# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `math` Chapter.

``math`` - Mathematical Constants

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the provision of mathematical constants
required by other functions within the NAG Library.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x01/x01intro.html
"""

# NAG Copyright 2017-2019.

import sys as _sys

from ..base import utils

def pi():
    u"""
    Provides the mathematical constant pi.

    For full information please refer to the NAG Library document for
    x01aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x01/x01aaf.html

    Returns
    -------
    pi_nag : float
        The numerical value of pi.
    """

    excs = []
    fun_name = 'naginterfaces.base.math.pi'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('x01aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Output, pi_nag.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _pi_nag = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _pi_nag,
    )
    utils._handle_cb_exception(excs)
    _pi_nag_py = utils._EngineFloat64ScalarType.to_py(_pi_nag)
    return_data = _pi_nag_py
    return return_data

def euler():
    u"""
    Provides the mathematical constant gamma (Euler's constant).

    For full information please refer to the NAG Library document for
    x01ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x01/x01abf.html

    Returns
    -------
    euler : float
        The numerical value of gamma (Euler's constant).
    """

    excs = []
    fun_name = 'naginterfaces.base.math.euler'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('x01abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Output, euler.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _euler = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _euler,
    )
    utils._handle_cb_exception(excs)
    _euler_py = utils._EngineFloat64ScalarType.to_py(_euler)
    return_data = _euler_py
    return return_data
