# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `dot` Chapter.

``dot`` - Inner Products

This module is concerned with the calculation of innerproducts required
by other functions within the NAG Library.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x03/x03intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def real_prec(a, b, n, istepa, istepb, c1, c2, sw):
    u"""
    Real inner product added to initial value, basic/additional
    precision.

    For full information please refer to the NAG Library document for
    x03aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x03/x03aaf.html

    Parameters
    ----------
    a : float, array-like, shape (isizea)
        The elements of the first vector.

        The ith vector element is stored in the array element
        `a`[(i-1)*`istepa`].

        In your function from which ``real_prec`` is called, `a` can be
        part of a multidimensional array and the actual argument must be
        the array element containing the first vector element.

    b : float, array-like, shape (isizeb)
        The elements of the second vector.

        The ith vector element is stored in the array element
        `b`[(i-1)*`istepb`].

        In your function from which ``real_prec`` is called, `b` can be
        part of a multidimensional array and the actual argument must be
        the array element containing the first vector element.

    n : int
        n, the number of elements in the scalar product.

    istepa : int
        The step length between elements of the first vector in array
        `a`.

    istepb : int
        The step length between elements of the second vector in array
        `b`.

    c1 : float
        `c1` and `c2` must specify the initial value c: c = `c1`+`c2`.
        Normally, if c is in additional precision, `c1` specifies the
        most significant part and `c2` the least significant part; if c
        is in basic precision, then `c1` specifies c and `c2` must have
        the value 0.0. Both `c1` and `c2` must be defined on entry.

    c2 : float
        `c1` and `c2` must specify the initial value c: c = `c1`+`c2`.
        Normally, if c is in additional precision, `c1` specifies the
        most significant part and `c2` the least significant part; if c
        is in basic precision, then `c1` specifies c and `c2` must have
        the value 0.0. Both `c1` and `c2` must be defined on entry.

    sw : bool
        The precision to be used in the calculation.

        `sw` = True

          additional precision.

        `sw` = False

          basic precision.

    Returns
    -------
    d1 : float
        The result d.

        If the calculation is in additional precision (`sw` = True),

        - `d1` = d rounded to basic precision;

        - `d2` = d-`d1`,

        thus `d1` holds the correctly rounded basic precision result and
        the sum `d1`+`d2` gives the result in additional precision. `d2`
        may have the opposite sign to `d1`.

        If the calculation is in basic precision (`sw` = False),

        - `d1` = d;

        - `d2` = 0.0.

    d2 : float
        The result d.

        If the calculation is in additional precision (`sw` = True),

        - `d1` = d rounded to basic precision;

        - `d2` = d-`d1`,

        thus `d1` holds the correctly rounded basic precision result and
        the sum `d1`+`d2` gives the result in additional precision. `d2`
        may have the opposite sign to `d1`.

        If the calculation is in basic precision (`sw` = False),

        - `d1` = d;

        - `d2` = 0.0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `istepb` = *<value>*.

            Constraint: `istepb` > 0.

        (`errno` 1)
            On entry, `istepa` = *<value>*.

            Constraint: `istepa` > 0.

        (`errno` 2)
            On entry, isizeb = *<value>*, `n` = *<value>* and `istepb` =
            *<value>*.

            Constraint: isizeb >= (`n`-1)*`istepb`+1.

        (`errno` 2)
            On entry, isizea = *<value>*, `n` = *<value>* and `istepa` =
            *<value>*.

            Constraint: isizea >= (`n`-1)*`istepa`+1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.dot import real_prec as b_real_prec

    fun_name = 'naginterfaces.library.dot.real_prec'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _isizea_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _isizea = _b_utils._EngineIntScalarType(
        _isizea_py, {'fun_name': fun_name, 'entity': '_isizea'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _isizeb_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _isizeb = _b_utils._EngineIntScalarType(
        _isizeb_py, {'fun_name': fun_name, 'entity': '_isizeb'},
    ).to_ctypes()
    _return_scalars = b_real_prec(
        _a,
        _isizea,
        _b,
        _isizeb,
        n,
        istepa,
        istepb,
        c1,
        c2,
        sw,
    )
    _d1_py = _return_scalars.d1
    _d2_py = _return_scalars.d2
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['d1', 'd2'],
    )(
        _d1_py,
        _d2_py,
    )
    return _return_data

def complex_prec(a, b, n, istepa, istepb, cx, sw):
    u"""
    Complex inner product added to initial value, basic/additional
    precision.

    For full information please refer to the NAG Library document for
    x03ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x03/x03abf.html

    Parameters
    ----------
    a : complex, array-like, shape (isizea)
        The elements of the first vector.

        The ith vector element is stored in the array element
        `a`[(i-1)*`istepa`].

        In your function from which ``complex_prec`` is called, `a` can
        be part of a multidimensional array and the actual argument must
        be the array element containing the first vector element.

    b : complex, array-like, shape (isizeb)
        The elements of the second vector.

        The ith vector element is stored in the array element
        `b`[(i-1)*`istepb`].

        In your function from which ``complex_prec`` is called, `b` can
        be part of a multidimensional array and the actual argument must
        be the array element containing the first vector element.

    n : int
        n, the number of elements in the scalar product.

    istepa : int
        The step length between elements of the first vector in array
        `a`.

    istepb : int
        The step length between elements of the second vector in array
        `b`.

    cx : complex
        The initial value c.

    sw : bool
        The precision to be used in the calculation.

        `sw` = True

          additional precision.

        `sw` = False

          basic precision.

    Returns
    -------
    dx : complex
        The result d.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `istepb` = *<value>*.

            Constraint: `istepb` > 0.

        (`errno` 1)
            On entry, `istepa` = *<value>*.

            Constraint: `istepa` > 0.

        (`errno` 2)
            On entry, isizeb = *<value>*, `n` = *<value>* and `istepb` =
            *<value>*.

            Constraint: isizeb >= (`n`-1)*`istepb`+1.

        (`errno` 2)
            On entry, isizea = *<value>*, `n` = *<value>* and `istepa` =
            *<value>*.

            Constraint: isizea >= (`n`-1)*`istepa`+1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.dot import complex_prec as b_complex_prec

    fun_name = 'naginterfaces.library.dot.complex_prec'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _isizea_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _isizea = _b_utils._EngineIntScalarType(
        _isizea_py, {'fun_name': fun_name, 'entity': '_isizea'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _isizeb_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _isizeb = _b_utils._EngineIntScalarType(
        _isizeb_py, {'fun_name': fun_name, 'entity': '_isizeb'},
    ).to_ctypes()
    _return_scalar = b_complex_prec(
        _a,
        _isizea,
        _b,
        _isizeb,
        n,
        istepa,
        istepb,
        cx,
        sw,
    )
    _dx_py = _return_scalar
    return _dx_py
