# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `interp` Chapter.

``interp`` - Interpolation

This module is concerned with the interpolation of a function of one or
more variables.
When provided with the value of the function (and possibly one or more
of its lowest-order derivatives) at each of a number of values of the
variable(s), the NAG Library functions provide either an interpolating
function or an interpolated value.
For some of the interpolating functions, there are supporting NAG
Library functions to evaluate, differentiate or integrate them.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01intro.html
"""

# NAG Copyright 2017-2019.

from __future__ import absolute_import

import collections as _collections

from math import sqrt

from ..base import utils as _b_utils
from ..library import _utils

def dim1_aitken(a, b, x):
    u"""
    Interpolated values, Aitken's technique, unequally spaced data, one
    variable.

    For full information please refer to the NAG Library document for
    e01aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01aaf.html

    Parameters
    ----------
    a : float, array-like, shape (n+1)
        `a`[i-1] must contain the x-component of the ith data point,
        x_i, for i = 1,2,...,n+1.

    b : float, array-like, shape (n+1)
        `b`[i-1] must contain the y-component (function value) of the
        ith data point, y_i, for i = 1,2,...,n+1.

    x : float
        The point x at which the interpolation is required. Note that
        `x` may lie outside the interval defined by the minimum and
        maximum values in `a`, in which case an extrapolated value will
        be computed; extrapolated results should be treated with
        considerable caution since there is no information on the
        behaviour of the function outside the defined interval.

    Returns
    -------
    a : float, ndarray, shape (n+1)
        `a`[i-1] contains the value x_i-x, for i = 1,2,...,n+1.

    b : float, ndarray, shape (n+1)
        The contents of `b` are unspecified.

    c : float, ndarray, shape (n*(n+1)/2)
        - `c`[0],...,`c`[n-1] contain the first set of linear
          interpolations,

        - `c`[n],...,`c`[2*n-2] contain the second set of linear
          interpolations,

        - `c`[2n-1],...,`c`[3*n-4] contain the third set of linear
          interpolations,

        -

        - `c`[n*(n+1)/2-1] contains the interpolated function value at
          the point x.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, error in parameter n.

            Constraint: n > 0.
    """

    from ..base.interp import dim1_aitken as b_dim1_aitken

    fun_name = 'naginterfaces.library.interp.dim1_aitken'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py*(_n_py+1)//2,),
    )
    b_dim1_aitken(
        _a,
        _b,
        _c,
        _n,
        x,
    )
    _a_py = _a
    _b_py = _b
    _c_py = _c
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'b', 'c'],
    )(
        _a_py,
        _b_py,
        _c_py,
    )
    return _return_data

def dim1_everett(p, a):
    u"""
    Interpolated values, Everett's formula, equally spaced data, one
    variable.

    For full information please refer to the NAG Library document for
    e01ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01abf.html

    Parameters
    ----------
    p : float
        The point p at which the interpolated function value is
        required, i.e., p = (x-x_0)/h with -1.0 < p < 1.0.

    a : float, array-like, shape (2*n)
        `a`[i-1] must be set to the function value y_{i-n}, for i =
        1,2,...,2n.

    Returns
    -------
    a : float, ndarray, shape (2*n)
        The contents of `a` are unspecified.

    g : float, ndarray, shape (2*n+1)
        The array contains

        [table omitted]

        The interpolated function value y_p is stored in `g`[2n].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` <= 1.0.

        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` >= -1.0.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n > 0.
    """

    from ..base.interp import dim1_everett as b_dim1_everett

    fun_name = 'naginterfaces.library.interp.dim1_everett'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]//2, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _g = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2*_n_py+1,),
    )
    b_dim1_everett(
        _n,
        p,
        _a,
        _g,
    )
    _a_py = _a
    _g_py = _g
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'g'],
    )(
        _a_py,
        _g_py,
    )
    return _return_data

def dim1_cheb(xmin, xmax, x, y, ip, lwrk, liwrk, itmin=0, itmax=0):
    u"""
    Interpolating functions, polynomial interpolant, data may include
    derivative values, one variable.

    For full information please refer to the NAG Library document for
    e01ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01aef.html

    Parameters
    ----------
    xmin : float
        The lower and upper end points, respectively, of the interval
        [x_min,x_max]. If they are not determined by your problem, it is
        recommended that they be set respectively to the smallest and
        largest values among the x_i.

    xmax : float
        The lower and upper end points, respectively, of the interval
        [x_min,x_max]. If they are not determined by your problem, it is
        recommended that they be set respectively to the smallest and
        largest values among the x_i.

    x : float, array-like, shape (m)
        The value of x_i, for i = 1,2,...,m. The `x`[i-1] need not be
        ordered.

    y : float, array-like, shape (n)
        The given values of the dependent variable, and derivatives, as
        follows:

        The first p_1+1 elements contain y_1,y_1^(1),...,y_1^(p_1) in
        that order.

        The next p_2+1 elements contain y_2,y_2^(1),...,y_2^(p_2) in
        that order.

        ...

        The last p_m+1 elements contain y_m,y_m^(1),...,y_m^(p_m) in
        that order.

    ip : int, array-like, shape (m)
        p_i, the order of the highest-order derivative whose value is
        given at x_i, for i = 1,2,...,m. If the value of y only is given
        for some x_i then the corresponding value of `ip`[i-1] must be
        zero.

    lwrk : int
        The dimension of the array `wrk`.

    liwrk : int
        The dimension of the array `iwrk`.

    itmin : int, optional
        Respectively the minimum and maximum number of iterations to be
        performed by the function (for full details see [Further
        Comments]). Setting `itmin` and/or `itmax` negative or zero
        invokes default value(s) of 2 and/or 10, respectively.

        The default values will be satisfactory for most problems, but
        occasionally significant improvement will result from using
        higher values.

    itmax : int, optional
        Respectively the minimum and maximum number of iterations to be
        performed by the function (for full details see [Further
        Comments]). Setting `itmin` and/or `itmax` negative or zero
        invokes default value(s) of 2 and/or 10, respectively.

        The default values will be satisfactory for most problems, but
        occasionally significant improvement will result from using
        higher values.

    Returns
    -------
    a : float, ndarray, shape (n)
        `a`[i] contains the coefficient a_i in the Chebyshev series
        representation of q(x), for i = 0,...,n-1.

    wrk : float, ndarray, shape (`lwrk`)
        Used as workspace, but see also [Further Comments].

    iwrk : int, ndarray, shape (`liwrk`)
        Used as workspace, but see also [Further Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `liwrk` is too small. `liwrk` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, `lwrk` is too small. `lwrk` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, n = *<value>* and m+`ip`[0]+`ip`[1]+...+`ip`[m-1]
            = *<value>*.

            Constraint: n = m+`ip`[0]+`ip`[1]+...+`ip`[m-1].

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, I = *<value>* and `ip`[I-1] = *<value>*.

            Constraint: `ip`[I-1] >= 0.

        (`errno` 3)
            On entry, I = *<value>*, J = *<value>* and `x`[I-1] =
            *<value>*.

            Constraint: `x`[I-1] != `x`[J-1].

        (`errno` 3)
            On entry, I = *<value>*, `x`[I-1] = *<value>*, `xmin` =
            *<value>* and `xmax` = *<value>*.

            Constraint: `xmin` <= `x`[I-1] <= `xmax`.

        (`errno` 3)
            On entry, `xmin` = *<value>* and `xmax` = *<value>*.

            Constraint: `xmin` < `xmax`.

        (`errno` 5)
            The computation has been terminated because the iterative
            process appears to be diverging.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Not all the performance indices are less than eight times
            the machine precision, although `itmax` iterations have been
            performed. A more accurate solution may possibly be obtained
            by increasing `itmax` and recalling the function.
    """

    from ..base.interp import dim1_cheb as b_dim1_cheb

    fun_name = 'naginterfaces.library.interp.dim1_cheb'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ip = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ip'},
        ip,
        copy=False,
    )
    _a = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _lwrk = _b_utils._EngineIntScalarType(
        lwrk, {'fun_name': fun_name, 'entity': 'lwrk'},
    ).to_ctypes()
    _lwrk_py = _b_utils._EngineIntScalarType.to_py(_lwrk)
    _wrk = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lwrk_py,),
    )
    _liwrk = _b_utils._EngineIntScalarType(
        liwrk, {'fun_name': fun_name, 'entity': 'liwrk'},
    ).to_ctypes()
    _liwrk_py = _b_utils._EngineIntScalarType.to_py(_liwrk)
    _iwrk = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liwrk_py,),
    )
    try:
        b_dim1_cheb(
            _m,
            xmin,
            xmax,
            _x,
            _y,
            _ip,
            _n,
            itmin,
            itmax,
            _a,
            _wrk,
            _lwrk,
            _iwrk,
            _liwrk,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _wrk_py = _wrk
    _iwrk_py = _iwrk
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'wrk', 'iwrk'],
    )(
        _a_py,
        _wrk_py,
        _iwrk_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_spline(x, y):
    u"""
    Interpolating functions, cubic spline interpolant, one variable.

    For full information please refer to the NAG Library document for
    e01ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01baf.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        `x`[i-1] must be set to x_i, the ith data value of the
        independent variable x, for i = 1,2,...,m.

    y : float, array-like, shape (m)
        `y`[i-1] must be set to y_i, the ith data value of the dependent
        variable y, for i = 1,2,...,m.

    Returns
    -------
    lamda : float, ndarray, shape (m+4)
        The value of lambda_i, the ith knot, for i = 1,2,...,m+4.

    c : float, ndarray, shape (m)
        The coefficient c_i of the B-spline N_i(x), for i = 1,2,...,m.
        The remaining elements of the array are not used.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lck = *<value>* and m = *<value>*.

            Constraint: lck >= m+4.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 4.

        (`errno` 2)
            On entry, I = *<value>*, `x`[I-1] = *<value>* and `x`[I-2] =
            *<value>*.

            Constraint: `x`[I-1] > `x`[I-2].

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim1_spline as b_dim1_spline

    fun_name = 'naginterfaces.library.interp.dim1_spline'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _lck = _b_utils._EngineIntScalarType(
        _m_py+4, {'fun_name': fun_name, 'entity': '_lck'},
    ).to_ctypes()
    _lamda = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py+4,),
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py+4,),
    )
    b_dim1_spline(
        _m,
        _x,
        _y,
        _lamda,
        _c,
        _lck,
    )
    _lamda_py = _lamda
    _c_py = _c[:_m_py]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['lamda', 'c'],
    )(
        _lamda_py,
        _c_py,
    )
    return _return_data

def dim1_monotonic(x, f):
    u"""
    Interpolating functions, monotonicity-preserving, piecewise cubic
    Hermite, one variable.

    For full information please refer to the NAG Library document for
    e01be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01bef.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x`[r-1] must be set to x_r, the rth value of the independent
        variable (abscissa), for r = 1,2,...,n.

    f : float, array-like, shape (n)
        `f`[r-1] must be set to f_r, the rth value of the dependent
        variable (ordinate), for r = 1,2,...,n.

    Returns
    -------
    d : float, ndarray, shape (n)
        Estimates of derivatives at the data points. `d`[r-1] contains
        the derivative at `x`[r-1].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, r = *<value>*, `x`[r-2] = *<value>* and `x`[r-1] =
            *<value>*.

            Constraint: `x`[r-2] < `x`[r-1] for all r.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim1_monotonic as b_dim1_monotonic

    fun_name = 'naginterfaces.library.interp.dim1_monotonic'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dim1_monotonic(
        _n,
        _x,
        _f,
        _d,
    )
    _d_py = _d
    return _d_py

def dim1_monotonic_eval(x, f, d, px):
    u"""
    Interpolated values, interpolant computed by ``dim1_monotonic``,
    function only, one variable.

    For full information please refer to the NAG Library document for
    e01bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01bff.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    f : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    d : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    px : float, array-like, shape (m)
        The m values of x at which the interpolant is to be evaluated.

    Returns
    -------
    pf : float, ndarray, shape (m)
        `pf`[i-1] contains the value of the interpolant evaluated at the
        point `px`[i-1], for i = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, r = *<value>*, `x`[r-2] = *<value>* and `x`[r-1] =
            *<value>*.

            Constraint: `x`[r-2] < `x`[r-1] for all r.

        (`errno` 3)
            On entry, m = *<value>*.

            Constraint: m >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Warning -- some points in array `px` lie outside the range
            `x`[0]...`x`[n-1]. Values at these points are unreliable
            because computed by extrapolation.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim1_monotonic_eval as b_dim1_monotonic_eval

    fun_name = 'naginterfaces.library.interp.dim1_monotonic_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
    )
    _px = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'px'},
        px,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_px, {'fun_name': fun_name, 'entity': 'px'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _pf = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    try:
        b_dim1_monotonic_eval(
            _n,
            _x,
            _f,
            _d,
            _m,
            _px,
            _pf,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _pf_py = _pf
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['pf'],
    )(
        _pf_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _pf_py

def dim1_monotonic_deriv(x, f, d, px):
    u"""
    Interpolated values, interpolant computed by ``dim1_monotonic``,
    function and first derivative, one variable.

    For full information please refer to the NAG Library document for
    e01bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01bgf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    f : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    d : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    px : float, array-like, shape (m)
        The m values of x at which the interpolant is to be evaluated.

    Returns
    -------
    pf : float, ndarray, shape (m)
        `pf`[i-1] contains the value of the interpolant evaluated at the
        point `px`[i-1], for i = 1,2,...,m.

    pd : float, ndarray, shape (m)
        `pd`[i-1] contains the first derivative of the interpolant
        evaluated at the point `px`[i-1], for i = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, r = *<value>*, `x`[r-2] = *<value>* and `x`[r-1] =
            *<value>*.

            Constraint: `x`[r-2] < `x`[r-1] for all r.

        (`errno` 3)
            On entry, m = *<value>*.

            Constraint: m >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Warning -- some points in array `px` lie outside the range
            `x`[0]...`x`[n-1]. Values at these points are unreliable
            because computed by extrapolation.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim1_monotonic_deriv as b_dim1_monotonic_deriv

    fun_name = 'naginterfaces.library.interp.dim1_monotonic_deriv'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
    )
    _px = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'px'},
        px,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_px, {'fun_name': fun_name, 'entity': 'px'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _pf = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _pd = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    try:
        b_dim1_monotonic_deriv(
            _n,
            _x,
            _f,
            _d,
            _m,
            _px,
            _pf,
            _pd,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _pf_py = _pf
    _pd_py = _pd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['pf', 'pd'],
    )(
        _pf_py,
        _pd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_monotonic_intg(x, f, d, a, b):
    u"""
    Interpolated values, interpolant computed by ``dim1_monotonic``,
    definite integral, one variable.

    For full information please refer to the NAG Library document for
    e01bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01bhf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    f : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    d : float, array-like, shape (n)
        n, `x`, `f` and `d` must be unchanged from the previous call of
        ``dim1_monotonic``.

    a : float
        The interval [a,b] over which integration is to be performed.

    b : float
        The interval [a,b] over which integration is to be performed.

    Returns
    -------
    pint : float
        The value of the definite integral of the interpolant over the
        interval [a,b].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, r = *<value>*, `x`[r-2] = *<value>* and `x`[r-1] =
            *<value>*.

            Constraint: `x`[r-2] < `x`[r-1] for all r.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Warning -- either `a` or `b` is outside the range
            `x`[0]...`x`[n-1]. The result has been computed by
            extrapolation and is unreliable. `a` = *<value>* `b` =
            *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim1_monotonic_intg as b_dim1_monotonic_intg

    fun_name = 'naginterfaces.library.interp.dim1_monotonic_intg'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
    )
    try:
        _return_scalar = b_dim1_monotonic_intg(
            _n,
            _x,
            _f,
            _d,
            a,
            b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _pint_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['pint'],
    )(
        _pint_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _pint_py

def dim1_monconv_disc(negfor, yfor, x, y, lam=0.2):
    u"""
    Interpolating variables, monotone convex Hagan--West procedure, one
    variable.

    For full information please refer to the NAG Library document for
    e01ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01cef.html

    Parameters
    ----------
    negfor : bool
        Determines whether or not to allow negative forward rates.

        `negfor` = True

          Negative forward rates are permitted.

        `negfor` = False

          Forward rates calculated must be non-negative.

    yfor : bool
        Determines whether the array `y` contains values, y, or discrete
        forward rates f^d.

        `yfor` = True

          `y` contains the discrete forward rates f_i^d, for i =
          1,2,...,n.

        `yfor` = False

          `y` contains the values y_i, for i = 1,2,...,n.

    x : float, array-like, shape (n)
        x, the (possibly unordered) set of data points.

    y : float, array-like, shape (n)
        If `yfor` = True, the discrete forward rates f_i^d corresponding
        to the data points x_i, for i = 1,2,...,n.

        If `yfor` = False, the data values y_i corresponding to the data
        points x_i, for i = 1,2,...,n.

    lam : float, optional
        lambda, the amelioration (smoothing) parameter. Forward rates
        are first computed using (2) and then, if lambda > 0, a limiting
        filter is applied which depends on neighbouring discrete forward
        values. This filter has a smoothing effect on the curve that
        increases with lambda.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `lam` = *<value>*.

            Constraint: 0.0 <= `lam` <= 1.0.

        (`errno` 3)
            On entry, `x` contains duplicate data points.
    """

    from ..base.interp import dim1_monconv_disc as b_dim1_monconv_disc

    fun_name = 'naginterfaces.library.interp.dim1_monconv_disc'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _return_scalar = b_dim1_monconv_disc(
        _n,
        lam,
        negfor,
        yfor,
        _x,
        _y,
    )
    _comm_py = _return_scalar
    return _comm_py

def dim1_monconv_eval(x, comm):
    u"""
    Interpolated values, variables computed by ``dim1_monconv_disc``,
    monotone convex Hagan--West procedure, one variable.

    For full information please refer to the NAG Library document for
    e01cf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01cff.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        x, the points at which the interpolant is to be evaluated.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_monconv_disc``.

    Returns
    -------
    val : float, ndarray, shape (m)
        The values of the interpolant at x.

    fwd : float, ndarray, shape (m)
        The values of the forward rates at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either ``dim1_monconv_disc`` was not called first or the
            communication array has become corrupted.
    """

    from ..base.interp import dim1_monconv_eval as b_dim1_monconv_eval

    fun_name = 'naginterfaces.library.interp.dim1_monconv_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _val = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _fwd = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    b_dim1_monconv_eval(
        _m,
        _x,
        _val,
        _fwd,
        comm,
    )
    _val_py = _val
    _fwd_py = _fwd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['val', 'fwd'],
    )(
        _val_py,
        _fwd_py,
    )
    return _return_data

def dim2_spline_grid(x, y, f):
    u"""
    Interpolating functions, fitting bicubic spline, data on rectangular
    grid.

    For full information please refer to the NAG Library document for
    e01da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01daf.html

    Parameters
    ----------
    x : float, array-like, shape (mx)
        `x`[q-1] and `y`[r-1] must contain x_q, for q = 1,2,...,m_x, and
        y_r, for r = 1,2,...,m_y, respectively.

    y : float, array-like, shape (my)
        `x`[q-1] and `y`[r-1] must contain x_q, for q = 1,2,...,m_x, and
        y_r, for r = 1,2,...,m_y, respectively.

    f : float, array-like, shape (mx*my)
        `f`[m_y*(q-1)+r-1] must contain f_{q,r}, for r = 1,2,...,m_y,
        for q = 1,2,...,m_x.

    Returns
    -------
    px : int
        `px` and `py` contain m_x+4 and m_y+4, the total number of knots
        of the computed spline with respect to the x and y variables,
        respectively.

    py : int
        `px` and `py` contain m_x+4 and m_y+4, the total number of knots
        of the computed spline with respect to the x and y variables,
        respectively.

    lamda : float, ndarray, shape (mx+4)
        `lamda` contains the complete set of knots lambda_i associated
        with the x variable

    mu : float, ndarray, shape (my+4)
        `mu` contains the complete set of knots mu_i associated with the
        y variable

    c : float, ndarray, shape (mx*my)
        The coefficients of the spline interpolant. `c`[m_y*(i-1)+j-1]
        contains the coefficient c_{ij} described in [Description].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, my = *<value>*.

            Constraint: my >= 4.

        (`errno` 1)
            On entry, mx = *<value>*.

            Constraint: mx >= 4.

        (`errno` 2)
            On entry, the `x` or the `y` mesh points are not in strictly
            ascending order.

        (`errno` 3)
            An intermediate set of linear equations is singular -- the
            data is too ill-conditioned to compute B-spline
            coefficients.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim2_spline_grid as b_dim2_spline_grid

    fun_name = 'naginterfaces.library.interp.dim2_spline_grid'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _mx_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _mx = _b_utils._EngineIntScalarType(
        _mx_py, {'fun_name': fun_name, 'entity': '_mx'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _my_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=1)[0]
    _my = _b_utils._EngineIntScalarType(
        _my_py, {'fun_name': fun_name, 'entity': '_my'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _lamda = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_mx_py+4,),
    )
    _mu = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_my_py+4,),
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_mx_py*_my_py,),
    )
    _return_scalars = b_dim2_spline_grid(
        _mx,
        _my,
        _x,
        _y,
        _f,
        _lamda,
        _mu,
        _c,
    )
    _px_py = _return_scalars.px
    _py_py = _return_scalars.py
    _lamda_py = _lamda
    _mu_py = _mu
    _c_py = _c
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['px', 'py', 'lamda', 'mu', 'c'],
    )(
        _px_py,
        _py_py,
        _lamda_py,
        _mu_py,
        _c_py,
    )
    return _return_data

def dim2_triangulate(x, y):
    u"""
    Triangulation of two-dimensional scattered grid, method of Renka and
    Cline.

    For full information please refer to the NAG Library document for
    e01ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01eaf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The x coordinates of the n data points.

    y : float, array-like, shape (n)
        The y coordinates of the n data points.

    Returns
    -------
    triang : int, ndarray, shape (7*n)
        A data structure defining the computed triangulation, in a form
        suitable for passing to ``dim2_triang_bary_eval``. Details of
        how the triangulation is encoded in this argument are given in
        [Further Comments]. These details are most likely to be of use
        when plotting the computed triangulation which is demonstrated
        in [Example].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 3.

        (`errno` 2)
            On entry, all the (x,y) pairs are collinear.
    """

    from ..base.interp import dim2_triangulate as b_dim2_triangulate

    fun_name = 'naginterfaces.library.interp.dim2_triangulate'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _triang = _b_utils._EngineIntArrayType.empty_ndarray(
        (7*_n_py,),
    )
    b_dim2_triangulate(
        _n,
        _x,
        _y,
        _triang,
    )
    _triang_py = _triang
    return _triang_py

def dim2_triang_bary_eval(x, y, f, triang, px, py):
    u"""
    Barycentric interpolation on function values provided on a
    two-dimensional scattered grid.

    For full information please refer to the NAG Library document for
    e01eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01ebf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The coordinates of the rth data point, (x_r,y_r), for r =
        1,2,...,n. `x` and `y` must be unchanged from the previous call
        of ``dim2_triangulate``.

    y : float, array-like, shape (n)
        The coordinates of the rth data point, (x_r,y_r), for r =
        1,2,...,n. `y` and `y` must be unchanged from the previous call
        of ``dim2_triangulate``.

    f : float, array-like, shape (n)
        The function values f_r at (x_r,y_r), for r = 1,2,...,n.

    triang : int, array-like, shape (7*n)
        The triangulation computed by the previous call of
        ``dim2_triangulate``. See [Further Comments] for details of how
        the triangulation used is encoded in `triang`.

    px : float, array-like, shape (m)
        The coordinates (px_i,py_i), for i = 1,2,...,m, at which
        interpolated function values are sought.

    py : float, array-like, shape (m)
        The coordinates (px_i,py_i), for i = 1,2,...,m, at which
        interpolated function values are sought.

    Returns
    -------
    pf : float, ndarray, shape (m)
        The interpolated values F(px_i,py_i), for i = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 3.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, the triangulation information held in the array
            `triang` does not specify a valid triangulation of the data
            points. `triang` has been corrupted since the call to
            ``dim2_triangulate``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            At least one evaluation point lies outside the nodal
            triangulation. For each such point the value returned in
            `pf` is that corresponding to a node on the closest boundary
            line segment.
    """

    from ..base.interp import dim2_triang_bary_eval as b_dim2_triang_bary_eval

    fun_name = 'naginterfaces.library.interp.dim2_triang_bary_eval'

    _px = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'px'},
        px,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_px, {'fun_name': fun_name, 'entity': 'px'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _triang = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'triang'},
        triang,
        copy=False,
    )
    _py = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'py'},
        py,
        copy=False,
    )
    _pf = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    try:
        b_dim2_triang_bary_eval(
            _m,
            _n,
            _x,
            _y,
            _f,
            _triang,
            _px,
            _py,
            _pf,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _pf_py = _pf
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['pf'],
    )(
        _pf_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _pf_py

def dim1_ratnl(x, f):
    u"""
    Interpolating functions, rational interpolant, one variable.

    For full information please refer to the NAG Library document for
    e01ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01raf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x`[i-1] must be set to the value of the ith data abscissa, x_i,
        for i = 1,2,...,n.

    f : float, array-like, shape (n)
        `f`[i-1] must be set to the value of the data ordinate, f_i,
        corresponding to x_i, for i = 1,2,...,n.

    Returns
    -------
    m : int
        m, the number of terms in the continued fraction representation
        of R(x).

    a : float, ndarray, shape (n)
        `a`[j-1] contains the value of the parameter a_j in R(x), for j
        = 1,2,...,m. The remaining elements of `a`, if any, are set to
        zero.

    u : float, ndarray, shape (n)
        `u`[j-1] contains the value of the parameter u_j in R(x), for j
        = 1,2,...,m-1. The u_j are a permuted subset of the elements of
        `x`. The remaining n-m+1 locations contain a permutation of the
        remaining x_i, which can be ignored.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 2)
            On entry, `x`[I-1] is very close to `x`[J-1]: I = *<value>*,
            `x`[I-1] = *<value>*, J = *<value>* and `x`[J-1] =
            *<value>*.

        (`errno` 3)
            A continued fraction of the required form does not exist.
    """

    from ..base.interp import dim1_ratnl as b_dim1_ratnl

    fun_name = 'naginterfaces.library.interp.dim1_ratnl'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _a = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalar = b_dim1_ratnl(
        _n,
        _x,
        _f,
        _a,
        _u,
    )
    _m_py = _return_scalar
    _a_py = _a
    _u_py = _u
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['m', 'a', 'u'],
    )(
        _m_py,
        _a_py,
        _u_py,
    )
    return _return_data

def dim1_ratnl_eval(a, u, x):
    u"""
    Interpolated values, evaluate rational interpolant computed by
    ``dim1_ratnl``, one variable.

    For full information please refer to the NAG Library document for
    e01rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01rbf.html

    Parameters
    ----------
    a : float, array-like, shape (m)
        `a`[j-1] must be set to the value of the parameter a_j in the
        continued fraction, for j = 1,2,...,m.

    u : float, array-like, shape (m)
        `u`[j-1] must be set to the value of the parameter u_j in the
        continued fraction, for j = 1,2,...,m-1. (The element `u`[m-1]
        is not used).

    x : float
        The value of x at which the continued fraction is to be
        evaluated.

    Returns
    -------
    f : float
        The value of the continued fraction corresponding to the value
        of x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            `x` corresponds to a pole of R(x), or is very close. `x` =
            *<value>*.
    """

    from ..base.interp import dim1_ratnl_eval as b_dim1_ratnl_eval

    fun_name = 'naginterfaces.library.interp.dim1_ratnl_eval'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _u = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
        copy=False,
    )
    _return_scalar = b_dim1_ratnl_eval(
        _m,
        _a,
        _u,
        x,
    )
    _f_py = _return_scalar
    return _f_py

def dim2_scat(x, y, f):
    u"""
    Interpolating functions, method of Renka and Cline, two variables.

    For full information please refer to the NAG Library document for
    e01sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01saf.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        The coordinates of the rth data point, for r = 1,2,...,m. The
        data points are accepted in any order, but see [Further
        Comments].

    y : float, array-like, shape (m)
        The coordinates of the rth data point, for r = 1,2,...,m. The
        data points are accepted in any order, but see [Further
        Comments].

    f : float, array-like, shape (m)
        The coordinates of the rth data point, for r = 1,2,...,m. The
        data points are accepted in any order, but see [Further
        Comments].

    Returns
    -------
    triang : int, ndarray, shape (7*m)
        A data structure defining the computed triangulation, in a form
        suitable for passing to ``dim2_scat_eval``.

    grads : float, ndarray, shape (2, m)
        The estimated partial derivatives at the nodes, in a form
        suitable for passing to ``dim2_scat_eval``. The derivatives at
        node r with respect to x and y are contained in `grads`[0,r-1]
        and `grads`[1,r-1] respectively, for r = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 3.

        (`errno` 2)
            All nodes are collinear. There is no unique solution.

        (`errno` 3)
            On entry, (`x`[I-1],`y`[I-1]) = (`x`[J-1],`y`[J-1]), for I,J
            = *<value>**<value>*, `x`[I-1], `y`[I-1] =
            *<value>**<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim2_scat as b_dim2_scat

    fun_name = 'naginterfaces.library.interp.dim2_scat'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _triang = _b_utils._EngineIntArrayType.empty_ndarray(
        (7*_m_py,),
    )
    _grads = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2, _m_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_dim2_scat(
        _m,
        _x,
        _y,
        _f,
        _triang,
        _grads,
    )
    _triang_py = _triang
    _grads_py = _grads
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['triang', 'grads'],
    )(
        _triang_py,
        _grads_py,
    )
    return _return_data

def dim2_scat_eval(x, y, f, triang, grads, px, py, ist=1):
    u"""
    Interpolated values, evaluate interpolant computed by ``dim2_scat``,
    two variables.

    For full information please refer to the NAG Library document for
    e01sb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01sbf.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        `x` must be unchanged from the previous call of ``dim2_scat``

    y : float, array-like, shape (m)
        `y` must be unchanged from the previous call of ``dim2_scat``

    f : float, array-like, shape (m)
        `f` must be unchanged from the previous call of ``dim2_scat``

    triang : int, array-like, shape (7*m)
        `triang` must be unchanged from the previous call of
        ``dim2_scat``

    grads : float, array-like, shape (2, m)
        `grads` must be unchanged from the previous call of
        ``dim2_scat``

    px : float
        The point (px,py) at which the interpolant is to be evaluated.

    py : float
        The point (px,py) at which the interpolant is to be evaluated.

    ist : int, optional
        The index of the starting node in the search for a triangle
        containing the point (px,py). On the first call to
        ``dim2_scat_eval``, `ist` must be set to 1. For efficiency on
        subsequent calls to ``dim2_scat_eval`` an updated value of `ist`
        as returned by ``dim2_scat_eval`` may be supplied instead. An
        input value outside the range 1 <= `ist` <= m will be treated as
        1.

    Returns
    -------
    ist : int
        The index of one of the vertices of the triangle containing the
        point (px,py).

    pf : float
        The value of the interpolant evaluated at the point (px,py).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 3.

        (`errno` 2)
            On entry, `triang` does not contain a valid data point
            triangulation; `triang` may have been corrupted since the
            call to ``dim2_scat``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Warning -- the evaluation point (*<value>*,*<value>*) lies
            outside the triangulation boundary. The returned value was
            computed by extrapolation.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.interp import dim2_scat_eval as b_dim2_scat_eval

    fun_name = 'naginterfaces.library.interp.dim2_scat_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _triang = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'triang'},
        triang,
        copy=False,
    )
    _grads = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'grads'},
        grads,
        copy=False,
    )
    try:
        _return_scalars = b_dim2_scat_eval(
            _m,
            _x,
            _y,
            _f,
            _triang,
            _grads,
            px,
            py,
            ist,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ist_py = _return_scalars.ist
    _pf_py = _return_scalars.pf
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ist', 'pf'],
    )(
        _ist_py,
        _pf_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim2_scat_shep(x, y, f, nw, nq):
    u"""
    Interpolating functions, modified Shepard's method, two variables.

    For full information please refer to the NAG Library document for
    e01sg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01sgf.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        The Cartesian coordinates of the data points (x_r,y_r), for r =
        1,2,...,m.

    y : float, array-like, shape (m)
        The Cartesian coordinates of the data points (x_r,y_r), for r =
        1,2,...,m.

    f : float, array-like, shape (m)
        `f`[r-1] must be set to the data value f_r, for r = 1,2,...,m.

    nw : int
        The number N_w of data points that determines each radius of
        influence R_w, appearing in the definition of each of the
        weights w_r, for r = 1,2,...,m (see [Description]). Note that
        R_w is different for each weight. If `nw` <= 0 the default value
        `nw` = min(19, m-1) is used instead.

    nq : int
        The number N_q of data points to be used in the least squares
        fit for coefficients defining the nodal functions q_r(x,y) (see
        [Description]). If `nq` <= 0 the default value `nq` = min(13,
        m-1) is used instead.

    Returns
    -------
    iq : int, ndarray, shape (2*m+1)
        Integer data defining the interpolant Q(x,y).

    rq : float, ndarray, shape (6*m+5)
        Real data defining the interpolant Q(x,y).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lrq is too small: lrq = *<value>*.

        (`errno` 1)
            On entry, liq is too small: liq = *<value>*.

        (`errno` 1)
            On entry, `nw` = *<value>* and m = *<value>*.

            Constraint: `nw` <= min(40, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>* and m = *<value>*.

            Constraint: `nq` <= min(40, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>*.

            Constraint: `nq` <= 0 or `nq` >= 5.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 6.

        (`errno` 2)
            There are duplicate nodes in the dataset.
            (`x`[I-1],`y`[I-1]) = (`x`[J-1],`y`[J-1]), for I = *<value>*
            and J = *<value>*. The interpolant cannot be derived.

        (`errno` 3)
            All nodes are collinear. There is no unique solution.
    """

    from ..base.interp import dim2_scat_shep as b_dim2_scat_shep

    fun_name = 'naginterfaces.library.interp.dim2_scat_shep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _liq = _b_utils._EngineIntScalarType(
        2*_m_py+1, {'fun_name': fun_name, 'entity': '_liq'},
    ).to_ctypes()
    _iq = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*_m_py+1,),
    )
    _lrq = _b_utils._EngineIntScalarType(
        6*_m_py+5, {'fun_name': fun_name, 'entity': '_lrq'},
    ).to_ctypes()
    _rq = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (6*_m_py+5,),
    )
    b_dim2_scat_shep(
        _m,
        _x,
        _y,
        _f,
        nw,
        nq,
        _iq,
        _liq,
        _rq,
        _lrq,
    )
    _iq_py = _iq
    _rq_py = _rq
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iq', 'rq'],
    )(
        _iq_py,
        _rq_py,
    )
    return _return_data

def dim2_scat_shep_eval(x, y, f, iq, rq, u, v):
    u"""
    Interpolated values, evaluate interpolant computed by
    ``dim2_scat_shep``, function and first derivatives, two variables.

    For full information please refer to the NAG Library document for
    e01sh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01shf.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        m, `x`, `y` and `f` must be the same values as were supplied in
        the preceding call to ``dim2_scat_shep``.

    y : float, array-like, shape (m)
        m, `x`, `y` and `f` must be the same values as were supplied in
        the preceding call to ``dim2_scat_shep``.

    f : float, array-like, shape (m)
        m, `x`, `y` and `f` must be the same values as were supplied in
        the preceding call to ``dim2_scat_shep``.

    iq : int, array-like, shape (2*m+1)
        Must be unchanged from the value returned from a previous call
        to ``dim2_scat_shep``.

    rq : float, array-like, shape (6*m+5)
        Must be unchanged from the value returned from a previous call
        to ``dim2_scat_shep``.

    u : float, array-like, shape (n)
        The evaluation points (u_i,v_i), for i = 1,2,...,n.

    v : float, array-like, shape (n)
        The evaluation points (u_i,v_i), for i = 1,2,...,n.

    Returns
    -------
    q : float, ndarray, shape (n)
        The values of the interpolant at (u_i,v_i), for i = 1,2,...,n.
        If any of these evaluation points lie outside the region of
        definition of the interpolant the corresponding entries in `q`
        are set to the largest machine representable number (see
        ``machine.real_largest``), and ``dim2_scat_shep_eval`` returns
        with `errno` = 3.

    qx : float, ndarray, shape (n)
        The values of the partial derivatives of the interpolant Q(x,y)
        at (u_i,v_i), for i = 1,2,...,n. If any of these evaluation
        points lie outside the region of definition of the interpolant,
        the corresponding entries in `qx` and `qy` are set to the
        largest machine representable number (see
        ``machine.real_largest``), and ``dim2_scat_shep_eval`` returns
        with `errno` = 3.

    qy : float, ndarray, shape (n)
        The values of the partial derivatives of the interpolant Q(x,y)
        at (u_i,v_i), for i = 1,2,...,n. If any of these evaluation
        points lie outside the region of definition of the interpolant,
        the corresponding entries in `qx` and `qy` are set to the
        largest machine representable number (see
        ``machine.real_largest``), and ``dim2_scat_shep_eval`` returns
        with `errno` = 3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lrq is too small: lrq = *<value>*.

        (`errno` 1)
            On entry, liq is too small: liq = *<value>*.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 6.

        (`errno` 2)
            On entry, values in `rq` appear to be invalid. Check that
            `rq` has not been corrupted between calls to
            ``dim2_scat_shep`` and ``dim2_scat_shep_eval``.

        (`errno` 2)
            On entry, values in `iq` appear to be invalid. Check that
            `iq` has not been corrupted between calls to
            ``dim2_scat_shep`` and ``dim2_scat_shep_eval``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, at least one evaluation point lies outside the
            region of definition of the interpolant. At such points the
            corresponding values in `q` and `qx` contain extrapolated
            approximations. Points should be evaluated one by one to
            identify extrapolated values.
    """

    from ..base.interp import dim2_scat_shep_eval as b_dim2_scat_shep_eval

    fun_name = 'naginterfaces.library.interp.dim2_scat_shep_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iq'},
        iq,
        copy=False,
    )
    _liq_py = _b_utils._EngineIntArrayType.get_shape(_iq, {'fun_name': fun_name, 'entity': 'iq'}, exp_rank=1)[0]
    _liq = _b_utils._EngineIntScalarType(
        _liq_py, {'fun_name': fun_name, 'entity': '_liq'},
    ).to_ctypes()
    _rq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rq'},
        rq,
        copy=False,
    )
    _lrq_py = _b_utils._EngineFloat64ArrayType.get_shape(_rq, {'fun_name': fun_name, 'entity': 'rq'}, exp_rank=1)[0]
    _lrq = _b_utils._EngineIntScalarType(
        _lrq_py, {'fun_name': fun_name, 'entity': '_lrq'},
    ).to_ctypes()
    _u = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=False,
    )
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qy = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_dim2_scat_shep_eval(
            _m,
            _x,
            _y,
            _f,
            _iq,
            _liq,
            _rq,
            _lrq,
            _n,
            _u,
            _v,
            _q,
            _qx,
            _qy,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _q_py = _q
    _qx_py = _qx
    _qy_py = _qy
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'qx', 'qy'],
    )(
        _q_py,
        _qx_py,
        _qy_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim3_scat_shep(x, y, z, f, nw, nq):
    u"""
    Interpolating functions, modified Shepard's method, three variables.

    For full information please refer to the NAG Library document for
    e01tg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01tgf.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        `x`[r-1], `y`[r-1], `z`[r-1] must be set to the Cartesian
        coordinates of the data point (x_r,y_r,z_r), for r = 1,2,...,m.

    y : float, array-like, shape (m)
        `x`[r-1], `y`[r-1], `z`[r-1] must be set to the Cartesian
        coordinates of the data point (x_r,y_r,z_r), for r = 1,2,...,m.

    z : float, array-like, shape (m)
        `x`[r-1], `y`[r-1], `z`[r-1] must be set to the Cartesian
        coordinates of the data point (x_r,y_r,z_r), for r = 1,2,...,m.

    f : float, array-like, shape (m)
        `f`[r-1] must be set to the data value f_r, for r = 1,2,...,m.

    nw : int
        The number N_w of data points that determines each radius of
        influence R_w, appearing in the definition of each of the
        weights w_r, for r = 1,2,...,m (see [Description]). Note that
        R_w is different for each weight. If `nw` <= 0 the default value
        `nw` = min(32, m-1) is used instead.

    nq : int
        The number N_q of data points to be used in the least squares
        fit for coefficients defining the nodal functions q_r(x,y,z)
        (see [Description]). If `nq` <= 0 the default value `nq` =
        min(17, m-1) is used instead.

    Returns
    -------
    iq : int, ndarray, shape (2*m+1)
        Integer data defining the interpolant Q(x,y,z).

    rq : float, ndarray, shape (10*m+7)
        Real data defining the interpolant Q(x,y,z).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lrq is too small: lrq = *<value>*.

        (`errno` 1)
            On entry, liq is too small: liq = *<value>*.

        (`errno` 1)
            On entry, `nw` = *<value>* and m = *<value>*.

            Constraint: `nw` <= min(40, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>* and m = *<value>*.

            Constraint: `nq` <= min(40, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>*.

            Constraint: `nq` <= 0 or `nq` >= 9.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 10.

        (`errno` 2)
            There are duplicate nodes in the dataset.
            (`x`[I-1],`y`[I-1],`z`[I-1]) = (`x`[J-1],`y`[J-1],`z`[J-1])
            for: I = *<value>* and J = *<value>*. The interpolant cannot
            be derived.

        (`errno` 3)
            All nodes are coplanar. There is no unique solution.
    """

    from ..base.interp import dim3_scat_shep as b_dim3_scat_shep

    fun_name = 'naginterfaces.library.interp.dim3_scat_shep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _liq = _b_utils._EngineIntScalarType(
        2*_m_py+1, {'fun_name': fun_name, 'entity': '_liq'},
    ).to_ctypes()
    _iq = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*_m_py+1,),
    )
    _lrq = _b_utils._EngineIntScalarType(
        10*_m_py+7, {'fun_name': fun_name, 'entity': '_lrq'},
    ).to_ctypes()
    _rq = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (10*_m_py+7,),
    )
    b_dim3_scat_shep(
        _m,
        _x,
        _y,
        _z,
        _f,
        nw,
        nq,
        _iq,
        _liq,
        _rq,
        _lrq,
    )
    _iq_py = _iq
    _rq_py = _rq
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iq', 'rq'],
    )(
        _iq_py,
        _rq_py,
    )
    return _return_data

def dim3_scat_shep_eval(x, y, z, f, iq, rq, u, v, w):
    u"""
    Interpolated values, evaluate interpolant computed by
    ``dim3_scat_shep``, function and first derivatives, three variables.

    For full information please refer to the NAG Library document for
    e01th

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01thf.html

    Parameters
    ----------
    x : float, array-like, shape (m)
        m, `x`, `y`, `z` and `f` must be the same values as were
        supplied in the preceding call to ``dim3_scat_shep``.

    y : float, array-like, shape (m)
        m, `x`, `y`, `z` and `f` must be the same values as were
        supplied in the preceding call to ``dim3_scat_shep``.

    z : float, array-like, shape (m)
        m, `x`, `y`, `z` and `f` must be the same values as were
        supplied in the preceding call to ``dim3_scat_shep``.

    f : float, array-like, shape (m)
        m, `x`, `y`, `z` and `f` must be the same values as were
        supplied in the preceding call to ``dim3_scat_shep``.

    iq : int, array-like, shape (2*m+1)
        Must be unchanged from the value returned from a previous call
        to ``dim3_scat_shep``.

    rq : float, array-like, shape (10*m+7)
        Must be unchanged from the value returned from a previous call
        to ``dim3_scat_shep``.

    u : float, array-like, shape (n)
        `u`[i-1], `v`[i-1], `w`[i-1] must be set to the evaluation point
        (u_i,v_i,w_i), for i = 1,2,...,n.

    v : float, array-like, shape (n)
        `u`[i-1], `v`[i-1], `w`[i-1] must be set to the evaluation point
        (u_i,v_i,w_i), for i = 1,2,...,n.

    w : float, array-like, shape (n)
        `u`[i-1], `v`[i-1], `w`[i-1] must be set to the evaluation point
        (u_i,v_i,w_i), for i = 1,2,...,n.

    Returns
    -------
    q : float, ndarray, shape (n)
        `q`[i-1] contains the value of the interpolant, at
        (u_i,v_i,w_i), for i = 1,2,...,n. If any of these evaluation
        points lie outside the region of definition of the interpolant
        the corresponding entries in `q` are set to the largest machine
        representable number (see ``machine.real_largest``), and
        ``dim3_scat_shep_eval`` returns with `errno` = 3.

    qx : float, ndarray, shape (n)
        `qx`[i-1], `qy`[i-1], `qz`[i-1] contains the value of the
        partial derivatives of the interpolant Q(x,y,z) at
        (u_i,v_i,w_i), for i = 1,2,...,n. If any of these evaluation
        points lie outside the region of definition of the interpolant,
        the corresponding entries in `qx`, `qy` and `qz` are set to the
        largest machine representable number (see
        ``machine.real_largest``), and ``dim3_scat_shep_eval`` returns
        with `errno` = 3.

    qy : float, ndarray, shape (n)
        `qx`[i-1], `qy`[i-1], `qz`[i-1] contains the value of the
        partial derivatives of the interpolant Q(x,y,z) at
        (u_i,v_i,w_i), for i = 1,2,...,n. If any of these evaluation
        points lie outside the region of definition of the interpolant,
        the corresponding entries in `qx`, `qy` and `qz` are set to the
        largest machine representable number (see
        ``machine.real_largest``), and ``dim3_scat_shep_eval`` returns
        with `errno` = 3.

    qz : float, ndarray, shape (n)
        `qx`[i-1], `qy`[i-1], `qz`[i-1] contains the value of the
        partial derivatives of the interpolant Q(x,y,z) at
        (u_i,v_i,w_i), for i = 1,2,...,n. If any of these evaluation
        points lie outside the region of definition of the interpolant,
        the corresponding entries in `qx`, `qy` and `qz` are set to the
        largest machine representable number (see
        ``machine.real_largest``), and ``dim3_scat_shep_eval`` returns
        with `errno` = 3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lrq is too small: lrq = *<value>*.

        (`errno` 1)
            On entry, liq is too small: liq = *<value>*.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 10.

        (`errno` 2)
            On entry, values in `rq` appear to be invalid. Check that
            `rq` has not been corrupted between calls to
            ``dim3_scat_shep`` and ``dim3_scat_shep_eval``.

        (`errno` 2)
            On entry, values in `iq` appear to be invalid. Check that
            `iq` has not been corrupted between calls to
            ``dim3_scat_shep`` and ``dim3_scat_shep_eval``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, at least one evaluation point lies outside the
            region of definition of the interpolant. At such points the
            corresponding values in `q` and `qx` contain extrapolated
            approximations. Points should be evaluated one by one to
            identify extrapolated values.
    """

    from ..base.interp import dim3_scat_shep_eval as b_dim3_scat_shep_eval

    fun_name = 'naginterfaces.library.interp.dim3_scat_shep_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iq'},
        iq,
        copy=False,
    )
    _liq_py = _b_utils._EngineIntArrayType.get_shape(_iq, {'fun_name': fun_name, 'entity': 'iq'}, exp_rank=1)[0]
    _liq = _b_utils._EngineIntScalarType(
        _liq_py, {'fun_name': fun_name, 'entity': '_liq'},
    ).to_ctypes()
    _rq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rq'},
        rq,
        copy=False,
    )
    _lrq_py = _b_utils._EngineFloat64ArrayType.get_shape(_rq, {'fun_name': fun_name, 'entity': 'rq'}, exp_rank=1)[0]
    _lrq = _b_utils._EngineIntScalarType(
        _lrq_py, {'fun_name': fun_name, 'entity': '_lrq'},
    ).to_ctypes()
    _u = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=False,
    )
    _w = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'w'},
        w,
        copy=False,
    )
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qy = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_dim3_scat_shep_eval(
            _m,
            _x,
            _y,
            _z,
            _f,
            _iq,
            _liq,
            _rq,
            _lrq,
            _n,
            _u,
            _v,
            _w,
            _q,
            _qx,
            _qy,
            _qz,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _q_py = _q
    _qx_py = _qx
    _qy_py = _qy
    _qz_py = _qz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'qx', 'qy', 'qz'],
    )(
        _q_py,
        _qx_py,
        _qy_py,
        _qz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim4_scat_shep(x, f, nw, nq):
    u"""
    Interpolating functions, modified Shepard's method, four variables.

    For full information please refer to the NAG Library document for
    e01tk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01tkf.html

    Parameters
    ----------
    x : float, array-like, shape (4, m)
        `x`[0,r-1],...,`x`[3,r-1] must be set to the Cartesian
        coordinates of the data point x_r, for r = 1,2,...,m.

    f : float, array-like, shape (m)
        `f`[r-1] must be set to the data value f_r, for r = 1,2,...,m.

    nw : int
        The number N_w of data points that determines each radius of
        influence R_w, appearing in the definition of each of the
        weights w_r, for r = 1,2,...,m (see [Description]). Note that
        R_w is different for each weight. If `nw` <= 0 the default value
        `nw` = min(32, m-1) is used instead.

    nq : int
        The number N_q of data points to be used in the least squares
        fit for coefficients defining the quadratic functions q_r(x)
        (see [Description]). If `nq` <= 0 the default value `nq` =
        min(38, m-1) is used instead.

    Returns
    -------
    iq : int, ndarray, shape (2*m+1)
        Integer data defining the interpolant Q(x).

    rq : float, ndarray, shape (15*m+9)
        Real data defining the interpolant Q(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nw` = *<value>* and m = *<value>*.

            Constraint: `nw` <= min(50, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>* and m = *<value>*.

            Constraint: `nq` <= min(50, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>*.

            Constraint: `nq` <= 0 or `nq` >= 14.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 16.

        (`errno` 2)
            There are duplicate nodes in the dataset. `x`[i-1,k-1] =
            `x`[j-1,k-1], for i = *<value>*, j = *<value>* and k =
            1,2,...,4. The interpolant cannot be derived.

        (`errno` 3)
            On entry, all the data points lie on the same
            three-dimensional hypersurface. No unique solution exists.
    """

    from ..base.interp import dim4_scat_shep as b_dim4_scat_shep

    fun_name = 'naginterfaces.library.interp.dim4_scat_shep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*_m_py+1,),
    )
    _rq = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (15*_m_py+9,),
    )
    b_dim4_scat_shep(
        _m,
        _x,
        _f,
        nw,
        nq,
        _iq,
        _rq,
    )
    _iq_py = _iq
    _rq_py = _rq
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iq', 'rq'],
    )(
        _iq_py,
        _rq_py,
    )
    return _return_data

def dim4_scat_shep_eval(x, f, iq, rq, xe):
    u"""
    Interpolated values, evaluate interpolant computed by
    ``dim4_scat_shep``, function and first derivatives, four variables.

    For full information please refer to the NAG Library document for
    e01tl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01tlf.html

    Parameters
    ----------
    x : float, array-like, shape (4, m)
        Must be the same array supplied as argument x in the preceding
        call to ``dim4_scat_shep``. It must remain unchanged between
        calls.

    f : float, array-like, shape (m)
        Must be the same array supplied as argument f in the preceding
        call to ``dim4_scat_shep``. It must remain unchanged between
        calls.

    iq : int, array-like, shape (2*m+1)
        Must be the same array returned as argument iq in the preceding
        call to ``dim4_scat_shep``. It must remain unchanged between
        calls.

    rq : float, array-like, shape (15*m+9)
        Must be the same array returned as argument rq in the preceding
        call to ``dim4_scat_shep``. It must remain unchanged between
        calls.

    xe : float, array-like, shape (4, n)
        `xe`[0,r-1],...,`xe`[3,r-1] must be set to the evaluation point
        x_i, for i = 1,2,...,n.

    Returns
    -------
    q : float, ndarray, shape (n)
        `q`[i-1] contains the value of the interpolant, at x_i, for i =
        1,2,...,n. If any of these evaluation points lie outside the
        region of definition of the interpolant the corresponding
        entries in `q` are set to the largest machine representable
        number (see ``machine.real_largest``), and
        ``dim4_scat_shep_eval`` returns with `errno` = 3.

    qx : float, ndarray, shape (4, n)
        `qx`[j-1,i-1] contains the value of the partial derivatives with
        respect to x_j of the interpolant Q(x) at x_i, for i =
        1,2,...,n, and for each of the four partial derivatives j =
        1,2,3,4. If any of these evaluation points lie outside the
        region of definition of the interpolant, the corresponding
        entries in `qx` are set to the largest machine representable
        number (see ``machine.real_largest``), and
        ``dim4_scat_shep_eval`` returns with `errno` = 3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 16.

        (`errno` 2)
            On entry, values in `rq` appear to be invalid. Check that
            `rq` has not been corrupted between calls to
            ``dim4_scat_shep`` and ``dim4_scat_shep_eval``.

        (`errno` 2)
            On entry, values in `iq` appear to be invalid. Check that
            `iq` has not been corrupted between calls to
            ``dim4_scat_shep`` and ``dim4_scat_shep_eval``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, at least one evaluation point lies outside the
            region of definition of the interpolant. At such points the
            corresponding values in `q` and `qx` contain extrapolated
            approximations. Points should be evaluated one by one to
            identify extrapolated values.
    """

    from ..base.interp import dim4_scat_shep_eval as b_dim4_scat_shep_eval

    fun_name = 'naginterfaces.library.interp.dim4_scat_shep_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _xe = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xe'},
        xe,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_xe, 'xe')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iq'},
        iq,
        copy=False,
    )
    _rq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rq'},
        rq,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_xe, {'fun_name': fun_name, 'entity': 'xe'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (4, _n_py),
        sorder=_m_sorder,
    )
    try:
        b_dim4_scat_shep_eval(
            _m,
            _x,
            _f,
            _iq,
            _rq,
            _n,
            _xe,
            _q,
            _qx,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _q_py = _q
    _qx_py = _qx
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'qx'],
    )(
        _q_py,
        _qx_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim5_scat_shep(x, f, nw, nq):
    u"""
    Interpolating functions, modified Shepard's method, five variables.

    For full information please refer to the NAG Library document for
    e01tm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01tmf.html

    Parameters
    ----------
    x : float, array-like, shape (5, m)
        `x`[0,r-1],...,`x`[4,r-1] must be set to the Cartesian
        coordinates of the data point x_r, for r = 1,2,...,m.

    f : float, array-like, shape (m)
        `f`[r-1] must be set to the data value f_r, for r = 1,2,...,m.

    nw : int
        The number N_w of data points that determines each radius of
        influence R_w, appearing in the definition of each of the
        weights w_r, for r = 1,2,...,m (see [Description]). Note that
        R_w is different for each weight. If `nw` <= 0 the default value
        `nw` = min(32, m-1) is used instead.

    nq : int
        The number N_q of data points to be used in the least squares
        fit for coefficients defining the quadratic functions q_r(x)
        (see [Description]). If `nq` <= 0 the default value `nq` =
        min(50, m-1) is used instead.

    Returns
    -------
    iq : int, ndarray, shape (2*m+1)
        Integer data defining the interpolant Q(x).

    rq : float, ndarray, shape (21*m+11)
        Real data defining the interpolant Q(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nw` = *<value>* and m = *<value>*.

            Constraint: `nw` <= min(50, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>* and m = *<value>*.

            Constraint: `nq` <= min(70, m-1).

        (`errno` 1)
            On entry, `nq` = *<value>*.

            Constraint: `nq` <= 0 or `nq` >= 20.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 23.

        (`errno` 2)
            There are duplicate nodes in the dataset. `x`[i-1,k-1] =
            `x`[j-1,k-1], for i = *<value>*, j = *<value>* and k =
            1,2,...,5. The interpolant cannot be derived.

        (`errno` 3)
            On entry, all the data points lie on the same
            four-dimensional hypersurface. No unique solution exists.
    """

    from ..base.interp import dim5_scat_shep as b_dim5_scat_shep

    fun_name = 'naginterfaces.library.interp.dim5_scat_shep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*_m_py+1,),
    )
    _rq = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (21*_m_py+11,),
    )
    b_dim5_scat_shep(
        _m,
        _x,
        _f,
        nw,
        nq,
        _iq,
        _rq,
    )
    _iq_py = _iq
    _rq_py = _rq
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iq', 'rq'],
    )(
        _iq_py,
        _rq_py,
    )
    return _return_data

def dim5_scat_shep_eval(x, f, iq, rq, xe):
    u"""
    Interpolated values, evaluate interpolant computed by
    ``dim5_scat_shep``, function and first derivatives, five variables.

    For full information please refer to the NAG Library document for
    e01tn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01tnf.html

    Parameters
    ----------
    x : float, array-like, shape (5, m)
        Must be the same array supplied as argument x in the preceding
        call to ``dim5_scat_shep``. It must remain unchanged between
        calls.

    f : float, array-like, shape (m)
        Must be the same array supplied as argument f in the preceding
        call to ``dim5_scat_shep``. It must remain unchanged between
        calls.

    iq : int, array-like, shape (2*m+1)
        Must be the same array returned as argument iq in the preceding
        call to ``dim5_scat_shep``. It must remain unchanged between
        calls.

    rq : float, array-like, shape (21*m+11)
        Must be the same array returned as argument rq in the preceding
        call to ``dim5_scat_shep``. It must remain unchanged between
        calls.

    xe : float, array-like, shape (5, n)
        `xe`[0,i-1],...,`xe`[4,i-1] must be set to the evaluation point
        x_i, for i = 1,2,...,n.

    Returns
    -------
    q : float, ndarray, shape (n)
        `q`[i-1] contains the value of the interpolant, at x_i, for i =
        1,2,...,n. If any of these evaluation points lie outside the
        region of definition of the interpolant the corresponding
        entries in `q` are set to the largest machine representable
        number (see ``machine.real_largest``), and
        ``dim5_scat_shep_eval`` returns with `errno` = 3.

    qx : float, ndarray, shape (5, n)
        `qx`[j-1,i-1] contains the value of the partial derivatives with
        respect to x_j of the interpolant Q(x) at x_i, for i =
        1,2,...,n, and for each of the five partial derivatives j =
        1,2,3,4,5. If any of these evaluation points lie outside the
        region of definition of the interpolant, the corresponding
        entries in `qx` are set to the largest machine representable
        number (see ``machine.real_largest``), and
        ``dim5_scat_shep_eval`` returns with `errno` = 3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 23.

        (`errno` 2)
            On entry, values in `rq` appear to be invalid. Check that
            `rq` has not been corrupted between calls to
            ``dim5_scat_shep`` and ``dim5_scat_shep_eval``.

        (`errno` 2)
            On entry, values in `iq` appear to be invalid. Check that
            `iq` has not been corrupted between calls to
            ``dim5_scat_shep`` and ``dim5_scat_shep_eval``.

        (`errno` 3)
            On entry, at least one evaluation point lies outside the
            region of definition of the interpolant. At such points the
            corresponding values in `q` and `qx` contain extrapolated
            approximations. Points should be evaluated one by one to
            identify extrapolated values.
    """

    from ..base.interp import dim5_scat_shep_eval as b_dim5_scat_shep_eval

    fun_name = 'naginterfaces.library.interp.dim5_scat_shep_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _xe = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xe'},
        xe,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_xe, 'xe')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iq'},
        iq,
        copy=False,
    )
    _rq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rq'},
        rq,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_xe, {'fun_name': fun_name, 'entity': 'xe'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (5, _n_py),
        sorder=_m_sorder,
    )
    b_dim5_scat_shep_eval(
        _m,
        _x,
        _f,
        _iq,
        _rq,
        _n,
        _xe,
        _q,
        _qx,
    )
    _q_py = _q
    _qx_py = _qx
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'qx'],
    )(
        _q_py,
        _qx_py,
    )
    return _return_data

def dimn_grid(narr, uniform, axis, v, point, method, k=1, wf=0.0):
    u"""
    Interpolates an n-dimensional point on grid data using either
    linear, cubic or modified Shepard's method.

    For full information please refer to the NAG Library document for
    e01za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01zaf.html

    Parameters
    ----------
    narr : int, array-like, shape (d)
        The number of data ordinates in each dimension, with `narr`[i-1]
        = n_i, for i = 1,2,...,d.

    uniform : bool
        States whether the data points are uniformly spaced.

        `uniform` = True

          The data points are uniformly spaced.

        `uniform` = False

          The data points are not uniformly spaced.

    axis : float, array-like, shape (lx)
        Defines the axis. If the data points are uniformly spaced (see
        argument `uniform`) `axis` should contain the start and end of
        each dimension (x_{1 1},x_{1 n_1},...,x_{d 1},x_{d n_d}).

        If the data points are not uniformly spaced, `axis` should
        contain all the data ordinates for each dimension
        (x_{1 1},x_{1 2},...,x_{1 n_1},...,x_{d 1},x_{d 2},...,x_{d n_d}).

    v : float, array-like, shape (sum(`narr`))
        Holds the values of the data points in such an order that the
        index of a data value with coordinates (z_1,z_2,...,z_d) is

            sum(z_iprod_{n in S_i}n),

        where S_i = {`narr`[l-1]:l = 1,...,i-1} e.g.,
        ((x_{11},x_{21},...,x_{d1}),(x_{12},x_{21},...,x_{d1}),...,(x_{1n_d},x_{21},...,x_{d1}),(x_{11},x_{22},...,x_{d1}),(x_{12},x_{22},...,x_{d1}),...,(x_{1n_d},x_{2n_d},...,x_{dn_d})).

    point : float, array-like, shape (d)
        x, the point at which the data value is to be interpolated.

    method : int
        The method to be used.

        `method` = 1

          Weighted Average.

        `method` = 2

          Linear Interpolation.

        `method` = 3

          Cubic Interpolation.

    k : int, optional
        If `method` = 1, `k` controls the number of data points used in
        the Weighted Average method, with `k` points used in each
        dimension, either side of the interpolation point. The total
        number of data points used for the interpolation will therefore
        be (2*`k`)^d.

        If `method` != 1, then `k` is not referenced and need not be
        set.

    wf : float, optional
        The power used for the weighted average such that a high power
        will cause closer points to be more heavily weighted.

    Returns
    -------
    ans : float
        Holds the result of the interpolation.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, d = *<value>*.

            Constraint: d >= 2.

        (`errno` 2)
            On entry, `narr`[*<value>*] = *<value>*.

            Constraint: `narr`[i-1] >= 2.

        (`errno` 4)
            On entry, `axis` decreases in dimension *<value>*.

            Constraint: `axis` definition must be strictly increasing.

        (`errno` 5)
            On entry, lx = *<value>*, sum of `narr`= *<value>*.

            Constraint: if `uniform` = False, lx = sum of `narr`.

        (`errno` 5)
            On entry, lx = *<value>*, d = *<value>*.

            Constraint: if `uniform` = True, lx = 2*d.

        (`errno` 7)
            On entry, `k` = *<value>*.

            Constraint: if `method` = 1, `k` >= 1.

        (`errno` 8)
            On entry, `point`[*<value>*] = *<value>* and data range =
            [*<value>*,*<value>*].

            Constraint: `point` must be within the data range.

        (`errno` 9)
            On entry, `method` = 3 and `uniform` = False.

            Constraint: if `method` = 3, `uniform` must be True.

        (`errno` 9)
            On entry, `method` = *<value>*.

            Constraint: `method` = 1, 2 or 3.

        (`errno` 10)
            On entry, `wf` = *<value>*.

            Constraint: if `method` = 1, 1.0 <= `wf` <= 15.0.

        (`errno` 101)
            Cubic Interpolation method does not have enough data
            surrounding `point`; interpolation not possible.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 201)
            Warning: the size of `k` has been reduced, due to too few
            data points around `point`.
    """

    from ..base.interp import dimn_grid as b_dimn_grid

    fun_name = 'naginterfaces.library.interp.dimn_grid'

    _narr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'narr'},
        narr,
        copy=False,
    )
    _d_py = _b_utils._EngineIntArrayType.get_shape(_narr, {'fun_name': fun_name, 'entity': 'narr'}, exp_rank=1)[0]
    _d = _b_utils._EngineIntScalarType(
        _d_py, {'fun_name': fun_name, 'entity': '_d'},
    ).to_ctypes()
    _axis = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'axis'},
        axis,
        copy=False,
    )
    _lx_py = _b_utils._EngineFloat64ArrayType.get_shape(_axis, {'fun_name': fun_name, 'entity': 'axis'}, exp_rank=1)[0]
    _lx = _b_utils._EngineIntScalarType(
        _lx_py, {'fun_name': fun_name, 'entity': '_lx'},
    ).to_ctypes()
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=False,
    )
    _point = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'point'},
        point,
        copy=False,
    )
    try:
        _return_scalar = b_dimn_grid(
            _d,
            _narr,
            uniform,
            _axis,
            _lx,
            _v,
            _point,
            method,
            k,
            wf,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ans_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ans'],
    )(
        _ans_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _ans_py

def dimn_scat_shep(x, f, nw=-1, nq=-1):
    u"""
    Interpolating function, modified Shepard's method, d dimensions.

    For full information please refer to the NAG Library document for
    e01zm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01zmf.html

    Parameters
    ----------
    x : float, array-like, shape (d, m)
        The d components of the first data point must be stored in
        elements 0,1,...,d-1 of `x`. The second data point must be
        stored in elements d,d+1,...,2*d-1 of `x`, and so on. In
        general, the m data points must be stored in `x`[j,i], for j =
        0,1,...,d-1, for i = 0,1,...,m-1.

    f : float, array-like, shape (m)
        `f`[r-1] must be set to the data value f_r, for r = 1,2,...,m.

    nw : int, optional
        The number N_w of data points that determines each radius of
        influence R_w, appearing in the definition of each of the
        weights w_r, for r = 1,2,...,m (see [Description]). Note that
        R_w is different for each weight. If `nw` <= 0 the default value
        `nw` = min(2*(d+1)*(d+2), m-1) is used instead.

    nq : int, optional
        The number N_q of data points to be used in the least squares
        fit for coefficients defining the quadratic functions q_r(x)
        (see [Description]). If `nq` <= 0 the default value `nq` =
        min((d+1)*(d+2)*6/5, m-1) is used instead.

    Returns
    -------
    iq : int, ndarray, shape (2*m+1)
        Integer data defining the interpolant Q(x).

    rq : float, ndarray, shape (lrq)
        Real data defining the interpolant Q(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nw` = *<value>* and m = *<value>*.

            Constraint: `nw` <= m-1.

        (`errno` 1)
            On entry, `nq` = *<value>* and m = *<value>*.

            Constraint: `nq` <= m-1.

        (`errno` 1)
            On entry, `nq` = *<value>* and d = *<value>*.

            Constraint: `nq` <= 0 or `nq` >= (d+1)*(d+2)/2-1.

        (`errno` 1)
            On entry, m = *<value>* and d = *<value>*.

            Constraint: m >= (d+1)*(d+2)/2+2.

        (`errno` 1)
            On entry, ((d+1)*(d+2)/2)*m+2*d+1 exceeds the largest
            machine integer.

            d = *<value>* and m = *<value>*.

        (`errno` 1)
            On entry, d = *<value>*.

            Constraint: d >= 2.

        (`errno` 2)
            There are duplicate nodes in the dataset. `x`[k-1,i-1] =
            `x`[k-1,j-1], for i = *<value>*, j = *<value>* and k =
            1,2,...,d. The interpolant cannot be derived.

        (`errno` 3)
            On entry, all the data points lie on the same hypersurface.
            No unique solution exists.
    """

    from ..base.interp import dimn_scat_shep as b_dimn_scat_shep
    from ..base.machine import integer_max as _b_integer_max

    fun_name = 'naginterfaces.library.interp.dimn_scat_shep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _d_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _d = _b_utils._EngineIntScalarType(
        _d_py, {'fun_name': fun_name, 'entity': '_d'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*_m_py+1,),
    )
    if _d_py <= int(float((sqrt(float(_m_py)/float(2))*sqrt(float(_b_integer_max())/float(2))-1))/float(float(_m_py)/float(2))):
        _vl_lrq_py = ((_d_py+1)*(_d_py+2)//2)*_m_py+2*_d_py+1
    else:
        _vl_lrq_py = 0
    _rq = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_lrq_py,),
    )
    b_dimn_scat_shep(
        _d,
        _m,
        _x,
        _f,
        nw,
        nq,
        _iq,
        _rq,
    )
    _iq_py = _iq
    _rq_py = _rq
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iq', 'rq'],
    )(
        _iq_py,
        _rq_py,
    )
    return _return_data

def dimn_scat_shep_eval(x, f, iq, rq, xe):
    u"""
    Interpolated values, evaluate interpolant computed by
    ``dimn_scat_shep``, function and first derivatives, d dimensions.

    For full information please refer to the NAG Library document for
    e01zn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e01/e01znf.html

    Parameters
    ----------
    x : float, array-like, shape (d, m)
        Must be the same array supplied as argument x in the preceding
        call to ``dimn_scat_shep``. It must remain unchanged between
        calls.

    f : float, array-like, shape (m)
        Must be the same array supplied as argument f in the preceding
        call to ``dimn_scat_shep``. It must remain unchanged between
        calls.

    iq : int, array-like, shape (2*m+1)
        Must be the same array returned as argument iq in the preceding
        call to ``dimn_scat_shep``. It must remain unchanged between
        calls.

    rq : float, array-like, shape (lrq)
        Must be the same array returned as argument rq in the preceding
        call to ``dimn_scat_shep``. It must remain unchanged between
        calls.

    xe : float, array-like, shape (d, n)
        `xe`[0,j-1],...,`xe`[d-1,j-1] must be set to the evaluation
        point x_j, for j = 1,2,...,n.

    Returns
    -------
    q : float, ndarray, shape (n)
        `q`[i-1] contains the value of the interpolant, at x_i, for i =
        1,2,...,n. If any of these evaluation points lie outside the
        region of definition of the interpolant the corresponding
        entries in `q` are set to an extrapolated approximation, and
        ``dimn_scat_shep_eval`` returns with `errno` = 3.

    qx : float, ndarray, shape (d, n)
        `qx`[i-1,j-1] contains the value of the partial derivatives with
        respect to the ith independent variable (dimension) of the
        interpolant Q(x) at x_j, for j = 1,2,...,n, and for each of the
        partial derivatives i = 1,2,...,d. If any of these evaluation
        points lie outside the region of definition of the interpolant,
        the corresponding entries in `qx` are set to extrapolated
        approximations to the partial derivatives, and
        ``dimn_scat_shep_eval`` returns with `errno` = 3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, m = *<value>* and d = *<value>*.

            Constraint: m >= (d+1)*(d+2)/2+2.

        (`errno` 1)
            On entry, ((d+1)*(d+2)/2)*m+2*d+1 exceeds the largest
            machine integer.

            d = *<value>* and m = *<value>*.

        (`errno` 1)
            On entry, d = *<value>*.

            Constraint: d >= 2.

        (`errno` 2)
            On entry, values in `rq` appear to be invalid. Check that
            `rq` has not been corrupted between calls to
            ``dimn_scat_shep`` and ``dimn_scat_shep_eval``.

        (`errno` 2)
            On entry, values in `iq` appear to be invalid. Check that
            `iq` has not been corrupted between calls to
            ``dimn_scat_shep`` and ``dimn_scat_shep_eval``.

        (`errno` 3)
            On entry, at least one evaluation point lies outside the
            region of definition of the interpolant. At such points the
            corresponding values in `q` and `qx` contain extrapolated
            approximations. Points should be evaluated one by one to
            identify extrapolated values.
    """

    from ..base.interp import dimn_scat_shep_eval as b_dimn_scat_shep_eval
    from ..base.machine import integer_max as _b_integer_max

    fun_name = 'naginterfaces.library.interp.dimn_scat_shep_eval'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _xe = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xe'},
        xe,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_xe, 'xe')),
    )
    _d_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _d = _b_utils._EngineIntScalarType(
        _d_py, {'fun_name': fun_name, 'entity': '_d'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    _iq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iq'},
        iq,
        copy=False,
    )
    if _d_py <= int(float((sqrt(float(_m_py)/float(2))*sqrt(float(_b_integer_max())/float(2))-1))/float(float(_m_py)/float(2))):
        _vl_lrq_py = ((_d_py+1)*(_d_py+2)//2)*_m_py+2*_d_py+1
    else:
        _vl_lrq_py = 0
    _rq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rq'},
        rq,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_xe, {'fun_name': fun_name, 'entity': 'xe'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _qx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_d_py, _n_py),
        sorder=_m_sorder,
    )
    b_dimn_scat_shep_eval(
        _d,
        _m,
        _x,
        _f,
        _iq,
        _rq,
        _n,
        _xe,
        _q,
        _qx,
    )
    _q_py = _q
    _qx_py = _qx
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'qx'],
    )(
        _q_py,
        _qx_py,
    )
    return _return_data
