# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `surviv` Chapter.

``surviv`` - Survival Analysis

This module is concerned with statistical techniques used in the
analysis of survival/reliability/failure time data.

Other modules contain functions which are also used to analyse this type
of data. Submodule ``correg`` contains generalized linear models,
submodule ``univar`` contains functions to fit distribution models, and
submodule ``nonpar`` contains rank based methods.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12intro.html

See Also
--------
``naginterfaces.library.examples.surviv`` :
    This subpackage contains examples for the ``surviv`` module.
    See also the :ref:`library_surviv_ex` subsection.
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import machine
from ..library import _utils

def kaplanmeier(t, ic, freq, ifreq):
    u"""
    Computes Kaplan--Meier (product-limit) estimates of survival
    probabilities.

    For full information please refer to the NAG Library document for
    g12aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12aaf.html

    Parameters
    ----------
    t : float, array-like, shape (n)
        The failure and censored times; these need not be ordered.

    ic : int, array-like, shape (n)
        `ic`[i-1] contains the censoring code of the ith observation,
        for i = 1,2,...,n.

        `ic`[i-1] = 0

          The ith observation is a failure time.

        `ic`[i-1] = 1

          The ith observation is right-censored.

    freq : str, length 1
        Indicates whether frequencies are provided for each time point.

        `freq` = 'F'

          Frequencies are provided for each failure and censored time.

        `freq` = 'S'

          The failure and censored times are considered as single
          observations, i.e., a frequency of 1 is assumed.

    ifreq : int, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `freq`='F': n; if `freq`='S': 1; otherwise: 0.

        If `freq` = 'F', `ifreq`[i] must contain the frequency of the
        ith observation.

        If `ifreq` = 'S', a frequency of 1 is assumed and `ifreq` is not
        referenced.

    Returns
    -------
    nd : int
        The number of distinct failure times, n_d.

    tp : float, ndarray, shape (n)
        `tp`[i-1] contains the ith ordered distinct failure time, t_i,
        for i = 1,2,...,n_d.

    p : float, ndarray, shape (n)
        `p`[i-1] contains the Kaplan--Meier estimate of the survival
        probability, S^(t), for time `tp`[i-1], for i = 1,2,...,n_d.

    psig : float, ndarray, shape (n)
        `psig`[i-1] contains an estimate of the standard deviation of
        `p`[i-1], for i = 1,2,...,n_d.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `freq` = *<value>*.

            Constraint: `freq` = 'F' or 'S'.

        (`errno` 3)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 4)
            On entry, i = *<value>* and `ifreq`[i-1] = *<value>*.

            Constraint: `ifreq`[i-1] >= 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.surviv import kaplanmeier as b_kaplanmeier

    fun_name = 'naginterfaces.library.surviv.kaplanmeier'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ic = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ic'},
        ic,
        copy=False,
    )
    _ifreq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ifreq'},
        ifreq,
        copy=False,
    )
    _tp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _psig = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalar = b_kaplanmeier(
        _n,
        _t,
        _ic,
        freq,
        _ifreq,
        _tp,
        _p,
        _psig,
    )
    _nd_py = _return_scalar
    _tp_py = _tp
    _p_py = _p
    _psig_py = _psig
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nd', 'tp', 'p', 'psig'],
    )(
        _nd_py,
        _tp_py,
        _p_py,
        _psig_py,
    )
    return _return_data

def logrank(t, ic, grp, ngrp, ifreq=None, wt=None, ldn=None):
    u"""
    Computes rank statistics for comparing survival curves.

    For full information please refer to the NAG Library document for
    g12ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12abf.html

    Parameters
    ----------
    t : float, array-like, shape (n)
        The observed failure and censored times; these need not be
        ordered.

    ic : int, array-like, shape (n)
        `ic`[i-1] contains the censoring code of the ith observation,
        for i = 1,2,...,n.

        `ic`[i-1] = 0

          the ith observation is a failure time.

        `ic`[i-1] = 1

          the ith observation is right-censored.

    grp : int, array-like, shape (n)
        `grp`[i-1] contains a flag indicating which group the ith
        observation belongs in, for i = 1,2,...,n.

    ngrp : int
        g, the number of groups.

    ifreq : None or int, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `ifreq` is not None: n; otherwise: 0.

        Optionally, the frequency (number of observations) that each
        entry in `t` corresponds to. If `ifreq` is None then each entry
        in `t` is assumed to correspond to a single observation, i.e., a
        frequency of 1 is assumed.

    wt : None or float, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `wt` is not None: `ldn`; otherwise: 0.

        Optionally, the n_d weights, w_i, where n_d is the number of
        distinct failure times. If `wt` is None then w_i = 1 for all i.

    ldn : int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: n.

        The size of arrays `di` and `ni`. As n_d <= n, if n_d is not
        known a priori then a value of n can safely be used for `ldn`.

    Returns
    -------
    ts : float
        T, the test statistic.

    df : int
        nu, the degrees of freedom.

    p : float
        P(X >= T), when X~chi_nu^2, i.e., the probability associated
        with `ts`.

    obsd : float, ndarray, shape (`ngrp`)
        O_i, the observed number of failures in each group.

    expt : float, ndarray, shape (`ngrp`)
        E_i, the expected number of failures in each group.

    nd : int
        n_d, the number of distinct failure times.

    di : int, ndarray, shape (`ldn`)
        The first `nd` elements of `di` contain d_i, the number of
        failures, across all groups, at time t_i.

    ni : int, ndarray, shape (`ldn`)
        The first `nd` elements of `ni` contain n_i, the size of the
        risk set, across all groups, at time t_i.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, all the times in `t` are the same.

        (`errno` 3)
            On entry, `ic`[*<value>*] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 4)
            On entry, `grp`[*<value>*] = *<value>* and `ngrp` =
            *<value>*.

            Constraint: 1 <= `grp`[i-1] <= `ngrp`.

        (`errno` 5)
            On entry, `ngrp` = *<value>* and n = *<value>*.

            Constraint: 2 <= `ngrp` <= n.

        (`errno` 6)
            On entry, lfreq = *<value>* and n = *<value>*.

            Constraint: lfreq = 0 or lfreq >= n.

        (`errno` 7)
            On entry, `ifreq`[*<value>*] = *<value>*.

            Constraint: `ifreq`[i-1] >= 0.

        (`errno` 8)
            On entry, lwt = *<value>* and `ldn` = *<value>*.

            Constraint: lwt = 0 or lwt >= `ldn`.

        (`errno` 9)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 11)
            The degrees of freedom are zero.

        (`errno` 18)
            On entry, `ldn` = *<value>*.

            Constraint: `ldn` >= *<value>*.

        (`errno` 31)
            On entry, all observations are censored.

        (`errno` 41)
            On entry, group *<value>* has no observations.
    """

    from ..base.surviv import logrank as b_logrank

    fun_name = 'naginterfaces.library.surviv.logrank'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ic = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ic'},
        ic,
        copy=False,
    )
    _grp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'grp'},
        grp,
        copy=False,
    )
    _ifreq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ifreq'},
        ifreq,
        copy=False,
        allow_none=True,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _ngrp = _b_utils._EngineIntScalarType(
        ngrp, {'fun_name': fun_name, 'entity': 'ngrp'},
    ).to_ctypes()
    _ngrp_py = _b_utils._EngineIntScalarType.to_py(_ngrp)
    _obsd = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ngrp_py,),
    )
    _expt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ngrp_py,),
    )
    _ldn = _n_py if ldn is None else ldn
    _ldn = _b_utils._EngineIntScalarType(
        _ldn, {'fun_name': fun_name, 'entity': '_ldn'},
    ).to_ctypes()
    _ldn_py = _b_utils._EngineIntScalarType.to_py(_ldn)
    _di = _b_utils._EngineIntArrayType.empty_ndarray(
        (_ldn_py,),
    )
    _ni = _b_utils._EngineIntArrayType.empty_ndarray(
        (_ldn_py,),
    )
    _return_scalars = b_logrank(
        _n,
        _t,
        _ic,
        _grp,
        _ngrp,
        _ifreq,
        _wt,
        _obsd,
        _expt,
        _di,
        _ni,
        _ldn,
    )
    _ts_py = _return_scalars.ts
    _df_py = _return_scalars.df
    _p_py = _return_scalars.p
    _obsd_py = _obsd
    _expt_py = _expt
    _nd_py = _return_scalars.nd
    _di_py = _di
    _ni_py = _ni
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ts', 'df', 'p', 'obsd', 'expt', 'nd', 'di', 'ni'],
    )(
        _ts_py,
        _df_py,
        _p_py,
        _obsd_py,
        _expt_py,
        _nd_py,
        _di_py,
        _ni_py,
    )
    return _return_data

def coxmodel(ns, z, isz, t, ic, isi, b, ndmax, omega=None, tol=100*machine.precision(), maxit=1000, iprint=0, io_manager=None):
    u"""
    Fits Cox's proportional hazard model.

    For full information please refer to the NAG Library document for
    g12ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12baf.html

    Parameters
    ----------
    ns : int
        The number of strata. If `ns` > 0 then the stratum for each
        observation must be supplied in `isi`.

    z : float, array-like, shape (n, m)
        The ith row must contain the covariates which are associated
        with the ith failure time given in `t`.

    isz : int, array-like, shape (m)
        Indicates which subset of covariates is to be included in the
        model.

        `isz`[j-1] >= 1

          The jth covariate is included in the model.

        `isz`[j-1] = 0

          The jth covariate is excluded from the model and not
          referenced.

    t : float, array-like, shape (n)
        The vector of n failure censoring times.

    ic : int, array-like, shape (n)
        The status of the individual at time t given in `t`.

        `ic`[i-1] = 0

          The ith individual has failed at time `t`[i-1].

        `ic`[i-1] = 1

          The ith individual has been censored at time `t`[i-1].

    isi : int, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ns`>0: n; otherwise: 1.

        If `ns` > 0, the stratum indicators which also allow data points
        to be excluded from the analysis.

        If `ns` = 0, `isi` is not referenced.

        `isi`[i-1] = k

          The ith data point is in the kth stratum, where k =
          1,2,...,`ns`.

        `isi`[i-1] = 0

          The ith data point is omitted from the analysis.

    b : float, array-like, shape (ip)
        Initial estimates of the covariate coefficient parameters beta.
        `b`[j-1] must contain the initial estimate of the coefficient of
        the covariate in `z` corresponding to the jth nonzero value of
        `isz`.

    ndmax : int
        The dimension of the array `tp`.

        The first dimension of the array `sur`.

    omega : None or float, array-like, shape (n), optional
        If `omega` is not None, the offset, omega_i, for i = 1,2,...,n.
        Otherwise `omega` is not referenced.

    tol : float, optional
        Indicates the accuracy required for the estimation. Convergence
        is assumed when the decrease in deviance is less than
        `tol`*(1.0+CurrentDeviance). This corresponds approximately to
        an absolute precision if the deviance is small and a relative
        precision if the deviance is large.

    maxit : int, optional
        The maximum number of iterations to be used for computing the
        estimates. If `maxit` is set to 0 then the standard errors,
        score functions, variance-covariance matrix and the survival
        function are computed for the input value of beta in `b` but
        beta is not updated.

    iprint : int, optional
        Indicates if the printing of information on the iterations is
        required.

        `iprint` <= 0

          No printing.

        `iprint` >= 1

          The deviance and the current estimates are printed every
          `iprint` iterations. When printing occurs the output is
          directed to the file object associated with the advisory I/O
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    dev : float
        The deviance, that is -2*(maximized log marginal likelihood).

    b : float, ndarray, shape (ip)
        `b`[j-1] contains the estimate beta^_i, the coefficient of the
        covariate stored in the ith column of `z` where i is the jth
        nonzero value in the array `isz`.

    se : float, ndarray, shape (ip)
        `se`[j-1] is the asymptotic standard error of the estimate
        contained in `b`[j-1] and score function in `sc`[j-1], for j =
        1,2,...,ip.

    sc : float, ndarray, shape (ip)
        `sc`[j-1] is the value of the score function, U_j(beta), for the
        estimate contained in `b`[j-1].

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The variance-covariance matrix of the parameter estimates in `b`
        stored in packed form by column, i.e., the covariance between
        the parameter estimates given in `b`[i-1] and `b`[j-1], j >= i,
        is stored in `cov`[j(j-1)/2+i-1].

    res : float, ndarray, shape (n)
        The residuals, r(t_l), for l = 1,2,...,n.

    nd : int
        The number of distinct failure times.

    tp : float, ndarray, shape (`ndmax`)
        `tp`[i-1] contains the ith distinct failure time, for i =
        1,2,...,`nd`.

    sur : float, ndarray, shape (`ndmax`, max(`ns`, 1))
        If `ns` = 0, `sur`[i-1,0] contains the estimated survival
        function for the ith distinct failure time.

        If `ns` > 0, `sur`[i-1,k-1] contains the estimated survival
        function for the ith distinct failure time in the kth stratum.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 10*machine precision.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 0.

        (`errno` 1)
            On entry, `ns` = *<value>*.

            Constraint: `ns` >= 0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, there are not ip values of `isz` > 0.

        (`errno` 2)
            On entry, i = *<value>* and `isz`[i-1] = *<value>*.

            Constraint: `isz`[i-1] >= 0.

        (`errno` 2)
            On entry too few observations included in model.

        (`errno` 2)
            On entry, i = *<value>*, `isi`[i-1] = *<value>* and `ns` =
            *<value>*.

            Constraint: 0 <= `isi`[i-1] <= `ns`.

        (`errno` 2)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 2)
            On entry, `ndmax` = *<value>* and minimum value for `ndmax`
            = *<value>*.

            Constraint: `ndmax` >= number of distinct failure times.

        (`errno` 2)
            All observations are censored.

        (`errno` 3)
            The matrix of second partial derivative is singular.

        (`errno` 4)
            Overflow has been detected in the calculations.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            Convergence not achieved in *<value>* iterations.

        (`errno` 6)
            Too many step halvings required.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.surviv.coxmodel_ex.main`
    """

    from ..base.surviv import coxmodel as b_coxmodel

    fun_name = 'naginterfaces.library.surviv.coxmodel'

    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_z, 'z'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isz = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isz'},
        isz,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _ic = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ic'},
        ic,
        copy=False,
    )
    _omega = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'omega'},
        omega,
        copy=False,
        allow_none=True,
    )
    _isi = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isi'},
        isi,
        copy=False,
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _sc = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py*(_ip_py+1)//2,),
    )
    _res = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ndmax = _b_utils._EngineIntScalarType(
        ndmax, {'fun_name': fun_name, 'entity': 'ndmax'},
    ).to_ctypes()
    _ndmax_py = _b_utils._EngineIntScalarType.to_py(_ndmax)
    _tp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ndmax_py,),
    )
    _ns = _b_utils._EngineIntScalarType(
        ns, {'fun_name': fun_name, 'entity': 'ns'},
    ).to_ctypes()
    _ns_py = _b_utils._EngineIntScalarType.to_py(_ns)
    _sur = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ndmax_py, max(_ns_py, 1)),
        sorder=_m_sorder,
    )
    try:
        _return_scalars = b_coxmodel(
            _n,
            _m,
            _ns,
            _z,
            _isz,
            _ip,
            _t,
            _ic,
            _omega,
            _isi,
            _b,
            _se,
            _sc,
            _cov,
            _res,
            _tp,
            _sur,
            _ndmax,
            tol,
            maxit,
            iprint,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _b_py = _b
    _dev_py = _return_scalars.dev
    _se_py = _se
    _sc_py = _sc
    _cov_py = _cov
    _res_py = _res
    _nd_py = _return_scalars.nd
    _tp_py = _tp
    _sur_py = _sur
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dev', 'b', 'se', 'sc', 'cov', 'res', 'nd', 'tp', 'sur'],
    )(
        _dev_py,
        _b_py,
        _se_py,
        _sc_py,
        _cov_py,
        _res_py,
        _nd_py,
        _tp_py,
        _sur_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def coxmodel_risksets(ns, z, isz, ip, t, ic, isi, mxn):
    u"""
    Creates the risk sets associated with the Cox proportional hazards
    model for fixed covariates.

    For full information please refer to the NAG Library document for
    g12za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12zaf.html

    Parameters
    ----------
    ns : int
        The number of strata. If `ns` > 0 then the stratum for each
        observation must be supplied in `isi`.

    z : float, array-like, shape (n, m)
        The ith row must contain the covariates which are associated
        with the ith failure time given in `t`.

    isz : int, array-like, shape (m)
        Indicates which subset of covariates are to be included in the
        model.

        `isz`[j-1] >= 1

          The jth covariate is included in the model.

        `isz`[j-1] = 0

          The jth covariate is excluded from the model and not
          referenced.

    ip : int
        p, the number of covariates included in the model as indicated
        by `isz`.

    t : float, array-like, shape (n)
        The vector of n failure censoring times.

    ic : int, array-like, shape (n)
        The status of the individual at time t given in `t`.

        `ic`[i-1] = 0

          Indicates that the ith individual has failed at time `t`[i-1].

        `ic`[i-1] = 1

          Indicates that the ith individual has been censored at time
          `t`[i-1].

    isi : int, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ns`>0: n; otherwise: 1.

        If `ns` > 0, the stratum indicators which also allow data points
        to be excluded from the analysis.

        If `ns` = 0, `isi` is not referenced.

        `isi`[i] = k

          Indicates that the ith data point is in the kth stratum, where
          k = 1,2,...,`ns`.

        `isi`[i] = 0

          Indicates that the ith data point is omitted from the
          analysis.

    mxn : int
        The first dimension of the array `x`.

        The dimension of the arrays `ixs` and `fid`.

    Returns
    -------
    num : int
        The number of values in the combined risk sets.

    ixs : int, ndarray, shape (`mxn`)
        The factor giving the risk sets/strata for the data in `x` and
        `fid`.

        If `ns` = 0 or 1, `ixs`[i-1] = l for members of the lth risk
        set.

        If `ns` > 1, `ixs`[i-1] = (j-1)*`nd`+l for the observations in
        the lth risk set for the jth strata.

    nxs : int
        The number of levels for the risk sets/strata factor given in
        `ixs`.

    x : float, ndarray, shape (`num`, `ip`)
        The first `num` rows contain the values of the covariates for
        the members of the risk sets.

    fid : int, ndarray, shape (`mxn`)
        Indicates if the member of the risk set given in `x` failed.

        `fid`[i-1] = 1 if the member of the risk set failed at the time
        defining the risk set and `fid`[i-1] = 0 otherwise.

    nd : int
        The number of distinct failure times, i.e., the number of risk
        sets.

    tp : float, ndarray, shape (n)
        `tp`[i-1] contains the ith distinct failure time, for i =
        1,2,...,`nd`.

    irs : int, ndarray, shape (n)
        Indicates rows in `x` and elements in `ixs` and `fid`
        corresponding to the risk sets. The first risk set corresponding
        to failure time `tp`[0] is given by rows 1 to `irs`[0]. The lth
        risk set is given by rows `irs`[l-2]+1 to `irs`[l-1], for l =
        1,2,...,`nd`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ns` = *<value>*.

            Constraint: `ns` >= 0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, i = *<value>*, `isi`[i-1] = *<value>* and `ns` =
            *<value>*.

            Constraint: 0 <= `isi`[i-1] <= `ns`.

        (`errno` 2)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 2)
            On entry, there are not `ip` values of `isz` > 0.

        (`errno` 2)
            On entry, i = *<value>* and `isz`[i-1] = *<value>*.

            Constraint: `isz`[j-1] >= 0.

        (`errno` 3)
            On entry, `mxn` = *<value>* and minimum value for `mxn` =
            *<value>*.

            Constraint: `mxn` must be sufficiently large for the arrays
            to contain the expanded risk set.
    """

    from ..base.surviv import coxmodel_risksets as b_coxmodel_risksets

    fun_name = 'naginterfaces.library.surviv.coxmodel_risksets'

    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_z, 'z'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isz = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isz'},
        isz,
        copy=False,
    )
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _ic = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ic'},
        ic,
        copy=False,
    )
    _isi = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isi'},
        isi,
        copy=False,
    )
    _mxn = _b_utils._EngineIntScalarType(
        mxn, {'fun_name': fun_name, 'entity': 'mxn'},
    ).to_ctypes()
    _mxn_py = _b_utils._EngineIntScalarType.to_py(_mxn)
    _ixs = _b_utils._EngineIntArrayType.empty_ndarray(
        (_mxn_py,),
    )
    _ip = _b_utils._EngineIntScalarType(
        ip, {'fun_name': fun_name, 'entity': 'ip'},
    ).to_ctypes()
    _ip_py = _b_utils._EngineIntScalarType.to_py(_ip)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_mxn_py, _ip_py),
        sorder=_m_sorder,
    )
    _fid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_mxn_py,),
    )
    _tp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _irs = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_coxmodel_risksets(
        _n,
        _m,
        ns,
        _z,
        _isz,
        _ip,
        _t,
        _ic,
        _isi,
        _ixs,
        _x,
        _mxn,
        _fid,
        _tp,
        _irs,
    )
    _num_py = _return_scalars.num
    _ixs_py = _ixs
    _nxs_py = _return_scalars.nxs
    _x_py = _x[:_num_py, :]
    _fid_py = _fid
    _nd_py = _return_scalars.nd
    _tp_py = _tp
    _irs_py = _irs
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['num', 'ixs', 'nxs', 'x', 'fid', 'nd', 'tp', 'irs'],
    )(
        _num_py,
        _ixs_py,
        _nxs_py,
        _x_py,
        _fid_py,
        _nd_py,
        _tp_py,
        _irs_py,
    )
    return _return_data
