# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `univar` Chapter.

``univar`` - Univariate Estimation

This module deals with the estimation of unknown parameters of a
univariate distribution.
It includes both point and interval estimation using maximum likelihood
and robust methods.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07intro.html

See Also
--------
``naginterfaces.library.examples.univar`` :
    This subpackage contains examples for the ``univar`` module.
    See also the :ref:`library_univar_ex` subsection.
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def ci_binomial(n, k, clevel):
    u"""
    Computes confidence interval for the parameter of a binomial
    distribution.

    For full information please refer to the NAG Library document for
    g07aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07aaf.html

    Parameters
    ----------
    n : int
        n, the number of trials.

    k : int
        k, the number of successes.

    clevel : float
        The confidence level, (1-alpha), for two-sided interval
        estimate. For example `clevel` = 0.95 will give a 95% confidence
        interval.

    Returns
    -------
    pl : float
        The lower limit, p_l, of the confidence interval.

    pu : float
        The upper limit, p_u, of the confidence interval.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `clevel` = *<value>*.

            Constraint: 0.0 < `clevel` < 1.0.

        (`errno` 1)
            On entry, `n` = *<value>* and `k` = *<value>*.

            Constraint: `n` >= `k`.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            When using the relationship with the gamma distribution the
            series to calculate the gamma probabilities has failed to
            converge.
    """

    from ..base.univar import ci_binomial as b_ci_binomial

    _return_scalars = b_ci_binomial(
        n,
        k,
        clevel,
    )
    _pl_py = _return_scalars.pl
    _pu_py = _return_scalars.pu
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['pl', 'pu'],
    )(
        _pl_py,
        _pu_py,
    )
    return _return_data

def ci_poisson(n, xmean, clevel):
    u"""
    Computes confidence interval for the parameter of a Poisson
    distribution.

    For full information please refer to the NAG Library document for
    g07ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07abf.html

    Parameters
    ----------
    n : int
        n, the sample size.

    xmean : float
        The sample mean, x_.

    clevel : float
        The confidence level, (1-alpha), for two-sided interval
        estimate. For example `clevel` = 0.95 gives a 95% confidence
        interval.

    Returns
    -------
    tl : float
        The lower limit, theta_l, of the confidence interval.

    tu : float
        The upper limit, theta_u, of the confidence interval.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `clevel` = *<value>*.

            Constraint: 0.0 < `clevel` < 1.0.

        (`errno` 1)
            On entry, `xmean` = *<value>*.

            Constraint: `xmean` >= 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            When using the relationship with the gamma distribution the
            series to calculate the gamma probabilities has failed to
            converge.
    """

    from ..base.univar import ci_poisson as b_ci_poisson

    _return_scalars = b_ci_poisson(
        n,
        xmean,
        clevel,
    )
    _tl_py = _return_scalars.tl
    _tu_py = _return_scalars.tu
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['tl', 'tu'],
    )(
        _tl_py,
        _tu_py,
    )
    return _return_data

def estim_normal(method, x, xc, ic, xmu, xsig, tol, maxit):
    u"""
    Computes maximum likelihood estimates for parameters of the Normal
    distribution from grouped and/or censored data.

    For full information please refer to the NAG Library document for
    g07bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07bbf.html

    Parameters
    ----------
    method : str, length 1
        Indicates whether the Newton--Raphson or EM algorithm should be
        used.

        If `method` = 'N', the Newton--Raphson algorithm is used.

        If `method` = 'E', the EM algorithm is used.

    x : float, array-like, shape (n)
        The observations x_i, L_i or U_i, for i = 1,2,...,n.

        If the observation is exactly specified -- the exact value, x_i.

        If the observation is right-censored -- the lower value, L_i.

        If the observation is left-censored -- the upper value, U_i.

        If the observation is interval-censored -- the lower or upper
        value, L_i or U_i, (see `xc`).

    xc : float, array-like, shape (n)
        If the jth observation, for j = 1,2,...,n is an
        interval-censored observation then `xc`[j-1] should contain the
        complementary value to `x`[j-1], that is, if `x`[j-1] <
        `xc`[j-1], then `xc`[j-1] contains upper value, U_i, and if
        `x`[j-1] > `xc`[j-1], then `xc`[j-1] contains lower value, L_i.
        Otherwise if the jth observation is exact or right - or
        left-censored `xc`[j-1] need not be set.

    ic : int, array-like, shape (n)
        `ic`[i-1] contains the censoring codes for the ith observation,
        for i = 1,2,...,n.

        If `ic`[i-1] = 0, the observation is exactly specified.

        If `ic`[i-1] = 1, the observation is right-censored.

        If `ic`[i-1] = 2, the observation is left-censored.

        If `ic`[i-1] = 3, the observation is interval-censored.

    xmu : float
        If `xsig` > 0.0 the initial estimate of the mean, mu; otherwise
        `xmu` need not be set.

    xsig : float
        Specifies whether an initial estimate of mu and sigma are to be
        supplied.

        `xsig` > 0.0

          `xsig` is the initial estimate of sigma and `xmu` must contain
          an initial estimate of mu.

        `xsig` <= 0.0

          Initial estimates of `xmu` and `xsig` are calculated
          internally from:

          - the exact observations, if the number of exactly specified
            observations is >= 2; or

          - the interval-censored observations; if the number of
            interval-censored observations is >= 1; or

          - they are set to 0.0 and 1.0 respectively.

    tol : float
        The relative precision required for the final estimates of mu
        and sigma. Convergence is assumed when the absolute relative
        changes in the estimates of both mu and sigma are less than
        `tol`.

        If `tol` = 0.0, a relative precision of 0.000005 is used.

    maxit : int
        The maximum number of iterations.

        If `maxit` <= 0, a value of 25 is used.

    Returns
    -------
    xmu : float
        The maximum likelihood estimate, mu^, of mu.

    xsig : float
        The maximum likelihood estimate, sigma^, of sigma.

    sexmu : float
        The estimate of the standard error of mu^.

    sexsig : float
        The estimate of the standard error of sigma^.

    corr : float
        The estimate of the correlation between mu^ and sigma^.

    dev : float
        The maximized log-likelihood, L(mu^,sigma^).

    nobs : int, ndarray, shape (4)
        The number of the different types of each observation;

        `nobs`[0] contains number of right-censored observations.

        `nobs`[1] contains number of left-censored observations.

        `nobs`[2] contains number of interval-censored observations.

        `nobs`[3] contains number of exactly specified observations.

    nit : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'N' or 'E'.

        (`errno` 1)
            On entry, effective number of observations < 2.

        (`errno` 1)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0, 1, 2 or 3.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: machine precision < `tol` <= 1.0 or `tol` = 0.0.

        (`errno` 2)
            The chosen method has not converged in *<value>* iterations.

        (`errno` 3)
            The process has diverged.

        (`errno` 3)
            The EM process has failed.

        (`errno` 4)
            Standard errors cannot be computed.
    """

    from ..base.univar import estim_normal as b_estim_normal

    fun_name = 'naginterfaces.library.univar.estim_normal'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _xc = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xc'},
        xc,
        copy=False,
    )
    _ic = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ic'},
        ic,
        copy=False,
    )
    _nobs = _b_utils._EngineIntArrayType.empty_ndarray(
        (4,),
    )
    _return_scalars = b_estim_normal(
        method,
        _n,
        _x,
        _xc,
        _ic,
        xmu,
        xsig,
        tol,
        maxit,
        _nobs,
    )
    _xmu_py = _return_scalars.xmu
    _xsig_py = _return_scalars.xsig
    _sexmu_py = _return_scalars.sexmu
    _sexsig_py = _return_scalars.sexsig
    _corr_py = _return_scalars.corr
    _dev_py = _return_scalars.dev
    _nobs_py = _nobs
    _nit_py = _return_scalars.nit
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xmu', 'xsig', 'sexmu', 'sexsig', 'corr', 'dev', 'nobs', 'nit'],
    )(
        _xmu_py,
        _xsig_py,
        _sexmu_py,
        _sexsig_py,
        _corr_py,
        _dev_py,
        _nobs_py,
        _nit_py,
    )
    return _return_data

def estim_weibull(cens, x, ic, gamma, tol, maxit):
    u"""
    Computes maximum likelihood estimates for parameters of the Weibull
    distribution.

    For full information please refer to the NAG Library document for
    g07be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07bef.html

    Parameters
    ----------
    cens : str, length 1
        Indicates whether the data is censored or non-censored.

        `cens` = 'N'

          Each observation is assumed to be exactly specified. `ic` is
          not referenced.

        `cens` = 'C'

          Each observation is censored according to the value contained
          in `ic`[i-1], for i = 1,2,...,n.

    x : float, array-like, shape (n)
        `x`[i-1] contains the ith observation, x_i, for i = 1,2,...,n.

    ic : int, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `cens`='C': n; otherwise: 1.

        If `cens` = 'C', `ic`[i-1] contains the censoring codes for the
        ith observation, for i = 1,2,...,n.

        If `ic`[i-1] = 0, the ith observation is exactly specified.

        If `ic`[i-1] = 1, the ith observation is right-censored.

        If `cens` = 'N', `ic` is not referenced.

    gamma : float
        Indicates whether an initial estimate of gamma is provided.

        If `gamma` > 0.0, it is taken as the initial estimate of gamma
        and an initial estimate of beta is calculated from this value of
        gamma.

        If `gamma` <= 0.0, initial estimates of gamma and beta are
        calculated, internally, providing the data contains at least two
        distinct exact observations. (If there are only two distinct
        exact observations, the largest observation must not be exactly
        specified.) See [Further Comments] for further details.

    tol : float
        The relative precision required for the final estimates of beta
        and gamma. Convergence is assumed when the absolute relative
        changes in the estimates of both beta and gamma are less than
        `tol`.

        If `tol` = 0.0, a relative precision of 0.000005 is used.

    maxit : int
        The maximum number of iterations allowed.

        If `maxit` <= 0, a value of 25 is used.

    Returns
    -------
    beta : float
        The maximum likelihood estimate, beta^, of beta.

    gamma : float
        Contains the maximum likelihood estimate, gamma^, of gamma.

    sebeta : float
        An estimate of the standard error of beta^.

    segam : float
        An estimate of the standard error of gamma^.

    corr : float
        An estimate of the correlation between beta^ and gamma^.

    dev : float
        The maximized kernel log-likelihood, L(beta^,gamma^).

    nit : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: machine precision < `tol` <= 1.0 or `tol` = 0.0.

        (`errno` 1)
            On entry, `cens` = *<value>*.

            Constraint: `cens` = 'N' or 'C'.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 2)
            On entry, i = *<value>* and `x`[i-1] = *<value>*.

            Constraint: `x`[i-1] > 0.0.

        (`errno` 3)
            Unable to calculate initial values.

        (`errno` 3)
            On entry, there are no exactly specified observations.

        (`errno` 4)
            The chosen method has not converged in *<value>* iterations.

        (`errno` 5)
            The process has diverged.

        (`errno` 5)
            Hessian matrix of the Newton--Raphson process is singular.

        (`errno` 6)
            Potential overflow detected.

    See Also
    --------
    :meth:`naginterfaces.library.examples.univar.estim_weibull_ex.main`
    """

    from ..base.univar import estim_weibull as b_estim_weibull

    fun_name = 'naginterfaces.library.univar.estim_weibull'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ic = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ic'},
        ic,
        copy=False,
    )
    _return_scalars = b_estim_weibull(
        cens,
        _n,
        _x,
        _ic,
        gamma,
        tol,
        maxit,
    )
    _gamma_py = _return_scalars.gamma
    _beta_py = _return_scalars.beta
    _sebeta_py = _return_scalars.sebeta
    _segam_py = _return_scalars.segam
    _corr_py = _return_scalars.corr
    _dev_py = _return_scalars.dev
    _nit_py = _return_scalars.nit
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['beta', 'gamma', 'sebeta', 'segam', 'corr', 'dev', 'nit'],
    )(
        _beta_py,
        _gamma_py,
        _sebeta_py,
        _segam_py,
        _corr_py,
        _dev_py,
        _nit_py,
    )
    return _return_data

def estim_genpareto(y, optopt):
    u"""
    Estimates parameter values of the generalized Pareto distribution.

    For full information please refer to the NAG Library document for
    g07bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07bff.html

    Parameters
    ----------
    y : float, array-like, shape (n)
        The n observations y_i, for i = 1,2,...,n, assumed to follow a
        generalized Pareto distribution.

    optopt : int
        Determines the method of estimation, set:

        `optopt` = -2

          For the method of probability-weighted moments.

        `optopt` = -1

          For the method of moments.

        `optopt` = 1

          For maximum likelihood with starting values given by the
          method of moments estimates.

        `optopt` = 2

          For maximum likelihood with starting values given by the
          method of probability-weighted moments.

    Returns
    -------
    xi : float
        The parameter estimate xi^.

    beta : float
        The parameter estimate beta^.

    asvc : float, ndarray, shape (4)
        The variance-covariance of the asymptotic Normal distribution of
        xi^ and beta^. `asvc`[0] contains the variance of xi^; `asvc`[3]
        contains the variance of beta^; `asvc`[1] and `asvc`[2] contain
        the covariance of xi^ and beta^.

    obsvc : float, ndarray, shape (4)
        If maximum likelihood estimates are requested, the observed
        variance-covariance of xi^ and beta^. `obsvc`[0] contains the
        variance of xi^; `obsvc`[3] contains the variance of beta^;
        `obsvc`[1] and `obsvc`[2] contain the covariance of xi^ and
        beta^.

    ll : float
        If maximum likelihood estimates are requested, this argument
        contains the log-likelihood value L at the end of the
        optimization; otherwise this argument is set to -1.0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 2)
            On entry, `y`[*<value>*] = *<value>*.

            Constraint: `y`[i-1] >= 0.0 for all i.

        (`errno` 3)
            On entry, `optopt` = *<value>*.

            Constraint: `optopt` = -2, -1, 1 or 2.

        (`errno` 9)
            The optimization of log-likelihood failed to converge; no
            maximum likelihood estimates are returned. Try using the
            other maximum likelihood option by resetting `optopt`. If
            this also fails, moments-based estimates can be returned by
            an appropriate setting of `optopt`.

        (`errno` 10)
            Variance of data in `y` is too low for method of moments
            optimization.

        (`errno` 11)
            The sum of `y` is zero within machine precision.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            The asymptotic distribution is not available for the
            returned parameter estimates.

        (`errno` 7)
            The distribution of maximum likelihood estimates cannot be
            calculated for the returned parameter estimates because the
            Hessian matrix could not be inverted.

        (`errno` 8)
            The asymptotic distribution of parameter estimates is
            invalid and the distribution of maximum likelihood estimates
            cannot be calculated for the returned parameter estimates
            because the Hessian matrix could not be inverted.
    """

    from ..base.univar import estim_genpareto as b_estim_genpareto

    fun_name = 'naginterfaces.library.univar.estim_genpareto'

    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _asvc = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (4,),
    )
    _obsvc = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (4,),
    )
    try:
        _return_scalars = b_estim_genpareto(
            _n,
            _y,
            optopt,
            _asvc,
            _obsvc,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _xi_py = _return_scalars.xi
    _beta_py = _return_scalars.beta
    _asvc_py = _asvc
    _obsvc_py = _obsvc
    _ll_py = _return_scalars.ll
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xi', 'beta', 'asvc', 'obsvc', 'll'],
    )(
        _xi_py,
        _beta_py,
        _asvc_py,
        _obsvc_py,
        _ll_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def ttest_2normal(tail, equal, nx, ny, xmean, ymean, xstd, ystd, clevel):
    u"""
    Computes t-test statistic for a difference in means between two
    Normal populations, confidence interval.

    For full information please refer to the NAG Library document for
    g07ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07caf.html

    Parameters
    ----------
    tail : str, length 1
        Indicates which tail probability is to be calculated, and thus
        which alternative hypothesis is to be used.

        `tail` = 'T'

          The two tail probability, i.e., H_1:mu_x != mu_y.

        `tail` = 'U'

          The upper tail probability, i.e., H_1:mu_x > mu_y.

        `tail` = 'L'

          The lower tail probability, i.e., H_1:mu_x < mu_y.

    equal : str, length 1
        Indicates whether the population variances are assumed to be
        equal or not.

        `equal` = 'E'

          The population variances are assumed to be equal, that is
          sigma_x^2 = sigma_y^2.

        `equal` = 'U'

          The population variances are not assumed to be equal.

    nx : int
        n_x, the size of the X sample.

    ny : int
        n_y, the size of the Y sample.

    xmean : float
        x_, the mean of the X sample.

    ymean : float
        y_, the mean of the Y sample.

    xstd : float
        s_x, the standard deviation of the X sample.

    ystd : float
        s_y, the standard deviation of the Y sample.

    clevel : float
        The confidence level, 1-alpha, for the specified tail. For
        example `clevel` = 0.95 will give a 95% confidence interval.

    Returns
    -------
    t : float
        Contains the test statistic, t_obs or t'_obs.

    df : float
        Contains the degrees of freedom for the test statistic.

    prob : float
        Contains the significance level, that is the tail probability,
        p, as defined by `tail`.

    dl : float
        Contains the lower confidence limit for mu_x-mu_y.

    du : float
        Contains the upper confidence limit for mu_x-mu_y.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `clevel` = *<value>*.

            Constraint: 0.0 < `clevel` < 1.0.

        (`errno` 1)
            On entry, `ystd` = *<value>*.

            Constraint: `ystd` > 0.0.

        (`errno` 1)
            On entry, `xstd` = *<value>*.

            Constraint: `xstd` > 0.0.

        (`errno` 1)
            On entry, `ny` = *<value>*.

            Constraint: `ny` >= 2.

        (`errno` 1)
            On entry, `nx` = *<value>*.

            Constraint: `nx` >= 2.

        (`errno` 1)
            On entry, `equal` = *<value>*.

            Constraint: `equal` = 'E' or 'U'.

        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'T', 'U' or 'L'.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.univar import ttest_2normal as b_ttest_2normal

    _return_scalars = b_ttest_2normal(
        tail,
        equal,
        nx,
        ny,
        xmean,
        ymean,
        xstd,
        ystd,
        clevel,
    )
    _t_py = _return_scalars.t
    _df_py = _return_scalars.df
    _prob_py = _return_scalars.prob
    _dl_py = _return_scalars.dl
    _du_py = _return_scalars.du
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['t', 'df', 'prob', 'dl', 'du'],
    )(
        _t_py,
        _df_py,
        _prob_py,
        _dl_py,
        _du_py,
    )
    return _return_data

def robust_1var_median(x):
    u"""
    Robust estimation, median, median absolute deviation, robust
    standard deviation.

    For full information please refer to the NAG Library document for
    g07da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07daf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The vector of observations, x_1,x_2,...,x_n.

    Returns
    -------
    y : float, ndarray, shape (n)
        The observations sorted into ascending order.

    xme : float
        The median, theta_med.

    xmd : float
        The median absolute deviation, sigma_med.

    xsd : float
        The robust estimate of the standard deviation, sigma'_med.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.univar import robust_1var_median as b_robust_1var_median

    fun_name = 'naginterfaces.library.univar.robust_1var_median'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_robust_1var_median(
        _n,
        _x,
        _y,
    )
    _y_py = _y
    _xme_py = _return_scalars.xme
    _xmd_py = _return_scalars.xmd
    _xsd_py = _return_scalars.xsd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['y', 'xme', 'xmd', 'xsd'],
    )(
        _y_py,
        _xme_py,
        _xmd_py,
        _xsd_py,
    )
    return _return_data

def robust_1var_mestim(isigma, x, ipsi, c, h1, h2, h3, dchi, theta, sigma, tol, maxit=50):
    u"""
    Robust estimation, M-estimates for location and scale parameters,
    standard weight functions.

    For full information please refer to the NAG Library document for
    g07db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07dbf.html

    Parameters
    ----------
    isigma : int
        The value assigned to `isigma` determines whether sigma^ is to
        be simultaneously estimated.

        `isigma` = 0

          The estimation of sigma^ is bypassed and `sigma` is set equal
          to sigma_c.

        `isigma` = 1

          sigma^ is estimated simultaneously.

    x : float, array-like, shape (n)
        The vector of observations, x_1,x_2,...,x_n.

    ipsi : int
        Which psi function is to be used.

        `ipsi` = 0

          psi(t) = t.

        `ipsi` = 1

          Huber's function.

        `ipsi` = 2

          Hampel's piecewise linear function.

        `ipsi` = 3

          Andrew's sine wave,

        `ipsi` = 4

          Tukey's bi-weight.

    c : float
        If `ipsi` = 1, `c` must specify the parameter, c, of Huber's psi
        function. `c` is not referenced if `ipsi` != 1.

    h1 : float
        If `ipsi` = 2, `h1`, `h2` and `h3` must specify the parameters,
        h_1, h_2, and h_3, of Hampel's piecewise linear psi function.
        `h1`, `h2` and `h3` are not referenced if `ipsi` != 2.

    h2 : float
        If `ipsi` = 2, `h1`, `h2` and `h3` must specify the parameters,
        h_1, h_2, and h_3, of Hampel's piecewise linear psi function.
        `h1`, `h2` and `h3` are not referenced if `ipsi` != 2.

    h3 : float
        If `ipsi` = 2, `h1`, `h2` and `h3` must specify the parameters,
        h_1, h_2, and h_3, of Hampel's piecewise linear psi function.
        `h1`, `h2` and `h3` are not referenced if `ipsi` != 2.

    dchi : float
        d, the parameter of the chi function. `dchi` is not referenced
        if `ipsi` = 0.

    theta : float
        If `sigma` > 0 then `theta` must be set to the required starting
        value of the estimation of the location parameter theta^. A
        reasonable initial value for theta^ will often be the sample
        mean or median.

    sigma : float
        The role of `sigma` depends on the value assigned to `isigma`,
        as follows:

        - if `isigma` = 1, `sigma` must be assigned a value which
          determines the values of the starting points for the
          calculations of theta^ and sigma^. If `sigma` <= 0.0 then
          ``robust_1var_mestim`` will determine the starting points of
          theta^ and sigma^. Otherwise the value assigned to `sigma`
          will be taken as the starting point for sigma^, and `theta`
          must be assigned a value before entry, see above;

        - if `isigma` = 0, `sigma` must be assigned a value which
          determines the value of sigma_c, which is held fixed during
          the iterations, and the starting value for the calculation of
          theta^. If `sigma` <= 0, ``robust_1var_mestim`` will determine
          the value of sigma_c as the median absolute deviation adjusted
          to reduce bias (see ``robust_1var_median``) and the starting
          point for theta^. Otherwise, the value assigned to `sigma`
          will be taken as the value of sigma_c and `theta` must be
          assigned a relevant value before entry, see above.

    tol : float
        The relative precision for the final estimates. Convergence is
        assumed when the increments for `theta`, and `sigma` are less
        than `tol`*max(1.0, sigma_{k-1}).

    maxit : int, optional
        The maximum number of iterations that should be used during the
        estimation.

    Returns
    -------
    theta : float
        The M-estimate of the location parameter, theta^.

    sigma : float
        Contains the M-estimate of the scale parameter, sigma^, if
        `isigma` was assigned the value 1 on entry, otherwise `sigma`
        will contain the initial fixed value sigma_c.

    rs : float, ndarray, shape (n)
        The Winsorized residuals.

    nit : int
        The number of iterations that were used during the estimation.

    wrk : float, ndarray, shape (n)
        If `sigma` <= 0.0 on entry, `wrk` will contain the n
        observations in ascending order.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ipsi` = *<value>*.

            Constraint: `ipsi` = 0, 1, 2, 3 or 4.

        (`errno` 1)
            On entry, `isigma` = *<value>*.

            Constraint: `isigma` = 0 or 1.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 2)
            On entry, `dchi` = *<value>*.

            Constraint: `dchi` > 0.0.

        (`errno` 2)
            On entry, `h1` = *<value>*, `h2` = *<value>* and `h3` =
            *<value>*.

            Constraint: 0 <= `h1` <= `h2` <= `h3` and `h3` > 0.0.

        (`errno` 2)
            On entry, `c` = *<value>*.

            Constraint: `c` > 0.0.

        (`errno` 3)
            All elements of `x` are equal.

        (`errno` 4)
            Current estimate of `sigma` is zero or negative: `sigma` =
            *<value>*.

        (`errno` 5)
            Number of iterations required exceeds `maxit`: `maxit` =
            *<value>*.

        (`errno` 6)
            All winsorized residuals are zero.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.univar import robust_1var_mestim as b_robust_1var_mestim

    fun_name = 'naginterfaces.library.univar.robust_1var_mestim'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _rs = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wrk = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_robust_1var_mestim(
        isigma,
        _n,
        _x,
        ipsi,
        c,
        h1,
        h2,
        h3,
        dchi,
        theta,
        sigma,
        maxit,
        tol,
        _rs,
        _wrk,
    )
    _theta_py = _return_scalars.theta
    _sigma_py = _return_scalars.sigma
    _rs_py = _rs
    _nit_py = _return_scalars.nit
    _wrk_py = _wrk
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['theta', 'sigma', 'rs', 'nit', 'wrk'],
    )(
        _theta_py,
        _sigma_py,
        _rs_py,
        _nit_py,
        _wrk_py,
    )
    return _return_data

def robust_1var_mestim_wgt(chi, psi, isigma, x, beta, theta, sigma, tol, maxit=50, data=None):
    u"""
    Robust estimation, M-estimates for location and scale parameters,
    user-defined weight functions.

    For full information please refer to the NAG Library document for
    g07dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07dcf.html

    Parameters
    ----------
    chi : callable retval = chi(t, data=None)
        `chi` must return the value of the weight function chi for a
        given value of its argument.

        The value of chi must be non-negative.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which `chi` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the weight function phi evaluated at `t`.

    psi : callable retval = psi(t, data=None)
        `psi` must return the value of the weight function psi for a
        given value of its argument.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which `psi` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the weight function psi evaluated at `t`.

    isigma : int
        The value assigned to `isigma` determines whether sigma^ is to
        be simultaneously estimated.

        `isigma` = 0

          The estimation of sigma^ is bypassed and `sigma` is set equal
          to sigma_c.

        `isigma` = 1

          sigma^ is estimated simultaneously.

    x : float, array-like, shape (n)
        The vector of observations, x_1,x_2,...,x_n.

    beta : float
        The value of the constant beta of the chosen `chi` function.

    theta : float
        If `sigma` > 0, `theta` must be set to the required starting
        value of the estimate of the location parameter theta^. A
        reasonable initial value for theta^ will often be the sample
        mean or median.

    sigma : float
        The role of `sigma` depends on the value assigned to `isigma` as
        follows.

        If `isigma` = 1, `sigma` must be assigned a value which
        determines the values of the starting points for the calculation
        of theta^ and sigma^.

        If `sigma` <= 0.0, ``robust_1var_mestim_wgt`` will determine the
        starting points of theta^ and sigma^.

        Otherwise, the value assigned to `sigma` will be taken as the
        starting point for sigma^, and `theta` must be assigned a
        relevant value before entry, see above.

        If `isigma` = 0, `sigma` must be assigned a value which
        determines the values of sigma_c, which is held fixed during the
        iterations, and the starting value for the calculation of
        theta^.

        If `sigma` <= 0, ``robust_1var_mestim_wgt`` will determine the
        value of sigma_c as the median absolute deviation adjusted to
        reduce bias (see ``robust_1var_median``) and the starting point
        for theta.

        Otherwise, the value assigned to `sigma` will be taken as the
        value of sigma_c and `theta` must be assigned a relevant value
        before entry, see above.

    tol : float
        The relative precision for the final estimates. Convergence is
        assumed when the increments for `theta`, and `sigma` are less
        than `tol`*max(1.0, sigma_{k-1}).

    maxit : int, optional
        The maximum number of iterations that should be used during the
        estimation.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    theta : float
        The M-estimate of the location parameter theta^.

    sigma : float
        The M-estimate of the scale parameter sigma^, if `isigma` was
        assigned the value 1 on entry, otherwise `sigma` will contain
        the initial fixed value sigma_c.

    rs : float, ndarray, shape (n)
        The Winsorized residuals.

    nit : int
        The number of iterations that were used during the estimation.

    wrk : float, ndarray, shape (n)
        If `sigma` <= 0.0 on entry, `wrk` will contain the n
        observations in ascending order.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `isigma` = *<value>*.

            Constraint: `isigma` = 0 or 1.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 2)
            On entry, `beta` = *<value>*.

            Constraint: `beta` > 0.0.

        (`errno` 3)
            All elements of `x` are equal.

        (`errno` 4)
            Current estimate of `sigma` is zero or negative: `sigma` =
            *<value>*.

        (`errno` 5)
            Number of iterations required exceeds `maxit`: `maxit` =
            *<value>*.

        (`errno` 6)
            All winsorized residuals are zero.

        (`errno` 7)
            The `chi` function returned a negative value: `chi` =
            *<value>*.
    """

    from ..base.univar import robust_1var_mestim_wgt as b_robust_1var_mestim_wgt

    def cb_chi(t, data=None):
        """Wrapping user's chi. Called as base chi."""
        if data is not None:
            _retval_chi = chi(
                t,
                data,
            )
        else:
            _retval_chi = chi(
                t,
            )
        return _retval_chi

    def cb_psi(t, data=None):
        """Wrapping user's psi. Called as base psi."""
        if data is not None:
            _retval_psi = psi(
                t,
                data,
            )
        else:
            _retval_psi = psi(
                t,
            )
        return _retval_psi

    fun_name = 'naginterfaces.library.univar.robust_1var_mestim_wgt'

    _b_utils._check_callable(chi, {'fun_name': fun_name, 'entity': 'chi'})
    _chi = cb_chi
    _b_utils._check_callable(psi, {'fun_name': fun_name, 'entity': 'psi'})
    _psi = cb_psi
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _rs = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wrk = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_robust_1var_mestim_wgt(
        _chi,
        _psi,
        isigma,
        _n,
        _x,
        beta,
        theta,
        sigma,
        maxit,
        tol,
        _rs,
        _wrk,
        data,
    )
    _theta_py = _return_scalars.theta
    _sigma_py = _return_scalars.sigma
    _rs_py = _rs
    _nit_py = _return_scalars.nit
    _wrk_py = _wrk
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['theta', 'sigma', 'rs', 'nit', 'wrk'],
    )(
        _theta_py,
        _sigma_py,
        _rs_py,
        _nit_py,
        _wrk_py,
    )
    return _return_data

def robust_1var_trimmed(x, alpha):
    u"""
    Computes a trimmed and winsorized mean of a single sample with
    estimates of their variance.

    For full information please refer to the NAG Library document for
    g07dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07ddf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The sample observations, x_i, for i = 1,2,...,n.

    alpha : float
        alpha, the proportion of observations to be trimmed at each end
        of the sorted sample.

    Returns
    -------
    tmean : float
        The alpha-trimmed mean, x__t.

    wmean : float
        The alpha-Winsorized mean, x__w.

    tvar : float
        Contains an estimate of the variance of the trimmed mean.

    wvar : float
        Contains an estimate of the variance of the Winsorized mean.

    k : int
        Contains the number of observations trimmed at each end, k.

    sx : float, ndarray, shape (n)
        Contains the sample observations sorted into ascending order.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n <= 1.

        (`errno` 2)
            On entry, `alpha` = *<value>*.

            Constraint: 0.0 <= `alpha` < 0.5.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.univar import robust_1var_trimmed as b_robust_1var_trimmed

    fun_name = 'naginterfaces.library.univar.robust_1var_trimmed'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _sx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_robust_1var_trimmed(
        _n,
        _x,
        alpha,
        _sx,
    )
    _tmean_py = _return_scalars.tmean
    _wmean_py = _return_scalars.wmean
    _tvar_py = _return_scalars.tvar
    _wvar_py = _return_scalars.wvar
    _k_py = _return_scalars.k
    _sx_py = _sx
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['tmean', 'wmean', 'tvar', 'wvar', 'k', 'sx'],
    )(
        _tmean_py,
        _wmean_py,
        _tvar_py,
        _wvar_py,
        _k_py,
        _sx_py,
    )
    return _return_data

def robust_1var_ci(method, x, clevel):
    u"""
    Robust confidence intervals, one-sample.

    For full information please refer to the NAG Library document for
    g07ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07eaf.html

    Parameters
    ----------
    method : str, length 1
        Specifies the method to be used.

        `method` = 'E'

          The exact algorithm is used.

        `method` = 'A'

          The iterative algorithm is used.

    x : float, array-like, shape (n)
        The sample observations, x_i, for i = 1,2,...,n.

    clevel : float
        The confidence interval desired.

        For example, for a 95% confidence interval set `clevel` = 0.95.

    Returns
    -------
    theta : float
        The estimate of the location, theta^.

    thetal : float
        The estimate of the lower limit of the confidence interval,
        theta_l.

    thetau : float
        The estimate of the upper limit of the confidence interval,
        theta_u.

    estcl : float
        An estimate of the actual percentage confidence of the interval
        found, as a proportion between (0.0,1.0).

    wlower : float
        The upper value of the Wilcoxon test statistic, W_u,
        corresponding to the lower limit of the confidence interval.

    wupper : float
        The lower value of the Wilcoxon test statistic, W_l,
        corresponding to the upper limit of the confidence interval.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `clevel` = *<value>*.

            Constraint: 0.0 < `clevel` < 1.0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'E' or 'A'.

        (`errno` 2)
            Not enough information to compute an interval estimate since
            the whole sample is identical. The common value is returned
            in `theta`, `thetal` and `thetau`.

        (`errno` 3)
            The iterative procedure used to estimate theta has not
            converged.

        (`errno` 3)
            The iterative procedure used to estimate, theta_u, the upper
            confidence limit has not converged.

        (`errno` 3)
            The iterative procedure used to estimate, theta_l, the lower
            confidence limit has not converged.
    """

    from ..base.univar import robust_1var_ci as b_robust_1var_ci

    fun_name = 'naginterfaces.library.univar.robust_1var_ci'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_robust_1var_ci(
        method,
        _n,
        _x,
        clevel,
    )
    _theta_py = _return_scalars.theta
    _thetal_py = _return_scalars.thetal
    _thetau_py = _return_scalars.thetau
    _estcl_py = _return_scalars.estcl
    _wlower_py = _return_scalars.wlower
    _wupper_py = _return_scalars.wupper
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['theta', 'thetal', 'thetau', 'estcl', 'wlower', 'wupper'],
    )(
        _theta_py,
        _thetal_py,
        _thetau_py,
        _estcl_py,
        _wlower_py,
        _wupper_py,
    )
    return _return_data

def robust_2var_ci(method, x, y, clevel):
    u"""
    Robust confidence intervals, two-sample.

    For full information please refer to the NAG Library document for
    g07eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07ebf.html

    Parameters
    ----------
    method : str, length 1
        Specifies the method to be used.

        `method` = 'E'

          The exact algorithm is used.

        `method` = 'A'

          The iterative algorithm is used.

    x : float, array-like, shape (n)
        The observations of the first sample, x_i, for i = 1,2,...,n.

    y : float, array-like, shape (m)
        The observations of the second sample, y_j, for j = 1,2,...,m.

    clevel : float
        The confidence interval required, 1-alpha; e.g., for a 95%
        confidence interval set `clevel` = 0.95.

    Returns
    -------
    theta : float
        The estimate of the difference in the location of the two
        populations, theta^.

    thetal : float
        The estimate of the lower limit of the confidence interval,
        theta_l.

    thetau : float
        The estimate of the upper limit of the confidence interval,
        theta_u.

    estcl : float
        An estimate of the actual percentage confidence of the interval
        found, as a proportion between (0.0,1.0).

    ulower : float
        The value of the Mann--Whitney U statistic corresponding to the
        lower confidence limit, U_l.

    uupper : float
        The value of the Mann--Whitney U statistic corresponding to the
        upper confidence limit, U_u.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, `clevel` = *<value>*.

            Constraint: 0.0 < `clevel` < 1.0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'E' or 'A'.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Not enough information to compute an interval estimate since
            each sample has identical values. The common difference is
            returned in `theta`, `thetal` and `thetau`.

        (`errno` 3)
            The iterative procedure used to estimate theta has not
            converged.

        (`errno` 3)
            The iterative procedure used to estimate, theta_u, the upper
            confidence limit has not converged.

        (`errno` 3)
            The iterative procedure used to estimate, theta_l, the lower
            confidence limit has not converged.
    """

    from ..base.univar import robust_2var_ci as b_robust_2var_ci

    fun_name = 'naginterfaces.library.univar.robust_2var_ci'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    try:
        _return_scalars = b_robust_2var_ci(
            method,
            _n,
            _x,
            _m,
            _y,
            clevel,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _theta_py = _return_scalars.theta
    _thetal_py = _return_scalars.thetal
    _thetau_py = _return_scalars.thetau
    _estcl_py = _return_scalars.estcl
    _ulower_py = _return_scalars.ulower
    _uupper_py = _return_scalars.uupper
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['theta', 'thetal', 'thetau', 'estcl', 'ulower', 'uupper'],
    )(
        _theta_py,
        _thetal_py,
        _thetau_py,
        _estcl_py,
        _ulower_py,
        _uupper_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def outlier_peirce_1var(p, y, ldiff, mean=0.0, var=0.0):
    u"""
    Outlier detection using method of Peirce, raw data or single
    variance supplied.

    For full information please refer to the NAG Library document for
    g07ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07gaf.html

    Parameters
    ----------
    p : int
        p, the number of parameters in the model used in obtaining the
        y. If y is an observed set of values, as opposed to the
        residuals from fitting a model with p parameters, then p should
        be set to 1, i.e., as if a model just containing the mean had
        been used.

    y : float, array-like, shape (n)
        y, the data being tested.

    ldiff : int
        The maximum number of values to be returned in arrays `diff` and
        `llamb`.

        If `ldiff` <= 0, arrays `diff` and `llamb` are not referenced.

    mean : float, optional
        If `var` > 0.0, `mean` must contain mu, the mean of y, otherwise
        `mean` is not referenced and the mean is calculated from the
        data supplied in `y`.

    var : float, optional
        If `var` > 0.0, `var` must contain sigma^2, the variance of y,
        otherwise the variance is calculated from the data supplied in
        `y`.

    Returns
    -------
    iout : int, ndarray, shape (n)
        The indices of the values in `y` sorted in descending order of
        the absolute difference from the mean, therefore |`y`[
        `iout`[i-2] -1] - mu| >= |`y`[ `iout`[i-1] -1] - mu|, for i =
        2,3,...,n.

    niout : int
        The number of potential outliers. The indices for these
        potential outliers are held in the first `niout` elements of
        `iout`. By construction there can be at most n-`p`-1 values
        flagged as outliers.

    diff : float, ndarray, shape (`ldiff`)
        `diff`[i-1] holds |y-mu|-sigma^2z for observation
        `y`[`iout`[i-1]-1], for i = 1,2,...,min(`ldiff`, `niout`+1,
        n-`p`-1).

    llamb : float, ndarray, shape (`ldiff`)
        `llamb`[i-1] holds log(lambda^2) for observation
        `y`[`iout`[i-1]-1], for i = 1,2,...,min(`ldiff`, `niout`+1,
        n-`p`-1).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 3.

        (`errno` 2)
            On entry, `p` = *<value>* and n = *<value>*.

            Constraint: 1 <= `p` <= n-2.
    """

    from ..base.univar import outlier_peirce_1var as b_outlier_peirce_1var

    fun_name = 'naginterfaces.library.univar.outlier_peirce_1var'

    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _iout = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _ldiff = _b_utils._EngineIntScalarType(
        ldiff, {'fun_name': fun_name, 'entity': 'ldiff'},
    ).to_ctypes()
    _ldiff_py = _b_utils._EngineIntScalarType.to_py(_ldiff)
    _diff = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ldiff_py,),
    )
    _llamb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ldiff_py,),
    )
    _return_scalar = b_outlier_peirce_1var(
        _n,
        p,
        _y,
        mean,
        var,
        _iout,
        _ldiff,
        _diff,
        _llamb,
    )
    _iout_py = _iout
    _niout_py = _return_scalar
    _diff_py = _diff
    _llamb_py = _llamb
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iout', 'niout', 'diff', 'llamb'],
    )(
        _iout_py,
        _niout_py,
        _diff_py,
        _llamb_py,
    )
    return _return_data

def outlier_peirce_2var(n, e, var1, var2):
    u"""
    Outlier detection using method of Peirce, two variances supplied.

    For full information please refer to the NAG Library document for
    g07gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g07/g07gbf.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    e : float
        e~, the value being tested.

    var1 : float
        sigma^2, the residual variance on fitting model M to y.

    var2 : float
        sigma~^2, the residual variance on fitting model M to y~.

    Returns
    -------
    outl : bool
        True if the value being tested is an outlier.

    x : float
        An estimated value of x, the cutoff that indicates an outlier.

    lx : float
        l, the lower limit for x.

    ux : float
        u, the upper limit for x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 3.

        (`errno` 3)
            On entry, `var1` = *<value>*.

            Constraint: `var1` > 0.0.

        (`errno` 4)
            On entry, `var2` = *<value>*.

            Constraint: `var2` > 0.0.

        (`errno` 4)
            On entry, `var1` = *<value>*, `var2` = *<value>*.

            Constraint: `var2` < `var1`.
    """

    from ..base.univar import outlier_peirce_2var as b_outlier_peirce_2var

    _return_scalars = b_outlier_peirce_2var(
        n,
        e,
        var1,
        var2,
    )
    _outl_py = _return_scalars.outl
    _x_py = _return_scalars.x
    _lx_py = _return_scalars.lx
    _ux_py = _return_scalars.ux
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['outl', 'x', 'lx', 'ux'],
    )(
        _outl_py,
        _x_py,
        _lx_py,
        _ux_py,
    )
    return _return_data
