# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `wav` Chapter.

``wav`` - Wavelet Transforms

This module is concerned with the analysis of datasets (or functions or
operators) in terms of frequency and scale components using wavelet
transforms.
Wavelet transforms have been applied in many fields from time series
analysis to image processing and the localization in either frequency or
scale that they provide is useful for data compression or denoising.
In general the standard wavelet transform uses dilation and scaling of a
chosen function, psi(t), (called the mother wavelet) such that

    psi_{a,b}(t) = 1/(sqrt(a))psi((t-b)/a)

where a gives the scaling and b determines the translation.
Wavelet methods can be divided into continuous transforms and discrete
transforms.
In the continuous case, the pair a and b are real numbers with a>0.
For the discrete transform, a and b can be chosen as a = 2^(-j), b =
k2^(-j) for integers j, k

    psi_{j,k}(t) = 2^(j/2)psi(2^jt-k).

The continuous real valued, one-dimensional wavelet transform (CWT) is
included in this module.
The discrete wavelet transform (DWT) at a single level together with its
inverse and the multi-level DWT with inverse are also provided for one,
two and three dimensions.
The Maximal Overlap DWT (MODWT) together with its inverse and the
multi-level MODWT with inverse are provided for one dimension.
The choice of wavelet for CWT includes the Morlet wavelet and
derivatives of a Gaussian while the DWT and MODWT offer the orthogonal
wavelets of Daubechies and a selection of biorthogonal wavelets.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def dim1_init(wavnam, wtrans, mode, n):
    u"""
    One-dimensional wavelet filter initialization.

    For full information please refer to the NAG Library document for
    c09aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09aaf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'HAAR'

          Haar wavelet, also known as 'DB1' as a special case of the
          Daubechies wavelet.

        `wavnam` = 'DBn', where n = 2,3,...,38

          Daubechies wavelet with n vanishing moments (2n coefficients).
          For example, `wavnam` = 'DB4' is the name for the Daubechies
          wavelet with 4 vanishing moments (8 coefficients).

        `wavnam` = 'COIFn', where n = 1,2,...,17

          Coiflet wavelet of order n.

        `wavnam` = 'BEYL'

          Beylkin wavelet.

        `wavnam` = 'VAID'

          Vaidyanathan wavelet.

        `wavnam` = 'SYMn', where n = 2,3,...,20

          Symlet wavelet of order n.

        `wavnam` = 'BIORx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Biorthogonal wavelet of order x.y. For example `wavnam` =
          'BIOR3.1' is the name for the biorthogonal wavelet of order
          3.1.

        `wavnam` = 'RBIOx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Reverse biorthogonal wavelet of order x.y. For example
          `wavnam` = 'RBIO3.1' is the name for the reverse biorthogonal
          wavelet of order 3.1.

    wtrans : str, length 1
        The type of discrete wavelet transform that is to be applied.

        `wtrans` = 'S'

          Single-level decomposition or reconstruction by discrete
          wavelet transform.

        `wtrans` = 'M'

          Multiresolution, by a multi-level DWT or its inverse.

        `wtrans` = 'T'

          Single-level decomposition or reconstruction by maximal
          overlap discrete wavelet transform.

        `wtrans` = 'U'

          Multi-level resolution by a maximal overlap discrete wavelet
          transform or its inverse.

    mode : str, length 1
        The end extension method. Note that only periodic end extension
        is currently available for the MODWT.

        `mode` = 'P'

          Periodic end extension.

        `mode` = 'H'

          Half-point symmetric end extension.

        `mode` = 'W'

          Whole-point symmetric end extension.

        `mode` = 'Z'

          Zero end extension.

    n : int
        The number of elements, n, in the input data array, x.

    Returns
    -------
    nwlmax : int
        The maximum number of levels of resolution, l_max, that can be
        computed when a multi-level discrete wavelet transform is
        applied. It is such that 2^(l_max) <= n < 2^(l_max+1), for l_max
        an integer.

    nf : int
        The filter length, n_f, for the supplied mother wavelet. This is
        used to determine the number of coefficients to be generated by
        the chosen transform.

    nwc : int
        For a single-level transform (`wtrans` = 'S' or 'T'), the number
        of approximation coefficients that would be generated for the
        given problem size, mother wavelet, extension method and type of
        transform; this is also the corresponding number of detail
        coefficients. For a multi-level transform (`wtrans` = 'M' or
        'U') the total number of coefficients that would be generated
        over l_max levels and with ikeep = 1 for MODWT.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wtrans` = *<value>*.

            Constraint: `wtrans` = 'S', 'M', 'T' or 'U'.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 'P', 'H', 'W' or 'Z'.

        (`errno` 3)
            On entry, `wtrans` = 'T' or 'U' and `mode` != 'P'.

            Constraint: `mode` = 'P' when `wtrans` = 'T' or 'U'.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.
    """

    from ..base.wav import dim1_init as b_dim1_init

    _return_scalars = b_dim1_init(
        wavnam,
        wtrans,
        mode,
        n,
    )
    _nwlmax_py = _return_scalars.nwlmax
    _nf_py = _return_scalars.nf
    _nwc_py = _return_scalars.nwc
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nwlmax', 'nf', 'nwc', 'comm'],
    )(
        _nwlmax_py,
        _nf_py,
        _nwc_py,
        _comm_py,
    )
    return _return_data

def dim2_init(wavnam, wtrans, mode, m, n):
    u"""
    Two-dimensional wavelet filter initialization.

    For full information please refer to the NAG Library document for
    c09ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09abf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'HAAR'

          Haar wavelet, also known as 'DB1' as a special case of the
          Daubechies wavelet.

        `wavnam` = 'DBn', where n = 2,3,...,38

          Daubechies wavelet with n vanishing moments (2n coefficients).
          For example, `wavnam` = 'DB4' is the name for the Daubechies
          wavelet with 4 vanishing moments (8 coefficients).

        `wavnam` = 'COIFn', where n = 1,2,...,17

          Coiflet wavelet of order n.

        `wavnam` = 'BEYL'

          Beylkin wavelet.

        `wavnam` = 'VAID'

          Vaidyanathan wavelet.

        `wavnam` = 'SYMn', where n = 2,3,...,20

          Symlet wavelet of order n.

        `wavnam` = 'BIORx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Biorthogonal wavelet of order x.y. For example `wavnam` =
          'BIOR3.1' is the name for the biorthogonal wavelet of order
          3.1.

        `wavnam` = 'RBIOx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Reverse biorthogonal wavelet of order x.y. For example
          `wavnam` = 'RBIO3.1' is the name for the reverse biorthogonal
          wavelet of order 3.1.

    wtrans : str, length 1
        The type of discrete wavelet transform that is to be applied.

        `wtrans` = 'S'

          Single-level decomposition or reconstruction by discrete
          wavelet transform.

        `wtrans` = 'M'

          Multiresolution, by a multi-level DWT or its inverse.

    mode : str, length 1
        The end extension method.

        `mode` = 'P'

          Periodic end extension.

        `mode` = 'H'

          Half-point symmetric end extension.

        `mode` = 'W'

          Whole-point symmetric end extension.

        `mode` = 'Z'

          Zero end extension.

    m : int
        The number of elements, m, in the first dimension (number of
        rows of data matrix A) of the input data.

    n : int
        The number of elements, n, in the second dimension (number of
        columns of data matrix A) of the input data.

    Returns
    -------
    nwlmax : int
        The maximum number of levels of resolution, l_max, that can be
        computed if a multi-level discrete wavelet transform is applied
        (`wtrans` = 'M'). It is such that 2^(l_max) <= min(m, n) <
        2^(l_max+1), for l_max an integer.

        If `wtrans` = 'S', `nwlmax` is not set.

    nf : int
        The filter length, n_f, for the supplied mother wavelet. This is
        used to determine the number of coefficients to be generated by
        the chosen transform.

    nwct : int
        The total number of wavelet coefficients, n_ct, that will be
        generated. When `wtrans` = 'S' the number of rows required in
        each of the output coefficient matrices can be calculated as
        n_cm = n_ct/(4n_cn). When `wtrans` = 'M' the length of the array
        used to store all of the coefficient matrices must be at least
        n_ct.

    nwcn : int
        For a single-level transform (`wtrans` = 'S'), the number of
        coefficients that would be generated in the second dimension,
        n_cn, for each coefficient type. For a multi-level transform
        (`wtrans` = 'M') this is set to 1.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wtrans` = *<value>*.

            Constraint: `wtrans` = 'S' or 'M'.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 'P', 'H', 'W' or 'Z'.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 2.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.
    """

    from ..base.wav import dim2_init as b_dim2_init

    _return_scalars = b_dim2_init(
        wavnam,
        wtrans,
        mode,
        m,
        n,
    )
    _nwlmax_py = _return_scalars.nwlmax
    _nf_py = _return_scalars.nf
    _nwct_py = _return_scalars.nwct
    _nwcn_py = _return_scalars.nwcn
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nwlmax', 'nf', 'nwct', 'nwcn', 'comm'],
    )(
        _nwlmax_py,
        _nf_py,
        _nwct_py,
        _nwcn_py,
        _comm_py,
    )
    return _return_data

def dim3_init(wavnam, wtrans, mode, m, n, fr):
    u"""
    Three-dimensional wavelet filter initialization.

    For full information please refer to the NAG Library document for
    c09ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09acf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'HAAR'

          Haar wavelet, also known as 'DB1' as a special case of the
          Daubechies wavelet.

        `wavnam` = 'DBn', where n = 2,3,...,38

          Daubechies wavelet with n vanishing moments (2n coefficients).
          For example, `wavnam` = 'DB4' is the name for the Daubechies
          wavelet with 4 vanishing moments (8 coefficients).

        `wavnam` = 'COIFn', where n = 1,2,...,17

          Coiflet wavelet of order n.

        `wavnam` = 'BEYL'

          Beylkin wavelet.

        `wavnam` = 'VAID'

          Vaidyanathan wavelet.

        `wavnam` = 'SYMn', where n = 2,3,...,20

          Symlet wavelet of order n.

        `wavnam` = 'BIORx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Biorthogonal wavelet of order x.y. For example `wavnam` =
          'BIOR3.1' is the name for the biorthogonal wavelet of order
          3.1.

        `wavnam` = 'RBIOx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Reverse biorthogonal wavelet of order x.y. For example
          `wavnam` = 'RBIO3.1' is the name for the reverse biorthogonal
          wavelet of order 3.1.

    wtrans : str, length 1
        The type of discrete wavelet transform that is to be applied.

        `wtrans` = 'S'

          Single-level decomposition or reconstruction by discrete
          wavelet transform.

        `wtrans` = 'M'

          Multiresolution, by a multi-level DWT or its inverse.

    mode : str, length 1
        The end extension method.

        `mode` = 'P'

          Periodic end extension.

        `mode` = 'H'

          Half-point symmetric end extension.

        `mode` = 'W'

          Whole-point symmetric end extension.

        `mode` = 'Z'

          Zero end extension.

    m : int
        The number of elements, m, in the first dimension (number of
        rows of each two-dimensional frame) of the input data, A.

    n : int
        The number of elements, n, in the second dimension (number of
        columns of each two-dimensional frame) of the input data, A.

    fr : int
        The number of elements, fr, in the third dimension (number of
        frames) of the input data, A.

    Returns
    -------
    nwlmax : int
        The maximum number of levels of resolution, l_max, that can be
        computed if a multi-level discrete wavelet transform is applied
        (`wtrans` = 'M'). It is such that 2^(l_max) <= min(m,n,fr) <
        2^(l_max+1), for l_max an integer.

        If `wtrans` = 'S', `nwlmax` is not set.

    nf : int
        The filter length, n_f, for the supplied mother wavelet. This is
        used to determine the number of coefficients to be generated by
        the chosen transform.

    nwct : int
        The total number of wavelet coefficients, n_ct, that will be
        generated. When `wtrans` = 'S' the number of rows required in
        each of the output coefficient arrays can be calculated as n_cm
        = n_ct/(8*n_cn*n_cfr). When `wtrans` = 'M' the length of the
        array used to store all of the coefficient matrices must be at
        least n_ct.

    nwcn : int
        For a single-level transform (`wtrans` = 'S'), the number of
        coefficients that would be generated in the second dimension,
        n_cn, for each coefficient type. For a multi-level transform
        (`wtrans` = 'M') this is set to 1.

    nwcfr : int
        For a single-level transform (`wtrans` = 'S'), the number of
        coefficients that would be generated in the third dimension,
        n_cfr, for each coefficient type. For a multi-level transform
        (`wtrans` = 'M') this is set to 1.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wtrans` = *<value>*.

            Constraint: `wtrans` = 'S' or 'M'.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 'P', 'H', 'W' or 'Z'.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 2.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 4)
            On entry, `fr` = *<value>*.

            Constraint: `fr` >= 2.
    """

    from ..base.wav import dim3_init as b_dim3_init

    _return_scalars = b_dim3_init(
        wavnam,
        wtrans,
        mode,
        m,
        n,
        fr,
    )
    _nwlmax_py = _return_scalars.nwlmax
    _nf_py = _return_scalars.nf
    _nwct_py = _return_scalars.nwct
    _nwcn_py = _return_scalars.nwcn
    _nwcfr_py = _return_scalars.nwcfr
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nwlmax', 'nf', 'nwct', 'nwcn', 'nwcfr', 'comm'],
    )(
        _nwlmax_py,
        _nf_py,
        _nwct_py,
        _nwcn_py,
        _nwcfr_py,
        _comm_py,
    )
    return _return_data

def dim1_cont(wavnam, wparam, x, scales):
    u"""
    One-dimensional real continuous wavelet transform.

    For full information please refer to the NAG Library document for
    c09ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09baf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'MORLET'

          Morlet wavelet.

        `wavnam` = 'DGAUSS'

          Derivative of a Gaussian wavelet.

        `wavnam` = 'MEXHAT'

          Mexican hat wavelet.

    wparam : int
        The nondimensional wave number for the Morlet wavelet or the
        order of the derivative for the Gaussian wavelet. It is not
        referenced when `wavnam` = 'MEXHAT'.

    x : float, array-like, shape (n)
        `x` contains the input dataset `x`[j-1] = x_j, for j =
        1,2,...,n.

    scales : int, array-like, shape (nscal)
        The scales at which the transform is to be computed.

    Returns
    -------
    c : float, ndarray, shape (nscal, n)
        The transform coefficients at the requested scales, where
        `c`[i-1,j-1] is the transform coefficient C_{i,j} at scale i and
        position j.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wavnam` = 'MORLET' and `wparam` = *<value>*.

            Constraint: if `wavnam` = 'MORLET', 5 <= `wparam` <= 20.

        (`errno` 2)
            On entry, `wavnam` = 'DGAUSS' and `wparam` = *<value>*.

            Constraint: if `wavnam` = 'DGAUSS', 1 <= `wparam` <= 8.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 5)
            On entry, nscal = *<value>*.

            Constraint: nscal >= 1.
    """

    from ..base.wav import dim1_cont as b_dim1_cont

    fun_name = 'naginterfaces.library.wav.dim1_cont'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _scales = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'scales'},
        scales,
        copy=False,
    )
    _nscal_py = _b_utils._EngineIntArrayType.get_shape(_scales, {'fun_name': fun_name, 'entity': 'scales'}, exp_rank=1)[0]
    _nscal = _b_utils._EngineIntScalarType(
        _nscal_py, {'fun_name': fun_name, 'entity': '_nscal'},
    ).to_ctypes()
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nscal_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_dim1_cont(
        wavnam,
        wparam,
        _n,
        _x,
        _nscal,
        _scales,
        _c,
    )
    _c_py = _c
    return _c_py

def dim1_sngl_fwd(x, comm):
    u"""
    One-dimensional discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09caf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x` contains the input dataset x_i, for i = 1,2,...,n.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    ca : float, ndarray, shape (:)
        `ca`[i-1] contains the ith approximation coefficient, C_a(i),
        for i = 1,2,...,n_c.

    cd : float, ndarray, shape (:)
        `cd`[i-1] contains the ith detail coefficient, C_d(i), for i =
        1,2,...,n_c.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n is inconsistent with the value passed to the
            initialization function: n = *<value>*, n should be
            *<value>*.

        (`errno` 6)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or array `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim1_sngl_fwd as b_dim1_sngl_fwd

    fun_name = 'naginterfaces.library.wav.dim1_sngl_fwd'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _lenc = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False), {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _ca = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
    )
    _cd = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
    )
    b_dim1_sngl_fwd(
        _n,
        _x,
        _lenc,
        _ca,
        _cd,
        comm,
    )
    _ca_py = _ca
    _cd_py = _cd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ca', 'cd'],
    )(
        _ca_py,
        _cd_py,
    )
    return _return_data

def dim1_sngl_inv(ca, cd, n, comm):
    u"""
    One-dimensional inverse discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09cbf.html

    Parameters
    ----------
    ca : float, array-like, shape (lenc)
        The n_c approximation coefficients, C_a. These will normally be
        the result of some transformation on the coefficients computed
        by ``dim1_sngl_fwd``.

    cd : float, array-like, shape (lenc)
        The n_c detail coefficients, C_d. These will normally be the
        result of some transformation on the coefficients computed by
        ``dim1_sngl_fwd``.

    n : int
        n, the length of the original data array from which the wavelet
        coefficients were computed by ``dim1_sngl_fwd`` and the length
        of the data array `y` that is to be reconstructed by this
        function.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    y : float, ndarray, shape (`n`)
        The reconstructed data based on approximation and detail
        coefficients C_a and C_d and the transform options supplied to
        the initialization function ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, array dimension lenc not large enough: lenc =
            *<value>* but must be at least *<value>*.

        (`errno` 4)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 6)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or array `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim1_sngl_inv as b_dim1_sngl_inv

    fun_name = 'naginterfaces.library.wav.dim1_sngl_inv'

    _ca = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ca'},
        ca,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_ca, {'fun_name': fun_name, 'entity': 'ca'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _cd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cd'},
        cd,
        copy=False,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dim1_sngl_inv(
        _lenc,
        _ca,
        _cd,
        _n,
        _y,
        comm,
    )
    _y_py = _y
    return _y_py

def dim1_multi_fwd(x, nwl, comm):
    u"""
    One-dimensional multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ccf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x` contains the one-dimensional input dataset x_i, for i =
        1,2,...,n.

    nwl : int
        The number of levels, n_fwd, in the multi-level resolution to be
        performed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    c : float, ndarray, shape (n_c)
        Let q(i) denote the number of coefficients (of each type)
        produced by the wavelet transform at level i, for i =
        n_fwd,...,1. These values are returned in `dwtlev`. Setting k_1
        = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-j+1), for j =
        1,2,...,n_fwd, the coefficients are stored as follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_1+1,...,k_2

          Contains the level n_fwd detail coefficients d_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-j+1 detail coefficients, for j =
          2,3,...,n_fwd.

    dwtlev : int, ndarray, shape (`nwl`+1)
        The number of transform coefficients at each level. `dwtlev`[0]
        and `dwtlev`[1] contain the number, q(n_fwd), of approximation
        and detail coefficients respectively, for the final level of
        resolution (these are equal); `dwtlev`[i-1] contains the number
        of detail coefficients, q(n_fwd-i+2), for the (n_fwd-i+2)th
        level, for i = 3,4,...,n_fwd+1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n is inconsistent with the value passed to the
            initialization function: n = *<value>*, n should be
            *<value>*.

        (`errno` 5)
            On entry, `nwl` is larger than the maximum number of levels
            returned by the initialization function: `nwl` = *<value>*,
            maximum = *<value>*.

        (`errno` 5)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 7)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 7)
            Either the initialization function was called with `wtrans`
            = 'S' or array `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim1_multi_fwd as b_dim1_multi_fwd

    fun_name = 'naginterfaces.library.wav.dim1_multi_fwd'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _lenc = _b_utils._EngineIntScalarType.c_type(0)
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _nwl = _b_utils._EngineIntScalarType(
        nwl, {'fun_name': fun_name, 'entity': 'nwl'},
    ).to_ctypes()
    _nwl_py = _b_utils._EngineIntScalarType.to_py(_nwl)
    _dwtlev = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nwl_py+1,),
    )
    _return_scalar = b_dim1_multi_fwd(
        _n,
        _x,
        _lenc,
        _c,
        _nwl,
        _dwtlev,
        comm,
    )
    _csize_py = _return_scalar
    _lenc = _b_utils._EngineIntScalarType(
        _csize_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _return_scalar = b_dim1_multi_fwd(
        _n,
        _x,
        _lenc,
        _c,
        _nwl,
        _dwtlev,
        comm,
    )
    _c_py = _c
    _dwtlev_py = _dwtlev
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['c', 'dwtlev'],
    )(
        _c_py,
        _dwtlev_py,
    )
    return _return_data

def dim1_multi_inv(nwlinv, c, n, comm):
    u"""
    One-dimensional inverse multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09cdf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim1_multi_fwd``. The data will be reconstructed to level
        (nwl-`nwlinv`), where level 0 is the original input dataset
        provided to ``dim1_multi_fwd``.

    c : float, array-like, shape (lenc)
        The coefficients of a multi-level wavelet transform of the
        dataset.

        Let q(i) be the number of coefficients (of each type) at level
        i, for i = n_fwd,...,1.

        Then, setting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-j+1), for
        j = 1,2,...,n_fwd, the coefficients are stored in `c` as
        follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_1+1,...,k_2

          Contains the level n_fwd detail coefficients d_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-j+1 detail coefficients, for j =
          2,3,...,n_fwd.

        The values q(i), for i = n_fwd,...,1, are contained in dwtlev
        which is produced as output by a preceding call to
        ``dim1_multi_fwd``.

        See ``dim1_multi_fwd`` for details.

    n : int
        n, the length of the data array, y, to be reconstructed. For a
        full reconstruction of nwl levels, where nwl is as supplied to
        ``dim1_multi_fwd``, this must be the same as argument n used in
        the call to ``dim1_multi_fwd``. For a partial reconstruction of
        `nwlinv` < nwl, this must be equal to dwtlev[`nwlinv`+1], as
        returned from ``dim1_multi_fwd``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    y : float, ndarray, shape (`n`)
        The dataset reconstructed from the multi-level wavelet transform
        coefficients and the transformation options supplied to the
        initialization function ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` is larger than the number of levels
            computed by the preceding call to ``dim1_multi_fwd``:
            `nwlinv` = *<value>*, expected = *<value>*.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 2)
            On entry, lenc is set too small: lenc = *<value>*.

            Constraint: lenc >= *<value>*.

        (`errno` 4)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 6)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or array `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim1_multi_inv as b_dim1_multi_inv

    fun_name = 'naginterfaces.library.wav.dim1_multi_inv'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dim1_multi_inv(
        nwlinv,
        _lenc,
        _c,
        _n,
        _y,
        comm,
    )
    _y_py = _y
    return _y_py

def dim1_mxolap_fwd(x, comm):
    u"""
    One-dimensional maximal overlap discrete wavelet transform (MODWT).

    For full information please refer to the NAG Library document for
    c09da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09daf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x` contains the input dataset x_i, for i = 1,2,...,n.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    ca : float, ndarray, shape (:)
        `ca`[i-1] contains the ith approximation coefficient, C_a(i),
        for i = 1,2,...,n_c.

    cd : float, ndarray, shape (:)
        `cd`[i-1] contains the ith detail coefficient, C_d(i), for i =
        1,2,...,n_c.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n is inconsistent with the value passed to the
            initialization function: n = *<value>*, n should be
            *<value>*.

        (`errno` 6)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'T', or the communication array `comm`['icomm'] has become
            corrupted.
    """

    from ..base.wav import dim1_mxolap_fwd as b_dim1_mxolap_fwd

    fun_name = 'naginterfaces.library.wav.dim1_mxolap_fwd'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _lenc = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False), {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _ca = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
    )
    _cd = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
    )
    b_dim1_mxolap_fwd(
        _n,
        _x,
        _lenc,
        _ca,
        _cd,
        comm,
    )
    _ca_py = _ca
    _cd_py = _cd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ca', 'cd'],
    )(
        _ca_py,
        _cd_py,
    )
    return _return_data

def dim1_mxolap_inv(ca, cd, n, comm):
    u"""
    One-dimensional inverse maximal overlap discrete wavelet transform
    (IMODWT).

    For full information please refer to the NAG Library document for
    c09db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09dbf.html

    Parameters
    ----------
    ca : float, array-like, shape (lenc)
        The n_c approximation coefficients, C_a. These will normally be
        the result of some transformation on the coefficients computed
        by ``dim1_mxolap_fwd``.

    cd : float, array-like, shape (lenc)
        The n_c detail coefficients, C_d. These will normally be the
        result of some transformation on the coefficients computed by
        ``dim1_mxolap_fwd``.

    n : int
        n, the length of the original data array from which the wavelet
        coefficients were computed by ``dim1_mxolap_fwd`` and the length
        of the data array `y` that is to be reconstructed by this
        function.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    y : float, ndarray, shape (`n`)
        The reconstructed data based on approximation and detail
        coefficients C_a and C_d and the transform options supplied to
        the initialization function ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, array dimension lenc not large enough: lenc =
            *<value>* but must be at least *<value>*.

        (`errno` 4)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 6)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'T', or the communication array `comm`['icomm'] has become
            corrupted.
    """

    from ..base.wav import dim1_mxolap_inv as b_dim1_mxolap_inv

    fun_name = 'naginterfaces.library.wav.dim1_mxolap_inv'

    _ca = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ca'},
        ca,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_ca, {'fun_name': fun_name, 'entity': 'ca'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _cd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cd'},
        cd,
        copy=False,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dim1_mxolap_inv(
        _lenc,
        _ca,
        _cd,
        _n,
        _y,
        comm,
    )
    _y_py = _y
    return _y_py

def dim1_mxolap_multi_fwd(x, keepa, nwl, comm):
    u"""
    One-dimensional multi-level maximal overlap discrete wavelet
    transform (MODWT).

    For full information please refer to the NAG Library document for
    c09dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09dcf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x` contains the input dataset x_i, for i = 1,2,...,n.

    keepa : str, length 1
        Determines whether the approximation coefficients are stored in
        array `c` for every level of the computed transform or else only
        for the final level. In both cases, the detail coefficients are
        stored in `c` for every level computed.

        `keepa` = 'A'

          Retain approximation coefficients for all levels computed.

        `keepa` = 'F'

          Retain approximation coefficients for only the final level
          computed.

    nwl : int
        The number of levels, n_l, in the multi-level resolution to be
        performed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    c : float, ndarray, shape (lenc)
        The coefficients of a multi-level wavelet transform of the
        dataset.

        The coefficients are stored in this argument as follows:

        If `keepa` = 'F',

        `c`[0:n_a]

          Contains the level n_l approximation coefficients;

        `c`[ n_a+(i-1)*n_d+1 -1: n_a+i*n_d ]

          Contains the level (n_l-i+1) detail coefficients, for i =
          1,2,...,n_l;

        If `keepa` = 'A',

        `c`[ (i-1)*n_a+1 -1: i*n_a ]

          Contains the level (n_l-i+1) approximation coefficients, for i
          = 1,2,...,n_l;

        `c`[ n_l*n_a+(i-1)*n_d+1 -1: n_l*n_a+i*n_d ]

          Contains the level i detail coefficients, for i = 1,2,...,n_l;

        The values n_a and n_d denote the numbers of approximation and
        detail coefficients respectively, which are equal and returned
        in `na`.

    na : int
        `na` contains the number of approximation coefficients, n_a, at
        each level which is equal to the number of detail coefficients,
        n_d. With periodic end extension (`mode` = 'P' in ``dim1_init``)
        this is the same as the length, n, of the data array, `x`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n is inconsistent with the value passed to the
            initialization function: n = *<value>*, n should be
            *<value>*.

        (`errno` 6)
            On entry, `nwl` is larger than the maximum number of levels
            returned by the initialization function: `nwl` = *<value>*,
            maximum = *<value>*.

        (`errno` 6)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 8)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'U', or the communication array `comm`['icomm'] has become
            corrupted.

        (`errno` 31)
            On entry, `keepa` = *<value>*.

            Constraint: `keepa` = 'F' or 'A'.
    """

    from ..base.wav import dim1_mxolap_multi_fwd as b_dim1_mxolap_multi_fwd

    fun_name = 'naginterfaces.library.wav.dim1_mxolap_multi_fwd'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _lenc = _b_utils._EngineIntScalarType.c_type(0)
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _return_scalars = b_dim1_mxolap_multi_fwd(
        _n,
        _x,
        keepa,
        _lenc,
        _c,
        nwl,
        comm,
    )
    _csize_py = _return_scalars.csize
    _lenc = _b_utils._EngineIntScalarType(
        _csize_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _return_scalars = b_dim1_mxolap_multi_fwd(
        _n,
        _x,
        keepa,
        _lenc,
        _c,
        nwl,
        comm,
    )
    _c_py = _c
    _na_py = _return_scalars.na
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['c', 'na'],
    )(
        _c_py,
        _na_py,
    )
    return _return_data

def dim1_mxolap_multi_inv(nwlinv, keepa, c, n, comm):
    u"""
    One-dimensional inverse multi-level maximal overlap discrete wavelet
    transform (IMODWT).

    For full information please refer to the NAG Library document for
    c09dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ddf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim1_mxolap_multi_fwd``. The data will be reconstructed to
        level (nwl-`nwlinv`), where level 0 is the original input
        dataset provided to ``dim1_mxolap_multi_fwd``.

    keepa : str, length 1
        Determines whether the approximation coefficients are stored in
        array `c` for every level of the computed transform or else only
        for the final level. In both cases, the detail coefficients are
        stored in `c` for every level computed.

        `keepa` = 'A'

          Retain approximation coefficients for all levels computed.

        `keepa` = 'F'

          Retain approximation coefficients for only the final level
          computed.

    c : float, array-like, shape (lenc)
        The coefficients of a multi-level wavelet transform of the
        dataset.

        The coefficients are stored in `c` as follows:

        If `keepa` = 'F',

        `c`[0:n_a]

          Contains the level n_l approximation coefficients;

        `c`[ n_a + (i-1)*n_d+1 -1: n_a+i*n_d ]

          Contains the level (n_l-i+1) detail coefficients, for i =
          1,2,...,n_l;

        If `keepa` = 'A',

        `c`[(i-1) *n_a:i*n_a]

          Contains the level (n_l-i+1) approximation coefficients, for i
          = 1,2,...,n_l;

        `c`[n_l *n_a+(i-1) *n_d:n_l *n_a+i*n_d]

          Contains the level i detail coefficients, for i = 1,2,...,n_l.

        The values n_a and n_d denote the numbers of approximation and
        detail coefficients respectively, which are equal.

        This number is returned as output in na from a preceding call to
        ``dim1_mxolap_multi_fwd``.

        See ``dim1_mxolap_multi_fwd`` for details.

    n : int
        n, the length of the data array, y, to be reconstructed.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    y : float, ndarray, shape (`n`)
        The dataset reconstructed from the multi-level wavelet transform
        coefficients and the transformation options supplied to the
        initialization function ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` is larger than the number of levels
            computed by the preceding call to ``dim1_mxolap_multi_fwd``:
            `nwlinv` = *<value>*, expected *<value>*.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 3)
            On entry, lenc is set too small: lenc = *<value>*.

            Constraint: lenc >= *<value>*.

        (`errno` 5)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 7)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'U', or the communication array `comm`['icomm'] has become
            corrupted.

        (`errno` 21)
            On entry, `keepa` = *<value>*.

            Constraint: `keepa` = 'F' or 'A'.
    """

    from ..base.wav import dim1_mxolap_multi_inv as b_dim1_mxolap_multi_inv

    fun_name = 'naginterfaces.library.wav.dim1_mxolap_multi_inv'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dim1_mxolap_multi_inv(
        nwlinv,
        keepa,
        _lenc,
        _c,
        _n,
        _y,
        comm,
    )
    _y_py = _y
    return _y_py

def dim2_sngl_fwd(a, comm):
    u"""
    Two-dimensional discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09eaf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m by n data matrix A.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Returns
    -------
    ca : float, ndarray, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of approximation coefficients,
        C_a.

    ch : float, ndarray, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of horizontal coefficients,
        C_h.

    cv : float, ndarray, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of vertical coefficients, C_v.

    cd : float, ndarray, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of diagonal coefficients, C_d.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n = *<value>*, the value of n on initialization
            (see ``dim2_init``).

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m = *<value>*, the value of m on initialization
            (see ``dim2_init``).

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim2_sngl_fwd as b_dim2_sngl_fwd

    fun_name = 'naginterfaces.library.wav.dim2_sngl_fwd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    _vl_n_cn_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    _ca = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_n_cm_py, _vl_n_cn_py),
        sorder=_m_sorder,
    )
    _ch = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_n_cm_py, _vl_n_cn_py),
        sorder=_m_sorder,
    )
    _cv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_n_cm_py, _vl_n_cn_py),
        sorder=_m_sorder,
    )
    _cd = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_n_cm_py, _vl_n_cn_py),
        sorder=_m_sorder,
    )
    b_dim2_sngl_fwd(
        _m,
        _n,
        _a,
        _ca,
        _ch,
        _cv,
        _cd,
        comm,
    )
    _ca_py = _ca
    _ch_py = _ch
    _cv_py = _cv
    _cd_py = _cd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ca', 'ch', 'cv', 'cd'],
    )(
        _ca_py,
        _ch_py,
        _cv_py,
        _cd_py,
    )
    return _return_data

def dim2_sngl_inv(m, n, ca, ch, cv, cd, comm):
    u"""
    Two-dimensional inverse discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ebf.html

    Parameters
    ----------
    m : int
        Number of rows, m, of data matrix B.

    n : int
        Number of columns, n, of data matrix B.

    ca : float, array-like, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of approximation coefficients,
        C_a. This array will normally be the result of some
        transformation on the coefficients computed by function
        ``dim2_sngl_fwd``.

    ch : float, array-like, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of horizontal coefficients,
        C_h. This array will normally be the result of some
        transformation on the coefficients computed by function
        ``dim2_sngl_fwd``.

    cv : float, array-like, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of vertical coefficients, C_v.
        This array will normally be the result of some transformation on
        the coefficients computed by function ``dim2_sngl_fwd``.

    cd : float, array-like, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of diagonal coefficients, C_d.
        This array will normally be the result of some transformation on
        the coefficients computed by function ``dim2_sngl_fwd``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Returns
    -------
    b : float, ndarray, shape (`m`, `n`)
        The m by n reconstructed matrix, B, based on the input
        approximation, horizontal, vertical and diagonal coefficients
        and the transform options supplied to the initialization
        function ``dim2_init``.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of n on
            initialization (see ``dim2_init``).

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of m on
            initialization (see ``dim2_init``).

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim2_sngl_inv as b_dim2_sngl_inv

    fun_name = 'naginterfaces.library.wav.dim2_sngl_inv'

    _ca = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ca'},
        ca,
        copy=False,
    )
    _ch = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ch'},
        ch,
        copy=False,
    )
    _cv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cv'},
        cv,
        copy=False,
    )
    _cd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cd'},
        cd,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ca, 'ca'), (_ch, 'ch'), (_cv, 'cv'), (_cd, 'cd')),
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_m_sorder,
    )
    b_dim2_sngl_inv(
        _m,
        _n,
        _ca,
        _ch,
        _cv,
        _cd,
        _b,
        comm,
    )
    _b_py = _b
    return _b_py

def dim2_multi_fwd(a, nwl, comm):
    u"""
    Two-dimensional multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ecf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m by n data matrix A.

    nwl : int
        The number of levels, n_fwd, in the multi-level resolution to be
        performed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Returns
    -------
    c : float, ndarray, shape (n_ct)
        The coefficients of the discrete wavelet transform. If you need
        to access or modify the approximation coefficients or any
        specific set of detail coefficients then the use of
        ``dim2_coeff_ext`` or ``dim2_coeff_ins`` is recommended. For
        completeness the following description provides details of
        precisely how the coefficient are stored in this argument but
        this information should only be required in rare cases.

        Let q(i) denote the number of coefficients (of each type) at
        level i, for i = 1,2,...,n_fwd, such that q(i) = `dwtlvm`[ n_fwd
        - i + 1 -1]*`dwtlvn`[ n_fwd - i + 1 -1].

        Then, letting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-⌈j/3⌉+1),
        for j = 1,2,...,3n_fwd, the coefficients are stored in this
        argument as follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-⌈j/3⌉+1 vertical, horizontal and
          diagonal coefficients. These are:

          - vertical coefficients if mod(j, 3) = 1;

          - horizontal coefficients if mod(j, 3) = 2;

          - diagonal coefficients if mod(j, 3) = 0,

        for j = 1,...,3*n_fwd.

    dwtlvm : int, ndarray, shape (`nwl`)
        The number of coefficients in the first dimension for each
        coefficient type at each level. `dwtlvm`[i-1] contains the
        number of coefficients in the first dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd. Thus for the first n_fwd-1
        levels of resolution, `dwtlvm`[n_fwd-i] is the size of the first
        dimension of the matrices of vertical, horizontal and diagonal
        coefficients computed at this level; for the final level of
        resolution, `dwtlvm`[0] is the size of the first dimension of
        the matrices of approximation, vertical, horizontal and diagonal
        coefficients computed.

    dwtlvn : int, ndarray, shape (`nwl`)
        The number of coefficients in the second dimension for each
        coefficient type at each level. `dwtlvn`[i-1] contains the
        number of coefficients in the second dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd. Thus for the first n_fwd-1
        levels of resolution, `dwtlvn`[n_fwd-i] is the size of the
        second dimension of the matrices of vertical, horizontal and
        diagonal coefficients computed at this level; for the final
        level of resolution, `dwtlvn`[0] is the size of the second
        dimension of the matrices of approximation, vertical, horizontal
        and diagonal coefficients computed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n = *<value>*, the value of n on initialization
            (see ``dim2_init``).

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m = *<value>*, the value of m on initialization
            (see ``dim2_init``).

        (`errno` 5)
            On entry, `nwl` = *<value>* and nwlmax = *<value>* in
            ``dim2_init``.

            Constraint: `nwl` <= nwlmax in ``dim2_init``.

        (`errno` 5)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 7)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 7)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim2_multi_fwd as b_dim2_multi_fwd

    fun_name = 'naginterfaces.library.wav.dim2_multi_fwd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _lenc = _b_utils._EngineIntScalarType.c_type(0)
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _nwl = _b_utils._EngineIntScalarType(
        nwl, {'fun_name': fun_name, 'entity': 'nwl'},
    ).to_ctypes()
    _nwl_py = _b_utils._EngineIntScalarType.to_py(_nwl)
    _dwtlvm = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nwl_py,),
    )
    _dwtlvn = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nwl_py,),
    )
    _return_scalar = b_dim2_multi_fwd(
        _m,
        _n,
        _a,
        _lenc,
        _c,
        _nwl,
        _dwtlvm,
        _dwtlvn,
        comm,
    )
    _csize_py = _return_scalar
    _lenc = _b_utils._EngineIntScalarType(
        _csize_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _return_scalar = b_dim2_multi_fwd(
        _m,
        _n,
        _a,
        _lenc,
        _c,
        _nwl,
        _dwtlvm,
        _dwtlvn,
        comm,
    )
    _c_py = _c
    _dwtlvm_py = _dwtlvm
    _dwtlvn_py = _dwtlvn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['c', 'dwtlvm', 'dwtlvn'],
    )(
        _c_py,
        _dwtlvm_py,
        _dwtlvn_py,
    )
    return _return_data

def dim2_multi_inv(nwlinv, c, m, n, comm):
    u"""
    Two-dimensional inverse multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ed

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09edf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim2_multi_fwd``. The data will be reconstructed to level
        (nwl-`nwlinv`), where level 0 is the original input dataset
        provided to ``dim2_multi_fwd``.

    c : float, array-like, shape (lenc)
        The coefficients of a multi-level wavelet transform of the
        original matrix, A, which may have been filtered or otherwise
        manipulated.

        Let q(i) be the number of coefficients (of each type) at level
        i, for i = n_fwd,...,1.

        Then, setting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-⌈j/3⌉+1),
        for j = 1,2,...,3n_fwd, the coefficients are stored in `c` as
        follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-⌈j/3⌉+1 vertical, horizontal and
          diagonal coefficients. These are:

          - vertical coefficients if mod(j, 3) = 1;

          - horizontal coefficients if mod(j, 3) = 2;

          - diagonal coefficients if mod(j, 3) = 0,

        for j = 1,...,3*n_fwd.

        Note that the coefficients in `c` may be extracted according to
        level and type into two-dimensional arrays using
        ``dim2_coeff_ext``, and inserted using ``dim2_coeff_ins``.

    m : int
        The number of elements, m, in the first dimension of the
        reconstructed matrix B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim2_multi_fwd``, this must be the
        same as argument m used in the call to ``dim2_multi_fwd``. For a
        partial reconstruction of `nwlinv` < nwl levels, this must be
        equal to dwtlvm[`nwlinv`], as returned from ``dim2_multi_fwd``.

    n : int
        The number of elements, n, in the second dimension of the
        reconstructed matrix B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument n used in the call to ``dim2_multi_fwd``. For a
        partial reconstruction of `nwlinv` < nwl, this must be equal to
        dwtlvn[`nwlinv`], as returned from ``dim2_multi_fwd``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Returns
    -------
    b : float, ndarray, shape (`m`, `n`)
        The m by n reconstructed matrix, B, based on the input
        multi-level wavelet transform coefficients and the transform
        options supplied to the initialization function ``dim2_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` = *<value>* and n_fwd = *<value>*.

            Constraint: `nwlinv` <= n_fwd.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 3)
            On entry, lenc = *<value>*.

            Constraint: lenc >= *<value>*, the total number of
            coefficients generated by the preceding call to
            ``dim2_multi_fwd``.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= *<value>*, the number of coefficients in
            the second dimension at the required level of
            reconstruction.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= *<value>*, the number of coefficients in
            the first dimension at the required level of reconstruction.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim2_multi_inv as b_dim2_multi_inv

    fun_name = 'naginterfaces.library.wav.dim2_multi_inv'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_dim2_multi_inv(
        nwlinv,
        _lenc,
        _c,
        _m,
        _n,
        _b,
        comm,
    )
    _b_py = _b
    return _b_py

def dim2_coeff_ext(ilev, cindex, c, comm):
    u"""
    Two-dimensional discrete wavelet transform coefficient extraction.

    For full information please refer to the NAG Library document for
    c09ey

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09eyf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be extracted.

    cindex : int
        Identifies which coefficients to extract. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns and rows of the original matrix
          (LL). The approximation coefficients are available only for
          `ilev` = nwl, where nwl is the value used in a preceding call
          to ``dim2_multi_fwd``.

        `cindex` = 1

          The vertical detail coefficients produced by applying the low
          pass filter over columns of the original matrix and the high
          pass filter over rows (LH).

        `cindex` = 2

          The horizontal detail coefficients produced by applying the
          high pass filter over columns of the original matrix and the
          low pass filter over rows (HL).

        `cindex` = 3

          The diagonal detail coefficients produced by applying the high
          pass filter over columns and rows of the original matrix (HH).

    c : float, array-like, shape (lenc)
        DWT coefficients, as computed by a preceding call to
        ``dim2_multi_fwd``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Returns
    -------
    d : float, ndarray, shape (n_cm, n_cn)
        The requested coefficients.

        If `ilev` = nwl (as used in ``dim2_multi_fwd``) and `cindex` =
        0, the n_cm by n_cn approximation coefficients a_{ij} are stored
        in `d`[i-1,j-1], for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

        Otherwise the n_cm by n_cn level `ilev` detail coefficients (of
        type specified by `cindex`) d_{ij} are stored in `d`[i-1,j-1],
        for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` >= 1.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim2_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 3.

        (`errno` 3)
            On entry, lenc = *<value>* and n_ct = *<value>*.

            Constraint: lenc >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim2_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim2_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim2_coeff_ext as b_dim2_coeff_ext

    fun_name = 'naginterfaces.library.wav.dim2_coeff_ext'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev = _b_utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _ilev_py = _b_utils._EngineIntScalarType.to_py(_ilev)
    _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _vl_n_cn_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((20+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_n_cm_py, _vl_n_cn_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_dim2_coeff_ext(
        _ilev,
        cindex,
        _lenc,
        _c,
        _d,
        comm,
    )
    _d_py = _d
    return _d_py

def dim2_coeff_ins(ilev, cindex, c, d, comm):
    u"""
    Two-dimensional discrete wavelet transform coefficient insertion.

    For full information please refer to the NAG Library document for
    c09ez

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ezf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be inserted.

    cindex : int
        Identifies which coefficients to insert. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns and rows of the original matrix
          (LL). The approximation coefficients are present only for
          `ilev` = nwl, where nwl is the value used in a preceding call
          to ``dim2_multi_fwd``.

        `cindex` = 1

          The vertical detail coefficients produced by applying the low
          pass filter over columns of the original matrix and the high
          pass filter over rows (LH).

        `cindex` = 2

          The horizontal detail coefficients produced by applying the
          high pass filter over columns of the original matrix and the
          low pass filter over rows (HL).

        `cindex` = 3

          The diagonal detail coefficients produced by applying the high
          pass filter over columns and rows of the original matrix (HH).

    c : float, array-like, shape (lenc)
        Contains the DWT coefficients inserted by previous calls to
        ``dim2_coeff_ins``, or computed by a previous call to
        ``dim2_multi_fwd``.

    d : float, array-like, shape (n_cm, n_cn)
        The coefficients to be inserted.

        If `ilev` = nwl (as used in ``dim2_multi_fwd``) and `cindex` =
        0, the n_cm by n_cn manipulated approximation coefficients
        a_{ij} must be stored in `d`[i-1,j-1], for i = 1,2,...,n_cn, for
        i = 1,2,...,n_cm.

        Otherwise the n_cm by n_cn manipulated level `ilev` detail
        coefficients (of type specified by `cindex`) d_{ij} must be
        stored in `d`[i-1,j-1], for j = 1,2,...,n_cn, for i =
        1,2,...,n_cm.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Returns
    -------
    c : float, ndarray, shape (lenc)
        Contains the same DWT coefficients provided on entry except for
        those identified by `ilev` and `cindex`, which are updated with
        the values supplied in `d`, inserted into the correct locations
        as expected by the reconstruction function ``dim2_multi_inv``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` >= 1.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim2_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 3.

        (`errno` 3)
            On entry, lenc = *<value>* and n_ct = *<value>*.

            Constraint: lenc >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim2_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim2_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim2_coeff_ins as b_dim2_coeff_ins

    fun_name = 'naginterfaces.library.wav.dim2_coeff_ins'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev = _b_utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _ilev_py = _b_utils._EngineIntScalarType.to_py(_ilev)
    _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
    )
    b_dim2_coeff_ins(
        _ilev,
        cindex,
        _lenc,
        _c,
        _d,
        comm,
    )
    _c_py = _c
    return _c_py

def dim3_sngl_fwd(a, comm):
    u"""
    Three-dimensional discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09faf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n, fr)
        The m by n by fr three-dimensional input data A, where A_{ijk}
        is stored in `a`[i-1,j-1,k-1].

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Returns
    -------
    c : float, ndarray, shape (:)
        The coefficients of the discrete wavelet transform. If you need
        to access or modify the approximation coefficients or any
        specific set of detail coefficients then the use of
        ``dim3_coeff_ext`` or ``dim3_coeff_ins`` is recommended. For
        completeness the following description provides details of
        precisely how the coefficients are stored in this argument but
        this information should only be required in rare cases.

        The 8 sets of coefficients are stored in the following order:
        approximation coefficients (LLL) first, followed by 7 sets of
        detail coefficients: LLH, LHL, LHH, HLL, HLH, HHL, HHH, where L
        indicates the low pass filter, and H the high pass filter being
        applied to, respectively, the columns of length m, the rows of
        length n and then the frames of length fr.

        Note that for computational efficiency reasons each set of
        coefficients is stored in the order n_cfr*n_cm*n_cn (see output
        arguments nwcfr, nwct and nwcn in ``dim3_init``).

        See [Example] for details of how to access each set of
        coefficients in order to perform extraction from this argument
        following a call to this function, or insertion into this
        argument before a call to the three-dimensional inverse function
        ``dim3_sngl_inv``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n = *<value>*, the value of n on initialization
            (see ``dim3_init``).

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m = *<value>*, the value of m on initialization
            (see ``dim3_init``).

        (`errno` 1)
            On entry, fr = *<value>*.

            Constraint: fr = *<value>*, the value of fr on
            initialization (see ``dim3_init``).

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or `wtrans` = 'M' was used in the prior call to
            the initialization function ``dim3_init``.
    """

    from ..base.wav import dim3_sngl_fwd as b_dim3_sngl_fwd

    fun_name = 'naginterfaces.library.wav.dim3_sngl_fwd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fr_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[2]
    _fr = _b_utils._EngineIntScalarType(
        _fr_py, {'fun_name': fun_name, 'entity': '_fr'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _lenc = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (10,), validate=False), {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (10,), validate=False),),
    )
    b_dim3_sngl_fwd(
        _m,
        _n,
        _fr,
        _a,
        _lenc,
        _c,
        comm,
    )
    _c_py = _c
    return _c_py

def dim3_sngl_inv(m, n, fr, c, comm):
    u"""
    Three-dimensional inverse discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fbf.html

    Parameters
    ----------
    m : int
        The number of rows of each two-dimensional frame.

    n : int
        The number of columns of each two-dimensional frame.

    fr : int
        The number two-dimensional frames.

    c : float, array-like, shape (lenc)
        The coefficients of the discrete wavelet transform. This will
        normally be the result of some transformation on the
        coefficients computed by function ``dim3_sngl_fwd``.

        Note that the coefficients in `c` may be extracted according to
        type into three-dimensional arrays using ``dim3_coeff_ext``, and
        inserted using ``dim3_coeff_ins``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Returns
    -------
    b : float, ndarray, shape (`m`, `n`, `fr`)
        The m by n by fr reconstructed array, B, with B_{ijk} stored in
        `b`[i-1,j-1,k-1]. The reconstruction is based on the input
        wavelet coefficients and the transform options supplied to the
        initialization function ``dim3_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of `n` on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of `m` on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `fr` = *<value>*.

            Constraint: `fr` = *<value>*, the value of `fr` on
            initialization (see ``dim3_init``).

        (`errno` 3)
            On entry, lenc = *<value>* and n_ct = *<value>*.

            Constraint: lenc >= n_ct, where n_ct is the number of DWT
            coefficients returned by ``dim3_init`` in argument nwct.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or `wtrans` = 'M' was used in the prior call to
            the initialization function ``dim3_init``.
    """

    from ..base.wav import dim3_sngl_inv as b_dim3_sngl_inv

    fun_name = 'naginterfaces.library.wav.dim3_sngl_inv'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _fr = _b_utils._EngineIntScalarType(
        fr, {'fun_name': fun_name, 'entity': 'fr'},
    ).to_ctypes()
    _fr_py = _b_utils._EngineIntScalarType.to_py(_fr)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py, _fr_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_dim3_sngl_inv(
        _m,
        _n,
        _fr,
        _lenc,
        _c,
        _b,
        comm,
    )
    _b_py = _b
    return _b_py

def dim3_multi_fwd(a, nwl, comm):
    u"""
    Three-dimensional multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fcf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n, fr)
        The m by n by fr three-dimensional input data A, where with
        A_{ijk} stored in `a`[i-1,j-1,k-1].

    nwl : int
        The number of levels, n_fwd, in the multi-level resolution to be
        performed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Returns
    -------
    c : float, ndarray, shape (n_ct)
        The coefficients of the discrete wavelet transform. If you need
        to access or modify the approximation coefficients or any
        specific set of detail coefficients then the use of
        ``dim3_coeff_ext`` or ``dim3_coeff_ins`` is recommended. For
        completeness the following description provides details of
        precisely how the coefficients are stored in this argument but
        this information should only be required in rare cases.

        Let q(i) denote the number of coefficients of each type at level
        i, for i = 1,2,...,n_fwd, such that q(i) =
        `dwtlvm`[n_fwd-i]*`dwtlvn`[n_fwd-i]*`dwtlvfr`[n_fwd-i].

        Then, letting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-⌈j/7⌉+1),
        for j = 1,2,...,7n_fwd, the coefficients are stored in this
        argument as follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.
          Note that for computational efficiency reasons these
          coefficients are stored as
          `dwtlvm`[0]*`dwtlvn`[0]*`dwtlvfr`[0] in this argument.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-⌈j/7⌉+1 detail coefficients. These
          are:

          - LLH coefficients if mod(j, 7) = 1;

          - LHL coefficients if mod(j, 7) = 2;

          - LHH coefficients if mod(j, 7) = 3;

          - HLL coefficients if mod(j, 7) = 4;

          - HLH coefficients if mod(j, 7) = 5;

          - HHL coefficients if mod(j, 7) = 6;

          - HHH coefficients if mod(j, 7) = 0,

        for j = 1,...,7*n_fwd. See [the C09 Introduction] for a
        description of how these coefficients are produced.

          Note that for computational efficiency reasons these
          coefficients are stored as `dwtlvfr`[ ⌈j/7⌉ -1]*`dwtlvm`[
          ⌈j/7⌉ -1]*`dwtlvn`[ ⌈j/7⌉ -1] in this argument.

    dwtlvm : int, ndarray, shape (`nwl`)
        The number of coefficients in the first dimension for each
        coefficient type at each level. `dwtlvm`[i-1] contains the
        number of coefficients in the first dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd.

    dwtlvn : int, ndarray, shape (`nwl`)
        The number of coefficients in the second dimension for each
        coefficient type at each level. `dwtlvn`[i-1] contains the
        number of coefficients in the second dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd.

    dwtlvfr : int, ndarray, shape (`nwl`)
        The number of coefficients in the third dimension for each
        coefficient type at each level. `dwtlvfr`[i-1] contains the
        number of coefficients in the third dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n = *<value>*, the value of n on initialization
            (see ``dim3_init``).

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m = *<value>*, the value of m on initialization
            (see ``dim3_init``).

        (`errno` 1)
            On entry, fr = *<value>*.

            Constraint: fr = *<value>*, the value of fr on
            initialization (see ``dim3_init``).

        (`errno` 5)
            On entry, `nwl` = *<value>* and nwlmax = *<value>* in
            ``dim3_init``.

            Constraint: `nwl` <= nwlmax in ``dim3_init``.

        (`errno` 5)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or the communication array `comm`['icomm'] has been
            corrupted.
    """

    from ..base.wav import dim3_multi_fwd as b_dim3_multi_fwd

    fun_name = 'naginterfaces.library.wav.dim3_multi_fwd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fr_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[2]
    _fr = _b_utils._EngineIntScalarType(
        _fr_py, {'fun_name': fun_name, 'entity': '_fr'},
    ).to_ctypes()
    _lenc = _b_utils._EngineIntScalarType.c_type(0)
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _nwl = _b_utils._EngineIntScalarType(
        nwl, {'fun_name': fun_name, 'entity': 'nwl'},
    ).to_ctypes()
    _nwl_py = _b_utils._EngineIntScalarType.to_py(_nwl)
    _dwtlvm = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nwl_py,),
    )
    _dwtlvn = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nwl_py,),
    )
    _dwtlvfr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nwl_py,),
    )
    _return_scalar = b_dim3_multi_fwd(
        _m,
        _n,
        _fr,
        _a,
        _lenc,
        _c,
        _nwl,
        _dwtlvm,
        _dwtlvn,
        _dwtlvfr,
        comm,
    )
    _csize_py = _return_scalar
    _lenc = _b_utils._EngineIntScalarType(
        _csize_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _lenc_py = _b_utils._EngineIntScalarType.to_py(_lenc)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lenc_py,),
    )
    _return_scalar = b_dim3_multi_fwd(
        _m,
        _n,
        _fr,
        _a,
        _lenc,
        _c,
        _nwl,
        _dwtlvm,
        _dwtlvn,
        _dwtlvfr,
        comm,
    )
    _c_py = _c
    _dwtlvm_py = _dwtlvm
    _dwtlvn_py = _dwtlvn
    _dwtlvfr_py = _dwtlvfr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['c', 'dwtlvm', 'dwtlvn', 'dwtlvfr'],
    )(
        _c_py,
        _dwtlvm_py,
        _dwtlvn_py,
        _dwtlvfr_py,
    )
    return _return_data

def dim3_mxolap_multi_inv(nwlinv, c, m, n, fr, comm):
    u"""
    Three-dimensional inverse multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fdf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim3_multi_fwd``. The data will be reconstructed to level
        (nwl-`nwlinv`), where level 0 is the original input dataset
        provided to ``dim3_multi_fwd``.

    c : float, array-like, shape (lenc)
        The coefficients of the multi-level discrete wavelet transform.
        This will normally be the result of some transformation on the
        coefficients computed by function ``dim3_multi_fwd``.

        Note that the coefficients in `c` may be extracted according to
        level and type into three-dimensional arrays using
        ``dim3_coeff_ext``, and inserted using ``dim3_coeff_ins``.

    m : int
        The number of elements, m, in the first dimension of the
        reconstructed array B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument m used in a preceding call to
        ``dim3_multi_fwd``. For a partial reconstruction of `nwlinv` <
        nwl levels, this must be equal to dwtlvm[`nwlinv`], as returned
        from ``dim3_multi_fwd``

    n : int
        The number of elements, n, in the second dimension of the
        reconstructed array B. For a full reconstruction of nwl, levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument n used in a preceding call to
        ``dim3_multi_fwd``. For a partial reconstruction of `nwlinv` <
        nwl levels, this must be equal to dwtlvn[`nwlinv`], as returned
        from ``dim3_multi_fwd``.

    fr : int
        The number of elements, fr, in the third dimension of the
        reconstructed array B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument `fr` used in a preceding call to
        ``dim3_multi_fwd``. For a partial reconstruction of `nwlinv` <
        nwl levels, this must be equal to dwtlvfr[`nwlinv`], as returned
        from ``dim3_multi_fwd``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Returns
    -------
    b : float, ndarray, shape (`m`, `n`, `fr`)
        The m by n by fr reconstructed array, B, with B_{ijk} stored in
        `b`[i-1,j-1,k-1]. The reconstruction is based on the input
        multi-level wavelet transform coefficients and the transform
        options supplied to the initialization function ``dim3_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` = *<value>* and nwl = *<value>* where nwl
            is as used in the computation of the wavelet coefficients by
            a call to ``dim3_multi_fwd``.

            Constraint: `nwlinv` <= nwl as used in the call to
            ``dim3_multi_fwd``.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 3)
            On entry, lenc = *<value>*.

            Constraint: lenc >= *<value>*, the number of wavelet
            coefficients required for a transform operating on `nwlinv`
            levels. If `nwlinv` = nwlmax, the maximum number of levels
            as returned by the initial call to ``dim3_init``, lenc must
            be at least n_ct, the value returned in nwct by the same
            call to ``dim3_init``.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= *<value>*, the number of coefficients in
            the second dimension at the required level of
            reconstruction.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= *<value>*, the number of coefficients in
            the first dimension at the required level of reconstruction.

        (`errno` 4)
            On entry, `fr` = *<value>*.

            Constraint: `fr` >= *<value>*, the number of coefficients in
            the third dimension at the required level of reconstruction.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or the communication array `comm`['icomm'] has been
            corrupted.
    """

    from ..base.wav import dim3_mxolap_multi_inv as b_dim3_mxolap_multi_inv

    fun_name = 'naginterfaces.library.wav.dim3_mxolap_multi_inv'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _fr = _b_utils._EngineIntScalarType(
        fr, {'fun_name': fun_name, 'entity': 'fr'},
    ).to_ctypes()
    _fr_py = _b_utils._EngineIntScalarType.to_py(_fr)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py, _fr_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_dim3_mxolap_multi_inv(
        nwlinv,
        _lenc,
        _c,
        _m,
        _n,
        _fr,
        _b,
        comm,
    )
    _b_py = _b
    return _b_py

def dim3_coeff_ext(ilev, cindex, c, comm):
    u"""
    Three-dimensional discrete wavelet transform coefficient extraction.

    For full information please refer to the NAG Library document for
    c09fy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fyf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be extracted.

        If `ilev` = 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the single level function
        ``dim3_sngl_fwd``.

        If `ilev` > 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the multi-level function
        ``dim3_multi_fwd``.

    cindex : int
        Identifies which coefficients to extract. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns, rows and frames of A (LLL).
          After a call to the multi-level transform function
          ``dim3_multi_fwd`` (which implies that `ilev` > 0) the
          approximation coefficients are available only for `ilev` =
          nwl, where nwl is the value used in a preceding call to
          ``dim3_multi_fwd``.

        `cindex` = 1

          The detail coefficients produced by applying the low pass
          filter over columns and rows of A and the high pass filter
          over frames (LLH).

        `cindex` = 2

          The detail coefficients produced by applying the low pass
          filter over columns, high pass filter over rows and low pass
          filter over frames of A (LHL).

        `cindex` = 3

          The detail coefficients produced by applying the low pass
          filter over columns of A and high pass filter over rows and
          frames (LHH).

        `cindex` = 4

          The detail coefficients produced by applying the high pass
          filter over columns of A and low pass filter over rows and
          frames (HLL).

        `cindex` = 5

          The detail coefficients produced by applying the high pass
          filter over columns, low pass filter over rows and high pass
          filter over frames of A (HLH).

        `cindex` = 6

          The detail coefficients produced by applying the high pass
          filter over columns and rows of A and the low pass filter over
          frames (HHL).

        `cindex` = 7

          The detail coefficients produced by applying the high pass
          filter over columns, rows and frames of A (HHH).

    c : float, array-like, shape (lenc)
        DWT coefficients, as computed by ``dim3_sngl_fwd`` or
        ``dim3_multi_fwd``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Returns
    -------
    d : float, ndarray, shape (n_cm, n_cn, n_cfr)
        The requested coefficients.

        If the DWT coefficients were computed by ``dim3_sngl_fwd`` then

        - if `cindex` = 0, the approximation coefficients are stored in
          `d`[i-1,j-1,k-1], for k = 1,2,...,n_cfr, for j = 1,2,...,n_cn,
          for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, are stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

        If the DWT coefficients were computed by ``dim3_multi_fwd`` then

        - if `cindex` = 0 and `ilev` = nwl, the approximation
          coefficients are stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, for level `ilev` are stored in `d`[i-1,j-1,k-1],
          for k = 1,2,...,n_cfr, for j = 1,2,...,n_cn, for i =
          1,2,...,n_cm.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` = 0 following a call to the single level
            function ``dim3_sngl_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` > 0 following a call to the multi-level
            function ``dim3_multi_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim3_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 7.

        (`errno` 3)
            On entry, lenc = *<value>* and n_ct = *<value>*.

            Constraint: lenc >= n_ct, where n_ct is the number of DWT
            coefficients computed in the preceding call to
            ``dim3_sngl_fwd``.

        (`errno` 3)
            On entry, lenc = *<value>* and n_ct = *<value>*.

            Constraint: lenc >= n_ct, where n_ct is the number of DWT
            coefficients computed in the preceding call to
            ``dim3_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim3_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim3_coeff_ext as b_dim3_coeff_ext

    fun_name = 'naginterfaces.library.wav.dim3_coeff_ext'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev = _b_utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _ilev_py = _b_utils._EngineIntScalarType.to_py(_ilev)
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    else:
        _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cn_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _vl_n_cn_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((20+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cfr_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (12,), validate=False)
    else:
        _vl_n_cfr_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((180+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_n_cm_py, _vl_n_cn_py, _vl_n_cfr_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_dim3_coeff_ext(
        _ilev,
        cindex,
        _lenc,
        _c,
        _d,
        comm,
    )
    _d_py = _d
    return _d_py

def dim3_coeff_ins(ilev, cindex, c, d, comm):
    u"""
    Three-dimensional discrete wavelet transform coefficient insertion.

    For full information please refer to the NAG Library document for
    c09fz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fzf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be inserted.

        If `ilev` = 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the single level function
        ``dim3_sngl_fwd``.

        If `ilev` > 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the multi-level function
        ``dim3_multi_fwd``.

    cindex : int
        Identifies which coefficients to insert. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns, rows and frames of A (LLL).
          After a call to the multi-level transform function
          ``dim3_multi_fwd`` (which implies that `ilev` > 0) the
          approximation coefficients are present only for `ilev` = nwl,
          where nwl is the value used in a preceding call to
          ``dim3_multi_fwd``.

        `cindex` = 1

          The detail coefficients produced by applying the low pass
          filter over columns and rows of A and the high pass filter
          over frames (LLH).

        `cindex` = 2

          The detail coefficients produced by applying the low pass
          filter over columns, high pass filter over rows and low pass
          filter over frames of A (LHL).

        `cindex` = 3

          The detail coefficients produced by applying the low pass
          filter over columns of A and high pass filter over rows and
          frames (LHH).

        `cindex` = 4

          The detail coefficients produced by applying the high pass
          filter over columns of A and low pass filter over rows and
          frames (HLL).

        `cindex` = 5

          The detail coefficients produced by applying the high pass
          filter over columns, low pass filter over rows and high pass
          filter over frames of A (HLH).

        `cindex` = 6

          The detail coefficients produced by applying the high pass
          filter over columns and rows of A and the low pass filter over
          frames (HHL).

        `cindex` = 7

          The detail coefficients produced by applying the high pass
          filter over columns, rows and frames of A (HHH).

    c : float, array-like, shape (lenc)
        Contains the DWT coefficients inserted by previous calls to
        ``dim3_coeff_ins``, or computed by a previous call to either
        ``dim3_sngl_fwd`` or ``dim3_multi_fwd``.

    d : float, array-like, shape (n_cm, n_cn, n_cfr)
        The coefficients to be inserted.

        If the DWT coefficients were computed by ``dim3_sngl_fwd`` then

        - if `cindex` = 0, the approximation coefficients must be stored
          in `d`[i-1,j-1,k-1], for k = 1,2,...,n_cfr, for j =
          1,2,...,n_cn, for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, must be stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

        If the DWT coefficients were computed by ``dim3_multi_fwd`` then

        - if `cindex` = 0 and `ilev` = nwl, the approximation
          coefficients must be stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, for level `ilev` must be stored in
          `d`[i-1,j-1,k-1], for k = 1,2,...,n_cfr, for j = 1,2,...,n_cn,
          for i = 1,2,...,n_cm.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Returns
    -------
    c : float, ndarray, shape (lenc)
        Contains the same DWT coefficients provided on entry except for
        those identified by `ilev` and `cindex`, which are updated with
        the values supplied in `d`, inserted into the correct locations
        as expected by one of the reconstruction functions
        ``dim3_sngl_inv`` (if ``dim3_sngl_fwd`` was called previously)
        or ``dim3_mxolap_multi_inv`` (if ``dim3_multi_fwd`` was called
        previously).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` = 0 following a call to the single level
            function ``dim3_sngl_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` > 0 following a call to the multi-level
            function ``dim3_multi_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim3_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 7.

        (`errno` 3)
            On entry, lenc = *<value>* and n_ct = *<value>*.

            Constraint: lenc >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim3_sngl_fwd``.

        (`errno` 3)
            On entry, lenc = *<value>* and n_ct = *<value>*.

            Constraint: lenc >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim3_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim3_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.
    """

    from ..base.wav import dim3_coeff_ins as b_dim3_coeff_ins

    fun_name = 'naginterfaces.library.wav.dim3_coeff_ins'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _lenc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _lenc = _b_utils._EngineIntScalarType(
        _lenc_py, {'fun_name': fun_name, 'entity': '_lenc'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev = _b_utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _ilev_py = _b_utils._EngineIntScalarType.to_py(_ilev)
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    else:
        _vl_n_cm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cn_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _vl_n_cn_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((20+_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
    )
    b_dim3_coeff_ins(
        _ilev,
        cindex,
        _lenc,
        _c,
        _d,
        comm,
    )
    _c_py = _c
    return _c_py
