"""
Deprecated wrappers for the public NAG C Library API, Mark 26.0
algorithmic contents.

nag4py
======

Deprecated Python wrappers to the NAG C Library Mark 26.0 contents

https://www.nag.com/numeric/CL/nagdoc_cl26/html/FRONTMATTER/manconts.html

(Functions withdrawn from the NAG C Library since Mark 26.0 will also
not be present in this subpackage.)

The subpackage will raise RuntimeError if the integer width used in the
associated NAG C Library does not match that used in the target numpy
installation.

Basic Usage
===========

Each Chapter from the NAG C Library is available as a submodule:
for example, for the Roots of One or More Transcendental Equations
(c05) Chapter

 >>> import naginterfaces.nag4py.c05
 >>> help(naginterfaces.nag4py.c05) # doctest: +SKIP

The util submodule supplies definitions for the support
objects - structures, named constants, and so on - needed by some
functions. For example, to find the zero of a continuous function
using c05auc, the Python function whose zero is to be found must
be cast using NAG_C05AUC_FUN to make it suitable for passing to
the nag4py wrapper. You may import this function-type object
directly using

 >>> from naginterfaces.nag4py.util import NAG_C05AUC_FUN

or indirectly from the util submodule via the c05 submodule

 >>> from naginterfaces.nag4py.c05 import NAG_C05AUC_FUN

Error or warning cases detected by the NAG C Library are handled
in nag4py using a similar NagError mechanism as in the Library
itself. The util submodule provides two convenience functions
('quiet_fail' and 'noisy_fail') to create a NagError instance with
printing of messages disabled or enabled, respectively.

Upon exit from a nag4py function that has a 'fail' argument you
are strongly advised to test the 'code' member of 'fail'. This
will facilitate programmatical management of success or failure
from the function.

Further Usage Notes
===================

The nag4py wrappers strive to provide interfaces that match those
in the NAG C Library as closely as possible. As a consequence
there are some additional usage points to be aware of.

* Output arguments must be declared before appearing in a nag4py
  call; i.e., these arguments appear on the right-hand side of the
  Python call.

* Integer, real and complex arguments to the NAG C Library may
  need to be declared before a nag4py Python call to have the
  correct data type.

  Specifically this is the case for
    integer: Input array, Input/Output any rank, Output any rank;
    real: Input/Output any rank, Output any rank;
    complex: any.

  See the documentation of numpy.ndarray for more details. (The
  numpy.ndarray default dtype is float, hence this must be
  overridden explictly when declaring other data.)

  A nag_int_type is provided, which matches the integer type of
  the NAG C Library. This should be used as the nag4py integer
  type for casting.

  Enumerated types should be cast to the correct NAG enumeration
  name.

  A Complex class is provided, and this should be used as the
  nag4py complex type for casting.

  TypeError will be raised by the nag4py wrapper whenever an
  argument is incorrectly typed.

* A nag_opt_handle_object function is provided and should be used
  to declare 'handle' arguments to the NAG C Library.

  TypeError will be raised by the nag4py wrapper whenever a
  handle argument is incorrectly typed.

* Arguments listed as Input/Output or Output by the NAG C Library
  must be declared before a nag4py Python call to be instances of
  numpy.ndarray (or a subclass thereof) to achieve
  pass-by-reference correctly. This holds for scalar and array
  data alike.

  Complex data cast in this way (resulting from a call to, say,
  numpy.array) will appear as numpy.void. The real and imaginary
  parts of such an object must then be referenced as array element
  0 and 1 of the object, respectively: the fields 're' and 'im' of
  the object will no longer exist.

  TypeError will be raised by the nag4py wrapper whenever a
  numpy.ndarray is expected but not received.

* Arguments listed as Input/Output or Output by the NAG C Library
  will on output from a nag4py Python call be modified in place
  (on the Python function's right-hand side).

* Members of structures from the NAG C Library must be accessed as
  ctypes objects, not as native Python types.

  The function nag_char_cast in the util submodule should be
  used to cast arrays (e.g., lists or tuples) of strings to the
  correct ctypes (char **) format for insertion into structures when
  necessary.

* Data from callback functions has to be returned in the C-like
  by-reference fashion, with scalar data having to appear as an
  array of length 1 to facilitate this.

* The facility in the NAG C Library where NULL may be used in
  some functions to represent non-present non-scalar optional
  data is available for character, integer and real data in
  the nag4py interfaces.  To use this facility you may pass None
  as the value for such arguments.  Note that when not supported
  this may mean certain selections of algorithms are unavailable
  (e.g., the computation selected in the NAG C Library upon
  specifying arimav=NULL on entry to the Library function
  g13bec).

* The NULLFN mechanism used in certain places in the NAG C
  Library to select an optional callback function is also
  available in the nag4py interfaces, again by passing None for
  the callback in question.

* In Python 3 string arguments to nag4py Python functions must be
  byte strings.

* Some wrapper functions may be unavailable or not supported on
  certain systems. If that is the case, ImportError will be
  raised.

Available Submodules
====================

a00 - Library Identification.
a02 - Complex Arithmetic.
c02 - Zeros of Polynomials.
c05 - Roots of One or More Transcendental Equations.
c06 - Fourier Transforms.
c09 - Wavelet Transforms.
d01 - Quadrature.
d02 - Ordinary Differential Equations.
d03 - Partial Differential Equations.
d04 - Numerical Differentiation.
d05 - Integral Equations.
d06 - Mesh Generation.
e01 - Interpolation.
e02 - Curve and Surface Fitting.
e04 - Minimizing or Maximizing a Function.
e05 - Global Optimization of a Function.
f01 - Matrix Operations, Including Inversion.
f02 - Eigenvalues and Eigenvectors.
f03 - Determinants.
f04 - Simultaneous Linear Equations.
f06 - Linear Algebra Support Functions.
f07 - Linear Equations (LAPACK).
f08 - Least Squares and Eigenvalue Problems (LAPACK).
f11 - Large Scale Linear Systems.
f12 - Large Scale Eigenproblems.
f16 - NAG Interface to BLAS.
g01 - Simple Calculations on Statistical Data.
g02 - Correlation and Regression Analysis.
g03 - Multivariate Methods.
g04 - Analysis of Variance.
g05 - Random Number Generators.
g07 - Univariate Estimation.
g08 - Nonparametric Statistics.
g10 - Smoothing in Statistics.
g11 - Contingency Table Analysis.
g12 - Survival Analysis.
g13 - Time Series Analysis.
h02 - Integer Programming.
h03 - Transportation and Shortest Path Problems.
h05 - Feature Selection.
m01 - Sorting and Searching.
s - Approximations of Special Functions.
util - Definitions of support objects.
x01 - Mathematical Constants.
x02 - Machine Constants.
x04 - Input/Output Utilities.
x06 - OpenMP Utilities.
x07 - IEEE Arithmetic.

Further Support
===============

Please direct any feedback or queries to one of the NAG support desks:

NAG Ltd (Oxford, UK): support@nag.co.uk

NAG Inc (Lisle, IL): support@nag.com

Nihon NAG KK (Tokyo, JP): naghelp@nag-j.co.jp

NAG Copyright 2017.
"""

from ..base import utils as b_utils

b_utils._warnings.warn(
    b_utils.NagWithdrawalWarning(
        {},
        'This subpackage is scheduled for withdrawal, no earlier than Mark 28.',
    )
)
