"""
Deprecated wrappers for the NAG C Library (Mark 26.0) c05 Chapter.

c05 - Roots of One or More Transcendental Equations.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/C05/c05conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Nag_Comm
from .util import NagError
from .util import NAG_C05AUC_FUN
from .util import Integer
from .util import NAG_C05AWC_FUN
from .util import Nag_ErrorControl
from .util import NAG_C05AYC_FUN
from .util import Nag_Boolean
from .util import Complex
from .util import NAG_C05QBC_FUN
from .util import NAG_C05QCC_FUN
from .util import Nag_ScaleType
from .util import NAG_C05QSC_FUN
from .util import NAG_C05RBC_FUN
from .util import NAG_C05RCC_FCN
from .util import Nag_Complex_Type

_library_cl = get_library()

c05auc_ctypes = _library_cl.c05auc
c05auc_ctypes.restype = None
c05auc_ctypes.argtypes = [POINTER(c_double), c_double, c_double, c_double,
                          NAG_C05AUC_FUN, POINTER(c_double),
                          POINTER(c_double), POINTER(Nag_Comm),
                          POINTER(NagError)]


def c05auc(x, h, eps, eta, f, a, b, comm, fail):
    """
    Zero of continuous function, Brent algorithm, from a given starting
    value, binary search for interval.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05auc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, a, b
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05auc', x, 'x')
    _a = nag_double_type_check_and_cast('c05auc', a, 'a')
    _b = nag_double_type_check_and_cast('c05auc', b, 'b')
    c05auc_ctypes(_x, h, eps, eta, f, _a, _b, comm, fail)

c05avc_ctypes = _library_cl.c05avc
c05avc_ctypes.restype = None
c05avc_ctypes.argtypes = [POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def c05avc(x, fx, h, boundl, boundu, y, c, ind, fail):
    """
    Binary search for interval containing zero of continuous function
    (reverse communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05avc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, h, y, c, ind
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, h, y, c
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ind
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05avc', x, 'x')
    _h = nag_double_type_check_and_cast('c05avc', h, 'h')
    _y = nag_double_type_check_and_cast('c05avc', y, 'y')
    _c = nag_double_type_check_and_cast('c05avc', c, 'c')
    _ind = nag_integer_type_check_and_cast('c05avc', ind, 'ind')
    c05avc_ctypes(_x, fx, _h, boundl, boundu, _y, _c, _ind, fail)

c05awc_ctypes = _library_cl.c05awc
c05awc_ctypes.restype = None
c05awc_ctypes.argtypes = [POINTER(c_double), c_double, c_double, NAG_C05AWC_FUN, Integer, POINTER(Nag_Comm), POINTER(NagError)]


def c05awc(x, eps, eta, f, nfmax, comm, fail):
    """
    Zero of continuous function, continuation method, from a given starting
    value.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05awc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05awc', x, 'x')
    c05awc_ctypes(_x, eps, eta, f, nfmax, comm, fail)

c05axc_ctypes = _library_cl.c05axc
c05axc_ctypes.restype = None
c05axc_ctypes.argtypes = [POINTER(c_double), c_double, c_double, Nag_ErrorControl, c_double, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def c05axc(x, fx, tol, ir, scal, c, ind, fail):
    """
    Zero of continuous function, continuation method, from a given starting
    value (reverse communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05axc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, c, ind
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, c
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ind
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05axc', x, 'x')
    _c = nag_double_type_check_and_cast('c05axc', c, 'c')
    _ind = nag_integer_type_check_and_cast('c05axc', ind, 'ind')
    c05axc_ctypes(_x, fx, tol, ir, scal, _c, _ind, fail)

c05ayc_ctypes = _library_cl.c05ayc
c05ayc_ctypes.restype = None
c05ayc_ctypes.argtypes = [c_double, c_double, c_double, c_double, NAG_C05AYC_FUN, POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def c05ayc(a, b, eps, eta, f, x, comm, fail):
    """
    Zero of continuous function in a given interval, Brent algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05ayc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05ayc', x, 'x')
    c05ayc_ctypes(a, b, eps, eta, f, _x, comm, fail)

c05azc_ctypes = _library_cl.c05azc
c05azc_ctypes.restype = None
c05azc_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), c_double, c_double, Nag_ErrorControl, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def c05azc(x, y, fx, tolx, ir, c, ind, fail):
    """
    Zero of continuous function in a given interval, Brent algorithm
    (reverse communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05azc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, c, ind
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, c
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ind
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05azc', x, 'x')
    _y = nag_double_type_check_and_cast('c05azc', y, 'y')
    _c = nag_double_type_check_and_cast('c05azc', c, 'c')
    _ind = nag_integer_type_check_and_cast('c05azc', ind, 'ind')
    c05azc_ctypes(_x, _y, fx, tolx, ir, _c, _ind, fail)

c05bac_ctypes = _library_cl.c05bac
c05bac_ctypes.restype = c_double
c05bac_ctypes.argtypes = [c_double, Integer, Nag_Boolean, POINTER(NagError)]


def c05bac(x, branch, offset, fail):
    """
    Real values of Lambert's W function, W(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05bac.html
    """
    return_value = c05bac_ctypes(x, branch, offset, fail)
    return return_value

c05bbc_ctypes = _library_cl.c05bbc
c05bbc_ctypes.restype = None
c05bbc_ctypes.argtypes = [Integer, Nag_Boolean, Nag_Complex_Type, POINTER(Complex), POINTER(c_double), POINTER(NagError)]


def c05bbc(branch, offset, z, w, resid, fail):
    """
    Values of Lambert's W function, W(z).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        w, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        resid
    TypeError will be raised when the following arguments are not of data type
    Complex:
        w
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _w = nag_complex_type_check_and_cast('c05bbc', w, 'w')
    _resid = nag_double_type_check_and_cast('c05bbc', resid, 'resid')
    c05bbc_ctypes(branch, offset, z, _w, _resid, fail)

c05qbc_ctypes = _library_cl.c05qbc
c05qbc_ctypes.restype = None
c05qbc_ctypes.argtypes = [NAG_C05QBC_FUN, Integer, POINTER(c_double), POINTER(c_double), c_double, POINTER(Nag_Comm), POINTER(NagError)]


def c05qbc(fcn, n, x, fvec, xtol, comm, fail):
    """
    Solution of a system of nonlinear equations using function values only
    (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05qbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fvec
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05qbc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05qbc', fvec, 'fvec')
    c05qbc_ctypes(fcn, n, _x, _fvec, xtol, comm, fail)

c05qcc_ctypes = _library_cl.c05qcc
c05qcc_ctypes.restype = None
c05qcc_ctypes.argtypes = [NAG_C05QCC_FUN, Integer, POINTER(c_double), POINTER(c_double), c_double, Integer, Integer, Integer, c_double, Nag_ScaleType, POINTER(c_double), c_double, Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def c05qcc(fcn, n, x, fvec, xtol, maxfev, ml, mu, epsfcn, scalemode, diag, factor, nprint, nfev, fjac, r, qtf, comm, fail):
    """
    Solution of a system of nonlinear equations using function values only
    (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05qcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fvec, diag, nfev, fjac, r, qtf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, diag, fjac, r, qtf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nfev
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05qcc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05qcc', fvec, 'fvec')
    _diag = nag_double_type_check_and_cast('c05qcc', diag, 'diag')
    _nfev = nag_integer_type_check_and_cast('c05qcc', nfev, 'nfev')
    _fjac = nag_double_type_check_and_cast('c05qcc', fjac, 'fjac')
    _r = nag_double_type_check_and_cast('c05qcc', r, 'r')
    _qtf = nag_double_type_check_and_cast('c05qcc', qtf, 'qtf')
    c05qcc_ctypes(fcn, n, _x, _fvec, xtol, maxfev, ml, mu, epsfcn, scalemode, _diag, factor, nprint, _nfev, _fjac, _r, _qtf, comm, fail)

c05qdc_ctypes = _library_cl.c05qdc
c05qdc_ctypes.restype = None
c05qdc_ctypes.argtypes = [POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), c_double, Integer, Integer, c_double, Nag_ScaleType, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def c05qdc(irevcm, n, x, fvec, xtol, ml, mu, epsfcn, scalemode, diag, factor, fjac, r, qtf, iwsav, rwsav, fail):
    """
    Solution of a system of nonlinear equations using function values only
    (reverse communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05qdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, x, fvec, diag, fjac, r, qtf, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, diag, fjac, r, qtf, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('c05qdc', irevcm, 'irevcm')
    _x = nag_double_type_check_and_cast('c05qdc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05qdc', fvec, 'fvec')
    _diag = nag_double_type_check_and_cast('c05qdc', diag, 'diag')
    _fjac = nag_double_type_check_and_cast('c05qdc', fjac, 'fjac')
    _r = nag_double_type_check_and_cast('c05qdc', r, 'r')
    _qtf = nag_double_type_check_and_cast('c05qdc', qtf, 'qtf')
    _iwsav = nag_integer_type_check_and_cast('c05qdc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('c05qdc', rwsav, 'rwsav')
    c05qdc_ctypes(_irevcm, n, _x, _fvec, xtol, ml, mu, epsfcn, scalemode, _diag, factor, _fjac, _r, _qtf, _iwsav, _rwsav, fail)

c05qsc_ctypes = _library_cl.c05qsc
c05qsc_ctypes.restype = None
c05qsc_ctypes.argtypes = [NAG_C05QSC_FUN, Integer, POINTER(c_double), POINTER(c_double), c_double, Nag_Boolean, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(Nag_Comm), POINTER(NagError)]


def c05qsc(fcn, n, x, fvec, xtol, init, rcomm, lrcomm, icomm, licomm, comm, fail):
    """
    Solution of a sparse system of nonlinear equations using function values
    only (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05qsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fvec, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05qsc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05qsc', fvec, 'fvec')
    _rcomm = nag_double_type_check_and_cast('c05qsc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('c05qsc', icomm, 'icomm')
    c05qsc_ctypes(fcn, n, _x, _fvec, xtol, init, _rcomm, lrcomm, _icomm, licomm, comm, fail)

c05rbc_ctypes = _library_cl.c05rbc
c05rbc_ctypes.restype = None
c05rbc_ctypes.argtypes = [NAG_C05RBC_FUN, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(Nag_Comm), POINTER(NagError)]


def c05rbc(fcn, n, x, fvec, fjac, xtol, comm, fail):
    """
    Solution of a system of nonlinear equations using first derivatives
    (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05rbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fvec, fjac
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, fjac
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05rbc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05rbc', fvec, 'fvec')
    _fjac = nag_double_type_check_and_cast('c05rbc', fjac, 'fjac')
    c05rbc_ctypes(fcn, n, _x, _fvec, _fjac, xtol, comm, fail)

c05rcc_ctypes = _library_cl.c05rcc
c05rcc_ctypes.restype = None
c05rcc_ctypes.argtypes = [NAG_C05RCC_FCN, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, Integer, Nag_ScaleType, POINTER(c_double), c_double, Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def c05rcc(fcn, n, x, fvec, fjac, xtol, maxfev, scalemode, diag, factor, nprint, nfev, njev, r, qtf, comm, fail):
    """
    Solution of a system of nonlinear equations using first derivatives
    (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05rcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fvec, fjac, diag, nfev, njev, r, qtf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, fjac, diag, r, qtf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nfev, njev
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05rcc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05rcc', fvec, 'fvec')
    _fjac = nag_double_type_check_and_cast('c05rcc', fjac, 'fjac')
    _diag = nag_double_type_check_and_cast('c05rcc', diag, 'diag')
    _nfev = nag_integer_type_check_and_cast('c05rcc', nfev, 'nfev')
    _njev = nag_integer_type_check_and_cast('c05rcc', njev, 'njev')
    _r = nag_double_type_check_and_cast('c05rcc', r, 'r')
    _qtf = nag_double_type_check_and_cast('c05rcc', qtf, 'qtf')
    c05rcc_ctypes(fcn, n, _x, _fvec, _fjac, xtol, maxfev, scalemode, _diag, factor, nprint, _nfev, _njev, _r, _qtf, comm, fail)

c05rdc_ctypes = _library_cl.c05rdc
c05rdc_ctypes.restype = None
c05rdc_ctypes.argtypes = [POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, Nag_ScaleType, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def c05rdc(irevcm, n, x, fvec, fjac, xtol, scalemode, diag, factor, r, qtf, iwsav, rwsav, fail):
    """
    Solution of a system of nonlinear equations using first derivatives
    (reverse communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05rdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, x, fvec, fjac, diag, r, qtf, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, fjac, diag, r, qtf, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('c05rdc', irevcm, 'irevcm')
    _x = nag_double_type_check_and_cast('c05rdc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05rdc', fvec, 'fvec')
    _fjac = nag_double_type_check_and_cast('c05rdc', fjac, 'fjac')
    _diag = nag_double_type_check_and_cast('c05rdc', diag, 'diag')
    _r = nag_double_type_check_and_cast('c05rdc', r, 'r')
    _qtf = nag_double_type_check_and_cast('c05rdc', qtf, 'qtf')
    _iwsav = nag_integer_type_check_and_cast('c05rdc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('c05rdc', rwsav, 'rwsav')
    c05rdc_ctypes(_irevcm, n, _x, _fvec, _fjac, xtol, scalemode, _diag, factor, _r, _qtf, _iwsav, _rwsav, fail)

c05zdc_ctypes = _library_cl.c05zdc
c05zdc_ctypes.restype = None
c05zdc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def c05zdc(mode, m, n, x, fvec, fjac, xp, fvecp, err, fail):
    """
    Check user's function for calculating first derivatives of a set of
    nonlinear functions of several variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c05/c05zdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fvec, fjac, xp, fvecp, err
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, fjac, xp, fvecp, err
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c05zdc', x, 'x')
    _fvec = nag_double_type_check_and_cast('c05zdc', fvec, 'fvec')
    _fjac = nag_double_type_check_and_cast('c05zdc', fjac, 'fjac')
    _xp = nag_double_type_check_and_cast('c05zdc', xp, 'xp')
    _fvecp = nag_double_type_check_and_cast('c05zdc', fvecp, 'fvecp')
    _err = nag_double_type_check_and_cast('c05zdc', err, 'err')
    c05zdc_ctypes(mode, m, n, _x, _fvec, _fjac, _xp, _fvecp, _err, fail)

nag_zero_cont_func_brent_binsrch = c05auc
nag_interval_zero_cont_func = c05avc
nag_zero_cont_func_cntin = c05awc
nag_zero_cont_func_cntin_rcomm = c05axc
nag_zero_cont_func_brent = c05ayc
nag_zero_cont_func_brent_rcomm = c05azc
nag_lambertW = c05bac
nag_lambertW_complex = c05bbc
nag_zero_nonlin_eqns_easy = c05qbc
nag_zero_nonlin_eqns_expert = c05qcc
nag_zero_nonlin_eqns_rcomm = c05qdc
nag_zero_sparse_nonlin_eqns_easy = c05qsc
nag_zero_nonlin_eqns_deriv_easy = c05rbc
nag_zero_nonlin_eqns_deriv_expert = c05rcc
nag_zero_nonlin_eqns_deriv_rcomm = c05rdc
nag_check_derivs = c05zdc
__all__ = ['c05auc', 'c05avc', 'c05awc', 'c05axc', 'c05ayc',
           'c05azc', 'c05bac', 'c05bbc', 'c05qbc', 'c05qcc', 'c05qdc',
           'c05qsc', 'c05rbc', 'c05rcc', 'c05rdc',
           'c05zdc', 'nag_check_derivs',
           'nag_interval_zero_cont_func', 'nag_lambertW',
           'nag_lambertW_complex',
           'nag_zero_cont_func_brent',
           'nag_zero_cont_func_brent_binsrch',
           'nag_zero_cont_func_brent_rcomm',
           'nag_zero_cont_func_cntin',
           'nag_zero_cont_func_cntin_rcomm',
           'nag_zero_nonlin_eqns_deriv_easy',
           'nag_zero_nonlin_eqns_deriv_expert',
           'nag_zero_nonlin_eqns_deriv_rcomm',
           'nag_zero_nonlin_eqns_easy', 'nag_zero_nonlin_eqns_expert',
           'nag_zero_nonlin_eqns_rcomm',
           'nag_zero_sparse_nonlin_eqns_easy']
