"""
Deprecated wrappers for the NAG C Library (Mark 26.0) c06 Chapter.

c06 - Fourier Transforms.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/C06/c06conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Integer
from .util import NagError
from .util import Nag_Series
from .util import Nag_VectorOp
from .util import Nag_TransformDirection
from .util import Complex

_library_cl = get_library()

c06dcc_ctypes = _library_cl.c06dcc
c06dcc_ctypes.restype = None
c06dcc_ctypes.argtypes = [POINTER(c_double), Integer, c_double, c_double, POINTER(c_double), Integer, Nag_Series, POINTER(c_double), POINTER(NagError)]


def c06dcc(x, lx, xmin, xmax, c, n, s, res, fail):
    """
    Sum of a Chebyshev series at a set of points.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, c, res
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, c, res
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06dcc', x, 'x')
    _c = nag_double_type_check_and_cast('c06dcc', c, 'c')
    _res = nag_double_type_check_and_cast('c06dcc', res, 'res')
    c06dcc_ctypes(_x, lx, xmin, xmax, _c, n, s, _res, fail)

c06fkc_ctypes = _library_cl.c06fkc
c06fkc_ctypes.restype = None
c06fkc_ctypes.argtypes = [Nag_VectorOp, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def c06fkc(job, x, y, n, fail):
    """
    Circular convolution or correlation of two real vectors, no restrictions
    on n.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06fkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06fkc', x, 'x')
    _y = nag_double_type_check_and_cast('c06fkc', y, 'y')
    c06fkc_ctypes(job, _x, _y, n, fail)

c06fpc_ctypes = _library_cl.c06fpc
c06fpc_ctypes.restype = None
c06fpc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def c06fpc(m, n, x, trig, fail):
    """
    Multiple one-dimensional real discrete Fourier transforms.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06fpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, trig
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, trig
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06fpc', x, 'x')
    _trig = nag_double_type_check_and_cast('c06fpc', trig, 'trig')
    c06fpc_ctypes(m, n, _x, _trig, fail)

c06fqc_ctypes = _library_cl.c06fqc
c06fqc_ctypes.restype = None
c06fqc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def c06fqc(m, n, x, trig, fail):
    """
    Multiple one-dimensional Hermitian discrete Fourier transforms.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06fqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, trig
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, trig
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06fqc', x, 'x')
    _trig = nag_double_type_check_and_cast('c06fqc', trig, 'trig')
    c06fqc_ctypes(m, n, _x, _trig, fail)

c06gqc_ctypes = _library_cl.c06gqc
c06gqc_ctypes.restype = None
c06gqc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(NagError)]


def c06gqc(m, n, x, fail):
    """
    Complex conjugate of multiple Hermitian sequences.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06gqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06gqc', x, 'x')
    c06gqc_ctypes(m, n, _x, fail)

c06gsc_ctypes = _library_cl.c06gsc
c06gsc_ctypes.restype = None
c06gsc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def c06gsc(m, n, x, u, v, fail):
    """
    Convert Hermitian sequences to general complex sequences.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06gsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, u, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, u, v
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06gsc', x, 'x')
    _u = nag_double_type_check_and_cast('c06gsc', u, 'u')
    _v = nag_double_type_check_and_cast('c06gsc', v, 'v')
    c06gsc_ctypes(m, n, _x, _u, _v, fail)

c06gzc_ctypes = _library_cl.c06gzc
c06gzc_ctypes.restype = None
c06gzc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(NagError)]


def c06gzc(n, trig, fail):
    """
    Initialization function for other c06 functions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06gzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        trig
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        trig
    """
    from .util import nag_double_type_check_and_cast
    _trig = nag_double_type_check_and_cast('c06gzc', trig, 'trig')
    c06gzc_ctypes(n, _trig, fail)

c06pac_ctypes = _library_cl.c06pac
c06pac_ctypes.restype = None
c06pac_ctypes.argtypes = [Nag_TransformDirection, POINTER(c_double), Integer, POINTER(NagError)]


def c06pac(direct, x, n, fail):
    """
    Single one-dimensional real and Hermitian complex discrete Fourier
    transform, using complex storage format for Hermitian sequences.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06pac', x, 'x')
    c06pac_ctypes(direct, _x, n, fail)

c06pcc_ctypes = _library_cl.c06pcc
c06pcc_ctypes.restype = None
c06pcc_ctypes.argtypes = [Nag_TransformDirection, POINTER(Complex), Integer, POINTER(NagError)]


def c06pcc(direct, x, n, fail):
    """
    Single one-dimensional complex discrete Fourier transform, complex data
    type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('c06pcc', x, 'x')
    c06pcc_ctypes(direct, _x, n, fail)

c06pfc_ctypes = _library_cl.c06pfc
c06pfc_ctypes.restype = None
c06pfc_ctypes.argtypes = [Nag_TransformDirection, Integer, Integer, POINTER(Integer), Integer, POINTER(Complex), POINTER(NagError)]


def c06pfc(direct, ndim, l, nd, n, x, fail):
    """
    One-dimensional complex discrete Fourier transform of multidimensional
    data (using complex data type).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nd, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nd
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nd = nag_integer_type_check_and_cast('c06pfc', nd, 'nd')
    _x = nag_complex_type_check_and_cast('c06pfc', x, 'x')
    c06pfc_ctypes(direct, ndim, l, _nd, n, _x, fail)

c06pjc_ctypes = _library_cl.c06pjc
c06pjc_ctypes.restype = None
c06pjc_ctypes.argtypes = [Nag_TransformDirection, Integer, POINTER(Integer), Integer, POINTER(Complex), POINTER(NagError)]


def c06pjc(direct, ndim, nd, n, x, fail):
    """
    Multidimensional complex discrete Fourier transform of multidimensional
    data (using complex data type).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nd, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nd
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nd = nag_integer_type_check_and_cast('c06pjc', nd, 'nd')
    _x = nag_complex_type_check_and_cast('c06pjc', x, 'x')
    c06pjc_ctypes(direct, ndim, _nd, n, _x, fail)

c06psc_ctypes = _library_cl.c06psc
c06psc_ctypes.restype = None
c06psc_ctypes.argtypes = [Nag_TransformDirection, Integer, Integer, POINTER(Complex), POINTER(NagError)]


def c06psc(direct, n, m, x, fail):
    """
    Multiple one-dimensional complex discrete Fourier transforms, complex
    data type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06psc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('c06psc', x, 'x')
    c06psc_ctypes(direct, n, m, _x, fail)

c06puc_ctypes = _library_cl.c06puc
c06puc_ctypes.restype = None
c06puc_ctypes.argtypes = [Nag_TransformDirection, Integer, Integer, POINTER(Complex), POINTER(NagError)]


def c06puc(direct, m, n, x, fail):
    """
    Two-dimensional complex discrete Fourier transform, complex data type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06puc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('c06puc', x, 'x')
    c06puc_ctypes(direct, m, n, _x, fail)

c06pvc_ctypes = _library_cl.c06pvc
c06pvc_ctypes.restype = None
c06pvc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(Complex), POINTER(NagError)]


def c06pvc(m, n, x, y, fail):
    """
    Two-dimensional real-to-complex discrete Fourier transform.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        y
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06pvc', x, 'x')
    _y = nag_complex_type_check_and_cast('c06pvc', y, 'y')
    c06pvc_ctypes(m, n, _x, _y, fail)

c06pwc_ctypes = _library_cl.c06pwc
c06pwc_ctypes.restype = None
c06pwc_ctypes.argtypes = [Integer, Integer, POINTER(Complex), POINTER(c_double), POINTER(NagError)]


def c06pwc(m, n, y, x, fail):
    """
    Two-dimensional complex-to-real discrete Fourier transform.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        y
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _y = nag_complex_type_check_and_cast('c06pwc', y, 'y')
    _x = nag_double_type_check_and_cast('c06pwc', x, 'x')
    c06pwc_ctypes(m, n, _y, _x, fail)

c06pxc_ctypes = _library_cl.c06pxc
c06pxc_ctypes.restype = None
c06pxc_ctypes.argtypes = [Nag_TransformDirection, Integer, Integer, Integer, POINTER(Complex), POINTER(NagError)]


def c06pxc(direct, n1, n2, n3, x, fail):
    """
    Three-dimensional complex discrete Fourier transform, complex data type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('c06pxc', x, 'x')
    c06pxc_ctypes(direct, n1, n2, n3, _x, fail)

c06pyc_ctypes = _library_cl.c06pyc
c06pyc_ctypes.restype = None
c06pyc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), POINTER(Complex), POINTER(NagError)]


def c06pyc(n1, n2, n3, x, y, fail):
    """
    Three-dimensional real-to-complex discrete Fourier transform.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        y
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06pyc', x, 'x')
    _y = nag_complex_type_check_and_cast('c06pyc', y, 'y')
    c06pyc_ctypes(n1, n2, n3, _x, _y, fail)

c06pzc_ctypes = _library_cl.c06pzc
c06pzc_ctypes.restype = None
c06pzc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(Complex), POINTER(c_double), POINTER(NagError)]


def c06pzc(n1, n2, n3, y, x, fail):
    """
    Three-dimensional complex-to-real discrete Fourier transform.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06pzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        y
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _y = nag_complex_type_check_and_cast('c06pzc', y, 'y')
    _x = nag_double_type_check_and_cast('c06pzc', x, 'x')
    c06pzc_ctypes(n1, n2, n3, _y, _x, fail)

c06rec_ctypes = _library_cl.c06rec
c06rec_ctypes.restype = None
c06rec_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(NagError)]


def c06rec(m, n, x, fail):
    """
    Multiple discrete sine transforms, simple.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06rec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06rec', x, 'x')
    c06rec_ctypes(m, n, _x, fail)

c06rfc_ctypes = _library_cl.c06rfc
c06rfc_ctypes.restype = None
c06rfc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(NagError)]


def c06rfc(m, n, x, fail):
    """
    Multiple discrete cosine transforms, simple.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06rfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06rfc', x, 'x')
    c06rfc_ctypes(m, n, _x, fail)

c06rgc_ctypes = _library_cl.c06rgc
c06rgc_ctypes.restype = None
c06rgc_ctypes.argtypes = [Nag_TransformDirection, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def c06rgc(direct, m, n, x, fail):
    """
    Multiple discrete quarter-wave sine transforms, simple.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06rgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06rgc', x, 'x')
    c06rgc_ctypes(direct, m, n, _x, fail)

c06rhc_ctypes = _library_cl.c06rhc
c06rhc_ctypes.restype = None
c06rhc_ctypes.argtypes = [Nag_TransformDirection, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def c06rhc(direct, m, n, x, fail):
    """
    Multiple discrete quarter-wave cosine transforms, simple.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c06/c06rhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('c06rhc', x, 'x')
    c06rhc_ctypes(direct, m, n, _x, fail)

nag_sum_cheby_series = c06dcc
nag_sum_convcorr_real = c06fkc
nag_fft_multiple_real = c06fpc
nag_fft_multiple_hermitian = c06fqc
nag_multiple_conjugate_hermitian = c06gqc
nag_multiple_hermitian_to_complex = c06gsc
nag_fft_init_trig = c06gzc
nag_sum_fft_realherm_1d = c06pac
nag_sum_fft_complex_1d = c06pcc
nag_fft_multid_single = c06pfc
nag_fft_multid_full = c06pjc
nag_sum_fft_complex_1d_multi = c06psc
nag_sum_fft_complex_2d = c06puc
nag_sum_fft_real_2d = c06pvc
nag_sum_fft_hermitian_2d = c06pwc
nag_fft_3d = c06pxc
nag_sum_fft_real_3d = c06pyc
nag_sum_fft_hermitian_3d = c06pzc
nag_sum_fft_sine = c06rec
nag_sum_fft_cosine = c06rfc
nag_sum_fft_qtrsine = c06rgc
nag_sum_fft_qtrcosine = c06rhc
__all__ = ['c06dcc',
           'c06fkc', 'c06fpc', 'c06fqc',
           'c06gqc', 'c06gsc', 'c06gzc',
           'c06pac',
           'c06pcc', 'c06pfc', 'c06pjc', 'c06psc', 'c06puc',
           'c06pvc', 'c06pwc', 'c06pxc', 'c06pyc', 'c06pzc', 'c06rec',
           'c06rfc', 'c06rgc', 'c06rhc',
           'nag_fft_3d',
           'nag_fft_init_trig',
           'nag_fft_multid_full', 'nag_fft_multid_single',
           'nag_fft_multiple_hermitian',
           'nag_fft_multiple_real',
           'nag_multiple_conjugate_hermitian',
           'nag_multiple_hermitian_to_complex',
           'nag_sum_cheby_series',
           'nag_sum_convcorr_real', 'nag_sum_fft_complex_1d',
           'nag_sum_fft_complex_1d_multi', 'nag_sum_fft_complex_2d',
           'nag_sum_fft_cosine', 'nag_sum_fft_hermitian_2d',
           'nag_sum_fft_hermitian_3d', 'nag_sum_fft_qtrcosine',
           'nag_sum_fft_qtrsine', 'nag_sum_fft_real_2d',
           'nag_sum_fft_real_3d', 'nag_sum_fft_realherm_1d',
           'nag_sum_fft_sine']
