"""
Deprecated wrappers for the NAG C Library (Mark 26.0) d01 Chapter.

d01 - Quadrature.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/D01/d01conts.html

NAG Copyright 2017.
"""
from ctypes import c_char_p
from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Integer
from .util import Nag_Comm
from .util import NagError
from .util import NAG_D01BDC_F
from .util import NAG_D01DAC_PHI1
from .util import NAG_D01DAC_PHI2
from .util import NAG_D01DAC_F
from .util import NAG_D01FBC_FUN
from .util import NAG_D01FDC_F
from .util import NAG_D01FDC_REGION
from .util import Nag_MCMethod
from .util import Nag_Start
from .util import Nag_Boolean
from .util import NAG_D01GDC_VECFUN
from .util import NAG_D01GDC_VECREG
from .util import Nag_QuadType
from .util import NAG_D01PAC_FUNCTN
from .util import NAG_D01RGC_F
from .util import NAG_D01SJC_FUN
from .util import Nag_QuadProgress
from .util import Nag_User
from .util import NAG_D01SKC_FUN
from .util import NAG_D01SLC_FUN
from .util import NAG_D01SMC_FUN
from .util import Nag_BoundInterval
from .util import NAG_D01SNC_FUN
from .util import Nag_TrigTransform
from .util import NAG_D01SPC_FUN
from .util import Nag_QuadWeight
from .util import NAG_D01SQC_FUN
from .util import NAG_D01SSC_FUN
from .util import Nag_QuadSubProgress
from .util import Nag_GaussFormulae
from .util import NAG_D01TAC_FUN
from .util import NAG_D01UAC_F
from .util import NAG_D01WCC_FUN
from .util import NAG_D01XBC_FUN
from .util import Nag_VariableType
from .util import NAG_D01ESC_F
from .util import NAG_D01UBC_F

_library_cl = get_library()

STRING = c_char_p

d01bdc_ctypes = _library_cl.d01bdc
d01bdc_ctypes.restype = None
d01bdc_ctypes.argtypes = [NAG_D01BDC_F, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm)]


def d01bdc(f, a, b, epsabs, epsrel, result, abserr, comm):
    """
    One-dimensional quadrature, non-adaptive, finite interval.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01bdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01bdc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01bdc', abserr, 'abserr')
    d01bdc_ctypes(f, a, b, epsabs, epsrel, _result, _abserr, comm)

d01dac_ctypes = _library_cl.d01dac
d01dac_ctypes.restype = None
d01dac_ctypes.argtypes = [c_double, c_double, NAG_D01DAC_PHI1, NAG_D01DAC_PHI2, NAG_D01DAC_F, c_double, POINTER(c_double), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def d01dac(ya, yb, phi1, phi2, f, absacc, ans, npts, comm, fail):
    """
    Two-dimensional quadrature, finite region.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ans, npts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ans
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        npts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ans = nag_double_type_check_and_cast('d01dac', ans, 'ans')
    _npts = nag_integer_type_check_and_cast('d01dac', npts, 'npts')
    d01dac_ctypes(ya, yb, phi1, phi2, f, absacc, _ans, _npts, comm, fail)

d01esc_ctypes = _library_cl.d01esc
d01esc_ctypes.restype = None
d01esc_ctypes.argtypes = [Integer, Integer, NAG_D01ESC_F, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def d01esc(ni, ndim, f, maxdlv, dinest, errest, ivalid, iopts, opts, comm, fail):
    """
    Multi-dimensional quadrature using sparse grids.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01esc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        maxdlv, dinest, errest, ivalid, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dinest, errest, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        maxdlv, ivalid, iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dinest = nag_double_type_check_and_cast('d01esc', dinest, 'dinest')
    _errest = nag_double_type_check_and_cast('d01esc', errest, 'errest')
    _opts = nag_double_type_check_and_cast('d01esc', opts, 'opts')
    _maxdlv = nag_integer_type_check_and_cast('d01esc', maxdlv, 'maxdlv')
    _ivalid = nag_integer_type_check_and_cast('d01esc', ivalid, 'ivalid')
    _iopts = nag_integer_type_check_and_cast('d01esc', iopts, 'iopts')
    d01esc_ctypes(ni, ndim, f, _maxdlv, _dinest, _errest, _ivalid, _iopts, _opts, comm, fail)

nag_quad_md_sgq_multi_vec = d01esc

d01fbc_ctypes = _library_cl.d01fbc
d01fbc_ctypes.restype = c_double
d01fbc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), NAG_D01FBC_FUN, POINTER(Nag_Comm), POINTER(NagError)]


def d01fbc(ndim, nptvec, lwa, weight, abscis, fun, comm, fail):
    """
    Multidimensional Gaussian quadrature over hyper-rectangle.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01fbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nptvec, weight, abscis
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        weight, abscis
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nptvec
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nptvec = nag_integer_type_check_and_cast('d01fbc', nptvec, 'nptvec')
    _weight = nag_double_type_check_and_cast('d01fbc', weight, 'weight')
    _abscis = nag_double_type_check_and_cast('d01fbc', abscis, 'abscis')
    return_value = d01fbc_ctypes(ndim, _nptvec, lwa, _weight, _abscis, fun, comm, fail)
    return return_value

d01fdc_ctypes = _library_cl.d01fdc
d01fdc_ctypes.restype = None
d01fdc_ctypes.argtypes = [Integer, NAG_D01FDC_F, c_double, NAG_D01FDC_REGION, Integer, c_double, c_double, POINTER(c_double), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def d01fdc(ndim, f, sigma, region, limit, r0, u, result, ncalls, comm, fail):
    """
    Multidimensional quadrature, Sag--Szekeres method, general product
    region or n-sphere.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01fdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, ncalls
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ncalls
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    if region is None:
        region = cast(None, NAG_D01FDC_REGION)
    _result = nag_double_type_check_and_cast('d01fdc', result, 'result')
    _ncalls = nag_integer_type_check_and_cast('d01fdc', ncalls, 'ncalls')
    d01fdc_ctypes(ndim, f, sigma, region, limit, r0, u, _result, _ncalls, comm, fail)

d01gac_ctypes = _library_cl.d01gac
d01gac_ctypes.restype = None
d01gac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d01gac(n, x, y, ans, er, fail):
    """
    One-dimensional integration of a function defined by data values only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, ans, er
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, ans, er
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('d01gac', x, 'x')
    _y = nag_double_type_check_and_cast('d01gac', y, 'y')
    _ans = nag_double_type_check_and_cast('d01gac', ans, 'ans')
    _er = nag_double_type_check_and_cast('d01gac', er, 'er')
    d01gac_ctypes(n, _x, _y, _ans, _er, fail)

d01gdc_ctypes = _library_cl.d01gdc
d01gdc_ctypes.restype = None
d01gdc_ctypes.argtypes = [Integer, NAG_D01GDC_VECFUN, NAG_D01GDC_VECREG, Integer, POINTER(c_double), Integer, Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def d01gdc(ndim, vecfun, vecreg, npts, vk, nrand, transform, res, err, comm, fail):
    """
    Multidimensional quadrature, general product region, number-theoretic
    method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01gdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vk, res, err
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        vk, res, err
    """
    from .util import nag_double_type_check_and_cast
    _vk = nag_double_type_check_and_cast('d01gdc', vk, 'vk')
    _res = nag_double_type_check_and_cast('d01gdc', res, 'res')
    _err = nag_double_type_check_and_cast('d01gdc', err, 'err')
    d01gdc_ctypes(ndim, vecfun, vecreg, npts, _vk, nrand, transform, _res, _err, comm, fail)

d01gyc_ctypes = _library_cl.d01gyc
d01gyc_ctypes.restype = None
d01gyc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(NagError)]


def d01gyc(ndim, npts, vk, fail):
    """
    Korobov optimal coefficients for use in d01gdc, when number of points is
    prime.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01gyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        vk
    """
    from .util import nag_double_type_check_and_cast
    _vk = nag_double_type_check_and_cast('d01gyc', vk, 'vk')
    d01gyc_ctypes(ndim, npts, _vk, fail)

d01gzc_ctypes = _library_cl.d01gzc
d01gzc_ctypes.restype = None
d01gzc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def d01gzc(ndim, np1, np2, vk, fail):
    """
    Korobov optimal coefficients for use in d01gdc, when number of points is
    product of two primes.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01gzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        vk
    """
    from .util import nag_double_type_check_and_cast
    _vk = nag_double_type_check_and_cast('d01gzc', vk, 'vk')
    d01gzc_ctypes(ndim, np1, np2, _vk, fail)

d01pac_ctypes = _library_cl.d01pac
d01pac_ctypes.restype = None
d01pac_ctypes.argtypes = [Integer, POINTER(c_double), NAG_D01PAC_FUNCTN, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def d01pac(ndim, vert, functn, minord, maxord, finvls, esterr, comm, fail):
    """
    Multidimensional quadrature over an n-simplex.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01pac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vert, minord, finvls, esterr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        vert, finvls, esterr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        minord
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _vert = nag_double_type_check_and_cast('d01pac', vert, 'vert')
    _minord = nag_integer_type_check_and_cast('d01pac', minord, 'minord')
    _finvls = nag_double_type_check_and_cast('d01pac', finvls, 'finvls')
    _esterr = nag_double_type_check_and_cast('d01pac', esterr, 'esterr')
    d01pac_ctypes(ndim, _vert, functn, _minord, maxord, _finvls, _esterr, comm, fail)

d01rac_ctypes = _library_cl.d01rac
d01rac_ctypes.restype = None
d01rac_ctypes.argtypes = [POINTER(Integer), Integer, c_double, c_double, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def d01rac(irevcm, ni, a, b, sid, needi, x, lenx, nx, fm, ldf, dinest, errest, iopts, opts, icom, licom, com, lcom, fail):
    """
    One-dimensional quadrature, adaptive, finite interval, multiple
    integrands, vectorized abscissae, reverse communication.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01rac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, sid, needi, x, nx, fm, dinest, errest, iopts, opts, icom, com
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fm, dinest, errest, opts, com
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, sid, needi, nx, iopts, icom
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('d01rac', irevcm, 'irevcm')
    _sid = nag_integer_type_check_and_cast('d01rac', sid, 'sid')
    _needi = nag_integer_type_check_and_cast('d01rac', needi, 'needi')
    _x = nag_double_type_check_and_cast('d01rac', x, 'x')
    _nx = nag_integer_type_check_and_cast('d01rac', nx, 'nx')
    _fm = nag_double_type_check_and_cast('d01rac', fm, 'fm')
    _dinest = nag_double_type_check_and_cast('d01rac', dinest, 'dinest')
    _errest = nag_double_type_check_and_cast('d01rac', errest, 'errest')
    _iopts = nag_integer_type_check_and_cast('d01rac', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('d01rac', opts, 'opts')
    _icom = nag_integer_type_check_and_cast('d01rac', icom, 'icom')
    _com = nag_double_type_check_and_cast('d01rac', com, 'com')
    d01rac_ctypes(_irevcm, ni, a, b, _sid, _needi, _x, lenx, _nx, _fm, ldf, _dinest, _errest, _iopts, _opts, _icom, licom, _com, lcom, fail)

d01rcc_ctypes = _library_cl.d01rcc
d01rcc_ctypes.restype = None
d01rcc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d01rcc(ni, lenxrq, ldfmrq, sdfmrq, licmin, licmax, lcmin, lcmax, iopts, opts, fail):
    """
    Determine required array dimensions for d01rac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01rcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lenxrq, ldfmrq, sdfmrq, licmin, licmax, lcmin, lcmax, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        lenxrq, ldfmrq, sdfmrq, licmin, licmax, lcmin, lcmax, iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lenxrq = nag_integer_type_check_and_cast('d01rcc', lenxrq, 'lenxrq')
    _ldfmrq = nag_integer_type_check_and_cast('d01rcc', ldfmrq, 'ldfmrq')
    _sdfmrq = nag_integer_type_check_and_cast('d01rcc', sdfmrq, 'sdfmrq')
    _licmin = nag_integer_type_check_and_cast('d01rcc', licmin, 'licmin')
    _licmax = nag_integer_type_check_and_cast('d01rcc', licmax, 'licmax')
    _lcmin = nag_integer_type_check_and_cast('d01rcc', lcmin, 'lcmin')
    _lcmax = nag_integer_type_check_and_cast('d01rcc', lcmax, 'lcmax')
    _iopts = nag_integer_type_check_and_cast('d01rcc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('d01rcc', opts, 'opts')
    d01rcc_ctypes(ni, _lenxrq, _ldfmrq, _sdfmrq, _licmin, _licmax, _lcmin, _lcmax, _iopts, _opts, fail)

d01rgc_ctypes = _library_cl.d01rgc
d01rgc_ctypes.restype = None
d01rgc_ctypes.argtypes = [c_double, c_double, NAG_D01RGC_F, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def d01rgc(a, b, f, epsabs, epsrel, dinest, errest, nevals, comm, fail):
    """
    One-dimensional quadrature, adaptive, finite interval, strategy due to
    Gonnet, allowing for badly behaved integrands.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01rgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dinest, errest, nevals
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dinest, errest
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nevals
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dinest = nag_double_type_check_and_cast('d01rgc', dinest, 'dinest')
    _errest = nag_double_type_check_and_cast('d01rgc', errest, 'errest')
    _nevals = nag_integer_type_check_and_cast('d01rgc', nevals, 'nevals')
    d01rgc_ctypes(a, b, f, epsabs, epsrel, _dinest, _errest, _nevals, comm, fail)

d01sjc_ctypes = _library_cl.d01sjc
d01sjc_ctypes.restype = None
d01sjc_ctypes.argtypes = [NAG_D01SJC_FUN, c_double, c_double, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadProgress), POINTER(Nag_User), POINTER(NagError)]


def d01sjc(f, a, b, epsabs, epsrel, maxnumsubint, result, abserr, qp, comm, fail):
    """
    One-dimensional quadrature, adaptive, finite interval, strategy due to
    Piessens and de Doncker, allowing for badly behaved integrands.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01sjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01sjc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01sjc', abserr, 'abserr')
    d01sjc_ctypes(f, a, b, epsabs, epsrel, maxnumsubint, _result, _abserr, qp, comm, fail)

d01skc_ctypes = _library_cl.d01skc
d01skc_ctypes.restype = None
d01skc_ctypes.argtypes = [NAG_D01SKC_FUN, c_double, c_double, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadProgress), POINTER(Nag_User), POINTER(NagError)]


def d01skc(f, a, b, epsabs, epsrel, maxnumsubint, result, abserr, qp, comm, fail):
    """
    One-dimensional quadrature, adaptive, finite interval, method suitable
    for oscillating functions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01skc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01skc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01skc', abserr, 'abserr')
    d01skc_ctypes(f, a, b, epsabs, epsrel, maxnumsubint, _result, _abserr, qp, comm, fail)

d01slc_ctypes = _library_cl.d01slc
d01slc_ctypes.restype = None
d01slc_ctypes.argtypes = [NAG_D01SLC_FUN, c_double, c_double, Integer, POINTER(c_double), c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadProgress), POINTER(Nag_User), POINTER(NagError)]


def d01slc(f, a, b, nbrkpts, brkpts, epsabs, epsrel, maxnumsubint, result, abserr, qp, comm, fail):
    """
    One-dimensional quadrature, adaptive, finite interval, allowing for
    singularities at user-specified break-points.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01slc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        brkpts, result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        brkpts, result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _brkpts = nag_double_type_check_and_cast('d01slc', brkpts, 'brkpts')
    _result = nag_double_type_check_and_cast('d01slc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01slc', abserr, 'abserr')
    d01slc_ctypes(f, a, b, nbrkpts, _brkpts, epsabs, epsrel, maxnumsubint, _result, _abserr, qp, comm, fail)

d01smc_ctypes = _library_cl.d01smc
d01smc_ctypes.restype = None
d01smc_ctypes.argtypes = [NAG_D01SMC_FUN, Nag_BoundInterval, c_double, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadProgress), POINTER(Nag_User), POINTER(NagError)]


def d01smc(f, boundinf, bound, epsabs, epsrel, maxnumsubint, result, abserr, qp, comm, fail):
    """
    One-dimensional adaptive quadrature over infinite or semi-infinite
    interval.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01smc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01smc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01smc', abserr, 'abserr')
    d01smc_ctypes(f, boundinf, bound, epsabs, epsrel, maxnumsubint, _result, _abserr, qp, comm, fail)

d01snc_ctypes = _library_cl.d01snc
d01snc_ctypes.restype = None
d01snc_ctypes.argtypes = [NAG_D01SNC_FUN, c_double, c_double, c_double, Nag_TrigTransform, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadProgress), POINTER(Nag_User), POINTER(NagError)]


def d01snc(g, a, b, omega, wtfunc, epsabs, epsrel, maxnumsubint, result, abserr, qp, comm, fail):
    """
    One-dimensional adaptive quadrature, finite interval, sine or cosine
    weight functions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01snc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01snc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01snc', abserr, 'abserr')
    d01snc_ctypes(g, a, b, omega, wtfunc, epsabs, epsrel, maxnumsubint, _result, _abserr, qp, comm, fail)

d01spc_ctypes = _library_cl.d01spc
d01spc_ctypes.restype = None
d01spc_ctypes.argtypes = [NAG_D01SPC_FUN, c_double, c_double, c_double, c_double, Nag_QuadWeight, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadProgress), POINTER(Nag_User), POINTER(NagError)]


def d01spc(g, a, b, alfa, beta, wtfunc, epsabs, epsrel, maxnumsubint, result, abserr, qp, comm, fail):
    """
    One-dimensional adaptive quadrature, weight function with end-point
    singularities of algebraic-logarithmic type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01spc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01spc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01spc', abserr, 'abserr')
    d01spc_ctypes(g, a, b, alfa, beta, wtfunc, epsabs, epsrel, maxnumsubint, _result, _abserr, qp, comm, fail)

d01sqc_ctypes = _library_cl.d01sqc
d01sqc_ctypes.restype = None
d01sqc_ctypes.argtypes = [NAG_D01SQC_FUN, c_double, c_double, c_double, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadProgress), POINTER(Nag_User), POINTER(NagError)]


def d01sqc(g, a, b, c, epsabs, epsrel, maxnumsubint, result, abserr, qp, comm, fail):
    """
    One-dimensional adaptive quadrature, weight function 1/(x-c), Cauchy
    principal value.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01sqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01sqc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01sqc', abserr, 'abserr')
    d01sqc_ctypes(g, a, b, c, epsabs, epsrel, maxnumsubint, _result, _abserr, qp, comm, fail)

d01ssc_ctypes = _library_cl.d01ssc
d01ssc_ctypes.restype = None
d01ssc_ctypes.argtypes = [NAG_D01SSC_FUN, c_double, c_double, Nag_TrigTransform, Integer, Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(Nag_QuadSubProgress), POINTER(Nag_User), POINTER(NagError)]


def d01ssc(g, a, omega, wtfunc, maxintervals, maxnumsubint, epsabs, result, abserr, qpsub, comm, fail):
    """
    One-dimensional adaptive quadrature, semi-infinite interval, sine or
    cosine weight function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01ssc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        result, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        result, abserr
    """
    from .util import nag_double_type_check_and_cast
    _result = nag_double_type_check_and_cast('d01ssc', result, 'result')
    _abserr = nag_double_type_check_and_cast('d01ssc', abserr, 'abserr')
    d01ssc_ctypes(g, a, omega, wtfunc, maxintervals, maxnumsubint, epsabs, _result, _abserr, qpsub, comm, fail)

d01tac_ctypes = _library_cl.d01tac
d01tac_ctypes.restype = c_double
d01tac_ctypes.argtypes = [Nag_GaussFormulae, NAG_D01TAC_FUN, c_double, c_double, Integer, POINTER(Nag_User), POINTER(NagError)]


def d01tac(quadrule, f, a, b, npts, comm, fail):
    """
    One-dimensional Gaussian quadrature, choice of weight functions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01tac.html
    """
    return_value = d01tac_ctypes(quadrule, f, a, b, npts, comm, fail)
    return return_value

d01tbc_ctypes = _library_cl.d01tbc
d01tbc_ctypes.restype = None
d01tbc_ctypes.argtypes = [Nag_QuadType, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d01tbc(quadtype, a, b, n, weight, abscis, fail):
    """
    Pre-computed weights and abscissae for Gaussian quadrature rules,
    restricted choice of rule.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01tbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        weight, abscis
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        weight, abscis
    """
    from .util import nag_double_type_check_and_cast
    _weight = nag_double_type_check_and_cast('d01tbc', weight, 'weight')
    _abscis = nag_double_type_check_and_cast('d01tbc', abscis, 'abscis')
    d01tbc_ctypes(quadtype, a, b, n, _weight, _abscis, fail)

d01tcc_ctypes = _library_cl.d01tcc
d01tcc_ctypes.restype = None
d01tcc_ctypes.argtypes = [Nag_QuadType, c_double, c_double, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d01tcc(quadtype, a, b, c, d, n, weight, abscis, fail):
    """
    Calculation of weights and abscissae for Gaussian quadrature rules,
    general choice of rule.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01tcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        weight, abscis
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        weight, abscis
    """
    from .util import nag_double_type_check_and_cast
    _weight = nag_double_type_check_and_cast('d01tcc', weight, 'weight')
    _abscis = nag_double_type_check_and_cast('d01tcc', abscis, 'abscis')
    d01tcc_ctypes(quadtype, a, b, c, d, n, _weight, _abscis, fail)

d01tdc_ctypes = _library_cl.d01tdc
d01tdc_ctypes.restype = None
d01tdc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double),
                          POINTER(c_double), c_double, POINTER(c_double),
                          POINTER(c_double), POINTER(NagError)]


def d01tdc(n, a, b, c, muzero, weight, abscis, fail):
    """
    Calculation of weights and abscissae for Gaussian quadrature rules,
    method of Golub and Welsch.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/d01/d01tdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, weight, abscis
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c, weight, abscis
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('d01tdc', a, 'a')
    _b = nag_double_type_check_and_cast('d01tdc', b, 'b')
    _c = nag_double_type_check_and_cast('d01tdc', c, 'c')
    _weight = nag_double_type_check_and_cast('d01tdc', weight, 'weight')
    _abscis = nag_double_type_check_and_cast('d01tdc', abscis, 'abscis')
    d01tdc_ctypes(n, _a, _b, _c, muzero, _weight, _abscis, fail)

d01tec_ctypes = _library_cl.d01tec
d01tec_ctypes.restype = None
d01tec_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double),
                          POINTER(c_double), POINTER(c_double),
                          POINTER(NagError)]


def d01tec(n, mu, a, b, c, fail):
    """
    Generates recursion coefficients needed by nag_quad_1d_gauss_wrec (d01tdc)
    to calculate a Gaussian quadrature rule.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/d01/d01tec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        mu, a, b, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        mu, a, b, c
    """
    from .util import nag_double_type_check_and_cast
    _mu = nag_double_type_check_and_cast('d01tec', mu, 'mu')
    _a = nag_double_type_check_and_cast('d01tec', a, 'a')
    _b = nag_double_type_check_and_cast('d01tec', b, 'b')
    _c = nag_double_type_check_and_cast('d01tec', c, 'c')
    d01tec_ctypes(n, _mu, _a, _b, _c, fail)

d01uac_ctypes = _library_cl.d01uac
d01uac_ctypes.restype = None
d01uac_ctypes.argtypes = [Nag_QuadType, c_double, c_double, Integer, NAG_D01UAC_F, POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def d01uac(quadtype, a, b, n, f, dinest, comm, fail):
    """
    One-dimensional Gaussian quadrature, choice of weight functions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01uac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dinest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dinest
    """
    from .util import nag_double_type_check_and_cast
    _dinest = nag_double_type_check_and_cast('d01uac', dinest, 'dinest')
    d01uac_ctypes(quadtype, a, b, n, f, _dinest, comm, fail)

d01ubc_ctypes = _library_cl.d01ubc
d01ubc_ctypes.restype = None
d01ubc_ctypes.argtypes = [NAG_D01UBC_F, Integer, POINTER(c_double),
                          POINTER(Nag_Comm), POINTER(NagError)]


def d01ubc(fun, n, ans, comm, fail):
    r"""
    Returns the Gaussian quadrature approximation for the specific problem
    int_{0}^{\inf}\exp{-x^2}f(x)dx
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01ubc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ans
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ans
    """
    from .util import nag_double_type_check_and_cast
    _ans = nag_double_type_check_and_cast('d01ubc', ans, 'ans')
    d01ubc_ctypes(fun, n, _ans, comm, fail)

d01wcc_ctypes = _library_cl.d01wcc
d01wcc_ctypes.restype = None
d01wcc_ctypes.argtypes = [Integer, NAG_D01WCC_FUN, POINTER(c_double), POINTER(c_double), POINTER(Integer), Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(Nag_User), POINTER(NagError)]


def d01wcc(ndim, f, a, b, minpts, maxpts, eps, finval, acc, comm, fail):
    """
    Multidimensional adaptive quadrature.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01wcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, minpts, finval, acc
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, finval, acc
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        minpts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('d01wcc', a, 'a')
    _b = nag_double_type_check_and_cast('d01wcc', b, 'b')
    _minpts = nag_integer_type_check_and_cast('d01wcc', minpts, 'minpts')
    _finval = nag_double_type_check_and_cast('d01wcc', finval, 'finval')
    _acc = nag_double_type_check_and_cast('d01wcc', acc, 'acc')
    d01wcc_ctypes(ndim, f, _a, _b, _minpts, maxpts, eps, _finval, _acc, comm, fail)

d01xbc_ctypes = _library_cl.d01xbc
d01xbc_ctypes.restype = None
d01xbc_ctypes.argtypes = [Integer, NAG_D01XBC_FUN, Nag_MCMethod, Nag_Start, POINTER(c_double), POINTER(c_double), POINTER(Integer), Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(POINTER(c_double)), POINTER(Nag_User), POINTER(NagError)]


def d01xbc(ndim, f, method, cont, a, b, mincls, maxcls, eps, finest, acc, commarr, comm, fail):
    """
    Multidimensional quadrature, using Monte--Carlo method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01xbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, mincls, finest, acc, commarr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, finest, acc, commarr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        mincls
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('d01xbc', a, 'a')
    _b = nag_double_type_check_and_cast('d01xbc', b, 'b')
    _mincls = nag_integer_type_check_and_cast('d01xbc', mincls, 'mincls')
    _finest = nag_double_type_check_and_cast('d01xbc', finest, 'finest')
    _acc = nag_double_type_check_and_cast('d01xbc', acc, 'acc')
    _commarr = nag_double_type_check_and_cast('d01xbc', commarr, 'commarr')
    d01xbc_ctypes(ndim, f, method, cont, _a, _b, _mincls, maxcls, eps, _finest, _acc, _commarr, comm, fail)

d01zkc_ctypes = _library_cl.d01zkc
d01zkc_ctypes.restype = None
d01zkc_ctypes.argtypes = [STRING, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def d01zkc(optstr, iopts, liopts, opts, lopts, fail):
    """
    Option setting function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01zkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iopts = nag_integer_type_check_and_cast('d01zkc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('d01zkc', opts, 'opts')
    d01zkc_ctypes(optstr, _iopts, liopts, _opts, lopts, fail)

d01zlc_ctypes = _library_cl.d01zlc
d01zlc_ctypes.restype = None
d01zlc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(c_double), STRING, Integer, POINTER(Nag_VariableType), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d01zlc(optstr, ivalue, rvalue, cvalue, lcvalue, optype, iopts, opts, fail):
    """
    Option getting function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d01/d01zlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue, rvalue, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue, iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('d01zlc', ivalue, 'ivalue')
    _rvalue = nag_double_type_check_and_cast('d01zlc', rvalue, 'rvalue')
    _iopts = nag_integer_type_check_and_cast('d01zlc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('d01zlc', opts, 'opts')
    d01zlc_ctypes(optstr, _ivalue, _rvalue, cvalue, lcvalue, optype, _iopts, _opts, fail)

nag_quad_1d_fin_smooth = d01bdc
nag_quad_2d_fin = d01dac
nag_quad_md_gauss = d01fbc
nag_quad_md_sphere = d01fdc
nag_1d_quad_vals = d01gac
nag_quad_md_numth_vec = d01gdc
nag_quad_md_numth_coeff_prime = d01gyc
nag_quad_md_numth_coeff_2prime = d01gzc
nag_quad_md_simplex = d01pac
nag_1d_quad_gen_1 = d01sjc
nag_1d_quad_osc_1 = d01skc
nag_1d_quad_brkpts_1 = d01slc
nag_1d_quad_inf_1 = d01smc
nag_1d_quad_wt_trig_1 = d01snc
nag_1d_quad_wt_alglog_1 = d01spc
nag_1d_quad_wt_cauchy_1 = d01sqc
nag_1d_quad_inf_wt_trig_1 = d01ssc
nag_quad_1d_gen_vec_multi_rcomm = d01rac
nag_quad_1d_gen_vec_multi_dimreq = d01rcc
nag_quad_1d_fin_gonnet_vec = d01rgc
nag_1d_quad_gauss_1 = d01tac
nag_quad_1d_gauss_wset = d01tbc
nag_quad_1d_gauss_wgen = d01tcc
nag_quad_1d_gauss_wrec = d01tdc
nag_quad_1d_gauss_recm = d01tec
nag_quad_1d_gauss_vec = d01uac
nag_quad_1d_inf_exp_wt = d01ubc
nag_multid_quad_adapt_1 = d01wcc
nag_multid_quad_monte_carlo_1 = d01xbc
nag_quad_opt_set = d01zkc
nag_quad_opt_get = d01zlc
__all__ = ['d01bdc', 'd01dac', 'd01fbc',
           'd01fdc', 'd01gac', 'd01gdc', 'd01gyc',
           'd01gzc', 'd01pac', 'd01rac', 'd01rcc',
           'd01rgc', 'd01sjc', 'd01skc', 'd01slc', 'd01smc', 'd01snc',
           'd01spc', 'd01sqc', 'd01ssc', 'd01tac', 'd01tbc', 'd01tcc',
           'd01tdc', 'd01tec',
           'd01uac', 'd01ubc', 'd01wcc', 'd01xbc', 'd01zkc', 'd01zlc',
           'nag_1d_quad_brkpts_1', 'nag_1d_quad_gauss_1',
           'nag_1d_quad_gen_1', 'nag_1d_quad_inf_1',
           'nag_1d_quad_inf_wt_trig_1', 'nag_1d_quad_osc_1',
           'nag_1d_quad_vals', 'nag_1d_quad_wt_alglog_1',
           'nag_1d_quad_wt_cauchy_1', 'nag_1d_quad_wt_trig_1',
           'nag_multid_quad_adapt_1',
           'nag_multid_quad_monte_carlo_1',
           'nag_quad_1d_fin_gonnet_vec', 'nag_quad_1d_fin_smooth',
           'nag_quad_1d_gauss_vec', 'nag_quad_1d_gauss_wgen',
           'nag_quad_1d_gauss_wset',
           'nag_quad_1d_gen_vec_multi_dimreq',
           'nag_quad_1d_gen_vec_multi_rcomm',
           'nag_quad_2d_fin',
           'nag_quad_md_gauss',
           'nag_quad_md_numth_coeff_2prime',
           'nag_quad_md_numth_coeff_prime', 'nag_quad_md_numth_vec',
           'nag_quad_md_simplex', 'nag_quad_md_sphere',
           'nag_quad_opt_get',
           'nag_quad_opt_set',
           'nag_quad_1d_gauss_wrec',
           'nag_quad_1d_gauss_recm',
           'nag_quad_1d_inf_exp_wt']
