"""
Deprecated wrappers for the NAG C Library (Mark 26.0) d02 Chapter.

d02 - Ordinary Differential Equations.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/D02/d02conts.html

NAG Copyright 2017.
"""

from ctypes import c_double
from ctypes import POINTER
from ctypes import c_char_p
from .util import get_library
from .util import Integer
from .util import Nag_Comm
from .util import NagError
from .util import Nag_ErrorControl
from .util import NAG_D02CJC_FUN
from .util import NAG_D02CJC_OUTFUN
from .util import NAG_D02CJC_GFUN
from .util import Nag_User
from .util import NAG_D02EJC_FUN
from .util import NAG_D02EJC_PFUN
from .util import NAG_D02EJC_OUTFUN
from .util import NAG_D02EJC_GFUN
from .util import NAG_D02GAC_FUN
from .util import NAG_D02GBC_FUN
from .util import NAG_D02GBC_GFUN
from .util import Nag_Boolean
from .util import Nag_EvaluateJacobian
from .util import NAG_D02NEC_RES
from .util import NAG_D02NEC_JAC
from .util import NAG_D02PEC_F
from .util import NAG_D02PFC_F
from .util import Nag_RK_method
from .util import Nag_ErrorAssess
from .util import Nag_SolDeriv
from .util import NAG_D02PSC_F
from .util import NAG_D02QFC_FUN
from .util import NAG_D02QFC_GFUN
from .util import Nag_ODE_Adams
from .util import Nag_Start
from .util import NAG_D02RAC_FUN
from .util import NAG_D02RAC_GFUN
from .util import Nag_MeshSet
from .util import NAG_D02RAC_JFUN
from .util import NAG_D02RAC_JGFUN
from .util import NAG_D02RAC_JEPSFUN
from .util import NAG_D02RAC_JGEPSFUN
from .util import NAG_D02TLC_FFUN
from .util import NAG_D02TLC_FJAC
from .util import NAG_D02TLC_GAFUN
from .util import NAG_D02TLC_GBFUN
from .util import NAG_D02TLC_GAJAC
from .util import NAG_D02TLC_GBJAC
from .util import NAG_D02TLC_GUESS

_library_cl = get_library()

STRING = c_char_p

d02cjc_ctypes = _library_cl.d02cjc
d02cjc_ctypes.restype = None
d02cjc_ctypes.argtypes = [Integer, NAG_D02CJC_FUN, POINTER(c_double), POINTER(c_double), c_double, c_double, Nag_ErrorControl, NAG_D02CJC_OUTFUN, NAG_D02CJC_GFUN, POINTER(Nag_User), POINTER(NagError)]


def d02cjc(neq, fcn, x, y, xend, tol, errc, output, g, comm, fail):
    """
    Ordinary differential equation solver using a variable-order variable-
    step Adams' method (Black Box).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02cjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('d02cjc', x, 'x')
    _y = nag_double_type_check_and_cast('d02cjc', y, 'y')
    if output is None:
        output = cast(None, NAG_D02CJC_OUTFUN)
    if g is None:
        g = cast(None, NAG_D02CJC_GFUN)
    d02cjc_ctypes(neq, fcn, _x, _y, xend, tol, errc, output, g, comm, fail)

d02ejc_ctypes = _library_cl.d02ejc
d02ejc_ctypes.restype = None
d02ejc_ctypes.argtypes = [Integer, NAG_D02EJC_FUN, NAG_D02EJC_PFUN, POINTER(c_double), POINTER(c_double), c_double, c_double, Nag_ErrorControl, NAG_D02EJC_OUTFUN, NAG_D02EJC_GFUN, POINTER(Nag_User), POINTER(NagError)]


def d02ejc(neq, fcn, pederv, x, y, xend, tol, errc, output, g, comm, fail):
    """
    Ordinary differential equations solver, stiff, initial value problems
    using the Backward Differentiation Formulae.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02ejc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('d02ejc', x, 'x')
    _y = nag_double_type_check_and_cast('d02ejc', y, 'y')
    if pederv is None:
        pederv = cast(None, NAG_D02EJC_PFUN)
    if output is None:
        output = cast(None, NAG_D02EJC_OUTFUN)
    if g is None:
        g = cast(None, NAG_D02EJC_GFUN)
    d02ejc_ctypes(neq, fcn, pederv, _x, _y, xend, tol, errc, output, g, comm, fail)

d02gac_ctypes = _library_cl.d02gac
d02gac_ctypes.restype = None
d02gac_ctypes.argtypes = [Integer, NAG_D02GAC_FUN, c_double, c_double, POINTER(c_double), POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), c_double, POINTER(Nag_User), POINTER(NagError)]


def d02gac(neq, fcn, a, b, u, v, mnp, np, x, y, tol, comm, fail):
    """
    Ordinary differential equations solver, for simple nonlinear two-point
    boundary value problems, using a finite difference technique with
    deferred correction.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        u, v, np, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        u, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        v, np
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _u = nag_double_type_check_and_cast('d02gac', u, 'u')
    _v = nag_integer_type_check_and_cast('d02gac', v, 'v')
    _np = nag_integer_type_check_and_cast('d02gac', np, 'np')
    _x = nag_double_type_check_and_cast('d02gac', x, 'x')
    _y = nag_double_type_check_and_cast('d02gac', y, 'y')
    d02gac_ctypes(neq, fcn, a, b, _u, _v, mnp, _np, _x, _y, tol, comm, fail)

d02gbc_ctypes = _library_cl.d02gbc
d02gbc_ctypes.restype = None
d02gbc_ctypes.argtypes = [Integer, NAG_D02GBC_FUN, NAG_D02GBC_GFUN, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), c_double, POINTER(Nag_User), POINTER(NagError)]


def d02gbc(neq, fcnf, fcng, a, b, c, d, gam, mnp, np, x, y, tol, comm, fail):
    """
    Ordinary differential equations solver, for general linear two-point
    boundary value problems, using a finite difference technique with
    deferred correction.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02gbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        c, d, gam, np, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c, d, gam, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        np
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    if fcng is None:
        fcng = cast(None, NAG_D02GBC_GFUN)
    _c = nag_double_type_check_and_cast('d02gbc', c, 'c')
    _d = nag_double_type_check_and_cast('d02gbc', d, 'd')
    _gam = nag_double_type_check_and_cast('d02gbc', gam, 'gam')
    _np = nag_integer_type_check_and_cast('d02gbc', np, 'np')
    _x = nag_double_type_check_and_cast('d02gbc', x, 'x')
    _y = nag_double_type_check_and_cast('d02gbc', y, 'y')
    d02gbc_ctypes(neq, fcnf, fcng, a, b, _c, _d, _gam, mnp, _np, _x, _y, tol, comm, fail)

d02mcc_ctypes = _library_cl.d02mcc
d02mcc_ctypes.restype = None
d02mcc_ctypes.argtypes = [POINTER(Integer)]


def d02mcc(icom):
    """
    DASSL method continuation resetting function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02mcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icom
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icom
    """
    from .util import nag_integer_type_check_and_cast
    _icom = nag_integer_type_check_and_cast('d02mcc', icom, 'icom')
    d02mcc_ctypes(_icom)

d02mwc_ctypes = _library_cl.d02mwc
d02mwc_ctypes.restype = None
d02mwc_ctypes.argtypes = [Integer, Integer, Nag_EvaluateJacobian, c_double, c_double, Nag_Boolean, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def d02mwc(neq, maxord, jceval, hmax, h0, vectortol, icom, licom, com, lcom, fail):
    """
    Implicit ordinary differential equations/DAEs, initial value problem,
    setup for d02nec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02mwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icom, com
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        com
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icom
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _icom = nag_integer_type_check_and_cast('d02mwc', icom, 'icom')
    _com = nag_double_type_check_and_cast('d02mwc', com, 'com')
    d02mwc_ctypes(neq, maxord, jceval, hmax, h0, vectortol, _icom, licom, _com, lcom, fail)

d02nec_ctypes = _library_cl.d02nec
d02nec_ctypes.restype = None
d02nec_ctypes.argtypes = [Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), NAG_D02NEC_RES, NAG_D02NEC_JAC, POINTER(Integer), POINTER(c_double), Integer, POINTER(Nag_Comm), POINTER(NagError)]


def d02nec(neq, t, tout, y, ydot, rtol, atol, itask, res, jac, icom, com, lcom, comm, fail):
    """
    Implicit ordinary differential equations/DAEs, initial value problem,
    DASSL method integrator.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02nec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t, y, ydot, rtol, atol, itask, icom, com
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, y, ydot, rtol, atol, com
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        itask, icom
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _t = nag_double_type_check_and_cast('d02nec', t, 't')
    _y = nag_double_type_check_and_cast('d02nec', y, 'y')
    _ydot = nag_double_type_check_and_cast('d02nec', ydot, 'ydot')
    _rtol = nag_double_type_check_and_cast('d02nec', rtol, 'rtol')
    _atol = nag_double_type_check_and_cast('d02nec', atol, 'atol')
    _itask = nag_integer_type_check_and_cast('d02nec', itask, 'itask')
    if jac is None:
        jac = cast(None, NAG_D02NEC_JAC)
    _icom = nag_integer_type_check_and_cast('d02nec', icom, 'icom')
    _com = nag_double_type_check_and_cast('d02nec', com, 'com')
    d02nec_ctypes(neq, _t, tout, _y, _ydot, _rtol, _atol, _itask, res, jac, _icom, _com, lcom, comm, fail)

d02npc_ctypes = _library_cl.d02npc
d02npc_ctypes.restype = None
d02npc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(Integer), Integer, POINTER(NagError)]


def d02npc(neq, ml, mu, icom, licom, fail):
    """
    Implicit ordinary differential equations/DAEs, initial value problem
    linear algebra setup function for d02nec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02npc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icom
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icom
    """
    from .util import nag_integer_type_check_and_cast
    _icom = nag_integer_type_check_and_cast('d02npc', icom, 'icom')
    d02npc_ctypes(neq, ml, mu, _icom, licom, fail)

d02pec_ctypes = _library_cl.d02pec
d02pec_ctypes.restype = None
d02pec_ctypes.argtypes = [NAG_D02PEC_F, Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d02pec(f, n, twant, tgot, ygot, ypgot, ymax, comm, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, Runge--Kutta
    method, integration over range with output.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02pec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tgot, ygot, ypgot, ymax, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        tgot, ygot, ypgot, ymax, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _tgot = nag_double_type_check_and_cast('d02pec', tgot, 'tgot')
    _ygot = nag_double_type_check_and_cast('d02pec', ygot, 'ygot')
    _ypgot = nag_double_type_check_and_cast('d02pec', ypgot, 'ypgot')
    _ymax = nag_double_type_check_and_cast('d02pec', ymax, 'ymax')
    _iwsav = nag_integer_type_check_and_cast('d02pec', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02pec', rwsav, 'rwsav')
    d02pec_ctypes(f, n, twant, _tgot, _ygot, _ypgot, _ymax, comm, _iwsav, _rwsav, fail)

d02pfc_ctypes = _library_cl.d02pfc
d02pfc_ctypes.restype = None
d02pfc_ctypes.argtypes = [NAG_D02PFC_F, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d02pfc(f, n, tnow, ynow, ypnow, comm, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, Runge--Kutta
    method, integration over one step.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02pfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tnow, ynow, ypnow, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        tnow, ynow, ypnow, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _tnow = nag_double_type_check_and_cast('d02pfc', tnow, 'tnow')
    _ynow = nag_double_type_check_and_cast('d02pfc', ynow, 'ynow')
    _ypnow = nag_double_type_check_and_cast('d02pfc', ypnow, 'ypnow')
    _iwsav = nag_integer_type_check_and_cast('d02pfc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02pfc', rwsav, 'rwsav')
    d02pfc_ctypes(f, n, _tnow, _ynow, _ypnow, comm, _iwsav, _rwsav, fail)

d02pgc_ctypes = _library_cl.d02pgc
d02pgc_ctypes.restype = None
d02pgc_ctypes.argtypes = [POINTER(Integer), Integer, POINTER(c_double),
                          POINTER(c_double), POINTER(c_double), POINTER(Integer),
                          POINTER(c_double), POINTER(NagError)]


def d02pgc(irevcm, n, t, y, yp, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, Runge--Kutta
    method, integration by reverse communication.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02pgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, t, y, yp, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, y, yp, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('d02pgc', irevcm, 'irevcm')
    _t = nag_double_type_check_and_cast('d02pgc', t, 't')
    _y = nag_double_type_check_and_cast('d02pgc', y, 'y')
    _yp = nag_double_type_check_and_cast('d02pgc', yp, 'yp')
    _iwsav = nag_integer_type_check_and_cast('d02pgc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02pgc', rwsav, 'rwsav')
    d02pgc_ctypes(_irevcm, n, _t, _y, _yp, _iwsav, _rwsav, fail)

d02phc_ctypes = _library_cl.d02phc
d02phc_ctypes.restype = None
d02phc_ctypes.argtypes = [POINTER(Integer), Integer, Integer,
                          POINTER(c_double), POINTER(c_double),
                          POINTER(c_double), POINTER(c_double),
                          Integer, POINTER(Integer), POINTER(c_double),
                          POINTER(NagError)]


def d02phc(irevcm, n, nwant, t, y, yp, wcomm, lwcomm, iwsav, rwsav, fail):
    """
    Set up interpolant by reverse communication for solution and derivative
    evaluations at points within the range of the last integration step taken
    by d02pgc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02phc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevm, t, y, yp, wcomm, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, y, yp, wcomm, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevm, iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('d02phc', irevcm, 'irevcm')
    _t = nag_double_type_check_and_cast('d02phc', t, 't')
    _y = nag_double_type_check_and_cast('d02phc', y, 'y')
    _yp = nag_double_type_check_and_cast('d02phc', yp, 'yp')
    _wcomm = nag_double_type_check_and_cast('d02phc', wcomm, 'wcomm')
    _iwsav = nag_integer_type_check_and_cast('d02phc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02phc', rwsav, 'rwsav')
    d02phc_ctypes(_irevcm, n, nwant, _t, _y, _yp, _wcomm, lwcomm,
                  _iwsav, _rwsav, fail)

d02pjc_ctypes = _library_cl.d02pjc
d02pjc_ctypes.restype = None
d02pjc_ctypes.argtypes = [Nag_Boolean, Integer, Integer, c_double,
                          Integer, POINTER(c_double), POINTER(c_double),
                          Integer, POINTER(Integer), POINTER(c_double),
                          POINTER(NagError)]


def d02pjc(icheck, n, nwant, t, ideriv, sol, wcomm, lwcomm, iwsav, rwsav,
           fail):
    """
    Evaluate interpolant, set up using d02pqc, to approximate solution and/or
    solution derivatives at a point within the range of the last integration
    step taken by d02pgc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02pjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        sol, wcomm, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        sol, wcomm, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _sol = nag_double_type_check_and_cast('d02pjc', sol, 'sol')
    _wcomm = nag_double_type_check_and_cast('d02pjc', wcomm, 'wcomm')
    _iwsav = nag_integer_type_check_and_cast('d02pjc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02pjc', rwsav, 'rwsav')
    d02pjc_ctypes(icheck, n, nwant, t, ideriv, _sol, _wcomm, lwcomm,
                  _iwsav, _rwsav, fail)

d02pqc_ctypes = _library_cl.d02pqc
d02pqc_ctypes.restype = None
d02pqc_ctypes.argtypes = [Integer, c_double, c_double, POINTER(c_double), c_double, POINTER(c_double), Nag_RK_method, Nag_ErrorAssess, c_double, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d02pqc(n, tstart, tend, yinit, tol, thresh, method, errass, hstart, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, setup for d02pec
    and d02pfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02pqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        yinit, thresh, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        yinit, thresh, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _yinit = nag_double_type_check_and_cast('d02pqc', yinit, 'yinit')
    _thresh = nag_double_type_check_and_cast('d02pqc', thresh, 'thresh')
    _iwsav = nag_integer_type_check_and_cast('d02pqc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02pqc', rwsav, 'rwsav')
    d02pqc_ctypes(n, tstart, tend, _yinit, tol, _thresh, method, errass, hstart, _iwsav, _rwsav, fail)

d02prc_ctypes = _library_cl.d02prc
d02prc_ctypes.restype = None
d02prc_ctypes.argtypes = [c_double, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d02prc(tendnu, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, resets end of
    range for d02pfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02prc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iwsav = nag_integer_type_check_and_cast('d02prc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02prc', rwsav, 'rwsav')
    d02prc_ctypes(tendnu, _iwsav, _rwsav, fail)

d02psc_ctypes = _library_cl.d02psc
d02psc_ctypes.restype = None
d02psc_ctypes.argtypes = [Integer, c_double, Nag_SolDeriv, Integer, POINTER(c_double), POINTER(c_double), NAG_D02PSC_F, POINTER(c_double), Integer, POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d02psc(n, twant, reqest, nwant, ywant, ypwant, f, wcomm, lwcomm, comm, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, interpolation
    for d02pfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02psc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ywant, ypwant, wcomm, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ywant, ypwant, wcomm, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ywant = nag_double_type_check_and_cast('d02psc', ywant, 'ywant')
    _ypwant = nag_double_type_check_and_cast('d02psc', ypwant, 'ypwant')
    _wcomm = nag_double_type_check_and_cast('d02psc', wcomm, 'wcomm')
    _iwsav = nag_integer_type_check_and_cast('d02psc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02psc', rwsav, 'rwsav')
    d02psc_ctypes(n, twant, reqest, nwant, _ywant, _ypwant, f, _wcomm, lwcomm, comm, _iwsav, _rwsav, fail)

d02ptc_ctypes = _library_cl.d02ptc
d02ptc_ctypes.restype = None
d02ptc_ctypes.argtypes = [POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d02ptc(fevals, stepcost, waste, stepsok, hnext, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, integration
    diagnostics for d02pec and d02pfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02ptc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        fevals, stepcost, waste, stepsok, hnext, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        waste, hnext, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        fevals, stepcost, stepsok, iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _fevals = nag_integer_type_check_and_cast('d02ptc', fevals, 'fevals')
    _stepcost = nag_integer_type_check_and_cast('d02ptc', stepcost, 'stepcost')
    _waste = nag_double_type_check_and_cast('d02ptc', waste, 'waste')
    _stepsok = nag_integer_type_check_and_cast('d02ptc', stepsok, 'stepsok')
    _hnext = nag_double_type_check_and_cast('d02ptc', hnext, 'hnext')
    _iwsav = nag_integer_type_check_and_cast('d02ptc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02ptc', rwsav, 'rwsav')
    d02ptc_ctypes(_fevals, _stepcost, _waste, _stepsok, _hnext, _iwsav, _rwsav, fail)

d02puc_ctypes = _library_cl.d02puc
d02puc_ctypes.restype = None
d02puc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def d02puc(n, rmserr, errmax, terrmx, iwsav, rwsav, fail):
    """
    Ordinary differential equations, initial value problem, error assessment
    diagnostics for d02pec and d02pfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02puc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rmserr, errmax, terrmx, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rmserr, errmax, terrmx, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _rmserr = nag_double_type_check_and_cast('d02puc', rmserr, 'rmserr')
    _errmax = nag_double_type_check_and_cast('d02puc', errmax, 'errmax')
    _terrmx = nag_double_type_check_and_cast('d02puc', terrmx, 'terrmx')
    _iwsav = nag_integer_type_check_and_cast('d02puc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('d02puc', rwsav, 'rwsav')
    d02puc_ctypes(n, _rmserr, _errmax, _terrmx, _iwsav, _rwsav, fail)

d02qfc_ctypes = _library_cl.d02qfc
d02qfc_ctypes.restype = None
d02qfc_ctypes.argtypes = [Integer, NAG_D02QFC_FUN, POINTER(c_double), POINTER(c_double), c_double, NAG_D02QFC_GFUN, POINTER(Nag_User), POINTER(Nag_ODE_Adams), POINTER(NagError)]


def d02qfc(neqf, fcn, t, y, tout, g, comm, opt, fail):
    """
    Ordinary differential equation solver using Adams' method (sophisticated
    use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02qfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, y
    """
    from .util import nag_double_type_check_and_cast
    _t = nag_double_type_check_and_cast('d02qfc', t, 't')
    _y = nag_double_type_check_and_cast('d02qfc', y, 'y')
    d02qfc_ctypes(neqf, fcn, _t, _y, tout, g, comm, opt, fail)

d02qwc_ctypes = _library_cl.d02qwc
d02qwc_ctypes.restype = None
d02qwc_ctypes.argtypes = [POINTER(Nag_Start), Integer, Nag_Boolean, POINTER(c_double), POINTER(c_double), Nag_Boolean, Nag_Boolean, c_double, c_double, Integer, Integer, POINTER(Nag_Boolean), Nag_Boolean, POINTER(Nag_ODE_Adams), POINTER(NagError)]


def d02qwc(state, neqf, vectol, atol, rtol, onestep, crit, tcrit, hmax, maxstep, neqg, alterg, sophist, opt, fail):
    """
    Setup function for d02qfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02qwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        atol, rtol
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        atol, rtol
    """
    from .util import nag_double_type_check_and_cast
    _atol = nag_double_type_check_and_cast('d02qwc', atol, 'atol')
    _rtol = nag_double_type_check_and_cast('d02qwc', rtol, 'rtol')
    d02qwc_ctypes(state, neqf, vectol, _atol, _rtol, onestep, crit, tcrit, hmax, maxstep, neqg, alterg, sophist, opt, fail)

d02qyc_ctypes = _library_cl.d02qyc
d02qyc_ctypes.restype = None
d02qyc_ctypes.argtypes = [POINTER(Nag_ODE_Adams)]


def d02qyc(opt):
    """
    Freeing function for use with d02qfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02qyc.html
    """
    d02qyc_ctypes(opt)

d02qzc_ctypes = _library_cl.d02qzc
d02qzc_ctypes.restype = None
d02qzc_ctypes.argtypes = [Integer, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_ODE_Adams), POINTER(NagError)]


def d02qzc(neqf, twant, nwant, ywant, ypwant, opt, fail):
    """
    Interpolation function for use with d02qfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02qzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ywant, ypwant
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ywant, ypwant
    """
    from .util import nag_double_type_check_and_cast
    _ywant = nag_double_type_check_and_cast('d02qzc', ywant, 'ywant')
    _ypwant = nag_double_type_check_and_cast('d02qzc', ypwant, 'ypwant')
    d02qzc_ctypes(neqf, twant, nwant, _ywant, _ypwant, opt, fail)

d02rac_ctypes = _library_cl.d02rac
d02rac_ctypes.restype = None
d02rac_ctypes.argtypes = [Integer, POINTER(c_double), NAG_D02RAC_FUN, Integer, Integer, NAG_D02RAC_GFUN, Nag_MeshSet, Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), NAG_D02RAC_JFUN, NAG_D02RAC_JGFUN, NAG_D02RAC_JEPSFUN, NAG_D02RAC_JGEPSFUN, POINTER(Nag_User), POINTER(NagError)]


def d02rac(neq, deleps, fcn, numbeg, nummix, g, init, mnp, np, x, y, tol, abt, jacobf, jacobg, jaceps, jacgep, comm, fail):
    """
    Ordinary differential equations solver, for general nonlinear two-point
    boundary value problems, using a finite difference technique with
    deferred correction.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02rac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        deleps, np, x, y, abt
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        deleps, x, y, abt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        np
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _deleps = nag_double_type_check_and_cast('d02rac', deleps, 'deleps')
    _np = nag_integer_type_check_and_cast('d02rac', np, 'np')
    _x = nag_double_type_check_and_cast('d02rac', x, 'x')
    _y = nag_double_type_check_and_cast('d02rac', y, 'y')
    _abt = nag_double_type_check_and_cast('d02rac', abt, 'abt')
    if jacobf is None:
        jacobf = cast(None, NAG_D02RAC_JFUN)
    if jacobg is None:
        jacobg = cast(None, NAG_D02RAC_JGFUN)
    if jaceps is None:
        jaceps = cast(None, NAG_D02RAC_JEPSFUN)
    if jacgep is None:
        jacgep = cast(None, NAG_D02RAC_JGEPSFUN)
    d02rac_ctypes(neq, _deleps, fcn, numbeg, nummix, g, init, mnp, _np, _x, _y, tol, _abt, jacobf, jacobg, jaceps, jacgep, comm, fail)

d02tlc_ctypes = _library_cl.d02tlc
d02tlc_ctypes.restype = None
d02tlc_ctypes.argtypes = [NAG_D02TLC_FFUN, NAG_D02TLC_FJAC, NAG_D02TLC_GAFUN, NAG_D02TLC_GBFUN, NAG_D02TLC_GAJAC, NAG_D02TLC_GBJAC, NAG_D02TLC_GUESS, POINTER(c_double), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def d02tlc(ffun, fjac, gafun, gbfun, gajac, gbjac, guess, rcomm, icomm, comm, fail):
    """
    Ordinary differential equations, general nonlinear boundary value
    problem, collocation technique (thread safe).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02tlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _rcomm = nag_double_type_check_and_cast('d02tlc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('d02tlc', icomm, 'icomm')
    d02tlc_ctypes(ffun, fjac, gafun, gbfun, gajac, gbjac, guess, _rcomm, _icomm, comm, fail)

d02tvc_ctypes = _library_cl.d02tvc
d02tvc_ctypes.restype = None
d02tvc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, Integer, Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(NagError)]


def d02tvc(neq, m, nlbc, nrbc, ncol, tols, mxmesh, nmesh, mesh, ipmesh, rcomm, lrcomm, icomm, licomm, fail):
    """
    Ordinary differential equations, general nonlinear boundary value
    problem, setup for d02tlc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02tvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        m, tols, mesh, ipmesh, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        tols, mesh, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, ipmesh, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _m = nag_integer_type_check_and_cast('d02tvc', m, 'm')
    _tols = nag_double_type_check_and_cast('d02tvc', tols, 'tols')
    _mesh = nag_double_type_check_and_cast('d02tvc', mesh, 'mesh')
    _ipmesh = nag_integer_type_check_and_cast('d02tvc', ipmesh, 'ipmesh')
    _rcomm = nag_double_type_check_and_cast('d02tvc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('d02tvc', icomm, 'icomm')
    d02tvc_ctypes(neq, _m, nlbc, nrbc, ncol, _tols, mxmesh, nmesh, _mesh, _ipmesh, _rcomm, lrcomm, _icomm, licomm, fail)

d02txc_ctypes = _library_cl.d02txc
d02txc_ctypes.restype = None
d02txc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def d02txc(mxmesh, nmesh, mesh, ipmesh, rcomm, icomm, fail):
    """
    Ordinary differential equations, general nonlinear boundary value
    problem, continuation facility for d02tlc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02txc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        mesh, ipmesh, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        mesh, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipmesh, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _mesh = nag_double_type_check_and_cast('d02txc', mesh, 'mesh')
    _ipmesh = nag_integer_type_check_and_cast('d02txc', ipmesh, 'ipmesh')
    _rcomm = nag_double_type_check_and_cast('d02txc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('d02txc', icomm, 'icomm')
    d02txc_ctypes(mxmesh, nmesh, _mesh, _ipmesh, _rcomm, _icomm, fail)

d02tyc_ctypes = _library_cl.d02tyc
d02tyc_ctypes.restype = None
d02tyc_ctypes.argtypes = [c_double, POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def d02tyc(x, y, neq, mmax, rcomm, icomm, fail):
    """
    Ordinary differential equations, general nonlinear boundary value
    problem, interpolation for d02tlc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02tyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _y = nag_double_type_check_and_cast('d02tyc', y, 'y')
    _rcomm = nag_double_type_check_and_cast('d02tyc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('d02tyc', icomm, 'icomm')
    d02tyc_ctypes(x, _y, neq, mmax, _rcomm, _icomm, fail)

d02tzc_ctypes = _library_cl.d02tzc
d02tzc_ctypes.restype = None
d02tzc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def d02tzc(mxmesh, nmesh, mesh, ipmesh, ermx, iermx, ijermx, rcomm, icomm, fail):
    """
    Ordinary differential equations, general nonlinear boundary value
    problem, diagnostics for d02tlc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02tzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nmesh, mesh, ipmesh, ermx, iermx, ijermx, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        mesh, ermx, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nmesh, ipmesh, iermx, ijermx, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nmesh = nag_integer_type_check_and_cast('d02tzc', nmesh, 'nmesh')
    _mesh = nag_double_type_check_and_cast('d02tzc', mesh, 'mesh')
    _ipmesh = nag_integer_type_check_and_cast('d02tzc', ipmesh, 'ipmesh')
    _ermx = nag_double_type_check_and_cast('d02tzc', ermx, 'ermx')
    _iermx = nag_integer_type_check_and_cast('d02tzc', iermx, 'iermx')
    _ijermx = nag_integer_type_check_and_cast('d02tzc', ijermx, 'ijermx')
    _rcomm = nag_double_type_check_and_cast('d02tzc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('d02tzc', icomm, 'icomm')
    d02tzc_ctypes(mxmesh, _nmesh, _mesh, _ipmesh, _ermx, _iermx, _ijermx, _rcomm, _icomm, fail)

d02uac_ctypes = _library_cl.d02uac
d02uac_ctypes.restype = None
d02uac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d02uac(n, f, c, fail):
    """
    Coefficients of Chebyshev interpolating polynomial from function values
    on Chebyshev grid.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02uac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        f, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        f, c
    """
    from .util import nag_double_type_check_and_cast
    _f = nag_double_type_check_and_cast('d02uac', f, 'f')
    _c = nag_double_type_check_and_cast('d02uac', c, 'c')
    d02uac_ctypes(n, _f, _c, fail)

d02ubc_ctypes = _library_cl.d02ubc
d02ubc_ctypes.restype = None
d02ubc_ctypes.argtypes = [Integer, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d02ubc(n, a, b, q, c, f, fail):
    """
    Function or low-order-derivative values on Chebyshev grid from
    coefficients of Chebyshev interpolating polynomial.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02ubc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        c, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c, f
    """
    from .util import nag_double_type_check_and_cast
    _c = nag_double_type_check_and_cast('d02ubc', c, 'c')
    _f = nag_double_type_check_and_cast('d02ubc', f, 'f')
    d02ubc_ctypes(n, a, b, q, _c, _f, fail)

d02ucc_ctypes = _library_cl.d02ucc
d02ucc_ctypes.restype = None
d02ucc_ctypes.argtypes = [Integer, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def d02ucc(n, a, b, x, fail):
    """
    Chebyshev Gauss--Lobatto grid generation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02ucc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('d02ucc', x, 'x')
    d02ucc_ctypes(n, a, b, _x, fail)

d02udc_ctypes = _library_cl.d02udc
d02udc_ctypes.restype = None
d02udc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d02udc(n, f, fd, fail):
    """
    Differentiate a function by the FFT using function values on Chebyshev
    grid.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02udc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        f, fd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        f, fd
    """
    from .util import nag_double_type_check_and_cast
    _f = nag_double_type_check_and_cast('d02udc', f, 'f')
    _fd = nag_double_type_check_and_cast('d02udc', fd, 'fd')
    d02udc_ctypes(n, _f, _fd, fail)

d02uec_ctypes = _library_cl.d02uec
d02uec_ctypes.restype = None
d02uec_ctypes.argtypes = [Integer, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d02uec(n, a, b, m, c, bmat, y, bvec, f, uc, resid, fail):
    """
    Solve linear constant coefficient boundary value problem on Chebyshev
    grid, Integral formulation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02uec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        c, bmat, y, bvec, f, uc, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c, bmat, y, bvec, f, uc, resid
    """
    from .util import nag_double_type_check_and_cast
    _c = nag_double_type_check_and_cast('d02uec', c, 'c')
    _bmat = nag_double_type_check_and_cast('d02uec', bmat, 'bmat')
    _y = nag_double_type_check_and_cast('d02uec', y, 'y')
    _bvec = nag_double_type_check_and_cast('d02uec', bvec, 'bvec')
    _f = nag_double_type_check_and_cast('d02uec', f, 'f')
    _uc = nag_double_type_check_and_cast('d02uec', uc, 'uc')
    _resid = nag_double_type_check_and_cast('d02uec', resid, 'resid')
    d02uec_ctypes(n, a, b, m, _c, _bmat, _y, _bvec, _f, _uc, _resid, fail)

d02uwc_ctypes = _library_cl.d02uwc
d02uwc_ctypes.restype = None
d02uwc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d02uwc(n, nip, x, f, xip, fip, fail):
    """
    Interpolate a function from Chebyshev grid to uniform grid using
    barycentric Lagrange interpolation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02uwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, xip, fip
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, xip, fip
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('d02uwc', x, 'x')
    _f = nag_double_type_check_and_cast('d02uwc', f, 'f')
    _xip = nag_double_type_check_and_cast('d02uwc', xip, 'xip')
    _fip = nag_double_type_check_and_cast('d02uwc', fip, 'fip')
    d02uwc_ctypes(n, nip, _x, _f, _xip, _fip, fail)

d02uyc_ctypes = _library_cl.d02uyc
d02uyc_ctypes.restype = None
d02uyc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(NagError)]


def d02uyc(n, w, fail):
    """
    Clenshaw--Curtis quadrature weights for integration using computed
    Chebyshev coefficients.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02uyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    """
    from .util import nag_double_type_check_and_cast
    _w = nag_double_type_check_and_cast('d02uyc', w, 'w')
    d02uyc_ctypes(n, _w, fail)

d02uzc_ctypes = _library_cl.d02uzc
d02uzc_ctypes.restype = None
d02uzc_ctypes.argtypes = [Integer, c_double, POINTER(c_double), POINTER(NagError)]


def d02uzc(k, x, t, fail):
    """
    Chebyshev polynomial evaluation, T_k(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d02/d02uzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t
    """
    from .util import nag_double_type_check_and_cast
    _t = nag_double_type_check_and_cast('d02uzc', t, 't')
    d02uzc_ctypes(k, x, _t, fail)

nag_ode_ivp_adams_gen = d02cjc
nag_ode_ivp_bdf_gen = d02ejc
nag_ode_bvp_fd_nonlin_fixedbc = d02gac
nag_ode_bvp_fd_lin_gen = d02gbc
nag_dae_ivp_dassl_cont = d02mcc
nag_dae_ivp_dassl_setup = d02mwc
nag_dae_ivp_dassl_gen = d02nec
nag_dae_ivp_dassl_linalg = d02npc
nag_ode_ivp_rkts_range = d02pec
nag_ode_ivp_rkts_onestep = d02pfc
nag_ode_ivp_rk_step_revcomm = d02pgc
nag_ode_ivp_rk_interp_setup = d02phc
nag_ode_ivp_rk_interp_eval = d02pjc
nag_ode_ivp_rkts_setup = d02pqc
nag_ode_ivp_rkts_reset_tend = d02prc
nag_ode_ivp_rkts_interp = d02psc
nag_ode_ivp_rkts_diag = d02ptc
nag_ode_ivp_rkts_errass = d02puc
nag_ode_ivp_adams_roots = d02qfc
nag_ode_ivp_adams_setup = d02qwc
nag_ode_ivp_adams_free = d02qyc
nag_ode_ivp_adams_interp = d02qzc
nag_ode_bvp_fd_nonlin_gen = d02rac
nag_ode_bvp_coll_nlin_solve = d02tlc
nag_ode_bvp_coll_nlin_setup = d02tvc
nag_ode_bvp_coll_nlin_contin = d02txc
nag_ode_bvp_coll_nlin_interp = d02tyc
nag_ode_bvp_coll_nlin_diag = d02tzc
nag_ode_bvp_ps_lin_coeffs = d02uac
nag_ode_bvp_ps_lin_cgl_vals = d02ubc
nag_ode_bvp_ps_lin_cgl_grid = d02ucc
nag_ode_bvp_ps_lin_cgl_deriv = d02udc
nag_ode_bvp_ps_lin_solve = d02uec
nag_ode_bvp_ps_lin_grid_vals = d02uwc
nag_ode_bvp_ps_lin_quad_weights = d02uyc
nag_ode_bvp_ps_lin_cheb_eval = d02uzc
__all__ = ['d02cjc', 'd02ejc',
           'd02gac', 'd02gbc',
           'd02mcc', 'd02mwc', 'd02nec',
           'd02npc',
           'd02pec', 'd02pfc', 'd02pgc', 'd02phc', 'd02pjc',
           'd02pqc', 'd02prc', 'd02psc', 'd02ptc', 'd02puc',
           'd02qfc', 'd02qwc',
           'd02qyc', 'd02qzc', 'd02rac', 'd02tlc',
           'd02tvc', 'd02txc', 'd02tyc', 'd02tzc', 'd02uac', 'd02ubc',
           'd02ucc', 'd02udc', 'd02uec', 'd02uwc', 'd02uyc', 'd02uzc',
           'nag_dae_ivp_dassl_cont', 'nag_dae_ivp_dassl_gen',
           'nag_dae_ivp_dassl_linalg', 'nag_dae_ivp_dassl_setup',
           'nag_ode_bvp_coll_nlin_contin',
           'nag_ode_bvp_coll_nlin_diag',
           'nag_ode_bvp_coll_nlin_interp',
           'nag_ode_bvp_coll_nlin_setup',
           'nag_ode_bvp_coll_nlin_solve',
           'nag_ode_bvp_fd_lin_gen', 'nag_ode_bvp_fd_nonlin_fixedbc',
           'nag_ode_bvp_fd_nonlin_gen',
           'nag_ode_bvp_ps_lin_cgl_deriv',
           'nag_ode_bvp_ps_lin_cgl_grid',
           'nag_ode_bvp_ps_lin_cgl_vals',
           'nag_ode_bvp_ps_lin_cheb_eval',
           'nag_ode_bvp_ps_lin_coeffs',
           'nag_ode_bvp_ps_lin_grid_vals',
           'nag_ode_bvp_ps_lin_quad_weights',
           'nag_ode_bvp_ps_lin_solve',
           'nag_ode_ivp_adams_free',
           'nag_ode_ivp_adams_gen', 'nag_ode_ivp_adams_interp',
           'nag_ode_ivp_adams_roots', 'nag_ode_ivp_adams_setup',
           'nag_ode_ivp_bdf_gen',
           'nag_ode_ivp_rkts_diag',
           'nag_ode_ivp_rkts_errass', 'nag_ode_ivp_rkts_interp',
           'nag_ode_ivp_rkts_onestep', 'nag_ode_ivp_rkts_range',
           'nag_ode_ivp_rkts_reset_tend', 'nag_ode_ivp_rkts_setup',
           'nag_ode_ivp_rk_step_revcomm',
           'nag_ode_ivp_rk_interp_setup',
           'nag_ode_ivp_rk_interp_eval']
