"""
Deprecated wrappers for the NAG C Library (Mark 26.0) d04 Chapter.

d04 - Numerical Differentiation.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/D04/d04conts.html

NAG Copyright 2017.
"""

from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Integer
from .util import NAG_D04AAC_FUN
from .util import Nag_Comm
from .util import NagError

_library_cl = get_library()

d04aac_ctypes = _library_cl.d04aac
d04aac_ctypes.restype = None
d04aac_ctypes.argtypes = [c_double, Integer, c_double, POINTER(c_double), POINTER(c_double), NAG_D04AAC_FUN, POINTER(Nag_Comm), POINTER(NagError)]


def d04aac(xval, nder, hbase, der, erest, fun, comm, fail):
    """
    Numerical differentiation, derivatives up to order 14, function of one
    real variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d04/d04aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        der, erest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        der, erest
    """
    from .util import nag_double_type_check_and_cast
    _der = nag_double_type_check_and_cast('d04aac', der, 'der')
    _erest = nag_double_type_check_and_cast('d04aac', erest, 'erest')
    d04aac_ctypes(xval, nder, hbase, _der, _erest, fun, comm, fail)

d04bac_ctypes = _library_cl.d04bac
d04bac_ctypes.restype = None
d04bac_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d04bac(xval, fval, der, erest, fail):
    """
    Numerical differentiation, user-supplied function values, derivatives up
    to order 14, derivatives with respect to one real variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d04/d04bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xval, fval, der, erest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xval, fval, der, erest
    """
    from .util import nag_double_type_check_and_cast
    _xval = nag_double_type_check_and_cast('d04bac', xval, 'xval')
    _fval = nag_double_type_check_and_cast('d04bac', fval, 'fval')
    _der = nag_double_type_check_and_cast('d04bac', der, 'der')
    _erest = nag_double_type_check_and_cast('d04bac', erest, 'erest')
    d04bac_ctypes(_xval, _fval, _der, _erest, fail)

d04bbc_ctypes = _library_cl.d04bbc
d04bbc_ctypes.restype = None
d04bbc_ctypes.argtypes = [c_double, c_double, POINTER(c_double)]


def d04bbc(x0, hbase, xval):
    """
    Generates sample points for function evaluations by d04bac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d04/d04bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xval
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xval
    """
    from .util import nag_double_type_check_and_cast
    _xval = nag_double_type_check_and_cast('d04bbc', xval, 'xval')
    d04bbc_ctypes(x0, hbase, _xval)

nag_numdiff_1d_real = d04aac
nag_numdiff_1d_real_eval = d04bac
nag_numdiff_1d_real_absci = d04bbc
__all__ = ['d04aac', 'd04bac', 'd04bbc', 'nag_numdiff_1d_real',
           'nag_numdiff_1d_real_absci', 'nag_numdiff_1d_real_eval']
