"""
Deprecated wrappers for the NAG C Library (Mark 26.0) e04 Chapter.

e04 - Minimizing or Maximizing a Function.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/E04/e04conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from ctypes import c_ulong
from ctypes import c_char_p
from ctypes import c_void_p
from ctypes import byref
from .util import _reset_units
from .util import get_library
from .util import Integer
from .util import Nag_BoundType
from .util import Nag_E04_Opt
from .util import Nag_Comm
from .util import NagError
from .util import NAG_E04KBC_FUN
from .util import NAG_E04ABC_FUN
from .util import NAG_E04BBC_FUN
from .util import NAG_E04CBC_FUNCT
from .util import NAG_E04CBC_MONIT
from .util import NAG_E04DGC_FUN
from .util import Nag_FileID
from .util import NAG_E04FCC_FUN
from .util import NAG_E04GBC_FUN
from .util import NAG_E04JCC_OBJFUN
from .util import NAG_E04JCC_MONFUN
from .util import NAG_E04LBC_FUN
from .util import NAG_E04LBC_HESS
from .util import NAG_E04NFC_FUN
from .util import NAG_E04NKC_HESSFUN
from .util import Nag_E04State
from .util import Nag_Start
from .util import NAG_E04NQC_QPHX
from .util import Nag_RegularizedType
from .util import NAG_E04UCC_FUN
from .util import NAG_E04UCC_CONFUN
from .util import Nag_Boolean
from .util import NAG_E04UGC_CONFUN
from .util import NAG_E04UGC_OBJFUN
from .util import NAG_E04UNC_OBJFUN
from .util import NAG_E04UNC_CONFUN
from .util import NAG_E04VHC_USRFUN
from .util import NAG_E04VJC_USRFUN
from .util import NAG_E04WDC_CONFUN
from .util import NAG_E04WDC_OBJFUN
from .util import NAG_E04XAC_OBJFUN
from .util import Nag_DerivInfo
from .util import NAG_E04YAC_FUN
from .util import NAG_E04HCC_FUN
from .util import Nag_VariableType
from .util import NAG_E04STC_OBJFUN
from .util import NAG_E04STC_OBJGRD
from .util import NAG_E04STC_CONFUN
from .util import NAG_E04STC_CONGRD
from .util import NAG_E04STC_HESS
from .util import NAG_E04STC_MON

_library_cl = get_library()

STRING = c_char_p

size_t = c_ulong

e04kbc_ctypes = _library_cl.e04kbc
e04kbc_ctypes.restype = None
e04kbc_ctypes.argtypes = [Integer, NAG_E04KBC_FUN, Nag_BoundType, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04kbc(n, sfun, bound, bl, bu, x, f, g, options, usercomm, fail):
    """
    Bound constrained nonlinear minimization (first derivatives required).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04kbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bl, bu, x, f, g
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        bl, bu, x, f, g
    """
    from .util import nag_double_type_check_and_cast
    _bl = nag_double_type_check_and_cast('e04kbc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04kbc', bu, 'bu')
    _x = nag_double_type_check_and_cast('e04kbc', x, 'x')
    _f = nag_double_type_check_and_cast('e04kbc', f, 'f')
    _g = nag_double_type_check_and_cast('e04kbc', g, 'g')
    e04kbc_ctypes(n, sfun, bound, _bl, _bu, _x, _f, _g, options, usercomm, fail)

e04abc_ctypes = _library_cl.e04abc
e04abc_ctypes.restype = None
e04abc_ctypes.argtypes = [NAG_E04ABC_FUN, c_double, c_double, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def e04abc(funct, e1, e2, a, b, maxfun, x, f, comm, fail):
    """
    Minimizes a function of one variable, using function values only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, f
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04abc', a, 'a')
    _b = nag_double_type_check_and_cast('e04abc', b, 'b')
    _x = nag_double_type_check_and_cast('e04abc', x, 'x')
    _f = nag_double_type_check_and_cast('e04abc', f, 'f')
    e04abc_ctypes(funct, e1, e2, _a, _b, maxfun, _x, _f, comm, fail)

e04bbc_ctypes = _library_cl.e04bbc
e04bbc_ctypes.restype = None
e04bbc_ctypes.argtypes = [NAG_E04BBC_FUN, c_double, c_double, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def e04bbc(funct, e1, e2, a, b, maxfun, x, f, g, comm, fail):
    """
    Minimizes a function of one variable, requires first derivatives.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, x, f, g
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, f, g
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04bbc', a, 'a')
    _b = nag_double_type_check_and_cast('e04bbc', b, 'b')
    _x = nag_double_type_check_and_cast('e04bbc', x, 'x')
    _f = nag_double_type_check_and_cast('e04bbc', f, 'f')
    _g = nag_double_type_check_and_cast('e04bbc', g, 'g')
    e04bbc_ctypes(funct, e1, e2, _a, _b, maxfun, _x, _f, _g, comm, fail)

e04cbc_ctypes = _library_cl.e04cbc
e04cbc_ctypes.restype = None
e04cbc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), c_double, c_double, NAG_E04CBC_FUNCT, NAG_E04CBC_MONIT, Integer, POINTER(Nag_Comm), POINTER(NagError)]


def e04cbc(n, x, f, tolf, tolx, funct, monit, maxcal, comm, fail):
    """
    Unconstrained minimization using simplex algorithm, function of several
    variables using function values only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04cbc', x, 'x')
    _f = nag_double_type_check_and_cast('e04cbc', f, 'f')
    if monit is None:
        monit = cast(None, NAG_E04CBC_MONIT)
    e04cbc_ctypes(n, _x, _f, tolf, tolx, funct, monit, maxcal, comm, fail)

e04dgc_ctypes = _library_cl.e04dgc
e04dgc_ctypes.restype = None
e04dgc_ctypes.argtypes = [Integer, NAG_E04DGC_FUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04dgc(n, objfun, x, objf, g, options, comm, fail):
    """
    Unconstrained minimization using conjugate gradients.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04dgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, objf, g
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, objf, g
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04dgc', x, 'x')
    _objf = nag_double_type_check_and_cast('e04dgc', objf, 'objf')
    _g = nag_double_type_check_and_cast('e04dgc', g, 'g')
    e04dgc_ctypes(n, objfun, _x, _objf, _g, options, comm, fail)

e04fcc_ctypes = _library_cl.e04fcc
e04fcc_ctypes.restype = None
e04fcc_ctypes.argtypes = [Integer, Integer, NAG_E04FCC_FUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04fcc(m, n, lsqfun, x, fsumsq, fvec, fjac, tdfjac, options, comm, fail):
    """
    Unconstrained nonlinear least squares (no derivatives required).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04fcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fsumsq, fvec, fjac
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fsumsq, fvec, fjac
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04fcc', x, 'x')
    _fsumsq = nag_double_type_check_and_cast('e04fcc', fsumsq, 'fsumsq')
    _fvec = nag_double_type_check_and_cast('e04fcc', fvec, 'fvec')
    _fjac = nag_double_type_check_and_cast('e04fcc', fjac, 'fjac')
    e04fcc_ctypes(m, n, lsqfun, _x, _fsumsq, _fvec, _fjac, tdfjac, options, comm, fail)

e04gbc_ctypes = _library_cl.e04gbc
e04gbc_ctypes.restype = None
e04gbc_ctypes.argtypes = [Integer, Integer, NAG_E04GBC_FUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04gbc(m, n, lsqfun, x, fsumsq, fvec, fjac, tdfjac, options, comm, fail):
    """
    Unconstrained nonlinear least squares (first derivatives required).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04gbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fsumsq, fvec, fjac
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fsumsq, fvec, fjac
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04gbc', x, 'x')
    _fsumsq = nag_double_type_check_and_cast('e04gbc', fsumsq, 'fsumsq')
    _fvec = nag_double_type_check_and_cast('e04gbc', fvec, 'fvec')
    _fjac = nag_double_type_check_and_cast('e04gbc', fjac, 'fjac')
    e04gbc_ctypes(m, n, lsqfun, _x, _fsumsq, _fvec, _fjac, tdfjac, options, comm, fail)

e04jcc_ctypes = _library_cl.e04jcc
e04jcc_ctypes.restype = None
e04jcc_ctypes.argtypes = [NAG_E04JCC_OBJFUN, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, c_double, NAG_E04JCC_MONFUN, Integer, POINTER(c_double), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def e04jcc(objfun, n, npt, x, bl, bu, rhobeg, rhoend, monfun, maxcal, f, nf, comm, fail):
    """
    Minimum by quadratic approximation, function of several variables,
    simple bounds, using function values only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04jcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, bl, bu, f, nf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, bl, bu, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nf
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04jcc', x, 'x')
    _bl = nag_double_type_check_and_cast('e04jcc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04jcc', bu, 'bu')
    if monfun is None:
        monfun = cast(None, NAG_E04JCC_MONFUN)
    _f = nag_double_type_check_and_cast('e04jcc', f, 'f')
    _nf = nag_integer_type_check_and_cast('e04jcc', nf, 'nf')
    e04jcc_ctypes(objfun, n, npt, _x, _bl, _bu, rhobeg, rhoend, monfun, maxcal, _f, _nf, comm, fail)

e04lbc_ctypes = _library_cl.e04lbc
e04lbc_ctypes.restype = None
e04lbc_ctypes.argtypes = [Integer, NAG_E04LBC_FUN, NAG_E04LBC_HESS, Nag_BoundType, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04lbc(n, objfun, hessfun, bound, bl, bu, x, objf, g, options, comm, fail):
    """
    Solves bound constrained problems (first and second derivatives
    required).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04lbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bl, bu, x, objf, g
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        bl, bu, x, objf, g
    """
    from .util import nag_double_type_check_and_cast
    _bl = nag_double_type_check_and_cast('e04lbc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04lbc', bu, 'bu')
    _x = nag_double_type_check_and_cast('e04lbc', x, 'x')
    _objf = nag_double_type_check_and_cast('e04lbc', objf, 'objf')
    _g = nag_double_type_check_and_cast('e04lbc', g, 'g')
    e04lbc_ctypes(n, objfun, hessfun, bound, _bl, _bu, _x, _objf, _g, options, comm, fail)

e04mfc_ctypes = _library_cl.e04mfc
e04mfc_ctypes.restype = None
e04mfc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04mfc(n, nclin, a, tda, bl, bu, cvec, x, objf, options, comm, fail):
    """
    Linear programming.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04mfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, cvec, x, objf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, cvec, x, objf
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04mfc', a, 'a')
    _bl = nag_double_type_check_and_cast('e04mfc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04mfc', bu, 'bu')
    _cvec = nag_double_type_check_and_cast('e04mfc', cvec, 'cvec')
    _x = nag_double_type_check_and_cast('e04mfc', x, 'x')
    _objf = nag_double_type_check_and_cast('e04mfc', objf, 'objf')
    e04mfc_ctypes(n, nclin, _a, tda, _bl, _bu, _cvec, _x, _objf, options, comm, fail)

e04mwc_ctypes = _library_cl.e04mwc
e04mwc_ctypes.restype = None
e04mwc_ctypes.argtypes = [Nag_FileID, Integer, Integer, Integer, Integer,
                          Integer, Integer, Integer, POINTER(Integer),
                          POINTER(c_double), Integer, POINTER(c_double),
                          POINTER(Integer), POINTER(Integer),
                          POINTER(c_double), POINTER(c_double),
                          STRING,
                          STRING,
                          POINTER(c_double), POINTER(Integer),
                          POINTER(Integer), Integer, POINTER(Integer),
                          POINTER(NagError)]


def e04mwc(fileid, n, m, nnzc, nnza, ncolh, nnzh, lintvar, idxc, c, iobj, a,
           irowa, iccola, bl, bu, pnames, crname, h, irowh, iccolh, minmax,
           intvar, fail):
    """
    Write MPS data file defining LP, QP, MILP or MIQP problem.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04mwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        idxc, c, a, irowa, iccola, bl, bu, h, irowh, iccolh, intvar
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c, a, bl, bu, h
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        idxc, irowa, iccola, irowh, iccolh, intvar
    TypeError will be raised when the following arguments are not arrays of
    strings:
        pnames, crname
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_char2array_type_check_and_cast
    _idxc = nag_integer_type_check_and_cast('e04mwc', idxc, 'idxc')
    _c = nag_double_type_check_and_cast('e04mwc', c, 'c')
    _a = nag_double_type_check_and_cast('e04mwc', a, 'a')
    _irowa = nag_integer_type_check_and_cast('e04mwc', irowa, 'irowa')
    _iccola = nag_integer_type_check_and_cast('e04mwc', iccola, 'iccola')
    _bl = nag_double_type_check_and_cast('e04mwc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04mwc', bu, 'bu')
    _pnames = nag_char2array_type_check_and_cast('e04mwc', pnames, 'pnames', 9, 5)
    _crname = nag_char2array_type_check_and_cast('e04mwc', crname, 'crname', 9, n+m)
    _h = nag_double_type_check_and_cast('e04mwc', h, 'h')
    _irowh = nag_integer_type_check_and_cast('e04mwc', irowh, 'irowh')
    _iccolh = nag_integer_type_check_and_cast('e04mwc', iccolh, 'iccolh')
    _intvar = nag_integer_type_check_and_cast('e04mwc', intvar, 'intvar')
    e04mwc_ctypes(fileid, n, m, nnzc, nnza, ncolh, nnzh, lintvar, _idxc,
                  _c, iobj, _a, _irowa, _iccola, _bl, _bu, _pnames,
                  _crname, _h, _irowh, _iccolh, minmax, _intvar, fail)


e04myc_ctypes = _library_cl.e04myc
e04myc_ctypes.restype = None
e04myc_ctypes.argtypes = [POINTER(POINTER(c_double)), POINTER(POINTER(Integer)), POINTER(POINTER(Integer)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double))]


def e04myc(a, ha, ka, bl, bu, xs):
    """
    Free memory allocated by e04mzc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04myc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ha, ka, bl, bu, xs
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, xs
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ha, ka
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04myc', a, 'a')
    _ha = nag_integer_type_check_and_cast('e04myc', ha, 'ha')
    _ka = nag_integer_type_check_and_cast('e04myc', ka, 'ka')
    _bl = nag_double_type_check_and_cast('e04myc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04myc', bu, 'bu')
    _xs = nag_double_type_check_and_cast('e04myc', xs, 'xs')
    e04myc_ctypes(_a, _ha, _ka, _bl, _bu, _xs)

e04mxc_ctypes = _library_cl.e04mxc
e04mxc_ctypes.restype = None
e04mxc_ctypes.argtypes = [Nag_FileID, Integer, Integer, Integer, Integer,
                          Integer, Integer, Integer, POINTER(Integer),
                          POINTER(Integer),
                          POINTER(Integer), POINTER(Integer),
                          POINTER(Integer), POINTER(Integer),
                          POINTER(Integer), POINTER(c_double),
                          POINTER(Integer), POINTER(Integer),
                          POINTER(c_double), POINTER(c_double),
                          STRING, POINTER(Integer),
                          STRING,
                          POINTER(c_double), POINTER(Integer),
                          POINTER(Integer), POINTER(Integer),
                          POINTER(Integer), POINTER(NagError)]


def e04mxc(infile, maxn, maxm, maxnnz, maxncolh, maxnnzh, maxlintvar, mpslst, n, m, nnz, ncolh, nnzh, lintvar, iobj, a, irowa, iccola, bl, bu, pnames, nname, crname, h, irowh, iccolh, minmax, intvar, fail):
    """
    Reads MPS data file defining LP, QP, MILP or MIQP problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04mxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        n, m, nnz, ncolh, nnzh, lintvar, iobj, a, irowa, iccola, bl, bu, nname, h, irowh, iccolh, minmax, intvar
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, h
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        n, m, nnz, ncolh, nnzh, lintvar, iobj, irowa, iccola, nname, irowh, iccolh, minmax, intvar
    TypeError will be raised when the following arguments are not arrays of
    strings:
        pnames, crname
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_char2array_type_check_and_cast
    _n = nag_integer_type_check_and_cast('e04mxc', n, 'n')
    _m = nag_integer_type_check_and_cast('e04mxc', m, 'm')
    _nnz = nag_integer_type_check_and_cast('e04mxc', nnz, 'nnz')
    _ncolh = nag_integer_type_check_and_cast('e04mxc', ncolh, 'ncolh')
    _nnzh = nag_integer_type_check_and_cast('e04mxc', nnzh, 'nnzh')
    _lintvar = nag_integer_type_check_and_cast('e04mxc', lintvar, 'lintvar')
    _iobj = nag_integer_type_check_and_cast('e04mxc', iobj, 'iobj')
    _a = nag_double_type_check_and_cast('e04mxc', a, 'a')
    _irowa = nag_integer_type_check_and_cast('e04mxc', irowa, 'irowa')
    _iccola = nag_integer_type_check_and_cast('e04mxc', iccola, 'iccola')
    _bl = nag_double_type_check_and_cast('e04mxc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04mxc', bu, 'bu')
    _pnames = nag_char2array_type_check_and_cast('e04mxc', pnames, 'pnames', 9, 5)
    _nname = nag_integer_type_check_and_cast('e04mxc', nname, 'nname')
    _crname = nag_char2array_type_check_and_cast('e04mxc', crname, 'crname', 9, n+m)
    _h = nag_double_type_check_and_cast('e04mxc', h, 'h')
    _irowh = nag_integer_type_check_and_cast('e04mxc', irowh, 'irowh')
    _iccolh = nag_integer_type_check_and_cast('e04mxc', iccolh, 'iccolh')
    _minmax = nag_integer_type_check_and_cast('e04mxc', minmax, 'minmax')
    _intvar = nag_integer_type_check_and_cast('e04mxc', intvar, 'intvar')
    e04mxc_ctypes(infile, maxn, maxm, maxnnz, maxncolh, maxnnzh, maxlintvar, mpslst, _n, _m, _nnz, _ncolh, _nnzh, _lintvar, _iobj, _a, _irowa, _iccola, _bl, _bu, _pnames, _nname, _crname, _h, _irowh, _iccolh, _minmax, _intvar, fail)

e04mzc_ctypes = _library_cl.e04mzc
e04mzc_ctypes.restype = None
e04mzc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(POINTER(c_double)), POINTER(POINTER(Integer)), POINTER(POINTER(Integer)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(Nag_E04_Opt), POINTER(NagError)]


def e04mzc(mpsfile, n, m, nnz, iobj, a, ha, ka, bl, bu, xs, options, fail):
    """
    Read MPSX data for sparse LP or QP problem from a file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04mzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        n, m, nnz, iobj, a, ha, ka, bl, bu, xs
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, xs
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        n, m, nnz, iobj, ha, ka
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _n = nag_integer_type_check_and_cast('e04mzc', n, 'n')
    _m = nag_integer_type_check_and_cast('e04mzc', m, 'm')
    _nnz = nag_integer_type_check_and_cast('e04mzc', nnz, 'nnz')
    _iobj = nag_integer_type_check_and_cast('e04mzc', iobj, 'iobj')
    _a = nag_double_type_check_and_cast('e04mzc', a, 'a')
    _ha = nag_integer_type_check_and_cast('e04mzc', ha, 'ha')
    _ka = nag_integer_type_check_and_cast('e04mzc', ka, 'ka')
    _bl = nag_double_type_check_and_cast('e04mzc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04mzc', bu, 'bu')
    _xs = nag_double_type_check_and_cast('e04mzc', xs, 'xs')
    e04mzc_ctypes(mpsfile, _n, _m, _nnz, _iobj, _a, _ha, _ka, _bl, _bu, _xs, options, fail)

e04ncc_ctypes = _library_cl.e04ncc
e04ncc_ctypes.restype = None
e04ncc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04ncc(m, n, nclin, a, tda, bl, bu, cvec, b, h, tdh, kx, x, objf, options, comm, fail):
    """
    Solves linear least squares and convex quadratic programming problems
    (non-sparse).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04ncc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, cvec, b, h, kx, x, objf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, cvec, b, h, x, objf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        kx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04ncc', a, 'a')
    _bl = nag_double_type_check_and_cast('e04ncc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04ncc', bu, 'bu')
    _cvec = nag_double_type_check_and_cast('e04ncc', cvec, 'cvec')
    _b = nag_double_type_check_and_cast('e04ncc', b, 'b')
    _h = nag_double_type_check_and_cast('e04ncc', h, 'h')
    _kx = nag_integer_type_check_and_cast('e04ncc', kx, 'kx')
    _x = nag_double_type_check_and_cast('e04ncc', x, 'x')
    _objf = nag_double_type_check_and_cast('e04ncc', objf, 'objf')
    e04ncc_ctypes(m, n, nclin, _a, tda, _bl, _bu, _cvec, _b, _h, tdh, _kx, _x, _objf, options, comm, fail)

e04nfc_ctypes = _library_cl.e04nfc
e04nfc_ctypes.restype = None
e04nfc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, NAG_E04NFC_FUN, POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04nfc(n, nclin, a, tda, bl, bu, cvec, h, tdh, qphess, x, objf, options, comm, fail):
    """
    Quadratic programming.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, cvec, h, x, objf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, cvec, h, x, objf
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04nfc', a, 'a')
    _bl = nag_double_type_check_and_cast('e04nfc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04nfc', bu, 'bu')
    _cvec = nag_double_type_check_and_cast('e04nfc', cvec, 'cvec')
    _h = nag_double_type_check_and_cast('e04nfc', h, 'h')
    if qphess is None:
        qphess = cast(None, NAG_E04NFC_FUN)
    _x = nag_double_type_check_and_cast('e04nfc', x, 'x')
    _objf = nag_double_type_check_and_cast('e04nfc', objf, 'objf')
    e04nfc_ctypes(n, nclin, _a, tda, _bl, _bu, _cvec, _h, tdh, qphess, _x, _objf, options, comm, fail)

e04nkc_ctypes = _library_cl.e04nkc
e04nkc_ctypes.restype = None
e04nkc_ctypes.argtypes = [Integer, Integer, Integer, Integer, Integer, NAG_E04NKC_HESSFUN, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04nkc(n, m, nnz, iobj, ncolh, qphx, a, ha, ka, bl, bu, xs, ninf, sinf, obj, options, comm, fail):
    """
    Solves sparse linear programming or convex quadratic programming
    problems.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ha, ka, bl, bu, xs, ninf, sinf, obj
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, xs, sinf, obj
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ha, ka, ninf
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    if qphx is None:
        qphx = cast(None, NAG_E04NKC_HESSFUN)
    _a = nag_double_type_check_and_cast('e04nkc', a, 'a')
    _ha = nag_integer_type_check_and_cast('e04nkc', ha, 'ha')
    _ka = nag_integer_type_check_and_cast('e04nkc', ka, 'ka')
    _bl = nag_double_type_check_and_cast('e04nkc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04nkc', bu, 'bu')
    _xs = nag_double_type_check_and_cast('e04nkc', xs, 'xs')
    _ninf = nag_integer_type_check_and_cast('e04nkc', ninf, 'ninf')
    _sinf = nag_double_type_check_and_cast('e04nkc', sinf, 'sinf')
    _obj = nag_double_type_check_and_cast('e04nkc', obj, 'obj')
    e04nkc_ctypes(n, m, nnz, iobj, ncolh, qphx, _a, _ha, _ka, _bl, _bu, _xs, _ninf, _sinf, _obj, options, comm, fail)

e04npc_ctypes = _library_cl.e04npc
e04npc_ctypes.restype = None
e04npc_ctypes.argtypes = [POINTER(Nag_E04State), POINTER(NagError)]


def e04npc(state, fail):
    """
    Initialization function for e04nqc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04npc.html
    """
    e04npc_ctypes(state, fail)

e04nqc_ctypes = _library_cl.e04nqc
e04nqc_ctypes.restype = None
e04nqc_ctypes.argtypes = [Nag_Start, NAG_E04NQC_QPHX, Integer, Integer, Integer, Integer, Integer, Integer, Integer, c_double, STRING, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(STRING), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04State), POINTER(Nag_Comm), POINTER(NagError)]


def e04nqc(start, qphx, m, n, ne, nname, lenc, ncolh, iobj, objadd, prob, acol, inda, loca, bl, bu, c, names, helast, hs, x, pi, rc, ns, ninf, sinf, obj, state, comm, fail):
    """
    LP or QP problem (suitable for sparse problems).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        acol, inda, loca, bl, bu, c, helast, hs, x, pi, rc, ns, ninf, sinf, obj
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        acol, bl, bu, c, x, pi, rc, sinf, obj
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        inda, loca, helast, hs, ns, ninf
    TypeError will be raised when the following argumets are not arrays of
    strings:
        names
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    if qphx is None:
        qphx = cast(None, NAG_E04NQC_QPHX)
    _acol = nag_double_type_check_and_cast('e04nqc', acol, 'acol')
    _inda = nag_integer_type_check_and_cast('e04nqc', inda, 'inda')
    _loca = nag_integer_type_check_and_cast('e04nqc', loca, 'loca')
    _bl = nag_double_type_check_and_cast('e04nqc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04nqc', bu, 'bu')
    _c = nag_double_type_check_and_cast('e04nqc', c, 'c')
    _names = nag_char_type_check_and_cast('e04nqc', names, 'names')
    _helast = nag_integer_type_check_and_cast('e04nqc', helast, 'helast')
    _hs = nag_integer_type_check_and_cast('e04nqc', hs, 'hs')
    _x = nag_double_type_check_and_cast('e04nqc', x, 'x')
    _pi = nag_double_type_check_and_cast('e04nqc', pi, 'pi')
    _rc = nag_double_type_check_and_cast('e04nqc', rc, 'rc')
    _ns = nag_integer_type_check_and_cast('e04nqc', ns, 'ns')
    _ninf = nag_integer_type_check_and_cast('e04nqc', ninf, 'ninf')
    _sinf = nag_double_type_check_and_cast('e04nqc', sinf, 'sinf')
    _obj = nag_double_type_check_and_cast('e04nqc', obj, 'obj')
    e04nqc_ctypes(start, qphx, m, n, ne, nname, lenc, ncolh, iobj, objadd, prob, _acol, _inda, _loca, _bl, _bu, _c, _names, _helast, _hs, _x, _pi, _rc, _ns, _ninf, _sinf, _obj, state, comm, fail)

e04nrc_ctypes = _library_cl.e04nrc
e04nrc_ctypes.restype = None
e04nrc_ctypes.argtypes = [Nag_FileID, POINTER(Nag_E04State), POINTER(NagError)]


def e04nrc(fileid, state, fail):
    """
    Supply optional argument values for e04nqc from external file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nrc.html
    """
    e04nrc_ctypes(fileid, state, fail)

e04nsc_ctypes = _library_cl.e04nsc
e04nsc_ctypes.restype = None
e04nsc_ctypes.argtypes = [STRING, POINTER(Nag_E04State), POINTER(NagError)]


def e04nsc(string, state, fail):
    """
    Set a single option for e04nqc from a character string.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nsc.html
    """
    e04nsc_ctypes(string, state, fail)

e04ntc_ctypes = _library_cl.e04ntc
e04ntc_ctypes.restype = None
e04ntc_ctypes.argtypes = [STRING, Integer, POINTER(Nag_E04State), POINTER(NagError)]


def e04ntc(string, ivalue, state, fail):
    """
    Set a single option for e04nqc from an integer argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04ntc.html
    """
    e04ntc_ctypes(string, ivalue, state, fail)

e04nuc_ctypes = _library_cl.e04nuc
e04nuc_ctypes.restype = None
e04nuc_ctypes.argtypes = [STRING, c_double, POINTER(Nag_E04State), POINTER(NagError)]


def e04nuc(string, rvalue, state, fail):
    """
    Set a single option for e04nqc from a real argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nuc.html
    """
    e04nuc_ctypes(string, rvalue, state, fail)

e04nxc_ctypes = _library_cl.e04nxc
e04nxc_ctypes.restype = None
e04nxc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(Nag_E04State), POINTER(NagError)]


def e04nxc(string, ivalue, state, fail):
    """
    Get the setting of an integer valued option of e04nqc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue
    """
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('e04nxc', ivalue, 'ivalue')
    e04nxc_ctypes(string, _ivalue, state, fail)

e04nyc_ctypes = _library_cl.e04nyc
e04nyc_ctypes.restype = None
e04nyc_ctypes.argtypes = [STRING, POINTER(c_double), POINTER(Nag_E04State), POINTER(NagError)]


def e04nyc(string, rvalue, state, fail):
    """
    Get the setting of a real valued option of e04nqc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04nyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue
    """
    from .util import nag_double_type_check_and_cast
    _rvalue = nag_double_type_check_and_cast('e04nyc', rvalue, 'rvalue')
    e04nyc_ctypes(string, _rvalue, state, fail)

e04pcc_ctypes = _library_cl.e04pcc
e04pcc_ctypes.restype = None
e04pcc_ctypes.argtypes = [Nag_RegularizedType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def e04pcc(itype, m, n, a, tda, b, bl, bu, tol, x, rnorm, nfree, w, indx, fail):
    """
    Computes the least squares solution to a set of linear equations subject
    to fixed upper and lower bounds on the variables. An option is provided
    to return a minimal length solution if a solution is not unique.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04pcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, bl, bu, x, rnorm, nfree, w, indx
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, bl, bu, x, rnorm, w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nfree, indx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04pcc', a, 'a')
    _b = nag_double_type_check_and_cast('e04pcc', b, 'b')
    _bl = nag_double_type_check_and_cast('e04pcc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04pcc', bu, 'bu')
    _x = nag_double_type_check_and_cast('e04pcc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('e04pcc', rnorm, 'rnorm')
    _nfree = nag_integer_type_check_and_cast('e04pcc', nfree, 'nfree')
    _w = nag_double_type_check_and_cast('e04pcc', w, 'w')
    _indx = nag_integer_type_check_and_cast('e04pcc', indx, 'indx')
    e04pcc_ctypes(itype, m, n, _a, tda, _b, _bl, _bu, tol, _x, _rnorm, _nfree, _w, _indx, fail)

e04ucc_ctypes = _library_cl.e04ucc
e04ucc_ctypes.restype = None
e04ucc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), NAG_E04UCC_FUN, NAG_E04UCC_CONFUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04ucc(n, nclin, ncnlin, a, tda, bl, bu, objfun, confun, x, objf, g, options, comm, fail):
    """
    Minimization with nonlinear constraints using a sequential QP method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04ucc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, x, objf, g
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, x, objf, g
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04ucc', a, 'a')
    _bl = nag_double_type_check_and_cast('e04ucc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04ucc', bu, 'bu')
    if confun is None:
        confun = cast(None, NAG_E04UCC_CONFUN)
    _x = nag_double_type_check_and_cast('e04ucc', x, 'x')
    _objf = nag_double_type_check_and_cast('e04ucc', objf, 'objf')
    _g = nag_double_type_check_and_cast('e04ucc', g, 'g')
    e04ucc_ctypes(n, nclin, ncnlin, _a, tda, _bl, _bu, objfun, confun, _x, _objf, _g, options, comm, fail)

e04udc_ctypes = _library_cl.e04udc
e04udc_ctypes.restype = None
e04udc_ctypes.argtypes = [Nag_FileID, POINTER(Nag_Boolean), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e04udc(fileid, lwsav, iwsav, rwsav, fail):
    """
    Supply optional argument values for e04ucc or e04ufc from external file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04udc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lwsav, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        lwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lwsav = nag_enum_type_check_and_cast('e04udc', lwsav, 'lwsav', Nag_Boolean)
    _iwsav = nag_integer_type_check_and_cast('e04udc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('e04udc', rwsav, 'rwsav')
    e04udc_ctypes(fileid, _lwsav, _iwsav, _rwsav, fail)

e04uec_ctypes = _library_cl.e04uec
e04uec_ctypes.restype = None
e04uec_ctypes.argtypes = [STRING, POINTER(Nag_Boolean), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e04uec(_str, lwsav, iwsav, rwsav, fail):
    """
    Supply optional argument values to e04ucc or e04ufc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04uec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lwsav, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        lwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lwsav = nag_enum_type_check_and_cast('e04uec', lwsav, 'lwsav', Nag_Boolean)
    _iwsav = nag_integer_type_check_and_cast('e04uec', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('e04uec', rwsav, 'rwsav')
    e04uec_ctypes(_str, _lwsav, _iwsav, _rwsav, fail)

e04ufc_ctypes = _library_cl.e04ufc
e04ufc_ctypes.restype = None
e04ufc_ctypes.argtypes = [POINTER(Integer), Integer, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), Integer, POINTER(c_double), Integer, STRING, POINTER(Nag_Boolean), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


@_reset_units
def e04ufc(irevcm, n, nclin, ncnln, tda, tdcj, tdr, a, bl, bu, _iter, istate, c, cjac, clamda, objf, objgrd, r, x, needc, icomm, licomm, rcomm, lrcomm, cwsav, lwsav, iwsav, rwsav, fail):
    """
    Minimum, function of several variables, sequential QP method, nonlinear
    constraints, using function values and optionally first derivatives
    (reverse communication, comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04ufc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, a, bl, bu, _iter, istate, c, cjac, clamda, objf, objgrd, r, x, needc, icomm, rcomm, lwsav, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, c, cjac, clamda, objf, objgrd, r, x, rcomm, rwsav
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        lwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, _iter, istate, needc, icomm, iwsav
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('e04ufc', irevcm, 'irevcm')
    _a = nag_double_type_check_and_cast('e04ufc', a, 'a')
    _bl = nag_double_type_check_and_cast('e04ufc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04ufc', bu, 'bu')
    __iter = nag_integer_type_check_and_cast('e04ufc', _iter, '_iter')
    _istate = nag_integer_type_check_and_cast('e04ufc', istate, 'istate')
    _c = nag_double_type_check_and_cast('e04ufc', c, 'c')
    _cjac = nag_double_type_check_and_cast('e04ufc', cjac, 'cjac')
    _clamda = nag_double_type_check_and_cast('e04ufc', clamda, 'clamda')
    _objf = nag_double_type_check_and_cast('e04ufc', objf, 'objf')
    _objgrd = nag_double_type_check_and_cast('e04ufc', objgrd, 'objgrd')
    _r = nag_double_type_check_and_cast('e04ufc', r, 'r')
    _x = nag_double_type_check_and_cast('e04ufc', x, 'x')
    _needc = nag_integer_type_check_and_cast('e04ufc', needc, 'needc')
    _icomm = nag_integer_type_check_and_cast('e04ufc', icomm, 'icomm')
    _rcomm = nag_double_type_check_and_cast('e04ufc', rcomm, 'rcomm')
    _lwsav = nag_enum_type_check_and_cast('e04uec', lwsav, 'lwsav', Nag_Boolean)
    _iwsav = nag_integer_type_check_and_cast('e04ufc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('e04ufc', rwsav, 'rwsav')
    e04ufc_ctypes(_irevcm, n, nclin, ncnln, tda, tdcj, tdr, _a, _bl, _bu, __iter, _istate, _c, _cjac, _clamda, _objf, _objgrd, _r, _x, _needc, _icomm, licomm, _rcomm, lrcomm, cwsav, _lwsav, _iwsav, _rwsav, fail)

e04ugc_ctypes = _library_cl.e04ugc
e04ugc_ctypes.restype = None
e04ugc_ctypes.argtypes = [NAG_E04UGC_CONFUN, NAG_E04UGC_OBJFUN, Integer, Integer, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(Nag_E04_Opt), POINTER(NagError)]


def e04ugc(confun, objfun, n, m, ncnln, nonln, njnln, iobj, nnz, a, ha, ka, bl, bu, xs, ninf, sinf, objf, comm, options, fail):
    """
    NLP problem (sparse).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04ugc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ha, ka, bl, bu, xs, ninf, sinf, objf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, xs, sinf, objf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ha, ka, ninf
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    if confun is None:
        confun = cast(None, NAG_E04UGC_CONFUN)
    if objfun is None:
        objfun = cast(None, NAG_E04UGC_OBJFUN)
    _a = nag_double_type_check_and_cast('e04ugc', a, 'a')
    _ha = nag_integer_type_check_and_cast('e04ugc', ha, 'ha')
    _ka = nag_integer_type_check_and_cast('e04ugc', ka, 'ka')
    _bl = nag_double_type_check_and_cast('e04ugc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04ugc', bu, 'bu')
    _xs = nag_double_type_check_and_cast('e04ugc', xs, 'xs')
    _ninf = nag_integer_type_check_and_cast('e04ugc', ninf, 'ninf')
    _sinf = nag_double_type_check_and_cast('e04ugc', sinf, 'sinf')
    _objf = nag_double_type_check_and_cast('e04ugc', objf, 'objf')
    e04ugc_ctypes(confun, objfun, n, m, ncnln, nonln, njnln, iobj, nnz, _a, _ha, _ka, _bl, _bu, _xs, _ninf, _sinf, _objf, comm, options, fail)

e04unc_ctypes = _library_cl.e04unc
e04unc_ctypes.restype = None
e04unc_ctypes.argtypes = [Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), NAG_E04UNC_OBJFUN, NAG_E04UNC_CONFUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04unc(m, n, nclin, ncnlin, a, tda, bl, bu, y, objfun, confun, x, objf, f, fjac, tdfjac, options, comm, fail):
    """
    Solves nonlinear least squares problems using the sequential QP method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04unc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, y, x, objf, f, fjac
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, y, x, objf, f, fjac
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04unc', a, 'a')
    _bl = nag_double_type_check_and_cast('e04unc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04unc', bu, 'bu')
    _y = nag_double_type_check_and_cast('e04unc', y, 'y')
    if confun is None:
        confun = cast(None, NAG_E04UNC_CONFUN)
    _x = nag_double_type_check_and_cast('e04unc', x, 'x')
    _objf = nag_double_type_check_and_cast('e04unc', objf, 'objf')
    _f = nag_double_type_check_and_cast('e04unc', f, 'f')
    _fjac = nag_double_type_check_and_cast('e04unc', fjac, 'fjac')
    e04unc_ctypes(m, n, nclin, ncnlin, _a, tda, _bl, _bu, _y, objfun, confun, _x, _objf, _f, _fjac, tdfjac, options, comm, fail)

e04vgc_ctypes = _library_cl.e04vgc
e04vgc_ctypes.restype = None
e04vgc_ctypes.argtypes = [POINTER(Nag_E04State), POINTER(NagError)]


def e04vgc(state, fail):
    """
    Initialization function for e04vhc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vgc.html
    """
    e04vgc_ctypes(state, fail)

e04vhc_ctypes = _library_cl.e04vhc
e04vhc_ctypes.restype = None
e04vhc_ctypes.argtypes = [Nag_Start, Integer, Integer, Integer, Integer, c_double, Integer, STRING, NAG_E04VHC_USRFUN, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(Integer), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(STRING), POINTER(c_double), POINTER(c_double), POINTER(STRING), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Nag_E04State), POINTER(Nag_Comm), POINTER(NagError)]


def e04vhc(start, nf, n, nxname, nfname, objadd, objrow, prob, usrfun, iafun, javar, a, lena, nea, igfun, jgvar, leng, neg, xlow, xupp, xnames, flow, fupp, fnames, x, xstate, xmul, f, fstate, fmul, ns, ninf, sinf, state, comm, fail):
    """
    General sparse nonlinear optimizer.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iafun, javar, a, igfun, jgvar, xlow, xupp, flow, fupp, x, xstate, xmul, f, fstate, fmul, ns, ninf, sinf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, xlow, xupp, flow, fupp, x, xmul, f, fmul, sinf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iafun, javar, igfun, jgvar, xstate, fstate, ns, ninf
    TypeError will be raised when the following argumets are not arrays of
    strings:
        xnames, fnames
    """
    from .util import nag_char_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iafun = nag_integer_type_check_and_cast('e04vhc', iafun, 'iafun')
    _javar = nag_integer_type_check_and_cast('e04vhc', javar, 'javar')
    _a = nag_double_type_check_and_cast('e04vhc', a, 'a')
    _igfun = nag_integer_type_check_and_cast('e04vhc', igfun, 'igfun')
    _jgvar = nag_integer_type_check_and_cast('e04vhc', jgvar, 'jgvar')
    _xlow = nag_double_type_check_and_cast('e04vhc', xlow, 'xlow')
    _xupp = nag_double_type_check_and_cast('e04vhc', xupp, 'xupp')
    _xnames = nag_char_type_check_and_cast('e04vhc', xnames, 'xnames')
    _flow = nag_double_type_check_and_cast('e04vhc', flow, 'flow')
    _fupp = nag_double_type_check_and_cast('e04vhc', fupp, 'fupp')
    _fnames = nag_char_type_check_and_cast('e04vhc', fnames, 'fnames')
    _x = nag_double_type_check_and_cast('e04vhc', x, 'x')
    _xstate = nag_integer_type_check_and_cast('e04vhc', xstate, 'xstate')
    _xmul = nag_double_type_check_and_cast('e04vhc', xmul, 'xmul')
    _f = nag_double_type_check_and_cast('e04vhc', f, 'f')
    _fstate = nag_integer_type_check_and_cast('e04vhc', fstate, 'fstate')
    _fmul = nag_double_type_check_and_cast('e04vhc', fmul, 'fmul')
    _ns = nag_integer_type_check_and_cast('e04vhc', ns, 'ns')
    _ninf = nag_integer_type_check_and_cast('e04vhc', ninf, 'ninf')
    _sinf = nag_double_type_check_and_cast('e04vhc', sinf, 'sinf')
    e04vhc_ctypes(start, nf, n, nxname, nfname, objadd, objrow, prob, usrfun, _iafun, _javar, _a, lena, nea, _igfun, _jgvar, leng, neg, _xlow, _xupp, _xnames, _flow, _fupp, _fnames, _x, _xstate, _xmul, _f, _fstate, _fmul, _ns, _ninf, _sinf, state, comm, fail)

e04vjc_ctypes = _library_cl.e04vjc
e04vjc_ctypes.restype = None
e04vjc_ctypes.argtypes = [Integer, Integer, NAG_E04VJC_USRFUN, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04State), POINTER(Nag_Comm), POINTER(NagError)]


def e04vjc(nf, n, usrfun, iafun, javar, a, lena, nea, igfun, jgvar, leng, neg, x, xlow, xupp, state, comm, fail):
    """
    Determine the pattern of nonzeros in the Jacobian matrix for e04vhc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iafun, javar, a, nea, igfun, jgvar, neg, x, xlow, xupp
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, x, xlow, xupp
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iafun, javar, nea, igfun, jgvar, neg
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iafun = nag_integer_type_check_and_cast('e04vjc', iafun, 'iafun')
    _javar = nag_integer_type_check_and_cast('e04vjc', javar, 'javar')
    _a = nag_double_type_check_and_cast('e04vjc', a, 'a')
    _nea = nag_integer_type_check_and_cast('e04vjc', nea, 'nea')
    _igfun = nag_integer_type_check_and_cast('e04vjc', igfun, 'igfun')
    _jgvar = nag_integer_type_check_and_cast('e04vjc', jgvar, 'jgvar')
    _neg = nag_integer_type_check_and_cast('e04vjc', neg, 'neg')
    _x = nag_double_type_check_and_cast('e04vjc', x, 'x')
    _xlow = nag_double_type_check_and_cast('e04vjc', xlow, 'xlow')
    _xupp = nag_double_type_check_and_cast('e04vjc', xupp, 'xupp')
    e04vjc_ctypes(nf, n, usrfun, _iafun, _javar, _a, lena, _nea, _igfun, _jgvar, leng, _neg, _x, _xlow, _xupp, state, comm, fail)

e04vkc_ctypes = _library_cl.e04vkc
e04vkc_ctypes.restype = None
e04vkc_ctypes.argtypes = [Nag_FileID, POINTER(Nag_E04State), POINTER(NagError)]


def e04vkc(fileid, state, fail):
    """
    Supply optional argument values for e04vhc from external file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vkc.html
    """
    e04vkc_ctypes(fileid, state, fail)

e04vlc_ctypes = _library_cl.e04vlc
e04vlc_ctypes.restype = None
e04vlc_ctypes.argtypes = [STRING, POINTER(Nag_E04State), POINTER(NagError)]


def e04vlc(string, state, fail):
    """
    Set a single option for e04vhc from a character string.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vlc.html
    """
    e04vlc_ctypes(string, state, fail)

e04vmc_ctypes = _library_cl.e04vmc
e04vmc_ctypes.restype = None
e04vmc_ctypes.argtypes = [STRING, Integer, POINTER(Nag_E04State), POINTER(NagError)]


def e04vmc(string, ivalue, state, fail):
    """
    Set a single option for e04vhc from an integer argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vmc.html
    """
    e04vmc_ctypes(string, ivalue, state, fail)

e04vnc_ctypes = _library_cl.e04vnc
e04vnc_ctypes.restype = None
e04vnc_ctypes.argtypes = [STRING, c_double, POINTER(Nag_E04State), POINTER(NagError)]


def e04vnc(string, rvalue, state, fail):
    """
    Set a single option for e04vhc from a real argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vnc.html
    """
    e04vnc_ctypes(string, rvalue, state, fail)

e04vrc_ctypes = _library_cl.e04vrc
e04vrc_ctypes.restype = None
e04vrc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(Nag_E04State), POINTER(NagError)]


def e04vrc(string, ivalue, state, fail):
    """
    Get the setting of an integer valued option of e04vhc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vrc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue
    """
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('e04vrc', ivalue, 'ivalue')
    e04vrc_ctypes(string, _ivalue, state, fail)

e04vsc_ctypes = _library_cl.e04vsc
e04vsc_ctypes.restype = None
e04vsc_ctypes.argtypes = [STRING, POINTER(c_double), POINTER(Nag_E04State), POINTER(NagError)]


def e04vsc(string, rvalue, state, fail):
    """
    Get the setting of a real valued option of e04vhc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04vsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue
    """
    from .util import nag_double_type_check_and_cast
    _rvalue = nag_double_type_check_and_cast('e04vsc', rvalue, 'rvalue')
    e04vsc_ctypes(string, _rvalue, state, fail)

e04wbc_ctypes = _library_cl.e04wbc
e04wbc_ctypes.restype = None
e04wbc_ctypes.argtypes = [STRING, STRING, Integer, POINTER(Nag_Boolean), Integer, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def e04wbc(rname, cwsav, lcwsav, lwsav, llwsav, iwsav, liwsav, rwsav, lrwsav, fail):
    """
    Initialization function for e04ufc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lwsav, iwsav, rwsav
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        lwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwsav
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lwsav = nag_enum_type_check_and_cast('e04wbc', lwsav, 'lwsav', Nag_Boolean)
    _iwsav = nag_integer_type_check_and_cast('e04wbc', iwsav, 'iwsav')
    _rwsav = nag_double_type_check_and_cast('e04wbc', rwsav, 'rwsav')
    e04wbc_ctypes(rname, cwsav, lcwsav, _lwsav, llwsav, _iwsav, liwsav, _rwsav, lrwsav, fail)

e04wcc_ctypes = _library_cl.e04wcc
e04wcc_ctypes.restype = None
e04wcc_ctypes.argtypes = [POINTER(Nag_E04State), POINTER(NagError)]


def e04wcc(state, fail):
    """
    Initialization function for e04wdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wcc.html
    """
    e04wcc_ctypes(state, fail)

e04wdc_ctypes = _library_cl.e04wdc
e04wdc_ctypes.restype = None
e04wdc_ctypes.argtypes = [Integer, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), NAG_E04WDC_CONFUN, NAG_E04WDC_OBJFUN, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_E04State), POINTER(Nag_Comm), POINTER(NagError)]


def e04wdc(n, nclin, ncnln, tda, tdcj, tdh, a, bl, bu, confun, objfun, majits, istate, ccon, cjac, clamda, objf, grad, h, x, state, comm, fail):
    """
    Solves the nonlinear programming (NLP) problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, majits, istate, ccon, cjac, clamda, objf, grad, h, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, ccon, cjac, clamda, objf, grad, h, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        majits, istate
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e04wdc', a, 'a')
    _bl = nag_double_type_check_and_cast('e04wdc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04wdc', bu, 'bu')
    if confun is None:
        confun = cast(None, NAG_E04WDC_CONFUN)
    _majits = nag_integer_type_check_and_cast('e04wdc', majits, 'majits')
    _istate = nag_integer_type_check_and_cast('e04wdc', istate, 'istate')
    _ccon = nag_double_type_check_and_cast('e04wdc', ccon, 'ccon')
    _cjac = nag_double_type_check_and_cast('e04wdc', cjac, 'cjac')
    _clamda = nag_double_type_check_and_cast('e04wdc', clamda, 'clamda')
    _objf = nag_double_type_check_and_cast('e04wdc', objf, 'objf')
    _grad = nag_double_type_check_and_cast('e04wdc', grad, 'grad')
    _h = nag_double_type_check_and_cast('e04wdc', h, 'h')
    _x = nag_double_type_check_and_cast('e04wdc', x, 'x')
    e04wdc_ctypes(n, nclin, ncnln, tda, tdcj, tdh, _a, _bl, _bu, confun, objfun, _majits, _istate, _ccon, _cjac, _clamda, _objf, _grad, _h, _x, state, comm, fail)

e04wec_ctypes = _library_cl.e04wec
e04wec_ctypes.restype = None
e04wec_ctypes.argtypes = [Nag_FileID, POINTER(Nag_E04State), POINTER(NagError)]


def e04wec(fileid, state, fail):
    """
    Supply optional argument values for e04wdc from external file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wec.html
    """
    e04wec_ctypes(fileid, state, fail)

e04wfc_ctypes = _library_cl.e04wfc
e04wfc_ctypes.restype = None
e04wfc_ctypes.argtypes = [STRING, POINTER(Nag_E04State), POINTER(NagError)]


def e04wfc(string, state, fail):
    """
    Set a single option for e04wdc from a character string.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wfc.html
    """
    e04wfc_ctypes(string, state, fail)

e04wgc_ctypes = _library_cl.e04wgc
e04wgc_ctypes.restype = None
e04wgc_ctypes.argtypes = [STRING, Integer, POINTER(Nag_E04State), POINTER(NagError)]


def e04wgc(string, ivalue, state, fail):
    """
    Set a single option for e04wdc from an integer argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wgc.html
    """
    e04wgc_ctypes(string, ivalue, state, fail)

e04whc_ctypes = _library_cl.e04whc
e04whc_ctypes.restype = None
e04whc_ctypes.argtypes = [STRING, c_double, POINTER(Nag_E04State), POINTER(NagError)]


def e04whc(string, rvalue, state, fail):
    """
    Set a single option for e04wdc from a real argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04whc.html
    """
    e04whc_ctypes(string, rvalue, state, fail)

e04wkc_ctypes = _library_cl.e04wkc
e04wkc_ctypes.restype = None
e04wkc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(Nag_E04State), POINTER(NagError)]


def e04wkc(string, ivalue, state, fail):
    """
    Get the setting of an integer valued option of e04wdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue
    """
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('e04wkc', ivalue, 'ivalue')
    e04wkc_ctypes(string, _ivalue, state, fail)

e04wlc_ctypes = _library_cl.e04wlc
e04wlc_ctypes.restype = None
e04wlc_ctypes.argtypes = [STRING, POINTER(c_double), POINTER(Nag_E04State), POINTER(NagError)]


def e04wlc(string, rvalue, state, fail):
    """
    Get the setting of a real valued option of e04wdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04wlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue
    """
    from .util import nag_double_type_check_and_cast
    _rvalue = nag_double_type_check_and_cast('e04wlc', rvalue, 'rvalue')
    e04wlc_ctypes(string, _rvalue, state, fail)

e04xac_ctypes = _library_cl.e04xac
e04xac_ctypes.restype = None
e04xac_ctypes.argtypes = [Integer, POINTER(c_double), NAG_E04XAC_OBJFUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_DerivInfo), POINTER(Nag_E04_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def e04xac(n, x, objfun, objf, g, hforward, hcentral, hess, tdhess, derivinfo, options, comm, fail):
    """
    Computes an approximation to the gradient vector and/or the Hessian
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04xac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, objf, g, hforward, hcentral, hess, derivinfo
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, objf, g, hforward, hcentral, hess
    TypeError will be raised when the following arguments are not of data type
    Nag_DerivInfo:
        derivinfo
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04xac', x, 'x')
    _objf = nag_double_type_check_and_cast('e04xac', objf, 'objf')
    _g = nag_double_type_check_and_cast('e04xac', g, 'g')
    _hforward = nag_double_type_check_and_cast('e04xac', hforward, 'hforward')
    _hcentral = nag_double_type_check_and_cast('e04xac', hcentral, 'hcentral')
    _hess = nag_double_type_check_and_cast('e04xac', hess, 'hess')
    _derivinfo = nag_enum_type_check_and_cast('e04xac', derivinfo, 'derivinfo', Nag_DerivInfo)
    e04xac_ctypes(n, _x, objfun, _objf, _g, _hforward, _hcentral, _hess, tdhess, _derivinfo, options, comm, fail)

e04xxc_ctypes = _library_cl.e04xxc
e04xxc_ctypes.restype = None
e04xxc_ctypes.argtypes = [POINTER(Nag_E04_Opt)]


def e04xxc(options):
    """
    Initialization function for option setting.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04xxc.html
    """
    e04xxc_ctypes(options)

e04xyc_ctypes = _library_cl.e04xyc
e04xyc_ctypes.restype = None
e04xyc_ctypes.argtypes = [STRING, STRING, POINTER(Nag_E04_Opt), Nag_Boolean, STRING, POINTER(NagError)]


def e04xyc(name, optfile, options, _print, outfile, fail):
    """
    Read options from a text file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04xyc.html
    """
    e04xyc_ctypes(name, optfile, options, _print, outfile, fail)

e04xzc_ctypes = _library_cl.e04xzc
e04xzc_ctypes.restype = None
e04xzc_ctypes.argtypes = [POINTER(Nag_E04_Opt), STRING, POINTER(NagError)]


def e04xzc(options, pname, fail):
    """
    Memory freeing function for use with option setting.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04xzc.html
    """
    e04xzc_ctypes(options, pname, fail)

e04yac_ctypes = _library_cl.e04yac
e04yac_ctypes.restype = None
e04yac_ctypes.argtypes = [Integer, Integer, NAG_E04YAC_FUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_Comm), POINTER(NagError)]


def e04yac(m, n, lsqfun, x, fvec, fjac, tdfjac, comm, fail):
    """
    Least squares derivative checker for use with e04gbc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04yac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fvec, fjac
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fvec, fjac
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04yac', x, 'x')
    _fvec = nag_double_type_check_and_cast('e04yac', fvec, 'fvec')
    _fjac = nag_double_type_check_and_cast('e04yac', fjac, 'fjac')
    e04yac_ctypes(m, n, lsqfun, _x, _fvec, _fjac, tdfjac, comm, fail)

e04ycc_ctypes = _library_cl.e04ycc
e04ycc_ctypes.restype = None
e04ycc_ctypes.argtypes = [Integer, Integer, Integer, c_double, POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(NagError)]


def e04ycc(job, m, n, fsumsq, cj, options, fail):
    """
    Covariance matrix for nonlinear least squares.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04ycc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cj
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        cj
    """
    from .util import nag_double_type_check_and_cast
    _cj = nag_double_type_check_and_cast('e04ycc', cj, 'cj')
    e04ycc_ctypes(job, m, n, fsumsq, _cj, options, fail)

e04hcc_ctypes = _library_cl.e04hcc
e04hcc_ctypes.restype = None
e04hcc_ctypes.argtypes = [Integer, NAG_E04HCC_FUN, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def e04hcc(n, sfun, x, f, g, usercomm, fail):
    """
    Derivative checker.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04hcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, g
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, g
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04hcc', x, 'x')
    _f = nag_double_type_check_and_cast('e04hcc', f, 'f')
    _g = nag_double_type_check_and_cast('e04hcc', g, 'g')
    e04hcc_ctypes(n, sfun, _x, _f, _g, usercomm, fail)

e04hdc_ctypes = _library_cl.e04hdc
e04hdc_ctypes.restype = None
e04hdc_ctypes.argtypes = [Integer, NAG_E04LBC_FUN, NAG_E04LBC_HESS, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def e04hdc(n, sfun, shess, x, g, hesl, hesd, usercomm, fail):
    """
    Checks second derivatives of a user-defined function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e04/e04hdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, g, hesl, hesd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, g, hesl, hesd
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e04hdc', x, 'x')
    _g = nag_double_type_check_and_cast('e04hdc', g, 'g')
    _hesl = nag_double_type_check_and_cast('e04hdc', hesl, 'hesl')
    _hesd = nag_double_type_check_and_cast('e04hdc', hesd, 'hesd')
    e04hdc_ctypes(n, sfun, shess, _x, _g, _hesl, _hesd, usercomm, fail)

e04rac_ctypes = _library_cl.e04rac
e04rac_ctypes.restype = None
e04rac_ctypes.argtypes = [POINTER(c_void_p),
                          Integer,
                          POINTER(NagError)]


@_reset_units
def e04rac(handle, nvar, fail):
    """
    Initialization of a handle for the NAG optimization modelling suite for
    problems, such as, quadratic programming (QP), nonlinear programming (NLP),
    linear semidefinite programming (SDP) or SDP with bilinear matrix
    inequalities (BMI-SDP)
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rac.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    """
    from .util import nag_handle_check
    nag_handle_check('e04rac', 'handle', handle)
    e04rac_ctypes(byref(handle), nvar, fail)


e04rdc_ctypes = _library_cl.e04rdc
e04rdc_ctypes.restype = None
e04rdc_ctypes.argtypes = [Nag_FileID,
                          Integer,
                          Integer,
                          Integer,
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(c_double),
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(c_double),
                          POINTER(Integer),
                          POINTER(NagError)]


def e04rdc(infile, maxnvar, maxnblk, maxnnz, filelst, nvar, nblk, nnz, cvec,
           nnza, irowa, icola, a, blksizea, fail):
    """
    A reader of sparse SDPA data files for linear SDP problems.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nvar, nblk, nnz, cvec, nnza, irowa, icola, a, blksizea
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        cvec, a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nvar, nblk, nnz, nnza, irowa, icola, blksizea
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nvar = nag_integer_type_check_and_cast('e04rdc', nvar, 'nvar')
    _nblk = nag_integer_type_check_and_cast('e04rdc', nblk, 'nblk')
    _nnz = nag_integer_type_check_and_cast('e04rdc', nnz, 'nnz')
    _cvec = nag_double_type_check_and_cast('e04rdc', cvec, 'cvec')
    _nnza = nag_integer_type_check_and_cast('e04rdc', nnza, 'nnza')
    _irowa = nag_integer_type_check_and_cast('e04rdc', irowa, 'irowa')
    _icola = nag_integer_type_check_and_cast('e04rdc', icola, 'icola')
    _a = nag_double_type_check_and_cast('e04rdc', a, 'a')
    _blksizea = nag_integer_type_check_and_cast('e04rdc', blksizea,
                                                'blksizea')

    e04rdc_ctypes(infile, maxnvar, maxnblk, maxnnz, filelst, _nvar, _nblk,
                  _nnz, _cvec, _nnza, _irowa, _icola, _a, _blksizea, fail)

e04rec_ctypes = _library_cl.e04rec
e04rec_ctypes.restype = None
e04rec_ctypes.argtypes = [c_void_p,
                          Integer,
                          POINTER(c_double),
                          POINTER(NagError)]


def e04rec(handle, nvar, cvec, fail):
    """
    Define a linear objective function to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rec.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cvex
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        cvec
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast
    nag_handle_check('e04rec', 'handle', handle)
    _cvec = nag_double_type_check_and_cast('e04rec', cvec, 'cvec')
    e04rec_ctypes(handle, nvar, _cvec, fail)

e04rfc_ctypes = _library_cl.e04rfc
e04rfc_ctypes.restype = None
e04rfc_ctypes.argtypes = [c_void_p,
                          Integer,
                          POINTER(Integer),
                          POINTER(c_double),
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(c_double),
                          POINTER(NagError)]


def e04rfc(handle, nnzc, idxc, c, nnzh, irowh, icolh, h, fail):
    """
    Define a linear or a quadratic objective function to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rfc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        idxc, c, h, irowh, icolh
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c, h
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        idxc, irowh, iccolh
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast

    nag_handle_check('e04rfc', 'handle', handle)
    _idxc = nag_integer_type_check_and_cast('e04rfc', idxc, 'idxc')
    _c = nag_double_type_check_and_cast('e04rfc', c, 'c')
    _irowh = nag_integer_type_check_and_cast('e04rfc', irowh, 'irowh')
    _icolh = nag_integer_type_check_and_cast('e04rfc', icolh, 'icolh')
    _h = nag_double_type_check_and_cast('e04rfc', h, 'h')
    e04rfc_ctypes(handle, nnzc, _idxc, _c, nnzh, _irowh, _icolh, _h, fail)

e04rgc_ctypes = _library_cl.e04rgc
e04rgc_ctypes.restype = None
e04rgc_ctypes.argtypes = [c_void_p, Integer, POINTER(Integer),
                          POINTER(NagError)]


def e04rgc(handle, nnzfd, idxfd, fail):
    """
    Define a nonlinear objective function to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rgc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        idxfd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        idxfd
    """
    from .util import nag_handle_check
    from .util import nag_integer_type_check_and_cast

    nag_handle_check('e04rgc', 'handle', handle)
    _idxfd = nag_integer_type_check_and_cast('e04rgc', idxfd, 'idxfd')
    e04rgc_ctypes(handle, nnzfd, _idxfd, fail)

e04rhc_ctypes = _library_cl.e04rhc
e04rhc_ctypes.restype = None
e04rhc_ctypes.argtypes = [c_void_p, Integer, POINTER(c_double),
                          POINTER(c_double), POINTER(NagError)]


def e04rhc(handle, nvar, bl, bu, fail):
    """
    Define bounds of variables of a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rhc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bl, bu
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        bl, bu
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast

    nag_handle_check('e04rhc', 'handle', handle)
    _bl = nag_double_type_check_and_cast('e04rhc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04rhc', bu, 'bu')
    e04rhc_ctypes(handle, nvar, _bl, _bu, fail)

e04rjc_ctypes = _library_cl.e04rjc
e04rjc_ctypes.restype = None
e04rjc_ctypes.argtypes = [c_void_p, Integer, POINTER(c_double),
                          POINTER(c_double), Integer, POINTER(Integer),
                          POINTER(Integer), POINTER(c_double),
                          POINTER(Integer), POINTER(NagError)]


def e04rjc(handle, nclin, bl, bu, nnzb, irowb, icolb, b, idlc, fail):
    """
    Define a block of linear constraints to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rjc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bl, bu, irowb, icolb, b, idlc
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        bl, bu, b
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irowb, icolb, idlc
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast

    nag_handle_check('e04rjc', 'handle', handle)

    _bl = nag_double_type_check_and_cast('e04rjc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04rjc', bu, 'bu')
    _irowb = nag_integer_type_check_and_cast('e04rjc', irowb, 'irowb')
    _icolb = nag_integer_type_check_and_cast('e04rjc', icolb, 'icolb')
    _b = nag_double_type_check_and_cast('e04rjc', b, 'b')
    _idlc = nag_integer_type_check_and_cast('e04rjc', idlc, 'idlc')
    e04rjc_ctypes(handle, nclin, _bl, _bu, nnzb, _irowb, _icolb, _b,
                  _idlc, fail)

e04rkc_ctypes = _library_cl.e04rkc
e04rkc_ctypes.restype = None
e04rkc_ctypes.argtypes = [c_void_p, Integer, POINTER(c_double),
                          POINTER(c_double), Integer, POINTER(Integer),
                          POINTER(Integer), POINTER(NagError)]


def e04rkc(handle, ncnln, bl, bu, nnzgd, irowgd, icolgd, fail):
    """
    Define a block of nonlinear constraints to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rkc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised if the following arguments are not a numpy array:
        bl, bu, irowgd, icolgd
    TypeError will be raised if the following arguments are not of datatype
    float:
        bl, bu
    TypeError will be raised if the following arguments are not of datatype
    int32 or int64:
        irowgd, icolgd
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast

    nag_handle_check('e04rkc', 'handle', handle)
    _bl = nag_double_type_check_and_cast('e04rkc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e04rkc', bu, 'bu')
    _irowgd = nag_integer_type_check_and_cast('e04rkc', irowgd, 'irowgd')
    _icolgd = nag_integer_type_check_and_cast('e04rkc', icolgd, 'icolgd')
    e04rkc_ctypes(handle, ncnln, _bl, _bu, nnzgd, _irowgd, _icolgd, fail)

e04rlc_ctypes = _library_cl.e04rlc
e04rlc_ctypes.restype = None
e04rlc_ctypes.argtypes = [c_void_p,
                          Integer,
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(NagError)]


def e04rlc(handle, idf, nnzh, irowh, icolh, fail):
    """
    Define a structure of Hessian of the objective,
    constraints or the Lagrangian to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rlc.html

    TypeError will be raised if the following are not a numpy array:
        irowh, icolh
    TypeError will be raised if the following are not of datatype float:
        irowh, icolh
    """
    from .util import nag_handle_check
    from .util import nag_integer_type_check_and_cast

    nag_handle_check('e04rlc', 'handle', handle)
    _irowh = nag_integer_type_check_and_cast('e04rlc', irowh, 'irowh')
    _icolh = nag_integer_type_check_and_cast('e04rlc', icolh, 'icolh')
    e04rlc_ctypes(handle, idf, nnzh, _irowh, _icolh, fail)

e04rnc_ctypes = _library_cl.e04rnc
e04rnc_ctypes.restype = None
e04rnc_ctypes.argtypes = [c_void_p,
                          Integer,
                          Integer,
                          POINTER(Integer),
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(c_double),
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(NagError)]


def e04rnc(handle, nvar, dima, nnza, nnzasum, irowa, icola, a, nblk, blksizea,
           idblk, fail):
    """
    Add one or more linear matrix inequality constraints to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rnc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nnza, irowa, icola, a, blksizea, idblk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nnza, irow, icola, blksizea, idblk
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast

    nag_handle_check('e04rnc', 'handle', handle)
    _nnza = nag_integer_type_check_and_cast('e04rnc', nnza, 'nnza')
    _irowa = nag_integer_type_check_and_cast('e04rnc', irowa, 'irowa')
    _icola = nag_integer_type_check_and_cast('e04rnc', icola, 'icola')
    _a = nag_double_type_check_and_cast('e04rnc', a, 'a')
    _blksizea = nag_integer_type_check_and_cast('e04rnc', blksizea, 'blksizea')
    _idblk = nag_integer_type_check_and_cast('e04rnc', idblk, 'idblk')
    e04rnc_ctypes(handle, nvar, dima, _nnza, nnzasum, _irowa, _icola, _a,
                  nblk, _blksizea, _idblk, fail)

e04rpc_ctypes = _library_cl.e04rpc
e04rpc_ctypes.restype = None
e04rpc_ctypes.argtypes = [c_void_p,
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          Integer,
                          POINTER(Integer),
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(c_double),
                          POINTER(Integer),
                          POINTER(NagError)]


def e04rpc(handle, nq, qi, qj, dimq, nnzq, nnzqsum, irowq, icolq,
           q, idblk, fail):
    """
    Define bilinear matrix terms to a problem
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rpc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        qi, qj, nnzq, irowq, icolq, q, idblk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        q
    TypeError will be raised when the following arguments are not of data type
    numpy.int32, numpy.int64
        qi, qj, nnzq, irowq, icolq, idblk
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast

    nag_handle_check('e04rpc', 'handle', handle)
    _qi = nag_integer_type_check_and_cast('e04rpc', qi, 'qi')
    _qj = nag_integer_type_check_and_cast('e04rpc', qj, 'qj')
    _nnzq = nag_integer_type_check_and_cast('e04rpc', nnzq, 'nnzq')
    _irowq = nag_integer_type_check_and_cast('e04rpc', irowq, 'irowq')
    _icolq = nag_integer_type_check_and_cast('e04rpc', icolq, 'icolq')
    _q = nag_double_type_check_and_cast('e04rpc', q, 'q')
    _idblk = nag_integer_type_check_and_cast('e04rpc', idblk, 'idblk')
    e04rpc_ctypes(handle, nq, _qi, _qj, dimq, _nnzq, nnzqsum,
                  _irowq, _icolq, _q, _idblk, fail)

e04ryc_ctypes = _library_cl.e04ryc
e04ryc_ctypes.restype = None
e04ryc_ctypes.argtypes = [c_void_p, Nag_FileID, STRING,
                          POINTER(NagError)]

@_reset_units
def e04ryc(handle, fileid, cmdstr, fail):
    """
    Print information about a problem handle
    initialized by e04rac.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04ryc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    """
    from .util import nag_handle_check
    nag_handle_check('e04ryc', 'handle', handle)
    e04ryc_ctypes(handle, fileid, cmdstr, fail)

e04rzc_ctypes = _library_cl.e04rzc
e04rzc_ctypes.restype = None
e04rzc_ctypes.argtypes = [POINTER(c_void_p), POINTER(NagError)]


def e04rzc(handle, fail):
    """
    Destroy the problem handle initialized by e04rac and deallocate all the
    memory used.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04rzc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    """
    from .util import nag_handle_check
    nag_handle_check('e04rzc', 'handle', handle)
    e04rzc_ctypes(byref(handle), fail)

e04stc_ctypes = _library_cl.e04stc
e04stc_ctypes.restype = None
e04stc_ctypes.argtypes = [c_void_p,
                          NAG_E04STC_OBJFUN,
                          NAG_E04STC_OBJGRD,
                          NAG_E04STC_CONFUN,
                          NAG_E04STC_CONGRD,
                          NAG_E04STC_HESS,
                          NAG_E04STC_MON,
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(Nag_Comm),
                          POINTER(NagError)]


def e04stc(handle, objfun, objgrd, confun, congrd, hess, mon, nvar, x,
           nnzu, u, rinfo, stats, comm, fail):
    """
    Run an interior point solver on a sparse nonlinear programming problem (NLP)
    initialized by e04rac and defined by other functions from the suite.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04stc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    The following parameters can be passed as None:
        objfun, objgrd, confun, congrd, hess, mon
    TypeError will be raised if the following are not of type numpy.array:
        x, u, rinfo, stats
    TypeError will be raised if the following are not floats:
        x, u, rinfo, stats
    """
    from ctypes import cast
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast

    nag_handle_check('e04stc', 'handle', handle)
    if objfun is None:
        objfun = cast(None, NAG_E04STC_OBJFUN)
    if objgrd is None:
        objgrd = cast(None, NAG_E04STC_OBJGRD)
    if confun is None:
        confun = cast(None, NAG_E04STC_CONFUN)
    if congrd is None:
        congrd = cast(None, NAG_E04STC_CONGRD)
    if hess is None:
        hess = cast(None, NAG_E04STC_HESS)
    if mon is None:
        mon = cast(None, NAG_E04STC_MON)
    _x = nag_double_type_check_and_cast('e04stc', x, 'x')
    _u = nag_double_type_check_and_cast('e04stc', u, 'u')
    _rinfo = nag_double_type_check_and_cast('e04stc', rinfo, 'rinfo')
    _stats = nag_double_type_check_and_cast('e04stc', stats, 'stats')
    e04stc_ctypes(handle, objfun, objgrd, confun, congrd, hess, mon, nvar, _x,
                  nnzu, _u, _rinfo, _stats, comm, fail)

e04svc_ctypes = _library_cl.e04svc
e04svc_ctypes.restype = None
e04svc_ctypes.argtypes = [c_void_p,
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(Integer),
                          POINTER(NagError)]


@_reset_units
def e04svc(handle, nvar, x, nnzu, u, nnzuc, uc, nnzua, ua, rinfo,
           stats, inform, fail):
    """
    Run the Pennon solver on a compatible problem initialized by e04rac and
    defined by other functions from the suite, such as,
    semidefinite programming (SDP) and SDP with bilinear matrix inequalities
    (BMI).
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04svc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, u, uc, ua, rinfo, stats, inform
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, u, uc, rinfo, stats
    TypeError will be raised when the following arguments are not of data type
    numpy.int32, numpy.int64
        inform
    """
    from .util import nag_handle_check
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    nag_handle_check('e04svc', 'handle', handle)
    _x = nag_double_type_check_and_cast('e04svc', x, 'x')
    _u = nag_double_type_check_and_cast('e04svc', u, 'u')
    _uc = nag_double_type_check_and_cast('e04svc', uc, 'uc')
    _ua = nag_double_type_check_and_cast('e04svc', ua, 'ua')
    _rinfo = nag_double_type_check_and_cast('e04svc', rinfo, 'rinfo')
    _stats = nag_double_type_check_and_cast('e04svc', stats, 'stats')
    _inform = nag_integer_type_check_and_cast('e04svc', inform, 'inform')
    e04svc_ctypes(handle, nvar, _x, nnzu, _u, nnzuc, _uc, nnzua, _ua, _rinfo,
                  _stats, _inform, fail)


e04zmc_ctypes = _library_cl.e04zmc
e04zmc_ctypes.restype = None
e04zmc_ctypes.argtypes = [c_void_p, STRING, POINTER(NagError)]


@_reset_units
def e04zmc(handle, optstr, fail):
    """
    Option setting routine for the solvers from the NAG optimization modelling
    suite.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04zmc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    """
    from .util import nag_handle_check
    nag_handle_check('e04zmc', 'handle', handle)
    e04zmc_ctypes(handle, optstr, fail)

e04znc_ctypes = _library_cl.e04znc
e04znc_ctypes.restype = None
e04znc_ctypes.argtypes = [c_void_p,
                          STRING,
                          POINTER(Integer),
                          POINTER(c_double),
                          STRING,
                          Integer,
                          POINTER(Nag_VariableType),
                          POINTER(NagError)]


def e04znc(handle, optstr, ivalue, rvalue, cvalue, lcvalue, optype, fail):
    """
    Option getting routine for the solvers from the
    NAG optimization modelling suite.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04znc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue, rvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_handle_check

    nag_handle_check('e04znc', 'handle', handle)
    _ivalue = nag_integer_type_check_and_cast('e04znc', ivalue, 'ivalue')
    _rvalue = nag_double_type_check_and_cast('e04znc', rvalue, 'rvalue')
    e04znc_ctypes(handle, optstr, _ivalue, _rvalue, cvalue,
                  lcvalue, optype, fail)

e04zpc_ctypes = _library_cl.e04zpc
e04zpc_ctypes.restype = None
e04zpc_ctypes.argtypes = [c_void_p, Nag_FileID, POINTER(NagError)]


def e04zpc(handle, fileid, fail):
    """
    Option setting routine for the solvers from the
    NAG optimization modelling suite from external file.
    https://www.nag.com/numeric/cl/nagdoc_cl26/html/e04/e04zpc.html

    TypeError will be raised when the following arguments are not instances
    of nag_opt_handle_object:
        handle
    """
    from .util import nag_handle_check

    nag_handle_check('e04zpc', 'handle', handle)
    e04zpc_ctypes(handle, fileid, fail)

nag_opt_one_var_no_deriv = e04abc
nag_opt_one_var_deriv = e04bbc
nag_opt_simplex_easy = e04cbc
nag_opt_conj_grad = e04dgc
nag_opt_lsq_no_deriv = e04fcc
nag_opt_lsq_deriv = e04gbc
nag_opt_check_deriv = e04hcc
nag_opt_check_2nd_deriv = e04hdc
nag_opt_bounds_qa_no_deriv = e04jcc
nag_opt_bounds_deriv = e04kbc
nag_opt_bounds_2nd_deriv = e04lbc
nag_opt_lp = e04mfc
nag_opt_miqp_mps_write = e04mwc
nag_opt_miqp_mps_read = e04mxc
nag_opt_sparse_mps_free = e04myc
nag_opt_sparse_mps_read = e04mzc
nag_opt_lin_lsq = e04ncc
nag_opt_qp = e04nfc
nag_opt_sparse_convex_qp = e04nkc
nag_opt_sparse_convex_qp_init = e04npc
nag_opt_sparse_convex_qp_solve = e04nqc
nag_opt_sparse_convex_qp_option_set_file = e04nrc
nag_opt_sparse_convex_qp_option_set_string = e04nsc
nag_opt_sparse_convex_qp_option_set_integer = e04ntc
nag_opt_sparse_convex_qp_option_set_double = e04nuc
nag_opt_sparse_convex_qp_option_get_integer = e04nxc
nag_opt_sparse_convex_qp_option_get_double = e04nyc
nag_opt_bnd_lin_lsq = e04pcc
nag_opt_handle_init = e04rac
nag_opt_sdp_read_sdpa = e04rdc
nag_opt_handle_set_linobj = e04rec
nag_opt_handle_set_quadobj = e04rfc
nag_opt_handle_set_nlnobj = e04rgc
nag_opt_handle_set_simplebounds = e04rhc
nag_opt_handle_set_linconstr = e04rjc
nag_opt_handle_set_nlnconstr = e04rkc
nag_opt_handle_set_nlnhess = e04rlc
nag_opt_handle_set_linmatineq = e04rnc
nag_opt_handle_set_quadmatineq = e04rpc
nag_opt_handle_print = e04ryc
nag_opt_handle_free = e04rzc
nag_opt_handle_solve_ipopt = e04stc
nag_opt_handle_solve_pennon = e04svc
nag_opt_nlp = e04ucc
nag_opt_nlp_revcomm_option_set_file = e04udc
nag_opt_nlp_revcomm_option_set_string = e04uec
nag_opt_nlp_revcomm = e04ufc
nag_opt_nlp_sparse = e04ugc
nag_opt_nlin_lsq = e04unc
nag_opt_sparse_nlp_init = e04vgc
nag_opt_sparse_nlp_solve = e04vhc
nag_opt_sparse_nlp_jacobian = e04vjc
nag_opt_sparse_nlp_option_set_file = e04vkc
nag_opt_sparse_nlp_option_set_string = e04vlc
nag_opt_sparse_nlp_option_set_integer = e04vmc
nag_opt_sparse_nlp_option_set_double = e04vnc
nag_opt_sparse_nlp_option_get_integer = e04vrc
nag_opt_sparse_nlp_option_get_double = e04vsc
nag_opt_nlp_revcomm_init = e04wbc
nag_opt_nlp_init = e04wcc
nag_opt_nlp_solve = e04wdc
nag_opt_nlp_option_set_file = e04wec
nag_opt_nlp_option_set_string = e04wfc
nag_opt_nlp_option_set_integer = e04wgc
nag_opt_nlp_option_set_double = e04whc
nag_opt_nlp_option_get_integer = e04wkc
nag_opt_nlp_option_get_double = e04wlc
nag_opt_estimate_deriv = e04xac
nag_opt_init = e04xxc
nag_opt_read = e04xyc
nag_opt_free = e04xzc
nag_opt_lsq_check_deriv = e04yac
nag_opt_lsq_covariance = e04ycc
nag_opt_handle_opt_set = e04zmc
nag_opt_handle_opt_get = e04znc
nag_opt_handle_opt_set_file = e04zpc
__all__ = ['e04abc', 'e04bbc', 'e04cbc', 'e04dgc',
           'e04fcc', 'e04gbc',
           'e04hcc', 'e04hdc', 'e04jcc',
           'e04kbc', 'e04lbc',
           'e04mfc', 'e04mwc', 'e04mxc', 'e04myc',
           'e04mzc', 'e04ncc', 'e04nfc',
           'e04nkc', 'e04npc', 'e04nqc',
           'e04nrc', 'e04nsc', 'e04ntc', 'e04nuc', 'e04nxc', 'e04nyc',
           'e04pcc', 'e04rac', 'e04rdc', 'e04rec', 'e04rfc', 'e04rgc',
           'e04rhc', 'e04rjc', 'e04rkc', 'e04rlc',
           'e04rnc', 'e04rpc',
           'e04ryc',
           'e04rzc', 'e04stc', 'e04svc',
           'e04ucc', 'e04udc', 'e04uec', 'e04ufc', 'e04ugc',
           'e04unc',
           'e04vgc', 'e04vhc', 'e04vjc', 'e04vkc', 'e04vlc', 'e04vmc',
           'e04vnc', 'e04vrc', 'e04vsc', 'e04wbc', 'e04wcc', 'e04wdc',
           'e04wec', 'e04wfc', 'e04wgc', 'e04whc', 'e04wkc', 'e04wlc',
           'e04xac', 'e04xxc', 'e04xyc', 'e04xzc', 'e04yac',
           'e04ycc', 'e04zmc', 'e04znc', 'e04zpc',
           'nag_opt_bnd_lin_lsq',
           'nag_opt_bounds_2nd_deriv', 'nag_opt_bounds_deriv',
           'nag_opt_bounds_qa_no_deriv',
           'nag_opt_check_2nd_deriv', 'nag_opt_check_deriv',
           'nag_opt_conj_grad', 'nag_opt_estimate_deriv',
           'nag_opt_free', 'nag_opt_init', 'nag_opt_lin_lsq',
           'nag_opt_lp',
           'nag_opt_lsq_check_deriv',
           'nag_opt_lsq_covariance',
           'nag_opt_lsq_deriv',
           'nag_opt_lsq_no_deriv',
           'nag_opt_miqp_mps_read', 'nag_opt_nlin_lsq', 'nag_opt_nlp',
           'nag_opt_nlp_init',
           'nag_opt_nlp_option_get_double',
           'nag_opt_nlp_option_get_integer',
           'nag_opt_nlp_option_set_double',
           'nag_opt_nlp_option_set_file',
           'nag_opt_nlp_option_set_integer',
           'nag_opt_nlp_option_set_string', 'nag_opt_nlp_revcomm',
           'nag_opt_nlp_revcomm_init',
           'nag_opt_nlp_revcomm_option_set_file',
           'nag_opt_nlp_revcomm_option_set_string',
           'nag_opt_nlp_solve', 'nag_opt_nlp_sparse',
           'nag_opt_one_var_deriv', 'nag_opt_one_var_no_deriv',
           'nag_opt_qp',
           'nag_opt_read',
           'nag_opt_simplex_easy', 'nag_opt_sparse_convex_qp',
           'nag_opt_sparse_convex_qp_init',
           'nag_opt_sparse_convex_qp_option_get_double',
           'nag_opt_sparse_convex_qp_option_get_integer',
           'nag_opt_sparse_convex_qp_option_set_double',
           'nag_opt_sparse_convex_qp_option_set_file',
           'nag_opt_sparse_convex_qp_option_set_integer',
           'nag_opt_sparse_convex_qp_option_set_string',
           'nag_opt_sparse_convex_qp_solve',
           'nag_opt_sparse_mps_free', 'nag_opt_sparse_mps_read',
           'nag_opt_sparse_nlp_init', 'nag_opt_sparse_nlp_jacobian',
           'nag_opt_sparse_nlp_option_get_double',
           'nag_opt_sparse_nlp_option_get_integer',
           'nag_opt_sparse_nlp_option_set_double',
           'nag_opt_sparse_nlp_option_set_file',
           'nag_opt_sparse_nlp_option_set_integer',
           'nag_opt_sparse_nlp_option_set_string',
           'nag_opt_sparse_nlp_solve',
           'nag_opt_sdp_read_sdpa',
           'nag_opt_handle_init',
           'nag_opt_handle_set_linobj',
           'nag_opt_handle_set_linmatineq',
           'nag_opt_handle_print',
           'nag_opt_handle_opt_set',
           'nag_opt_handle_solve_pennon',
           'nag_opt_handle_free',
           'nag_opt_handle_set_quadmatineq',
           'nag_opt_handle_set_quadobj',
           'nag_opt_handle_opt_get',
           'nag_opt_handle_set_simplebounds',
           'nag_opt_handle_set_nlnobj',
           'nag_opt_handle_set_linconstr',
           'nag_opt_handle_set_nlnconstr',
           'nag_opt_handle_set_nlnhess',
           'nag_opt_handle_solve_ipopt',
           'nag_opt_handle_opt_set_file']
