"""
Deprecated wrappers for the NAG C Library (Mark 26.0) g01 Chapter.

g01 - Simple Calculations on Statistical Data.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/G01/g01conts.html

NAG Copyright 2017.
"""
from ctypes import POINTER
from ctypes import c_double

from .util import get_library
from .util import Nag_TS_URProbMethod
from .util import Nag_TS_URTestType

from .util import Integer
from .util import NagError
from .util import Nag_ClassBoundary
from .util import Nag_Boolean
from .util import Nag_Scores
from .util import Nag_Ties
from .util import Nag_TailProbability
from .util import Nag_LCCMethod
from .util import Nag_MatrixType
from .util import Nag_OrderType
from .util import Nag_SelectMoments
from .util import Nag_IncludeMean
from .util import Nag_MomentType
from .util import Nag_Weightstype

_library_cl = get_library()

g01adc_ctypes = _library_cl.g01adc
g01adc_ctypes.restype = None
g01adc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01adc(k, x, ifreq, xmean, xsd, xskew, xkurt, n, fail):
    """
    Mean, variance, skewness, kurtosis, etc., one variable, from frequency
    table.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01adc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ifreq, xmean, xsd, xskew, xkurt, n
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, xmean, xsd, xskew, xkurt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ifreq, n
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01adc', x, 'x')
    _ifreq = nag_integer_type_check_and_cast('g01adc', ifreq, 'ifreq')
    _xmean = nag_double_type_check_and_cast('g01adc', xmean, 'xmean')
    _xsd = nag_double_type_check_and_cast('g01adc', xsd, 'xsd')
    _xskew = nag_double_type_check_and_cast('g01adc', xskew, 'xskew')
    _xkurt = nag_double_type_check_and_cast('g01adc', xkurt, 'xkurt')
    _n = nag_integer_type_check_and_cast('g01adc', n, 'n')
    g01adc_ctypes(k, _x, _ifreq, _xmean, _xsd, _xskew, _xkurt, _n, fail)

g01aec_ctypes = _library_cl.g01aec
g01aec_ctypes.restype = None
g01aec_ctypes.argtypes = [Integer, POINTER(c_double), Integer, Nag_ClassBoundary, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01aec(n, x, numclass, classb, cint, ifreq, xmin, xmax, fail):
    """
    Frequency table from raw data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01aec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, cint, ifreq, xmin, xmax
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, cint, xmin, xmax
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ifreq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01aec', x, 'x')
    _cint = nag_double_type_check_and_cast('g01aec', cint, 'cint')
    _ifreq = nag_integer_type_check_and_cast('g01aec', ifreq, 'ifreq')
    _xmin = nag_double_type_check_and_cast('g01aec', xmin, 'xmin')
    _xmax = nag_double_type_check_and_cast('g01aec', xmax, 'xmax')
    g01aec_ctypes(n, _x, numclass, classb, _cint, _ifreq, _xmin, _xmax, fail)

g01alc_ctypes = _library_cl.g01alc
g01alc_ctypes.restype = None
g01alc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01alc(n, x, res, fail):
    """
    Five-point summary (median, hinges and extremes).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01alc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, res
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, res
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01alc', x, 'x')
    _res = nag_double_type_check_and_cast('g01alc', res, 'res')
    g01alc_ctypes(n, _x, _res, fail)

g01amc_ctypes = _library_cl.g01amc
g01amc_ctypes.restype = None
g01amc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01amc(n, rv, nq, q, qv, fail):
    """
    Quantiles of a set of unordered values.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01amc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rv, q, qv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rv, q, qv
    """
    from .util import nag_double_type_check_and_cast
    _rv = nag_double_type_check_and_cast('g01amc', rv, 'rv')
    _q = nag_double_type_check_and_cast('g01amc', q, 'q')
    _qv = nag_double_type_check_and_cast('g01amc', qv, 'qv')
    g01amc_ctypes(n, _rv, nq, _q, _qv, fail)

g01anc_ctypes = _library_cl.g01anc
g01anc_ctypes.restype = None
g01anc_ctypes.argtypes = [POINTER(Integer), Integer, POINTER(c_double), Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(NagError)]


def g01anc(ind, n, rv, nb, eps, np, q, qv, nq, rcomm, lrcomm, icomm, licomm, fail):
    """
    Calculates approximate quantiles from a data stream of known size.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01anc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ind, rv, np, q, qv, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rv, q, qv, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ind, np, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ind = nag_integer_type_check_and_cast('g01anc', ind, 'ind')
    _rv = nag_double_type_check_and_cast('g01anc', rv, 'rv')
    _np = nag_integer_type_check_and_cast('g01anc', np, 'np')
    _q = nag_double_type_check_and_cast('g01anc', q, 'q')
    _qv = nag_double_type_check_and_cast('g01anc', qv, 'qv')
    _rcomm = nag_double_type_check_and_cast('g01anc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('g01anc', icomm, 'icomm')
    g01anc_ctypes(_ind, n, _rv, nb, eps, _np, _q, _qv, nq, _rcomm, lrcomm, _icomm, licomm, fail)

g01apc_ctypes = _library_cl.g01apc
g01apc_ctypes.restype = None
g01apc_ctypes.argtypes = [POINTER(Integer), POINTER(c_double), Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(NagError)]


def g01apc(ind, rv, nb, eps, np, q, qv, nq, rcomm, lrcomm, icomm, licomm, fail):
    """
    Calculates approximate quantiles from a data stream of unknown size.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01apc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ind, rv, np, q, qv, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rv, q, qv, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ind, np, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ind = nag_integer_type_check_and_cast('g01apc', ind, 'ind')
    _rv = nag_double_type_check_and_cast('g01apc', rv, 'rv')
    _np = nag_integer_type_check_and_cast('g01apc', np, 'np')
    _q = nag_double_type_check_and_cast('g01apc', q, 'q')
    _qv = nag_double_type_check_and_cast('g01apc', qv, 'qv')
    _rcomm = nag_double_type_check_and_cast('g01apc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('g01apc', icomm, 'icomm')
    g01apc_ctypes(_ind, _rv, nb, eps, _np, _q, _qv, nq, _rcomm, lrcomm, _icomm, licomm, fail)

g01atc_ctypes = _library_cl.g01atc
g01atc_ctypes.restype = None
g01atc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01atc(nb, x, wt, pn, xmean, xsd, xskew, xkurt, xmin, xmax, rcomm, fail):
    """
    Computes univariate summary information: mean, variance, skewness,
    kurtosis.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01atc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, wt, pn, xmean, xsd, xskew, xkurt, xmin, xmax, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, wt, xmean, xsd, xskew, xkurt, xmin, xmax, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        pn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01atc', x, 'x')
    _wt = nag_double_type_check_and_cast('g01atc', wt, 'wt')
    _pn = nag_integer_type_check_and_cast('g01atc', pn, 'pn')
    _xmean = nag_double_type_check_and_cast('g01atc', xmean, 'xmean')
    _xsd = nag_double_type_check_and_cast('g01atc', xsd, 'xsd')
    _xskew = nag_double_type_check_and_cast('g01atc', xskew, 'xskew')
    _xkurt = nag_double_type_check_and_cast('g01atc', xkurt, 'xkurt')
    _xmin = nag_double_type_check_and_cast('g01atc', xmin, 'xmin')
    _xmax = nag_double_type_check_and_cast('g01atc', xmax, 'xmax')
    _rcomm = nag_double_type_check_and_cast('g01atc', rcomm, 'rcomm')
    g01atc_ctypes(nb, _x, _wt, _pn, _xmean, _xsd, _xskew, _xkurt, _xmin, _xmax, _rcomm, fail)

g01auc_ctypes = _library_cl.g01auc
g01auc_ctypes.restype = None
g01auc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01auc(b, mrcomm, pn, xmean, xsd, xskew, xkurt, xmin, xmax, rcomm, fail):
    """
    Combines multiple sets of summary information, for use after g01atc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01auc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        mrcomm, pn, xmean, xsd, xskew, xkurt, xmin, xmax, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        mrcomm, xmean, xsd, xskew, xkurt, xmin, xmax, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        pn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _mrcomm = nag_double_type_check_and_cast('g01auc', mrcomm, 'mrcomm')
    _pn = nag_integer_type_check_and_cast('g01auc', pn, 'pn')
    _xmean = nag_double_type_check_and_cast('g01auc', xmean, 'xmean')
    _xsd = nag_double_type_check_and_cast('g01auc', xsd, 'xsd')
    _xskew = nag_double_type_check_and_cast('g01auc', xskew, 'xskew')
    _xkurt = nag_double_type_check_and_cast('g01auc', xkurt, 'xkurt')
    _xmin = nag_double_type_check_and_cast('g01auc', xmin, 'xmin')
    _xmax = nag_double_type_check_and_cast('g01auc', xmax, 'xmax')
    _rcomm = nag_double_type_check_and_cast('g01auc', rcomm, 'rcomm')
    g01auc_ctypes(b, _mrcomm, _pn, _xmean, _xsd, _xskew, _xkurt, _xmin, _xmax, _rcomm, fail)

g01bjc_ctypes = _library_cl.g01bjc
g01bjc_ctypes.restype = None
g01bjc_ctypes.argtypes = [Integer, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01bjc(n, p, k, plek, pgtk, peqk, fail):
    """
    Binomial distribution function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01bjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        plek, pgtk, peqk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        plek, pgtk, peqk
    """
    from .util import nag_double_type_check_and_cast
    _plek = nag_double_type_check_and_cast('g01bjc', plek, 'plek')
    _pgtk = nag_double_type_check_and_cast('g01bjc', pgtk, 'pgtk')
    _peqk = nag_double_type_check_and_cast('g01bjc', peqk, 'peqk')
    g01bjc_ctypes(n, p, k, _plek, _pgtk, _peqk, fail)

g01bkc_ctypes = _library_cl.g01bkc
g01bkc_ctypes.restype = None
g01bkc_ctypes.argtypes = [c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01bkc(rlamda, k, plek, pgtk, peqk, fail):
    """
    Poisson distribution function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01bkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        plek, pgtk, peqk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        plek, pgtk, peqk
    """
    from .util import nag_double_type_check_and_cast
    _plek = nag_double_type_check_and_cast('g01bkc', plek, 'plek')
    _pgtk = nag_double_type_check_and_cast('g01bkc', pgtk, 'pgtk')
    _peqk = nag_double_type_check_and_cast('g01bkc', peqk, 'peqk')
    g01bkc_ctypes(rlamda, k, _plek, _pgtk, _peqk, fail)

g01blc_ctypes = _library_cl.g01blc
g01blc_ctypes.restype = None
g01blc_ctypes.argtypes = [Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01blc(n, l, m, k, plek, pgtk, peqk, fail):
    """
    Hypergeometric distribution function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01blc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        plek, pgtk, peqk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        plek, pgtk, peqk
    """
    from .util import nag_double_type_check_and_cast
    _plek = nag_double_type_check_and_cast('g01blc', plek, 'plek')
    _pgtk = nag_double_type_check_and_cast('g01blc', pgtk, 'pgtk')
    _peqk = nag_double_type_check_and_cast('g01blc', peqk, 'peqk')
    g01blc_ctypes(n, l, m, k, _plek, _pgtk, _peqk, fail)

g01dac_ctypes = _library_cl.g01dac
g01dac_ctypes.restype = None
g01dac_ctypes.argtypes = [Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def g01dac(n, pp, etol, errest, fail):
    """
    Normal scores, accurate values.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        pp, errest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        pp, errest
    """
    from .util import nag_double_type_check_and_cast
    _pp = nag_double_type_check_and_cast('g01dac', pp, 'pp')
    _errest = nag_double_type_check_and_cast('g01dac', errest, 'errest')
    g01dac_ctypes(n, _pp, etol, _errest, fail)

g01dcc_ctypes = _library_cl.g01dcc
g01dcc_ctypes.restype = None
g01dcc_ctypes.argtypes = [Integer, c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def g01dcc(n, exp1, exp2, sumssq, vec, fail):
    """
    Normal scores, approximate variance-covariance matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vec
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        vec
    """
    from .util import nag_double_type_check_and_cast
    _vec = nag_double_type_check_and_cast('g01dcc', vec, 'vec')
    g01dcc_ctypes(n, exp1, exp2, sumssq, _vec, fail)

g01ddc_ctypes = _library_cl.g01ddc
g01ddc_ctypes.restype = None
g01ddc_ctypes.argtypes = [Integer, POINTER(c_double), Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01ddc(n, x, calcwts, a, w, pw, fail):
    """
    Shapiro and Wilk's W test for Normality.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ddc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, a, w, pw
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, a, w, pw
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01ddc', x, 'x')
    _a = nag_double_type_check_and_cast('g01ddc', a, 'a')
    _w = nag_double_type_check_and_cast('g01ddc', w, 'w')
    _pw = nag_double_type_check_and_cast('g01ddc', pw, 'pw')
    g01ddc_ctypes(n, _x, calcwts, _a, _w, _pw, fail)

g01dhc_ctypes = _library_cl.g01dhc
g01dhc_ctypes.restype = None
g01dhc_ctypes.argtypes = [Nag_Scores, Nag_Ties, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01dhc(scores, ties, n, x, r, fail):
    """
    Ranks, Normal scores, approximate Normal scores or exponential (Savage)
    scores.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01dhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, r
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01dhc', x, 'x')
    _r = nag_double_type_check_and_cast('g01dhc', r, 'r')
    g01dhc_ctypes(scores, ties, n, _x, _r, fail)

g01eac_ctypes = _library_cl.g01eac
g01eac_ctypes.restype = c_double
g01eac_ctypes.argtypes = [Nag_TailProbability, c_double, POINTER(NagError)]


def g01eac(tail, x, fail):
    """
    Probabilities for the standard Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01eac.html
    """
    return_value = g01eac_ctypes(tail, x, fail)
    return return_value

g01ebc_ctypes = _library_cl.g01ebc
g01ebc_ctypes.restype = c_double
g01ebc_ctypes.argtypes = [Nag_TailProbability, c_double, c_double, POINTER(NagError)]


def g01ebc(tail, t, df, fail):
    """
    Probabilities for Student's t-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ebc.html
    """
    return_value = g01ebc_ctypes(tail, t, df, fail)
    return return_value

g01ecc_ctypes = _library_cl.g01ecc
g01ecc_ctypes.restype = c_double
g01ecc_ctypes.argtypes = [Nag_TailProbability, c_double, c_double, POINTER(NagError)]


def g01ecc(tail, x, df, fail):
    """
    Probabilities for chi**2 distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ecc.html
    """
    return_value = g01ecc_ctypes(tail, x, df, fail)
    return return_value

g01edc_ctypes = _library_cl.g01edc
g01edc_ctypes.restype = c_double
g01edc_ctypes.argtypes = [Nag_TailProbability, c_double, c_double, c_double, POINTER(NagError)]


def g01edc(tail, f, df1, df2, fail):
    """
    Probabilities for F-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01edc.html
    """
    return_value = g01edc_ctypes(tail, f, df1, df2, fail)
    return return_value

g01eec_ctypes = _library_cl.g01eec
g01eec_ctypes.restype = None
g01eec_ctypes.argtypes = [c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01eec(x, a, b, tol, p, q, pdf, fail):
    """
    Upper and lower tail probabilities and probability density function for
    the beta distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01eec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        p, q, pdf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, q, pdf
    """
    from .util import nag_double_type_check_and_cast
    _p = nag_double_type_check_and_cast('g01eec', p, 'p')
    _q = nag_double_type_check_and_cast('g01eec', q, 'q')
    _pdf = nag_double_type_check_and_cast('g01eec', pdf, 'pdf')
    g01eec_ctypes(x, a, b, tol, _p, _q, _pdf, fail)

g01efc_ctypes = _library_cl.g01efc
g01efc_ctypes.restype = c_double
g01efc_ctypes.argtypes = [Nag_TailProbability, c_double, c_double, c_double, POINTER(NagError)]


def g01efc(tail, g, a, b, fail):
    """
    Probabilities for the gamma distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01efc.html
    """
    return_value = g01efc_ctypes(tail, g, a, b, fail)
    return return_value

g01emc_ctypes = _library_cl.g01emc
g01emc_ctypes.restype = c_double
g01emc_ctypes.argtypes = [c_double, c_double, Integer, POINTER(NagError)]


def g01emc(q, v, ir, fail):
    """
    Computes probability for the Studentized range statistic.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01emc.html
    """
    return_value = g01emc_ctypes(q, v, ir, fail)
    return return_value

g01epc_ctypes = _library_cl.g01epc
g01epc_ctypes.restype = None
g01epc_ctypes.argtypes = [Integer, Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01epc(n, ip, d, pdl, pdu, fail):
    """
    Computes bounds for the significance of a Durbin--Watson statistic.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01epc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        pdl, pdu
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        pdl, pdu
    """
    from .util import nag_double_type_check_and_cast
    _pdl = nag_double_type_check_and_cast('g01epc', pdl, 'pdl')
    _pdu = nag_double_type_check_and_cast('g01epc', pdu, 'pdu')
    g01epc_ctypes(n, ip, d, _pdl, _pdu, fail)

g01erc_ctypes = _library_cl.g01erc
g01erc_ctypes.restype = c_double
g01erc_ctypes.argtypes = [c_double, c_double, POINTER(NagError)]


def g01erc(t, vk, fail):
    """
    Computes probability for von Mises distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01erc.html
    """
    return_value = g01erc_ctypes(t, vk, fail)
    return return_value

g01etc_ctypes = _library_cl.g01etc
g01etc_ctypes.restype = c_double
g01etc_ctypes.argtypes = [c_double]


def g01etc(x):
    """
    Landau distribution function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01etc.html
    """
    return_value = g01etc_ctypes(x)
    return return_value

g01euc_ctypes = _library_cl.g01euc
g01euc_ctypes.restype = c_double
g01euc_ctypes.argtypes = [c_double, POINTER(c_double)]


def g01euc(x, commarr):
    """
    Vavilov distribution function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01euc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        commarr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        commarr
    """
    from .util import nag_double_type_check_and_cast
    _commarr = nag_double_type_check_and_cast('g01euc', commarr, 'commarr')
    return_value = g01euc_ctypes(x, _commarr)
    return return_value

g01ewc_ctypes = _library_cl.g01ewc
g01ewc_ctypes.restype = c_double
g01ewc_ctypes.argtypes = [Nag_TS_URProbMethod, Nag_TS_URTestType, Integer, c_double, Integer, POINTER(Integer), POINTER(NagError)]


def g01ewc(method, _type, n, ts, nsamp, state, fail):
    """
    Computes probabilities for the Dickey--Fuller unit root test.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ewc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        state
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        state
    """
    from .util import nag_integer_type_check_and_cast
    _state = nag_integer_type_check_and_cast('g01ewc', state, 'state')
    return_val = g01ewc_ctypes(method, _type, n, ts, nsamp, _state, fail)
    return return_val

nag_prob_dickey_fuller_unit = g01ewc

g01eyc_ctypes = _library_cl.g01eyc
g01eyc_ctypes.restype = c_double
g01eyc_ctypes.argtypes = [Integer, c_double, POINTER(NagError)]


def g01eyc(n, d, fail):
    """
    Computes probabilities for the one-sample Kolmogorov--Smirnov
    distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01eyc.html
    """
    return_value = g01eyc_ctypes(n, d, fail)
    return return_value

g01ezc_ctypes = _library_cl.g01ezc
g01ezc_ctypes.restype = c_double
g01ezc_ctypes.argtypes = [Integer, Integer, c_double, POINTER(NagError)]


def g01ezc(n1, n2, d, fail):
    """
    Computes probabilities for the two-sample Kolmogorov--Smirnov
    distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ezc.html
    """
    return_value = g01ezc_ctypes(n1, n2, d, fail)
    return return_value

g01fac_ctypes = _library_cl.g01fac
g01fac_ctypes.restype = c_double
g01fac_ctypes.argtypes = [Nag_TailProbability, c_double, POINTER(NagError)]


def g01fac(tail, p, fail):
    """
    Deviates for the Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01fac.html
    """
    return_value = g01fac_ctypes(tail, p, fail)
    return return_value

g01fbc_ctypes = _library_cl.g01fbc
g01fbc_ctypes.restype = c_double
g01fbc_ctypes.argtypes = [Nag_TailProbability, c_double, c_double, POINTER(NagError)]


def g01fbc(tail, p, df, fail):
    """
    Deviates for Student's t-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01fbc.html
    """
    return_value = g01fbc_ctypes(tail, p, df, fail)
    return return_value

g01fcc_ctypes = _library_cl.g01fcc
g01fcc_ctypes.restype = c_double
g01fcc_ctypes.argtypes = [c_double, c_double, POINTER(NagError)]


def g01fcc(p, df, fail):
    """
    Deviates for the chi**2 distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01fcc.html
    """
    return_value = g01fcc_ctypes(p, df, fail)
    return return_value

g01fdc_ctypes = _library_cl.g01fdc
g01fdc_ctypes.restype = c_double
g01fdc_ctypes.argtypes = [c_double, c_double, c_double, POINTER(NagError)]


def g01fdc(p, df1, df2, fail):
    """
    Deviates for the F-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01fdc.html
    """
    return_value = g01fdc_ctypes(p, df1, df2, fail)
    return return_value

g01fec_ctypes = _library_cl.g01fec
g01fec_ctypes.restype = c_double
g01fec_ctypes.argtypes = [c_double, c_double, c_double, c_double, POINTER(NagError)]


def g01fec(p, a, b, tol, fail):
    """
    Deviates for the beta distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01fec.html
    """
    return_value = g01fec_ctypes(p, a, b, tol, fail)
    return return_value

g01ffc_ctypes = _library_cl.g01ffc
g01ffc_ctypes.restype = c_double
g01ffc_ctypes.argtypes = [c_double, c_double, c_double, c_double, POINTER(NagError)]


def g01ffc(p, a, b, tol, fail):
    """
    Deviates for the gamma distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ffc.html
    """
    return_value = g01ffc_ctypes(p, a, b, tol, fail)
    return return_value

g01fmc_ctypes = _library_cl.g01fmc
g01fmc_ctypes.restype = c_double
g01fmc_ctypes.argtypes = [c_double, c_double, Integer, POINTER(NagError)]


def g01fmc(p, v, ir, fail):
    """
    Computes deviates for the Studentized range statistic.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01fmc.html
    """
    return_value = g01fmc_ctypes(p, v, ir, fail)
    return return_value

g01ftc_ctypes = _library_cl.g01ftc
g01ftc_ctypes.restype = c_double
g01ftc_ctypes.argtypes = [c_double, POINTER(NagError)]


def g01ftc(x, fail):
    """
    Landau inverse function Psi(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ftc.html
    """
    return_value = g01ftc_ctypes(x, fail)
    return return_value

g01gbc_ctypes = _library_cl.g01gbc
g01gbc_ctypes.restype = c_double
g01gbc_ctypes.argtypes = [c_double, c_double, c_double, c_double, Integer, POINTER(NagError)]


def g01gbc(t, df, delta, tol, maxiter, fail):
    """
    Computes probabilities for the non-central Student's t-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01gbc.html
    """
    return_value = g01gbc_ctypes(t, df, delta, tol, maxiter, fail)
    return return_value

g01gcc_ctypes = _library_cl.g01gcc
g01gcc_ctypes.restype = c_double
g01gcc_ctypes.argtypes = [c_double, c_double, c_double, c_double, Integer, POINTER(NagError)]


def g01gcc(x, df, _lambda, tol, maxiter, fail):
    """
    Computes probabilities for the non-central chi**2 distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01gcc.html
    """
    return_value = g01gcc_ctypes(x, df, _lambda, tol, maxiter, fail)
    return return_value

g01gdc_ctypes = _library_cl.g01gdc
g01gdc_ctypes.restype = c_double
g01gdc_ctypes.argtypes = [c_double, c_double, c_double, c_double, c_double, Integer, POINTER(NagError)]


def g01gdc(f, df1, df2, _lambda, tol, maxiter, fail):
    """
    Computes probabilities for the non-central F-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01gdc.html
    """
    return_value = g01gdc_ctypes(f, df1, df2, _lambda, tol, maxiter, fail)
    return return_value

g01gec_ctypes = _library_cl.g01gec
g01gec_ctypes.restype = c_double
g01gec_ctypes.argtypes = [c_double, c_double, c_double, c_double, c_double, Integer, POINTER(NagError)]


def g01gec(x, a, b, _lambda, tol, maxiter, fail):
    """
    Computes probabilities for the non-central beta distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01gec.html
    """
    return_value = g01gec_ctypes(x, a, b, _lambda, tol, maxiter, fail)
    return return_value

g01hac_ctypes = _library_cl.g01hac
g01hac_ctypes.restype = c_double
g01hac_ctypes.argtypes = [c_double, c_double, c_double, POINTER(NagError)]


def g01hac(x, y, rho, fail):
    """
    Probability for the bivariate Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01hac.html
    """
    return_value = g01hac_ctypes(x, y, rho, fail)
    return return_value

g01hbc_ctypes = _library_cl.g01hbc
g01hbc_ctypes.restype = c_double
g01hbc_ctypes.argtypes = [Nag_TailProbability, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, c_double, Integer, POINTER(NagError)]


def g01hbc(tail, n, a, b, mean, sigma, tdsig, tol, maxpts, fail):
    """
    Computes probabilities for the multivariate Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01hbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, mean, sigma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, mean, sigma
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('g01hbc', a, 'a')
    _b = nag_double_type_check_and_cast('g01hbc', b, 'b')
    _mean = nag_double_type_check_and_cast('g01hbc', mean, 'mean')
    _sigma = nag_double_type_check_and_cast('g01hbc', sigma, 'sigma')
    return_value = g01hbc_ctypes(tail, n, _a, _b, _mean, _sigma, tdsig, tol, maxpts, fail)
    return return_value

g01hcc_ctypes = _library_cl.g01hcc
g01hcc_ctypes.restype = c_double
g01hcc_ctypes.argtypes = [Nag_TailProbability, POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(NagError)]


def g01hcc(tail, a, b, df, rho, fail):
    """
    Computes probabilities for the bivariate Student's t-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01hcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('g01hcc', a, 'a')
    _b = nag_double_type_check_and_cast('g01hcc', b, 'b')
    return_value = g01hcc_ctypes(tail, _a, _b, df, rho, fail)
    return return_value

g01hdc_ctypes = _library_cl.g01hdc
g01hdc_ctypes.restype = c_double
g01hdc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), Nag_Boolean, POINTER(c_double), Integer, c_double, c_double, Integer, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def g01hdc(n, tail, a, b, nu, delta, iscov, rc, ldrc, epsabs, epsrel, numsub, nsampl, fmax, errest, fail):
    """
    Computes the probability for the multivariate Student's t-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01hdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, delta, rc, errest, tail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, delta, rc, errest
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01hdc', tail, 'tail', Nag_TailProbability)
    _a = nag_double_type_check_and_cast('g01hdc', a, 'a')
    _b = nag_double_type_check_and_cast('g01hdc', b, 'b')
    _delta = nag_double_type_check_and_cast('g01hdc', delta, 'delta')
    _rc = nag_double_type_check_and_cast('g01hdc', rc, 'rc')
    _errest = nag_double_type_check_and_cast('g01hdc', errest, 'errest')
    return_value = g01hdc_ctypes(n, _tail, _a, _b, nu, _delta, iscov, _rc, ldrc, epsabs, epsrel, numsub, nsampl, fmax, _errest, fail)
    return return_value

g01jcc_ctypes = _library_cl.g01jcc
g01jcc_ctypes.restype = None
g01jcc_ctypes.argtypes = [POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(c_double), c_double, Integer, POINTER(NagError)]


def g01jcc(a, mult, rlamda, n, c, p, pdf, tol, maxit, fail):
    """
    Computes probability for a positive linear combination of chi**2
    variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01jcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, mult, rlamda, p, pdf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, rlamda, p, pdf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        mult
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('g01jcc', a, 'a')
    _mult = nag_integer_type_check_and_cast('g01jcc', mult, 'mult')
    _rlamda = nag_double_type_check_and_cast('g01jcc', rlamda, 'rlamda')
    _p = nag_double_type_check_and_cast('g01jcc', p, 'p')
    _pdf = nag_double_type_check_and_cast('g01jcc', pdf, 'pdf')
    g01jcc_ctypes(_a, _mult, _rlamda, n, c, _p, _pdf, tol, maxit, fail)

g01jdc_ctypes = _library_cl.g01jdc
g01jdc_ctypes.restype = None
g01jdc_ctypes.argtypes = [Nag_LCCMethod, Integer, POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(NagError)]


def g01jdc(method, n, rlam, d, c, prob, fail):
    """
    Computes lower tail probability for a linear combination of (central)
    chi**2 variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01jdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rlam, prob
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rlam, prob
    """
    from .util import nag_double_type_check_and_cast
    _rlam = nag_double_type_check_and_cast('g01jdc', rlam, 'rlam')
    _prob = nag_double_type_check_and_cast('g01jdc', prob, 'prob')
    g01jdc_ctypes(method, n, _rlam, d, c, _prob, fail)

g01kac_ctypes = _library_cl.g01kac
g01kac_ctypes.restype = c_double
g01kac_ctypes.argtypes = [c_double, c_double, c_double, POINTER(NagError)]


def g01kac(x, xmean, xstd, fail):
    """
    Calculates the value for the probability density function of the Normal
    distribution at a chosen point.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01kac.html
    """
    return_value = g01kac_ctypes(x, xmean, xstd, fail)
    return return_value

g01kfc_ctypes = _library_cl.g01kfc
g01kfc_ctypes.restype = c_double
g01kfc_ctypes.argtypes = [c_double, c_double, c_double, POINTER(NagError)]


def g01kfc(x, a, b, fail):
    """
    Calculates the value for the probability density function of the gamma
    distribution at a chosen point.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01kfc.html
    """
    return_value = g01kfc_ctypes(x, a, b, fail)
    return return_value

g01kkc_ctypes = _library_cl.g01kkc
g01kkc_ctypes.restype = None
g01kkc_ctypes.argtypes = [Nag_Boolean, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01kkc(ilog, lx, x, la, a, lb, b, pdf, ivalid, fail):
    """
    Computes a vector of values for the probability density function of the
    gamma distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01kkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, a, b, pdf, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, a, b, pdf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01kkc', x, 'x')
    _a = nag_double_type_check_and_cast('g01kkc', a, 'a')
    _b = nag_double_type_check_and_cast('g01kkc', b, 'b')
    _pdf = nag_double_type_check_and_cast('g01kkc', pdf, 'pdf')
    _ivalid = nag_integer_type_check_and_cast('g01kkc', ivalid, 'ivalid')
    g01kkc_ctypes(ilog, lx, _x, la, _a, lb, _b, _pdf, _ivalid, fail)

g01kqc_ctypes = _library_cl.g01kqc
g01kqc_ctypes.restype = None
g01kqc_ctypes.argtypes = [Nag_Boolean, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01kqc(ilog, lx, x, lxmu, xmu, lxstd, xstd, pdf, ivalid, fail):
    """
    Computes a vector of values for the probability density function of the
    Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01kqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, xmu, xstd, pdf, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, xmu, xstd, pdf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01kqc', x, 'x')
    _xmu = nag_double_type_check_and_cast('g01kqc', xmu, 'xmu')
    _xstd = nag_double_type_check_and_cast('g01kqc', xstd, 'xstd')
    _pdf = nag_double_type_check_and_cast('g01kqc', pdf, 'pdf')
    _ivalid = nag_integer_type_check_and_cast('g01kqc', ivalid, 'ivalid')
    g01kqc_ctypes(ilog, lx, _x, lxmu, _xmu, lxstd, _xstd, _pdf, _ivalid, fail)

g01lbc_ctypes = _library_cl.g01lbc
g01lbc_ctypes.restype = None
g01lbc_ctypes.argtypes = [Nag_Boolean, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Nag_MatrixType, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01lbc(ilog, k, n, x, pdx, xmu, iuld, sig, pdsig, pdf, rank, fail):
    """
    Computes a vector of values for the probability density function of the
    multivariate Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01lbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, xmu, sig, pdf, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, xmu, sig, pdf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01lbc', x, 'x')
    _xmu = nag_double_type_check_and_cast('g01lbc', xmu, 'xmu')
    _sig = nag_double_type_check_and_cast('g01lbc', sig, 'sig')
    _pdf = nag_double_type_check_and_cast('g01lbc', pdf, 'pdf')
    _rank = nag_integer_type_check_and_cast('g01lbc', rank, 'rank')
    g01lbc_ctypes(ilog, k, n, _x, pdx, _xmu, iuld, _sig, pdsig, _pdf, _rank, fail)

g01mbc_ctypes = _library_cl.g01mbc
g01mbc_ctypes.restype = c_double
g01mbc_ctypes.argtypes = [c_double]


def g01mbc(x):
    """
    Computes reciprocal of Mills' Ratio.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01mbc.html
    """
    return_value = g01mbc_ctypes(x)
    return return_value

g01mtc_ctypes = _library_cl.g01mtc
g01mtc_ctypes.restype = c_double
g01mtc_ctypes.argtypes = [c_double]


def g01mtc(x):
    """
    Landau density function phi(lambda).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01mtc.html
    """
    return_value = g01mtc_ctypes(x)
    return return_value

g01muc_ctypes = _library_cl.g01muc
g01muc_ctypes.restype = c_double
g01muc_ctypes.argtypes = [c_double, POINTER(c_double)]


def g01muc(x, commarr):
    """
    Vavilov density function phi_V(lambda;kappa,beta**2).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01muc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        commarr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        commarr
    """
    from .util import nag_double_type_check_and_cast
    _commarr = nag_double_type_check_and_cast('g01muc', commarr, 'commarr')
    return_value = g01muc_ctypes(x, _commarr)
    return return_value

g01nac_ctypes = _library_cl.g01nac
g01nac_ctypes.restype = None
g01nac_ctypes.argtypes = [Nag_OrderType, Nag_SelectMoments, Nag_IncludeMean, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01nac(order, mom, mean, n, a, pda, emu, sigma, pdsig, l, rkum, rmom, fail):
    """
    Cumulants and moments of quadratic forms in Normal variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01nac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, emu, sigma, rkum, rmom
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, emu, sigma, rkum, rmom
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('g01nac', a, 'a')
    _emu = nag_double_type_check_and_cast('g01nac', emu, 'emu')
    _sigma = nag_double_type_check_and_cast('g01nac', sigma, 'sigma')
    _rkum = nag_double_type_check_and_cast('g01nac', rkum, 'rkum')
    _rmom = nag_double_type_check_and_cast('g01nac', rmom, 'rmom')
    g01nac_ctypes(order, mom, mean, n, _a, pda, _emu, _sigma, pdsig, l, _rkum, _rmom, fail)

g01nbc_ctypes = _library_cl.g01nbc
g01nbc_ctypes.restype = None
g01nbc_ctypes.argtypes = [Nag_OrderType, Nag_MomentType, Nag_IncludeMean, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), c_double, POINTER(NagError)]


def g01nbc(order, ratiotype, mean, n, a, pda, b, pdb, c, pdc, ela, emu, sigma, pdsig, l1, l2, lmax, rmom, abserr, eps, fail):
    """
    Moments of ratios of quadratic forms in Normal variables, and related
    statistics.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01nbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, ela, emu, sigma, lmax, rmom, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c, ela, emu, sigma, rmom, abserr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        lmax
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('g01nbc', a, 'a')
    _b = nag_double_type_check_and_cast('g01nbc', b, 'b')
    _c = nag_double_type_check_and_cast('g01nbc', c, 'c')
    _ela = nag_double_type_check_and_cast('g01nbc', ela, 'ela')
    _emu = nag_double_type_check_and_cast('g01nbc', emu, 'emu')
    _sigma = nag_double_type_check_and_cast('g01nbc', sigma, 'sigma')
    _lmax = nag_integer_type_check_and_cast('g01nbc', lmax, 'lmax')
    _rmom = nag_double_type_check_and_cast('g01nbc', rmom, 'rmom')
    _abserr = nag_double_type_check_and_cast('g01nbc', abserr, 'abserr')
    g01nbc_ctypes(order, ratiotype, mean, n, _a, pda, _b, pdb, _c, pdc, _ela, _emu, _sigma, pdsig, l1, l2, _lmax, _rmom, _abserr, eps, fail)

g01ptc_ctypes = _library_cl.g01ptc
g01ptc_ctypes.restype = c_double
g01ptc_ctypes.argtypes = [c_double]


def g01ptc(x):
    """
    Landau first moment function Phi_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01ptc.html
    """
    return_value = g01ptc_ctypes(x)
    return return_value

g01qtc_ctypes = _library_cl.g01qtc
g01qtc_ctypes.restype = c_double
g01qtc_ctypes.argtypes = [c_double]


def g01qtc(x):
    """
    Landau second moment function Phi_2(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01qtc.html
    """
    return_value = g01qtc_ctypes(x)
    return return_value

g01rtc_ctypes = _library_cl.g01rtc
g01rtc_ctypes.restype = c_double
g01rtc_ctypes.argtypes = [c_double]


def g01rtc(x):
    """
    Landau derivative function phi'(lambda).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01rtc.html
    """
    return_value = g01rtc_ctypes(x)
    return return_value

g01sac_ctypes = _library_cl.g01sac
g01sac_ctypes.restype = None
g01sac_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01sac(ltail, tail, lx, x, lxmu, xmu, lxstd, xstd, p, ivalid, fail):
    """
    Computes a vector of probabilities for the standard Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01sac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, x, xmu, xstd, p, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, xmu, xstd, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01sac', tail, 'tail', Nag_TailProbability)
    _x = nag_double_type_check_and_cast('g01sac', x, 'x')
    _xmu = nag_double_type_check_and_cast('g01sac', xmu, 'xmu')
    _xstd = nag_double_type_check_and_cast('g01sac', xstd, 'xstd')
    _p = nag_double_type_check_and_cast('g01sac', p, 'p')
    _ivalid = nag_integer_type_check_and_cast('g01sac', ivalid, 'ivalid')
    g01sac_ctypes(ltail, _tail, lx, _x, lxmu, _xmu, lxstd, _xstd, _p, _ivalid, fail)

g01sbc_ctypes = _library_cl.g01sbc
g01sbc_ctypes.restype = None
g01sbc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01sbc(ltail, tail, lt, t, ldf, df, p, ivalid, fail):
    """
    Computes a vector of probabilities for the Student's t-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01sbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, t, df, p, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, df, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01sbc', tail, 'tail', Nag_TailProbability)
    _t = nag_double_type_check_and_cast('g01sbc', t, 't')
    _df = nag_double_type_check_and_cast('g01sbc', df, 'df')
    _p = nag_double_type_check_and_cast('g01sbc', p, 'p')
    _ivalid = nag_integer_type_check_and_cast('g01sbc', ivalid, 'ivalid')
    g01sbc_ctypes(ltail, _tail, lt, _t, ldf, _df, _p, _ivalid, fail)

g01scc_ctypes = _library_cl.g01scc
g01scc_ctypes.restype = None
g01scc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01scc(ltail, tail, lx, x, ldf, df, p, ivalid, fail):
    """
    Computes a vector of probabilities for chi**2 distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01scc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, x, df, p, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, df, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01scc', tail, 'tail', Nag_TailProbability)
    _x = nag_double_type_check_and_cast('g01scc', x, 'x')
    _df = nag_double_type_check_and_cast('g01scc', df, 'df')
    _p = nag_double_type_check_and_cast('g01scc', p, 'p')
    _ivalid = nag_integer_type_check_and_cast('g01scc', ivalid, 'ivalid')
    g01scc_ctypes(ltail, _tail, lx, _x, ldf, _df, _p, _ivalid, fail)

g01sdc_ctypes = _library_cl.g01sdc
g01sdc_ctypes.restype = None
g01sdc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01sdc(ltail, tail, lf, f, ldf1, df1, ldf2, df2, p, ivalid, fail):
    """
    Computes a vector of probabilities for F-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01sdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, f, df1, df2, p, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        f, df1, df2, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01sdc', tail, 'tail', Nag_TailProbability)
    _f = nag_double_type_check_and_cast('g01sdc', f, 'f')
    _df1 = nag_double_type_check_and_cast('g01sdc', df1, 'df1')
    _df2 = nag_double_type_check_and_cast('g01sdc', df2, 'df2')
    _p = nag_double_type_check_and_cast('g01sdc', p, 'p')
    _ivalid = nag_integer_type_check_and_cast('g01sdc', ivalid, 'ivalid')
    g01sdc_ctypes(ltail, _tail, lf, _f, ldf1, _df1, ldf2, _df2, _p, _ivalid, fail)

g01sec_ctypes = _library_cl.g01sec
g01sec_ctypes.restype = None
g01sec_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01sec(ltail, tail, lbeta, beta, la, a, lb, b, p, ivalid, fail):
    """
    Computes a vector of probabilities for the beta distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01sec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, beta, a, b, p, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        beta, a, b, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01sec', tail, 'tail', Nag_TailProbability)
    _beta = nag_double_type_check_and_cast('g01sec', beta, 'beta')
    _a = nag_double_type_check_and_cast('g01sec', a, 'a')
    _b = nag_double_type_check_and_cast('g01sec', b, 'b')
    _p = nag_double_type_check_and_cast('g01sec', p, 'p')
    _ivalid = nag_integer_type_check_and_cast('g01sec', ivalid, 'ivalid')
    g01sec_ctypes(ltail, _tail, lbeta, _beta, la, _a, lb, _b, _p, _ivalid, fail)

g01sfc_ctypes = _library_cl.g01sfc
g01sfc_ctypes.restype = None
g01sfc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01sfc(ltail, tail, lg, g, la, a, lb, b, p, ivalid, fail):
    """
    Computes a vector of probabilities for the gamma distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01sfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, g, a, b, p, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        g, a, b, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01sfc', tail, 'tail', Nag_TailProbability)
    _g = nag_double_type_check_and_cast('g01sfc', g, 'g')
    _a = nag_double_type_check_and_cast('g01sfc', a, 'a')
    _b = nag_double_type_check_and_cast('g01sfc', b, 'b')
    _p = nag_double_type_check_and_cast('g01sfc', p, 'p')
    _ivalid = nag_integer_type_check_and_cast('g01sfc', ivalid, 'ivalid')
    g01sfc_ctypes(ltail, _tail, lg, _g, la, _a, lb, _b, _p, _ivalid, fail)

g01sjc_ctypes = _library_cl.g01sjc
g01sjc_ctypes.restype = None
g01sjc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01sjc(ln, n, lp, p, lk, k, plek, pgtk, peqk, ivalid, fail):
    """
    Computes a vector of probabilities for the binomial distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01sjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        n, p, k, plek, pgtk, peqk, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, plek, pgtk, peqk
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        n, k, ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _n = nag_integer_type_check_and_cast('g01sjc', n, 'n')
    _p = nag_double_type_check_and_cast('g01sjc', p, 'p')
    _k = nag_integer_type_check_and_cast('g01sjc', k, 'k')
    _plek = nag_double_type_check_and_cast('g01sjc', plek, 'plek')
    _pgtk = nag_double_type_check_and_cast('g01sjc', pgtk, 'pgtk')
    _peqk = nag_double_type_check_and_cast('g01sjc', peqk, 'peqk')
    _ivalid = nag_integer_type_check_and_cast('g01sjc', ivalid, 'ivalid')
    g01sjc_ctypes(ln, _n, lp, _p, lk, _k, _plek, _pgtk, _peqk, _ivalid, fail)

g01skc_ctypes = _library_cl.g01skc
g01skc_ctypes.restype = None
g01skc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01skc(ll, l, lk, k, plek, pgtk, peqk, ivalid, fail):
    """
    Computes a vector of probabilities for the Poisson distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01skc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        l, k, plek, pgtk, peqk, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        l, plek, pgtk, peqk
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _l = nag_double_type_check_and_cast('g01skc', l, 'l')
    _k = nag_integer_type_check_and_cast('g01skc', k, 'k')
    _plek = nag_double_type_check_and_cast('g01skc', plek, 'plek')
    _pgtk = nag_double_type_check_and_cast('g01skc', pgtk, 'pgtk')
    _peqk = nag_double_type_check_and_cast('g01skc', peqk, 'peqk')
    _ivalid = nag_integer_type_check_and_cast('g01skc', ivalid, 'ivalid')
    g01skc_ctypes(ll, _l, lk, _k, _plek, _pgtk, _peqk, _ivalid, fail)

g01slc_ctypes = _library_cl.g01slc
g01slc_ctypes.restype = None
g01slc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(Integer), Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01slc(ln, n, ll, l, lm, m, lk, k, plek, pgtk, peqk, ivalid, fail):
    """
    Computes a vector of probabilities for the hypergeometric distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01slc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        n, l, m, k, plek, pgtk, peqk, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        plek, pgtk, peqk
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        n, l, m, k, ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _n = nag_integer_type_check_and_cast('g01slc', n, 'n')
    _l = nag_integer_type_check_and_cast('g01slc', l, 'l')
    _m = nag_integer_type_check_and_cast('g01slc', m, 'm')
    _k = nag_integer_type_check_and_cast('g01slc', k, 'k')
    _plek = nag_double_type_check_and_cast('g01slc', plek, 'plek')
    _pgtk = nag_double_type_check_and_cast('g01slc', pgtk, 'pgtk')
    _peqk = nag_double_type_check_and_cast('g01slc', peqk, 'peqk')
    _ivalid = nag_integer_type_check_and_cast('g01slc', ivalid, 'ivalid')
    g01slc_ctypes(ln, _n, ll, _l, lm, _m, lk, _k, _plek, _pgtk, _peqk, _ivalid, fail)

g01tac_ctypes = _library_cl.g01tac
g01tac_ctypes.restype = None
g01tac_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01tac(ltail, tail, lp, p, lxmu, xmu, lxstd, xstd, x, ivalid, fail):
    """
    Computes a vector of deviates for the standard Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01tac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, p, xmu, xstd, x, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, xmu, xstd, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01tac', tail, 'tail', Nag_TailProbability)
    _p = nag_double_type_check_and_cast('g01tac', p, 'p')
    _xmu = nag_double_type_check_and_cast('g01tac', xmu, 'xmu')
    _xstd = nag_double_type_check_and_cast('g01tac', xstd, 'xstd')
    _x = nag_double_type_check_and_cast('g01tac', x, 'x')
    _ivalid = nag_integer_type_check_and_cast('g01tac', ivalid, 'ivalid')
    g01tac_ctypes(ltail, _tail, lp, _p, lxmu, _xmu, lxstd, _xstd, _x, _ivalid, fail)

g01tbc_ctypes = _library_cl.g01tbc
g01tbc_ctypes.restype = None
g01tbc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01tbc(ltail, tail, lp, p, ldf, df, t, ivalid, fail):
    """
    Computes a vector of deviates for Student's t-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01tbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, p, df, t, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, df, t
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01tbc', tail, 'tail', Nag_TailProbability)
    _p = nag_double_type_check_and_cast('g01tbc', p, 'p')
    _df = nag_double_type_check_and_cast('g01tbc', df, 'df')
    _t = nag_double_type_check_and_cast('g01tbc', t, 't')
    _ivalid = nag_integer_type_check_and_cast('g01tbc', ivalid, 'ivalid')
    g01tbc_ctypes(ltail, _tail, lp, _p, ldf, _df, _t, _ivalid, fail)

g01tcc_ctypes = _library_cl.g01tcc
g01tcc_ctypes.restype = None
g01tcc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01tcc(ltail, tail, lp, p, ldf, df, x, ivalid, fail):
    """
    Computes a vector of deviates for chi**2 distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01tcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, p, df, x, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, df, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01tcc', tail, 'tail', Nag_TailProbability)
    _p = nag_double_type_check_and_cast('g01tcc', p, 'p')
    _df = nag_double_type_check_and_cast('g01tcc', df, 'df')
    _x = nag_double_type_check_and_cast('g01tcc', x, 'x')
    _ivalid = nag_integer_type_check_and_cast('g01tcc', ivalid, 'ivalid')
    g01tcc_ctypes(ltail, _tail, lp, _p, ldf, _df, _x, _ivalid, fail)

g01tdc_ctypes = _library_cl.g01tdc
g01tdc_ctypes.restype = None
g01tdc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01tdc(ltail, tail, lp, p, ldf1, df1, ldf2, df2, f, ivalid, fail):
    """
    Computes a vector of deviates for F-distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01tdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, p, df1, df2, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, df1, df2, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01tdc', tail, 'tail', Nag_TailProbability)
    _p = nag_double_type_check_and_cast('g01tdc', p, 'p')
    _df1 = nag_double_type_check_and_cast('g01tdc', df1, 'df1')
    _df2 = nag_double_type_check_and_cast('g01tdc', df2, 'df2')
    _f = nag_double_type_check_and_cast('g01tdc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('g01tdc', ivalid, 'ivalid')
    g01tdc_ctypes(ltail, _tail, lp, _p, ldf1, _df1, ldf2, _df2, _f, _ivalid, fail)

g01tec_ctypes = _library_cl.g01tec
g01tec_ctypes.restype = None
g01tec_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01tec(ltail, tail, lp, p, la, a, lb, b, tol, beta, ivalid, fail):
    """
    Computes a vector of deviates for the beta distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01tec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, p, a, b, beta, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, a, b, beta
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01tec', tail, 'tail', Nag_TailProbability)
    _p = nag_double_type_check_and_cast('g01tec', p, 'p')
    _a = nag_double_type_check_and_cast('g01tec', a, 'a')
    _b = nag_double_type_check_and_cast('g01tec', b, 'b')
    _beta = nag_double_type_check_and_cast('g01tec', beta, 'beta')
    _ivalid = nag_integer_type_check_and_cast('g01tec', ivalid, 'ivalid')
    g01tec_ctypes(ltail, _tail, lp, _p, la, _a, lb, _b, tol, _beta, _ivalid, fail)

g01tfc_ctypes = _library_cl.g01tfc
g01tfc_ctypes.restype = None
g01tfc_ctypes.argtypes = [Integer, POINTER(Nag_TailProbability), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g01tfc(ltail, tail, lp, p, la, a, lb, b, tol, g, ivalid, fail):
    """
    Computes a vector of deviates for the gamma distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01tfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tail, p, a, b, g, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, a, b, g
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    TypeError will be raised when the following arguments are not of data type
    Nag_TailProbability:
        tail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _tail = nag_enum_type_check_and_cast('g01tfc', tail, 'tail', Nag_TailProbability)
    _p = nag_double_type_check_and_cast('g01tfc', p, 'p')
    _a = nag_double_type_check_and_cast('g01tfc', a, 'a')
    _b = nag_double_type_check_and_cast('g01tfc', b, 'b')
    _g = nag_double_type_check_and_cast('g01tfc', g, 'g')
    _ivalid = nag_integer_type_check_and_cast('g01tfc', ivalid, 'ivalid')
    g01tfc_ctypes(ltail, _tail, lp, _p, la, _a, lb, _b, tol, _g, _ivalid, fail)

g01zuc_ctypes = _library_cl.g01zuc
g01zuc_ctypes.restype = None
g01zuc_ctypes.argtypes = [c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01zuc(rkappa, beta2, mode, xl, xu, commarr, fail):
    """
    Initialization function for g01muc and g01euc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01zuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xl, xu, commarr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xl, xu, commarr
    """
    from .util import nag_double_type_check_and_cast
    _xl = nag_double_type_check_and_cast('g01zuc', xl, 'xl')
    _xu = nag_double_type_check_and_cast('g01zuc', xu, 'xu')
    _commarr = nag_double_type_check_and_cast('g01zuc', commarr, 'commarr')
    g01zuc_ctypes(rkappa, beta2, mode, _xl, _xu, _commarr, fail)

g01wac_ctypes = _library_cl.g01wac
g01wac_ctypes.restype = None
g01wac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Nag_Weightstype, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g01wac(m, nb, x, iwt, wt, pn, rmean, rsd, rcomm, fail):
    """
    Computes the mean and standard deviation using a rolling window.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g01/g01wac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, wt, pn, rmean, rsd, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, wt, rmean, rsd, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        pn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g01wac', x, 'x')
    _wt = nag_double_type_check_and_cast('g01wac', wt, 'wt')
    _pn = nag_integer_type_check_and_cast('g01wac', pn, 'pn')
    _rmean = nag_double_type_check_and_cast('g01wac', rmean, 'rmean')
    _rsd = nag_double_type_check_and_cast('g01wac', rsd, 'rsd')
    _rcomm = nag_double_type_check_and_cast('g01wac', rcomm, 'rcomm')
    g01wac_ctypes(m, nb, _x, iwt, _wt, _pn, _rmean, _rsd, _rcomm, fail)

nag_summary_stats_freq = g01adc
nag_frequency_table = g01aec
nag_5pt_summary_stats = g01alc
nag_double_quantiles = g01amc
nag_approx_quantiles_fixed = g01anc
nag_approx_quantiles_arbitrary = g01apc
nag_summary_stats_onevar = g01atc
nag_summary_stats_onevar_combine = g01auc
nag_binomial_dist = g01bjc
nag_poisson_dist = g01bkc
nag_hypergeom_dist = g01blc
nag_normal_scores_exact = g01dac
nag_normal_scores_var = g01dcc
nag_shapiro_wilk_test = g01ddc
nag_ranks_and_scores = g01dhc
nag_prob_normal = g01eac
nag_prob_students_t = g01ebc
nag_prob_chi_sq = g01ecc
nag_prob_f_dist = g01edc
nag_prob_beta_dist = g01eec
nag_gamma_dist = g01efc
nag_prob_studentized_range = g01emc
nag_prob_durbin_watson = g01epc
nag_prob_von_mises = g01erc
nag_prob_landau = g01etc
nag_prob_vavilov = g01euc
nag_prob_1_sample_ks = g01eyc
nag_prob_2_sample_ks = g01ezc
nag_deviates_normal = g01fac
nag_deviates_students_t = g01fbc
nag_deviates_chi_sq = g01fcc
nag_deviates_f_dist = g01fdc
nag_deviates_beta = g01fec
nag_deviates_gamma_dist = g01ffc
nag_deviates_studentized_range = g01fmc
nag_deviates_landau = g01ftc
nag_prob_non_central_students_t = g01gbc
nag_prob_non_central_chi_sq = g01gcc
nag_prob_non_central_f_dist = g01gdc
nag_prob_non_central_beta_dist = g01gec
nag_bivariate_normal_dist = g01hac
nag_multi_normal = g01hbc
nag_bivariate_students_t = g01hcc
nag_multi_students_t = g01hdc
nag_prob_lin_non_central_chi_sq = g01jcc
nag_prob_lin_chi_sq = g01jdc
nag_normal_pdf = g01kac
nag_gamma_pdf = g01kfc
nag_gamma_pdf_vector = g01kkc
nag_normal_pdf_vector = g01kqc
nag_multi_normal_pdf_vector = g01lbc
nag_mills_ratio = g01mbc
nag_prob_density_landau = g01mtc
nag_prob_density_vavilov = g01muc
nag_moments_quad_form = g01nac
nag_moments_ratio_quad_forms = g01nbc
nag_moment_1_landau = g01ptc
nag_moment_2_landau = g01qtc
nag_prob_der_landau = g01rtc
nag_prob_normal_vector = g01sac
nag_prob_students_t_vector = g01sbc
nag_prob_chi_sq_vector = g01scc
nag_prob_f_vector = g01sdc
nag_prob_beta_vector = g01sec
nag_prob_gamma_vector = g01sfc
nag_prob_binomial_vector = g01sjc
nag_prob_poisson_vector = g01skc
nag_prob_hypergeom_vector = g01slc
nag_deviates_normal_vector = g01tac
nag_deviates_students_t_vector = g01tbc
nag_deviates_chi_sq_vector = g01tcc
nag_deviates_f_vector = g01tdc
nag_deviates_beta_vector = g01tec
nag_deviates_gamma_vector = g01tfc
nag_moving_average = g01wac
nag_init_vavilov = g01zuc
__all__ = ['g01adc', 'g01aec', 'g01alc',
           'g01amc', 'g01anc', 'g01apc', 'g01atc', 'g01auc', 'g01bjc',
           'g01bkc', 'g01blc', 'g01dac', 'g01dcc', 'g01ddc',
           'g01dhc', 'g01eac', 'g01ebc', 'g01ecc', 'g01edc', 'g01eec',
           'g01efc', 'g01emc', 'g01epc', 'g01erc', 'g01etc', 'g01euc',
           'g01eyc', 'g01ezc', 'g01fac', 'g01fbc', 'g01fcc',
           'g01fdc', 'g01fec', 'g01ffc', 'g01fmc', 'g01ftc', 'g01gbc',
           'g01gcc', 'g01gdc', 'g01gec', 'g01hac', 'g01hbc', 'g01hcc',
           'g01hdc', 'g01jcc', 'g01jdc', 'g01kac', 'g01kfc', 'g01kkc',
           'g01kqc', 'g01lbc', 'g01mbc', 'g01mtc', 'g01muc', 'g01nac',
           'g01nbc', 'g01ptc', 'g01qtc', 'g01rtc', 'g01sac', 'g01sbc',
           'g01scc', 'g01sdc', 'g01sec', 'g01sfc', 'g01sjc', 'g01skc',
           'g01slc', 'g01tac', 'g01tbc', 'g01tcc', 'g01tdc', 'g01tec',
           'g01tfc', 'g01wac', 'g01zuc', 'nag_5pt_summary_stats',
           'nag_approx_quantiles_arbitrary',
           'nag_approx_quantiles_fixed', 'nag_binomial_dist',
           'nag_bivariate_normal_dist', 'nag_bivariate_students_t',
           'nag_deviates_beta', 'nag_deviates_beta_vector',
           'nag_deviates_chi_sq', 'nag_deviates_chi_sq_vector',
           'nag_deviates_f_dist', 'nag_deviates_f_vector',
           'nag_deviates_gamma_dist', 'nag_deviates_gamma_vector',
           'nag_deviates_landau', 'nag_deviates_normal',
           'nag_deviates_normal_vector',
           'nag_deviates_studentized_range',
           'nag_deviates_students_t',
           'nag_deviates_students_t_vector', 'nag_double_quantiles',
           'nag_frequency_table', 'nag_gamma_dist', 'nag_gamma_pdf',
           'nag_gamma_pdf_vector', 'nag_hypergeom_dist',
           'nag_init_vavilov', 'nag_mills_ratio',
           'nag_moment_1_landau', 'nag_moment_2_landau',
           'nag_moments_quad_form', 'nag_moments_ratio_quad_forms',
           'nag_moving_average', 'nag_multi_normal',
           'nag_multi_normal_pdf_vector', 'nag_multi_students_t',
           'nag_normal_pdf', 'nag_normal_pdf_vector',
           'nag_normal_scores_exact',
           'nag_normal_scores_var', 'nag_poisson_dist',
           'nag_prob_1_sample_ks', 'nag_prob_2_sample_ks',
           'nag_prob_beta_dist', 'nag_prob_beta_vector',
           'nag_prob_binomial_vector', 'nag_prob_chi_sq',
           'nag_prob_chi_sq_vector', 'nag_prob_density_landau',
           'nag_prob_density_vavilov', 'nag_prob_der_landau',
           'nag_prob_durbin_watson',
           'nag_prob_f_dist', 'nag_prob_f_vector',
           'nag_prob_gamma_vector', 'nag_prob_hypergeom_vector',
           'nag_prob_landau', 'nag_prob_lin_chi_sq',
           'nag_prob_lin_non_central_chi_sq',
           'nag_prob_non_central_beta_dist',
           'nag_prob_non_central_chi_sq',
           'nag_prob_non_central_f_dist',
           'nag_prob_non_central_students_t', 'nag_prob_normal',
           'nag_prob_normal_vector', 'nag_prob_poisson_vector',
           'nag_prob_studentized_range', 'nag_prob_students_t',
           'nag_prob_students_t_vector', 'nag_prob_vavilov',
           'nag_prob_von_mises', 'nag_ranks_and_scores',
           'nag_shapiro_wilk_test',
           'nag_summary_stats_freq', 'nag_summary_stats_onevar',
           'nag_summary_stats_onevar_combine']
