"""
Deprecated wrappers for the NAG C Library (Mark 26.0) g13 Chapter.

g13 - Time Series Analysis.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/G13/g13conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from ctypes import c_char_p
from .util import Nag_TS_ChangeType
from .util import get_library
from .util import Nag_TS_URTestType
from .util import NAG_G13EKC_F
from .util import NAG_G13EKC_H
from .util import Nag_Comm
from .util import NAG_G13NBC_CONSTFN
from .util import Integer
from .util import NagError
from .util import Nag_InitialValues
from .util import Nag_ExpSmoothType
from .util import Nag_ArimaOrder
from .util import Nag_RangeStat
from .util import Nag_TransfOrder
from .util import Nag_G13_Opt
from .util import Nag_LoggedSpectra
from .util import NagMeanOrTrend
from .util import Complex
from .util import Nag_IncludeMean
from .util import Nag_Boolean
from .util import NAG_G13NEC_CHGPFN
from .util import Nag_CovOrCorr
from .util import Nag_ab_input
from .util import Nag_ObserverForm
from .util import Nag_ControllerForm
from .util import Nag_Garch_Stationary_Type
from .util import Nag_Garch_Est_Initial_Type
from .util import Nag_TS_Interpolation
from .util import Nag_OrderType
from .util import Nag_TS_Transform

STRING = c_char_p

_library_cl = get_library()

g13aac_ctypes = _library_cl.g13aac
g13aac_ctypes.restype = None
g13aac_ctypes.argtypes = [POINTER(c_double), Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g13aac(x, nx, d, ds, s, xd, nxd, fail):
    """
    Univariate time series, seasonal and non-seasonal differencing.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, xd, nxd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, xd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nxd
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g13aac', x, 'x')
    _xd = nag_double_type_check_and_cast('g13aac', xd, 'xd')
    _nxd = nag_integer_type_check_and_cast('g13aac', nxd, 'nxd')
    g13aac_ctypes(_x, nx, d, ds, s, _xd, _nxd, fail)

g13abc_ctypes = _library_cl.g13abc
g13abc_ctypes.restype = None
g13abc_ctypes.argtypes = [POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13abc(x, nx, nk, mean, var, r, stat, fail):
    """
    Sample autocorrelation function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, mean, var, r, stat
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, mean, var, r, stat
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g13abc', x, 'x')
    _mean = nag_double_type_check_and_cast('g13abc', mean, 'mean')
    _var = nag_double_type_check_and_cast('g13abc', var, 'var')
    _r = nag_double_type_check_and_cast('g13abc', r, 'r')
    _stat = nag_double_type_check_and_cast('g13abc', stat, 'stat')
    g13abc_ctypes(_x, nx, nk, _mean, _var, _r, _stat, fail)

g13acc_ctypes = _library_cl.g13acc
g13acc_ctypes.restype = None
g13acc_ctypes.argtypes = [POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g13acc(r, nk, nl, p, v, ar, nvl, fail):
    """
    Partial autocorrelation function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13acc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        r, p, v, ar, nvl
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r, p, v, ar
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nvl
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _r = nag_double_type_check_and_cast('g13acc', r, 'r')
    _p = nag_double_type_check_and_cast('g13acc', p, 'p')
    _v = nag_double_type_check_and_cast('g13acc', v, 'v')
    _ar = nag_double_type_check_and_cast('g13acc', ar, 'ar')
    _nvl = nag_integer_type_check_and_cast('g13acc', nvl, 'nvl')
    g13acc_ctypes(_r, nk, nl, _p, _v, _ar, _nvl, fail)

g13amc_ctypes = _library_cl.g13amc
g13amc_ctypes.restype = None
g13amc_ctypes.argtypes = [Nag_InitialValues, Nag_ExpSmoothType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13amc(mode, itype, p, param, n, y, k, init, nf, fv, fse, yhat, res, dv, ad, r, fail):
    """
    Univariate time series, exponential smoothing.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13amc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        param, y, init, fv, fse, yhat, res, dv, ad, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        param, y, init, fv, fse, yhat, res, dv, ad, r
    """
    from .util import nag_double_type_check_and_cast
    _param = nag_double_type_check_and_cast('g13amc', param, 'param')
    _y = nag_double_type_check_and_cast('g13amc', y, 'y')
    _init = nag_double_type_check_and_cast('g13amc', init, 'init')
    _fv = nag_double_type_check_and_cast('g13amc', fv, 'fv')
    _fse = nag_double_type_check_and_cast('g13amc', fse, 'fse')
    _yhat = nag_double_type_check_and_cast('g13amc', yhat, 'yhat')
    _res = nag_double_type_check_and_cast('g13amc', res, 'res')
    _dv = nag_double_type_check_and_cast('g13amc', dv, 'dv')
    _ad = nag_double_type_check_and_cast('g13amc', ad, 'ad')
    _r = nag_double_type_check_and_cast('g13amc', r, 'r')
    g13amc_ctypes(mode, itype, p, _param, n, _y, k, _init, nf, _fv, _fse, _yhat, _res, _dv, _ad, _r, fail)

g13asc_ctypes = _library_cl.g13asc
g13asc_ctypes.restype = None
g13asc_ctypes.argtypes = [POINTER(Nag_ArimaOrder), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13asc(arimav, n, v, m, par, narma, r, rc, tdrc, chi, df, siglev, fail):
    """
    Univariate time series, diagnostic checking of residuals, following
    g13bec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13asc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        v, par, r, rc, chi, df, siglev
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        v, par, r, rc, chi, siglev
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        df
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _v = nag_double_type_check_and_cast('g13asc', v, 'v')
    _par = nag_double_type_check_and_cast('g13asc', par, 'par')
    _r = nag_double_type_check_and_cast('g13asc', r, 'r')
    _rc = nag_double_type_check_and_cast('g13asc', rc, 'rc')
    _chi = nag_double_type_check_and_cast('g13asc', chi, 'chi')
    _df = nag_integer_type_check_and_cast('g13asc', df, 'df')
    _siglev = nag_double_type_check_and_cast('g13asc', siglev, 'siglev')
    g13asc_ctypes(arimav, n, _v, m, _par, narma, _r, _rc, tdrc, _chi, _df, _siglev, fail)

g13auc_ctypes = _library_cl.g13auc
g13auc_ctypes.restype = None
g13auc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, Nag_RangeStat, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13auc(n, z, m, rs, y, mean, fail):
    """
    Computes quantities needed for range-mean or standard deviation-mean
    plot.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13auc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        z, y, mean
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        z, y, mean
    """
    from .util import nag_double_type_check_and_cast
    _z = nag_double_type_check_and_cast('g13auc', z, 'z')
    _y = nag_double_type_check_and_cast('g13auc', y, 'y')
    _mean = nag_double_type_check_and_cast('g13auc', mean, 'mean')
    g13auc_ctypes(n, _z, m, rs, _y, _mean, fail)

g13awc_ctypes = _library_cl.g13awc
g13awc_ctypes.restype = c_double
g13awc_ctypes.argtypes = [Nag_TS_URTestType, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def g13awc(_type, p, n, y, fail):
    """
    Computes (augmented) Dickey--Fuller unit root test statistic.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13awc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g13awc', y, 'y')
    return_val = g13awc_ctypes(_type, p, n, _y, fail)
    return return_val

nag_tsa_dickey_fuller_unit = g13awc

g13bac_ctypes = _library_cl.g13bac
g13bac_ctypes.restype = None
g13bac_ctypes.argtypes = [POINTER(c_double), Integer, POINTER(Nag_ArimaOrder), POINTER(Nag_ArimaOrder), POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def g13bac(y, ny, arimaf, arimav, par, npar, cy, b, nb, fail):
    """
    Multivariate time series, filtering (pre-whitening) by an ARIMA model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, par, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, par, b
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g13bac', y, 'y')
    _par = nag_double_type_check_and_cast('g13bac', par, 'par')
    _b = nag_double_type_check_and_cast('g13bac', b, 'b')
    g13bac_ctypes(_y, ny, arimaf, arimav, _par, npar, cy, _b, nb, fail)

g13bbc_ctypes = _library_cl.g13bbc
g13bbc_ctypes.restype = None
g13bbc_ctypes.argtypes = [POINTER(c_double), Integer, POINTER(Nag_TransfOrder), POINTER(Nag_ArimaOrder), POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def g13bbc(y, ny, transfv, arimas, par, npar, cy, b, nb, fail):
    """
    Multivariate time series, filtering by a transfer function model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, par, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, par, b
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g13bbc', y, 'y')
    _par = nag_double_type_check_and_cast('g13bbc', par, 'par')
    _b = nag_double_type_check_and_cast('g13bbc', b, 'b')
    g13bbc_ctypes(_y, ny, transfv, arimas, _par, npar, cy, _b, nb, fail)

g13bcc_ctypes = _library_cl.g13bcc
g13bcc_ctypes.restype = None
g13bcc_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13bcc(x, y, nxy, nl, s, r0, r, stat, fail):
    """
    Multivariate time series, cross-correlations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, s, r0, r, stat
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, s, r0, r, stat
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g13bcc', x, 'x')
    _y = nag_double_type_check_and_cast('g13bcc', y, 'y')
    _s = nag_double_type_check_and_cast('g13bcc', s, 's')
    _r0 = nag_double_type_check_and_cast('g13bcc', r0, 'r0')
    _r = nag_double_type_check_and_cast('g13bcc', r, 'r')
    _stat = nag_double_type_check_and_cast('g13bcc', stat, 'stat')
    g13bcc_ctypes(_x, _y, nxy, nl, _s, _r0, _r, _stat, fail)

g13bdc_ctypes = _library_cl.g13bdc
g13bdc_ctypes.restype = None
g13bdc_ctypes.argtypes = [c_double, POINTER(c_double), Integer, POINTER(Nag_TransfOrder), c_double, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g13bdc(r0, r, nl, transfv, s, wds, isf, fail):
    """
    Multivariate time series, preliminary estimation of transfer function
    model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        r, wds, isf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r, wds
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isf
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _r = nag_double_type_check_and_cast('g13bdc', r, 'r')
    _wds = nag_double_type_check_and_cast('g13bdc', wds, 'wds')
    _isf = nag_integer_type_check_and_cast('g13bdc', isf, 'isf')
    g13bdc_ctypes(r0, _r, nl, transfv, s, _wds, _isf, fail)

g13bec_ctypes = _library_cl.g13bec
g13bec_ctypes.restype = None
g13bec_ctypes.argtypes = [POINTER(Nag_ArimaOrder), Integer, POINTER(Nag_TransfOrder), POINTER(c_double), Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_G13_Opt), POINTER(NagError)]


def g13bec(arimav, nseries, transfv, para, npara, nxxy, xxy, tdxxy, sd, rss, objf, df, options, fail):
    """
    Estimation for time series models.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        para, xxy, sd, rss, objf, df
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        para, xxy, sd, rss, objf, df
    """
    from .util import nag_double_type_check_and_cast
    _para = nag_double_type_check_and_cast('g13bec', para, 'para')
    _xxy = nag_double_type_check_and_cast('g13bec', xxy, 'xxy')
    _sd = nag_double_type_check_and_cast('g13bec', sd, 'sd')
    _rss = nag_double_type_check_and_cast('g13bec', rss, 'rss')
    _objf = nag_double_type_check_and_cast('g13bec', objf, 'objf')
    _df = nag_double_type_check_and_cast('g13bec', df, 'df')
    g13bec_ctypes(arimav, nseries, transfv, _para, npara, nxxy, _xxy, tdxxy, _sd, _rss, _objf, _df, options, fail)

g13bgc_ctypes = _library_cl.g13bgc
g13bgc_ctypes.restype = None
g13bgc_ctypes.argtypes = [POINTER(Nag_ArimaOrder), Integer, POINTER(Nag_TransfOrder), POINTER(c_double), Integer, Integer, POINTER(c_double), Integer, Integer, POINTER(Nag_G13_Opt), POINTER(NagError)]


def g13bgc(arimav, nser, transfv, para, npara, nnv, xxyn, tdxxyn, kzef, options, fail):
    """
    Multivariate time series, update state set for forecasting from multi-
    input model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        para, xxyn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        para, xxyn
    """
    from .util import nag_double_type_check_and_cast
    _para = nag_double_type_check_and_cast('g13bgc', para, 'para')
    _xxyn = nag_double_type_check_and_cast('g13bgc', xxyn, 'xxyn')
    g13bgc_ctypes(arimav, nser, transfv, _para, npara, nnv, _xxyn, tdxxyn, kzef, options, fail)

g13bjc_ctypes = _library_cl.g13bjc
g13bjc_ctypes.restype = None
g13bjc_ctypes.argtypes = [POINTER(Nag_ArimaOrder), Integer, POINTER(Nag_TransfOrder), POINTER(c_double), Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_G13_Opt), POINTER(NagError)]


def g13bjc(arimav, nseries, transfv, para, npara, nev, nfv, xxy, tdxxy, rmsxy, mrx, tdmrx, parx, ldparx, tdparx, fva, fsd, options, fail):
    """
    Forecasting function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        para, xxy, rmsxy, mrx, parx, fva, fsd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        para, xxy, rmsxy, parx, fva, fsd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        mrx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _para = nag_double_type_check_and_cast('g13bjc', para, 'para')
    _xxy = nag_double_type_check_and_cast('g13bjc', xxy, 'xxy')
    _rmsxy = nag_double_type_check_and_cast('g13bjc', rmsxy, 'rmsxy')
    _mrx = nag_integer_type_check_and_cast('g13bjc', mrx, 'mrx')
    _parx = nag_double_type_check_and_cast('g13bjc', parx, 'parx')
    _fva = nag_double_type_check_and_cast('g13bjc', fva, 'fva')
    _fsd = nag_double_type_check_and_cast('g13bjc', fsd, 'fsd')
    g13bjc_ctypes(arimav, nseries, transfv, _para, npara, nev, nfv, _xxy, tdxxy, _rmsxy, _mrx, tdmrx, _parx, ldparx, tdparx, _fva, _fsd, options, fail)

g13bxc_ctypes = _library_cl.g13bxc
g13bxc_ctypes.restype = None
g13bxc_ctypes.argtypes = [POINTER(Nag_G13_Opt)]


def g13bxc(options):
    """
    Initialization function for option setting.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bxc.html
    """
    g13bxc_ctypes(options)

g13byc_ctypes = _library_cl.g13byc
g13byc_ctypes.restype = None
g13byc_ctypes.argtypes = [Integer, POINTER(Nag_TransfOrder), POINTER(NagError)]


def g13byc(nseries, transfv, fail):
    """
    Allocates memory to transfer function model orders.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13byc.html
    """
    g13byc_ctypes(nseries, transfv, fail)

g13bzc_ctypes = _library_cl.g13bzc
g13bzc_ctypes.restype = None
g13bzc_ctypes.argtypes = [POINTER(Nag_TransfOrder)]


def g13bzc(transfv):
    """
    Freeing function for the structure holding the transfer function model
    orders.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13bzc.html
    """
    g13bzc_ctypes(transfv)

g13cac_ctypes = _library_cl.g13cac
g13cac_ctypes.restype = None
g13cac_ctypes.argtypes = [Integer, Integer, c_double, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, Integer, Nag_LoggedSpectra, Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13cac(nx, mtx, px, iw, mw, ic, nc, c, kc, l, lgspect, nxg, xg, ng, stats, fail):
    """
    Univariate time series, smoothed sample spectrum using rectangular,
    Bartlett, Tukey or Parzen lag window.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        c, xg, ng, stats
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c, xg, stats
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ng
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _c = nag_double_type_check_and_cast('g13cac', c, 'c')
    _xg = nag_double_type_check_and_cast('g13cac', xg, 'xg')
    _ng = nag_integer_type_check_and_cast('g13cac', ng, 'ng')
    _stats = nag_double_type_check_and_cast('g13cac', stats, 'stats')
    g13cac_ctypes(nx, mtx, px, iw, mw, ic, nc, _c, kc, l, lgspect, nxg, _xg, _ng, _stats, fail)

g13cbc_ctypes = _library_cl.g13cbc
g13cbc_ctypes.restype = None
g13cbc_ctypes.argtypes = [Integer, NagMeanOrTrend, c_double, Integer, c_double, Integer, Integer, Nag_LoggedSpectra, POINTER(c_double), POINTER(POINTER(c_double)), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13cbc(nx, mtcorrection, px, mw, pw, l, kc, lgspect, x, g, ng, stats, fail):
    """
    Univariate time series, smoothed sample spectrum using spectral
    smoothing by the trapezium frequency (Daniell) window.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, g, ng, stats
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, g, stats
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ng
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g13cbc', x, 'x')
    _g = nag_double_type_check_and_cast('g13cbc', g, 'g')
    _ng = nag_integer_type_check_and_cast('g13cbc', ng, 'ng')
    _stats = nag_double_type_check_and_cast('g13cbc', stats, 'stats')
    g13cbc_ctypes(nx, mtcorrection, px, mw, pw, l, kc, lgspect, _x, _g, _ng, _stats, fail)

g13ccc_ctypes = _library_cl.g13ccc
g13ccc_ctypes.restype = None
g13ccc_ctypes.argtypes = [Integer, NagMeanOrTrend, c_double, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), POINTER(Integer), POINTER(NagError)]


def g13ccc(nxy, mtxycorrection, pxy, iw, mw, ish, ic, nc, cxy, cyx, kc, l, xg, yg, g, ng, fail):
    """
    Multivariate time series, smoothed sample cross spectrum using
    rectangular, Bartlett, Tukey or Parzen lag window.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ccc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cxy, cyx, xg, yg, g, ng
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        cxy, cyx, xg, yg
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ng
    TypeError will be raised when the following arguments are not of data type
    Complex:
        g
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cxy = nag_double_type_check_and_cast('g13ccc', cxy, 'cxy')
    _cyx = nag_double_type_check_and_cast('g13ccc', cyx, 'cyx')
    _xg = nag_double_type_check_and_cast('g13ccc', xg, 'xg')
    _yg = nag_double_type_check_and_cast('g13ccc', yg, 'yg')
    _g = nag_complex_type_check_and_cast('g13ccc', g, 'g')
    _ng = nag_integer_type_check_and_cast('g13ccc', ng, 'ng')
    g13ccc_ctypes(nxy, mtxycorrection, pxy, iw, mw, ish, ic, nc, _cxy, _cyx, kc, l, _xg, _yg, _g, _ng, fail)

g13cdc_ctypes = _library_cl.g13cdc
g13cdc_ctypes.restype = None
g13cdc_ctypes.argtypes = [Integer, NagMeanOrTrend, c_double, Integer, Integer, c_double, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(POINTER(Complex)), POINTER(Integer), POINTER(NagError)]


def g13cdc(nxy, mtcorrection, pxy, mw, _is, pw, l, kc, x, y, g, ng, fail):
    """
    Multivariate time series, smoothed sample cross spectrum using spectral
    smoothing by the trapezium frequency (Daniell) window.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13cdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, g, ng
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ng
    TypeError will be raised when the following arguments are not of data type
    Complex:
        g
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g13cdc', x, 'x')
    _y = nag_double_type_check_and_cast('g13cdc', y, 'y')
    _g = nag_complex_type_check_and_cast('g13cdc', g, 'g')
    _ng = nag_integer_type_check_and_cast('g13cdc', ng, 'ng')
    g13cdc_ctypes(nxy, mtcorrection, pxy, mw, _is, pw, l, kc, _x, _y, _g, _ng, fail)

g13cec_ctypes = _library_cl.g13cec
g13cec_ctypes.restype = None
g13cec_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13cec(xg, yg, xyg, ng, stats, ca, calw, caup, t, sc, sclw, scup, fail):
    """
    Multivariate time series, cross amplitude spectrum, squared coherency,
    bounds, univariate and bivariate (cross) spectra.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13cec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xg, yg, xyg, stats, ca, calw, caup, t, sc, sclw, scup
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xg, yg, stats, ca, calw, caup, t, sc, sclw, scup
    TypeError will be raised when the following arguments are not of data type
    Complex:
        xyg
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _xg = nag_double_type_check_and_cast('g13cec', xg, 'xg')
    _yg = nag_double_type_check_and_cast('g13cec', yg, 'yg')
    _xyg = nag_complex_type_check_and_cast('g13cec', xyg, 'xyg')
    _stats = nag_double_type_check_and_cast('g13cec', stats, 'stats')
    _ca = nag_double_type_check_and_cast('g13cec', ca, 'ca')
    _calw = nag_double_type_check_and_cast('g13cec', calw, 'calw')
    _caup = nag_double_type_check_and_cast('g13cec', caup, 'caup')
    _t = nag_double_type_check_and_cast('g13cec', t, 't')
    _sc = nag_double_type_check_and_cast('g13cec', sc, 'sc')
    _sclw = nag_double_type_check_and_cast('g13cec', sclw, 'sclw')
    _scup = nag_double_type_check_and_cast('g13cec', scup, 'scup')
    g13cec_ctypes(_xg, _yg, _xyg, ng, _stats, _ca, _calw, _caup, _t, _sc, _sclw, _scup, fail)

g13cfc_ctypes = _library_cl.g13cfc
g13cfc_ctypes.restype = None
g13cfc_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13cfc(xg, yg, xyg, ng, stats, gn, gnlw, gnup, ph, phlw, phup, fail):
    """
    Multivariate time series, gain, phase, bounds, univariate and bivariate
    (cross) spectra.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13cfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xg, yg, xyg, stats, gn, gnlw, gnup, ph, phlw, phup
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xg, yg, stats, gn, gnlw, gnup, ph, phlw, phup
    TypeError will be raised when the following arguments are not of data type
    Complex:
        xyg
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _xg = nag_double_type_check_and_cast('g13cfc', xg, 'xg')
    _yg = nag_double_type_check_and_cast('g13cfc', yg, 'yg')
    _xyg = nag_complex_type_check_and_cast('g13cfc', xyg, 'xyg')
    _stats = nag_double_type_check_and_cast('g13cfc', stats, 'stats')
    _gn = nag_double_type_check_and_cast('g13cfc', gn, 'gn')
    _gnlw = nag_double_type_check_and_cast('g13cfc', gnlw, 'gnlw')
    _gnup = nag_double_type_check_and_cast('g13cfc', gnup, 'gnup')
    _ph = nag_double_type_check_and_cast('g13cfc', ph, 'ph')
    _phlw = nag_double_type_check_and_cast('g13cfc', phlw, 'phlw')
    _phup = nag_double_type_check_and_cast('g13cfc', phup, 'phup')
    g13cfc_ctypes(_xg, _yg, _xyg, ng, _stats, _gn, _gnlw, _gnup, _ph, _phlw, _phup, fail)

g13cgc_ctypes = _library_cl.g13cgc
g13cgc_ctypes.restype = None
g13cgc_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13cgc(xg, yg, xyg, ng, stats, l, n, er, erlw, erup, rf, rfse, fail):
    """
    Multivariate time series, noise spectrum, bounds, impulse response
    function and its standard error.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13cgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xg, yg, xyg, stats, er, erlw, erup, rf, rfse
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xg, yg, stats, er, erlw, erup, rf, rfse
    TypeError will be raised when the following arguments are not of data type
    Complex:
        xyg
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _xg = nag_double_type_check_and_cast('g13cgc', xg, 'xg')
    _yg = nag_double_type_check_and_cast('g13cgc', yg, 'yg')
    _xyg = nag_complex_type_check_and_cast('g13cgc', xyg, 'xyg')
    _stats = nag_double_type_check_and_cast('g13cgc', stats, 'stats')
    _er = nag_double_type_check_and_cast('g13cgc', er, 'er')
    _erlw = nag_double_type_check_and_cast('g13cgc', erlw, 'erlw')
    _erup = nag_double_type_check_and_cast('g13cgc', erup, 'erup')
    _rf = nag_double_type_check_and_cast('g13cgc', rf, 'rf')
    _rfse = nag_double_type_check_and_cast('g13cgc', rfse, 'rfse')
    g13cgc_ctypes(_xg, _yg, _xyg, ng, _stats, l, n, _er, _erlw, _erup, _rf, _rfse, fail)

g13dbc_ctypes = _library_cl.g13dbc
g13dbc_ctypes.restype = None
g13dbc_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g13dbc(c0, c, ns, nl, nk, p, v0, v, d, db, w, wb, nvp, fail):
    """
    Multivariate time series, multiple squared partial autocorrelations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        c0, c, p, v0, v, d, db, w, wb, nvp
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c0, c, p, v0, v, d, db, w, wb
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nvp
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _c0 = nag_double_type_check_and_cast('g13dbc', c0, 'c0')
    _c = nag_double_type_check_and_cast('g13dbc', c, 'c')
    _p = nag_double_type_check_and_cast('g13dbc', p, 'p')
    _v0 = nag_double_type_check_and_cast('g13dbc', v0, 'v0')
    _v = nag_double_type_check_and_cast('g13dbc', v, 'v')
    _d = nag_double_type_check_and_cast('g13dbc', d, 'd')
    _db = nag_double_type_check_and_cast('g13dbc', db, 'db')
    _w = nag_double_type_check_and_cast('g13dbc', w, 'w')
    _wb = nag_double_type_check_and_cast('g13dbc', wb, 'wb')
    _nvp = nag_integer_type_check_and_cast('g13dbc', nvp, 'nvp')
    g13dbc_ctypes(_c0, _c, ns, nl, nk, _p, _v0, _v, _d, _db, _w, _wb, _nvp, fail)

g13ddc_ctypes = _library_cl.g13ddc
g13ddc_ctypes.restype = None
g13ddc_ctypes.argtypes = [Integer, Integer, Integer, Integer, Nag_IncludeMean, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Nag_Boolean), Nag_Boolean, Integer, c_double, Integer, Integer, STRING, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g13ddc(k, n, ip, iq, mean, par, npar, qq, kmax, w, parhld, exact, iprint, cgetol, maxcal, ishow, outfile, niter, rlogl, v, g, cm, pdcm, fail):
    """
    Multivariate time series, estimation of VARMA model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ddc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        par, qq, w, parhld, niter, rlogl, v, g, cm
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        parhld
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        par, qq, w, rlogl, v, g, cm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        niter
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _par = nag_double_type_check_and_cast('g13ddc', par, 'par')
    _qq = nag_double_type_check_and_cast('g13ddc', qq, 'qq')
    _w = nag_double_type_check_and_cast('g13ddc', w, 'w')
    _parhld = nag_enum_type_check_and_cast('g13ddc', parhld, 'parhld', Nag_Boolean)
    _niter = nag_integer_type_check_and_cast('g13ddc', niter, 'niter')
    _rlogl = nag_double_type_check_and_cast('g13ddc', rlogl, 'rlogl')
    _v = nag_double_type_check_and_cast('g13ddc', v, 'v')
    _g = nag_double_type_check_and_cast('g13ddc', g, 'g')
    _cm = nag_double_type_check_and_cast('g13ddc', cm, 'cm')
    g13ddc_ctypes(k, n, ip, iq, mean, _par, npar, _qq, kmax, _w, _parhld, exact, iprint, cgetol, maxcal, ishow, outfile, _niter, _rlogl, _v, _g, _cm, pdcm, fail)

g13djc_ctypes = _library_cl.g13djc
g13djc_ctypes.restype = None
g13djc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, Integer, Nag_IncludeMean, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g13djc(k, n, z, kmax, tr, _id, delta, ip, iq, mean, par, lpar, qq, v, lmax, predz, sefz, ref, lref, fail):
    """
    Multivariate time series, forecasts and their standard errors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13djc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        z, tr, _id, delta, par, qq, v, predz, sefz, ref
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        z, delta, par, qq, v, predz, sefz, ref
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        tr, _id
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _z = nag_double_type_check_and_cast('g13djc', z, 'z')
    _tr = nag_integer_type_check_and_cast('g13djc', tr, 'tr')
    __id = nag_integer_type_check_and_cast('g13djc', _id, '_id')
    _delta = nag_double_type_check_and_cast('g13djc', delta, 'delta')
    _par = nag_double_type_check_and_cast('g13djc', par, 'par')
    _qq = nag_double_type_check_and_cast('g13djc', qq, 'qq')
    _v = nag_double_type_check_and_cast('g13djc', v, 'v')
    _predz = nag_double_type_check_and_cast('g13djc', predz, 'predz')
    _sefz = nag_double_type_check_and_cast('g13djc', sefz, 'sefz')
    _ref = nag_double_type_check_and_cast('g13djc', ref, 'ref')
    g13djc_ctypes(k, n, _z, kmax, _tr, __id, _delta, ip, iq, mean, _par, lpar, _qq, _v, lmax, _predz, _sefz, _ref, lref, fail)

g13dkc_ctypes = _library_cl.g13dkc
g13dkc_ctypes.restype = None
g13dkc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13dkc(k, lmax, m, mlast, z, kmax, ref, lref, v, predz, sefz, fail):
    """
    Multivariate time series, updates forecasts and their standard errors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        mlast, z, ref, v, predz, sefz
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        z, ref, v, predz, sefz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        mlast
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _mlast = nag_integer_type_check_and_cast('g13dkc', mlast, 'mlast')
    _z = nag_double_type_check_and_cast('g13dkc', z, 'z')
    _ref = nag_double_type_check_and_cast('g13dkc', ref, 'ref')
    _v = nag_double_type_check_and_cast('g13dkc', v, 'v')
    _predz = nag_double_type_check_and_cast('g13dkc', predz, 'predz')
    _sefz = nag_double_type_check_and_cast('g13dkc', sefz, 'sefz')
    g13dkc_ctypes(k, lmax, m, _mlast, _z, kmax, _ref, lref, _v, _predz, _sefz, fail)

g13dlc_ctypes = _library_cl.g13dlc
g13dlc_ctypes.restype = None
g13dlc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g13dlc(k, n, z, tr, _id, delta, w, nd, fail):
    """
    Multivariate time series, differences and/or transforms.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        z, tr, _id, delta, w, nd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        z, delta, w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        tr, _id, nd
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _z = nag_double_type_check_and_cast('g13dlc', z, 'z')
    _tr = nag_integer_type_check_and_cast('g13dlc', tr, 'tr')
    __id = nag_integer_type_check_and_cast('g13dlc', _id, '_id')
    _delta = nag_double_type_check_and_cast('g13dlc', delta, 'delta')
    _w = nag_double_type_check_and_cast('g13dlc', w, 'w')
    _nd = nag_integer_type_check_and_cast('g13dlc', nd, 'nd')
    g13dlc_ctypes(k, n, _z, _tr, __id, _delta, _w, _nd, fail)

g13dmc_ctypes = _library_cl.g13dmc
g13dmc_ctypes.restype = None
g13dmc_ctypes.argtypes = [Nag_CovOrCorr, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13dmc(matrix, k, n, m, w, wmean, r0, r, fail):
    """
    Multivariate time series, sample cross-correlation or cross-covariance
    matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        w, wmean, r0, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w, wmean, r0, r
    """
    from .util import nag_double_type_check_and_cast
    _w = nag_double_type_check_and_cast('g13dmc', w, 'w')
    _wmean = nag_double_type_check_and_cast('g13dmc', wmean, 'wmean')
    _r0 = nag_double_type_check_and_cast('g13dmc', r0, 'r0')
    _r = nag_double_type_check_and_cast('g13dmc', r, 'r')
    g13dmc_ctypes(matrix, k, n, m, _w, _wmean, _r0, _r, fail)

g13dnc_ctypes = _library_cl.g13dnc
g13dnc_ctypes.restype = None
g13dnc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13dnc(k, n, m, r0, r, maxlag, parlag, x, pvalue, fail):
    """
    Multivariate time series, sample partial lag correlation matrices,
    chi**2 statistics and significance levels.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        r0, r, maxlag, parlag, x, pvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r0, r, parlag, x, pvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        maxlag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _r0 = nag_double_type_check_and_cast('g13dnc', r0, 'r0')
    _r = nag_double_type_check_and_cast('g13dnc', r, 'r')
    _maxlag = nag_integer_type_check_and_cast('g13dnc', maxlag, 'maxlag')
    _parlag = nag_double_type_check_and_cast('g13dnc', parlag, 'parlag')
    _x = nag_double_type_check_and_cast('g13dnc', x, 'x')
    _pvalue = nag_double_type_check_and_cast('g13dnc', pvalue, 'pvalue')
    g13dnc_ctypes(k, n, m, _r0, _r, _maxlag, _parlag, _x, _pvalue, fail)

g13dpc_ctypes = _library_cl.g13dpc
g13dpc_ctypes.restype = None
g13dpc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13dpc(k, n, z, m, maxlag, parlag, se, qq, x, pvalue, loglhd, fail):
    """
    Multivariate time series, partial autoregression matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        z, maxlag, parlag, se, qq, x, pvalue, loglhd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        z, parlag, se, qq, x, pvalue, loglhd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        maxlag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _z = nag_double_type_check_and_cast('g13dpc', z, 'z')
    _maxlag = nag_integer_type_check_and_cast('g13dpc', maxlag, 'maxlag')
    _parlag = nag_double_type_check_and_cast('g13dpc', parlag, 'parlag')
    _se = nag_double_type_check_and_cast('g13dpc', se, 'se')
    _qq = nag_double_type_check_and_cast('g13dpc', qq, 'qq')
    _x = nag_double_type_check_and_cast('g13dpc', x, 'x')
    _pvalue = nag_double_type_check_and_cast('g13dpc', pvalue, 'pvalue')
    _loglhd = nag_double_type_check_and_cast('g13dpc', loglhd, 'loglhd')
    g13dpc_ctypes(k, n, _z, m, _maxlag, _parlag, _se, _qq, _x, _pvalue, _loglhd, fail)

g13dsc_ctypes = _library_cl.g13dsc
g13dsc_ctypes.restype = None
g13dsc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(Nag_Boolean), POINTER(c_double), Integer, STRING, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13dsc(k, n, v, kmax, ip, iq, m, par, parhld, qq, ishow, outfile, r0, r, rcm, pdrcm, chi, idf, siglev, fail):
    """
    Multivariate time series, diagnostic checking of residuals, following
    g13ddc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        v, par, parhld, qq, r0, r, rcm, chi, idf, siglev
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        parhld
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        v, par, qq, r0, r, rcm, chi, siglev
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        idf
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _v = nag_double_type_check_and_cast('g13dsc', v, 'v')
    _par = nag_double_type_check_and_cast('g13dsc', par, 'par')
    _parhld = nag_enum_type_check_and_cast('g13dsc', parhld, 'parhld', Nag_Boolean)
    _qq = nag_double_type_check_and_cast('g13dsc', qq, 'qq')
    _r0 = nag_double_type_check_and_cast('g13dsc', r0, 'r0')
    _r = nag_double_type_check_and_cast('g13dsc', r, 'r')
    _rcm = nag_double_type_check_and_cast('g13dsc', rcm, 'rcm')
    _chi = nag_double_type_check_and_cast('g13dsc', chi, 'chi')
    _idf = nag_integer_type_check_and_cast('g13dsc', idf, 'idf')
    _siglev = nag_double_type_check_and_cast('g13dsc', siglev, 'siglev')
    g13dsc_ctypes(k, n, _v, kmax, ip, iq, m, _par, _parhld, _qq, ishow, outfile, _r0, _r, _rcm, pdrcm, _chi, _idf, _siglev, fail)

g13dxc_ctypes = _library_cl.g13dxc
g13dxc_ctypes.restype = None
g13dxc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13dxc(k, ip, par, rr, ri, rmod, fail):
    """
    Calculates the zeros of a vector autoregressive (or moving average)
    operator.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13dxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        par, rr, ri, rmod
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        par, rr, ri, rmod
    """
    from .util import nag_double_type_check_and_cast
    _par = nag_double_type_check_and_cast('g13dxc', par, 'par')
    _rr = nag_double_type_check_and_cast('g13dxc', rr, 'rr')
    _ri = nag_double_type_check_and_cast('g13dxc', ri, 'ri')
    _rmod = nag_double_type_check_and_cast('g13dxc', rmod, 'rmod')
    g13dxc_ctypes(k, ip, _par, _rr, _ri, _rmod, fail)

g13eac_ctypes = _library_cl.g13eac
g13eac_ctypes.restype = None
g13eac_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(NagError)]


def g13eac(n, m, p, s, tds, a, tda, b, tdb, q, tdq, c, tdc, r, tdr, k, tdk, h, tdh, tol, fail):
    """
    One iteration step of the time-varying Kalman filter recursion using the
    square root covariance implementation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13eac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        s, a, b, q, c, r, k, h
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s, a, b, q, c, r, k, h
    """
    from .util import nag_double_type_check_and_cast
    _s = nag_double_type_check_and_cast('g13eac', s, 's')
    _a = nag_double_type_check_and_cast('g13eac', a, 'a')
    _b = nag_double_type_check_and_cast('g13eac', b, 'b')
    _q = nag_double_type_check_and_cast('g13eac', q, 'q')
    _c = nag_double_type_check_and_cast('g13eac', c, 'c')
    _r = nag_double_type_check_and_cast('g13eac', r, 'r')
    _k = nag_double_type_check_and_cast('g13eac', k, 'k')
    _h = nag_double_type_check_and_cast('g13eac', h, 'h')
    g13eac_ctypes(n, m, p, _s, tds, _a, tda, _b, tdb, _q, tdq, _c, tdc, _r, tdr, _k, tdk, _h, tdh, tol, fail)

g13ebc_ctypes = _library_cl.g13ebc
g13ebc_ctypes.restype = None
g13ebc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(NagError)]


def g13ebc(n, m, p, s, tds, a, tda, b, tdb, q, tdq, c, tdc, r, tdr, k, tdk, h, tdh, tol, fail):
    """
    One iteration step of the time-invariant Kalman filter recursion using
    the square root covariance implementation with (A,C) in lower observer
    Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ebc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        s, a, b, q, c, r, k, h
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s, a, b, q, c, r, k, h
    """
    from .util import nag_double_type_check_and_cast
    _s = nag_double_type_check_and_cast('g13ebc', s, 's')
    _a = nag_double_type_check_and_cast('g13ebc', a, 'a')
    _b = nag_double_type_check_and_cast('g13ebc', b, 'b')
    _q = nag_double_type_check_and_cast('g13ebc', q, 'q')
    _c = nag_double_type_check_and_cast('g13ebc', c, 'c')
    _r = nag_double_type_check_and_cast('g13ebc', r, 'r')
    _k = nag_double_type_check_and_cast('g13ebc', k, 'k')
    _h = nag_double_type_check_and_cast('g13ebc', h, 'h')
    g13ebc_ctypes(n, m, p, _s, tds, _a, tda, _b, tdb, _q, tdq, _c, tdc, _r, tdr, _k, tdk, _h, tdh, tol, fail)

g13ecc_ctypes = _library_cl.g13ecc
g13ecc_ctypes.restype = None
g13ecc_ctypes.argtypes = [Integer, Integer, Integer, Nag_ab_input, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(NagError)]


def g13ecc(n, m, p, inpab, t, tdt, ainv, tda, b, tdb, rinv, tdr, c, tdc, qinv, tdq, x, rinvy, z, tol, fail):
    """
    One iteration step of the time-varying Kalman filter recursion using the
    square root information implementation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ecc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t, ainv, b, rinv, c, qinv, x, rinvy, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, ainv, b, rinv, c, qinv, x, rinvy, z
    """
    from .util import nag_double_type_check_and_cast
    _t = nag_double_type_check_and_cast('g13ecc', t, 't')
    _ainv = nag_double_type_check_and_cast('g13ecc', ainv, 'ainv')
    _b = nag_double_type_check_and_cast('g13ecc', b, 'b')
    _rinv = nag_double_type_check_and_cast('g13ecc', rinv, 'rinv')
    _c = nag_double_type_check_and_cast('g13ecc', c, 'c')
    _qinv = nag_double_type_check_and_cast('g13ecc', qinv, 'qinv')
    _x = nag_double_type_check_and_cast('g13ecc', x, 'x')
    _rinvy = nag_double_type_check_and_cast('g13ecc', rinvy, 'rinvy')
    _z = nag_double_type_check_and_cast('g13ecc', z, 'z')
    g13ecc_ctypes(n, m, p, inpab, _t, tdt, _ainv, tda, _b, tdb, _rinv, tdr, _c, tdc, _qinv, tdq, _x, _rinvy, _z, tol, fail)

g13edc_ctypes = _library_cl.g13edc
g13edc_ctypes.restype = None
g13edc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(NagError)]


def g13edc(n, m, p, t, tdt, ainv, tda, ainvb, tdai, rinv, tdr, c, tdc, qinv, tdq, x, rinvy, z, tol, fail):
    """
    One iteration step of the time-invariant Kalman filter recursion using
    the square root information implementation with (A**(-1),A**(-1)B) in
    upper controller Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13edc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t, ainv, ainvb, rinv, c, qinv, x, rinvy, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, ainv, ainvb, rinv, c, qinv, x, rinvy, z
    """
    from .util import nag_double_type_check_and_cast
    _t = nag_double_type_check_and_cast('g13edc', t, 't')
    _ainv = nag_double_type_check_and_cast('g13edc', ainv, 'ainv')
    _ainvb = nag_double_type_check_and_cast('g13edc', ainvb, 'ainvb')
    _rinv = nag_double_type_check_and_cast('g13edc', rinv, 'rinv')
    _c = nag_double_type_check_and_cast('g13edc', c, 'c')
    _qinv = nag_double_type_check_and_cast('g13edc', qinv, 'qinv')
    _x = nag_double_type_check_and_cast('g13edc', x, 'x')
    _rinvy = nag_double_type_check_and_cast('g13edc', rinvy, 'rinvy')
    _z = nag_double_type_check_and_cast('g13edc', z, 'z')
    g13edc_ctypes(n, m, p, _t, tdt, _ainv, tda, _ainvb, tdai, _rinv, tdr, _c, tdc, _qinv, tdq, _x, _rinvy, _z, tol, fail)

g13ejc_ctypes = _library_cl.g13ejc
g13ejc_ctypes.restype = None
g13ejc_ctypes.argtypes = [POINTER(Integer), Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g13ejc(irevcm, mx, my, y, lx, pdlx, ly, pdly, x, st, pdst, n, xt, pdxt, fxt, pdfxt, ropt, lropt, icomm, licomm, rcomm, lrcomm, fail):
    """
    Combined time and measurement update, one iteration of the Unscented
    Kalman Filter for a nonlinear state space model, with additive noise
    (reverse communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ejc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, y, lx, ly, x, st, n, xt, fxt, ropt, icomm, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, lx, ly, x, st, xt, fxt, ropt, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, n, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _y = nag_double_type_check_and_cast('g13ejc', y, 'y')
    _lx = nag_double_type_check_and_cast('g13ejc', lx, 'lx')
    _ly = nag_double_type_check_and_cast('g13ejc', ly, 'ly')
    _x = nag_double_type_check_and_cast('g13ejc', x, 'x')
    _st = nag_double_type_check_and_cast('g13ejc', st, 'st')
    _xt = nag_double_type_check_and_cast('g13ejc', xt, 'xt')
    _fxt = nag_double_type_check_and_cast('g13ejc', fxt, 'fxt')
    _ropt = nag_double_type_check_and_cast('g13ejc', ropt, 'ropt')
    _rcomm = nag_double_type_check_and_cast('g13ejc', rcomm, 'rcomm')
    _irevcm = nag_integer_type_check_and_cast('g13ejc', irevcm, 'irevcm')
    _n = nag_integer_type_check_and_cast('g13ejc', n, 'n')
    _icomm = nag_integer_type_check_and_cast('g13ejc', icomm, 'icomm')
    g13ejc_ctypes(_irevcm, mx, my, _y, _lx, pdlx, _ly, pdly, _x, _st, pdst, _n, _xt, pdxt, _fxt, pdfxt, _ropt, lropt, _icomm, licomm, _rcomm, lrcomm, fail)

nag_kalman_unscented_state_revcom = g13ejc

g13ekc_ctypes = _library_cl.g13ekc
g13ekc_ctypes.restype = None
g13ekc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), NAG_G13EKC_F, NAG_G13EKC_H, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def g13ekc(mx, my, y, lx, ly, f, h, x, st, comm, fail):
    """
    Combined time and measurement update, one iteration of the Unscented
    Kalman Filter for a nonlinear state space model, with additive noise.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ekc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, lx, ly, x, st
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, lx, ly, x, st
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g13ekc', y, 'y')
    _lx = nag_double_type_check_and_cast('g13ekc', lx, 'lx')
    _ly = nag_double_type_check_and_cast('g13ekc', ly, 'ly')
    _x = nag_double_type_check_and_cast('g13ekc', x, 'x')
    _st = nag_double_type_check_and_cast('g13ekc', st, 'st')
    g13ekc_ctypes(mx, my, _y, _lx, _ly, f, h, _x, _st, comm, fail)

nag_kalman_unscented_state = g13ekc

g13ewc_ctypes = _library_cl.g13ewc
g13ewc_ctypes.restype = None
g13ewc_ctypes.argtypes = [Integer, Integer, Nag_ObserverForm, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g13ewc(n, p, reduceto, a, tda, c, tdc, u, tdu, fail):
    """
    Unitary state-space transformation to reduce (A,C) to lower or upper
    observer Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ewc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, c, u
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, c, u
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('g13ewc', a, 'a')
    _c = nag_double_type_check_and_cast('g13ewc', c, 'c')
    _u = nag_double_type_check_and_cast('g13ewc', u, 'u')
    g13ewc_ctypes(n, p, reduceto, _a, tda, _c, tdc, _u, tdu, fail)

g13exc_ctypes = _library_cl.g13exc
g13exc_ctypes.restype = None
g13exc_ctypes.argtypes = [Integer, Integer, Nag_ControllerForm, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g13exc(n, m, reduceto, a, tda, b, tdb, u, tdu, fail):
    """
    Unitary state-space transformation to reduce (B,A) to lower or upper
    controller Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13exc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, u
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, u
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('g13exc', a, 'a')
    _b = nag_double_type_check_and_cast('g13exc', b, 'b')
    _u = nag_double_type_check_and_cast('g13exc', u, 'u')
    g13exc_ctypes(n, m, reduceto, _a, tda, _b, tdb, _u, tdu, fail)

g13fac_ctypes = _library_cl.g13fac
g13fac_ctypes.restype = None
g13fac_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), Integer, Integer, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Nag_Garch_Stationary_Type, Nag_Garch_Est_Initial_Type, Integer, c_double, POINTER(NagError)]


def g13fac(yt, x, tdx, num, p, q, nreg, mn, isym, theta, se, sc, covar, tdc, hp, et, ht, lgf, statopt, estopt, maxiter, tol, fail):
    """
    Univariate time series, parameter estimation for either a symmetric
    GARCH process or a GARCH process with asymmetry of the form
    (epsilon_(t-1)+gamma)**2.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13fac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        yt, x, theta, se, sc, covar, hp, et, ht, lgf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        yt, x, theta, se, sc, covar, hp, et, ht, lgf
    """
    from .util import nag_double_type_check_and_cast
    _yt = nag_double_type_check_and_cast('g13fac', yt, 'yt')
    _x = nag_double_type_check_and_cast('g13fac', x, 'x')
    _theta = nag_double_type_check_and_cast('g13fac', theta, 'theta')
    _se = nag_double_type_check_and_cast('g13fac', se, 'se')
    _sc = nag_double_type_check_and_cast('g13fac', sc, 'sc')
    _covar = nag_double_type_check_and_cast('g13fac', covar, 'covar')
    _hp = nag_double_type_check_and_cast('g13fac', hp, 'hp')
    _et = nag_double_type_check_and_cast('g13fac', et, 'et')
    _ht = nag_double_type_check_and_cast('g13fac', ht, 'ht')
    _lgf = nag_double_type_check_and_cast('g13fac', lgf, 'lgf')
    g13fac_ctypes(_yt, _x, tdx, num, p, q, nreg, mn, isym, _theta, _se, _sc, _covar, tdc, _hp, _et, _ht, _lgf, statopt, estopt, maxiter, tol, fail)

g13fbc_ctypes = _library_cl.g13fbc
g13fbc_ctypes.restype = None
g13fbc_ctypes.argtypes = [Integer, Integer, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13fbc(num, nt, p, q, theta, gamma, fht, ht, et, fail):
    """
    Univariate time series, forecast function for either a symmetric GARCH
    process or a GARCH process with asymmetry of the form
    (epsilon_(t-1)+gamma)**2.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13fbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        theta, fht, ht, et
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        theta, fht, ht, et
    """
    from .util import nag_double_type_check_and_cast
    _theta = nag_double_type_check_and_cast('g13fbc', theta, 'theta')
    _fht = nag_double_type_check_and_cast('g13fbc', fht, 'fht')
    _ht = nag_double_type_check_and_cast('g13fbc', ht, 'ht')
    _et = nag_double_type_check_and_cast('g13fbc', et, 'et')
    g13fbc_ctypes(num, nt, p, q, _theta, gamma, _fht, _ht, _et, fail)

g13fcc_ctypes = _library_cl.g13fcc
g13fcc_ctypes.restype = None
g13fcc_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), Integer, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Nag_Garch_Stationary_Type, Nag_Garch_Est_Initial_Type, Integer, c_double, POINTER(NagError)]


def g13fcc(yt, x, tdx, num, p, q, nreg, mn, theta, se, sc, covar, tdc, hp, et, ht, lgf, statopt, estopt, maxiter, tol, fail):
    """
    Univariate time series, parameter estimation for a GARCH process with
    asymmetry of the form (abs(epsilon_(t-1))+gammaepsilon_(t-1))**2.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13fcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        yt, x, theta, se, sc, covar, hp, et, ht, lgf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        yt, x, theta, se, sc, covar, hp, et, ht, lgf
    """
    from .util import nag_double_type_check_and_cast
    _yt = nag_double_type_check_and_cast('g13fcc', yt, 'yt')
    _x = nag_double_type_check_and_cast('g13fcc', x, 'x')
    _theta = nag_double_type_check_and_cast('g13fcc', theta, 'theta')
    _se = nag_double_type_check_and_cast('g13fcc', se, 'se')
    _sc = nag_double_type_check_and_cast('g13fcc', sc, 'sc')
    _covar = nag_double_type_check_and_cast('g13fcc', covar, 'covar')
    _hp = nag_double_type_check_and_cast('g13fcc', hp, 'hp')
    _et = nag_double_type_check_and_cast('g13fcc', et, 'et')
    _ht = nag_double_type_check_and_cast('g13fcc', ht, 'ht')
    _lgf = nag_double_type_check_and_cast('g13fcc', lgf, 'lgf')
    g13fcc_ctypes(_yt, _x, tdx, num, p, q, nreg, mn, _theta, _se, _sc, _covar, tdc, _hp, _et, _ht, _lgf, statopt, estopt, maxiter, tol, fail)

g13fdc_ctypes = _library_cl.g13fdc
g13fdc_ctypes.restype = None
g13fdc_ctypes.argtypes = [Integer, Integer, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13fdc(num, nt, p, q, theta, gamma, fht, ht, et, fail):
    """
    Univariate time series, forecast function for a GARCH process with
    asymmetry of the form (abs(epsilon_(t-1))+gammaepsilon_(t-1))**2.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13fdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        theta, fht, ht, et
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        theta, fht, ht, et
    """
    from .util import nag_double_type_check_and_cast
    _theta = nag_double_type_check_and_cast('g13fdc', theta, 'theta')
    _fht = nag_double_type_check_and_cast('g13fdc', fht, 'fht')
    _ht = nag_double_type_check_and_cast('g13fdc', ht, 'ht')
    _et = nag_double_type_check_and_cast('g13fdc', et, 'et')
    g13fdc_ctypes(num, nt, p, q, _theta, gamma, _fht, _ht, _et, fail)

g13fec_ctypes = _library_cl.g13fec
g13fec_ctypes.restype = None
g13fec_ctypes.argtypes = [POINTER(c_double), POINTER(c_double), Integer, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Nag_Garch_Stationary_Type, Nag_Garch_Est_Initial_Type, Integer, c_double, POINTER(NagError)]


def g13fec(yt, x, tdx, num, p, q, nreg, mn, theta, se, sc, covar, tdc, hp, et, ht, lgf, statopt, estopt, maxiter, tol, fail):
    """
    Univariate time series, parameter estimation for an asymmetric Glosten,
    Jagannathan and Runkle (GJR) GARCH process.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13fec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        yt, x, theta, se, sc, covar, hp, et, ht, lgf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        yt, x, theta, se, sc, covar, hp, et, ht, lgf
    """
    from .util import nag_double_type_check_and_cast
    _yt = nag_double_type_check_and_cast('g13fec', yt, 'yt')
    _x = nag_double_type_check_and_cast('g13fec', x, 'x')
    _theta = nag_double_type_check_and_cast('g13fec', theta, 'theta')
    _se = nag_double_type_check_and_cast('g13fec', se, 'se')
    _sc = nag_double_type_check_and_cast('g13fec', sc, 'sc')
    _covar = nag_double_type_check_and_cast('g13fec', covar, 'covar')
    _hp = nag_double_type_check_and_cast('g13fec', hp, 'hp')
    _et = nag_double_type_check_and_cast('g13fec', et, 'et')
    _ht = nag_double_type_check_and_cast('g13fec', ht, 'ht')
    _lgf = nag_double_type_check_and_cast('g13fec', lgf, 'lgf')
    g13fec_ctypes(_yt, _x, tdx, num, p, q, nreg, mn, _theta, _se, _sc, _covar, tdc, _hp, _et, _ht, _lgf, statopt, estopt, maxiter, tol, fail)

g13ffc_ctypes = _library_cl.g13ffc
g13ffc_ctypes.restype = None
g13ffc_ctypes.argtypes = [Integer, Integer, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13ffc(num, nt, p, q, theta, gamma, fht, ht, et, fail):
    """
    Univariate time series, forecast function for an asymmetric Glosten,
    Jagannathan and Runkle (GJR) GARCH process.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ffc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        theta, fht, ht, et
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        theta, fht, ht, et
    """
    from .util import nag_double_type_check_and_cast
    _theta = nag_double_type_check_and_cast('g13ffc', theta, 'theta')
    _fht = nag_double_type_check_and_cast('g13ffc', fht, 'fht')
    _ht = nag_double_type_check_and_cast('g13ffc', ht, 'ht')
    _et = nag_double_type_check_and_cast('g13ffc', et, 'et')
    g13ffc_ctypes(num, nt, p, q, _theta, gamma, _fht, _ht, _et, fail)

g13mec_ctypes = _library_cl.g13mec
g13mec_ctypes.restype = None
g13mec_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), c_double, Integer, POINTER(c_double), POINTER(Nag_TS_Interpolation), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13mec(nb, iema, t, tau, m, sinit, inter, pn, rcomm, fail):
    """
    Computes the iterated exponential moving average for a univariate
    inhomogeneous time series.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13mec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iema, t, sinit, pn, rcomm, inter
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        iema, t, sinit, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        pn
    TypeError will be raised when the following arguments are not of data type
    Nag_TS_Interpolation:
        inter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _iema = nag_double_type_check_and_cast('g13mec', iema, 'iema')
    _t = nag_double_type_check_and_cast('g13mec', t, 't')
    _sinit = nag_double_type_check_and_cast('g13mec', sinit, 'sinit')
    _pn = nag_integer_type_check_and_cast('g13mec', pn, 'pn')
    _inter = nag_enum_type_check_and_cast('g13mec', inter, 'inter', Nag_TS_Interpolation)
    _rcomm = nag_double_type_check_and_cast('g13mec', rcomm, 'rcomm')
    g13mec_ctypes(nb, _iema, _t, tau, m, _sinit, _inter, _pn, _rcomm, fail)

g13mfc_ctypes = _library_cl.g13mfc
g13mfc_ctypes.restype = None
g13mfc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), c_double, Integer, Integer, POINTER(c_double), POINTER(Nag_TS_Interpolation), Nag_TS_Transform, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13mfc(order, nb, z, iema, pdiema, t, tau, m1, m2, sinit, inter, ftype, p, x, pn, rcomm, fail):
    """
    Computes the iterated exponential moving average for a univariate
    inhomogeneous time series, intermediate results are also returned.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13mfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        z, iema, t, sinit, inter, p, x, pn, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        z, iema, t, sinit, p, x, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        pn
    TypeError will be raised when the following arguments are not of data type
    Nag_TS_Interpolation:
        inter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _z = nag_double_type_check_and_cast('g13mfc', z, 'z')
    _iema = nag_double_type_check_and_cast('g13mfc', iema, 'iema')
    _t = nag_double_type_check_and_cast('g13mfc', t, 't')
    _sinit = nag_double_type_check_and_cast('g13mfc', sinit, 'sinit')
    _inter = nag_enum_type_check_and_cast('g13mfc', inter, 'inter', Nag_TS_Interpolation)
    _p = nag_double_type_check_and_cast('g13mfc', p, 'p')
    _x = nag_double_type_check_and_cast('g13mfc', x, 'x')
    _pn = nag_integer_type_check_and_cast('g13mfc', pn, 'pn')
    _rcomm = nag_double_type_check_and_cast('g13mfc', rcomm, 'rcomm')
    g13mfc_ctypes(order, nb, _z, _iema, pdiema, _t, tau, m1, m2, _sinit, _inter, ftype, _p, _x, _pn, _rcomm, fail)

g13mgc_ctypes = _library_cl.g13mgc
g13mgc_ctypes.restype = None
g13mgc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), c_double, Integer, Integer, POINTER(c_double), POINTER(Nag_TS_Interpolation), Nag_TS_Transform, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g13mgc(nb, ma, t, tau, m1, m2, sinit, inter, ftype, p, pn, wma, rcomm, fail):
    """
    Computes the exponential moving average for a univariate inhomogeneous
    time series.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13mgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ma, t, sinit, inter, p, pn, wma, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ma, t, sinit, p, wma, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        pn
    TypeError will be raised when the following arguments are not of data type
    Nag_TS_Interpolation:
        inter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _ma = nag_double_type_check_and_cast('g13mgc', ma, 'ma')
    _t = nag_double_type_check_and_cast('g13mgc', t, 't')
    _sinit = nag_double_type_check_and_cast('g13mgc', sinit, 'sinit')
    _inter = nag_enum_type_check_and_cast('g13mgc', inter, 'inter', Nag_TS_Interpolation)
    _p = nag_double_type_check_and_cast('g13mgc', p, 'p')
    _pn = nag_integer_type_check_and_cast('g13mgc', pn, 'pn')
    _wma = nag_double_type_check_and_cast('g13mgc', wma, 'wma')
    _rcomm = nag_double_type_check_and_cast('g13mgc', rcomm, 'rcomm')
    g13mgc_ctypes(nb, _ma, _t, tau, m1, m2, _sinit, _inter, ftype, _p, _pn, _wma, _rcomm, fail)

g13nac_ctypes = _library_cl.g13nac
g13nac_ctypes.restype = None
g13nac_ctypes.argtypes = [Nag_TS_ChangeType, Integer, POINTER(c_double), c_double, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13nac(ctype, n, y, beta, minss, param, ntau, tau, sparam, fail):
    """
    Change point detection, using the PELT algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13nac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, param, ntau, tau, sparam
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, param, sparam
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ntau, tau
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _y = nag_double_type_check_and_cast('g13nac', y, 'y')
    _param = nag_double_type_check_and_cast('g13nac', param, 'param')
    _sparam = nag_double_type_check_and_cast('g13nac', sparam, 'sparam')
    _ntau = nag_integer_type_check_and_cast('g13nac', ntau, 'ntau')
    _tau = nag_integer_type_check_and_cast('g13nac', tau, 'tau')
    g13nac_ctypes(ctype, n, _y, beta, minss, _param, _ntau, _tau, _sparam, fail)

nag_tsa_cp_pelt = g13nac

g13nbc_ctypes = _library_cl.g13nbc
g13nbc_ctypes.restype = None
g13nbc_ctypes.argtypes = [Integer, c_double, Integer, c_double, NAG_G13NBC_CONSTFN, POINTER(Integer), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def g13nbc(n, beta, minss, k, costfn, ntau, tau, comm, fail):
    """
    Change points detection using the PELT algorithm, user supplied cost
    function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13nbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ntau, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ntau, tau
    """
    from .util import nag_integer_type_check_and_cast
    _ntau = nag_integer_type_check_and_cast('g13nbc', ntau, 'ntau')
    _tau = nag_integer_type_check_and_cast('g13nbc', tau, 'tau')
    g13nbc_ctypes(n, beta, minss, k, costfn, _ntau, _tau, comm, fail)

nag_tsa_cp_pelt_user = g13nbc

g13ndc_ctypes = _library_cl.g13ndc
g13ndc_ctypes.restype = None
g13ndc_ctypes.argtypes = [Nag_TS_ChangeType, Integer, POINTER(c_double), c_double, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g13ndc(ctype, n, y, beta, minss, param, mdepth, ntau, tau, sparam, fail):
    """
    Change point detection, using binary segmentation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13ndc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, param, ntau, tau, sparam
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, param, sparam
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ntau, tau
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _y = nag_double_type_check_and_cast('g13ndc', y, 'y')
    _param = nag_double_type_check_and_cast('g13ndc', param, 'param')
    _sparam = nag_double_type_check_and_cast('g13ndc', sparam, 'sparam')
    _ntau = nag_integer_type_check_and_cast('g13ndc', ntau, 'ntau')
    _tau = nag_integer_type_check_and_cast('g13ndc', tau, 'tau')
    g13ndc_ctypes(ctype, n, _y, beta, minss, _param, mdepth, _ntau, _tau, _sparam, fail)

nag_tsa_cp_binary = g13ndc

g13nec_ctypes = _library_cl.g13nec
g13nec_ctypes.restype = None
g13nec_ctypes.argtypes = [Integer, c_double, Integer, Integer, NAG_G13NEC_CHGPFN, POINTER(Integer), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def g13nec(n, beta, minss, mdepth, chgpfn, ntau, tau, comm, fail):
    """
    Change point detection, using binary segmentation, user supplied cost
    function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13nec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ntau, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ntau, tau
    """
    from .util import nag_integer_type_check_and_cast
    _ntau = nag_integer_type_check_and_cast('g13nec', ntau, 'ntau')
    _tau = nag_integer_type_check_and_cast('g13nec', tau, 'tau')
    g13nec_ctypes(n, beta, minss, mdepth, chgpfn, _ntau, _tau, comm, fail)

nag_tsa_cp_binary_user = g13nec

g13xzc_ctypes = _library_cl.g13xzc
g13xzc_ctypes.restype = None
g13xzc_ctypes.argtypes = [POINTER(Nag_G13_Opt)]


def g13xzc(options):
    """
    Freeing function for use with g13 option setting.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g13/g13xzc.html
    """
    g13xzc_ctypes(options)

nag_tsa_diff = g13aac
nag_tsa_auto_corr = g13abc
nag_tsa_auto_corr_part = g13acc
nag_tsa_exp_smooth = g13amc
nag_tsa_resid_corr = g13asc
nag_tsa_mean_range = g13auc
nag_tsa_arma_filter = g13bac
nag_tsa_transf_filter = g13bbc
nag_tsa_cross_corr = g13bcc
nag_tsa_transf_prelim_fit = g13bdc
nag_tsa_multi_inp_model_estim = g13bec
nag_tsa_multi_inp_update = g13bgc
nag_tsa_multi_inp_model_forecast = g13bjc
nag_tsa_options_init = g13bxc
nag_tsa_transf_orders = g13byc
nag_tsa_trans_free = g13bzc
nag_tsa_spectrum_univar_cov = g13cac
nag_tsa_spectrum_univar = g13cbc
nag_tsa_spectrum_bivar_cov = g13ccc
nag_tsa_spectrum_bivar = g13cdc
nag_tsa_cross_spectrum_bivar = g13cec
nag_tsa_gain_phase_bivar = g13cfc
nag_tsa_noise_spectrum_bivar = g13cgc
nag_tsa_multi_auto_corr_part = g13dbc
nag_tsa_varma_estimate = g13ddc
nag_tsa_varma_forecast = g13djc
nag_tsa_varma_update = g13dkc
nag_tsa_multi_diff = g13dlc
nag_tsa_multi_cross_corr = g13dmc
nag_tsa_multi_part_lag_corr = g13dnc
nag_tsa_multi_part_regsn = g13dpc
nag_tsa_varma_diagnostic = g13dsc
nag_tsa_arma_roots = g13dxc
nag_kalman_sqrt_filt_cov_var = g13eac
nag_kalman_sqrt_filt_cov_invar = g13ebc
nag_kalman_sqrt_filt_info_var = g13ecc
nag_kalman_sqrt_filt_info_invar = g13edc
nag_trans_hessenberg_observer = g13ewc
nag_trans_hessenberg_controller = g13exc
nag_estimate_agarchI = g13fac
nag_forecast_agarchI = g13fbc
nag_estimate_agarchII = g13fcc
nag_forecast_agarchII = g13fdc
nag_estimate_garchGJR = g13fec
nag_forecast_garchGJR = g13ffc
nag_tsa_inhom_iema = g13mec
nag_tsa_inhom_iema_all = g13mfc
nag_tsa_inhom_ma = g13mgc
nag_tsa_free = g13xzc
__all__ = ['g13aac', 'g13abc', 'g13acc',
           'g13amc', 'g13asc', 'g13auc',
           'g13bac', 'g13bbc', 'g13bcc', 'g13bdc', 'g13bec',
           'g13bgc', 'g13bjc', 'g13bxc', 'g13byc', 'g13bzc', 'g13cac',
           'g13cbc', 'g13ccc', 'g13cdc', 'g13cec', 'g13cfc', 'g13cgc',
           'g13dbc', 'g13ddc', 'g13djc', 'g13dkc', 'g13dlc', 'g13dmc',
           'g13dnc', 'g13dpc', 'g13dsc', 'g13dxc', 'g13eac', 'g13ebc',
           'g13ecc', 'g13edc', 'g13ewc', 'g13exc', 'g13fac', 'g13fbc',
           'g13fcc', 'g13fdc', 'g13fec', 'g13ffc', 'g13mec', 'g13mfc',
           'g13mgc', 'g13xzc',
           'nag_estimate_agarchI', 'nag_estimate_agarchII',
           'nag_estimate_garchGJR', 'nag_forecast_agarchI',
           'nag_forecast_agarchII', 'nag_forecast_garchGJR',
           'nag_kalman_sqrt_filt_cov_invar',
           'nag_kalman_sqrt_filt_cov_var',
           'nag_kalman_sqrt_filt_info_invar',
           'nag_kalman_sqrt_filt_info_var',
           'nag_trans_hessenberg_controller',
           'nag_trans_hessenberg_observer', 'nag_tsa_arma_filter',
           'nag_tsa_arma_roots', 'nag_tsa_auto_corr',
           'nag_tsa_auto_corr_part',
           'nag_tsa_cross_corr',
           'nag_tsa_cross_spectrum_bivar', 'nag_tsa_diff',
           'nag_tsa_exp_smooth', 'nag_tsa_free',
           'nag_tsa_gain_phase_bivar', 'nag_tsa_inhom_iema',
           'nag_tsa_inhom_iema_all', 'nag_tsa_inhom_ma',
           'nag_tsa_mean_range', 'nag_tsa_multi_auto_corr_part',
           'nag_tsa_multi_cross_corr', 'nag_tsa_multi_diff',
           'nag_tsa_multi_inp_model_estim',
           'nag_tsa_multi_inp_model_forecast',
           'nag_tsa_multi_inp_update', 'nag_tsa_multi_part_lag_corr',
           'nag_tsa_multi_part_regsn', 'nag_tsa_noise_spectrum_bivar',
           'nag_tsa_options_init', 'nag_tsa_resid_corr',
           'nag_tsa_spectrum_bivar', 'nag_tsa_spectrum_bivar_cov',
           'nag_tsa_spectrum_univar', 'nag_tsa_spectrum_univar_cov',
           'nag_tsa_trans_free', 'nag_tsa_transf_filter',
           'nag_tsa_transf_orders', 'nag_tsa_transf_prelim_fit',
           'nag_tsa_varma_diagnostic', 'nag_tsa_varma_estimate',
           'nag_tsa_varma_forecast', 'nag_tsa_varma_update']
