#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the correg submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2019.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestG02a(IntegrationTest):
    """
    Abstract suite of integration tests for the g02a wrappers.
    """

    # pylint: disable=too-few-public-methods

    def set_up(self):
        # pylint: disable=attribute-defined-outside-init
        IntegrationTest.set_up(self)
        self.input_g = [
            [2.0, -1.0, 0.0, 0.0],
            [-1.0, 2.0, -1.0, 0.0],
            [0.0, -1.0, 2.0, -1.0],
            [0.0, 0.0, -1.0, 2.0],
        ]
        self.weights = [100.0, 20.0, 20.0, 20.0]

class TestG02aa(TestG02a): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g02aa wrapper.
    """

class TestG02aaBase(TestG02aa, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import correg as chapter
        TestG02aa.set_up(self)
        self.fun = chapter.corrmat_nearest
        self.default_args()

    def default_args(self):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        n = len(self.input_g)
        self.args = [
            np.array(self.input_g), # g, 0
            n, # n, 1
            0.0, # errtol, 2
            0, # maxits, 3
            0, # maxit, 4
            np.empty((n, n)), # x, 5
        ]

    def test_good(self):
        """
        Good call.
        """
        return_data = self.fun(*self.args)
        np.testing.assert_allclose(
            self.args[5],
            np.array([
                [1.0000, -0.8084, 0.1916, 0.1068],
                [-0.8084, 1.0000, -0.6562, 0.1916],
                [0.1916, -0.6562, 1.0000, -0.8084],
                [0.1068, 0.1916, -0.8084, 1.0000],
            ]),
            rtol=1e-3,
        )
        self.assertLess(return_data.itera, 5)
        self.assertLess(return_data.feval, 5)

class TestG02ab(TestG02a):
    """
    Abstract suite of integration tests for the g02ab wrapper.
    """

    # pylint: disable=too-few-public-methods

    exp_x_a = np.array([
        [1., -0.80246336, 0.19753664, 0.10966256],
        [-0.80246336, 1., -0.64417462, 0.19753664],
        [0.19753664, -0.64417462, 1., -0.80246336],
        [0.10966256, 0.19753664, -0.80246336, 1.],
    ])
    exp_x_b = np.array([
        [1., -0.93710158, 0.02616978, 0.00874155],
        [-0.93710158, 1., -0.30685327, 0.23153336],
        [0.02616978, -0.30685327, 1., -0.90366727],
        [0.00874155, 0.23153336, -0.90366727, 1.],
    ])
    exp_x_w = np.array([
        [1.0000, -0.9464, 0.0247, 0.0089],
        [-0.9464, 1.0000, -0.3374, 0.2374],
        [0.0247, -0.3374, 1.0000, -0.8904],
        [0.0089, 0.2374, -0.8904, 1.0000],
    ])

class TestG02abBase(TestG02ab, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import correg as chapter
        TestG02ab.set_up(self)
        self.fun = chapter.corrmat_nearest_bounded
        self.default_args()

    def default_args(self):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        n = len(self.input_g)
        self.args = [
            np.array(self.input_g), # g, 0
            n, # n, 1
            'B', # opt, 2
            0.02, # alpha, 3
            np.array(self.weights), # w, 4
            0.0, # errtol, 5
            0, # maxits, 6
            0, # maxit, 7
            np.empty((n, n)), # x, 8
        ]

    def test_good(self):
        """
        Good call.
        """
        return_data = self.fun(*self.args)
        np.testing.assert_allclose(
            self.args[8],
            self.exp_x_b,
            rtol=1e-2,
        )
        self.assertLess(return_data.itera, 10)
        self.assertLess(return_data.feval, 10)

class TestG02abNp(TestG02ab, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02ab.set_up(self)
        self.fun = chapter.corrmat_nearest_bounded

    def test_good_a(self):
        """
        Good call, opt = 'A'.
        """
        return_data = self.fun(self.input_g, 'A', alpha=0.02)
        np.testing.assert_allclose(
            return_data.x,
            self.exp_x_a,
            rtol=1e-3,
        )
        self.assertLess(return_data.itera, 10)
        self.assertLess(return_data.feval, 10)

    def test_good_b(self):
        """
        Good call, opt = 'B'.
        """
        return_data = self.fun(self.input_g, 'B', alpha=0.02, w=self.weights)
        np.testing.assert_allclose(
            return_data.x,
            self.exp_x_b,
            rtol=1e-2,
        )
        self.assertLess(return_data.itera, 10)
        self.assertLess(return_data.feval, 10)

    def test_good_w(self):
        """
        Good call, opt = 'W'.
        """
        return_data = self.fun(self.input_g, 'W', w=self.weights)
        np.testing.assert_allclose(
            return_data.x,
            self.exp_x_w,
            rtol=1e-2,
        )
        self.assertLess(return_data.itera, 10)
        self.assertLess(return_data.feval, 10)

class TestG02aj(TestG02a):
    """
    Abstract suite of integration tests for the g02aj wrapper.
    """

    # pylint: disable=too-few-public-methods

    input_h = [
        [0.0, 10.0, 0.0, 0.0],
        [10.0, 0.0, 1.5, 1.5],
        [0.0, 1.5, 0.0, 0.0],
        [0.0, 1.5, 0.0, 0.0],
    ]
    exp_x = np.array([
        [1.00000000e+00, -9.59999999e-01, 9.59992547e-01, -3.46563350e-04],
        [-9.59999999e-01, 1.00000000e+00, -9.59992361e-01, 2.99675665e-04],
        [9.59992547e-01, -9.59992361e-01, 1.00000000e+00, -4.12942115e-03],
        [-3.46563350e-04, 2.99675665e-04, -4.12942115e-03, 1.00000000e+00],
    ])

class TestG02ajBase(TestG02aj, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import correg as chapter
        TestG02aj.set_up(self)
        self.fun = chapter.corrmat_h_weight
        self.default_args()

    def default_args(self):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        n = len(self.input_g)
        self.args = [
            np.array(self.input_g), # g, 0
            n, # n, 1
            0.04, # alpha, 2
            np.array(self.input_h), # h, 3
            0.0, # errtol, 4
            0, # maxit, 5
            np.empty((n, n)), # x, 6
        ]

    def test_good(self):
        """
        Good call.
        """
        return_data = self.fun(*self.args)
        np.testing.assert_allclose(
            self.args[6],
            self.exp_x,
            rtol=1e-5,
        )
        self.assertLess(return_data.itera, 75)

class TestG02ajNp(TestG02aj, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02aj.set_up(self)
        self.fun = chapter.corrmat_h_weight

    def test_good(self):
        """
        Good call.
        """
        return_data = self.fun(self.input_g, 0.04, self.input_h)
        np.testing.assert_allclose(
            return_data.x,
            self.exp_x,
            rtol=1e-5,
        )
        self.assertLess(return_data.itera, 75)

class TestG02ak(TestG02a): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g02ak wrapper.
    """

class TestG02akNp(TestG02ak, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02ak.set_up(self)
        self.fun = chapter.corrmat_nearest_rank

    def test_good(self):
        """
        Good call.
        """
        np.testing.assert_allclose(
            self.fun(self.input_g, 2).x,
            [
                [1.0000, -0.9021, 0.2448, 0.1975],
                [-0.9021, 1.0000, -0.6392, 0.2448],
                [0.2448, -0.6392, 1.0000, -0.9021],
                [0.1975, 0.2448, -0.9021, 1.0000],
            ],
            rtol=1e-3,
        )

class TestG02an(TestG02a):
    """
    Abstract suite of integration tests for the g02an wrapper.
    """

    # pylint: disable=too-few-public-methods

    an_input_g = [
        [1.0000, -0.0991, 0.5665, -0.5653, -0.3441],
        [-0.0991, 1.0000, -0.4273, 0.8474, 0.4975],
        [0.5665, -0.4273, 1.0000, -0.1837, -0.0585],
        [-0.5653, 0.8474, -0.1837, 1.0000, -0.2713],
        [-0.3441, 0.4975, -0.0585, -0.2713, 1.0000],
    ]
    exp_x = np.array([
        [1.0000, -0.0991, 0.5665, -0.3826, -0.2329],
        [-0.0991, 1.0000, -0.4273, 0.5735, 0.3367],
        [0.5665, -0.4273, 1.0000, -0.1243, -0.0396],
        [-0.3826, 0.5735, -0.1243, 1.0000, -0.1836],
        [-0.2329, 0.3367, -0.0396, -0.1836, 1.0000],
    ])

class TestG02anBase(TestG02an, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import correg as chapter
        TestG02an.set_up(self)
        self.input_g = self.an_input_g
        self.fun = chapter.corrmat_shrinking
        self.default_args()

    def default_args(self):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        n = len(self.input_g)
        self.args = [
            np.array(self.input_g), # g, 0
            n, # n, 1
            3, # k, 2
            0.0, # errtol, 3
            0.0, # eigtol, 4
            np.empty((n, n)), # x, 5
        ]

    def test_good(self):
        """
        Good call.
        """
        return_data = self.fun(*self.args)
        np.testing.assert_allclose(
            self.args[5],
            self.exp_x,
            rtol=1e-3,
        )
        self.assertLess(return_data.itera, 30)

class TestG02anNp(TestG02an, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02an.set_up(self)
        self.fun = chapter.corrmat_shrinking

    def test_good(self):
        """
        Good call.
        """
        return_data = self.fun(self.an_input_g, 3)
        np.testing.assert_allclose(
            return_data.x,
            self.exp_x,
            rtol=1e-3,
        )
        self.assertLess(return_data.itera, 30)

    def test_good_f(self):
        """
        Good call, Fortran ordered.
        """
        return_data = self.fun(
            np.array(self.an_input_g, order='F'),
            3
        )
        np.testing.assert_allclose(
            return_data.x,
            self.exp_x,
            rtol=1e-3,
        )
        self.assertLess(return_data.itera, 30)

class TestG02as(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g02as wrapper.
    """

class TestG02asNp(TestG02as, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02as.set_up(self)
        self.fun = chapter.corrmat_fixed

    def test_good(self):
        """
        Good call.
        """
        g = [
            [1., -0.0991, 0.5665, -0.5653, -0.3441],
            [-0.0991, 1., -0.4273, 0.8474, 0.4975],
            [0.5665, -0.4273, 1., -0.1837, -0.0585],
            [-0.5653, 0.8474, -0.1837, 1., -0.2713],
            [-0.3441, 0.4975, -0.0585, -0.2713, 1.],
        ]
        h = [
            [0, 0, 0, 0, 0],
            [1, 0, 0, 0, 0],
            [0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0],
            [0, 0, 0, 1, 0],
        ]
        np.testing.assert_allclose(
            self.fun(g, 0.04, h, -1).x,
            [
                [1.0000, -0.0991, 0.5116, -0.4019, -0.2257],
                [-0.0991, 1.0000, -0.3498, 0.6170, 0.3305],
                [0.5116, -0.3498, 1.0000, -0.2656, -0.1179],
                [-0.4019, 0.6170, -0.2656, 1.0000, -0.2713],
                [-0.2257, 0.3305, -0.1179, -0.2713, 1.0000],
            ],
            rtol=1e-3,
        )

class TestG02ea(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g02ea wrapper.
    """

class TestG02eaNp(TestG02ea, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02ea.set_up(self)
        self.fun = chapter.linregm_rssq

    def test_good(self):
        """
        Good call.
        """
        x = [
            [0., 1125., 232., 7160., 85.9, 8905.],
            [7., 920., 268., 8804., 86.5, 7388.],
            [15., 835., 271., 8108., 85.2, 5348.],
            [22., 1000., 237., 6370., 83.8, 8056.],
            [29., 1150., 192., 6441., 82.1, 6960.],
            [37., 990., 202., 5154., 79.2, 5690.],
            [44., 840., 184., 5896., 81.2, 6932.],
            [58., 650., 200., 5336., 80.6, 5400.],
            [65., 640., 180., 5041., 78.4, 3177.],
            [72., 583., 165., 5012., 79.3, 4461.],
            [80., 570., 151., 4825., 78.7, 3901.],
            [86., 570., 171., 4391., 78., 5002.],
            [93., 510., 243., 4320., 72.3, 4665.],
            [100., 555., 147., 3709., 74.9, 4642.],
            [107., 460., 286., 3969., 74.4, 4840.],
            [122., 275., 198., 3558., 72.5, 4479.],
            [129., 510., 196., 4361., 57.7, 4200.],
            [151., 165., 210., 3301., 71.8, 3410.],
            [171., 244., 327., 2964., 72.5, 3360.],
            [220., 79., 334., 2777., 71.9, 2599.],
        ]
        y = [
            1.5563,
            0.8976,
            0.7482,
            0.7160,
            0.3010,
            0.3617,
            0.1139,
            0.1139,
            -0.2218,
            -0.1549,
            0.,
            0.,
            -0.0969,
            -0.2218,
            -0.3979,
            -0.1549,
            -0.2218,
            -0.3979,
            -0.5229,
            -0.0458,
        ]
        isx = [0, 1, 1, 1, 1, 1]
        vname = ['DAY', 'BOD', 'TKN', 'TS', 'TVS', 'COD']
        soln = self.fun(x, vname, isx, y)
        np.testing.assert_equal(
            soln.modl[-1, :soln.nterms[-1]],
            ['BOD', 'TKN', 'TS', 'TVS', 'COD'],
        )

    def test_bad_vname(self):
        """
        vname incorrect type.
        """
        n = 2
        m = 3
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            'must be an instance of .*str',
            self.fun,
            np.ones((n, m)), list(range(m)), [1]*m, [42.]*n,
        )

class TestG02kb(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g02kb wrapper.
    """

class TestG02kbNp(TestG02kb, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02kb.set_up(self)
        self.fun = chapter.ridge

        m = 3
        self.args = [
            [
                [19.5, 43.1, 29.1],
                [24.7, 49.8, 28.2],
                [30.7, 51.9, 37.0],
                [29.8, 54.3, 31.1],
                [19.1, 42.2, 30.9],
                [25.6, 53.9, 23.7],
                [31.4, 58.5, 27.6],
                [27.9, 52.1, 30.6],
                [22.1, 49.9, 23.2],
                [25.5, 53.5, 24.8],
                [31.1, 56.6, 30.0],
                [30.4, 56.7, 28.3],
                [18.7, 46.5, 23.0],
                [19.7, 44.2, 28.6],
                [14.6, 42.7, 21.3],
                [29.5, 54.4, 30.1],
                [27.7, 55.3, 25.7],
                [30.2, 58.6, 24.6],
                [22.7, 48.2, 27.1],
                [25.2, 51.0, 27.5],
            ], # x, 0
            [1]*m, # isx, 1
            [
                11.9,
                22.8,
                18.7,
                20.1,
                12.9,
                21.7,
                27.1,
                25.4,
                21.3,
                19.3,
                25.4,
                27.2,
                11.7,
                17.8,
                12.8,
                23.9,
                22.6,
                25.4,
                14.8,
                21.1,
            ], # y, 2
            [
                0.0, 0.002, 0.004, 0.006,
                0.008, 0.010, 0.012, 0.014,
                0.016, 0.018, 0.020, 0.022,
                0.024, 0.026, 0.028, 0.030,
            ], # h, 3
            1, # wantb, 4
            1, # wantvf, 5
        ]

    def test_good(self):
        """
        Good call.
        """
        soln = self.fun(*self.args, pec=['L', 'G', 'U', 'F', 'B'])
        np.testing.assert_allclose(
            soln.nep,
            [
                4., 3.26338456, 3.14752741, 3.09873525, 3.07090491,
                3.05231728, 3.03861989, 3.02782466, 3.01889433, 3.01123473,
                3.004481, 2.9983967, 2.99282187, 2.9876444, 2.98278341,
                2.97817903,
            ],
        )
        np.testing.assert_allclose(
            soln.pe,
            [
                [
                    8.03682804, 7.54641935, 7.55750354, 7.56564679, 7.57005088,
                    7.57228869, 7.57324885, 7.57341932, 7.57308308, 7.57241373,
                    7.57152305, 7.570486, 7.56935457, 7.56816583, 7.56694685,
                    7.56571781,
                ],
                [
                    7.68788189, 7.4237779, 7.45203446, 7.46684422, 7.47492264,
                    7.47955854, 7.48225152, 7.48376245, 7.48451476, 7.48476178,
                    7.48466395, 7.48432761, 7.48382591, 7.48321073, 7.48251981,
                    7.48178121,
                ],
                [
                    6.15030551, 6.21244579, 6.27926032, 6.30995555, 6.32718381,
                    6.33805925, 6.3454656, 6.35078642, 6.3547668, 6.35784305,
                    6.36028742, 6.36227845, 6.36393802, 6.3653521, 6.36658301,
                    6.36767701,
                ],
                [
                    7.38036661, 7.22612577, 7.26746753, 7.28759963, 7.2986928,
                    7.30534764, 7.3095385, 7.31223981, 7.31398527, 7.31509094,
                    7.31575555, 7.31611018, 7.31624466, 7.31622253, 7.31608992,
                    7.31588112,
                ],
                [
                    8.60517815, 8.23547965, 8.25145733, 8.2610714, 8.26605566,
                    8.26850791, 8.269497, 8.26958998, 8.26911007, 8.26825356,
                    8.26714601, 8.26587123, 8.26448712, 8.2630349, 8.26154459,
                    8.26003855,
                ],
            ],
        )

    def test_good_pec_none(self):
        """
        Good call.
        """
        soln = self.fun(*self.args)
        self.assertIsNone(soln.pe)

class TestG02mb(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g02mb wrapper.
    """

class TestG02mbNp(TestG02mb, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02mb.set_up(self)
        self.fun = chapter.lars_xtx

        d = [
            [10.28, 1.77, 9.69, 15.58, 8.23, 10.44],
            [9.08, 8.99, 11.53, 6.57, 15.89, 12.58],
            [17.98, 13.10, 1.04, 10.45, 10.12, 16.68],
            [14.82, 13.79, 12.23, 7.00, 8.14, 7.79],
            [17.53, 9.41, 6.24, 3.75, 13.12, 17.08],
            [7.78, 10.38, 9.83, 2.58, 10.13, 4.25],
            [11.95, 21.71, 8.83, 11.00, 12.59, 10.52],
            [14.60, 10.09, -2.70, 9.89, 14.67, 6.49],
            [3.63, 9.07, 12.59, 14.09, 9.06, 8.19],
            [6.35, 9.79, 9.40, 12.79, 8.38, 16.79],
            [4.66, 3.55, 16.82, 13.83, 21.39, 13.88],
            [8.32, 14.04, 17.17, 7.93, 7.39, -1.09],
            [10.86, 13.68, 5.75, 10.44, 10.36, 10.06],
            [4.76, 4.92, 17.83, 2.90, 7.58, 11.97],
            [5.05, 10.41, 9.89, 9.04, 7.90, 13.12],
            [5.41, 9.32, 5.27, 15.53, 5.06, 19.84],
            [9.77, 2.37, 9.54, 20.23, 9.33, 8.82],
            [14.28, 4.34, 14.23, 14.95, 18.16, 11.03],
            [10.17, 6.80, 3.17, 8.57, 16.07, 15.93],
            [5.39, 2.67, 6.37, 13.56, 10.68, 7.35],
        ]
        y = [
            -46.47, -35.80, -129.22, -42.44, -73.51,
            -26.61, -63.90, -76.73, -32.64, -83.29,
            -16.31, -5.82, -47.75, 18.38, -54.71,
            -55.62, -45.28, -22.76, -104.32, -55.94,
        ]
        dy = [drow + [yrow] for drow, yrow in zip(d, y)]
        dytdy = chapter.ssqmat(dy).c
        m = 6
        self.pm = (m*m+m)//2

        self.args = [
            1, # mtype, 0
            20, # n, 1
            [dytdy[:self.pm]], # dtd, 2
            dytdy[self.pm:self.pm+m], # dty, 3
            dytdy[-1], # yty, 4
        ]

        self.exp_fitsum_el = 7.24456263e+01

    def test_good_row_row(self):
        """
        Good call, dtd as a row vector stored by rows.
        """
        self.args[2] = np.array(self.args[2]).reshape((1, self.pm))
        soln = self.fun(*self.args)
        self.assertAlmostEqual(
            soln.fitsum[0, 0], self.exp_fitsum_el,
        )

    def test_good_row_col(self):
        """
        Good call, dtd as a row vector stored by columns.
        """
        self.args[2] = np.array(self.args[2]).reshape((1, self.pm), order='F')
        soln = self.fun(*self.args)
        self.assertAlmostEqual(
            soln.fitsum[0, 0], self.exp_fitsum_el,
        )

    def test_good_col_row(self):
        """
        Good call, dtd as a column vector stored by rows.
        """
        self.args[2] = np.array(self.args[2]).reshape((self.pm, 1))
        soln = self.fun(*self.args)
        self.assertAlmostEqual(
            soln.fitsum[0, 0], self.exp_fitsum_el,
        )

    def test_good_col_col(self):
        """
        Good call, dtd as a column vector stored by columns.
        """
        self.args[2] = np.array(self.args[2]).reshape((self.pm, 1), order='F')
        soln = self.fun(*self.args)
        self.assertAlmostEqual(
            soln.fitsum[0, 0], self.exp_fitsum_el,
        )

class TestG02qg(IntegrationTest):
    """
    Abstract suite of integration tests for the g02qg wrapper.
    """

    # pylint: disable=too-few-public-methods

    dat = [
        [-1.08706912e+00, -2.09398141e+00, -2.21892417e+00, -2.01675718e+00,
         -7.54572987e-02],
        [2.05731809e+00, 2.41944003e+00, -1.70306294e+00, 2.93309008e+00,
         -2.95781935e+00],
        [-3.30731975e+00, 7.18592416e-01, 6.59568222e-02, 2.25845718e+00,
         4.25727762e-01],
        [-5.39295531e+00, 1.36518166e+00, -2.25660293e-01, 2.76551410e-01,
         2.06312509e+00],
        [3.12413454e+00, -2.76559040e+00, -1.16227180e+00, 2.05631499e+00,
         1.38776726e+00],
        [-1.16416380e+00, 6.34806769e-01, -1.03150502e+00, -1.74121142e+00,
         -3.56324996e+00],
        [6.61050911e-01, -4.94639108e-01, 6.74055874e-01, 5.75342267e-01,
         1.47983316e+00],
        [3.51063870e-01, -1.62612119e-01, 1.20371812e+00, -1.29035014e+00,
         1.19700989e-01],
        [-5.67855419e-02, 4.80356930e-01, 3.02757080e+00, 7.01338605e-01,
         -2.23910406e+00],
        [3.24786890e-01, -1.62148070e+00, -1.72181967e-01, -2.10956316e+00,
         -1.81687465e+00],
        [-5.96141446e-01, 3.30090696e+00, -4.12869981e-01, 1.10350960e+00,
         -2.60794711e+00],
        [1.32861977e+00, -1.31332878e+00, -2.10228689e+00, 7.45681392e-02,
         -1.08515370e+00],
        [-1.49324009e+00, 2.07549890e+00, 1.16002283e+00, 1.11224282e+00,
         1.95299368e+00],
        [-9.65325891e-01, -1.82163443e+00, -2.73241229e+00, 1.43186352e+00,
         3.59902243e+00],
        [-1.13479144e+00, -7.58928476e-01, -8.82501584e-01, 1.16854593e+00,
         -1.25558381e+00],
        [-1.32674007e-03, 5.96368893e-03, 1.57040461e+00, 8.12803924e-01,
         -1.00433197e-01],
        [-2.30961578e+00, 2.77606344e+00, 1.47191206e+00, -2.38685501e+00,
         -8.36942005e-01],
        [7.19199159e-01, -1.33988468e+00, -1.08313900e+00, 2.61333722e+00,
         -2.66060848e-01],
        [-5.35527129e-02, 4.23101472e-01, 1.97460324e+00, 9.74032463e-01,
         2.93278796e-01],
        [2.33095939e-01, 2.65146436e+00, -2.14555596e+00, 3.77149242e-01,
         2.36991090e+00],
        [-3.21611366e+00, 8.97623546e-01, -1.52139174e+00, 7.32188931e-01,
         8.84363103e-01],
        [-3.06958179e-01, 2.65718306e-03, -4.02366353e+00, 2.93067531e+00,
         -2.47582374e+00],
        [2.34054727e+00, -3.79753310e+00, 1.28987620e+00, -2.41869340e-01,
         8.99389497e-01],
        [-1.68740304e-01, -3.69803054e+00, 3.92430991e-01, -4.11809790e-01,
         1.84049141e+00],
        [-8.78946585e-02, 5.04346490e-01, -4.43093069e-02, -4.52389024e+00,
         3.69153104e+00],
        [2.33511702e+00, 2.05189739e+00, 4.16388335e+00, -1.72725495e+00,
         -3.00826188e-01],
        [5.93680297e+00, 1.31035004e+00, -1.78996891e+00, -1.65410390e+00,
         -5.11009586e-01],
        [-8.59894531e-01, 2.14506174e+00, -3.74072916e-01, -2.61934968e+00,
         -2.97528508e+00],
        [3.87059506e+00, 5.69304393e-01, 1.92019307e+00, -1.08959053e+00,
         2.09056534e+00],
        [2.56760253e+00, 3.10728523e+00, -1.06880788e+00, 3.15207558e+00,
         -5.04739814e-01],
    ]
    y = [
        -0.7877887, -1.84724306, -0.54159935, 0.11743427, 1.54088859,
        1.34282617,
        3.02187751, 0.00524155, -0.66147648, -1.00639836, -0.51076796,
        3.5073064,
        -2.45566551, -3.4764284, 0.98971684, 4.08685784, -1.44939978,
        0.60580839,
        0.8102041, -2.65197515, 0.53834684, 0.82620637, 1.03820745,
        -0.39157927,
        -3.56033408, 0.43587325, 0.52196984, -3.38171296, 2.72014672,
        0.16890433,
    ]

class TestG02qgBase(TestG02qg, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import correg as chapter
        TestG02qg.set_up(self)
        self.fun = chapter.quantile_linreg
        self.optset = chapter.optset
        self.comm = {}
        self.optset('Initialize = quantile_linreg', self.comm)

    def test_good_b(self):
        """
        b supplied.
        """
        # pylint: disable=too-many-locals
        import warnings
        dat = np.array(self.dat)
        m = len(self.dat[0])
        n = len(self.y)
        isx = [1]*m
        tau = [0.5]
        ntau = len(tau)
        ip = m + 1
        b = np.ones((ip, ntau))
        bl = np.ones((ip, ntau))
        bu = np.ones((ip, ntau))
        ch = np.empty((ip, ip, 0))
        res = np.ones((n, 0))
        info = np.empty(ntau, dtype=int)
        self.optset('Iteration Limit = 1', self.comm)
        self.optset('Calculate Initial Values = No', self.comm)
        with warnings.catch_warnings(record=True):
            self.fun(
                1, 'Y', n, m, dat, isx, ip, self.y, None,
                ntau, tau, b, bl, bu, ch, res,
                self.comm, None, info
            )
        np.testing.assert_allclose(
            bl,
            [
                [-0.71565257],
                [0.21052154],
                [-0.01825634],
                [-0.04386288],
                [0.21337317],
                [0.19674137],
            ],
            rtol=1e-5,
        )

class TestG02qgNp(TestG02qg, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import correg as chapter
        TestG02qg.set_up(self)
        self.fun = chapter.quantile_linreg
        self.optset = chapter.optset
        self.comm = {}
        self.optset('Initialize = quantile_linreg', self.comm)

    def test_bad_b_req(self):
        """
        b not supplied but required.
        """
        self.optset('Calculate Initial Values = No', self.comm)
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            r'.*library\.correg.*b\) must not be None.*',
            self.fun,
            1, [[42.]], [1], [42.], [0.1], comm=self.comm,
        )

    def test_good_b(self):
        """
        b supplied.
        """
        import warnings
        m = len(self.dat[0])
        isx = [1]*m
        tau = 0.5
        b = [[1.]]*(m+1)
        self.optset('Iteration Limit = 1', self.comm)
        self.optset('Calculate Initial Values = No', self.comm)
        with warnings.catch_warnings(record=True):
            regn = self.fun(
                1, self.dat, isx, self.y, tau, comm=self.comm, b=b,
            )
        np.testing.assert_allclose(
            regn.bl,
            [
                [-0.71565257],
                [0.21052154],
                [-0.01825634],
                [-0.04386288],
                [0.21337317],
                [0.19674137],
            ],
            rtol=1e-5,
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
