#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the matop submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2019.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestF01cr(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f01cr wrapper.
    """

class TestF01crBase(TestF01cr, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import matop as chapter
        TestF01cr.set_up(self)
        self.fun = chapter.real_gen_trans_inplace

    def test_good(self):
        """
        Good call.
        """
        m = 3
        n = 7
        a = np.array(range(1, m*n+1), dtype=float)
        self.fun(a, m, n, m*n)
        np.testing.assert_equal(
            a,
            [
                1.0, 4.0, 7.0, 10.0, 13.0, 16.0, 19.0,
                2.0, 5.0, 8.0, 11.0, 14.0, 17.0, 20.0,
                3.0, 6.0, 9.0, 12.0, 15.0, 18.0, 21.0,
            ],
        )

class TestF01sa(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f01sa wrapper.
    """

class TestF01saNp(TestF01sa, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import matop as chapter
        TestF01sa.set_up(self)
        self.fun = chapter.real_nmf
        self.a = [
            [8., 10., 5., 10., 5.],
            [9., 21., 7., 17., 10.],
            [12., 17., 8., 14., 6.],
            [14., 18., 9., 16., 7.],
            [13., 29., 10., 23., 13.],
            [10., 17., 7., 14., 7.],
        ]
        self.w = [
            [1.13762312, 1.18752198, 0.50625744],
            [3.05022932, 2.28974284, 1.12067648],
            [1.65246573, 2.61887843, 2.66302154],
            [0.29028784, 2.76216426, 1.39868493],
            [2.19317303, 2.90910074, 0.48745224],
            [2.08269566, 2.47982721, 1.08634255],
        ]
        self.h = [
            [2.37956027, 1.24992201, 3.43080604, 1.25784511, 1.40833516],
            [3.38788601, 1.81046744, 2.77209202, 3.48391992, 1.08437095],
            [0.45468824, 0.42640333, 0.27647956, 3.12439008, 1.3233102],
        ]
        self.exp_w = [
            [0.1611, 0.2912, 0.1945],
            [0.4777, 0.3146, 0.5818],
            [0.3179, 0.4475, 0.1284],
            [0.3098, 0.5199, 0.1631],
            [0.6549, 0.4609, 0.7068],
            [0.3464, 0.3651, 0.2847],
        ]
        self.exp_h = [
            [6.2177E-16, 3.0274E+01, 4.2556E+00, 6.5413E+00, 2.5214E+00],
            [2.6573E+01, 1.5723E+01, 1.4575E+01, 2.3361E+01, 8.6885E+00],
            [1.0847E+00, 2.7314E+00, 6.7700E-01, 1.1231E+01, 1.0406E+01],
        ]

    def test_good(self):
        """
        Good call.
        """
        import warnings
        k = 3
        warnings.simplefilter('ignore', b_utils.NagAlgorithmicWarning)
        soln = self.fun(self.a, k=k)
        warnings.simplefilter('default', b_utils.NagAlgorithmicWarning)
        np.testing.assert_allclose(
            soln.w,
            self.exp_w,
            rtol=5e-3,
        )
        np.testing.assert_allclose(
            soln.h,
            self.exp_h,
            rtol=1e-4,
        )

    def test_good_iterates(self):
        """
        Good call, initial iterates supplied. seed should be ignored.
        """
        import warnings
        warnings.simplefilter('ignore', b_utils.NagAlgorithmicWarning)
        soln = self.fun(self.a, w=self.w, h=self.h, seed=42)
        warnings.simplefilter('default', b_utils.NagAlgorithmicWarning)
        np.testing.assert_allclose(
            soln.w,
            self.exp_w,
            rtol=5e-3,
        )
        np.testing.assert_allclose(
            soln.h,
            self.exp_h,
            rtol=1e-4,
        )

    def test_bad_k(self):
        """
        Bad call, k None.
        """
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            'argument h. multidimensional array',
            self.fun, self.a,
        )

    def test_bad_seed(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            'argument seed.*condition is not permitted: seed <= 0',
            self.fun, self.a, k=3, seed=-1,
        )

    def test_bad_iterparity(self):
        """
        Bad call, w None.
        """
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            'not permitted.*w is None',
            self.fun, self.a, h=self.h, seed=-42,
        )

class TestF01sb(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f01sb wrapper.
    """

class TestF01sbNp(TestF01sb, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import matop as chapter
        TestF01sb.set_up(self)
        self.fun = chapter.real_nmf_rcomm
        self.a = [
            [8., 10., 5., 10., 5.],
            [9., 21., 7., 17., 10.],
            [12., 17., 8., 14., 6.],
            [14., 18., 9., 16., 7.],
            [13., 29., 10., 23., 13.],
            [10., 17., 7., 14., 7.],
        ]

    def test_good(self):
        """
        Good call.
        """
        import warnings
        a = np.array(self.a)
        k = 3
        w = np.empty((a.shape[0], k))
        h = np.empty((k, a.shape[1]))
        ht = np.empty((a.shape[1], k))
        comm = {}
        irevcm = 0
        warnings.simplefilter('ignore', b_utils.NagAlgorithmicWarning)
        while True:
            irevcm = self.fun(irevcm, w, h, ht, comm)
            if irevcm == 0:
                break
            if irevcm == 1:
                continue
            if irevcm == 2:
                ht = np.matmul(a.transpose(), w)
                continue
            if irevcm == 3:
                w = np.matmul(a, ht)
                continue
            raise ValueError(
                'NYI for irevcm ' + str(irevcm)
            ) # pragma: no cover

        warnings.simplefilter('default', b_utils.NagAlgorithmicWarning)
        np.testing.assert_allclose(
            w,
            [
                [0.159505, 0.29356, 0.190679],
                [0.478135, 0.312574, 0.583245],
                [0.317544, 0.448035, 0.125572],
                [0.308465, 0.522147, 0.158176],
                [0.655799, 0.457909, 0.708717],
                [0.346248, 0.364783, 0.283547],
            ],
            rtol=5e-3,
        )
        np.testing.assert_allclose(
            h,
            [
                [6.215108e-16, 2.997519e+01,
                 5.273596e+00, 7.212119e+00, 1.938524e+00],
                [2.642846e+01, 1.587965e+01,
                 1.408101e+01, 2.309246e+01, 9.011448e+00],
                [1.267314e+00, 2.921997e+00,
                 1.322868e-01, 1.085913e+01, 1.072685e+01],
            ],
            rtol=1e-4,
        )

class TestF01vm(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f01vm wrapper.
    """

class TestF01vmBase(TestF01vm, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import matop as chapter
        TestF01vm.set_up(self)
        self.fun = chapter.ztrttp

    def test_good(self):
        """
        Good call.
        """
        a = np.array([
            [1.1 + 1.1j, 1.1 + 1.2j, 1.3 + 1.3j, 1.4 + 1.4j],
            [0., 2.2 + 2.2j, 2.3 + 2.3j, 2.4 + 2.4j],
            [0., 0., 3.3 + 3.3j, 3.4 + 3.4j],
            [0., 0., 0., 4.4 + 4.4j],
        ])
        n = a.shape[0]
        ap = np.empty(n*(n+1)//2, dtype=complex)
        self.fun('U', n, a, ap)
        np.testing.assert_allclose(
            ap,
            [
                1.1 + 1.1j, 1.1 + 1.2j, 2.2 + 2.2j, 1.3 + 1.3j, 2.3 + 2.3j,
                3.3 + 3.3j, 1.4 + 1.4j, 2.4 + 2.4j, 3.4 + 3.4j, 4.4 + 4.4j,
            ],
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
