#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the rnla submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2019.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestF10ca(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f10ca wrapper.
    """

class TestF10caNp(TestF10ca, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import rand, rnla as chap
        TestF10ca.set_up(self)
        self.fun = chap.svd_rowext_real
        self.a = [
            [2.27, 0.28, -0.48, 1.07, -2.35, 0.62],
            [-1.54, -1.67, -3.09, 1.22, 2.93, -7.39],
            [1.15, 0.94, 0.99, 0.79, -1.45, 1.03],
            [-1.94, -0.78, -0.21, 0.63, 2.30, -2.57],
            [-1.94, -0.78, -0.21, 0.63, 2.30, -2.57],
            [-1.94, -0.78, -0.21, 0.63, 2.30, -2.57],
        ]
        self.k = 5
        self.statecomm = rand.init_repeat(genid=1, seed=[1762543])
        self.exp_s_val = 11.396036330675097

    def test_good(self):
        """
        Good call.
        """
        soln = self.fun('S', 'S', self.a, self.k, self.statecomm)
        self.assertEqual(soln.r, 4)
        self.assertEqual(len(soln.s), soln.r)
        self.assertEqual(soln.s[0], self.exp_s_val)
        self.assertEqual(soln.u.shape, (len(self.a), soln.r))
        self.assertEqual(soln.vt.shape, (soln.r, len(self.a[0])))

    def test_good_nou(self):
        """
        Good call, u declined.
        """
        soln = self.fun('N', 'S', self.a, self.k, self.statecomm)
        self.assertEqual(soln.s[0], self.exp_s_val)
        self.assertIsNone(soln.u)
        self.assertIsNotNone(soln.vt)

    def test_good_novt(self):
        """
        Good call, vt declined.
        """
        soln = self.fun('S', 'N', self.a, self.k, self.statecomm)
        self.assertEqual(soln.s[0], self.exp_s_val)
        self.assertIsNotNone(soln.u)
        self.assertIsNone(soln.vt)

class TestF10da(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f10da wrapper.
    """

class TestF10daNp(TestF10da, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import rand, rnla as chap
        TestF10da.set_up(self)
        self.fun = chap.randproj_dct_real
        self.a = [
            [2.27, 0.28, -0.48, 1.07, -2.35, 0.62],
            [-1.54, -1.67, -3.09, 1.22, 2.93, -7.39],
            [1.15, 0.94, 0.99, 0.79, -1.45, 1.03],
            [-1.94, -0.78, -0.21, 0.63, 2.30, -2.57],
            [-1.94, -0.78, -0.21, 0.63, 2.30, -2.57],
            [-1.94, -0.78, -0.21, 0.63, 2.30, -2.57],
        ]
        self.k = 5
        self.statecomm = rand.init_repeat(genid=1, seed=[1762543])

    def test_bad_trans(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'Constraint: trans = \'',
            self.fun,
            'Z', self.a, self.k, self.statecomm,
        )

    def test_good_notrans(self):
        """
        Good call.
        """
        a = self.fun('N', self.a, self.k, self.statecomm)
        np.testing.assert_equal(
            a.shape,
            (len(self.a), self.k),
        )
        np.testing.assert_allclose(
            a,
            [
                [
                    1.5875000000000004, -0.8655939751330899,
                    1.9825000000000002,
                    1.5714595129879212, 0.2037700067183081,
                ],
                [
                    1.7449999999999988, -1.3736605307173191,
                    -0.3050000000000002,
                    -2.4622290468889538, -1.0683409977338392,
                ],
                [
                    0.9174999999999999, -0.6849873734152915,
                    0.5175000000000001,
                    0.9580737311095229, 0.7234993135154149,
                ],
                [
                    0.2500000000000002, -0.1905992052357225,
                    -1.5299999999999998,
                    -1.4561634731337458, -0.5483563003098524,
                ],
                [
                    0.2500000000000002, -0.1905992052357225,
                    -1.5299999999999998,
                    -1.4561634731337458, -0.5483563003098524,
                ],
                [
                    0.2500000000000002, -0.1905992052357225,
                    -1.5299999999999998,
                    -1.4561634731337458, -0.5483563003098524,
                ],
            ],
        )

    def test_good_trans(self):
        """
        Good call.
        """
        a = self.fun('T', self.a, self.k, self.statecomm)
        np.testing.assert_equal(
            a.shape,
            (self.k, len(self.a[0])),
        )
        np.testing.assert_allclose(
            a,
            [
                [
                    2.8825000000000003, 1.7650000000000001, 2.0249999999999999,
                    -0.2625000000000001, -3.9274999999999989,
                    5.6500000000000004,
                ],
                [
                    0.1419722215136416, 0.2704341622907671, 0.8674799769332159,
                    0.1511648634762061, -0.4734114344382924,
                    1.4827595564842930,
                ],
                [
                    0.9625000000000000, -1.0000000000000009E-02,
                    -0.5099999999999999,
                    -0.4424999999999999, -1.0125000000000000,
                    0.9249999999999999,
                ],
                [
                    -1.9260789351752803, -1.3025068885413305,
                    -1.2552784615187746,
                    0.8036738206561291, 2.6107065695510636,
                    -3.9975456734540855,
                ],
                [
                    1.0051050179368470, -7.0606145244476615E-02,
                    -1.1170125002459281,
                    0.8692831310105646, -0.6268407956709172,
                    -1.9110784591398526,
                ],
            ],
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
