#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the stat submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2019.

# pylint: disable=invalid-name

import unittest

import numpy as np

from test_functionality_core import IntegrationTest

class TestG01al(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g01al wrapper.
    """

class TestG01alBase(TestG01al, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import stat as chapter
        TestG01al.set_up(self)
        self.fun = chapter.five_point_summary

    def test_good(self):
        """
        Good call.
        """
        x = [12., 9., 2., 5., 6., 8., 2., 7., 3., 1., 11., 10.]
        res = np.empty(5)
        self.fun(len(x), x, res)
        self.assertEqual(res[0], 1.)
        self.assertEqual(res[1], 2.5)
        self.assertEqual(res[2], 6.5)
        self.assertEqual(res[3], 9.5)
        self.assertEqual(res[4], 12.)

class TestG01an(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g01an wrapper.
    """

class TestG01anNp(TestG01an, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import stat as chapter
        TestG01an.set_up(self)
        self.fun = chapter.quantiles_stream_fixed

    def test_good(self):
        """
        Good call.
        """
        # pylint: disable=invalid-name,too-many-locals
        trv = [
            [
                34.01, 57.95, 44.88, 22.04, 28.84, 4.43, 0.32, 20.82, 20.53,
                13.08, 7.99, 54.03, 23.21, 26.73, 39.72, 0.97,
            ],
            [
                39.05, 38.78,
                19.38, 51.34, 24.08, 12.41, 58.11, 35.90, 40.38, 27.41, 19.80,
                6.02, 45.33, 36.34, 43.14, 53.84, 39.49, 9.04, 36.74, 58.72,
                59.95, 15.41, 33.05, 39.54,
            ],
            [
                33.24, 58.67, 54.12, 39.48, 43.73,
                24.15, 55.72, 8.87,
            ],
            [
                40.47, 46.18, 20.36, 6.95, 36.86, 49.24,
                56.83, 43.87, 29.86, 22.49, 25.29, 33.17,
            ],
        ]
        n = sum([len(trv_block) for trv_block in trv])
        eps = 0.2

        ind = 0
        rcomm = np.empty(0)
        icomm = np.zeros(2, dtype=int)
        tr = self.fun(ind, n, [], 0, eps, [], 0, rcomm, icomm)
        rcomm = np.empty(icomm[0])
        icomm = np.zeros(icomm[1], dtype=int)

        for trv_block in trv:
            tr = self.fun(
                tr.ind, n, trv_block, len(trv_block), eps, [], 0, rcomm, icomm,
            )

        ind = 3
        q = [0.25, 0.5, 1.0]
        tr = self.fun(ind, n, [], 0, eps, q, len(q), rcomm, icomm)

        np.testing.assert_allclose(
            tr.qv,
            [22.49, 36.86, 59.95],
        )

class TestG01ap(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g01ap wrapper.
    """

class TestG01apNp(TestG01ap, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import stat as chapter
        TestG01ap.set_up(self)
        self.fun = chapter.quantiles_stream_arbitrary

    def test_good(self):
        """
        Good call.
        """
        # pylint: disable=invalid-name
        trv = [
            [
                34.01, 57.95, 44.88, 22.04, 28.84, 4.43, 0.32, 20.82, 20.53,
                13.08, 7.99, 54.03, 23.21, 26.73, 39.72, 0.97, 39.05, 38.78,
                19.38, 51.34,
            ],
            [
                24.08, 12.41, 58.11, 35.90, 40.38, 27.41, 19.80,
                6.02, 45.33, 36.34, 43.14, 53.84, 39.49, 9.04, 36.74, 58.72,
                59.95, 15.41, 33.05, 39.54,
            ],
            [
                33.24, 58.67, 54.12, 39.48, 43.73,
                24.15, 55.72, 8.87, 40.47, 46.18, 20.36, 6.95, 36.86, 49.24,
                56.83, 43.87, 29.86, 22.49, 25.29, 33.17,
            ],
        ]
        eps = 0.2
        ind = 0
        n_block = 0

        while n_block < len(trv) - 1:

            if ind == 0:
                rcomm = np.empty(1)
                icomm = np.zeros(10, dtype=int)
            elif ind == 1:
                n_block += 1
            elif ind == 2:
                rcomm = np.resize(rcomm, icomm[0])
                icomm = np.resize(icomm, icomm[1])
            else:
                self.fail('inconceivable') # pragma: no cover

            tr = self.fun(
                ind, trv[n_block], len(trv[n_block]), eps, [], 0, rcomm, icomm,
            )
            ind = tr.ind

        ind = 3
        q = [0.25, 0.5, 1.0]
        tr = self.fun(ind, [], 0, eps, q, len(q), rcomm, icomm)

        np.testing.assert_allclose(
            tr.qv,
            [22.49, 39.54, 59.95],
        )

class TestG01at(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g01at wrapper.
    """

class TestG01atNp(TestG01at, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import stat as chapter
        TestG01at.set_up(self)
        self.fun = chapter.summary_onevar

        self.x_blocks = [
            [
                -0.6200000000000000, -1.9199999999999999, -1.7200000000000000,
                -6.3499999999999996, 2.0000000000000000, 7.6500000000000004,
                6.1500000000000004, 3.8100000000000001, 4.8700000000000001,
                -0.5100000000000000, 6.8799999999999999, -5.8499999999999996,
                -0.7200000000000000, 0.6600000000000000, 2.2300000000000000,
                -1.6100000000000001, -0.1500000000000000, -1.1499999999999999,
                -8.7400000000000002, -3.9399999999999999, 3.6099999999999999,
            ],
            [
                -0.6600000000000000, -2.3900000000000001, -6.2500000000000000,
                1.2300000000000000, 2.2700000000000000, -2.2700000000000000,
                10.1199999999999992, 8.2899999999999991, -2.9900000000000002,
                8.7100000000000009, -0.7400000000000000, 2.00000000000000E-02,
                1.2200000000000000, 1.7000000000000000, 4.2999999999999998,
                2.9900000000000002, -0.8300000000000000, -1.0000000000000000,
                6.5700000000000003, 2.3199999999999998, -3.4700000000000002,
                -1.4099999999999999, -5.2599999999999998, 0.5300000000000000,
                1.8000000000000000, 4.7900000000000000, -3.0400000000000000,
                1.2000000000000000, -3.2100000000000000, -3.7500000000000000,
                0.8600000000000000, 1.2700000000000000, -5.9500000000000002,
                -5.2699999999999996, 1.6299999999999999, 3.5899999999999999,
                -1.0000000000000000E-02, -1.3799999999999999, -4.7100000000000,
                -4.8200000000000003, 3.5499999999999998, 0.4600000000000000,
                2.5699999999999998, 1.7600000000000000, -4.0499999999999998,
                1.2300000000000000, -1.9900000000000000, 3.2000000000000002,
                -0.6500000000000000, 8.4199999999999999, -6.0099999999999998,
            ],
            [
                1.1299999999999999, -8.8599999999999994, 5.9199999999999999,
                -1.7100000000000000, -3.9900000000000002, 6.5700000000000003,
                -2.0099999999999998, -2.2900000000000000, -1.1100000000000001,
                7.1399999999999997, 4.8399999999999999, -4.4400000000000004,
                -3.3199999999999998, 10.2500000000000000, -2.1099999999999999,
                8.0199999999999996, -7.3099999999999996, 2.7999999999999998,
                -1.2000000000000000, 1.0100000000000000, 1.3700000000000001,
                -2.2799999999999998, 1.2800000000000000, -3.9500000000000002,
                3.4300000000000002, -0.6100000000000000, 4.8499999999999996,
                -0.1100000000000000,
            ],
        ]

        self.wt_blocks = [
            [
                4.9100000000000001, 0.2500000000000000, 3.8999999999999999,
                3.7500000000000000, 1.1699999999999999, 3.1899999999999999,
                2.6600000000000001, 2.0000000000000000E-02, 3.5899999999999999,
                3.6299999999999999, 4.8300000000000001, 3.7200000000000002,
                1.7200000000000000, 0.7800000000000000, 4.7400000000000002,
                1.7200000000000000, 3.9399999999999999, 1.3300000000000001,
                0.5100000000000000, 2.3999999999999999, 3.8999999999999999,
            ],
            None,
            None,
        ]

    def check_results(self, summary):
        """
        Compare the summary output.
        """
        self.assertAlmostEqual(summary.xmean, 0.51, places=2)
        self.assertAlmostEqual(summary.xsd, 4.24, places=2)
        self.assertAlmostEqual(summary.xskew, 0.18, places=2)
        self.assertAlmostEqual(summary.xkurt, -0.59, places=2)
        self.assertAlmostEqual(summary.xmin, -8.86, places=2)
        self.assertAlmostEqual(summary.xmax, 10.25, places=2)

    def test_good_whole(self):
        """
        Good call, all data supplied in one go.
        """
        all_x = []
        all_wt = []
        for block_i, x_block in enumerate(self.x_blocks):
            all_x += x_block
            if self.wt_blocks[block_i] is None:
                all_wt += [1.]*len(x_block)
            else:
                all_wt += self.wt_blocks[block_i]
        self.check_results(self.fun(all_x, wt=all_wt))

    def test_good_blocked(self):
        """
        Good call, all data supplied in one go.
        """
        pn = 0
        rcomm = np.empty(20)
        for block_i, x_block in enumerate(self.x_blocks):
            summary = self.fun(
                x_block,
                pn=pn, wt=self.wt_blocks[block_i], rcomm=rcomm,
            )
            pn = summary.pn
        self.check_results(summary)

class TestG01eb(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g01eb wrapper.
    """

class TestG01ebNp(TestG01eb, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import stat as chapter
        TestG01eb.set_up(self)
        self.fun = chapter.prob_students_t

    def test_good(self):
        """
        Good call.
        """
        self.assertAlmostEqual(
            self.fun(0.85, 20.),
            0.7973069793597106,
        )

class TestG01wa(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g01wa wrapper.
    """

class TestG01waBase(TestG01wa, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import stat as chapter
        TestG01wa.set_up(self)
        self.fun = chapter.moving_average
        self.default_args()

    def default_args(self, m=1, nb=1, pn=0, lrsd=0):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        self.args = [
            m, # m, 0
            nb, # nb, 1
            [42.]*nb, # x, 2
            0, # iwt, 3
            np.empty(0), # wt, 4
            pn, # pn, 5
            np.empty(max(0, nb + min(0, pn - m + 1))), # rmean, 6
            None if lrsd == 0 else np.empty(lrsd), # rsd, 7
            None, # comm, 8
        ]

    def test_good_no_weights(self):
        """
        Test correct operation when no weights supplied.
        """
        self.args[3] = 0
        self.args[5] = self.fun(*self.args)
        self.assertEqual(self.args[5], 1)
        self.assertEqual(self.args[6], [42.])
        self.assertIsNone(self.args[8])

    def test_good_pos_weights_comm(self):
        """
        Test correct operation when inferred positional weights used,
        comm supplied
        """
        self.default_args(m=2, nb=2)
        self.args[2] = [42., 43.]
        self.args[3] = 3
        self.args[8] = {}
        self.args[5] = self.fun(*self.args)
        self.assertIn('rcomm', self.args[8])
        self.args[6] = np.empty(2)
        self.args[2] = [44., 45.]
        self.args[5] = self.fun(*self.args)
        self.assertEqual(self.args[5], 4)
        np.testing.assert_allclose(
            self.args[6],
            [43.666667, 44.666667]
        )
        self.assertIn('rcomm', self.args[8])

class TestG01waNp(TestG01wa, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import stat as chapter
        TestG01wa.set_up(self)
        self.fun = chapter.moving_average
        self.default_args()

    def default_args(self, m=1, nb=1, pn=0, lrsd=0):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        self.args = [
            m, # m, 0
            nb, # nb, 1
            [42.]*nb, # x, 2
            0, # iwt, 3
            np.empty(0), # wt, 4
            pn, # pn, 5
            np.empty(max(0, nb + min(0, pn - m + 1))), # rmean, 6
            None if lrsd == 0 else np.empty(lrsd), # rsd, 7
            None, # comm, 8
        ]

    def test_good_no_weights(self):
        """
        Test correct operation when no weights supplied.
        """
        self.assertEqual(
            self.fun(1, [42.]),
            (1, [42.], None),
        )

    def test_good_pos_weights_sd(self):
        """
        Test correct operation when weights supplied, SD requested.
        """
        import math
        pn, rmean, rsd = self.fun(
            2, [42., 43.], wt=[0.1, 0.2], iwt=1, wantsd=True
        )
        self.assertEqual(pn, 2)
        np.testing.assert_allclose(
            rmean,
            [42.666667]
        )
        np.testing.assert_allclose(
            rsd,
            [1./math.sqrt(2.)]
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
