# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `blast` Chapter.

``blast`` - Further Linear Algebra Support Routines

This module is concerned with basic linear algebra functions which
perform elementary algebraic operations involving scalars, vectors and
matrices.
Most functions for such operations conform either to the specifications
of the BLAS (Basic Linear Algebra Subprograms) or to the specifications
of the BLAST (Basic Linear Algebra Subprograms Technical) Forum.
This module includes functions from the BLAST specifications. Most
(BLAS) functions for such operations are available in submodule
``blas``.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def isum(x):
    u"""
    Sum elements of integer vector.

    For full information please refer to the NAG Library document for
    f16dl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16dlf.html

    Parameters
    ----------
    x : int, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    sumx : int
        The sum of the elements of x.
    """

    from ..base.blast import isum as b_isum

    fun_name = 'naginterfaces.library.blast.isum'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalar = b_isum(
        _n,
        _x,
    )
    return _return_scalar

def imax_val(x):
    u"""
    Maximum value and location, integer vector.

    For full information please refer to the NAG Library document for
    f16dn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16dnf.html

    Parameters
    ----------
    x : int, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the largest component
        of x. If n <= 0 on input then this argument is returned as 0.

    i : int
        i, the largest component of x. If n <= 0 on input then this
        argument is returned as 0.
    """

    from ..base.blast import imax_val as b_imax_val

    fun_name = 'naginterfaces.library.blast.imax_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_imax_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _i_py = _return_scalars.i
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'i'],
    )(
        _k_py,
        _i_py,
    )
    return _return_data

def imin_val(x):
    u"""
    Minimum value and location, integer vector.

    For full information please refer to the NAG Library document for
    f16dp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16dpf.html

    Parameters
    ----------
    x : int, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the smallest
        component of x. If n <= 0 on input then this argument is
        returned as 0.

    i : int
        i, the smallest component of x. If n <= 0 on input then this
        argument is returned as 0.
    """

    from ..base.blast import imin_val as b_imin_val

    fun_name = 'naginterfaces.library.blast.imin_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_imin_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _i_py = _return_scalars.i
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'i'],
    )(
        _k_py,
        _i_py,
    )
    return _return_data

def iamax_val(x):
    u"""
    Maximum absolute value and location, integer vector.

    For full information please refer to the NAG Library document for
    f16dq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16dqf.html

    Parameters
    ----------
    x : int, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the largest component
        of x with respect to absolute value. If n <= 0 on input then
        this argument is returned as 0.

    i : int
        i, the largest component of x with respect to absolute value. If
        n <= 0 on input then this argument is returned as 0.
    """

    from ..base.blast import iamax_val as b_iamax_val

    fun_name = 'naginterfaces.library.blast.iamax_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_iamax_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _i_py = _return_scalars.i
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'i'],
    )(
        _k_py,
        _i_py,
    )
    return _return_data

def iamin_val(x):
    u"""
    Minimum absolute value and location, integer vector.

    For full information please refer to the NAG Library document for
    f16dr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16drf.html

    Parameters
    ----------
    x : int, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the smallest
        component of x with respect to absolute value. If n <= 0 on
        input then this argument is returned as 0.

    i : int
        i, the smallest component of x with respect to absolute value.
        If n <= 0 on input then this argument is returned as 0.
    """

    from ..base.blast import iamin_val as b_iamin_val

    fun_name = 'naginterfaces.library.blast.iamin_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_iamin_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _i_py = _return_scalars.i
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'i'],
    )(
        _k_py,
        _i_py,
    )
    return _return_data

def ddot(conj, alpha, x, beta, y, r):
    u"""
    Dot product of two vectors, allows scaling and accumulation.

    For full information please refer to the NAG Library document for
    f16ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16eaf.html

    Parameters
    ----------
    conj : int
        `conj` is not referenced and need not be set. The presence of
        this argument in the BLAST standard is for consistency with the
        interface of the complex variant of this function.

    alpha : float
        The scalar alpha.

    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If alpha = 0.0 or n = 0, `x` is not referenced.

    beta : float
        The scalar beta.

    y : float, array-like, shape (n)
        The n-element vector y.

        If incy > 0, y_i must be stored in `y`[(i-1)*incy], for i =
        1,2,...,n.

        If incy < 0, y_i must be stored in `y`[(n-i)*|incy|], for i =
        1,2,...,n.

        Intermediate elements of `y` are not referenced.

        If alpha = 0.0 or n = 0, `y` is not referenced.

    r : float
        The initial value, r, to be updated. If beta = 0.0, `r` need not
        be set on entry.

    Returns
    -------
    r : float
        The value r, scaled by beta and updated by the scaled dot
        product of x and y.
    """

    from ..base.blast import ddot as b_ddot

    fun_name = 'naginterfaces.library.blast.ddot'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _return_scalar = b_ddot(
        conj,
        _n,
        alpha,
        _x,
        beta,
        _y,
        r,
    )
    _r_py = _return_scalar
    return _r_py

def daxpby(alpha, x, beta, y):
    u"""
    Real weighted vector addition.

    For full information please refer to the NAG Library document for
    f16ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16ecf.html

    Parameters
    ----------
    alpha : float
        The scalar alpha.

    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    beta : float
        The scalar beta.

    y : float, ndarray, shape (n), modified in place
        On entry: the n-element vector y.

        If incy > 0, y_i must be stored in `y`[(i-1)*incy], for i =
        1,2,...,n.

        If incy < 0, y_i must be stored in `y`[(n-i)*|incy|], for i =
        1,2,...,n.

        Intermediate elements of `y` are not referenced.

        On exit: the updated vector y stored in the array elements used
        to supply the original vector y.

        Intermediate elements of `y` are unchanged.
    """

    from ..base.blast import daxpby as b_daxpby

    fun_name = 'naginterfaces.library.blast.daxpby'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
    )
    b_daxpby(
        _n,
        alpha,
        _x,
        beta,
        y,
    )

def dwaxpby(alpha, x, beta, y):
    u"""
    Real weighted vector addition preserving input.

    For full information please refer to the NAG Library document for
    f16eh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16ehf.html

    Parameters
    ----------
    alpha : float
        The scalar alpha.

    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    beta : float
        The scalar beta.

    y : float, array-like, shape (n)
        The n-element vector y.

        If incy > 0, y_i must be stored in `y`[(i-1)*incy], for i =
        1,2,...,n.

        If incy < 0, y_i must be stored in `y`[(n-i)*|incy|], for i =
        1,2,...,n.

        Intermediate elements of `y` are not referenced.

        If beta = 0.0 or n = 0, `y` is not referenced.

    Returns
    -------
    w : float, ndarray, shape (n)
        The elements w_i of the vector w will be stored in `w` as
        follows.

        If incw > 0, w_i is in `w`[(i-1)*incw], for i = 1,2,...,n.

        If incw < 0, w_i is in `w`[(n-i)*|incw|], for i = 1,2,...,n.

        Intermediate elements of this argument are not referenced.
    """

    from ..base.blast import dwaxpby as b_dwaxpby

    fun_name = 'naginterfaces.library.blast.dwaxpby'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dwaxpby(
        _n,
        alpha,
        _x,
        beta,
        _y,
        _w,
    )
    _w_py = _w
    return _w_py

def dsum(x):
    u"""
    Sum elements of real vector.

    For full information please refer to the NAG Library document for
    f16el

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16elf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    sumx : float
        The sum of the elements of x.
    """

    from ..base.blast import dsum as b_dsum

    fun_name = 'naginterfaces.library.blast.dsum'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalar = b_dsum(
        _n,
        _x,
    )
    return _return_scalar

def zaxpby(alpha, x, beta, y):
    u"""
    Complex weighted vector addition.

    For full information please refer to the NAG Library document for
    f16gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16gcf.html

    Parameters
    ----------
    alpha : complex
        The scalar alpha.

    x : complex, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    beta : complex
        The scalar beta.

    y : complex, array-like, shape (n)
        The n-element vector y.

        If incy > 0, y_i must be stored in `y`[(i-1)*incy], for i =
        1,2,...,n.

        If incy < 0, y_i must be stored in `y`[(n-i)*|incy|], for i =
        1,2,...,n.

        Intermediate elements of `y` are not referenced.

    Returns
    -------
    y : complex, ndarray, shape (n)
        The updated vector y stored in the array elements used to supply
        the original vector y.

        Intermediate elements of `y` are unchanged.
    """

    from ..base.blast import zaxpby as b_zaxpby

    fun_name = 'naginterfaces.library.blast.zaxpby'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
    )
    b_zaxpby(
        _n,
        alpha,
        _x,
        beta,
        _y,
    )
    _y_py = _y
    return _y_py

def zwaxpby(alpha, x, beta, y):
    u"""
    Complex weighted vector addition preserving input.

    For full information please refer to the NAG Library document for
    f16gh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16ghf.html

    Parameters
    ----------
    alpha : complex
        The scalar alpha.

    x : complex, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    beta : complex
        The scalar beta.

    y : complex, array-like, shape (n)
        The n-element vector y.

        If incy > 0, y_i must be stored in `y`[(i-1)*incy], for i =
        1,2,...,n.

        If incy < 0, y_i must be stored in `y`[(n-i)*|incy|], for i =
        1,2,...,n.

        Intermediate elements of `y` are not referenced.

        If beta = 0.0 or n = 0, `y` is not referenced.

    Returns
    -------
    w : complex, ndarray, shape (n)
        The elements w_i of the vector w will be stored in `w` as
        follows.

        If incw > 0, w_i is in `w`[(i-1)*incw], for i = 1,2,...,n.

        If incw < 0, w_i is in `w`[(n-i)*|incw|], for i = 1,2,...,n.

        Intermediate elements of this argument are not referenced.
    """

    from ..base.blast import zwaxpby as b_zwaxpby

    fun_name = 'naginterfaces.library.blast.zwaxpby'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
    )
    _w = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_zwaxpby(
        _n,
        alpha,
        _x,
        beta,
        y,
        _w,
    )
    _w_py = _w
    return _w_py

def zsum(x):
    u"""
    Sum elements of complex vector.

    For full information please refer to the NAG Library document for
    f16gl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16glf.html

    Parameters
    ----------
    x : complex, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    sx : complex
        The sum of the elements of the complex vector x.
    """

    from ..base.blast import zsum as b_zsum

    fun_name = 'naginterfaces.library.blast.zsum'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalar = b_zsum(
        _n,
        _x,
    )
    return _return_scalar

def dmax_val(x):
    u"""
    Maximum value and location, real vector.

    For full information please refer to the NAG Library document for
    f16jn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16jnf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the largest component
        of x. If n <= 0 on input then this argument is returned as 0.

    r : float
        r, the largest component of x. If n <= 0 on input then this
        argument is returned as 0.0.
    """

    from ..base.blast import dmax_val as b_dmax_val

    fun_name = 'naginterfaces.library.blast.dmax_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_dmax_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _r_py = _return_scalars.r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'r'],
    )(
        _k_py,
        _r_py,
    )
    return _return_data

def dmin_val(x):
    u"""
    Minimum value and location, real vector.

    For full information please refer to the NAG Library document for
    f16jp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16jpf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the smallest
        component of x. If n <= 0 on input then this argument is
        returned as 0.

    r : float
        r, the smallest component of x. If n <= 0 on input then this
        argument is returned as 0.0.
    """

    from ..base.blast import dmin_val as b_dmin_val

    fun_name = 'naginterfaces.library.blast.dmin_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_dmin_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _r_py = _return_scalars.r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'r'],
    )(
        _k_py,
        _r_py,
    )
    return _return_data

def damax_val(x):
    u"""
    Maximum absolute value and location, real vector.

    For full information please refer to the NAG Library document for
    f16jq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16jqf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the largest component
        of x with respect to absolute value. If n <= 0 on input then
        this argument is returned as 0.

    r : float
        r, the largest component of x with respect to absolute value. If
        n <= 0 on input then this argument is returned as 0.0.
    """

    from ..base.blast import damax_val as b_damax_val

    fun_name = 'naginterfaces.library.blast.damax_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_damax_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _r_py = _return_scalars.r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'r'],
    )(
        _k_py,
        _r_py,
    )
    return _return_data

def damin_val(x):
    u"""
    Minimum absolute value and location, real vector.

    For full information please refer to the NAG Library document for
    f16jr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16jrf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the smallest
        component of x with respect to absolute value. If n <= 0 on
        input then this argument is returned as 0.

    r : float
        r, the smallest component of x with respect to absolute value.
        If n <= 0 on input then this argument is returned as 0.0.
    """

    from ..base.blast import damin_val as b_damin_val

    fun_name = 'naginterfaces.library.blast.damin_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_damin_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _r_py = _return_scalars.r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'r'],
    )(
        _k_py,
        _r_py,
    )
    return _return_data

def zamax_val(x):
    u"""
    Maximum absolute value and location, complex vector.

    For full information please refer to the NAG Library document for
    f16js

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16jsf.html

    Parameters
    ----------
    x : complex, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the largest component
        of x with respect to absolute value. If n <= 0 on input then
        this argument is returned as 0.

    r : float
        r, the largest component of x with respect to absolute value. If
        n <= 0 on input then this argument is returned as 0.0.
    """

    from ..base.blast import zamax_val as b_zamax_val

    fun_name = 'naginterfaces.library.blast.zamax_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_zamax_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _r_py = _return_scalars.r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'r'],
    )(
        _k_py,
        _r_py,
    )
    return _return_data

def zamin_val(x):
    u"""
    Minimum absolute value and location, complex vector.

    For full information please refer to the NAG Library document for
    f16jt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16jtf.html

    Parameters
    ----------
    x : complex, array-like, shape (n)
        The n-element vector x.

        If incx < 0, x_i must be stored in `x`[(n-i)*|incx|], for i =
        1,2,...,n.

        If n = 0, `x` is not referenced.

    Returns
    -------
    k : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        k, the index, from the set {1,2,...,n}, of the smallest
        component of x with respect to absolute value. If n <= 0 on
        input then this argument is returned as 0.

    r : float
        r, the smallest component of x with respect to absolute value.
        If n <= 0 on input then this argument is returned as 0.0.
    """

    from ..base.blast import zamin_val as b_zamin_val

    fun_name = 'naginterfaces.library.blast.zamin_val'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_zamin_val(
        _n,
        _x,
    )
    _k_py = _return_scalars.k
    _r_py = _return_scalars.r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k', 'r'],
    )(
        _k_py,
        _r_py,
    )
    return _return_data

def dgb_norm(inorm, m, kl, ku, ab):
    u"""
    1-norm, infinity-norm, Frobenius norm, largest absolute element,
    real band matrix.

    For full information please refer to the NAG Library document for
    f16rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16rbf.html

    Parameters
    ----------
    inorm : int
        Specifies the value to be returned.

        `inorm` = 171

          The 1-norm.

        `inorm` = 173

          The 2-norm of a row or column vector.

        `inorm` = 174

          The Frobenius (or Euclidean) norm.

        `inorm` = 175

          The infinity-norm.

        `inorm` = 177

          The value max_{i,j}(|a_{ij}|) (not a norm).

    m : int
        m, the number of rows of the matrix A. If `m` <= 0 on input,
        ``dgb_norm`` returns 0.

    kl : int
        k_l, the number of subdiagonals within the band of A. If `kl` <=
        0 on input, ``dgb_norm`` returns 0.

    ku : int
        k_u, the number of superdiagonals within the band of A. If `ku`
        <= 0 on input, ``dgb_norm`` returns 0.

    ab : float, array-like, shape (`kl`+`ku`+1, n)
        The m by n band matrix A.

    Returns
    -------
    nrm : float
        The value of the required norm of the matrix supplied in `ab`.
    """

    from ..base.blast import dgb_norm as b_dgb_norm

    fun_name = 'naginterfaces.library.blast.dgb_norm'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalar = b_dgb_norm(
        inorm,
        m,
        _n,
        kl,
        ku,
        _ab,
    )
    return _return_scalar

def zgb_norm(inorm, m, kl, ku, ab):
    u"""
    1-norm, infinity-norm, Frobenius norm, largest absolute element,
    complex band matrix.

    For full information please refer to the NAG Library document for
    f16ub

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f16/f16ubf.html

    Parameters
    ----------
    inorm : int
        Specifies the value to be returned.

        `inorm` = 171

          The 1-norm.

        `inorm` = 173

          The 2-norm of a row or column vector.

        `inorm` = 174

          The Frobenius (or Euclidean) norm.

        `inorm` = 175

          The infinity-norm.

        `inorm` = 177

          The value max_{i,j}(|a_{ij}|) (not a norm).

    m : int
        m, the number of rows of the matrix A. If `m` <= 0 on input,
        ``zgb_norm`` returns 0.

    kl : int
        k_l, the number of subdiagonals within the band of A. If `kl` <=
        0 on input, ``zgb_norm`` returns 0.

    ku : int
        k_u, the number of superdiagonals within the band of A. If `ku`
        <= 0 on input, ``zgb_norm`` returns 0.

    ab : complex, array-like, shape (`kl`+`ku`+1, n)
        The m by n band matrix A.

    Returns
    -------
    nrm : float
        The value of the required norm of the matrix supplied in `ab`.
    """

    from ..base.blast import zgb_norm as b_zgb_norm

    fun_name = 'naginterfaces.library.blast.zgb_norm'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalar = b_zgb_norm(
        inorm,
        m,
        _n,
        kl,
        ku,
        _ab,
    )
    return _return_scalar
