# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `lapacklin` Chapter.

``lapacklin`` - Linear Equations (LAPACK)

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module provides functions for the solution of systems of
simultaneous linear equations, and associated computations.
It provides functions for

- matrix factorizations;

- solution of linear equations;

- estimating matrix condition numbers;

- computing error bounds for the solution of linear equations;

- matrix inversion;

- computing scaling factors to equilibrate a matrix.

Functions are provided for both real and complex data.

For a general introduction to the solution of systems of linear
equations, you should turn first to [the F04 Introduction].
The decision trees, in [the F04 Introduction], direct you to the most
appropriate functions in submodule ``linsys`` or submodule ``lapacklin``
for solving your particular problem.
In particular, submodule ``linsys`` and submodule ``lapacklin`` contain
Black Box (or driver) functions which enable some standard types of
problem to be solved by a call to a single function.
Where possible, functions in submodule ``linsys`` call submodule
``lapacklin`` functions to perform the necessary computational tasks.

There are two types of driver functions in this module: simple drivers
which just return the solution to the linear equations; and expert
drivers which also return condition and error estimates and, in many
cases, also allow equilibration.
The simple drivers for real matrices have names of the form and for
complex matrices have names of the form The expert drivers for real
matrices have names of the form and for complex matrices have names of
the form

The functions in this module (submodule ``lapacklin``) handle only dense
and band matrices (not matrices with more specialised structures, or
general sparse matrices).

The functions in this module have all been derived from the LAPACK
project.
They have been designed to be efficient on a wide range of
high-performance computers, without compromising efficiency on
conventional serial machines.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def dgesv(n, nrhs, a, ipiv, b):
    u"""
    Computes the solution to a real system of linear equations.

    For full information please refer to the NAG Library document for
    f07aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07aaf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        On exit: the factors L and U from the factorization A = PLU; the
        unit diagonal elements of L are not stored.

    ipiv : int, shape (`n`), modified in place
        On exit: if no constraints are violated, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution could not be computed.

    See Also
    --------
    :meth:`naginterfaces.library.examples.lapacklin.dgesv_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgesv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgesvx(fact, trans, n, nrhs, a, af, ipiv, equed, r, c, b, x, ferr, berr):
    u"""
    Uses the LU factorization to compute the solution, error-bound and
    condition estimate for a real system of linear equations.

    For full information please refer to the NAG Library document for
    f07ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07abf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `af` and `ipiv` contain the factorized form of A. If `equed`
          != 'N', the matrix A has been equilibrated with scaling
          factors given by `r` and `c`. `a`, `af` and `ipiv` are not
          modified.

        `fact` = 'N'

          The matrix A will be copied to `af` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `af` and factorized.

    trans : str, length 1
        Specifies the form of the system of equations.

        `trans` = 'N'

          AX = B (No transpose).

        `trans` = 'T' or 'C'

          A^TX = B (Transpose).

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        If `fact` = 'F' and `equed` != 'N', `a` must have been
        equilibrated by the scaling factors in `r` and/or `c`.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `a` is not modified.

        If `fact` = 'E' or `equed` != 'N', A is scaled as follows:

        - if `equed` = 'R', A = D_R A;

        - if `equed` = 'C', A = AD_C;

        - if `equed` = 'B', A = D_R AD_C.

    af : None or float, shape (`n`, `n`), modified in place
        On entry: if `fact` = 'F', `af` contains the factors L and U
        from the factorization A = PLU as computed by ``dgetrf``. If
        `equed` != 'N', `af` is the factorized form of the equilibrated
        matrix A.

        If `fact` = 'N' or 'E', `af` need not be set.

        On exit: if `fact` = 'N', `af` returns the factors L and U from
        the factorization A = PLU of the original matrix A.

        If `fact` = 'E', `af` returns the factors L and U from the
        factorization A = PLU of the equilibrated matrix A (see the
        description of `a` for the form of the equilibrated matrix).

        If `fact` = 'F', `af` is unchanged from entry.

    ipiv : None or int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains the pivot indices
        from the factorization A = PLU as computed by ``dgetrf``; at the
        ith step row i of the matrix was interchanged with row
        `ipiv`[i-1]. `ipiv`[i-1] = i indicates a row interchange was not
        required.

        If `fact` = 'N' or 'E', `ipiv` need not be set.

        On exit: if `fact` = 'N', `ipiv` contains the pivot indices from
        the factorization A = PLU of the original matrix A.

        If `fact` = 'E', `ipiv` contains the pivot indices from the
        factorization A = PLU of the equilibrated matrix A.

        If `fact` = 'F', `ipiv` is unchanged from entry.

    equed : None or str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'R', row equilibration, i.e., A has been
          premultiplied by D_R;

        - if `equed` = 'C', column equilibration, i.e., A has been
          postmultiplied by D_C;

        - if `equed` = 'B', both row and column equilibration, i.e., A
          has been replaced by D_R AD_C.

    r : None or float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `r` need not be set.

        If `fact` = 'F' and `equed` = 'R' or 'B', `r` must contain the
        row scale factors for A, D_R; each element of `r` must be
        positive.

        On exit: if `fact` = 'F', `r` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'R' or
        'B', `r` contains the row scale factors for A, D_R, such that A
        is multiplied on the left by D_R; each element of `r` is
        positive.

    c : None or float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `c` need not be set.

        If `fact` = 'F' and `equed` = 'C' or 'B', `c` must contain the
        column scale factors for A, D_C; each element of `c` must be
        positive.

        On exit: if `fact` = 'F', `c` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'C' or
        'B', `c` contains the row scale factors for A, D_C; each element
        of `c` is positive.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `trans` = 'N' and `equed` = 'R' or 'B', `b` is overwritten by
        D_R B.

        If `trans` = 'T' or 'C' and `equed` = 'C' or 'B', `b` is
        overwritten by D_C B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` != 'N', and the solution to the equilibrated system is
        D_C^-1X if `trans` = 'N' and `equed` = 'C' or 'B', or D_R^-1X if
        `trans` = 'T' or 'C' and `equed` = 'R' or 'B'.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : None or str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    recip_growth_factor : float
        If no exception or warning is raised, the reciprocal pivot
        growth factor ||A||/||U||, where ||.|| denotes the maximum
        absolute element norm. If `recip_growth_factor`≪1, the stability
        of the LU factorization of (equilibrated) A could be poor. This
        also means that the solution `x`, condition estimate `rcond`,
        and forward error bound `ferr` could be unreliable. If the
        factorization fails with `errno` in 1 ... `n`, then
        `recip_growth_factor` contains the reciprocal pivot growth
        factor for the leading errno columns of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -10)
            On entry, error in parameter `equed`.

            Constraint: this argument = 'N', 'R', 'C' or 'B'.

        (`errno` -11)
            On entry, error in parameter `r`.

        (`errno` -12)
            On entry, error in parameter `c`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            U is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgesvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, work[max(1, 4*n)].
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    if _fact_py in ('F', 'f'):
        if af is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'af'},
                'must not be None when fact in (\'F\', \'f\')',
            )
    if _fact_py not in ('F', 'f'):
        if af is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'af'},
                'must not be None when fact not in (\'F\', \'f\')',
            )
    if af is None:
        _af = utils._EngineFloat64ArrayType.empty_ctypes((_n_py)*(_n_py))
    else:
        _af_cls = utils._EngineFloat64ArrayType(
            af, {'fun_name': fun_name, 'entity': 'af'},
            exp_shape=(_n_py, _n_py),
            mutual_sorder=_m_sorder,
            check_contents=_fact_py in ('F', 'f'),
            allow_none=True,
        )
        _af = _af_cls.to_ctypes(can_reference_contents=_fact_py in ('F', 'f'))
    if _fact_py in ('F', 'f'):
        if ipiv is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'ipiv'},
                'must not be None when fact in (\'F\', \'f\')',
            )
    if _fact_py not in ('F', 'f'):
        if ipiv is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'ipiv'},
                'must not be None when fact not in (\'F\', \'f\')',
            )
    if ipiv is None:
        _ipiv = utils._EngineIntArrayType.empty_ctypes(_n_py)
    else:
        _ipiv_cls = utils._EngineIntArrayType(
            ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
            exp_shape=(_n_py,),
            check_contents=_fact_py in ('F', 'f'),
            allow_none=True,
        )
        _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=_fact_py in ('F', 'f'))
    if _fact_py in ('F', 'f'):
        if equed is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'equed'},
                'must not be None when fact in (\'F\', \'f\')',
            )
    if equed is None:
        _equed = utils._EngineCharScalarType(
            'Z', {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
            allow_none=True,
        ).to_ctypes()
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
    else:
        _equed = utils._EngineCharScalarType(
            equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
            allow_none=True,
        ).to_ctypes()
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        if _fact_py in ('F', 'f'):
            if _equed_py not in ('N', 'n', 'R', 'r', 'C', 'c', 'B', 'b'):
                utils._EngineErrorExit(
                    fun_name=fun_name,
                    msg_lines=[
                        utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'R\', \'r\', \'C\', \'c\', \'B\', \'b\').'),
                    ],
                    errcodes=(-10,),
                    entity='equed',
                ).manage()
    if (_fact_py in ('F', 'f')) and _equed_py in ('R', 'r', 'B', 'b'):
        if r is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'r'},
                'must not be None when (fact in (\'F\', \'f\')) and equed in (\'R\', \'r\', \'B\', \'b\')',
            )
    if _fact_py not in ('F', 'f') and _equed_py in ('R', 'r', 'B', 'b'):
        if r is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'r'},
                'must not be None when fact not in (\'F\', \'f\') and equed in (\'R\', \'r\', \'B\', \'b\')',
            )
    if r is None:
        _r = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _r_cls = utils._EngineFloat64ArrayType(
            r, {'fun_name': fun_name, 'entity': 'r'},
            exp_shape=(_n_py,),
            check_contents=(_fact_py in ('F', 'f')) and _equed_py in ('R', 'r', 'B', 'b'),
            allow_none=True,
        )
        _r = _r_cls.to_ctypes(can_reference_contents=(_fact_py in ('F', 'f')) and _equed_py in ('R', 'r', 'B', 'b'))
        if (_fact_py in ('F', 'f')) and _equed_py in ('R', 'r', 'B', 'b'):
            if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(r, {'fun_name': fun_name, 'entity': 'r'}, (_vl_j_py-1,)) < 0]) == 0:
                utils._EngineErrorExit(
                    fun_name=fun_name,
                    msg_lines=[
                        'Argument fails constraint check.'
                    ],
                    errcodes=(-11,),
                    entity='r',
                ).manage()
    if (_fact_py in ('F', 'f')) and _equed_py in ('C', 'c', 'B', 'b'):
        if c is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'c'},
                'must not be None when (fact in (\'F\', \'f\')) and equed in (\'C\', \'c\', \'B\', \'b\')',
            )
    if _fact_py not in ('F', 'f') and _equed_py in ('C', 'c', 'B', 'b'):
        if c is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'c'},
                'must not be None when fact not in (\'F\', \'f\') and equed in (\'C\', \'c\', \'B\', \'b\')',
            )
    if c is None:
        _c = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _c_cls = utils._EngineFloat64ArrayType(
            c, {'fun_name': fun_name, 'entity': 'c'},
            exp_shape=(_n_py,),
            check_contents=(_fact_py in ('F', 'f')) and _equed_py in ('C', 'c', 'B', 'b'),
            allow_none=True,
        )
        _c = _c_cls.to_ctypes(can_reference_contents=(_fact_py in ('F', 'f')) and _equed_py in ('C', 'c', 'B', 'b'))
        if (_fact_py in ('F', 'f')) and _equed_py in ('C', 'c', 'B', 'b'):
            if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(c, {'fun_name': fun_name, 'entity': 'c'}, (_vl_j_py-1,)) < 0]) == 0:
                utils._EngineErrorExit(
                    fun_name=fun_name,
                    msg_lines=[
                        'Argument fails constraint check.'
                    ],
                    errcodes=(-12,),
                    entity='c',
                ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(max(1, 4*_n_py))
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _trans,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _equed,
        _r,
        _c,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _fact._length_,
        _trans._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        if af is not None:
            _af_cls.refresh_py(_af)
        if ipiv is not None:
            _ipiv_cls.refresh_py(_ipiv)
        if _fact_py not in ('F', 'f'):
            _equed_py = utils._EngineCharScalarType.to_py(_equed)
        else:
            _equed_py = None
        if r is not None:
            _r_cls.refresh_py(_r)
        if c is not None:
            _c_cls.refresh_py(_c)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _recip_growth_factor_py = utils._EngineFloat64ArrayType.get_element(_work, {'fun_name': fun_name, 'entity': '_work'}, (0,), validate=False)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond', 'recip_growth_factor'],
        )(
            _equed_py,
            _rcond_py,
            _recip_growth_factor_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dsgesv(n, nrhs, a, ipiv, b, x):
    u"""
    Computes the solution to a real system of linear equations using
    mixed precision arithmetic.

    For full information please refer to the NAG Library document for
    f07ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07acf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        On exit: if iterative refinement has been successfully used
        (i.e., if no exception or warning is raised and `itera` >= 0),
        then A is unchanged. If double precision factorization has been
        used (when no exception or warning is raised and `itera` < 0), A
        contains the factors L and U from the factorization A = PLU; the
        unit diagonal elements of L are not stored.

    ipiv : int, shape (`n`), modified in place
        On exit: if no constraints are violated, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required. `ipiv` corresponds
        either to the single precision factorization (if no exception or
        warning is raised and `itera` >= 0) or to the double precision
        factorization (if no exception or warning is raised and `itera`
        < 0).

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Returns
    -------
    itera : int
        If `itera` > 0, iterative refinement has been successfully used
        and this argument is the number of iterations carried out.

        If `itera` < 0, iterative refinement has failed for one of the
        reasons given below and double precision factorization has been
        carried out instead.

        `itera` = -1

          Taking into account machine parameters, and the values of `n`
          and `nrhs`, it is not worth working in single precision.

        `itera` = -2

          Overflow of an entry occurred when moving from double to
          single precision.

        `itera` = -3

          An intermediate single precision factorization failed.

        `itera` = -31

          The maximum permitted number of iterations was exceeded.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsgesv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineFloat32ArrayType.c_type, # Workspace, swork.
        utils._BYREF_INT, # Output, itera.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _swork = utils._EngineFloat32ArrayType.empty_ctypes(0)
    _itera = utils._EngineIntScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _work,
        _swork,
        _itera,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        return_data = _itera_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['itera'],
            )(
                _itera_py,
            )
    _err_class.manage()
    return return_data

def dgetrf(m, n, a, ipiv):
    u"""
    LU factorization of real m by n matrix.

    For full information please refer to the NAG Library document for
    f07ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07adf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the factors L and U from the factorization A = PLU; the
        unit diagonal elements of L are not stored.

    ipiv : int, shape (min(`m`, `n`)), modified in place
        On exit: the pivot indices that define the permutation matrix.
        At the ith step, if `ipiv`[i-1] > i then row i of the matrix A
        was interchanged with row `ipiv`[i-1], for i = 1,2,...,min(m,
        n). `ipiv`[i-1] <= i indicates that, at the ith step, a row
        interchange was not required.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, and division by zero will occur if it is
            used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgetrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[min(m, n)].
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _ipiv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, and division by zero will occur if it is used to solve',
                    'a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgetrs(trans, n, nrhs, a, ipiv, b):
    u"""
    Solution of real system of linear equations, multiple right-hand
    sides, matrix already factorized by ``dgetrf``.

    For full information please refer to the NAG Library document for
    f07ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07aef.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          AX = B is solved for X.

        `trans` = 'T' or 'C'

          A^TX = B is solved for X.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        The LU factorization of A, as returned by ``dgetrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``dgetrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgetrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dgeequ(m, n, a, r, c):
    u"""
    Computes row and column scalings intended to equilibrate a general
    real matrix and reduce its condition number.

    For full information please refer to the NAG Library document for
    f07af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07aff.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`)
        The matrix A whose scaling factors are to be computed.

    r : float, shape (`m`), modified in place
        On exit: if no exception or warning is raised or `errno` > `m`,
        `r` contains the row scale factors, the diagonal elements of
        D_R. The elements of `r` will be positive.

    c : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `c` contains the
        column scale factors, the diagonal elements of D_C. The elements
        of `c` will be positive.

    Returns
    -------
    rowcnd : float
        If no exception or warning is raised or `errno` > `m`, this
        argument contains the ratio of the smallest value of `r`[i-1] to
        the largest value of `r`[i-1]. If `rowcnd` >= 0.1 and `amax` is
        neither too large nor too small, it is not worth scaling by D_R.

    colcnd : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `c`[i-1] to the largest value of
        `c`[i-1].

        If `colcnd` >= 0.1, it is not worth scaling by D_C.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`m`))
            Row *<value>* of A is exactly zero.

        (`errno` i > `m`)
            Column *<value>* of A is exactly zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgeequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, r[m].
        utils._EngineFloat64ArrayType.c_type, # Output, c[n].
        utils._BYREF_FLOAT64, # Output, rowcnd.
        utils._BYREF_FLOAT64, # Output, colcnd.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _rowcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _colcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _r,
        _c,
        _rowcnd,
        _colcnd,
        _amax,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Row {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Column {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _r_cls.refresh_py(_r)
        _c_cls.refresh_py(_c)
        _rowcnd_py = utils._EngineFloat64ScalarType.to_py(_rowcnd)
        _colcnd_py = utils._EngineFloat64ScalarType.to_py(_colcnd)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rowcnd', 'colcnd', 'amax'],
        )(
            _rowcnd_py,
            _colcnd_py,
            _amax_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgecon(norm, n, a, anorm):
    u"""
    Estimate condition number of real matrix, matrix already factorized
    by ``dgetrf``.

    For full information please refer to the NAG Library document for
    f07ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07agf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`)
        The LU factorization of A, as returned by ``dgetrf``.

    anorm : float
        If `norm` = '1' or 'O', the 1-norm of the original matrix A.

        If `norm` = 'I', the infinity-norm of the original matrix A.

        `anorm` may be computed by calling ``blas.dlange`` with the same
        value for the argument `norm`.

        `anorm` must be computed either before calling ``dgetrf`` or
        else from a copy of the original matrix A (see [Example]).

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgecon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-5,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _n,
        _a,
        _lda,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _norm._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dgerfs(trans, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ahf.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the linear equations for which X is the
        computed solution.

        `trans` = 'N'

          The linear equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The linear equations are of the form A^TX = B.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        The n by n original matrix A as supplied to ``dgetrf``.

    af : float, shape (`n`, `n`)
        The LU factorization of A, as returned by ``dgetrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``dgetrf``.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``dgetrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgerfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ahft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af = utils._EngineFloat64ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dgetri(n, a, ipiv):
    u"""
    Inverse of real matrix, matrix already factorized by ``dgetrf``.

    For full information please refer to the NAG Library document for
    f07aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ajf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the LU factorization of A, as returned by ``dgetrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``dgetrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is zero. U is singular,
            and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgetri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ajft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'U is singular, and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgesv(n, nrhs, a, ipiv, b):
    u"""
    Computes the solution to a complex system of linear equations.

    For full information please refer to the NAG Library document for
    f07an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07anf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        On exit: the factors L and U from the factorization A = PLU; the
        unit diagonal elements of L are not stored.

    ipiv : int, shape (`n`), modified in place
        On exit: if no constraints are violated, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgesv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgesvx(fact, trans, n, nrhs, a, af, ipiv, equed, r, c, b, x, ferr, berr):
    u"""
    Uses the LU factorization to compute the solution, error-bound and
    condition estimate for a complex system of linear equations.

    For full information please refer to the NAG Library document for
    f07ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07apf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `af` and `ipiv` contain the factorized form of A. If `equed`
          != 'N', the matrix A has been equilibrated with scaling
          factors given by `r` and `c`. `a`, `af` and `ipiv` are not
          modified.

        `fact` = 'N'

          The matrix A will be copied to `af` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `af` and factorized.

    trans : str, length 1
        Specifies the form of the system of equations.

        `trans` = 'N'

          AX = B (No transpose).

        `trans` = 'T'

          A^TX = B (Transpose).

        `trans` = 'C'

          A^HX = B (Conjugate transpose).

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        If `fact` = 'F' and `equed` != 'N', `a` must have been
        equilibrated by the scaling factors in `r` and/or `c`.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `a` is not modified.

        If `fact` = 'E' or `equed` != 'N', A is scaled as follows:

        - if `equed` = 'R', A = D_R A;

        - if `equed` = 'C', A = AD_C;

        - if `equed` = 'B', A = D_R AD_C.

    af : complex, shape (`n`, `n`), modified in place
        On entry: if `fact` = 'F', `af` contains the factors L and U
        from the factorization A = PLU as computed by ``zgetrf``. If
        `equed` != 'N', `af` is the factorized form of the equilibrated
        matrix A.

        If `fact` = 'N' or 'E', `af` need not be set.

        On exit: if `fact` = 'N', `af` returns the factors L and U from
        the factorization A = PLU of the original matrix A.

        If `fact` = 'E', `af` returns the factors L and U from the
        factorization A = PLU of the equilibrated matrix A (see the
        description of `a` for the form of the equilibrated matrix).

        If `fact` = 'F', `af` is unchanged from entry.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains the pivot indices
        from the factorization A = PLU as computed by ``zgetrf``; at the
        ith step row i of the matrix was interchanged with row
        `ipiv`[i-1]. `ipiv`[i-1] = i indicates a row interchange was not
        required.

        If `fact` = 'N' or 'E', `ipiv` need not be set.

        On exit: if `fact` = 'N', `ipiv` contains the pivot indices from
        the factorization A = PLU of the original matrix A.

        If `fact` = 'E', `ipiv` contains the pivot indices from the
        factorization A = PLU of the equilibrated matrix A.

        If `fact` = 'F', `ipiv` is unchanged from entry.

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'R', row equilibration, i.e., A has been
          premultiplied by D_R;

        - if `equed` = 'C', column equilibration, i.e., A has been
          postmultiplied by D_C;

        - if `equed` = 'B', both row and column equilibration, i.e., A
          has been replaced by D_R AD_C.

    r : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `r` need not be set.

        If `fact` = 'F' and `equed` = 'R' or 'B', `r` must contain the
        row scale factors for A, D_R; each element of `r` must be
        positive.

        On exit: if `fact` = 'F', `r` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'R' or
        'B', `r` contains the row scale factors for A, D_R, such that A
        is multiplied on the left by D_R; each element of `r` is
        positive.

    c : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `c` need not be set.

        If `fact` = 'F' and `equed` = 'C' or 'B', `c` must contain the
        column scale factors for A, D_C; each element of `c` must be
        positive.

        On exit: if `fact` = 'F', `c` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'C' or
        'B', `c` contains the row scale factors for A, D_C; each element
        of `c` is positive.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `trans` = 'N' and `equed` = 'R' or 'B', `b` is overwritten by
        D_R B.

        If `trans` = 'T' or 'C' and `equed` = 'C' or 'B', `b` is
        overwritten by D_C B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` != 'N', and the solution to the equilibrated system is
        D_C^-1X if `trans` = 'N' and `equed` = 'C' or 'B', or D_R^-1X if
        `trans` = 'T' or 'C' and `equed` = 'R' or 'B'.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    recip_growth_factor : float
        If no exception or warning is raised, the reciprocal pivot
        growth factor ||A||/||U||, where ||.|| denotes the maximum
        absolute element norm. If `recip_growth_factor`≪1, the stability
        of the LU factorization of (equilibrated) A could be poor. This
        also means that the solution `x`, condition estimate `rcond`,
        and forward error bound `ferr` could be unreliable. If the
        factorization fails with `errno` in 1 ... `n`, then
        `recip_growth_factor` contains the reciprocal pivot growth
        factor for the leading errno columns of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -10)
            On entry, error in parameter `equed`.

            Constraint: this argument = 'N', 'R', 'C' or 'B'.

        (`errno` -11)
            On entry, error in parameter `r`.

        (`errno` -12)
            On entry, error in parameter `c`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            U is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgesvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[max(1, 2*n)].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af_cls = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _af = _af_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'R', 'r', 'C', 'c', 'B', 'b'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'R\', \'r\', \'C\', \'c\', \'B\', \'b\').'),
                ],
                errcodes=(-10,),
                entity='equed',
            ).manage()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
    )
    _r = _r_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and _equed_py in ('R', 'r', 'B', 'b'):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(r, {'fun_name': fun_name, 'entity': 'r'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-11,),
                entity='r',
            ).manage()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
    )
    _c = _c_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and _equed_py in ('C', 'c', 'B', 'b'):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(c, {'fun_name': fun_name, 'entity': 'c'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-12,),
                entity='c',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(max(1, 2*_n_py))
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _trans,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _equed,
        _r,
        _c,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _trans._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _af_cls.refresh_py(_af)
        _ipiv_cls.refresh_py(_ipiv)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _r_cls.refresh_py(_r)
        _c_cls.refresh_py(_c)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _recip_growth_factor_py = utils._EngineFloat64ArrayType.get_element(_rwork, {'fun_name': fun_name, 'entity': '_rwork'}, (0,), validate=False)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond', 'recip_growth_factor'],
        )(
            _equed_py,
            _rcond_py,
            _recip_growth_factor_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zcgesv(n, nrhs, a, ipiv, b, x):
    u"""
    Computes the solution to a complex system of linear equations using
    mixed precision arithmetic.

    For full information please refer to the NAG Library document for
    f07aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07aqf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        On exit: if iterative refinement has been successfully used
        (i.e., if no exception or warning is raised and `itera` >= 0),
        then A is unchanged. If double precision factorization has been
        used (when no exception or warning is raised and `itera` < 0), A
        contains the factors L and U from the factorization A = PLU; the
        unit diagonal elements of L are not stored.

    ipiv : int, shape (`n`), modified in place
        On exit: if no constraints are violated, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required. `ipiv` corresponds
        either to the single precision factorization (if no exception or
        warning is raised and `itera` >= 0) or to the double precision
        factorization (if no exception or warning is raised and `itera`
        < 0).

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Returns
    -------
    itera : int
        If `itera` > 0, iterative refinement has been successfully used
        and this argument is the number of iterations carried out.

        If `itera` < 0, iterative refinement has failed for one of the
        reasons given below and double precision factorization has been
        carried out instead.

        `itera` = -1

          Taking into account machine parameters, and the values of `n`
          and `nrhs`, it is not worth working in single precision.

        `itera` = -2

          Overflow of an entry occurred when moving from double to
          single precision.

        `itera` = -3

          An intermediate single precision factorization failed.

        `itera` = -31

          The maximum permitted number of iterations was exceeded.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zcgesv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineComplex64ArrayType.c_type, # Workspace, swork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, itera.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _swork = utils._EngineComplex64ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _itera = utils._EngineIntScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _work,
        _swork,
        _rwork,
        _itera,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        return_data = _itera_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['itera'],
            )(
                _itera_py,
            )
    _err_class.manage()
    return return_data

def zgetrf(m, n, a, ipiv):
    u"""
    LU factorization of complex m by n matrix.

    For full information please refer to the NAG Library document for
    f07ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07arf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the factors L and U from the factorization A = PLU; the
        unit diagonal elements of L are not stored.

    ipiv : int, shape (min(`m`, `n`)), modified in place
        On exit: the pivot indices that define the permutation matrix.
        At the ith step, if `ipiv`[i-1] > i then row i of the matrix A
        was interchanged with row `ipiv`[i-1], for i = 1,2,...,min(m,
        n). `ipiv`[i-1] <= i indicates that, at the ith step, a row
        interchange was not required.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, and division by zero will occur if it is
            used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgetrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[min(m, n)].
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _ipiv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, and division by zero will occur if it is used to solve',
                    'a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgetrs(trans, n, nrhs, a, ipiv, b):
    u"""
    Solution of complex system of linear equations, multiple right-hand
    sides, matrix already factorized by ``zgetrf``.

    For full information please refer to the NAG Library document for
    f07as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07asf.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          AX = B is solved for X.

        `trans` = 'T'

          A^TX = B is solved for X.

        `trans` = 'C'

          A^HX = B is solved for X.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The LU factorization of A, as returned by ``zgetrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``zgetrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgetrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zgeequ(m, n, a, r, c):
    u"""
    Computes row and column scalings intended to equilibrate a general
    complex matrix and reduce its condition number.

    For full information please refer to the NAG Library document for
    f07at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07atf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`)
        The matrix A whose scaling factors are to be computed.

    r : float, shape (`m`), modified in place
        On exit: if no exception or warning is raised or `errno` > `m`,
        `r` contains the row scale factors, the diagonal elements of
        D_R. The elements of `r` will be positive.

    c : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `c` contains the
        column scale factors, the diagonal elements of D_C. The elements
        of `c` will be positive.

    Returns
    -------
    rowcnd : float
        If no exception or warning is raised or `errno` > `m`, this
        argument contains the ratio of the smallest value of `r`[i-1] to
        the largest value of `r`[i-1]. If `rowcnd` >= 0.1 and `amax` is
        neither too large nor too small, it is not worth scaling by D_R.

    colcnd : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `c`[i-1] to the largest value of
        `c`[i-1].

        If `colcnd` >= 0.1, it is not worth scaling by D_C.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`m`))
            Row *<value>* of A is exactly zero.

        (`errno` i > `m`)
            Column *<value>* of A is exactly zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgeequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, r[m].
        utils._EngineFloat64ArrayType.c_type, # Output, c[n].
        utils._BYREF_FLOAT64, # Output, rowcnd.
        utils._BYREF_FLOAT64, # Output, colcnd.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _rowcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _colcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _r,
        _c,
        _rowcnd,
        _colcnd,
        _amax,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Row {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Column {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _r_cls.refresh_py(_r)
        _c_cls.refresh_py(_c)
        _rowcnd_py = utils._EngineFloat64ScalarType.to_py(_rowcnd)
        _colcnd_py = utils._EngineFloat64ScalarType.to_py(_colcnd)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rowcnd', 'colcnd', 'amax'],
        )(
            _rowcnd_py,
            _colcnd_py,
            _amax_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgecon(norm, n, a, anorm):
    u"""
    Estimate condition number of complex matrix, matrix already
    factorized by ``zgetrf``.

    For full information please refer to the NAG Library document for
    f07au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07auf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        The LU factorization of A, as returned by ``zgetrf``.

    anorm : float
        If `norm` = '1' or 'O', the 1-norm of the original matrix A.

        If `norm` = 'I', the infinity-norm of the original matrix A.

        `anorm` may be computed by calling ``blas.zlange`` with the same
        value for the argument `norm`.

        `anorm` must be computed either before calling ``zgetrf`` or
        else from a copy of the original matrix A (see [Example]).

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgecon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-5,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _n,
        _a,
        _lda,
        _anorm,
        _rcond,
        _work,
        _rwork,
        _info,
        _norm._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zgerfs(trans, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07av

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07avf.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the linear equations for which X is the
        computed solution as follows:

        `trans` = 'N'

          The linear equations are of the form AX = B.

        `trans` = 'T'

          The linear equations are of the form A^TX = B.

        `trans` = 'C'

          The linear equations are of the form A^HX = B.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The n by n original matrix A as supplied to ``zgetrf``.

    af : complex, shape (`n`, `n`)
        The LU factorization of A, as returned by ``zgetrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``zgetrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zgetrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgerfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07avft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zgetri(n, a, ipiv):
    u"""
    Inverse of complex matrix, matrix already factorized by ``zgetrf``.

    For full information please refer to the NAG Library document for
    f07aw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07awf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the LU factorization of A, as returned by ``zgetrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``zgetrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is zero. U is singular,
            and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgetri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07awft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'U is singular, and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgbsv(n, kl, ku, nrhs, ab, ipiv, b):
    u"""
    Computes the solution to a real banded system of linear equations.

    For full information please refer to the NAG Library document for
    f07ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07baf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : float, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        See [Further Comments] for further details.

        On exit: if `errno` >= 0, `ab` is overwritten by details of the
        factorization.

    ipiv : int, shape (`n`), modified in place
        On exit: if no constraints are violated, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -3)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgbsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-2,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-3,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _ipiv,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgbsvx(fact, trans, n, kl, ku, nrhs, ab, afb, ipiv, equed, r, c, b, x, ferr, berr):
    u"""
    Uses the LU factorization to compute the solution, error-bound and
    condition estimate for a real banded system of linear equations.

    For full information please refer to the NAG Library document for
    f07bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `afb` and `ipiv` contain the factorized form of A. If `equed`
          != 'N', the matrix A has been equilibrated with scaling
          factors given by `r` and `c`. `ab`, `afb` and `ipiv` are not
          modified.

        `fact` = 'N'

          The matrix A will be copied to `afb` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `afb` and factorized.

    trans : str, length 1
        Specifies the form of the system of equations.

        `trans` = 'N'

          AX = B (No transpose).

        `trans` = 'T' or 'C'

          A^TX = B (Transpose).

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : float, shape (`kl`+`ku`+1, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        See [Further Comments] for further details.

        If `fact` = 'F' and `equed` != 'N', A must have been
        equilibrated by the scaling factors in `r` and/or `c`.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `ab` is not modified.

        If `equed` != 'N' then, if no constraints are violated, A is
        scaled as follows:

        - if `equed` = 'R', A = D_r A;

        - if `equed` = 'C', A = AD_c;

        - if `equed` = 'B', A = D_r AD_c.

    afb : float, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: if `fact` = 'N' or 'E', `afb` need not be set.

        If `fact` = 'F', details of the LU factorization of the n by n
        band matrix A, as computed by ``dgbtrf``.

        If `equed` != 'N', `afb` is the factorized form of the
        equilibrated matrix A.

        On exit: if `fact` = 'F', `afb` is unchanged from entry.

        Otherwise, if no constraints are violated, then if `fact` = 'N',
        `afb` returns details of the LU factorization of the band matrix
        A, and if `fact` = 'E', `afb` returns details of the LU
        factorization of the equilibrated band matrix A (see the
        description of `ab` for the form of the equilibrated matrix).

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `ipiv` need not be set.

        If `fact` = 'F', `ipiv` contains the pivot indices from the
        factorization A = LU, as computed by ``dgbtrf``; row i of the
        matrix was interchanged with row `ipiv`[i-1].

        On exit: if `fact` = 'F', `ipiv` is unchanged from entry.

        Otherwise, if no constraints are violated, `ipiv` contains the
        pivot indices that define the permutation matrix P; at the ith
        step row i of the matrix was interchanged with row `ipiv`[i-1].
        `ipiv`[i-1] = i indicates a row interchange was not required.

        If `fact` = 'N', the pivot indices are those corresponding to
        the factorization A = LU of the original matrix A.

        If `fact` = 'E', the pivot indices are those corresponding to
        the factorization of A = LU of the equilibrated matrix A.

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'R', row equilibration, i.e., A has been
          premultiplied by D_R;

        - if `equed` = 'C', column equilibration, i.e., A has been
          postmultiplied by D_C;

        - if `equed` = 'B', both row and column equilibration, i.e., A
          has been replaced by D_R AD_C.

    r : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `r` need not be set.

        If `fact` = 'F' and `equed` = 'R' or 'B', `r` must contain the
        row scale factors for A, D_R; each element of `r` must be
        positive.

        On exit: if `fact` = 'F', `r` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'R' or
        'B', `r` contains the row scale factors for A, D_R, such that A
        is multiplied on the left by D_R; each element of `r` is
        positive.

    c : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `c` need not be set.

        If `fact` = 'F' and `equed` = 'C' or 'B', `c` must contain the
        column scale factors for A, D_C; each element of `c` must be
        positive.

        On exit: if `fact` = 'F', `c` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'C' or
        'B', `c` contains the row scale factors for A, D_C; each element
        of `c` is positive.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `trans` = 'N' and `equed` = 'R' or 'B', `b` is overwritten by
        D_R B.

        If `trans` = 'T' or 'C' and `equed` = 'C' or 'B', `b` is
        overwritten by D_C B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` != 'N', and the solution to the equilibrated system is
        D_C^-1X if `trans` = 'N' and `equed` = 'C' or 'B', or D_R^-1X if
        `trans` = 'T' or 'C' and `equed` = 'R' or 'B'.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    recip_growth_factor : float
        If no exception or warning is raised, the reciprocal pivot
        growth factor ||A||/||U||, where ||.|| denotes the maximum
        absolute element norm. If `recip_growth_factor`≪1, the stability
        of the LU factorization of (equilibrated) A could be poor. This
        also means that the solution `x`, condition estimate `rcond`,
        and forward error bound `ferr` could be unreliable. If the
        factorization fails with `errno` in 1 ... `n`, then
        `recip_growth_factor` contains the reciprocal pivot growth
        factor for the leading errno columns of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -5)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -6)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -12)
            On entry, error in parameter `equed`.

            Constraint: this argument = 'N', 'R', 'C' or 'B'.

        (`errno` -13)
            On entry, error in parameter `r`.

        (`errno` -14)
            On entry, error in parameter `c`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            U is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgbsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, work[max(1, 3*n)].
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-4,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-5,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-6,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldafb_py = utils._EngineFloat64ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb_cls = utils._EngineFloat64ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _afb = _afb_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'R', 'r', 'C', 'c', 'B', 'b'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'R\', \'r\', \'C\', \'c\', \'B\', \'b\').'),
                ],
                errcodes=(-12,),
                entity='equed',
            ).manage()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
    )
    _r = _r_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and _equed_py in ('R', 'r', 'B', 'b'):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(r, {'fun_name': fun_name, 'entity': 'r'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-13,),
                entity='r',
            ).manage()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
    )
    _c = _c_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and _equed_py in ('C', 'c', 'B', 'b'):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(c, {'fun_name': fun_name, 'entity': 'c'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-14,),
                entity='c',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(max(1, 3*_n_py))
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _trans,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _ipiv,
        _equed,
        _r,
        _c,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _fact._length_,
        _trans._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _afb_cls.refresh_py(_afb)
        _ipiv_cls.refresh_py(_ipiv)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _r_cls.refresh_py(_r)
        _c_cls.refresh_py(_c)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _recip_growth_factor_py = utils._EngineFloat64ArrayType.get_element(_work, {'fun_name': fun_name, 'entity': '_work'}, (0,), validate=False)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond', 'recip_growth_factor'],
        )(
            _equed_py,
            _rcond_py,
            _recip_growth_factor_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgbtrf(m, n, kl, ku, ab, ipiv):
    u"""
    LU factorization of real m by n band matrix.

    For full information please refer to the NAG Library document for
    f07bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bdf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    ab : float, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: the m by n matrix A.

        See [Further Comments] for further details.

        On exit: if `errno` >= 0, `ab` is overwritten by details of the
        factorization.

    ipiv : int, shape (min(`m`, `n`)), modified in place
        On exit: the pivot indices that define the permutation matrix.
        At the ith step, if `ipiv`[i-1] > i then row i of the matrix A
        was interchanged with row `ipiv`[i-1], for i = 1,2,...,min(m,
        n). `ipiv`[i-1] <= i indicates that, at the ith step, a row
        interchange was not required.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, and division by zero will occur if it is
            used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgbtrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Output, ipiv[min(m, n)].
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _kl,
        _ku,
        _ab,
        _ldab,
        _ipiv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, and division by zero will occur if it is used to solve',
                    'a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgbtrs(trans, n, kl, ku, nrhs, ab, ipiv, b):
    u"""
    Solution of real band system of linear equations, multiple
    right-hand sides, matrix already factorized by ``dgbtrf``.

    For full information please refer to the NAG Library document for
    f07be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bef.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          AX = B is solved for X.

        `trans` = 'T' or 'C'

          A^TX = B is solved for X.

    n : int
        n, the order of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : float, shape (2*`kl`+`ku`+1, `n`)
        The LU factorization of A, as returned by ``dgbtrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``dgbtrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgbtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07beft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _ipiv,
        _b,
        _ldb,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dgbequ(m, n, kl, ku, ab, r, c):
    u"""
    Computes row and column scalings intended to equilibrate a real
    banded matrix and reduce its condition number.

    For full information please refer to the NAG Library document for
    f07bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bff.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    kl : int
        k_l, the number of subdiagonals of the matrix A.

    ku : int
        k_u, the number of superdiagonals of the matrix A.

    ab : float, shape (`kl`+`ku`+1, `n`)
        The m by n band matrix A whose scaling factors are to be
        computed.

        See [Further Comments] for further details.

    r : float, shape (`m`), modified in place
        On exit: if no exception or warning is raised or `errno` > `m`,
        `r` contains the row scale factors, the diagonal elements of
        D_R. The elements of `r` will be positive.

    c : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `c` contains the
        column scale factors, the diagonal elements of D_C. The elements
        of `c` will be positive.

    Returns
    -------
    rowcnd : float
        If no exception or warning is raised or `errno` > `m`, this
        argument contains the ratio of the smallest value of `r`[i-1] to
        the largest value of `r`[i-1]. If `rowcnd` >= 0.1 and `amax` is
        neither too large nor too small, it is not worth scaling by D_R.

    colcnd : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `c`[i-1] to the largest value of
        `c`[i-1].

        If `colcnd` >= 0.1, it is not worth scaling by D_C.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`m`))
            Row *<value>* of A is exactly zero.

        (`errno` i > `m`)
            Column *<value>* of A is exactly zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgbequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, r[m].
        utils._EngineFloat64ArrayType.c_type, # Output, c[n].
        utils._BYREF_FLOAT64, # Output, rowcnd.
        utils._BYREF_FLOAT64, # Output, colcnd.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _rowcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _colcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _kl,
        _ku,
        _ab,
        _ldab,
        _r,
        _c,
        _rowcnd,
        _colcnd,
        _amax,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Row {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Column {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _r_cls.refresh_py(_r)
        _c_cls.refresh_py(_c)
        _rowcnd_py = utils._EngineFloat64ScalarType.to_py(_rowcnd)
        _colcnd_py = utils._EngineFloat64ScalarType.to_py(_colcnd)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rowcnd', 'colcnd', 'amax'],
        )(
            _rowcnd_py,
            _colcnd_py,
            _amax_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgbcon(norm, n, kl, ku, ab, ipiv, anorm):
    u"""
    Estimate condition number of real band matrix, matrix already
    factorized by ``dgbtrf``.

    For full information please refer to the NAG Library document for
    f07bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bgf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    n : int
        n, the order of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    ab : float, shape (2*`kl`+`ku`+1, `n`)
        The LU factorization of A, as returned by ``dgbtrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``dgbtrf``.

    anorm : float
        If `norm` = '1' or 'O', the 1-norm of the original matrix A.

        If `norm` = 'I', the infinity-norm of the original matrix A.

        `anorm` may be computed by calling ``blas.dlangb`` with the same
        value for the argument `norm`.

        `anorm` must be computed either before calling ``dgbtrf`` or
        else from a copy of the original matrix A (see [Example]).

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -8)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgbcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-8,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _n,
        _kl,
        _ku,
        _ab,
        _ldab,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _norm._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dgbrfs(trans, n, kl, ku, nrhs, ab, afb, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real band system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bhf.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the linear equations for which X is the
        computed solution.

        `trans` = 'N'

          The linear equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The linear equations are of the form A^TX = B.

    n : int
        n, the order of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : float, shape (`kl`+`ku`+1, `n`)
        The original n by n band matrix A as supplied to ``dgbtrf`` but
        with reduced requirements since the matrix is not factorized.

        See [Further Comments] for further details.

    afb : float, shape (2*`kl`+`ku`+1, `n`)
        The LU factorization of A, as returned by ``dgbtrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``dgbtrf``.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``dgbtrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgbrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldafb_py = utils._EngineFloat64ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb = utils._EngineFloat64ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zgbsv(n, kl, ku, nrhs, ab, ipiv, b):
    u"""
    Computes the solution to a complex banded system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07bn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bnf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : complex, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        See [Further Comments] for further details.

        On exit: if `errno` >= 0, `ab` is overwritten by details of the
        factorization.

    ipiv : int, shape (`n`), modified in place
        On exit: if no constraints are violated, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -3)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgbsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-2,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-3,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _ipiv,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgbsvx(fact, trans, n, kl, ku, nrhs, ab, afb, ipiv, equed, r, c, b, x, ferr, berr):
    u"""
    Uses the LU factorization to compute the solution, error-bound and
    condition estimate for a complex banded system of linear equations.

    For full information please refer to the NAG Library document for
    f07bp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `afb` and `ipiv` contain the factorized form of A. If `equed`
          != 'N', the matrix A has been equilibrated with scaling
          factors given by `r` and `c`. `ab`, `afb` and `ipiv` are not
          modified.

        `fact` = 'N'

          The matrix A will be copied to `afb` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `afb` and factorized.

    trans : str, length 1
        Specifies the form of the system of equations.

        `trans` = 'N'

          AX = B (No transpose).

        `trans` = 'T'

          A^TX = B (Transpose).

        `trans` = 'C'

          A^HX = B (Conjugate transpose).

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : complex, shape (`kl`+`ku`+1, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        See [Further Comments] for further details.

        If `fact` = 'F' and `equed` != 'N', A must have been
        equilibrated by the scaling factors in `r` and/or `c`.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `ab` is not modified.

        If `equed` != 'N' then, if no constraints are violated, A is
        scaled as follows:

        - if `equed` = 'R', A = D_r A;

        - if `equed` = 'C', A = AD_c;

        - if `equed` = 'B', A = D_r AD_c.

    afb : complex, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: if `fact` = 'N' or 'E', `afb` need not be set.

        If `fact` = 'F', details of the LU factorization of the n by n
        band matrix A, as computed by ``zgbtrf``.

        If `equed` != 'N', `afb` is the factorized form of the
        equilibrated matrix A.

        On exit: if `fact` = 'F', `afb` is unchanged from entry.

        Otherwise, if no constraints are violated, then if `fact` = 'N',
        `afb` returns details of the LU factorization of the band matrix
        A, and if `fact` = 'E', `afb` returns details of the LU
        factorization of the equilibrated band matrix A (see the
        description of `ab` for the form of the equilibrated matrix).

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `ipiv` need not be set.

        If `fact` = 'F', `ipiv` contains the pivot indices from the
        factorization A = LU, as computed by ``dgbtrf``; row i of the
        matrix was interchanged with row `ipiv`[i-1].

        On exit: if `fact` = 'F', `ipiv` is unchanged from entry.

        Otherwise, if no constraints are violated, `ipiv` contains the
        pivot indices that define the permutation matrix P; at the ith
        step row i of the matrix was interchanged with row `ipiv`[i-1].
        `ipiv`[i-1] = i indicates a row interchange was not required.

        If `fact` = 'N', the pivot indices are those corresponding to
        the factorization A = LU of the original matrix A.

        If `fact` = 'E', the pivot indices are those corresponding to
        the factorization of A = LU of the equilibrated matrix A.

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'R', row equilibration, i.e., A has been
          premultiplied by D_R;

        - if `equed` = 'C', column equilibration, i.e., A has been
          postmultiplied by D_C;

        - if `equed` = 'B', both row and column equilibration, i.e., A
          has been replaced by D_R AD_C.

    r : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `r` need not be set.

        If `fact` = 'F' and `equed` = 'R' or 'B', `r` must contain the
        row scale factors for A, D_R; each element of `r` must be
        positive.

        On exit: if `fact` = 'F', `r` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'R' or
        'B', `r` contains the row scale factors for A, D_R, such that A
        is multiplied on the left by D_R; each element of `r` is
        positive.

    c : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `c` need not be set.

        If `fact` = 'F' and `equed` = 'C' or 'B', `c` must contain the
        column scale factors for A, D_C; each element of `c` must be
        positive.

        On exit: if `fact` = 'F', `c` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'C' or
        'B', `c` contains the row scale factors for A, D_C; each element
        of `c` is positive.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `trans` = 'N' and `equed` = 'R' or 'B', `b` is overwritten by
        D_R B.

        If `trans` = 'T' or 'C' and `equed` = 'C' or 'B', `b` is
        overwritten by D_C B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` != 'N', and the solution to the equilibrated system is
        D_C^-1X if `trans` = 'N' and `equed` = 'C' or 'B', or D_R^-1X if
        `trans` = 'T' or 'C' and `equed` = 'R' or 'B'.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    recip_growth_factor : float
        If no exception or warning is raised, the reciprocal pivot
        growth factor ||A||/||U||, where ||.|| denotes the maximum
        absolute element norm. If `recip_growth_factor`≪1, the stability
        of the LU factorization of (equilibrated) A could be poor. This
        also means that the solution `x`, condition estimate `rcond`,
        and forward error bound `ferr` could be unreliable. If the
        factorization fails with `errno` in 1 ... `n`, then
        `recip_growth_factor` contains the reciprocal pivot growth
        factor for the leading errno columns of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -5)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -6)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -12)
            On entry, error in parameter `equed`.

            Constraint: this argument = 'N', 'R', 'C' or 'B'.

        (`errno` -13)
            On entry, error in parameter `r`.

        (`errno` -14)
            On entry, error in parameter `c`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            U is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgbsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[max(1, n)].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-4,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-5,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-6,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldafb_py = utils._EngineComplex128ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb_cls = utils._EngineComplex128ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _afb = _afb_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'R', 'r', 'C', 'c', 'B', 'b'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'R\', \'r\', \'C\', \'c\', \'B\', \'b\').'),
                ],
                errcodes=(-12,),
                entity='equed',
            ).manage()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
    )
    _r = _r_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and _equed_py in ('R', 'r', 'B', 'b'):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(r, {'fun_name': fun_name, 'entity': 'r'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-13,),
                entity='r',
            ).manage()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
    )
    _c = _c_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and _equed_py in ('C', 'c', 'B', 'b'):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(c, {'fun_name': fun_name, 'entity': 'c'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-14,),
                entity='c',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(max(1, _n_py))
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _trans,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _ipiv,
        _equed,
        _r,
        _c,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _trans._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _afb_cls.refresh_py(_afb)
        _ipiv_cls.refresh_py(_ipiv)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _r_cls.refresh_py(_r)
        _c_cls.refresh_py(_c)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _recip_growth_factor_py = utils._EngineFloat64ArrayType.get_element(_rwork, {'fun_name': fun_name, 'entity': '_rwork'}, (0,), validate=False)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond', 'recip_growth_factor'],
        )(
            _equed_py,
            _rcond_py,
            _recip_growth_factor_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgbtrf(m, n, kl, ku, ab, ipiv):
    u"""
    LU factorization of complex m by n band matrix.

    For full information please refer to the NAG Library document for
    f07br

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07brf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    ab : complex, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: the m by n matrix A.

        See [Further Comments] for further details.

        On exit: if `errno` >= 0, `ab` is overwritten by details of the
        factorization.

    ipiv : int, shape (min(`m`, `n`)), modified in place
        On exit: the pivot indices that define the permutation matrix.
        At the ith step, if `ipiv`[i-1] > i then row i of the matrix A
        was interchanged with row `ipiv`[i-1], for i = 1,2,...,min(m,
        n). `ipiv`[i-1] <= i indicates that, at the ith step, a row
        interchange was not required.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, and division by zero will occur if it is
            used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgbtrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07brft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Output, ipiv[min(m, n)].
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _kl,
        _ku,
        _ab,
        _ldab,
        _ipiv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, and division by zero will occur if it is used to solve',
                    'a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgbtrs(trans, n, kl, ku, nrhs, ab, ipiv, b):
    u"""
    Solution of complex band system of linear equations, multiple
    right-hand sides, matrix already factorized by ``zgbtrf``.

    For full information please refer to the NAG Library document for
    f07bs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bsf.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          AX = B is solved for X.

        `trans` = 'T'

          A^TX = B is solved for X.

        `trans` = 'C'

          A^HX = B is solved for X.

    n : int
        n, the order of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : complex, shape (2*`kl`+`ku`+1, `n`)
        The LU factorization of A, as returned by ``zgbtrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``zgbtrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgbtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _ipiv,
        _b,
        _ldb,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zgbequ(m, n, kl, ku, ab, r, c):
    u"""
    Computes row and column scalings intended to equilibrate a complex
    banded matrix and reduce its condition number.

    For full information please refer to the NAG Library document for
    f07bt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07btf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    kl : int
        k_l, the number of subdiagonals of the matrix A.

    ku : int
        k_u, the number of superdiagonals of the matrix A.

    ab : complex, shape (`kl`+`ku`+1, `n`)
        The m by n band matrix A whose scaling factors are to be
        computed.

        See [Further Comments] for further details.

    r : float, shape (`m`), modified in place
        On exit: if no exception or warning is raised or `errno` > `m`,
        `r` contains the row scale factors, the diagonal elements of
        D_R. The elements of `r` will be positive.

    c : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `c` contains the
        column scale factors, the diagonal elements of D_C. The elements
        of `c` will be positive.

    Returns
    -------
    rowcnd : float
        If no exception or warning is raised or `errno` > `m`, this
        argument contains the ratio of the smallest value of `r`[i-1] to
        the largest value of `r`[i-1]. If `rowcnd` >= 0.1 and `amax` is
        neither too large nor too small, it is not worth scaling by D_R.

    colcnd : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `c`[i-1] to the largest value of
        `c`[i-1].

        If `colcnd` >= 0.1, it is not worth scaling by D_C.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`m`))
            Row *<value>* of A is exactly zero.

        (`errno` i > `m`)
            Column *<value>* of A is exactly zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgbequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07btft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, r[m].
        utils._EngineFloat64ArrayType.c_type, # Output, c[n].
        utils._BYREF_FLOAT64, # Output, rowcnd.
        utils._BYREF_FLOAT64, # Output, colcnd.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _rowcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _colcnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _kl,
        _ku,
        _ab,
        _ldab,
        _r,
        _c,
        _rowcnd,
        _colcnd,
        _amax,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Row {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _m_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Column {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _r_cls.refresh_py(_r)
        _c_cls.refresh_py(_c)
        _rowcnd_py = utils._EngineFloat64ScalarType.to_py(_rowcnd)
        _colcnd_py = utils._EngineFloat64ScalarType.to_py(_colcnd)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rowcnd', 'colcnd', 'amax'],
        )(
            _rowcnd_py,
            _colcnd_py,
            _amax_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgbcon(norm, n, kl, ku, ab, ipiv, anorm):
    u"""
    Estimate condition number of complex band matrix, matrix already
    factorized by ``zgbtrf``.

    For full information please refer to the NAG Library document for
    f07bu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07buf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    n : int
        n, the order of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    ab : complex, shape (2*`kl`+`ku`+1, `n`)
        The LU factorization of A, as returned by ``zgbtrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``zgbtrf``.

    anorm : float
        If `norm` = '1' or 'O', the 1-norm of the original matrix A.

        If `norm` = 'I', the infinity-norm of the original matrix A.

        `anorm` may be computed by calling ``blas.zlangb`` with the same
        value for the argument `norm`.

        `anorm` must be computed either before calling ``zgbtrf`` or
        else from a copy of the original matrix A (see [Example]).

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -8)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgbcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07buft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-8,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _n,
        _kl,
        _ku,
        _ab,
        _ldab,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _rwork,
        _info,
        _norm._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zgbrfs(trans, n, kl, ku, nrhs, ab, afb, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex band system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07bv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07bvf.html

    Parameters
    ----------
    trans : str, length 1
        Indicates the form of the linear equations for which X is the
        computed solution as follows:

        `trans` = 'N'

          The linear equations are of the form AX = B.

        `trans` = 'T'

          The linear equations are of the form A^TX = B.

        `trans` = 'C'

          The linear equations are of the form A^HX = B.

    n : int
        n, the order of the matrix A.

    kl : int
        k_l, the number of subdiagonals within the band of the matrix A.

    ku : int
        k_u, the number of superdiagonals within the band of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : complex, shape (`kl`+`ku`+1, `n`)
        The original n by n band matrix A as supplied to ``zgbtrf`` but
        with reduced requirements since the matrix is not factorized.

        See [Further Comments] for further details.

    afb : complex, shape (2*`kl`+`ku`+1, `n`)
        The LU factorization of A, as returned by ``zgbtrf``.

    ipiv : int, shape (`n`)
        The pivot indices, as returned by ``zgbtrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zgbtrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -4)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgbrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07bvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-3,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-4,),
            entity='ku',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldafb_py = utils._EngineComplex128ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb = utils._EngineComplex128ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dgtsv(n, nrhs, dl, d, du, b):
    u"""
    Computes the solution to a real tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07caf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: if no constraints are violated, `dl` is overwritten by
        the (n-2) elements of the second superdiagonal of the upper
        triangular matrix U from the LU factorization of A, in
        `dl`[0],`dl`[1],...,`dl`[n-3].

    d : float, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: if no constraints are violated, `d` is overwritten by
        the n diagonal elements of the upper triangular matrix U from
        the LU factorization of A.

    du : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) superdiagonal elements of the
        matrix A.

        On exit: if no constraints are violated, `du` is overwritten by
        the (n-1) elements of the first superdiagonal of U.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero, and the
            solution has not been computed. The factorization has not
            been completed unless `n` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgtsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dl[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, du[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _dl_cls = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    )
    _dl = _dl_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _du_cls = utils._EngineFloat64ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    )
    _du = _du_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the solution has not been computed. The factorization has not been',
                    'completed unless n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dl_cls.refresh_py(_dl)
        _d_cls.refresh_py(_d)
        _du_cls.refresh_py(_du)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, x, ferr, berr):
    u"""
    Uses the LU factorization to compute the solution, error-bound and
    condition estimate for a real tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `dlf`, `df`, `duf`, `du2` and `ipiv` contain the factorized
          form of the matrix A. `dlf`, `df`, `duf`, `du2` and `ipiv`
          will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `dlf`, `df` and `duf` and
          factorized.

    trans : str, length 1
        Specifies the form of the system of equations.

        `trans` = 'N'

          AX = B (No transpose).

        `trans` = 'T' or 'C'

          A^TX = B (Transpose).

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : float, shape (`n`-1)
        The (n-1) subdiagonal elements of A.

    d : float, shape (`n`)
        The n diagonal elements of A.

    du : float, shape (`n`-1)
        The (n-1) superdiagonal elements of A.

    dlf : float, shape (`n`-1), modified in place
        On entry: if `fact` = 'F', `dlf` contains the (n-1) multipliers
        that define the matrix L from the LU factorization of A.

        On exit: if `fact` = 'N', `dlf` contains the (n-1) multipliers
        that define the matrix L from the LU factorization of A.

    df : float, shape (`n`), modified in place
        On entry: if `fact` = 'F', `df` contains the n diagonal elements
        of the upper triangular matrix U from the LU factorization of A.

        On exit: if `fact` = 'N', `df` contains the n diagonal elements
        of the upper triangular matrix U from the LU factorization of A.

    duf : float, shape (`n`-1), modified in place
        On entry: if `fact` = 'F', `duf` contains the (n-1) elements of
        the first superdiagonal of U.

        On exit: if `fact` = 'N', `duf` contains the (n-1) elements of
        the first superdiagonal of U.

    du2 : float, shape (`n`-2), modified in place
        On entry: if `fact` = 'F', `du2` contains the (n-2) elements of
        the second superdiagonal of U.

        On exit: if `fact` = 'N', `du2` contains the (n-2) elements of
        the second superdiagonal of U.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains the pivot indices
        from the LU factorization of A.

        On exit: if `fact` = 'N', `ipiv` contains the pivot indices from
        the LU factorization of A; row i of the matrix was interchanged
        with row `ipiv`[i-1]. `ipiv`[i-1] will always be either i or
        i+1; `ipiv`[i-1] = i indicates a row interchange was not
        required.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i < `n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has not been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` (i > 0) and (`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            U is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgtsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, dl[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, du[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dlf[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, df[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, duf[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _dl = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineFloat64ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _dlf_cls = utils._EngineFloat64ArrayType(
        dlf, {'fun_name': fun_name, 'entity': 'dlf'},
        exp_shape=(_n_py-1,),
    )
    _dlf = _dlf_cls.to_ctypes()
    _df_cls = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    )
    _df = _df_cls.to_ctypes()
    _duf_cls = utils._EngineFloat64ArrayType(
        duf, {'fun_name': fun_name, 'entity': 'duf'},
        exp_shape=(_n_py-1,),
    )
    _duf = _duf_cls.to_ctypes()
    _du2_cls = utils._EngineFloat64ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    )
    _du2 = _du2_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _trans,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _dlf,
        _df,
        _duf,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _fact._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py < _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has not been completed, but the factor U is exactly',
                    'singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > 0 and _info_py == _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dlf_cls.refresh_py(_dlf)
        _df_cls.refresh_py(_df)
        _duf_cls.refresh_py(_duf)
        _du2_cls.refresh_py(_du2)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def dgttrf(n, dl, d, du, du2, ipiv):
    u"""
    LU factorization of real tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f07cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cdf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    dl : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: is overwritten by the (n-1) multipliers that define the
        matrix L of the LU factorization of A.

    d : float, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: is overwritten by the n diagonal elements of the upper
        triangular matrix U from the LU factorization of A.

    du : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) superdiagonal elements of the
        matrix A.

        On exit: is overwritten by the (n-1) elements of the first
        superdiagonal of U.

    du2 : float, shape (`n`-2), modified in place
        On exit: contains the (n-2) elements of the second superdiagonal
        of U.

    ipiv : int, shape (`n`), modified in place
        On exit: contains the n pivot indices that define the
        permutation matrix P. At the ith step, row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] will always be
        either i or (i+1), `ipiv`[i-1] = i indicating that a row
        interchange was not performed.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, and division by zero will occur if it is
            used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgttrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07cdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dl[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, du[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, du2[n-2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._BYREF_INT, # Output, info.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _dl_cls = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    )
    _dl = _dl_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _du_cls = utils._EngineFloat64ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    )
    _du = _du_cls.to_ctypes()
    _du2_cls = utils._EngineFloat64ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
        check_contents=False,
    )
    _du2 = _du2_cls.to_ctypes(can_reference_contents=False)
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, and division by zero will occur if it is used to solve',
                    'a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dl_cls.refresh_py(_dl)
        _d_cls.refresh_py(_d)
        _du_cls.refresh_py(_du)
        _du2_cls.refresh_py(_du2)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b):
    u"""
    Solves a real tridiagonal system of linear equations using the LU
    factorization computed by ``dgttrf``.

    For full information please refer to the NAG Library document for
    f07ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cef.html

    Parameters
    ----------
    trans : str, length 1
        Specifies the equations to be solved as follows:

        `trans` = 'N'

          Solve AX = B for X.

        `trans` = 'T' or 'C'

          Solve A^TX = B for X.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : float, shape (`n`-1)
        Must contain the (n-1) multipliers that define the matrix L of
        the LU factorization of A.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the upper triangular
        matrix U from the LU factorization of A.

    du : float, shape (`n`-1)
        Must contain the (n-1) elements of the first superdiagonal of U.

    du2 : float, shape (`n`-2)
        Must contain the (n-2) elements of the second superdiagonal of
        U.

    ipiv : int, shape (`n`)
        Must contain the n pivot indices that define the permutation
        matrix P. At the ith step, row i of the matrix was interchanged
        with row `ipiv`[i-1], and `ipiv`[i-1] must always be either i or
        (i+1), `ipiv`[i-1] = i indicating that a row interchange was not
        performed.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgttrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ceft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, dl[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, du[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _dl = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineFloat64ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _du2 = utils._EngineFloat64ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dgtcon(norm, n, dl, d, du, du2, ipiv, anorm):
    u"""
    Estimates the reciprocal of the condition number of a real
    tridiagonal matrix using the LU factorization computed by
    ``dgttrf``.

    For full information please refer to the NAG Library document for
    f07cg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cgf.html

    Parameters
    ----------
    norm : str, length 1
        Specifies the norm to be used to estimate kappa(A).

        `norm` = '1' or 'O'

          Estimate kappa_1(A).

        `norm` = 'I'

          Estimate kappa_infinity (A).

    n : int
        n, the order of the matrix A.

    dl : float, shape (`n`-1)
        Must contain the (n-1) multipliers that define the matrix L of
        the LU factorization of A.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the upper triangular
        matrix U from the LU factorization of A.

    du : float, shape (`n`-1)
        Must contain the (n-1) elements of the first superdiagonal of U.

    du2 : float, shape (`n`-2)
        Must contain the (n-2) elements of the second superdiagonal of
        U.

    ipiv : int, shape (`n`)
        Must contain the n pivot indices that define the permutation
        matrix P. At the ith step, row i of the matrix was interchanged
        with row `ipiv`[i-1], and `ipiv`[i-1] must always be either i or
        (i+1), `ipiv`[i-1] = i indicating that a row interchange was not
        performed.

    anorm : float
        If `norm` = '1' or 'O', the 1-norm of the original matrix A.

        If `norm` = 'I', the infinity-norm of the original matrix A.

        `anorm` may be computed by calling ``blas.dlangt`` with the same
        value for the argument `norm`.

        `anorm` must be computed either before calling ``dgttrf`` or
        else from a copy of the original matrix A (see [Example]).

    Returns
    -------
    rcond : float
        Contains an estimate of the reciprocal condition number.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgtcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07cgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, dl[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, du[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _call_vendor = utils._EngineState._literal_true_cint
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _dl = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineFloat64ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _du2 = utils._EngineFloat64ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-8,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _norm,
        _n,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _norm._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real tridiagonal system of
    linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07ch

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07chf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies the equations to be solved as follows:

        `trans` = 'N'

          Solve AX = B for X.

        `trans` = 'T' or 'C'

          Solve A^TX = B for X.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : float, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the matrix A.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the matrix A.

    du : float, shape (`n`-1)
        Must contain the (n-1) superdiagonal elements of the matrix A.

    dlf : float, shape (`n`-1)
        Must contain the (n-1) multipliers that define the matrix L of
        the LU factorization of A.

    df : float, shape (`n`)
        Must contain the n diagonal elements of the upper triangular
        matrix U from the LU factorization of A.

    duf : float, shape (`n`-1)
        Must contain the (n-1) elements of the first superdiagonal of U.

    du2 : float, shape (`n`-2)
        Must contain the (n-2) elements of the second superdiagonal of
        U.

    ipiv : int, shape (`n`)
        Must contain the n pivot indices that define the permutation
        matrix P. At the ith step, row i of the matrix was interchanged
        with row `ipiv`[i-1], and `ipiv`[i-1] must always be either i or
        (i+1), `ipiv`[i-1] = i indicating that a row interchange was not
        performed.

    b : float, shape (`n`, `nrhs`)
        The n by r matrix of right-hand sides B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r initial solution matrix X.

        On exit: the n by r refined solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x^_j||_infinity <= `ferr`[j-1], where
        x^_j is the jth column of the computed solution returned in the
        array `x` and x_j is the corresponding column of the exact
        solution X. The estimate is almost always a slight overestimate
        of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: estimate of the component-wise relative backward error
        of each computed solution vector x^_j (i.e., the smallest
        relative change in any element of A or B that makes x^_j an
        exact solution).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dgtrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07chft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, dl[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, du[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, dlf[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, df[n].
        utils._EngineFloat64ArrayType.c_type, # Input, duf[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _dl = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineFloat64ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _dlf = utils._EngineFloat64ArrayType(
        dlf, {'fun_name': fun_name, 'entity': 'dlf'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _df = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _duf = utils._EngineFloat64ArrayType(
        duf, {'fun_name': fun_name, 'entity': 'duf'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _du2 = utils._EngineFloat64ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _dlf,
        _df,
        _duf,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zgtsv(n, nrhs, dl, d, du, b):
    u"""
    Computes the solution to a complex tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07cn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cnf.html

    Parameters
    ----------
    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: if no constraints are violated, `dl` is overwritten by
        the (n-2) elements of the second superdiagonal of the upper
        triangular matrix U from the LU factorization of A, in
        `dl`[0],`dl`[1],...,`dl`[n-3].

    d : complex, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: if no constraints are violated, `d` is overwritten by
        the n diagonal elements of the upper triangular matrix U from
        the LU factorization of A.

    du : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) superdiagonal elements of the
        matrix A.

        On exit: if no constraints are violated, `du` is overwritten by
        the (n-1) elements of the first superdiagonal of U.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero, and the
            solution has not been computed. The factorization has not
            been completed unless `n` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgtsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07cnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, dl[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, du[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _dl_cls = utils._EngineComplex128ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    )
    _dl = _dl_cls.to_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _du_cls = utils._EngineComplex128ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    )
    _du = _du_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the solution has not been computed. The factorization has not been',
                    'completed unless n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dl_cls.refresh_py(_dl)
        _d_cls.refresh_py(_d)
        _du_cls.refresh_py(_du)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, x, ferr, berr):
    u"""
    Uses the LU factorization to compute the solution, error-bound and
    condition estimate for a complex tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07cp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `dlf`, `df`, `duf`, `du2` and `ipiv` contain the factorized
          form of the matrix A. `dlf`, `df`, `duf`, `du2` and `ipiv`
          will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `dlf`, `df` and `duf` and
          factorized.

    trans : str, length 1
        Specifies the form of the system of equations.

        `trans` = 'N'

          AX = B (No transpose).

        `trans` = 'T'

          A^TX = B (Transpose).

        `trans` = 'C'

          A^HX = B (Conjugate transpose).

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : complex, shape (`n`-1)
        The (n-1) subdiagonal elements of A.

    d : complex, shape (`n`)
        The n diagonal elements of A.

    du : complex, shape (`n`-1)
        The (n-1) superdiagonal elements of A.

    dlf : complex, shape (`n`-1), modified in place
        On entry: if `fact` = 'F', `dlf` contains the (n-1) multipliers
        that define the matrix L from the LU factorization of A.

        On exit: if `fact` = 'N', `dlf` contains the (n-1) multipliers
        that define the matrix L from the LU factorization of A.

    df : complex, shape (`n`), modified in place
        On entry: if `fact` = 'F', `df` contains the n diagonal elements
        of the upper triangular matrix U from the LU factorization of A.

        On exit: if `fact` = 'N', `df` contains the n diagonal elements
        of the upper triangular matrix U from the LU factorization of A.

    duf : complex, shape (`n`-1), modified in place
        On entry: if `fact` = 'F', `duf` contains the (n-1) elements of
        the first superdiagonal of U.

        On exit: if `fact` = 'N', `duf` contains the (n-1) elements of
        the first superdiagonal of U.

    du2 : complex, shape (`n`-2), modified in place
        On entry: if `fact` = 'F', `du2` contains the (n-2) elements of
        the second superdiagonal of U.

        On exit: if `fact` = 'N', `du2` contains the (n-2) elements of
        the second superdiagonal of U.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains the pivot indices
        from the LU factorization of A.

        On exit: if `fact` = 'N', `ipiv` contains the pivot indices from
        the LU factorization of A; row i of the matrix was interchanged
        with row `ipiv`[i-1]. `ipiv`[i-1] will always be either i or
        i+1; `ipiv`[i-1] = i indicates a row interchange was not
        required.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i < `n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has not been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` (i > 0) and (`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            U is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgtsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07cpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, dl[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, du[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, dlf[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, df[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, duf[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _dl = utils._EngineComplex128ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineComplex128ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _dlf_cls = utils._EngineComplex128ArrayType(
        dlf, {'fun_name': fun_name, 'entity': 'dlf'},
        exp_shape=(_n_py-1,),
    )
    _dlf = _dlf_cls.to_ctypes()
    _df_cls = utils._EngineComplex128ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    )
    _df = _df_cls.to_ctypes()
    _duf_cls = utils._EngineComplex128ArrayType(
        duf, {'fun_name': fun_name, 'entity': 'duf'},
        exp_shape=(_n_py-1,),
    )
    _duf = _duf_cls.to_ctypes()
    _du2_cls = utils._EngineComplex128ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    )
    _du2 = _du2_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _trans,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _dlf,
        _df,
        _duf,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py < _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has not been completed, but the factor U is exactly',
                    'singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > 0 and _info_py == _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dlf_cls.refresh_py(_dlf)
        _df_cls.refresh_py(_df)
        _duf_cls.refresh_py(_duf)
        _du2_cls.refresh_py(_du2)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def zgttrf(n, dl, d, du, du2, ipiv):
    u"""
    LU factorization of complex tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f07cr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07crf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    dl : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: is overwritten by the (n-1) multipliers that define the
        matrix L of the LU factorization of A.

    d : complex, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: is overwritten by the n diagonal elements of the upper
        triangular matrix U from the LU factorization of A.

    du : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) superdiagonal elements of the
        matrix A.

        On exit: is overwritten by the (n-1) elements of the first
        superdiagonal of U.

    du2 : complex, shape (`n`-2), modified in place
        On exit: contains the (n-2) elements of the second superdiagonal
        of U.

    ipiv : int, shape (`n`), modified in place
        On exit: contains the n pivot indices that define the
        permutation matrix P. At the ith step, row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] will always be
        either i or (i+1), `ipiv`[i-1] = i indicating that a row
        interchange was not performed.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor U is
            exactly singular, and division by zero will occur if it is
            used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgttrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07crft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, dl[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, du[n-1].
        utils._EngineComplex128ArrayType.c_type, # Output, du2[n-2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._BYREF_INT, # Output, info.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _dl_cls = utils._EngineComplex128ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    )
    _dl = _dl_cls.to_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _du_cls = utils._EngineComplex128ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    )
    _du = _du_cls.to_ctypes()
    _du2_cls = utils._EngineComplex128ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
        check_contents=False,
    )
    _du2 = _du2_cls.to_ctypes(can_reference_contents=False)
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor U',
                    'is exactly singular, and division by zero will occur if it is used to solve',
                    'a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dl_cls.refresh_py(_dl)
        _d_cls.refresh_py(_d)
        _du_cls.refresh_py(_du)
        _du2_cls.refresh_py(_du2)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b):
    u"""
    Solves a complex tridiagonal system of linear equations using the LU
    factorization computed by ``dgttrf``.

    For full information please refer to the NAG Library document for
    f07cs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07csf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies the equations to be solved as follows:

        `trans` = 'N'

          Solve AX = B for X.

        `trans` = 'T'

          Solve A^TX = B for X.

        `trans` = 'C'

          Solve A^HX = B for X.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : complex, shape (`n`-1)
        Must contain the (n-1) multipliers that define the matrix L of
        the LU factorization of A.

    d : complex, shape (`n`)
        Must contain the n diagonal elements of the upper triangular
        matrix U from the LU factorization of A.

    du : complex, shape (`n`-1)
        Must contain the (n-1) elements of the first superdiagonal of U.

    du2 : complex, shape (`n`-2)
        Must contain the (n-2) elements of the second superdiagonal of
        U.

    ipiv : int, shape (`n`)
        Must contain the n pivot indices that define the permutation
        matrix P. At the ith step, row i of the matrix was interchanged
        with row `ipiv`[i-1], and `ipiv`[i-1] must always be either i or
        (i+1), `ipiv`[i-1] = i indicating that a row interchange was not
        performed.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgttrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07csft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, dl[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, du[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _dl = utils._EngineComplex128ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineComplex128ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _du2 = utils._EngineComplex128ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zgtcon(norm, n, dl, d, du, du2, ipiv, anorm):
    u"""
    Estimates the reciprocal of the condition number of a complex
    tridiagonal matrix using the LU factorization computed by
    ``dgttrf``.

    For full information please refer to the NAG Library document for
    f07cu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cuf.html

    Parameters
    ----------
    norm : str, length 1
        Specifies the norm to be used to estimate kappa(A).

        `norm` = '1' or 'O'

          Estimate kappa_1(A).

        `norm` = 'I'

          Estimate kappa_infinity (A).

    n : int
        n, the order of the matrix A.

    dl : complex, shape (`n`-1)
        Must contain the (n-1) multipliers that define the matrix L of
        the LU factorization of A.

    d : complex, shape (`n`)
        Must contain the n diagonal elements of the upper triangular
        matrix U from the LU factorization of A.

    du : complex, shape (`n`-1)
        Must contain the (n-1) elements of the first superdiagonal of U.

    du2 : complex, shape (`n`-2)
        Must contain the (n-2) elements of the second superdiagonal of
        U.

    ipiv : int, shape (`n`)
        Must contain the n pivot indices that define the permutation
        matrix P. At the ith step, row i of the matrix was interchanged
        with row `ipiv`[i-1], and `ipiv`[i-1] must always be either i or
        (i+1), `ipiv`[i-1] = i indicating that a row interchange was not
        performed.

    anorm : float
        If `norm` = '1' or 'O', the 1-norm of the original matrix A.

        If `norm` = 'I', the infinity-norm of the original matrix A.

        `anorm` may be computed by calling ``blas.zlangt`` with the same
        value for the argument `norm`.

        `anorm` must be computed either before calling ``zgttrf`` or
        else from a copy of the original matrix A (see [Example]).

    Returns
    -------
    rcond : float
        Contains an estimate of the reciprocal condition number.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgtcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07cuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, dl[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, du[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _call_vendor = utils._EngineState._literal_true_cint
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _dl = utils._EngineComplex128ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineComplex128ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _du2 = utils._EngineComplex128ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-8,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _norm,
        _n,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _info,
        _norm._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex tridiagonal system of
    linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07cv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07cvf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies the equations to be solved as follows:

        `trans` = 'N'

          Solve AX = B for X.

        `trans` = 'T'

          Solve A^TX = B for X.

        `trans` = 'C'

          Solve A^HX = B for X.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    dl : complex, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the matrix A.

    d : complex, shape (`n`)
        Must contain the n diagonal elements of the matrix A.

    du : complex, shape (`n`-1)
        Must contain the (n-1) superdiagonal elements of the matrix A.

    dlf : complex, shape (`n`-1)
        Must contain the (n-1) multipliers that define the matrix L of
        the LU factorization of A.

    df : complex, shape (`n`)
        Must contain the n diagonal elements of the upper triangular
        matrix U from the LU factorization of A.

    duf : complex, shape (`n`-1)
        Must contain the (n-1) elements of the first superdiagonal of U.

    du2 : complex, shape (`n`-2)
        Must contain the (n-2) elements of the second superdiagonal of
        U.

    ipiv : int, shape (`n`)
        Must contain the n pivot indices that define the permutation
        matrix P. At the ith step, row i of the matrix was interchanged
        with row `ipiv`[i-1], and `ipiv`[i-1] must always be either i or
        (i+1), `ipiv`[i-1] = i indicating that a row interchange was not
        performed.

    b : complex, shape (`n`, `nrhs`)
        The n by r matrix of right-hand sides B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r initial solution matrix X.

        On exit: the n by r refined solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x^_j||_infinity <= `ferr`[j-1], where
        x^_j is the jth column of the computed solution returned in the
        array `x` and x_j is the corresponding column of the exact
        solution X. The estimate is almost always a slight overestimate
        of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: estimate of the component-wise relative backward error
        of each computed solution vector x^_j (i.e., the smallest
        relative change in any element of A or B that makes x^_j an
        exact solution).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zgtrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07cvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, dl[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, du[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, dlf[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, df[n].
        utils._EngineComplex128ArrayType.c_type, # Input, duf[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, du2[n-2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _dl = utils._EngineComplex128ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _d = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _du = utils._EngineComplex128ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _dlf = utils._EngineComplex128ArrayType(
        dlf, {'fun_name': fun_name, 'entity': 'dlf'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _df = utils._EngineComplex128ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _duf = utils._EngineComplex128ArrayType(
        duf, {'fun_name': fun_name, 'entity': 'duf'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _du2 = utils._EngineComplex128ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _dlf,
        _df,
        _duf,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dposv(uplo, n, nrhs, a, b):
    u"""
    Computes the solution to a real symmetric positive definite system
    of linear equations.

    For full information please refer to the NAG Library document for
    f07fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07faf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^TU or A = LL^T.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dposv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dposvx(fact, uplo, n, nrhs, a, af, equed, s, b, x, ferr, berr):
    u"""
    Uses the Cholesky factorization to compute the solution, error-bound
    and condition estimate for a real symmetric positive definite system
    of linear equations.

    For full information please refer to the NAG Library document for
    f07fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `af` contains the factorized form of A. If `equed` = 'Y', the
          matrix A has been equilibrated with scaling factors given by
          `s`. `a` and `af` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `af` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `af` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        If `fact` = 'F' and `equed` = 'Y', `a` must have been
        equilibrated by the scaling factor in `s` as D_S AD_S.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `a` is not modified.

        If `fact` = 'E' and `equed` = 'Y', `a` is overwritten by D_S
        AD_S.

    af : float, shape (`n`, `n`), modified in place
        On entry: if `fact` = 'F', `af` contains the triangular factor U
        or L from the Cholesky factorization A = U^TU or A = LL^T, in
        the same storage format as `a`. If `equed` != 'N', `af` is the
        factorized form of the equilibrated matrix D_S AD_S.

        On exit: if `fact` = 'N', `af` returns the triangular factor U
        or L from the Cholesky factorization A = U^TU or A = LL^T of the
        original matrix A.

        If `fact` = 'E', `af` returns the triangular factor U or L from
        the Cholesky factorization A = U^TU or A = LL^T of the
        equilibrated matrix A (see the description of `a` for the form
        of the equilibrated matrix).

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'Y', equilibration was performed, i.e., A has
          been replaced by D_S AD_S.

    s : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `s` need not be set.

        If `fact` = 'F' and `equed` = 'Y', `s` must contain the scale
        factors, D_S, for A; each element of `s` must be positive.

        On exit: if `fact` = 'F', `s` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'Y', `s`
        contains the scale factors, D_S, for A; each element of `s` is
        positive.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `equed` = 'Y', `b` is overwritten by D_S B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` = 'Y', and the solution to the equilibrated system is
        D_S^-1X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of the equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -9)
            On entry, error in parameter `equed`.

            Constraint: `equed` = 'N' or 'Y'.

        (`errno` -10)
            On entry, error in parameter `s`.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            U (or L) is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dposvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af_cls = utils._EngineFloat64ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _af = _af_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'Y', 'y'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'Y\', \'y\').'),
                ],
                errcodes=(-9,),
                entity='equed',
            ).manage()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
    )
    _s = _s_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and (_equed_py in ('Y', 'y')):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(s, {'fun_name': fun_name, 'entity': 's'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-10,),
                entity='s',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _equed,
        _s,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _fact._length_,
        _uplo._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U (or L) is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _af_cls.refresh_py(_af)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _s_cls.refresh_py(_s)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond'],
        )(
            _equed_py,
            _rcond_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dsposv(uplo, n, nrhs, a, b, x):
    u"""
    Computes the solution to a real symmetric positive definite system
    of linear equations using mixed precision arithmetic.

    For full information please refer to the NAG Library document for
    f07fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fcf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric positive definite matrix A.

        On exit: if iterative refinement has been successfully used (no
        exception or warning is raised and `itera` >= 0), then `a` is
        unchanged. If full precision factorization has been used (no
        exception or warning is raised and `itera` < 0), then the array
        A contains the factor U or L from the Cholesky factorization A =
        U^TU or A = LL^T.

    b : float, shape (`n`, `nrhs`)
        The right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Returns
    -------
    itera : int
        Information on the progress of the interative refinement
        process.

        `itera` < 0

          Iterative refinement has failed for one of the reasons given
          below, full precision factorization has been performed
          instead.

        [table omitted]

        `itera` > 0

          Iterative refinement has been sucessfully used. `itera`
          returns the number of iterations.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsposv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineFloat32ArrayType.c_type, # Workspace, swork.
        utils._BYREF_INT, # Output, itera.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _swork = utils._EngineFloat32ArrayType.empty_ctypes(0)
    _itera = utils._EngineIntScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _x,
        _ldx,
        _work,
        _swork,
        _itera,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _x_cls.refresh_py(_x)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        return_data = _itera_py
    _err_class.manage()
    return return_data

def dpotrf(uplo, n, a):
    u"""
    Cholesky factorization of real symmetric positive definite matrix.

    For full information please refer to the NAG Library document for
    f07fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fdf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric positive definite matrix A.

        On exit: the upper or lower triangle of A is overwritten by the
        Cholesky factor U or L as specified by `uplo`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. To factorize a symmetric
            matrix which is not positive definite, call ``dsytrf``
            instead.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpotrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the factorization could not be completed. Hence A itself',
                    'is not positive definite. This may indicate an error in forming the',
                    'matrix A. To factorize a symmetric matrix which is not',
                    'positive definite, call dsytrf instead.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dpotrs(uplo, n, nrhs, a, b):
    u"""
    Solution of real symmetric positive definite system of linear
    equations, multiple right-hand sides, matrix already factorized by
    ``dpotrf``.

    For full information please refer to the NAG Library document for
    f07fe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        The Cholesky factor of A, as returned by ``dpotrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpotrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07feft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dpoequ(n, a, s):
    u"""
    Computes row and column scalings intended to equilibrate a real
    symmetric positive definite matrix and reduce its condition number.

    For full information please refer to the NAG Library document for
    f07ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fff.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`)
        The matrix A whose scaling factors are to be computed. Only the
        diagonal elements of the array `a` are referenced.

    s : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `s` contains the
        diagonal elements of the scaling matrix S.

    Returns
    -------
    scond : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `s` to the largest value of `s`.
        If `scond` >= 0.1 and `amax` is neither too large nor too small,
        it is not worth scaling by S.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The *<value>*th diagonal element of A is not positive (and
            hence A cannot be positive definite).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpoequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ffft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, s[n].
        utils._BYREF_FLOAT64, # Output, scond.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _scond = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _a,
        _lda,
        _s,
        _scond,
        _amax,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}th diagonal element of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    '(and hence A cannot be positive definite).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _scond_py = utils._EngineFloat64ScalarType.to_py(_scond)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scond', 'amax'],
        )(
            _scond_py,
            _amax_py,
        )
    _err_class.manage()
    return return_data

def dpocon(uplo, n, a, anorm):
    u"""
    Estimate condition number of real symmetric positive definite
    matrix, matrix already factorized by ``dpotrf``.

    For full information please refer to the NAG Library document for
    f07fg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fgf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`)
        The Cholesky factor of A, as returned by ``dpotrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.dlansy`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``dpotrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpocon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-5,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dporfs(uplo, n, nrhs, a, af, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real symmetric positive
    definite system of linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07fh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fhf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        The n by n original symmetric positive definite matrix A as
        supplied to ``dpotrf``.

    af : float, shape (`n`, `n`)
        The Cholesky factor of A, as returned by ``dpotrf``.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``dpotrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dporfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af = utils._EngineFloat64ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dpotri(uplo, n, a):
    u"""
    Inverse of real symmetric positive definite matrix, matrix already
    factorized by ``dpotrf``.

    For full information please refer to the NAG Library document for
    f07fj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fjf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the upper triangular matrix U if `uplo` = 'U' or the
        lower triangular matrix L if `uplo` = 'L', as returned by
        ``dpotrf``.

        On exit: U is overwritten by the upper triangle of A^-1 if
        `uplo` = 'U'; L is overwritten by the lower triangle of A^-1 if
        `uplo` = 'L'.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of the Cholesky factor is zero;
            the Cholesky factor is singular and the inverse of A cannot
            be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpotri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the Cholesky factor is zero;'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the Cholesky factor is singular and the inverse of A cannot',
                    'be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zposv(uplo, n, nrhs, a, b):
    u"""
    Computes the solution to a complex Hermitian positive definite
    system of linear equations.

    For full information please refer to the NAG Library document for
    f07fn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fnf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^HU or A = LL^H.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zposv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zposvx(fact, uplo, n, nrhs, a, af, equed, s, b, x, ferr, berr):
    u"""
    Uses the Cholesky factorization to compute the solution, error-bound
    and condition estimate for a complex Hermitian positive definite
    system of linear equations.

    For full information please refer to the NAG Library document for
    f07fp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `af` contains the factorized form of A. If `equed` = 'Y', the
          matrix A has been equilibrated with scaling factors given by
          `s`. `a` and `af` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `af` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `af` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        If `fact` = 'F' and `equed` = 'Y', `a` must have been
        equilibrated by the scaling factor in `s` as D_S AD_S.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `a` is not modified.

        If `fact` = 'E' and `equed` = 'Y', `a` is overwritten by D_S
        AD_S.

    af : complex, shape (`n`, `n`), modified in place
        On entry: if `fact` = 'F', `af` contains the triangular factor U
        or L from the Cholesky factorization A = U^HU or A = LL^H, in
        the same storage format as `a`. If `equed` != 'N', `af` is the
        factorized form of the equilibrated matrix D_S AD_S.

        On exit: if `fact` = 'N', `af` returns the triangular factor U
        or L from the Cholesky factorization A = U^HU or A = LL^H of the
        original matrix A.

        If `fact` = 'E', `af` returns the triangular factor U or L from
        the Cholesky factorization A = U^HU or A = LL^H of the
        equilibrated matrix A (see the description of `a` for the form
        of the equilibrated matrix).

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'Y', equilibration was performed, i.e., A has
          been replaced by D_S AD_S.

    s : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `s` need not be set.

        If `fact` = 'F' and `equed` = 'Y', `s` must contain the scale
        factors, D_S, for A; each element of `s` must be positive.

        On exit: if `fact` = 'F', `s` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'Y', `s`
        contains the scale factors, D_S, for A; each element of `s` is
        positive.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `equed` = 'Y', `b` is overwritten by D_S B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` = 'Y', and the solution to the equilibrated system is
        D_S^-1X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of the equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -9)
            On entry, error in parameter `equed`.

            Constraint: `equed` = 'N' or 'Y'.

        (`errno` -10)
            On entry, error in parameter `s`.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            U (or L) is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zposvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af_cls = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _af = _af_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'Y', 'y'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'Y\', \'y\').'),
                ],
                errcodes=(-9,),
                entity='equed',
            ).manage()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
    )
    _s = _s_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and (_equed_py in ('Y', 'y')):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(s, {'fun_name': fun_name, 'entity': 's'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-10,),
                entity='s',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _equed,
        _s,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _uplo._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U (or L) is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _af_cls.refresh_py(_af)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _s_cls.refresh_py(_s)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond'],
        )(
            _equed_py,
            _rcond_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zcposv(uplo, n, nrhs, a, b, x):
    u"""
    Computes the solution to a complex Hermitian positive definite
    system of linear equations using mixed precision arithmetic.

    For full information please refer to the NAG Library document for
    f07fq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fqf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian positive definite matrix A.

        On exit: if iterative refinement has been successfully used (no
        exception or warning is raised and `itera` >= 0), then `a` is
        unchanged. If full precision factorization has been used (no
        exception or warning is raised and `itera` < 0), then the array
        A contains the factor U or L from the Cholesky factorization A =
        U^HU or A = LL^H.

    b : complex, shape (`n`, `nrhs`)
        The right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Returns
    -------
    itera : int
        Information on the progress of the interative refinement
        process.

        `itera` < 0

          Iterative refinement has failed for one of the reasons given
          below, full precision factorization has been performed
          instead.

        [table omitted]

        `itera` > 0

          Iterative refinement has been sucessfully used. `itera`
          returns the number of iterations.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zcposv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineComplex64ArrayType.c_type, # Workspace, swork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, itera.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _swork = utils._EngineComplex64ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _itera = utils._EngineIntScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _x,
        _ldx,
        _work,
        _swork,
        _rwork,
        _itera,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _x_cls.refresh_py(_x)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        return_data = _itera_py
    _err_class.manage()
    return return_data

def zpotrf(uplo, n, a):
    u"""
    Cholesky factorization of complex Hermitian positive definite
    matrix.

    For full information please refer to the NAG Library document for
    f07fr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07frf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian positive definite matrix A.

        On exit: the upper or lower triangle of A is overwritten by the
        Cholesky factor U or L as specified by `uplo`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. To factorize a Hermitian
            matrix which is not positive definite, call ``zhetrf``
            instead.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpotrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07frft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the factorization could not be completed. Hence A itself',
                    'is not positive definite. This may indicate an error in forming the',
                    'matrix A. To factorize a Hermitian matrix which is not',
                    'positive definite, call zhetrf instead.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zpotrs(uplo, n, nrhs, a, b):
    u"""
    Solution of complex Hermitian positive definite system of linear
    equations, multiple right-hand sides, matrix already factorized by
    ``zpotrf``.

    For full information please refer to the NAG Library document for
    f07fs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The Cholesky factor of A, as returned by ``zpotrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpotrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zpoequ(n, a, s):
    u"""
    Computes row and column scalings intended to equilibrate a complex
    Hermitian positive definite matrix and reduce its condition number.

    For full information please refer to the NAG Library document for
    f07ft

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ftf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        The matrix A whose scaling factors are to be computed. Only the
        diagonal elements of the array `a` are referenced.

    s : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `s` contains the
        diagonal elements of the scaling matrix S.

    Returns
    -------
    scond : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `s` to the largest value of `s`.
        If `scond` >= 0.1 and `amax` is neither too large nor too small,
        it is not worth scaling by S.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The *<value>*th diagonal element of A is not positive (and
            hence A cannot be positive definite).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpoequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ftft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, s[n].
        utils._BYREF_FLOAT64, # Output, scond.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _scond = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _a,
        _lda,
        _s,
        _scond,
        _amax,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}th diagonal element of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    '(and hence A cannot be positive definite).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _scond_py = utils._EngineFloat64ScalarType.to_py(_scond)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scond', 'amax'],
        )(
            _scond_py,
            _amax_py,
        )
    _err_class.manage()
    return return_data

def zpocon(uplo, n, a, anorm):
    u"""
    Estimate condition number of complex Hermitian positive definite
    matrix, matrix already factorized by ``zpotrf``.

    For full information please refer to the NAG Library document for
    f07fu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fuf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        The Cholesky factor of A, as returned by ``zpotrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlanhe`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zpotrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpocon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-5,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _anorm,
        _rcond,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zporfs(uplo, n, nrhs, a, af, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex Hermitian positive
    definite system of linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07fv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The n by n original Hermitian positive definite matrix A as
        supplied to ``zpotrf``.

    af : complex, shape (`n`, `n`)
        The Cholesky factor of A, as returned by ``zpotrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zpotrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zporfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zpotri(uplo, n, a):
    u"""
    Inverse of complex Hermitian positive definite matrix, matrix
    already factorized by ``zpotrf``.

    For full information please refer to the NAG Library document for
    f07fw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07fwf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the upper triangular matrix U if `uplo` = 'U' or the
        lower triangular matrix L if `uplo` = 'L', as returned by
        ``zpotrf``.

        On exit: U is overwritten by the upper triangle of A^-1 if
        `uplo` = 'U'; L is overwritten by the lower triangle of A^-1 if
        `uplo` = 'L'.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of the Cholesky factor is zero;
            the Cholesky factor is singular and the inverse of A cannot
            be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpotri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07fwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the Cholesky factor is zero;'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the Cholesky factor is singular and the inverse of A cannot',
                    'be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dppsv(uplo, n, nrhs, ap, b):
    u"""
    Computes the solution to a real symmetric positive definite system
    of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f07ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gaf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed by columns.

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^TU or A = LL^T, in the
        same storage format as A.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dppsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, x, ferr, berr):
    u"""
    Uses the Cholesky factorization to compute the solution, error-bound
    and condition estimate for a real symmetric positive definite system
    of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f07gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `afp` contains the factorized form of A. If `equed` = 'Y', the
          matrix A has been equilibrated with scaling factors given by
          `s`. `ap` and `afp` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `afp` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `afp` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: if `fact` = 'F' and `equed` = 'Y', `ap` must contain
        the equilibrated matrix D_S AD_S; otherwise, `ap` must contain
        the n by n symmetric matrix A, packed by columns.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `ap` is not modified.

        If `fact` = 'E' and `equed` = 'Y', `ap` is overwritten by D_S
        AD_S.

    afp : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: if `fact` = 'F', `afp` contains the triangular factor
        U or L from the Cholesky factorization A = U^TU or A = LL^T, in
        the same storage format as `ap`. If `equed` = 'Y', `afp` is the
        factorized form of the equilibrated matrix D_S AD_S.

        On exit: if `fact` = 'N' or if `fact` = 'E' and `equed` = 'N',
        `afp` returns the triangular factor U or L from the Cholesky
        factorization A = U^TU or A = LL^T of the original matrix A.

        If `fact` = 'E' and `equed` = 'Y', `afp` returns the triangular
        factor U or L from the Cholesky factorization A = U^TU or A =
        LL^T of the equilibrated matrix A (see the description of `ap`
        for the form of the equilibrated matrix).

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'Y', equilibration was performed, i.e., A has
          been replaced by D_S AD_S.

    s : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `s` need not be set.

        If `fact` = 'F' and `equed` = 'Y', `s` must contain the scale
        factors, D_S, for A; each element of `s` must be positive.

        On exit: if `fact` = 'F', `s` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'Y', `s`
        contains the scale factors, D_S, for A; each element of `s` is
        positive.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `equed` = 'Y', `b` is overwritten by D_S B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` = 'Y', and the solution to the equilibrated system is
        D_S^-1X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of the equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -7)
            On entry, error in parameter `equed`.

            Constraint: `equed` = 'N' or 'Y'.

        (`errno` -8)
            On entry, error in parameter `s`.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            U (or L) is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dppsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, afp[n*(n+1)//2].
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _afp_cls = utils._EngineFloat64ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _afp = _afp_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'Y', 'y'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'Y\', \'y\').'),
                ],
                errcodes=(-7,),
                entity='equed',
            ).manage()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
    )
    _s = _s_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and (_equed_py in ('Y', 'y')):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(s, {'fun_name': fun_name, 'entity': 's'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-8,),
                entity='s',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _equed,
        _s,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _fact._length_,
        _uplo._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U (or L) is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _afp_cls.refresh_py(_afp)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _s_cls.refresh_py(_s)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond'],
        )(
            _equed_py,
            _rcond_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dpptrf(uplo, n, ap):
    u"""
    Cholesky factorization of real symmetric positive definite matrix,
    packed storage.

    For full information please refer to the NAG Library document for
    f07gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gdf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed by columns.

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^TU or A = LL^T, in the
        same storage format as A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. To factorize a symmetric
            matrix which is not positive definite, call ``dsptrf``
            instead.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpptrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the factorization could not be completed. Hence A itself',
                    'is not positive definite. This may indicate an error in forming the',
                    'matrix A. To factorize a symmetric matrix which is not',
                    'positive definite, call dsptrf instead.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def dpptrs(uplo, n, nrhs, ap, b):
    u"""
    Solution of real symmetric positive definite system of linear
    equations, multiple right-hand sides, matrix already factorized by
    ``dpptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07ge

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : float, shape (`n`*(`n`+1)/2)
        The Cholesky factor of A stored in packed form, as returned by
        ``dpptrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpptrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07geft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dppequ(uplo, n, ap, s):
    u"""
    Computes row and column scalings intended to equilibrate a real
    symmetric positive definite matrix and reduce its condition number,
    packed storage.

    For full information please refer to the NAG Library document for
    f07gf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gff.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored in the array `ap`, as follows:

        `uplo` = 'U'

          The upper triangle of A is stored.

        `uplo` = 'L'

          The lower triangle of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n symmetric matrix A, packed by columns.

        Only the elements of `ap` corresponding to the diagonal elements
        A are referenced.

    s : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `s` contains the
        diagonal elements of the scaling matrix S.

    Returns
    -------
    scond : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `s` to the largest value of `s`.
        If `scond` >= 0.1 and `amax` is neither too large nor too small,
        it is not worth scaling by S.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The *<value>*th diagonal element of A is not positive (and
            hence A cannot be positive definite).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dppequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, s[n].
        utils._BYREF_FLOAT64, # Output, scond.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _scond = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _s,
        _scond,
        _amax,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}th diagonal element of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    '(and hence A cannot be positive definite).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _scond_py = utils._EngineFloat64ScalarType.to_py(_scond)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scond', 'amax'],
        )(
            _scond_py,
            _amax_py,
        )
    _err_class.manage()
    return return_data

def dppcon(uplo, n, ap, anorm):
    u"""
    Estimate condition number of real symmetric positive definite
    matrix, matrix already factorized by ``dpptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07gg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ggf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2)
        The Cholesky factor of A stored in packed form, as returned by
        ``dpptrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.dlansp`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``dpptrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dppcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ggft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-4,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dpprfs(uplo, n, nrhs, ap, afp, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real symmetric positive
    definite system of linear equations, multiple right-hand sides,
    packed storage.

    For full information please refer to the NAG Library document for
    f07gh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ghf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n original symmetric positive definite matrix A as
        supplied to ``dpptrf``.

    afp : float, shape (`n`*(`n`+1)/2)
        The Cholesky factor of A stored in packed form, as returned by
        ``dpptrf``.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``dpptrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpprfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ghft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input, afp[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp = utils._EngineFloat64ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dpptri(uplo, n, ap):
    u"""
    Inverse of real symmetric positive definite matrix, matrix already
    factorized by ``dpptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07gj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gjf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the Cholesky factor of A stored in packed form, as
        returned by ``dpptrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of the Cholesky factor is zero;
            the Cholesky factor is singular and the inverse of A cannot
            be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpptri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the Cholesky factor is zero;'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the Cholesky factor is singular and the inverse of A cannot',
                    'be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zppsv(uplo, n, nrhs, ap, b):
    u"""
    Computes the solution to a complex Hermitian positive definite
    system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f07gn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gnf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n Hermitian matrix A, packed by columns.

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^HU or A = LL^H, in the
        same storage format as A.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zppsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, x, ferr, berr):
    u"""
    Uses the Cholesky factorization to compute the solution, error-bound
    and condition estimate for a complex Hermitian positive definite
    system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f07gp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `afp` contains the factorized form of A. If `equed` = 'Y', the
          matrix A has been equilibrated with scaling factors given by
          `s`. `ap` and `afp` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `afp` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `afp` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: if `fact` = 'F' and `equed` = 'Y', `ap` must contain
        the equilibrated matrix D_S AD_S; otherwise, `ap` must contain
        the n by n Hermitian matrix A, packed by columns.

        On exit: if `fact` = 'F' or 'N', or if `fact` = 'E' and `equed`
        = 'N', `ap` is not modified.

        If `fact` = 'E' and `equed` = 'Y', `ap` is overwritten by D_S
        AD_S.

    afp : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: if `fact` = 'F', `afp` contains the triangular factor
        U or L from the Cholesky factorization A = U^HU or A = LL^H, in
        the same storage format as `ap`. If `equed` = 'Y', `afp` is the
        factorized form of the equilibrated matrix D_S AD_S.

        On exit: if `fact` = 'N' or if `fact` = 'E' and `equed` = 'N',
        `afp` returns the triangular factor U or L from the Cholesky
        factorization A = U^HU or A = LL^H of the original matrix A.

        If `fact` = 'E' and `equed` = 'Y', `afp` returns the triangular
        factor U or L from the Cholesky factorization A = U^HU or A =
        LL^H of the equilibrated matrix A (see the description of `ap`
        for the form of the equilibrated matrix).

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'Y', equilibration was performed, i.e., A has
          been replaced by D_S AD_S.

    s : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `s` need not be set.

        If `fact` = 'F' and `equed` = 'Y', `s` must contain the scale
        factors, D_S, for A; each element of `s` must be positive.

        On exit: if `fact` = 'F', `s` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'Y', `s`
        contains the scale factors, D_S, for A; each element of `s` is
        positive.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `equed` = 'Y', `b` is overwritten by D_S B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` = 'Y', and the solution to the equilibrated system is
        D_S^-1X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of the equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -7)
            On entry, error in parameter `equed`.

            Constraint: `equed` = 'N' or 'Y'.

        (`errno` -8)
            On entry, error in parameter `s`.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            U (or L) is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zppsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, afp[n*(n+1)//2].
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _afp_cls = utils._EngineComplex128ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _afp = _afp_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'Y', 'y'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'Y\', \'y\').'),
                ],
                errcodes=(-7,),
                entity='equed',
            ).manage()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
    )
    _s = _s_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and (_equed_py in ('Y', 'y')):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(s, {'fun_name': fun_name, 'entity': 's'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-8,),
                entity='s',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _equed,
        _s,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _uplo._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U (or L) is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _afp_cls.refresh_py(_afp)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _s_cls.refresh_py(_s)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond'],
        )(
            _equed_py,
            _rcond_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zpptrf(uplo, n, ap):
    u"""
    Cholesky factorization of complex Hermitian positive definite
    matrix, packed storage.

    For full information please refer to the NAG Library document for
    f07gr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07grf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n Hermitian matrix A, packed by columns.

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^HU or A = LL^H, in the
        same storage format as A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. To factorize a Hermitian
            matrix which is not positive definite, call ``zhptrf``
            instead.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpptrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07grft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the factorization could not be completed. Hence A itself',
                    'is not positive definite. This may indicate an error in forming the',
                    'matrix A. To factorize a Hermitian matrix which is not',
                    'positive definite, call zhptrf instead.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def zpptrs(uplo, n, nrhs, ap, b):
    u"""
    Solution of complex Hermitian positive definite system of linear
    equations, multiple right-hand sides, matrix already factorized by
    ``zpptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07gs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The Cholesky factor of A stored in packed form, as returned by
        ``zpptrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpptrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zppequ(uplo, n, ap, s):
    u"""
    Computes row and column scalings intended to equilibrate a complex
    Hermitian positive definite matrix and reduce its condition number,
    packed storage.

    For full information please refer to the NAG Library document for
    f07gt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gtf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored in the array `ap`, as follows:

        `uplo` = 'U'

          The upper triangle of A is stored.

        `uplo` = 'L'

          The lower triangle of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n Hermitian matrix A, packed by columns.

        Only the elements of `ap` corresponding to the diagonal elements
        A are referenced.

    s : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `s` contains the
        diagonal elements of the scaling matrix S.

    Returns
    -------
    scond : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `s` to the largest value of `s`.
        If `scond` >= 0.1 and `amax` is neither too large nor too small,
        it is not worth scaling by S.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The *<value>*th diagonal element of A is not positive (and
            hence A cannot be positive definite).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zppequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, s[n].
        utils._BYREF_FLOAT64, # Output, scond.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _scond = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _s,
        _scond,
        _amax,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}th diagonal element of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    '(and hence A cannot be positive definite).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _scond_py = utils._EngineFloat64ScalarType.to_py(_scond)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scond', 'amax'],
        )(
            _scond_py,
            _amax_py,
        )
    _err_class.manage()
    return return_data

def zppcon(uplo, n, ap, anorm):
    u"""
    Estimate condition number of complex Hermitian positive definite
    matrix, matrix already factorized by ``zpptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07gu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07guf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2)
        The Cholesky factor of A stored in packed form, as returned by
        ``zpptrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlanhp`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zpptrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zppcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07guft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-4,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _anorm,
        _rcond,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zpprfs(uplo, n, nrhs, ap, afp, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex Hermitian positive
    definite system of linear equations, multiple right-hand sides,
    packed storage.

    For full information please refer to the NAG Library document for
    f07gv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n original Hermitian positive definite matrix A as
        supplied to ``zpptrf``.

    afp : complex, shape (`n`*(`n`+1)/2)
        The Cholesky factor of A stored in packed form, as returned by
        ``zpptrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zpptrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpprfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input, afp[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp = utils._EngineComplex128ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zpptri(uplo, n, ap):
    u"""
    Inverse of complex Hermitian positive definite matrix, matrix
    already factorized by ``zpptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07gw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07gwf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the Cholesky factor of A stored in packed form, as
        returned by ``zpptrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of the Cholesky factor is zero;
            the Cholesky factor is singular and the inverse of A cannot
            be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpptri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07gwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the Cholesky factor is zero;'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the Cholesky factor is singular and the inverse of A cannot',
                    'be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dpbsv(uplo, n, kd, nrhs, ab, b):
    u"""
    Computes the solution to a real symmetric positive definite banded
    system of linear equations.

    For full information please refer to the NAG Library document for
    f07ha

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07haf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the symmetric band
        matrix A.

        On exit: if no exception or warning is raised, the triangular
        factor U or L from the Cholesky factorization A = U^TU or A =
        LL^T of the band matrix A, in the same storage format as A.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpbsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07haft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dpbsvx(fact, uplo, n, kd, nrhs, ab, afb, equed, s, b, x, ferr, berr):
    u"""
    Uses the Cholesky factorization to compute the solution, error-bound
    and condition estimate for a real symmetric positive definite banded
    system of linear equations.

    For full information please refer to the NAG Library document for
    f07hb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `afb` contains the factorized form of A. If `equed` = 'Y', the
          matrix A has been equilibrated with scaling factors given by
          `s`. `ab` and `afb` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `afb` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `afb` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the symmetric band
        matrix A, except if `fact` = 'F' and `equed` = 'Y', in which
        case `ab` must contain the equilibrated matrix D_S AD_S.

        On exit: if `fact` = 'E' and `equed` = 'Y', `ab` is overwritten
        by D_S AD_S.

    afb : float, shape (`kd`+1, `n`), modified in place
        On entry: if `fact` = 'F', `afb` contains the triangular factor
        U or L from the Cholesky factorization A = U^TU or A = LL^T of
        the band matrix A, in the same storage format as A. If `equed` =
        'Y', `afb` is the factorized form of the equilibrated matrix A.

        On exit: if `fact` = 'N', `afb` returns the triangular factor U
        or L from the Cholesky factorization A = U^TU or A = LL^T.

        If `fact` = 'E', `afb` returns the triangular factor U or L from
        the Cholesky factorization A = U^TU or A = LL^T of the
        equilibrated matrix A (see the description of `ab` for the form
        of the equilibrated matrix).

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'Y', equilibration was performed, i.e., A has
          been replaced by D_S AD_S.

    s : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `s` need not be set.

        If `fact` = 'F' and `equed` = 'Y', `s` must contain the scale
        factors, D_S, for A; each element of `s` must be positive.

        On exit: if `fact` = 'F', `s` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'Y', `s`
        contains the scale factors, D_S, for A; each element of `s` is
        positive.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `equed` = 'Y', `b` is overwritten by D_S B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` = 'Y', and the solution to the equilibrated system is
        D_S^-1X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of the equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -10)
            On entry, error in parameter `equed`.

            Constraint: `equed` = 'N' or 'Y'.

        (`errno` -11)
            On entry, error in parameter `s`.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            U (or L) is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpbsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldafb_py = utils._EngineFloat64ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb_cls = utils._EngineFloat64ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _afb = _afb_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'Y', 'y'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'Y\', \'y\').'),
                ],
                errcodes=(-10,),
                entity='equed',
            ).manage()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
    )
    _s = _s_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and (_equed_py in ('Y', 'y')):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(s, {'fun_name': fun_name, 'entity': 's'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-11,),
                entity='s',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _equed,
        _s,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _fact._length_,
        _uplo._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U (or L) is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _afb_cls.refresh_py(_afb)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _s_cls.refresh_py(_s)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond'],
        )(
            _equed_py,
            _rcond_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dpbtrf(uplo, n, kd, ab):
    u"""
    Cholesky factorization of real symmetric positive definite band
    matrix.

    For full information please refer to the NAG Library document for
    f07hd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hdf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the n by n symmetric positive definite band matrix A.

        On exit: the upper or lower triangle of A is overwritten by the
        Cholesky factor U or L as specified by `uplo`, using the same
        storage format as described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. There is no function
            specifically designed to factorize a symmetric band matrix
            which is not positive definite; the matrix must be treated
            either as a nonsymmetric band matrix, by calling ``dgbtrf``
            or as a full symmetric matrix, by calling ``dsytrf``.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpbtrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed. Hence A itself is not positive',
                    'definite. This may indicate an error in forming the matrix A. There is no',
                    'function specifically designed to factorize a symmetric band matrix which is',
                    'not positive definite; the matrix must be treated either as a nonsymmetric',
                    'band matrix, by calling dgbtrf or as a full symmetric matrix, by',
                    'calling dsytrf.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        return_data = None
    _err_class.manage()
    return return_data

def dpbtrs(uplo, n, kd, nrhs, ab, b):
    u"""
    Solution of real symmetric positive definite band system of linear
    equations, multiple right-hand sides, matrix already factorized by
    ``dpbtrf``.

    For full information please refer to the NAG Library document for
    f07he

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : float, shape (`kd`+1, `n`)
        The Cholesky factor of A, as returned by ``dpbtrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpbtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07heft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dpbequ(uplo, n, kd, ab, s):
    u"""
    Computes row and column scalings intended to equilibrate a real
    symmetric positive definite banded matrix and reduce its condition
    number.

    For full information please refer to the NAG Library document for
    f07hf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hff.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored in the array `ab`, as follows:

        `uplo` = 'U'

          The upper triangle of A is stored.

        `uplo` = 'L'

          The lower triangle of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    ab : float, shape (`kd`+1, `n`)
        The upper or lower triangle of the symmetric positive definite
        band matrix A whose scaling factors are to be computed.

        Only the elements of the array `ab` corresponding to the
        diagonal elements of A are referenced. (Row (k_d+1) of `ab` when
        `uplo` = 'U', row 1 of `ab` when `uplo` = 'L'.)

    s : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `s` contains the
        diagonal elements of the scaling matrix S.

    Returns
    -------
    scond : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `s` to the largest value of `s`.
        If `scond` >= 0.1 and `amax` is neither too large nor too small,
        it is not worth scaling by S.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The *<value>*th diagonal element of A is not positive (and
            hence A cannot be positive definite).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpbequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, s[n].
        utils._BYREF_FLOAT64, # Output, scond.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _scond = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _s,
        _scond,
        _amax,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}th diagonal element of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    '(and hence A cannot be positive definite).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _scond_py = utils._EngineFloat64ScalarType.to_py(_scond)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scond', 'amax'],
        )(
            _scond_py,
            _amax_py,
        )
    _err_class.manage()
    return return_data

def dpbcon(uplo, n, kd, ab, anorm):
    u"""
    Estimate condition number of real symmetric positive definite band
    matrix, matrix already factorized by ``dpbtrf``.

    For full information please refer to the NAG Library document for
    f07hg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hgf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    ab : float, shape (`kd`+1, `n`)
        The Cholesky factor of A, as returned by ``dpbtrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.dlansb`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``dpbtrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -6)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpbcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-6,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dpbrfs(uplo, n, kd, nrhs, ab, afb, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real symmetric positive
    definite band system of linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07hh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hhf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : float, shape (`kd`+1, `n`)
        The n by n original symmetric positive definite band matrix A as
        supplied to ``dpbtrf``.

    afb : float, shape (`kd`+1, `n`)
        The Cholesky factor of A, as returned by ``dpbtrf``.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``dpbtrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpbrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldafb_py = utils._EngineFloat64ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb = utils._EngineFloat64ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zpbsv(uplo, n, kd, nrhs, ab, b):
    u"""
    Computes the solution to a complex Hermitian positive definite
    banded system of linear equations.

    For full information please refer to the NAG Library document for
    f07hn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hnf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the Hermitian band
        matrix A.

        On exit: if no exception or warning is raised, the triangular
        factor U or L from the Cholesky factorization A = U^HU or A =
        LL^H of the band matrix A, in the same storage format as A.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpbsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zpbsvx(fact, uplo, n, kd, nrhs, ab, afb, equed, s, b, x, ferr, berr):
    u"""
    Uses the Cholesky factorization to compute the solution, error-bound
    and condition estimate for a complex Hermitian positive definite
    banded system of linear equations.

    For full information please refer to the NAG Library document for
    f07hp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A is
        supplied on entry, and if not, whether the matrix A should be
        equilibrated before it is factorized.

        `fact` = 'F'

          `afb` contains the factorized form of A. If `equed` = 'Y', the
          matrix A has been equilibrated with scaling factors given by
          `s`. `ab` and `afb` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `afb` and factorized.

        `fact` = 'E'

          The matrix A will be equilibrated if necessary, then copied to
          `afb` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the Hermitian band
        matrix A, except if `fact` = 'F' and `equed` = 'Y', in which
        case `ab` must contain the equilibrated matrix D_S AD_S.

        On exit: if `fact` = 'E' and `equed` = 'Y', `ab` is overwritten
        by D_S AD_S.

    afb : complex, shape (`kd`+1, `n`), modified in place
        On entry: if `fact` = 'F', `afb` contains the triangular factor
        U or L from the Cholesky factorization A = U^HU or A = LL^H of
        the band matrix A, in the same storage format as A. If `equed` =
        'Y', `afb` is the factorized form of the equilibrated matrix A.

        On exit: if `fact` = 'N', `afb` returns the triangular factor U
        or L from the Cholesky factorization A = U^HU or A = LL^H.

        If `fact` = 'E', `afb` returns the triangular factor U or L from
        the Cholesky factorization A = U^HU or A = LL^H of the
        equilibrated matrix A (see the description of `ab` for the form
        of the equilibrated matrix).

    equed : str, length 1
        If `fact` = 'N' or 'E', `equed` need not be set.

        If `fact` = 'F', `equed` must specify the form of the
        equilibration that was performed as follows:

        - if `equed` = 'N', no equilibration;

        - if `equed` = 'Y', equilibration was performed, i.e., A has
          been replaced by D_S AD_S.

    s : float, shape (`n`), modified in place
        On entry: if `fact` = 'N' or 'E', `s` need not be set.

        If `fact` = 'F' and `equed` = 'Y', `s` must contain the scale
        factors, D_S, for A; each element of `s` must be positive.

        On exit: if `fact` = 'F', `s` is unchanged from entry.

        Otherwise, if no constraints are violated and `equed` = 'Y', `s`
        contains the scale factors, D_S, for A; each element of `s` is
        positive.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if `equed` = 'N', `b` is not modified.

        If `equed` = 'Y', `b` is overwritten by D_S B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X to the original system of
        equations. Note that the arrays A and B are modified on exit if
        `equed` = 'Y', and the solution to the equilibrated system is
        D_S^-1X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    equed : str, length 1
        If `fact` = 'F', this argument is unchanged from entry.

        Otherwise, if no constraints are violated, this argument
        specifies the form of the equilibration that was performed as
        specified above.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal
        condition number of the matrix A (after equilibration if that is
        performed), computed as `rcond` = 1.0/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F', 'N' or 'E'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` -10)
            On entry, error in parameter `equed`.

            Constraint: `equed` = 'N' or 'Y'.

        (`errno` -11)
            On entry, error in parameter `s`.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            U (or L) is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpbsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._BYREF_STRING, # Input/Output, equed(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equed.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\', \'E\', \'e\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldafb_py = utils._EngineComplex128ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb_cls = utils._EngineComplex128ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _afb = _afb_cls.to_ctypes()
    _equed = utils._EngineCharScalarType(
        equed, {'fun_name': fun_name, 'entity': 'equed'}, exp_chrlen=1,
    ).to_ctypes()
    _equed_py = utils._EngineCharScalarType.to_py(_equed)
    if _fact_py in ('F', 'f'):
        if _equed_py not in ('N', 'n', 'Y', 'y'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('equed in (\'N\', \'n\', \'Y\', \'y\').'),
                ],
                errcodes=(-10,),
                entity='equed',
            ).manage()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
    )
    _s = _s_cls.to_ctypes()
    if (_fact_py in ('F', 'f')) and (_equed_py in ('Y', 'y')):
        if not len([_vl_j_py for _vl_j_py in range(1, _n_py+1) if utils._EngineFloat64ArrayType.get_element(s, {'fun_name': fun_name, 'entity': 's'}, (_vl_j_py-1,)) < 0]) == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument fails constraint check.'
                ],
                errcodes=(-11,),
                entity='s',
            ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _equed,
        _s,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _uplo._length_,
        _equed._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'U (or L) is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _afb_cls.refresh_py(_afb)
        _equed_py = utils._EngineCharScalarType.to_py(_equed)
        _s_cls.refresh_py(_s)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['equed', 'rcond'],
        )(
            _equed_py,
            _rcond_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zpbtrf(uplo, n, kd, ab):
    u"""
    Cholesky factorization of complex Hermitian positive definite band
    matrix.

    For full information please refer to the NAG Library document for
    f07hr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hrf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the n by n Hermitian positive definite band matrix A.

        On exit: the upper or lower triangle of A is overwritten by the
        Cholesky factor U or L as specified by `uplo`, using the same
        storage format as described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. There is no function
            specifically designed to factorize a Hermitian band matrix
            which is not positive definite; the matrix must be treated
            either as a nonsymmetric band matrix, by calling ``zgbtrf``
            or as a full Hermitian matrix, by calling ``zhetrf``.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpbtrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed. Hence A itself is not positive',
                    'definite. This may indicate an error in forming the matrix A. There is no',
                    'function specifically designed to factorize a Hermitian band matrix which is',
                    'not positive definite; the matrix must be treated either as a nonsymmetric',
                    'band matrix, by calling zgbtrf or as a full Hermitian matrix, by',
                    'calling zhetrf.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        return_data = None
    _err_class.manage()
    return return_data

def zpbtrs(uplo, n, kd, nrhs, ab, b):
    u"""
    Solution of complex Hermitian positive definite band system of
    linear equations, multiple right-hand sides, matrix already
    factorized by ``zpbtrf``.

    For full information please refer to the NAG Library document for
    f07hs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : complex, shape (`kd`+1, `n`)
        The Cholesky factor of A, as returned by ``zpbtrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpbtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zpbequ(uplo, n, kd, ab, s):
    u"""
    Computes row and column scalings intended to equilibrate a complex
    Hermitian positive definite banded matrix and reduce its condition
    number.

    For full information please refer to the NAG Library document for
    f07ht

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07htf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored in the array `ab`, as follows:

        `uplo` = 'U'

          The upper triangle of A is stored.

        `uplo` = 'L'

          The lower triangle of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    ab : complex, shape (`kd`+1, `n`)
        The upper or lower triangle of the Hermitian positive definite
        band matrix A whose scaling factors are to be computed.

        Only the elements of the array `ab` corresponding to the
        diagonal elements of A are referenced. (Row (k_d+1) of `ab` when
        `uplo` = 'U', row 1 of `ab` when `uplo` = 'L'.)

    s : float, shape (`n`), modified in place
        On exit: if no exception or warning is raised, `s` contains the
        diagonal elements of the scaling matrix S.

    Returns
    -------
    scond : float
        If no exception or warning is raised, this argument contains the
        ratio of the smallest value of `s` to the largest value of `s`.
        If `scond` >= 0.1 and `amax` is neither too large nor too small,
        it is not worth scaling by S.

    amax : float
        max|a_{ij}|. If this argument is very close to overflow or
        underflow, the matrix A should be scaled.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The *<value>*th diagonal element of A is not positive (and
            hence A cannot be positive definite).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpbequ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07htft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, s[n].
        utils._BYREF_FLOAT64, # Output, scond.
        utils._BYREF_FLOAT64, # Output, amax.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _scond = utils._EngineFloat64ScalarType.empty_ctypes()
    _amax = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _s,
        _scond,
        _amax,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}th diagonal element of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    '(and hence A cannot be positive definite).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _scond_py = utils._EngineFloat64ScalarType.to_py(_scond)
        _amax_py = utils._EngineFloat64ScalarType.to_py(_amax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scond', 'amax'],
        )(
            _scond_py,
            _amax_py,
        )
    _err_class.manage()
    return return_data

def zpbcon(uplo, n, kd, ab, anorm):
    u"""
    Estimate condition number of complex Hermitian positive definite
    band matrix, matrix already factorized by ``zpbtrf``.

    For full information please refer to the NAG Library document for
    f07hu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07huf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    ab : complex, shape (`kd`+1, `n`)
        The Cholesky factor of A, as returned by ``zpbtrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlanhb`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zpbtrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -6)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpbcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07huft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-6,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _anorm,
        _rcond,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zpbrfs(uplo, n, kd, nrhs, ab, afb, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex Hermitian positive
    definite band system of linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07hv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07hvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals or subdiagonals of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides.

    ab : complex, shape (`kd`+1, `n`)
        The n by n original Hermitian positive definite band matrix A as
        supplied to ``zpbtrf``.

    afb : complex, shape (`kd`+1, `n`)
        The Cholesky factor of A, as returned by ``zpbtrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zpbtrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpbrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07hvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input, afb[ldafb, n].
        utils._BYREF_INT, # Input, ldafb.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (afb, 'afb'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-3,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldafb_py = utils._EngineComplex128ArrayType.get_nd(afb, {'fun_name': fun_name, 'entity': 'afb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldafb = utils._EngineIntScalarType(
        _ldafb_py, {'fun_name': fun_name, 'entity': '_ldafb'},
    ).to_ctypes()
    _afb = utils._EngineComplex128ArrayType(
        afb, {'fun_name': fun_name, 'entity': 'afb'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _afb,
        _ldafb,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dptsv(n, nrhs, d, e, b):
    u"""
    Computes the solution to a real symmetric positive definite
    tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f07ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jaf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix A.

        On exit: the n diagonal elements of the diagonal matrix D from
        the factorization A = LDL^T.

    e : float, shape (`n`-1), modified in place
        On entry: the (n-1) subdiagonal elements of the tridiagonal
        matrix A.

        On exit: the (n-1) subdiagonal elements of the unit bidiagonal
        factor L from the LDL^T factorization of A. (`e` can also be
        regarded as the superdiagonal of the unit bidiagonal factor U
        from the U^TDU factorization of A.)

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite, and the solution has not been computed. The
            factorization has not been completed unless `n` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dptsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _d,
        _e,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the solution has not been computed.',
                    'The factorization has not been completed unless n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dptsvx(fact, n, nrhs, d, e, df, ef, b, x, ferr, berr):
    u"""
    Uses the LDL^T factorization to compute the solution, error-bound
    and condition estimate for a real symmetric positive definite
    tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f07jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `df` and `ef` contain the factorized form of the matrix A.
          `df` and `ef` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `df` and `ef` and factorized.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`)
        The n diagonal elements of the tridiagonal matrix A.

    e : float, shape (`n`-1)
        The (n-1) subdiagonal elements of the tridiagonal matrix A.

    df : float, shape (`n`), modified in place
        On entry: if `fact` = 'F', `df` must contain the n diagonal
        elements of the diagonal matrix D from the LDL^T factorization
        of A.

        On exit: if `fact` = 'N', `df` contains the n diagonal elements
        of the diagonal matrix D from the LDL^T factorization of A.

    ef : float, shape (`n`-1), modified in place
        On entry: if `fact` = 'F', `ef` must contain the (n-1)
        subdiagonal elements of the unit bidiagonal factor L from the
        LDL^T factorization of A.

        On exit: if `fact` = 'N', `ef` contains the (n-1) subdiagonal
        elements of the unit bidiagonal factor L from the LDL^T
        factorization of A.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: the forward error bound for each solution vector x^_j
        (the jth column of the solution matrix X). If x_j is the true
        solution corresponding to x^_j, `ferr`[j-1] is an estimated
        upper bound for the magnitude of the largest element in
        (x^_j-x_j) divided by the magnitude of the largest element in
        x^_j.

    berr : float, shape (`nrhs`), modified in place
        On exit: the component-wise relative backward error of each
        solution vector x^_j (i.e., the smallest relative change in any
        element of A or B that makes x^_j an exact solution).

    Returns
    -------
    rcond : float
        The reciprocal condition number of the matrix A. If `rcond` is
        less than the machine precision (in particular, if `rcond` =
        0.0), the matrix is singular to working precision. This
        condition is indicated by a return code of `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dptsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, df[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ef[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _df_cls = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    )
    _df = _df_cls.to_ctypes()
    _ef_cls = utils._EngineFloat64ArrayType(
        ef, {'fun_name': fun_name, 'entity': 'ef'},
        exp_shape=(_n_py-1,),
    )
    _ef = _ef_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _n,
        _nrhs,
        _d,
        _e,
        _df,
        _ef,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _info,
        _fact._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _df_cls.refresh_py(_df)
        _ef_cls.refresh_py(_ef)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def dpttrf(n, d, e):
    u"""
    Computes the LDL^T factorization of a real symmetric positive
    definite tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f07jd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jdf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    d : float, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: is overwritten by the n diagonal elements of the
        diagonal matrix D from the LDL^T factorization of A.

    e : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: is overwritten by the (n-1) subdiagonal elements of the
        lower bidiagonal matrix L. (`e` can also be regarded as
        containing the (n-1) superdiagonal elements of the upper
        bidiagonal matrix U.)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i < `n`))
            The leading minor of order *<value>* is not positive
            definite, the factorization could not be completed.

        (`errno` (i > 0) and (`n`))
            The leading minor of order n is not positive definite, the
            factorization was completed, but `d`[`n`-1] <= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpttrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._BYREF_INT, # Output, info.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _d,
        _e,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py < _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > 0 and _info_py == _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order n is not positive definite, the factorization was',
                    'completed, but d[n-1] <= 0.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        return_data = None
    _err_class.manage()
    return return_data

def dpttrs(n, nrhs, d, e, b):
    u"""
    Solves a real symmetric positive definite tridiagonal system using
    the LDL^T factorization computed by ``dpttrf``.

    For full information please refer to the NAG Library document for
    f07je

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jef.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the diagonal matrix D
        from the LDL^T factorization of A.

    e : float, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the unit lower
        bidiagonal matrix L. (`e` can also be regarded as the
        superdiagonal of the unit upper bidiagonal matrix U from the
        U^TDU factorization of A.)

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpttrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _d,
        _e,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dptcon(n, d, e, anorm):
    u"""
    Computes the reciprocal of the condition number of a real symmetric
    positive definite tridiagonal system using the LDL^T factorization
    computed by ``dpttrf``.

    For full information please refer to the NAG Library document for
    f07jg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jgf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the diagonal matrix D
        from the LDL^T factorization of A.

    e : float, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the unit lower
        bidiagonal matrix L. (`e` can also be regarded as the
        superdiagonal of the unit upper bidiagonal matrix U from the
        U^TDU factorization of A.)

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.dlanst`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``dpttrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        The reciprocal condition number, 1/kappa_1(A) =
        1/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dptcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _call_vendor = utils._EngineState._literal_true_cint
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-4,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _n,
        _d,
        _e,
        _anorm,
        _rcond,
        _work,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dptrfs(n, nrhs, d, e, df, ef, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real symmetric positive
    definite tridiagonal system of linear equations, multiple right-hand
    sides.

    For full information please refer to the NAG Library document for
    f07jh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jhf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the matrix of A.

    e : float, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the matrix A.

    df : float, shape (`n`)
        Must contain the n diagonal elements of the diagonal matrix D
        from the LDL^T factorization of A.

    ef : float, shape (`n`)
        Must contain the (n-1) subdiagonal elements of the unit
        bidiagonal matrix L from the LDL^T factorization of A.

    b : float, shape (`n`, `nrhs`)
        The n by r matrix of right-hand sides B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r initial solution matrix X.

        On exit: the n by r refined solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x^_j||_infinity <= `ferr`[j-1], where
        x^_j is the jth column of the computed solution returned in the
        array `x` and x_j is the corresponding column of the exact
        solution X. The estimate is almost always a slight overestimate
        of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: estimate of the component-wise relative backward error
        of each computed solution vector x^_j (i.e., the smallest
        relative change in any element of A or B that makes x^_j an
        exact solution).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dptrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, df[n].
        utils._EngineFloat64ArrayType.c_type, # Input, ef[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _df = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ef = utils._EngineFloat64ArrayType(
        ef, {'fun_name': fun_name, 'entity': 'ef'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _d,
        _e,
        _df,
        _ef,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zptsv(n, nrhs, d, e, b):
    u"""
    Computes the solution to a complex Hermitian positive definite
    tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f07jn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jnf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix A.

        On exit: the n diagonal elements of the diagonal matrix D from
        the factorization A = LDL^H.

    e : complex, shape (`n`-1), modified in place
        On entry: the (n-1) subdiagonal elements of the tridiagonal
        matrix A.

        On exit: the (n-1) subdiagonal elements of the unit bidiagonal
        factor L from the LDL^H factorization of A. (`e` can also be
        regarded as the superdiagonal of the unit bidiagonal factor U
        from the U^HDU factorization of A.)

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite, and the solution has not been computed. The
            factorization has not been completed unless `n` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zptsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-2,),
            entity='nrhs',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _nrhs,
        _d,
        _e,
        _b,
        _ldb,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the solution has not been computed.',
                    'The factorization has not been completed unless n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zptsvx(fact, n, nrhs, d, e, df, ef, b, x, ferr, berr):
    u"""
    Uses the LDL^T factorization to compute the solution, error-bound
    and condition estimate for a complex Hermitian positive definite
    tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f07jp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `df` and `ef` contain the factorized form of the matrix A.
          `df` and `ef` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `df` and `ef` and factorized.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`)
        The n diagonal elements of the tridiagonal matrix A.

    e : complex, shape (`n`-1)
        The (n-1) subdiagonal elements of the tridiagonal matrix A.

    df : float, shape (`n`), modified in place
        On entry: if `fact` = 'F', `df` must contain the n diagonal
        elements of the diagonal matrix D from the LDL^H factorization
        of A.

        On exit: if `fact` = 'N', `df` contains the n diagonal elements
        of the diagonal matrix D from the LDL^H factorization of A.

    ef : complex, shape (`n`-1), modified in place
        On entry: if `fact` = 'F', `ef` must contain the (n-1)
        subdiagonal elements of the unit bidiagonal factor L from the
        LDL^H factorization of A.

        On exit: if `fact` = 'N', `ef` contains the (n-1) subdiagonal
        elements of the unit bidiagonal factor L from the LDL^H
        factorization of A.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: the forward error bound for each solution vector x^_j
        (the jth column of the solution matrix X). If x_j is the true
        solution corresponding to x^_j, `ferr`[j-1] is an estimated
        upper bound for the magnitude of the largest element in
        (x^_j-x_j) divided by the magnitude of the largest element in
        x^_j.

    berr : float, shape (`nrhs`), modified in place
        On exit: the component-wise relative backward error of each
        solution vector x^_j (i.e., the smallest relative change in any
        element of A or B that makes x^_j an exact solution).

    Returns
    -------
    rcond : float
        The reciprocal condition number of the matrix A. If `rcond` is
        less than the machine precision (in particular, if `rcond` =
        0.0), the matrix is singular to working precision. This
        condition is indicated by a return code of `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The leading minor of order *<value>* of A is not positive
            definite, so the factorization could not be completed, and
            the solution has not been computed. `rcond` = 0.0 is
            returned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zptsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, df[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ef[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _df_cls = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    )
    _df = _df_cls.to_ctypes()
    _ef_cls = utils._EngineComplex128ArrayType(
        ef, {'fun_name': fun_name, 'entity': 'ef'},
        exp_shape=(_n_py-1,),
    )
    _ef = _ef_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _n,
        _nrhs,
        _d,
        _e,
        _df,
        _ef,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} of A is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite, so the factorization could not be completed, and the solution has',
                    'not been computed. rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _df_cls.refresh_py(_df)
        _ef_cls.refresh_py(_ef)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def zpttrf(n, d, e):
    u"""
    Computes the LDL^H factorization of a complex Hermitian positive
    definite tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f07jr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jrf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    d : float, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: is overwritten by the n diagonal elements of the
        diagonal matrix D from the LDL^H factorization of A.

    e : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: is overwritten by the (n-1) subdiagonal elements of the
        lower bidiagonal matrix L. (`e` can also be regarded as
        containing the (n-1) superdiagonal elements of the upper
        bidiagonal matrix U.)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i < `n`))
            The leading minor of order *<value>* is not positive
            definite, the factorization could not be completed.

        (`errno` (i > 0) and (`n`))
            The leading minor of order n is not positive definite, the
            factorization was completed, but `d`[`n`-1] <= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpttrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, e[n-1].
        utils._BYREF_INT, # Output, info.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _d,
        _e,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py < _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > 0 and _info_py == _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order n is not positive definite, the factorization was',
                    'completed, but d[n-1] <= 0.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        return_data = None
    _err_class.manage()
    return return_data

def zpttrs(uplo, n, nrhs, d, e, b):
    u"""
    Solves a complex Hermitian positive definite tridiagonal system
    using the LDL^H factorization computed by ``zpttrf``.

    For full information please refer to the NAG Library document for
    f07js

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies the form of the factorization as follows:

        `uplo` = 'U'

          A = U^HDU.

        `uplo` = 'L'

          A = LDL^H.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the diagonal matrix D
        from the LDL^H or U^HDU factorization of A.

    e : complex, shape (`n`-1)
        If `uplo` = 'U', `e` must contain the (n-1) superdiagonal
        elements of the unit upper bidiagonal matrix U from the U^HDU
        factorization of A.

        If `uplo` = 'L', `e` must contain the (n-1) subdiagonal elements
        of the unit lower bidiagonal matrix L from the LDL^H
        factorization of A.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpttrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _d,
        _e,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zptcon(n, d, e, anorm):
    u"""
    Computes the reciprocal of the condition number of a complex
    Hermitian positive definite tridiagonal system using the LDL^H
    factorization computed by ``zpttrf``.

    For full information please refer to the NAG Library document for
    f07ju

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07juf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the diagonal matrix D
        from the LDL^H factorization of A.

    e : complex, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the unit lower
        bidiagonal matrix L. (`e` can also be regarded as the
        superdiagonal of the unit upper bidiagonal matrix U from the
        U^HDU factorization of A.)

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlanht`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zpttrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        The reciprocal condition number, 1/kappa_1(A) =
        1/(||A||_1||A^-1||_1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zptcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07juft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, e[n-1].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _call_vendor = utils._EngineState._literal_true_cint
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-4,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _n,
        _d,
        _e,
        _anorm,
        _rcond,
        _rwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zptrfs(uplo, n, nrhs, d, e, df, ef, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex Hermitian positive
    definite tridiagonal system of linear equations, multiple right-hand
    sides.

    For full information please refer to the NAG Library document for
    f07jv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07jvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies the form of the factorization as follows:

        `uplo` = 'U'

          A = U^HDU.

        `uplo` = 'L'

          A = LDL^H.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    d : float, shape (`n`)
        Must contain the n diagonal elements of the matrix of A.

    e : complex, shape (`n`-1)
        If `uplo` = 'U', `e` must contain the (n-1) superdiagonal
        elements of the matrix A.

        If `uplo` = 'L', `e` must contain the (n-1) subdiagonal elements
        of the matrix A.

    df : float, shape (`n`)
        Must contain the n diagonal elements of the diagonal matrix D
        from the LDL^T factorization of A.

    ef : complex, shape (`n`-1)
        If `uplo` = 'U', `ef` must contain the (n-1) superdiagonal
        elements of the unit upper bidiagonal matrix U from the U^HDU
        factorization of A.

        If `uplo` = 'L', `ef` must contain the (n-1) subdiagonal
        elements of the unit lower bidiagonal matrix L from the LDL^H
        factorization of A.

    b : complex, shape (`n`, `nrhs`)
        The n by r matrix of right-hand sides B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r initial solution matrix X.

        On exit: the n by r refined solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x^_j||_infinity <= `ferr`[j-1], where
        x^_j is the jth column of the computed solution returned in the
        array `x` and x_j is the corresponding column of the exact
        solution X. The estimate is almost always a slight overestimate
        of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: estimate of the component-wise relative backward error
        of each computed solution vector x^_j (i.e., the smallest
        relative change in any element of A or B that makes x^_j an
        exact solution).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zptrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07jvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input, df[n].
        utils._EngineComplex128ArrayType.c_type, # Input, ef[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _df = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ef = utils._EngineComplex128ArrayType(
        ef, {'fun_name': fun_name, 'entity': 'ef'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _d,
        _e,
        _df,
        _ef,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dpstrf(uplo, n, a, piv, tol):
    u"""
    Cholesky factorization, with complete pivoting, of a real,
    symmetric, positive semidefinite matrix.

    For full information please refer to the NAG Library document for
    f07kd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07kdf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric positive semidefinite matrix A.

        On exit: if `uplo` = 'U', the first `rank` rows of the upper
        triangle of A are overwritten with the nonzero elements of the
        Cholesky factor U, and the remaining rows of the triangle are
        destroyed.

        If `uplo` = 'L', the first `rank` columns of the lower triangle
        of A are overwritten with the nonzero elements of the Cholesky
        factor L, and the remaining columns of the triangle are
        destroyed.

    piv : int, shape (`n`), modified in place
        On exit: `piv` is such that the nonzero entries of P are
        P(`piv`[k-1],k) = 1, for k = 1,2,...,n.

    tol : float
        User defined tolerance. If `tol` < 0, n*max_{k =
        1,n}|A_{kk}|*machine precision will be used. The algorithm
        terminates at the rth step if the (r+1)th step pivot < `tol`.

    Returns
    -------
    rank : int
        The computed rank of A given by the number of steps the
        algorithm completed.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The matrix A is not positive definite. It is either positive
            semidefinite with computed rank as returned in `rank` and
            less than n, or it may be indefinite, see [Further
            Comments].
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpstrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07kdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, piv[n].
        utils._BYREF_INT, # Output, rank.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _piv_cls = utils._EngineIntArrayType(
        piv, {'fun_name': fun_name, 'entity': 'piv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _piv = _piv_cls.to_ctypes(can_reference_contents=False)
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _piv,
        _rank,
        _tol,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is not positive definite. It is either positive semidefinite',
                    'with computed rank as returned in rank and less than n, or it may be',
                    'indefinite, see [Further Comments].',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _piv_cls.refresh_py(_piv)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rank'],
            )(
                _rank_py,
            )
    _err_class.manage()
    return return_data

def zpstrf(uplo, n, a, piv, tol):
    u"""
    Cholesky factorization of complex Hermitian positive semidefinite
    matrix.

    For full information please refer to the NAG Library document for
    f07kr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07krf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian positive semidefinite matrix A.

        On exit: if `uplo` = 'U', the first `rank` rows of the upper
        triangle of A are overwritten with the nonzero elements of the
        Cholesky factor U, and the remaining rows of the triangle are
        destroyed.

        If `uplo` = 'L', the first `rank` columns of the lower triangle
        of A are overwritten with the nonzero elements of the Cholesky
        factor L, and the remaining columns of the triangle are
        destroyed.

    piv : int, shape (`n`), modified in place
        On exit: `piv` is such that the nonzero entries of P are
        P(`piv`[k-1],k) = 1, for k = 1,2,...,n.

    tol : float
        User defined tolerance. If `tol` < 0, n*max_{k =
        1,n}|A_{kk}|*machine precision will be used. The algorithm
        terminates at the rth step if the (r+1)th step pivot < `tol`.

    Returns
    -------
    rank : int
        The computed rank of A given by the number of steps the
        algorithm completed.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The matrix A is not positive definite. It is either positive
            semidefinite with computed rank as returned in `rank` and
            less than n, or it may be indefinite, see [Further
            Comments].
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpstrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07krft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, piv[n].
        utils._BYREF_INT, # Output, rank.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _piv_cls = utils._EngineIntArrayType(
        piv, {'fun_name': fun_name, 'entity': 'piv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _piv = _piv_cls.to_ctypes(can_reference_contents=False)
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _piv,
        _rank,
        _tol,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is not positive definite. It is either positive semidefinite',
                    'with computed rank as returned in rank and less than n, or it may be',
                    'indefinite, see [Further Comments].',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _piv_cls.refresh_py(_piv)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rank'],
            )(
                _rank_py,
            )
    _err_class.manage()
    return return_data

def dsysv(uplo, n, nrhs, a, ipiv, b):
    u"""
    Computes the solution to a real symmetric system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07ma

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07maf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if no exception or warning is raised, the block
        diagonal matrix D and the multipliers used to obtain the factor
        U or L from the factorization A = UDU^T or A = LDL^T as computed
        by ``dsytrf``.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, so the solution could not be
            computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsysv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07maft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dsysvx(fact, uplo, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Uses the diagonal pivoting factorization to compute the solution to
    a real symmetric system of linear equations.

    For full information please refer to the NAG Library document for
    f07mb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `af` and `ipiv` contain the factorized form of the matrix A.
          `af` and `ipiv` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `af` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : float, shape (`n`, `n`)
        The n by n symmetric matrix A.

    af : float, shape (`n`, `n`), modified in place
        On entry: if `fact` = 'F', `af` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization `a` = UDU^T or `a` = LDL^T as computed by
        ``dsytrf``.

        On exit: if `fact` = 'N', `af` returns the block diagonal matrix
        D and the multipliers used to obtain the factor U or L from the
        factorization `a` = UDU^T or `a` = LDL^T.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``dsytrf``.

        On exit: if `fact` = 'N', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``dsytrf``, as described above.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor D is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsysvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af_cls = utils._EngineFloat64ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _af = _af_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _lwork,
        _iwork,
        _info,
        _fact._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor D is exactly singular,',
                    'so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _af_cls.refresh_py(_af)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def dsytrf(uplo, n, a, ipiv):
    u"""
    Bunch--Kaufman factorization of real symmetric indefinite matrix.

    For full information please refer to the NAG Library document for
    f07md

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mdf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric indefinite matrix A.

        On exit: the upper or lower triangle of A is overwritten by
        details of the block diagonal matrix D and the multipliers used
        to obtain the factor U or L as specified by `uplo`.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, and division by zero will
            occur if it is used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsytrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, and division by zero will occur if it is',
                    'used to solve a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dsytrs(uplo, n, nrhs, a, ipiv, b):
    u"""
    Solution of real symmetric indefinite system of linear equations,
    multiple right-hand sides, matrix already factorized by ``dsytrf``.

    For full information please refer to the NAG Library document for
    f07me

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``dsytrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsytrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsytrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07meft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dsycon(uplo, n, a, ipiv, anorm):
    u"""
    Estimate condition number of real symmetric indefinite matrix,
    matrix already factorized by ``dsytrf``.

    For full information please refer to the NAG Library document for
    f07mg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mgf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``dsytrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsytrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.dlansy`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``dsytrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsycon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-6,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dsyrfs(uplo, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real symmetric indefinite
    system of linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07mh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mhf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        The n by n original symmetric matrix A as supplied to
        ``dsytrf``.

    af : float, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``dsytrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsytrf``.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``dsytrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsyrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af = utils._EngineFloat64ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dsytri(uplo, n, a, ipiv):
    u"""
    Inverse of real symmetric indefinite matrix, matrix already
    factorized by ``dsytrf``.

    For full information please refer to the NAG Library document for
    f07mj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mjf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: details of the factorization of A, as returned by
        ``dsytrf``.

        On exit: the factorization is overwritten by the n by n
        symmetric matrix A^-1.

        If `uplo` = 'U', the upper triangle of A^-1 is stored in the
        upper triangular part of the array.

        If `uplo` = 'L', the lower triangle of A^-1 is stored in the
        lower triangular part of the array.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsytrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. D is
            singular and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsytri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        elif _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'D is singular and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zhesv(uplo, n, nrhs, a, ipiv, b):
    u"""
    Computes the solution to a complex Hermitian system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07mn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mnf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if no exception or warning is raised, the block
        diagonal matrix D and the multipliers used to obtain the factor
        U or L from the factorization A = UDU^H or A = LDL^H as computed
        by ``zhetrf``.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, so the solution could not be
            computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhesv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zhesvx(fact, uplo, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Uses the diagonal pivoting factorization to compute the solution to
    a complex Hermitian system of linear equations.

    For full information please refer to the NAG Library document for
    f07mp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `af` and `ipiv` contain the factorized form of the matrix A.
          `af` and `ipiv` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `af` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`)
        The n by n Hermitian matrix A.

    af : complex, shape (`n`, `n`), modified in place
        On entry: if `fact` = 'F', `af` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization `a` = UDU^H or `a` = LDL^H as computed by
        ``zhetrf``.

        On exit: if `fact` = 'N', `af` returns the block diagonal matrix
        D and the multipliers used to obtain the factor U or L from the
        factorization `a` = UDU^H or `a` = LDL^H.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zhetrf``.

        On exit: if `fact` = 'N', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zhetrf``, as described above.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor D is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhesvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af_cls = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _af = _af_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _lwork,
        _rwork,
        _info,
        _fact._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor D is exactly singular,',
                    'so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _af_cls.refresh_py(_af)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def zhetrf(uplo, n, a, ipiv):
    u"""
    Bunch--Kaufman factorization of complex Hermitian indefinite matrix.

    For full information please refer to the NAG Library document for
    f07mr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mrf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian indefinite matrix A.

        On exit: the upper or lower triangle of A is overwritten by
        details of the block diagonal matrix D and the multipliers used
        to obtain the factor U or L as specified by `uplo`.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, and division by zero will
            occur if it is used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhetrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, and division by zero will occur if it is',
                    'used to solve a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zhetrs(uplo, n, nrhs, a, ipiv, b):
    u"""
    Solution of complex Hermitian indefinite system of linear equations,
    multiple right-hand sides, matrix already factorized by ``zhetrf``.

    For full information please refer to the NAG Library document for
    f07ms

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07msf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``zhetrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhetrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhetrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07msft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zhecon(uplo, n, a, ipiv, anorm):
    u"""
    Estimate condition number of complex Hermitian indefinite matrix,
    matrix already factorized by ``zhetrf``.

    For full information please refer to the NAG Library document for
    f07mu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07muf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``zhetrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhetrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlanhe`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zhetrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhecon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07muft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-6,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zherfs(uplo, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex Hermitian indefinite
    system of linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07mv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The n by n original Hermitian matrix A as supplied to
        ``zhetrf``.

    af : complex, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``zhetrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhetrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zhetrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zherfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zhetri(uplo, n, a, ipiv):
    u"""
    Inverse of complex Hermitian indefinite matrix, matrix already
    factorized by ``zhetrf``.

    For full information please refer to the NAG Library document for
    f07mw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07mwf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: details of the factorization of A, as returned by
        ``zhetrf``.

        On exit: the factorization is overwritten by the n by n
        Hermitian matrix A^-1.

        If `uplo` = 'U', the upper triangle of A^-1 is stored in the
        upper triangular part of the array.

        If `uplo` = 'L', the lower triangle of A^-1 is stored in the
        lower triangular part of the array.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhetrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. D is
            singular and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhetri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07mwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        elif _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'D is singular and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zsysv(uplo, n, nrhs, a, ipiv, b):
    u"""
    Computes the solution to a complex symmetric system of linear
    equations.

    For full information please refer to the NAG Library document for
    f07nn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07nnf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if no exception or warning is raised, the block
        diagonal matrix D and the multipliers used to obtain the factor
        U or L from the factorization A = UDU^H or A = LDL^H as computed
        by ``zsytrf``.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, so the solution could not be
            computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsysv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07nnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zsysvx(fact, uplo, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Uses the diagonal pivoting factorization to compute the solution to
    a complex symmetric system of linear equations.

    For full information please refer to the NAG Library document for
    f07np

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07npf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `af` and `ipiv` contain the factorized form of the matrix A.
          `af` and `ipiv` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `af` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    a : complex, shape (`n`, `n`)
        The n by n symmetric matrix A.

    af : complex, shape (`n`, `n`), modified in place
        On entry: if `fact` = 'F', `af` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization `a` = UDU^H or `a` = LDL^H as computed by
        ``zsytrf``.

        On exit: if `fact` = 'N', `af` returns the block diagonal matrix
        D and the multipliers used to obtain the factor U or L from the
        factorization `a` = UDU^H or `a` = LDL^H.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zsytrf``.

        On exit: if `fact` = 'N', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zsytrf``, as described above.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor D is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsysvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07npft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af_cls = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _af = _af_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _lwork,
        _rwork,
        _info,
        _fact._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor D is exactly singular,',
                    'so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _af_cls.refresh_py(_af)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def zsytrf(uplo, n, a, ipiv):
    u"""
    Bunch--Kaufman factorization of complex symmetric matrix.

    For full information please refer to the NAG Library document for
    f07nr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07nrf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric indefinite matrix A.

        On exit: the upper or lower triangle of A is overwritten by
        details of the block diagonal matrix D and the multipliers used
        to obtain the factor U or L as specified by `uplo`.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, and division by zero will
            occur if it is used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsytrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07nrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, and division by zero will occur if it is',
                    'used to solve a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zsytrs(uplo, n, nrhs, a, ipiv, b):
    u"""
    Solution of complex symmetric system of linear equations, multiple
    right-hand sides, matrix already factorized by ``zsytrf``.

    For full information please refer to the NAG Library document for
    f07ns

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07nsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``zsytrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsytrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsytrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07nsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zsycon(uplo, n, a, ipiv, anorm):
    u"""
    Estimate condition number of complex symmetric matrix, matrix
    already factorized by ``zsytrf``.

    For full information please refer to the NAG Library document for
    f07nu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07nuf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``zsytrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsytrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlansy`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zsytrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsycon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07nuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-6,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zsyrfs(uplo, n, nrhs, a, af, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex symmetric system of
    linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07nv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07nvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The n by n original symmetric matrix A as supplied to
        ``zsytrf``.

    af : complex, shape (`n`, `n`)
        Details of the factorization of A, as returned by ``zsytrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsytrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zsytrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsyrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07nvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, af[ldaf, n].
        utils._BYREF_INT, # Input, ldaf.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (af, 'af'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldaf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(af, {'fun_name': fun_name, 'entity': 'af'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaf'},
    ).to_ctypes()
    _af = utils._EngineComplex128ArrayType(
        af, {'fun_name': fun_name, 'entity': 'af'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _af,
        _ldaf,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zsytri(uplo, n, a, ipiv):
    u"""
    Inverse of complex symmetric matrix, matrix already factorized by
    ``zsytrf``.

    For full information please refer to the NAG Library document for
    f07nw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07nwf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: details of the factorization of A, as returned by
        ``zsytrf``.

        On exit: the factorization is overwritten by the n by n
        symmetric matrix A^-1.

        If `uplo` = 'U', the upper triangle of A^-1 is stored in the
        upper triangular part of the array.

        If `uplo` = 'L', the lower triangle of A^-1 is stored in the
        lower triangular part of the array.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsytrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. D is
            singular and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsytri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07nwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ipiv,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        elif _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'D is singular and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dspsv(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Computes the solution to a real symmetric system of linear
    equations, packed storage.

    For full information please refer to the NAG Library document for
    f07pa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07paf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed by columns.

        On exit: the block diagonal matrix D and the multipliers used to
        obtain the factor U or L from the factorization A = UDU^T or A =
        LDL^T as computed by ``dsptrf``, stored as a packed triangular
        matrix in the same storage format as A.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, so the solution could not be
            computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dspsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07paft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, x, ferr, berr):
    u"""
    Uses the diagonal pivoting factorization to compute the solution to
    a real symmetric system of linear equations, packed storage. Error
    bounds and a condition estimate are also computed.

    For full information please refer to the NAG Library document for
    f07pb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pbf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `afp` and `ipiv` contain the factorized form of the matrix A.
          `afp` and `ipiv` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `afp` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n symmetric matrix A, packed by columns.

    afp : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: if `fact` = 'F', `afp` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization A = UDU^T or A = LDL^T as computed by
        ``dsptrf``, stored as a packed triangular matrix in the same
        storage format as A.

        On exit: if `fact` = 'N', `afp` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization A = UDU^T or A = LDL^T as computed by
        ``dsptrf``, stored as a packed triangular matrix in the same
        storage format as A.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``dsptrf``.

        On exit: if `fact` = 'N', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``dsptrf``, as described above.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor D is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dspsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07pbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, afp[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp_cls = utils._EngineFloat64ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _afp = _afp_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _fact._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor D is exactly singular,',
                    'so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _afp_cls.refresh_py(_afp)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def dsptrf(uplo, n, ap, ipiv):
    u"""
    Bunch--Kaufman factorization of real symmetric indefinite matrix,
    packed storage.

    For full information please refer to the NAG Library document for
    f07pd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pdf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed by columns.

        On exit: A is overwritten by details of the block diagonal
        matrix D and the multipliers used to obtain the factor U or L as
        specified by `uplo`.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, and division by zero will
            occur if it is used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsptrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07pdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, and division by zero will occur if it is',
                    'used to solve a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dsptrs(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Solution of real symmetric indefinite system of linear equations,
    multiple right-hand sides, matrix already factorized by ``dsptrf``,
    packed storage.

    For full information please refer to the NAG Library document for
    f07pe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : float, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``dsptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsptrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsptrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07peft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dspcon(uplo, n, ap, ipiv, anorm):
    u"""
    Estimate condition number of real symmetric indefinite matrix,
    matrix already factorized by ``dsptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07pg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pgf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``dsptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsptrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.dlansp`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``dsptrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dspcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07pgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-5,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dsprfs(uplo, n, nrhs, ap, afp, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of real symmetric indefinite
    system of linear equations, multiple right-hand sides, packed
    storage.

    For full information please refer to the NAG Library document for
    f07ph

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07phf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n original symmetric matrix A as supplied to
        ``dsptrf``.

    afp : float, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``dsptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsptrf``.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``dsptrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsprfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07phft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input, afp[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp = utils._EngineFloat64ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dsptri(uplo, n, ap, ipiv):
    u"""
    Inverse of real symmetric indefinite matrix, matrix already
    factorized by ``dsptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07pj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pjf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^TP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^TP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the factorization of A stored in packed form, as
        returned by ``dsptrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``dsptrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. D is
            singular and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dsptri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07pjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'D is singular and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zhpsv(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Computes the solution to a complex Hermitian system of linear
    equations, packed storage.

    For full information please refer to the NAG Library document for
    f07pn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pnf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n Hermitian matrix A, packed by columns.

        On exit: the block diagonal matrix D and the multipliers used to
        obtain the factor U or L from the factorization A = UDU^H or A =
        LDL^H as computed by ``zhptrf``, stored as a packed triangular
        matrix in the same storage format as A.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, so the solution could not be
            computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhpsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07pnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zhpsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, x, ferr, berr):
    u"""
    Uses the diagonal pivoting factorization to compute the solution to
    a complex, Hermitian, system of linear equations, error bounds and
    condition estimates. Packed storage.

    For full information please refer to the NAG Library document for
    f07pp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ppf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `afp` and `ipiv` contain the factorized form of the matrix A.
          `afp` and `ipiv` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `afp` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n Hermitian matrix A, packed by columns.

    afp : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: if `fact` = 'F', `afp` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization A = UDU^H or A = LDL^H as computed by
        ``zhptrf``, stored as a packed triangular matrix in the same
        storage format as A.

        On exit: if `fact` = 'N', `afp` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization A = UDU^H or A = LDL^H as computed by
        ``zhptrf``, stored as a packed triangular matrix in the same
        storage format as A.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zhptrf``.

        On exit: if `fact` = 'N', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zhptrf``, as described above.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor D is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhpsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ppft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, afp[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp_cls = utils._EngineComplex128ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _afp = _afp_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor D is exactly singular,',
                    'so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _afp_cls.refresh_py(_afp)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def zhptrf(uplo, n, ap, ipiv):
    u"""
    Bunch--Kaufman factorization of complex Hermitian indefinite matrix,
    packed storage.

    For full information please refer to the NAG Library document for
    f07pr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07prf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n Hermitian matrix A, packed by columns.

        On exit: A is overwritten by details of the block diagonal
        matrix D and the multipliers used to obtain the factor U or L as
        specified by `uplo`.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, and division by zero will
            occur if it is used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhptrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07prft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, and division by zero will occur if it is',
                    'used to solve a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zhptrs(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Solution of complex Hermitian indefinite system of linear equations,
    multiple right-hand sides, matrix already factorized by ``zhptrf``,
    packed storage.

    For full information please refer to the NAG Library document for
    f07ps

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07psf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``zhptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhptrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhptrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07psft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zhpcon(uplo, n, ap, ipiv, anorm):
    u"""
    Estimate condition number of complex Hermitian indefinite matrix,
    matrix already factorized by ``zhptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07pu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07puf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``zhptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhptrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlanhp`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zhptrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhpcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07puft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-5,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zhprfs(uplo, n, nrhs, ap, afp, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex Hermitian indefinite
    system of linear equations, multiple right-hand sides, packed
    storage.

    For full information please refer to the NAG Library document for
    f07pv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n original Hermitian matrix A as supplied to
        ``zhptrf``.

    afp : complex, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``zhptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhptrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zhptrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhprfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07pvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input, afp[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp = utils._EngineComplex128ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zhptri(uplo, n, ap, ipiv):
    u"""
    Inverse of complex Hermitian indefinite matrix, matrix already
    factorized by ``zhptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07pw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07pwf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the factorization of A stored in packed form, as
        returned by ``zhptrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zhptrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. D is
            singular and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zhptri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07pwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'D is singular and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zspsv(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Computes the solution to a complex symmetric system of linear
    equations, packed storage.

    For full information please refer to the NAG Library document for
    f07qn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07qnf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed by columns.

        On exit: the block diagonal matrix D and the multipliers used to
        obtain the factor U or L from the factorization A = UDU^H or A =
        LDL^H as computed by ``zsptrf``, stored as a packed triangular
        matrix in the same storage format as A.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: if no exception or warning is raised, the n by r
        solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, so the solution could not be
            computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zspsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07qnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, so the solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, x, ferr, berr):
    u"""
    Uses the diagonal pivoting factorization to compute the solution to
    a complex, symmetric, system of linear equations, error bounds and
    condition estimates. Packed storage.

    For full information please refer to the NAG Library document for
    f07qp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07qpf.html

    Parameters
    ----------
    fact : str, length 1
        Specifies whether or not the factorized form of the matrix A has
        been supplied.

        `fact` = 'F'

          `afp` and `ipiv` contain the factorized form of the matrix A.
          `afp` and `ipiv` will not be modified.

        `fact` = 'N'

          The matrix A will be copied to `afp` and factorized.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of A is stored.

        If `uplo` = 'L', the lower triangle of A is stored.

    n : int
        n, the number of linear equations, i.e., the order of the matrix
        A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n symmetric matrix A, packed by columns.

    afp : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: if `fact` = 'F', `afp` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization A = UDU^H or A = LDL^H as computed by
        ``zsptrf``, stored as a packed triangular matrix in the same
        storage format as A.

        On exit: if `fact` = 'N', `afp` contains the block diagonal
        matrix D and the multipliers used to obtain the factor U or L
        from the factorization A = UDU^H or A = LDL^H as computed by
        ``zsptrf``, stored as a packed triangular matrix in the same
        storage format as A.

    ipiv : int, shape (`n`), modified in place
        On entry: if `fact` = 'F', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zsptrf``.

        On exit: if `fact` = 'N', `ipiv` contains details of the
        interchanges and the block structure of D, as determined by
        ``zsptrf``, as described above.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the forward error bound for each computed
        solution vector, such that
        ||x^_j-x_j||_infinity/||x_j||_infinity <= `ferr`[j-1] where x^_j
        is the jth column of the computed solution returned in the array
        `x` and x_j is the corresponding column of the exact solution X.
        The estimate is as reliable as the estimate for `rcond`, and is
        almost always a slight overestimate of the true error.

    berr : float, shape (`nrhs`), modified in place
        On exit: if the function exits successfully or `errno` = `n` +
        1, an estimate of the component-wise relative backward error of
        each computed solution vector x^_j (i.e., the smallest relative
        change in any element of A or B that makes x^_j an exact
        solution).

    Returns
    -------
    rcond : float
        The estimate of the reciprocal condition number of the matrix A.
        If `rcond` = 0.0, the matrix may be exactly singular. This
        condition is indicated by `errno` in 1 ... `n`. Otherwise, if
        `rcond` is less than the machine precision, the matrix is
        singular to working precision. This condition is indicated by
        `errno` = `n` + 1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `fact`.

            Constraint: `fact` = 'F' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the factor D is
            exactly singular, so the solution and error bounds could not
            be computed. `rcond` = 0.0 is returned.

        (`errno` `n`+1)
            D is nonsingular, but `rcond` is less than machine
            precision, meaning that the matrix is singular to working
            precision. Nevertheless, the solution and error bounds are
            computed because there are a number of situations where the
            computed solution can be more accurate than the value of
            `rcond` would suggest.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zspsvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07qpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, fact(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, afp[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input/Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of fact.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fact = utils._EngineCharScalarType(
        fact, {'fun_name': fun_name, 'entity': 'fact'}, exp_chrlen=1,
    ).to_ctypes()
    _fact_py = utils._EngineCharScalarType.to_py(_fact)
    if _fact_py not in ('F', 'f', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('fact in (\'F\', \'f\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='fact',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp_cls = utils._EngineComplex128ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _afp = _afp_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    )
    _ipiv = _ipiv_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _fact,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _rcond,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _fact._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the factor D is exactly singular,',
                    'so the solution and error bounds could not be computed.',
                    'rcond = 0.0 is returned.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'D is nonsingular, but rcond is less than',
                    'machine precision, meaning that the matrix is singular to working precision.',
                    'Nevertheless, the solution and error bounds are computed because there',
                    'are a number of situations where the computed solution can be more accurate',
                    'than the value of rcond would suggest.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _afp_cls.refresh_py(_afp)
        _ipiv_cls.refresh_py(_ipiv)
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rcond'],
            )(
                _rcond_py,
            )
    _err_class.manage()
    return return_data

def zsptrf(uplo, n, ap, ipiv):
    u"""
    Bunch--Kaufman factorization of complex symmetric matrix, packed
    storage.

    For full information please refer to the NAG Library document for
    f07qr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07qrf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed by columns.

        On exit: A is overwritten by details of the block diagonal
        matrix D and the multipliers used to obtain the factor U or L as
        specified by `uplo`.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the interchanges and the block structure of
        D.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. The
            factorization has been completed, but the block diagonal
            matrix D is exactly singular, and division by zero will
            occur if it is used to solve a system of equations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsptrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07qrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The factorization has been completed, but the block diagonal matrix D',
                    'is exactly singular, and division by zero will occur if it is',
                    'used to solve a system of equations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zsptrs(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Solution of complex symmetric system of linear equations, multiple
    right-hand sides, matrix already factorized by ``zsptrf``, packed
    storage.

    For full information please refer to the NAG Library document for
    f07qs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07qsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``zsptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsptrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsptrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07qsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zspcon(uplo, n, ap, ipiv, anorm):
    u"""
    Estimate condition number of complex symmetric matrix, matrix
    already factorized by ``zsptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07qu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07quf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``zsptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsptrf``.

    anorm : float
        The 1-norm of the original matrix A, which may be computed by
        calling ``blas.zlansp`` with its argument norm = '1'. `anorm`
        must be computed either before calling ``zsptrf`` or else from a
        copy of the original matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `anorm`.

            Constraint: `anorm` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zspcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07quft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._BYREF_FLOAT64, # Input, anorm.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _anorm = utils._EngineFloat64ScalarType(
        anorm, {'fun_name': fun_name, 'entity': 'anorm'},
    ).to_ctypes()
    _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
    if not _anorm_py >= 0.0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('anorm >= 0.0.'),
            ],
            errcodes=(-5,),
            entity='anorm',
        ).manage()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _anorm,
        _rcond,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def zsprfs(uplo, n, nrhs, ap, afp, ipiv, b, x, ferr, berr):
    u"""
    Refined solution with error bounds of complex symmetric system of
    linear equations, multiple right-hand sides, packed storage.

    For full information please refer to the NAG Library document for
    f07qv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07qvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored and how A is to be factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and A is factorized
          as PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored and A is factorized
          as PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n original symmetric matrix A as supplied to
        ``zsptrf``.

    afp : complex, shape (`n`*(`n`+1)/2)
        The factorization of A stored in packed form, as returned by
        ``zsptrf``.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsptrf``.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r solution matrix X, as returned by
        ``zsptrs``.

        On exit: the improved solution matrix X.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsprfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07qvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input, afp[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _afp = utils._EngineComplex128ArrayType(
        afp, {'fun_name': fun_name, 'entity': 'afp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _afp,
        _ipiv,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def zsptri(uplo, n, ap, ipiv):
    u"""
    Inverse of complex symmetric matrix, matrix already factorized by
    ``zsptrf``, packed storage.

    For full information please refer to the NAG Library document for
    f07qw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07qwf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = PUDU^HP^T, where U is upper triangular.

        `uplo` = 'L'

          A = PLDL^HP^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the factorization of A stored in packed form, as
        returned by ``zsptrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1.

    ipiv : int, shape (`n`)
        Details of the interchanges and the block structure of D, as
        returned by ``zsptrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. D is
            singular and the inverse of A cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zsptri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07qwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _ipiv,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'D is singular and the inverse of A cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtrtrs(uplo, trans, diag, n, nrhs, a, b):
    u"""
    Solution of real triangular system of linear equations, multiple
    right-hand sides.

    For full information please refer to the NAG Library document for
    f07te

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07tef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The equations are of the form A^TX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        The n by n triangular matrix A.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular and the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtrtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07teft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular and the solution has not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtrcon(norm, uplo, diag, n, a):
    u"""
    Estimate condition number of real triangular matrix.

    For full information please refer to the NAG Library document for
    f07tg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07tgf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`)
        The n by n triangular matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or if
        the estimate underflows. If `rcond` is less than machine
        precision, A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtrcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07tgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _uplo,
        _diag,
        _n,
        _a,
        _lda,
        _rcond,
        _work,
        _iwork,
        _info,
        _norm._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dtrrfs(uplo, trans, diag, n, nrhs, a, b, x, ferr, berr):
    u"""
    Error bounds for solution of real triangular system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07th

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07thf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The equations are of the form A^TX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : float, shape (`n`, `n`)
        The n by n triangular matrix A.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`)
        The n by r solution matrix X, as returned by ``dtrtrs``.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtrrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07thft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dtrtri(uplo, diag, n, a):
    u"""
    Inverse of real triangular matrix.

    For full information please refer to the NAG Library document for
    f07tj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07tjf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n triangular matrix A.

        On exit: A is overwritten by A^-1, using the same storage format
        as described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular its inverse cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtrtri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07tjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-2,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _diag,
        _n,
        _a,
        _lda,
        _info,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular its inverse cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ztrtrs(uplo, trans, diag, n, nrhs, a, b):
    u"""
    Solution of complex triangular system of linear equations, multiple
    right-hand sides.

    For full information please refer to the NAG Library document for
    f07ts

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07tsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T'

          The equations are of the form A^TX = B.

        `trans` = 'C'

          The equations are of the form A^HX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The n by n triangular matrix A.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular and the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztrtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07tsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular and the solution has not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ztrcon(norm, uplo, diag, n, a):
    u"""
    Estimate condition number of complex triangular matrix.

    For full information please refer to the NAG Library document for
    f07tu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07tuf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        The n by n triangular matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztrcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07tuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _uplo,
        _diag,
        _n,
        _a,
        _lda,
        _rcond,
        _work,
        _rwork,
        _info,
        _norm._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def ztrrfs(uplo, trans, diag, n, nrhs, a, b, x, ferr, berr):
    u"""
    Error bounds for solution of complex triangular system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07tv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07tvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T'

          The equations are of the form A^TX = B.

        `trans` = 'C'

          The equations are of the form A^HX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    a : complex, shape (`n`, `n`)
        The n by n triangular matrix A.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`)
        The n by r solution matrix X, as returned by ``ztrtrs``.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztrrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07tvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def ztrtri(uplo, diag, n, a):
    u"""
    Inverse of complex triangular matrix.

    For full information please refer to the NAG Library document for
    f07tw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07twf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n triangular matrix A.

        On exit: A is overwritten by A^-1, using the same storage format
        as described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular its inverse cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztrtri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07twft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-2,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _diag,
        _n,
        _a,
        _lda,
        _info,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular its inverse cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtptrs(uplo, trans, diag, n, nrhs, ap, b):
    u"""
    Solution of real triangular system of linear equations, multiple
    right-hand sides, packed storage.

    For full information please refer to the NAG Library document for
    f07ue

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07uef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The equations are of the form A^TX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular and the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtptrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ueft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular and the solution has not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtpcon(norm, uplo, diag, n, ap):
    u"""
    Estimate condition number of real triangular matrix, packed storage.

    For full information please refer to the NAG Library document for
    f07ug

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ugf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtpcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ugft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _uplo,
        _diag,
        _n,
        _ap,
        _rcond,
        _work,
        _iwork,
        _info,
        _norm._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dtprfs(uplo, trans, diag, n, nrhs, ap, b, x, ferr, berr):
    u"""
    Error bounds for solution of real triangular system of linear
    equations, multiple right-hand sides, packed storage.

    For full information please refer to the NAG Library document for
    f07uh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07uhf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The equations are of the form A^TX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`)
        The n by r solution matrix X, as returned by ``dtptrs``.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtprfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07uhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dtptri(uplo, diag, n, ap):
    u"""
    Inverse of real triangular matrix, packed storage.

    For full information please refer to the NAG Library document for
    f07uj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07ujf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n triangular matrix A, packed by columns.

        On exit: A is overwritten by A^-1, using the same storage format
        as described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular its inverse cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtptri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07ujft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-2,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _diag,
        _n,
        _ap,
        _info,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular its inverse cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ztptrs(uplo, trans, diag, n, nrhs, ap, b):
    u"""
    Solution of complex triangular system of linear equations, multiple
    right-hand sides, packed storage.

    For full information please refer to the NAG Library document for
    f07us

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07usf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T'

          The equations are of the form A^TX = B.

        `trans` = 'C'

          The equations are of the form A^HX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular and the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztptrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07usft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular and the solution has not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ztpcon(norm, uplo, diag, n, ap):
    u"""
    Estimate condition number of complex triangular matrix, packed
    storage.

    For full information please refer to the NAG Library document for
    f07uu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07uuf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztpcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07uuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _uplo,
        _diag,
        _n,
        _ap,
        _rcond,
        _work,
        _rwork,
        _info,
        _norm._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def ztprfs(uplo, trans, diag, n, nrhs, ap, b, x, ferr, berr):
    u"""
    Error bounds for solution of complex triangular system of linear
    equations, multiple right-hand sides, packed storage.

    For full information please refer to the NAG Library document for
    f07uv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07uvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T'

          The equations are of the form A^TX = B.

        `trans` = 'C'

          The equations are of the form A^HX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`)
        The n by r solution matrix X, as returned by ``ztptrs``.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztprfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07uvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-5,),
            entity='nrhs',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def ztptri(uplo, diag, n, ap):
    u"""
    Inverse of complex triangular matrix, packed storage.

    For full information please refer to the NAG Library document for
    f07uw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07uwf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n triangular matrix A, packed by columns.

        On exit: A is overwritten by A^-1, using the same storage format
        as described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular its inverse cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztptri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07uwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-2,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _diag,
        _n,
        _ap,
        _info,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular its inverse cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtbtrs(uplo, trans, diag, n, kd, nrhs, ab, b):
    u"""
    Solution of real band triangular system of linear equations,
    multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07ve

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07vef.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The equations are of the form A^TX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides.

    ab : float, shape (`kd`+1, `n`)
        The n by n triangular band matrix A.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -6)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular and the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtbtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07veft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-6,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular and the solution has not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtbcon(norm, uplo, diag, n, kd, ab):
    u"""
    Estimate condition number of real band triangular matrix.

    For full information please refer to the NAG Library document for
    f07vg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07vgf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    ab : float, shape (`kd`+1, `n`)
        The n by n triangular band matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtbcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07vgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _uplo,
        _diag,
        _n,
        _kd,
        _ab,
        _ldab,
        _rcond,
        _work,
        _iwork,
        _info,
        _norm._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def dtbrfs(uplo, trans, diag, n, kd, nrhs, ab, b, x, ferr, berr):
    u"""
    Error bounds for solution of real band triangular system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07vh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07vhf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T' or 'C'

          The equations are of the form A^TX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides.

    ab : float, shape (`kd`+1, `n`)
        The n by n triangular band matrix A.

    b : float, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : float, shape (`n`, `nrhs`)
        The n by r solution matrix X, as returned by ``dtbtrs``.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -6)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtbrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07vhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-6,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _iwork,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def ztbtrs(uplo, trans, diag, n, kd, nrhs, ab, b):
    u"""
    Solution of complex band triangular system of linear equations,
    multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07vs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07vsf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T'

          The equations are of the form A^TX = B.

        `trans` = 'C'

          The equations are of the form A^HX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides.

    ab : complex, shape (`kd`+1, `n`)
        The n by n triangular band matrix A.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -6)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Element *<value>* of the diagonal is exactly zero. A is
            singular and the solution has not been computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztbtrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07vsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-6,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Element {:s} of the diagonal is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular and the solution has not been computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ztbcon(norm, uplo, diag, n, kd, ab):
    u"""
    Estimate condition number of complex band triangular matrix.

    For full information please refer to the NAG Library document for
    f07vu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07vuf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    ab : complex, shape (`kd`+1, `n`)
        The n by n triangular band matrix A.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `norm`.

            Constraint: `norm` = '1', 'O' or 'I'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztbcon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07vuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _norm_py = utils._EngineCharScalarType.to_py(_norm)
    if _norm_py not in ('1', '1', 'O', 'o', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('norm in (\'1\', \'1\', \'O\', \'o\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='norm',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _norm,
        _uplo,
        _diag,
        _n,
        _kd,
        _ab,
        _ldab,
        _rcond,
        _work,
        _rwork,
        _info,
        _norm._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        return_data = _rcond_py
    _err_class.manage()
    return return_data

def ztbrfs(uplo, trans, diag, n, kd, nrhs, ab, b, x, ferr, berr):
    u"""
    Error bounds for solution of complex band triangular system of
    linear equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f07vv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07vvf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    trans : str, length 1
        Indicates the form of the equations.

        `trans` = 'N'

          The equations are of the form AX = B.

        `trans` = 'T'

          The equations are of the form A^TX = B.

        `trans` = 'C'

          The equations are of the form A^HX = B.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    kd : int
        k_d, the number of superdiagonals of the matrix A if `uplo` =
        'U', or the number of subdiagonals if `uplo` = 'L'.

    nrhs : int
        r, the number of right-hand sides.

    ab : complex, shape (`kd`+1, `n`)
        The n by n triangular band matrix A.

    b : complex, shape (`n`, `nrhs`)
        The n by r right-hand side matrix B.

    x : complex, shape (`n`, `nrhs`)
        The n by r solution matrix X, as returned by ``ztbtrs``.

    ferr : float, shape (`nrhs`), modified in place
        On exit: `ferr`[j-1] contains an estimated error bound for the
        jth solution vector, that is, the jth column of X, for j =
        1,2,...,r.

    berr : float, shape (`nrhs`), modified in place
        On exit: `berr`[j-1] contains the component-wise backward error
        bound beta for the jth solution vector, that is, the jth column
        of X, for j = 1,2,...,r.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -6)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztbrfs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07vvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input, x[ldx, nrhs].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, ferr[nrhs].
        utils._EngineFloat64ArrayType.c_type, # Output, berr[nrhs].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-6,),
            entity='nrhs',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ferr_cls = utils._EngineFloat64ArrayType(
        ferr, {'fun_name': fun_name, 'entity': 'ferr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _ferr = _ferr_cls.to_ctypes(can_reference_contents=False)
    _berr_cls = utils._EngineFloat64ArrayType(
        berr, {'fun_name': fun_name, 'entity': 'berr'},
        exp_shape=(_nrhs_py,),
        check_contents=False,
    )
    _berr = _berr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _trans,
        _diag,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _x,
        _ldx,
        _ferr,
        _berr,
        _work,
        _rwork,
        _info,
        _uplo._length_,
        _trans._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ferr_cls.refresh_py(_ferr)
        _berr_cls.refresh_py(_berr)
        return_data = None
    _err_class.manage()
    return return_data

def dpftrf(transr, uplo, n, ar):
    u"""
    Cholesky factorization of real symmetric positive definite matrix,
    Rectangular Full Packed format.

    For full information please refer to the NAG Library document for
    f07wd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wdf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the RFP representation of A is normal or
        transposed.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'T'

          The matrix A is stored in transposed RFP format.

    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored, and A is factorized
          as U^TU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored, and A is factorized
          as LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangular part (as specified by
        `uplo`) of the n by n symmetric matrix A, in either normal or
        transposed RFP format (as specified by `transr`). The storage
        format is described in detail in [the F07 Introduction].

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^TU or A = LL^T, in the
        same storage format as A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. There is no function
            specifically designed to factorize a symmetric matrix stored
            in RFP format which is not positive definite; the matrix
            must be treated as a full symmetric matrix, by calling
            ``dsytrf``.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpftrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07wdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar_cls = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ar = _ar_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed. Hence A itself is not positive',
                    'definite. This may indicate an error in forming the matrix A. There is no',
                    'function specifically designed to factorize a symmetric matrix stored in',
                    'RFP format which is not positive definite; the matrix must be treated as a',
                    'full symmetric matrix, by calling dsytrf.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ar_cls.refresh_py(_ar)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dpftrs(transr, uplo, n, nrhs, ar, b):
    u"""
    Solution of real symmetric positive definite system of linear
    equations, multiple right-hand sides, coefficient matrix already
    factorized by ``dpftrf``, Rectangular Full Packed format.

    For full information please refer to the NAG Library document for
    f07we

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wef.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the RFP representation of A is normal or
        transposed.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'T'

          The matrix A is stored in transposed RFP format.

    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ar : float, shape (`n`*(`n`+1)/2)
        The Cholesky factorization of A stored in RFP format, as
        returned by ``dpftrf``.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpftrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07weft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input, ar[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ar = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _nrhs,
        _ar,
        _b,
        _ldb,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dpftri(transr, uplo, n, ar):
    u"""
    Inverse of real symmetric positive definite matrix, matrix already
    factorized by ``dpftrf``, Rectangular Full Packed format.

    For full information please refer to the NAG Library document for
    f07wj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wjf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the RFP representation of A is normal or
        transposed.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'T'

          The matrix A is stored in transposed RFP format.

    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^TU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^T, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the Cholesky factorization of A stored in RFP format,
        as returned by ``dpftrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1 stored in RFP format.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. There is no function
            specifically designed to invert a symmetric matrix stored in
            RFP format which is not positive definite; the matrix must
            be treated as a full symmetric matrix, by calling
            ``dsytri``.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dpftri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07wjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar_cls = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ar = _ar_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed. Hence A itself is not positive',
                    'definite. This may indicate an error in forming the matrix A. There is no',
                    'function specifically designed to invert a symmetric matrix stored in',
                    'RFP format which is not positive definite; the matrix must be treated as a',
                    'full symmetric matrix, by calling dsytri.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ar_cls.refresh_py(_ar)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtftri(transr, uplo, diag, n, ar):
    u"""
    Inverse of real triangular matrix, Rectangular Full Packed format.

    For full information please refer to the NAG Library document for
    f07wk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wkf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the RFP representation of A is normal or
        transposed.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'T'

          The matrix A is stored in transposed RFP format.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    ar : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangular part (as specified by
        `uplo`) of the n by n symmetric matrix A, in either normal or
        transposed RFP format (as specified by `transr`). The storage
        format is described in detail in [the F07 Introduction].

        On exit: A is overwritten by A^-1, in the same storage format as
        A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of A is exactly zero. A is
            singular its inverse cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.dtftri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07wkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ar_cls = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ar = _ar_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _diag,
        _n,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular its inverse cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ar_cls.refresh_py(_ar)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zpftrf(transr, uplo, n, ar):
    u"""
    Cholesky factorization of complex Hermitian positive definite
    matrix, Rectangular Full Packed format.

    For full information please refer to the NAG Library document for
    f07wr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wrf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored, and A is factorized
          as U^HU, where U is upper triangular.

        `uplo` = 'L'

          The lower triangular part of A is stored, and A is factorized
          as LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangular part (as specified by
        `uplo`) of the n by n Hermitian matrix A, in either normal or
        transposed RFP format (as specified by `transr`). The storage
        format is described in detail in [the F07 Introduction].

        On exit: if no exception or warning is raised, the factor U or L
        from the Cholesky factorization A = U^HU or A = LL^H, in the
        same storage format as A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. There is no function
            specifically designed to factorize a Hermitian matrix stored
            in RFP format which is not positive definite; the matrix
            must be treated as a full Hermitian matrix, by calling
            ``zhetrf``.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpftrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07wrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar_cls = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ar = _ar_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed. Hence A itself is not positive',
                    'definite. This may indicate an error in forming the matrix A. There is no',
                    'function specifically designed to factorize a Hermitian matrix stored in',
                    'RFP format which is not positive definite; the matrix must be treated as a',
                    'full Hermitian matrix, by calling zhetrf.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ar_cls.refresh_py(_ar)
        return_data = None
    _err_class.manage()
    return return_data

def zpftrs(transr, uplo, n, nrhs, ar, b):
    u"""
    Solution of complex Hermitian positive definite system of linear
    equations, multiple right-hand sides, coefficient matrix already
    factorized by ``zpftrf``, Rectangular Full Packed format.

    For full information please refer to the NAG Library document for
    f07ws

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wsf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    nrhs : int
        r, the number of right-hand sides.

    ar : complex, shape (`n`*(`n`+1)/2)
        The Cholesky factorization of A stored in RFP format, as
        returned by ``zpftrf``.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpftrs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07wsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input, ar[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _ar = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _nrhs,
        _ar,
        _b,
        _ldb,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zpftri(transr, uplo, n, ar):
    u"""
    Inverse of complex Hermitian positive definite matrix, matrix
    already factorized by ``zpftrf``, Rectangular Full Packed format.

    For full information please refer to the NAG Library document for
    f07ww

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wwf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies how A has been factorized.

        `uplo` = 'U'

          A = U^HU, where U is upper triangular.

        `uplo` = 'L'

          A = LL^H, where L is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the Cholesky factorization of A stored in RFP format,
        as returned by ``zpftrf``.

        On exit: the factorization is overwritten by the n by n matrix
        A^-1 stored in RFP format.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The leading minor of order *<value>* is not positive
            definite and the factorization could not be completed. Hence
            A itself is not positive definite. This may indicate an
            error in forming the matrix A. There is no function
            specifically designed to invert a Hermitian matrix stored in
            RFP format which is not positive definite; the matrix must
            be treated as a full Hermitian matrix, by calling
            ``zhetri``.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.zpftri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07wwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar_cls = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ar = _ar_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the factorization could not be completed. Hence A itself is not positive',
                    'definite. This may indicate an error in forming the matrix A. There is no',
                    'function specifically designed to invert a Hermitian matrix stored in',
                    'RFP format which is not positive definite; the matrix must be treated as a',
                    'full Hermitian matrix, by calling zhetri.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ar_cls.refresh_py(_ar)
        return_data = None
    _err_class.manage()
    return return_data

def ztftri(transr, uplo, diag, n, ar):
    u"""
    Inverse of complex triangular matrix, Rectangular Full Packed
    format.

    For full information please refer to the NAG Library document for
    f07wx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f07/f07wxf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The matrix A is stored in normal RFP format.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    diag : str, length 1
        Indicates whether A is a nonunit or unit triangular matrix.

        `diag` = 'N'

          A is a nonunit triangular matrix.

        `diag` = 'U'

          A is a unit triangular matrix; the diagonal elements are not
          referenced and are assumed to be 1.

    n : int
        n, the order of the matrix A.

    ar : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangular part (as specified by
        `uplo`) of the n by n Hermitian matrix A, in either normal or
        transposed RFP format (as specified by `transr`). The storage
        format is described in detail in [the F07 Introduction].

        On exit: A is overwritten by A^-1, in the same storage format as
        A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `diag`.

            Constraint: `diag` = 'N' or 'U'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of A is exactly zero. A is
            singular its inverse cannot be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapacklin.ztftri'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f07wxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _diag_py = utils._EngineCharScalarType.to_py(_diag)
    if _diag_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('diag in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='diag',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ar_cls = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ar = _ar_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _diag,
        _n,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
        _diag._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of A is exactly zero.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'A is singular its inverse cannot be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ar_cls.refresh_py(_ar)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data
