# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `matop` Chapter.

``matop`` - Matrix Operations, Including Inversion

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module provides facilities for four types of problem:

- matrix inversion;

- matrix factorizations;

- matrix arithmetic and manipulation;

- matrix functions.

See [Background to the Problems] where these problems are discussed.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def real_symm_posdef_inv(a, n, b):
    u"""
    Inverse of real symmetric positive definite matrix using iterative
    refinement.

    For full information please refer to the NAG Library document for
    f01ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01abf.html

    Parameters
    ----------
    a : float, shape (`n`+1, `n`), modified in place
        On entry: the upper triangle of the n by n positive definite
        symmetric matrix A. The elements of the array below the diagonal
        need not be set.

        On exit: the lower triangle of the inverse matrix X is stored in
        the elements of the array below the diagonal, in rows 2 to n+1;
        x_{ij} is stored in `a`[i,j-1] for i >= j. The upper triangle of
        the original matrix is unchanged.

    n : int
        n, the order of the matrix A.

    b : float, shape (`n`, `n`), modified in place
        On exit: the lower triangle of the inverse matrix X, with x_{ij}
        stored in `b`[i-1,j-1], for i >= j.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The matrix A is not positive definite, possibly due to
            rounding errors.

        (`errno` 2)
            The refinement process failed to converge. The matrix A is
            ill-conditioned.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_symm_posdef_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Workspace, z.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    _z = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _lda,
        _n,
        _b,
        _ldb,
        _z,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is not positive definite, possibly due to rounding errors.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The refinement process failed to converge.',
                    'The matrix A is ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1201:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def real_symm_posdef_inv_noref(n, a):
    u"""
    Inverse of real symmetric positive definite matrix.

    For full information please refer to the NAG Library document for
    f01ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01adf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`+1, `n`), modified in place
        On entry: the upper triangle of the n by n positive definite
        symmetric matrix A. The elements of the array below the diagonal
        need not be set.

        On exit: the lower triangle of the inverse matrix X is stored in
        the elements of the array below the diagonal, in rows 2 to n+1;
        x_{ij} is stored in `a`[i,j-1] for i >= j. The upper triangle of
        the original matrix is unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The matrix A is not positive definite, possibly due to
            rounding errors.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_symm_posdef_inv_noref'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is not positive definite, possibly due to rounding errors.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_pseudinv(m, n, t, a, aijmax, inc):
    u"""
    Pseudo-inverse and rank of real m by n matrix (m >= n).

    For full information please refer to the NAG Library document for
    f01bl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01blf.html

    Parameters
    ----------
    m : int
        m and n, the number of rows and columns in the matrix A.

    n : int
        m and n, the number of rows and columns in the matrix A.

    t : float
        The tolerance used to decide when elements can be regarded as
        zero (see [Further Comments]).

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n rectangular matrix A.

        On exit: the transpose of the pseudo-inverse of A.

    aijmax : float, shape (`n`), modified in place
        On exit: `aijmax`[i-1] contains the element of largest modulus
        in the reduced matrix at the ith stage. If r < n, then only the
        first r+1 elements of `aijmax` have values assigned to them; the
        remaining elements are unused. The ratio
        `aijmax`[0]/`aijmax`[r-1] usually gives an indication of the
        condition number of the original matrix (see [Further
        Comments]).

    inc : int, shape (`n`), modified in place
        On exit: the record of the column interchanges in the
        Householder factorization.

    Returns
    -------
    irank : int
        r, the rank of A as determined using the tolerance `t`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Inverse not found. Incorrect `irank`.

        (`errno` 2)
            Invalid tolerance, `t` too large: `t` = *<value>*.

        (`errno` 2)
            Invalid tolerance, `t` < 0.0: `t` = *<value>*.

        (`errno` 3)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` >= `n`.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_pseudinv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01blft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, aijmax[n].
        utils._BYREF_INT, # Output, irank.
        utils._EngineIntArrayType.c_type, # Output, inc[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, d.
        utils._EngineFloat64ArrayType.c_type, # Workspace, u.
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Workspace, du.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _aijmax_cls = utils._EngineFloat64ArrayType(
        aijmax, {'fun_name': fun_name, 'entity': 'aijmax'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _aijmax = _aijmax_cls.to_ctypes(can_reference_contents=False)
    _irank = utils._EngineIntScalarType.empty_ctypes()
    _inc_cls = utils._EngineIntArrayType(
        inc, {'fun_name': fun_name, 'entity': 'inc'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _inc = _inc_cls.to_ctypes(can_reference_contents=False)
    _d = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _u = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ldu = utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    _du = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _t,
        _a,
        _lda,
        _aijmax,
        _irank,
        _inc,
        _d,
        _u,
        _ldu,
        _du,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Inverse not found. Incorrect irank.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Invalid tolerance, t too large:',
                    't = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Invalid tolerance, t < 0.0:',
                    't = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _aijmax_cls.refresh_py(_aijmax)
        _inc_cls.refresh_py(_inc)
        _irank_py = utils._EngineIntScalarType.to_py(_irank)
        return_data = _irank_py
    _err_class.manage()
    return return_data

def real_gen_sparse_lu(n, nz, warnlev, errlev, a, licn, irn, lirn, icn, pivot, w, lblock, grow, abort, io_manager=None):
    u"""
    LU factorization of real sparse matrix.

    For full information please refer to the NAG Library document for
    f01br

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01brf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nz : int
        The number of nonzero elements in the matrix A.

    warnlev : int
        Setting `warnlev` = 0 disables output of warning messages during
        the process of the computation. Setting `warnlev` = 1 enables
        these messages.

    errlev : int
        Setting `errlev` = 0 disables output of error messages during
        the process of the computation. Setting `errlev` = 1 enables
        these messages.

    a : float, shape (`licn`), modified in place
        On entry: `a`[i-1], for i = 1,2,...,`nz`, must contain the
        nonzero elements of the sparse matrix A. They can be in any
        order since ``real_gen_sparse_lu`` will reorder them.

        On exit: the nonzero elements in the LU factorization. The array
        must not be changed by you between a call of
        ``real_gen_sparse_lu`` and a call of
        ``linsys.real_sparse_fac_solve``.

    licn : int
        Since the factorization is returned in `a` and `comm`['icn'],
        `licn` should be large enough to accommodate this and should
        ordinarily be 2 to 4 times as large as `nz`.

    irn : int, shape (`lirn`), modified in place
        On entry: `irn`[i-1], for i = 1,2,...,`nz`, must contain the row
        index of the nonzero element stored in `a`[i-1].

        On exit: `irn` is overwritten and is not needed for subsequent
        calls of ``real_gen_sparse_lu_reuse`` or
        ``linsys.real_sparse_fac_solve``.

    lirn : int
        It need not be as large as `licn`; normally it will not need to
        be very much greater than `nz`.

    icn : int, shape (`licn`), modified in place
        On entry: `icn`[i-1], for i = 1,2,...,`nz`, must contain the
        column index of the nonzero element stored in `a`[i-1].

        On exit: the column indices of the nonzero elements in the
        factorization

    pivot : float
        Should have a value in the range 0.0 <= `pivot` <= 0.9999 and is
        used to control the choice of pivots. If `pivot` < 0.0, the
        value 0.0 is assumed, and if `pivot` > 0.9999, the value 0.9999
        is assumed. When searching a row for a pivot, any element is
        excluded which is less than `pivot` times the largest of those
        elements in the row available as pivots. Thus decreasing `pivot`
        biases the algorithm to maintaining sparsity at the expense of
        stability.

    w : float, shape (`n`), modified in place
        On exit: if `grow` = True, `w`[0] contains an estimate (an upper
        bound) of the increase in size of elements encountered during
        the factorization (see `grow`); the rest of the array is used as
        workspace.

        If `grow` = False, the array is not used.

    lblock : bool
        If `lblock` = True, the matrix is preordered into block lower
        triangular form before the LU factorization is performed;
        otherwise the entire matrix is factorized.

    grow : bool
        If `grow` = True, then on exit `w`[0] contains an estimate (an
        upper bound) of the increase in size of elements encountered
        during the factorization. If the matrix is well-scaled (see
        [Further Comments]), then a high value for `w`[0] indicates that
        the LU factorization may be inaccurate and you should be wary of
        the results and perhaps increase the argument `pivot` for
        subsequent runs (see [Accuracy]).

    abort : bool, shape (4)
        If `abort`[0] = True, ``real_gen_sparse_lu`` will exit
        immediately on detecting a structural singularity (one that
        depends on the pattern of nonzeros) and return `errno` = 1;
        otherwise it will complete the factorization (see [Further
        Comments]).

        If `abort`[1] = True, ``real_gen_sparse_lu`` will exit
        immediately on detecting a numerical singularity (one that
        depends on the numerical values) and return `errno` = 2;
        otherwise it will complete the factorization (see [Further
        Comments]).

        If `abort`[2] = True, ``real_gen_sparse_lu`` will exit
        immediately (with `errno` = 5) when the arrays `a` and
        `comm`['icn'] are filled up by the previously factorized, active
        and unfactorized parts of the matrix; otherwise it continues so
        that better guidance on necessary array sizes can be given in
        `comm`['idisp'][5] and `comm`['idisp'][6], and will exit with
        errno in the range 4 to 6.

        Note that there is always an immediate error exit if the array
        `irn` is too small.

        If `abort`[3] = True, ``real_gen_sparse_lu`` exits immediately
        (with `errno` = 13) if it finds duplicate elements in the input
        matrix.

        If `abort`[3] = False, ``real_gen_sparse_lu`` proceeds using a
        value equal to the sum of the duplicate elements.

        In either case details of each duplicate element are output on
        the file object associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`), unless
        suppressed by the value of errno on entry.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix is structurally singular -- decomposition aborted
            RANK = *<value>*.

        (`errno` 1)
            Matrix is structurally singular -- decomposition aborted.

        (`errno` 2)
            Matrix is numerically singular -- decomposition aborted.

        (`errno` 3)
            `lirn` too small. Decomposition aborted at stage *<value>*
            in block *<value>* with first row *<value>* and last row
            *<value>*. To continue set `lirn` to at least *<value>*.

        (`errno` 3)
            `lirn` too small. Decomposition aborted at stage *<value>*
            in block *<value>* with first row *<value>* and last row
            *<value>*.

        (`errno` 4)
            `licn` much too small. Decomposition aborted at stage
            *<value>* in block *<value>* with first row *<value>* and
            last row *<value>*. To continue set `licn` to at least
            *<value>*.

        (`errno` 4)
            `licn` much too small. Decomposition aborted at stage
            *<value>* in block *<value>* with first row *<value>* and
            last row *<value>*.

        (`errno` 5)
            `licn` too small. Decomposition aborted at stage *<value>*
            in block *<value>* with first row *<value>* and last row
            *<value>*.

        (`errno` 5)
            `licn` too small. Decomposition aborted at stage *<value>*
            in block *<value>* with first row *<value>* and last row
            *<value>*. To continue set `licn` to at least *<value>*.

        (`errno` 5)
            `licn` too small. For successful decomposition set `licn` to
            at least *<value>*.

        (`errno` 6)
            `licn` and `lirn` too small. Decomposition aborted at stage
            *<value>* in block *<value>* with first row *<value>* and
            last row *<value>*. To continue set `lirn` to at least
            *<value>*.

        (`errno` 6)
            `licn` and `lirn` too small. Decomposition aborted at stage
            *<value>* in block *<value>* with first row *<value>* and
            last row *<value>*.

        (`errno` 7)
            `licn` not big enough for permutation -- increase by
            *<value>*.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 9)
            On entry, `nz` = *<value>*.

            Constraint: `nz` > 0.

        (`errno` 10)
            On entry, `licn` = *<value>* and `nz` = *<value>*.

            Constraint: `licn` >= `nz`.

        (`errno` 11)
            On entry, `lirn` = *<value>* and `nz` = *<value>*.

            Constraint: `lirn` >= `nz`.

        (`errno` 12)
            On entry, `irn`[I-1] or `icn`[I-1] is out of range: I =
            *<value>*, `a`[I-1] = *<value>* `irn`[I-1] = *<value>*,
            `icn`[I-1] = *<value>*.

        (`errno` 13)
            On entry, duplicate elements found -- see advisory messages.

        (`errno` 14)
            On entry, `warnlev` = *<value>*.

            Constraint: `warnlev` = 0 or 1.

        (`errno` 15)
            On entry, `errlev` = *<value>*.

            Constraint: `errlev` = 0 or 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -2)
            Matrix is numerically singular -- decomposition completed.

        (`errno` -1)
            Matrix is structurally singular -- decomposition completed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_sparse_lu'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01brft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nz.
        utils._BYREF_INT, # Input, warnlev.
        utils._BYREF_INT, # Input, errlev.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[licn].
        utils._BYREF_INT, # Input, licn.
        utils._EngineIntArrayType.c_type, # Input/Output, irn[lirn].
        utils._BYREF_INT, # Input, lirn.
        utils._EngineIntArrayType.c_type, # Input/Output, icn[licn].
        utils._BYREF_FLOAT64, # Input, pivot.
        utils._EngineIntArrayType.c_type, # Output, ikeep[5*n].
        utils._EngineIntArrayType.c_type, # Workspace, iw.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._BYREF_LOGICAL, # Input, lblock.
        utils._BYREF_LOGICAL, # Input, grow.
        utils._EngineLogicalArrayType.c_type, # Input, abort[4].
        utils._EngineIntArrayType.c_type, # Output, idisp[10].
        utils._BYREF_INT, # Input, nadv.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nz = utils._EngineIntScalarType(
        nz, {'fun_name': fun_name, 'entity': 'nz'},
    ).to_ctypes()
    _warnlev = utils._EngineIntScalarType(
        warnlev, {'fun_name': fun_name, 'entity': 'warnlev'},
    ).to_ctypes()
    _errlev = utils._EngineIntScalarType(
        errlev, {'fun_name': fun_name, 'entity': 'errlev'},
    ).to_ctypes()
    _licn = utils._EngineIntScalarType(
        licn, {'fun_name': fun_name, 'entity': 'licn'},
    ).to_ctypes()
    _licn_py = utils._EngineIntScalarType.to_py(_licn)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_licn_py,),
    )
    _a = _a_cls.to_ctypes()
    _lirn = utils._EngineIntScalarType(
        lirn, {'fun_name': fun_name, 'entity': 'lirn'},
    ).to_ctypes()
    _lirn_py = utils._EngineIntScalarType.to_py(_lirn)
    _irn_cls = utils._EngineIntArrayType(
        irn, {'fun_name': fun_name, 'entity': 'irn'},
        exp_shape=(_lirn_py,),
    )
    _irn = _irn_cls.to_ctypes()
    _icn_cls = utils._EngineIntArrayType(
        icn, {'fun_name': fun_name, 'entity': 'icn'},
        exp_shape=(_licn_py,),
    )
    _icn = _icn_cls.to_ctypes()
    _comm_py = {}
    _comm_py['icn'] = _icn
    _pivot = utils._EngineFloat64ScalarType(
        pivot, {'fun_name': fun_name, 'entity': 'pivot'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ikeep = utils._EngineIntArrayType.empty_ctypes(5*_n_py)
    _comm_py['ikeep'] = _ikeep
    _iw = utils._EngineIntArrayType.empty_ctypes(0)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _lblock = utils._EngineLogicalScalarType(
        lblock, {'fun_name': fun_name, 'entity': 'lblock'},
    ).to_ctypes()
    _grow = utils._EngineLogicalScalarType(
        grow, {'fun_name': fun_name, 'entity': 'grow'},
    ).to_ctypes()
    _abort = utils._EngineLogicalArrayType(
        abort, {'fun_name': fun_name, 'entity': 'abort'},
        exp_shape=(4,),
    ).to_ctypes()
    _idisp = utils._EngineIntArrayType.empty_ctypes(10)
    _comm_py['idisp'] = _idisp
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _n,
        _nz,
        _warnlev,
        _errlev,
        _a,
        _licn,
        _irn,
        _lirn,
        _icn,
        _pivot,
        _ikeep,
        _iw,
        _w,
        _lblock,
        _grow,
        _abort,
        _idisp,
        _nadv,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix is numerically singular -- decomposition completed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix is structurally singular -- decomposition completed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix is structurally singular -- decomposition aborted',
                    'RANK = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix is structurally singular -- decomposition aborted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix is numerically singular -- decomposition aborted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'lirn too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'row {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'To continue set lirn to at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'lirn too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'row {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn much too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'row {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'To continue set licn to at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn much too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'row {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'row {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'row {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'To continue set licn to at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn too small.',
                    'For successful decomposition set licn to at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn and lirn too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'row {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'To continue set lirn to at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn and lirn too small.',
                    'Decomposition aborted at stage {:s} in block'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} with first row {:s} and last row'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'licn not big enough for permutation -- increase by {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nz > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, licn = {:s} and nz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: licn >= nz.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lirn = {:s} and nz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lirn >= nz.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irn[I-1] or icn[I-1] is out of range:',
                    'I = {:s}, a[I-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'irn[I-1] = {:s}, icn[I-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, duplicate elements found -- see advisory messages.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, warnlev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: warnlev = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, errlev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: errlev = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _irn_cls.refresh_py(_irn)
        _w_cls.refresh_py(_w)
        return_data = _comm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['comm'],
            )(
                _comm_py,
            )
    _err_class.manage()
    return return_data

def real_gen_sparse_lu_reuse(n, nz, warnlev, a, licn, ivect, jvect, comm, w, grow, eta, abort, io_manager=None):
    u"""
    LU factorization of real sparse matrix with known sparsity pattern.

    For full information please refer to the NAG Library document for
    f01bs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01bsf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nz : int
        The number of nonzero elements in the matrix A.

    warnlev : int
        Setting `warnlev` = 0 disables output of warning messages during
        the process of the computation. Setting `warnlev` = 1 enables
        these messages.

    a : float, shape (`licn`), modified in place
        On entry: `a`[i-1], for i = 1,2,...,`nz`, must contain the
        nonzero elements of the sparse matrix A. They can be in any
        order since ``real_gen_sparse_lu_reuse`` will reorder them.

        On exit: the nonzero elements in the LU factorization. The array
        must not be changed by you between a call of
        ``real_gen_sparse_lu_reuse`` and a call of
        ``linsys.real_sparse_fac_solve``.

    licn : int
        It should have the same value as it had for
        ``real_gen_sparse_lu``.

    ivect : int, shape (`nz`)
        `ivect`[i-1] and `jvect`[i-1], for i = 1,2,...,`nz`, must
        contain the row index and the column index respectively of the
        nonzero element stored in `a`[i-1].

    jvect : int, shape (`nz`)
        `ivect`[i-1] and `jvect`[i-1], for i = 1,2,...,`nz`, must
        contain the row index and the column index respectively of the
        nonzero element stored in `a`[i-1].

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_gen_sparse_lu``.

    w : float, shape (`n`), modified in place
        On exit: if `grow` = True, `w`[0] contains an estimate (an upper
        bound) of the increase in size of elements encountered during
        the factorization (see `grow`); the rest of the array is used as
        workspace.

        If `grow` = False, the array is not used.

    grow : bool
        If `grow` = True, then on exit `w`[0] contains an estimate (an
        upper bound) of the increase in size of elements encountered
        during the factorization. If the matrix is well-scaled (see
        [Further Comments]), then a high value for `w`[0] indicates that
        the LU factorization may be inaccurate and you should be wary of
        the results and perhaps increase the argument pivot for
        subsequent runs (see [Accuracy]).

    eta : float
        The relative pivot threshold below which an error diagnostic is
        provoked and errno is set to `errno` = 7. If `eta` is greater
        than 1.0, then no check on pivot size is made.

    abort : bool
        If `abort` = True, ``real_gen_sparse_lu_reuse`` exits
        immediately (with `errno` = 8) if it finds duplicate elements in
        the input matrix.

        If `abort` = False, ``real_gen_sparse_lu_reuse`` proceeds using
        a value equal to the sum of the duplicate elements.

        In either case details of each duplicate element are output on
        the file object associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`), unless
        suppressed by the value of errno on entry.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    rpmin : float
        If `eta` is less than 1.0, then `rpmin` gives the smallest ratio
        of the pivot to the largest element in the row of the
        corresponding upper triangular factor thus monitoring the
        stability of the factorization. If `rpmin` is very small it may
        be advisable to perform a new factorization using
        ``real_gen_sparse_lu``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nz` = *<value>*.

            Constraint: `nz` > 0.

        (`errno` 3)
            On entry, `licn` = *<value>* and `nz` = *<value>*.

            Constraint: `licn` >= `nz`.

        (`errno` 4)
            On entry, irn[I-1] in ``real_gen_sparse_lu`` or
            `comm`['icn'][I-1] is out of range: I = *<value>*, `a`[I-1]
            = *<value>* irn[I-1] = *<value>* in ``real_gen_sparse_lu``,
            `comm`['icn'][I-1] = *<value>*.

        (`errno` 5)
            Nonzero element (*<value>*, *<value>*) was not in L/U
            pattern.

        (`errno` 5)
            Nonzero element (*<value>*, *<value>*) in zero off-diagonal
            block.

        (`errno` 6)
            Numerical singularity in row *<value>* - decomposition
            aborted.

        (`errno` 8)
            On entry, duplicate elements found -- see advisory messages.

        (`errno` 9)
            On entry, `warnlev` = *<value>*.

            Constraint: `warnlev` = 0 or 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 7)
            Subthreshold pivot in row *<value>* - decomposition
            completed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_sparse_lu_reuse'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01bsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nz.
        utils._BYREF_INT, # Input, warnlev.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[licn].
        utils._BYREF_INT, # Input, licn.
        utils._EngineIntArrayType.c_type, # Input, ivect[nz].
        utils._EngineIntArrayType.c_type, # Input, jvect[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, icn[licn].
        utils._EngineIntArrayType.c_type, # Input, ikeep[5*n].
        utils._EngineIntArrayType.c_type, # Workspace, iw.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._BYREF_LOGICAL, # Input, grow.
        utils._BYREF_FLOAT64, # Input, eta.
        utils._BYREF_FLOAT64, # Output, rpmin.
        utils._BYREF_LOGICAL, # Input, abort.
        utils._EngineIntArrayType.c_type, # Input, idisp[2].
        utils._BYREF_INT, # Input, nadv.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nz = utils._EngineIntScalarType(
        nz, {'fun_name': fun_name, 'entity': 'nz'},
    ).to_ctypes()
    _warnlev = utils._EngineIntScalarType(
        warnlev, {'fun_name': fun_name, 'entity': 'warnlev'},
    ).to_ctypes()
    _licn = utils._EngineIntScalarType(
        licn, {'fun_name': fun_name, 'entity': 'licn'},
    ).to_ctypes()
    _licn_py = utils._EngineIntScalarType.to_py(_licn)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_licn_py,),
    )
    _a = _a_cls.to_ctypes()
    _nz_py = utils._EngineIntScalarType.to_py(_nz)
    _ivect = utils._EngineIntArrayType(
        ivect, {'fun_name': fun_name, 'entity': 'ivect'},
        exp_shape=(_nz_py,),
    ).to_ctypes()
    _jvect = utils._EngineIntArrayType(
        jvect, {'fun_name': fun_name, 'entity': 'jvect'},
        exp_shape=(_nz_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icn', 'ikeep', 'idisp'),
    )
    _icn = comm['icn']
    _ikeep = comm['ikeep']
    _iw = utils._EngineIntArrayType.empty_ctypes(0)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _grow = utils._EngineLogicalScalarType(
        grow, {'fun_name': fun_name, 'entity': 'grow'},
    ).to_ctypes()
    _eta = utils._EngineFloat64ScalarType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
    ).to_ctypes()
    _rpmin = utils._EngineFloat64ScalarType.empty_ctypes()
    _abort = utils._EngineLogicalScalarType(
        abort, {'fun_name': fun_name, 'entity': 'abort'},
    ).to_ctypes()
    _idisp = comm['idisp']
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _n,
        _nz,
        _warnlev,
        _a,
        _licn,
        _ivect,
        _jvect,
        _icn,
        _ikeep,
        _iw,
        _w,
        _grow,
        _eta,
        _rpmin,
        _abort,
        _idisp,
        _nadv,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nz > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, licn = {:s} and nz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: licn >= nz.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irn[I-1] in real_gen_sparse_lu or comm[\'icn\'][I-1] is out of range:',
                    'I = {:s}, a[I-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'irn[I-1] = {:s} in real_gen_sparse_lu, comm[\'icn\'][I-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Nonzero element ({:s}, {:s}) was not in'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'L/U pattern.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Nonzero element ({:s}, {:s}) in zero'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'off-diagonal block.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Numerical singularity in row {:s} - decomposition aborted.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Subthreshold pivot in row {:s} - decomposition completed.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, duplicate elements found -- see advisory messages.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, warnlev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: warnlev = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _rpmin_py = utils._EngineFloat64ScalarType.to_py(_rpmin)
        return_data = _rpmin_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['rpmin'],
            )(
                _rpmin_py,
            )
    _err_class.manage()
    return return_data

def real_symm_posdef_fac(n, m1, k, a):
    u"""
    ULDL^TU^T factorization of real symmetric positive definite band
    matrix.

    For full information please refer to the NAG Library document for
    f01bu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01buf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    m1 : int
        m+1, where m is the number of nonzero superdiagonals in A.
        Normally `m1`≪`n`.

    k : int
        k, the change-over point in the decomposition.

    a : float, shape (`m1`, `n`), modified in place
        On entry: the upper triangle of the n by n symmetric band matrix
        A, with the diagonal of the matrix stored in the (m+1)th row of
        the array, and the m superdiagonals within the band stored in
        the first m rows of the array. Each column of the matrix is
        stored in the corresponding column of the array. For example, if
        n = 6 and m = 2, the storage scheme is

            [table omitted]

        Elements in the top left corner of the array are not used. The
        following code assigns the matrix elements within the band to
        the correct elements of the array:

        ::

          for j in range(A.shape[1]):
              for i in range(max(0, j+1-m1), j+1):
                  a[i-j+m1-1, j] = A[i, j]

        On exit: A is overwritten by the corresponding elements of L, D
        and U.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>* and `n` = *<value>*.

            Constraint: `k` <= `n`.

        (`errno` 1)
            On entry, `k` = *<value>* and `m1`-1 = *<value>*.

            Constraint: `k` >= `m1`-1.

        (`errno` 2)
            The matrix A is not positive definite.

        (`errno` 3)
            The matrix A is not positive definite.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_symm_posdef_fac'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01buft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m1.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m1 = utils._EngineIntScalarType(
        m1, {'fun_name': fun_name, 'entity': 'm1'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m1_py = utils._EngineIntScalarType.to_py(_m1)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m1_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _m1,
        _k,
        _a,
        _lda,
        _w,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and m1-1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k >= m1-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def real_symm_posdef_geneig(n, ma1, mb1, k, a, b):
    u"""
    Reduction to standard form, generalized real symmetric-definite
    banded eigenproblem.

    For full information please refer to the NAG Library document for
    f01bv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01bvf.html

    Parameters
    ----------
    n : int
        n, the order of the matrices A, B and C.

    ma1 : int
        m_A+1, where m_A is the number of nonzero superdiagonals in A.
        Normally `ma1`≪`n`.

    mb1 : int
        m_B+1, where m_B is the number of nonzero superdiagonals in B.

    k : int
        k, the change-over point in the transformations. It must be the
        same as the value used by ``real_symm_posdef_fac`` in the
        decomposition of B.

    a : float, shape (`ma1`, `n`), modified in place
        On entry: the upper triangle of the n by n symmetric band matrix
        A, with the diagonal of the matrix stored in the (m_A+1)th row
        of the array, and the m_A superdiagonals within the band stored
        in the first m_A rows of the array. Each column of the matrix is
        stored in the corresponding column of the array. For example, if
        n = 6 and m_A = 2, the storage scheme is

            [table omitted]

        Elements in the top left corner of the array need not be set.
        The following code assigns the matrix elements within the band
        to the correct elements of the array:

        ::

          for j in range(A.shape[1]):
              for i in range(max(0, j+1-ma1), j+1):
                  a[i-j+ma1-1, j] = A[i, j]

        On exit: is overwritten by the corresponding elements of C.

    b : float, shape (`mb1`, `n`), modified in place
        On entry: the elements of the decomposition of matrix B as
        returned by ``real_symm_posdef_fac``.

        On exit: the elements of `b` will have been permuted.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mb1` = *<value>* and `ma1` = *<value>*.

            Constraint: `mb1` <= `ma1`.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_symm_posdef_geneig'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01bvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ma1.
        utils._BYREF_INT, # Input, mb1.
        utils._BYREF_INT, # Input, m3.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Workspace, v.
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ma1 = utils._EngineIntScalarType(
        ma1, {'fun_name': fun_name, 'entity': 'ma1'},
    ).to_ctypes()
    _mb1 = utils._EngineIntScalarType(
        mb1, {'fun_name': fun_name, 'entity': 'mb1'},
    ).to_ctypes()
    _m3 = utils._EngineIntScalarType.c_type(0)
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _ma1_py = utils._EngineIntScalarType.to_py(_ma1)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_ma1_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _mb1_py = utils._EngineIntScalarType.to_py(_mb1)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_mb1_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _v = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _vl_ma_py = _ma1_py-1
    _vl_mb_py = _mb1_py-1
    _ldv = utils._EngineIntScalarType(
        _vl_ma_py+_vl_mb_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _ma1,
        _mb1,
        _m3,
        _k,
        _a,
        _lda,
        _b,
        _ldb,
        _v,
        _ldv,
        _w,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mb1 = {:s} and ma1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mb1 <= ma1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_matmul(a, b, c, n, p, m, opt):
    u"""
    Multiplication of real matrices.

    For full information please refer to the NAG Library document for
    f01ck

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01ckf.html

    Parameters
    ----------
    a : float, shape (`n`, `p`), modified in place
        On exit: if `opt` = 1, `a` contains the result of the matrix
        multiplication.

    b : float, shape (`n`, `m`), modified in place
        On entry: the n by m matrix B.

        On exit: if `opt` = 2, `b` contains the result of the
        multiplication.

    c : float, shape (`m`, `p`), modified in place
        On entry: the m by p matrix C.

        On exit: if `opt` = 3, `c` contains the result of the
        multiplication.

    n : int
        n, the number of rows of the array A and of the array B.

    p : int
        p, the number of columns of the array A and of the array C.

    m : int
        m, the number of columns of the array B and rows of the array C.

    opt : int
        The value of `opt` determines which array is to contain the
        final result.

        `opt` = 1

          `a` must be distinct from `b` and `c` and, on exit, contains
          the result. `b` and `c` need not be distinct in this case.

        `opt` = 2

          `b` must be distinct from `c` and on exit, contains the
          result. `a` is not used in this case and need not be distinct
          from `b` or `c`.

        `opt` = 3

          `c` must be distinct from `b` and on exit, contains the
          result. `a` is not used in this case and need not be distinct
          from `b` or `c`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `m` = *<value>* and `p` = *<value>*.

            Constraint: if `opt` = 2, `m` = `p`.

        (`errno` 3)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: if `opt` = 3, `n` = `m`.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matmul'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01ckft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Output, a[n, p].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[n, m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[m, p].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Workspace, z.
        utils._BYREF_INT, # Input, iz.
        utils._BYREF_INT, # Input, opt.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _p_py = utils._EngineIntScalarType.to_py(_p)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _p_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _p_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _z = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iz = utils._EngineIntScalarType.c_type(0)
    _opt = utils._EngineIntScalarType(
        opt, {'fun_name': fun_name, 'entity': 'opt'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _c,
        _n,
        _p,
        _m,
        _z,
        _iz,
        _opt,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if opt = 2, m = p.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if opt = 3, n = m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_trans_inplace(a, m, n, mn):
    u"""
    Transposition of a real matrix.

    For full information please refer to the NAG Library document for
    f01cr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01crf.html

    Parameters
    ----------
    a : float, shape (`mn`), modified in place
        On entry: the elements of the m by n matrix A, stored by
        columns.

        On exit: the elements of the transpose matrix, also stored by
        columns.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    mn : int
        mn, the value m*n.

    Raises
    ------
    NagValueError
        (`errno` i < 0)
            A serious internal error has occurred in
            ``real_gen_trans_inplace``.

        (`errno` 1)
            On entry, `m` = *<value>*, `n` = *<value>* and `mn` =
            *<value>*.

            Constraint: `mn` = `m`*`n`.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_trans_inplace'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01crft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[mn].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, mn.
        utils._EngineIntArrayType.c_type, # Workspace, move.
        utils._BYREF_INT, # Input, lmove.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _mn = utils._EngineIntScalarType(
        mn, {'fun_name': fun_name, 'entity': 'mn'},
    ).to_ctypes()
    _mn_py = utils._EngineIntScalarType.to_py(_mn)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_mn_py,),
    )
    _a = _a_cls.to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _move = utils._EngineIntArrayType.empty_ctypes(0)
    _lmove = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _m,
        _n,
        _mn,
        _move,
        _lmove,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py < 0 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious internal error has occurred in real_gen_trans_inplace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'mn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: mn = m*n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def real_addsub(transa, transb, m, n, alpha, a, beta, b, c):
    u"""
    Sum or difference of two real matrices, optional scaling and
    transposition.

    For full information please refer to the NAG Library document for
    f01ct

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01ctf.html

    Parameters
    ----------
    transa : str, length 1
        `transa` and `transb` must specify whether or not the matrix A
        and the matrix B, respectively, are to be transposed before
        addition.

        `transa` or `transb` = 'N'

          The matrix will not be transposed.

        `transa` or `transb` = 'T' or 'C'

          The matrix will be transposed.

    transb : str, length 1
        `transa` and `transb` must specify whether or not the matrix A
        and the matrix B, respectively, are to be transposed before
        addition.

        `transa` or `transb` = 'N'

          The matrix will not be transposed.

        `transa` or `transb` = 'T' or 'C'

          The matrix will be transposed.

    m : int
        m, the number of rows of the matrices A and B or their
        transposes. Also the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrices A and B or their
        transposes. Also the number of columns of the matrix C.

    alpha : float
        The scalar alpha, by which matrix A is multiplied before
        addition.

    a : float, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `transa`='N': `m`; otherwise: `n`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `transa`='N': `n`; otherwise: `m`.

        If alpha = 0.0, the elements of array `a` need not be assigned.
        If alpha != 0.0, then if `transa` = 'N', the leading m by n part
        of `a` must contain the matrix A, otherwise the leading n by m
        part of `a` must contain the matrix A.

    beta : float
        The scalar beta, by which matrix B is multiplied before
        addition.

    b : float, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `transb`='N': `m`; otherwise: `n`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `transb`='N': `n`; otherwise: `m`.

        If beta = 0.0, the elements of array `b` need not be assigned.
        If beta != 0.0, then if `transa` = 'N', the leading m by n part
        of `b` must contain the matrix B, otherwise the leading n by m
        part of `b` must contain the matrix B.

    c : float, shape (`m`, `n`), modified in place
        On exit: the elements of the m by n matrix C.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `transb` = *<value>*.

            Constraint: `transb` = 'N', 'T' or 'C'.

        (`errno` 1)
            On entry, `transa` = *<value>*.

            Constraint: `transa` = 'N', 'T' or 'C'.

        (`errno` 2)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` >= 0 and `n` >= 0.

        (`errno` 3)
            On entry, lda = *<value>* and `n` = *<value>*.

            Constraint: if `transa` = *<value>*, lda >= max(1, `n`).

        (`errno` 3)
            On entry, lda = *<value>* and `m` = *<value>*.

            Constraint: if `transa` = *<value>*, lda >= max(1, `m`).

        (`errno` 4)
            On entry, ldb = *<value>* and `n` = *<value>*.

            Constraint: if `transb` = *<value>*, ldb >= max(1, `n`).

        (`errno` 4)
            On entry, ldb = *<value>* and `m` = *<value>*.

            Constraint: if `transb` = *<value>*, ldb >= max(1, `m`).

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_addsub'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01ctft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transa(len=1).
        utils._BYREF_STRING, # Input, transb(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, alpha.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, beta.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, *].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transa.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transb.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transa = utils._EngineCharScalarType(
        transa, {'fun_name': fun_name, 'entity': 'transa'}, exp_chrlen=1,
    ).to_ctypes()
    _transb = utils._EngineCharScalarType(
        transb, {'fun_name': fun_name, 'entity': 'transb'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _alpha = utils._EngineFloat64ScalarType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _transa_py = utils._EngineCharScalarType.to_py(_transa)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if _transa_py in ('N', 'n'):
        _a_dim1 = _m_py
    else:
        _a_dim1 = _n_py
    if _transa_py in ('N', 'n'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = _m_py
    exp_shape_a = (_a_dim1, _a_dim2)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _beta = utils._EngineFloat64ScalarType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _transb_py = utils._EngineCharScalarType.to_py(_transb)
    if _transb_py in ('N', 'n'):
        _b_dim1 = _m_py
    else:
        _b_dim1 = _n_py
    if _transb_py in ('N', 'n'):
        _b_dim2 = _n_py
    else:
        _b_dim2 = _m_py
    exp_shape_b = (_b_dim1, _b_dim2)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=exp_shape_b,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _transa,
        _transb,
        _m,
        _n,
        _alpha,
        _a,
        _lda,
        _beta,
        _b,
        _ldb,
        _c,
        _ldc,
        _errbuf,
        _ierr,
        _transa._length_,
        _transb._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, transb = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: transb = \'N\', \'T\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, transa = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: transa = \'N\', \'T\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= 0 and n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transa = \'{:s}\', lda >= max(1, n).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transa = \'{:s}\', lda >= max(1, m).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transb = \'{:s}\', ldb >= max(1, n).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transb = \'{:s}\', ldb >= max(1, m).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def complex_addsub(transa, transb, m, n, alpha, a, beta, b, c):
    u"""
    Sum or difference of two complex matrices, optional scaling and
    transposition.

    For full information please refer to the NAG Library document for
    f01cw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01cwf.html

    Parameters
    ----------
    transa : str, length 1
        `transa` and `transb` must specify whether or not the matrix A
        and the matrix B, respectively, are to be transposed before
        addition.

        `transa` or `transb` = 'N'

          The matrix will not be transposed.

        `transa` or `transb` = 'T'

          The matrix will be transposed.

        `transa` or `transb` = 'C'

          The matrix will be transposed and conjugated.

    transb : str, length 1
        `transa` and `transb` must specify whether or not the matrix A
        and the matrix B, respectively, are to be transposed before
        addition.

        `transa` or `transb` = 'N'

          The matrix will not be transposed.

        `transa` or `transb` = 'T'

          The matrix will be transposed.

        `transa` or `transb` = 'C'

          The matrix will be transposed and conjugated.

    m : int
        m, the number of rows of the matrices A and B or their
        transposes. Also the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrices A and B or their
        transposes. Also the number of columns of the matrix C.

    alpha : complex
        The scalar alpha, by which matrix A is multiplied before
        addition.

    a : complex, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `transa`='N': `m`; otherwise: `n`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `alpha`!=0 and `transa`='N': `n`; if
        `alpha`!=0 and `transa` in ('T', 'C'): `m`; if `alpha`=0: 1;
        otherwise: 0.

        The matrix A. If alpha = 0, the array `a` is not referenced.

    beta : complex
        The scalar beta, by which matrix B is multiplied before
        addition.

    b : complex, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `transb`='N': `m`; otherwise: `n`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `beta`!=0 and `transb`='N': `n`; if
        `beta`!=0 and `transb` in ('T', 'C'): `m`; if `beta`=0: 1;
        otherwise: 0.

        The matrix B. If beta = 0, the array `b` is not referenced.

    c : complex, shape (`m`, `n`), modified in place
        On exit: the elements of the m by n matrix C.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `transb` = *<value>*.

            Constraint: `transb` = 'N', 'T' or 'C'.

        (`errno` 1)
            On entry, `transa` = *<value>*.

            Constraint: `transa` = 'N', 'T' or 'C'.

        (`errno` 2)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` >= 0 and `n` >= 0.

        (`errno` 3)
            On entry, lda = *<value>* and `n` = *<value>*.

            Constraint: if `transa` = *<value>*, lda >= max(1, `n`).

        (`errno` 3)
            On entry, lda = *<value>* and `m` = *<value>*.

            Constraint: if `transa` = *<value>*, lda >= max(1, `m`).

        (`errno` 4)
            On entry, ldb = *<value>* and `n` = *<value>*.

            Constraint: if `transb` = *<value>*, ldb >= max(1, `n`).

        (`errno` 4)
            On entry, ldb = *<value>* and `m` = *<value>*.

            Constraint: if `transb` = *<value>*, ldb >= max(1, `m`).

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_addsub'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01cwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transa(len=1).
        utils._BYREF_STRING, # Input, transb(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_COMPLEX128, # Input, alpha.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_COMPLEX128, # Input, beta.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, *].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transa.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transb.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transa = utils._EngineCharScalarType(
        transa, {'fun_name': fun_name, 'entity': 'transa'}, exp_chrlen=1,
    ).to_ctypes()
    _transb = utils._EngineCharScalarType(
        transb, {'fun_name': fun_name, 'entity': 'transb'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _alpha = utils._EngineComplex128ScalarType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _transa_py = utils._EngineCharScalarType.to_py(_transa)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _alpha_py = utils._EngineComplex128ScalarType.to_py(_alpha)
    if _transa_py in ('N', 'n'):
        _a_dim1 = _m_py
    else:
        _a_dim1 = _n_py
    if _alpha_py != 0 and (_transa_py in ('N', 'n')):
        _a_dim2 = _n_py
    elif _alpha_py != 0 and _transa_py in ('T', 't', 'C', 'c'):
        _a_dim2 = _m_py
    elif _alpha_py == 0:
        _a_dim2 = 1
    else:
        _a_dim2 = 0
    exp_shape_a = (_a_dim1, _a_dim2)
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _beta = utils._EngineComplex128ScalarType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _transb_py = utils._EngineCharScalarType.to_py(_transb)
    _beta_py = utils._EngineComplex128ScalarType.to_py(_beta)
    if _transb_py in ('N', 'n'):
        _b_dim1 = _m_py
    else:
        _b_dim1 = _n_py
    if _beta_py != 0 and (_transb_py in ('N', 'n')):
        _b_dim2 = _n_py
    elif _beta_py != 0 and _transb_py in ('T', 't', 'C', 'c'):
        _b_dim2 = _m_py
    elif _beta_py == 0:
        _b_dim2 = 1
    else:
        _b_dim2 = 0
    exp_shape_b = (_b_dim1, _b_dim2)
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=exp_shape_b,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _transa,
        _transb,
        _m,
        _n,
        _alpha,
        _a,
        _lda,
        _beta,
        _b,
        _ldb,
        _c,
        _ldc,
        _errbuf,
        _ierr,
        _transa._length_,
        _transb._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, transb = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: transb = \'N\', \'T\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, transa = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: transa = \'N\', \'T\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= 0 and n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transa = \'{:s}\', lda >= max(1, n).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transa = \'{:s}\', lda >= max(1, m).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transb = \'{:s}\', ldb >= max(1, n).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if transb = \'{:s}\', ldb >= max(1, m).'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def real_tri_matmul_inplace(side, uplo, transa, n, alpha, a, b):
    u"""
    Matrix-matrix product, two real lower or upper triangular matrices.

    For full information please refer to the NAG Library document for
    f01dg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01dgf.html

    Parameters
    ----------
    side : str, length 1
        Specifies whether B is operated on from the left or the right.

        `side` = 'L'

          B is pre-multiplied from the left.

        `side` = 'R'

          B is post-multiplied from the right.

    uplo : str, length 1
        Specifies whether A and B are upper or lower triangular.

        `uplo` = 'U'

          A and B are upper triangular.

        `uplo` = 'L'

          A and B are lower triangular.

    transa : str, length 1
        Specifies whether the operation involves A or A^T.

        `transa` = 'N'

          The operation involves A.

        `transa` = 'T' or 'C'

          The operation involves A^T.

    n : int
        n, the order of the triangular matrices A and B.

    alpha : float
        The scalar alpha.

    a : float, shape (`n`, `n`)
        The n by n triangular matrix A.

    b : float, shape (`n`, `n`), modified in place
        On entry: the n by n triangular matrix B.

        If `alpha` = 0, `b` need not be set.

        On exit: B is overwritten.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `side` = *<value>*.

            Constraint: `side` = 'L' or 'R'.

        (`errno` 2)
            On entry, `uplo` = *<value>*.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` 3)
            On entry, `transa` = *<value>*.

            Constraint: `transa` = 'N', 'T' or 'C'.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_tri_matmul_inplace'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01dgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, transa(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, alpha.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transa.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _transa = utils._EngineCharScalarType(
        transa, {'fun_name': fun_name, 'entity': 'transa'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _alpha = utils._EngineFloat64ScalarType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _side,
        _uplo,
        _transa,
        _n,
        _alpha,
        _a,
        _lda,
        _b,
        _ldb,
        _errbuf,
        _ierr,
        _side._length_,
        _uplo._length_,
        _transa._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, side = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: side = \'L\' or \'R\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uplo = \'U\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, transa = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: transa = \'N\', \'T\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def complex_tri_matmul_inplace(side, uplo, transa, n, alpha, a, b):
    u"""
    Matrix-matrix product, two complex lower or upper triangular
    matrices.

    For full information please refer to the NAG Library document for
    f01du

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01duf.html

    Parameters
    ----------
    side : str, length 1
        Specifies whether B is operated on from the left or the right.

        `side` = 'L'

          B is pre-multiplied from the left.

        `side` = 'R'

          B is post-multiplied from the right.

    uplo : str, length 1
        Specifies whether A and B are upper or lower triangular.

        `uplo` = 'U'

          A and B are upper triangular.

        `uplo` = 'L'

          A and B are lower triangular.

    transa : str, length 1
        Specifies whether the operation involves A, A^T or A^H.

        `transa` = 'N'

          The operation involves A.

        `transa` = 'T'

          The operation involves A^T.

        `transa` = 'C'

          The operation involves A^H.

    n : int
        n, the order of the triangular matrices A and B.

    alpha : complex
        The scalar alpha.

    a : complex, shape (`n`, `n`)
        The n by n triangular matrix A.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the n by n triangular matrix B.

        If `alpha` = 0, `b` need not be set.

        On exit: B is overwritten.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `side` = *<value>*.

            Constraint: `side` = 'L' or 'R'.

        (`errno` 2)
            On entry, `uplo` = *<value>*.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` 3)
            On entry, `transa` = *<value>*.

            Constraint: `transa` = 'N', 'T' or 'C'.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_tri_matmul_inplace'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01duft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, transa(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_COMPLEX128, # Input, alpha.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transa.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _transa = utils._EngineCharScalarType(
        transa, {'fun_name': fun_name, 'entity': 'transa'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _alpha = utils._EngineComplex128ScalarType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _side,
        _uplo,
        _transa,
        _n,
        _alpha,
        _a,
        _lda,
        _b,
        _ldb,
        _errbuf,
        _ierr,
        _side._length_,
        _uplo._length_,
        _transa._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, side = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: side = \'L\' or \'R\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uplo = \'U\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, transa = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: transa = \'N\', \'T\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_matrix_exp(n, a):
    u"""
    Real matrix exponential.

    For full information please refer to the NAG Library document for
    f01ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01ecf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix exponential e^A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            The linear equations to be solved for the Padé approximant
            are singular; it is likely that this function has been
            called incorrectly.

        (`errno` 2)
            The linear equations to be solved are nearly singular and
            the Padé approximant probably has no correct figures; it is
            likely that this function has been called incorrectly.

        (`errno` 4)
            An unexpected internal error has occurred. Please contact
            NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix exponential.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01ecft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved for the Padé approximant are singular;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved are nearly singular and the',
                    'Padé approximant probably has no correct figures;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix exponential.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error has occurred. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_symm_matrix_exp(uplo, n, a):
    u"""
    Real symmetric matrix exponential.

    For full information please refer to the NAG Library document for
    f01ed

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01edf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if no exception or warning is raised, the upper or
        lower triangular part of the n by n matrix exponential, e^A.

    Raises
    ------
    NagValueError
        (`errno` -3)
            An internal error occurred when computing the spectral
            factorization. Please contact NAG.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` = *<value>*.

            Constraint: `uplo` = 'L' or 'U'.

        (`errno` i > 0)
            The computation of the spectral factorization failed to
            converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_symm_matrix_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01edft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 1210:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when computing the spectral factorization.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uplo = \'L\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of the spectral factorization failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def real_symm_matrix_fun(uplo, n, a, f, data=None):
    u"""
    Function of a real symmetric matrix.

    For full information please refer to the NAG Library document for
    f01ef

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01eff.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if no exception or warning is raised, the upper or
        lower triangular part of the n by n matrix function, f(A).

    f : callable f(n, x, fx, data=None)
        The function `f` evaluates f(z_i) at a number of points z_i.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of function values required.

        x : float, ndarray, shape (`n`)
            The n points x_1,x_2,...,x_n at which the function f is to
            be evaluated.

        fx : float, ndarray, shape (`n`), to be modified in place
            On exit: the n function values. this argument[i-1] should
            return the value f(x_i), for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` -3)
            An internal error occurred when computing the spectral
            factorization. Please contact NAG.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` = *<value>*.

            Constraint: `uplo` = 'L' or 'U'.

        (`errno` i > 0)
            The computation of the spectral factorization failed to
            converge.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` -6)
            Termination requested in `f`.
    """

    from ..base import _nag_enum_consts

    excs = []
    fun_name = 'naginterfaces.base.matop.real_symm_matrix_fun'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fx[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, iflag, n, x, fx, _iuser, _ruser):
        """Helper for f."""
        try:
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fh,),
            )
            _fx_fh = utils._EngineFloat64ArrayType.from_pointer(
                fx,
                (_n_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _n_fh,
                    _x_fh,
                    _fx_fh,
                    data,
                )
            else:
                f(
                    _n_fh,
                    _x_fh,
                    _fx_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fx_fh, {'fun_name': cb_f_name, 'returned': 'fx'}, fx,
            )
            utils._unlink_pointer(fx)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01efft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, sorder.
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _sorder = utils._EngineIntScalarType.c_type(_nag_enum_consts.nag_col_major)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _sorder,
        _uplo,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -6:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == -3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when computing the spectral factorization.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uplo = \'L\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of the spectral factorization failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_log(n, a):
    u"""
    Real matrix logarithm.

    For full information please refer to the NAG Library document for
    f01ej

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01ejf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix logarithm, log(A).

    Returns
    -------
    imnorm : float
        If the function has given a reliable answer then `imnorm` = 0.0.
        If this argument differs from 0.0 by more than unit roundoff (as
        returned by ``machine.precision``) then the computed matrix
        logarithm is unreliable.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A is singular so the logarithm cannot be computed.

        (`errno` 2)
            A was found to have eigenvalues on the negative real line.
            The principal logarithm is not defined in this case.
            ``complex_gen_matrix_log`` can be used to find a complex
            non-principal logarithm.

        (`errno` 4)
            An unexpected internal error occurred. Please contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix logarithm.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_log'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01ejft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, imnorm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _imnorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _imnorm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _ierr_fl = 1
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the logarithm cannot be computed.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A was found to have eigenvalues on the negative real line.',
                    'The principal logarithm is not defined in this case.',
                    'complex_gen_matrix_log can be used to find a complex non-principal logarithm.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7:
            _ierr_fl = 3
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix logarithm.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _ierr_fl = 4
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _imnorm_py = utils._EngineFloat64ScalarType.to_py(_imnorm)
        return_data = _imnorm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['imnorm'],
            )(
                _imnorm_py,
            )
    _err_class.manage()
    return return_data

def real_gen_matrix_fun_std(fun, n, a):
    u"""
    Exponential, sine, cosine, sinh or cosh of a real matrix
    (Schur--Parlett algorithm).

    For full information please refer to the NAG Library document for
    f01ek

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01ekf.html

    Parameters
    ----------
    fun : str
        Indicates which matrix function will be computed.

        `fun` = 'EXP'

          The matrix exponential, e^A, will be computed.

        `fun` = 'SIN'

          The matrix sine, sin(A), will be computed.

        `fun` = 'COS'

          The matrix cosine, cos(A), will be computed.

        `fun` = 'SINH'

          The hyperbolic matrix sine, sinh(A), will be computed.

        `fun` = 'COSH'

          The hyperbolic matrix cosine, cosh(A), will be computed.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    Returns
    -------
    imnorm : float
        If A has complex eigenvalues, ``real_gen_matrix_fun_std`` will
        use complex arithmetic to compute the matrix function. The
        imaginary part is discarded at the end of the computation,
        because it will theoretically vanish. this argument contains the
        1-norm of the imaginary part, which should be used to check that
        the routine has given a reliable answer.

        If A has real eigenvalues, ``real_gen_matrix_fun_std`` uses real
        arithmetic and `imnorm` = 0.

    Raises
    ------
    NagValueError
        (`errno` -2)
            Input argument number *<value>* is invalid.

        (`errno` -1)
            Input argument `fun` is invalid.

        (`errno` 1)
            A Taylor series failed to converge.

        (`errno` 2)
            An unexpected internal error occurred when evaluating the
            function at a point. Please contact NAG.

        (`errno` 3)
            There was an error whilst reordering the Schur form of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 4)
            The function was unable to compute the Schur decomposition
            of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 5)
            An unexpected internal error occurred. Please contact NAG.

        (`errno` 6)
            The linear equations to be solved are nearly singular and
            the Padé approximant used to compute the exponential may
            have no correct figures.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_fun_std'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01ekft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ifun.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, imnorm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fun = utils._EngineCharScalarType(
        fun, {'fun_name': fun_name, 'entity': 'fun'}, exp_chrlen=None,
    ).to_ctypes()
    _fun_py = utils._EngineCharScalarType.to_py(_fun)
    if _fun_py.upper() == 'EXP':
        _ifun = utils._EngineIntScalarType.c_type(1)
    elif _fun_py.upper() == 'SIN':
        _ifun = utils._EngineIntScalarType.c_type(2)
    elif _fun_py.upper() == 'COS':
        _ifun = utils._EngineIntScalarType.c_type(3)
    elif _fun_py.upper() == 'SINH':
        _ifun = utils._EngineIntScalarType.c_type(4)
    elif _fun_py.upper() == 'COSH':
        _ifun = utils._EngineIntScalarType.c_type(5)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Input argument fun is invalid.',
            ],
            errcodes=(-1,),
        )
        _err_class.manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _imnorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ifun,
        _n,
        _a,
        _lda,
        _imnorm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A Taylor series failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred when evaluating the function at a',
                    'point. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There was an error whilst reordering the Schur form of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was unable to compute the Schur decomposition of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved are nearly singular and the Padé',
                    'approximant used to compute the exponential may have no correct figures.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _imnorm_py = utils._EngineFloat64ScalarType.to_py(_imnorm)
        return_data = _imnorm_py
    _err_class.manage()
    return return_data

def real_gen_matrix_fun_num(n, a, f, data=None):
    u"""
    Function of a real matrix (using numerical differentiation).

    For full information please refer to the NAG Library document for
    f01el

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01elf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(nz, z, fz, data=None)
        The function `f` evaluates f(z_i) at a number of points z_i.

        Parameters
        ~~~~~~~~~~
        nz : int
            n_z, the number of function values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function values. this argument[i-1] should
            return the value f(z_i), for i = 1,2,...,n_z. If z_i lies on
            the real line, then so must f(z_i).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    imnorm : float
        If A has complex eigenvalues, ``real_gen_matrix_fun_num`` will
        use complex arithmetic to compute f(A). The imaginary part is
        discarded at the end of the computation, because it will
        theoretically vanish. this argument contains the 1-norm of the
        imaginary part, which should be used to check that the routine
        has given a reliable answer.

        If `a` has real eigenvalues, ``real_gen_matrix_fun_num`` uses
        real arithmetic and `imnorm` = 0.

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            A Taylor series failed to converge after 40 terms. Further
            Taylor series coefficients can no longer reliably be
            obtained by numerical differentiation.

        (`errno` 3)
            There was an error whilst reordering the Schur form of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 4)
            The function was unable to compute the Schur decomposition
            of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 5)
            An unexpected internal error occurred. Please contact NAG.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 2)
            Termination requested in `f`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_fun_num'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, norder.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, norder, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01elft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._BYREF_FLOAT64, # Output, imnorm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _imnorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _imnorm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A Taylor series failed to converge after 40 terms.',
                    'Further Taylor series coefficients can no longer reliably be obtained',
                    'by numerical differentiation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There was an error whilst reordering the Schur form of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was unable to compute the Schur decomposition of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _imnorm_py = utils._EngineFloat64ScalarType.to_py(_imnorm)
        return_data = _imnorm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['imnorm'],
            )(
                _imnorm_py,
            )
    _err_class.manage()
    return return_data

def real_gen_matrix_fun_usd(n, a, f, data=None):
    u"""
    Function of a real matrix (using user-supplied derivatives).

    For full information please refer to the NAG Library document for
    f01em

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01emf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(m, nz, z, fz, data=None)
        Given an integer m, the function `f` evaluates f^(m)(z_i) at a
        number of points z_i.

        Parameters
        ~~~~~~~~~~
        m : int
            The order, m, of the derivative required.

            If `m` = 0, f(z_i) should be returned.

            For `m` > 0, f^(m)(z_i) should be returned.

        nz : int
            n_z, the number of function or derivative values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function or derivative values. this
            argument[i-1] should return the value f^(m)(z_i), for i =
            1,2,...,n_z. If z_i lies on the real line, then so must
            f^(m)(z_i).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    imnorm : float
        If A has complex eigenvalues, ``real_gen_matrix_fun_usd`` will
        use complex arithmetic to compute f(A). The imaginary part is
        discarded at the end of the computation, because it will
        theoretically vanish. this argument contains the 1-norm of the
        imaginary part, which should be used to check that the function
        has given a reliable answer.

        If A has real eigenvalues, ``real_gen_matrix_fun_usd`` uses real
        arithmetic and `imnorm` = 0.

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            A Taylor series failed to converge.

        (`errno` 3)
            There was an error whilst reordering the Schur form of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 4)
            The routine was unable to compute the Schur decomposition of
            A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 5)
            An unexpected internal error occurred. Please contact NAG.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 2)
            Termination requested in `f`.

    See Also
    --------
    :meth:`naginterfaces.library.examples.matop.real_gen_matrix_fun_usd_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_fun_usd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, m, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _m_fh = utils._EngineIntScalarType.to_py(m.contents)
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01emft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._BYREF_FLOAT64, # Output, imnorm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _imnorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _imnorm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A Taylor series failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There was an error whilst reordering the Schur form of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The routine was unable to compute the Schur decomposition of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _imnorm_py = utils._EngineFloat64ScalarType.to_py(_imnorm)
        return_data = _imnorm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['imnorm'],
            )(
                _imnorm_py,
            )
    _err_class.manage()
    return return_data

def real_gen_matrix_sqrt(n, a):
    u"""
    Real matrix square root.

    For full information please refer to the NAG Library document for
    f01en

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01enf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: contains, if no exception or warning is raised, the n
        by n principal matrix square root, A^(1/2). Alternatively, if
        `errno` = 1, contains an n by n non-principal square root of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A has a defective vanishing eigenvalue. The square root
            cannot be found in this case.

        (`errno` 3)
            A has a negative real eigenvalue. The principal square root
            is not defined. ``complex_gen_matrix_sqrt`` can be used to
            return a complex, non-principal square root.

        (`errno` 4)
            An internal error occurred. It is likely that the function
            was called incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has a semisimple vanishing eigenvalue. A non-principal
            square root is returned.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_sqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01enft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 9999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a semisimple vanishing eigenvalue.',
                    'A non-principal square root is returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 9999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a defective vanishing eigenvalue.',
                    'The square root cannot be found in this case.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 9999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a negative real eigenvalue. The principal square root is not defined.',
                    'complex_gen_matrix_sqrt can be used to return a complex, non-principal square root.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 9999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred. It is likely that the function was called',
                    'incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_tri_matrix_sqrt(n, a):
    u"""
    Real upper quasi-triangular matrix square root.

    For full information please refer to the NAG Library document for
    f01ep

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01epf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n upper quasi-triangular matrix A.

        On exit: the n by n principal matrix square root A^(1/2).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A has negative or vanishing eigenvalues. The principal
            square root is not defined in this case.
            ``real_gen_matrix_sqrt`` or ``complex_gen_matrix_sqrt`` may
            be able to provide further information.

        (`errno` 2)
            An internal error occurred. It is likely that the function
            was called incorrectly.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_tri_matrix_sqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01epft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 9001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has negative or vanishing eigenvalues.',
                    'The principal square root is not defined in this case.',
                    'real_gen_matrix_sqrt or complex_gen_matrix_sqrt may be able to provide further information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 9999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has negative or vanishing eigenvalues.',
                    'The principal square root is not defined in this case.',
                    'real_gen_matrix_sqrt or complex_gen_matrix_sqrt may be able to provide further information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred. It is likely that the function was called',
                    'incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_matrix_pow(n, a, p):
    u"""
    General power of a real matrix.

    For full information please refer to the NAG Library document for
    f01eq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01eqf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix pth power, A^p.

    p : float
        The required power of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A has eigenvalues on the negative real line. The principal
            pth power is not defined. ``complex_gen_matrix_pow`` can be
            used to find a complex, non-principal pth power.

        (`errno` 2)
            A is singular so the pth power cannot be computed.

        (`errno` 4)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            A^p has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_pow'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01eqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line.',
                    'The principal pth power is not defined.',
                    'complex_gen_matrix_pow can be used to find a complex, non-principal pth power.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the pth power cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A^p has been computed using',
                    'an IEEE double precision Padé approximant, although the',
                    'arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_exp(n, a):
    u"""
    Complex matrix exponential.

    For full information please refer to the NAG Library document for
    f01fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fcf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: with no exception or warning is raised, the n by n
        matrix exponential e^A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            The linear equations to be solved for the Padé approximant
            are singular; it is likely that this function has been
            called incorrectly.

        (`errno` 2)
            The linear equations to be solved are nearly singular and
            the Padé approximant probably has no correct figures; it is
            likely that this function has been called incorrectly.

        (`errno` 4)
            An unexpected internal error has occurred. Please contact
            NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix exponential.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved for the Padé approximant are singular;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved are nearly singular and the',
                    'Padé approximant probably has no correct figures;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix exponential.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error has occurred. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_herm_matrix_exp(uplo, n, a):
    u"""
    Complex Hermitian matrix exponential.

    For full information please refer to the NAG Library document for
    f01fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fdf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if no exception or warning is raised, the upper or
        lower triangular part of the n by n matrix exponential, e^A.

    Raises
    ------
    NagValueError
        (`errno` -3)
            An internal error occurred when computing the spectral
            factorization. Please contact NAG.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` = *<value>*.

            Constraint: `uplo` = 'L' or 'U'.

        (`errno` i > 0)
            The computation of the spectral factorization failed to
            converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_herm_matrix_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when computing the spectral factorization.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uplo = \'L\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of the spectral factorization failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def complex_herm_matrix_fun(uplo, n, a, f, data=None):
    u"""
    Function of a complex Hermitian matrix.

    For full information please refer to the NAG Library document for
    f01ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fff.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if no exception or warning is raised, the upper or
        lower triangular part of the n by n matrix function, f(A).

    f : callable f(n, x, fx, data=None)
        The function `f` evaluates f(z_i) at a number of points z_i.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of function values required.

        x : float, ndarray, shape (`n`)
            The n points x_1,x_2,...,x_n at which the function f is to
            be evaluated.

        fx : float, ndarray, shape (`n`), to be modified in place
            On exit: the n function values. this argument[i-1] should
            return the value f(x_i), for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` -3)
            An internal error occurred when computing the spectral
            factorization. Please contact NAG.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` = *<value>*.

            Constraint: `uplo` = 'L' or 'U'.

        (`errno` i > 0)
            The computation of the spectral factorization failed to
            converge.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` -6)
            Termination requested in `f`.
    """

    from ..base import _nag_enum_consts

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_herm_matrix_fun'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fx[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, iflag, n, x, fx, _iuser, _ruser):
        """Helper for f."""
        try:
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fh,),
            )
            _fx_fh = utils._EngineFloat64ArrayType.from_pointer(
                fx,
                (_n_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _n_fh,
                    _x_fh,
                    _fx_fh,
                    data,
                )
            else:
                f(
                    _n_fh,
                    _x_fh,
                    _fx_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fx_fh, {'fun_name': cb_f_name, 'returned': 'fx'}, fx,
            )
            utils._unlink_pointer(fx)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01ffft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, sorder.
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _sorder = utils._EngineIntScalarType.c_type(_nag_enum_consts.nag_col_major)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _sorder,
        _uplo,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -6:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == -3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when computing the spectral factorization.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uplo = \'L\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of the spectral factorization failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_log(n, a):
    u"""
    Complex matrix logarithm.

    For full information please refer to the NAG Library document for
    f01fj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fjf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix logarithm, log(A), unless
        `errno` = 2, in which case a non-principal logarithm is
        returned.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A is singular so the logarithm cannot be computed.

        (`errno` 4)
            An unexpected internal error has occurred. Please contact
            NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            A was found to have eigenvalues on the negative real line.
            The principal logarithm is not defined in this case, so a
            non-principal logarithm was returned.

        (`errno` 3)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix logarithm.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_log'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01fjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the logarithm cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A was found to have eigenvalues on the negative real line.',
                    'The principal logarithm is not defined in this case,',
                    'so a non-principal logarithm was returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix logarithm.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error has occurred. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_fun_std(fun, n, a):
    u"""
    Exponential, sine, cosine, sinh or cosh of a complex matrix
    (Schur--Parlett algorithm).

    For full information please refer to the NAG Library document for
    f01fk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fkf.html

    Parameters
    ----------
    fun : str
        Indicates which matrix function will be computed.

        `fun` = 'EXP'

          The matrix exponential, e^A, will be computed.

        `fun` = 'SIN'

          The matrix sine, sin(A), will be computed.

        `fun` = 'COS'

          The matrix cosine, cos(A), will be computed.

        `fun` = 'SINH'

          The hyperbolic matrix sine, sinh(A), will be computed.

        `fun` = 'COSH'

          The hyperbolic matrix cosine, cosh(A), will be computed.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    Raises
    ------
    NagValueError
        (`errno` -2)
            Input argument number *<value>* is invalid.

        (`errno` -1)
            Input argument `fun` is invalid.

        (`errno` 1)
            A Taylor series failed to converge.

        (`errno` 2)
            An unexpected internal error occurred when evaluating the
            function at a point. Please contact NAG.

        (`errno` 3)
            There was an error whilst reordering the Schur form of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 4)
            The function was unable to compute the Schur decomposition
            of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 5)
            An unexpected internal error occurred. Please contact NAG.

        (`errno` 6)
            The linear equations to be solved are nearly singular and
            the Padé approximant used to compute the exponential may
            have no correct figures.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_fun_std'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01fkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ifun.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fun = utils._EngineCharScalarType(
        fun, {'fun_name': fun_name, 'entity': 'fun'}, exp_chrlen=None,
    ).to_ctypes()
    _fun_py = utils._EngineCharScalarType.to_py(_fun)
    if _fun_py.upper() == 'EXP':
        _ifun = utils._EngineIntScalarType.c_type(1)
    elif _fun_py.upper() == 'SIN':
        _ifun = utils._EngineIntScalarType.c_type(2)
    elif _fun_py.upper() == 'COS':
        _ifun = utils._EngineIntScalarType.c_type(3)
    elif _fun_py.upper() == 'SINH':
        _ifun = utils._EngineIntScalarType.c_type(4)
    elif _fun_py.upper() == 'COSH':
        _ifun = utils._EngineIntScalarType.c_type(5)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Input argument fun is invalid.',
            ],
            errcodes=(-1,),
        )
        _err_class.manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ifun,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A Taylor series failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred when evaluating the function at a',
                    'point. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There was an error whilst reordering the Schur form of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was unable to compute the Schur decomposition of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved are nearly singular and the Padé',
                    'approximant used to compute the exponential may have no correct figures.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def complex_gen_matrix_fun_num(n, a, f, data=None):
    u"""
    Function of a complex matrix (using numerical differentiation).

    For full information please refer to the NAG Library document for
    f01fl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01flf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(nz, z, fz, data=None)
        The function `f` evaluates f(z_i) at a number of points z_i.

        Parameters
        ~~~~~~~~~~
        nz : int
            n_z, the number of function values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function values. this argument[i-1] should
            return the value f(z_i), for i = 1,2,...,n_z.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            A Taylor series failed to converge after 40 terms. Further
            Taylor series coefficients can no longer reliably be
            obtained by numerical differentiation.

        (`errno` 3)
            There was an error whilst reordering the Schur form of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 4)
            The function was unable to compute the Schur decomposition
            of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 5)
            An unexpected internal error occurred. Please contact NAG.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 2)
            Termination requested in `f`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_fun_num'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, norder.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, norder, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01flft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A Taylor series failed to converge after 40 terms.',
                    'Further Taylor series coefficients can no longer reliably be obtained',
                    'by numerical differentiation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There was an error whilst reordering the Schur form of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was unable to compute the Schur decomposition of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_fun_usd(n, a, f, data=None):
    u"""
    Function of a complex matrix (using user-supplied derivatives).

    For full information please refer to the NAG Library document for
    f01fm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fmf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(m, nz, z, fz, data=None)
        Given an integer m, the function `f` evaluates f^(m)(z_i) at a
        number of points z_i.

        Parameters
        ~~~~~~~~~~
        m : int
            The order, m, of the derivative required.

            If `m` = 0, f(z_i) should be returned.

            For `m` > 0, f^(m)(z_i) should be returned.

        nz : int
            n_z, the number of function or derivative values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function or derivative values. this
            argument[i-1] should return the value f^(m)(z_i), for i =
            1,2,...,n_z.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            A Taylor series failed to converge.

        (`errno` 3)
            There was an error whilst reordering the Schur form of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 4)
            The function was unable to compute the Schur decomposition
            of A.

            Note: this failure should not occur and suggests that the
            function has been called incorrectly.

        (`errno` 5)
            An unexpected internal error occurred. Please contact NAG.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 2)
            Termination requested in `f`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_fun_usd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, m, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _m_fh = utils._EngineIntScalarType.to_py(m.contents)
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01fmft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A Taylor series failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There was an error whilst reordering the Schur form of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was unable to compute the Schur decomposition of A.',
                    'Note: this failure should not occur and suggests that the function has been',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_sqrt(n, a):
    u"""
    Complex matrix square root.

    For full information please refer to the NAG Library document for
    f01fn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fnf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: contains, if no exception or warning is raised, the n
        by n principal matrix square root, A^(1/2). Alternatively, if
        `errno` = 1, contains an n by n non-principal square root of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A has a defective vanishing eigenvalue. The square root
            cannot be found in this case.

        (`errno` 3)
            An internal error occurred. It is likely that the function
            was called incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has a negative or semisimple vanishing eigenvalue. A
            non-principal square root is returned.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_sqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01fnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a negative or semisimple vanishing eigenvalue.',
                    'A non-principal square root is returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 9999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a defective vanishing eigenvalue.',
                    'The square root cannot be found in this case.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred. It is likely that the function was called',
                    'incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_tri_matrix_sqrt(n, a):
    u"""
    Complex upper triangular matrix square root.

    For full information please refer to the NAG Library document for
    f01fp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fpf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix A.

        On exit: contains, if no exception or warning is raised, the n
        by n principal matrix square root, A^(1/2). Alternatively, if
        `errno` = 1, contains an n by n non-principal square root of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A has a defective vanishing eigenvalue. The square root
            cannot be found in this case.

        (`errno` 3)
            An internal error occurred. It is likely that the function
            was called incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has negative or semisimple, vanishing eigenvalues. The
            principal square root is not defined in this case; a
            non-principal square root is returned.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_tri_matrix_sqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01fpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has negative or semisimple, vanishing eigenvalues.',
                    'The principal square root is not defined in this case;',
                    'a non-principal square root is returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 9999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a defective vanishing eigenvalue.',
                    'The square root cannot be found in this case.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred. It is likely that the function was called',
                    'incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_pow(n, a, p):
    u"""
    General power of a complex matrix.

    For full information please refer to the NAG Library document for
    f01fq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01fqf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: if no exception or warning is raised, the n by n matrix
        pth power, A^p. Alternatively, if `errno` = 1, contains an n by
        n non-principal power of A.

    p : float
        The required power of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A is singular so the pth power cannot be computed.

        (`errno` 4)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has eigenvalues on the negative real line. The principal
            pth power is not defined so a non-principal power is
            returned.

        (`errno` 3)
            A^p has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_pow'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01fqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line.',
                    'The principal pth power is not defined so a',
                    'non-principal power is returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the pth power cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A^p has been computed using',
                    'an IEEE double precision Padé approximant, although the',
                    'arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_actexp(n, m, a, b, t):
    u"""
    Action of a real matrix exponential on a real matrix.

    For full information please refer to the NAG Library document for
    f01ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01gaf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    m : int
        m, the number of columns of the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: A is overwritten during the computation.

    b : float, shape (`n`, `m`), modified in place
        On entry: the n by m matrix B.

        On exit: the n by m matrix e^(tA)B.

    t : float
        The scalar t.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            e^(tA)B has been computed using an IEEE double precision
            Taylor series, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_actexp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, m].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _m,
        _a,
        _lda,
        _b,
        _ldb,
        _t,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT.format(_m_py),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1101:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'e^(tA)B has been computed using an IEEE double precision Taylor series,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_actexp_rcomm(irevcm, n, m, b, t, tr, b2, x, y, p, r, z, comm):
    u"""
    Action of a real matrix exponential on a real matrix (reverse
    communication).

    For full information please refer to the NAG Library document for
    f01gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01gbf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must be set to 0.

    n : int
        n, the order of the matrix A.

    m : int
        The number of columns of the matrix B.

    b : float, shape (`n`, `m`), modified in place
        On initial entry: the n by m matrix B.

        On intermediate exit: if `irevcm` = 1, contains the n by m
        matrix B.

        On intermediate entry: must not be changed.

        On final exit: the n by m matrix e^(tA)B.

    t : float
        The scalar t.

    tr : float
        The trace of A. If this is not available then any number can be
        supplied (0.0 is a reasonable default); however, in the trivial
        case, n = 1, the result e^(`tr`t)B is immediately returned in
        the first row of B. See [Further Comments].

    b2 : float, shape (`n`, `m`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 1, must contain AB.

        On final exit: the array is undefined.

    x : float, shape (`n`, 2), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 2, contains the current n by
        2 matrix X.

        On intermediate entry: if `irevcm` = 3, must contain A^TY.

        On final exit: the array is undefined.

    y : float, shape (`n`, 2), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 3, contains the current n by
        2 matrix Y.

        On intermediate entry: if `irevcm` = 2, must contain AX.

        On final exit: the array is undefined.

    p : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 4, must contain Az.

        On final exit: the array is undefined.

    r : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 5, must contain A^Tz.

        On final exit: the array is undefined.

    z : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 4 or 5, contains the vector
        z.

        On intermediate entry: must not be changed.

        On final exit: the array is undefined.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 1, 2, 3, 4 or 5. The calling
        program must:

        - if `irevcm` = 1: evaluate B_2 = AB, where B_2 is an n by m
          matrix, and store the result in `b2`;

          if `irevcm` = 2: evaluate Y = AX, where X and Y are n by 2
          matrices, and store the result in `y`;

          if `irevcm` = 3: evaluate X = A^TY and store the result in
          `x`;

          if `irevcm` = 4: evaluate p = Az and store the result in `p`;

          if `irevcm` = 5: evaluate r = A^Tz and store the result in
          `r`.

        - call ``real_gen_matrix_actexp_rcomm`` again with all other
          parameters unchanged.

        On final exit: `irevcm` = 0.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On initial entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -2)
            On initial entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On intermediate re-entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3, 4 or 5.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            e^(tA)B has been computed using an IEEE double precision
            Taylor series, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_actexp_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, m].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, tr.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b2[ldb2, m].
        utils._BYREF_INT, # Input, ldb2.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, 2].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[ldy, 2].
        utils._BYREF_INT, # Input, ldy.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, p[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[n*m+3*n+12].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[2*n+40].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (b2, 'b2'), (x, 'x'), (y, 'y')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tr = utils._EngineFloat64ScalarType(
        tr, {'fun_name': fun_name, 'entity': 'tr'},
    ).to_ctypes()
    _ldb2_py = utils._EngineFloat64ArrayType.get_nd(b2, {'fun_name': fun_name, 'entity': 'b2'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb2 = utils._EngineIntScalarType(
        _ldb2_py, {'fun_name': fun_name, 'entity': '_ldb2'},
    ).to_ctypes()
    _b2_cls = utils._EngineFloat64ArrayType(
        b2, {'fun_name': fun_name, 'entity': 'b2'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _b2 = _b2_cls.to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineFloat64ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_n_py,),
    )
    _p = _p_cls.to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
    )
    _r = _r_cls.to_ctypes()
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py,),
    )
    _z = _z_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('comm', 'icomm'),
    )
    if _comm_needs_init:
        _comm = utils._EngineFloat64ArrayType.empty_ctypes(_n_py*_m_py+3*_n_py+12)
        comm['comm'] = _comm
    else:
        _comm = comm['comm']
    if _comm_needs_init:
        _icomm = utils._EngineIntArrayType.empty_ctypes(2*_n_py+40)
        comm['icomm'] = _icomm
    else:
        _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _n,
        _m,
        _b,
        _ldb,
        _t,
        _tr,
        _b2,
        _ldb2,
        _x,
        _ldx,
        _y,
        _ldy,
        _p,
        _r,
        _z,
        _comm,
        _icomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == -3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1234:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate re-entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1235:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1101:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'e^(tA)B has been computed using an IEEE double precision Taylor series,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        _b2_cls.refresh_py(_b2)
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _p_cls.refresh_py(_p)
        _r_cls.refresh_py(_r)
        _z_cls.refresh_py(_z)
        return_data = _irevcm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['irevcm'],
            )(
                _irevcm_py,
            )
    _err_class.manage()
    return return_data

def complex_gen_matrix_actexp(n, m, a, b, t):
    u"""
    Action of a complex matrix exponential on a complex matrix.

    For full information please refer to the NAG Library document for
    f01ha

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01haf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    m : int
        m, the number of columns of the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: A is overwritten during the computation.

    b : complex, shape (`n`, `m`), modified in place
        On entry: the n by m matrix B.

        On exit: the n by m matrix e^(tA)B.

    t : complex
        The scalar t.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            e^(tA)B has been computed using an IEEE double precision
            Taylor series, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_actexp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01haft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, m].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_COMPLEX128, # Input, t.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _t = utils._EngineComplex128ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _m,
        _a,
        _lda,
        _b,
        _ldb,
        _t,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT.format(_m_py),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1101:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'e^(tA)B has been computed using an IEEE double precision Taylor series,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_actexp_rcomm(irevcm, n, m, b, t, tr, b2, x, y, p, r, z, comm):
    u"""
    Action of a complex matrix exponential on a complex matrix (reverse
    communication).

    For full information please refer to the NAG Library document for
    f01hb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01hbf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must be set to 0.

    n : int
        n, the order of the matrix A.

    m : int
        The number of columns of the matrix B.

    b : complex, shape (`n`, `m`), modified in place
        On initial entry: the n by m matrix B.

        On intermediate exit: if `irevcm` = 1, contains the n by m
        matrix B.

        On intermediate entry: must not be changed.

        On final exit: the n by m matrix e^(tA)B.

    t : complex
        The scalar t.

    tr : complex
        The trace of A. If this is not available then any number can be
        supplied (0.0 is a reasonable default); however, in the trivial
        case, n = 1, the result e^(`tr`t)B is immediately returned in
        the first row of B. See [Further Comments].

    b2 : complex, shape (`n`, `m`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 1, must contain AB.

        On final exit: the array is undefined.

    x : complex, shape (`n`, 2), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 2, contains the current n by
        2 matrix X.

        On intermediate entry: if `irevcm` = 3, must contain A^HY.

        On final exit: the array is undefined.

    y : complex, shape (`n`, 2), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 3, contains the current n by
        2 matrix Y.

        On intermediate entry: if `irevcm` = 2, must contain AX.

        On final exit: the array is undefined.

    p : complex, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 4, must contain Az.

        On final exit: the array is undefined.

    r : complex, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 5, must contain A^Hz.

        On final exit: the array is undefined.

    z : complex, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 4 or 5, contains the vector
        z.

        On intermediate entry: must not be changed.

        On final exit: the array is undefined.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 1, 2, 3, 4 or 5. The calling
        program must:

        - if `irevcm` = 1: evaluate B_2 = AB, where B_2 is an n by m
          matrix, and store the result in `b2`;

          if `irevcm` = 2: evaluate Y = AX, where X and Y are n by 2
          matrices, and store the result in `y`;

          if `irevcm` = 3: evaluate X = A^HY and store the result in
          `x`;

          if `irevcm` = 4: evaluate p = Az and store the result in `p`;

          if `irevcm` = 5: evaluate r = A^Hz and store the result in
          `r`.

        - call ``complex_gen_matrix_actexp_rcomm`` again with all other
          parameters unchanged.

        On final exit: `irevcm` = 0.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On initial entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -2)
            On initial entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On intermediate re-entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3, 4 or 5.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            e^(tA)B has been computed using an IEEE double precision
            Taylor series, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_actexp_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01hbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, m].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_COMPLEX128, # Input, t.
        utils._BYREF_COMPLEX128, # Input, tr.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b2[ldb2, m].
        utils._BYREF_INT, # Input, ldb2.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, 2].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, 2].
        utils._BYREF_INT, # Input, ldy.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, p[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, r[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ccomm[n*(m+2)].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[3*n+14].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[2*n+40].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (b2, 'b2'), (x, 'x'), (y, 'y')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldb_py = utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _t = utils._EngineComplex128ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tr = utils._EngineComplex128ScalarType(
        tr, {'fun_name': fun_name, 'entity': 'tr'},
    ).to_ctypes()
    _ldb2_py = utils._EngineComplex128ArrayType.get_nd(b2, {'fun_name': fun_name, 'entity': 'b2'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb2 = utils._EngineIntScalarType(
        _ldb2_py, {'fun_name': fun_name, 'entity': '_ldb2'},
    ).to_ctypes()
    _b2_cls = utils._EngineComplex128ArrayType(
        b2, {'fun_name': fun_name, 'entity': 'b2'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _b2 = _b2_cls.to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _p_cls = utils._EngineComplex128ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_n_py,),
    )
    _p = _p_cls.to_ctypes()
    _r_cls = utils._EngineComplex128ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
    )
    _r = _r_cls.to_ctypes()
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py,),
    )
    _z = _z_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('ccomm', 'comm', 'icomm'),
    )
    if _comm_needs_init:
        _ccomm = utils._EngineComplex128ArrayType.empty_ctypes(_n_py*(_m_py+2))
        comm['ccomm'] = _ccomm
    else:
        _ccomm = comm['ccomm']
    if _comm_needs_init:
        _comm = utils._EngineFloat64ArrayType.empty_ctypes(3*_n_py+14)
        comm['comm'] = _comm
    else:
        _comm = comm['comm']
    if _comm_needs_init:
        _icomm = utils._EngineIntArrayType.empty_ctypes(2*_n_py+40)
        comm['icomm'] = _icomm
    else:
        _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _n,
        _m,
        _b,
        _ldb,
        _t,
        _tr,
        _b2,
        _ldb2,
        _x,
        _ldx,
        _y,
        _ldy,
        _p,
        _r,
        _z,
        _ccomm,
        _comm,
        _icomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == -3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1234:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate re-entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1235:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1101:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'e^(tA)B has been computed using an IEEE double precision Taylor series,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        _b2_cls.refresh_py(_b2)
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _p_cls.refresh_py(_p)
        _r_cls.refresh_py(_r)
        _z_cls.refresh_py(_z)
        return_data = _irevcm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['irevcm'],
            )(
                _irevcm_py,
            )
    _err_class.manage()
    return return_data

def real_gen_matrix_cond_std(fun, n, a):
    u"""
    Condition number for the exponential, logarithm, sine, cosine, sinh
    or cosh of a real matrix.

    For full information please refer to the NAG Library document for
    f01ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jaf.html

    Parameters
    ----------
    fun : str
        Indicates which matrix function will be used.

        `fun` = 'EXP'

          The matrix exponential, e^A, will be used.

        `fun` = 'SIN'

          The matrix sine, sin(A), will be used.

        `fun` = 'COS'

          The matrix cosine, cos(A), will be used.

        `fun` = 'SINH'

          The hyperbolic matrix sine, sinh(A), will be used.

        `fun` = 'COSH'

          The hyperbolic matrix cosine, cosh(A), will be used.

        `fun` = 'LOG'

          The matrix logarithm, log(A), will be used.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    Returns
    -------
    conda : float
        An estimate of the absolute condition number of f at A.

    norma : float
        The 1-norm of A.

    normfa : float
        The 1-norm of f(A).

    Raises
    ------
    NagValueError
        (`errno` -2)
            Input argument number *<value>* is invalid.

        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            An internal error occurred when evaluating the matrix
            function f(A). Please contact NAG.

        (`errno` 2)
            An internal error occurred when estimating the norm of the
            Fréchet derivative of f at A. Please contact NAG.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_cond_std'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ifun.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, conda.
        utils._BYREF_FLOAT64, # Output, norma.
        utils._BYREF_FLOAT64, # Output, normfa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fun = utils._EngineCharScalarType(
        fun, {'fun_name': fun_name, 'entity': 'fun'}, exp_chrlen=None,
    ).to_ctypes()
    _fun_py = utils._EngineCharScalarType.to_py(_fun)
    if _fun_py.upper() == 'EXP':
        _ifun = utils._EngineIntScalarType.c_type(1)
    elif _fun_py.upper() == 'SIN':
        _ifun = utils._EngineIntScalarType.c_type(2)
    elif _fun_py.upper() == 'COS':
        _ifun = utils._EngineIntScalarType.c_type(3)
    elif _fun_py.upper() == 'SINH':
        _ifun = utils._EngineIntScalarType.c_type(4)
    elif _fun_py.upper() == 'COSH':
        _ifun = utils._EngineIntScalarType.c_type(5)
    elif _fun_py.upper() == 'LOG':
        _ifun = utils._EngineIntScalarType.c_type(6)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Invalid value for fun.',
            ],
            errcodes=(-1,),
        )
        _err_class.manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _conda = utils._EngineFloat64ScalarType.empty_ctypes()
    _norma = utils._EngineFloat64ScalarType.empty_ctypes()
    _normfa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ifun,
        _n,
        _a,
        _lda,
        _conda,
        _norma,
        _normfa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when evaluating the matrix function',
                    'f(A). Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when estimating the norm of the Fréchet',
                    'derivative of f at A. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _conda_py = utils._EngineFloat64ScalarType.to_py(_conda)
        _norma_py = utils._EngineFloat64ScalarType.to_py(_norma)
        _normfa_py = utils._EngineFloat64ScalarType.to_py(_normfa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['conda', 'norma', 'normfa'],
        )(
            _conda_py,
            _norma_py,
            _normfa_py,
        )
    _err_class.manage()
    return return_data

def real_gen_matrix_cond_num(n, a, f, data=None):
    u"""
    Condition number for a function of a real matrix (using numerical
    differentiation).

    For full information please refer to the NAG Library document for
    f01jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jbf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(nz, z, fz, data=None)
        The function `f` evaluates f(z_i) at a number of points z_i.

        Parameters
        ~~~~~~~~~~
        nz : int
            n_z, the number of function values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function values. this argument[i-1] should
            return the value f(z_i), for i = 1,2,...,n_z. If z_i lies on
            the real line, then so must f(z_i).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    conda : float
        An estimate of the absolute condition number of f at A.

    norma : float
        The 1-norm of A.

    normfa : float
        The 1-norm of f(A).

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            An internal error occurred when estimating the norm of the
            Fréchet derivative of f at A. Please contact NAG.

        (`errno` 2)
            An internal error occurred when evaluating the matrix
            function f(A). You can investigate further by calling
            ``real_gen_matrix_fun_num`` with the matrix A and the
            function f.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 3)
            Termination requested in `f`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_cond_num'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, norder.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, norder, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01jbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._BYREF_FLOAT64, # Output, conda.
        utils._BYREF_FLOAT64, # Output, norma.
        utils._BYREF_FLOAT64, # Output, normfa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _conda = utils._EngineFloat64ScalarType.empty_ctypes()
    _norma = utils._EngineFloat64ScalarType.empty_ctypes()
    _normfa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _conda,
        _norma,
        _normfa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when estimating the norm of the Fréchet',
                    'derivative of f at A. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when evaluating the matrix function f(A).',
                    'You can investigate further by calling real_gen_matrix_fun_num with the matrix A and',
                    'the function f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _conda_py = utils._EngineFloat64ScalarType.to_py(_conda)
        _norma_py = utils._EngineFloat64ScalarType.to_py(_norma)
        _normfa_py = utils._EngineFloat64ScalarType.to_py(_normfa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['conda', 'norma', 'normfa'],
        )(
            _conda_py,
            _norma_py,
            _normfa_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_cond_usd(n, a, f, data=None):
    u"""
    Condition number for a function of a real matrix (using
    user-supplied derivatives).

    For full information please refer to the NAG Library document for
    f01jc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jcf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(m, nz, z, fz, data=None)
        Given an integer m, the function `f` evaluates f^(m)(z_i) at a
        number of points z_i.

        Parameters
        ~~~~~~~~~~
        m : int
            The order, m, of the derivative required.

            If `m` = 0, f(z_i) should be returned.

            For `m` > 0, f^(m)(z_i) should be returned.

        nz : int
            n_z, the number of function or derivative values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function or derivative values. this
            argument[i-1] should return the value f^(m)(z_i), for i =
            1,2,...,n_z. If z_i lies on the real line, then so must
            f^(m)(z_i).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    conda : float
        An estimate of the absolute condition number of f at A.

    norma : float
        The 1-norm of A.

    normfa : float
        The 1-norm of f(A).

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            An internal error occurred when estimating the norm of the
            Fréchet derivative of f at A. Please contact NAG.

        (`errno` 2)
            An internal error occurred when evaluating the matrix
            function f(A). You can investigate further by calling
            ``real_gen_matrix_fun_usd`` with the matrix A and the
            function f.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 3)
            Termination requested in `f`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_cond_usd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, m, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _m_fh = utils._EngineIntScalarType.to_py(m.contents)
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01jcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._BYREF_FLOAT64, # Output, conda.
        utils._BYREF_FLOAT64, # Output, norma.
        utils._BYREF_FLOAT64, # Output, normfa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _conda = utils._EngineFloat64ScalarType.empty_ctypes()
    _norma = utils._EngineFloat64ScalarType.empty_ctypes()
    _normfa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _conda,
        _norma,
        _normfa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when estimating the norm of the Fréchet',
                    'derivative of f at A. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when evaluating the matrix function f(A).',
                    'You can investigate further by calling real_gen_matrix_fun_usd with the matrix A and',
                    'the function f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _conda_py = utils._EngineFloat64ScalarType.to_py(_conda)
        _norma_py = utils._EngineFloat64ScalarType.to_py(_norma)
        _normfa_py = utils._EngineFloat64ScalarType.to_py(_normfa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['conda', 'norma', 'normfa'],
        )(
            _conda_py,
            _norma_py,
            _normfa_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_cond_sqrt(n, a):
    u"""
    Condition number for square root of real matrix.

    For full information please refer to the NAG Library document for
    f01jd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jdf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: contains, if no exception or warning is raised, the n
        by n principal matrix square root, A^(1/2). Alternatively, if
        `errno` = 1, contains an n by n non-principal square root of A.

    Returns
    -------
    alpha : float
        An estimate of the stability of the relative residual for the
        computed principal (if no exception or warning is raised) or
        non-principal (if `errno` = 1) matrix square root, alpha.

    condsa : float
        An estimate of the relative condition number, in the Frobenius
        norm, of the principal (if no exception or warning is raised) or
        non-principal (if `errno` = 1) matrix square root at A,
        kappa_A^(1/2).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A has a defective vanishing eigenvalue. The square root and
            condition number cannot be found in this case.

        (`errno` 3)
            A has a negative real eigenvalue. The principal square root
            is not defined. ``complex_gen_matrix_cond_sqrt`` can be used
            to return a complex, non-principal square root.

        (`errno` 4)
            An error occurred when computing the matrix square root.
            Consequently, `alpha` and `condsa` could not be computed. It
            is likely that the function was called incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has a semisimple vanishing eigenvalue. A non-principal
            square root was returned.

        (`errno` 5)
            An error occurred when computing the condition number. The
            matrix square root was still returned but you should use
            ``real_gen_matrix_sqrt`` to check if it is the principal
            matrix square root.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_cond_sqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, alpha.
        utils._BYREF_FLOAT64, # Output, condsa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _alpha = utils._EngineFloat64ScalarType.empty_ctypes()
    _condsa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _alpha,
        _condsa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a semisimple vanishing eigenvalue. A non-principal square root',
                    'was returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a defective vanishing eigenvalue. The square root and condition',
                    'number cannot be found in this case.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a negative real eigenvalue. The principal square root is not defined.',
                    'complex_gen_matrix_cond_sqrt can be used to return a complex, non-principal square root.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error occurred when computing the matrix square root. Consequently, alpha',
                    'and condsa could not be computed. It is likely that the function was',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error occurred when computing the condition number. The matrix square',
                    'root was still returned but you should use real_gen_matrix_sqrt to check if it is the',
                    'principal matrix square root.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _alpha_py = utils._EngineFloat64ScalarType.to_py(_alpha)
        _condsa_py = utils._EngineFloat64ScalarType.to_py(_condsa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['alpha', 'condsa'],
        )(
            _alpha_py,
            _condsa_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_cond_pow(n, a, p):
    u"""
    Condition number for real matrix power.

    For full information please refer to the NAG Library document for
    f01je

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jef.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix pth power, A^p.

    p : float
        The required power of A.

    Returns
    -------
    condpa : float
        If the function exits successfully or `errno` = 3, an estimate
        of the relative condition number of the matrix pth power,
        kappa_A^p. Alternatively, if `errno` = 4, the absolute condition
        number of the matrix pth power.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A has eigenvalues on the negative real line. The principal
            pth power is not defined in this case;
            ``complex_gen_matrix_cond_pow`` can be used to find a
            complex, non-principal pth power.

        (`errno` 2)
            A is singular so the pth power cannot be computed.

        (`errno` 4)
            The relative condition number is infinite. The absolute
            condition number was returned instead.

        (`errno` 5)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            A^p has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_cond_pow'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Output, condpa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _condpa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _p,
        _condpa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line.',
                    'The principal pth power is not defined in this case;',
                    'complex_gen_matrix_cond_pow can be used to find a complex, non-principal pth power.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the pth power cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A^p has been computed using',
                    'an IEEE double precision Padé approximant, although the',
                    'arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The relative condition number is infinite. The absolute condition number',
                    'was returned instead.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _condpa_py = utils._EngineFloat64ScalarType.to_py(_condpa)
        return_data = _condpa_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['condpa'],
            )(
                _condpa_py,
            )
    _err_class.manage()
    return return_data

def real_gen_matrix_frcht_pow(n, a, e, p):
    u"""
    Fréchet derivative of real matrix power.

    For full information please refer to the NAG Library document for
    f01jf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jff.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix pth power, A^p.

    e : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix E.

        On exit: the Fréchet derivative L(A, E).

    p : float
        The required power of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A has eigenvalues on the negative real line. The principal
            pth power is not defined in this case;
            ``complex_gen_matrix_frcht_pow`` can be used to find a
            complex, non-principal pth power.

        (`errno` 2)
            A is singular so the pth power cannot be computed.

        (`errno` 4)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            A^p has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_frcht_pow'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._BYREF_FLOAT64, # Input, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (e, 'e')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _lde_py = utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _e = _e_cls.to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _e,
        _lde,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line.',
                    'The principal pth power is not defined in this case;',
                    'complex_gen_matrix_frcht_pow can be used to find a complex, non-principal pth power.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the pth power cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A^p has been computed using',
                    'an IEEE double precision Padé approximant, although the',
                    'arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _e_cls.refresh_py(_e)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_cond_exp(n, a):
    u"""
    Condition number for real matrix exponential.

    For full information please refer to the NAG Library document for
    f01jg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jgf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix exponential e^A.

    Returns
    -------
    condea : float
        An estimate of the relative condition number of the matrix
        exponential kappa_exp(A).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            The linear equations to be solved for the Padé approximant
            are singular; it is likely that this function has been
            called incorrectly.

        (`errno` 3)
            An unexpected internal error has occurred. Please contact
            NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix exponential.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_cond_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, condea.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _condea = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _condea,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved for the Padé approximant are singular;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 9999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix exponential.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error has occurred. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _condea_py = utils._EngineFloat64ScalarType.to_py(_condea)
        return_data = _condea_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['condea'],
            )(
                _condea_py,
            )
    _err_class.manage()
    return return_data

def real_gen_matrix_frcht_exp(n, a, e):
    u"""
    Fréchet derivative of real matrix exponential.

    For full information please refer to the NAG Library document for
    f01jh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jhf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix exponential e^A.

    e : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix E

        On exit: the Fréchet derivative L(A, E)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            The linear equations to be solved for the Padé approximant
            are singular; it is likely that this function has been
            called incorrectly.

        (`errno` 3)
            An unexpected internal error has occurred. Please contact
            NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix exponential.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_frcht_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (e, 'e')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _lde_py = utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _e = _e_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _e,
        _lde,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved for the Padé approximant are singular;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 9999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix exponential.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error has occurred. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _e_cls.refresh_py(_e)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_matrix_cond_log(n, a):
    u"""
    Condition number for real matrix logarithm.

    For full information please refer to the NAG Library document for
    f01jj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jjf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix logarithm, log(A).

    Returns
    -------
    condla : float
        With the function exits successfully or `errno` = 3, an estimate
        of the relative condition number of the matrix logarithm,
        kappa_log(A). Alternatively, if `errno` = 4, contains the
        absolute condition number of the matrix logarithm.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A is singular so the logarithm cannot be computed.

        (`errno` 2)
            A has eigenvalues on the negative real line. The principal
            logarithm is not defined in this case;
            ``complex_gen_matrix_cond_log`` can be used to return a
            complex, non-principal log.

        (`errno` 5)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            log(A) has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.

        (`errno` 4)
            The relative condition number is infinite. The absolute
            condition number was returned instead.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_cond_log'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, condla.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _condla = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _condla,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the logarithm cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line. The principal logarithm is not',
                    'defined in this case; complex_gen_matrix_cond_log can be used to return a complex,',
                    'non-principal log.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'log(A) has been computed using an IEEE double precision Padé approximant,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The relative condition number is infinite. The absolute condition number',
                    'was returned instead.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _condla_py = utils._EngineFloat64ScalarType.to_py(_condla)
        return_data = _condla_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['condla'],
            )(
                _condla_py,
            )
    _err_class.manage()
    return return_data

def real_gen_matrix_frcht_log(n, a, e):
    u"""
    Fréchet derivative of real matrix logarithm.

    For full information please refer to the NAG Library document for
    f01jk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01jkf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix logarithm, log(A).

    e : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix E

        On exit: the Fréchet derivative L(A, E)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A is singular so the logarithm cannot be computed.

        (`errno` 2)
            A has eigenvalues on the negative real line. The principal
            logarithm is not defined in this case;
            ``complex_gen_matrix_frcht_log`` can be used to return a
            complex, non-principal log.

        (`errno` 4)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            log(A) has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_matrix_frcht_log'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01jkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (e, 'e')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _lde_py = utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _e = _e_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _e,
        _lde,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the logarithm cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line. The principal logarithm is not',
                    'defined in this case; complex_gen_matrix_frcht_log can be used to return a complex,',
                    'non-principal log.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'log(A) has been computed using an IEEE double precision Padé approximant,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _e_cls.refresh_py(_e)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_cond_std(fun, n, a):
    u"""
    Condition number for the exponential, logarithm, sine, cosine, sinh
    or cosh of a complex matrix.

    For full information please refer to the NAG Library document for
    f01ka

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kaf.html

    Parameters
    ----------
    fun : str
        Indicates which matrix function will be used.

        `fun` = 'EXP'

          The matrix exponential, e^A, will be used.

        `fun` = 'SIN'

          The matrix sine, sin(A), will be used.

        `fun` = 'COS'

          The matrix cosine, cos(A), will be used.

        `fun` = 'SINH'

          The hyperbolic matrix sine, sinh(A), will be used.

        `fun` = 'COSH'

          The hyperbolic matrix cosine, cosh(A), will be used.

        `fun` = 'LOG'

          The matrix logarithm, log(A), will be used.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    Returns
    -------
    conda : float
        An estimate of the absolute condition number of f at A.

    norma : float
        The 1-norm of A.

    normfa : float
        The 1-norm of f(A).

    Raises
    ------
    NagValueError
        (`errno` -2)
            Input argument number *<value>* is invalid.

        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            An internal error occurred when estimating the norm of the
            Fréchet derivative of f at A. Please contact NAG.

        (`errno` 2)
            An internal error occurred when evaluating the matrix
            function f(A). You can investigate further by calling
            ``complex_gen_matrix_exp``, ``complex_gen_matrix_log`` or
            ``complex_gen_matrix_fun_std`` with the matrix A.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_cond_std'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01kaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ifun.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, conda.
        utils._BYREF_FLOAT64, # Output, norma.
        utils._BYREF_FLOAT64, # Output, normfa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _fun = utils._EngineCharScalarType(
        fun, {'fun_name': fun_name, 'entity': 'fun'}, exp_chrlen=None,
    ).to_ctypes()
    _fun_py = utils._EngineCharScalarType.to_py(_fun)
    if _fun_py.upper() == 'EXP':
        _ifun = utils._EngineIntScalarType.c_type(1)
    elif _fun_py.upper() == 'SIN':
        _ifun = utils._EngineIntScalarType.c_type(2)
    elif _fun_py.upper() == 'COS':
        _ifun = utils._EngineIntScalarType.c_type(3)
    elif _fun_py.upper() == 'SINH':
        _ifun = utils._EngineIntScalarType.c_type(4)
    elif _fun_py.upper() == 'COSH':
        _ifun = utils._EngineIntScalarType.c_type(5)
    elif _fun_py.upper() == 'LOG':
        _ifun = utils._EngineIntScalarType.c_type(6)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Invalid value for fun.',
            ],
            errcodes=(-1,),
        )
        _err_class.manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _conda = utils._EngineFloat64ScalarType.empty_ctypes()
    _norma = utils._EngineFloat64ScalarType.empty_ctypes()
    _normfa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ifun,
        _n,
        _a,
        _lda,
        _conda,
        _norma,
        _normfa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when estimating the norm of the Fréchet',
                    'derivative of f at A. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when evaluating the matrix function f(A).',
                    'You can investigate further by calling',
                    'complex_gen_matrix_exp, complex_gen_matrix_log or complex_gen_matrix_fun_std with the matrix A.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _conda_py = utils._EngineFloat64ScalarType.to_py(_conda)
        _norma_py = utils._EngineFloat64ScalarType.to_py(_norma)
        _normfa_py = utils._EngineFloat64ScalarType.to_py(_normfa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['conda', 'norma', 'normfa'],
        )(
            _conda_py,
            _norma_py,
            _normfa_py,
        )
    _err_class.manage()
    return return_data

def complex_gen_matrix_cond_num(n, a, f, data=None):
    u"""
    Condition number for a function of a complex matrix (using numerical
    differentiation).

    For full information please refer to the NAG Library document for
    f01kb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kbf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(nz, z, fz, data=None)
        The function `f` evaluates f(z_i) at a number of points z_i.

        Parameters
        ~~~~~~~~~~
        nz : int
            n_z, the number of function values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function values. this argument[i-1] should
            return the value f(z_i), for i = 1,2,...,n_z.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    conda : float
        An estimate of the absolute condition number of f at A.

    norma : float
        The 1-norm of A.

    normfa : float
        The 1-norm of f(A).

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            An internal error occurred when estimating the norm of the
            Fréchet derivative of f at A. Please contact NAG.

        (`errno` 2)
            An internal error occurred while evaluating the matrix
            function f(A). You can investigate further by calling
            ``complex_gen_matrix_fun_num`` with the matrix A and the
            function f.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 3)
            Termination requested in `f`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_cond_num'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, norder.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, norder, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01kbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._BYREF_FLOAT64, # Output, conda.
        utils._BYREF_FLOAT64, # Output, norma.
        utils._BYREF_FLOAT64, # Output, normfa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _conda = utils._EngineFloat64ScalarType.empty_ctypes()
    _norma = utils._EngineFloat64ScalarType.empty_ctypes()
    _normfa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _conda,
        _norma,
        _normfa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when estimating the norm of the Fréchet',
                    'derivative of f at A. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred while evaluating the matrix function f(A).',
                    'You can investigate further by calling',
                    'complex_gen_matrix_fun_num with the matrix A and the function f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _conda_py = utils._EngineFloat64ScalarType.to_py(_conda)
        _norma_py = utils._EngineFloat64ScalarType.to_py(_norma)
        _normfa_py = utils._EngineFloat64ScalarType.to_py(_normfa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['conda', 'norma', 'normfa'],
        )(
            _conda_py,
            _norma_py,
            _normfa_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_cond_usd(n, a, f, data=None):
    u"""
    Condition number for a function of a complex matrix (using
    user-supplied derivatives).

    For full information please refer to the NAG Library document for
    f01kc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kcf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix, f(A).

    f : callable f(m, nz, z, fz, data=None)
        Given an integer m, the function `f` evaluates f^(m)(z_i) at a
        number of points z_i.

        Parameters
        ~~~~~~~~~~
        m : int
            The order, m, of the derivative required.

            If `m` = 0, f(z_i) should be returned.

            For `m` > 0, f^(m)(z_i) should be returned.

        nz : int
            n_z, the number of function or derivative values required.

        z : complex, ndarray, shape (`nz`)
            The n_z points z_1,z_2,...,z_n_z at which the function f is
            to be evaluated.

        fz : complex, ndarray, shape (`nz`), to be modified in place
            On exit: the n_z function or derivative values. this
            argument[i-1] should return the value f^(m)(z_i), for i =
            1,2,...,n_z.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    conda : float
        An estimate of the absolute condition number of f at A.

    norma : float
        The 1-norm of A.

    normfa : float
        The 1-norm of f(A).

    Raises
    ------
    NagValueError
        (`errno` -1)
            Input argument number *<value>* is invalid.

        (`errno` 1)
            An internal error occurred when estimating the norm of the
            Fréchet derivative of f at A. Please contact NAG.

        (`errno` 2)
            An internal error occurred when evaluating the matrix
            function f(A). You can investigate further by calling
            ``complex_gen_matrix_fun_usd`` with the matrix A and the
            function f.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 3)
            Termination requested in `f`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_cond_usd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, nz.
        utils._EngineComplex128ArrayType.c_type, # Input, z[nz].
        utils._EngineComplex128ArrayType.c_type, # Output, fz[nz].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, m, iflag, nz, z, fz, _iuser, _ruser):
        """Helper for f."""
        try:
            _m_fh = utils._EngineIntScalarType.to_py(m.contents)
            _nz_fh = utils._EngineIntScalarType.to_py(nz.contents)
            _z_fh = utils._EngineComplex128ArrayType.from_pointer(
                z,
                (_nz_fh,),
            )
            _fz_fh = utils._EngineComplex128ArrayType.from_pointer(
                fz,
                (_nz_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                    data,
                )
            else:
                f(
                    _m_fh,
                    _nz_fh,
                    _z_fh,
                    _fz_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineComplex128ArrayType.refresh_ctypes_buffer(
                _fz_fh, {'fun_name': cb_f_name, 'returned': 'fz'}, fz,
            )
            utils._unlink_pointer(fz)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f01kcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, iflag.
        utils._BYREF_FLOAT64, # Output, conda.
        utils._BYREF_FLOAT64, # Output, norma.
        utils._BYREF_FLOAT64, # Output, normfa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _iflag = utils._EngineIntScalarType.empty_ctypes()
    _conda = utils._EngineFloat64ScalarType.empty_ctypes()
    _norma = utils._EngineFloat64ScalarType.empty_ctypes()
    _normfa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _f,
        _fh,
        _iuser,
        _ruser,
        _iflag,
        _conda,
        _norma,
        _normfa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Input argument number {:s} is invalid.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when estimating the norm of the Fréchet',
                    'derivative of f at A. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when evaluating the matrix function f(A).',
                    'You can investigate further by calling complex_gen_matrix_fun_usd with the matrix A and',
                    'the function f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _conda_py = utils._EngineFloat64ScalarType.to_py(_conda)
        _norma_py = utils._EngineFloat64ScalarType.to_py(_norma)
        _normfa_py = utils._EngineFloat64ScalarType.to_py(_normfa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['conda', 'norma', 'normfa'],
        )(
            _conda_py,
            _norma_py,
            _normfa_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_cond_sqrt(n, a):
    u"""
    Condition number for square root of complex matrix.

    For full information please refer to the NAG Library document for
    f01kd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kdf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix square root A^(1/2).
        Alternatively, if `errno` = 1, contains an n by n non-principal
        square root of A.

    Returns
    -------
    alpha : float
        An estimate of the stability of the relative residual for the
        computed principal (if no exception or warning is raised) or
        non-principal (if `errno` = 1) matrix square root, alpha.

    condsa : float
        An estimate of the relative condition number, in the Frobenius
        norm, of the principal (if no exception or warning is raised) or
        non-principal (if `errno` = 1) matrix square root at A,
        kappa_A^(1/2).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A has a defective vanishing eigenvalue. The square root and
            condition number cannot be found in this case.

        (`errno` 3)
            An error occurred when computing the matrix square root.
            Consequently, `alpha` and `condsa` could not be computed. It
            is likely that the function was called incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has a negative or semisimple vanishing eigenvalue. A
            non-principal square root was returned.

        (`errno` 4)
            An error occurred when computing the condition number. The
            matrix square root was still returned but you should use
            ``complex_gen_matrix_sqrt`` to check if it is the principal
            matrix square root.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_cond_sqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01kdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, alpha.
        utils._BYREF_FLOAT64, # Output, condsa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _alpha = utils._EngineFloat64ScalarType.empty_ctypes()
    _condsa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _alpha,
        _condsa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a negative or semisimple vanishing eigenvalue. A non-principal',
                    'square root was returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has a defective vanishing eigenvalue. The square root and condition',
                    'number cannot be found in this case.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error occurred when computing the matrix square root. Consequently, alpha',
                    'and condsa could not be computed. It is likely that the function was',
                    'called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error occurred when computing the condition number. The matrix square',
                    'root was still returned but you should use complex_gen_matrix_sqrt to check if it is the',
                    'principal matrix square root.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _alpha_py = utils._EngineFloat64ScalarType.to_py(_alpha)
        _condsa_py = utils._EngineFloat64ScalarType.to_py(_condsa)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['alpha', 'condsa'],
        )(
            _alpha_py,
            _condsa_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_cond_pow(n, a, p):
    u"""
    Condition number for complex matrix power.

    For full information please refer to the NAG Library document for
    f01ke

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kef.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix pth power, A^p, unless
        `errno` = 1, in which case a non-principal pth power is
        returned.

    p : float
        The required power of A.

    Returns
    -------
    condpa : float
        If the function exits successfully or `errno` = 3, an estimate
        of the relative condition number of the matrix pth power,
        kappa_A^p. Alternatively, if `errno` = 4, the absolute condition
        number of the matrix pth power.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A is singular so the pth power cannot be computed.

        (`errno` 5)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has eigenvalues on the negative real line. The principal
            pth power is not defined in this case, so a non-principal
            power was returned.

        (`errno` 3)
            A^p has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.

        (`errno` 4)
            The relative condition number is infinite. The absolute
            condition number was returned instead.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_cond_pow'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01keft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Output, condpa.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _condpa = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _p,
        _condpa,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line.',
                    'The principal pth power is not defined in this case,',
                    'so a non-principal power was returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the pth power cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A^p has been computed using',
                    'an IEEE double precision Padé approximant, although the',
                    'arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The relative condition number is infinite. The absolute condition number',
                    'was returned instead.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _condpa_py = utils._EngineFloat64ScalarType.to_py(_condpa)
        return_data = _condpa_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['condpa'],
            )(
                _condpa_py,
            )
    _err_class.manage()
    return return_data

def complex_gen_matrix_frcht_pow(n, a, e, p):
    u"""
    Fréchet derivative of complex matrix power.

    For full information please refer to the NAG Library document for
    f01kf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kff.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix pth power, A^p.
        Alternatively if `errno` = 1, a non-principal pth power is
        returned.

    e : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix E.

        On exit: the Fréchet derivative L(A, E).

    p : float
        The required power of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            A is singular so the pth power cannot be computed.

        (`errno` 4)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A has eigenvalues on the negative real line. The principal
            pth power is not defined in this case, so a non-principal
            power was returned.

        (`errno` 3)
            A^p has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_frcht_pow'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01kfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._BYREF_FLOAT64, # Input, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (e, 'e')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _lde_py = utils._EngineComplex128ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e_cls = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _e = _e_cls.to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _e,
        _lde,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line.',
                    'The principal pth power is not defined in this case,',
                    'so a non-principal power was returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the pth power cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A^p has been computed using',
                    'an IEEE double precision Padé approximant, although the',
                    'arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _e_cls.refresh_py(_e)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_cond_exp(n, a):
    u"""
    Condition number for complex matrix exponential.

    For full information please refer to the NAG Library document for
    f01kg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kgf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix exponential e^A.

    Returns
    -------
    condea : float
        An estimate of the relative condition number of the matrix
        exponential kappa_exp(A).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            The linear equations to be solved for the Padé approximant
            are singular; it is likely that this function has been
            called incorrectly.

        (`errno` 3)
            An unexpected internal error has occurred. Please contact
            NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix exponential.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_cond_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01kgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, condea.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _condea = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _condea,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved for the Padé approximant are singular;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 9999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix exponential.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error has occurred. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _condea_py = utils._EngineFloat64ScalarType.to_py(_condea)
        return_data = _condea_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['condea'],
            )(
                _condea_py,
            )
    _err_class.manage()
    return return_data

def complex_gen_matrix_frcht_exp(n, a, e):
    u"""
    Fréchet derivative of complex matrix exponential.

    For full information please refer to the NAG Library document for
    f01kh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01khf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n matrix exponential e^A.

    e : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix E

        On exit: the Fréchet derivative L(A, E)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            The linear equations to be solved for the Padé approximant
            are singular; it is likely that this function has been
            called incorrectly.

        (`errno` 3)
            An unexpected internal error has occurred. Please contact
            NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The arithmetic precision is higher than that used for the
            Padé approximant computed matrix exponential.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_frcht_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01khft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (e, 'e')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _lde_py = utils._EngineComplex128ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e_cls = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _e = _e_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _e,
        _lde,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations to be solved for the Padé approximant are singular;',
                    'it is likely that this function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 9999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arithmetic precision is higher than that used for the',
                    'Padé approximant computed matrix exponential.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error has occurred. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _e_cls.refresh_py(_e)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_gen_matrix_cond_log(n, a):
    u"""
    Condition number for complex matrix logarithm.

    For full information please refer to the NAG Library document for
    f01kj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kjf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix logarithm, log(A).
        Alternatively, if `errno` = 2, a non-principal logarithm is
        returned.

    Returns
    -------
    condla : float
        With the function exits successfully or `errno` = 2 or 3, an
        estimate of the relative condition number of the matrix
        logarithm, kappa_log(A). Alternatively, if `errno` = 4, contains
        the absolute condition number of the matrix logarithm.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A is singular so the logarithm cannot be computed.

        (`errno` 5)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            A has eigenvalues on the negative real line. The principal
            logarithm is not defined in this case, so a non-principal
            logarithm was returned.

        (`errno` 3)
            log(A) has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.

        (`errno` 4)
            The relative condition number is infinite. The absolute
            condition number was returned instead.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_cond_log'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01kjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Output, condla.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _condla = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _condla,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the logarithm cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1100:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line. The principal logarithm is not',
                    'defined in this case, so a non-principal logarithm was returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'log(A) has been computed using an IEEE double precision Padé approximant,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The relative condition number is infinite. The absolute condition number',
                    'was returned instead.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _condla_py = utils._EngineFloat64ScalarType.to_py(_condla)
        return_data = _condla_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['condla'],
            )(
                _condla_py,
            )
    _err_class.manage()
    return return_data

def complex_gen_matrix_frcht_log(n, a, e):
    u"""
    Fréchet derivative of complex matrix logarithm.

    For full information please refer to the NAG Library document for
    f01kk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01kkf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: the n by n principal matrix logarithm, log(A).
        Alterntively, if `errno` = 2, a non-principal logarithm is
        returned.

    e : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix E

        On exit: with the function exits successfully or `errno` = 2 or
        3, the Fréchet derivative L(A, E)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            A is singular so the logarithm cannot be computed.

        (`errno` 4)
            An unexpected internal error occurred. This failure should
            not occur and suggests that the function has been called
            incorrectly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            A has eigenvalues on the negative real line. The principal
            logarithm is not defined in this case, so a non-principal
            logarithm was returned.

        (`errno` 3)
            log(A) has been computed using an IEEE double precision Padé
            approximant, although the arithmetic precision is higher
            than IEEE double precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_matrix_frcht_log'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01kkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (e, 'e')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _lde_py = utils._EngineComplex128ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e_cls = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _e = _e_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _e,
        _lde,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is singular so the logarithm cannot be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1100:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has eigenvalues on the negative real line. The principal logarithm is not',
                    'defined in this case, so a non-principal logarithm was returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'log(A) has been computed using an IEEE double precision Padé approximant,',
                    'although the arithmetic precision is higher than IEEE double precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred. This failure should not occur and',
                    'suggests that the function has been called incorrectly.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _e_cls.refresh_py(_e)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_tridiag_lu(n, a, lamda, b, c, tol, d, ipiv):
    u"""
    LU factorization of real tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f01le

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01lef.html

    Parameters
    ----------
    n : int
        n, the order of the matrix T.

    a : float, shape (`n`), modified in place
        On entry: the diagonal elements of T.

        On exit: the diagonal elements of the upper triangular matrix U.

    lamda : float
        The scalar lambda. ``real_gen_tridiag_lu`` factorizes T-lambda
        I.

    b : float, shape (`n`), modified in place
        On entry: the superdiagonal elements of T, stored in `b`[1] to
        `b`[n-1]; `b`[0] is not used.

        On exit: the elements of the first superdiagonal of U, stored in
        `b`[1] to `b`[n-1].

    c : float, shape (`n`), modified in place
        On entry: the subdiagonal elements of T, stored in `c`[1] to
        `c`[n-1]; `c`[0] is not used.

        On exit: the subdiagonal elements of L, stored in `c`[1] to
        `c`[n-1].

    tol : float
        A relative tolerance used to indicate whether or not the matrix
        (T-lambda I) is nearly singular. `tol` should normally be chosen
        as approximately the largest relative error in the elements of
        T. For example, if the elements of T are correct to about 4
        significant figures, then `tol` should be set to about
        5*10^(-4). See [Further Comments] for further details on how
        `tol` is used. If `tol` is supplied as less than epsilon, where
        epsilon is the machine precision, then the value epsilon is used
        in place of `tol`.

    d : float, shape (`n`), modified in place
        On exit: the elements of the second superdiagonal of U, stored
        in `d`[2] to `d`[n-1]; `d`[0] and `d`[1] are not used.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the permutation matrix P. If an interchange
        occurred at the kth step of the elimination, then `ipiv`[k-1] =
        1, otherwise `ipiv`[k-1] = 0. If a diagonal element of U is
        small, indicating that (T-lambda I) is nearly singular, then the
        element `ipiv`[n-1] is returned as positive. Otherwise
        `ipiv`[n-1] is returned as 0. See [Further Comments] for further
        details. If the application is such that it is important that
        (T-lambda I) is not nearly singular, then it is strongly
        recommended that `ipiv`[n-1] is inspected on return.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_tridiag_lu'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01left')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[n].
        utils._BYREF_FLOAT64, # Input, lamda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    )
    _a = _a_cls.to_ctypes()
    _lamda = utils._EngineFloat64ScalarType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    )
    _b = _b_cls.to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
    )
    _c = _c_cls.to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lamda,
        _b,
        _c,
        _tol,
        _d,
        _ipiv,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        _d_cls.refresh_py(_d)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_blkdiag_lu(n, nbloks, blkstr, a, lena, pivot, tol):
    u"""
    LU factorization of real almost block diagonal matrix.

    For full information please refer to the NAG Library document for
    f01lh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01lhf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nbloks : int
        n, the total number of blocks of the matrix A.

    blkstr : int, shape (3, `nbloks`)
        Information which describes the block structure of A as follows:

        - `blkstr`[0,k-1] must contain the number of rows in the kth
          block, k = 1,2,...,`nbloks`;

        - `blkstr`[1,k-1] must contain the number of columns in the kth
          block, k = 1,2,...,`nbloks`;

        - `blkstr`[2,k-1] must contain the number of columns of overlap
          between the kth and (k+1)th blocks, k = 1,2,...,`nbloks`-1.
          `blkstr`[2,`nbloks`-1] need not be set.

        The following conditions delimit the structure of A:

        - `blkstr`[0,k-1],`blkstr`[1,k-1] > 0, k = 1,2,...,`nbloks`,

        - `blkstr`[2,k-1] >= 0, k = 1,2,...,`nbloks`-1,

        (there must be at least one column and one row in each block and
        a non-negative number of columns of overlap);

        - `blkstr`[2,k-2]+`blkstr`[2,k-1] <= `blkstr`[1,k-1], k =
          2,3,...,`nbloks`-1,

        (the total number of columns in overlaps in each block must not
        exceed the number of columns in that block);

        - `blkstr`[1,0] >= `blkstr`[0,0],

        - `blkstr`[1,0]+sum_k = 2^j[`blkstr`[1,k -1]-`blkstr`[2,k -1
          -1]] >= sum_k = 1^j`blkstr`[0,k -1], j = 2,3,...,`nbloks`-1,

        - sum_k = 1^j[`blkstr`[1,k-1]-`blkstr`[2,k-1]] <= sum_k =
          1^j`blkstr`[0,k-1], j = 1,2,...,`nbloks`-1,

        (the index of the first column of the overlap between the jth
        and (j+1)th blocks must be <= the index of the last row of the
        jth block, and the index of the last column of overlap must be
        >= the index of the last row of the jth block);

        - sum_k = 1^`nbloks``blkstr`[0,k -1] = n,

        - `blkstr`[1,0]+sum_k = 2^`nbloks`[`blkstr`[1,k -1]-`blkstr`[2,k
          -1 -1]] = nk,

        (both the number of rows and the number of columns of A must
        equal n).

    a : float, shape (`lena`), modified in place
        On entry: the elements of the almost block diagonal matrix
        stored block by block, with each block stored column by column.
        The sizes of the blocks and the overlaps are defined by the
        argument `blkstr`.

        If a_{rs} is the first element in the kth block, then an
        arbitrary element a_{ij} in the kth block must be stored in the
        array element:

            `a`[ p_k + (j-r) m_k + (i-s)+1 -1]

        where

            p_k = sum_l = 1^k-1`blkstr`[0,l-1]*`blkstr`[1,l-1]

        is the base address of the kth block, and

            m_k = `blkstr`[0,k-1]

        is the number of rows of the kth block.

        See [Further Comments] for comments on scaling.

        On exit: the factorized form of the matrix.

    lena : int
        The dimension of the array `a`.

    pivot : int, shape (`n`), modified in place
        On exit: details of the interchanges.

    tol : float
        A relative tolerance to be used to indicate whether or not the
        matrix is singular. For a discussion on how `tol` is used see
        [Further Comments]. If `tol` is non-positive, `tol` is reset to
        10 epsilon, where epsilon is the machine precision.

    Returns
    -------
    tol : float
        Unchanged unless `tol` <= 0.0 on entry, in which case it is set
        to 10*epsilon.

    kpivot : int
        If `errno` = 2, `kpivot` contains the value k, where k is the
        first position on the diagonal of the matrix A where too small a
        pivot was detected. Otherwise `kpivot` is set to 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lena` is too small. `lena` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, the following equality does not hold:
            `blkstr`[1,0]+sum(`blkstr`[1,k-1]-`blkstr`[2,k-2]:k = 2,
            `nbloks`) = `n`.

        (`errno` 1)
            On entry, the following equality does not hold:
            sum(`blkstr`[0,k-1]:k = 1,`nbloks`) = `n`.

        (`errno` 1)
            On entry, K = *<value>* `blkstr`[2,K-1] = *<value>*
            `blkstr`[2,K-2] = *<value>* and `blkstr`[1,K-1] = *<value>*.

            Constraint: `blkstr`[2,K-1]+`blkstr`[2,K-2] >=
            `blkstr`[1,K-1].

        (`errno` 1)
            On entry, the following inequality was not satisfied for: J
            = *<value>*. sum(`blkstr`[1,k-1]-`blkstr`[2,k-1]:k = 1, J)
            <= sum(`blkstr`[0,k-1]:k = 1, J) <=
            `blkstr`[1,0]+sum(`blkstr`[1,k-1]-`blkstr`[2,k-2]:k = 2, J).

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[1,K-1] = *<value>*
            `blkstr`[2,K-1] = *<value>* and `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1]-`blkstr`[2,K-1] <=
            `blkstr`[0,K-1].

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[1,K-1] = *<value>* and
            `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1] >= `blkstr`[0,K-1].

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[2,K-1] = *<value>*.

            Constraint: `blkstr`[2,K-1] >= 0.

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[1,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1] >= 1.

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[0,K-1] >= 1.

        (`errno` 1)
            On entry, `n` = *<value>* and `nbloks` = *<value>*.

            Constraint: `n` >= `nbloks`.

        (`errno` 1)
            On entry, `nbloks` = *<value>*.

            Constraint: `nbloks` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Factorization completed, but pivot in diagonal I was small:
            I = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_blkdiag_lu'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01lhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nbloks.
        utils._EngineIntArrayType.c_type, # Input, blkstr[3, nbloks].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lena].
        utils._BYREF_INT, # Input, lena.
        utils._EngineIntArrayType.c_type, # Output, pivot[n].
        utils._BYREF_FLOAT64, # Input/Output, tol.
        utils._BYREF_INT, # Output, kpivot.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((blkstr, 'blkstr'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nbloks = utils._EngineIntScalarType(
        nbloks, {'fun_name': fun_name, 'entity': 'nbloks'},
    ).to_ctypes()
    _nbloks_py = utils._EngineIntScalarType.to_py(_nbloks)
    _blkstr = utils._EngineIntArrayType(
        blkstr, {'fun_name': fun_name, 'entity': 'blkstr'},
        exp_shape=(3, _nbloks_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lena = utils._EngineIntScalarType(
        lena, {'fun_name': fun_name, 'entity': 'lena'},
    ).to_ctypes()
    _lena_py = utils._EngineIntScalarType.to_py(_lena)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_lena_py,),
    )
    _a = _a_cls.to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _pivot_cls = utils._EngineIntArrayType(
        pivot, {'fun_name': fun_name, 'entity': 'pivot'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _pivot = _pivot_cls.to_ctypes(can_reference_contents=False)
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _kpivot = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nbloks,
        _blkstr,
        _a,
        _lena,
        _pivot,
        _tol,
        _kpivot,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lena is too small.',
                    'lena = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the following equality does not hold:',
                    'blkstr[1,0]+sum(blkstr[1,k-1]-blkstr[2,k-2]:k = 2, nbloks) = n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the following equality does not hold:',
                    'sum(blkstr[0,k-1]:k = 1,nbloks) = n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s} blkstr[2,K-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'blkstr[2,K-2] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'blkstr[1,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: blkstr[2,K-1]+blkstr[2,K-2] >= blkstr[1,K-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the following inequality was not satisfied for:',
                    'J = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'sum(blkstr[1,k-1]-blkstr[2,k-1]:k = 1, J) <=',
                    'sum(blkstr[0,k-1]:k = 1, J) <=',
                    'blkstr[1,0]+sum(blkstr[1,k-1]-blkstr[2,k-2]:k = 2, J).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s}, blkstr[1,K-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'blkstr[2,K-1] = {:s} and blkstr[0,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: blkstr[1,K-1]-blkstr[2,K-1] <= blkstr[0,K-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s}, blkstr[1,K-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and blkstr[0,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: blkstr[1,K-1] >= blkstr[0,K-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'blkstr[2,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: blkstr[2,K-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'blkstr[1,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: blkstr[1,K-1] >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'blkstr[0,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: blkstr[0,K-1] >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and nbloks = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= nbloks.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nbloks = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nbloks >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Factorization completed, but pivot in diagonal I was small:',
                    'I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _pivot_cls.refresh_py(_pivot)
        _tol_py = utils._EngineFloat64ScalarType.to_py(_tol)
        _kpivot_py = utils._EngineIntScalarType.to_py(_kpivot)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['tol', 'kpivot'],
        )(
            _tol_py,
            _kpivot_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_vband_posdef_fac(n, a, lal, nrow, al, d):
    u"""
    LDL^T factorization of real symmetric positive definite
    variable-bandwidth matrix.

    For full information please refer to the NAG Library document for
    f01mc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01mcf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`lal`)
        The elements within the envelope of the lower triangle of the
        positive definite symmetric matrix A, taken in row by row order.

        See also [Further Comments].

    lal : int
        The dimension of the arrays `a` and `al`.

    nrow : int, shape (`n`), modified in place
        `nrow`[i-1] must contain the width of row i of the matrix A,
        i.e., the number of elements between the first (leftmost)
        nonzero element and the element on the diagonal, inclusive.

    al : float, shape (`lal`), modified in place
        On exit: the elements within the envelope of the lower
        triangular matrix L, taken in row by row order. The envelope of
        L is identical to that of the lower triangle of A. The unit
        diagonal elements of L are stored explicitly. See also [Further
        Comments].

    d : float, shape (`n`), modified in place
        On exit: the diagonal elements of the diagonal matrix D. Note
        that the determinant of A is equal to the product of these
        diagonal elements. If the value of the determinant is required
        it should not be determined by forming the product explicitly,
        because of the possibility of overflow or underflow. The
        logarithm of the determinant may safely be formed from the sum
        of the logarithms of the diagonal elements.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lal` = *<value>* and `nrow`[0]+...+`nrow`[`n`-1]
            = *<value>*.

            Constraint: `lal` >= `nrow`[0]+...+`nrow`[`n`-1].

        (`errno` 1)
            On entry, I = *<value>* and `nrow`[I-1] = *<value>*.

            Constraint: `nrow`[I-1] >= 1 and `nrow`[I-1] <= I.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            A is not positive definite. Factorization abandoned.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            A is not positive definite. Factorization completed.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_vband_posdef_fac'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01mcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lal].
        utils._BYREF_INT, # Input, lal.
        utils._EngineIntArrayType.c_type, # Input/Output, nrow[n].
        utils._EngineFloat64ArrayType.c_type, # Output, al[lal].
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lal = utils._EngineIntScalarType(
        lal, {'fun_name': fun_name, 'entity': 'lal'},
    ).to_ctypes()
    _lal_py = utils._EngineIntScalarType.to_py(_lal)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_lal_py,),
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _nrow_cls = utils._EngineIntArrayType(
        nrow, {'fun_name': fun_name, 'entity': 'nrow'},
        exp_shape=(_n_py,),
    )
    _nrow = _nrow_cls.to_ctypes()
    _al_cls = utils._EngineFloat64ArrayType(
        al, {'fun_name': fun_name, 'entity': 'al'},
        exp_shape=(_lal_py,),
        check_contents=False,
    )
    _al = _al_cls.to_ctypes(can_reference_contents=False)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lal,
        _nrow,
        _al,
        _d,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lal = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nrow[0]+...+nrow[n-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lal >= nrow[0]+...+nrow[n-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s} and nrow[I-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nrow[I-1] >= 1 and nrow[I-1] <= I.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is not positive definite. Factorization abandoned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A is not positive definite. Factorization completed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _nrow_cls.refresh_py(_nrow)
        _al_cls.refresh_py(_al)
        _d_cls.refresh_py(_d)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_trapez_rq(m, n, a, zeta):
    u"""
    RQ factorization of real m by n upper trapezoidal matrix (m <= n).

    For full information please refer to the NAG Library document for
    f01qg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01qgf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

        When `m` = 0 then an immediate return is effected.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the leading m by n upper trapezoidal part of the array
        `a` must contain the matrix to be factorized.

        On exit: the m by m upper triangular part of `a` will contain
        the upper triangular matrix R, and the m by (n-m) upper
        trapezoidal part of `a` will contain details of the
        factorization as described in [Description].

    zeta : float, shape (`m`), modified in place
        On exit: `zeta`[k-1] contains the scalar zeta_k for the
        (m-k+1)th transformation. If T_k = I then `zeta`[k-1] = 0.0,
        otherwise `zeta`[k-1] contains zeta_k as described in
        [Description] and zeta_k is always in the range (1.0,sqrt(2.0)).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` -1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_trapez_rq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01qgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, zeta[m].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _zeta_cls = utils._EngineFloat64ArrayType(
        zeta, {'fun_name': fun_name, 'entity': 'zeta'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _zeta = _zeta_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _zeta,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _zeta_cls.refresh_py(_zeta)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_rq(m, n, a, zeta):
    u"""
    RQ factorization of real m by n matrix (m <= n).

    For full information please refer to the NAG Library document for
    f01qj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01qjf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

        When `m` = 0 then an immediate return is effected.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the leading m by n part of the array `a` must contain
        the matrix to be factorized.

        On exit: the m by m upper triangular part of `a` will contain
        the upper triangular matrix R, and the m by m strictly lower
        triangular part of `a` and the m by (n-m) rectangular part of
        `a` to the right of the upper triangular part will contain
        details of the factorization as described in [Description].

    zeta : float, shape (`m`), modified in place
        On exit: `zeta`[k-1] contains the scalar zeta_k for the
        (m-k+1)th transformation. If P_k = I then `zeta`[k-1] = 0.0,
        otherwise `zeta`[k-1] contains zeta_k as described in
        [Description] and zeta_k is always in the range (1.0,sqrt(2.0)).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` -1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_rq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01qjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, zeta[m].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _zeta_cls = utils._EngineFloat64ArrayType(
        zeta, {'fun_name': fun_name, 'entity': 'zeta'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _zeta = _zeta_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _zeta,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _zeta_cls.refresh_py(_zeta)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_rq_formq(wheret, m, n, nrowp, a, zeta):
    u"""
    Operations with orthogonal matrices, form rows of Q, after RQ
    factorization by ``real_gen_rq``.

    For full information please refer to the NAG Library document for
    f01qk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01qkf.html

    Parameters
    ----------
    wheret : str, length 1
        Indicates where the elements of zeta are to be found.

        `wheret` = 'I' (In `a`)

          The elements of zeta are in `a`.

        `wheret` = 'S' (Separate)

          The elements of zeta are separate from `a`, in `zeta`.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrowp : int
        l, the required number of rows of P.

        If `nrowp` = 0, an immediate return is effected.

    a : float, shape (max(`m`, `nrowp`), `n`), modified in place
        On entry: the leading m by m strictly lower triangular part of
        the array `a`, and the m by (n-m) rectangular part of `a` with
        top left-hand corner at element `a`[0,`m`] must contain details
        of the matrix P. In addition, if `wheret` = 'I', the diagonal
        elements of `a` must contain the elements of zeta.

        On exit: the first `nrowp` rows of the array `a` are overwritten
        by the first `nrowp` rows of the n by n orthogonal matrix P^T.

    zeta : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `wheret`='S': `m`; otherwise: 1.

        With `wheret` = 'S', the array `zeta` must contain the elements
        of zeta. If `zeta`[k-1] = 0.0 then P_k is assumed to be I,
        otherwise `zeta`[k-1] is assumed to contain zeta_k.

        When `wheret` = 'I', the array `zeta` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, lda = *<value>*, `m` = *<value>* and `nrowp` =
            *<value>*.

            Constraint: lda >= max(`m`, `nrowp`).

        (`errno` -1)
            On entry, `nrowp` = *<value>* and `n` = *<value>*.

            Constraint: `nrowp` >= 0 and `nrowp` <= `n`.

        (`errno` -1)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` -1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -1)
            On entry, `wheret` = *<value>*.

            Constraint: `wheret` = 'I' or 'S'.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_gen_rq_formq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01qkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, wheret(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrowp.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, zeta[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of wheret.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _wheret = utils._EngineCharScalarType(
        wheret, {'fun_name': fun_name, 'entity': 'wheret'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrowp = utils._EngineIntScalarType(
        nrowp, {'fun_name': fun_name, 'entity': 'nrowp'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _nrowp_py = utils._EngineIntScalarType.to_py(_nrowp)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(max(_m_py, _nrowp_py), _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _wheret_py = utils._EngineCharScalarType.to_py(_wheret)
    if _wheret_py in ('S', 's'):
        _zeta_dim1 = _m_py
    else:
        _zeta_dim1 = 1
    exp_shape_zeta = (_zeta_dim1,)
    _zeta = utils._EngineFloat64ArrayType(
        zeta, {'fun_name': fun_name, 'entity': 'zeta'},
        exp_shape=exp_shape_zeta,
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _wheret,
        _m,
        _n,
        _nrowp,
        _a,
        _lda,
        _zeta,
        _work,
        _errbuf,
        _ierr,
        _wheret._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s}, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'nrowp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lda >= max(m, nrowp).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrowp = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nrowp >= 0 and nrowp <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wheret = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wheret = \'I\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def complex_trapez_rq(m, n, a, theta):
    u"""
    RQ factorization of complex m by n upper trapezoidal matrix (m <=
    n).

    For full information please refer to the NAG Library document for
    f01rg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01rgf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

        When `m` = 0 then an immediate return is effected.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the leading m by n upper trapezoidal part of the array
        `a` must contain the matrix to be factorized.

        On exit: the m by m upper triangular part of `a` will contain
        the upper triangular matrix R, and the m by (n-m) upper
        trapezoidal part of `a` will contain details of the
        factorization as described in [Description].

    theta : complex, shape (`m`), modified in place
        On exit: `theta`[k-1] contains the scalar theta_k for the
        (m-k+1)th transformation. If T_k = I then `theta`[k-1] = 0.0; if

            T_k = [table omitted], Re(alpha) < 0.0

        then `theta`[k-1] = alpha, otherwise `theta`[k-1] contains
        theta_k as described in [Description] and Re(theta_k) is always
        in the range (1.0,sqrt(2.0)).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` -1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_trapez_rq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01rgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, theta[m].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _theta_cls = utils._EngineComplex128ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _theta,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _theta_cls.refresh_py(_theta)
        return_data = None
    _err_class.manage()
    return return_data

def complex_gen_rq(m, n, a, theta):
    u"""
    RQ factorization of complex m by n matrix (m <= n).

    For full information please refer to the NAG Library document for
    f01rj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01rjf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

        When `m` = 0 then an immediate return is effected.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the leading m by n part of the array `a` must contain
        the matrix to be factorized.

        On exit: the m by m upper triangular part of `a` will contain
        the upper triangular matrix R, and the m by m strictly lower
        triangular part of `a` and the m by (n-m) rectangular part of
        `a` to the right of the upper triangular part will contain
        details of the factorization as described in [Description].

    theta : complex, shape (`m`), modified in place
        On exit: `theta`[k-1] contains the scalar theta_k for the
        (m-k+1)th transformation. If P_k = I then `theta`[k-1] = 0.0; if

            T_k = [table omitted], Re(alpha) < 0.0

        then `theta`[k-1] = alpha, otherwise `theta`[k-1] contains
        theta_k as described in [Description] and theta_k is always in
        the range (1.0,sqrt(2.0)).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` -1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_rq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01rjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, theta[m].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _theta_cls = utils._EngineComplex128ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _theta,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _theta_cls.refresh_py(_theta)
        return_data = None
    _err_class.manage()
    return return_data

def complex_gen_rq_formq(wheret, m, n, nrowp, a, theta):
    u"""
    Operations with unitary matrices, form rows of Q, after RQ
    factorization by ``complex_gen_rq``.

    For full information please refer to the NAG Library document for
    f01rk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01rkf.html

    Parameters
    ----------
    wheret : str, length 1
        Indicates where the elements of theta are to be found.

        `wheret` = 'I' (In `a`)

          The elements of theta are in `a`.

        `wheret` = 'S' (Separate)

          The elements of theta are separate from `a`, in `theta`.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrowp : int
        l, the required number of rows of P.

        If `nrowp` = 0, an immediate return is effected.

    a : complex, shape (max(`m`, `nrowp`), `n`), modified in place
        On entry: the leading m by m strictly lower triangular part of
        the array `a`, and the m by (n-m) rectangular part of `a` with
        top left-hand corner at element `a`[0,`m`] must contain details
        of the matrix P. In addition, if `wheret` = 'I', the diagonal
        elements of `a` must contain the elements of theta.

        On exit: the first `nrowp` rows of the array `a` are overwritten
        by the first `nrowp` rows of the n by n unitary matrix P^H.

    theta : complex, shape (:)
        Note: the required length for this argument is determined as
        follows: if `wheret`='S': `m`; otherwise: 1.

        If `wheret` = 'S', the array `theta` must contain the elements
        of theta. If `theta`[k-1] = 0.0, P_k is assumed to be I, if
        `theta`[k-1] = alpha and Re(alpha) < 0.0, P_k is assumed to be
        of the form

            P_k = [table omitted],

        otherwise `theta`[k-1] is assumed to contain theta_k given by

            theta_k = (zeta_k,Im(gamma_k)).

        If `wheret` = 'I', the array `theta` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, lda = *<value>*, `m` = *<value>* and `nrowp` =
            *<value>*.

            Constraint: lda >= max(`m`, `nrowp`).

        (`errno` -1)
            On entry, `nrowp` = *<value>* and `n` = *<value>*.

            Constraint: `nrowp` >= 0 and `nrowp` <= `n`.

        (`errno` -1)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` -1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -1)
            On entry, `wheret` = *<value>*.

            Constraint: `wheret` = 'I' or 'S'.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_gen_rq_formq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01rkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, wheret(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrowp.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, theta[*].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of wheret.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _wheret = utils._EngineCharScalarType(
        wheret, {'fun_name': fun_name, 'entity': 'wheret'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrowp = utils._EngineIntScalarType(
        nrowp, {'fun_name': fun_name, 'entity': 'nrowp'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _nrowp_py = utils._EngineIntScalarType.to_py(_nrowp)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(max(_m_py, _nrowp_py), _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _wheret_py = utils._EngineCharScalarType.to_py(_wheret)
    if _wheret_py in ('S', 's'):
        _theta_dim1 = _m_py
    else:
        _theta_dim1 = 1
    exp_shape_theta = (_theta_dim1,)
    _theta = utils._EngineComplex128ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=exp_shape_theta,
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _wheret,
        _m,
        _n,
        _nrowp,
        _a,
        _lda,
        _theta,
        _work,
        _errbuf,
        _ierr,
        _wheret._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s}, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'nrowp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lda >= max(m, nrowp).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrowp = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nrowp >= 0 and nrowp <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wheret = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wheret = \'I\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def real_nmf(m, n, k, a, w, h, seed, errtol, maxit):
    u"""
    Non-negative matrix factorization of real non-negative matrix.

    For full information please refer to the NAG Library document for
    f01sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01saf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A. Also the number of rows
        of the matrix W.

    n : int
        n, the number of columns of the matrix A. Also the number of
        columns of the matrix H.

    k : int
        k, the factorization rank.

    a : float, shape (`m`, `n`)
        The m by n non-negative matrix A.

    w : float, shape (`m`, `k`), modified in place
        On entry:

        - if `seed` <= 0, `w` should be set to an initial iterate for
          the non-negative matrix factor, W.

        - If `seed` >= 1, `w` need not be set. ``real_nmf`` will
          generate a random initial iterate.

        On exit: the non-negative matrix factor, W.

    h : float, shape (`k`, `n`), modified in place
        On entry:

        - if `seed` <= 0, `h` should be set to an initial iterate for
          the non-negative matrix factor, H.

        - If `seed` >= 1, `h` need not be set. ``real_nmf`` will
          generate a random initial iterate.

        On exit: the non-negative matrix factor, H.

    seed : int
        - If `seed` <= 0, the supplied values of W and H are used for
          the initial iterate.

        - If `seed` >= 1, the value of `seed` is used to seed a random
          number generator for the initial iterates W and H. See
          [Generating Random Initial Iterates] for further details.

    errtol : float
        The convergence tolerance for when the Hierarchical Alternating
        Least Squares iteration has reached a stationary point. If
        `errtol` <= 0.0, max(`m`, `n`)*sqrt(machine precision) is used.

    maxit : int
        Specifies the maximum number of iterations to be used. If
        `maxit` <= 0, 200 is used.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 2.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 3)
            On entry, `k` = *<value>*, `m` = *<value>* and `n` =
            *<value>*.

            Constraint: 1 <= `k` < min(`m`, `n`).

        (`errno` 8)
            An internal error occurred when generating initial values
            for `w` and `h`. Please contact NAG.

        (`errno` 9)
            On entry, one of more of the elements of `a`, `w` or `h`
            were negative.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 7)
            The function has failed to converge after *<value>*
            iterations. The factorization given by `w` and `h` may still
            be a good enough approximation to be useful. Alternatively
            an improved factorization may be obtained by increasing
            `maxit` or using different initial choices of `w` and `h`.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_nmf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01saft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, w[ldw, k].
        utils._BYREF_INT, # Input, ldw.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, h[ldh, n].
        utils._BYREF_INT, # Input, ldh.
        utils._BYREF_INT, # Input, seed.
        utils._BYREF_FLOAT64, # Input, errtol.
        utils._BYREF_INT, # Input, maxit.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (w, 'w'), (h, 'h')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldw_py = utils._EngineFloat64ArrayType.get_nd(w, {'fun_name': fun_name, 'entity': 'w'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldw = utils._EngineIntScalarType(
        _ldw_py, {'fun_name': fun_name, 'entity': '_ldw'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _seed = utils._EngineIntScalarType(
        seed, {'fun_name': fun_name, 'entity': 'seed'},
    ).to_ctypes()
    _seed_py = utils._EngineIntScalarType.to_py(_seed)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py, _k_py),
        mutual_sorder=_m_sorder,
        check_contents=_seed_py <= 0,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=_seed_py <= 0)
    _ldh_py = utils._EngineFloat64ArrayType.get_nd(h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldh = utils._EngineIntScalarType(
        _ldh_py, {'fun_name': fun_name, 'entity': '_ldh'},
    ).to_ctypes()
    _h_cls = utils._EngineFloat64ArrayType(
        h, {'fun_name': fun_name, 'entity': 'h'},
        exp_shape=(_k_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=_seed_py <= 0,
    )
    _h = _h_cls.to_ctypes(can_reference_contents=_seed_py <= 0)
    _errtol = utils._EngineFloat64ScalarType(
        errtol, {'fun_name': fun_name, 'entity': 'errtol'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _w,
        _ldw,
        _h,
        _ldh,
        _seed,
        _errtol,
        _maxit,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= k < min(m, n).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function has failed to converge after {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The factorization given by w and h may still be a good enough approximation',
                    'to be useful. Alternatively an improved factorization may be obtained by',
                    'increasing maxit or using different initial choices of w and h.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when generating initial values for w and h.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, one of more of the elements of a, w or h were negative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _h_cls.refresh_py(_h)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_nmf_rcomm(irevcm, m, n, k, w, h, ht, seed, errtol, comm):
    u"""
    Non-negative matrix factorization of real non-negative matrix
    (reverse communication).

    For full information please refer to the NAG Library document for
    f01sb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01sbf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must be set to 0.

    m : int
        m, the number of rows of the matrix A. Also the number of rows
        of the matrix W.

    n : int
        n, the number of columns of the matrix A. Also the number of
        columns of the matrix H.

    k : int
        k, the number of columns of the matrix W. Also, the number of
        rows of the matrix H. See [Choice of kk] for further details.

    w : float, shape (`m`, `k`), modified in place
        On initial entry:

        - if `seed` <= 0, `w` should be set to an initial iterate for
          the non-negative matrix factor, W.

        - If `seed` >= 1, `w` need not be set. ``real_nmf_rcomm`` will
          generate a random initial iterate.

        On intermediate exit: if `irevcm` = 1 or 2, `w` contains the
        current iterate of the m*k non-negative matrix W.

        On intermediate entry:

        - if `irevcm` = 3, `w` must contain AH^T, where H^T is stored in
          ht.

        - If `irevcm` = 0, 1 or 2, `w` must not be changed.

        On final exit: `w` contains the m*k non-negative matrix W.

    h : float, shape (`k`, `n`), modified in place
        On initial entry:

        - if `seed` <= 0, `h` should be set to an initial iterate for
          the non-negative matrix factor, H.

        - If `seed` >= 1, `h` need not be set. ``real_nmf_rcomm`` will
          generate a random initial iterate.

        On intermediate exit: if `irevcm` = 1, `h` contains the current
        iterate of the k*n non-negative matrix H.

        On intermediate entry: `h` must not be changed.

        On final exit: `h` contains the k*n non-negative matrix H.

    ht : float, shape (`n`, `k`), modified in place
        On initial entry: `ht` need not be set.

        On intermediate exit: if `irevcm` = 3, `ht` contains the n*k
        non-negative matrix H^T, which is required in order to form
        AH^T.

        On intermediate entry: if `irevcm` = 2, `ht` must contain A^TW.

        If `irevcm` = 0, 1 or 3, `ht` must not be changed.

        On final exit: `ht` is undefined.

    seed : int
        On initial entry:

        - if `seed` <= 0, the supplied values of W and H are used for
          the initial iterate.

        - If `seed` >= 1, the value of `seed` is used to seed a random
          number generator for the initial iterates W and H. See
          [Generating Random Initial Iterates] for further details.

    errtol : float
        The convergence tolerance for when the Hierarchical Alternating
        Least Squares iteration has reached a stationary point. If
        `errtol` <= 0.0, max(`m`, `n`)*sqrt(machine precision) is used.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``real_nmf_rcomm`` with `irevcm` unchanged. The
        value of `irevcm` should be interpreted as follows:

        `irevcm` = 1

          Indicates the start of a new iteration. No action is required
          by you, but `w` and `h` are available for printing, and a
          limit on the number of iterations can be applied.

        `irevcm` = 2

          Indicates that before re-entry to ``real_nmf_rcomm``, the
          product A^TW must be computed and stored in `ht`.

        `irevcm` = 3

          Indicates that before re-entry to ``real_nmf_rcomm``, the
          product AH^T must be computed and stored in `w`.

        On final exit: `irevcm` = 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On intermediate re-entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2 or 3.

        (`errno` 1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 2.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 4)
            On entry, `k` = *<value>*, `m` = *<value>* and `n` =
            *<value>*.

            Constraint: 1 <= `k` < min(`m`, `n`).

        (`errno` 8)
            An internal error occurred when generating initial values
            for `w` and `h`. Please contact NAG.

        (`errno` 9)
            On entry, one of more of the elements of `w` or `h` were
            negative.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_nmf_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01sbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, w[ldw, k].
        utils._BYREF_INT, # Input, ldw.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, h[ldh, n].
        utils._BYREF_INT, # Input, ldh.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ht[ldht, k].
        utils._BYREF_INT, # Input, ldht.
        utils._BYREF_INT, # Input, seed.
        utils._BYREF_FLOAT64, # Input, errtol.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[(2*m+n)*k+3].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[9].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((w, 'w'), (h, 'h'), (ht, 'ht')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _ldw_py = utils._EngineFloat64ArrayType.get_nd(w, {'fun_name': fun_name, 'entity': 'w'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldw = utils._EngineIntScalarType(
        _ldw_py, {'fun_name': fun_name, 'entity': '_ldw'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py, _k_py),
        mutual_sorder=_m_sorder,
    )
    _w = _w_cls.to_ctypes()
    _ldh_py = utils._EngineFloat64ArrayType.get_nd(h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldh = utils._EngineIntScalarType(
        _ldh_py, {'fun_name': fun_name, 'entity': '_ldh'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _h_cls = utils._EngineFloat64ArrayType(
        h, {'fun_name': fun_name, 'entity': 'h'},
        exp_shape=(_k_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _h = _h_cls.to_ctypes()
    _ldht_py = utils._EngineFloat64ArrayType.get_nd(ht, {'fun_name': fun_name, 'entity': 'ht'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldht = utils._EngineIntScalarType(
        _ldht_py, {'fun_name': fun_name, 'entity': '_ldht'},
    ).to_ctypes()
    _ht_cls = utils._EngineFloat64ArrayType(
        ht, {'fun_name': fun_name, 'entity': 'ht'},
        exp_shape=(_n_py, _k_py),
        mutual_sorder=_m_sorder,
    )
    _ht = _ht_cls.to_ctypes()
    _seed = utils._EngineIntScalarType(
        seed, {'fun_name': fun_name, 'entity': 'seed'},
    ).to_ctypes()
    _errtol = utils._EngineFloat64ScalarType(
        errtol, {'fun_name': fun_name, 'entity': 'errtol'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('comm', 'icomm'),
    )
    if _comm_needs_init:
        _comm = utils._EngineFloat64ArrayType.empty_ctypes((2*_m_py+_n_py)*_k_py+3)
        comm['comm'] = _comm
    else:
        _comm = comm['comm']
    if _comm_needs_init:
        _icomm = utils._EngineIntArrayType.empty_ctypes(9)
        comm['icomm'] = _icomm
    else:
        _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _m,
        _n,
        _k,
        _w,
        _ldw,
        _h,
        _ldh,
        _ht,
        _ldht,
        _seed,
        _errtol,
        _comm,
        _icomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate re-entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= k < min(m, n).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred when generating initial values for w and h.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, one of more of the elements of w or h were negative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _w_cls.refresh_py(_w)
        _h_cls.refresh_py(_h)
        _ht_cls.refresh_py(_ht)
        return_data = _irevcm_py
    _err_class.manage()
    return return_data

def dtrttp(uplo, n, a, ap):
    u"""
    Copies a real triangular matrix from full format to packed format.

    For full information please refer to the NAG Library document for
    f01va

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vaf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`)
        The triangular matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On exit: the n by n triangular matrix A, packed by columns.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.dtrttp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ap = _ap_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ap,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def ztrttp(uplo, n, a, ap):
    u"""
    Copies a complex triangular matrix from full format to packed
    format.

    For full information please refer to the NAG Library document for
    f01vb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vbf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        The triangular matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On exit: the n by n triangular matrix A, packed by columns.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.ztrttp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ap = _ap_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _ap,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def dtpttr(uplo, n, ap, a):
    u"""
    Copies a real triangular matrix from packed format to full format.

    For full information please refer to the NAG Library document for
    f01vc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vcf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    a : float, shape (`n`, `n`), modified in place
        On exit: the triangular matrix A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.dtpttr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _a,
        _lda,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def ztpttr(uplo, n, ap, a):
    u"""
    Copies a complex triangular matrix from packed format to full
    format.

    For full information please refer to the NAG Library document for
    f01vd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vdf.html

    Parameters
    ----------
    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    a : complex, shape (`n`, `n`), modified in place
        On exit: the triangular matrix A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.ztpttr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _a,
        _lda,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dtrttf(transr, uplo, n, a, ar):
    u"""
    Copies a real triangular matrix from full format to Rectangular Full
    Packed format.

    For full information please refer to the NAG Library document for
    f01ve

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vef.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'T'

          The transpose of the RFP representation of the matrix A is
          stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`)
        The triangular matrix A.

    ar : float, shape (`n`*(`n`+1)/2), modified in place
        On exit: the upper or lower n by n triangular matrix A (as
        specified by `uplo`) in either normal or transposed RFP format
        (as specified by `transr`). The storage format is described in
        [the F07 Introduction].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.dtrttf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01veft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ar_cls = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ar = _ar_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _a,
        _lda,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ar_cls.refresh_py(_ar)
        return_data = None
    _err_class.manage()
    return return_data

def ztrttf(transr, uplo, n, a, ar):
    u"""
    Copies a complex triangular matrix from full format to Rectangular
    Full Packed format.

    For full information please refer to the NAG Library document for
    f01vf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vff.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`)
        The triangular matrix A.

    ar : complex, shape (`n`*(`n`+1)/2), modified in place
        On exit: the upper or lower n by n triangular matrix A (as
        specified by `uplo`) in either normal or transposed RFP format
        (as specified by `transr`). The storage format is described in
        [the F07 Introduction].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.ztrttf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ar_cls = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ar = _ar_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _a,
        _lda,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ar_cls.refresh_py(_ar)
        return_data = None
    _err_class.manage()
    return return_data

def dtfttr(transr, uplo, n, ar, a):
    u"""
    Copies a real triangular matrix from Rectangular Full Packed format
    to full format.

    For full information please refer to the NAG Library document for
    f01vg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vgf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'T'

          The transpose of the RFP representation of the matrix A is
          stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : float, shape (`n`*(`n`+1)/2)
        The upper or lower n by n triangular matrix A (as specified by
        `uplo`) in either normal or transposed RFP format (as specified
        by `transr`). The storage format is described in [the F07
        Introduction].

    a : float, shape (`n`, `n`), modified in place
        On exit: the triangular matrix A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.dtfttr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ar[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _a,
        _lda,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def ztfttr(transr, uplo, n, ar, a):
    u"""
    Copies a complex triangular matrix from Rectangular Full Packed
    format to full format.

    For full information please refer to the NAG Library document for
    f01vh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vhf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : complex, shape (`n`*(`n`+1)/2)
        The upper or lower n by n triangular matrix A (as specified by
        `uplo`) in either normal or transposed RFP format (as specified
        by `transr`). The storage format is described in [the F07
        Introduction].

    a : complex, shape (`n`, `n`), modified in place
        On exit: the triangular matrix A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.ztfttr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ar[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _a,
        _lda,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dtpttf(transr, uplo, n, ap, ar):
    u"""
    Copies a real triangular matrix from packed format to Rectangular
    Full Packed format.

    For full information please refer to the NAG Library document for
    f01vj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vjf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'T'

          The transpose of the RFP representation of the matrix A is
          stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    ar : float, shape (`n`*(`n`+1)/2), modified in place
        On exit: the upper or lower n by n triangular matrix A (as
        specified by `uplo`) in either normal or transposed RFP format
        (as specified by `transr`). The storage format is described in
        [the F07 Introduction].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.dtpttf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ar_cls = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ar = _ar_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ap,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ar_cls.refresh_py(_ar)
        return_data = None
    _err_class.manage()
    return return_data

def ztpttf(transr, uplo, n, ap, ar):
    u"""
    Copies a complex triangular matrix from packed format to Rectangular
    Full Packed format.

    For full information please refer to the NAG Library document for
    f01vk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vkf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2)
        The n by n triangular matrix A, packed by columns.

    ar : complex, shape (`n`*(`n`+1)/2), modified in place
        On exit: the upper or lower n by n triangular matrix A (as
        specified by `uplo`) in either normal or transposed RFP format
        (as specified by `transr`). The storage format is described in
        [the F07 Introduction].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.ztpttf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Output, ar[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ar_cls = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ar = _ar_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ap,
        _ar,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ar_cls.refresh_py(_ar)
        return_data = None
    _err_class.manage()
    return return_data

def dtfttp(transr, uplo, n, ar, ap):
    u"""
    Copies a real triangular matrix from Rectangular Full Packed format
    to packed format.

    For full information please refer to the NAG Library document for
    f01vl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vlf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'T'

          The transpose of the RFP representation of the matrix A is
          stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : float, shape (`n`*(`n`+1)/2)
        The upper or lower n by n triangular matrix A (as specified by
        `uplo`) in either normal or transposed RFP format (as specified
        by `transr`). The storage format is described in [the F07
        Introduction].

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On exit: the n by n triangular matrix A, packed by columns.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.dtfttp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vlft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ar[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar = utils._EngineFloat64ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ap = _ap_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _ap,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def ztfttp(transr, uplo, n, ar, ap):
    u"""
    Copies a complex triangular matrix from Rectangular Full Packed
    format to packed format.

    For full information please refer to the NAG Library document for
    f01vm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01vmf.html

    Parameters
    ----------
    transr : str, length 1
        Specifies whether the normal RFP representation of A or its
        conjugate transpose is stored.

        `transr` = 'N'

          The RFP representation of the matrix A is stored.

        `transr` = 'C'

          The conjugate transpose of the RFP representation of the
          matrix A is stored.

    uplo : str, length 1
        Specifies whether A is upper or lower triangular.

        `uplo` = 'U'

          A is upper triangular.

        `uplo` = 'L'

          A is lower triangular.

    n : int
        n, the order of the matrix A.

    ar : complex, shape (`n`*(`n`+1)/2)
        The upper or lower n by n triangular matrix A (as specified by
        `uplo`) in either normal or transposed RFP format (as specified
        by `transr`). The storage format is described in [the F07
        Introduction].

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On exit: the n by n triangular matrix A, packed by columns.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `transr`.

            Constraint: `transr` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.ztfttp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01vmft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, transr(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ar[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Output, ap[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _transr = utils._EngineCharScalarType(
        transr, {'fun_name': fun_name, 'entity': 'transr'}, exp_chrlen=1,
    ).to_ctypes()
    _transr_py = utils._EngineCharScalarType.to_py(_transr)
    if _transr_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('transr in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='transr',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ar = utils._EngineComplex128ArrayType(
        ar, {'fun_name': fun_name, 'entity': 'ar'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _ap = _ap_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _transr,
        _uplo,
        _n,
        _ar,
        _ap,
        _info,
        _transr._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def real_tri_pack(job, uplo, diag, n, a, b):
    u"""
    Convert real matrix between packed triangular and square storage
    formats.

    For full information please refer to the NAG Library document for
    f01za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01zaf.html

    Parameters
    ----------
    job : str, length 1
        Specifies whether the triangular matrix is to be packed or
        unpacked.

        `job` = 'P' (Pack)

          The matrix is to be packed into array `b`.

        `job` = 'U' (Unpack)

          The matrix is to be unpacked into array `a`.

    uplo : str, length 1
        Specifies the type of the matrix to be copied

        `uplo` = 'L' (Lower)

          The matrix is lower triangular. In this case the packed vector
          holds, or will hold on exit, the matrix elements in the
          following order:
          (1,1),(2,1),...,(`n`,1),(2,2),(3,2),...,(`n`,2), etc..

        `uplo` = 'U' (Upper)

          The matrix is upper triangular. In this case the packed vector
          holds, or will hold on exit, the matrix elements in the
          following order: (1,1), (1,2), (2,2), (1,3), (2,3), (3,3),
          (1,4), etc..

    diag : str, length 1
        Must specify whether the diagonal elements of the matrix are to
        be copied.

        `diag` = 'B' (Blank)

          The diagonal elements of the matrix are not referenced and not
          copied.

        `diag` = 'U' (Unit diagonal)

          The diagonal elements of the matrix are not referenced, but
          are assumed all to be unity, and are copied as such.

        `diag` = 'N' (Non-unit diagonal)

          The diagonal elements of the matrix are referenced and copied.

    n : int
        n, the number of rows and columns of the triangular matrix.

    a : float, shape (`n`, `n`), modified in place
        On entry: if `job` = 'P', the leading `n` by `n` part of `a`
        must contain the matrix to be copied, stored in unpacked form,
        in the upper or lower triangle depending on argument `uplo`. The
        opposite triangle of `a` is not referenced and need not be
        assigned.

        On exit: if `job` = 'U', the leading `n` by `n` part of array
        `a` contains the copied matrix, stored in unpacked form, in the
        upper or lower triangle depending on argument `uplo`. The
        opposite triangle of `a` is not referenced.

    b : float, shape ((`n`*(`n`+1))/2), modified in place
        On entry: if `job` = 'U', `b` must contain the triangular matrix
        packed by column.

        On exit: if `job` = 'P', `b` contains the triangular matrix
        packed by column.

        Note that `b` must have space for the diagonal elements of the
        matrix, even if these are not stored.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `job` != 'P' or 'U': `job` = *<value>*.

        (`errno` 2)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` 3)
            On entry, `diag` != 'N', 'U' or 'B': `diag` = *<value>*.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_tri_pack'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01zaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[(n*(n+1))//2].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=((_n_py*(_n_py+1))//2,),
    )
    _b = _b_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _job,
        _uplo,
        _diag,
        _n,
        _a,
        _lda,
        _b,
        _errbuf,
        _ierr,
        _job._length_,
        _uplo._length_,
        _diag._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, job != \'P\' or \'U\': job = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo != \'U\' or \'L\': uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, diag != \'N\', \'U\' or \'B\': diag = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def complex_tri_pack(job, uplo, diag, n, a, b):
    u"""
    Convert complex matrix between packed triangular and square storage
    formats.

    For full information please refer to the NAG Library document for
    f01zb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01zbf.html

    Parameters
    ----------
    job : str, length 1
        Specifies whether the triangular matrix is to be packed or
        unpacked.

        `job` = 'P' (Pack)

          The matrix is to be packed into array `b`.

        `job` = 'U' (Unpack)

          The matrix is to be unpacked into array `a`.

    uplo : str, length 1
        Specifies the type of the matrix to be copied

        `uplo` = 'L' (Lower)

          The matrix is lower triangular. In this case the packed vector
          holds, or will hold on exit, the matrix elements in the
          following order:
          (1,1),(2,1),...,(`n`,1),(2,2),(3,2),...,(`n`,2), etc..

        `uplo` = 'U' (Upper)

          The matrix is upper triangular. In this case the packed vector
          holds, or will hold on exit, the matrix elements in the
          following order: (1,1), (1,2), (2,2), (1,3), (2,3), (3,3),
          (1,4), etc..

    diag : str, length 1
        Must specify whether the diagonal elements of the matrix are to
        be copied.

        `diag` = 'B' (Blank)

          The diagonal elements of the matrix are not referenced and not
          copied.

        `diag` = 'U' (Unit diagonal)

          The diagonal elements of the matrix are not referenced, but
          are assumed all to be unity, and are copied as such.

        `diag` = 'N' (Non-unit diagonal)

          The diagonal elements of the matrix are referenced and copied.

    n : int
        `n` must specify the number of rows and columns of the
        triangular matrix.

    a : complex, shape (`n`, `n`), modified in place
        On entry: if `job` = 'P', the leading `n` by `n` part of `a`
        must contain the matrix to be copied, stored in unpacked form,
        in the upper or lower triangle depending on argument `uplo`. The
        opposite triangle of `a` is not referenced and need not be
        assigned.

        On exit: if `job` = 'U', the leading `n` by `n` part of array
        `a` contains the copied matrix, stored in unpacked form, in the
        upper or lower triangle depending on argument `uplo`. The
        opposite triangle of `a` is not referenced.

    b : complex, shape ((`n`*(`n`+1))/2), modified in place
        On entry: if `job` = 'U', `b` must contain the triangular matrix
        packed by column.

        On exit: if `job` = 'P', `b` contains the triangular matrix
        packed by column.

        Note that `b` must have space for the diagonal elements of the
        matrix, even if these are not stored.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `job` != 'P' or 'U': `job` = *<value>*.

        (`errno` 2)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` 3)
            On entry, `diag` = *<value>*.

            Constraint: `diag` = 'N', 'U' or 'B'.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_tri_pack'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01zbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, diag(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[(n*(n+1))//2].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of diag.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _diag = utils._EngineCharScalarType(
        diag, {'fun_name': fun_name, 'entity': 'diag'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=((_n_py*(_n_py+1))//2,),
    )
    _b = _b_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _job,
        _uplo,
        _diag,
        _n,
        _a,
        _lda,
        _b,
        _errbuf,
        _ierr,
        _job._length_,
        _uplo._length_,
        _diag._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, job != \'P\' or \'U\': job = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo != \'U\' or \'L\': uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, diag = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: diag = \'N\', \'U\' or \'B\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def real_band_pack(job, m, n, kl, ku, a, b):
    u"""
    Convert real matrix between packed banded and rectangular storage
    formats.

    For full information please refer to the NAG Library document for
    f01zc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01zcf.html

    Parameters
    ----------
    job : str, length 1
        Specifies whether the band matrix is to be packed or unpacked.

        `job` = 'P' (Pack)

          The band matrix is to be packed into array `b`.

        `job` = 'U' (Unpack)

          The band matrix is to be unpacked into array `a`.

    m : int
        m and n, the number of rows and columns of the band matrix,
        respectively.

    n : int
        m and n, the number of rows and columns of the band matrix,
        respectively.

    kl : int
        k_l, the number of subdiagonals of the band matrix.

    ku : int
        k_u, the number of superdiagonals of the band matrix.

    a : float, shape (`m`, `n`), modified in place
        On entry: if `job` = 'P', the leading m by n part of `a` must
        contain the band matrix stored in unpacked form. Elements of the
        array that lie outside the banded part of the matrix are not
        referenced and need not be assigned.

        On exit: if `job` = 'U', the leading m by n part of `a` contains
        the band matrix stored in unpacked form. Elements of the leading
        m by n part of `a` that are not within the banded part of the
        matrix are assigned the value zero.

    b : float, shape ((`kl`+`ku`+1), min(`m`+`ku`, `n`)), modified in
    place
        On entry: if `job` = 'U', `b` must contain the band matrix in
        packed form, in the leading (k_l+k_u+1) by min(m+k_u, n) part of
        the array. The matrix is packed column by column, with the
        leading diagonal of the matrix in row (k_u+1) of `b`, the first
        superdiagonal starting at position 2 in row k_u, the first
        subdiagonal starting at position 1 in row (k_u+2), and so on.
        Elements of `b` that are not needed to store the band matrix,
        for instance the leading k_u by k_u triangle, are not referenced
        and need not be assigned.

        On exit: if `job` = 'P', `b` contains the band matrix stored in
        packed form. Elements of `b` that are not needed to store the
        band matrix are not referenced.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `job` = *<value>*.

            Constraint: `job` = 'P' or 'U'.

        (`errno` 2)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 3)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 5)
            On entry, ldb = *<value>*, `kl` = *<value>* and `ku` =
            *<value>*.

            Constraint: ldb >= `kl`+`ku`+1.

        (`errno` 6)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 6)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.real_band_pack'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01zcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, min(m+ku, n)].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=((_kl_py+_ku_py+1), min(_m_py+_ku_py, _n_py)),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _job,
        _m,
        _n,
        _kl,
        _ku,
        _a,
        _lda,
        _b,
        _ldb,
        _errbuf,
        _ierr,
        _job._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, job = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: job = \'P\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s}, kl = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ldb >= kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def complex_band_pack(job, m, n, kl, ku, a, b):
    u"""
    Convert complex matrix between packed banded and rectangular storage
    formats.

    For full information please refer to the NAG Library document for
    f01zd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f01/f01zdf.html

    Parameters
    ----------
    job : str, length 1
        Specifies whether the band matrix is to be packed or unpacked.

        `job` = 'P' (Pack)

          The band matrix is to be packed into array `b`.

        `job` = 'U' (Unpack)

          The band matrix is to be unpacked into array `a`.

    m : int
        m and n, the number of rows and columns of the band matrix,
        respectively.

    n : int
        m and n, the number of rows and columns of the band matrix,
        respectively.

    kl : int
        k_l, the number of subdiagonals of the band matrix.

    ku : int
        k_u, the number of superdiagonals of the band matrix.

    a : complex, shape (`m`, `n`), modified in place
        On entry: if `job` = 'P', the leading m by n part of `a` must
        contain the band matrix stored in unpacked form. Elements of the
        array that lie outside the banded part of the matrix are not
        referenced and need not be assigned.

        On exit: if `job` = 'U', the leading m by n part of `a` contains
        the band matrix stored in unpacked form. Elements of the leading
        m by n part of `a` that are not within the banded part of the
        matrix are assigned the value zero.

    b : complex, shape ((`kl`+`ku`+1), min(`m`+`ku`, `n`)), modified in
    place
        On entry: if `job` = 'U', `b` must contain the band matrix in
        packed form, in the leading (k_l+k_u+1) by min(m+k_u, n) part of
        the array. The matrix is packed column by column, with the
        leading diagonal of the matrix in row (k_u+1) of `b`, the first
        superdiagonal starting at position 2 in row k_u, the first
        subdiagonal starting at position 1 in row (k_u+2), and so on.
        Elements of `b` that are not needed to store the band matrix,
        for instance the leading k_u by k_u triangle, are not referenced
        and need not be assigned.

        On exit: if `job` = 'P', `b` contains the band matrix stored in
        packed form. Elements of `b` that are not needed to store the
        band matrix are not referenced.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `job` = *<value>*.

            Constraint: `job` = 'P' or 'U'.

        (`errno` 2)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 3)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 5)
            On entry, ldb = *<value>*, `kl` = *<value>* and `ku` =
            *<value>*.

            Constraint: ldb >= `kl`+`ku`+1.

        (`errno` 6)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 6)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.matop.complex_band_pack'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f01zdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, min(m+ku, n)].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=((_kl_py+_ku_py+1), min(_m_py+_ku_py, _n_py)),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _job,
        _m,
        _n,
        _kl,
        _ku,
        _a,
        _lda,
        _b,
        _ldb,
        _errbuf,
        _ierr,
        _job._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, job = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: job = \'P\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s}, kl = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ldb >= kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data
