# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `mesh` Chapter.

``mesh`` - Mesh Generation

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with automatic mesh generation

- with line segments, over the boundary of a closed two-dimensional
  connected polygonal domain;

- with triangles, over a given two-dimensional region using only its
  boundary mesh.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def dim2_gen_inc(nvb, nvmax, nedge, edge, coor, conn, bspace, smooth, coef, power, itrace, io_manager=None):
    u"""
    Generates a two-dimensional mesh using a simple incremental method.

    For full information please refer to the NAG Library document for
    d06aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06aaf.html

    Parameters
    ----------
    nvb : int
        The number of vertices in the input boundary mesh.

    nvmax : int
        The maximum number of vertices in the mesh to be generated.

    nedge : int
        The number of boundary edges in the input mesh.

    edge : int, shape (3, `nedge`)
        The specification of the boundary edges. `edge`[0,j-1] and
        `edge`[1,j-1] contain the vertex numbers of the two end points
        of the jth boundary edge. `edge`[2,j-1] is a user-supplied tag
        for the jth boundary edge and is not used by ``dim2_gen_inc``.

    coor : float, shape (2, `nvmax`), modified in place
        On entry: `coor`[0,i-1] contains the x coordinate of the ith
        input boundary mesh vertex; while `coor`[1,i-1] contains the
        corresponding y coordinate, for i = 1,...,`nvb`.

        On exit: `coor`[0,i-1] will contain the x coordinate of the
        (i-`nvb`)th generated interior mesh vertex; while `coor`[1,i-1]
        will contain the corresponding y coordinate, for i =
        `nvb`+1,...,`nv`. The remaining elements are unchanged.

    conn : int, shape (3, 2*(`nvmax`-1)), modified in place
        On exit: the connectivity of the mesh between triangles and
        vertices. For each triangle j, `conn`[i-1,j-1] gives the indices
        of its three vertices (in anticlockwise order), for j =
        1,...,`nelt`, for i = 1,2,...,3.

    bspace : float, shape (`nvb`)
        The desired mesh spacing (triangle diameter, which is the length
        of the longer edge of the triangle) near the boundary vertices.

    smooth : bool
        Indicates whether or not mesh smoothing should be performed.

        If `smooth` = True, the smoothing is performed; otherwise no
        smoothing is performed.

    coef : float
        The coefficient in the stopping criteria for the generation of
        interior vertices. This argument controls the triangle density
        and the number of triangles generated is in O(`coef`^2). The
        mesh will be finer if `coef` is greater than 0.7165 and 0.75 is
        a good value.

    power : float
        Controls the rate of change of the mesh size during the
        generation of interior vertices. The smaller the value of
        `power`, the faster the decrease in element size away from the
        boundary.

    itrace : int
        The level of trace information required from ``dim2_gen_inc``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Output from the meshing solver is printed. This output
          contains details of the vertices and triangles generated by
          the process.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv : int
        The total number of vertices in the output mesh (including both
        boundary and interior vertices). If `nvb` = `nvmax`, no interior
        vertices will be generated and `nv` = `nvb`.

    nelt : int
        The number of triangular elements in the mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nvb` = *<value>* and `nvmax` = *<value>*.

            Constraint: 3 <= `nvb` <= `nvmax`.

        (`errno` 1)
            On entry, `nedge` = *<value>*.

            Constraint: `nedge` >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nvb` = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= `nvb`.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `bspace`[I-1] = *<value>* and I = *<value>*.

            Constraint: `bspace`[I-1] > 0.0.

        (`errno` 1)
            On entry, `power` = *<value>*.

            Constraint: `power` <= 10.0.

        (`errno` 1)
            On entry, `power` = *<value>*.

            Constraint: `power` >= 0.1.

        (`errno` 2)
            An error has occurred during the generation of the interior
            mesh. Check the definition of the boundary (arguments `coor`
            and `edge`) as well as the orientation of the boundary
            (especially in the case of a multiple connected component
            boundary). Setting `itrace` > 0 may provide more details.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_gen_inc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, nvb.
        utils._BYREF_INT, # Input, nvmax.
        utils._BYREF_INT, # Input, nedge.
        utils._EngineIntArrayType.c_type, # Input, edge[3, nedge].
        utils._BYREF_INT, # Output, nv.
        utils._BYREF_INT, # Output, nelt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, coor[2, nvmax].
        utils._EngineIntArrayType.c_type, # Output, conn[3, 2*(nvmax-1)].
        utils._EngineFloat64ArrayType.c_type, # Input, bspace[nvb].
        utils._BYREF_LOGICAL, # Input, smooth.
        utils._BYREF_FLOAT64, # Input, coef.
        utils._BYREF_FLOAT64, # Input, power.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((edge, 'edge'), (coor, 'coor'), (conn, 'conn')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _nvb = utils._EngineIntScalarType(
        nvb, {'fun_name': fun_name, 'entity': 'nvb'},
    ).to_ctypes()
    _nvmax = utils._EngineIntScalarType(
        nvmax, {'fun_name': fun_name, 'entity': 'nvmax'},
    ).to_ctypes()
    _nedge = utils._EngineIntScalarType(
        nedge, {'fun_name': fun_name, 'entity': 'nedge'},
    ).to_ctypes()
    _nedge_py = utils._EngineIntScalarType.to_py(_nedge)
    _edge = utils._EngineIntArrayType(
        edge, {'fun_name': fun_name, 'entity': 'edge'},
        exp_shape=(3, _nedge_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nv = utils._EngineIntScalarType.empty_ctypes()
    _nelt = utils._EngineIntScalarType.empty_ctypes()
    _nvmax_py = utils._EngineIntScalarType.to_py(_nvmax)
    _coor_cls = utils._EngineFloat64ArrayType(
        coor, {'fun_name': fun_name, 'entity': 'coor'},
        exp_shape=(2, _nvmax_py),
        mutual_sorder=_m_sorder,
    )
    _coor = _coor_cls.to_ctypes()
    _conn_cls = utils._EngineIntArrayType(
        conn, {'fun_name': fun_name, 'entity': 'conn'},
        exp_shape=(3, 2*(_nvmax_py-1)),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _conn = _conn_cls.to_ctypes(can_reference_contents=False)
    _nvb_py = utils._EngineIntScalarType.to_py(_nvb)
    _bspace = utils._EngineFloat64ArrayType(
        bspace, {'fun_name': fun_name, 'entity': 'bspace'},
        exp_shape=(_nvb_py,),
    ).to_ctypes()
    _smooth = utils._EngineLogicalScalarType(
        smooth, {'fun_name': fun_name, 'entity': 'smooth'},
    ).to_ctypes()
    _coef = utils._EngineFloat64ScalarType(
        coef, {'fun_name': fun_name, 'entity': 'coef'},
    ).to_ctypes()
    _power = utils._EngineFloat64ScalarType(
        power, {'fun_name': fun_name, 'entity': 'power'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _nvb,
        _nvmax,
        _nedge,
        _edge,
        _nv,
        _nelt,
        _coor,
        _conn,
        _bspace,
        _smooth,
        _coef,
        _power,
        _itrace,
        _nadv,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvb = {:s} and nvmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 3 <= nvb <= nvmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edge(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nvb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edge(I,J) >= 1 and edge(I,J) <= nvb.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of the edge J have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, bspace[I-1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: bspace[I-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, power = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: power <= 10.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, power = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: power >= 0.1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the generation of the interior mesh.',
                    'Check the definition of the boundary (arguments coor and edge) as well as',
                    'the orientation of the boundary (especially in the case of a multiple',
                    'connected component boundary). Setting itrace > 0 may provide more details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coor_cls.refresh_py(_coor)
        _conn_cls.refresh_py(_conn)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        _nelt_py = utils._EngineIntScalarType.to_py(_nelt)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nv', 'nelt'],
        )(
            _nv_py,
            _nelt_py,
        )
    _err_class.manage()
    return return_data

def dim2_gen_delaunay(nvb, nvint, nvmax, nedge, edge, coor, conn, weight, npropa, itrace, io_manager=None):
    u"""
    Generates a two-dimensional mesh using a Delaunay--Voronoi process.

    For full information please refer to the NAG Library document for
    d06ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06abf.html

    Parameters
    ----------
    nvb : int
        The number of vertices in the input boundary mesh.

    nvint : int
        The number of fixed interior mesh vertices to which a weight
        will be applied.

    nvmax : int
        The maximum number of vertices in the mesh to be generated.

    nedge : int
        The number of boundary edges in the input mesh.

    edge : int, shape (3, `nedge`)
        The specification of the boundary edges. `edge`[0,j-1] and
        `edge`[1,j-1] contain the vertex numbers of the two end points
        of the jth boundary edge. `edge`[2,j-1] is a user-supplied tag
        for the jth boundary edge and is not used by
        ``dim2_gen_delaunay``.

    coor : float, shape (2, `nvmax`), modified in place
        On entry: `coor`[0,i-1] contains the x coordinate of the ith
        input boundary mesh vertex, for i = 1,...,`nvb`. `coor`[0,i-1]
        contains the x coordinate of the (i-`nvb`)th fixed interior
        vertex, for i = `nvb`+1,...,`nvb`+`nvint`. For boundary and
        interior vertices, `coor`[1,i-1] contains the corresponding y
        coordinate, for i = 1,2,...,`nvb`+`nvint`.

        On exit: `coor`[0,i-1] will contain the x coordinate of the
        (i-`nvb`-`nvint`)th generated interior mesh vertex, for i =
        `nvb`+`nvint`+1,...,`nv`; while `coor`[1,i-1] will contain the
        corresponding y coordinate. The remaining elements are
        unchanged.

    conn : int, shape (3, 2*`nvmax`+5), modified in place
        On exit: the connectivity of the mesh between triangles and
        vertices. For each triangle j, `conn`[i-1,j-1] gives the indices
        of its three vertices (in anticlockwise order), for j =
        1,...,`nelt`, for i = 1,2,...,3.

    weight : float, shape (`nvint`)
        The weight of fixed interior vertices. It is the diameter of
        triangles (length of the longer edge) created around each of the
        given interior vertices.

    npropa : int
        The propagation type and coefficient, the argument `npropa` is
        used when the internal points are created. They are distributed
        in a geometric manner if `npropa` is positive and in an
        arithmetic manner if it is negative. For more details see
        [Further Comments].

    itrace : int
        The level of trace information required from
        ``dim2_gen_delaunay``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Output from the meshing solver is printed. This output
          contains details of the vertices and triangles generated by
          the process.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv : int
        The total number of vertices in the output mesh (including both
        boundary and interior vertices). If `nvb`+`nvint` = `nvmax`, no
        interior vertices will be generated and `nv` = `nvmax`.

    nelt : int
        The number of triangular elements in the mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nvb` = *<value>*, `nvint` = *<value>* and `nvmax`
            = *<value>*.

            Constraint: `nvmax` >= `nvb`+`nvint`.

        (`errno` 1)
            On entry, `nedge` = *<value>*.

            Constraint: `nedge` >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nvb` = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= `nvb`.

        (`errno` 1)
            On entry, `nvb` = *<value>*.

            Constraint: `nvb` >= 3.

        (`errno` 1)
            On entry, `nvint` = *<value>*.

            Constraint: `nvint` >= 0.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `weight`[I-1] = *<value>* and I = *<value>*.

            Constraint: `weight`[I-1] > 0.0.

        (`errno` 1)
            On entry, `npropa` = 0.

        (`errno` 2)
            An error has occurred during the generation of the interior
            mesh. Check the definition of the boundary (arguments `coor`
            and `edge`) as well as the orientation of the boundary
            (especially in the case of a multiple connected component
            boundary). Setting `itrace` > 0 may provide more details.

        (`errno` 3)
            An error has occurred during the generation of the boundary
            mesh. It appears that `nvmax` is not large enough: `nvmax` =
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_gen_delaunay'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, nvb.
        utils._BYREF_INT, # Input, nvint.
        utils._BYREF_INT, # Input, nvmax.
        utils._BYREF_INT, # Input, nedge.
        utils._EngineIntArrayType.c_type, # Input, edge[3, nedge].
        utils._BYREF_INT, # Output, nv.
        utils._BYREF_INT, # Output, nelt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, coor[2, nvmax].
        utils._EngineIntArrayType.c_type, # Output, conn[3, 2*nvmax+5].
        utils._EngineFloat64ArrayType.c_type, # Input, weight[nvint].
        utils._BYREF_INT, # Input, npropa.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((edge, 'edge'), (coor, 'coor'), (conn, 'conn')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _nvb = utils._EngineIntScalarType(
        nvb, {'fun_name': fun_name, 'entity': 'nvb'},
    ).to_ctypes()
    _nvint = utils._EngineIntScalarType(
        nvint, {'fun_name': fun_name, 'entity': 'nvint'},
    ).to_ctypes()
    _nvmax = utils._EngineIntScalarType(
        nvmax, {'fun_name': fun_name, 'entity': 'nvmax'},
    ).to_ctypes()
    _nedge = utils._EngineIntScalarType(
        nedge, {'fun_name': fun_name, 'entity': 'nedge'},
    ).to_ctypes()
    _nedge_py = utils._EngineIntScalarType.to_py(_nedge)
    _edge = utils._EngineIntArrayType(
        edge, {'fun_name': fun_name, 'entity': 'edge'},
        exp_shape=(3, _nedge_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nv = utils._EngineIntScalarType.empty_ctypes()
    _nelt = utils._EngineIntScalarType.empty_ctypes()
    _nvmax_py = utils._EngineIntScalarType.to_py(_nvmax)
    _coor_cls = utils._EngineFloat64ArrayType(
        coor, {'fun_name': fun_name, 'entity': 'coor'},
        exp_shape=(2, _nvmax_py),
        mutual_sorder=_m_sorder,
    )
    _coor = _coor_cls.to_ctypes()
    _conn_cls = utils._EngineIntArrayType(
        conn, {'fun_name': fun_name, 'entity': 'conn'},
        exp_shape=(3, 2*_nvmax_py+5),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _conn = _conn_cls.to_ctypes(can_reference_contents=False)
    _nvint_py = utils._EngineIntScalarType.to_py(_nvint)
    _weight = utils._EngineFloat64ArrayType(
        weight, {'fun_name': fun_name, 'entity': 'weight'},
        exp_shape=(_nvint_py,),
    ).to_ctypes()
    _npropa = utils._EngineIntScalarType(
        npropa, {'fun_name': fun_name, 'entity': 'npropa'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _nvb,
        _nvint,
        _nvmax,
        _nedge,
        _edge,
        _nv,
        _nelt,
        _coor,
        _conn,
        _weight,
        _npropa,
        _itrace,
        _nadv,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvb = {:s}, nvint = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and nvmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: nvmax >= nvb+nvint.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edge(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nvb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edge(I,J) >= 1 and edge(I,J) <= nvb.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvb >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvint = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvint >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of the edge J have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, weight[I-1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: weight[I-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npropa = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the generation of the interior mesh.',
                    'Check the definition of the boundary (arguments coor and edge) as well as',
                    'the orientation of the boundary (especially in the case of a multiple',
                    'connected component boundary). Setting itrace > 0 may provide more details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the generation of the boundary mesh.',
                    'It appears that nvmax is not large enough:',
                    'nvmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coor_cls.refresh_py(_coor)
        _conn_cls.refresh_py(_conn)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        _nelt_py = utils._EngineIntScalarType.to_py(_nelt)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nv', 'nelt'],
        )(
            _nv_py,
            _nelt_py,
        )
    _err_class.manage()
    return return_data

def dim2_gen_front(nvb, nvint, nvmax, nedge, edge, coor, conn, weight, itrace, io_manager=None):
    u"""
    Generates a two-dimensional mesh using an Advancing-front method.

    For full information please refer to the NAG Library document for
    d06ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06acf.html

    Parameters
    ----------
    nvb : int
        The number of vertices in the input boundary mesh.

    nvint : int
        The number of fixed interior mesh vertices to which a weight
        will be applied.

    nvmax : int
        The maximum number of vertices in the mesh to be generated.

    nedge : int
        The number of boundary edges in the input mesh.

    edge : int, shape (3, `nedge`)
        The specification of the boundary edges. `edge`[0,j-1] and
        `edge`[1,j-1] contain the vertex numbers of the two end points
        of the jth boundary edge. `edge`[2,j-1] is a user-supplied tag
        for the jth boundary edge and is not used by ``dim2_gen_front``.

    coor : float, shape (2, `nvmax`), modified in place
        On entry: `coor`[0,i-1] contains the x coordinate of the ith
        input boundary mesh vertex, for i = 1,...,`nvb`. `coor`[0,i-1]
        contains the x coordinate of the (i-`nvb`)th fixed interior
        vertex, for i = `nvb`+1,...,`nvb`+`nvint`. For boundary and
        interior vertices, `coor`[1,i-1] contains the corresponding y
        coordinate, for i = 1,2,...,`nvb`+`nvint`.

        On exit: `coor`[0,i-1] will contain the x coordinate of the
        (i-`nvb`-`nvint`)th generated interior mesh vertex, for i =
        `nvb`+`nvint`+1,...,`nv`; while `coor`[1,i-1] will contain the
        corresponding y coordinate. The remaining elements are
        unchanged.

    conn : int, shape (3, 2*`nvmax`+5), modified in place
        On exit: the connectivity of the mesh between triangles and
        vertices. For each triangle j, `conn`[i-1,j-1] gives the indices
        of its three vertices (in anticlockwise order), for j =
        1,...,`nelt`, for i = 1,2,...,3.

    weight : float, shape (`nvint`)
        The weight of fixed interior vertices. It is the diameter of
        triangles (length of the longer edge) created around each of the
        given interior vertices.

    itrace : int
        The level of trace information required from ``dim2_gen_front``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Output from the meshing solver is printed. This output
          contains details of the vertices and triangles generated by
          the process.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv : int
        The total number of vertices in the output mesh (including both
        boundary and interior vertices). If `nvb`+`nvint` = `nvmax`, no
        interior vertices will be generated and `nv` = `nvmax`.

    nelt : int
        The number of triangular elements in the mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nv` = *<value>*, `nvint` = *<value>* and `nvmax`
            = *<value>*.

            Constraint: `nvb`+`nvint` <= `nvmax`.

        (`errno` 1)
            On entry, `nedge` = *<value>*.

            Constraint: `nedge` >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nvb` = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= `nvb`.

        (`errno` 1)
            On entry, `nvb` = *<value>*.

            Constraint: `nvb` >= 3.

        (`errno` 1)
            On entry, `nvint` = *<value>*.

            Constraint: `nvint` >= 0.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `weight`[I-1] = *<value>* and I = *<value>*.

            Constraint: `weight`[I-1] > 0.0.

        (`errno` 2)
            An error has occurred during the generation of the interior
            mesh. Check the definition of the boundary (arguments `coor`
            and `edge`) as well as the orientation of the boundary
            (especially in the case of a multiple connected component
            boundary). Setting `itrace` > 0 may provide more details.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_gen_front'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, nvb.
        utils._BYREF_INT, # Input, nvint.
        utils._BYREF_INT, # Input, nvmax.
        utils._BYREF_INT, # Input, nedge.
        utils._EngineIntArrayType.c_type, # Input, edge[3, nedge].
        utils._BYREF_INT, # Output, nv.
        utils._BYREF_INT, # Output, nelt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, coor[2, nvmax].
        utils._EngineIntArrayType.c_type, # Output, conn[3, 2*nvmax+5].
        utils._EngineFloat64ArrayType.c_type, # Input, weight[nvint].
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((edge, 'edge'), (coor, 'coor'), (conn, 'conn')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _nvb = utils._EngineIntScalarType(
        nvb, {'fun_name': fun_name, 'entity': 'nvb'},
    ).to_ctypes()
    _nvint = utils._EngineIntScalarType(
        nvint, {'fun_name': fun_name, 'entity': 'nvint'},
    ).to_ctypes()
    _nvmax = utils._EngineIntScalarType(
        nvmax, {'fun_name': fun_name, 'entity': 'nvmax'},
    ).to_ctypes()
    _nedge = utils._EngineIntScalarType(
        nedge, {'fun_name': fun_name, 'entity': 'nedge'},
    ).to_ctypes()
    _nedge_py = utils._EngineIntScalarType.to_py(_nedge)
    _edge = utils._EngineIntArrayType(
        edge, {'fun_name': fun_name, 'entity': 'edge'},
        exp_shape=(3, _nedge_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nv = utils._EngineIntScalarType.empty_ctypes()
    _nelt = utils._EngineIntScalarType.empty_ctypes()
    _nvmax_py = utils._EngineIntScalarType.to_py(_nvmax)
    _coor_cls = utils._EngineFloat64ArrayType(
        coor, {'fun_name': fun_name, 'entity': 'coor'},
        exp_shape=(2, _nvmax_py),
        mutual_sorder=_m_sorder,
    )
    _coor = _coor_cls.to_ctypes()
    _conn_cls = utils._EngineIntArrayType(
        conn, {'fun_name': fun_name, 'entity': 'conn'},
        exp_shape=(3, 2*_nvmax_py+5),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _conn = _conn_cls.to_ctypes(can_reference_contents=False)
    _nvint_py = utils._EngineIntScalarType.to_py(_nvint)
    _weight = utils._EngineFloat64ArrayType(
        weight, {'fun_name': fun_name, 'entity': 'weight'},
        exp_shape=(_nvint_py,),
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _nvb,
        _nvint,
        _nvmax,
        _nedge,
        _edge,
        _nv,
        _nelt,
        _coor,
        _conn,
        _weight,
        _itrace,
        _nadv,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}, nvint = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and nvmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: nvb+nvint <= nvmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edge(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nvb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edge(I,J) >= 1 and edge(I,J) <= nvb.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvb >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvint = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvint >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of the edge J have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, weight[I-1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: weight[I-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the generation of the interior mesh.',
                    'Check the definition of the boundary (arguments coor and edge) as well as',
                    'the orientation of the boundary (especially in the case of a multiple',
                    'connected component boundary). Setting itrace > 0 may provide more details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coor_cls.refresh_py(_coor)
        _conn_cls.refresh_py(_conn)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        _nelt_py = utils._EngineIntScalarType.to_py(_nelt)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nv', 'nelt'],
        )(
            _nv_py,
            _nelt_py,
        )
    _err_class.manage()
    return return_data

def dim2_gen_boundary(nlines, coorch, lined, fbnd, crus, sdcrus, rate, ncomp, nlcomp, lcomp, nvmax, nedmx, coor, edge, itrace, data=None, io_manager=None):
    u"""
    Generates a boundary mesh.

    For full information please refer to the NAG Library document for
    d06ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06baf.html

    Parameters
    ----------
    nlines : int
        m, the number of lines that define the boundary of the closed
        connected subdomain (this equals the number of characteristic
        points which separate the entire boundary d Omega into lines).

    coorch : float, shape (2, `nlines`)
        `coorch`[0,i-1] contains the x coordinate of the ith
        characteristic point, for i = 1,2,...,`nlines`; while
        `coorch`[1,i-1] contains the corresponding y coordinate.

    lined : int, shape (4, `nlines`)
        The description of the lines that define the boundary domain.
        The line i, for i = 1,...,m, is defined as follows:

        `lined`[0,i-1]

          The number of points on the line, including two end points.

        `lined`[1,i-1]

          The first end point of the line. If `lined`[1,i-1] = j, the
          coordinates of the first end point are those stored in
          `coorch`[0,j-1],`coorch`[1,j-1].

        `lined`[2,i-1]

          The second end point of the line. If `lined`[2,i-1] = k, the
          coordinates of the second end point are those stored in
          `coorch`[0,k-1],`coorch`[1,k-1].

        `lined`[3,i-1]

          This defines the type of line segment connecting the end
          points. Additional information is conveyed by the numerical
          value of `lined`[3,i-1] as follows:

          - `lined`[3,i-1] > 0, the line is described in `fbnd` with
            `lined`[3,i-1] as the index. In this case, the line must be
            described in the trigonometric (anticlockwise) direction;

          - `lined`[3,i-1] = 0, the line is a straight line;

          - if `lined`[3,i-1] < 0, say (-p), then the line is a
            polygonal arc joining the end points and interior points
            specified in `crus`. In this case the line contains the
            points whose coordinates are stored in `coorch`[z,j-1],
            `crus`[z,p-1], `crus`[z,p],...,`crus`[z,p+r-4],
            `coorch`[z,k-1], where z in {0,1}, r = `lined`[0,i-1], j =
            `lined`[1,i-1] and k = `lined`[2,i-1].

    fbnd : None or callable retval = fbnd(i, x, y, data=None)
        `fbnd` must be supplied to calculate the value of the function
        which describes the curve {(x,y) in R^2; such that f(x,y) = 0}
        on segments of the boundary for which `lined`[3,i-1] > 0.

        Parameters
        ~~~~~~~~~~
        i : int
            `lined`[3,i-1], the reference index of the line (portion of
            the contour) i described.

        x : float
            The values of x and y at which f(x,y) is to be evaluated.

        y : float
            The values of x and y at which f(x,y) is to be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x,y) at the specified point.

    crus : float, shape (2, `sdcrus`)
        The coordinates of the intermediate points for polygonal arc
        lines. For a line i defined as a polygonal arc (i.e.,
        `lined`[3,i-1] < 0), if p = -`lined`[3,i-1], `crus`[0,k-1], for
        k = p,...,p+`lined`[0,i-1]-3, must contain the x coordinate of
        the consecutive intermediate points for this line. Similarly
        `crus`[1,k-1], for k = p,...,p+`lined`[0,i-1]-3, must contain
        the corresponding y coordinate.

    sdcrus : int
        The second dimension of the array `crus`.

    rate : float, shape (`nlines`)
        `rate`[i-1] is the geometric progression ratio between the
        points to be generated on the line i, for i = 1,2,...,m and
        `lined`[3,i-1] >= 0.

        If `lined`[3,i-1] < 0, `rate`[i-1] is not referenced.

    ncomp : int
        n, the number of separately connected components of the
        boundary.

    nlcomp : int, shape (`ncomp`)
        |`nlcomp`[k-1]| is the number of line segments in component k of
        the contour. The line i of component k runs in the direction
        `lined`[1,i-1] to `lined`[2,i-1] if `nlcomp`[k-1] > 0, and in
        the opposite direction otherwise; for k = 1,2,...,n.

    lcomp : int, shape (`nlines`)
        `lcomp` must contain the list of line numbers for the each
        component of the boundary. Specifically, the line numbers for
        the kth component of the boundary, for k = 1,2,...,`ncomp`, must
        be in elements l1-1 to l2-1 of `lcomp`, where l2 = sum_i =
        1^k|`nlcomp`[i-1]| and l1 = l2+1-|`nlcomp`[k-1]|.

    nvmax : int
        The maximum number of the boundary mesh vertices to be
        generated.

    nedmx : int
        The maximum number of boundary edges in the boundary mesh to be
        generated.

    coor : float, shape (2, `nvmax`), modified in place
        On exit: `coor`[0,i-1] will contain the x coordinate of the ith
        boundary mesh vertex generated, for i = 1,...,`nvb`; while
        `coor`[1,i-1] will contain the corresponding y coordinate.

    edge : int, shape (3, `nedmx`), modified in place
        On exit: the specification of the boundary edges. `edge`[0,j-1]
        and `edge`[1,j-1] will contain the vertex numbers of the two end
        points of the jth boundary edge. `edge`[2,j-1] is a reference
        number for the jth boundary edge and

        - `edge`[2,j-1] = `lined`[3,i-1], where i and j are such that
          the jth edges is part of the ith line of the boundary and
          `lined`[3,i-1] >= 0;

        - `edge`[2,j-1] = 100+|`lined`[3,i-1]|, where i and j are such
          that the jth edges is part of the ith line of the boundary and
          `lined`[3,i-1] < 0.

    itrace : int
        The level of trace information required from
        ``dim2_gen_boundary``.

        `itrace` = 0 or `itrace` < -1

          No output is generated.

        `itrace` = 1

          Output from the boundary mesh generator is printed. This
          output contains the input information of each line and each
          connected component of the boundary.

        `itrace` = -1

          An analysis of the output boundary mesh is printed on the file
          object associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`). This
          analysis includes the orientation (clockwise or anticlockwise)
          of each connected component of the boundary. This information
          could be of interest to you, especially if an interior meshing
          is carried out using the output of this function, calling
          either ``dim2_gen_inc``, ``dim2_gen_delaunay`` or
          ``dim2_gen_front``.

        `itrace` > 1

          The output is similar to that produced when `itrace` = 1, but
          the coordinates of the generated vertices on the boundary are
          also output.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nvb : int
        The total number of boundary mesh vertices generated.

    nedge : int
        The total number of boundary edges in the boundary mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, the indices of the extremities of line *<value>*
            are both equal to *<value>*.

        (`errno` 1)
            On entry, the sum of absolute values of all numbers of line
            segments is different from `nlines`. The sum of all the
            elements of `nlcomp` = *<value>*. `nlines` = *<value>*.

        (`errno` 1)
            On entry, `ncomp` = *<value>*.

            Constraint: `ncomp` >= 1.

        (`errno` 1)
            On entry, `nlines` = *<value>*.

            Constraint: `nlines` >= 1.

        (`errno` 1)
            On entry, the geometric progression ratio between the points
            to be generated on line *<value>* is *<value>*. It should be
            greater than 0 unless the line segment is defined by
            user-supplied points.

        (`errno` 1)
            On entry, the index of the second end point of line
            *<value>* is *<value>*. It should be greater than or equal
            to 1 and less than or equal to `nlines` = *<value>*.

        (`errno` 1)
            On entry, the index of the first end point of line *<value>*
            is *<value>*. It should be greater than or equal to 1 and
            less than or equal to `nlines` = *<value>*.

        (`errno` 1)
            On entry, the number of points on line *<value>* is
            *<value>*. It should be greater than or equal to 2.

        (`errno` 1)
            On entry, `nvmax` = *<value>* and `nlines` = *<value>*.

            Constraint: `nvmax` >= `nlines`.

        (`errno` 1)
            On entry, `nedmx` = *<value>*.

            Constraint: `nedmx` >= 1.

        (`errno` 1)
            On entry, the line list for the separate connected component
            of the boundary is badly set: `lcomp`[l-1] = *<value>* and l
            = *<value>*. It should be less than or equal to `nlines` and
            greater than or equal to 1.

        (`errno` 1)
            On entry, end point 1, with index K, does not lie on the
            curve representing line I with index J: K = *<value>*, I =
            *<value>*, J = *<value>*, f(x,y) = *<value>*.

        (`errno` 1)
            On entry, end point 2, with index K, does not lie on the
            curve representing line I with index J: K = *<value>*, I =
            *<value>*, J = *<value>*, f(x,y) = *<value>*.

        (`errno` 1)
            On entry, `sdcrus` = *<value>* and nusmin = *<value>*.

            Constraint: `sdcrus` >= nusmin.

        (`errno` 1)
            On entry, there is a correlation problem between the
            user-supplied coordinates and the specification of the
            polygonal arc representing line I = *<value>* with the index
            in `crus` = *<value>*.

        (`errno` 1)
            On entry, the absolute number of line segments in the kth
            component of the contour should be less than or equal to
            `nlines` and greater than 0. k = *<value>*, `nlcomp`[k-1] =
            *<value>* and `nlines` = *<value>*.

        (`errno` 2)
            An error has occurred during the generation of the boundary
            mesh. It appears that `nedmx` is not large enough: `nedmx` =
            *<value>*.

        (`errno` 3)
            An error has occurred during the generation of the boundary
            mesh. It appears that `nvmax` is not large enough: `nvmax` =
            *<value>*.

        (`errno` 4)
            An error has occurred during the generation of the boundary
            mesh. Check the definition of each line (the argument
            `lined`) and each connected component of the boundary (the
            arguments `nlcomp`, and `lcomp`, as well as the coordinates
            of the characteristic points. Setting `itrace` > 0 may
            provide more details.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_gen_boundary'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fbnd_name = fun_name + ':fbnd'

    fbndh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, i.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if fbnd is None:
        fbnd_auxil_ctypes = utils._EngineState._get_symbol('d06badt')
        fbnd_auxil_ctypes.restype = None
        fbnd_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, i.
            utils._BYREF_FLOAT64, # Input, x.
            utils._BYREF_FLOAT64, # Input, y.
            utils._BYREF_FLOAT64, # Output, retval.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_fbndh(_fbnd, _fbndsub, en_data, i, x, y, retval, iuser, ruser):
        """Helper for fbnd."""
        try:
            if fbnd is None:
                fbnd_auxil_ctypes(
                    en_data,
                    i,
                    x,
                    y,
                    retval,
                    iuser,
                    ruser,
                )
                return
            _i_fbndh = utils._EngineIntScalarType.to_py(i.contents)
            _x_fbndh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fbndh = utils._EngineFloat64ScalarType.to_py(y.contents)
            if data is not None:
                _retval_fbndh = fbnd(
                    _i_fbndh,
                    _x_fbndh,
                    _y_fbndh,
                    data,
                )
            else:
                _retval_fbndh = fbnd(
                    _i_fbndh,
                    _x_fbndh,
                    _y_fbndh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fbndh, {'fun_name': cb_fbnd_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d06baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, nlines.
        utils._EngineFloat64ArrayType.c_type, # Input, coorch[2, nlines].
        utils._EngineIntArrayType.c_type, # Input, lined[4, nlines].
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fbndh,
        utils._EngineFloat64ArrayType.c_type, # Input, crus[2, sdcrus].
        utils._BYREF_INT, # Input, sdcrus.
        utils._EngineFloat64ArrayType.c_type, # Input, rate[nlines].
        utils._BYREF_INT, # Input, ncomp.
        utils._EngineIntArrayType.c_type, # Input, nlcomp[ncomp].
        utils._EngineIntArrayType.c_type, # Input, lcomp[nlines].
        utils._BYREF_INT, # Input, nvmax.
        utils._BYREF_INT, # Input, nedmx.
        utils._BYREF_INT, # Output, nvb.
        utils._EngineFloat64ArrayType.c_type, # Output, coor[2, nvmax].
        utils._BYREF_INT, # Output, nedge.
        utils._EngineIntArrayType.c_type, # Output, edge[3, nedmx].
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((coorch, 'coorch'), (lined, 'lined'), (crus, 'crus'), (coor, 'coor'), (edge, 'edge')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _nlines = utils._EngineIntScalarType(
        nlines, {'fun_name': fun_name, 'entity': 'nlines'},
    ).to_ctypes()
    _nlines_py = utils._EngineIntScalarType.to_py(_nlines)
    _coorch = utils._EngineFloat64ArrayType(
        coorch, {'fun_name': fun_name, 'entity': 'coorch'},
        exp_shape=(2, _nlines_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lined = utils._EngineIntArrayType(
        lined, {'fun_name': fun_name, 'entity': 'lined'},
        exp_shape=(4, _nlines_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    utils._check_callable(fbnd, {'fun_name': fun_name, 'entity': 'fbnd'}, allow_none=True)
    _fbnd = utils._UNUSED_CALLBACK
    _fbndsub = utils._UNUSED_CALLBACK
    _fbndh = utils._cast_callback(
        fbndh, cb_fbndh,
        {'fun_name': fun_name, 'entity': 'fbnd'},
    )
    _sdcrus = utils._EngineIntScalarType(
        sdcrus, {'fun_name': fun_name, 'entity': 'sdcrus'},
    ).to_ctypes()
    _sdcrus_py = utils._EngineIntScalarType.to_py(_sdcrus)
    _crus = utils._EngineFloat64ArrayType(
        crus, {'fun_name': fun_name, 'entity': 'crus'},
        exp_shape=(2, _sdcrus_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _rate = utils._EngineFloat64ArrayType(
        rate, {'fun_name': fun_name, 'entity': 'rate'},
        exp_shape=(_nlines_py,),
    ).to_ctypes()
    _ncomp = utils._EngineIntScalarType(
        ncomp, {'fun_name': fun_name, 'entity': 'ncomp'},
    ).to_ctypes()
    _ncomp_py = utils._EngineIntScalarType.to_py(_ncomp)
    _nlcomp = utils._EngineIntArrayType(
        nlcomp, {'fun_name': fun_name, 'entity': 'nlcomp'},
        exp_shape=(_ncomp_py,),
    ).to_ctypes()
    _lcomp = utils._EngineIntArrayType(
        lcomp, {'fun_name': fun_name, 'entity': 'lcomp'},
        exp_shape=(_nlines_py,),
    ).to_ctypes()
    _nvmax = utils._EngineIntScalarType(
        nvmax, {'fun_name': fun_name, 'entity': 'nvmax'},
    ).to_ctypes()
    _nedmx = utils._EngineIntScalarType(
        nedmx, {'fun_name': fun_name, 'entity': 'nedmx'},
    ).to_ctypes()
    _nvb = utils._EngineIntScalarType.empty_ctypes()
    _nvmax_py = utils._EngineIntScalarType.to_py(_nvmax)
    _coor_cls = utils._EngineFloat64ArrayType(
        coor, {'fun_name': fun_name, 'entity': 'coor'},
        exp_shape=(2, _nvmax_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _coor = _coor_cls.to_ctypes(can_reference_contents=False)
    _nedge = utils._EngineIntScalarType.empty_ctypes()
    _nedmx_py = utils._EngineIntScalarType.to_py(_nedmx)
    _edge_cls = utils._EngineIntArrayType(
        edge, {'fun_name': fun_name, 'entity': 'edge'},
        exp_shape=(3, _nedmx_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _edge = _edge_cls.to_ctypes(can_reference_contents=False)
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iuser = None
    _ruser = None
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _nlines,
        _coorch,
        _lined,
        _fbnd,
        _fbndsub,
        _fbndh,
        _crus,
        _sdcrus,
        _rate,
        _ncomp,
        _nlcomp,
        _lcomp,
        _nvmax,
        _nedmx,
        _nvb,
        _coor,
        _nedge,
        _edge,
        _itrace,
        _nadv,
        _iuser,
        _ruser,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the indices of the extremities of line {:s} are'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'both equal to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the sum of absolute values of all numbers of line segments is',
                    'different from nlines.',
                    'The sum of all the elements of nlcomp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nlines = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncomp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncomp >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nlines = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nlines >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the geometric progression ratio between the points',
                    'to be generated on line {:s} is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'It should be greater than 0 unless the line segment is defined',
                    'by user-supplied points.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the index of the second end point of line {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'is {:s}. It should be greater than or equal to 1'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and less than or equal to nlines = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the index of the first end point of line {:s} is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}. It should be greater than or equal to 1 and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'less than or equal to nlines = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the number of points on line {:s} is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}. It should be greater than or equal to 2.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvmax = {:s} and nlines = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nvmax >= nlines.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedmx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedmx >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the line list for the separate connected component',
                    'of the boundary is badly set:',
                    'lcomp[l-1] = {:s} and l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'It should be less than or equal to nlines and greater than or',
                    'equal to 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, end point 1, with index K, does not lie on the curve representing',
                    'line I with index J: K = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'f(x,y) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, end point 2, with index K, does not lie on the curve representing',
                    'line I with index J: K = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'f(x,y) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sdcrus = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nusmin = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sdcrus >= nusmin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, there is a correlation problem between the user-supplied',
                    'coordinates and the specification of the polygonal arc representing',
                    'line I = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'with the index in crus = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the absolute number of line segments in the kth component',
                    'of the contour should be less than or equal to nlines and greater',
                    'than 0.',
                    'k = {:s}, nlcomp[k-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and nlines = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the generation of the boundary mesh.',
                    'It appears that nedmx is not large enough:',
                    'nedmx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the generation of the boundary mesh.',
                    'It appears that nvmax is not large enough:',
                    'nvmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the generation of the boundary mesh. Check the',
                    'definition of each line (the argument lined) and each connected component of',
                    'the boundary (the arguments nlcomp, and lcomp, as well as the coordinates of',
                    'the characteristic points. Setting itrace > 0 may provide more details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coor_cls.refresh_py(_coor)
        _edge_cls.refresh_py(_edge)
        _nvb_py = utils._EngineIntScalarType.to_py(_nvb)
        _nedge_py = utils._EngineIntScalarType.to_py(_nedge)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nvb', 'nedge'],
        )(
            _nvb_py,
            _nedge_py,
        )
    _err_class.manage()
    return return_data

def dim2_smooth_bary(nv, nelt, nedge, coor, edge, conn, nvfix, numfix, itrace, nqint, io_manager=None):
    u"""
    Uses a barycentering technique to smooth a given mesh.

    For full information please refer to the NAG Library document for
    d06ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06caf.html

    Parameters
    ----------
    nv : int
        The total number of vertices in the input mesh.

    nelt : int
        The number of triangles in the input mesh.

    nedge : int
        The number of the boundary and interface edges in the input
        mesh.

    coor : float, shape (2, `nv`), modified in place
        On entry: `coor`[0,i-1] contains the x coordinate of the ith
        input mesh vertex, for i = 1,2,...,`nv`; while `coor`[1,i-1]
        contains the corresponding y coordinate.

        On exit: `coor`[0,i-1] will contain the x coordinate of the ith
        smoothed mesh vertex, for i = 1,2,...,`nv`; while `coor`[1,i-1]
        will contain the corresponding y coordinate. Note that the
        coordinates of boundary and interface edge vertices, as well as
        those specified by you (see the description of `numfix`), are
        unchanged by the process.

    edge : int, shape (3, `nedge`)
        The specification of the boundary or interface edges.
        `edge`[0,j-1] and `edge`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edge`[2,j-1] is a
        user-supplied tag for the jth boundary or interface edge:
        `edge`[2,j-1] = 0 for an interior edge and has a nonzero tag
        otherwise.

    conn : int, shape (3, `nelt`)
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,2,...,`nelt`, for i
        = 1,2,...,3.

    nvfix : int
        The number of fixed vertices in the input mesh.

    numfix : int, shape (`nvfix`)
        The indices in `coor` of fixed interior vertices of the input
        mesh.

    itrace : int
        The level of trace information required from
        ``dim2_smooth_bary``.

        `itrace` <= 0

          No output is generated.

        `itrace` = 1

          A histogram of the triangular element qualities is printed on
          the file object associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) before and
          after smoothing. This histogram gives the lowest and the
          highest triangle quality as well as the number of elements
          lying in each of the `nqint` equal intervals between the
          extremes.

        `itrace` > 1

          The output is similar to that produced when `itrace` = 1 but
          the connectivity between vertices and triangles (for each
          vertex, the list of triangles in which it appears) is given.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element meshes.

    nqint : int
        The number of intervals between the extreme quality values for
        the input and the smoothed mesh.

        If `itrace` = 0, `nqint` is not referenced.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 3.

        (`errno` 1)
            On entry, `nedge` = *<value>*.

            Constraint: `nedge` >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= `nv`.

        (`errno` 1)
            On entry, `nelt` = *<value>* and `nv` = *<value>*.

            Constraint: `nelt` <= 2*`nv`-1.

        (`errno` 1)
            On entry, `conn`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `conn`(I,J) >= 1 and `conn`(I,J) <= `nv`.

        (`errno` 1)
            On entry, `numfix`[I-1] = *<value>*, I = *<value>* and `nv`
            = *<value>*.

            Constraint: `numfix`[I-1] >= 1 and `numfix`[I-1] <= `nv`.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nvfix` = *<value>*.

            Constraint: 0 <= `nvfix` <= `nv`.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            A serious error has occurred in an internal call to an
            auxiliary function. Check the input mesh especially the
            connectivity. Seek expert help.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_smooth_bary'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, nv.
        utils._BYREF_INT, # Input, nelt.
        utils._BYREF_INT, # Input, nedge.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, coor[2, nv].
        utils._EngineIntArrayType.c_type, # Input, edge[3, nedge].
        utils._EngineIntArrayType.c_type, # Input, conn[3, nelt].
        utils._BYREF_INT, # Input, nvfix.
        utils._EngineIntArrayType.c_type, # Input, numfix[nvfix].
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input, nqint.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((coor, 'coor'), (edge, 'edge'), (conn, 'conn')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    _nelt = utils._EngineIntScalarType(
        nelt, {'fun_name': fun_name, 'entity': 'nelt'},
    ).to_ctypes()
    _nedge = utils._EngineIntScalarType(
        nedge, {'fun_name': fun_name, 'entity': 'nedge'},
    ).to_ctypes()
    _nv_py = utils._EngineIntScalarType.to_py(_nv)
    _coor_cls = utils._EngineFloat64ArrayType(
        coor, {'fun_name': fun_name, 'entity': 'coor'},
        exp_shape=(2, _nv_py),
        mutual_sorder=_m_sorder,
    )
    _coor = _coor_cls.to_ctypes()
    _nedge_py = utils._EngineIntScalarType.to_py(_nedge)
    _edge = utils._EngineIntArrayType(
        edge, {'fun_name': fun_name, 'entity': 'edge'},
        exp_shape=(3, _nedge_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nelt_py = utils._EngineIntScalarType.to_py(_nelt)
    _conn = utils._EngineIntArrayType(
        conn, {'fun_name': fun_name, 'entity': 'conn'},
        exp_shape=(3, _nelt_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nvfix = utils._EngineIntScalarType(
        nvfix, {'fun_name': fun_name, 'entity': 'nvfix'},
    ).to_ctypes()
    _nvfix_py = utils._EngineIntScalarType.to_py(_nvfix)
    _numfix = utils._EngineIntArrayType(
        numfix, {'fun_name': fun_name, 'entity': 'numfix'},
        exp_shape=(_nvfix_py,),
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _nqint = utils._EngineIntScalarType(
        nqint, {'fun_name': fun_name, 'entity': 'nqint'},
    ).to_ctypes()
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _nv,
        _nelt,
        _nedge,
        _coor,
        _edge,
        _conn,
        _nvfix,
        _numfix,
        _itrace,
        _nadv,
        _nqint,
        _iwork,
        _liwork,
        _rwork,
        _lrwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edge(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edge(I,J) >= 1 and edge(I,J) <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nelt = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nelt <= 2*nv-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, conn(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: conn(I,J) >= 1 and conn(I,J) <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, numfix[I-1] = {:s}, I = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: numfix[I-1] >= 1 and numfix[I-1] <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nvfix = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 <= nvfix <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of the edge J have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 2 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 2 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error has occurred in an internal call to an auxiliary function.',
                    'Check the input mesh especially the connectivity. Seek expert help.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coor_cls.refresh_py(_coor)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_sparsity(nv, nelt, nnzmax, conn, irow, icol):
    u"""
    Generates a sparsity pattern of a Finite Element matrix associated
    with a given mesh.

    For full information please refer to the NAG Library document for
    d06cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06cbf.html

    Parameters
    ----------
    nv : int
        The total number of vertices in the input mesh.

    nelt : int
        The number of triangles in the input mesh.

    nnzmax : int
        The maximum number of nonzero entries in the matrix based on the
        input mesh. It is the dimension of the arrays `irow` and `icol`
        as declared in the function from which ``dim2_sparsity`` is
        called.

    conn : int, shape (3, `nelt`)
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,2,...,`nelt`, for i
        = 1,2,...,3.

    irow : int, shape (`nnzmax`), modified in place
        On exit: the first `nnz` elements contain the row and column
        indices of the nonzero elements supplied in the finite element
        matrix A.

    icol : int, shape (`nnzmax`), modified in place
        On exit: the first `nnz` elements contain the row and column
        indices of the nonzero elements supplied in the finite element
        matrix A.

    Returns
    -------
    nnz : int
        The number of nonzero entries in the matrix associated with the
        input mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `conn`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `conn`(I,J) >= 1 and `conn`(I,J) <= `nv`.

        (`errno` 1)
            On entry, `nelt` = *<value>* and `nv` = *<value>*.

            Constraint: `nelt` <= 2*`nv`-1.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `nnzmax` = *<value>* and `nv` = *<value>*.

            Constraint: `nnzmax` <= `nv`^2.

        (`errno` 1)
            On entry, `nnzmax` = *<value>*, `nelt` = *<value>* and `nv`
            = *<value>*.

            Constraint: `nnzmax` >= (4*`nelt`+`nv`).

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 3.

        (`errno` 2)
            A serious error has occurred in an internal call to an
            auxiliary function. Check the input mesh, especially the
            connectivity between triangles and vertices (the argument
            `conn`). Array dimensions should also be checked. If the
            problem persists, contact NAG.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_sparsity'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nv.
        utils._BYREF_INT, # Input, nelt.
        utils._BYREF_INT, # Input, nnzmax.
        utils._EngineIntArrayType.c_type, # Input, conn[3, nelt].
        utils._BYREF_INT, # Output, nnz.
        utils._EngineIntArrayType.c_type, # Output, irow[nnzmax].
        utils._EngineIntArrayType.c_type, # Output, icol[nnzmax].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((conn, 'conn'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    _nelt = utils._EngineIntScalarType(
        nelt, {'fun_name': fun_name, 'entity': 'nelt'},
    ).to_ctypes()
    _nnzmax = utils._EngineIntScalarType(
        nnzmax, {'fun_name': fun_name, 'entity': 'nnzmax'},
    ).to_ctypes()
    _nelt_py = utils._EngineIntScalarType.to_py(_nelt)
    _conn = utils._EngineIntArrayType(
        conn, {'fun_name': fun_name, 'entity': 'conn'},
        exp_shape=(3, _nelt_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nnz = utils._EngineIntScalarType.empty_ctypes()
    _nnzmax_py = utils._EngineIntScalarType.to_py(_nnzmax)
    _irow_cls = utils._EngineIntArrayType(
        irow, {'fun_name': fun_name, 'entity': 'irow'},
        exp_shape=(_nnzmax_py,),
        check_contents=False,
    )
    _irow = _irow_cls.to_ctypes(can_reference_contents=False)
    _icol_cls = utils._EngineIntArrayType(
        icol, {'fun_name': fun_name, 'entity': 'icol'},
        exp_shape=(_nnzmax_py,),
        check_contents=False,
    )
    _icol = _icol_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nv,
        _nelt,
        _nnzmax,
        _conn,
        _nnz,
        _irow,
        _icol,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, conn(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: conn(I,J) >= 1 and conn(I,J) <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nelt = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nelt <= 2*nv-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 2 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 2 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nnzmax = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nnzmax <= nv^2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nnzmax = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nelt = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: nnzmax >= (4*nelt+nv).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error has occurred in an internal call to an auxiliary function.',
                    'Check the input mesh, especially the connectivity between triangles and',
                    'vertices (the argument conn). Array dimensions should also be checked. If',
                    'the problem persists, contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _irow_cls.refresh_py(_irow)
        _icol_cls.refresh_py(_icol)
        _nnz_py = utils._EngineIntScalarType.to_py(_nnz)
        return_data = _nnz_py
    _err_class.manage()
    return return_data

def dim2_renumber(nv, nelt, nedge, nnzmax, coor, edge, conn, irow, icol, itrace, io_manager=None):
    u"""
    Renumbers a given mesh using Gibbs method.

    For full information please refer to the NAG Library document for
    d06cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06ccf.html

    Parameters
    ----------
    nv : int
        The total number of vertices in the input mesh.

    nelt : int
        The number of triangles in the input mesh.

    nedge : int
        The number of boundary edges in the input mesh.

    nnzmax : int
        The maximum number of nonzero entries in the matrix based on the
        input mesh. It is the dimension of the arrays `irow` and `icol`
        as declared in the function from which ``dim2_renumber`` is
        called.

    coor : float, shape (2, `nv`), modified in place
        On entry: `coor`[0,i-1] contains the x coordinate of the ith
        input mesh vertex, for i = 1,...,`nv`; while `coor`[1,i-1]
        contains the corresponding y coordinate.

        On exit: `coor`[0,i-1] will contain the x coordinate of the ith
        renumbered mesh vertex, for i = 1,...,`nv`; while `coor`[1,i-1]
        will contain the corresponding y coordinate.

    edge : int, shape (3, `nedge`), modified in place
        On entry: the specification of the boundary or interface edges.
        `edge`[0,j-1] and `edge`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edge`[2,j-1] is a
        user-supplied tag for the jth boundary or interface edge:
        `edge`[2,j-1] = 0 for an interior edge and has a nonzero tag
        otherwise.

        On exit: the renumbered specification of the boundary or
        interface edges.

    conn : int, shape (3, `nelt`), modified in place
        On entry: the connectivity of the mesh between triangles and
        vertices. For each triangle j, `conn`[i-1,j-1] gives the indices
        of its three vertices (in anticlockwise order), for j =
        1,2,...,`nelt`, for i = 1,2,...,3.

        On exit: the renumbered connectivity of the mesh between
        triangles and vertices.

    irow : int, shape (`nnzmax`), modified in place
        On exit: the first `nnz` elements contain the row and column
        indices of the nonzero elements supplied in the finite element
        matrix A.

    icol : int, shape (`nnzmax`), modified in place
        On exit: the first `nnz` elements contain the row and column
        indices of the nonzero elements supplied in the finite element
        matrix A.

    itrace : int
        The level of trace information required from ``dim2_renumber``.

        `itrace` <= 0

          No output is generated.

        `itrace` = 1

          Information about the effect of the renumbering on the finite
          element matrix are output. This information includes the half
          bandwidth and the sparsity structure of this matrix before and
          after renumbering.

        `itrace` > 1

          The output is similar to that produced when `itrace` = 1 but
          the sparsities (for each row of the matrix, indices of nonzero
          entries) of the matrix before and after renumbering are also
          output.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nnz : int
        The number of nonzero entries in the matrix based on the input
        mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 3.

        (`errno` 1)
            On entry, `nedge` = *<value>*.

            Constraint: `nedge` >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= `nv`.

        (`errno` 1)
            On entry, `nelt` = *<value>* and `nv` = *<value>*.

            Constraint: `nelt` <= 2*`nv`-1.

        (`errno` 1)
            On entry, `conn`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `conn`(I,J) >= 1 and `conn`(I,J) <= `nv`.

        (`errno` 1)
            On entry, `nnzmax` = *<value>*, `nelt` = *<value>* and `nv`
            = *<value>*.

            Constraint: `nnzmax` >= (4*`nelt`+`nv`) and `nnzmax` <=
            `nv`^2.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            An error has occurred during the computation of the compact
            sparsity of the finite element matrix. Check the
            Triangle/Vertices connectivity.

        (`errno` 3)
            A serious error has occurred in an internal call to the
            renumbering function. Check the input mesh especially the
            connectivity. Seek expert help.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_renumber'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, nv.
        utils._BYREF_INT, # Input, nelt.
        utils._BYREF_INT, # Input, nedge.
        utils._BYREF_INT, # Input, nnzmax.
        utils._BYREF_INT, # Output, nnz.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, coor[2, nv].
        utils._EngineIntArrayType.c_type, # Input/Output, edge[3, nedge].
        utils._EngineIntArrayType.c_type, # Input/Output, conn[3, nelt].
        utils._EngineIntArrayType.c_type, # Output, irow[nnzmax].
        utils._EngineIntArrayType.c_type, # Output, icol[nnzmax].
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((coor, 'coor'), (edge, 'edge'), (conn, 'conn')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    _nelt = utils._EngineIntScalarType(
        nelt, {'fun_name': fun_name, 'entity': 'nelt'},
    ).to_ctypes()
    _nedge = utils._EngineIntScalarType(
        nedge, {'fun_name': fun_name, 'entity': 'nedge'},
    ).to_ctypes()
    _nnzmax = utils._EngineIntScalarType(
        nnzmax, {'fun_name': fun_name, 'entity': 'nnzmax'},
    ).to_ctypes()
    _nnz = utils._EngineIntScalarType.empty_ctypes()
    _nv_py = utils._EngineIntScalarType.to_py(_nv)
    _coor_cls = utils._EngineFloat64ArrayType(
        coor, {'fun_name': fun_name, 'entity': 'coor'},
        exp_shape=(2, _nv_py),
        mutual_sorder=_m_sorder,
    )
    _coor = _coor_cls.to_ctypes()
    _nedge_py = utils._EngineIntScalarType.to_py(_nedge)
    _edge_cls = utils._EngineIntArrayType(
        edge, {'fun_name': fun_name, 'entity': 'edge'},
        exp_shape=(3, _nedge_py),
        mutual_sorder=_m_sorder,
    )
    _edge = _edge_cls.to_ctypes()
    _nelt_py = utils._EngineIntScalarType.to_py(_nelt)
    _conn_cls = utils._EngineIntArrayType(
        conn, {'fun_name': fun_name, 'entity': 'conn'},
        exp_shape=(3, _nelt_py),
        mutual_sorder=_m_sorder,
    )
    _conn = _conn_cls.to_ctypes()
    _nnzmax_py = utils._EngineIntScalarType.to_py(_nnzmax)
    _irow_cls = utils._EngineIntArrayType(
        irow, {'fun_name': fun_name, 'entity': 'irow'},
        exp_shape=(_nnzmax_py,),
        check_contents=False,
    )
    _irow = _irow_cls.to_ctypes(can_reference_contents=False)
    _icol_cls = utils._EngineIntArrayType(
        icol, {'fun_name': fun_name, 'entity': 'icol'},
        exp_shape=(_nnzmax_py,),
        check_contents=False,
    )
    _icol = _icol_cls.to_ctypes(can_reference_contents=False)
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _nv,
        _nelt,
        _nedge,
        _nnzmax,
        _nnz,
        _coor,
        _edge,
        _conn,
        _irow,
        _icol,
        _itrace,
        _nadv,
        _iwork,
        _liwork,
        _rwork,
        _lrwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edge(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edge(I,J) >= 1 and edge(I,J) <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nelt = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nelt <= 2*nv-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, conn(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: conn(I,J) >= 1 and conn(I,J) <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nnzmax = {:s}, nelt = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: nnzmax >= (4*nelt+nv) and nnzmax <= nv^2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of the edge J have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 2 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 2 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error has occurred during the computation of the compact sparsity of the',
                    'finite element matrix. Check the Triangle/Vertices connectivity.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error has occurred in an internal call to the renumbering',
                    'function. Check the input mesh especially the connectivity.',
                    'Seek expert help.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coor_cls.refresh_py(_coor)
        _edge_cls.refresh_py(_edge)
        _conn_cls.refresh_py(_conn)
        _irow_cls.refresh_py(_irow)
        _icol_cls.refresh_py(_icol)
        _nnz_py = utils._EngineIntScalarType.to_py(_nnz)
        return_data = _nnz_py
    _err_class.manage()
    return return_data

def dim2_transform_affine(mode, nv, nedge, nelt, ntrans, itype, trans, coori, edgei, conni, cooro, edgeo, conno, itrace, io_manager=None):
    u"""
    Generates a mesh resulting from an affine transformation of a given
    mesh.

    For full information please refer to the NAG Library document for
    d06da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06daf.html

    Parameters
    ----------
    mode : int
        If `mode` = 1, the arguments `coori`, `edgei` and `conni` are
        overwritten on exit by the output values described in `cooro`,
        `edgeo` and `conno` respectively. In this case `cooro`, `edgeo`
        and `conno` are not referenced, and you can save storage space.

        If `mode` != 1, no such aliasing is assumed.

    nv : int
        The total number of vertices in the input mesh.

    nedge : int
        The number of the boundary or interface edges in the input mesh.

    nelt : int
        The number of triangles in the input mesh.

    ntrans : int
        The number of transformations of the input mesh.

    itype : int, shape (`ntrans`)
        `itype`[i-1], for i = 1,2,...,`ntrans`, indicates the type of
        each transformation as follows:

        `itype`[i-1] = 0

          Identity transformation.

        `itype`[i-1] = 1

          Translation.

        `itype`[i-1] = 2

          Symmetric transformation with respect to a user-supplied line.

        `itype`[i-1] = 3

          Rotation.

        `itype`[i-1] = 4

          Scaling.

        `itype`[i-1] = 10

          User-supplied analytic transformation.

        Note that the transformations are applied in the order described
        in `itype`.

    trans : float, shape (6, `ntrans`)
        The arguments for each transformation. For i = 1,2,...,`ntrans`,
        `trans`[0,i-1] to `trans`[5,i-1] contain the arguments of the
        ith transformation.

        If `itype`[i-1] = 0, elements `trans`[0,i-1] to `trans`[5,i-1]
        are not referenced.

        If `itype`[i-1] = 1, the translation vector is vec(u) = (a;b),
        where a = `trans`[0,i-1] and b = `trans`[1,i-1], while elements
        `trans`[2,i-1] to `trans`[5,i-1] are not referenced.

        If `itype`[i-1] = 2, the user-supplied line is the curve {(x,y)
        in R^2; such that ax+by+c = 0}, where a = `trans`[0,i-1], b =
        `trans`[1,i-1] and c = `trans`[2,i-1], while elements
        `trans`[3,i-1] to `trans`[5,i-1] are not referenced.

        If `itype`[i-1] = 3, the centre of the rotation is (x_0,y_0)
        where x_0 = `trans`[0,i-1] and y_0 = `trans`[1,i-1], theta =
        `trans`[2,i-1] is its angle in degrees, while elements
        `trans`[3,i-1] to `trans`[5,i-1] are not referenced.

        If `itype`[i-1] = 4, a = `trans`[0,i-1] is the scaling
        coefficient in the x-direction, b = `trans`[1,i-1] is the
        scaling coefficient in the y-direction, and (x_0,y_0) are the
        scaling centre coordinates, with x_0 = `trans`[2,i-1] and y_0 =
        `trans`[3,i-1]; while elements `trans`[4,i-1] to `trans`[5,i-1]
        are not referenced.

        If `itype`[i-1] = 10, the user-supplied analytic affine
        transformation Y = A*X+B is such that A = (a_{kl})_{1 <= k,l <=
        2} and B = (b_k)_{1 <= k <= 2} wherea_{kl} =
        `trans`[2*(k-1)+l-1,i-1], and b_k = `trans`[4+k-1,i-1] with k,l
        = 1,2.

    coori : float, shape (2, `nv`), modified in place
        On entry: `coori`[0,i-1] contains the x coordinate of the ith
        vertex of the input mesh, for i = 1,2,...,`nv`; while
        `coori`[1,i-1] contains the corresponding y coordinate.

        On exit: see [Further Comments].

    edgei : int, shape (3, `nedge`), modified in place
        On entry: the specification of the boundary or interface edges.
        `edgei`[0,j-1] and `edgei`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edgei`[2,j-1] is a
        user-supplied tag for the jth boundary edge.

        On exit: if `mode` = 1, `edgei` holds the output values
        described in `edgeo`.

    conni : int, shape (3, `nelt`), modified in place
        On entry: the connectivity of the input mesh between triangles
        and vertices. For each triangle j, `conni`[i-1,j-1] gives the
        indices of its three vertices (in anticlockwise order), for j =
        1,2,...,`nelt`, for i = 1,2,...,3.

        On exit: if `mode` = 1, `conni` holds the output values
        described in `conno`.

    cooro : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `mode`!=1: 2; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `mode`!=1: `nv`; otherwise: 0.

        On exit: `cooro`[0,i-1] will contain the x coordinate of the ith
        vertex of the transformed mesh, for i = 1,2,...,`nv`; while
        `cooro`[1,i-1] will contain the corresponding y coordinate.

    edgeo : None or int, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `mode`!=1: 3; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `mode`!=1: `nedge`; otherwise: 0.

        On exit: the specification of the boundary or interface edges of
        the transformed mesh. If the number of symmetric transformations
        is even or zero then `edgeo`[i-1,j-1] = `edgei`[i-1,j-1], for j
        = 1,2,...,`nedge`, for i = 1,2,...,3; otherwise `edgeo`[0,j-1] =
        `edgei`[1,j-1],`edgeo`[1,j-1] = `edgei`[0,j-1] and
        `edgeo`[2,j-1] = `edgei`[2,j-1], for j = 1,2,...,`nedge`.

    conno : None or int, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `mode`!=1: 3; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `mode`!=1: `nelt`; otherwise: 1.

        On exit: the connectivity of the transformed mesh between
        triangles and vertices. If the number of symmetric
        transformations is even or zero then `conno`[i-1,j-1] =
        `conni`[i-1,j-1], for j = 1,2,...,`nelt`, for i = 1,2,...,3;
        otherwise `conno`[0,j-1] = `conni`[0,j-1], `conno`[1,j-1] =
        `conni`[2,j-1] and `conno`[2,j-1] = `conni`[1,j-1], for j =
        1,2,...,`nelt`.

    itrace : int
        The level of trace information required from
        ``dim2_transform_affine``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Details of each transformation, the matrix A and the vector B
          of the final transformation, which is the composition of all
          the `ntrans` transformations, are printed.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 3.

        (`errno` 1)
            On entry, `itype`[I-1] = *<value>* and I = *<value>*.

            Constraint: `itype`[I-1] = 0, 1, 2, 3, 4 or 10.

        (`errno` 1)
            On entry, `nedge` = *<value>*.

            Constraint: `nedge` >= 1.

        (`errno` 1)
            On entry, `nelt` = *<value>* and `nv` = *<value>*.

            Constraint: `nelt` <= 2*`nv`-1.

        (`errno` 1)
            On entry, `edgei`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `edgei`(I,J) >= 1 and `edgei`(I,J) <= `nv`.

        (`errno` 1)
            On entry, `conni`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `conni`(I,J) >= 1 and `conni`(I,J) <= `nv`.

        (`errno` 1)
            On entry, `ntrans` = *<value>*.

            Constraint: `ntrans` > 0.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            A serious error has occurred in an internal call to an
            auxiliary function. Check the input mesh especially the
            connectivities and the details of each transformations.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_transform_affine'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, mode.
        utils._BYREF_INT, # Input, nv.
        utils._BYREF_INT, # Input, nedge.
        utils._BYREF_INT, # Input, nelt.
        utils._BYREF_INT, # Input, ntrans.
        utils._EngineIntArrayType.c_type, # Input, itype[ntrans].
        utils._EngineFloat64ArrayType.c_type, # Input, trans[6, ntrans].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, coori[2, nv].
        utils._EngineIntArrayType.c_type, # Input/Output, edgei[3, nedge].
        utils._EngineIntArrayType.c_type, # Input/Output, conni[3, nelt].
        utils._EngineFloat64ArrayType.c_type, # Output, cooro[ldcooro, *].
        utils._BYREF_INT, # Input, ldcooro.
        utils._EngineIntArrayType.c_type, # Output, edgeo[ldedgeo, *].
        utils._BYREF_INT, # Input, ldedgeo.
        utils._EngineIntArrayType.c_type, # Output, conno[ldconno, *].
        utils._BYREF_INT, # Input, ldconno.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((trans, 'trans'), (coori, 'coori'), (edgei, 'edgei'), (conni, 'conni'), (cooro, 'cooro'), (edgeo, 'edgeo'), (conno, 'conno')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    _nedge = utils._EngineIntScalarType(
        nedge, {'fun_name': fun_name, 'entity': 'nedge'},
    ).to_ctypes()
    _nelt = utils._EngineIntScalarType(
        nelt, {'fun_name': fun_name, 'entity': 'nelt'},
    ).to_ctypes()
    _ntrans = utils._EngineIntScalarType(
        ntrans, {'fun_name': fun_name, 'entity': 'ntrans'},
    ).to_ctypes()
    _ntrans_py = utils._EngineIntScalarType.to_py(_ntrans)
    _itype = utils._EngineIntArrayType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
        exp_shape=(_ntrans_py,),
    ).to_ctypes()
    _trans = utils._EngineFloat64ArrayType(
        trans, {'fun_name': fun_name, 'entity': 'trans'},
        exp_shape=(6, _ntrans_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nv_py = utils._EngineIntScalarType.to_py(_nv)
    _coori_cls = utils._EngineFloat64ArrayType(
        coori, {'fun_name': fun_name, 'entity': 'coori'},
        exp_shape=(2, _nv_py),
        mutual_sorder=_m_sorder,
    )
    _coori = _coori_cls.to_ctypes()
    _nedge_py = utils._EngineIntScalarType.to_py(_nedge)
    _edgei_cls = utils._EngineIntArrayType(
        edgei, {'fun_name': fun_name, 'entity': 'edgei'},
        exp_shape=(3, _nedge_py),
        mutual_sorder=_m_sorder,
    )
    _edgei = _edgei_cls.to_ctypes()
    _nelt_py = utils._EngineIntScalarType.to_py(_nelt)
    _conni_cls = utils._EngineIntArrayType(
        conni, {'fun_name': fun_name, 'entity': 'conni'},
        exp_shape=(3, _nelt_py),
        mutual_sorder=_m_sorder,
    )
    _conni = _conni_cls.to_ctypes()
    _ldcooro_py = utils._EngineFloat64ArrayType.get_nd(cooro, {'fun_name': fun_name, 'entity': 'cooro'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldcooro = utils._EngineIntScalarType(
        _ldcooro_py, {'fun_name': fun_name, 'entity': '_ldcooro'},
    ).to_ctypes()
    _mode_py = utils._EngineIntScalarType.to_py(_mode)
    if _mode_py != 1:
        if cooro is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'cooro'},
                'must not be None when mode != 1',
            )
    if _mode_py != 1:
        _cooro_dim1 = 2
    else:
        _cooro_dim1 = 0
    if _mode_py != 1:
        _cooro_dim2 = _nv_py
    else:
        _cooro_dim2 = 0
    exp_shape_cooro = (_cooro_dim1, _cooro_dim2)
    _cooro_cls = utils._EngineFloat64ArrayType(
        cooro, {'fun_name': fun_name, 'entity': 'cooro'},
        exp_shape=exp_shape_cooro,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _cooro = _cooro_cls.to_ctypes(can_reference_contents=False)
    _ldedgeo_py = utils._EngineIntArrayType.get_nd(edgeo, {'fun_name': fun_name, 'entity': 'edgeo'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldedgeo = utils._EngineIntScalarType(
        _ldedgeo_py, {'fun_name': fun_name, 'entity': '_ldedgeo'},
    ).to_ctypes()
    if _mode_py != 1:
        if edgeo is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'edgeo'},
                'must not be None when mode != 1',
            )
    if _mode_py != 1:
        _edgeo_dim1 = 3
    else:
        _edgeo_dim1 = 0
    if _mode_py != 1:
        _edgeo_dim2 = _nedge_py
    else:
        _edgeo_dim2 = 0
    exp_shape_edgeo = (_edgeo_dim1, _edgeo_dim2)
    _edgeo_cls = utils._EngineIntArrayType(
        edgeo, {'fun_name': fun_name, 'entity': 'edgeo'},
        exp_shape=exp_shape_edgeo,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _edgeo = _edgeo_cls.to_ctypes(can_reference_contents=False)
    _ldconno_py = utils._EngineIntArrayType.get_nd(conno, {'fun_name': fun_name, 'entity': 'conno'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldconno = utils._EngineIntScalarType(
        _ldconno_py, {'fun_name': fun_name, 'entity': '_ldconno'},
    ).to_ctypes()
    if _mode_py != 1:
        if conno is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'conno'},
                'must not be None when mode != 1',
            )
    if _mode_py != 1:
        _conno_dim1 = 3
    else:
        _conno_dim1 = 0
    if _mode_py != 1:
        _conno_dim2 = _nelt_py
    else:
        _conno_dim2 = 1
    exp_shape_conno = (_conno_dim1, _conno_dim2)
    _conno_cls = utils._EngineIntArrayType(
        conno, {'fun_name': fun_name, 'entity': 'conno'},
        exp_shape=exp_shape_conno,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _conno = _conno_cls.to_ctypes(can_reference_contents=False)
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _mode,
        _nv,
        _nedge,
        _nelt,
        _ntrans,
        _itype,
        _trans,
        _coori,
        _edgei,
        _conni,
        _cooro,
        _ldcooro,
        _edgeo,
        _ldedgeo,
        _conno,
        _ldconno,
        _itrace,
        _nadv,
        _rwork,
        _lrwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itype[I-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: itype[I-1] = 0, 1, 2, 3, 4 or 10.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nelt = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nelt <= 2*nv-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edgei(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edgei(I,J) >= 1 and edgei(I,J) <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, conni(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: conni(I,J) >= 1 and conni(I,J) <= nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ntrans = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ntrans > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of the edge J have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 2 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 3 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 2 of the triangle K have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error has occurred in an internal call to an auxiliary function.',
                    'Check the input mesh especially the connectivities and the details of',
                    'each transformations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coori_cls.refresh_py(_coori)
        _edgei_cls.refresh_py(_edgei)
        _conni_cls.refresh_py(_conni)
        if cooro is not None:
            _cooro_cls.refresh_py(_cooro)
        if edgeo is not None:
            _edgeo_cls.refresh_py(_edgeo)
        if conno is not None:
            _conno_cls.refresh_py(_conno)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_join(eps, nv1, nelt1, nedge1, coor1, edge1, conn1, reft1, nv2, nelt2, nedge2, coor2, edge2, conn2, reft2, coor3, edge3, conn3, reft3, itrace, io_manager=None):
    u"""
    Joins together two given adjacent (possibly overlapping) meshes.

    For full information please refer to the NAG Library document for
    d06db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06dbf.html

    Parameters
    ----------
    eps : float
        The relative precision of the restitching of the two input
        meshes (see [Further Comments]).

    nv1 : int
        The total number of vertices in the first input mesh.

    nelt1 : int
        The number of triangular elements in the first input mesh.

    nedge1 : int
        The number of boundary edges in the first input mesh.

    coor1 : float, shape (2, `nv1`)
        `coor1`[0,i-1] contains the x coordinate of the ith vertex of
        the first input mesh, for i = 1,2,...,`nv1`; while
        `coor1`[1,i-1] contains the corresponding y coordinate.

    edge1 : int, shape (3, `nedge1`)
        The specification of the boundary edges of the first input mesh.
        `edge1`[0,j-1] and `edge1`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edge1`[2,j-1] is a
        user-supplied tag for the jth boundary edge.

    conn1 : int, shape (3, `nelt1`)
        The connectivity between triangles and vertices of the first
        input mesh. For each triangle j, `conn1`[i-1,j-1] gives the
        indices of its three vertices (in anticlockwise order), for j =
        1,2,...,`nelt1`, for i = 1,2,...,3.

    reft1 : int, shape (`nelt1`)
        `reft1`[k-1] contains the user-supplied tag of the kth triangle
        from the first input mesh, for k = 1,2,...,`nelt1`.

    nv2 : int
        The total number of vertices in the second input mesh.

    nelt2 : int
        The number of triangular elements in the second input mesh.

    nedge2 : int
        The number of boundary edges in the second input mesh.

    coor2 : float, shape (2, `nv2`)
        `coor2`[0,i-1] contains the x coordinate of the ith vertex of
        the second input mesh, for i = 1,2,...,`nv2`; while
        `coor2`[1,i-1] contains the corresponding y coordinate.

    edge2 : int, shape (3, `nedge2`)
        The specification of the boundary edges of the second input
        mesh. `edge2`[0,j-1] and `edge2`[1,j-1] contain the vertex
        numbers of the two end points of the jth boundary edge.
        `edge2`[2,j-1] is a user-supplied tag for the jth boundary edge.

    conn2 : int, shape (3, `nelt2`)
        The connectivity between triangles and vertices of the second
        input mesh. For each triangle j, `conn2`[i-1,j-1] gives the
        indices of its three vertices (in anticlockwise order), for j =
        1,2,...,`nelt2`, for i = 1,2,...,3.

    reft2 : int, shape (`nelt2`)
        `reft2`[k-1] contains the user-supplied tag of the kth triangle
        from the second input mesh, for k = 1,2,...,`nelt2`.

    coor3 : float, shape (2, `nv1`+`nv2`), modified in place
        On exit: `coor3`[0,i-1] will contain the x coordinate of the ith
        vertex of the resulting mesh, for i = 1,2,...,`nv3`; while
        `coor3`[1,i-1] will contain the corresponding y coordinate.

    edge3 : int, shape (3, `nedge1`+`nedge2`), modified in place
        On exit: the specification of the boundary edges of the
        resulting mesh. `edge3`[i-1,j-1] will contain the vertex number
        of the ith end point (i = 1,2) of the jth boundary or interface
        edge.

        If the two meshes overlap, `edge3`[2,j-1] will contain the same
        tag as the corresponding edge belonging to the first and/or the
        second input mesh.

        If the two meshes are adjacent,

        - if the jth edge is part of the partition interface,
          `edge3`[2,j-1] will contain the value 1000*k_1+k_2 where k_1
          and k_2 are the tags for the same edge of the first and the
          second mesh respectively;

        - otherwise, `edge3`[2,j-1] will contain the same tag as the
          corresponding edge belonging to the first and/or the second
          input mesh.

    conn3 : int, shape (3, `nelt1`+`nelt2`), modified in place
        On exit: the connectivity between triangles and vertices of the
        resulting mesh. `conn3`[i-1,j-1] will give the indices of its
        three vertices (in anticlockwise order), for j =
        1,2,...,`nelt3`, for i = 1,2,...,3.

    reft3 : int, shape (`nelt1`+`nelt2`), modified in place
        On exit: if the two meshes form a partition, `reft3`[k-1] will
        contain the same tag as the corresponding triangle belonging to
        the first or the second input mesh, for k = 1,2,...,`nelt3`. If
        the two meshes overlap, `reft3`[k-1] will contain the value
        1000*k_1+k_2 where k_1 and k_2 are the user-supplied tags for
        the same triangle of the first and the second mesh respectively,
        for k = 1,2,...,`nelt3`.

    itrace : int
        The level of trace information required from ``dim2_join``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Details about the common vertices, edges and triangles to both
          meshes are printed.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv3 : int
        The total number of vertices in the resulting mesh.

    nelt3 : int
        The number of triangular elements in the resulting mesh.

    nedge3 : int
        The number of boundary edges in the resulting mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nv2` = *<value>*.

            Constraint: `nv2` >= 3.

        (`errno` 1)
            On entry, `nedge2` = *<value>*.

            Constraint: `nedge2` >= 1.

        (`errno` 1)
            On entry, `edge2`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv2` = *<value>*.

            Constraint: `edge2`(I,J) >= 1 and `edge2`(I,J) <= `nv2`.

        (`errno` 1)
            On entry, `nelt2` = *<value>* and `nv2` = *<value>*.

            Constraint: `nelt2` <= (2*`nv2`-1).

        (`errno` 1)
            On entry, `conn2`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv2` = *<value>*.

            Constraint: `conn2`(I,J) >= 1 and `conn2`(I,J) <= `nv2`.

        (`errno` 1)
            On entry, `nv1` = *<value>*.

            Constraint: `nv1` >= 3.

        (`errno` 1)
            On entry, `nedge1` = *<value>*.

            Constraint: `nedge1` >= 1.

        (`errno` 1)
            On entry, `edge1`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv1` = *<value>*.

            Constraint: `edge1`(I,J) >= 1 and `edge1`(I,J) <= `nv1`.

        (`errno` 1)
            On entry, `nelt1` = *<value>* and `nv1` = *<value>*.

            Constraint: `nelt1` <= (2*`nv1`-1).

        (`errno` 1)
            On entry, `conn1`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv1` = *<value>*.

            Constraint: `conn1`(I,J) >= 1 and `conn1`(I,J) <= `nv1`.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, the end points of edge J in the second mesh have
            the same index I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of triangle K in the second mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of triangle K in the second mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of triangle K in the second mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, the end points of edge J in the first mesh have
            the same index I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of triangle K in the first mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of triangle K in the first mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of triangle K in the first mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            The function has detected only *<value>* coincident vertices
            with a precision `eps` = *<value>*. Either `eps` should be
            changed or the two meshes are not restitchable.

        (`errno` 3)
            A serious error has occurred in an internal call to the
            restitching routine. Check the input of the two meshes,
            especially the edges/vertices and/or the triangles/vertices
            connectivities. Seek expert help.

        (`errno` 4)
            The function has detected a different number of coincident
            triangles from the two meshes in the overlapping zone
            *<value>* *<value>*. Check the input of the two meshes,
            especially the triangles/vertices connectivity.

        (`errno` 5)
            The function has detected a different number of coincident
            edges from the two meshes on the partition interface
            *<value>* *<value>*. Check the input of the two meshes,
            especially the edges/vertices connectivity.
    """

    excs = []
    fun_name = 'naginterfaces.base.mesh.dim2_join'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d06dbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_INT, # Input, nv1.
        utils._BYREF_INT, # Input, nelt1.
        utils._BYREF_INT, # Input, nedge1.
        utils._EngineFloat64ArrayType.c_type, # Input, coor1[2, nv1].
        utils._EngineIntArrayType.c_type, # Input, edge1[3, nedge1].
        utils._EngineIntArrayType.c_type, # Input, conn1[3, nelt1].
        utils._EngineIntArrayType.c_type, # Input, reft1[nelt1].
        utils._BYREF_INT, # Input, nv2.
        utils._BYREF_INT, # Input, nelt2.
        utils._BYREF_INT, # Input, nedge2.
        utils._EngineFloat64ArrayType.c_type, # Input, coor2[2, nv2].
        utils._EngineIntArrayType.c_type, # Input, edge2[3, nedge2].
        utils._EngineIntArrayType.c_type, # Input, conn2[3, nelt2].
        utils._EngineIntArrayType.c_type, # Input, reft2[nelt2].
        utils._BYREF_INT, # Output, nv3.
        utils._BYREF_INT, # Output, nelt3.
        utils._BYREF_INT, # Output, nedge3.
        utils._EngineFloat64ArrayType.c_type, # Output, coor3[2, nv1+nv2].
        utils._EngineIntArrayType.c_type, # Output, edge3[3, nedge1+nedge2].
        utils._EngineIntArrayType.c_type, # Output, conn3[3, nelt1+nelt2].
        utils._EngineIntArrayType.c_type, # Output, reft3[nelt1+nelt2].
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((coor1, 'coor1'), (edge1, 'edge1'), (conn1, 'conn1'), (coor2, 'coor2'), (edge2, 'edge2'), (conn2, 'conn2'), (coor3, 'coor3'), (edge3, 'edge3'), (conn3, 'conn3')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _nv1 = utils._EngineIntScalarType(
        nv1, {'fun_name': fun_name, 'entity': 'nv1'},
    ).to_ctypes()
    _nelt1 = utils._EngineIntScalarType(
        nelt1, {'fun_name': fun_name, 'entity': 'nelt1'},
    ).to_ctypes()
    _nedge1 = utils._EngineIntScalarType(
        nedge1, {'fun_name': fun_name, 'entity': 'nedge1'},
    ).to_ctypes()
    _nv1_py = utils._EngineIntScalarType.to_py(_nv1)
    _coor1 = utils._EngineFloat64ArrayType(
        coor1, {'fun_name': fun_name, 'entity': 'coor1'},
        exp_shape=(2, _nv1_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nedge1_py = utils._EngineIntScalarType.to_py(_nedge1)
    _edge1 = utils._EngineIntArrayType(
        edge1, {'fun_name': fun_name, 'entity': 'edge1'},
        exp_shape=(3, _nedge1_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nelt1_py = utils._EngineIntScalarType.to_py(_nelt1)
    _conn1 = utils._EngineIntArrayType(
        conn1, {'fun_name': fun_name, 'entity': 'conn1'},
        exp_shape=(3, _nelt1_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _reft1 = utils._EngineIntArrayType(
        reft1, {'fun_name': fun_name, 'entity': 'reft1'},
        exp_shape=(_nelt1_py,),
    ).to_ctypes()
    _nv2 = utils._EngineIntScalarType(
        nv2, {'fun_name': fun_name, 'entity': 'nv2'},
    ).to_ctypes()
    _nelt2 = utils._EngineIntScalarType(
        nelt2, {'fun_name': fun_name, 'entity': 'nelt2'},
    ).to_ctypes()
    _nedge2 = utils._EngineIntScalarType(
        nedge2, {'fun_name': fun_name, 'entity': 'nedge2'},
    ).to_ctypes()
    _nv2_py = utils._EngineIntScalarType.to_py(_nv2)
    _coor2 = utils._EngineFloat64ArrayType(
        coor2, {'fun_name': fun_name, 'entity': 'coor2'},
        exp_shape=(2, _nv2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nedge2_py = utils._EngineIntScalarType.to_py(_nedge2)
    _edge2 = utils._EngineIntArrayType(
        edge2, {'fun_name': fun_name, 'entity': 'edge2'},
        exp_shape=(3, _nedge2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nelt2_py = utils._EngineIntScalarType.to_py(_nelt2)
    _conn2 = utils._EngineIntArrayType(
        conn2, {'fun_name': fun_name, 'entity': 'conn2'},
        exp_shape=(3, _nelt2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _reft2 = utils._EngineIntArrayType(
        reft2, {'fun_name': fun_name, 'entity': 'reft2'},
        exp_shape=(_nelt2_py,),
    ).to_ctypes()
    _nv3 = utils._EngineIntScalarType.empty_ctypes()
    _nelt3 = utils._EngineIntScalarType.empty_ctypes()
    _nedge3 = utils._EngineIntScalarType.empty_ctypes()
    _coor3_cls = utils._EngineFloat64ArrayType(
        coor3, {'fun_name': fun_name, 'entity': 'coor3'},
        exp_shape=(2, _nv1_py+_nv2_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _coor3 = _coor3_cls.to_ctypes(can_reference_contents=False)
    _edge3_cls = utils._EngineIntArrayType(
        edge3, {'fun_name': fun_name, 'entity': 'edge3'},
        exp_shape=(3, _nedge1_py+_nedge2_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _edge3 = _edge3_cls.to_ctypes(can_reference_contents=False)
    _conn3_cls = utils._EngineIntArrayType(
        conn3, {'fun_name': fun_name, 'entity': 'conn3'},
        exp_shape=(3, _nelt1_py+_nelt2_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _conn3 = _conn3_cls.to_ctypes(can_reference_contents=False)
    _reft3_cls = utils._EngineIntArrayType(
        reft3, {'fun_name': fun_name, 'entity': 'reft3'},
        exp_shape=(_nelt1_py+_nelt2_py,),
        check_contents=False,
    )
    _reft3 = _reft3_cls.to_ctypes(can_reference_contents=False)
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _eps,
        _nv1,
        _nelt1,
        _nedge1,
        _coor1,
        _edge1,
        _conn1,
        _reft1,
        _nv2,
        _nelt2,
        _nedge2,
        _coor2,
        _edge2,
        _conn2,
        _reft2,
        _nv3,
        _nelt3,
        _nedge3,
        _coor3,
        _edge3,
        _conn3,
        _reft3,
        _itrace,
        _nadv,
        _iwork,
        _liwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv2 >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge2 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edge2(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edge2(I,J) >= 1 and edge2(I,J) <= nv2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nelt2 = {:s} and nv2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nelt2 <= (2*nv2-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, conn2(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: conn2(I,J) >= 1 and conn2(I,J) <= nv2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv1 >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nedge1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nedge1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, edge1(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: edge1(I,J) >= 1 and edge1(I,J) <= nv1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nelt1 = {:s} and nv1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nelt1 <= (2*nv1-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, conn1(I,J) = {:s}, I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and nv1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: conn1(I,J) >= 1 and conn1(I,J) <= nv1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eps > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of edge J in the second mesh have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 2 and 3 of triangle K in the second mesh',
                    'have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 3 of triangle K in the second mesh',
                    'have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 2 of triangle K in the second mesh',
                    'have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the end points of edge J in the first mesh have the same index I:',
                    'J = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 2 and 3 of triangle K in the first mesh',
                    'have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 3 of triangle K in the first mesh',
                    'have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vertices 1 and 2 of triangle K in the first mesh',
                    'have the same index I:',
                    'K = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function has detected only {:s} coincident vertices'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'with a precision eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Either eps should be changed or the two meshes are not restitchable.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error has occurred in an internal call to the restitching',
                    'routine. Check the input of the two meshes, especially the',
                    'edges/vertices and/or the triangles/vertices',
                    'connectivities. Seek expert help.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function has detected a different number of coincident triangles from',
                    'the two meshes in the overlapping zone {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}. Check the input of the two meshes, especially the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'triangles/vertices connectivity.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function has detected a different number of coincident edges from',
                    'the two meshes on the partition interface {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}. Check the input of the two meshes, especially the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'edges/vertices connectivity.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _coor3_cls.refresh_py(_coor3)
        _edge3_cls.refresh_py(_edge3)
        _conn3_cls.refresh_py(_conn3)
        _reft3_cls.refresh_py(_reft3)
        _nv3_py = utils._EngineIntScalarType.to_py(_nv3)
        _nelt3_py = utils._EngineIntScalarType.to_py(_nelt3)
        _nedge3_py = utils._EngineIntScalarType.to_py(_nedge3)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nv3', 'nelt3', 'nedge3'],
        )(
            _nv3_py,
            _nelt3_py,
            _nedge3_py,
        )
    _err_class.manage()
    return return_data
