# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `mv` Chapter.

``mv`` - Multivariate Methods

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with methods for studying multivariate data.
A multivariate dataset consists of several variables recorded on a
number of objects or individuals.
Multivariate methods can be classified as those that seek to examine the
relationships between the variables (e.g., principal components), known
as variable-directed methods, and those that seek to examine the
relationships between the objects (e.g., cluster analysis), known as
individual-directed methods.

Multiple regression is not included in this module as it involves the
relationship of a single variable, known as the response variable, to
the other variables in the dataset, the explanatory variables.
Routines for multiple regression are provided in submodule ``correg``.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def prin_comp(matrix, std, n, m, x, isx, s, wt, nvar, e, p, v):
    u"""
    Performs principal component analysis.

    For full information please refer to the NAG Library document for
    g03aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03aaf.html

    Parameters
    ----------
    matrix : str, length 1
        Indicates for which type of matrix the principal component
        analysis is to be carried out.

        `matrix` = 'C'

          It is for the correlation matrix.

        `matrix` = 'S'

          It is for a standardized matrix, with standardizations given
          by `s`.

        `matrix` = 'U'

          It is for the sums of squares and cross-products matrix.

        `matrix` = 'V'

          It is for the variance-covariance matrix.

    std : str, length 1
        Indicates if the principal component scores are to be
        standardized.

        `std` = 'S'

          The principal component scores are standardized so that F'F =
          I, i.e., F = X_s PLambda^-1 = V.

        `std` = 'U'

          The principal component scores are unstandardized, i.e., F =
          X_s P = V Lambda.

        `std` = 'Z'

          The principal component scores are standardized so that they
          have unit variance.

        `std` = 'E'

          The principal component scores are standardized so that they
          have variance equal to the corresponding eigenvalue.

    n : int
        n, the number of observations.

    m : int
        m, the number of variables in the data matrix.

    x : float, shape (`n`, `m`)
        `x`[i-1,j-1] must contain the ith observation for the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

    isx : int, shape (`m`)
        `isx`[j-1] indicates whether or not the jth variable is to be
        included in the analysis.

        If `isx`[j-1] > 0, the variable contained in the jth column of
        `x` is included in the principal component analysis, for j =
        1,2,...,m.

    s : float, shape (`m`), modified in place
        On entry: the standardizations to be used, if any.

        If `matrix` = 'S', the first m elements of `s` must contain the
        standardization coefficients, the diagonal elements of sigma.

        On exit: if `matrix` = 'S', `s` is unchanged on exit.

        If `matrix` = 'C', `s` contains the variances of the selected
        variables. `s`[j-1] contains the variance of the variable in the
        jth column of `x` if `isx`[j-1] > 0.

        If `matrix` = 'U' or 'V', `s` is not referenced.

    wt : None or float, shape (`n`)
        If `wt` is not None, the first n elements of `wt` must contain
        the weights to be used in the principal component analysis.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        analysis.

        The effective number of observations is the sum of the weights.

        If `wt` is None the effective number of observations is n.

    nvar : int
        p, the number of variables in the principal component analysis.

    e : float, shape (`nvar`, 6), modified in place
        On exit: the statistics of the principal component analysis.

        `e`[i-1,0]

          The eigenvalues associated with the ith principal component,
          lambda_i^2, for i = 1,2,...,p.

        `e`[i-1,1]

          The proportion of variation explained by the ith principal
          component, for i = 1,2,...,p.

        `e`[i-1,2]

          The cumulative proportion of variation explained by the first
          ith principal components, for i = 1,2,...,p.

        `e`[i-1,3]

          The chi^2 statistics, for i = 1,2,...,p.

        `e`[i-1,4]

          The degrees of freedom for the chi^2 statistics, for i =
          1,2,...,p.

        If `matrix` != 'C', `e`[i-1,5] contains significance level for
        the chi^2 statistic, for i = 1,2,...,p.

        If `matrix` = 'C', `e`[i-1,5] is returned as zero.

    p : float, shape (`nvar`, `nvar`), modified in place
        On exit: the first `nvar` columns of `p` contain the principal
        component loadings, a_i. The jth column of `p` contains the
        `nvar` coefficients for the jth principal component.

    v : float, shape (`n`, `nvar`), modified in place
        On exit: the first `nvar` columns of `v` contain the principal
        component scores. The jth column of `v` contains the `n` scores
        for the jth principal component.

        If `wt` is not None, any rows for which `wt`[i-1] is zero will
        be set to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nvar` = *<value>* and `m` = *<value>*.

            Constraint: `nvar` <= `m`.

        (`errno` 1)
            On entry, `nvar` = *<value>*.

            Constraint: `nvar` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>* and `nvar` = *<value>*.

            Constraint: `n` > `nvar`.

        (`errno` 1)
            On entry, `std` = *<value>*.

            Constraint: `std` = 'E', 'S', 'U' or 'Z'.

        (`errno` 1)
            On entry, `matrix` = *<value>*.

            Constraint: `matrix` = 'C', 'S', 'U' or 'V'.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, i = *<value>* and `wt`[i-1] < 0.0.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 3)
            Number of selected variables >= effective number of
            observations.

        (`errno` 3)
            On entry, `nvar` = *<value>* and *<value>* values of `isx` >
            0.

            Constraint: exactly `nvar` elements of `isx` > 0.

        (`errno` 4)
            On entry, j = *<value>* and `s`[j-1] <= 0.0.

            Constraint: `s`[j-1] > 0.0.

        (`errno` 5)
            The singular value decomposition has failed to converge.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            All eigenvalues/singular values are zero. This will be
            caused by all the variables being constant.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.mv.prin_comp_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.prin_comp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, matrix(len=1).
        utils._BYREF_STRING, # Input, std(len=1).
        utils._BYREF_STRING, # Input, weight(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[m].
        utils._EngineFloat64ArrayType.c_type, # Input, wt[n].
        utils._BYREF_INT, # Input, nvar.
        utils._EngineFloat64ArrayType.c_type, # Output, e[lde, 6].
        utils._BYREF_INT, # Input, lde.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, nvar].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, nvar].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of matrix.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of std.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (e, 'e'), (p, 'p'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _matrix = utils._EngineCharScalarType(
        matrix, {'fun_name': fun_name, 'entity': 'matrix'}, exp_chrlen=1,
    ).to_ctypes()
    _std = utils._EngineCharScalarType(
        std, {'fun_name': fun_name, 'entity': 'std'}, exp_chrlen=1,
    ).to_ctypes()
    if wt is None:
        _weight = utils._EngineCharScalarType(
            'U', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _weight = utils._EngineCharScalarType(
            'W', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_m_py,),
    )
    _s = _s_cls.to_ctypes()
    _wt = utils._EngineFloat64ArrayType(
        wt, {'fun_name': fun_name, 'entity': 'wt'},
        exp_shape=(_n_py,),
        allow_none=True,
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _lde_py = utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_nvar_py, 6),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_nvar_py, _nvar_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _nvar_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _matrix,
        _std,
        _weight,
        _n,
        _m,
        _x,
        _ldx,
        _isx,
        _s,
        _wt,
        _nvar,
        _e,
        _lde,
        _p,
        _ldp,
        _v,
        _ldv,
        _errbuf,
        _ierr,
        _matrix._length_,
        _std._length_,
        _weight._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nvar <= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvar >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n > nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, std = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: std = \'E\', \'S\', \'U\' or \'Z\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, matrix = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: matrix = \'C\', \'S\', \'U\' or \'V\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and wt[i-1] < 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wt[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Number of selected variables >= effective number of observations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s} values of isx > 0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: exactly nvar elements of isx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, j = {:s} and s[j-1] <= 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: s[j-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The singular value decomposition has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All eigenvalues/singular values are zero.',
                    'This will be caused by all the variables being constant.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _s_cls.refresh_py(_s)
        _e_cls.refresh_py(_e)
        _p_cls.refresh_py(_p)
        _v_cls.refresh_py(_v)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def canon_var(weight, n, m, x, isx, ing, ng, wt, nig, cvm, e, cvx, tol):
    u"""
    Performs canonical variate analysis.

    For full information please refer to the NAG Library document for
    g03ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03acf.html

    Parameters
    ----------
    weight : str, length 1
        Indicates if weights are to be used.

        `weight` = 'U'

          No weights are used.

        `weight` = 'W' or 'V'

          Weights are used and must be supplied in `wt`.

        If `weight` = 'W', the weights are treated as frequencies and
        the effective number of observations is the sum of the weights.

        If `weight` = 'V', the weights are treated as being inversely
        proportional to the variance of the observations and the
        effective number of observations is the number of observations
        with nonzero weights.

    n : int
        n, the number of observations.

    m : int
        m, the total number of variables.

    x : float, shape (`n`, `m`)
        `x`[i-1,j-1] must contain the ith observation for the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

    isx : int, shape (`m`)
        `isx`[j-1] indicates whether or not the jth variable is to be
        included in the analysis.

        If `isx`[j-1] > 0, the variables contained in the jth column of
        `x` is included in the canonical variate analysis, for j =
        1,2,...,m.

    ing : int, shape (`n`)
        `ing`[i-1] indicates which group the ith observation is in, for
        i = 1,2,...,n. The effective number of groups is the number of
        groups with nonzero membership.

    ng : int
        The number of groups, n_g.

    wt : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `weight` in ('W', 'V'): `n`; otherwise: 1.

        If `weight` = 'W' or 'V', the first n elements of `wt` must
        contain the weights to be used in the analysis.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        analysis.

        If `weight` = 'U', `wt` is not referenced.

    nig : int, shape (`ng`), modified in place
        On exit: `nig`[j-1] gives the number of observations in group j,
        for j = 1,2,...,n_g.

    cvm : float, shape (`ng`, nx), modified in place
        On exit: `cvm`[i-1,j-1] contains the mean of the jth canonical
        variate for the ith group, for j = 1,2,...,l, for i =
        1,2,...,n_g; the remaining columns, if any, are used as
        workspace.

    e : float, shape (min(nx, `ng`-1), 6), modified in place
        On exit: the statistics of the canonical variate analysis.

        `e`[i-1,0]

          The canonical correlations, delta_i, for i = 1,2,...,l.

        `e`[i-1,1]

          The eigenvalues of the within-group sum of squares matrix,
          lambda_i^2, for i = 1,2,...,l.

        `e`[i-1,2]

          The proportion of variation explained by the ith canonical
          variate, for i = 1,2,...,l.

        `e`[i-1,3]

          The chi^2 statistic for the ith canonical variate, for i =
          1,2,...,l.

        `e`[i-1,4]

          The degrees of freedom for chi^2 statistic for the ith
          canonical variate, for i = 1,2,...,l.

        `e`[i-1,5]

          The significance level for the chi^2 statistic for the ith
          canonical variate, for i = 1,2,...,l.

    cvx : float, shape (nx, `ng`-1), modified in place
        On exit: the canonical variate loadings. `cvx`[i-1,j-1] contains
        the loading coefficient for the ith variable on the jth
        canonical variate, for j = 1,2,...,l, for i = 1,2,...,n_x; the
        remaining columns, if any, are used as workspace.

    tol : float
        The value of `tol` is used to decide if the variables are of
        full rank and, if not, what is the rank of the variables. The
        smaller the value of `tol` the stricter the criterion for
        selecting the singular value decomposition. If a non-negative
        value of `tol` less than machine precision is entered, the
        square root of machine precision is used instead.

    Returns
    -------
    ncv : int
        The number of canonical variates, l. This will be the minimum of
        n_g-1 and the rank of `x`.

    irankx : int
        The rank of the dependent variables.

        If the variables are of full rank then `irankx` = nx.

        If the variables are not of full rank then `irankx` is an
        estimate of the rank of the dependent variables. `irankx` is
        calculated as the number of singular values greater than
        `tol`*(largest singular value).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, `weight` = *<value>*.

            Constraint: `weight` = 'U', 'W' or 'V'.

        (`errno` 1)
            On entry, lde = *<value>* and min(nx, `ng`-1) = *<value>*.

            Constraint: lde >= min(nx, `ng`-1).

        (`errno` 1)
            On entry, `n` = *<value>* and nx+`ng` = *<value>*.

            Constraint: `n` >= nx+`ng`.

        (`errno` 1)
            On entry, `m` = *<value>* and nx = *<value>*.

            Constraint: `m` >= nx.

        (`errno` 1)
            On entry, `ng` = *<value>*.

            Constraint: `ng` >= 2.

        (`errno` 1)
            On entry, nx = *<value>*.

            Constraint: nx >= 1.

        (`errno` 2)
            On entry, i = *<value>* and `wt`[i-1] < 0.0.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 3)
            On entry, i = *<value>*, `ing`[i-1] = *<value>* and `ng` =
            *<value>*.

            Constraint: 1 <= `ing`[i-1] <= `ng`.

        (`errno` 4)
            On entry, nx = *<value>*, expected value = *<value>*.

            Constraint: nx must be consistent with `isx`.

        (`errno` 5)
            The singular value decomposition has failed to converge.

        (`errno` 7)
            The effective number of observations is less than the
            effective number of groups plus number of variables.

        (`errno` 7)
            Less than 2 groups have nonzero membership.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            A canonical correlation is equal to 1.0.

        (`errno` 8)
            The rank of `x` is 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.canon_var'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_STRING, # Input, weight(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._BYREF_INT, # Input, nx.
        utils._EngineIntArrayType.c_type, # Input, ing[n].
        utils._BYREF_INT, # Input, ng.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
        utils._EngineIntArrayType.c_type, # Output, nig[ng].
        utils._EngineFloat64ArrayType.c_type, # Output, cvm[ldcvm, nx].
        utils._BYREF_INT, # Input, ldcvm.
        utils._EngineFloat64ArrayType.c_type, # Output, e[lde, 6].
        utils._BYREF_INT, # Input, lde.
        utils._BYREF_INT, # Output, ncv.
        utils._EngineFloat64ArrayType.c_type, # Output, cvx[ldcvx, ng-1].
        utils._BYREF_INT, # Input, ldcvx.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Output, irankx.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._BYREF_INT, # Input, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (cvm, 'cvm'), (e, 'e'), (cvx, 'cvx')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _weight = utils._EngineCharScalarType(
        weight, {'fun_name': fun_name, 'entity': 'weight'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _vl_nx_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if utils._EngineIntArrayType.get_element(isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_nx_py = _vl_nx_py + 1
    _nx = utils._EngineIntScalarType(
        _vl_nx_py, {'fun_name': fun_name, 'entity': '_nx'},
    ).to_ctypes()
    _ing = utils._EngineIntArrayType(
        ing, {'fun_name': fun_name, 'entity': 'ing'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ng = utils._EngineIntScalarType(
        ng, {'fun_name': fun_name, 'entity': 'ng'},
    ).to_ctypes()
    _weight_py = utils._EngineCharScalarType.to_py(_weight)
    if _weight_py in ('W', 'w', 'V', 'v'):
        _wt_dim1 = _n_py
    else:
        _wt_dim1 = 1
    exp_shape_wt = (_wt_dim1,)
    _wt = utils._EngineFloat64ArrayType(
        wt, {'fun_name': fun_name, 'entity': 'wt'},
        exp_shape=exp_shape_wt,
    ).to_ctypes()
    _ng_py = utils._EngineIntScalarType.to_py(_ng)
    _nig_cls = utils._EngineIntArrayType(
        nig, {'fun_name': fun_name, 'entity': 'nig'},
        exp_shape=(_ng_py,),
        check_contents=False,
    )
    _nig = _nig_cls.to_ctypes(can_reference_contents=False)
    _ldcvm_py = utils._EngineFloat64ArrayType.get_nd(cvm, {'fun_name': fun_name, 'entity': 'cvm'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcvm = utils._EngineIntScalarType(
        _ldcvm_py, {'fun_name': fun_name, 'entity': '_ldcvm'},
    ).to_ctypes()
    _nx_py = utils._EngineIntScalarType.to_py(_nx)
    _cvm_cls = utils._EngineFloat64ArrayType(
        cvm, {'fun_name': fun_name, 'entity': 'cvm'},
        exp_shape=(_ng_py, _nx_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _cvm = _cvm_cls.to_ctypes(can_reference_contents=False)
    _lde_py = utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(min(_nx_py, _ng_py-1), 6),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _ncv = utils._EngineIntScalarType.empty_ctypes()
    _ldcvx_py = utils._EngineFloat64ArrayType.get_nd(cvx, {'fun_name': fun_name, 'entity': 'cvx'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcvx = utils._EngineIntScalarType(
        _ldcvx_py, {'fun_name': fun_name, 'entity': '_ldcvx'},
    ).to_ctypes()
    _cvx_cls = utils._EngineFloat64ArrayType(
        cvx, {'fun_name': fun_name, 'entity': 'cvx'},
        exp_shape=(_nx_py, _ng_py-1),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _cvx = _cvx_cls.to_ctypes(can_reference_contents=False)
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _irankx = utils._EngineIntScalarType.empty_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwk = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _weight,
        _n,
        _m,
        _x,
        _ldx,
        _isx,
        _nx,
        _ing,
        _ng,
        _wt,
        _nig,
        _cvm,
        _ldcvm,
        _e,
        _lde,
        _ncv,
        _cvx,
        _ldcvx,
        _tol,
        _irankx,
        _wk,
        _iwk,
        _errbuf,
        _ierr,
        _weight._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, weight = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: weight = \'U\', \'W\' or \'V\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lde = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'min(nx, ng-1) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lde >= min(nx, ng-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nx+ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= nx+ng.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nx.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ng >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nx >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and wt[i-1] < 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wt[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, ing[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= ing[i-1] <= ng.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nx = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'expected value = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nx must be consistent with isx.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The singular value decomposition has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A canonical correlation is equal to 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The effective number of observations is less than',
                    'the effective number of groups plus number of variables.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Less than 2 groups have nonzero membership.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99982:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The rank of x is 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _nig_cls.refresh_py(_nig)
        _cvm_cls.refresh_py(_cvm)
        _e_cls.refresh_py(_e)
        _cvx_cls.refresh_py(_cvx)
        _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
        _irankx_py = utils._EngineIntScalarType.to_py(_irankx)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ncv', 'irankx'],
        )(
            _ncv_py,
            _irankx_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def canon_corr(n, m, z, isz, nx, ny, wt, e, cvx, mcv, cvy, tol):
    u"""
    Performs canonical correlation analysis.

    For full information please refer to the NAG Library document for
    g03ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03adf.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    m : int
        m, the total number of variables.

    z : float, shape (`n`, `m`)
        `z`[i-1,j-1] must contain the ith observation for the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

        Both x and y variables are to be included in `z`, the indicator
        array, `isz`, being used to assign the variables in `z` to the x
        or y sets as appropriate.

    isz : int, shape (`m`)
        `isz`[j-1] indicates whether or not the jth variable is included
        in the analysis and to which set of variables it belongs.

        `isz`[j-1] > 0

          The variable contained in the jth column of `z` is included as
          an x variable in the analysis.

        `isz`[j-1] < 0

          The variable contained in the jth column of `z` is included as
          a y variable in the analysis.

        `isz`[j-1] = 0

          The variable contained in the jth column of `z` is not
          included in the analysis.

    nx : int
        The number of x variables in the analysis, n_x.

    ny : int
        The number of y variables in the analysis, n_y.

    wt : None or float, shape (`n`)
        The elements of `wt` must contain the weights to be used in the
        analysis. The effective number of observations is the sum of the
        weights. If `wt`[i-1] = 0.0 then the ith observation is not
        included in the analysis.

        If weights are not provided then `wt` must be set to None and
        the effective number of observations is `n`.

    e : float, shape (min(`nx`, `ny`), 6), modified in place
        On exit: the statistics of the canonical variate analysis.

        `e`[i-1,0]

          The canonical correlations, delta_i, for i = 1,2,...,l.

        `e`[i-1,1]

          The eigenvalues of Sigma, lambda_i^2, for i = 1,2,...,l.

        `e`[i-1,2]

          The proportion of variation explained by the ith canonical
          variate, for i = 1,2,...,l.

        `e`[i-1,3]

          The chi^2 statistic for the ith canonical variate, for i =
          1,2,...,l.

        `e`[i-1,4]

          The degrees of freedom for chi^2 statistic for the ith
          canonical variate, for i = 1,2,...,l.

        `e`[i-1,5]

          The significance level for the chi^2 statistic for the ith
          canonical variate, for i = 1,2,...,l.

    cvx : float, shape (`nx`, `mcv`), modified in place
        On exit: the canonical variate loadings for the x variables.
        `cvx`[i-1,j-1] contains the loading coefficient for the ith x
        variable on the jth canonical variate.

    mcv : int
        An upper limit to the number of canonical variates.

    cvy : float, shape (`ny`, `mcv`), modified in place
        On exit: the canonical variate loadings for the y variables.
        `cvy`[i-1,j-1] contains the loading coefficient for the ith y
        variable on the jth canonical variate.

    tol : float
        The value of `tol` is used to decide if the variables are of
        full rank and, if not, what is the rank of the variables. The
        smaller the value of `tol` the stricter the criterion for
        selecting the singular value decomposition. If a non-negative
        value of `tol` less than machine precision is entered, the
        square root of machine precision is used instead.

    Returns
    -------
    ncv : int
        The number of canonical correlations, l. This will be the
        minimum of the rank of X and the rank of Y.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mcv` = *<value>* and min(`nx`, `ny`) = *<value>*.

            Constraint: `mcv` >= min(`nx`, `ny`).

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'U' or 'W'.

        (`errno` 1)
            On entry, lde = *<value>* and min(`nx`, `ny`) = *<value>*.

            Constraint: lde >= min(`nx`, `ny`).

        (`errno` 1)
            On entry, `n` = *<value>* and `nx`+`ny` = *<value>*.

            Constraint: `n` > `nx`+`ny`.

        (`errno` 1)
            On entry, `m` = *<value>* and `nx`+`ny` = *<value>*.

            Constraint: `m` >= `nx`+`ny`.

        (`errno` 1)
            On entry, `ny` = *<value>*.

            Constraint: `ny` >= 1.

        (`errno` 1)
            On entry, `nx` = *<value>*.

            Constraint: `nx` >= 1.

        (`errno` 2)
            On entry, i = *<value>* and `wt`[i-1] < 0.0.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 3)
            On entry, `ny` = *<value>*, expected value = *<value>*.

            Constraint: `ny` must be consistent with `isz`.

        (`errno` 3)
            On entry, `nx` = *<value>*, expected value = *<value>*.

            Constraint: `nx` must be consistent with `isz`.

        (`errno` 4)
            On entry, the effective number of observations is less than
            `nx`+`ny`+1.

        (`errno` 5)
            The singular value decomposition has failed to converge.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            A canonical correlation is equal to 1.0.

        (`errno` 7)
            On entry, the rank of the Y matrix is 0.

        (`errno` 7)
            On entry, the rank of the X matrix is 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.canon_corr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_STRING, # Input, weight(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, z[ldz, m].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Input, isz[m].
        utils._BYREF_INT, # Input, nx.
        utils._BYREF_INT, # Input, ny.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[n].
        utils._EngineFloat64ArrayType.c_type, # Output, e[lde, 6].
        utils._BYREF_INT, # Input, lde.
        utils._BYREF_INT, # Output, ncv.
        utils._EngineFloat64ArrayType.c_type, # Output, cvx[ldcvx, mcv].
        utils._BYREF_INT, # Input, ldcvx.
        utils._BYREF_INT, # Input, mcv.
        utils._EngineFloat64ArrayType.c_type, # Output, cvy[ldcvy, mcv].
        utils._BYREF_INT, # Input, ldcvy.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._BYREF_INT, # Input, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (e, 'e'), (cvx, 'cvx'), (cvy, 'cvy')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _iemode = utils._EngineIntScalarType.c_type(0)
    if wt is None:
        _weight = utils._EngineCharScalarType(
            'U', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _weight = utils._EngineCharScalarType(
            'W', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _z = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isz = utils._EngineIntArrayType(
        isz, {'fun_name': fun_name, 'entity': 'isz'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _nx = utils._EngineIntScalarType(
        nx, {'fun_name': fun_name, 'entity': 'nx'},
    ).to_ctypes()
    _ny = utils._EngineIntScalarType(
        ny, {'fun_name': fun_name, 'entity': 'ny'},
    ).to_ctypes()
    _wt = utils._EngineFloat64ArrayType(
        wt, {'fun_name': fun_name, 'entity': 'wt'},
        exp_shape=(_n_py,),
        allow_none=True,
    ).to_ctypes()
    _lde_py = utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _nx_py = utils._EngineIntScalarType.to_py(_nx)
    _ny_py = utils._EngineIntScalarType.to_py(_ny)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(min(_nx_py, _ny_py), 6),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _ncv = utils._EngineIntScalarType.empty_ctypes()
    _ldcvx_py = utils._EngineFloat64ArrayType.get_nd(cvx, {'fun_name': fun_name, 'entity': 'cvx'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcvx = utils._EngineIntScalarType(
        _ldcvx_py, {'fun_name': fun_name, 'entity': '_ldcvx'},
    ).to_ctypes()
    _mcv = utils._EngineIntScalarType(
        mcv, {'fun_name': fun_name, 'entity': 'mcv'},
    ).to_ctypes()
    _mcv_py = utils._EngineIntScalarType.to_py(_mcv)
    _cvx_cls = utils._EngineFloat64ArrayType(
        cvx, {'fun_name': fun_name, 'entity': 'cvx'},
        exp_shape=(_nx_py, _mcv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _cvx = _cvx_cls.to_ctypes(can_reference_contents=False)
    _ldcvy_py = utils._EngineFloat64ArrayType.get_nd(cvy, {'fun_name': fun_name, 'entity': 'cvy'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcvy = utils._EngineIntScalarType(
        _ldcvy_py, {'fun_name': fun_name, 'entity': '_ldcvy'},
    ).to_ctypes()
    _cvy_cls = utils._EngineFloat64ArrayType(
        cvy, {'fun_name': fun_name, 'entity': 'cvy'},
        exp_shape=(_ny_py, _mcv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _cvy = _cvy_cls.to_ctypes(can_reference_contents=False)
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwk = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _weight,
        _n,
        _m,
        _z,
        _ldz,
        _isz,
        _nx,
        _ny,
        _wt,
        _e,
        _lde,
        _ncv,
        _cvx,
        _ldcvx,
        _mcv,
        _cvy,
        _ldcvy,
        _tol,
        _wk,
        _iwk,
        _errbuf,
        _ierr,
        _weight._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mcv = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'min(nx, ny) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mcv >= min(nx, ny).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, weight = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: weight = \'U\' or \'W\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lde = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'min(nx, ny) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lde >= min(nx, ny).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and nx+ny = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n > nx+ny.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nx+ny = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nx+ny.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ny = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ny >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nx >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and wt[i-1] < 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wt[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ny = {:s}, expected value = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ny must be consistent with isz.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nx = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'expected value = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nx must be consistent with isz.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the effective number of observations is less than nx+ny+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The singular value decomposition has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A canonical correlation is equal to 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the rank of the Y matrix is 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the rank of the X matrix is 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _e_cls.refresh_py(_e)
        _cvx_cls.refresh_py(_cvx)
        _cvy_cls.refresh_py(_cvy)
        _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
        return_data = _ncv_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['ncv'],
            )(
                _ncv_py,
            )
    _err_class.manage()
    return return_data

def rot_orthomax(stand, g, nvar, k, fl, flr, r, acc, maxit):
    u"""
    Computes orthogonal rotations for loading matrix, generalized
    orthomax criterion.

    For full information please refer to the NAG Library document for
    g03ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03baf.html

    Parameters
    ----------
    stand : str, length 1
        Indicates if the matrix of loadings is to be row standardized
        before rotation.

        `stand` = 'S'

          The loadings are row standardized.

        `stand` = 'U'

          The loadings are left unstandardized.

    g : float
        gamma, the criterion constant with gamma = 1.0 giving varimax
        rotations and gamma = 0.0 giving quartimax rotations.

    nvar : int
        p, the number of original variables.

    k : int
        k, the number of derived variates or factors.

    fl : float, shape (`nvar`, `k`), modified in place
        On entry: Lambda, the matrix of loadings. `fl`[i-1,j-1] must
        contain the loading for the ith variable on the jth factor, for
        j = 1,2,...,k, for i = 1,2,...,p.

        On exit: if `stand` = 'S', the elements of `fl` are standardized
        so that the sum of squared elements for each row is 1.0 and then
        after the computation of the rotations are rescaled; this may
        lead to slight differences between the input and output values
        of `fl`.

        If `stand` = 'U', `fl` will be unchanged on exit.

    flr : float, shape (`nvar`, `k`), modified in place
        On exit: the rotated matrix of loadings, Lambda*. `flr`[i-1,j-1]
        will contain the rotated loading for the ith variable on the jth
        factor, for j = 1,2,...,k, for i = 1,2,...,p.

    r : float, shape (`k`, `k`), modified in place
        On exit: the matrix of rotations, R.

    acc : float
        Indicates the accuracy required. The iterative procedure of
        Cooley and Lohnes will be stopped and the final refinement
        computed when the change in V is less than `acc`*max(1.0, V). If
        `acc` is greater than or equal to 0.0 but less than machine
        precision or if `acc` is greater than 1.0, machine precision
        will be used instead.

    maxit : int
        The maximum number of iterations.

    Returns
    -------
    itera : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 1)
            On entry, `nvar` = *<value>* and `k` = *<value>*.

            Constraint: `nvar` >= `k`.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 2.

        (`errno` 1)
            On entry, `g` = *<value>*.

            Constraint: `g` >= 0.0.

        (`errno` 1)
            On entry, `stand` = *<value>*.

            Constraint: `stand` = 'S' or 'U'.

        (`errno` 1)
            On entry, `acc` = *<value>*.

            Constraint: `acc` >= 0.0.

        (`errno` 2)
            The singular value decomposition has failed to converge.

        (`errno` 3)
            The algorithm to find R has failed to reach the required
            accuracy in `maxit` iterations.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.rot_orthomax'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, stand(len=1).
        utils._BYREF_FLOAT64, # Input, g.
        utils._BYREF_INT, # Input, nvar.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fl[ldfl, k].
        utils._BYREF_INT, # Input, ldfl.
        utils._EngineFloat64ArrayType.c_type, # Output, flr[ldfl, k].
        utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, k].
        utils._BYREF_INT, # Input, ldr.
        utils._BYREF_FLOAT64, # Input, acc.
        utils._BYREF_INT, # Input, maxit.
        utils._BYREF_INT, # Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of stand.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fl, 'fl'), (flr, 'flr'), (r, 'r')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _stand = utils._EngineCharScalarType(
        stand, {'fun_name': fun_name, 'entity': 'stand'}, exp_chrlen=1,
    ).to_ctypes()
    _g = utils._EngineFloat64ScalarType(
        g, {'fun_name': fun_name, 'entity': 'g'},
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _ldfl_py = utils._EngineFloat64ArrayType.get_shape(fl, {'fun_name': fun_name, 'entity': 'fl'}, exp_rank=2)[0]
    _ldfl = utils._EngineIntScalarType(
        _ldfl_py, {'fun_name': fun_name, 'entity': '_ldfl'},
    ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _fl_cls = utils._EngineFloat64ArrayType(
        fl, {'fun_name': fun_name, 'entity': 'fl'},
        exp_shape=(_nvar_py, _k_py),
        mutual_sorder=_m_sorder,
    )
    _fl = _fl_cls.to_ctypes()
    _flr_cls = utils._EngineFloat64ArrayType(
        flr, {'fun_name': fun_name, 'entity': 'flr'},
        exp_shape=(_nvar_py, _k_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _flr = _flr_cls.to_ctypes(can_reference_contents=False)
    _ldr_py = utils._EngineFloat64ArrayType.get_nd(r, {'fun_name': fun_name, 'entity': 'r'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldr = utils._EngineIntScalarType(
        _ldr_py, {'fun_name': fun_name, 'entity': '_ldr'},
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_k_py, _k_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _acc = utils._EngineFloat64ScalarType(
        acc, {'fun_name': fun_name, 'entity': 'acc'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType.empty_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _stand,
        _g,
        _nvar,
        _k,
        _fl,
        _ldfl,
        _flr,
        _r,
        _ldr,
        _acc,
        _maxit,
        _itera,
        _wk,
        _errbuf,
        _ierr,
        _stand._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nvar >= k.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, g = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: g >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, stand = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: stand = \'S\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: acc >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The singular value decomposition has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm to find R has failed to reach the required',
                    'accuracy in maxit iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fl_cls.refresh_py(_fl)
        _flr_cls.refresh_py(_flr)
        _r_cls.refresh_py(_r)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        return_data = _itera_py
    _err_class.manage()
    return return_data

def rot_procrustes(stand, pscale, n, m, x, y, yhat, r, res):
    u"""
    Computes Procrustes rotations.

    For full information please refer to the NAG Library document for
    g03bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03bcf.html

    Parameters
    ----------
    stand : str, length 1
        Indicates if translation/normalization is required.

        `stand` = 'N'

          There is no translation or normalization.

        `stand` = 'Z'

          There is translation to the origin.

        `stand` = 'C'

          There is translation to origin and then to the Y centroid
          after rotation.

        `stand` = 'U'

          There is unit normalization.

        `stand` = 'S'

          There is translation and normalization.

        `stand` = 'M'

          There is translation and normalization to Y scale, then
          translation to the Y centroid after rotation.

    pscale : str, length 1
        Indicates if least squares scaling is to be applied after
        rotation.

        `pscale` = 'S'

          Scaling is applied.

        `pscale` = 'U'

          No scaling is applied.

    n : int
        n, the number of points.

    m : int
        m, the number of dimensions.

    x : float, shape (`n`, `m`), modified in place
        On entry: X, the matrix to be rotated.

        On exit: if `stand` = 'N', `x` will be unchanged.

        If `stand` = 'Z', 'C', 'S' or 'M', `x` will be translated to
        have zero column means.

        If `stand` = 'U' or 'S', `x` will be scaled to have unit sum of
        squares.

        If `stand` = 'M', `x` will be scaled to have the same sum of
        squares as `y`.

    y : float, shape (`n`, `m`), modified in place
        On entry: the target matrix, `y`.

        On exit: if `stand` = 'N', `y` will be unchanged.

        If `stand` = 'Z' or 'S', `y` will be translated to have zero
        column means.

        If `stand` = 'U' or 'S', `y` will be scaled to have unit sum of
        squares.

        If `stand` = 'C' or 'M', `y` will be translated and then after
        rotation translated back.

        The output `y` should be the same as the input `y` except for
        rounding errors.

    yhat : float, shape (`n`, `m`), modified in place
        On exit: the fitted matrix, Y^.

    r : float, shape (`m`, `m`), modified in place
        On exit: the matrix of rotations, R, see [Further Comments].

    res : float, shape (`n`), modified in place
        On exit: the residuals, r_i, for i = 1,2,...,n.

    Returns
    -------
    alpha : float
        If `pscale` = 'S' the scaling factor, alpha; otherwise `alpha`
        is not set.

    rss : float
        The residual sum of squares.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `pscale` = *<value>*.

            Constraint: `pscale` = 'S' or 'U'.

        (`errno` 1)
            On entry, `stand` = *<value>*.

            Constraint: `stand` = 'N', 'Z', 'C', 'U', 'S' or 'M'.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.

        (`errno` 1)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` 2)
            Only one distinct point (centred at zero) in `y` array.

        (`errno` 2)
            Only one distinct point (centred at zero) in `x` array.

        (`errno` 3)
            `yhat` contains only zero-points when least squares scaling
            is applied.

        (`errno` 4)
            The singular value decomposition has failed to converge.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.rot_procrustes'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03bcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, stand(len=1).
        utils._BYREF_STRING, # Input, pscale(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[ldy, m].
        utils._BYREF_INT, # Input, ldy.
        utils._EngineFloat64ArrayType.c_type, # Output, yhat[ldy, m].
        utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, m].
        utils._BYREF_INT, # Input, ldr.
        utils._BYREF_FLOAT64, # Output, alpha.
        utils._BYREF_FLOAT64, # Output, rss.
        utils._EngineFloat64ArrayType.c_type, # Output, res[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of stand.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of pscale.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (yhat, 'yhat'), (r, 'r')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _stand = utils._EngineCharScalarType(
        stand, {'fun_name': fun_name, 'entity': 'stand'}, exp_chrlen=1,
    ).to_ctypes()
    _pscale = utils._EngineCharScalarType(
        pscale, {'fun_name': fun_name, 'entity': 'pscale'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineFloat64ArrayType.get_shape(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2)[0]
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _yhat_cls = utils._EngineFloat64ArrayType(
        yhat, {'fun_name': fun_name, 'entity': 'yhat'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _yhat = _yhat_cls.to_ctypes(can_reference_contents=False)
    _ldr_py = utils._EngineFloat64ArrayType.get_nd(r, {'fun_name': fun_name, 'entity': 'r'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldr = utils._EngineIntScalarType(
        _ldr_py, {'fun_name': fun_name, 'entity': '_ldr'},
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _alpha = utils._EngineFloat64ScalarType.empty_ctypes()
    _rss = utils._EngineFloat64ScalarType.empty_ctypes()
    _res_cls = utils._EngineFloat64ArrayType(
        res, {'fun_name': fun_name, 'entity': 'res'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _res = _res_cls.to_ctypes(can_reference_contents=False)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _stand,
        _pscale,
        _n,
        _m,
        _x,
        _ldx,
        _y,
        _ldy,
        _yhat,
        _r,
        _ldr,
        _alpha,
        _rss,
        _res,
        _wk,
        _errbuf,
        _ierr,
        _stand._length_,
        _pscale._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, pscale = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: pscale = \'S\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, stand = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: stand = \'N\', \'Z\', \'C\', \'U\', \'S\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Only one distinct point (centred at zero) in y array.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Only one distinct point (centred at zero) in x array.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'yhat contains only zero-points when least squares scaling is applied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The singular value decomposition has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _yhat_cls.refresh_py(_yhat)
        _r_cls.refresh_py(_r)
        _res_cls.refresh_py(_res)
        _alpha_py = utils._EngineFloat64ScalarType.to_py(_alpha)
        _rss_py = utils._EngineFloat64ScalarType.to_py(_rss)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['alpha', 'rss'],
        )(
            _alpha_py,
            _rss_py,
        )
    _err_class.manage()
    return return_data

def rot_promax(stand, n, m, x, ro, power, fp, r, phi, fs, io_manager=None):
    u"""
    ProMax rotations.

    For full information please refer to the NAG Library document for
    g03bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03bdf.html

    Parameters
    ----------
    stand : str, length 1
        Indicates how loadings are normalized.

        `stand` = 'S'

          Rows of Y are (Kaiser) normalized by the communalities of the
          loadings.

        `stand` = 'U'

          Rows are not normalized.

    n : int
        n, the number of points.

    m : int
        m, the number of dimensions.

    x : float, shape (`n`, `m`)
        The loadings matrix following an orthogonal rotation, X.

    ro : float, shape (`m`, `m`)
        The orthogonal rotation matrix, O.

    power : float
        p, the value of exponent.

    fp : float, shape (`n`, `m`), modified in place
        On exit: the factor pattern matrix, P.

    r : float, shape (`m`, `m`), modified in place
        On exit: the ProMax rotation matrix, R.

    phi : float, shape (`m`, `m`), modified in place
        On exit: the matrix of inter-factor correlations, Phi.

    fs : float, shape (`n`, `m`), modified in place
        On exit: the factor structure matrix, S.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `stand` = *<value>*.

            Constraint: `stand` = 'U' or 'S'.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 1)
            On entry, `power` = *<value>*.

            Constraint: `power` > 1.0.

        (`errno` 2)
            On entry, `n` = *<value>* and `m` = *<value>*.

            Constraint: `n` >= `m`.

        (`errno` 20)
            The singular value decomposition has failed to converge.

        (`errno` 100)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.rot_promax'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03bdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, stand(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input, ro[ldro, m].
        utils._BYREF_INT, # Input, ldro.
        utils._BYREF_FLOAT64, # Input, power.
        utils._EngineFloat64ArrayType.c_type, # Output, fp[ldfp, m].
        utils._BYREF_INT, # Input, ldfp.
        utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, m].
        utils._BYREF_INT, # Input, ldr.
        utils._EngineFloat64ArrayType.c_type, # Output, phi[ldphi, m].
        utils._BYREF_INT, # Input, ldphi.
        utils._EngineFloat64ArrayType.c_type, # Output, fs[ldfs, m].
        utils._BYREF_INT, # Input, ldfs.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of stand.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (ro, 'ro'), (fp, 'fp'), (r, 'r'), (phi, 'phi'), (fs, 'fs')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _stand = utils._EngineCharScalarType(
        stand, {'fun_name': fun_name, 'entity': 'stand'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldro_py = utils._EngineFloat64ArrayType.get_nd(ro, {'fun_name': fun_name, 'entity': 'ro'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldro = utils._EngineIntScalarType(
        _ldro_py, {'fun_name': fun_name, 'entity': '_ldro'},
    ).to_ctypes()
    _ro = utils._EngineFloat64ArrayType(
        ro, {'fun_name': fun_name, 'entity': 'ro'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _power = utils._EngineFloat64ScalarType(
        power, {'fun_name': fun_name, 'entity': 'power'},
    ).to_ctypes()
    _ldfp_py = utils._EngineFloat64ArrayType.get_nd(fp, {'fun_name': fun_name, 'entity': 'fp'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfp = utils._EngineIntScalarType(
        _ldfp_py, {'fun_name': fun_name, 'entity': '_ldfp'},
    ).to_ctypes()
    _fp_cls = utils._EngineFloat64ArrayType(
        fp, {'fun_name': fun_name, 'entity': 'fp'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fp = _fp_cls.to_ctypes(can_reference_contents=False)
    _ldr_py = utils._EngineFloat64ArrayType.get_nd(r, {'fun_name': fun_name, 'entity': 'r'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldr = utils._EngineIntScalarType(
        _ldr_py, {'fun_name': fun_name, 'entity': '_ldr'},
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _ldphi_py = utils._EngineFloat64ArrayType.get_nd(phi, {'fun_name': fun_name, 'entity': 'phi'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldphi = utils._EngineIntScalarType(
        _ldphi_py, {'fun_name': fun_name, 'entity': '_ldphi'},
    ).to_ctypes()
    _phi_cls = utils._EngineFloat64ArrayType(
        phi, {'fun_name': fun_name, 'entity': 'phi'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _phi = _phi_cls.to_ctypes(can_reference_contents=False)
    _ldfs_py = utils._EngineFloat64ArrayType.get_nd(fs, {'fun_name': fun_name, 'entity': 'fs'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfs = utils._EngineIntScalarType(
        _ldfs_py, {'fun_name': fun_name, 'entity': '_ldfs'},
    ).to_ctypes()
    _fs_cls = utils._EngineFloat64ArrayType(
        fs, {'fun_name': fun_name, 'entity': 'fs'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fs = _fs_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _stand,
        _n,
        _m,
        _x,
        _ldx,
        _ro,
        _ldro,
        _power,
        _fp,
        _ldfp,
        _r,
        _ldr,
        _phi,
        _ldphi,
        _fs,
        _ldfs,
        _errbuf,
        _ierr,
        _stand._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, stand = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: stand = \'U\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, power = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: power > 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 20 and _ifmt == 110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The singular value decomposition has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 100 and _ifmt == 111:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred in this function.',
                    'Check the function call and any array sizes.',
                    'If the call is correct then please contact NAG for assistance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fp_cls.refresh_py(_fp)
        _r_cls.refresh_py(_r)
        _phi_cls.refresh_py(_phi)
        _fs_cls.refresh_py(_fs)
        return_data = None
    _err_class.manage()
    return return_data

def factor(matrix, n, m, x, nvar, isx, nfac, wt, e, stat, com, psi, res, fl, iop, io_manager=None):
    u"""
    Computes maximum likelihood estimates of the parameters of a factor
    analysis model, factor loadings, communalities and residual
    correlations.

    For full information please refer to the NAG Library document for
    g03ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03caf.html

    Parameters
    ----------
    matrix : str, length 1
        Selects the type of matrix on which factor analysis is to be
        performed.

        `matrix` = 'D'

          The data matrix will be input in `x` and factor analysis will
          be computed for the correlation matrix.

        `matrix` = 'S'

          The data matrix will be input in `x` and factor analysis will
          be computed for the covariance matrix, i.e., the results are
          scaled as described in [Further Comments].

        `matrix` = 'C'

          The correlation/variance-covariance matrix will be input in
          `x` and factor analysis computed for this matrix.

        See [Further Comments].

    n : int
        If `matrix` = 'D' or 'S' the number of observations in the data
        array `x`.

        If `matrix` = 'C' the (effective) number of observations used in
        computing the (possibly weighted)
        correlation/variance-covariance matrix input in `x`.

    m : int
        The number of variables in the
        data/correlation/variance-covariance matrix.

    x : float, shape (:, `m`)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `matrix` in ('D', 'S'): `n`; if
        `matrix`='C': `m`; otherwise: 0.

        The input matrix.

        If `matrix` = 'D' or 'S', `x` must contain the data matrix,
        i.e., `x`[i-1,j-1] must contain the ith observation for the jth
        variable, for j = 1,2,...,`m`, for i = 1,2,...,n.

        If `matrix` = 'C', `x` must contain the correlation or
        variance-covariance matrix.

        Only the upper triangular part is required.

    nvar : int
        p, the number of variables in the factor analysis.

    isx : int, shape (`m`)
        `isx`[j-1] indicates whether or not the jth variable is included
        in the factor analysis. If `isx`[j-1] >= 1, the variable
        represented by the jth column of `x` is included in the
        analysis; otherwise it is excluded, for j = 1,2,...,`m`.

    nfac : int
        k, the number of factors.

    wt : None or float, shape (:)
        Note: the required length for this argument is determined as
        follows: if weight='W' and `matrix` in ('D', 'S'): `n`;
        otherwise: 1.

        If `wt` is not None and `matrix` = 'D' or 'S', `wt` must contain
        the weights to be used in the factor analysis. The effective
        number of observations in the analysis will then be the sum of
        weights. If `wt`[i-1] = 0.0, the ith observation is not included
        in the analysis.

        If weight = 'U' or `matrix` = 'C', `wt` is not referenced and
        the effective number of observations is n.

    e : float, shape (`nvar`), modified in place
        On exit: the eigenvalues theta_i, for i = 1,2,...,p.

    stat : float, shape (4), modified in place
        On exit: the test statistics.

        `stat`[0]

          Contains the value F(Psi^).

        `stat`[1]

          Contains the test statistic, chi^2.

        `stat`[2]

          Contains the degrees of freedom associated with the test
          statistic.

        `stat`[3]

          Contains the significance level.

    com : float, shape (`nvar`), modified in place
        On exit: the communalities.

    psi : float, shape (`nvar`), modified in place
        On exit: the estimates of psi_i, for i = 1,2,...,p.

    res : float, shape (`nvar`*(`nvar`-1)/2), modified in place
        On exit: the residual correlations. The residual correlation for
        the ith and jth variables is stored in `res`[ (j-1) (j-2) /2+i
        -1], i < j.

    fl : float, shape (`nvar`, `nfac`), modified in place
        On exit: the factor loadings. `fl`[i-1,j-1] contains
        lambda_{ij}, for j = 1,2,...,k, for i = 1,2,...,p.

    iop : int, shape (5)
        Options for the optimization. There are four options to be set:

        [table omitted]

        Let epsilon = machine precision then if `iop`[0] = 0, the
        following default values are used:

        - iprint = -1

        - maxfun = 100p

        - acc = 10sqrt(epsilon)

        - eps = epsilon

        If `iop`[0] != 0, then

        - iprint = `iop`[1]

        - maxfun = `iop`[2]

        - acc = 10^(-l) where l = `iop`[3]

        - eps = 10^(-l) where l = `iop`[4]

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `iop`[0] != 1 and `iop`[4] = *<value>*.

            Constraint: 1 <= eps <= machine precision.

        (`errno` 1)
            On entry, `iop`[0] != 1 and `iop`[3] = *<value>*.

            Constraint: 1 <= acc <= machine precision.

        (`errno` 1)
            On entry, `iop`[0] != 1 and `iop`[2] = *<value>*.

            Constraint: maxfun >= 1.

        (`errno` 1)
            On entry, `nfac` = *<value>* and `nvar` = *<value>*.

            Constraint: `nfac` <= `nvar`.

        (`errno` 1)
            On entry, `nfac` = *<value>*.

            Constraint: `nfac` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>* and `nvar` = *<value>*.

            Constraint: `m` >= `nvar`.

        (`errno` 1)
            On entry, `matrix` = *<value>*.

            Constraint: `matrix` = 'D', 'S' or 'C'.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: when `matrix` = 'D' or 'S', weight = 'U' or 'W'.

        (`errno` 1)
            On entry, `n` = *<value>* and `nvar` = *<value>*.

            Constraint: `n` > `nvar`.

        (`errno` 1)
            On entry, `nvar` = *<value>*.

            Constraint: `nvar` > 1.

        (`errno` 2)
            On entry, i = *<value>* and `wt`[i-1] < 0.0.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 3)
            The effective number of observations <= 1.

        (`errno` 3)
            The number of variables >= number of included observations.

        (`errno` 3)
            On entry, `nvar` = *<value>* and *<value>* values of `isx` >
            0

            Constraint: exactly `nvar` elements of `isx` > 0.

        (`errno` 4)
            Two eigenvalues of S* are equal.

        (`errno` 4)
            On entry, the data matrix is not of full column rank or the
            input correlation/covariance matrix is not positive
            definite.

        (`errno` 5)
            The singular value decomposition has failed to converge.

        (`errno` 6)
            The estimation procedure has failed to converge in *<value>*
            iterations.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 7)
            The convergence is not certain but a lower point could not
            be found.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.factor'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_STRING, # Input, matrix(len=1).
        utils._BYREF_STRING, # Input, weight(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_INT, # Input, nvar.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._BYREF_INT, # Input, nfac.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
        utils._EngineFloat64ArrayType.c_type, # Output, e[nvar].
        utils._EngineFloat64ArrayType.c_type, # Output, stat[4].
        utils._EngineFloat64ArrayType.c_type, # Output, com[nvar].
        utils._EngineFloat64ArrayType.c_type, # Output, psi[nvar].
        utils._EngineFloat64ArrayType.c_type, # Output, res[nvar*(nvar-1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, fl[ldfl, nfac].
        utils._BYREF_INT, # Input, ldfl.
        utils._EngineIntArrayType.c_type, # Input, iop[5].
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._BYREF_INT, # Input, lwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of matrix.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (fl, 'fl')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _matrix = utils._EngineCharScalarType(
        matrix, {'fun_name': fun_name, 'entity': 'matrix'}, exp_chrlen=1,
    ).to_ctypes()
    if wt is None:
        _weight = utils._EngineCharScalarType(
            'U', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _weight = utils._EngineCharScalarType(
            'W', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _matrix_py = utils._EngineCharScalarType.to_py(_matrix)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if _matrix_py in ('D', 'd', 'S', 's'):
        _x_dim1 = _n_py
    elif _matrix_py in ('C', 'c'):
        _x_dim1 = _m_py
    else:
        _x_dim1 = 0
    exp_shape_x = (_x_dim1, _m_py)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _nfac = utils._EngineIntScalarType(
        nfac, {'fun_name': fun_name, 'entity': 'nfac'},
    ).to_ctypes()
    _weight_py = utils._EngineCharScalarType.to_py(_weight)
    if wt is None:
        if (_weight_py in ('W', 'w')) and _matrix_py in ('D', 'd', 'S', 's'):
            _wt_dim1 = _n_py
        else:
            _wt_dim1 = 1
        _wt = utils._EngineFloat64ArrayType.empty_ctypes(_wt_dim1)
    else:
        if (_weight_py in ('W', 'w')) and _matrix_py in ('D', 'd', 'S', 's'):
            _wt_dim1 = _n_py
        else:
            _wt_dim1 = 1
        exp_shape_wt = (_wt_dim1,)
        _wt = utils._EngineFloat64ArrayType(
            wt, {'fun_name': fun_name, 'entity': 'wt'},
            exp_shape=exp_shape_wt,
            allow_none=True,
        ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_nvar_py,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _stat_cls = utils._EngineFloat64ArrayType(
        stat, {'fun_name': fun_name, 'entity': 'stat'},
        exp_shape=(4,),
        check_contents=False,
    )
    _stat = _stat_cls.to_ctypes(can_reference_contents=False)
    _com_cls = utils._EngineFloat64ArrayType(
        com, {'fun_name': fun_name, 'entity': 'com'},
        exp_shape=(_nvar_py,),
        check_contents=False,
    )
    _com = _com_cls.to_ctypes(can_reference_contents=False)
    _psi_cls = utils._EngineFloat64ArrayType(
        psi, {'fun_name': fun_name, 'entity': 'psi'},
        exp_shape=(_nvar_py,),
        check_contents=False,
    )
    _psi = _psi_cls.to_ctypes(can_reference_contents=False)
    _res_cls = utils._EngineFloat64ArrayType(
        res, {'fun_name': fun_name, 'entity': 'res'},
        exp_shape=(_nvar_py*(_nvar_py-1)//2,),
        check_contents=False,
    )
    _res = _res_cls.to_ctypes(can_reference_contents=False)
    _ldfl_py = utils._EngineFloat64ArrayType.get_nd(fl, {'fun_name': fun_name, 'entity': 'fl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfl = utils._EngineIntScalarType(
        _ldfl_py, {'fun_name': fun_name, 'entity': '_ldfl'},
    ).to_ctypes()
    _nfac_py = utils._EngineIntScalarType.to_py(_nfac)
    _fl_cls = utils._EngineFloat64ArrayType(
        fl, {'fun_name': fun_name, 'entity': 'fl'},
        exp_shape=(_nvar_py, _nfac_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fl = _fl_cls.to_ctypes(can_reference_contents=False)
    _iop = utils._EngineIntArrayType(
        iop, {'fun_name': fun_name, 'entity': 'iop'},
        exp_shape=(5,),
    ).to_ctypes()
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwk = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _iemode,
        _matrix,
        _weight,
        _n,
        _m,
        _x,
        _ldx,
        _nvar,
        _isx,
        _nfac,
        _wt,
        _e,
        _stat,
        _com,
        _psi,
        _res,
        _fl,
        _ldfl,
        _iop,
        _iwk,
        _wk,
        _lwk,
        _errbuf,
        _ierr,
        _matrix._length_,
        _weight._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iop[0] != 1 and iop[4] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= eps <= machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iop[0] != 1 and iop[3] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= acc <= machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iop[0] != 1 and iop[2] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxfun >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nfac = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nfac <= nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nfac = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nfac >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, matrix = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: matrix = \'D\', \'S\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, weight = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when matrix = \'D\' or \'S\', weight = \'U\' or \'W\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n > nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvar > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and wt[i-1] < 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wt[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The effective number of observations <= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of variables >= number of included observations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} values of isx > 0'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: exactly nvar elements of isx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Two eigenvalues of S* are equal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the data matrix is not of full column rank or',
                    'the input correlation/covariance matrix is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The singular value decomposition has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The estimation procedure has failed to',
                    'converge in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99979:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The convergence is not certain but a lower point could not be found.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _e_cls.refresh_py(_e)
        _stat_cls.refresh_py(_stat)
        _com_cls.refresh_py(_com)
        _psi_cls.refresh_py(_psi)
        _res_cls.refresh_py(_res)
        _fl_cls.refresh_py(_fl)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def factor_score(method, rotate, nvar, nfac, fl, psi, e, r, fs):
    u"""
    Computes factor score coefficients (for use after ``factor``).

    For full information please refer to the NAG Library document for
    g03cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03ccf.html

    Parameters
    ----------
    method : str, length 1
        Indicates which method is to be used to compute the factor score
        coefficients.

        `method` = 'R'

          The regression method is used.

        `method` = 'B'

          Bartlett's method is used.

    rotate : str, length 1
        Indicates whether a rotation is to be applied.

        `rotate` = 'R'

          A rotation will be applied to the coefficients and the
          rotation matrix, R, must be given in `r`.

        `rotate` = 'U'

          No rotation is applied.

    nvar : int
        p, the number of observed variables in the factor analysis.

    nfac : int
        k, the number of factors in the factor analysis.

    fl : float, shape (`nvar`, `nfac`)
        Lambda, the matrix of unrotated factor loadings as returned by
        ``factor``.

    psi : float, shape (`nvar`)
        The diagonal elements of Psi, as returned by ``factor``.

    e : float, shape (`nvar`)
        The eigenvalues of the matrix S*, as returned by ``factor``.

    r : float, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `rotate`='R': `nfac`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `rotate`='U': 1; if `rotate`='R':
        `nfac`; otherwise: 0.

        If `rotate` = 'R', `r` must contain the orthogonal rotation
        matrix, R, as returned by ``rot_orthomax``.

        If `rotate` = 'U', `r` need not be set.

    fs : float, shape (`nvar`, `nfac`), modified in place
        On exit: the matrix of factor score coefficients, Phi.
        `fs`[i-1,j-1] contains the factor score coefficient for the jth
        factor and the ith observed variable, for j = 1,2,...,k, for i =
        1,2,...,p.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `rotate` = *<value>*.

            Constraint: `rotate` = 'R' or 'U'.

        (`errno` 1)
            On entry, `method` = *<value>*

            Constraint: `method` = 'B' or 'R'.

        (`errno` 1)
            On entry, `nvar` = *<value>* and `nfac` = *<value>*.

            Constraint: `nvar` >= `nfac`.

        (`errno` 1)
            On entry, `nfac` = *<value>*.

            Constraint: `nfac` >= 1.

        (`errno` 2)
            On entry, i = *<value>* and `psi`[i-1] <= 0.0.

            Constraint: `psi`[i-1] > 0.0.

        (`errno` 2)
            On entry, i = *<value>* and `e`[i-1] <= 1.0.

            Constraint: `e`[i-1] > 1.0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.factor_score'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, method(len=1).
        utils._BYREF_STRING, # Input, rotate(len=1).
        utils._BYREF_INT, # Input, nvar.
        utils._BYREF_INT, # Input, nfac.
        utils._EngineFloat64ArrayType.c_type, # Input, fl[ldfl, nfac].
        utils._BYREF_INT, # Input, ldfl.
        utils._EngineFloat64ArrayType.c_type, # Input, psi[nvar].
        utils._EngineFloat64ArrayType.c_type, # Input, e[nvar].
        utils._EngineFloat64ArrayType.c_type, # Input, r[ldr, *].
        utils._BYREF_INT, # Input, ldr.
        utils._EngineFloat64ArrayType.c_type, # Output, fs[ldfs, nfac].
        utils._BYREF_INT, # Input, ldfs.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of method.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of rotate.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fl, 'fl'), (r, 'r'), (fs, 'fs')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _method = utils._EngineCharScalarType(
        method, {'fun_name': fun_name, 'entity': 'method'}, exp_chrlen=1,
    ).to_ctypes()
    _rotate = utils._EngineCharScalarType(
        rotate, {'fun_name': fun_name, 'entity': 'rotate'}, exp_chrlen=1,
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _nfac = utils._EngineIntScalarType(
        nfac, {'fun_name': fun_name, 'entity': 'nfac'},
    ).to_ctypes()
    _ldfl_py = utils._EngineFloat64ArrayType.get_nd(fl, {'fun_name': fun_name, 'entity': 'fl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfl = utils._EngineIntScalarType(
        _ldfl_py, {'fun_name': fun_name, 'entity': '_ldfl'},
    ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _nfac_py = utils._EngineIntScalarType.to_py(_nfac)
    _fl = utils._EngineFloat64ArrayType(
        fl, {'fun_name': fun_name, 'entity': 'fl'},
        exp_shape=(_nvar_py, _nfac_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _psi = utils._EngineFloat64ArrayType(
        psi, {'fun_name': fun_name, 'entity': 'psi'},
        exp_shape=(_nvar_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_nvar_py,),
    ).to_ctypes()
    _ldr_py = utils._EngineFloat64ArrayType.get_nd(r, {'fun_name': fun_name, 'entity': 'r'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldr = utils._EngineIntScalarType(
        _ldr_py, {'fun_name': fun_name, 'entity': '_ldr'},
    ).to_ctypes()
    _rotate_py = utils._EngineCharScalarType.to_py(_rotate)
    if _rotate_py in ('R', 'r'):
        _r_dim1 = _nfac_py
    else:
        _r_dim1 = 1
    if _rotate_py in ('U', 'u'):
        _r_dim2 = 1
    elif _rotate_py in ('R', 'r'):
        _r_dim2 = _nfac_py
    else:
        _r_dim2 = 0
    exp_shape_r = (_r_dim1, _r_dim2)
    _r = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=exp_shape_r,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldfs_py = utils._EngineFloat64ArrayType.get_nd(fs, {'fun_name': fun_name, 'entity': 'fs'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfs = utils._EngineIntScalarType(
        _ldfs_py, {'fun_name': fun_name, 'entity': '_ldfs'},
    ).to_ctypes()
    _fs_cls = utils._EngineFloat64ArrayType(
        fs, {'fun_name': fun_name, 'entity': 'fs'},
        exp_shape=(_nvar_py, _nfac_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fs = _fs_cls.to_ctypes(can_reference_contents=False)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _method,
        _rotate,
        _nvar,
        _nfac,
        _fl,
        _ldfl,
        _psi,
        _e,
        _r,
        _ldr,
        _fs,
        _ldfs,
        _wk,
        _errbuf,
        _ierr,
        _method._length_,
        _rotate._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rotate = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rotate = \'R\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = \'{:s}\''.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: method = \'B\' or \'R\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and nfac = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nvar >= nfac.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nfac = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nfac >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and psi[i-1] <= 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: psi[i-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and e[i-1] <= 1.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: e[i-1] > 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fs_cls.refresh_py(_fs)
        return_data = None
    _err_class.manage()
    return return_data

def discrim(n, m, x, isx, nvar, ing, ng, wt, nig, gmn, det, gc):
    u"""
    Computes test statistic for equality of within-group covariance
    matrices and matrices for discriminant analysis.

    For full information please refer to the NAG Library document for
    g03da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03daf.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    m : int
        The number of variables in the data array `x`.

    x : float, shape (`n`, `m`)
        `x`[k-1,l-1] must contain the kth observation for the lth
        variable, for l = 1,2,...,`m`, for k = 1,2,...,n.

    isx : int, shape (`m`)
        `isx`[l-1] indicates whether or not the lth variable in `x` is
        to be included in the variance-covariance matrices.

        If `isx`[l-1] > 0 the lth variable is included, for l =
        1,2,...,`m`; otherwise it is not referenced.

    nvar : int
        p, the number of variables in the variance-covariance matrices.

    ing : int, shape (`n`)
        `ing`[k-1] indicates to which group the kth observation belongs,
        for k = 1,2,...,n.

    ng : int
        The number of groups, n_g.

    wt : None or float, shape (`n`)
        The elements of `wt` must contain the weights to be used in the
        analysis and the effective number of observations for a group is
        the sum of the weights of the observations in that group. If
        `wt`[k-1] = 0.0 then the kth observation is excluded from the
        calculations.

        If weights are not provided then `wt` must be set to None and
        the effective number of observations for a group is the number
        of observations in that group.

    nig : int, shape (`ng`), modified in place
        On exit: `nig`[j-1] contains the number of observations in the
        jth group, for j = 1,2,...,n_g.

    gmn : float, shape (`ng`, `nvar`), modified in place
        On exit: the jth row of `gmn` contains the means of the p
        selected variables for the jth group, for j = 1,2,...,n_g.

    det : float, shape (`ng`), modified in place
        On exit: the logarithm of the determinants of the within-group
        variance-covariance matrices.

    gc : float, shape ((`ng`+1)*`nvar`*(`nvar`+1)/2), modified in place
        On exit: the first p(p+1)/2 elements of `gc` contain R and the
        remaining n_g blocks of p(p+1)/2 elements contain the R_j
        matrices. All are stored in packed form by columns.

    Returns
    -------
    stat : float
        The likelihood-ratio test statistic, G.

    df : float
        The degrees of freedom for the distribution of G.

    sig : float
        The significance level for G.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'U' or 'W'.

        (`errno` 1)
            On entry, `m` = *<value>* and `nvar` = *<value>*.

            Constraint: `m` >= `nvar`.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `ng` = *<value>*.

            Constraint: `ng` >= 2.

        (`errno` 1)
            On entry, `nvar` = *<value>*.

            Constraint: `nvar` >= 1.

        (`errno` 2)
            On entry, i = *<value>* and `wt`[i-1] < 0.0.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 3)
            The number of observations for group *<value>* is less than
            `nvar`.

        (`errno` 3)
            The effective number of observations for group *<value>* is
            less than 1.

        (`errno` 3)
            On entry, i = *<value>*, `ing`[i-1] = *<value>* and `ng` =
            *<value>*.

            Constraint: 1 <= `ing`[i-1] <= `ng`.

        (`errno` 3)
            On entry, `nvar` = *<value>* and *<value>* values of `isx` >
            0

            Constraint: exactly `nvar` elements of `isx` > 0.

        (`errno` 4)
            R is not of full rank.

        (`errno` 4)
            R_j is not of full rank for j = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.mv.discrim_group_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.discrim'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, weight(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._BYREF_INT, # Input, nvar.
        utils._EngineIntArrayType.c_type, # Input, ing[n].
        utils._BYREF_INT, # Input, ng.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[n].
        utils._EngineIntArrayType.c_type, # Output, nig[ng].
        utils._EngineFloat64ArrayType.c_type, # Output, gmn[ldgmn, nvar].
        utils._BYREF_INT, # Input, ldgmn.
        utils._EngineFloat64ArrayType.c_type, # Output, det[ng].
        utils._EngineFloat64ArrayType.c_type, # Output, gc[(ng+1)*nvar*(nvar+1)//2].
        utils._BYREF_FLOAT64, # Output, stat.
        utils._BYREF_FLOAT64, # Output, df.
        utils._BYREF_FLOAT64, # Output, sig.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (gmn, 'gmn')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    if wt is None:
        _weight = utils._EngineCharScalarType(
            'U', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _weight = utils._EngineCharScalarType(
            'W', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _ing = utils._EngineIntArrayType(
        ing, {'fun_name': fun_name, 'entity': 'ing'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ng = utils._EngineIntScalarType(
        ng, {'fun_name': fun_name, 'entity': 'ng'},
    ).to_ctypes()
    if wt is None:
        _wt = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _wt = utils._EngineFloat64ArrayType(
            wt, {'fun_name': fun_name, 'entity': 'wt'},
            exp_shape=(_n_py,),
            allow_none=True,
        ).to_ctypes()
    _ng_py = utils._EngineIntScalarType.to_py(_ng)
    _nig_cls = utils._EngineIntArrayType(
        nig, {'fun_name': fun_name, 'entity': 'nig'},
        exp_shape=(_ng_py,),
        check_contents=False,
    )
    _nig = _nig_cls.to_ctypes(can_reference_contents=False)
    _ldgmn_py = utils._EngineFloat64ArrayType.get_nd(gmn, {'fun_name': fun_name, 'entity': 'gmn'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldgmn = utils._EngineIntScalarType(
        _ldgmn_py, {'fun_name': fun_name, 'entity': '_ldgmn'},
    ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _gmn_cls = utils._EngineFloat64ArrayType(
        gmn, {'fun_name': fun_name, 'entity': 'gmn'},
        exp_shape=(_ng_py, _nvar_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gmn = _gmn_cls.to_ctypes(can_reference_contents=False)
    _det_cls = utils._EngineFloat64ArrayType(
        det, {'fun_name': fun_name, 'entity': 'det'},
        exp_shape=(_ng_py,),
        check_contents=False,
    )
    _det = _det_cls.to_ctypes(can_reference_contents=False)
    _gc_cls = utils._EngineFloat64ArrayType(
        gc, {'fun_name': fun_name, 'entity': 'gc'},
        exp_shape=((_ng_py+1)*_nvar_py*(_nvar_py+1)//2,),
        check_contents=False,
    )
    _gc = _gc_cls.to_ctypes(can_reference_contents=False)
    _stat = utils._EngineFloat64ScalarType.empty_ctypes()
    _df = utils._EngineFloat64ScalarType.empty_ctypes()
    _sig = utils._EngineFloat64ScalarType.empty_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _weight,
        _n,
        _m,
        _x,
        _ldx,
        _isx,
        _nvar,
        _ing,
        _ng,
        _wt,
        _nig,
        _gmn,
        _ldgmn,
        _det,
        _gc,
        _stat,
        _df,
        _sig,
        _wk,
        _iwk,
        _errbuf,
        _ierr,
        _weight._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, weight = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: weight = \'U\' or \'W\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ng >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvar >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and wt[i-1] < 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wt[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of observations for group {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'is less than nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The effective number of observations for group {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'is less than 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, ing[i-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= ing[i-1] <= ng.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} values of isx > 0'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: exactly nvar elements of isx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'R is not of full rank.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'R_j is not of full rank for j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _nig_cls.refresh_py(_nig)
        _gmn_cls.refresh_py(_gmn)
        _det_cls.refresh_py(_det)
        _gc_cls.refresh_py(_gc)
        _stat_py = utils._EngineFloat64ScalarType.to_py(_stat)
        _df_py = utils._EngineFloat64ScalarType.to_py(_df)
        _sig_py = utils._EngineFloat64ScalarType.to_py(_sig)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['stat', 'df', 'sig'],
        )(
            _stat_py,
            _df_py,
            _sig_py,
        )
    _err_class.manage()
    return return_data

def discrim_mahal(equal, mode, nvar, ng, gmn, gc, nobs, m, isx, x, d):
    u"""
    Computes Mahalanobis squared distances for group or pooled
    variance-covariance matrices (for use after ``discrim``).

    For full information please refer to the NAG Library document for
    g03db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03dbf.html

    Parameters
    ----------
    equal : str, length 1
        Indicates whether or not the within-group variance-covariance
        matrices are assumed to be equal and the pooled
        variance-covariance matrix used.

        `equal` = 'E'

          The within-group variance-covariance matrices are assumed
          equal and the matrix R stored in the first p(p+1)/2 elements
          of `gc` is used.

        `equal` = 'U'

          The within-group variance-covariance matrices are assumed to
          be unequal and the matrices R_j, for j = 1,2,...,n_g, stored
          in the remainder of `gc` are used.

    mode : str, length 1
        Indicates whether distances from sample points are to be
        calculated or distances between the group means.

        `mode` = 'S'

          The distances between the sample points given in `x` and the
          group means are calculated.

        `mode` = 'M'

          The distances between the group means will be calculated.

    nvar : int
        p, the number of variables in the variance-covariance matrices
        as specified to ``discrim``.

    ng : int
        The number of groups, n_g.

    gmn : float, shape (`ng`, `nvar`)
        The jth row of `gmn` contains the means of the p selected
        variables for the jth group, for j = 1,2,...,n_g. These are
        returned by ``discrim``.

    gc : float, shape ((`ng`+1)*`nvar`*(`nvar`+1)/2)
        The first p(p+1)/2 elements of `gc` should contain the upper
        triangular matrix R and the next n_g blocks of p(p+1)/2 elements
        should contain the upper triangular matrices R_j. All matrices
        must be stored packed by column. These matrices are returned by
        ``discrim``. If `equal` = 'E' only the first p(p+1)/2 elements
        are referenced, if `equal` = 'U' only the elements p(p+1)/2+1 to
        (n_g+1)p(p+1)/2 are referenced.

    nobs : int
        If `mode` = 'S', the number of sample points in `x` for which
        distances are to be calculated.

        If `mode` = 'M', `nobs` is not referenced.

    m : int
        If `mode` = 'S', the number of variables in the data array `x`.

        If `mode` = 'M', `m` is not referenced.

    isx : int, shape (`m`)
        If `mode` = 'S', `isx`[l-1] indicates if the lth variable in `x`
        is to be included in the distance calculations. If `isx`[l-1] >
        0 the lth variable is included, for l = 1,2,...,`m`; otherwise
        the lth variable is not referenced.

        If `mode` = 'M', `isx` is not referenced.

    x : float, shape (:, `m`)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `mode`='S': `nobs`; otherwise: 1.

        If `mode` = 'S' the kth row of `x` must contain x_k. That is
        `x`[k-1,l-1] must contain the kth sample value for the lth
        variable, for l = 1,2,...,`m`, for k = 1,2,...,`nobs`. Otherwise
        `x` is not referenced.

    d : float, shape (:, `ng`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `mode`='S': `nobs`; if `mode`='M':
        `ng`; otherwise: 0.

        On exit: the squared distances.

        If `mode` = 'S', `d`[k-1,j-1] contains the squared distance of
        the kth sample point from the jth group mean, D_kj^2, for j =
        1,2,...,n_g, for k = 1,2,...,`nobs`.

        If `mode` = 'M' and `equal` = 'U', `d`[i-1,j-1] contains the
        squared distance between the ith mean and the jth mean, D_ij^2,
        for j = 1,2,...,i-1,i+1,...,n_g, for i = 1,2,...,n_g.

        The elements `d`[i-1,i-1] are not referenced, for i =
        1,2,...,n_g.

        If `mode` = 'M' and `equal` = 'E', `d`[i-1,j-1] contains the
        squared distance between the ith mean and the jth mean, D_ij^2,
        for j = 1,2,...,i-1, for i = 1,2,...,n_g.

        Since D_{ij} = D_{ji} the elements `d`[i-1,j-1] are not
        referenced, for j = i+1,...,n_g, for i = 1,2,...,n_g.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 'M' or 'S'.

        (`errno` 1)
            On entry, `equal` = *<value>*.

            Constraint: `equal` = 'E' or 'U'.

        (`errno` 1)
            On entry, `m` = *<value>* and `nvar` = *<value>*.

            Constraint: `m` >= `nvar`.

        (`errno` 1)
            On entry, `nobs` = *<value>*.

            Constraint: `nobs` >= 1.

        (`errno` 1)
            On entry, `ng` = *<value>*.

            Constraint: `ng` >= 2.

        (`errno` 1)
            On entry, `nvar` = *<value>*.

            Constraint: `nvar` >= 1.

        (`errno` 2)
            On entry, `nvar` = *<value>* and *<value>* values of `isx` >
            0.

            Constraint: exactly `nvar` elements of `isx` > 0.

        (`errno` 2)
            On entry, diagonal element *<value>* of R_j = 0 for j =
            *<value>*.

        (`errno` 2)
            On entry, diagonal element *<value>* of R = 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.discrim_mahal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03dbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, equal(len=1).
        utils._BYREF_STRING, # Input, mode(len=1).
        utils._BYREF_INT, # Input, nvar.
        utils._BYREF_INT, # Input, ng.
        utils._EngineFloat64ArrayType.c_type, # Input, gmn[ldgmn, nvar].
        utils._BYREF_INT, # Input, ldgmn.
        utils._EngineFloat64ArrayType.c_type, # Input, gc[(ng+1)*nvar*(nvar+1)//2].
        utils._BYREF_INT, # Input, nobs.
        utils._BYREF_INT, # Input, m.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, d[ldd, ng].
        utils._BYREF_INT, # Input, ldd.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mode.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((gmn, 'gmn'), (x, 'x'), (d, 'd')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _equal = utils._EngineCharScalarType(
        equal, {'fun_name': fun_name, 'entity': 'equal'}, exp_chrlen=1,
    ).to_ctypes()
    _mode = utils._EngineCharScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'}, exp_chrlen=1,
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _ng = utils._EngineIntScalarType(
        ng, {'fun_name': fun_name, 'entity': 'ng'},
    ).to_ctypes()
    _ldgmn_py = utils._EngineFloat64ArrayType.get_nd(gmn, {'fun_name': fun_name, 'entity': 'gmn'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldgmn = utils._EngineIntScalarType(
        _ldgmn_py, {'fun_name': fun_name, 'entity': '_ldgmn'},
    ).to_ctypes()
    _ng_py = utils._EngineIntScalarType.to_py(_ng)
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _gmn = utils._EngineFloat64ArrayType(
        gmn, {'fun_name': fun_name, 'entity': 'gmn'},
        exp_shape=(_ng_py, _nvar_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _gc = utils._EngineFloat64ArrayType(
        gc, {'fun_name': fun_name, 'entity': 'gc'},
        exp_shape=((_ng_py+1)*_nvar_py*(_nvar_py+1)//2,),
    ).to_ctypes()
    _nobs = utils._EngineIntScalarType(
        nobs, {'fun_name': fun_name, 'entity': 'nobs'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _mode_py = utils._EngineCharScalarType.to_py(_mode)
    _nobs_py = utils._EngineIntScalarType.to_py(_nobs)
    if _mode_py in ('S', 's'):
        _x_dim1 = _nobs_py
    else:
        _x_dim1 = 1
    exp_shape_x = (_x_dim1, _m_py)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldd_py = utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldd = utils._EngineIntScalarType(
        _ldd_py, {'fun_name': fun_name, 'entity': '_ldd'},
    ).to_ctypes()
    if _mode_py in ('S', 's'):
        _d_dim1 = _nobs_py
    elif _mode_py in ('M', 'm'):
        _d_dim1 = _ng_py
    else:
        _d_dim1 = 0
    exp_shape_d = (_d_dim1, _ng_py)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=exp_shape_d,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _equal,
        _mode,
        _nvar,
        _ng,
        _gmn,
        _ldgmn,
        _gc,
        _nobs,
        _m,
        _isx,
        _x,
        _ldx,
        _d,
        _ldd,
        _wk,
        _errbuf,
        _ierr,
        _equal._length_,
        _mode._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode = \'M\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, equal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: equal = \'E\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nobs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nobs >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ng >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvar >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s} values of isx > 0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: exactly nvar elements of isx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, diagonal element {:s} of R_j = 0'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'for j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, diagonal element {:s} of R = 0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        return_data = None
    _err_class.manage()
    return return_data

def discrim_group(typ, equal, priors, nvar, ng, nig, gmn, gc, det, nobs, m, isx, x, prior, p, iag, atiq, ati):
    u"""
    Allocates observations to groups according to selected rules (for
    use after ``discrim``).

    For full information please refer to the NAG Library document for
    g03dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03dcf.html

    Parameters
    ----------
    typ : str, length 1
        Whether the estimative or predictive approach is used.

        `typ` = 'E'

          The estimative approach is used.

        `typ` = 'P'

          The predictive approach is used.

    equal : str, length 1
        Indicates whether or not the within-group variance-covariance
        matrices are assumed to be equal and the pooled
        variance-covariance matrix used.

        `equal` = 'E'

          The within-group variance-covariance matrices are assumed
          equal and the matrix R stored in the first p(p+1)/2 elements
          of `gc` is used.

        `equal` = 'U'

          The within-group variance-covariance matrices are assumed to
          be unequal and the matrices R_i, for i = 1,2,...,n_g, stored
          in the remainder of `gc` are used.

    priors : str, length 1
        Indicates the form of the prior probabilities to be used.

        `priors` = 'E'

          Equal prior probabilities are used.

        `priors` = 'P'

          Prior probabilities proportional to the group sizes in the
          training set, n_j, are used.

        `priors` = 'I'

          The prior probabilities are input in `prior`.

    nvar : int
        p, the number of variables in the variance-covariance matrices.

    ng : int
        The number of groups, n_g.

    nig : int, shape (`ng`)
        The number of observations in each group in the training set,
        n_j.

    gmn : float, shape (`ng`, `nvar`)
        The jth row of `gmn` contains the means of the p variables for
        the jth group, for j = 1,2,...,n_j. These are returned by
        ``discrim``.

    gc : float, shape ((`ng`+1)*`nvar`*(`nvar`+1)/2)
        The first p(p+1)/2 elements of `gc` should contain the upper
        triangular matrix R and the next n_g blocks of p(p+1)/2 elements
        should contain the upper triangular matrices R_j.

        All matrices must be stored packed by column.

        These matrices are returned by ``discrim``.

        If `equal` = 'E' only the first p(p+1)/2 elements are
        referenced, if `equal` = 'U' only the elements p(p+1)/2+1 to
        (n_g+1)p(p+1)/2 are referenced.

    det : float, shape (`ng`)
        If `equal` = 'U'. the logarithms of the determinants of the
        within-group variance-covariance matrices as returned by
        ``discrim``. Otherwise `det` is not referenced.

    nobs : int
        The number of observations in `x` which are to be allocated.

    m : int
        The number of variables in the data array `x`.

    isx : int, shape (`m`)
        `isx`[l-1] indicates if the lth variable in `x` is to be
        included in the distance calculations.

        If `isx`[l-1] > 0, the lth variable is included, for l =
        1,2,...,`m`; otherwise the lth variable is not referenced.

    x : float, shape (`nobs`, `m`)
        `x`[k-1,l-1] must contain the kth observation for the lth
        variable, for l = 1,2,...,`m`, for k = 1,2,...,`nobs`.

    prior : float, shape (`ng`), modified in place
        On entry: if `priors` = 'I', the prior probabilities for the n_g
        groups.

        On exit: if `priors` = 'P', the computed prior probabilities in
        proportion to group sizes for the n_g groups.

        If `priors` = 'I', the input prior probabilities will be
        unchanged.

        If `priors` = 'E', `prior` is not set.

    p : float, shape (`nobs`, `ng`), modified in place
        On exit: `p`[k-1,j-1] contains the posterior probability p_{kj}
        for allocating the kth observation to the jth group, for j =
        1,2,...,n_g, for k = 1,2,...,`nobs`.

    iag : int, shape (`nobs`), modified in place
        On exit: the groups to which the observations have been
        allocated.

    atiq : bool
        `atiq` must be True if atypicality indices are required. If
        `atiq` is False the array `ati` is not set.

    ati : float, shape (`nobs`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `atiq`= True: `ng`; otherwise: 1.

        On exit: if `atiq` is True, `ati`[k-1,j-1] will contain the
        predictive atypicality index for the kth observation with
        respect to the jth group, for j = 1,2,...,n_g, for k =
        1,2,...,`nobs`.

        If `atiq` is False, `ati` is not set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `typ` = *<value>*.

            Constraint: `typ` = 'E' or 'P'.

        (`errno` 1)
            On entry, `priors` = *<value>*.

            Constraint: `priors` = 'E', 'I' or 'P'.

        (`errno` 1)
            On entry, `equal` = *<value>*.

            Constraint: `equal` = 'E' or 'U'.

        (`errno` 1)
            On entry, `m` = *<value>* and `nvar` = *<value>*.

            Constraint: `m` >= `nvar`.

        (`errno` 1)
            On entry, `nobs` = *<value>*.

            Constraint: `nobs` >= 1.

        (`errno` 1)
            On entry, `ng` = *<value>*.

            Constraint: `ng` >= 2.

        (`errno` 1)
            On entry, `nvar` = *<value>*.

            Constraint: `nvar` >= 1.

        (`errno` 2)
            On entry, i = *<value>* and `nig`[i-1] = *<value>*.

            Constraint: `nig`[i-1] > 0.

        (`errno` 2)
            On entry, sum_i(`nig`[i]) <= `ng`+`nvar`.

        (`errno` 2)
            On entry, i = *<value>*, `nig`[i-1] = *<value>* and `nvar` =
            *<value>*.

            Constraint: `nig`[i-1] < `nvar`.

        (`errno` 2)
            On entry, `nvar` = *<value>* and *<value>* values of `isx` >
            0.

            Constraint: exactly `nvar` elements of `isx` > 0.

        (`errno` 3)
            On entry, sum_j(`prior`[j]) != 1.0.

        (`errno` 3)
            On entry, j = *<value>* and `prior`[j-1] = *<value>*.

            Constraint: `prior`[j-1] > 0.

        (`errno` 4)
            On entry, a diagonal element of R or R_j is zero.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.mv.discrim_group_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.discrim_group'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03dcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, typ(len=1).
        utils._BYREF_STRING, # Input, equal(len=1).
        utils._BYREF_STRING, # Input, priors(len=1).
        utils._BYREF_INT, # Input, nvar.
        utils._BYREF_INT, # Input, ng.
        utils._EngineIntArrayType.c_type, # Input, nig[ng].
        utils._EngineFloat64ArrayType.c_type, # Input, gmn[ldgmn, nvar].
        utils._BYREF_INT, # Input, ldgmn.
        utils._EngineFloat64ArrayType.c_type, # Input, gc[(ng+1)*nvar*(nvar+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input, det[ng].
        utils._BYREF_INT, # Input, nobs.
        utils._BYREF_INT, # Input, m.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, prior[ng].
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, ng].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineIntArrayType.c_type, # Output, iag[nobs].
        utils._BYREF_LOGICAL, # Input, atiq.
        utils._EngineFloat64ArrayType.c_type, # Output, ati[ldati, *].
        utils._BYREF_INT, # Input, ldati.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of typ.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of equal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of priors.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((gmn, 'gmn'), (x, 'x'), (p, 'p'), (ati, 'ati')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _typ = utils._EngineCharScalarType(
        typ, {'fun_name': fun_name, 'entity': 'typ'}, exp_chrlen=1,
    ).to_ctypes()
    _equal = utils._EngineCharScalarType(
        equal, {'fun_name': fun_name, 'entity': 'equal'}, exp_chrlen=1,
    ).to_ctypes()
    _priors = utils._EngineCharScalarType(
        priors, {'fun_name': fun_name, 'entity': 'priors'}, exp_chrlen=1,
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _ng = utils._EngineIntScalarType(
        ng, {'fun_name': fun_name, 'entity': 'ng'},
    ).to_ctypes()
    _ng_py = utils._EngineIntScalarType.to_py(_ng)
    _nig = utils._EngineIntArrayType(
        nig, {'fun_name': fun_name, 'entity': 'nig'},
        exp_shape=(_ng_py,),
    ).to_ctypes()
    _ldgmn_py = utils._EngineFloat64ArrayType.get_nd(gmn, {'fun_name': fun_name, 'entity': 'gmn'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldgmn = utils._EngineIntScalarType(
        _ldgmn_py, {'fun_name': fun_name, 'entity': '_ldgmn'},
    ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _gmn = utils._EngineFloat64ArrayType(
        gmn, {'fun_name': fun_name, 'entity': 'gmn'},
        exp_shape=(_ng_py, _nvar_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _gc = utils._EngineFloat64ArrayType(
        gc, {'fun_name': fun_name, 'entity': 'gc'},
        exp_shape=((_ng_py+1)*_nvar_py*(_nvar_py+1)//2,),
    ).to_ctypes()
    _det = utils._EngineFloat64ArrayType(
        det, {'fun_name': fun_name, 'entity': 'det'},
        exp_shape=(_ng_py,),
    ).to_ctypes()
    _nobs = utils._EngineIntScalarType(
        nobs, {'fun_name': fun_name, 'entity': 'nobs'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _nobs_py = utils._EngineIntScalarType.to_py(_nobs)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_nobs_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _prior_cls = utils._EngineFloat64ArrayType(
        prior, {'fun_name': fun_name, 'entity': 'prior'},
        exp_shape=(_ng_py,),
    )
    _prior = _prior_cls.to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_nobs_py, _ng_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _iag_cls = utils._EngineIntArrayType(
        iag, {'fun_name': fun_name, 'entity': 'iag'},
        exp_shape=(_nobs_py,),
        check_contents=False,
    )
    _iag = _iag_cls.to_ctypes(can_reference_contents=False)
    _atiq = utils._EngineLogicalScalarType(
        atiq, {'fun_name': fun_name, 'entity': 'atiq'},
    ).to_ctypes()
    _ldati_py = utils._EngineFloat64ArrayType.get_nd(ati, {'fun_name': fun_name, 'entity': 'ati'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldati = utils._EngineIntScalarType(
        _ldati_py, {'fun_name': fun_name, 'entity': '_ldati'},
    ).to_ctypes()
    _atiq_py = utils._EngineLogicalScalarType.to_py(_atiq)
    if _atiq_py:
        _ati_dim2 = _ng_py
    else:
        _ati_dim2 = 1
    exp_shape_ati = (_nobs_py, _ati_dim2)
    _ati_cls = utils._EngineFloat64ArrayType(
        ati, {'fun_name': fun_name, 'entity': 'ati'},
        exp_shape=exp_shape_ati,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _ati = _ati_cls.to_ctypes(can_reference_contents=False)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _typ,
        _equal,
        _priors,
        _nvar,
        _ng,
        _nig,
        _gmn,
        _ldgmn,
        _gc,
        _det,
        _nobs,
        _m,
        _isx,
        _x,
        _ldx,
        _prior,
        _p,
        _ldp,
        _iag,
        _atiq,
        _ati,
        _ldati,
        _wk,
        _errbuf,
        _ierr,
        _typ._length_,
        _equal._length_,
        _priors._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, typ = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: typ = \'E\' or \'P\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, priors = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: priors = \'E\', \'I\' or \'P\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, equal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: equal = \'E\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nobs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nobs >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ng >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvar >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and nig[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nig[i-1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_i(nig[i]) <= ng+nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, nig[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: nig[i-1] < nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s} values of isx > 0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: exactly nvar elements of isx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_j(prior[j]) != 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, j = {:s} and prior[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: prior[j-1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a diagonal element of R or R_j is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _prior_cls.refresh_py(_prior)
        _p_cls.refresh_py(_p)
        _iag_cls.refresh_py(_iag)
        _ati_cls.refresh_py(_ati)
        return_data = None
    _err_class.manage()
    return return_data

def distance_mat(update, dist, scal, n, m, x, isx, s, d):
    u"""
    Computes distance matrix.

    For full information please refer to the NAG Library document for
    g03ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03eaf.html

    Parameters
    ----------
    update : str, length 1
        Indicates whether or not an existing matrix is to be updated.

        `update` = 'U'

          The matrix D is updated and distances are added to D.

        `update` = 'I'

          The matrix D is initialized to zero before the distances are
          added to D.

    dist : str, length 1
        Indicates which type of distances are computed.

        `dist` = 'A'

          Absolute distances.

        `dist` = 'E'

          Euclidean distances.

        `dist` = 'S'

          Euclidean squared distances.

    scal : str, length 1
        Indicates the standardization of the variables to be used.

        `scal` = 'S'

          Standard deviation.

        `scal` = 'R'

          Range.

        `scal` = 'G'

          Standardizations given in array `s`.

        `scal` = 'U'

          Unscaled.

    n : int
        n, the number of observations.

    m : int
        The total number of variables in array `x`.

    x : float, shape (`n`, `m`)
        `x`[i-1,j-1] must contain the value of the jth variable for the
        ith object, for j = 1,2,...,`m`, for i = 1,2,...,n.

    isx : int, shape (`m`)
        `isx`[j-1] indicates whether or not the jth variable in `x` is
        to be included in the distance computations.

        If `isx`[j-1] > 0 the jth variable is included, for j =
        1,2,...,`m`; otherwise it is not referenced.

    s : float, shape (`m`), modified in place
        On entry: if `scal` = 'G' and `isx`[j-1] > 0 then `s`[j-1] must
        contain the scaling for variable j, for j = 1,2,...,`m`.

        On exit: if `scal` = 'S' and `isx`[j-1] > 0 then `s`[j-1]
        contains the standard deviation of the variable in the jth
        column of `x`.

        If `scal` = 'R' and `isx`[j-1] > 0, `s`[j-1] contains the range
        of the variable in the jth column of `x`.

        If `scal` = 'U' and `isx`[j-1] > 0, `s`[j-1] = 1.0.

        If `scal` = 'G', `s` is unchanged.

    d : float, shape (`n`*(`n`-1)/2), modified in place
        On entry: if `update` = 'U', `d` must contain the strictly lower
        triangle of the distance matrix D to be updated. D must be
        stored packed by rows, i.e., `d`[ (i-1) (i-2) /2+j -1], i > j
        must contain d_{ij}.

        If `update` = 'I', `d` need not be set.

        On exit: the strictly lower triangle of the distance matrix D
        stored packed by rows, i.e., d_{ij} is contained in `d`[ (i-1)
        (i-2) /2+j -1], i > j.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `scal` = *<value>*.

            Constraint: `scal` = 'S', 'R', 'G' or 'U'.

        (`errno` 1)
            On entry, `update` = *<value>*.

            Constraint: `update` = 'U' or 'I'.

        (`errno` 1)
            On entry, `dist` = *<value>*.

            Constraint: `dist` = 'A', 'E' or 'S'

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            On entry, at least one element of `s` <= 0.0.

        (`errno` 2)
            Variable *<value>* is constant.

        (`errno` 2)
            On entry, at least one element of `d` < 0.0.

        (`errno` 2)
            On entry, `isx` does not contain a positive element.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.distance_mat'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, update(len=1).
        utils._BYREF_STRING, # Input, dist(len=1).
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n*(n-1)//2].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of update.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of dist.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _update = utils._EngineCharScalarType(
        update, {'fun_name': fun_name, 'entity': 'update'}, exp_chrlen=1,
    ).to_ctypes()
    _dist = utils._EngineCharScalarType(
        dist, {'fun_name': fun_name, 'entity': 'dist'}, exp_chrlen=1,
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_m_py,),
    )
    _s = _s_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py*(_n_py-1)//2,),
    )
    _d = _d_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _update,
        _dist,
        _scal,
        _n,
        _m,
        _x,
        _ldx,
        _isx,
        _s,
        _d,
        _errbuf,
        _ierr,
        _update._length_,
        _dist._length_,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: scal = \'S\', \'R\', \'G\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, update = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: update = \'U\' or \'I\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dist = \'A\', \'E\' or \'S\'',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of s <= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Variable {:s} is constant.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of d < 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, isx does not contain a positive element.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _d_cls.refresh_py(_d)
        return_data = None
    _err_class.manage()
    return return_data

def cluster_hier(method, n, d, ilc, iuc, cd, iord, dord):
    u"""
    Hierarchical cluster analysis.

    For full information please refer to the NAG Library document for
    g03ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03ecf.html

    Parameters
    ----------
    method : int
        Indicates which clustering method is used.

        `method` = 1

          Single link.

        `method` = 2

          Complete link.

        `method` = 3

          Group average.

        `method` = 4

          Centroid.

        `method` = 5

          Median.

        `method` = 6

          Minimum variance.

    n : int
        n, the number of objects.

    d : float, shape (`n`*(`n`-1)/2), modified in place
        On entry: the strictly lower triangle of the distance matrix. D
        must be stored packed by rows, i.e., `d`[ (i-1) (i-2) /2+j -1],
        i > j must contain d_{ij}.

        On exit: is overwritten.

    ilc : int, shape (`n`-1), modified in place
        On exit: `ilc`[l-1] contains the number, j, of the cluster
        merged with cluster k (see `iuc`), j < k, at step l, for l =
        1,2,...,n-1.

    iuc : int, shape (`n`-1), modified in place
        On exit: `iuc`[l-1] contains the number, k, of the cluster
        merged with cluster j, j < k, at step l, for l = 1,2,...,n-1.

    cd : float, shape (`n`-1), modified in place
        On exit: `cd`[l-1] contains the distance d_{jk}, between
        clusters j and k, j < k, merged at step l, for l = 1,2,...,n-1.

    iord : int, shape (`n`), modified in place
        On exit: the objects in dendrogram order.

    dord : float, shape (`n`), modified in place
        On exit: the clustering distances corresponding to the order in
        `iord`. `dord`[l-1] contains the distance at which cluster
        `iord`[l-1] and `iord`[l] merge, for l = 1,2,...,n-1.
        `dord`[n-1] contains the maximum distance.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 1, 2, 3, 4, 5 or 6.

        (`errno` 2)
            On entry, at least one element of `d` is negative.

        (`errno` 3)
            Minimum cluster distance not increasing, dendrogram invalid.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.cluster_hier'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03ecft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, method.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n*(n-1)//2].
        utils._EngineIntArrayType.c_type, # Output, ilc[n-1].
        utils._EngineIntArrayType.c_type, # Output, iuc[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, cd[n-1].
        utils._EngineIntArrayType.c_type, # Output, iord[n].
        utils._EngineFloat64ArrayType.c_type, # Output, dord[n].
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _method = utils._EngineIntScalarType(
        method, {'fun_name': fun_name, 'entity': 'method'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py*(_n_py-1)//2,),
    )
    _d = _d_cls.to_ctypes()
    _ilc_cls = utils._EngineIntArrayType(
        ilc, {'fun_name': fun_name, 'entity': 'ilc'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _ilc = _ilc_cls.to_ctypes(can_reference_contents=False)
    _iuc_cls = utils._EngineIntArrayType(
        iuc, {'fun_name': fun_name, 'entity': 'iuc'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _iuc = _iuc_cls.to_ctypes(can_reference_contents=False)
    _cd_cls = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _cd = _cd_cls.to_ctypes(can_reference_contents=False)
    _iord_cls = utils._EngineIntArrayType(
        iord, {'fun_name': fun_name, 'entity': 'iord'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _iord = _iord_cls.to_ctypes(can_reference_contents=False)
    _dord_cls = utils._EngineFloat64ArrayType(
        dord, {'fun_name': fun_name, 'entity': 'dord'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _dord = _dord_cls.to_ctypes(can_reference_contents=False)
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _method,
        _n,
        _d,
        _ilc,
        _iuc,
        _cd,
        _iord,
        _dord,
        _iwk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: method = 1, 2, 3, 4, 5 or 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of d is negative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Minimum cluster distance not increasing, dendrogram invalid.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _ilc_cls.refresh_py(_ilc)
        _iuc_cls.refresh_py(_iuc)
        _cd_cls.refresh_py(_cd)
        _iord_cls.refresh_py(_iord)
        _dord_cls.refresh_py(_dord)
        return_data = None
    _err_class.manage()
    return return_data

def cluster_kmeans(n, m, x, isx, nvar, k, cmeans, wt, inc, nic, css, csw, maxit):
    u"""
    K-means cluster analysis.

    For full information please refer to the NAG Library document for
    g03ef

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03eff.html

    Parameters
    ----------
    n : int
        n, the number of objects.

    m : int
        The total number of variables in array `x`.

    x : float, shape (`n`, `m`)
        `x`[i-1,j-1] must contain the value of the jth variable for the
        ith object, for j = 1,2,...,`m`, for i = 1,2,...,n.

    isx : int, shape (`m`)
        `isx`[j-1] indicates whether or not the jth variable is to be
        included in the analysis. If `isx`[j-1] > 0, the variable
        contained in the jth column of `x` is included, for j =
        1,2,...,`m`.

    nvar : int
        p, the number of variables included in the sums of squares
        calculations.

    k : int
        K, the number of clusters.

    cmeans : float, shape (`k`, `nvar`), modified in place
        On entry: `cmeans`[i-1,j-1] must contain the value of the jth
        variable for the ith initial cluster centre, for j = 1,2,...,p,
        for i = 1,2,...,K.

        On exit: `cmeans`[i-1,j-1] contains the value of the jth
        variable for the ith computed cluster centre, for j = 1,2,...,p,
        for i = 1,2,...,K.

    wt : None or float, shape (`n`)
        If `wt` is not None, the first n elements of `wt` must contain
        the weights to be used.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        analysis.

        The effective number of observation is the sum of the weights.

        If `wt` is None, `wt` is not referenced and the effective number
        of observations is n.

    inc : int, shape (`n`), modified in place
        On exit: `inc`[i-1] contains the cluster to which the ith object
        has been allocated, for i = 1,2,...,n.

    nic : int, shape (`k`), modified in place
        On exit: `nic`[i-1] contains the number of objects in the ith
        cluster, for i = 1,2,...,K.

    css : float, shape (`k`), modified in place
        On exit: `css`[i-1] contains the within-cluster (weighted) sum
        of squares of the ith cluster, for i = 1,2,...,K.

    csw : float, shape (`k`), modified in place
        On exit: `csw`[i-1] contains the within-cluster sum of weights
        of the ith cluster, for i = 1,2,...,K. If weight = 'U', the sum
        of weights is the number of objects in the cluster.

    maxit : int
        The maximum number of iterations allowed in the analysis.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 2.

        (`errno` 1)
            On entry, `m` = *<value>* and `nvar` = *<value>*.

            Constraint: `m` >= `nvar`.

        (`errno` 1)
            On entry, `nvar` = *<value>*.

            Constraint: `nvar` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'U' or 'W'.

        (`errno` 2)
            On entry, `wt` has less than two positive values.

        (`errno` 2)
            On entry, i = *<value>* and `wt`[i-1] < 0.0.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 3)
            On entry, `nvar` = *<value>* and *<value>* values of `isx` >
            0.

            Constraint: exactly `nvar` elements of `isx` > 0.

        (`errno` 4)
            At least one cluster is empty after the initial assignment.

        (`errno` 5)
            Convergence has not been achieved within the maximum number
            of iterations `maxit` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.mv.cluster_kmeans_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.cluster_kmeans'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03efft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, weight(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._BYREF_INT, # Input, nvar.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, cmeans[ldc, nvar].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[n].
        utils._EngineIntArrayType.c_type, # Output, inc[n].
        utils._EngineIntArrayType.c_type, # Output, nic[k].
        utils._EngineFloat64ArrayType.c_type, # Output, css[k].
        utils._EngineFloat64ArrayType.c_type, # Output, csw[k].
        utils._BYREF_INT, # Input, maxit.
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (cmeans, 'cmeans')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    if wt is None:
        _weight = utils._EngineCharScalarType(
            'U', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _weight = utils._EngineCharScalarType(
            'W', {'fun_name': fun_name, 'entity': '_weight'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(cmeans, {'fun_name': fun_name, 'entity': 'cmeans'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _cmeans_cls = utils._EngineFloat64ArrayType(
        cmeans, {'fun_name': fun_name, 'entity': 'cmeans'},
        exp_shape=(_k_py, _nvar_py),
        mutual_sorder=_m_sorder,
    )
    _cmeans = _cmeans_cls.to_ctypes()
    if wt is None:
        _wt = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _wt = utils._EngineFloat64ArrayType(
            wt, {'fun_name': fun_name, 'entity': 'wt'},
            exp_shape=(_n_py,),
            allow_none=True,
        ).to_ctypes()
    _inc_cls = utils._EngineIntArrayType(
        inc, {'fun_name': fun_name, 'entity': 'inc'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _inc = _inc_cls.to_ctypes(can_reference_contents=False)
    _nic_cls = utils._EngineIntArrayType(
        nic, {'fun_name': fun_name, 'entity': 'nic'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _nic = _nic_cls.to_ctypes(can_reference_contents=False)
    _css_cls = utils._EngineFloat64ArrayType(
        css, {'fun_name': fun_name, 'entity': 'css'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _css = _css_cls.to_ctypes(can_reference_contents=False)
    _csw_cls = utils._EngineFloat64ArrayType(
        csw, {'fun_name': fun_name, 'entity': 'csw'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _csw = _csw_cls.to_ctypes(can_reference_contents=False)
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _weight,
        _n,
        _m,
        _x,
        _ldx,
        _isx,
        _nvar,
        _k,
        _cmeans,
        _ldc,
        _wt,
        _inc,
        _nic,
        _css,
        _csw,
        _maxit,
        _iwk,
        _wk,
        _errbuf,
        _ierr,
        _weight._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvar >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, weight = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: weight = \'U\' or \'W\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wt has less than two positive values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and wt[i-1] < 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: wt[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s} values of isx > 0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: exactly nvar elements of isx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one cluster is empty after the initial assignment.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Convergence has not been achieved within the maximum number of',
                    'iterations maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _cmeans_cls.refresh_py(_cmeans)
        _inc_cls.refresh_py(_inc)
        _nic_cls.refresh_py(_nic)
        _css_cls.refresh_py(_css)
        _csw_cls.refresh_py(_csw)
        return_data = None
    _err_class.manage()
    return return_data

def cluster_hier_dendrogram(orient, n, dord, dmin, dstep, nsym, c, lenc):
    u"""
    Constructs dendrogram (for use after ``cluster_hier``).

    For full information please refer to the NAG Library document for
    g03eh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03ehf.html

    Parameters
    ----------
    orient : str, length 1
        Indicates which orientation the dendrogram is to take.

        `orient` = 'N'

          The end points of the dendrogram are to the north.

        `orient` = 'S'

          The end points of the dendrogram are to the south.

        `orient` = 'E'

          The end points of the dendrogram are to the east.

        `orient` = 'W'

          The end points of the dendrogram are to the west.

    n : int
        The number of objects in the cluster analysis.

    dord : float, shape (`n`)
        The array `dord` as output by ``cluster_hier``. `dord` contains
        the distances, in dendrogram order, at which clustering takes
        place.

    dmin : float
        The clustering distance at which the dendrogram begins.

    dstep : float
        The distance represented by one symbol of the dendrogram.

    nsym : int
        The number of character positions used in the dendrogram. Hence
        the clustering distance at which the dendrogram terminates is
        given by `dmin`+`nsym`*`dstep`.

    c : str, shape (`lenc`), modified in place
        On exit: the elements of `c` contain consecutive lines of the
        dendrogram.

    lenc : int
        The dimension of the array `c`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lenc` = *<value>* and `orient` = 'N' or 'S'.

            Constraint: if `orient` = 'N' or 'S', `lenc` >= `nsym`.

        (`errno` 1)
            On entry, the number of characters that can be stored in
            each element of `c` is insufficient for the requested
            orientation.

        (`errno` 1)
            On entry, `lenc` = *<value>* and `orient` = 'E' or 'W'.

            Constraint: if `orient` = 'E' or 'W', `lenc` >= `n`.

        (`errno` 1)
            On entry, `orient` = *<value>*.

            Constraint: `orient` = 'N', 'S', 'E' or 'W'.

        (`errno` 1)
            On entry, `dstep` = *<value>*.

            Constraint: `dstep` > 0.0.

        (`errno` 1)
            On entry, `dmin` = *<value>*.

            Constraint: `dmin` >= 0.0.

        (`errno` 1)
            On entry, `nsym` = *<value>*.

            Constraint: `nsym` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 2.

        (`errno` 2)
            On entry, `dord`[i-1] > `dord`[`n`-1], i < `n`.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.cluster_hier_dendrogram'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03ehft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, orient(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, dord[n].
        utils._BYREF_FLOAT64, # Input, dmin.
        utils._BYREF_FLOAT64, # Input, dstep.
        utils._BYREF_INT, # Input, nsym.
        utils._EngineCharArrayType.c_type, # Output, c(len=*)[lenc].
        utils._BYREF_INT, # Input, lenc.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of orient.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of c.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _orient = utils._EngineCharScalarType(
        orient, {'fun_name': fun_name, 'entity': 'orient'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _dord = utils._EngineFloat64ArrayType(
        dord, {'fun_name': fun_name, 'entity': 'dord'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _dmin = utils._EngineFloat64ScalarType(
        dmin, {'fun_name': fun_name, 'entity': 'dmin'},
    ).to_ctypes()
    _dstep = utils._EngineFloat64ScalarType(
        dstep, {'fun_name': fun_name, 'entity': 'dstep'},
    ).to_ctypes()
    _nsym = utils._EngineIntScalarType(
        nsym, {'fun_name': fun_name, 'entity': 'nsym'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _orient_py = utils._EngineCharScalarType.to_py(_orient)
    _nsym_py = utils._EngineIntScalarType.to_py(_nsym)
    if _orient_py in ('N', 'n', 'S', 's'):
        _c_dim1 = _nsym_py
    elif _orient_py in ('E', 'e', 'W', 'w'):
        _c_dim1 = _n_py
    else:
        _c_dim1 = 0
    exp_shape_c = (_c_dim1,)
    _c_cls = utils._EngineCharArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'}, exp_chrlen=None,
        exp_shape=exp_shape_c,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _c_chrlen = _c_cls.chrlen
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _orient,
        _n,
        _dord,
        _dmin,
        _dstep,
        _nsym,
        _c,
        _lenc,
        _errbuf,
        _ierr,
        _orient._length_,
        _c_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 19994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and orient = \'N\' or \'S\'.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if orient = \'N\' or \'S\', lenc >= nsym.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the number of characters that can be stored in each element',
                    'of c is insufficient for the requested orientation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and orient = \'E\' or \'W\'.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if orient = \'E\' or \'W\', lenc >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, orient = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: orient = \'N\', \'S\', \'E\' or \'W\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dstep = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dstep > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dmin >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nsym = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nsym >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dord[i-1] > dord[n-1], i < n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def cluster_hier_indicator(n, cd, iord, dord, k, dlevel, ic):
    u"""
    Computes cluster indicator variable (for use after
    ``cluster_hier``).

    For full information please refer to the NAG Library document for
    g03ej

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03ejf.html

    Parameters
    ----------
    n : int
        n, the number of objects.

    cd : float, shape (`n`-1)
        The clustering distances in increasing order as returned by
        ``cluster_hier``.

    iord : int, shape (`n`)
        The objects in dendrogram order as returned by ``cluster_hier``.

    dord : float, shape (`n`)
        The clustering distances corresponding to the order in `iord`.

    k : int
        Indicates if a specified number of clusters is required.

        If `k` > 0 then ``cluster_hier_indicator`` will attempt to find
        `k` clusters.

        If `k` <= 0 then ``cluster_hier_indicator`` will find the
        clusters based on the distance given in `dlevel`.

    dlevel : float
        If `k` <= 0, `dlevel` must contain the distance at which
        clusters are produced. Otherwise `dlevel` need not be set.

    ic : int, shape (`n`), modified in place
        On exit: `ic`[i-1] indicates to which of k clusters the ith
        object belongs, for i = 1,2,...,n.

    Returns
    -------
    k : int
        The number of clusters produced, k.

    dlevel : float
        If `k` > 0 on entry, `dlevel` contains the distance at which the
        required number of clusters are found. Otherwise `dlevel`
        remains unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 1)
            On entry, `k` < 0 and `dlevel` <= 0.0.

        (`errno` 1)
            On entry, `k` = *<value>* and `n` = *<value>*.

            Constraint: `k` <= `n`.

        (`errno` 2)
            On entry the values of `dord` and `cd` are not compatible.

        (`errno` 2)
            On entry the values of `cd` are not in increasing order.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            No clustering is performed when `k` = `n`.

        (`errno` 3)
            All data is merged when `k` = 1.

        (`errno` 3)
            All data merged into one cluster at `dlevel`, `dlevel` =
            *<value>*.

        (`errno` 3)
            No clustering takes place below `dlevel`, `dlevel` =
            *<value>*.

        (`errno` 4)
            The precise number of clusters requested is not possible
            because of tied clustering distances.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.cluster_hier_indicator'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03ejft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, cd[n-1].
        utils._EngineIntArrayType.c_type, # Input, iord[n].
        utils._EngineFloat64ArrayType.c_type, # Input, dord[n].
        utils._BYREF_INT, # Input/Output, k.
        utils._BYREF_FLOAT64, # Input/Output, dlevel.
        utils._EngineIntArrayType.c_type, # Output, ic[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _cd = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _iord = utils._EngineIntArrayType(
        iord, {'fun_name': fun_name, 'entity': 'iord'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _dord = utils._EngineFloat64ArrayType(
        dord, {'fun_name': fun_name, 'entity': 'dord'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _dlevel = utils._EngineFloat64ScalarType(
        dlevel, {'fun_name': fun_name, 'entity': 'dlevel'},
    ).to_ctypes()
    _ic_cls = utils._EngineIntArrayType(
        ic, {'fun_name': fun_name, 'entity': 'ic'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ic = _ic_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _cd,
        _iord,
        _dord,
        _k,
        _dlevel,
        _ic,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k < 0 and dlevel <= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry the values of dord and cd are not compatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry the values of cd are not in increasing order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No clustering is performed when k = n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All data is merged when k = 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All data merged into one cluster at dlevel,',
                    'dlevel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No clustering takes place below dlevel, dlevel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The precise number of clusters requested is not',
                    'possible because of tied clustering distances.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _dlevel_py = utils._EngineFloat64ScalarType.to_py(_dlevel)
        _ic_cls.refresh_py(_ic)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'dlevel'],
        )(
            _k_py,
            _dlevel_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def multidimscal_metric(roots, n, d, ndim, x, eigval):
    u"""
    Performs principal coordinate analysis, classical metric scaling.

    For full information please refer to the NAG Library document for
    g03fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03faf.html

    Parameters
    ----------
    roots : str, length 1
        Indicates if all the eigenvalues are to be computed or just the
        `ndim` largest.

        `roots` = 'A'

          All the eigenvalues are computed.

        `roots` = 'L'

          Only the largest `ndim` eigenvalues are computed.

    n : int
        n, the number of objects in the distance matrix.

    d : float, shape (`n`*(`n`-1)/2)
        The lower triangle of the distance matrix D stored packed by
        rows. That is `d`[ (i-1) * (i-2) /2+j -1] must contain d_{ij}
        for i = 2,3,...,n;j = 1,2,...,i-1.

    ndim : int
        k, the number of dimensions used to represent the data.

    x : float, shape (`n`, `ndim`), modified in place
        On exit: the ith row contains k coordinates for the ith point, i
        = 1,2,...,n.

    eigval : float, shape (`n`), modified in place
        On exit: if `roots` = 'A', `eigval` contains the n scaled
        eigenvalues of the matrix E.

        If `roots` = 'L', `eigval` contains the largest k scaled
        eigenvalues of the matrix E.

        In both cases the eigenvalues are divided by the sum of the
        eigenvalues (that is, the trace of E).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `roots` = *<value>*.

            Constraint: `roots` = 'A' or 'L'.

        (`errno` 1)
            On entry, `n` = *<value>* and `ndim` = *<value>*.

            Constraint: `n` > `ndim`.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, all the elements of `d` = 0.0.

        (`errno` 2)
            On entry, i = *<value>* and `d`[i-1] < 0.0.

            Constraint: `d`[i-1] >= 0.0.

        (`errno` 3)
            There are less than `ndim` eigenvalues greater than zero.

        (`errno` 4)
            The computation of the eigenvalues or eigenvectors has
            failed.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.multidimscal_metric'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, roots(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n*(n-1)//2].
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, ndim].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, eigval[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of roots.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _roots = utils._EngineCharScalarType(
        roots, {'fun_name': fun_name, 'entity': 'roots'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py*(_n_py-1)//2,),
    ).to_ctypes()
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _ndim_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _eigval_cls = utils._EngineFloat64ArrayType(
        eigval, {'fun_name': fun_name, 'entity': 'eigval'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _eigval = _eigval_cls.to_ctypes(can_reference_contents=False)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _roots,
        _n,
        _d,
        _ndim,
        _x,
        _ldx,
        _eigval,
        _wk,
        _iwk,
        _errbuf,
        _ierr,
        _roots._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, roots = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: roots = \'A\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n > ndim.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the elements of d = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and d[i-1] < 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: d[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There are less than ndim eigenvalues greater than zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of the eigenvalues or eigenvectors has failed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _eigval_cls.refresh_py(_eigval)
        return_data = None
    _err_class.manage()
    return return_data

def multidimscal_ordinal(typ, n, ndim, d, x, dfit, itera, iopt, io_manager=None):
    u"""
    Performs non-metric (ordinal) multidimensional scaling.

    For full information please refer to the NAG Library document for
    g03fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03fcf.html

    Parameters
    ----------
    typ : str, length 1
        Indicates whether `stress` or sstress is to be used as the
        criterion.

        `typ` = 'T'

          `stress` is used.

        `typ` = 'S'

          sstress is used.

    n : int
        n, the number of objects in the distance matrix.

    ndim : int
        m, the number of dimensions used to represent the data.

    d : float, shape (`n`*(`n`-1)/2)
        The lower triangle of the distance matrix D stored packed by
        rows. That is `d`[ (i-1) * (i-2) /2+j -1] must contain d_{ij},
        for j = 1,2,...,i-1, for i = 2,3,...,n. If d_{ij} is missing
        then set d_{ij} < 0; for further comments on missing values see
        [Further Comments].

    x : float, shape (`n`, `ndim`), modified in place
        On entry: the ith row must contain an initial estimate of the
        coordinates for the ith point, for i = 1,2,...,n. One method of
        computing these is to use ``multidimscal_metric``.

        On exit: the ith row contains m coordinates for the ith point,
        for i = 1,2,...,n.

    dfit : float, shape (2*`n`*(`n`-1)), modified in place
        On exit: auxiliary outputs.

        If `typ` = 'T', the first n(n-1)/2 elements contain the
        distances, d_{ij}^, for the points returned in `x`, the second
        set of n(n-1)/2 contains the distances d_{ij}^ ordered by the
        input distances, d_{ij}, the third set of n(n-1)/2 elements
        contains the monotonic distances, d_{ij}~, ordered by the input
        distances, d_{ij} and the final set of n(n-1)/2 elements
        contains fitted monotonic distances, d_{ij}~, for the points in
        `x`.

        The d_{ij}~ corresponding to distances which are input as
        missing are set to zero.

        If `typ` = 'S', the results are as above except that the squared
        distances are returned.

        Each distance matrix is stored in lower triangular packed form
        in the same way as the input matrix D.

    itera : int
        The maximum number of iterations in the optimization process.

        `itera` = 0

          A default value of 50 is used.

        `itera` < 0

          A default value of max(50, 5nm) (the default for
          ``opt.uncon_conjgrd_comp``) is used.

    iopt : int
        Selects the options, other than the number of iterations, that
        control the optimization.

        `iopt` = 0

          Default values are selected as described in [Further
          Comments]. In particular if an accuracy requirement of epsilon
          = 0.00001 is selected, see [Accuracy].

        `iopt` > 0

          The default values are used except that the accuracy is given
          by 10^(-i) where i = `iopt`.

        `iopt` < 0

          The option setting mechanism of ``opt.uncon_conjgrd_comp`` can
          be used to set all options except 'Iteration Limit'; this
          option is only recommended if you are an experienced user of
          NAG optimization functions. For further details see
          ``opt.uncon_conjgrd_comp``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    stress : float
        The value of `stress` or sstress at the final iteration.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `typ` = *<value>*.

            Constraint: `typ` = 'S' or 'T'.

        (`errno` 1)
            On entry, `n` = *<value>* and `ndim` = *<value>*.

            Constraint: `n` > `ndim`.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, all the elements of `d` <= 0.0.

        (`errno` 3)
            The optimization has failed to converge in `itera` function
            iterations.

        (`errno` 5)
            The optimization cannot begin from the initial
            configuration.

        (`errno` 6)
            The optimization has failed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The conditions for an acceptable solution have not been met
            but a lower point could not be found.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.multidimscal_ordinal'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, typ(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n*(n-1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, ndim].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, stress.
        utils._EngineFloat64ArrayType.c_type, # Output, dfit[2*n*(n-1)].
        utils._BYREF_INT, # Input, itera.
        utils._BYREF_INT, # Input, iopt.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of typ.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _typ = utils._EngineCharScalarType(
        typ, {'fun_name': fun_name, 'entity': 'typ'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py*(_n_py-1)//2,),
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _ndim_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _stress = utils._EngineFloat64ScalarType.empty_ctypes()
    _dfit_cls = utils._EngineFloat64ArrayType(
        dfit, {'fun_name': fun_name, 'entity': 'dfit'},
        exp_shape=(2*_n_py*(_n_py-1),),
        check_contents=False,
    )
    _dfit = _dfit_cls.to_ctypes(can_reference_contents=False)
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _iopt = utils._EngineIntScalarType(
        iopt, {'fun_name': fun_name, 'entity': 'iopt'},
    ).to_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _typ,
        _n,
        _ndim,
        _d,
        _x,
        _ldx,
        _stress,
        _dfit,
        _itera,
        _iopt,
        _wk,
        _iwk,
        _errbuf,
        _ierr,
        _typ._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, typ = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: typ = \'S\' or \'T\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n > ndim.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the elements of d <= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The optimization has failed to converge in itera function iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The conditions for an acceptable solution have not been met',
                    'but a lower point could not be found.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The optimization cannot begin from the initial configuration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The optimization has failed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _dfit_cls.refresh_py(_dfit)
        _stress_py = utils._EngineFloat64ScalarType.to_py(_stress)
        return_data = _stress_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['stress'],
            )(
                _stress_py,
            )
    _err_class.manage()
    return return_data

def gaussian_mixture(n, m, x, isx, nvar, ng, popt, prob, niter, riter, w, g, sopt, s, f, tol):
    u"""
    Fits a Gaussian mixture model.

    For full information please refer to the NAG Library document for
    g03ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03gaf.html

    Parameters
    ----------
    n : int
        n, the number of objects. There must be more objects than
        parameters in the model.

    m : int
        The total number of variables in array `x`.

    x : float, shape (`n`, `m`)
        `x`[i-1,j-1] must contain the value of the jth variable for the
        ith object, for j = 1,2,...,`m`, for i = 1,2,...,n.

    isx : None or int, shape (:)
        Note: the required length for this argument is determined as
        follows: if `nvar`<`m`: `m`; otherwise: 0.

        If `nvar` = `m` all available variables are included in the
        model and `isx` is not referenced; otherwise the jth variable
        will be included in the analysis if `isx`[j-1] = 1 and excluded
        if `isx`[j-1] = 0, for j = 1,2,...,`m`.

    nvar : int
        p, the number of variables included in the calculations.

    ng : int
        k, the number of groups in the mixture model.

    popt : int
        If `popt` = 1, the initial membership probabilities in `prob`
        are set internally; otherwise these probabilities must be
        supplied.

    prob : float, shape (`n`, `ng`), modified in place
        On entry: if `popt` != 1, `prob`[i-1,j-1] is the probability
        that the ith object belongs to the jth group. (These
        probabilities are normalised internally.)

        On exit: `prob`[i-1,j-1] is the probability of membership of the
        ith object to the jth group for the fitted model.

    niter : int
        The maximum number of iterations.

    riter : int
        If `riter` > 0, membership probabilities are rounded to 0.0 or
        1.0 after the completion of every `riter` iterations.

    w : float, shape (`ng`), modified in place
        On exit: w_j, the mixing probability for the jth group.

    g : float, shape (`nvar`, `ng`), modified in place
        On exit: `g`[i-1,j-1] gives the estimated mean of the ith
        variable in the jth group.

    sopt : int
        Determines the (co)variance structure:

        `sopt` = 1

          Groupwise covariance matrices.

        `sopt` = 2

          Pooled covariance matrix.

        `sopt` = 3

          Groupwise variances.

        `sopt` = 4

          Pooled variances.

        `sopt` = 5

          Overall variance.

    s : float, shape (:, :, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `sopt`=5: 1; otherwise: `nvar`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `sopt` in (1, 2): `nvar`; if `sopt`=3:
        `ng`; if `sopt` in (4, 5): 1; otherwise: 0.

        Note: the required extent for this argument in dimension 3 is
        determined as follows: if `sopt`=1: `ng`; otherwise: 1.

        On exit: if `sopt` = 1, `s`[i-1,j-1,k-1] gives the (i,j)th
        element of the kth group.

        If `sopt` = 2, `s`[i-1,j-1,0] gives the (i,j)th element of the
        pooled covariance.

        If `sopt` = 3, `s`[j-1,k-1,0] gives the jth variance in the kth
        group.

        If `sopt` = 4, `s`[j-1,0,0] gives the jth pooled variance.

        If `sopt` = 5, `s`[0,0,0] gives the overall variance.

    f : float, shape (`n`, `ng`), modified in place
        On exit: `f`[i-1,j-1] gives the p-variate Normal (Gaussian)
        density of the ith object in the jth group.

    tol : float
        Iterations cease the first time an improvement in log-likelihood
        is less than `tol`. If `tol` <= 0 a value of 10^(-3) is used.

    Returns
    -------
    niter : int
        The number of completed iterations.

    loglik : float
        The log-likelihood for the fitted mixture model.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>* and p = *<value>*.

            Constraint: `n` > p, the number of parameters, i.e., too few
            objects have been supplied for the model.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 5)
            On entry, `nvar` = *<value>* and `m` = *<value>*.

            Constraint: 1 <= `nvar` <= `m`.

        (`errno` 6)
            On entry, `nvar` != `m` and `isx` is invalid.

        (`errno` 7)
            On entry, `ng` = *<value>*.

            Constraint: `ng` >= 1.

        (`errno` 8)
            On entry, `popt` != 1 or 2.

        (`errno` 9)
            On entry, row *<value>* of supplied `prob` does not sum to
            1.

        (`errno` 11)
            On entry, `niter` = *<value>*.

            Constraint: `niter` >= 1.

        (`errno` 16)
            On entry, `sopt` != 1, 2, 3, 4 or 5.

        (`errno` 44)
            A covariance matrix is not positive definite, try a
            different initial allocation.

        (`errno` 45)
            An iteration cannot continue due to an empty group, try a
            different initial allocation.

    See Also
    --------
    :meth:`naginterfaces.library.examples.mv.gaussian_mixture_ex.main`
    """

    from ..base import _nag_enum_consts

    excs = []
    fun_name = 'naginterfaces.base.mv.gaussian_mixture'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, sorder.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, tdx].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_INT, # Input, tdx.
        utils._EngineIntArrayType.c_type, # Input, isx[*].
        utils._BYREF_INT, # Input, nvar.
        utils._BYREF_INT, # Input, ng.
        utils._BYREF_INT, # Input, popt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, prob[lprob, tdprob].
        utils._BYREF_INT, # Input, lprob.
        utils._BYREF_INT, # Input, tdprob.
        utils._BYREF_INT, # Input/Output, niter.
        utils._BYREF_INT, # Input, riter.
        utils._EngineFloat64ArrayType.c_type, # Output, w[ng].
        utils._EngineFloat64ArrayType.c_type, # Output, g[ldg, tdg].
        utils._BYREF_INT, # Input, ldg.
        utils._BYREF_INT, # Input, tdg.
        utils._BYREF_INT, # Input, sopt.
        utils._EngineFloat64ArrayType.c_type, # Output, s[lds, sds, *].
        utils._BYREF_INT, # Input, lds.
        utils._BYREF_INT, # Input, sds.
        utils._EngineFloat64ArrayType.c_type, # Output, f[ldf, tdf].
        utils._BYREF_INT, # Input, ldf.
        utils._BYREF_INT, # Input, tdf.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Output, loglik.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (prob, 'prob'), (g, 'g'), (s, 's'), (f, 'f')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _sorder = utils._EngineIntScalarType.c_type(_nag_enum_consts.nag_col_major)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _tdx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _tdx = utils._EngineIntScalarType(
        _tdx_py, {'fun_name': fun_name, 'entity': '_tdx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    if _nvar_py < _m_py:
        if isx is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'isx'},
                'must not be None when nvar < m',
            )
    if _nvar_py < _m_py:
        _isx_dim1 = _m_py
    else:
        _isx_dim1 = 0
    exp_shape_isx = (_isx_dim1,)
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=exp_shape_isx,
        check_contents=_nvar_py < _m_py,
        allow_none=True,
    ).to_ctypes(can_reference_contents=_nvar_py < _m_py)
    _ng = utils._EngineIntScalarType(
        ng, {'fun_name': fun_name, 'entity': 'ng'},
    ).to_ctypes()
    _popt = utils._EngineIntScalarType(
        popt, {'fun_name': fun_name, 'entity': 'popt'},
    ).to_ctypes()
    _lprob_py = utils._EngineFloat64ArrayType.get_nd(prob, {'fun_name': fun_name, 'entity': 'prob'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lprob = utils._EngineIntScalarType(
        _lprob_py, {'fun_name': fun_name, 'entity': '_lprob'},
    ).to_ctypes()
    _tdprob_py = utils._EngineFloat64ArrayType.get_nd(prob, {'fun_name': fun_name, 'entity': 'prob'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _tdprob = utils._EngineIntScalarType(
        _tdprob_py, {'fun_name': fun_name, 'entity': '_tdprob'},
    ).to_ctypes()
    _ng_py = utils._EngineIntScalarType.to_py(_ng)
    _popt_py = utils._EngineIntScalarType.to_py(_popt)
    _prob_cls = utils._EngineFloat64ArrayType(
        prob, {'fun_name': fun_name, 'entity': 'prob'},
        exp_shape=(_n_py, _ng_py),
        mutual_sorder=_m_sorder,
        check_contents=_popt_py != 1,
    )
    _prob = _prob_cls.to_ctypes(can_reference_contents=_popt_py != 1)
    _niter = utils._EngineIntScalarType(
        niter, {'fun_name': fun_name, 'entity': 'niter'},
    ).to_ctypes()
    _riter = utils._EngineIntScalarType(
        riter, {'fun_name': fun_name, 'entity': 'riter'},
    ).to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_ng_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldg_py = utils._EngineFloat64ArrayType.get_nd(g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldg = utils._EngineIntScalarType(
        _ldg_py, {'fun_name': fun_name, 'entity': '_ldg'},
    ).to_ctypes()
    _tdg_py = utils._EngineFloat64ArrayType.get_nd(g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _tdg = utils._EngineIntScalarType(
        _tdg_py, {'fun_name': fun_name, 'entity': '_tdg'},
    ).to_ctypes()
    _g_cls = utils._EngineFloat64ArrayType(
        g, {'fun_name': fun_name, 'entity': 'g'},
        exp_shape=(_nvar_py, _ng_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _g = _g_cls.to_ctypes(can_reference_contents=False)
    _sopt = utils._EngineIntScalarType(
        sopt, {'fun_name': fun_name, 'entity': 'sopt'},
    ).to_ctypes()
    _lds_py = utils._EngineFloat64ArrayType.get_nd(s, {'fun_name': fun_name, 'entity': 's'}, exp_rank=3, sorder=_m_sorder, dim_type='l')
    _lds = utils._EngineIntScalarType(
        _lds_py, {'fun_name': fun_name, 'entity': '_lds'},
    ).to_ctypes()
    _sds_py = utils._EngineFloat64ArrayType.get_nd(s, {'fun_name': fun_name, 'entity': 's'}, exp_rank=3, sorder=_m_sorder, dim_type='s')
    _sds = utils._EngineIntScalarType(
        _sds_py, {'fun_name': fun_name, 'entity': '_sds'},
    ).to_ctypes()
    _sopt_py = utils._EngineIntScalarType.to_py(_sopt)
    if _sopt_py == 5:
        _s_dim1 = 1
    else:
        _s_dim1 = _nvar_py
    if _sopt_py in (1, 2):
        _s_dim2 = _nvar_py
    elif _sopt_py == 3:
        _s_dim2 = _ng_py
    elif _sopt_py in (4, 5):
        _s_dim2 = 1
    else:
        _s_dim2 = 0
    if _sopt_py == 1:
        _s_dim3 = _ng_py
    else:
        _s_dim3 = 1
    exp_shape_s = (_s_dim1, _s_dim2, _s_dim3)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=exp_shape_s,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldf_py = utils._EngineFloat64ArrayType.get_nd(f, {'fun_name': fun_name, 'entity': 'f'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldf = utils._EngineIntScalarType(
        _ldf_py, {'fun_name': fun_name, 'entity': '_ldf'},
    ).to_ctypes()
    _tdf_py = utils._EngineFloat64ArrayType.get_nd(f, {'fun_name': fun_name, 'entity': 'f'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _tdf = utils._EngineIntScalarType(
        _tdf_py, {'fun_name': fun_name, 'entity': '_tdf'},
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py, _ng_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _loglik = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _sorder,
        _n,
        _m,
        _x,
        _ldx,
        _tdx,
        _isx,
        _nvar,
        _ng,
        _popt,
        _prob,
        _lprob,
        _tdprob,
        _niter,
        _riter,
        _w,
        _g,
        _ldg,
        _tdg,
        _sopt,
        _s,
        _lds,
        _sds,
        _f,
        _ldf,
        _tdf,
        _tol,
        _loglik,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n > p, the number of parameters,',
                    'i.e., too few objects have been supplied for the model.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= nvar <= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar != m and isx is invalid.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ng = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ng >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, popt != 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, row {:s} of supplied prob does not sum to 1.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, niter = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: niter >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 16:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sopt != 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 44 and _ifmt == 44:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A covariance matrix is not positive definite,',
                    'try a different initial allocation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 45 and _ifmt == 45:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An iteration cannot continue due to an empty group,',
                    'try a different initial allocation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _prob_cls.refresh_py(_prob)
        _niter_py = utils._EngineIntScalarType.to_py(_niter)
        _w_cls.refresh_py(_w)
        _g_cls.refresh_py(_g)
        _s_cls.refresh_py(_s)
        _f_cls.refresh_py(_f)
        _loglik_py = utils._EngineFloat64ScalarType.to_py(_loglik)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['niter', 'loglik'],
        )(
            _niter_py,
            _loglik_py,
        )
    _err_class.manage()
    return return_data

def z_scores(n, m, x, nvar, isx, s, e, z):
    u"""
    Produces standardized values (z-scores) for a data matrix.

    For full information please refer to the NAG Library document for
    g03za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g03/g03zaf.html

    Parameters
    ----------
    n : int
        n, the number of observations in the data matrix.

    m : int
        The number of variables in the data array `x`.

    x : float, shape (`n`, `m`)
        `x`[i-1,j-1] must contain the ith sample point for the jth
        variable, x_{ij}, for j = 1,2,...,`m`, for i = 1,2,...,n.

    nvar : int
        p, the number of variables to be standardized.

    isx : int, shape (`m`)
        `isx`[j-1] indicates whether or not the observations on the jth
        variable are included in the matrix of standardized values.

        If `isx`[j-1] != 0, the observations from the jth variable are
        included.

        If `isx`[j-1] = 0, the observations from the jth variable are
        not included.

    s : float, shape (`m`)
        If `isx`[j-1] != 0, `s`[j-1] must contain the scaling (standard
        deviation), sigma_j, for the jth variable.

        If `isx`[j-1] = 0, `s`[j-1] is not referenced.

    e : float, shape (`m`)
        If `isx`[j-1] != 0, `e`[j-1] must contain the location shift
        (mean), mu_j, for the jth variable.

        If `isx`[j-1] = 0, `e`[j-1] is not referenced.

    z : float, shape (`n`, `nvar`), modified in place
        On exit: the matrix of standardized values (z-scores), Z.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>* and `nvar` = *<value>*.

            Constraint: `m` >= `nvar`.

        (`errno` 1)
            On entry, `nvar` = *<value>*.

            Constraint: `nvar` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, `nvar` = *<value>* and *<value>* values of `isx` >
            0.

            Constraint: exactly `nvar` elements of `isx` > 0.

        (`errno` 3)
            On entry, j = *<value>* and `s`[i-1] <= 0.0.

            Constraint: `s`[j] > 0.0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.mv.z_scores'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g03zaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_INT, # Input, nvar.
        utils._EngineIntArrayType.c_type, # Input, isx[m].
        utils._EngineFloat64ArrayType.c_type, # Input, s[m].
        utils._EngineFloat64ArrayType.c_type, # Input, e[m].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, nvar].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nvar = utils._EngineIntScalarType(
        nvar, {'fun_name': fun_name, 'entity': 'nvar'},
    ).to_ctypes()
    _isx = utils._EngineIntArrayType(
        isx, {'fun_name': fun_name, 'entity': 'isx'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _nvar_py = utils._EngineIntScalarType.to_py(_nvar)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _nvar_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _m,
        _x,
        _ldx,
        _nvar,
        _isx,
        _s,
        _e,
        _z,
        _ldz,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= nvar.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nvar >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nvar = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s} values of isx > 0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: exactly nvar elements of isx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, j = {:s} and s[i-1] <= 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: s[j] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data
