# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `pde` Chapter.

``pde`` - Partial Differential Equations

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the numerical solution of partial
differential equations.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def dim2_laplace(stage1, ext, dorm, n, p, q, x, y, n1p1, phi, phid, alpha):
    u"""
    Elliptic PDE, Laplace's equation, two-dimensional arbitrary domain.

    For full information please refer to the NAG Library document for
    d03ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03eaf.html

    Parameters
    ----------
    stage1 : bool
        Indicates whether the function call is for stage 1 of the
        computation as defined in [Description].

        `stage1` = True

          The call is for stage 1.

        `stage1` = False

          The call is for stage 2.

    ext : bool
        The form of the domain. If `ext` = True, the domain is
        unbounded. Otherwise the domain is an interior one.

    dorm : bool
        The form of the boundary conditions. If `dorm` = True, the
        problem is a Dirichlet or mixed boundary value problem.
        Otherwise it is a Neumann problem.

    n : int
        The number of intervals into which the boundary is divided (see
        [Accuracy] and [Further Comments]).

    p : float
        To stage 2, `p` and `q` must specify the x and y coordinates
        respectively of a point at which the solution is required.

        When `stage1` is True, `p` and `q` are ignored.

    q : float
        To stage 2, `p` and `q` must specify the x and y coordinates
        respectively of a point at which the solution is required.

        When `stage1` is True, `p` and `q` are ignored.

    x : float, shape (`n1p1`)
        The x and y coordinates respectively of points on the one or
        more closed contours which define the domain of the problem.

        Note: each contour is described in such a manner that the
        subscripts of the coordinates increase when the domain is kept
        on the left. The final point on each contour coincides with the
        first and, if a further contour is to be described, the
        coordinates of this point are repeated in the arrays. In this
        way each interval is defined by three points, the second of
        which (the nodal point) always has an even subscript. In the
        case of the interior Neumann problem, the outermost boundary
        contour must be given first, if there is more than one.

    y : float, shape (`n1p1`)
        The x and y coordinates respectively of points on the one or
        more closed contours which define the domain of the problem.

        Note: each contour is described in such a manner that the
        subscripts of the coordinates increase when the domain is kept
        on the left. The final point on each contour coincides with the
        first and, if a further contour is to be described, the
        coordinates of this point are repeated in the arrays. In this
        way each interval is defined by three points, the second of
        which (the nodal point) always has an even subscript. In the
        case of the interior Neumann problem, the outermost boundary
        contour must be given first, if there is more than one.

    n1p1 : int
        The value 2*(`n`+M)-1, where M denotes the number of closed
        contours which make up the boundary.

    phi : float, shape (`n`), modified in place
        On entry: for stage 1, `phi` must contain the nodal values of
        phi or its normal derivative (into the domain) as prescribed in
        each interval. For stage 2 it must retain its output values from
        stage 1.

        On exit: from stage 1, it contains the constants which
        approximate phi in each interval. It remains unchanged on exit
        from stage 2.

    phid : float, shape (`n`), modified in place
        On entry: for stage 1, `phid`[i-1] must hold the value 0.0 or
        1.0 accordingly as `phi`[i-1] contains a value of phi or its
        normal derivative, for i = 1,2,...,`n`. For stage 2 it must
        retain its output values from stage 1.

        On exit: from stage 1, `phid` contains the constants which
        approximate the normal derivative of phi in each interval. It
        remains unchanged on exit from stage 2.

    alpha : float
        For stage 1, the use of `alpha` depends on the nature of the
        problem:

        - if `dorm` = True, `alpha` need not be set;

        - if `dorm` = False and `ext` = True, `alpha` must contain the
          prescribed constant c (see [Description]);

        - if `dorm` = False and `ext` = False, `alpha` must contain an
          appropriate value (often zero) for the integral of phi around
          the outermost boundary.

        For stage 2, on every call `alpha` must contain the value
        returned at stage 1.

    Returns
    -------
    alpha : float
        From stage 1:

        - if `ext` = False, `alpha` contains 0.0;

        - if `ext` = True and `dorm` = False `alpha` is unchanged;

        - if `ext` = True and `dorm` = True `alpha` contains a computed
          estimate for c.

        From stage 2:

        - `alpha` contains the computed value of phi at the point
          (`p`,`q`).

    Raises
    ------
    NagValueError
        (`errno` 1)
            Tolerance is too small: icint[0] = *<value>*.

        (`errno` 1)
            Tolerance is too large: icint[0] = 0.

        (`errno` 2)
            Incorrect rank: icint[0] = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_laplace'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, stage1.
        utils._BYREF_LOGICAL, # Input, ext.
        utils._BYREF_LOGICAL, # Input, dorm.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n1p1].
        utils._EngineFloat64ArrayType.c_type, # Input, y[n1p1].
        utils._BYREF_INT, # Input, n1p1.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, phi[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, phid[n].
        utils._BYREF_FLOAT64, # Input/Output, alpha.
        utils._EngineFloat64ArrayType.c_type, # Workspace, c.
        utils._BYREF_INT, # Input, ldc.
        utils._EngineIntArrayType.c_type, # Workspace, icint.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _stage1 = utils._EngineLogicalScalarType(
        stage1, {'fun_name': fun_name, 'entity': 'stage1'},
    ).to_ctypes()
    _ext = utils._EngineLogicalScalarType(
        ext, {'fun_name': fun_name, 'entity': 'ext'},
    ).to_ctypes()
    _dorm = utils._EngineLogicalScalarType(
        dorm, {'fun_name': fun_name, 'entity': 'dorm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _n1p1 = utils._EngineIntScalarType(
        n1p1, {'fun_name': fun_name, 'entity': 'n1p1'},
    ).to_ctypes()
    _n1p1_py = utils._EngineIntScalarType.to_py(_n1p1)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n1p1_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n1p1_py,),
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _phi_cls = utils._EngineFloat64ArrayType(
        phi, {'fun_name': fun_name, 'entity': 'phi'},
        exp_shape=(_n_py,),
    )
    _phi = _phi_cls.to_ctypes()
    _phid_cls = utils._EngineFloat64ArrayType(
        phid, {'fun_name': fun_name, 'entity': 'phid'},
        exp_shape=(_n_py,),
    )
    _phid = _phid_cls.to_ctypes()
    _alpha = utils._EngineFloat64ScalarType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ldc = utils._EngineIntScalarType(
        _n_py+1, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _icint = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _stage1,
        _ext,
        _dorm,
        _n,
        _p,
        _q,
        _x,
        _y,
        _n1p1,
        _phi,
        _phid,
        _alpha,
        _c,
        _ldc,
        _icint,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Tolerance is too small: icint[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Tolerance is too large: icint[0] = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Incorrect rank: icint[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _phi_cls.refresh_py(_phi)
        _phid_cls.refresh_py(_phid)
        _alpha_py = utils._EngineFloat64ScalarType.to_py(_alpha)
        return_data = _alpha_py
    _err_class.manage()
    return return_data

def dim2_ellip_fd(n1, n2, a, b, c, d, e, q, t, aparam, itmax, itcoun, ndir, ixn, iyn, conres, conchn, resids, chngs):
    u"""
    Elliptic PDE, solution of finite difference equations by SIP,
    five-point two-dimensional molecule, iterate to convergence.

    For full information please refer to the NAG Library document for
    d03eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03ebf.html

    Parameters
    ----------
    n1 : int
        The number of nodes in the first coordinate direction, n_1.

    n2 : int
        The number of nodes in the second coordinate direction, n_2.

    a : float, shape (`n1`, `n2`)
        `a`[i-1,j-1] must contain the coefficient of the 'southerly'
        term involving t_{i,j-1} in the (i,j)th equation of the system
        (1), for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of
        `a`, for j = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    b : float, shape (`n1`, `n2`)
        `b`[i-1,j-1] must contain the coefficient of the 'westerly' term
        involving t_{i-1,j} in the (i,j)th equation of the system (1),
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `b`,
        for i = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    c : float, shape (`n1`, `n2`)
        `c`[i-1,j-1] must contain the coefficient of the 'central' term
        involving t_{ij} in the (i,j)th equation of the system (1), for
        j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `c` are
        checked to ensure that they are nonzero. If any element is found
        to be zero, the corresponding algebraic equation is assumed to
        be t_{ij} = q_{ij}. This feature can be used to define the
        equations for nodes at which, for example, Dirichlet boundary
        conditions are applied, or for nodes external to the problem of
        interest, by setting `c`[i-1,j-1] = 0.0 at appropriate points,
        and the corresponding value of `q`[i-1,j-1] to the appropriate
        value, namely the prescribed value of `t`[i-1,j-1] in the
        Dirichlet case or zero at an external point.

    d : float, shape (`n1`, `n2`)
        `d`[i-1,j-1] must contain the coefficient of the 'easterly' term
        involving t_{i+1,j} in the (i,j)th equation of the system (1),
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `d`,
        for i = `n1`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    e : float, shape (`n1`, `n2`)
        `e`[i-1,j-1] must contain the coefficient of the 'northerly'
        term involving t_{i,j+1} in the (i,j)th equation of the system
        (1), for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of
        `e`, for j = `n2` must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    q : float, shape (`n1`, `n2`)
        `q`[i-1,j-1] must contain q_{ij}, for j = 1,2,...,`n2`, for i =
        1,2,...,`n1`, i.e., the source term values at the nodal points
        for the system (1).

    t : float, shape (`n1`, `n2`), modified in place
        On entry: `t`[i-1,j-1] must contain the element t_{ij} of the
        approximate solution to the equations supplied by the calling
        program as an initial starting value, for j = 1,2,...,`n2`, for
        i = 1,2,...,`n1`.

        If no better approximation is known, an array of zeros can be
        used.

        On exit: the solution derived by the function.

    aparam : float
        The iteration acceleration factor. A value of 1.0 is adequate
        for most typical problems. However, if convergence is slow, the
        value can be reduced, typically to 0.2 or 0.1. If divergence is
        obtained, the value can be increased, typically to 2.0, 5.0 or
        10.0.

    itmax : int
        The maximum number of iterations to be used by the function in
        seeking the solution. A reasonable value might be 30 if `n1` =
        `n2` = 10 or 100 if `n1` = `n2` = 50.

    itcoun : int
        On the first call of ``dim2_ellip_fd``, `itcoun` must be set to
        0. On subsequent entries, its value must be unchanged from the
        previous call.

    ndir : int
        Indicates whether or not the system of equations has a unique
        solution. For systems which have a unique solution, `ndir` must
        be set to any nonzero value. For systems derived from, for
        example, Laplace's equation with all Neumann boundary
        conditions, i.e., problems in which an arbitrary constant can be
        added to the solution, `ndir` should be set to 0 and the values
        of the next two arguments must be specified. For such problems
        the function subtracts the value of the function derived at the
        node (`ixn`, `iyn`) from the whole solution after each iteration
        to reduce the possibility of large rounding errors. You must
        also ensure that for such problems the appropriate consistency
        condition on the source terms `q` is satisfied.

    ixn : int
        Is ignored unless `ndir` is equal to zero, in which case it must
        specify the first index of the nodal point at which the solution
        is to be set to zero. The node should not correspond to a corner
        node, or to a node external to the region of interest.

    iyn : int
        Is ignored unless `ndir` is equal to zero, in which case it must
        specify the second index of the nodal point at which the
        solution is to be set to zero. The node should not correspond to
        a corner node, or to a node external to the region of interest.

    conres : float
        The convergence criterion to be used on the maximum absolute
        value of the normalized residual vector components. The latter
        is defined as the residual of the algebraic equation divided by
        the central coefficient when the latter is not equal to 0.0, and
        defined as the residual when the central coefficient is zero.

        Clearly `conres` should not be less than a reasonable multiple
        of the machine precision.

    conchn : float
        The convergence criterion to be used on the maximum absolute
        value of the change made at each iteration to the elements of
        the array `t`, namely the dependent variable. Clearly `conchn`
        should not be less than a reasonable multiple of the machine
        precision multiplied by the maximum value of `t` attained.

        Convergence is achieved when both the convergence criteria are
        satisfied.

        You can therefore set convergence on either the residual or on
        the change, or (as is recommended) on a requirement that both
        are below prescribed limits.

    resids : float, shape (`itmax`), modified in place
        On exit: the maximum absolute value of the residuals calculated
        at the ith iteration, for i = 1,2,...,`itused`. If you want to
        know the maximum absolute residual of the solution which is
        returned you must calculate this in the calling program. The
        sequence of values `resids` indicates the rate of convergence.

    chngs : float, shape (`itmax`), modified in place
        On exit: the maximum absolute value of the changes made to the
        components of the dependent variable `t` at the ith iteration,
        for i = 1,2,...,`itused`. The sequence of values `chngs`
        indicates the rate of convergence.

    Returns
    -------
    itcoun : int
        Its value is increased by the number of iterations used on this
        call (namely `itused`). It therefore stores the accumulated
        number of iterations actually used. For subsequent calls for the
        same problem, i.e., with the same `n1` and `n2` but possibly
        different coefficients and/or source terms, as occur with
        nonlinear systems or with time-dependent systems, `itcoun` is
        the accumulated number of iterations. This applies to the second
        and subsequent calls to ``dim2_ellip_fd``. In this way a
        suitable cycling of the sequence of iteration arguments is
        obtained in the calls to ``dim2_ellip_fd_iter``.

    itused : int
        The number of iterations actually used on that call.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` > 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` > 1.

        (`errno` 3)
            On entry, `aparam` = *<value>*.

            Constraint: `aparam` > 0.0.

        (`errno` 4)
            On entry, `aparam` = *<value>*, `n1` = *<value>* and `n2` =
            *<value>*.

            Constraint: `aparam` <= ((`n1`-1)^2+(`n2`-1)^2)/2.

        (`errno` 5)
            Convergence not achieved in `itmax` iterations: `itmax` =
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_ellip_fd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03ebft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, n2.
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, b[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, c[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, d[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, e[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, q[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, t[lda, n2].
        utils._BYREF_FLOAT64, # Input, aparam.
        utils._BYREF_INT, # Input, itmax.
        utils._BYREF_INT, # Input/Output, itcoun.
        utils._BYREF_INT, # Output, itused.
        utils._BYREF_INT, # Input, ndir.
        utils._BYREF_INT, # Input, ixn.
        utils._BYREF_INT, # Input, iyn.
        utils._BYREF_FLOAT64, # Input, conres.
        utils._BYREF_FLOAT64, # Input, conchn.
        utils._EngineFloat64ArrayType.c_type, # Output, resids[itmax].
        utils._EngineFloat64ArrayType.c_type, # Output, chngs[itmax].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp2.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp3.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (d, 'd'), (e, 'e'), (q, 'q'), (t, 't')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _n2_py = utils._EngineIntScalarType.to_py(_n2)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _q = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    )
    _t = _t_cls.to_ctypes()
    _aparam = utils._EngineFloat64ScalarType(
        aparam, {'fun_name': fun_name, 'entity': 'aparam'},
    ).to_ctypes()
    _itmax = utils._EngineIntScalarType(
        itmax, {'fun_name': fun_name, 'entity': 'itmax'},
    ).to_ctypes()
    _itcoun = utils._EngineIntScalarType(
        itcoun, {'fun_name': fun_name, 'entity': 'itcoun'},
    ).to_ctypes()
    _itused = utils._EngineIntScalarType.empty_ctypes()
    _ndir = utils._EngineIntScalarType(
        ndir, {'fun_name': fun_name, 'entity': 'ndir'},
    ).to_ctypes()
    _ixn = utils._EngineIntScalarType(
        ixn, {'fun_name': fun_name, 'entity': 'ixn'},
    ).to_ctypes()
    _iyn = utils._EngineIntScalarType(
        iyn, {'fun_name': fun_name, 'entity': 'iyn'},
    ).to_ctypes()
    _conres = utils._EngineFloat64ScalarType(
        conres, {'fun_name': fun_name, 'entity': 'conres'},
    ).to_ctypes()
    _conchn = utils._EngineFloat64ScalarType(
        conchn, {'fun_name': fun_name, 'entity': 'conchn'},
    ).to_ctypes()
    _itmax_py = utils._EngineIntScalarType.to_py(_itmax)
    _resids_cls = utils._EngineFloat64ArrayType(
        resids, {'fun_name': fun_name, 'entity': 'resids'},
        exp_shape=(_itmax_py,),
        check_contents=False,
    )
    _resids = _resids_cls.to_ctypes(can_reference_contents=False)
    _chngs_cls = utils._EngineFloat64ArrayType(
        chngs, {'fun_name': fun_name, 'entity': 'chngs'},
        exp_shape=(_itmax_py,),
        check_contents=False,
    )
    _chngs = _chngs_cls.to_ctypes(can_reference_contents=False)
    _wrksp1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp3 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _n2,
        _lda,
        _a,
        _b,
        _c,
        _d,
        _e,
        _q,
        _t,
        _aparam,
        _itmax,
        _itcoun,
        _itused,
        _ndir,
        _ixn,
        _iyn,
        _conres,
        _conchn,
        _resids,
        _chngs,
        _wrksp1,
        _wrksp2,
        _wrksp3,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: aparam > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}, n1 = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: aparam <= ((n1-1)^2+(n2-1)^2)/2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Convergence not achieved in itmax iterations:',
                    'itmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _t_cls.refresh_py(_t)
        _itcoun_py = utils._EngineIntScalarType.to_py(_itcoun)
        _resids_cls.refresh_py(_resids)
        _chngs_cls.refresh_py(_chngs)
        _itused_py = utils._EngineIntScalarType.to_py(_itused)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['itcoun', 'itused'],
        )(
            _itcoun_py,
            _itused_py,
        )
    _err_class.manage()
    return return_data

def dim3_ellip_fd(n1, n2, n3, sda, a, b, c, d, e, f, g, q, t, aparam, itmax, itcoun, ndir, ixn, iyn, izn, conres, conchn, resids, chngs):
    u"""
    Elliptic PDE, solution of finite difference equations by SIP for
    seven-point three-dimensional molecule, iterate to convergence.

    For full information please refer to the NAG Library document for
    d03ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03ecf.html

    Parameters
    ----------
    n1 : int
        The number of nodes in the first coordinate direction, n_1.

    n2 : int
        The number of nodes in the second coordinate direction, n_2.

    n3 : int
        The number of nodes in the third coordinate direction, n_3.

    sda : int
        The second dimension of the arrays `a`, `b`, `c`, `d`, `e`, `f`,
        `g`, `q`, `t`, `wrksp1`, `wrksp2`, `wrksp3` and `wrksp4`.

    a : float, shape (`n1`, `sda`, `n3`)
        `a`[i-1,j-1,k-1] must contain the coefficient of t_{ij,k-1} in
        the (i,j,k)th equation of the system (1), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `a`,
        for k = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    b : float, shape (`n1`, `sda`, `n3`)
        `b`[i-1,j-1,k-1] must contain the coefficient of t_{i,j-1,k} in
        the (i,j,k)th equation of the system (1), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `b`,
        for j = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    c : float, shape (`n1`, `sda`, `n3`)
        `c`[i-1,j-1,k-1] must contain the coefficient of t_{i-1,jk} in
        the (i,j,k)th equation of the system (1), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `c`,
        for i = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    d : float, shape (`n1`, `sda`, `n3`)
        `d`[i-1,j-1,k-1] must contain the coefficient of t_{ijk} (the
        'central' term) in the (i,j,k)th equation of the system (1), for
        k = 1,2,...,`n3`, for j = 1,2,...,`n2`, for i = 1,2,...,`n1`.
        The elements of `d` are checked to ensure that they are nonzero.
        If any element is found to be zero, the corresponding algebraic
        equation is assumed to be t_{ijk} = q_{ijk}. This feature can be
        used to define the equations for nodes at which, for example,
        Dirichlet boundary conditions are applied, or for nodes external
        to the problem of interest. Setting `d`[i-1,j-1,k-1] = 0.0 at
        appropriate points, and the corresponding value of
        `q`[i-1,j-1,k-1] to the appropriate value, namely the prescribed
        value of `t`[i-1,j-1,k-1] in the Dirichlet case, or to zero at
        an external point.

    e : float, shape (`n1`, `sda`, `n3`)
        `e`[i-1,j-1,k-1] must contain the coefficient of t_{i+1,jk} in
        the (i,j,k)th equation of the system (1), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `e`,
        for i = `n1`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    f : float, shape (`n1`, `sda`, `n3`)
        `f`[i-1,j-1,k-1] must contain the coefficient of t_{i,j+1,k} in
        the (i,j,k)th equation of the system (1), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `f`,
        for j = `n2`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    g : float, shape (`n1`, `sda`, `n3`)
        `g`[i-1,j-1,k-1] must contain the coefficient of t_{ij,k+1} in
        the (i,j,k)th equation of the system (1), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `g`,
        for k = `n3`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    q : float, shape (`n1`, `sda`, `n3`)
        `q`[i-1,j-1,k-1] must contain q_{ijk}, for k = 1,2,...,`n3`, for
        j = 1,2,...,`n2`, for i = 1,2,...,`n1`, i.e., the source-term
        values at the nodal points of the system (1).

    t : float, shape (`n1`, `sda`, `n3`), modified in place
        On entry: `t`[i-1,j-1,k-1] must contain the element t_{ijk} of
        an approximate solution to the equations, for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`.

        If no better approximation is known, an array of zeros can be
        used.

        On exit: the solution derived by the function.

    aparam : float
        The iteration acceleration factor. A value of 1.0 is adequate
        for most typical problems. However, if convergence is slow, the
        value can be reduced, typically to 0.2 or 0.1. If divergence is
        obtained, the value can be increased, typically to 2.0, 5.0 or
        10.0.

    itmax : int
        The maximum number of iterations to be used by the function in
        seeking the solution. A reasonable value might be 20 for a
        problem with 3000 nodes and convergence criteria of about
        10^(-3) of the original residual and change.

    itcoun : int
        On the first call of ``dim3_ellip_fd``, `itcoun` must be set to
        0. On subsequent entries, its value must be unchanged from the
        previous call.

    ndir : int
        Indicates whether or not the system of equations has a unique
        solution. For systems which have a unique solution, `ndir` must
        be set to any nonzero value. For systems derived from problems
        to which an arbitrary constant can be added to the solution, for
        example Poisson's equation with all Neumann boundary conditions,
        `ndir` should be set to 0 and the values of the next three
        arguments must be specified. For such problems the function
        subtracts the value of the function derived at the node (`ixn`,
        `iyn`, `izn`) from the whole solution after each iteration to
        reduce the possibility of large rounding errors. You must also
        ensure for such problems that the appropriate compatibility
        condition on the source terms `q` is satisfied. See the comments
        at the end of [Description].

    ixn : int
        Is ignored unless `ndir` is equal to zero, in which case it must
        specify the first index of the nodal point at which the solution
        is to be set to zero. The node should not correspond to a corner
        node, or to a node external to the region of interest.

    iyn : int
        Is ignored unless `ndir` is equal to zero, in which case it must
        specify the second index of the nodal point at which the
        solution is to be set to zero. The node should not correspond to
        a corner node, or to a node external to the region of interest.

    izn : int
        Is ignored unless `ndir` is equal to zero, in which case it must
        specify the third index of the nodal point at which the solution
        is to be set to zero. The node should not correspond to a corner
        node, or to a node external to the region of interest.

    conres : float
        The convergence criterion to be used on the maximum absolute
        value of the normalized residual vector components. The latter
        is defined as the residual of the algebraic equation divided by
        the central coefficient when the latter is not equal to 0.0, and
        defined as the residual when the central coefficient is zero.

        `conres` should not be less than a reasonable multiple of the
        machine precision.

    conchn : float
        The convergence criterion to be used on the maximum absolute
        value of the change made at each iteration to the elements of
        the array `t`, namely the dependent variable. `conchn` should
        not be less than a reasonable multiple of the machine accuracy
        multiplied by the maximum value of `t` attained.

        Convergence is achieved when both the convergence criteria are
        satisfied.

        You can therefore set convergence on either the residual or on
        the change, or (as is recommended) on a requirement that both
        are below prescribed limits.

    resids : float, shape (`itmax`), modified in place
        On exit: the maximum absolute value of the residuals calculated
        at the ith iteration, for i = 1,2,...,`itused`. If the residual
        of the solution is sought you must calculate this in the
        function from which ``dim3_ellip_fd`` is called. The sequence of
        values `resids` indicates the rate of convergence.

    chngs : float, shape (`itmax`), modified in place
        On exit: the maximum absolute value of the changes made to the
        components of the dependent variable `t` at the ith iteration,
        for i = 1,2,...,`itused`. The sequence of values `chngs`
        indicates the rate of convergence.

    Returns
    -------
    itcoun : int
        Its value is increased by the number of iterations used on this
        call (namely `itused`). It therefore stores the accumulated
        number of iterations actually used.

        For subsequent calls for the same problem, i.e., with the same
        `n1`, `n2` and `n3` but possibly different coefficients and/or
        source terms, as occur with nonlinear systems or with
        time-dependent systems, `itcoun` should not be reset, i.e., it
        must contain the accumulated number of iterations.

        In this way a suitable cycling of the sequence of iteration
        arguments is obtained in the calls to ``dim3_ellip_fd_iter``.

    itused : int
        The number of iterations actually used on that call.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n3` = *<value>*.

            Constraint: `n3` > 1.

        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` > 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` > 1.

        (`errno` 2)
            On entry, `sda` = *<value>* and `n2` = *<value>*.

            Constraint: `sda` >= `n2`.

        (`errno` 3)
            On entry, `aparam` = *<value>*.

            Constraint: `aparam` > 0.0.

        (`errno` 4)
            On entry, `aparam` = *<value>*, `n1` = *<value>*, `n2` =
            *<value>* and `n3` = *<value>*.

            Constraint: `aparam` <=
            ((`n1`-1)^2+(`n2`-1)^2+(`n3`-1)^2)/3.

        (`errno` 5)
            Convergence not achieved in `itmax` iterations: `itmax` =
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim3_ellip_fd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03ecft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, n2.
        utils._BYREF_INT, # Input, n3.
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, sda.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, b[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, c[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, d[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, e[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, f[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, g[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, q[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, t[lda, sda, n3].
        utils._BYREF_FLOAT64, # Input, aparam.
        utils._BYREF_INT, # Input, itmax.
        utils._BYREF_INT, # Input/Output, itcoun.
        utils._BYREF_INT, # Output, itused.
        utils._BYREF_INT, # Input, ndir.
        utils._BYREF_INT, # Input, ixn.
        utils._BYREF_INT, # Input, iyn.
        utils._BYREF_INT, # Input, izn.
        utils._BYREF_FLOAT64, # Input, conres.
        utils._BYREF_FLOAT64, # Input, conchn.
        utils._EngineFloat64ArrayType.c_type, # Output, resids[itmax].
        utils._EngineFloat64ArrayType.c_type, # Output, chngs[itmax].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp2.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp3.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp4.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (d, 'd'), (e, 'e'), (f, 'f'), (g, 'g'), (q, 'q'), (t, 't')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _n3 = utils._EngineIntScalarType(
        n3, {'fun_name': fun_name, 'entity': 'n3'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[0]
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _sda = utils._EngineIntScalarType(
        sda, {'fun_name': fun_name, 'entity': 'sda'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _n2_py = utils._EngineIntScalarType.to_py(_n2)
    _n3_py = utils._EngineIntScalarType.to_py(_n3)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _g = utils._EngineFloat64ArrayType(
        g, {'fun_name': fun_name, 'entity': 'g'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _q = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    )
    _t = _t_cls.to_ctypes()
    _aparam = utils._EngineFloat64ScalarType(
        aparam, {'fun_name': fun_name, 'entity': 'aparam'},
    ).to_ctypes()
    _itmax = utils._EngineIntScalarType(
        itmax, {'fun_name': fun_name, 'entity': 'itmax'},
    ).to_ctypes()
    _itcoun = utils._EngineIntScalarType(
        itcoun, {'fun_name': fun_name, 'entity': 'itcoun'},
    ).to_ctypes()
    _itused = utils._EngineIntScalarType.empty_ctypes()
    _ndir = utils._EngineIntScalarType(
        ndir, {'fun_name': fun_name, 'entity': 'ndir'},
    ).to_ctypes()
    _ixn = utils._EngineIntScalarType(
        ixn, {'fun_name': fun_name, 'entity': 'ixn'},
    ).to_ctypes()
    _iyn = utils._EngineIntScalarType(
        iyn, {'fun_name': fun_name, 'entity': 'iyn'},
    ).to_ctypes()
    _izn = utils._EngineIntScalarType(
        izn, {'fun_name': fun_name, 'entity': 'izn'},
    ).to_ctypes()
    _conres = utils._EngineFloat64ScalarType(
        conres, {'fun_name': fun_name, 'entity': 'conres'},
    ).to_ctypes()
    _conchn = utils._EngineFloat64ScalarType(
        conchn, {'fun_name': fun_name, 'entity': 'conchn'},
    ).to_ctypes()
    _itmax_py = utils._EngineIntScalarType.to_py(_itmax)
    _resids_cls = utils._EngineFloat64ArrayType(
        resids, {'fun_name': fun_name, 'entity': 'resids'},
        exp_shape=(_itmax_py,),
        check_contents=False,
    )
    _resids = _resids_cls.to_ctypes(can_reference_contents=False)
    _chngs_cls = utils._EngineFloat64ArrayType(
        chngs, {'fun_name': fun_name, 'entity': 'chngs'},
        exp_shape=(_itmax_py,),
        check_contents=False,
    )
    _chngs = _chngs_cls.to_ctypes(can_reference_contents=False)
    _wrksp1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp3 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp4 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _n2,
        _n3,
        _lda,
        _sda,
        _a,
        _b,
        _c,
        _d,
        _e,
        _f,
        _g,
        _q,
        _t,
        _aparam,
        _itmax,
        _itcoun,
        _itused,
        _ndir,
        _ixn,
        _iyn,
        _izn,
        _conres,
        _conchn,
        _resids,
        _chngs,
        _wrksp1,
        _wrksp2,
        _wrksp3,
        _wrksp4,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n3 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n3 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sda = {:s} and n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sda >= n2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: aparam > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}, n1 = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'n2 = {:s} and n3 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: aparam <= ((n1-1)^2+(n2-1)^2+(n3-1)^2)/3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Convergence not achieved in itmax iterations:',
                    'itmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _t_cls.refresh_py(_t)
        _itcoun_py = utils._EngineIntScalarType.to_py(_itcoun)
        _resids_cls.refresh_py(_resids)
        _chngs_cls.refresh_py(_chngs)
        _itused_py = utils._EngineIntScalarType.to_py(_itused)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['itcoun', 'itused'],
        )(
            _itcoun_py,
            _itused_py,
        )
    _err_class.manage()
    return return_data

def dim2_ellip_mgrid(ngx, ngy, a, rhs, ub, maxit, acc, us, u, iout, io_manager=None):
    u"""
    Elliptic PDE, solution of finite difference equations by a multigrid
    technique.

    For full information please refer to the NAG Library document for
    d03ed

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03edf.html

    Parameters
    ----------
    ngx : int
        The number of interior grid points in the x-direction, n_x.
        `ngx`-1 should preferably be divisible by as high a power of 2
        as possible.

    ngy : int
        The number of interior grid points in the y-direction, n_y.
        `ngy`-1 should preferably be divisible by as high a power of 2
        as possible.

    a : float, shape ((4*(`ngx`+1)*(`ngy`+1))/3, 7), modified in place
        On entry: `a`[i+(j-1)*`ngx`-1,k-1] must be set to `a`_ij^k, for
        k = 1,2,...,7, for j = 1,2,...,`ngy`, for i = 1,2,...,`ngx`.

        On exit: is overwritten.

    rhs : float, shape ((4*(`ngx`+1)*(`ngy`+1))/3), modified in place
        On entry: `rhs`[i+(j-1)*`ngx`-1] must be set to f_{ij}, for j =
        1,2,...,`ngy`, for i = 1,2,...,`ngx`.

        On exit: the first `ngx`*`ngy` elements are unchanged and the
        rest of the array is used as workspace.

    ub : float, shape (`ngx`*`ngy`), modified in place
        On entry: `ub`[i+(j-1)*`ngx`-1] must be set to the initial
        estimate for the solution u_{ij}.

        On exit: the corresponding component of the residual r = f-`a`u.

    maxit : int
        The maximum permitted number of multigrid iterations. If `maxit`
        = 0, no multigrid iterations are performed, but the coarse-grid
        approximations and incomplete Crout decompositions are computed,
        and may be output if `iout` is set accordingly.

    acc : float
        The required tolerance for convergence of the residual 2-norm:

            ||r||_2 = sqrt(sum_k = 1^`ngx`*`ngy`(r_k)^2)

        where r = f-Au and u is the computed solution. Note that the
        norm is not scaled by the number of equations. The function will
        stop after fewer than `maxit` iterations if the residual 2-norm
        is less than the specified tolerance. (If `maxit` > 0, at least
        one iteration is always performed.)

        If on entry `acc` = 0.0, the machine precision is used as a
        default value for the tolerance; if `acc` > 0.0, but `acc` is
        less than the machine precision, the function will stop when the
        residual 2-norm is less than the machine precision and errno
        will be set to 4.

    us : float, shape ((4*(`ngx`+1)*(`ngy`+1))/3), modified in place
        On exit: the residual 2-norm, stored in element `us`[0].

    u : float, shape ((4*(`ngx`+1)*(`ngy`+1))/3), modified in place
        On exit: the computed solution u_{ij} is returned in
        `u`[i+(j-1)*`ngx`-1], for j = 1,2,...,`ngy`, for i =
        1,2,...,`ngx`.

    iout : int
        Controls the output of printed information to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`):

        `iout` = 0

          No output.

        `iout` = 1

          The solution u_{ij}, for j = 1,2,...,`ngy`, for i =
          1,2,...,`ngx`.

        `iout` = 2

          The residual 2-norm after each iteration, with the reduction
          factor over the previous iteration.

        `iout` = 3

          As for `iout` = 1 and `iout` = 2.

        `iout` = 4

          As for `iout` = 3, plus the final residual (as returned in
          `ub`).

        `iout` = 5

          As for `iout` = 4, plus the initial elements of `a` and `rhs`.

        `iout` = 6

          As for `iout` = 5, plus the Galerkin coarse grid
          approximations.

        `iout` = 7

          As for `iout` = 6, plus the incomplete Crout decompositions.

        `iout` = 8

          As for `iout` = 7, plus the residual after each iteration.

        The elements `a`[p-1,k-1], the Galerkin coarse grid
        approximations and the incomplete Crout decompositions are
        output in the format:

        - Y-index = j

        - X-index =
          i`a`[p-1,0]`a`[p-1,1]`a`[p-1,2]`a`[p-1,3]`a`[p-1,4]`a`[p-1,5]`a`[p-1,6]

        - where p = i+(j-1)*`ngx`, for j = 1,2,...,`ngy`, for i =
          1,2,...,`ngx`.

        The vectors `u`[p-1], `ub`[p-1], `rhs`[p-1] are output in matrix
        form with `ngy` rows and `ngx` columns.

        Where `ngx` > 10, the `ngx` values for a given j value are
        produced in rows of 10.

        Values of `iout` > 4 may yield considerable amounts of output.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    numit : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 0.

        (`errno` 1)
            On entry, `acc` = *<value>*.

            Constraint: `acc` >= 0.0.

        (`errno` 1)
            On entry, `iout` = *<value>*.

            Constraint: 0 <= `iout` <= 8.

        (`errno` 1)
            On entry, `ngy` = *<value>*.

            Constraint: `ngy` >= 3.

        (`errno` 1)
            On entry, `ngx` = *<value>*.

            Constraint: `ngx` >= 3.

        (`errno` 3)
            After `maxit` iterations the residual norm is not less than
            the tolerance `maxit` = *<value>*, residual norm =
            *<value>*, tolerance = *<value>*. The residual norm
            increased at one or more iterations after the first.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            After `maxit` iterations the residual norm is not less than
            the tolerance `maxit` = *<value>*, residual norm =
            *<value>*, tolerance = *<value>*. The residual norm has
            decreased at each iteration after the first.

        (`errno` 4)
            On entry, `acc` is less than machine precision. The function
            terminated because the residual norm is less than machine
            precision. residual norm = *<value>*, machine precision =
            *<value>* and `acc` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_ellip_mgrid'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03edft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, ngx.
        utils._BYREF_INT, # Input, ngy.
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, 7].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rhs[lda].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ub[ngx*ngy].
        utils._BYREF_INT, # Input, maxit.
        utils._BYREF_FLOAT64, # Input, acc.
        utils._EngineFloat64ArrayType.c_type, # Output, us[lda].
        utils._EngineFloat64ArrayType.c_type, # Output, u[lda].
        utils._BYREF_INT, # Input, iout.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Output, numit.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _ngx = utils._EngineIntScalarType(
        ngx, {'fun_name': fun_name, 'entity': 'ngx'},
    ).to_ctypes()
    _ngy = utils._EngineIntScalarType(
        ngy, {'fun_name': fun_name, 'entity': 'ngy'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _ngx_py = utils._EngineIntScalarType.to_py(_ngx)
    _ngy_py = utils._EngineIntScalarType.to_py(_ngy)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=((4*(_ngx_py+1)*(_ngy_py+1))//3, 7),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _rhs_cls = utils._EngineFloat64ArrayType(
        rhs, {'fun_name': fun_name, 'entity': 'rhs'},
        exp_shape=((4*(_ngx_py+1)*(_ngy_py+1))//3,),
    )
    _rhs = _rhs_cls.to_ctypes()
    _ub_cls = utils._EngineFloat64ArrayType(
        ub, {'fun_name': fun_name, 'entity': 'ub'},
        exp_shape=(_ngx_py*_ngy_py,),
    )
    _ub = _ub_cls.to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _acc = utils._EngineFloat64ScalarType(
        acc, {'fun_name': fun_name, 'entity': 'acc'},
    ).to_ctypes()
    _us_cls = utils._EngineFloat64ArrayType(
        us, {'fun_name': fun_name, 'entity': 'us'},
        exp_shape=((4*(_ngx_py+1)*(_ngy_py+1))//3,),
        check_contents=False,
    )
    _us = _us_cls.to_ctypes(can_reference_contents=False)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=((4*(_ngx_py+1)*(_ngy_py+1))//3,),
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _iout = utils._EngineIntScalarType(
        iout, {'fun_name': fun_name, 'entity': 'iout'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _numit = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _ngx,
        _ngy,
        _lda,
        _a,
        _rhs,
        _ub,
        _maxit,
        _acc,
        _us,
        _u,
        _iout,
        _nadv,
        _numit,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: acc >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iout = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= iout <= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ngy = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ngy >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ngx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ngx >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After maxit iterations the residual norm is not less than the tolerance',
                    'maxit = {:s}, residual norm = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tolerance = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'The residual norm has decreased at each iteration after the first.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'After maxit iterations the residual norm is not less than the tolerance',
                    'maxit = {:s}, residual norm = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tolerance = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'The residual norm increased at one or more iterations after the first.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc is less than machine precision. The function terminated',
                    'because the residual norm is less than machine precision.',
                    'residual norm = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'machine precision = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _rhs_cls.refresh_py(_rhs)
        _ub_cls.refresh_py(_ub)
        _us_cls.refresh_py(_us)
        _u_cls.refresh_py(_u)
        _numit_py = utils._EngineIntScalarType.to_py(_numit)
        return_data = _numit_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['numit'],
            )(
                _numit_py,
            )
    _err_class.manage()
    return return_data

def dim2_ellip_discret(xmin, xmax, ymin, ymax, pdef, bndy, ngx, ngy, a, rhs, scheme, data=None):
    u"""
    Discretize a second-order elliptic PDE on a rectangle.

    For full information please refer to the NAG Library document for
    d03ee

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03eef.html

    Parameters
    ----------
    xmin : float
        The lower and upper x coordinates of the rectangular region
        respectively, x_A and x_B.

    xmax : float
        The lower and upper x coordinates of the rectangular region
        respectively, x_A and x_B.

    ymin : float
        The lower and upper y coordinates of the rectangular region
        respectively, y_A and y_B.

    ymax : float
        The lower and upper y coordinates of the rectangular region
        respectively, y_A and y_B.

    pdef : callable (alpha, beta, gamma, delta, epslon, phi, psi) =
        pdef(x, y, data=None)
        `pdef` must evaluate the functions alpha(x,y), beta(x,y),
        gamma(x,y), delta(x,y), epsilon(x,y), phi(x,y) and psi(x,y)
        which define the equation at a general point (x,y).

        Parameters
        ~~~~~~~~~~
        x : float
            The x and y coordinates of the point at which the
            coefficients of the partial differential equation are to be
            evaluated.

        y : float
            The x and y coordinates of the point at which the
            coefficients of the partial differential equation are to be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        alpha : float
            `alpha` must be set to the value of alpha(`x`,`y`) at the
            point specified by `x` and `y`.

        beta : float
            `beta` must be set to the value of beta(`x`,`y`) at the
            point specified by `x` and `y`.

        gamma : float
            `gamma` must be set to the value of gamma(`x`,`y`) at the
            point specified by `x` and `y`.

        delta : float
            `delta` must be set to the value of delta(`x`,`y`) at the
            point specified by `x` and `y`.

        epslon : float
            `epslon` must be set to the value of epislon(`x`,`y`) at the
            point specified by `x` and `y`.

        phi : float
            `phi` must be set to the value of phi(`x`,`y`) at the point
            specified by `x` and `y`.

        psi : float
            `psi` must be set to the value of psi(`x`,`y`) at the point
            specified by `x` and `y`.

    bndy : callable (a, b, c) = bndy(x, y, ibnd, data=None)
        `bndy` must evaluate the functions a(x,y), b(x,y), and c(x,y)
        involved in the boundary conditions.

        Parameters
        ~~~~~~~~~~
        x : float
            The x and y coordinates of the point at which the boundary
            conditions are to be evaluated.

        y : float
            The x and y coordinates of the point at which the boundary
            conditions are to be evaluated.

        ibnd : int
            Specifies on which boundary the point (`x`,`y`) lies. `ibnd`
            = 0, 1, 2 or 3 according as the point lies on the bottom,
            right, top or left boundary.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        a : float
            `a`, `b` and `c` must be set to the values of the functions
            appearing in the boundary conditions.

        b : float
            `a`, `b` and `c` must be set to the values of the functions
            appearing in the boundary conditions.

        c : float
            `a`, `b` and `c` must be set to the values of the functions
            appearing in the boundary conditions.

    ngx : int
        The number of interior grid points in the x- and y-directions
        respectively, n_x and n_y. If the seven-diagonal equations are
        to be solved by ``dim2_ellip_mgrid``, `ngx`-1 and `ngy`-1 should
        preferably be divisible by as high a power of 2 as possible.

    ngy : int
        The number of interior grid points in the x- and y-directions
        respectively, n_x and n_y. If the seven-diagonal equations are
        to be solved by ``dim2_ellip_mgrid``, `ngx`-1 and `ngy`-1 should
        preferably be divisible by as high a power of 2 as possible.

    a : float, shape ((4*(`ngx`+1)*(`ngy`+1))/3, 7), modified in place
        On exit: `a`[i-1,j-1], for j = 1,2,...,7, for i =
        1,2,...,`ngx`*`ngy`, contains the seven-diagonal linear
        equations produced by the discretization described above. If lda
        > `ngx`*`ngy`, the remaining elements are not referenced by the
        function, but if lda >= (4*(`ngx`+1)*(`ngy`+1))/3 then the array
        `a` can be passed directly to ``dim2_ellip_mgrid``, where these
        elements are used as workspace.

    rhs : float, shape ((4*(`ngx`+1)*(`ngy`+1))/3), modified in place
        On exit: the first `ngx`*`ngy` elements contain the right-hand
        sides of the seven-diagonal linear equations produced by the
        discretization described above. If lda > `ngx`*`ngy`, the
        remaining elements are not referenced by the function, but if
        lda >= (4*(`ngy`+1)*(`ngy`+1))/3 then the array `rhs` can be
        passed directly to ``dim2_ellip_mgrid``, where these elements
        are used as workspace.

    scheme : str, length 1
        The type of approximation to be used for the first derivatives
        which occur in the partial differential equation.

        `scheme` = 'C'

          Central differences are used.

        `scheme` = 'U'

          Upwind differences are used.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `scheme` = *<value>*.

            Constraint: `scheme` = 'C' or 'U'.

        (`errno` 1)
            On entry, `ngy` = *<value>*.

            Constraint: `ngy` >= 3.

        (`errno` 1)
            On entry, `ngx` = *<value>*.

            Constraint: `ngx` >= 3.

        (`errno` 1)
            On entry, `ngx`*`ngy` = *<value>* and lda = *<value>*.

            Constraint: lda must be at least `ngx`*`ngy`.

        (`errno` 1)
            On entry, `ymin` = *<value>* and `ymax` = *<value>*.

            Constraint: `ymin` < `ymax`.

        (`errno` 1)
            On entry, `xmin` = *<value>* and `xmax` = *<value>*.

            Constraint: `xmin` < `xmax`.

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at top left boundary. (x,y) = (*<value>*,
            *<value>*).

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at top right boundary. (x,y) = (*<value>*,
            *<value>*).

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at bottom left boundary. (x,y) = (*<value>*,
            *<value>*).

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at bottom right boundary. (x,y) = (*<value>*,
            *<value>*).

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at right boundary. (x,y) = (*<value>*, *<value>*).

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at left boundary. (x,y) = (*<value>*, *<value>*).

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at top boundary. (x,y) = (*<value>*, *<value>*).

        (`errno` 2)
            Mixed derivative in equation and derivative in boundary
            condition at bottom boundary. (x,y) = (*<value>*,
            *<value>*).

        (`errno` 3)
            Null boundary condition at left boundary, top end. (x,y) =
            (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at top boundary, left end. (x,y) =
            (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at right boundary, top end. (x,y) =
            (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at top boundary, right end. (x,y) =
            (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at left boundary, bottom end. (x,y)
            = (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at bottom boundary, left end. (x,y)
            = (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at right boundary, bottom end. (x,y)
            = (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at bottom boundary, right end. (x,y)
            = (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at right boundary. (x,y) =
            (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at left boundary. (x,y) =
            (*<value>*, *<value>*).

        (`errno` 3)
            Null boundary condition at top boundary. (x,y) = (*<value>*,
            *<value>*).

        (`errno` 3)
            Null boundary condition at bottom boundary. (x,y) =
            (*<value>*, *<value>*).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Equation not elliptic at some point.

        (`errno` 5)
            There is no unique solution with Neumann Boundary
            conditions.

        (`errno` 6)
            The linear equations were not diagonally dominant.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_ellip_discret'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdef_name = fun_name + ':pdef'

    pdefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Output, alpha.
        utils._BYREF_FLOAT64, # Output, beta.
        utils._BYREF_FLOAT64, # Output, gamma.
        utils._BYREF_FLOAT64, # Output, delta.
        utils._BYREF_FLOAT64, # Output, epslon.
        utils._BYREF_FLOAT64, # Output, phi.
        utils._BYREF_FLOAT64, # Output, psi.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdefh(_pdef, en_data, x, y, alpha, beta, gamma, delta, epslon, phi, psi, _iuser, _ruser):
        """Helper for pdef."""
        try:
            _x_pdefh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_pdefh = utils._EngineFloat64ScalarType.to_py(y.contents)
            if data is not None:
                _alpha_pdefh, _beta_pdefh, _gamma_pdefh, _delta_pdefh, _epslon_pdefh, _phi_pdefh, _psi_pdefh = pdef(
                    _x_pdefh,
                    _y_pdefh,
                    data,
                )
            else:
                _alpha_pdefh, _beta_pdefh, _gamma_pdefh, _delta_pdefh, _epslon_pdefh, _phi_pdefh, _psi_pdefh = pdef(
                    _x_pdefh,
                    _y_pdefh,
                )
            utils._EngineFloat64ScalarType(
                _alpha_pdefh, {'fun_name': cb_pdef_name, 'returned': 'alpha'},
            ).refresh_ctypes_buffer(alpha)
            utils._EngineFloat64ScalarType(
                _beta_pdefh, {'fun_name': cb_pdef_name, 'returned': 'beta'},
            ).refresh_ctypes_buffer(beta)
            utils._EngineFloat64ScalarType(
                _gamma_pdefh, {'fun_name': cb_pdef_name, 'returned': 'gamma'},
            ).refresh_ctypes_buffer(gamma)
            utils._EngineFloat64ScalarType(
                _delta_pdefh, {'fun_name': cb_pdef_name, 'returned': 'delta'},
            ).refresh_ctypes_buffer(delta)
            utils._EngineFloat64ScalarType(
                _epslon_pdefh, {'fun_name': cb_pdef_name, 'returned': 'epslon'},
            ).refresh_ctypes_buffer(epslon)
            utils._EngineFloat64ScalarType(
                _phi_pdefh, {'fun_name': cb_pdef_name, 'returned': 'phi'},
            ).refresh_ctypes_buffer(phi)
            utils._EngineFloat64ScalarType(
                _psi_pdefh, {'fun_name': cb_pdef_name, 'returned': 'psi'},
            ).refresh_ctypes_buffer(psi)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndy_name = fun_name + ':bndy'

    bndyh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Output, a.
        utils._BYREF_FLOAT64, # Output, b.
        utils._BYREF_FLOAT64, # Output, c.
        utils._BYREF_INT, # Input, ibnd.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndyh(_bndy, en_data, x, y, a, b, c, ibnd, _iuser, _ruser):
        """Helper for bndy."""
        try:
            _x_bndyh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_bndyh = utils._EngineFloat64ScalarType.to_py(y.contents)
            _ibnd_bndyh = utils._EngineIntScalarType.to_py(ibnd.contents)
            if data is not None:
                _a_bndyh, _b_bndyh, _c_bndyh = bndy(
                    _x_bndyh,
                    _y_bndyh,
                    _ibnd_bndyh,
                    data,
                )
            else:
                _a_bndyh, _b_bndyh, _c_bndyh = bndy(
                    _x_bndyh,
                    _y_bndyh,
                    _ibnd_bndyh,
                )
            utils._EngineFloat64ScalarType(
                _a_bndyh, {'fun_name': cb_bndy_name, 'returned': 'a'},
            ).refresh_ctypes_buffer(a)
            utils._EngineFloat64ScalarType(
                _b_bndyh, {'fun_name': cb_bndy_name, 'returned': 'b'},
            ).refresh_ctypes_buffer(b)
            utils._EngineFloat64ScalarType(
                _c_bndyh, {'fun_name': cb_bndy_name, 'returned': 'c'},
            ).refresh_ctypes_buffer(c)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03eeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, xmax.
        utils._BYREF_FLOAT64, # Input, ymin.
        utils._BYREF_FLOAT64, # Input, ymax.
        utils._UNUSED_FUNCTYPE,
        pdefh,
        utils._UNUSED_FUNCTYPE,
        bndyh,
        utils._BYREF_INT, # Input, ngx.
        utils._BYREF_INT, # Input, ngy.
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, a[lda, 7].
        utils._EngineFloat64ArrayType.c_type, # Output, rhs[lda].
        utils._BYREF_STRING, # Input, scheme(len=1).
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scheme.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _xmax = utils._EngineFloat64ScalarType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
    ).to_ctypes()
    _ymin = utils._EngineFloat64ScalarType(
        ymin, {'fun_name': fun_name, 'entity': 'ymin'},
    ).to_ctypes()
    _ymax = utils._EngineFloat64ScalarType(
        ymax, {'fun_name': fun_name, 'entity': 'ymax'},
    ).to_ctypes()
    utils._check_callable(pdef, {'fun_name': fun_name, 'entity': 'pdef'})
    _pdef = utils._UNUSED_CALLBACK
    _pdefh = utils._cast_callback(
        pdefh, cb_pdefh,
        {'fun_name': fun_name, 'entity': 'pdef'},
    )
    utils._check_callable(bndy, {'fun_name': fun_name, 'entity': 'bndy'})
    _bndy = utils._UNUSED_CALLBACK
    _bndyh = utils._cast_callback(
        bndyh, cb_bndyh,
        {'fun_name': fun_name, 'entity': 'bndy'},
    )
    _ngx = utils._EngineIntScalarType(
        ngx, {'fun_name': fun_name, 'entity': 'ngx'},
    ).to_ctypes()
    _ngy = utils._EngineIntScalarType(
        ngy, {'fun_name': fun_name, 'entity': 'ngy'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _ngx_py = utils._EngineIntScalarType.to_py(_ngx)
    _ngy_py = utils._EngineIntScalarType.to_py(_ngy)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=((4*(_ngx_py+1)*(_ngy_py+1))//3, 7),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _rhs_cls = utils._EngineFloat64ArrayType(
        rhs, {'fun_name': fun_name, 'entity': 'rhs'},
        exp_shape=((4*(_ngx_py+1)*(_ngy_py+1))//3,),
        check_contents=False,
    )
    _rhs = _rhs_cls.to_ctypes(can_reference_contents=False)
    _scheme = utils._EngineCharScalarType(
        scheme, {'fun_name': fun_name, 'entity': 'scheme'}, exp_chrlen=1,
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xmin,
        _xmax,
        _ymin,
        _ymax,
        _pdef,
        _pdefh,
        _bndy,
        _bndyh,
        _ngx,
        _ngy,
        _lda,
        _a,
        _rhs,
        _scheme,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _scheme._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scheme = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: scheme = \'C\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ngy = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ngy >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ngx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ngx >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ngx*ngy = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'lda = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lda must be at least ngx*ngy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ymin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ymax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ymin < ymax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xmin < xmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at top left boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at top right boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at bottom left boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at bottom right boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at right boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at left boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at top boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Mixed derivative in equation and derivative in boundary',
                    'condition at bottom boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at left boundary, top end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at top boundary, left end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at right boundary, top end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at top boundary, right end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at left boundary, bottom end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at bottom boundary, left end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at right boundary, bottom end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at bottom boundary, right end.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at right boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at left boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at top boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Null boundary condition at bottom boundary.',
                    '(x,y) = ({:s}, {:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99973:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Equation not elliptic at some point.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99972:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There is no unique solution with Neumann Boundary conditions.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear equations were not diagonally dominant.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _rhs_cls.refresh_py(_rhs)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim3_ellip_helmholtz(xs, xf, l, lbdcnd, bdxs, bdxf, ys, yf, m, mbdcnd, bdys, bdyf, zs, zf, n, nbdcnd, bdzs, bdzf, lamda, f):
    u"""
    Elliptic PDE, Helmholtz equation, three-dimensional Cartesian
    coordinates.

    For full information please refer to the NAG Library document for
    d03fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03faf.html

    Parameters
    ----------
    xs : float
        The lower bound of the range of x, i.e., `xs` <= x <= `xf`.

    xf : float
        The upper bound of the range of x, i.e., `xs` <= x <= `xf`.

    l : int
        The number of panels into which the interval (`xs`,`xf`) is
        subdivided. Hence, there will be `l`+1 grid points in the
        x-direction given by x_i = `xs`+(i-1)*delta x, for i =
        1,2,...,`l`+1, where delta x = (`xf`-`xs`)/`l` is the panel
        width.

    lbdcnd : int
        Indicates the type of boundary conditions at x = `xs` and x =
        `xf`.

        `lbdcnd` = 0

          If the solution is periodic in x, i.e., u(`xs`,y,z) =
          u(`xf`,y,z).

        `lbdcnd` = 1

          If the solution is specified at x = `xs` and x = `xf`.

        `lbdcnd` = 2

          If the solution is specified at x = `xs` and the derivative of
          the solution with respect to x is specified at x = `xf`.

        `lbdcnd` = 3

          If the derivative of the solution with respect to x is
          specified at x = `xs` and x = `xf`.

        `lbdcnd` = 4

          If the derivative of the solution with respect to x is
          specified at x = `xs` and the solution is specified at x =
          `xf`.

    bdxs : float, shape (`m`+1, `n`+1)
        The values of the derivative of the solution with respect to x
        at x = `xs`. When `lbdcnd` = 3 or 4, `bdxs`[j-1,k-1] =
        u_x(`xs`,y_j,z_k), for k = 1,2,...,`n`+1, for j = 1,2,...,`m`+1.

        When `lbdcnd` has any other value, `bdxs` is not referenced.

    bdxf : float, shape (`m`+1, `n`+1)
        The values of the derivative of the solution with respect to x
        at x = `xf`. When `lbdcnd` = 2 or 3, `bdxf`[j-1,k-1] =
        u_x(`xf`,y_j,z_k), for k = 1,2,...,`n`+1, for j = 1,2,...,`m`+1.

        When `lbdcnd` has any other value, `bdxf` is not referenced.

    ys : float
        The lower bound of the range of y, i.e., `ys` <= y <= `yf`.

    yf : float
        The upper bound of the range of y, i.e., `ys` <= y <= `yf`.

    m : int
        The number of panels into which the interval (`ys`,`yf`) is
        subdivided. Hence, there will be `m`+1 grid points in the
        y-direction given by y_j = `ys`+(j-1)*delta y, for j =
        1,2,...,`m`+1, where delta y = (`yf`-`ys`)/`m` is the panel
        width.

    mbdcnd : int
        Indicates the type of boundary conditions at y = `ys` and y =
        `yf`.

        `mbdcnd` = 0

          If the solution is periodic in y, i.e., u(x,`yf`,z) =
          u(x,`ys`,z).

        `mbdcnd` = 1

          If the solution is specified at y = `ys` and y = `yf`.

        `mbdcnd` = 2

          If the solution is specified at y = `ys` and the derivative of
          the solution with respect to y is specified at y = `yf`.

        `mbdcnd` = 3

          If the derivative of the solution with respect to y is
          specified at y = `ys` and y = `yf`.

        `mbdcnd` = 4

          If the derivative of the solution with respect to y is
          specified at y = `ys` and the solution is specified at y =
          `yf`.

    bdys : float, shape (`l`+1, `n`+1)
        The values of the derivative of the solution with respect to y
        at y = `ys`. When `mbdcnd` = 3 or 4, `bdys`[i-1,k-1] =
        u_y(x_i,`ys`,z_k), for k = 1,2,...,`n`+1, for i = 1,2,...,`l`+1.

        When `mbdcnd` has any other value, `bdys` is not referenced.

    bdyf : float, shape (`l`+1, `n`+1)
        The values of the derivative of the solution with respect to y
        at y = `yf`. When `mbdcnd` = 2 or 3, `bdyf`[i-1,k-1] =
        u_y(x_i,`yf`,z_k), for k = 1,2,...,`n`+1, for i = 1,2,...,`l`+1.

        When `mbdcnd` has any other value, `bdyf` is not referenced.

    zs : float
        The lower bound of the range of z, i.e., `zs` <= z <= `zf`.

    zf : float
        The upper bound of the range of z, i.e., `zs` <= z <= `zf`.

    n : int
        The number of panels into which the interval (`zs`,`zf`) is
        subdivided. Hence, there will be `n`+1 grid points in the
        z-direction given by z_k = `zs`+(k-1)*delta z, for k =
        1,2,...,`n`+1, where delta z = (`zf`-`zs`)/`n` is the panel
        width.

    nbdcnd : int
        Specifies the type of boundary conditions at z = `zs` and z =
        `zf`.

        `nbdcnd` = 0

          if the solution is periodic in z, i.e., u(x,y,`zf`) =
          u(x,y,`zs`).

        `nbdcnd` = 1

          if the solution is specified at z = `zs` and z = `zf`.

        `nbdcnd` = 2

          if the solution is specified at z = `zs` and the derivative of
          the solution with respect to z is specified at z = `zf`.

        `nbdcnd` = 3

          if the derivative of the solution with respect to z is
          specified at z = `zs` and z = `zf`.

        `nbdcnd` = 4

          if the derivative of the solution with respect to z is
          specified at z = `zs` and the solution is specified at z =
          `zf`.

    bdzs : float, shape (`l`+1, `m`+1)
        The values of the derivative of the solution with respect to z
        at z = `zs`. When `nbdcnd` = 3 or 4, `bdzs`[i-1,j-1] =
        u_z(x_i,y_j,`zs`), for j = 1,2,...,`m`+1, for i = 1,2,...,`l`+1.

        When `nbdcnd` has any other value, `bdzs` is not referenced.

    bdzf : float, shape (`l`+1, `m`+1)
        The values of the derivative of the solution with respect to z
        at z = `zf`. When `nbdcnd` = 2 or 3, `bdzf`[i-1,j-1] =
        u_z(x_i,y_j,`zf`), for j = 1,2,...,`m`+1, for i = 1,2,...,`l`+1.

        When `nbdcnd` has any other value, `bdzf` is not referenced.

    lamda : float
        The constant lambda in the Helmholtz equation. For certain
        positive values of lambda a solution to the differential
        equation may not exist, and close to these values the solution
        of the discretized problem will be extremely ill-conditioned. If
        lambda > 0, ``dim3_ellip_helmholtz`` will set `errno` = 3, but
        will still attempt to find a solution. However, since in general
        the values of lambda for which no solution exists cannot be
        predicted a priori, you are advised to treat any results
        computed with lambda > 0 with great caution.

    f : float, shape (`l`+1, `m`+1, `n`+1), modified in place
        On entry: the values of the right-side of the Helmholtz equation
        and boundary values (if any).

            `f`[i-1,j-1,k-1] = f(x_i,y_j,z_k) i = 2,3,...,`l`,j =
            2,3,...,`m` and k = 2,3,...,`n`.

        On the boundaries `f` is defined by

        [table omitted]

        Note: if the table calls for both the solution u and the
        right-hand side f on a boundary, the solution must be specified.

        On exit: contains the solution u(i,j,k) of the finite difference
        approximation for the grid point (x_i,y_j,z_k), for k =
        1,2,...,`n`+1, for j = 1,2,...,`m`+1, for i = 1,2,...,`l`+1.

    Returns
    -------
    pertrb : float
        `pertrb` = 0, unless a solution to Poisson's equation (lambda =
        0) is required with a combination of periodic or derivative
        boundary conditions (`lbdcnd`, `mbdcnd` and `nbdcnd` = 0 or 3).
        In this case a solution may not exist. `pertrb` is a constant,
        calculated and subtracted from the array `f`, which ensures that
        a solution exists. ``dim3_ellip_helmholtz`` then computes this
        solution, which is a least squares solution to the original
        approximation. This solution is not unique and is unnormalized.
        The value of `pertrb` should be small compared to the right-hand
        side `f`, otherwise a solution has been obtained to an
        essentially different problem. This comparison should always be
        made to ensure that a meaningful solution has been obtained.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nbdcnd` = *<value>*.

            Constraint: `nbdcnd` >= 0 and `nbdcnd` <= 4.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 5.

        (`errno` 1)
            On entry, `zs` = *<value>* and `zf` = *<value>*.

            Constraint: `zs` < `zf`.

        (`errno` 1)
            On entry, `mbdcnd` = *<value>*.

            Constraint: `mbdcnd` >= 0 and `mbdcnd` <= 4.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 5.

        (`errno` 1)
            On entry, `ys` = *<value>* and `yf` = *<value>*.

            Constraint: `ys` < `yf`

        (`errno` 1)
            On entry, `lbdcnd` = *<value>*.

            Constraint: `lbdcnd` >= 0 and `lbdcnd` <= 4.

        (`errno` 1)
            On entry, `l` = *<value>*.

            Constraint: `l` >= 5.

        (`errno` 1)
            On entry, `xs` = *<value>* and `xf` = *<value>*.

            Constraint: `xs` < `xf`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            lambda > 0.0 in Helmholtz's equation -- a solution may not
            exist.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim3_ellip_helmholtz'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xs.
        utils._BYREF_FLOAT64, # Input, xf.
        utils._BYREF_INT, # Input, l.
        utils._BYREF_INT, # Input, lbdcnd.
        utils._EngineFloat64ArrayType.c_type, # Input, bdxs[ldf2, n+1].
        utils._EngineFloat64ArrayType.c_type, # Input, bdxf[ldf2, n+1].
        utils._BYREF_FLOAT64, # Input, ys.
        utils._BYREF_FLOAT64, # Input, yf.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, mbdcnd.
        utils._EngineFloat64ArrayType.c_type, # Input, bdys[ldf, n+1].
        utils._EngineFloat64ArrayType.c_type, # Input, bdyf[ldf, n+1].
        utils._BYREF_FLOAT64, # Input, zs.
        utils._BYREF_FLOAT64, # Input, zf.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nbdcnd.
        utils._EngineFloat64ArrayType.c_type, # Input, bdzs[ldf, m+1].
        utils._EngineFloat64ArrayType.c_type, # Input, bdzf[ldf, m+1].
        utils._BYREF_FLOAT64, # Input, lamda.
        utils._BYREF_INT, # Input, ldf.
        utils._BYREF_INT, # Input, ldf2.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, f[ldf, ldf2, n+1].
        utils._BYREF_FLOAT64, # Output, pertrb.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((bdxs, 'bdxs'), (bdxf, 'bdxf'), (bdys, 'bdys'), (bdyf, 'bdyf'), (bdzs, 'bdzs'), (bdzf, 'bdzf'), (f, 'f')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _xs = utils._EngineFloat64ScalarType(
        xs, {'fun_name': fun_name, 'entity': 'xs'},
    ).to_ctypes()
    _xf = utils._EngineFloat64ScalarType(
        xf, {'fun_name': fun_name, 'entity': 'xf'},
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _lbdcnd = utils._EngineIntScalarType(
        lbdcnd, {'fun_name': fun_name, 'entity': 'lbdcnd'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldf2_py = utils._EngineFloat64ArrayType.get_shape(f, {'fun_name': fun_name, 'entity': 'f'}, exp_rank=3)[1]
    _ldf2 = utils._EngineIntScalarType(
        _ldf2_py, {'fun_name': fun_name, 'entity': '_ldf2'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _bdxs = utils._EngineFloat64ArrayType(
        bdxs, {'fun_name': fun_name, 'entity': 'bdxs'},
        exp_shape=(_m_py+1, _n_py+1),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _bdxf = utils._EngineFloat64ArrayType(
        bdxf, {'fun_name': fun_name, 'entity': 'bdxf'},
        exp_shape=(_m_py+1, _n_py+1),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ys = utils._EngineFloat64ScalarType(
        ys, {'fun_name': fun_name, 'entity': 'ys'},
    ).to_ctypes()
    _yf = utils._EngineFloat64ScalarType(
        yf, {'fun_name': fun_name, 'entity': 'yf'},
    ).to_ctypes()
    _mbdcnd = utils._EngineIntScalarType(
        mbdcnd, {'fun_name': fun_name, 'entity': 'mbdcnd'},
    ).to_ctypes()
    _ldf_py = utils._EngineFloat64ArrayType.get_shape(f, {'fun_name': fun_name, 'entity': 'f'}, exp_rank=3)[0]
    _ldf = utils._EngineIntScalarType(
        _ldf_py, {'fun_name': fun_name, 'entity': '_ldf'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    _bdys = utils._EngineFloat64ArrayType(
        bdys, {'fun_name': fun_name, 'entity': 'bdys'},
        exp_shape=(_l_py+1, _n_py+1),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _bdyf = utils._EngineFloat64ArrayType(
        bdyf, {'fun_name': fun_name, 'entity': 'bdyf'},
        exp_shape=(_l_py+1, _n_py+1),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _zs = utils._EngineFloat64ScalarType(
        zs, {'fun_name': fun_name, 'entity': 'zs'},
    ).to_ctypes()
    _zf = utils._EngineFloat64ScalarType(
        zf, {'fun_name': fun_name, 'entity': 'zf'},
    ).to_ctypes()
    _nbdcnd = utils._EngineIntScalarType(
        nbdcnd, {'fun_name': fun_name, 'entity': 'nbdcnd'},
    ).to_ctypes()
    _bdzs = utils._EngineFloat64ArrayType(
        bdzs, {'fun_name': fun_name, 'entity': 'bdzs'},
        exp_shape=(_l_py+1, _m_py+1),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _bdzf = utils._EngineFloat64ArrayType(
        bdzf, {'fun_name': fun_name, 'entity': 'bdzf'},
        exp_shape=(_l_py+1, _m_py+1),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lamda = utils._EngineFloat64ScalarType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_l_py+1, _m_py+1, _n_py+1),
        mutual_sorder=_m_sorder,
    )
    _f = _f_cls.to_ctypes()
    _pertrb = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xs,
        _xf,
        _l,
        _lbdcnd,
        _bdxs,
        _bdxf,
        _ys,
        _yf,
        _m,
        _mbdcnd,
        _bdys,
        _bdyf,
        _zs,
        _zf,
        _n,
        _nbdcnd,
        _bdzs,
        _bdzf,
        _lamda,
        _ldf,
        _ldf2,
        _f,
        _pertrb,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nbdcnd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nbdcnd >= 0 and nbdcnd <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, zs = {:s} and zf = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: zs < zf.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mbdcnd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mbdcnd >= 0 and mbdcnd <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ys = {:s} and yf = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ys < yf',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lbdcnd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lbdcnd >= 0 and lbdcnd <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l >= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xs = {:s} and xf = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xs < xf.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'lambda > 0.0 in Helmholtz\'s equation -- a solution may not exist.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _pertrb_py = utils._EngineFloat64ScalarType.to_py(_pertrb)
        return_data = _pertrb_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['pertrb'],
            )(
                _pertrb_py,
            )
    _err_class.manage()
    return return_data

def dim2_triangulate(h, m, n, nb, places, indx, sdindx, isin, data=None):
    u"""
    Triangulation of plane region.

    For full information please refer to the NAG Library document for
    d03ma

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03maf.html

    Parameters
    ----------
    h : float
        h, the required length for the sides of the triangles of the
        uniform mesh.

    m : int
        The value m such that all points inside the region satisfy the
        documented inequalities.

    n : int
        The value n such that all points inside the region satisfy the
        documented inequalities.

    nb : int
        The number of times a triangle side is bisected to find a point
        on the boundary. A value of 10 is adequate for most purposes
        (see [Accuracy]).

    places : float, shape (2, `sdindx`), modified in place
        On exit: the x and y coordinates respectively of the ith point
        of the triangulation.

    indx : int, shape (4, `sdindx`), modified in place
        On exit: `indx`[0,i-1] contains i if point i is inside the
        region and -i if it is on the boundary. For each triangle side
        between points i and j with j > i, `indx`[k-1,i-1], k > 1,
        contains j or -j according to whether point j is internal or on
        the boundary. There can never be more than three such points. If
        there are less, some values `indx`[k-1,i-1], k > 1, are zero.

    sdindx : int
        The second dimension of the arrays `places` and `indx`.

    isin : callable retval = isin(x, y, data=None)
        `isin` must return the value 1 if the given point (`x`,`y`) lies
        inside the region, and 0 if it lies outside.

        Parameters
        ~~~~~~~~~~
        x : float
            The coordinates of the given point.

        y : float
            The coordinates of the given point.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : int
            The value 1 if the given point (`x`,`y`) lies inside the
            region, and 0 if it lies outside.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    npts : int
        The number of points in the triangulation.

    Raises
    ------
    NagValueError
        (`errno` 1)
            `sdindx` is too small: `sdindx` = *<value>*.

        (`errno` 2)
            A point inside the region violates one of the constraints.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` > 2.

        (`errno` 5)
            On entry, `n` = *<value>*.

            Constraint: `n` > 2.

        (`errno` 6)
            On entry, `nb` = *<value>*.

            Constraint: `nb` > 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_triangulate'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_isin_name = fun_name + ':isin'

    isinh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_INT, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_isinh(_isin, _isinsub, en_data, x, y, retval, _iuser, _ruser):
        """Helper for isin."""
        try:
            _x_isinh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_isinh = utils._EngineFloat64ScalarType.to_py(y.contents)
            if data is not None:
                _retval_isinh = isin(
                    _x_isinh,
                    _y_isinh,
                    data,
                )
            else:
                _retval_isinh = isin(
                    _x_isinh,
                    _y_isinh,
                )
            utils._EngineIntScalarType(
                _retval_isinh, {'fun_name': cb_isin_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03maft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nb.
        utils._BYREF_INT, # Output, npts.
        utils._EngineFloat64ArrayType.c_type, # Output, places[2, sdindx].
        utils._EngineIntArrayType.c_type, # Output, indx[4, sdindx].
        utils._BYREF_INT, # Input, sdindx.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        isinh,
        utils._EngineFloat64ArrayType.c_type, # Workspace, dist.
        utils._BYREF_INT, # Input, sddist.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((places, 'places'), (indx, 'indx')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _npts = utils._EngineIntScalarType.empty_ctypes()
    _sdindx = utils._EngineIntScalarType(
        sdindx, {'fun_name': fun_name, 'entity': 'sdindx'},
    ).to_ctypes()
    _sdindx_py = utils._EngineIntScalarType.to_py(_sdindx)
    _places_cls = utils._EngineFloat64ArrayType(
        places, {'fun_name': fun_name, 'entity': 'places'},
        exp_shape=(2, _sdindx_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _places = _places_cls.to_ctypes(can_reference_contents=False)
    _indx_cls = utils._EngineIntArrayType(
        indx, {'fun_name': fun_name, 'entity': 'indx'},
        exp_shape=(4, _sdindx_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _indx = _indx_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(isin, {'fun_name': fun_name, 'entity': 'isin'})
    _isin = utils._UNUSED_CALLBACK
    _isinsub = utils._UNUSED_CALLBACK
    _isinh = utils._cast_callback(
        isinh, cb_isinh,
        {'fun_name': fun_name, 'entity': 'isin'},
    )
    _dist = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _sddist = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _h,
        _m,
        _n,
        _nb,
        _npts,
        _places,
        _indx,
        _sdindx,
        _isin,
        _isinsub,
        _isinh,
        _dist,
        _sddist,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'sdindx is too small: sdindx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A point inside the region violates one of the constraints.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _places_cls.refresh_py(_places)
        _indx_cls.refresh_py(_indx)
        _npts_py = utils._EngineIntScalarType.to_py(_npts)
        return_data = _npts_py
    _err_class.manage()
    return return_data

def dim1_blackscholes_fd(kopt, x, mesh, ns, s, nt, t, tdpar, r, q, sigma, alpha, ntkeep, f, theta, delta, gamma, lamda, rho):
    u"""
    Finite difference solution of the Black--Scholes equations.

    For full information please refer to the NAG Library document for
    d03nc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03ncf.html

    Parameters
    ----------
    kopt : int
        Specifies the kind of option to be valued.

        `kopt` = 1

          A European call option.

        `kopt` = 2

          An American call option.

        `kopt` = 3

          A European put option.

        `kopt` = 4

          An American put option.

    x : float
        The exercise price X.

    mesh : str, length 1
        Indicates the type of finite difference mesh to be used:

        `mesh` = 'U'

          Uniform mesh.

        `mesh` = 'C'

          Custom mesh supplied by you.

    ns : int
        The number of stock prices to be used in the finite difference
        mesh.

    s : float, shape (`ns`), modified in place
        On entry: if `mesh` = 'C', `s`[i-1] must contain the ith stock
        price in the mesh, for i = 1,2,...,`ns`. These values should be
        in increasing order, with `s`[0] = S_min and `s`[`ns`-1] =
        S_max.

        If `mesh` = 'U', `s`[0] must be set to S_min and `s`[`ns`-1] to
        S_max, but `s`[1],`s`[2],...,`s`[`ns`-2] need not be
        initialized, as they will be set internally by the function in
        order to define a uniform mesh.

        On exit: if `mesh` = 'U', the elements of `s` define a uniform
        mesh over [S_min,S_max].

        If `mesh` = 'C', the elements of `s` are unchanged.

    nt : int
        The number of time-steps to be used in the finite difference
        method.

    t : float, shape (`nt`), modified in place
        On entry: if `mesh` = 'C' then `t`[j-1] must contain the jth
        time in the mesh, for j = 1,2,...,`nt`. These values should be
        in increasing order, with `t`[0] = t_min and `t`[`nt`-1] =
        t_max.

        If `mesh` = 'U' then `t`[0] must be set to t_min and `t`[`nt`-1]
        to t_max, but `t`[1],`t`[2],...,`t`[`nt`-2] need not be
        initialized, as they will be set internally by the function in
        order to define a uniform mesh.

        On exit: if `mesh` = 'U', the elements of `t` define a uniform
        mesh over [t_min,t_max].

        If `mesh` = 'C', the elements of `t` are unchanged.

    tdpar : bool, shape (3)
        Specifies whether or not various arguments are time-dependent.
        More precisely, r is time-dependent if `tdpar`[0] = True and
        constant otherwise. Similarly, `tdpar`[1] specifies whether q is
        time-dependent and `tdpar`[2] specifies whether sigma is
        time-dependent.

    r : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `tdpar`[0]= True: `nt`; otherwise: 1.

        If `tdpar`[0] = True then `r`[j-1] must contain the value of the
        risk-free interest rate r(t) at the jth time in the mesh, for j
        = 1,2,...,`nt`.

        If `tdpar`[0] = False then `r`[0] must contain the constant
        value of the risk-free interest rate r.

        The remaining elements need not be set.

    q : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `tdpar`[1]= True: `nt`; otherwise: 1.

        If `tdpar`[1] = True then `q`[j-1] must contain the value of the
        continuous dividend q(t) at the jth time in the mesh, for j =
        1,2,...,`nt`.

        If `tdpar`[1] = False then `q`[0] must contain the constant
        value of the continuous dividend q.

        The remaining elements need not be set.

    sigma : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `tdpar`[2]= True: `nt`; otherwise: 1.

        If `tdpar`[2] = True then `sigma`[j-1] must contain the value of
        the volatility sigma(t) at the jth time in the mesh, for j =
        1,2,...,`nt`.

        If `tdpar`[2] = False then `sigma`[0] must contain the constant
        value of the volatility sigma.

        The remaining elements need not be set.

    alpha : float
        The value of lambda to be used in the time-stepping scheme.
        Typical values include:

        `alpha` = 0.0

          Explicit forward Euler scheme.

        `alpha` = 0.5

          Implicit Crank--Nicolson scheme.

        `alpha` = 1.0

          Implicit backward Euler scheme.

        The value 0.5 gives second-order accuracy in time.

        Values greater than 0.5 give unconditional stability.

        Since 0.5 is at the limit of unconditional stability this value
        does not damp oscillations.

    ntkeep : int
        The number of solutions to be stored in the time direction. The
        function calculates the solution backwards from `t`[`nt`-1] to
        `t`[0] at all times in the mesh. These time solutions and the
        corresponding Greeks will be stored at times `t`[i-1], for i =
        1,2,...,`ntkeep`, in the arrays `f`, `theta`, `delta`, `gamma`,
        `lamda` and `rho`. Other time solutions will be discarded. To
        store all time solutions set `ntkeep` = `nt`.

    f : float, shape (`ns`, `ntkeep`), modified in place
        On exit: `f`[i-1,j-1], for j = 1,2,...,`ntkeep`, for i =
        1,2,...,`ns`, contains the value f of the option at the ith mesh
        point `s`[i-1] at time `t`[j-1].

    theta : float, shape (`ns`, `ntkeep`), modified in place
        On exit: the values of various Greeks at the ith mesh point
        `s`[i-1] at time `t`[j-1], as follows:

            [table omitted]

    delta : float, shape (`ns`, `ntkeep`), modified in place
        On exit: the values of various Greeks at the ith mesh point
        `s`[i-1] at time `t`[j-1], as follows:

            [table omitted]

    gamma : float, shape (`ns`, `ntkeep`), modified in place
        On exit: the values of various Greeks at the ith mesh point
        `s`[i-1] at time `t`[j-1], as follows:

            [table omitted]

    lamda : float, shape (`ns`, `ntkeep`), modified in place
        On exit: the values of various Greeks at the ith mesh point
        `s`[i-1] at time `t`[j-1], as follows:

            [table omitted]

    rho : float, shape (`ns`, `ntkeep`), modified in place
        On exit: the values of various Greeks at the ith mesh point
        `s`[i-1] at time `t`[j-1], as follows:

            [table omitted]

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ntkeep` = *<value>* and `nt` = *<value>*.

            Constraint: `ntkeep` <= `nt`.

        (`errno` 1)
            On entry, `ntkeep` = *<value>*.

            Constraint: `ntkeep` >= 1.

        (`errno` 1)
            On entry, `alpha` = *<value>*.

            Constraint: `alpha` <= 1.0.

        (`errno` 1)
            On entry, `alpha` = *<value>*.

            Constraint: `alpha` >= 0.0.

        (`errno` 1)
            On entry, `mesh` = *<value>*.

            Constraint: `mesh` = 'U' or 'C'.

        (`errno` 1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] >= 0.0.

        (`errno` 1)
            On entry, `s`[0] = *<value>*.

            Constraint: `s`[0] >= 0.0.

        (`errno` 1)
            On entry, `nt` = *<value>*.

            Constraint: `nt` >= 2.

        (`errno` 1)
            On entry, `ns` = *<value>*.

            Constraint: `ns` >= 2.

        (`errno` 1)
            On entry, `kopt` = *<value>*.

            Constraint: `kopt` = 1, 2, 3 or 4.

        (`errno` 2)
            On entry, `t`[`nt`-1] = *<value>* and `t`[0] = *<value>*.

            Constraint: `t`[`nt`-1] > `t`[0].

        (`errno` 2)
            On entry, `s`[`ns`-1] = *<value>* and `s`[0] = *<value>*.

            Constraint: `s`[`ns`-1] > `s`[0].

        (`errno` 3)
            On entry, `t`[0] <= `t`[*<value>*].

            Constraint: when `mesh` = 'C', `t`[i-1] < `t`[i], for i =
            1,2,...,`nt`-1.

        (`errno` 3)
            On entry, `s`[0] <= `s`[*<value>*].

            Constraint: when `mesh` = 'C', `s`[i-1] < `s`[i], for i =
            1,2,...,`ns`-1.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_blackscholes_fd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03ncft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, kopt.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_STRING, # Input, mesh(len=1).
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, s[ns].
        utils._BYREF_INT, # Input, nt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, t[nt].
        utils._EngineLogicalArrayType.c_type, # Input, tdpar[3].
        utils._EngineFloat64ArrayType.c_type, # Input, r[*].
        utils._EngineFloat64ArrayType.c_type, # Input, q[*].
        utils._EngineFloat64ArrayType.c_type, # Input, sigma[*].
        utils._BYREF_FLOAT64, # Input, alpha.
        utils._BYREF_INT, # Input, ntkeep.
        utils._EngineFloat64ArrayType.c_type, # Output, f[ldf, ntkeep].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldf, ntkeep].
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldf, ntkeep].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldf, ntkeep].
        utils._EngineFloat64ArrayType.c_type, # Output, lamda[ldf, ntkeep].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldf, ntkeep].
        utils._BYREF_INT, # Input, ldf.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mesh.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((f, 'f'), (theta, 'theta'), (delta, 'delta'), (gamma, 'gamma'), (lamda, 'lamda'), (rho, 'rho')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _kopt = utils._EngineIntScalarType(
        kopt, {'fun_name': fun_name, 'entity': 'kopt'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _mesh = utils._EngineCharScalarType(
        mesh, {'fun_name': fun_name, 'entity': 'mesh'}, exp_chrlen=1,
    ).to_ctypes()
    _ns = utils._EngineIntScalarType(
        ns, {'fun_name': fun_name, 'entity': 'ns'},
    ).to_ctypes()
    _ns_py = utils._EngineIntScalarType.to_py(_ns)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_ns_py,),
    )
    _s = _s_cls.to_ctypes()
    _nt = utils._EngineIntScalarType(
        nt, {'fun_name': fun_name, 'entity': 'nt'},
    ).to_ctypes()
    _nt_py = utils._EngineIntScalarType.to_py(_nt)
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nt_py,),
    )
    _t = _t_cls.to_ctypes()
    _tdpar = utils._EngineLogicalArrayType(
        tdpar, {'fun_name': fun_name, 'entity': 'tdpar'},
        exp_shape=(3,),
    ).to_ctypes()
    if utils._EngineLogicalArrayType.get_element(tdpar, {'fun_name': fun_name, 'entity': 'tdpar'}, (0,)):
        _r_dim1 = _nt_py
    else:
        _r_dim1 = 1
    exp_shape_r = (_r_dim1,)
    _r = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=exp_shape_r,
    ).to_ctypes()
    if utils._EngineLogicalArrayType.get_element(tdpar, {'fun_name': fun_name, 'entity': 'tdpar'}, (1,)):
        _q_dim1 = _nt_py
    else:
        _q_dim1 = 1
    exp_shape_q = (_q_dim1,)
    _q = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
    ).to_ctypes()
    if utils._EngineLogicalArrayType.get_element(tdpar, {'fun_name': fun_name, 'entity': 'tdpar'}, (2,)):
        _sigma_dim1 = _nt_py
    else:
        _sigma_dim1 = 1
    exp_shape_sigma = (_sigma_dim1,)
    _sigma = utils._EngineFloat64ArrayType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
        exp_shape=exp_shape_sigma,
    ).to_ctypes()
    _alpha = utils._EngineFloat64ScalarType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
    ).to_ctypes()
    _ntkeep = utils._EngineIntScalarType(
        ntkeep, {'fun_name': fun_name, 'entity': 'ntkeep'},
    ).to_ctypes()
    _ldf_py = utils._EngineFloat64ArrayType.get_shape(f, {'fun_name': fun_name, 'entity': 'f'}, exp_rank=2)[0]
    _ldf = utils._EngineIntScalarType(
        _ldf_py, {'fun_name': fun_name, 'entity': '_ldf'},
    ).to_ctypes()
    _ntkeep_py = utils._EngineIntScalarType.to_py(_ntkeep)
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_ns_py, _ntkeep_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_ns_py, _ntkeep_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_ns_py, _ntkeep_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_ns_py, _ntkeep_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _lamda_cls = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_ns_py, _ntkeep_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _lamda = _lamda_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_ns_py, _ntkeep_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _kopt,
        _x,
        _mesh,
        _ns,
        _s,
        _nt,
        _t,
        _tdpar,
        _r,
        _q,
        _sigma,
        _alpha,
        _ntkeep,
        _f,
        _theta,
        _delta,
        _gamma,
        _lamda,
        _rho,
        _ldf,
        _work,
        _iwork,
        _errbuf,
        _ierr,
        _mesh._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ntkeep = {:s} and nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ntkeep <= nt.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ntkeep = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ntkeep >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, alpha = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: alpha <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, alpha = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: alpha >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mesh = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mesh = \'U\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: t[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: s[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nt >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ns = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ns >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kopt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kopt = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kopt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kopt = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[nt-1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    't[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: t[nt-1] > t[0].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s[ns-1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    's[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: s[ns-1] > s[0].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[0] <= t[{:s}].'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when mesh = \'C\', t[i-1] < t[i], for i = 1,2,...,nt-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s[0] <= s[{:s}].'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when mesh = \'C\', s[i-1] < s[i], for i = 1,2,...,ns-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _t_cls.refresh_py(_t)
        _f_cls.refresh_py(_f)
        _theta_cls.refresh_py(_theta)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _lamda_cls.refresh_py(_lamda)
        _rho_cls.refresh_py(_rho)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_blackscholes_closed(kopt, x, s, t, tmat, tdpar, r, q, sigma):
    u"""
    Analytic solution of the Black--Scholes equations.

    For full information please refer to the NAG Library document for
    d03nd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03ndf.html

    Parameters
    ----------
    kopt : int
        Specifies the kind of option to be valued:

        `kopt` = 1

          A European call option.

        `kopt` = 2

          An American call option.

        `kopt` = 3

          A European put option.

    x : float
        The exercise price X.

    s : float
        The stock price at which the option value and the Greeks should
        be evaluated.

    t : float
        The time at which the option value and the Greeks should be
        evaluated.

    tmat : float
        The maturity time of the option.

    tdpar : bool, shape (3)
        Specifies whether or not various arguments are time-dependent.
        More precisely, r is time-dependent if `tdpar`[0] = True and
        constant otherwise. Similarly, `tdpar`[1] specifies whether q is
        time-dependent and `tdpar`[2] specifies whether sigma is
        time-dependent.

    r : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `tdpar`[0]= True: 3; otherwise: 1.

        If `tdpar`[0] = False then `r`[0] must contain the constant
        value of r. The remaining elements need not be set.

        If `tdpar`[0] = True then `r`[0] must contain the value of r at
        time `t` and `r`[1] must contain its average instantaneous value
        over the remaining life of the option:

            r^= int_`t`^`tmat`r(zeta)d zeta.

        The auxiliary function ``dim1_blackscholes_means`` may be used
        to construct `r` from a set of values of r at discrete times.

    q : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `tdpar`[1]= True: 3; otherwise: 1.

        If `tdpar`[1] = False then `q`[0] must contain the constant
        value of q. The remaining elements need not be set.

        If `tdpar`[1] = True then `q`[0] must contain the constant value
        of q and `q`[1] must contain its average instantaneous value
        over the remaining life of the option:

            q^= int_`t`^`tmat`q(zeta)d zeta.

        The auxiliary function ``dim1_blackscholes_means`` may be used
        to construct `q` from a set of values of q at discrete times.

    sigma : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `tdpar`[2]= True: 3; otherwise: 1.

        If `tdpar`[2] = False then `sigma`[0] must contain the constant
        value of sigma. The remaining elements need not be set.

        If `tdpar`[2] = True then `sigma`[0] must contain the value of
        sigma at time `t`, `sigma`[1] the average instantaneous value
        sigma^, and `sigma`[2] the second-order average sigma_, where:

            sigma^= int_`t`^`tmat`sigma(zeta)d zeta,

            sigma_= (int_`t`^`tmat`sigma^2(zeta)d zeta)^(1/2).

        The auxiliary function ``dim1_blackscholes_means`` may be used
        to compute `sigma` from a set of values at discrete times.

    Returns
    -------
    f : float
        The value f of the option at the stock price `s` and time `t`.

    theta : float
        The values of various Greeks at the stock price `s` and time
        `t`, as follows:

            [table omitted]

    delta : float
        The values of various Greeks at the stock price `s` and time
        `t`, as follows:

            [table omitted]

    gamma : float
        The values of various Greeks at the stock price `s` and time
        `t`, as follows:

            [table omitted]

    lamda : float
        The values of various Greeks at the stock price `s` and time
        `t`, as follows:

            [table omitted]

    rho : float
        The values of various Greeks at the stock price `s` and time
        `t`, as follows:

            [table omitted]

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `sigma`[*<value>*] = *<value>*.

            Constraint: `sigma`[i-1] > 0.0.

        (`errno` 1)
            On entry, `tmat` = *<value>* and `t` = *<value>*.

            Constraint: `tmat` >= `t`.

        (`errno` 1)
            On entry, `t` = *<value>*.

            Constraint: `t` >= 0.0.

        (`errno` 1)
            On entry, `s` = *<value>*.

            Constraint: `s` >= 0.0.

        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

        (`errno` 1)
            On entry, `q`[0] is not equal to 0.0 with American call
            option. `q`[0] = *<value>*.

        (`errno` 1)
            On entry, `kopt` = *<value>*.

            Constraint: `kopt` = 1, 2 or 3.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_blackscholes_closed'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03ndft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, kopt.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, s.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, tmat.
        utils._EngineLogicalArrayType.c_type, # Input, tdpar[3].
        utils._EngineFloat64ArrayType.c_type, # Input, r[*].
        utils._EngineFloat64ArrayType.c_type, # Input, q[*].
        utils._EngineFloat64ArrayType.c_type, # Input, sigma[*].
        utils._BYREF_FLOAT64, # Output, f.
        utils._BYREF_FLOAT64, # Output, theta.
        utils._BYREF_FLOAT64, # Output, delta.
        utils._BYREF_FLOAT64, # Output, gamma.
        utils._BYREF_FLOAT64, # Output, lamda.
        utils._BYREF_FLOAT64, # Output, rho.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _kopt = utils._EngineIntScalarType(
        kopt, {'fun_name': fun_name, 'entity': 'kopt'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tmat = utils._EngineFloat64ScalarType(
        tmat, {'fun_name': fun_name, 'entity': 'tmat'},
    ).to_ctypes()
    _tdpar = utils._EngineLogicalArrayType(
        tdpar, {'fun_name': fun_name, 'entity': 'tdpar'},
        exp_shape=(3,),
    ).to_ctypes()
    if utils._EngineLogicalArrayType.get_element(tdpar, {'fun_name': fun_name, 'entity': 'tdpar'}, (0,)):
        _r_dim1 = 3
    else:
        _r_dim1 = 1
    exp_shape_r = (_r_dim1,)
    _r = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=exp_shape_r,
    ).to_ctypes()
    if utils._EngineLogicalArrayType.get_element(tdpar, {'fun_name': fun_name, 'entity': 'tdpar'}, (1,)):
        _q_dim1 = 3
    else:
        _q_dim1 = 1
    exp_shape_q = (_q_dim1,)
    _q = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
    ).to_ctypes()
    if utils._EngineLogicalArrayType.get_element(tdpar, {'fun_name': fun_name, 'entity': 'tdpar'}, (2,)):
        _sigma_dim1 = 3
    else:
        _sigma_dim1 = 1
    exp_shape_sigma = (_sigma_dim1,)
    _sigma = utils._EngineFloat64ArrayType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
        exp_shape=exp_shape_sigma,
    ).to_ctypes()
    _f = utils._EngineFloat64ScalarType.empty_ctypes()
    _theta = utils._EngineFloat64ScalarType.empty_ctypes()
    _delta = utils._EngineFloat64ScalarType.empty_ctypes()
    _gamma = utils._EngineFloat64ScalarType.empty_ctypes()
    _lamda = utils._EngineFloat64ScalarType.empty_ctypes()
    _rho = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _kopt,
        _x,
        _s,
        _t,
        _tmat,
        _tdpar,
        _r,
        _q,
        _sigma,
        _f,
        _theta,
        _delta,
        _gamma,
        _lamda,
        _rho,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma[i-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tmat = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tmat >= t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: t >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: s >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q[0] is not equal to 0.0 with American call option.',
                    'q[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kopt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kopt = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kopt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kopt = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_py = utils._EngineFloat64ScalarType.to_py(_f)
        _theta_py = utils._EngineFloat64ScalarType.to_py(_theta)
        _delta_py = utils._EngineFloat64ScalarType.to_py(_delta)
        _gamma_py = utils._EngineFloat64ScalarType.to_py(_gamma)
        _lamda_py = utils._EngineFloat64ScalarType.to_py(_lamda)
        _rho_py = utils._EngineFloat64ScalarType.to_py(_rho)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['f', 'theta', 'delta', 'gamma', 'lamda', 'rho'],
        )(
            _f_py,
            _theta_py,
            _delta_py,
            _gamma_py,
            _lamda_py,
            _rho_py,
        )
    _err_class.manage()
    return return_data

def dim1_blackscholes_means(t0, tmat, ntd, td, phid, phiav):
    u"""
    Compute average values for ``dim1_blackscholes_closed``.

    For full information please refer to the NAG Library document for
    d03ne

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03nef.html

    Parameters
    ----------
    t0 : float
        The current time t_0.

    tmat : float
        The maturity time T.

    ntd : int
        The number of discrete times at which phi is given.

    td : float, shape (`ntd`)
        The discrete times at which phi is specified.

    phid : float, shape (`ntd`)
        `phid`[i-1] must contain the value of phi at time `td`[i-1], for
        i = 1,2,...,`ntd`.

    phiav : float, shape (3), modified in place
        On exit: `phiav`[0] contains the value of phi interpolated to
        t_0, `phiav`[1] contains the first-order average phi^ and
        `phiav`[2] contains the second-order average phi_, where:

            phi^= 1/(T-t_0)int_t_0^T phi(zeta)d zeta, phi_=
            (1/(T-t_0)int_t_0^T phi^2(zeta)d zeta)^(1/2).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `td`[0] <= `td`[*<value>*].

            Constraint: `td`[0] < `td`[1] < ... < `td`[`ntd`-1].

        (`errno` 1)
            On entry, `ntd` = *<value>*.

            Constraint: `ntd` >= 2.

        (`errno` 1)
            On entry, `tmat` = *<value>*, `td`[0] = *<value>* and
            `td`[`ntd`-1] = *<value>*.

            Constraint: `td`[0] <= `tmat` <= `td`[`ntd`-1].

        (`errno` 1)
            On entry, `t0` = *<value>*, `td`[0] = *<value>* and
            `td`[`ntd`-1] = *<value>*.

            Constraint: `td`[0] <= `t0` <= `td`[`ntd`-1].

        (`errno` 2)
            Unexpected failure in internal call to spline function.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_blackscholes_means'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03neft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t0.
        utils._BYREF_FLOAT64, # Input, tmat.
        utils._BYREF_INT, # Input, ntd.
        utils._EngineFloat64ArrayType.c_type, # Input, td[ntd].
        utils._EngineFloat64ArrayType.c_type, # Input, phid[ntd].
        utils._EngineFloat64ArrayType.c_type, # Output, phiav[3].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _t0 = utils._EngineFloat64ScalarType(
        t0, {'fun_name': fun_name, 'entity': 't0'},
    ).to_ctypes()
    _tmat = utils._EngineFloat64ScalarType(
        tmat, {'fun_name': fun_name, 'entity': 'tmat'},
    ).to_ctypes()
    _ntd = utils._EngineIntScalarType(
        ntd, {'fun_name': fun_name, 'entity': 'ntd'},
    ).to_ctypes()
    _ntd_py = utils._EngineIntScalarType.to_py(_ntd)
    _td = utils._EngineFloat64ArrayType(
        td, {'fun_name': fun_name, 'entity': 'td'},
        exp_shape=(_ntd_py,),
    ).to_ctypes()
    _phid = utils._EngineFloat64ArrayType(
        phid, {'fun_name': fun_name, 'entity': 'phid'},
        exp_shape=(_ntd_py,),
    ).to_ctypes()
    _phiav_cls = utils._EngineFloat64ArrayType(
        phiav, {'fun_name': fun_name, 'entity': 'phiav'},
        exp_shape=(3,),
        check_contents=False,
    )
    _phiav = _phiav_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _t0,
        _tmat,
        _ntd,
        _td,
        _phid,
        _phiav,
        _work,
        _lwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, td[0] <= td[{:s}].'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: td[0] < td[1] < ... < td[ntd-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ntd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ntd >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry,',
                    'tmat = {:s}, td[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'td[ntd-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: td[0] <= tmat <= td[ntd-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t0 = {:s}, td[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'td[ntd-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: td[0] <= t0 <= td[ntd-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected failure in internal call to spline function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _phiav_cls.refresh_py(_phiav)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_parab_fd(npde, m, ts, tout, pdedef, bndary, u, npts, x, acc, comm, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of parabolic PDEs, method of lines, finite
    differences, one space variable.

    For full information please refer to the NAG Library document for
    d03pc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pcf.html

    Parameters
    ----------
    npde : int
        The number of PDEs in the system to be solved.

    m : int
        The coordinate system used:

        `m` = 0

          Indicates Cartesian coordinates.

        `m` = 1

          Indicates cylindrical polar coordinates.

        `m` = 2

          Indicates spherical polar coordinates.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, u, ux, p, q, r, ires,
        data=None)
        `pdedef` must compute the functions P_{i,j}, Q_i and R_i which
        define the system of PDEs. `pdedef` is called approximately
        midway between each pair of mesh points in turn by
        ``dim1_parab_fd``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`), to be modified in
        place
            On exit: `p`[i-1,j-1] must be set to the value of
            P_{i,j}(x,t,U,U_x), for j = 1,2,...,`npde`, for i =
            1,2,...,`npde`.

        q : float, ndarray, shape (`npde`), to be modified in place
            On exit: `q`[i-1] must be set to the value of
            Q_i(x,t,U,U_x), for i = 1,2,...,`npde`.

        r : float, ndarray, shape (`npde`), to be modified in place
            On exit: `r`[i-1] must be set to the value of
            R_i(x,t,U,U_x), for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    bndary : callable ires = bndary(npde, t, u, ux, ibnd, beta, gamma,
        ires, data=None)
        `bndary` must compute the functions beta_i and gamma_i which
        define the boundary conditions as in equation (3).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ibnd : int
            Determines the position of the boundary conditions.

            `ibnd` = 0

              `bndary` must set up the coefficients of the left-hand
              boundary, x = a.

            `ibnd` != 0

              Indicates that `bndary` must set up the coefficients of
              the right-hand boundary, x = b.

        beta : float, ndarray, shape (`npde`), to be modified in place
            On exit: `beta`[i-1] must be set to the value of beta_i(x,t)
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        gamma : float, ndarray, shape (`npde`), to be modified in place
            On exit: `gamma`[i-1] must be set to the value of
            gamma_i(x,t,U,U_x) at the boundary specified by `ibnd`, for
            i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    u : float, shape (`npde`, `npts`), modified in place
        On entry: the initial values of U(x,t) at t = `ts` and the mesh
        points `x`[j-1], for j = 1,2,...,`npts`.

        On exit: `u`[i-1,j-1] will contain the computed solution at t =
        `ts`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`)
        The mesh points in the spatial direction. `x`[0] must specify
        the left-hand boundary, a, and `x`[`npts`-1] must specify the
        right-hand boundary, b.

    acc : float
        A positive quantity for controlling the local error estimate in
        the time integration. If E(i,j) is the estimated error for U_i
        at the jth mesh point, the error test is:

            |E(i,j)| = `acc`*(1.0+|`u`[i-1,j-1]|).

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    itask : int
        Specifies the task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout`.

        `itask` = 2

          One step and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

    itrace : int
        The level of trace information required from ``dim1_parab_fd``
        and the underlying ODE solver. `itrace` may take the value -1,
        0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` > 0

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases. You are advised to set `itrace` = 0, unless you are
        experienced with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_fd``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `acc` = *<value>*.

            Constraint: `acc` > 0.0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `m` = *<value>* and `x`[0] = *<value>*.

            Constraint: `m` <= 0 or `x`[0] >= 0.0

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = 0, 1 or 2.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2 or 3.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `acc` was too small to start integration: `acc` = *<value>*.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef` or
            `bndary`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 14)
            Flux function appears to depend on time derivatives.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied value of `acc`. `ts` =
            *<value>*, `acc` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef` or `bndary`. Integration is successful as
            far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but a small change in `acc` is
            unlikely to result in a changed solution. `acc` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_fd'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, r[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, q, r, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            _q_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                q,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _r_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                )
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ux_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_bndaryh,),
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _beta_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                beta,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _gamma_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                gamma,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                )
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03pcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[npde, npts].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._BYREF_FLOAT64, # Input, acc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_npde_py, _npts_py),
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _acc = utils._EngineFloat64ScalarType(
        acc, {'fun_name': fun_name, 'entity': 'acc'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes((6*_npde_py+10)*_npde_py*_npts_py+(3*_npde_py+21)*_npde_py+7*_npts_py+54)
        comm['rsave'] = _rsave
    else:
        _rsave = comm['rsave']
    _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
    _lrsave = utils._EngineIntScalarType(
        _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave'},
    ).to_ctypes()
    if _comm_needs_init:
        _isave = utils._EngineIntArrayType.empty_ctypes(_npde_py*_npts_py+24)
        comm['isave'] = _isave
    else:
        _isave = comm['isave']
    _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
    _lisave = utils._EngineIntScalarType(
        _lisave_py, {'fun_name': fun_name, 'entity': '_lisave'},
    ).to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _m,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _u,
        _npts,
        _x,
        _acc,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: acc > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and x[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= 0 or x[0] >= 0.0',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied value of acc.',
                    'ts = {:s}, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef or bndary. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'acc was too small to start integration: acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99979:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but a small change in acc is unlikely',
                    'to result in a changed solution.',
                    'acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Flux function appears to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind'],
        )(
            _ts_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_coll(npde, m, ts, tout, pdedef, bndary, u, nbkpts, xbkpts, npoly, npts, x, uinit, acc, comm, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of parabolic PDEs, method of lines, Chebyshev C^0
    collocation, one space variable.

    For full information please refer to the NAG Library document for
    d03pd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pdf.html

    Parameters
    ----------
    npde : int
        The number of PDEs in the system to be solved.

    m : int
        The coordinate system used:

        `m` = 0

          Indicates Cartesian coordinates.

        `m` = 1

          Indicates cylindrical polar coordinates.

        `m` = 2

          Indicates spherical polar coordinates.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, nptl, u, ux, p, q, r,
        ires, data=None)
        `pdedef` must compute the values of the functions P_{i,j}, Q_i
        and R_i which define the system of PDEs.

        The functions may depend on x, t, U and U_x and must be
        evaluated at a set of points.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`nptl`)
            Contains a set of mesh points at which P_{i,j}, Q_i and R_i
            are to be evaluated. `x`[0] and `x`[`nptl`-1] contain
            successive user-supplied break-points and the elements of
            the array will satisfy `x`[0] < `x`[1] < ... <
            `x`[`nptl`-1].

        nptl : int
            The number of points at which evaluations are required (the
            value of `npoly`+1).

        u : float, ndarray, shape (`npde`, `nptl`)
            `u`[i-1,j-1] contains the value of the component U_i(x,t)
            where x = `x`[j-1], for j = 1,2,...,`nptl`, for i =
            1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`, `nptl`)
            `ux`[i-1,j-1] contains the value of the component
            dU_i(x,t)/dx where x = `x`[j-1], for j = 1,2,...,`nptl`, for
            i = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`, `nptl`), to be
        modified in place
            On exit: `p`[ `npde`*`npde`*(k-1)+ `npde`*(j-1)+ (i-1)] must
            be set to the value of P_{i,j}(x,t,U,U_x) where x =
            `x`[k-1], for k = 1,2,...,`nptl`, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npde`.

        q : float, ndarray, shape (`npde`, `nptl`), to be modified in
        place
            On exit: `q`[i-1,j-1] must be set to the value of
            Q_i(x,t,U,U_x) where x = `x`[j-1], for j = 1,2,...,`nptl`,
            for i = 1,2,...,`npde`.

        r : float, ndarray, shape (`npde`, `nptl`), to be modified in
        place
            On exit: `r`[i-1,j-1] must be set to the value of
            R_i(x,t,U,U_x) where x = `x`[j-1], for j = 1,2,...,`nptl`,
            for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_coll``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    bndary : callable ires = bndary(npde, t, u, ux, ibnd, beta, gamma,
        ires, data=None)
        `bndary` must compute the functions beta_i and gamma_i which
        define the boundary conditions as in equation (3).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must set up the coefficients of the left-hand
              boundary, x = a.

            `ibnd` != 0

              `bndary` must set up the coefficients of the right-hand
              boundary, x = b.

        beta : float, ndarray, shape (`npde`), to be modified in place
            On exit: `beta`[i-1] must be set to the value of beta_i(x,t)
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        gamma : float, ndarray, shape (`npde`), to be modified in place
            On exit: `gamma`[i-1] must be set to the value of
            gamma_i(x,t,U,U_x) at the boundary specified by `ibnd`, for
            i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_coll``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    u : float, shape (`npde`, `npts`), modified in place
        On entry: if `ind` = 1 the value of `u` must be unchanged from
        the previous call.

        On exit: `u`[i-1,j-1] will contain the computed solution at t =
        `ts`.

    nbkpts : int
        The number of break-points in the interval [a,b].

    xbkpts : float, shape (`nbkpts`)
        The values of the break-points in the space direction.
        `xbkpts`[0] must specify the left-hand boundary, a, and
        `xbkpts`[`nbkpts`-1] must specify the right-hand boundary, b.

    npoly : int
        The degree of the Chebyshev polynomial to be used in
        approximating the PDE solution between each pair of
        break-points.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`), modified in place
        On exit: the mesh points chosen by ``dim1_parab_coll`` in the
        spatial direction. The values of `x` will satisfy `x`[0] <
        `x`[1] < ... < `x`[`npts`-1].

    uinit : callable uinit(npde, npts, x, u, data=None)
        `uinit` must compute the initial values of the PDE components
        U_i(x_j,t_0), for j = 1,2,...,`npts`, for i = 1,2,...,`npde`.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        x : float, ndarray, shape (`npts`)
            `x`[j-1], contains the values of the jth mesh point, for j =
            1,2,...,`npts`.

        u : float, ndarray, shape (`npde`, `npts`), to be modified in
        place
            On exit: `u`[i-1,j-1] must be set to the initial value
            U_i(x_j,t_0), for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    acc : float
        A positive quantity for controlling the local error estimate in
        the time integration. If E(i,j) is the estimated error for U_i
        at the jth mesh point, the error test is:

            |E(i,j)| = `acc`*(1.0+|`u`[i-1,j-1]|).

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    itask : int
        Specifies the task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout`.

        `itask` = 2

          One step and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

    itrace : int
        The level of trace information required from ``dim1_parab_coll``
        and the underlying ODE solver. `itrace` may take the value -1,
        0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` > 0

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases. You are advised to set `itrace` = 0, unless you are
        experienced with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_coll``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, i = *<value>*, `xbkpts`[i-1] = *<value>*, j =
            *<value>* and `xbkpts`[j-1] = *<value>*.

            Constraint: `xbkpts`[0] < `xbkpts`[1] < ... <
            `xbkpts`[`nbkpts`-1].

        (`errno` 1)
            On entry, `acc` = *<value>*.

            Constraint: `acc` > 0.0.

        (`errno` 1)
            On entry, `npts` = *<value>*, `nbkpts` = *<value>* and
            `npoly` = *<value>*.

            Constraint: `npts` = (`nbkpts`-1)*`npoly`+1.

        (`errno` 1)
            On entry, `npoly` = *<value>*.

            Constraint: `npoly` <= 49.

        (`errno` 1)
            On entry, `npoly` = *<value>*.

            Constraint: `npoly` >= 1.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `nbkpts` = *<value>*.

            Constraint: `nbkpts` >= 2.

        (`errno` 1)
            On entry, `m` = *<value>* and `xbkpts`[0] = *<value>*.

            Constraint: `m` <= 0 or `xbkpts`[0] >= 0.0

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = 0, 1 or 2.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2 or 3.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `acc` was too small to start integration: `acc` = *<value>*.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef` or
            `bndary`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 14)
            Flux function appears to depend on time derivatives.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied value of `acc`. `ts` =
            *<value>*, `acc` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef` or `bndary`. Integration is successful as
            far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but a small change in `acc` is
            unlikely to result in a changed solution. `acc` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_coll'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, r[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, x, np, nptl, u, ut, ux, nv, v, vdot, p, q, r, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nptl_pdedefh = utils._EngineIntScalarType.to_py(nptl.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_nptl_pdedefh,),
            )
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh, _nptl_pdedefh),
                sorder=_m_sorder,
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh, _nptl_pdedefh),
                sorder=_m_sorder,
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh, _nptl_pdedefh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            _q_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                q,
                (_npde_pdedefh, _nptl_pdedefh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            _r_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_npde_pdedefh, _nptl_pdedefh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _nptl_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _nptl_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(u)
            utils._unlink_pointer(ux)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ux_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_bndaryh,),
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _beta_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                beta,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _gamma_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                gamma,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                )
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_uinit_name = fun_name + ':uinit'

    uinith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, u[npde, npts].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Output, v[nv].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_uinith(_uinit, en_data, npde, npts, nxi, x, xi, u, nv, v, _iuser, _ruser):
        """Helper for uinit."""
        try:
            _npde_uinith = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_uinith = utils._EngineIntScalarType.to_py(npts.contents)
            _x_uinith = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_uinith,),
            )
            _u_uinith = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_uinith, _npts_uinith),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                uinit(
                    _npde_uinith,
                    _npts_uinith,
                    _x_uinith,
                    _u_uinith,
                    data,
                )
            else:
                uinit(
                    _npde_uinith,
                    _npts_uinith,
                    _x_uinith,
                    _u_uinith,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _u_uinith, {'fun_name': cb_uinit_name, 'returned': 'u'}, u,
            )
            utils._unlink_pointer(u)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03pdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[npde, npts].
        utils._BYREF_INT, # Input, nbkpts.
        utils._EngineFloat64ArrayType.c_type, # Input, xbkpts[nbkpts].
        utils._BYREF_INT, # Input, npoly.
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Output, x[npts].
        utils._UNUSED_FUNCTYPE,
        uinith,
        utils._BYREF_FLOAT64, # Input, acc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _nbkpts = utils._EngineIntScalarType(
        nbkpts, {'fun_name': fun_name, 'entity': 'nbkpts'},
    ).to_ctypes()
    _nbkpts_py = utils._EngineIntScalarType.to_py(_nbkpts)
    _npoly = utils._EngineIntScalarType(
        npoly, {'fun_name': fun_name, 'entity': 'npoly'},
    ).to_ctypes()
    _npoly_py = utils._EngineIntScalarType.to_py(_npoly)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_npde_py, (_nbkpts_py-1)*_npoly_py+1),
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _xbkpts = utils._EngineFloat64ArrayType(
        xbkpts, {'fun_name': fun_name, 'entity': 'xbkpts'},
        exp_shape=(_nbkpts_py,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=((_nbkpts_py-1)*_npoly_py+1,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(uinit, {'fun_name': fun_name, 'entity': 'uinit'})
    _uinit = utils._UNUSED_CALLBACK
    _uinith = utils._cast_callback(
        uinith, cb_uinith,
        {'fun_name': fun_name, 'entity': 'uinit'},
    )
    _acc = utils._EngineFloat64ScalarType(
        acc, {'fun_name': fun_name, 'entity': 'acc'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
    )
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    if _comm_needs_init:
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes((3*_npde_py*(_npoly_py+1)+9)*_npde_py*_npts_py+3*(_npoly_py+1)**2+(_npoly_py+1)*(_npde_py*_npde_py+6*_npde_py+_nbkpts_py+1)+13*_npde_py+55)
        comm['rsave'] = _rsave
    else:
        _rsave = comm['rsave']
    _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
    _lrsave = utils._EngineIntScalarType(
        _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave'},
    ).to_ctypes()
    if _comm_needs_init:
        _isave = utils._EngineIntArrayType.empty_ctypes(_npde_py*_npts_py+24)
        comm['isave'] = _isave
    else:
        _isave = comm['isave']
    _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
    _lisave = utils._EngineIntScalarType(
        _lisave_py, {'fun_name': fun_name, 'entity': '_lisave'},
    ).to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _m,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _u,
        _nbkpts,
        _xbkpts,
        _npoly,
        _npts,
        _x,
        _uinit,
        _uinith,
        _acc,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xbkpts[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and xbkpts[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xbkpts[0] < xbkpts[1] < ... < xbkpts[nbkpts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: acc > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}, nbkpts = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: npts = (nbkpts-1)*npoly+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npoly <= 49.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npoly >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nbkpts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nbkpts >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'xbkpts[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= 0 or xbkpts[0] >= 0.0',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied value of acc.',
                    'ts = {:s}, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99980:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef or bndary. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'acc was too small to start integration: acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99976:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but a small change in acc is unlikely',
                    'to result in a changed solution.',
                    'acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Flux function appears to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _x_cls.refresh_py(_x)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind'],
        )(
            _ts_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_keller(npde, ts, tout, pdedef, bndary, u, npts, x, nleft, acc, comm, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of first-order PDEs, method of lines, Keller box
    discretization, one space variable.

    For full information please refer to the NAG Library document for
    d03pe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pef.html

    Parameters
    ----------
    npde : int
        The number of PDEs in the system to be solved.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, u, ut, ux, res, ires,
        data=None)
        `pdedef` must compute the functions G_i which define the system
        of PDEs. `pdedef` is called approximately midway between each
        pair of mesh points in turn by ``dim1_parab_keller``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ut : float, ndarray, shape (`npde`)
            `ut`[i-1] contains the value of the component dU_i(x,t)/dt,
            for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        res : float, ndarray, shape (`npde`), to be modified in place
            On exit: `res`[i-1] must contain the ith component of G, for
            i = 1,2,...,`npde`, where G is defined as

                G_i = sum_j = 1^`npde`P_{i,j}dU_j/dt,

            i.e., only terms depending explicitly on time derivatives,
            or

                G_i = sum_j = 1^`npde`P_{i,j}dU_j/dt+Q_i,

            i.e., all terms in equation (2).

            The definition of G is determined by the input value of
            `ires`.

        ires : int
            The form of G_i that must be returned in the array `res`.

            `ires` = -1

              Equation (8) must be used.

            `ires` = 1

              Equation (9) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions,
            as described below:

    bndary : callable ires = bndary(npde, t, ibnd, nobc, u, ut, res,
        ires, data=None)
        `bndary` must compute the functions G_i^L and G_i^R which define
        the boundary conditions as in equations (4) and (5).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        ibnd : int
            Determines the position of the boundary conditions.

            `ibnd` = 0

              `bndary` must compute the left-hand boundary condition at
              x = a.

            `ibnd` != 0

              Indicates that `bndary` must compute the right-hand
              boundary condition at x = b.

        nobc : int
            Specifies the number of boundary conditions at the boundary
            specified by `ibnd`.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ut : float, ndarray, shape (`npde`)
            `ut`[i-1] contains the value of the component dU_i(x,t)/dt
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        res : float, ndarray, shape (`nobc`), to be modified in place
            On exit: `res`[i-1] must contain the ith component of G^L or
            G^R, depending on the value of `ibnd`, for i =
            1,2,...,`nobc`, where G^L is defined as

                G_i^L = sum_j = 1^`npde`E_i,j^LdU_j/dt,

            i.e., only terms depending explicitly on time derivatives,
            or

                G_i^L = sum_j = 1^`npde`E_i,j^LdU_j/dt+S_i^L,

            i.e., all terms in equation (6), and similarly for G_i^R.

            The definitions of G^L and G^R are determined by the input
            value of `ires`.

        ires : int
            The form G_i^L (or G_i^R) that must be returned in the array
            `res`.

            `ires` = -1

              Equation (0) must be used.

            `ires` = 1

              Equation (1) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions,
            as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_keller``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    u : float, shape (`npde`, `npts`), modified in place
        On entry: the initial values of U(x,t) at t = `ts` and the mesh
        points `x`[j-1], for j = 1,2,...,`npts`.

        On exit: `u`[i-1,j-1] will contain the computed solution at t =
        `ts`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`)
        The mesh points in the spatial direction. `x`[0] must specify
        the left-hand boundary, a, and `x`[`npts`-1] must specify the
        right-hand boundary, b.

    nleft : int
        The number n_a of boundary conditions at the left-hand mesh
        point `x`[0].

    acc : float
        A positive quantity for controlling the local error estimate in
        the time integration. If E(i,j) is the estimated error for U_i
        at the jth mesh point, the error test is:

            |E(i,j)| = `acc`*(1.0+|`u`[i-1,j-1]|).

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    itask : int
        Specifies the task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout`.

        `itask` = 2

          Take one step and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_keller`` and the underlying ODE solver as follows:

        `itrace` <= -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` = 1

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        `itrace` = 2

          Output from the underlying ODE solver is similar to that
          produced when `itrace` = 1, except that the advisory messages
          are given in greater detail.

        `itrace` >= 3

          Output from the underlying ODE solver is similar to that
          produced when `itrace` = 2, except that the advisory messages
          are given in greater detail.

        You are advised to set `itrace` = 0.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_keller``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `acc` = *<value>*.

            Constraint: `acc` > 0.0.

        (`errno` 1)
            On entry, `nleft` = *<value>*, `npde` = *<value>*.

            Constraint: `nleft` <= `npde`.

        (`errno` 1)
            On entry, `nleft` = *<value>*.

            Constraint: `nleft` >= 0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2 or 3.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `acc` was too small to start integration: `acc` = *<value>*.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef` or
            `bndary`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied value of `acc`. `ts` =
            *<value>*, `acc` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef` or `bndary`. Integration is successful as
            far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but a small change in `acc` is
            unlikely to result in a changed solution. `acc` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_keller'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, res[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, q, res, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ut_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _res_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _res_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _res_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(ut)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nobc.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nobc].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ut, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nobc, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _nobc_bndaryh = utils._EngineIntScalarType.to_py(nobc.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ut_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npde_bndaryh,),
            )
            _res_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_nobc_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _ibnd_bndaryh,
                    _nobc_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _res_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _ibnd_bndaryh,
                    _nobc_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _res_bndaryh,
                    _ires_bndaryh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _res_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'res'}, res,
            )
            utils._unlink_pointer(res)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03peft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[npde, npts].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._BYREF_INT, # Input, nleft.
        utils._BYREF_FLOAT64, # Input, acc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_npde_py, _npts_py),
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _nleft = utils._EngineIntScalarType(
        nleft, {'fun_name': fun_name, 'entity': 'nleft'},
    ).to_ctypes()
    _acc = utils._EngineFloat64ScalarType(
        acc, {'fun_name': fun_name, 'entity': 'acc'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
    )
    _nleft_py = utils._EngineIntScalarType.to_py(_nleft)
    if _comm_needs_init:
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes((4*_npde_py+_nleft_py+14)*_npde_py*_npts_py+(3*_npde_py+21)*_npde_py+7*_npts_py+54)
        comm['rsave'] = _rsave
    else:
        _rsave = comm['rsave']
    _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
    _lrsave = utils._EngineIntScalarType(
        _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave'},
    ).to_ctypes()
    if _comm_needs_init:
        _isave = utils._EngineIntArrayType.empty_ctypes(_npde_py*_npts_py+24)
        comm['isave'] = _isave
    else:
        _isave = comm['isave']
    _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
    _lisave = utils._EngineIntScalarType(
        _lisave_py, {'fun_name': fun_name, 'entity': '_lisave'},
    ).to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _u,
        _npts,
        _x,
        _nleft,
        _acc,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: acc > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nleft = {:s}, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nleft <= npde.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nleft = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nleft >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied value of acc.',
                    'ts = {:s}, acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef or bndary. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'acc was too small to start integration: acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99979:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but a small change in acc is unlikely',
                    'to result in a changed solution.',
                    'acc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind'],
        )(
            _ts_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_convdiff(npde, ts, tout, pdedef, numflx, bndary, u, npts, x, acc, tsmax, comm, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of convection-diffusion PDEs with source terms in
    conservative form, method of lines, upwind scheme using numerical
    flux function based on Riemann solver, one space variable.

    For full information please refer to the NAG Library document for
    d03pf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pff.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : None or callable ires = pdedef(npde, t, x, u, ux, p, c, d,
        s, ires, data=None)
        `pdedef` must evaluate the functions P_{i,j}, C_i, D_i and S_i
        which partially define the system of PDEs. P_{i,j}, C_i and S_i
        may depend on x, t and U; D_i may depend on x, t, U and U_x.
        `pdedef` is called approximately midway between each pair of
        mesh points in turn by ``dim1_parab_convdiff``. None may be used
        for `pdedef` for problems in the form (2).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`), to be modified in
        place
            On exit: `p`[i-1,j-1] must be set to the value of
            P_{i,j}(x,t,U), for j = 1,2,...,`npde`, for i =
            1,2,...,`npde`.

        c : float, ndarray, shape (`npde`), to be modified in place
            On exit: `c`[i-1] must be set to the value of C_i(x,t,U),
            for i = 1,2,...,`npde`.

        d : float, ndarray, shape (`npde`), to be modified in place
            On exit: `d`[i-1] must be set to the value of
            D_i(x,t,U,U_x), for i = 1,2,...,`npde`.

        s : float, ndarray, shape (`npde`), to be modified in place
            On exit: `s`[i-1] must be set to the value of S_i(x,t,U),
            for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_convdiff``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    numflx : callable ires = numflx(npde, t, x, uleft, uright, flux,
        ires, data=None)
        `numflx` must supply the numerical flux for each PDE given the
        left and right values of the solution vector `u`. `numflx` is
        called approximately midway between each pair of mesh points in
        turn by ``dim1_parab_convdiff``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        uleft : float, ndarray, shape (`npde`)
            `uleft`[i-1] contains the left value of the component
            U_i(x), for i = 1,2,...,`npde`.

        uright : float, ndarray, shape (`npde`)
            `uright`[i-1] contains the right value of the component
            U_i(x), for i = 1,2,...,`npde`.

        flux : float, ndarray, shape (`npde`), to be modified in place
            On exit: `flux`[i-1] must be set to the numerical flux F^_i,
            for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_convdiff``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    bndary : callable ires = bndary(npde, npts, t, x, u, ibnd, g, ires,
        data=None)
        `bndary` must evaluate the functions G_i^L and G_i^R which
        describe the physical and numerical boundary conditions, as
        given by (7) and (8).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            The mesh points in the spatial direction. `x`[0] corresponds
            to the left-hand boundary, a, and `x`[`npts`-1] corresponds
            to the right-hand boundary, b.

        u : float, ndarray, shape (`npde`, 3)
            Contains the value of solution components in the boundary
            region.

            If `ibnd` = 0, `u`[i-1,j-1] contains the value of the
            component U_i(x,t) at x = `x`[j-1], for j = 1,2,...,3, for i
            = 1,2,...,`npde`.

            If `ibnd` != 0, `u`[i-1,j-1] contains the value of the
            component U_i(x,t) at x = `x`[`npts`-j], for j = 1,2,...,3,
            for i = 1,2,...,`npde`.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must evaluate the left-hand boundary condition at
              x = a.

            `ibnd` != 0

              `bndary` must evaluate the right-hand boundary condition
              at x = b.

        g : float, ndarray, shape (`npde`), to be modified in place
            On exit: `g`[i-1] must contain the ith component of either
            `g`^L or `g`^R in (7) and (8), depending on the value of
            `ibnd`, for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_convdiff``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    u : float, shape (`npde`, `npts`), modified in place
        On entry: `u`[i-1,j-1] must contain the initial value of
        U_i(x,t) at x = `x`[j-1] and t = `ts`, for j = 1,2,...,`npts`,
        for i = 1,2,...,`npde`.

        On exit: `u`[i-1,j-1] will contain the computed solution
        U_i(x,t) at x = `x`[j-1] and t = `ts`, for j = 1,2,...,`npts`,
        for i = 1,2,...,`npde`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`)
        The mesh points in the space direction. `x`[0] must specify the
        left-hand boundary, a, and `x`[`npts`-1] must specify the
        right-hand boundary, b.

    acc : float, shape (2)
        The components of `acc` contain the relative and absolute error
        tolerances used in the local error test in the time integration.

        If E(i,j) is the estimated error for U_i at the jth mesh point,
        the error test is

            E(i,j) = `acc`[0]*`u`[i-1,j-1]+`acc`[1].

    tsmax : float
        The maximum absolute step size to be allowed in the time
        integration. If `tsmax` = 0.0 then no maximum is imposed.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    itask : int
        The task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step in the time direction and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_convdiff`` and the underlying ODE solver. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` > 0

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases. You are advised to set `itrace` = 0, unless you are
        experienced with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_convdiff``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `tsmax` = *<value>*.

            Constraint: `tsmax` >= 0.0.

        (`errno` 1)
            On entry, `acc`[0] and `acc`[1] are both zero.

        (`errno` 1)
            On entry, `acc`[1] = *<value>*.

            Constraint: `acc`[1] >= 0.0.

        (`errno` 1)
            On entry, `acc`[0] = *<value>*.

            Constraint: `acc`[0] >= 0.0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2 or 3.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef`, `numflx`, or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            Values in `acc` are too small to start integration: `acc`[0]
            = *<value>*, `acc`[1] = *<value>*.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef`,
            `numflx`, or `bndary`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 14)
            The functions P, D, or C appear to depend on time
            derivatives.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `acc`. `ts` =
            *<value>*, `acc`[0] = *<value>*, `acc`[1] = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef`, `numflx`, or `bndary`. Integration is
            successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `acc` are
            unlikely to result in a changed solution. `acc`[0] =
            *<value>*, `acc`[1] = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_convdiff'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, c[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, d[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if pdedef is None:
        pdedef_auxil_ctypes = utils._EngineState._get_symbol('d03pfpt')
        pdedef_auxil_ctypes.restype = None
        pdedef_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, npde.
            utils._BYREF_FLOAT64, # Input, t.
            utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
            utils._BYREF_INT, # Input, np.
            utils._BYREF_INT, # Input, nptl.
            utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
            utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
            utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
            utils._BYREF_INT, # Input, nv.
            utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
            utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
            utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
            utils._EngineFloat64ArrayType.c_type, # Output, c[npde, np].
            utils._EngineFloat64ArrayType.c_type, # Output, d[npde, nptl].
            utils._BYREF_INT, # Input, ns.
            utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
            utils._BYREF_INT, # Input/Output, ires.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, c, d, ns, s, ires, iuser, ruser):
        """Helper for pdedef."""
        try:
            if pdedef is None:
                pdedef_auxil_ctypes(
                    en_data,
                    npde,
                    t,
                    xv,
                    np,
                    nptl,
                    u,
                    ut,
                    ux,
                    nv,
                    v,
                    vdot,
                    p,
                    c,
                    d,
                    ns,
                    s,
                    ires,
                    iuser,
                    ruser,
                )
                return
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            _c_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                c,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _d_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                d,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _s_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                s,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _p_pdedefh,
                    _c_pdedefh,
                    _d_pdedefh,
                    _s_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _p_pdedefh,
                    _c_pdedefh,
                    _d_pdedefh,
                    _s_pdedefh,
                    _ires_pdedefh,
                )
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_numflx_name = fun_name + ':numflx'

    numflxh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, uleft[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uright[npde].
        utils._EngineFloat64ArrayType.c_type, # Output, flux[npde].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
    )

    def cb_numflxh(_numflx, en_data, npde, t, x, nv, v, uleft, uright, flux, ires, _iuser, _ruser, iwsav):
        """Helper for numflx."""
        try:
            _npde_numflxh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_numflxh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_numflxh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _uleft_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                uleft,
                (_npde_numflxh,),
            )
            _uright_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                uright,
                (_npde_numflxh,),
            )
            _flux_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                flux,
                (_npde_numflxh,),
                can_reference_contents=False,
            )
            _ires_numflxh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_numflxh = numflx(
                    _npde_numflxh,
                    _t_numflxh,
                    _x_numflxh,
                    _uleft_numflxh,
                    _uright_numflxh,
                    _flux_numflxh,
                    _ires_numflxh,
                    data,
                )
            else:
                _ires_numflxh = numflx(
                    _npde_numflxh,
                    _t_numflxh,
                    _x_numflxh,
                    _uleft_numflxh,
                    _uright_numflxh,
                    _flux_numflxh,
                    _ires_numflxh,
                )
            utils._unlink_pointer(uleft)
            utils._unlink_pointer(uright)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _flux_numflxh, {'fun_name': cb_numflx_name, 'returned': 'flux'}, flux,
            )
            utils._unlink_pointer(flux)
            utils._EngineIntScalarType(
                _ires_numflxh, {'fun_name': cb_numflx_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, g[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, g, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_bndaryh = utils._EngineIntScalarType.to_py(npts.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_bndaryh,),
            )
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh, 3),
                sorder=_m_sorder,
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _g_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                g,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _npts_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _u_bndaryh,
                    _ibnd_bndaryh,
                    _g_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _npts_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _u_bndaryh,
                    _ibnd_bndaryh,
                    _g_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(x)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03pfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        numflxh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[npde, npts].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, acc[2].
        utils._BYREF_FLOAT64, # Input, tsmax.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'}, allow_none=True)
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(numflx, {'fun_name': fun_name, 'entity': 'numflx'})
    _numflx = utils._UNUSED_CALLBACK
    _numflxh = utils._cast_callback(
        numflxh, cb_numflxh,
        {'fun_name': fun_name, 'entity': 'numflx'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_npde_py, _npts_py),
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _acc = utils._EngineFloat64ArrayType(
        acc, {'fun_name': fun_name, 'entity': 'acc'},
        exp_shape=(2,),
    ).to_ctypes()
    _tsmax = utils._EngineFloat64ScalarType(
        tsmax, {'fun_name': fun_name, 'entity': 'tsmax'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes((11+9*_npde_py)*_npde_py*_npts_py+(32+3*_npde_py)*_npde_py+7*_npts_py+54)
        comm['rsave'] = _rsave
    else:
        _rsave = comm['rsave']
    _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
    _lrsave = utils._EngineIntScalarType(
        _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave'},
    ).to_ctypes()
    if _comm_needs_init:
        _isave = utils._EngineIntArrayType.empty_ctypes(_npde_py*_npts_py+24)
        comm['isave'] = _isave
    else:
        _isave = comm['isave']
    _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
    _lisave = utils._EngineIntScalarType(
        _lisave_py, {'fun_name': fun_name, 'entity': '_lisave'},
    ).to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _numflx,
        _numflxh,
        _bndary,
        _bndaryh,
        _u,
        _npts,
        _x,
        _acc,
        _tsmax,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tsmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tsmax >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc[0] and acc[1] are both zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: acc[1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, acc[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: acc[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of acc.',
                    'ts = {:s}, acc[0] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'acc[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef, numflx, or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99982:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef, numflx, or bndary. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Values in acc are too small to start integration:',
                    'acc[0] = {:s}, acc[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef, numflx, or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99978:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in acc are unlikely',
                    'to result in a changed solution.',
                    'acc[0] = {:s}, acc[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The functions P, D, or C appear to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind'],
        )(
            _ts_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_dae_fd(npde, m, ts, tout, pdedef, bndary, u, npts, x, nv, odedef, nxi, xi, neqn, rtol, atol, itol, norm, laopt, algopt, comm, lrsave, lisave, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of parabolic PDEs, coupled DAEs, method of lines,
    finite differences, one space variable.

    For full information please refer to the NAG Library document for
    d03ph

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03phf.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    m : int
        The coordinate system used:

        `m` = 0

          Indicates Cartesian coordinates.

        `m` = 1

          Indicates cylindrical polar coordinates.

        `m` = 2

          Indicates spherical polar coordinates.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, u, ux, nv, v, vdot, p,
        q, r, ires, data=None)
        `pdedef` must evaluate the functions P_{i,j}, Q_i and R_i which
        define the system of PDEs.

        The functions may depend on x, t, U, U_x and V. Q_i may depend
        linearly on V^.. `pdedef` is called approximately midway between
        each pair of mesh points in turn by ``dim1_parab_dae_fd``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in Q_j, for j = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`), to be modified in
        place
            On exit: `p`[i-1,j-1] must be set to the value of
            P_{i,j}(x,t,U,U_x,V), for j = 1,2,...,`npde`, for i =
            1,2,...,`npde`.

        q : float, ndarray, shape (`npde`), to be modified in place
            On exit: `q`[i-1] must be set to the value of
            Q_i(x,t,U,U_x,V,V^.), for i = 1,2,...,`npde`.

        r : float, ndarray, shape (`npde`), to be modified in place
            On exit: `r`[i-1] must be set to the value of
            R_i(x,t,U,U_x,V), for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_dae_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    bndary : callable ires = bndary(npde, t, u, ux, nv, v, vdot, ibnd,
        beta, gamma, ires, data=None)
        `bndary` must evaluate the functions beta_i and gamma_i which
        describe the boundary conditions, as given in [equation].

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in Q_j, for j = 1,2,...,`npde`.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must set up the coefficients of the left-hand
              boundary, x = a.

            `ibnd` != 0

              `bndary` must set up the coefficients of the right-hand
              boundary, x = b.

        beta : float, ndarray, shape (`npde`), to be modified in place
            On exit: `beta`[i-1] must be set to the value of beta_i(x,t)
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        gamma : float, ndarray, shape (`npde`), to be modified in place
            On exit: `gamma`[i-1] must be set to the value of
            gamma_i(x,t,U,U_x,V,V^.) at the boundary specified by
            `ibnd`, for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_dae_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    u : float, shape (`neqn`), modified in place
        On entry: the initial values of the dependent variables defined
        as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t_0), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t_0), for i = 1,2,...,`nv`.

        On exit: the computed solution U_i(x_j,t), for j =
        1,2,...,`npts`, for i = 1,2,...,`npde`, and V_k(t), for k =
        1,2,...,`nv`, evaluated at t = `ts`, as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t), for i = 1,2,...,`nv`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`)
        The mesh points in the space direction. `x`[0] must specify the
        left-hand boundary, a, and `x`[`npts`-1] must specify the
        right-hand boundary, b.

    nv : int
        The number of coupled ODE components.

    odedef : None or callable ires = odedef(npde, t, nv, v, vdot, nxi,
        xi, ucp, ucpx, rcp, ucpt, ucptx, f, ires, data=None)
        `odedef` must evaluate the functions F, which define the system
        of ODEs, as given in (3).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        nxi : int
            The number of ODE/PDE coupling points.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling
            points, xi_i, for i = 1,2,...,`nxi`.

        ucp : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucp`[i-1,j-1] contains the value of U_i(x,t)
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucpx : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpx`[i-1,j-1] contains the value of
            dU_i(x,t)/dx at the coupling point x = xi_j, for j =
            1,2,...,`nxi`, for i = 1,2,...,`npde`.

        rcp : float, ndarray, shape (`npde`, `nxi`)
            `rcp`[i-1,j-1] contains the value of the flux R_i at the
            coupling point x = xi_j, for j = 1,2,...,`nxi`, for i =
            1,2,...,`npde`.

        ucpt : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpt`[i-1,j-1] contains the value of dU_i/dt
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucptx : float, ndarray, shape (`npde`, `nxi`)
            `ucptx`[i-1,j-1] contains the value of d^2U_i/dxdt at the
            coupling point x = xi_j, for j = 1,2,...,`nxi`, for i =
            1,2,...,`npde`.

        f : float, ndarray, shape (`nv`), to be modified in place
            On exit, `f`[i-1] must contain the ith component of F as
            specified by the input value of `ires`.

        ires : int
            The form of F that must be returned in the array `f`.

            `ires` = 1

              Equation (5) must be used.

            `ires` = -1

              Equation (6) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may reset
            `ires` to force the integration function to take certain
            actions as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_dae_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    nxi : int
        The number of ODE/PDE coupling points.

    xi : float, shape (`nxi`)
        If `nxi` > 0, `xi`[i-1], for i = 1,2,...,`nxi`, must be set to
        the ODE/PDE coupling points.

    neqn : int
        The number of ODEs in the time direction.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; if `itol` in (3, 4): `neqn`;
        otherwise: 0.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; if `itol` in (2, 4): `neqn`;
        otherwise: 0.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``dim1_parab_dae_fd`` whether to interpret either
        or both of `rtol` or `atol` as a vector or scalar. The error
        test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is defined
        as follows:

        [table omitted]

        In the above, e_i denotes the estimated local error for the ith
        component of the coupled PDE/ODE system in time, `u`[i-1], for i
        = 1,2,...,`neqn`.

        The choice of norm used is defined by the argument `norm`.

    norm : str, length 1
        The type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L_2 norm.

        If `u`_norm denotes the norm of the vector `u` of length `neqn`,
        then for the averaged L_2 norm

            `u`_norm = sqrt(1/`neqn`sum_i = 1^`neqn`(`u`[i-1]/w_i)^2),

        while for the maximum norm

            `u`_norm = max_i(|`u`[i-1] / w_i|).

        See the description of `itol` for the formulation of the weight
        vector w.

    laopt : str, length 1
        The type of matrix algebra required.

        `laopt` = 'F'

          Full matrix methods to be used.

        `laopt` = 'B'

          Banded matrix methods to be used.

        `laopt` = 'S'

          Sparse matrix methods to be used.

    algopt : float, shape (30)
        May be set to control various options available in the
        integrator. If you wish to employ all the default options,
        `algopt`[0] should be set to 0.0. Default values will also be
        used for any other elements of `algopt` set to zero. The
        permissible values, default values, and meanings are as follows:

        `algopt`[0]

          Selects the ODE integration method to be used. If `algopt`[0]
          = 1.0, a BDF method is used and if `algopt`[0] = 2.0, a Theta
          method is used. The default value is `algopt`[0] = 1.0.

        If `algopt`[0] = 2.0, `algopt`[i-1], for i = 2,3,...,4 are not
        used.

        `algopt`[1]

          Specifies the maximum order of the BDF integration formula to
          be used. `algopt`[1] may be 1.0, 2.0, 3.0, 4.0 or 5.0. The
          default value is `algopt`[1] = 5.0.

        `algopt`[2]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the BDF method. If
          `algopt`[2] = 1.0 a modified Newton iteration is used and if
          `algopt`[2] = 2.0 a functional iteration method is used. If
          functional iteration is selected and the integrator encounters
          difficulty, there is an automatic switch to the modified
          Newton iteration. The default value is `algopt`[2] = 1.0.

        `algopt`[3]

          Specifies whether or not the Petzold error test is to be
          employed. The Petzold error test results in extra overhead but
          is more suitable when algebraic equations are present, such as
          P_{i,j} = 0.0, for j = 1,2,...,`npde`, for some i or when
          there is no V^._i(t) dependence in the coupled ODE system. If
          `algopt`[3] = 1.0, the Petzold test is used. If `algopt`[3] =
          2.0, the Petzold test is not used. The default value is
          `algopt`[3] = 1.0.

        If `algopt`[0] = 1.0, `algopt`[i-1], for i = 5,6,...,7, are not
        used.

        `algopt`[4]

          Specifies the value of Theta to be used in the Theta
          integration method. 0.51 <= `algopt`[4] <= 0.99. The default
          value is `algopt`[4] = 0.55.

        `algopt`[5]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the Theta method.
          If `algopt`[5] = 1.0, a modified Newton iteration is used and
          if `algopt`[5] = 2.0, a functional iteration method is used.
          The default value is `algopt`[5] = 1.0.

        `algopt`[6]

          Specifies whether or not the integrator is allowed to switch
          automatically between modified Newton and functional iteration
          methods in order to be more efficient. If `algopt`[6] = 1.0,
          switching is allowed and if `algopt`[6] = 2.0, switching is
          not allowed. The default value is `algopt`[6] = 1.0.

        `algopt`[10]

          Specifies a point in the time direction, t_crit, beyond which
          integration must not be attempted. The use of t_crit is
          described under the argument `itask`. If `algopt`[0] != 0.0, a
          value of 0.0 for `algopt`[10], say, should be specified even
          if `itask` subsequently specifies that t_crit will not be
          used.

        `algopt`[11]

          Specifies the minimum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[11]
          should be set to 0.0.

        `algopt`[12]

          Specifies the maximum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[12]
          should be set to 0.0.

        `algopt`[13]

          Specifies the initial step size to be attempted by the
          integrator. If `algopt`[13] = 0.0, the initial step size is
          calculated internally.

        `algopt`[14]

          Specifies the maximum number of steps to be attempted by the
          integrator in any one call. If `algopt`[14] = 0.0, no limit is
          imposed.

        `algopt`[22]

          Specifies what method is to be used to solve the nonlinear
          equations at the initial point to initialize the values of U,
          U_t, V and V^.. If `algopt`[22] = 1.0, a modified Newton
          iteration is used and if `algopt`[22] = 2.0, functional
          iteration is used. The default value is `algopt`[22] = 1.0.

        `algopt`[28] and `algopt`[29] are used only for the sparse
        matrix algebra option, `laopt` = 'S'.

        `algopt`[28]

          Governs the choice of pivots during the decomposition of the
          first Jacobian matrix. It should lie in the range 0.0 <
          `algopt`[28] < 1.0, with smaller values biasing the algorithm
          towards maintaining sparsity at the expense of numerical
          stability. If `algopt`[28] lies outside this range then the
          default value is used. If the functions regard the Jacobian
          matrix as numerically singular then increasing `algopt`[28]
          towards 1.0 may help, but at the cost of increased fill-in.
          The default value is `algopt`[28] = 0.1.

        `algopt`[29]

          Is used as a relative pivot threshold during subsequent
          Jacobian decompositions (see `algopt`[28]) below which an
          internal error is invoked. If `algopt`[29] is greater than 1.0
          no check is made on the pivot size, and this may be a
          necessary option if the Jacobian is found to be numerically
          singular (see `algopt`[28]). The default value is `algopt`[29]
          = 0.0001.

    comm : dict, communication object, modified in place
        Note: this argument will be (re-)initialized when it is an empty
        ``dict`` or under the following condition: `ind` = 0.

        Communication structure.

        On initial entry: need not be set.

    lrsave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['rsave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lrsave` is computed by the
        function.

        If your supplied `lrsave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lrsave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lrsave` set to at least the lower-bound value returned in
        `lrsave_min`.

    lisave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['isave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lisave` is computed by the
        function.

        If your supplied `lisave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lisave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lisave` set to at least the lower-bound value returned in
        `lisave_min`.

    itask : int
        Specifies the task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout`.

        `itask` = 2

          One step and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

        `itask` = 4

          Normal computation of output values `u` at t = `tout` but
          without overshooting t = t_crit where t_crit is described
          under the argument `algopt`.

        `itask` = 5

          Take one step in the time direction and return, without
          passing t_crit, where t_crit is described under the argument
          `algopt`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_dae_fd`` and the underlying ODE solver. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` > 0

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases. You are advised to set `itrace` = 0, unless you are
        experienced with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_dae_fd``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    lrsave_min : int
        A lower bound on the sufficient size for `comm`['rsave'].

    lisave_min : int
        A lower bound on the sufficient size for `comm`['isave'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `lrsave` = *<value>*.

            Constraint: `lrsave` >= *<value>*.

        (`errno` 1)
            On entry, `lisave` = *<value>*.

            Constraint: `lisave` >= *<value>*.

        (`errno` 1)
            On entry, at least one point in `xi` lies outside
            [`x`[0],`x`[`npts`-1]]: `x`[0] = *<value>* and `x`[`npts`-1]
            = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xi`[i] = *<value>* and `xi`[i-1] =
            *<value>*.

            Constraint: `xi`[i] > `xi`[i-1].

        (`errno` 1)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: corresponding elements `atol`[i-1] and
            `rtol`[j-1] cannot both be 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `rtol`[i-1] = *<value>*.

            Constraint: `rtol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `atol`[i-1] = *<value>*.

            Constraint: `atol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `neqn` = *<value>*, `npde` = *<value>*, `npts` =
            *<value>* and `nv` = *<value>*.

            Constraint: `neqn` = `npde`*`npts`+`nv`.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` = 0 when `nv` = 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` >= 0 when `nv` > 0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `m` = *<value>* and `x`[0] = *<value>*.

            Constraint: `m` <= 0 or `x`[0] >= 0.0

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = 0, 1 or 2.

        (`errno` 1)
            On entry, `laopt` = *<value>*.

            Constraint: `laopt` = 'F', 'B' or 'S'.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = 'A' or 'M'.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2, 3, 4 or 5.

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 0.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `atol` and `rtol` were too small to start integration.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef`,
            `bndary`, or `odedef`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 12)
            In solving ODE system, the maximum number of steps
            `algopt`[14] has been exceeded. `algopt`[14] = *<value>*.

        (`errno` 14)
            Flux function appears to depend on time derivatives.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `atol` and `rtol`.
            `ts` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef`, `bndary`, or `odedef`. Integration is
            successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `atol` or `rtol`
            are unlikely to result in a changed solution.

        (`errno` 13)
            Zero error weights encountered during time integration.

        (`errno` 15)
            When using the sparse option `lisave` or `lrsave` is too
            small: `lisave` = *<value>*, `lrsave` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_dae_fd'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, r[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, q, r, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _nv_pdedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_pdedefh,),
            )
            _vdot_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_pdedefh,),
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _q_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                q,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _r_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ux_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_bndaryh,),
            )
            _nv_bndaryh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_bndaryh,),
            )
            _vdot_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_bndaryh,),
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _beta_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                beta,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _gamma_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                gamma,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_odedef_name = fun_name + ':odedef'

    odedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, rcp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpt[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucptx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, f[nv].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_odedefh(_odedef, en_data, npde, t, nv, v, vdot, nxi, xi, ucp, ucpx, rcp, ucpt, ucptx, f, ires, _iuser, _ruser):
        """Helper for odedef."""
        if odedef is None:
            return
        try:
            _npde_odedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_odedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nv_odedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_odedefh,),
            )
            _vdot_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_odedefh,),
            )
            _nxi_odedefh = utils._EngineIntScalarType.to_py(nxi.contents)
            _xi_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_odedefh,),
            )
            _ucp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _rcp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                rcp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpt_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpt,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucptx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucptx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _f_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_nv_odedefh,),
                can_reference_contents=False,
            )
            _ires_odedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _rcp_odedefh,
                    _ucpt_odedefh,
                    _ucptx_odedefh,
                    _f_odedefh,
                    _ires_odedefh,
                    data,
                )
            else:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _rcp_odedefh,
                    _ucpt_odedefh,
                    _ucptx_odedefh,
                    _f_odedefh,
                    _ires_odedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._unlink_pointer(xi)
            utils._unlink_pointer(ucp)
            utils._unlink_pointer(ucpx)
            utils._unlink_pointer(rcp)
            utils._unlink_pointer(ucpt)
            utils._unlink_pointer(ucptx)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_odedefh, {'fun_name': cb_odedef_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._EngineIntScalarType(
                _ires_odedefh, {'fun_name': cb_odedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03phft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[neqn].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._BYREF_INT, # Input, nv.
        utils._UNUSED_FUNCTYPE,
        odedefh,
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._BYREF_INT, # Input, neqn.
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, laopt(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, algopt[30].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Output, lrsave_min.
        utils._BYREF_INT, # Output, lisave_min.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of laopt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _neqn = utils._EngineIntScalarType(
        neqn, {'fun_name': fun_name, 'entity': 'neqn'},
    ).to_ctypes()
    _neqn_py = utils._EngineIntScalarType.to_py(_neqn)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_neqn_py,),
    )
    _u = _u_cls.to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    utils._check_callable(odedef, {'fun_name': fun_name, 'entity': 'odedef'}, allow_none=True)
    _odedef = utils._UNUSED_CALLBACK
    _odedefh = utils._cast_callback(
        odedefh, cb_odedefh,
        {'fun_name': fun_name, 'entity': 'odedef'},
    )
    _nxi = utils._EngineIntScalarType(
        nxi, {'fun_name': fun_name, 'entity': 'nxi'},
    ).to_ctypes()
    _nxi_py = utils._EngineIntScalarType.to_py(_nxi)
    _xi = utils._EngineFloat64ArrayType(
        xi, {'fun_name': fun_name, 'entity': 'xi'},
        exp_shape=(_nxi_py,),
    ).to_ctypes()
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    elif _itol_py in (3, 4):
        _rtol_dim1 = _neqn_py
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    elif _itol_py in (2, 4):
        _atol_dim1 = _neqn_py
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _laopt = utils._EngineCharScalarType(
        laopt, {'fun_name': fun_name, 'entity': 'laopt'}, exp_chrlen=1,
    ).to_ctypes()
    _algopt = utils._EngineFloat64ArrayType(
        algopt, {'fun_name': fun_name, 'entity': 'algopt'},
        exp_shape=(30,),
    ).to_ctypes()
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _comm_needs_init = (
        utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
        ) or
        _ind_py == 0
    )
    if _comm_needs_init:
        if lrsave is None:
            _lrsave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lrsave = utils._EngineIntScalarType(
                lrsave, {'fun_name': fun_name, 'entity': 'lrsave'},
            ).to_ctypes()
        _lrsave_py = utils._EngineIntScalarType.to_py(_lrsave)
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
    else:
        _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = comm['rsave']
    if _comm_needs_init:
        if lisave is None:
            _lisave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lisave = utils._EngineIntScalarType(
                lisave, {'fun_name': fun_name, 'entity': 'lisave'},
            ).to_ctypes()
        _lisave_py = utils._EngineIntScalarType.to_py(_lisave)
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
    else:
        _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = comm['isave']
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iemode_py = 1 if _comm_needs_init else 0
    _iemode = utils._EngineIntScalarType(
        _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
    ).to_ctypes()
    _lrsave_min = utils._EngineIntScalarType.empty_ctypes()
    _lisave_min = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _m,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _u,
        _npts,
        _x,
        _nv,
        _odedef,
        _odedefh,
        _nxi,
        _xi,
        _neqn,
        _rtol,
        _atol,
        _itol,
        _norm,
        _laopt,
        _algopt,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iemode,
        _lrsave_min,
        _lisave_min,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _norm._length_,
        _laopt._length_,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS and _iemode_py == 1:
        _iemode_py = 0
        _iemode = utils._EngineIntScalarType(
            _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
        ).to_ctypes()
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lrsave_py = max(_lrsave_min_py, _lrsave_py)
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        _lisave_py = max(_lisave_min_py, _lisave_py)
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
        engine_ctypes(
            _en_data,
            utils._UNUSED_PRINT_REC,
            _print_rech,
            _npde,
            _m,
            _ts,
            _tout,
            _pdedef,
            _pdedefh,
            _bndary,
            _bndaryh,
            _u,
            _npts,
            _x,
            _nv,
            _odedef,
            _odedefh,
            _nxi,
            _xi,
            _neqn,
            _rtol,
            _atol,
            _itol,
            _norm,
            _laopt,
            _algopt,
            _rsave,
            _lrsave,
            _isave,
            _lisave,
            _itask,
            _itrace,
            _nadv,
            _ind,
            _iemode,
            _lrsave_min,
            _lisave_min,
            _iuser,
            _ruser,
            _cwsav,
            _lwsav,
            _iwsav,
            _rwsav,
            _errbuf,
            _ierr,
            _norm._length_,
            _laopt._length_,
            80,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        if _ierr_py == 1 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrsave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lisave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lisave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one point in xi lies outside [x[0],x[npts-1]]:',
                    'x[0] = {:s} and x[npts-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xi[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xi[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xi[i] > xi[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: corresponding elements atol[i-1] and rtol[j-1]',
                    'cannot both be 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and rtol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and atol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqn = {:s}, npde = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npts = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: neqn = npde*npts+nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi = 0 when nv = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi >= 0 when nv > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and x[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= 0 or x[0] >= 0.0',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laopt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: laopt = \'F\', \'B\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'A\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99976:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of atol and rtol.',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99975:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99972:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef, bndary, or odedef. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'atol and rtol were too small to start integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef, bndary, or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99968:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in atol or rtol are unlikely',
                    'to result in a changed solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99967:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In solving ODE system, the maximum number of steps',
                    'algopt[14] has been exceeded.',
                    'algopt[14] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99965:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Zero error weights encountered during time integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99964:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Flux function appears to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99963:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'When using the sparse option lisave or lrsave is too small:',
                    'lisave = {:s}, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind', 'lrsave_min', 'lisave_min'],
        )(
            _ts_py,
            _ind_py,
            _lrsave_min_py,
            _lisave_min_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_dae_coll(npde, m, ts, tout, pdedef, bndary, u, nbkpts, xbkpts, npoly, npts, x, nv, odedef, nxi, xi, neqn, uvinit, rtol, atol, itol, norm, laopt, algopt, comm, lrsave, lisave, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of parabolic PDEs, coupled DAEs, method of lines,
    Chebyshev C^0 collocation, one space variable.

    For full information please refer to the NAG Library document for
    d03pj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pjf.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    m : int
        The coordinate system used:

        `m` = 0

          Indicates Cartesian coordinates.

        `m` = 1

          Indicates cylindrical polar coordinates.

        `m` = 2

          Indicates spherical polar coordinates.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, nptl, u, ux, nv, v,
        vdot, p, q, r, ires, data=None)
        `pdedef` must compute the functions P_{i,j}, Q_i and R_i which
        define the system of PDEs.

        The functions may depend on x, t, U, U_x and V; Q_i may depend
        linearly on V^..

        The functions must be evaluated at a set of points.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`nptl`)
            Contains a set of mesh points at which P_{i,j}, Q_i and R_i
            are to be evaluated. `x`[0] and `x`[`nptl`-1] contain
            successive user-supplied break-points and the elements of
            the array will satisfy `x`[0] < `x`[1] < ... <
            `x`[`nptl`-1].

        nptl : int
            The number of points at which evaluations are required (the
            value of `npoly`+1).

        u : float, ndarray, shape (`npde`, `nptl`)
            `u`[i-1,j-1] contains the value of the component U_i(x,t)
            where x = `x`[j-1], for j = 1,2,...,`nptl`, for i =
            1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`, `nptl`)
            `ux`[i-1,j-1] contains the value of the component
            dU_i(x,t)/dx where x = `x`[j-1], for j = 1,2,...,`nptl`, for
            i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in Q_j, for j = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`, `nptl`), to be
        modified in place
            On exit: `p`[ `npde`*`npde`*(k-1)+ `npde`*(j-1)+ (i-1)] must
            be set to the value of P_{i,j}(x,t,U,U_x,V) where x =
            `x`[k-1], for k = 1,2,...,`nptl`, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npde`.

        q : float, ndarray, shape (`npde`, `nptl`), to be modified in
        place
            On exit: `q`[i-1,j-1] must be set to the value of
            Q_i(x,t,U,U_x,V,V^.) where x = `x`[j-1], for j =
            1,2,...,`nptl`, for i = 1,2,...,`npde`.

        r : float, ndarray, shape (`npde`, `nptl`), to be modified in
        place
            On exit: `r`[i-1,j-1] must be set to the value of
            R_i(x,t,U,U_x,V) where x = `x`[i-1], for j = 1,2,...,`nptl`,
            for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_dae_coll``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    bndary : callable ires = bndary(npde, t, u, ux, nv, v, vdot, ibnd,
        beta, gamma, ires, data=None)
        `bndary` must compute the functions beta_i and gamma_i which
        define the boundary conditions as in equation (4).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in Q_j, for j = 1,2,...,`npde`.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must set up the coefficients of the left-hand
              boundary, x = a.

            `ibnd` != 0

              `bndary` must set up the coefficients of the right-hand
              boundary, x = b.

        beta : float, ndarray, shape (`npde`), to be modified in place
            On exit: `beta`[i-1] must be set to the value of beta_i(x,t)
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        gamma : float, ndarray, shape (`npde`), to be modified in place
            On exit: `gamma`[i-1] must be set to the value of
            gamma_i(x,t,U,U_x,V,V^.) at the boundary specified by
            `ibnd`, for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_dae_coll``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    u : float, shape (`neqn`), modified in place
        On entry: if `ind` = 1 the value of `u` must be unchanged from
        the previous call.

        On exit: the computed solution U_i(x_j,t), for j =
        1,2,...,`npts`, for i = 1,2,...,`npde`, and V_k(t), for k =
        1,2,...,`nv`, evaluated at t = `ts`, as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t), for i = 1,2,...,`nv`.

    nbkpts : int
        The number of break-points in the interval [a,b].

    xbkpts : float, shape (`nbkpts`)
        The values of the break-points in the space direction.
        `xbkpts`[0] must specify the left-hand boundary, a, and
        `xbkpts`[`nbkpts`-1] must specify the right-hand boundary, b.

    npoly : int
        The degree of the Chebyshev polynomial to be used in
        approximating the PDE solution between each pair of
        break-points.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`), modified in place
        On exit: the mesh points chosen by ``dim1_parab_dae_coll`` in
        the spatial direction. The values of `x` will satisfy `x`[0] <
        `x`[1] < ... < `x`[`npts`-1].

    nv : int
        The number of coupled ODE components.

    odedef : None or callable ires = odedef(npde, t, nv, v, vdot, nxi,
        xi, ucp, ucpx, rcp, ucpt, ucptx, f, ires, data=None)
        `odedef` must evaluate the functions F, which define the system
        of ODEs, as given in (3).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        nxi : int
            The number of ODE/PDE coupling points.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling
            points, xi_i, for i = 1,2,...,`nxi`.

        ucp : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucp`[i-1,j-1] contains the value of U_i(x,t)
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucpx : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpx`[i-1,j-1] contains the value of
            dU_i(x,t)/dx at the coupling point x = xi_j, for j =
            1,2,...,`nxi`, for i = 1,2,...,`npde`.

        rcp : float, ndarray, shape (`npde`, `nxi`)
            `rcp`[i-1,j-1] contains the value of the flux R_i at the
            coupling point x = xi_j, for j = 1,2,...,`nxi`, for i =
            1,2,...,`npde`.

        ucpt : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpt`[i-1,j-1] contains the value of dU_i/dt
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucptx : float, ndarray, shape (`npde`, `nxi`)
            `ucptx`[i-1,j-1] contains the value of d^2U_i/dxdt at the
            coupling point x = xi_j, for j = 1,2,...,`nxi`, for i =
            1,2,...,`npde`.

        f : float, ndarray, shape (`nv`), to be modified in place
            On exit, `f`[i-1] must contain the ith component of F as
            specified by the input value of `ires`.

        ires : int
            The form of F that must be returned in the array `f`.

            `ires` = 1

              Equation (5) must be used.

            `ires` = -1

              Equation (6) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may reset
            `ires` to force the integration function to take certain
            actions as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_dae_coll``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    nxi : int
        The number of ODE/PDE coupling points.

    xi : float, shape (`nxi`)
        `xi`[i-1], for i = 1,2,...,`nxi`, must be set to the ODE/PDE
        coupling points.

    neqn : int
        The number of ODEs in the time direction.

    uvinit : callable uvinit(npde, npts, x, u, nv, v, data=None)
        `uvinit` must compute the initial values of the PDE and the ODE
        components U_i(x_j,t_0), for j = 1,2,...,`npts`, for i =
        1,2,...,`npde`, and V_k(t_0), for k = 1,2,...,`nv`.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        x : float, ndarray, shape (`npts`)
            `x`[i-1], for i = 1,2,...,`npts`, contains the current
            values of the space variable x_i.

        u : float, ndarray, shape (`npde`, `npts`), to be modified in
        place
            On exit: `u`[i-1,j-1] contains the value of the component
            U_i(x_j,t_0), for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`), to be modified in place
            On exit: `v`[i-1] contains the value of component V_i(t_0),
            for i = 1,2,...,`nv`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; if `itol` in (3, 4): `neqn`;
        otherwise: 0.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; if `itol` in (2, 4): `neqn`;
        otherwise: 0.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``dim1_parab_dae_coll`` whether to interpret either
        or both of `rtol` or `atol` as a vector or scalar. The error
        test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is defined
        as follows:

        [table omitted]

        In the above, e_i denotes the estimated local error for the ith
        component of the coupled PDE/ODE system in time, `u`[i-1], for i
        = 1,2,...,`neqn`.

        The choice of norm used is defined by the argument `norm`.

    norm : str, length 1
        The type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L_2 norm.

        If `u`_norm denotes the norm of the vector `u` of length `neqn`,
        then for the averaged L_2 norm

            `u`_norm = sqrt(1/`neqn`sum_i = 1^`neqn`(`u`[i-1]/w_i)^2),

        while for the maximum norm

            `u`_norm = max_i(|`u`[i-1] / w_i|).

        See the description of `itol` for the formulation of the weight
        vector w.

    laopt : str, length 1
        The type of matrix algebra required.

        `laopt` = 'F'

          Full matrix methods to be used.

        `laopt` = 'B'

          Banded matrix methods to be used.

        `laopt` = 'S'

          Sparse matrix methods to be used.

    algopt : float, shape (30)
        May be set to control various options available in the
        integrator. If you wish to employ all the default options,
        `algopt`[0] should be set to 0.0. Default values will also be
        used for any other elements of `algopt` set to zero. The
        permissible values, default values, and meanings are as follows:

        `algopt`[0]

          Selects the ODE integration method to be used. If `algopt`[0]
          = 1.0, a BDF method is used and if `algopt`[0] = 2.0, a Theta
          method is used. The default value is `algopt`[0] = 1.0.

        If `algopt`[0] = 2.0, `algopt`[i-1], for i = 2,3,...,4 are not
        used.

        `algopt`[1]

          Specifies the maximum order of the BDF integration formula to
          be used. `algopt`[1] may be 1.0, 2.0, 3.0, 4.0 or 5.0. The
          default value is `algopt`[1] = 5.0.

        `algopt`[2]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the BDF method. If
          `algopt`[2] = 1.0 a modified Newton iteration is used and if
          `algopt`[2] = 2.0 a functional iteration method is used. If
          functional iteration is selected and the integrator encounters
          difficulty, there is an automatic switch to the modified
          Newton iteration. The default value is `algopt`[2] = 1.0.

        `algopt`[3]

          Specifies whether or not the Petzold error test is to be
          employed. The Petzold error test results in extra overhead but
          is more suitable when algebraic equations are present, such as
          P_{i,j} = 0.0, for j = 1,2,...,`npde`, for some i or when
          there is no V^._i(t) dependence in the coupled ODE system. If
          `algopt`[3] = 1.0, the Petzold test is used. If `algopt`[3] =
          2.0, the Petzold test is not used. The default value is
          `algopt`[3] = 1.0.

        If `algopt`[0] = 1.0, `algopt`[i-1], for i = 5,6,...,7, are not
        used.

        `algopt`[4]

          Specifies the value of Theta to be used in the Theta
          integration method. 0.51 <= `algopt`[4] <= 0.99. The default
          value is `algopt`[4] = 0.55.

        `algopt`[5]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the Theta method.
          If `algopt`[5] = 1.0, a modified Newton iteration is used and
          if `algopt`[5] = 2.0, a functional iteration method is used.
          The default value is `algopt`[5] = 1.0.

        `algopt`[6]

          Specifies whether or not the integrator is allowed to switch
          automatically between modified Newton and functional iteration
          methods in order to be more efficient. If `algopt`[6] = 1.0,
          switching is allowed and if `algopt`[6] = 2.0, switching is
          not allowed. The default value is `algopt`[6] = 1.0.

        `algopt`[10]

          Specifies a point in the time direction, t_crit, beyond which
          integration must not be attempted. The use of t_crit is
          described under the argument `itask`. If `algopt`[0] != 0.0, a
          value of 0.0 for `algopt`[10], say, should be specified even
          if `itask` subsequently specifies that t_crit will not be
          used.

        `algopt`[11]

          Specifies the minimum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[11]
          should be set to 0.0.

        `algopt`[12]

          Specifies the maximum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[12]
          should be set to 0.0.

        `algopt`[13]

          Specifies the initial step size to be attempted by the
          integrator. If `algopt`[13] = 0.0, the initial step size is
          calculated internally.

        `algopt`[14]

          Specifies the maximum number of steps to be attempted by the
          integrator in any one call. If `algopt`[14] = 0.0, no limit is
          imposed.

        `algopt`[22]

          Specifies what method is to be used to solve the nonlinear
          equations at the initial point to initialize the values of U,
          U_t, V and V^.. If `algopt`[22] = 1.0, a modified Newton
          iteration is used and if `algopt`[22] = 2.0, functional
          iteration is used. The default value is `algopt`[22] = 1.0.

        `algopt`[28] and `algopt`[29] are used only for the sparse
        matrix algebra option, `laopt` = 'S'.

        `algopt`[28]

          Governs the choice of pivots during the decomposition of the
          first Jacobian matrix. It should lie in the range 0.0 <
          `algopt`[28] < 1.0, with smaller values biasing the algorithm
          towards maintaining sparsity at the expense of numerical
          stability. If `algopt`[28] lies outside this range then the
          default value is used. If the functions regard the Jacobian
          matrix as numerically singular then increasing `algopt`[28]
          towards 1.0 may help, but at the cost of increased fill-in.
          The default value is `algopt`[28] = 0.1.

        `algopt`[29]

          Is used as a relative pivot threshold during subsequent
          Jacobian decompositions (see `algopt`[28]) below which an
          internal error is invoked. If `algopt`[29] is greater than 1.0
          no check is made on the pivot size, and this may be a
          necessary option if the Jacobian is found to be numerically
          singular (see `algopt`[28]). The default value is `algopt`[29]
          = 0.0001.

    comm : dict, communication object, modified in place
        Note: this argument will be (re-)initialized when it is an empty
        ``dict`` or under the following condition: `ind` = 0.

        Communication structure.

        On initial entry: need not be set.

    lrsave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['rsave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lrsave` is computed by the
        function.

        If your supplied `lrsave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lrsave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lrsave` set to at least the lower-bound value returned in
        `lrsave_min`.

    lisave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['isave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lisave` is computed by the
        function.

        If your supplied `lisave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lisave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lisave` set to at least the lower-bound value returned in
        `lisave_min`.

    itask : int
        Specifies the task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout`.

        `itask` = 2

          One step and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

        `itask` = 4

          Normal computation of output values `u` at t = `tout` but
          without overshooting t = t_crit where t_crit is described
          under the argument `algopt`.

        `itask` = 5

          Take one step in the time direction and return, without
          passing t_crit, where t_crit is described under the argument
          `algopt`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_dae_coll`` and the underlying ODE solver. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` > 0

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases. You are advised to set `itrace` = 0, unless you are
        experienced with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_dae_coll``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    lrsave_min : int
        A lower bound on the sufficient size for `comm`['rsave'].

    lisave_min : int
        A lower bound on the sufficient size for `comm`['isave'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `lrsave` = *<value>*.

            Constraint: `lrsave` >= *<value>*.

        (`errno` 1)
            On entry, `lisave` = *<value>*.

            Constraint: `lisave` >= *<value>*.

        (`errno` 1)
            On entry, at least one point in `xi` lies outside
            [`xbkpts`[0],`xbkpts`[`nbkpts`-1]]: `xbkpts`[0] = *<value>*
            and `xbkpts`[`nbkpts`-1] = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xi`[i] = *<value>* and `xi`[i-1] =
            *<value>*.

            Constraint: `xi`[i] > `xi`[i-1].

        (`errno` 1)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: corresponding elements `atol`[i-1] and
            `rtol`[j-1] cannot both be 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `rtol`[i-1] = *<value>*.

            Constraint: `rtol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `atol`[i-1] = *<value>*.

            Constraint: `atol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, i = *<value>*, `xbkpts`[i-1] = *<value>*, j =
            *<value>* and `xbkpts`[j-1] = *<value>*.

            Constraint: `xbkpts`[0] < `xbkpts`[1] < ... <
            `xbkpts`[`nbkpts`-1].

        (`errno` 1)
            On entry, `neqn` = *<value>*, `npde` = *<value>*, `npts` =
            *<value>* and `nv` = *<value>*.

            Constraint: `neqn` = `npde`*`npts`+`nv`.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` = 0 when `nv` = 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` >= 0 when `nv` > 0.

        (`errno` 1)
            On entry, `npts` = *<value>*, `nbkpts` = *<value>* and
            `npoly` = *<value>*.

            Constraint: `npts` = (`nbkpts`-1)*`npoly`+1.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `nbkpts` = *<value>*.

            Constraint: `nbkpts` >= 2.

        (`errno` 1)
            On entry, `m` = *<value>* and `xbkpts`[0] = *<value>*.

            Constraint: `m` <= 0 or `xbkpts`[0] >= 0.0

        (`errno` 1)
            On entry, `npoly` = *<value>*.

            Constraint: `npoly` <= 49.

        (`errno` 1)
            On entry, `npoly` = *<value>*.

            Constraint: `npoly` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = 0, 1 or 2.

        (`errno` 1)
            On entry, `laopt` = *<value>*.

            Constraint: `laopt` = 'F', 'B' or 'S'.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = 'A' or 'M'.

        (`errno` 1)
            On entry, `algopt`[0] = *<value>*.

            Constraint: `algopt`[0] = 0.0, 1.0 or 2.0.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2, 3, 4 or 5.

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 0.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `atol` and `rtol` were too small to start integration.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef`,
            `bndary`, or `odedef`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 12)
            In solving ODE system, the maximum number of steps
            `algopt`[14] has been exceeded. `algopt`[14] = *<value>*.

        (`errno` 14)
            Flux function appears to depend on time derivatives.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `atol` and `rtol`.
            `ts` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef`, `bndary`, or `odedef`. Integration is
            successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `atol` or `rtol`
            are unlikely to result in a changed solution.

        (`errno` 13)
            Zero error weights encountered during time integration.

        (`errno` 15)
            When using the sparse option `lisave` or `lrsave` is too
            small: `lisave` = *<value>*, `lrsave` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_dae_coll'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Output, r[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, x, np, nptl, u, ut, ux, nv, v, vdot, p, q, r, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nptl_pdedefh = utils._EngineIntScalarType.to_py(nptl.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_nptl_pdedefh,),
            )
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh, _nptl_pdedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh, _nptl_pdedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _nv_pdedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_pdedefh,),
            )
            _vdot_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_pdedefh,),
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh, _nptl_pdedefh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _q_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                q,
                (_npde_pdedefh, _nptl_pdedefh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _r_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_npde_pdedefh, _nptl_pdedefh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _nptl_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _nptl_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(u)
            utils._unlink_pointer(ux)
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _p_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'p'}, p,
            )
            utils._unlink_pointer(p)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _q_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'q'}, q,
            )
            utils._unlink_pointer(q)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ux_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_bndaryh,),
            )
            _nv_bndaryh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_bndaryh,),
            )
            _vdot_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_bndaryh,),
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _beta_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                beta,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _gamma_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                gamma,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_odedef_name = fun_name + ':odedef'

    odedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, rcp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpt[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucptx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, f[nv].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_odedefh(_odedef, en_data, npde, t, nv, v, vdot, nxi, xi, ucp, ucpx, rcp, ucpt, ucptx, f, ires, _iuser, _ruser):
        """Helper for odedef."""
        if odedef is None:
            return
        try:
            _npde_odedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_odedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nv_odedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_odedefh,),
            )
            _vdot_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_odedefh,),
            )
            _nxi_odedefh = utils._EngineIntScalarType.to_py(nxi.contents)
            _xi_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_odedefh,),
            )
            _ucp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _rcp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                rcp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpt_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpt,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucptx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucptx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _f_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_nv_odedefh,),
                can_reference_contents=False,
            )
            _ires_odedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _rcp_odedefh,
                    _ucpt_odedefh,
                    _ucptx_odedefh,
                    _f_odedefh,
                    _ires_odedefh,
                    data,
                )
            else:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _rcp_odedefh,
                    _ucpt_odedefh,
                    _ucptx_odedefh,
                    _f_odedefh,
                    _ires_odedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._unlink_pointer(xi)
            utils._unlink_pointer(ucp)
            utils._unlink_pointer(ucpx)
            utils._unlink_pointer(rcp)
            utils._unlink_pointer(ucpt)
            utils._unlink_pointer(ucptx)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_odedefh, {'fun_name': cb_odedef_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._EngineIntScalarType(
                _ires_odedefh, {'fun_name': cb_odedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_uvinit_name = fun_name + ':uvinit'

    uvinith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, u[npde, npts].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Output, v[nv].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_uvinith(_uvinit, en_data, npde, npts, nxi, x, xi, u, nv, v, _iuser, _ruser):
        """Helper for uvinit."""
        try:
            _npde_uvinith = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_uvinith = utils._EngineIntScalarType.to_py(npts.contents)
            _x_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_uvinith,),
            )
            _u_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_uvinith, _npts_uvinith),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _nv_uvinith = utils._EngineIntScalarType.to_py(nv.contents)
            _v_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_uvinith,),
                can_reference_contents=False,
            )
            if data is not None:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _x_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                    data,
                )
            else:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _x_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _u_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'u'}, u,
            )
            utils._unlink_pointer(u)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _v_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'v'}, v,
            )
            utils._unlink_pointer(v)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03pjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[neqn].
        utils._BYREF_INT, # Input, nbkpts.
        utils._EngineFloat64ArrayType.c_type, # Input, xbkpts[nbkpts].
        utils._BYREF_INT, # Input, npoly.
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Output, x[npts].
        utils._BYREF_INT, # Input, nv.
        utils._UNUSED_FUNCTYPE,
        odedefh,
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._BYREF_INT, # Input, neqn.
        utils._UNUSED_FUNCTYPE,
        uvinith,
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, laopt(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, algopt[30].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Output, lrsave_min.
        utils._BYREF_INT, # Output, lisave_min.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of laopt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _neqn = utils._EngineIntScalarType(
        neqn, {'fun_name': fun_name, 'entity': 'neqn'},
    ).to_ctypes()
    _neqn_py = utils._EngineIntScalarType.to_py(_neqn)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_neqn_py,),
    )
    _u = _u_cls.to_ctypes()
    _nbkpts = utils._EngineIntScalarType(
        nbkpts, {'fun_name': fun_name, 'entity': 'nbkpts'},
    ).to_ctypes()
    _nbkpts_py = utils._EngineIntScalarType.to_py(_nbkpts)
    _xbkpts = utils._EngineFloat64ArrayType(
        xbkpts, {'fun_name': fun_name, 'entity': 'xbkpts'},
        exp_shape=(_nbkpts_py,),
    ).to_ctypes()
    _npoly = utils._EngineIntScalarType(
        npoly, {'fun_name': fun_name, 'entity': 'npoly'},
    ).to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    utils._check_callable(odedef, {'fun_name': fun_name, 'entity': 'odedef'}, allow_none=True)
    _odedef = utils._UNUSED_CALLBACK
    _odedefh = utils._cast_callback(
        odedefh, cb_odedefh,
        {'fun_name': fun_name, 'entity': 'odedef'},
    )
    _nxi = utils._EngineIntScalarType(
        nxi, {'fun_name': fun_name, 'entity': 'nxi'},
    ).to_ctypes()
    _nxi_py = utils._EngineIntScalarType.to_py(_nxi)
    _xi = utils._EngineFloat64ArrayType(
        xi, {'fun_name': fun_name, 'entity': 'xi'},
        exp_shape=(_nxi_py,),
    ).to_ctypes()
    utils._check_callable(uvinit, {'fun_name': fun_name, 'entity': 'uvinit'})
    _uvinit = utils._UNUSED_CALLBACK
    _uvinith = utils._cast_callback(
        uvinith, cb_uvinith,
        {'fun_name': fun_name, 'entity': 'uvinit'},
    )
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    elif _itol_py in (3, 4):
        _rtol_dim1 = _neqn_py
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    elif _itol_py in (2, 4):
        _atol_dim1 = _neqn_py
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _laopt = utils._EngineCharScalarType(
        laopt, {'fun_name': fun_name, 'entity': 'laopt'}, exp_chrlen=1,
    ).to_ctypes()
    _algopt = utils._EngineFloat64ArrayType(
        algopt, {'fun_name': fun_name, 'entity': 'algopt'},
        exp_shape=(30,),
    ).to_ctypes()
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _comm_needs_init = (
        utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
        ) or
        _ind_py == 0
    )
    if _comm_needs_init:
        if lrsave is None:
            _lrsave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lrsave = utils._EngineIntScalarType(
                lrsave, {'fun_name': fun_name, 'entity': 'lrsave'},
            ).to_ctypes()
        _lrsave_py = utils._EngineIntScalarType.to_py(_lrsave)
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
    else:
        _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = comm['rsave']
    if _comm_needs_init:
        if lisave is None:
            _lisave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lisave = utils._EngineIntScalarType(
                lisave, {'fun_name': fun_name, 'entity': 'lisave'},
            ).to_ctypes()
        _lisave_py = utils._EngineIntScalarType.to_py(_lisave)
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
    else:
        _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = comm['isave']
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iemode_py = 1 if _comm_needs_init else 0
    _iemode = utils._EngineIntScalarType(
        _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
    ).to_ctypes()
    _lrsave_min = utils._EngineIntScalarType.empty_ctypes()
    _lisave_min = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _m,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _u,
        _nbkpts,
        _xbkpts,
        _npoly,
        _npts,
        _x,
        _nv,
        _odedef,
        _odedefh,
        _nxi,
        _xi,
        _neqn,
        _uvinit,
        _uvinith,
        _rtol,
        _atol,
        _itol,
        _norm,
        _laopt,
        _algopt,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iemode,
        _lrsave_min,
        _lisave_min,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _norm._length_,
        _laopt._length_,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS and _iemode_py == 1:
        _iemode_py = 0
        _iemode = utils._EngineIntScalarType(
            _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
        ).to_ctypes()
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lrsave_py = max(_lrsave_min_py, _lrsave_py)
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        _lisave_py = max(_lisave_min_py, _lisave_py)
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
        engine_ctypes(
            _en_data,
            utils._UNUSED_PRINT_REC,
            _print_rech,
            _npde,
            _m,
            _ts,
            _tout,
            _pdedef,
            _pdedefh,
            _bndary,
            _bndaryh,
            _u,
            _nbkpts,
            _xbkpts,
            _npoly,
            _npts,
            _x,
            _nv,
            _odedef,
            _odedefh,
            _nxi,
            _xi,
            _neqn,
            _uvinit,
            _uvinith,
            _rtol,
            _atol,
            _itol,
            _norm,
            _laopt,
            _algopt,
            _rsave,
            _lrsave,
            _isave,
            _lisave,
            _itask,
            _itrace,
            _nadv,
            _ind,
            _iemode,
            _lrsave_min,
            _lisave_min,
            _iuser,
            _ruser,
            _cwsav,
            _lwsav,
            _iwsav,
            _rwsav,
            _errbuf,
            _ierr,
            _norm._length_,
            _laopt._length_,
            80,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        if _ierr_py == 1 and _ifmt == 99958:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrsave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lisave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lisave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one point in xi lies outside',
                    '[xbkpts[0],xbkpts[nbkpts-1]]:',
                    'xbkpts[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xbkpts[nbkpts-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xi[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xi[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xi[i] > xi[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: corresponding elements atol[i-1] and rtol[j-1]',
                    'cannot both be 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and rtol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and atol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xbkpts[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and xbkpts[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xbkpts[0] < xbkpts[1] < ... < xbkpts[nbkpts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqn = {:s}, npde = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npts = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: neqn = npde*npts+nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi = 0 when nv = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi >= 0 when nv > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}, nbkpts = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: npts = (nbkpts-1)*npoly+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nbkpts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nbkpts >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'xbkpts[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= 0 or xbkpts[0] >= 0.0',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npoly <= 49.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npoly >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laopt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: laopt = \'F\', \'B\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'A\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, algopt[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: algopt[0] = 0.0, 1.0 or 2.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99972:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of atol and rtol.',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99968:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef, bndary, or odedef. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99967:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'atol and rtol were too small to start integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef, bndary, or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99965:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99964:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in atol or rtol are unlikely',
                    'to result in a changed solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In solving ODE system, the maximum number of steps',
                    'algopt[14] has been exceeded.',
                    'algopt[14] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99961:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Zero error weights encountered during time integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Flux function appears to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99959:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'When using the sparse option lisave or lrsave is too small:',
                    'lisave = {:s}, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _x_cls.refresh_py(_x)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind', 'lrsave_min', 'lisave_min'],
        )(
            _ts_py,
            _ind_py,
            _lrsave_min_py,
            _lisave_min_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_dae_keller(npde, ts, tout, pdedef, bndary, u, npts, x, nleft, nv, odedef, nxi, xi, neqn, rtol, atol, itol, norm, laopt, algopt, comm, lrsave, lisave, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of first-order PDEs, coupled DAEs, method of lines,
    Keller box discretization, one space variable.

    For full information please refer to the NAG Library document for
    d03pk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pkf.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, u, ut, ux, nv, v, vdot,
        res, ires, data=None)
        `pdedef` must evaluate the functions G_i which define the system
        of PDEs. `pdedef` is called approximately midway between each
        pair of mesh points in turn by ``dim1_parab_dae_keller``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ut : float, ndarray, shape (`npde`)
            `ut`[i-1] contains the value of the component dU_i(x,t)/dt,
            for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        res : float, ndarray, shape (`npde`), to be modified in place
            On exit: `res`[i-1] must contain the ith component of G, for
            i = 1,2,...,`npde`, where G is defined as

                G_i = sum_j = 1^`npde`P_{i,j}dU_j/dt+sum_j =
                1^`nv`Q_{i,j}V^._j,

            i.e., only terms depending explicitly on time derivatives,
            or

                G_i = sum_j = 1^`npde`P_{i,j}dU_j/dt+sum_j =
                1^`nv`Q_{i,j}V^._j+S_i,

            i.e., all terms in equation (3).

            The definition of G is determined by the input value of
            `ires`.

        ires : int
            The form of G_i that must be returned in the array `res`.

            `ires` = -1

              Equation (9) must be used.

            `ires` = 1

              Equation (0) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions,
            as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_dae_keller`` returns to the calling function
              with the error indicator set to `errno` = 4.

    bndary : callable ires = bndary(npde, t, ibnd, nobc, u, ut, nv, v,
        vdot, res, ires, data=None)
        `bndary` must evaluate the functions G_i^L and G_i^R which
        describe the boundary conditions, as given in (5) and (6).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must compute the left-hand boundary condition at
              x = a.

            `ibnd` != 0

              `bndary` must compute the right-hand boundary condition at
              x = b.

        nobc : int
            Specifies the number of boundary conditions at the boundary
            specified by `ibnd`.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ut : float, ndarray, shape (`npde`)
            `ut`[i-1] contains the value of the component dU_i(x,t)/dt
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: `vdot`[i-1], for i = 1,2,...,`nv`, may only appear
            linearly as in (7) and (8).

        res : float, ndarray, shape (`nobc`), to be modified in place
            On exit: `res`[i-1] must contain the ith component of G^L or
            G^R, depending on the value of `ibnd`, for i =
            1,2,...,`nobc`, where G^L is defined as

                G_i^L = sum_j = 1^`npde`E_i,j^LdU_j/dt+sum_j =
                1^`nv`H_i,j^LV^._j,

            i.e., only terms depending explicitly on time derivatives,
            or

                G_i^L = sum_j = 1^`npde`E_i,j^LdU_j/dt+sum_j =
                1^`nv`H_i,j^LV^._j+K_i^L,

            i.e., all terms in equation (7), and similarly for G_i^R.

            The definitions of G^L and G^R are determined by the input
            value of `ires`.

        ires : int
            The form of G_i^L (or G_i^R) that must be returned in the
            array `res`.

            `ires` = -1

              Equation (1) must be used.

            `ires` = 1

              Equation (2) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_dae_keller`` returns to the calling function
              with the error indicator set to `errno` = 4.

    u : float, shape (`neqn`), modified in place
        On entry: the initial values of the dependent variables defined
        as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t_0), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t_0), for i = 1,2,...,`nv`.

        On exit: the computed solution U_i(x_j,t), for j =
        1,2,...,`npts`, for i = 1,2,...,`npde`, and V_k(t), for k =
        1,2,...,`nv`, evaluated at t = `ts`, as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t), for i = 1,2,...,`nv`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`)
        The mesh points in the space direction. `x`[0] must specify the
        left-hand boundary, a, and `x`[`npts`-1] must specify the
        right-hand boundary, b.

    nleft : int
        The number n_a of boundary conditions at the left-hand mesh
        point `x`[0].

    nv : int
        The number of coupled ODE components.

    odedef : None or callable ires = odedef(npde, t, nv, v, vdot, nxi,
        xi, ucp, ucpx, ucpt, r, ires, data=None)
        `odedef` must evaluate the functions R, which define the system
        of ODEs, as given in (4).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        nxi : int
            The number of ODE/PDE coupling points.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling
            points, xi_i, for i = 1,2,...,`nxi`.

        ucp : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucp`[i-1,j-1] contains the value of U_i(x,t)
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucpx : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpx`[i-1,j-1] contains the value of
            dU_i(x,t)/dx at the coupling point x = xi_j, for j =
            1,2,...,`nxi`, for i = 1,2,...,`npde`.

        ucpt : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpt`[i-1,j-1] contains the value of dU_i/dt
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        r : float, ndarray, shape (`nv`), to be modified in place
            On exit: if `nv` > 0, `r`[i-1] must contain the ith
            component of R, for i = 1,2,...,`nv`, where R is defined as

                R = -BV^.-CU_t^*,

            i.e., only terms depending explicitly on time derivatives,
            or

                R = A-BV^.-CU_t^*,

            i.e., all terms in equation (4). The definition of R is
            determined by the input value of `ires`.

        ires : int
            The form of R that must be returned in the array `r`.

            `ires` = -1

              Equation (3) must be used.

            `ires` = 1

              Equation (4) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may reset
            `ires` to force the integration function to take certain
            actions, as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_dae_keller`` returns to the calling function
              with the error indicator set to `errno` = 4.

    nxi : int
        The number of ODE/PDE coupling points.

    xi : float, shape (`nxi`)
        `xi`[i-1], for i = 1,2,...,`nxi`, must be set to the ODE/PDE
        coupling points, xi_i.

    neqn : int
        The number of ODEs in the time direction.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; if `itol` in (3, 4): `neqn`;
        otherwise: 0.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; if `itol` in (2, 4): `neqn`;
        otherwise: 0.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``dim1_parab_dae_keller`` whether to interpret
        either or both of `rtol` or `atol` as a vector or scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        In the above, e_i denotes the estimated local error for the ith
        component of the coupled PDE/ODE system in time, `u`[i-1], for i
        = 1,2,...,`neqn`.

        The choice of norm used is defined by the argument `norm`.

    norm : str, length 1
        The type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L_2 norm.

        If `u`_norm denotes the norm of the vector `u` of length `neqn`,
        then for the averaged L_2 norm

            `u`_norm = sqrt(1/`neqn`sum_i = 1^`neqn`(`u`[i-1]/w_i)^2),

        while for the maximum norm

            `u`_norm = max_i(|`u`[i-1] / w_i|).

        See the description of `itol` for the formulation of the weight
        vector w.

    laopt : str, length 1
        The type of matrix algebra required.

        `laopt` = 'F'

          Full matrix methods to be used.

        `laopt` = 'B'

          Banded matrix methods to be used.

        `laopt` = 'S'

          Sparse matrix methods to be used.

    algopt : float, shape (30)
        May be set to control various options available in the
        integrator. If you wish to employ all the default options,
        `algopt`[0] should be set to 0.0. Default values will also be
        used for any other elements of `algopt` set to zero. The
        permissible values, default values, and meanings are as follows:

        `algopt`[0]

          Selects the ODE integration method to be used. If `algopt`[0]
          = 1.0, a BDF method is used and if `algopt`[0] = 2.0, a Theta
          method is used. The default value is `algopt`[0] = 1.0.

        If `algopt`[0] = 2.0, then `algopt`[i-1], for i = 2,3,...,4, are
        not used.

        `algopt`[1]

          Specifies the maximum order of the BDF integration formula to
          be used. `algopt`[1] may be 1.0, 2.0, 3.0, 4.0 or 5.0. The
          default value is `algopt`[1] = 5.0.

        `algopt`[2]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the BDF method. If
          `algopt`[2] = 1.0 a modified Newton iteration is used and if
          `algopt`[2] = 2.0 a functional iteration method is used. If
          functional iteration is selected and the integrator encounters
          difficulty, there is an automatic switch to the modified
          Newton iteration. The default value is `algopt`[2] = 1.0.

        `algopt`[3]

          Specifies whether or not the Petzold error test is to be
          employed. The Petzold error test results in extra overhead but
          is more suitable when algebraic equations are present, such as
          P_{i,j} = 0.0, for j = 1,2,...,`npde`, for some i or when
          there is no V^._i(t) dependence in the coupled ODE system. If
          `algopt`[3] = 1.0, the Petzold test is used. If `algopt`[3] =
          2.0, the Petzold test is not used. The default value is
          `algopt`[3] = 1.0.

        If `algopt`[0] = 1.0, `algopt`[i-1], for i = 5,6,...,7, are not
        used.

        `algopt`[4]

          Specifies the value of Theta to be used in the Theta
          integration method. 0.51 <= `algopt`[4] <= 0.99. The default
          value is `algopt`[4] = 0.55.

        `algopt`[5]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the Theta method.
          If `algopt`[5] = 1.0, a modified Newton iteration is used and
          if `algopt`[5] = 2.0, a functional iteration method is used.
          The default value is `algopt`[5] = 1.0.

        `algopt`[6]

          Specifies whether or not the integrator is allowed to switch
          automatically between modified Newton and functional iteration
          methods in order to be more efficient. If `algopt`[6] = 1.0,
          switching is allowed and if `algopt`[6] = 2.0, switching is
          not allowed. The default value is `algopt`[6] = 1.0.

        `algopt`[10]

          Specifies a point in the time direction, t_crit, beyond which
          integration must not be attempted. The use of t_crit is
          described under the argument `itask`. If `algopt`[0] != 0.0, a
          value of 0.0, for `algopt`[10], say, should be specified even
          if `itask` subsequently specifies that t_crit will not be
          used.

        `algopt`[11]

          Specifies the minimum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[11]
          should be set to 0.0.

        `algopt`[12]

          Specifies the maximum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[12]
          should be set to 0.0.

        `algopt`[13]

          Specifies the initial step size to be attempted by the
          integrator. If `algopt`[13] = 0.0, the initial step size is
          calculated internally.

        `algopt`[14]

          Specifies the maximum number of steps to be attempted by the
          integrator in any one call. If `algopt`[14] = 0.0, no limit is
          imposed.

        `algopt`[22]

          Specifies what method is to be used to solve the nonlinear
          equations at the initial point to initialize the values of U,
          U_t, V and V^.. If `algopt`[22] = 1.0, a modified Newton
          iteration is used and if `algopt`[22] = 2.0, functional
          iteration is used. The default value is `algopt`[22] = 1.0.

        `algopt`[28] and `algopt`[29] are used only for the sparse
        matrix algebra option, i.e., `laopt` = 'S'.

        `algopt`[28]

          Governs the choice of pivots during the decomposition of the
          first Jacobian matrix. It should lie in the range 0.0 <
          `algopt`[28] < 1.0, with smaller values biasing the algorithm
          towards maintaining sparsity at the expense of numerical
          stability. If `algopt`[28] lies outside this range then the
          default value is used. If the functions regard the Jacobian
          matrix as numerically singular then increasing `algopt`[28]
          towards 1.0 may help, but at the cost of increased fill-in.
          The default value is `algopt`[28] = 0.1.

        `algopt`[29]

          Used as a relative pivot threshold during subsequent Jacobian
          decompositions (see `algopt`[28]) below which an internal
          error is invoked. `algopt`[29] must be greater than zero,
          otherwise the default value is used. If `algopt`[29] is
          greater than 1.0 no check is made on the pivot size, and this
          may be a necessary option if the Jacobian is found to be
          numerically singular (see `algopt`[28]). The default value is
          `algopt`[29] = 0.0001.

    comm : dict, communication object, modified in place
        Note: this argument will be (re-)initialized when it is an empty
        ``dict`` or under the following condition: `ind` = 0.

        Communication structure.

        On initial entry: need not be set.

    lrsave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['rsave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lrsave` is computed by the
        function.

        If your supplied `lrsave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lrsave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lrsave` set to at least the lower-bound value returned in
        `lrsave_min`.

    lisave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['isave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lisave` is computed by the
        function.

        If your supplied `lisave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lisave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lisave` set to at least the lower-bound value returned in
        `lisave_min`.

    itask : int
        The task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step in the time direction and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

        `itask` = 4

          Normal computation of output values `u` at t = `tout` but
          without overshooting t = t_crit where t_crit is described
          under the argument `algopt`.

        `itask` = 5

          Take one step in the time direction and return, without
          passing t_crit, where t_crit is described under the argument
          `algopt`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_dae_keller`` and the underlying ODE solver as
        follows:

        `itrace` <= -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` = 1

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        `itrace` = 2

          Output from the underlying ODE solver is similar to that
          produced when `itrace` = 1, except that the advisory messages
          are given in greater detail.

        `itrace` >= 3

          Output from the underlying ODE solver is similar to that
          produced when `itrace` = 2, except that the advisory messages
          are given in greater detail.

        You advised to set `itrace` = 0, unless you are experienced with
        submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_dae_keller``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution in `u`. Normally
        `ts` = `tout`.

    ind : int
        `ind` = 1.

    lrsave_min : int
        A lower bound on the sufficient size for `comm`['rsave'].

    lisave_min : int
        A lower bound on the sufficient size for `comm`['isave'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `lrsave` = *<value>*.

            Constraint: `lrsave` >= *<value>*.

        (`errno` 1)
            On entry, `lisave` = *<value>*.

            Constraint: `lisave` >= *<value>*.

        (`errno` 1)
            On entry, at least one point in `xi` lies outside
            [`x`[0],`x`[`npts`-1]]: `x`[0] = *<value>* and `x`[`npts`-1]
            = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xi`[i] = *<value>* and `xi`[i-1] =
            *<value>*.

            Constraint: `xi`[i] > `xi`[i-1].

        (`errno` 1)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: corresponding elements `atol`[i-1] and
            `rtol`[j-1] cannot both be 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `rtol`[i-1] = *<value>*.

            Constraint: `rtol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `atol`[i-1] = *<value>*.

            Constraint: `atol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `neqn` = *<value>*, `npde` = *<value>*, `npts` =
            *<value>* and `nv` = *<value>*.

            Constraint: `neqn` = `npde`*`npts`+`nv`.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` = 0 when `nv` = 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` >= 0 when `nv` > 0.

        (`errno` 1)
            On entry, `nleft` = *<value>*, `npde` = *<value>*.

            Constraint: `nleft` <= `npde`.

        (`errno` 1)
            On entry, `nleft` = *<value>*.

            Constraint: `nleft` >= 0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `laopt` = *<value>*.

            Constraint: `laopt` = 'F', 'B' or 'S'.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = 'A' or 'M'.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2, 3, 4 or 5.

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 0.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `atol` and `rtol` were too small to start integration.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef`,
            `bndary`, or `odedef`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 12)
            In solving ODE system, the maximum number of steps
            `algopt`[14] has been exceeded. `algopt`[14] = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `atol` and `rtol`.
            `ts` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef`, `bndary`, or `odedef`. Integration is
            successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `atol` or `rtol`
            are unlikely to result in a changed solution.

        (`errno` 13)
            Zero error weights encountered during time integration.

        (`errno` 15)
            When using the sparse option `lisave` or `lrsave` is too
            small: `lisave` = *<value>*, `lrsave` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_dae_keller'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, res[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, q, res, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ut_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _nv_pdedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_pdedefh,),
            )
            _vdot_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_pdedefh,),
            )
            _res_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _res_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _res_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(ut)
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nobc.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nobc].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ut, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nobc, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _nobc_bndaryh = utils._EngineIntScalarType.to_py(nobc.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ut_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npde_bndaryh,),
            )
            _nv_bndaryh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_bndaryh,),
            )
            _vdot_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_bndaryh,),
            )
            _res_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_nobc_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _ibnd_bndaryh,
                    _nobc_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _res_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _ibnd_bndaryh,
                    _nobc_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _res_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _res_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'res'}, res,
            )
            utils._unlink_pointer(res)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_odedef_name = fun_name + ':odedef'

    odedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, rcp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpt[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucptx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, r[nv].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_odedefh(_odedef, en_data, npde, t, nv, v, vdot, nxi, xi, ucp, ucpx, rcp, ucpt, ucptx, r, ires, _iuser, _ruser):
        """Helper for odedef."""
        if odedef is None:
            return
        try:
            _npde_odedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_odedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nv_odedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_odedefh,),
            )
            _vdot_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_odedefh,),
            )
            _nxi_odedefh = utils._EngineIntScalarType.to_py(nxi.contents)
            _xi_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_odedefh,),
            )
            _ucp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpt_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpt,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _r_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_nv_odedefh,),
                can_reference_contents=False,
            )
            _ires_odedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                    data,
                )
            else:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._unlink_pointer(xi)
            utils._unlink_pointer(ucp)
            utils._unlink_pointer(ucpx)
            utils._unlink_pointer(ucpt)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_odedefh, {'fun_name': cb_odedef_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_odedefh, {'fun_name': cb_odedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03pkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[neqn].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._BYREF_INT, # Input, nleft.
        utils._BYREF_INT, # Input, nv.
        utils._UNUSED_FUNCTYPE,
        odedefh,
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._BYREF_INT, # Input, neqn.
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, laopt(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, algopt[30].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Output, lrsave_min.
        utils._BYREF_INT, # Output, lisave_min.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of laopt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _neqn = utils._EngineIntScalarType(
        neqn, {'fun_name': fun_name, 'entity': 'neqn'},
    ).to_ctypes()
    _neqn_py = utils._EngineIntScalarType.to_py(_neqn)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_neqn_py,),
    )
    _u = _u_cls.to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _nleft = utils._EngineIntScalarType(
        nleft, {'fun_name': fun_name, 'entity': 'nleft'},
    ).to_ctypes()
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    utils._check_callable(odedef, {'fun_name': fun_name, 'entity': 'odedef'}, allow_none=True)
    _odedef = utils._UNUSED_CALLBACK
    _odedefh = utils._cast_callback(
        odedefh, cb_odedefh,
        {'fun_name': fun_name, 'entity': 'odedef'},
    )
    _nxi = utils._EngineIntScalarType(
        nxi, {'fun_name': fun_name, 'entity': 'nxi'},
    ).to_ctypes()
    _nxi_py = utils._EngineIntScalarType.to_py(_nxi)
    _xi = utils._EngineFloat64ArrayType(
        xi, {'fun_name': fun_name, 'entity': 'xi'},
        exp_shape=(_nxi_py,),
    ).to_ctypes()
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    elif _itol_py in (3, 4):
        _rtol_dim1 = _neqn_py
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    elif _itol_py in (2, 4):
        _atol_dim1 = _neqn_py
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _laopt = utils._EngineCharScalarType(
        laopt, {'fun_name': fun_name, 'entity': 'laopt'}, exp_chrlen=1,
    ).to_ctypes()
    _algopt = utils._EngineFloat64ArrayType(
        algopt, {'fun_name': fun_name, 'entity': 'algopt'},
        exp_shape=(30,),
    ).to_ctypes()
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _comm_needs_init = (
        utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
        ) or
        _ind_py == 0
    )
    if _comm_needs_init:
        if lrsave is None:
            _lrsave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lrsave = utils._EngineIntScalarType(
                lrsave, {'fun_name': fun_name, 'entity': 'lrsave'},
            ).to_ctypes()
        _lrsave_py = utils._EngineIntScalarType.to_py(_lrsave)
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
    else:
        _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = comm['rsave']
    if _comm_needs_init:
        if lisave is None:
            _lisave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lisave = utils._EngineIntScalarType(
                lisave, {'fun_name': fun_name, 'entity': 'lisave'},
            ).to_ctypes()
        _lisave_py = utils._EngineIntScalarType.to_py(_lisave)
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
    else:
        _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = comm['isave']
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iemode_py = 1 if _comm_needs_init else 0
    _iemode = utils._EngineIntScalarType(
        _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
    ).to_ctypes()
    _lrsave_min = utils._EngineIntScalarType.empty_ctypes()
    _lisave_min = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _u,
        _npts,
        _x,
        _nleft,
        _nv,
        _odedef,
        _odedefh,
        _nxi,
        _xi,
        _neqn,
        _rtol,
        _atol,
        _itol,
        _norm,
        _laopt,
        _algopt,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iemode,
        _lrsave_min,
        _lisave_min,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _norm._length_,
        _laopt._length_,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS and _iemode_py == 1:
        _iemode_py = 0
        _iemode = utils._EngineIntScalarType(
            _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
        ).to_ctypes()
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lrsave_py = max(_lrsave_min_py, _lrsave_py)
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        _lisave_py = max(_lisave_min_py, _lisave_py)
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
        engine_ctypes(
            _en_data,
            utils._UNUSED_PRINT_REC,
            _print_rech,
            _npde,
            _ts,
            _tout,
            _pdedef,
            _pdedefh,
            _bndary,
            _bndaryh,
            _u,
            _npts,
            _x,
            _nleft,
            _nv,
            _odedef,
            _odedefh,
            _nxi,
            _xi,
            _neqn,
            _rtol,
            _atol,
            _itol,
            _norm,
            _laopt,
            _algopt,
            _rsave,
            _lrsave,
            _isave,
            _lisave,
            _itask,
            _itrace,
            _nadv,
            _ind,
            _iemode,
            _lrsave_min,
            _lisave_min,
            _iuser,
            _ruser,
            _cwsav,
            _lwsav,
            _iwsav,
            _rwsav,
            _errbuf,
            _ierr,
            _norm._length_,
            _laopt._length_,
            80,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        if _ierr_py == 1 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrsave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lisave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lisave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one point in xi lies outside [x[0],x[npts-1]]:',
                    'x[0] = {:s} and x[npts-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xi[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xi[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xi[i] > xi[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: corresponding elements atol[i-1] and rtol[j-1]',
                    'cannot both be 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and rtol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and atol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqn = {:s}, npde = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npts = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: neqn = npde*npts+nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi = 0 when nv = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi >= 0 when nv > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nleft = {:s}, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nleft <= npde.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nleft = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nleft >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laopt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: laopt = \'F\', \'B\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'A\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99976:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of atol and rtol.',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99975:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99972:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef, bndary, or odedef. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'atol and rtol were too small to start integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef, bndary, or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99968:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in atol or rtol are unlikely',
                    'to result in a changed solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99967:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In solving ODE system, the maximum number of steps',
                    'algopt[14] has been exceeded.',
                    'algopt[14] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99965:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Zero error weights encountered during time integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99964:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'When using the sparse option lisave or lrsave is too small:',
                    'lisave = {:s}, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind', 'lrsave_min', 'lisave_min'],
        )(
            _ts_py,
            _ind_py,
            _lrsave_min_py,
            _lisave_min_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_convdiff_dae(npde, ts, tout, pdedef, numflx, bndary, u, npts, x, nv, odedef, nxi, xi, neqn, rtol, atol, itol, norm, laopt, algopt, comm, lrsave, lisave, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of convection-diffusion PDEs with source terms in
    conservative form, coupled DAEs, method of lines, upwind scheme
    using numerical flux function based on Riemann solver, one space
    variable.

    For full information please refer to the NAG Library document for
    d03pl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03plf.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : None or callable ires = pdedef(npde, t, x, u, ux, nv, v,
        vdot, p, c, d, s, ires, data=None)
        `pdedef` must evaluate the functions P_{i,j}, C_i, D_i and S_i
        which partially define the system of PDEs. P_{i,j} and C_i may
        depend on x, t, U and V; D_i may depend on x, t, U, U_x and V;
        and S_i may depend on x, t, U, V and linearly on V^.. `pdedef`
        is called approximately midway between each pair of mesh points
        in turn by ``dim1_parab_convdiff_dae``. None may be used for
        `pdedef` for problems in the form (2).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in S_j, for j = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`), to be modified in
        place
            On exit: `p`[i-1,j-1] must be set to the value of
            P_{i,j}(x,t,U,V), for j = 1,2,...,`npde`, for i =
            1,2,...,`npde`.

        c : float, ndarray, shape (`npde`), to be modified in place
            On exit: `c`[i-1] must be set to the value of C_i(x,t,U,V),
            for i = 1,2,...,`npde`.

        d : float, ndarray, shape (`npde`), to be modified in place
            On exit: `d`[i-1] must be set to the value of
            D_i(x,t,U,U_x,V), for i = 1,2,...,`npde`.

        s : float, ndarray, shape (`npde`), to be modified in place
            On exit: `s`[i-1] must be set to the value of
            S_i(x,t,U,V,V^.), for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_dae`` returns to the calling
              function with the error indicator set to `errno` = 4.

    numflx : callable ires = numflx(npde, t, x, nv, v, uleft, uright,
        flux, ires, data=None)
        `numflx` must supply the numerical flux for each PDE given the
        left and right values of the solution vector `u`. `numflx` is
        called approximately midway between each pair of mesh points in
        turn by ``dim1_parab_convdiff_dae``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        uleft : float, ndarray, shape (`npde`)
            `uleft`[i-1] contains the left value of the component
            U_i(x), for i = 1,2,...,`npde`.

        uright : float, ndarray, shape (`npde`)
            `uright`[i-1] contains the right value of the component
            U_i(x), for i = 1,2,...,`npde`.

        flux : float, ndarray, shape (`npde`), to be modified in place
            On exit: `flux`[i-1] must be set to the numerical flux F^_i,
            for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_dae`` returns to the calling
              function with the error indicator set to `errno` = 4.

    bndary : callable ires = bndary(npde, npts, t, x, u, nv, v, vdot,
        ibnd, g, ires, data=None)
        `bndary` must evaluate the functions G_i^L and G_i^R which
        describe the physical and numerical boundary conditions, as
        given by (9) and (0).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            The mesh points in the spatial direction. `x`[0] corresponds
            to the left-hand boundary, a, and `x`[`npts`-1] corresponds
            to the right-hand boundary, b.

        u : float, ndarray, shape (`npde`, `npts`)
            `u`[i-1,j-1] contains the value of the component U_i(x,t) at
            x = `x`[j-1], for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

            Note: if banded matrix algebra is to be used then the
            functions G_i^L and G_i^R may depend on the value of
            U_i(x,t) at the boundary point and the two adjacent points
            only.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in G_j^L and G_j^R, for j = 1,2,...,`npde`.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must evaluate the left-hand boundary condition at
              x = a.

            `ibnd` != 0

              `bndary` must evaluate the right-hand boundary condition
              at x = b.

        g : float, ndarray, shape (`npde`), to be modified in place
            On exit: `g`[i-1] must contain the ith component of either
            G_i^L or G_i^R in (9) and (0), depending on the value of
            `ibnd`, for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_dae`` returns to the calling
              function with the error indicator set to `errno` = 4.

    u : float, shape (`neqn`), modified in place
        On entry: the initial values of the dependent variables defined
        as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t_0), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t_0), for i = 1,2,...,`nv`.

        On exit: the computed solution U_i(x_j,t), for j =
        1,2,...,`npts`, for i = 1,2,...,`npde`, and V_k(t), for k =
        1,2,...,`nv`, evaluated at t = `ts`, as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t), for i = 1,2,...,`nv`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`)
        The mesh points in the space direction. `x`[0] must specify the
        left-hand boundary, a, and `x`[`npts`-1] must specify the
        right-hand boundary, b.

    nv : int
        The number of coupled ODE components.

    odedef : None or callable ires = odedef(npde, t, nv, v, vdot, nxi,
        xi, ucp, ucpx, ucpt, r, ires, data=None)
        `odedef` must evaluate the functions R, which define the system
        of ODEs, as given in (4).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        nxi : int
            The number of ODE/PDE coupling points.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling point,
            xi_i, for i = 1,2,...,`nxi`.

        ucp : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucp`[i-1,j-1] contains the value of U_i(x,t)
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucpx : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpx`[i-1,j-1] contains the value of
            dU_i(x,t)/dx at the coupling point x = xi_j, for j =
            1,2,...,`nxi`, for i = 1,2,...,`npde`.

        ucpt : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpt`[i-1,j-1] contains the value of dU_i/dt
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        r : float, ndarray, shape (`nv`), to be modified in place
            On exit: `r`[i-1] must contain the ith component of R, for i
            = 1,2,...,`nv`, where R is defined as

                R = L-MV^.-NU_t^*,

            or

                R = -MV^.-NU_t^*.

            The definition of R is determined by the input value of
            `ires`.

        ires : int
            The form of R that must be returned in the array `r`.

            `ires` = 1

              Equation (1) must be used.

            `ires` = -1

              Equation (2) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may reset
            `ires` to force the integration function to take certain
            actions, as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_dae`` returns to the calling
              function with the error indicator set to `errno` = 4.

    nxi : int
        The number of ODE/PDE coupling points.

    xi : float, shape (`nxi`)
        `xi`[i-1], for i = 1,2,...,`nxi`, must be set to the ODE/PDE
        coupling points.

    neqn : int
        The number of ODEs in the time direction.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; if `itol` in (3, 4): `neqn`;
        otherwise: 0.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; if `itol` in (2, 4): `neqn`;
        otherwise: 0.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. If e_i is
        the estimated local error for `u`[i-1], for i = 1,2,...,`neqn`,
        and || || denotes the norm, the error test to be satisfied is
        ||e_i|| < 1.0. `itol` indicates to ``dim1_parab_convdiff_dae``
        whether to interpret either or both of `rtol` and `atol` as a
        vector or scalar in the formation of the weights w_i used in the
        calculation of the norm (see the description of `norm`):

        [table omitted]

    norm : str, length 1
        The type of norm to be used.

        `norm` = '1'

          Averaged L_1 norm.

        `norm` = '2'

          Averaged L_2 norm.

        If U_norm denotes the norm of the vector `u` of length `neqn`,
        then for the averaged L_1 norm

            U_norm = 1/`neqn`sum_i = 1^`neqn``u`[i-1]/w_i,

        and for the averaged L_2 norm

            U_norm = sqrt(1/`neqn`sum_i = 1^`neqn`(`u`[i-1]/w_i)^2).

        See the description of `itol` for the formulation of the weight
        vector w.

    laopt : str, length 1
        The type of matrix algebra required.

        `laopt` = 'F'

          Full matrix methods to be used.

        `laopt` = 'B'

          Banded matrix methods to be used.

        `laopt` = 'S'

          Sparse matrix methods to be used.

    algopt : float, shape (30)
        May be set to control various options available in the
        integrator. If you wish to employ all the default options,
        `algopt`[0] should be set to 0.0. Default values will also be
        used for any other elements of `algopt` set to zero. The
        permissible values, default values, and meanings are as follows:

        `algopt`[0]

          Selects the ODE integration method to be used. If `algopt`[0]
          = 1.0, a BDF method is used and if `algopt`[0] = 2.0, a Theta
          method is used. The default is `algopt`[0] = 1.0.

        If `algopt`[0] = 2.0, then `algopt`[i-1], for i = 2,3,...,4, are
        not used.

        `algopt`[1]

          Specifies the maximum order of the BDF integration formula to
          be used. `algopt`[1] may be 1.0, 2.0, 3.0, 4.0 or 5.0. The
          default value is `algopt`[1] = 5.0.

        `algopt`[2]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the BDF method. If
          `algopt`[2] = 1.0 a modified Newton iteration is used and if
          `algopt`[2] = 2.0 a functional iteration method is used. If
          functional iteration is selected and the integrator encounters
          difficulty, there is an automatic switch to the modified
          Newton iteration. The default value is `algopt`[2] = 1.0.

        `algopt`[3]

          Specifies whether or not the Petzold error test is to be
          employed. The Petzold error test results in extra overhead but
          is more suitable when algebraic equations are present, such as
          P_{i,j} = 0.0, for j = 1,2,...,`npde`, for some i or when
          there is no V^._i(t) dependence in the coupled ODE system. If
          `algopt`[3] = 1.0, the Petzold test is used. If `algopt`[3] =
          2.0, the Petzold test is not used. The default value is
          `algopt`[3] = 1.0.

        If `algopt`[0] = 1.0, `algopt`[i-1], for i = 5,6,...,7, are not
        used.

        `algopt`[4]

          Specifies the value of Theta to be used in the Theta
          integration method. 0.51 <= `algopt`[4] <= 0.99. The default
          value is `algopt`[4] = 0.55.

        `algopt`[5]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the Theta method.
          If `algopt`[5] = 1.0, a modified Newton iteration is used and
          if `algopt`[5] = 2.0, a functional iteration method is used.
          The default value is `algopt`[5] = 1.0.

        `algopt`[6]

          Specifies whether or not the integrator is allowed to switch
          automatically between modified Newton and functional iteration
          methods in order to be more efficient. If `algopt`[6] = 1.0,
          switching is allowed and if `algopt`[6] = 2.0, switching is
          not allowed. The default value is `algopt`[6] = 1.0.

        `algopt`[10]

          Specifies a point in the time direction, t_crit, beyond which
          integration must not be attempted. The use of t_crit is
          described under the argument `itask`. If `algopt`[0] != 0.0, a
          value of 0.0 for `algopt`[10], say, should be specified even
          if `itask` subsequently specifies that t_crit will not be
          used.

        `algopt`[11]

          Specifies the minimum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[11]
          should be set to 0.0.

        `algopt`[12]

          Specifies the maximum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[12]
          should be set to 0.0.

        `algopt`[13]

          Specifies the initial step size to be attempted by the
          integrator. If `algopt`[13] = 0.0, the initial step size is
          calculated internally.

        `algopt`[14]

          Specifies the maximum number of steps to be attempted by the
          integrator in any one call. If `algopt`[14] = 0.0, no limit is
          imposed.

        `algopt`[22]

          Specifies what method is to be used to solve the nonlinear
          equations at the initial point to initialize the values of U,
          U_t, V and V^.. If `algopt`[22] = 1.0, a modified Newton
          iteration is used and if `algopt`[22] = 2.0, functional
          iteration is used. The default value is `algopt`[22] = 1.0.

        `algopt`[28] and `algopt`[29] are used only for the sparse
        matrix algebra option, i.e., `laopt` = 'S'.

        `algopt`[28]

          Governs the choice of pivots during the decomposition of the
          first Jacobian matrix. It should lie in the range 0.0 <
          `algopt`[28] < 1.0, with smaller values biasing the algorithm
          towards maintaining sparsity at the expense of numerical
          stability. If `algopt`[28] lies outside the range then the
          default value is used. If the functions regard the Jacobian
          matrix as numerically singular, increasing `algopt`[28]
          towards 1.0 may help, but at the cost of increased fill-in.
          The default value is `algopt`[28] = 0.1.

        `algopt`[29]

          Used as the relative pivot threshold during subsequent
          Jacobian decompositions (see `algopt`[28]) below which an
          internal error is invoked. `algopt`[29] must be greater than
          zero, otherwise the default value is used. If `algopt`[29] is
          greater than 1.0 no check is made on the pivot size, and this
          may be a necessary option if the Jacobian matrix is found to
          be numerically singular (see `algopt`[28]). The default value
          is `algopt`[29] = 0.0001.

    comm : dict, communication object, modified in place
        Note: this argument will be (re-)initialized when it is an empty
        ``dict`` or under the following condition: `ind` = 0.

        Communication structure.

        On initial entry: need not be set.

    lrsave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['rsave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lrsave` is computed by the
        function.

        If your supplied `lrsave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lrsave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lrsave` set to at least the lower-bound value returned in
        `lrsave_min`.

    lisave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['isave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lisave` is computed by the
        function.

        If your supplied `lisave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lisave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lisave` set to at least the lower-bound value returned in
        `lisave_min`.

    itask : int
        The task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step in the time direction and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

        `itask` = 4

          Normal computation of output values `u` at t = `tout` but
          without overshooting t = t_crit where t_crit is described
          under the argument `algopt`.

        `itask` = 5

          Take one step in the time direction and return, without
          passing t_crit, where t_crit is described under the argument
          `algopt`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_convdiff_dae`` and the underlying ODE solver.
        `itrace` may take the value -1, 0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` > 0

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases. You are advised to set `itrace` = 0, unless you are
        experienced with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` should be
          reset between calls to ``dim1_parab_convdiff_dae``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    lrsave_min : int
        A lower bound on the sufficient size for `comm`['rsave'].

    lisave_min : int
        A lower bound on the sufficient size for `comm`['isave'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `lrsave` = *<value>*.

            Constraint: `lrsave` >= *<value>*.

        (`errno` 1)
            On entry, `lisave` = *<value>*.

            Constraint: `lisave` >= *<value>*.

        (`errno` 1)
            On entry, at least one point in `xi` lies outside
            [`x`[0],`x`[`npts`-1]]: `x`[0] = *<value>* and `x`[`npts`-1]
            = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xi`[i] = *<value>* and `xi`[i-1] =
            *<value>*.

            Constraint: `xi`[i] > `xi`[i-1].

        (`errno` 1)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: corresponding elements `atol`[i-1] and
            `rtol`[j-1] cannot both be 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `rtol`[i-1] = *<value>*.

            Constraint: `rtol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `atol`[i-1] = *<value>*.

            Constraint: `atol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `neqn` = *<value>*, `npde` = *<value>*, `npts` =
            *<value>* and `nv` = *<value>*.

            Constraint: `neqn` = `npde`*`npts`+`nv`.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` = 0 when `nv` = 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` >= 0 when `nv` > 0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `laopt` = *<value>*.

            Constraint: `laopt` = 'F', 'B' or 'S'.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1' or '2'.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2, 3, 4 or 5.

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 0.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef`, `numflx`, `bndary` or
            `odedef`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `atol` and `rtol` were too small to start integration.

        (`errno` 8)
            `ires` set to an invalid value in a call to functions
            `pdedef`, `numflx`, `bndary` or `odedef`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 12)
            In solving ODE system, the maximum number of steps
            `algopt`[14] has been exceeded. `algopt`[14] = *<value>*.

        (`errno` 14)
            The functions P, D, or C appear to depend on time
            derivatives.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `atol` and `rtol`.
            `ts` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in functions `pdedef`, `numflx`, `bndary` or `odedef`.
            Integration is successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `atol` or `rtol`
            are unlikely to result in a changed solution.

        (`errno` 13)
            Zero error weights encountered during time integration.

        (`errno` 15)
            When using the sparse option `lisave` or `lrsave` is too
            small: `lisave` = *<value>*, `lrsave` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_convdiff_dae'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, c[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, d[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[npde].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if pdedef is None:
        pdedef_auxil_ctypes = utils._EngineState._get_symbol('d03plpt')
        pdedef_auxil_ctypes.restype = None
        pdedef_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, npde.
            utils._BYREF_FLOAT64, # Input, t.
            utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
            utils._BYREF_INT, # Input, np.
            utils._BYREF_INT, # Input, nptl.
            utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
            utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
            utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
            utils._BYREF_INT, # Input, nv.
            utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
            utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
            utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
            utils._EngineFloat64ArrayType.c_type, # Output, c[npde, np].
            utils._EngineFloat64ArrayType.c_type, # Output, d[npde, nptl].
            utils._BYREF_INT, # Input, ns.
            utils._EngineFloat64ArrayType.c_type, # Output, s[npde].
            utils._BYREF_INT, # Input/Output, ires.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, c, d, ns, s, ires, iuser, ruser):
        """Helper for pdedef."""
        try:
            if pdedef is None:
                pdedef_auxil_ctypes(
                    en_data,
                    npde,
                    t,
                    xv,
                    np,
                    nptl,
                    u,
                    ut,
                    ux,
                    nv,
                    v,
                    vdot,
                    p,
                    c,
                    d,
                    ns,
                    s,
                    ires,
                    iuser,
                    ruser,
                )
                return
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _nv_pdedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_pdedefh,),
            )
            _vdot_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_pdedefh,),
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _c_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                c,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _d_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                d,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _s_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                s,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _c_pdedefh,
                    _d_pdedefh,
                    _s_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _c_pdedefh,
                    _d_pdedefh,
                    _s_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _s_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 's'}, s,
            )
            utils._unlink_pointer(s)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_numflx_name = fun_name + ':numflx'

    numflxh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, uleft[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uright[npde].
        utils._EngineFloat64ArrayType.c_type, # Output, flux[npde].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
    )

    def cb_numflxh(_numflx, en_data, npde, t, x, nv, v, uleft, uright, flux, ires, _iuser, _ruser, iwsav):
        """Helper for numflx."""
        try:
            _npde_numflxh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_numflxh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_numflxh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _nv_numflxh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_numflxh,),
            )
            _uleft_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                uleft,
                (_npde_numflxh,),
            )
            _uright_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                uright,
                (_npde_numflxh,),
            )
            _flux_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                flux,
                (_npde_numflxh,),
                can_reference_contents=False,
            )
            _ires_numflxh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_numflxh = numflx(
                    _npde_numflxh,
                    _t_numflxh,
                    _x_numflxh,
                    _nv_numflxh,
                    _v_numflxh,
                    _uleft_numflxh,
                    _uright_numflxh,
                    _flux_numflxh,
                    _ires_numflxh,
                    data,
                )
            else:
                _ires_numflxh = numflx(
                    _npde_numflxh,
                    _t_numflxh,
                    _x_numflxh,
                    _nv_numflxh,
                    _v_numflxh,
                    _uleft_numflxh,
                    _uright_numflxh,
                    _flux_numflxh,
                    _ires_numflxh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(uleft)
            utils._unlink_pointer(uright)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _flux_numflxh, {'fun_name': cb_numflx_name, 'returned': 'flux'}, flux,
            )
            utils._unlink_pointer(flux)
            utils._EngineIntScalarType(
                _ires_numflxh, {'fun_name': cb_numflx_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, g[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, g, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_bndaryh = utils._EngineIntScalarType.to_py(npts.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_bndaryh,),
            )
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh, _npts_bndaryh),
                sorder=utils._EngineData.default_sorder,
            )
            _nv_bndaryh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_bndaryh,),
            )
            _vdot_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_bndaryh,),
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _g_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                g,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _npts_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _u_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _g_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _npts_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _u_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _g_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_odedef_name = fun_name + ':odedef'

    odedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, rcp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpt[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucptx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, r[nv].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_odedefh(_odedef, en_data, npde, t, nv, v, vdot, nxi, xi, ucp, ucpx, rcp, ucpt, ucptx, r, ires, _iuser, _ruser):
        """Helper for odedef."""
        if odedef is None:
            return
        try:
            _npde_odedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_odedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nv_odedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_odedefh,),
            )
            _vdot_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_odedefh,),
            )
            _nxi_odedefh = utils._EngineIntScalarType.to_py(nxi.contents)
            _xi_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_odedefh,),
            )
            _ucp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpt_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpt,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _r_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_nv_odedefh,),
                can_reference_contents=False,
            )
            _ires_odedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                    data,
                )
            else:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._unlink_pointer(xi)
            utils._unlink_pointer(ucp)
            utils._unlink_pointer(ucpx)
            utils._unlink_pointer(ucpt)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_odedefh, {'fun_name': cb_odedef_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_odedefh, {'fun_name': cb_odedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03plft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        numflxh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[neqn].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._BYREF_INT, # Input, nv.
        utils._UNUSED_FUNCTYPE,
        odedefh,
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._BYREF_INT, # Input, neqn.
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, laopt(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, algopt[30].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Output, lrsave_min.
        utils._BYREF_INT, # Output, lisave_min.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of laopt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'}, allow_none=True)
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(numflx, {'fun_name': fun_name, 'entity': 'numflx'})
    _numflx = utils._UNUSED_CALLBACK
    _numflxh = utils._cast_callback(
        numflxh, cb_numflxh,
        {'fun_name': fun_name, 'entity': 'numflx'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    _neqn = utils._EngineIntScalarType(
        neqn, {'fun_name': fun_name, 'entity': 'neqn'},
    ).to_ctypes()
    _neqn_py = utils._EngineIntScalarType.to_py(_neqn)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_neqn_py,),
    )
    _u = _u_cls.to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    utils._check_callable(odedef, {'fun_name': fun_name, 'entity': 'odedef'}, allow_none=True)
    _odedef = utils._UNUSED_CALLBACK
    _odedefh = utils._cast_callback(
        odedefh, cb_odedefh,
        {'fun_name': fun_name, 'entity': 'odedef'},
    )
    _nxi = utils._EngineIntScalarType(
        nxi, {'fun_name': fun_name, 'entity': 'nxi'},
    ).to_ctypes()
    _nxi_py = utils._EngineIntScalarType.to_py(_nxi)
    _xi = utils._EngineFloat64ArrayType(
        xi, {'fun_name': fun_name, 'entity': 'xi'},
        exp_shape=(_nxi_py,),
    ).to_ctypes()
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    elif _itol_py in (3, 4):
        _rtol_dim1 = _neqn_py
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    elif _itol_py in (2, 4):
        _atol_dim1 = _neqn_py
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _laopt = utils._EngineCharScalarType(
        laopt, {'fun_name': fun_name, 'entity': 'laopt'}, exp_chrlen=1,
    ).to_ctypes()
    _algopt = utils._EngineFloat64ArrayType(
        algopt, {'fun_name': fun_name, 'entity': 'algopt'},
        exp_shape=(30,),
    ).to_ctypes()
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _comm_needs_init = (
        utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
        ) or
        _ind_py == 0
    )
    if _comm_needs_init:
        if lrsave is None:
            _lrsave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lrsave = utils._EngineIntScalarType(
                lrsave, {'fun_name': fun_name, 'entity': 'lrsave'},
            ).to_ctypes()
        _lrsave_py = utils._EngineIntScalarType.to_py(_lrsave)
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
    else:
        _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = comm['rsave']
    if _comm_needs_init:
        if lisave is None:
            _lisave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lisave = utils._EngineIntScalarType(
                lisave, {'fun_name': fun_name, 'entity': 'lisave'},
            ).to_ctypes()
        _lisave_py = utils._EngineIntScalarType.to_py(_lisave)
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
    else:
        _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = comm['isave']
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iemode_py = 1 if _comm_needs_init else 0
    _iemode = utils._EngineIntScalarType(
        _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
    ).to_ctypes()
    _lrsave_min = utils._EngineIntScalarType.empty_ctypes()
    _lisave_min = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _numflx,
        _numflxh,
        _bndary,
        _bndaryh,
        _u,
        _npts,
        _x,
        _nv,
        _odedef,
        _odedefh,
        _nxi,
        _xi,
        _neqn,
        _rtol,
        _atol,
        _itol,
        _norm,
        _laopt,
        _algopt,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iemode,
        _lrsave_min,
        _lisave_min,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _norm._length_,
        _laopt._length_,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS and _iemode_py == 1:
        _iemode_py = 0
        _iemode = utils._EngineIntScalarType(
            _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
        ).to_ctypes()
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lrsave_py = max(_lrsave_min_py, _lrsave_py)
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        _lisave_py = max(_lisave_min_py, _lisave_py)
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
        engine_ctypes(
            _en_data,
            utils._UNUSED_PRINT_REC,
            _print_rech,
            _npde,
            _ts,
            _tout,
            _pdedef,
            _pdedefh,
            _numflx,
            _numflxh,
            _bndary,
            _bndaryh,
            _u,
            _npts,
            _x,
            _nv,
            _odedef,
            _odedefh,
            _nxi,
            _xi,
            _neqn,
            _rtol,
            _atol,
            _itol,
            _norm,
            _laopt,
            _algopt,
            _rsave,
            _lrsave,
            _isave,
            _lisave,
            _itask,
            _itrace,
            _nadv,
            _ind,
            _iemode,
            _lrsave_min,
            _lisave_min,
            _iuser,
            _ruser,
            _cwsav,
            _lwsav,
            _iwsav,
            _rwsav,
            _errbuf,
            _ierr,
            _norm._length_,
            _laopt._length_,
            80,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        if _ierr_py == 1 and _ifmt == 99964:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrsave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lisave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lisave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one point in xi lies outside [x[0],x[npts-1]]:',
                    'x[0] = {:s} and x[npts-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xi[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xi[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xi[i] > xi[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: corresponding elements atol[i-1] and rtol[j-1]',
                    'cannot both be 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and rtol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and atol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqn = {:s}, npde = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npts = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: neqn = npde*npts+nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi = 0 when nv = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi >= 0 when nv > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laopt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: laopt = \'F\', \'B\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'1\' or \'2\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99978:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of atol and rtol.',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99977:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef, numflx,',
                    'bndary or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99974:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'functions pdedef, numflx, bndary or odedef.',
                    'Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'atol and rtol were too small to start integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in a call to functions pdedef,',
                    'numflx, bndary or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99970:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in atol or rtol are unlikely',
                    'to result in a changed solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In solving ODE system, the maximum number of steps',
                    'algopt[14] has been exceeded.',
                    'algopt[14] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Zero error weights encountered during time integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The functions P, D, or C appear to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99965:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'When using the sparse option lisave or lrsave is too small:',
                    'lisave = {:s}, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind', 'lrsave_min', 'lisave_min'],
        )(
            _ts_py,
            _ind_py,
            _lrsave_min_py,
            _lisave_min_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_remesh_fd(npde, m, ts, tout, pdedef, bndary, uvinit, u, npts, x, nv, odedef, nxi, xi, neqn, rtol, atol, itol, norm, laopt, algopt, remesh, nxfix, xfix, nrmesh, dxmesh, trmesh, ipminf, xratio, con, monitf, comm, lrsave, lisave, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of parabolic PDEs, coupled DAEs, method of lines,
    finite differences, remeshing, one space variable.

    For full information please refer to the NAG Library document for
    d03pp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03ppf.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    m : int
        The coordinate system used:

        `m` = 0

          Indicates Cartesian coordinates.

        `m` = 1

          Indicates cylindrical polar coordinates.

        `m` = 2

          Indicates spherical polar coordinates.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, u, ux, nv, v, vdot, p,
        q, r, ires, data=None)
        `pdedef` must evaluate the functions P_{i,j}, Q_i and R_i which
        define the system of PDEs.

        The functions may depend on x, t, U, U_x and V. Q_i may depend
        linearly on V^.. `pdedef` is called approximately midway between
        each pair of mesh points in turn by ``dim1_parab_remesh_fd``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in Q_j, for j = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`), to be modified in
        place
            On exit: `p`[i-1,j-1] must be set to the value of
            P_{i,j}(x,t,U,U_x,V), for j = 1,2,...,`npde`, for i =
            1,2,...,`npde`.

        q : float, ndarray, shape (`npde`), to be modified in place
            On exit: `q`[i-1] must be set to the value of
            Q_i(x,t,U,U_x,V,V^.), for i = 1,2,...,`npde`.

        r : float, ndarray, shape (`npde`), to be modified in place
            On exit: `r`[i-1] must be set to the value of
            R_i(x,t,U,U_x,V), for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_remesh_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    bndary : callable ires = bndary(npde, t, u, ux, nv, v, vdot, ibnd,
        beta, gamma, ires, data=None)
        `bndary` must evaluate the functions beta_i and gamma_i which
        describe the boundary conditions, as given in [equation].

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            `vdot`[i-1] contains the value of component V^._i(t), for i
            = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in gamma_j, for j = 1,2,...,`npde`.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must set up the coefficients of the left-hand
              boundary, x = a.

            `ibnd` != 0

              `bndary` must set up the coefficients of the right-hand
              boundary, x = b.

        beta : float, ndarray, shape (`npde`), to be modified in place
            On exit: `beta`[i-1] must be set to the value of beta_i(x,t)
            at the boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        gamma : float, ndarray, shape (`npde`), to be modified in place
            On exit: `gamma`[i-1] must be set to the value of
            gamma_i(x,t,U,U_x,V,V^.) at the boundary specified by
            `ibnd`, for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_remesh_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    uvinit : callable uvinit(npde, npts, nxi, x, xi, u, nv, v,
        data=None)
        `uvinit` must supply the initial (t = t_0) values of U(x,t) and
        V(t) for all values of x in the interval a <= x <= b.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        nxi : int
            The number of ODE/PDE coupling points.

        x : float, ndarray, shape (`npts`)
            The current mesh. `x`[i-1] contains the value of x_i, for i
            = 1,2,...,`npts`.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the value of the ODE/PDE
            coupling point, xi_i, for i = 1,2,...,`nxi`.

        u : float, ndarray, shape (`npde`, `npts`), to be modified in
        place
            On exit: `u`[i-1,j-1] contains the value of the component
            U_i(x_j,t_0), for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`), to be modified in place
            On exit: `v`[i-1] contains the value of component V_i(t_0),
            for i = 1,2,...,`nv`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    u : float, shape (`neqn`), modified in place
        On entry: if `ind` = 1 the value of `u` must be unchanged from
        the previous call.

        On exit: the computed solution U_i(x_j,t), for j =
        1,2,...,`npts`, for i = 1,2,...,`npde`, and V_k(t), for k =
        1,2,...,`nv`, evaluated at t = `ts`, as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t), for i = 1,2,...,`nv`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`), modified in place
        On entry: the initial mesh points in the space direction. `x`[0]
        must specify the left-hand boundary, a, and `x`[`npts`-1] must
        specify the right-hand boundary, b.

        On exit: the final values of the mesh points.

    nv : int
        The number of coupled ODE in the system.

    odedef : None or callable ires = odedef(npde, t, nv, v, vdot, nxi,
        xi, ucp, ucpx, rcp, ucpt, ucptx, f, ires, data=None)
        `odedef` must evaluate the functions F, which define the system
        of ODEs, as given in (3).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        nxi : int
            The number of ODE/PDE coupling points.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling
            points, xi_i, for i = 1,2,...,`nxi`.

        ucp : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucp`[i-1,j-1] contains the value of U_i(x,t)
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucpx : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpx`[i-1,j-1] contains the value of
            dU_i(x,t)/dx at the coupling point x = xi_j, for j =
            1,2,...,`nxi`, for i = 1,2,...,`npde`.

        rcp : float, ndarray, shape (`npde`, `nxi`)
            `rcp`[i-1,j-1] contains the value of the flux R_i at the
            coupling point x = xi_j, for j = 1,2,...,`nxi`, for i =
            1,2,...,`npde`.

        ucpt : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpt`[i-1,j-1] contains the value of dU_i/dt
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucptx : float, ndarray, shape (`npde`, `nxi`)
            `ucptx`[i-1,j-1] contains the value of d^2U_i/dxdt at the
            coupling point x = xi_j, for j = 1,2,...,`nxi`, for i =
            1,2,...,`npde`.

        f : float, ndarray, shape (`nv`), to be modified in place
            On exit, `f`[i-1] must contain the ith component of F as
            specified by the input value of `ires`.

        ires : int
            The form of F that must be returned in the array `f`.

            `ires` = 1

              Equation (5) must be used.

            `ires` = -1

              Equation (6) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may reset
            `ires` to force the integration function to take certain
            actions as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3, ``dim1_parab_remesh_fd``
              returns to the calling function with the error indicator
              set to `errno` = 4.

    nxi : int
        The number of ODE/PDE coupling points.

    xi : float, shape (`nxi`)
        If `nxi` > 0, `xi`[i-1], for i = 1,2,...,`nxi`, must be set to
        the ODE/PDE coupling points.

    neqn : int
        The number of ODEs in the time direction.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; if `itol` in (3, 4): `neqn`;
        otherwise: 0.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; if `itol` in (2, 4): `neqn`;
        otherwise: 0.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``dim1_parab_remesh_fd`` whether to interpret
        either or both of `rtol` or `atol` as a vector or scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        In the above, e_i denotes the estimated local error for the ith
        component of the coupled PDE/ODE system in time, `u`[i-1], for i
        = 1,2,...,`neqn`.

        The choice of norm used is defined by the argument `norm`.

    norm : str, length 1
        The type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L_2 norm.

        If `u`_norm denotes the norm of the vector `u` of length `neqn`,
        then for the averaged L_2 norm

            `u`_norm = sqrt(1/`neqn`sum_i = 1^`neqn`(`u`[i-1]/w_i)^2),

        while for the maximum norm

            `u`_norm = max_i(|`u`[i-1] / w_i|).

        See the description of `itol` for the formulation of the weight
        vector w.

    laopt : str, length 1
        The type of matrix algebra required.

        `laopt` = 'F'

          Full matrix methods to be used.

        `laopt` = 'B'

          Banded matrix methods to be used.

        `laopt` = 'S'

          Sparse matrix methods to be used.

    algopt : float, shape (30)
        May be set to control various options available in the
        integrator. If you wish to employ all the default options,
        `algopt`[0] should be set to 0.0. Default values will also be
        used for any other elements of `algopt` set to zero. The
        permissible values, default values, and meanings are as follows:

        `algopt`[0]

          Selects the ODE integration method to be used. If `algopt`[0]
          = 1.0, a BDF method is used and if `algopt`[0] = 2.0, a Theta
          method is used. The default value is `algopt`[0] = 1.0.

        If `algopt`[0] = 2.0, `algopt`[i-1], for i = 2,3,...,4 are not
        used.

        `algopt`[1]

          Specifies the maximum order of the BDF integration formula to
          be used. `algopt`[1] may be 1.0, 2.0, 3.0, 4.0 or 5.0. The
          default value is `algopt`[1] = 5.0.

        `algopt`[2]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the BDF method. If
          `algopt`[2] = 1.0 a modified Newton iteration is used and if
          `algopt`[2] = 2.0 a functional iteration method is used. If
          functional iteration is selected and the integrator encounters
          difficulty, there is an automatic switch to the modified
          Newton iteration. The default value is `algopt`[2] = 1.0.

        `algopt`[3]

          Specifies whether or not the Petzold error test is to be
          employed. The Petzold error test results in extra overhead but
          is more suitable when algebraic equations are present, such as
          P_{i,j} = 0.0, for j = 1,2,...,`npde`, for some i or when
          there is no V^._i(t) dependence in the coupled ODE system. If
          `algopt`[3] = 1.0, the Petzold test is used. If `algopt`[3] =
          2.0, the Petzold test is not used. The default value is
          `algopt`[3] = 1.0.

        If `algopt`[0] = 1.0, `algopt`[i-1], for i = 5,6,...,7, are not
        used.

        `algopt`[4]

          Specifies the value of Theta to be used in the Theta
          integration method. 0.51 <= `algopt`[4] <= 0.99. The default
          value is `algopt`[4] = 0.55.

        `algopt`[5]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the Theta method.
          If `algopt`[5] = 1.0, a modified Newton iteration is used and
          if `algopt`[5] = 2.0, a functional iteration method is used.
          The default value is `algopt`[5] = 1.0.

        `algopt`[6]

          Specifies whether or not the integrator is allowed to switch
          automatically between modified Newton and functional iteration
          methods in order to be more efficient. If `algopt`[6] = 1.0,
          switching is allowed and if `algopt`[6] = 2.0, switching is
          not allowed. The default value is `algopt`[6] = 1.0.

        `algopt`[10]

          Specifies a point in the time direction, t_crit, beyond which
          integration must not be attempted. The use of t_crit is
          described under the argument `itask`. If `algopt`[0] != 0.0, a
          value of 0.0 for `algopt`[10], say, should be specified even
          if `itask` subsequently specifies that t_crit will not be
          used.

        `algopt`[11]

          Specifies the minimum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[11]
          should be set to 0.0.

        `algopt`[12]

          Specifies the maximum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[12]
          should be set to 0.0.

        `algopt`[13]

          Specifies the initial step size to be attempted by the
          integrator. If `algopt`[13] = 0.0, the initial step size is
          calculated internally.

        `algopt`[14]

          Specifies the maximum number of steps to be attempted by the
          integrator in any one call. If `algopt`[14] = 0.0, no limit is
          imposed.

        `algopt`[22]

          Specifies what method is to be used to solve the nonlinear
          equations at the initial point to initialize the values of U,
          U_t, V and V^.. If `algopt`[22] = 1.0, a modified Newton
          iteration is used and if `algopt`[22] = 2.0, functional
          iteration is used. The default value is `algopt`[22] = 1.0.

        `algopt`[28] and `algopt`[29] are used only for the sparse
        matrix algebra option, `laopt` = 'S'.

        `algopt`[28]

          Governs the choice of pivots during the decomposition of the
          first Jacobian matrix. It should lie in the range 0.0 <
          `algopt`[28] < 1.0, with smaller values biasing the algorithm
          towards maintaining sparsity at the expense of numerical
          stability. If `algopt`[28] lies outside this range then the
          default value is used. If the functions regard the Jacobian
          matrix as numerically singular then increasing `algopt`[28]
          towards 1.0 may help, but at the cost of increased fill-in.
          The default value is `algopt`[28] = 0.1.

        `algopt`[29]

          Is used as a relative pivot threshold during subsequent
          Jacobian decompositions (see `algopt`[28]) below which an
          internal error is invoked. If `algopt`[29] is greater than 1.0
          no check is made on the pivot size, and this may be a
          necessary option if the Jacobian is found to be numerically
          singular (see `algopt`[28]). The default value is `algopt`[29]
          = 0.0001.

    remesh : bool
        Indicates whether or not spatial remeshing should be performed.

        `remesh` = True

          Indicates that spatial remeshing should be performed as
          specified.

        `remesh` = False

          Indicates that spatial remeshing should be suppressed.

        Note: `remesh` should not be changed between consecutive calls
        to ``dim1_parab_remesh_fd``. Remeshing can be switched off or on
        at specified times by using appropriate values for the arguments
        `nrmesh` and `trmesh` at each call.

    nxfix : int
        The number of fixed mesh points.

    xfix : float, shape (`nxfix`)
        `xfix`[i-1], for i = 1,2,...,`nxfix`, must contain the value of
        the x coordinate at the ith fixed mesh point.

    nrmesh : int
        Specifies the spatial remeshing frequency and criteria for the
        calculation and adoption of a new mesh.

        `nrmesh` < 0

          Indicates that a new mesh is adopted according to the argument
          `dxmesh`. The mesh is tested every |`nrmesh`| timesteps.

        `nrmesh` = 0

          Indicates that remeshing should take place just once at the
          end of the first time step reached when t > `trmesh`.

        `nrmesh` > 0

          Indicates that remeshing will take place every `nrmesh` time
          steps, with no testing using `dxmesh`.

        Note: `nrmesh` may be changed between consecutive calls to
        ``dim1_parab_remesh_fd`` to give greater flexibility over the
        times of remeshing.

    dxmesh : float
        Determines whether a new mesh is adopted when `nrmesh` is set
        less than zero. A possible new mesh is calculated at the end of
        every |`nrmesh`| time steps, but is adopted only if

            x_i^(new) > x_i^(old)+`dxmesh`*(x_i+1^(old)-x_i^(old))

        or

            x_i^(new) < x_i^(old)-`dxmesh`*(x_i^(old)-x_i-1^(old))

        `dxmesh` thus imposes a lower limit on the difference between
        one mesh and the next.

    trmesh : float
        Specifies when remeshing will take place when `nrmesh` is set to
        zero. Remeshing will occur just once at the end of the first
        time step reached when t is greater than `trmesh`.

        Note: `trmesh` may be changed between consecutive calls to
        ``dim1_parab_remesh_fd`` to force remeshing at several specified
        times.

    ipminf : int
        The level of trace information regarding the adaptive remeshing.
        Details are directed to the file object associated with the
        advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

        `ipminf` = 0

          No trace information.

        `ipminf` = 1

          Brief summary of mesh characteristics.

        `ipminf` = 2

          More detailed information, including old and new mesh points,
          mesh sizes and monitor function values.

    xratio : float
        An input bound on the adjacent mesh ratio (greater than 1.0 and
        typically in the range 1.5 to 3.0). The remeshing functions will
        attempt to ensure that

            (x_i-x_{i-1})/`xratio` < x_{i+1}-x_i <
            `xratio`*(x_i-x_{i-1}).

    con : float
        An input bound on the sub-integral of the monitor function
        F^mon(x) over each space step. The remeshing functions will
        attempt to ensure that

            int_x_i^x_{i+1}F^mon(x)dx <=
            `con`int_x_1^x_`npts`F^mon(x)dx,

        . `con` gives you more control over the mesh distribution e.g.,
        decreasing `con` allows more clustering. A typical value is
        2/(`npts`-1), but you are encouraged to experiment with
        different values. Its value is not critical and the mesh should
        be qualitatively correct for all values in the range given
        below.

    monitf : None or callable monitf(t, npts, npde, x, u, r, fmon,
        data=None)
        `monitf` must supply and evaluate a remesh monitor function to
        indicate the solution behaviour of interest.

        Parameters
        ~~~~~~~~~~
        t : float
            The current value of the independent variable t.

        npts : int
            The number of mesh points in the interval [a,b].

        npde : int
            The number of PDEs in the system.

        x : float, ndarray, shape (`npts`)
            The current mesh. `x`[i-1] contains the value of x_i, for i
            = 1,2,...,`npts`.

        u : float, ndarray, shape (`npde`, `npts`)
            `u`[i-1,j-1] contains the value of U_i(x,t) at x = `x`[j-1]
            and time t, for j = 1,2,...,`npts`, for i = 1,2,...,`npde`.

        r : float, ndarray, shape (`npde`, `npts`)
            `r`[i-1,j-1] contains the value of R_i(x,t,U,U_x,V) at x =
            `x`[j-1] and time t, for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

        fmon : float, ndarray, shape (`npts`), to be modified in place
            On exit: `fmon`[i-1] must contain the value of the monitor
            function F^mon(x) at mesh point x = `x`[i-1].

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    comm : dict, communication object, modified in place
        Note: this argument will be (re-)initialized when it is an empty
        ``dict`` or under the following condition: `ind` = 0.

        Communication structure.

        On initial entry: need not be set.

    lrsave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['rsave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lrsave` is computed by the
        function.

        If your supplied `lrsave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lrsave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lrsave` set to at least the lower-bound value returned in
        `lrsave_min`.

    lisave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['isave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lisave` is computed by the
        function.

        If your supplied `lisave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lisave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lisave` set to at least the lower-bound value returned in
        `lisave_min`.

    itask : int
        Specifies the task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout`.

        `itask` = 2

          One step and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

        `itask` = 4

          Normal computation of output values `u` at t = `tout` but
          without overshooting t = t_crit where t_crit is described
          under the argument `algopt`.

        `itask` = 5

          Take one step in the time direction and return, without
          passing t_crit, where t_crit is described under the argument
          `algopt`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_remesh_fd`` and the underlying ODE solver:

        `itrace` <= -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` = 1

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        `itrace` = 2

          Output from the underlying ODE solver is similar to that
          produced when `itrace` = 1, except that the advisory messages
          are given in greater detail.

        `itrace` >= 3

          Output from the underlying ODE solver is similar to that
          produced when `itrace` = 2, except that the advisory messages
          are given in greater detail.

    ind : int
        Must be set to 0 or 1.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` and the
          remeshing arguments `nrmesh`, `dxmesh`, `trmesh`, `xratio` and
          `con` may be reset between calls to ``dim1_parab_remesh_fd``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    lrsave_min : int
        A lower bound on the sufficient size for `comm`['rsave'].

    lisave_min : int
        A lower bound on the sufficient size for `comm`['isave'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `lrsave` = *<value>*.

            Constraint: `lrsave` >= *<value>*.

        (`errno` 1)
            On entry, `lisave` = *<value>*.

            Constraint: `lisave` >= *<value>*.

        (`errno` 1)
            On entry, the point `xfix`[i-1] does not coincide with any
            `x`[j-1]: i = *<value>* and `xfix`[i-1] = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xfix`[i] = *<value>* and
            `xfix`[i-1] = *<value>*.

            Constraint: `xfix`[i] > `xfix`[i-1].

        (`errno` 1)
            On entry, at least one point in `xi` lies outside
            [`x`[0],`x`[`npts`-1]]: `x`[0] = *<value>* and `x`[`npts`-1]
            = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xi`[i] = *<value>* and `xi`[i-1] =
            *<value>*.

            Constraint: `xi`[i] > `xi`[i-1].

        (`errno` 1)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: corresponding elements `atol`[i-1] and
            `rtol`[j-1] cannot both be 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `rtol`[i-1] = *<value>*.

            Constraint: `rtol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `atol`[i-1] = *<value>*.

            Constraint: `atol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `neqn` = *<value>*, `npde` = *<value>*, `npts` =
            *<value>* and `nv` = *<value>*.

            Constraint: `neqn` = `npde`*`npts`+`nv`.

        (`errno` 1)
            On entry, `con` = *<value>*, `npts` = *<value>*.

            Constraint: `con` <= 10.0/(`npts`-1).

        (`errno` 1)
            On entry, `con` = *<value>*, `npts` = *<value>*.

            Constraint: `con` >= 0.1/(`npts`-1).

        (`errno` 1)
            On entry, `xratio` = *<value>*.

            Constraint: `xratio` > 1.0.

        (`errno` 1)
            On entry, `ipminf` = *<value>*.

            Constraint: `ipminf` = 0, 1 or 2.

        (`errno` 1)
            On entry, `dxmesh` = *<value>*.

            Constraint: `dxmesh` >= 0.0.

        (`errno` 1)
            On entry, `nxfix` = *<value>*, `npts` = *<value>*.

            Constraint: `nxfix` <= `npts`-2.

        (`errno` 1)
            On entry, `nxfix` = *<value>*.

            Constraint: `nxfix` >= 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` = 0 when `nv` = 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` >= 0 when `nv` > 0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `m` = *<value>* and `x`[0] = *<value>*.

            Constraint: `m` <= 0 or `x`[0] >= 0.0

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = 0, 1 or 2.

        (`errno` 1)
            On entry, `laopt` = *<value>*.

            Constraint: `laopt` = 'F', 'B' or 'S'.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = 'A' or 'M'.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2, 3, 4 or 5.

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 0.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `atol` and `rtol` were too small to start integration.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef`,
            `bndary`, or `odedef`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 12)
            In solving ODE system, the maximum number of steps
            `algopt`[14] has been exceeded. `algopt`[14] = *<value>*.

        (`errno` 14)
            Flux function appears to depend on time derivatives.

        (`errno` 16)
            `remesh` has been changed between calls to
            ``dim1_parab_remesh_fd``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `atol` and `rtol`.
            `ts` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef`, `bndary`, or `odedef`. Integration is
            successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `atol` or `rtol`
            are unlikely to result in a changed solution.

        (`errno` 13)
            Zero error weights encountered during time integration.

        (`errno` 15)
            When using the sparse option `lisave` or `lrsave` is too
            small: `lisave` = *<value>*, `lrsave` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_remesh_fd'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, r[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, q, r, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _nv_pdedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_pdedefh,),
            )
            _vdot_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_pdedefh,),
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _q_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                q,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _r_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _q_pdedefh,
                    _r_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ux_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_bndaryh,),
            )
            _nv_bndaryh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_bndaryh,),
            )
            _vdot_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_bndaryh,),
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _beta_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                beta,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _gamma_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                gamma,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _u_bndaryh,
                    _ux_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _beta_bndaryh,
                    _gamma_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_uvinit_name = fun_name + ':uvinit'

    uvinith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, u[npde, npts].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Output, v[nv].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_uvinith(_uvinit, en_data, npde, npts, nxi, x, xi, u, nv, v, _iuser, _ruser):
        """Helper for uvinit."""
        try:
            _npde_uvinith = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_uvinith = utils._EngineIntScalarType.to_py(npts.contents)
            _nxi_uvinith = utils._EngineIntScalarType.to_py(nxi.contents)
            _x_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_uvinith,),
            )
            _xi_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_uvinith,),
            )
            _u_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_uvinith, _npts_uvinith),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _nv_uvinith = utils._EngineIntScalarType.to_py(nv.contents)
            _v_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_uvinith,),
                can_reference_contents=False,
            )
            if data is not None:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _nxi_uvinith,
                    _x_uvinith,
                    _xi_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                    data,
                )
            else:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _nxi_uvinith,
                    _x_uvinith,
                    _xi_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(xi)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _u_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'u'}, u,
            )
            utils._unlink_pointer(u)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _v_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'v'}, v,
            )
            utils._unlink_pointer(v)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_odedef_name = fun_name + ':odedef'

    odedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, rcp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpt[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucptx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, f[nv].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_odedefh(_odedef, en_data, npde, t, nv, v, vdot, nxi, xi, ucp, ucpx, rcp, ucpt, ucptx, f, ires, _iuser, _ruser):
        """Helper for odedef."""
        if odedef is None:
            return
        try:
            _npde_odedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_odedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nv_odedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_odedefh,),
            )
            _vdot_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_odedefh,),
            )
            _nxi_odedefh = utils._EngineIntScalarType.to_py(nxi.contents)
            _xi_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_odedefh,),
            )
            _ucp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _rcp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                rcp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpt_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpt,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucptx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucptx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _f_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_nv_odedefh,),
                can_reference_contents=False,
            )
            _ires_odedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _rcp_odedefh,
                    _ucpt_odedefh,
                    _ucptx_odedefh,
                    _f_odedefh,
                    _ires_odedefh,
                    data,
                )
            else:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _rcp_odedefh,
                    _ucpt_odedefh,
                    _ucptx_odedefh,
                    _f_odedefh,
                    _ires_odedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._unlink_pointer(xi)
            utils._unlink_pointer(ucp)
            utils._unlink_pointer(ucpx)
            utils._unlink_pointer(rcp)
            utils._unlink_pointer(ucpt)
            utils._unlink_pointer(ucptx)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_odedefh, {'fun_name': cb_odedef_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._EngineIntScalarType(
                _ires_odedefh, {'fun_name': cb_odedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitf_name = fun_name + ':monitf'

    monitfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, npts].
        utils._BYREF_INT, # Input, nr.
        utils._EngineFloat64ArrayType.c_type, # Input, r[npde, npts].
        utils._EngineFloat64ArrayType.c_type, # Output, fmon[npts].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monitfh(_monitf, en_data, t, npts, npde, x, u, nr, r, fmon, _iuser, _ruser):
        """Helper for monitf."""
        if monitf is None:
            return
        try:
            _t_monitfh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _npts_monitfh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_monitfh = utils._EngineIntScalarType.to_py(npde.contents)
            _x_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_monitfh,),
            )
            _u_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_monitfh, _npts_monitfh),
                sorder=utils._EngineData.default_sorder,
            )
            _r_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_npde_monitfh, _npts_monitfh),
                sorder=utils._EngineData.default_sorder,
            )
            _fmon_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                fmon,
                (_npts_monitfh,),
                can_reference_contents=False,
            )
            if data is not None:
                monitf(
                    _t_monitfh,
                    _npts_monitfh,
                    _npde_monitfh,
                    _x_monitfh,
                    _u_monitfh,
                    _r_monitfh,
                    _fmon_monitfh,
                    data,
                )
            else:
                monitf(
                    _t_monitfh,
                    _npts_monitfh,
                    _npde_monitfh,
                    _x_monitfh,
                    _u_monitfh,
                    _r_monitfh,
                    _fmon_monitfh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(u)
            utils._unlink_pointer(r)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fmon_monitfh, {'fun_name': cb_monitf_name, 'returned': 'fmon'}, fmon,
            )
            utils._unlink_pointer(fmon)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03ppft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._UNUSED_FUNCTYPE,
        uvinith,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[neqn].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[npts].
        utils._BYREF_INT, # Input, nv.
        utils._UNUSED_FUNCTYPE,
        odedefh,
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._BYREF_INT, # Input, neqn.
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, laopt(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, algopt[30].
        utils._BYREF_LOGICAL, # Input, remesh.
        utils._BYREF_INT, # Input, nxfix.
        utils._EngineFloat64ArrayType.c_type, # Input, xfix[nxfix].
        utils._BYREF_INT, # Input, nrmesh.
        utils._BYREF_FLOAT64, # Input, dxmesh.
        utils._BYREF_FLOAT64, # Input, trmesh.
        utils._BYREF_INT, # Input, ipminf.
        utils._BYREF_FLOAT64, # Input, xratio.
        utils._BYREF_FLOAT64, # Input, con.
        utils._UNUSED_FUNCTYPE,
        monitfh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Output, lrsave_min.
        utils._BYREF_INT, # Output, lisave_min.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of laopt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    utils._check_callable(uvinit, {'fun_name': fun_name, 'entity': 'uvinit'})
    _uvinit = utils._UNUSED_CALLBACK
    _uvinith = utils._cast_callback(
        uvinith, cb_uvinith,
        {'fun_name': fun_name, 'entity': 'uvinit'},
    )
    _neqn = utils._EngineIntScalarType(
        neqn, {'fun_name': fun_name, 'entity': 'neqn'},
    ).to_ctypes()
    _neqn_py = utils._EngineIntScalarType.to_py(_neqn)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_neqn_py,),
    )
    _u = _u_cls.to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    )
    _x = _x_cls.to_ctypes()
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    utils._check_callable(odedef, {'fun_name': fun_name, 'entity': 'odedef'}, allow_none=True)
    _odedef = utils._UNUSED_CALLBACK
    _odedefh = utils._cast_callback(
        odedefh, cb_odedefh,
        {'fun_name': fun_name, 'entity': 'odedef'},
    )
    _nxi = utils._EngineIntScalarType(
        nxi, {'fun_name': fun_name, 'entity': 'nxi'},
    ).to_ctypes()
    _nxi_py = utils._EngineIntScalarType.to_py(_nxi)
    _xi = utils._EngineFloat64ArrayType(
        xi, {'fun_name': fun_name, 'entity': 'xi'},
        exp_shape=(_nxi_py,),
    ).to_ctypes()
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    elif _itol_py in (3, 4):
        _rtol_dim1 = _neqn_py
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    elif _itol_py in (2, 4):
        _atol_dim1 = _neqn_py
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _laopt = utils._EngineCharScalarType(
        laopt, {'fun_name': fun_name, 'entity': 'laopt'}, exp_chrlen=1,
    ).to_ctypes()
    _algopt = utils._EngineFloat64ArrayType(
        algopt, {'fun_name': fun_name, 'entity': 'algopt'},
        exp_shape=(30,),
    ).to_ctypes()
    _remesh = utils._EngineLogicalScalarType(
        remesh, {'fun_name': fun_name, 'entity': 'remesh'},
    ).to_ctypes()
    _nxfix = utils._EngineIntScalarType(
        nxfix, {'fun_name': fun_name, 'entity': 'nxfix'},
    ).to_ctypes()
    _nxfix_py = utils._EngineIntScalarType.to_py(_nxfix)
    _xfix = utils._EngineFloat64ArrayType(
        xfix, {'fun_name': fun_name, 'entity': 'xfix'},
        exp_shape=(_nxfix_py,),
    ).to_ctypes()
    _nrmesh = utils._EngineIntScalarType(
        nrmesh, {'fun_name': fun_name, 'entity': 'nrmesh'},
    ).to_ctypes()
    _dxmesh = utils._EngineFloat64ScalarType(
        dxmesh, {'fun_name': fun_name, 'entity': 'dxmesh'},
    ).to_ctypes()
    _trmesh = utils._EngineFloat64ScalarType(
        trmesh, {'fun_name': fun_name, 'entity': 'trmesh'},
    ).to_ctypes()
    _ipminf = utils._EngineIntScalarType(
        ipminf, {'fun_name': fun_name, 'entity': 'ipminf'},
    ).to_ctypes()
    _xratio = utils._EngineFloat64ScalarType(
        xratio, {'fun_name': fun_name, 'entity': 'xratio'},
    ).to_ctypes()
    _con = utils._EngineFloat64ScalarType(
        con, {'fun_name': fun_name, 'entity': 'con'},
    ).to_ctypes()
    utils._check_callable(monitf, {'fun_name': fun_name, 'entity': 'monitf'}, allow_none=True)
    _monitf = utils._UNUSED_CALLBACK
    _monitfh = utils._cast_callback(
        monitfh, cb_monitfh,
        {'fun_name': fun_name, 'entity': 'monitf'},
    )
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _comm_needs_init = (
        utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
        ) or
        _ind_py == 0
    )
    if _comm_needs_init:
        if lrsave is None:
            _lrsave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lrsave = utils._EngineIntScalarType(
                lrsave, {'fun_name': fun_name, 'entity': 'lrsave'},
            ).to_ctypes()
        _lrsave_py = utils._EngineIntScalarType.to_py(_lrsave)
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
    else:
        _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = comm['rsave']
    if _comm_needs_init:
        if lisave is None:
            _lisave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lisave = utils._EngineIntScalarType(
                lisave, {'fun_name': fun_name, 'entity': 'lisave'},
            ).to_ctypes()
        _lisave_py = utils._EngineIntScalarType.to_py(_lisave)
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
    else:
        _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = comm['isave']
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iemode_py = 1 if _comm_needs_init else 0
    _iemode = utils._EngineIntScalarType(
        _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
    ).to_ctypes()
    _lrsave_min = utils._EngineIntScalarType.empty_ctypes()
    _lisave_min = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _m,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _uvinit,
        _uvinith,
        _u,
        _npts,
        _x,
        _nv,
        _odedef,
        _odedefh,
        _nxi,
        _xi,
        _neqn,
        _rtol,
        _atol,
        _itol,
        _norm,
        _laopt,
        _algopt,
        _remesh,
        _nxfix,
        _xfix,
        _nrmesh,
        _dxmesh,
        _trmesh,
        _ipminf,
        _xratio,
        _con,
        _monitf,
        _monitfh,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iemode,
        _lrsave_min,
        _lisave_min,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _norm._length_,
        _laopt._length_,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS and _iemode_py == 1:
        _iemode_py = 0
        _iemode = utils._EngineIntScalarType(
            _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
        ).to_ctypes()
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lrsave_py = max(_lrsave_min_py, _lrsave_py)
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        _lisave_py = max(_lisave_min_py, _lisave_py)
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
        engine_ctypes(
            _en_data,
            utils._UNUSED_PRINT_REC,
            _print_rech,
            _npde,
            _m,
            _ts,
            _tout,
            _pdedef,
            _pdedefh,
            _bndary,
            _bndaryh,
            _uvinit,
            _uvinith,
            _u,
            _npts,
            _x,
            _nv,
            _odedef,
            _odedefh,
            _nxi,
            _xi,
            _neqn,
            _rtol,
            _atol,
            _itol,
            _norm,
            _laopt,
            _algopt,
            _remesh,
            _nxfix,
            _xfix,
            _nrmesh,
            _dxmesh,
            _trmesh,
            _ipminf,
            _xratio,
            _con,
            _monitf,
            _monitfh,
            _rsave,
            _lrsave,
            _isave,
            _lisave,
            _itask,
            _itrace,
            _nadv,
            _ind,
            _iemode,
            _lrsave_min,
            _lisave_min,
            _iuser,
            _ruser,
            _cwsav,
            _lwsav,
            _iwsav,
            _rwsav,
            _errbuf,
            _ierr,
            _norm._length_,
            _laopt._length_,
            80,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        if _ierr_py == 1 and _ifmt == 99952:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrsave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lisave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lisave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the point xfix[i-1] does not coincide with any x[j-1]:',
                    'i = {:s} and xfix[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xfix[i] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and xfix[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xfix[i] > xfix[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one point in xi lies outside [x[0],x[npts-1]]:',
                    'x[0] = {:s} and x[npts-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xi[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xi[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xi[i] > xi[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: corresponding elements atol[i-1] and rtol[j-1]',
                    'cannot both be 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and rtol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and atol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqn = {:s}, npde = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npts = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: neqn = npde*npts+nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con <= 10.0/(npts-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con >= 0.1/(npts-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xratio = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xratio > 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ipminf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ipminf = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dxmesh = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dxmesh >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxfix = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nxfix <= npts-2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxfix = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxfix >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi = 0 when nv = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi >= 0 when nv > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and x[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= 0 or x[0] >= 0.0',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laopt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: laopt = \'F\', \'B\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'A\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99966:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of atol and rtol.',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99965:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99964:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99962:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef, bndary, or odedef. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'atol and rtol were too small to start integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef, bndary, or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99959:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99958:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in atol or rtol are unlikely',
                    'to result in a changed solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99957:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99956:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In solving ODE system, the maximum number of steps',
                    'algopt[14] has been exceeded.',
                    'algopt[14] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99955:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Zero error weights encountered during time integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99954:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Flux function appears to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99953:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'When using the sparse option lisave or lrsave is too small:',
                    'lisave = {:s}, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 99967:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'remesh has been changed between calls to dim1_parab_remesh_fd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _x_cls.refresh_py(_x)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind', 'lrsave_min', 'lisave_min'],
        )(
            _ts_py,
            _ind_py,
            _lrsave_min_py,
            _lisave_min_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_remesh_keller(npde, ts, tout, pdedef, bndary, uvinit, u, npts, x, nleft, nv, odedef, nxi, xi, neqn, rtol, atol, itol, norm, laopt, algopt, remesh, nxfix, xfix, nrmesh, dxmesh, trmesh, ipminf, xratio, con, monitf, comm, lrsave, lisave, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of first-order PDEs, coupled DAEs, method of lines,
    Keller box discretization, remeshing, one space variable.

    For full information please refer to the NAG Library document for
    d03pr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03prf.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : callable ires = pdedef(npde, t, x, u, ut, ux, nv, v, vdot,
        res, ires, data=None)
        `pdedef` must evaluate the functions G_i which define the system
        of PDEs. `pdedef` is called approximately midway between each
        pair of mesh points in turn by ``dim1_parab_remesh_keller``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ut : float, ndarray, shape (`npde`)
            `ut`[i-1] contains the value of the component dU_i(x,t)/dt,
            for i = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        res : float, ndarray, shape (`npde`), to be modified in place
            On exit: `res`[i-1] must contain the ith component of G, for
            i = 1,2,...,`npde`, where G is defined as

                G_i = sum_j = 1^`npde`P_{i,j}dU_j/dt+sum_j =
                1^`nv`Q_{i,j}V^._j,

            i.e., only terms depending explicitly on time derivatives,
            or

                G_i = sum_j = 1^`npde`P_{i,j}dU_j/dt+sum_j =
                1^`nv`Q_{i,j}V^._j+S_i,

            i.e., all terms in equation (3).

            The definition of G is determined by the input value of
            `ires`.

        ires : int
            The form of G_i that must be returned in the array `res`.

            `ires` = -1

              Equation (9) must be used.

            `ires` = 1

              Equation (0) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions,
            as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_remesh_keller`` returns to the calling
              function with the error indicator set to `errno` = 4.

    bndary : callable ires = bndary(npde, t, ibnd, nobc, u, ut, nv, v,
        vdot, res, ires, data=None)
        `bndary` must evaluate the functions G_i^L and G_i^R which
        describe the boundary conditions, as given in (5) and (6).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must compute the left-hand boundary condition at
              x = a.

            `ibnd` != 0

              `bndary` must compute of the right-hand boundary condition
              at x = b.

        nobc : int
            Specifies the number n_a of boundary conditions at the
            boundary specified by `ibnd`.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t) at the
            boundary specified by `ibnd`, for i = 1,2,...,`npde`.

        ut : float, ndarray, shape (`npde`)
            `ut`[i-1] contains the value of the component dU_i(x,t)/dt,
            for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: `vdot`[i-1], for i = 1,2,...,`nv`, may only appear
            linearly as in (1) and (2).

        res : float, ndarray, shape (`nobc`), to be modified in place
            On exit: `res`[i-1] must contain the ith component of G^L or
            G^R, depending on the value of `ibnd`, for i =
            1,2,...,`nobc`, where G^L is defined as

                G_i^L = sum_j = 1^`npde`E_i,j^LdU_j/dt+sum_j =
                1^`nv`H_i,j^LV^._j,

            i.e., only terms depending explicitly on time derivatives,
            or

                G_i^L = sum_j = 1^`npde`E_i,j^LdU_j/dt+sum_j =
                1^`nv`H_i,j^LV^._j+K_i^L,

            i.e., all terms in equation (7), and similarly for G_i^R.

            The definitions of G^L and G^R are determined by the input
            value of `ires`.

        ires : int
            The form of G_i^L (or G_i^R) that must be returned in the
            array `res`.

            `ires` = -1

              Equation (1) must be used.

            `ires` = 1

              Equation (2) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_remesh_keller`` returns to the calling
              function with the error indicator set to `errno` = 4.

    uvinit : callable uvinit(npde, npts, nxi, x, xi, u, nv, v,
        data=None)
        `uvinit` must supply the initial (t = t_0) values of U(x,t) and
        V(t) for all values of x in the interval [a,b].

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        nxi : int
            The number of ODE/PDE coupling points.

        x : float, ndarray, shape (`npts`)
            The current mesh. `x`[i-1] contains the value of x_i, for i
            = 1,2,...,`npts`.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling point,
            xi_i, for i = 1,2,...,`nxi`.

        u : float, ndarray, shape (`npde`, `npts`), to be modified in
        place
            On exit: `u`[i-1,j-1] contains the value of the component
            U_i(x_j,t_0), for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`), to be modified in place
            On exit: if `nv` > 0, `v`[i-1] must contain the value of
            component V_i(t_0), for i = 1,2,...,`nv`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    u : float, shape (`neqn`), modified in place
        On entry: if `ind` = 1 the value of `u` must be unchanged from
        the previous call.

        On exit: the computed solution U_i(x_j,t), for j =
        1,2,...,`npts`, for i = 1,2,...,`npde`, and V_k(t), for k =
        1,2,...,`nv`, evaluated at t = `ts`, as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t), for i = 1,2,...,`nv`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`), modified in place
        On entry: the initial mesh points in the space direction. `x`[0]
        must specify the left-hand boundary, a, and `x`[`npts`-1] must
        specify the right-hand boundary, b.

        On exit: the final values of the mesh points.

    nleft : int
        The number n_a of boundary conditions at the left-hand mesh
        point `x`[0].

    nv : int
        The number of coupled ODE components.

    odedef : None or callable ires = odedef(npde, t, nv, v, vdot, nxi,
        xi, ucp, ucpx, ucpt, r, ires, data=None)
        `odedef` must evaluate the functions R, which define the system
        of ODEs, as given in (4).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        nxi : int
            The number of ODE/PDE coupling points.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling point,
            xi_i, for i = 1,2,...,`nxi`.

        ucp : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucp`[i-1,j-1] contains the value of U_i(x,t)
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucpx : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpx`[i-1,j-1] contains the value of
            dU_i(x,t)/dx at the coupling point x = xi_j, for j =
            1,2,...,`nxi`, for i = 1,2,...,`npde`.

        ucpt : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpt`[i-1,j-1] contains the value of dU_i/dt
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        r : float, ndarray, shape (`nv`), to be modified in place
            On exit: if `nv` > 0, `r`[i-1] must contain the ith
            component of R, for i = 1,2,...,`nv`, where R is defined as

                R = -BV^.-CU_t^*,

            i.e., only terms depending explicitly on time derivatives,
            or

                R = A-BV^.-CU_t^*,

            i.e., all terms in equation (4). The definition of R is
            determined by the input value of `ires`.

        ires : int
            The form of R that must be returned in the array `r`.

            `ires` = -1

              Equation (3) must be used.

            `ires` = 1

              Equation (4) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may reset
            `ires` to force the integration function to take certain
            actions, as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_remesh_keller`` returns to the calling
              function with the error indicator set to `errno` = 4.

    nxi : int
        The number of ODE/PDE coupling points.

    xi : float, shape (`nxi`)
        `xi`[i-1], for i = 1,2,...,`nxi`, must be set to the ODE/PDE
        coupling points, xi_i.

    neqn : int
        The number of ODEs in the time direction.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; if `itol` in (3, 4): `neqn`;
        otherwise: 0.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; if `itol` in (2, 4): `neqn`;
        otherwise: 0.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``dim1_parab_remesh_keller`` whether to interpret
        either or both of `rtol` or `atol` as a vector or scalar.

    norm : str, length 1
        The type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L_2 norm.

        If U_norm denotes the norm of the vector `u` of length `neqn`,
        then for the averaged L_2 norm

            U_norm = sqrt(1/`neqn`sum_i = 1^`neqn`(U(i)/w_i)^2),

        while for the maximum norm

            U_norm = max_i|`u`[i-1]/w_i|.

        See the description of `itol` for the formulation of the weight
        vector w.

    laopt : str, length 1
        The type of matrix algebra required.

        `laopt` = 'F'

          Full matrix methods to be used.

        `laopt` = 'B'

          Banded matrix methods to be used.

        `laopt` = 'S'

          Sparse matrix methods to be used.

    algopt : float, shape (30)
        May be set to control various options available in the
        integrator. If you wish to employ all the default options,
        `algopt`[0] should be set to 0.0. Default values will also be
        used for any other elements of `algopt` set to zero. The
        permissible values, default values, and meanings are as follows:

        `algopt`[0]

          Selects the ODE integration method to be used. If `algopt`[0]
          = 1.0, a BDF method is used and if `algopt`[0] = 2.0, a Theta
          method is used. The default value is `algopt`[0] = 1.0.

        If `algopt`[0] = 2.0, then `algopt`[i-1], for i = 2,3,...,4, are
        not used.

        `algopt`[1]

          Specifies the maximum order of the BDF integration formula to
          be used. `algopt`[1] may be 1.0, 2.0, 3.0, 4.0 or 5.0. The
          default value is `algopt`[1] = 5.0.

        `algopt`[2]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the BDF method. If
          `algopt`[2] = 1.0 a modified Newton iteration is used and if
          `algopt`[2] = 2.0 a functional iteration method is used. If
          functional iteration is selected and the integrator encounters
          difficulty, there is an automatic switch to the modified
          Newton iteration. The default value is `algopt`[2] = 1.0.

        `algopt`[3]

          Specifies whether or not the Petzold error test is to be
          employed. The Petzold error test results in extra overhead but
          is more suitable when algebraic equations are present, such as
          P_{i,j} = 0.0, for j = 1,2,...,`npde`, for some i or when
          there is no V^._i(t) dependence in the coupled ODE system. If
          `algopt`[3] = 1.0, the Petzold test is used. If `algopt`[3] =
          2.0, the Petzold test is not used. The default value is
          `algopt`[3] = 1.0.

        If `algopt`[0] = 1.0, `algopt`[i-1], for i = 5,6,...,7, are not
        used.

        `algopt`[4]

          Specifies the value of Theta to be used in the Theta
          integration method. 0.51 <= `algopt`[4] <= 0.99. The default
          value is `algopt`[4] = 0.55.

        `algopt`[5]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the Theta method.
          If `algopt`[5] = 1.0, a modified Newton iteration is used and
          if `algopt`[5] = 2.0, a functional iteration method is used.
          The default value is `algopt`[5] = 1.0.

        `algopt`[6]

          Specifies whether or not the integrator is allowed to switch
          automatically between modified Newton and functional iteration
          methods in order to be more efficient. If `algopt`[6] = 1.0,
          switching is allowed and if `algopt`[6] = 2.0, switching is
          not allowed. The default value is `algopt`[6] = 1.0.

        `algopt`[10]

          Specifies a point in the time direction, t_crit, beyond which
          integration must not be attempted. The use of t_crit is
          described under the argument `itask`. If `algopt`[0] != 0.0, a
          value of 0.0, for `algopt`[10], say, should be specified even
          if `itask` subsequently specifies that t_crit will not be
          used.

        `algopt`[11]

          Specifies the minimum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[11]
          should be set to 0.0.

        `algopt`[12]

          Specifies the maximum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[12]
          should be set to 0.0.

        `algopt`[13]

          Specifies the initial step size to be attempted by the
          integrator. If `algopt`[13] = 0.0, the initial step size is
          calculated internally.

        `algopt`[14]

          Specifies the maximum number of steps to be attempted by the
          integrator in any one call. If `algopt`[14] = 0.0, no limit is
          imposed.

        `algopt`[22]

          Specifies what method is to be used to solve the nonlinear
          equations at the initial point to initialize the values of U,
          U_t, V and V^.. If `algopt`[22] = 1.0, a modified Newton
          iteration is used and if `algopt`[22] = 2.0, functional
          iteration is used. The default value is `algopt`[22] = 1.0.

        `algopt`[28] and `algopt`[29] are used only for the sparse
        matrix algebra option, i.e., `laopt` = 'S'.

        `algopt`[28]

          Governs the choice of pivots during the decomposition of the
          first Jacobian matrix. It should lie in the range 0.0 <
          `algopt`[28] < 1.0, with smaller values biasing the algorithm
          towards maintaining sparsity at the expense of numerical
          stability. If `algopt`[28] lies outside this range then the
          default value is used. If the functions regard the Jacobian
          matrix as numerically singular then increasing `algopt`[28]
          towards 1.0 may help, but at the cost of increased fill-in.
          The default value is `algopt`[28] = 0.1.

        `algopt`[29]

          Used as a relative pivot threshold during subsequent Jacobian
          decompositions (see `algopt`[28]) below which an internal
          error is invoked. `algopt`[29] must be greater than zero,
          otherwise the default value is used. If `algopt`[29] is
          greater than 1.0 no check is made on the pivot size, and this
          may be a necessary option if the Jacobian is found to be
          numerically singular (see `algopt`[28]). The default value is
          `algopt`[29] = 0.0001.

    remesh : bool
        Indicates whether or not spatial remeshing should be performed.

        `remesh` = True

          Indicates that spatial remeshing should be performed as
          specified.

        `remesh` = False

          Indicates that spatial remeshing should be suppressed.

        Note: `remesh` should not be changed between consecutive calls
        to ``dim1_parab_remesh_keller``. Remeshing can be switched off
        or on at specified times by using appropriate values for the
        arguments `nrmesh` and `trmesh` at each call.

    nxfix : int
        The number of fixed mesh points.

    xfix : float, shape (`nxfix`)
        `xfix`[i-1], for i = 1,2,...,`nxfix`, must contain the value of
        the x coordinate at the ith fixed mesh point.

    nrmesh : int
        Indicates the form of meshing to be performed.

        `nrmesh` < 0

          Indicates that a new mesh is adopted according to the argument
          `dxmesh`. The mesh is tested every |`nrmesh`| timesteps.

        `nrmesh` = 0

          Indicates that remeshing should take place just once at the
          end of the first time step reached when t > `trmesh`.

        `nrmesh` > 0

          Indicates that remeshing will take place every `nrmesh` time
          steps, with no testing using `dxmesh`.

        Note: `nrmesh` may be changed between consecutive calls to
        ``dim1_parab_remesh_keller`` to give greater flexibility over
        the times of remeshing.

    dxmesh : float
        Determines whether a new mesh is adopted when `nrmesh` is set
        less than zero. A possible new mesh is calculated at the end of
        every |`nrmesh`| time steps, but is adopted only if

            x_i^new > x_i^old+`dxmesh`*(x_i+1^old-x_i^old),

        or

            x_i^new < x_i^old-`dxmesh`*(x_i^old-x_i-1^old).

        `dxmesh` thus imposes a lower limit on the difference between
        one mesh and the next.

    trmesh : float
        Specifies when remeshing will take place when `nrmesh` is set to
        zero. Remeshing will occur just once at the end of the first
        time step reached when t is greater than `trmesh`.

        Note: `trmesh` may be changed between consecutive calls to
        ``dim1_parab_remesh_keller`` to force remeshing at several
        specified times.

    ipminf : int
        The level of trace information regarding the adaptive remeshing.
        Details are directed to the file object associated with the
        advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

        `ipminf` = 0

          No trace information.

        `ipminf` = 1

          Brief summary of mesh characteristics.

        `ipminf` = 2

          More detailed information, including old and new mesh points,
          mesh sizes and monitor function values.

    xratio : float
        An input bound on the adjacent mesh ratio (greater than 1.0 and
        typically in the range 1.5 to 3.0). The remeshing functions will
        attempt to ensure that

            (x_i-x_{i-1})/`xratio` < x_{i+1}-x_i <
            `xratio`*(x_i-x_{i-1}).

    con : float
        An input bound on the sub-integral of the monitor function
        F^mon(x) over each space step. The remeshing functions will
        attempt to ensure that

            int_x_1^x_{i+1}F^mon(x)dx <=
            `con`int_x_1^x_`npts`F^mon(x)dx,

        . `con` gives you more control over the mesh distribution e.g.,
        decreasing `con` allows more clustering. A typical value is
        2/(`npts`-1), but you are encouraged to experiment with
        different values. Its value is not critical and the mesh should
        be qualitatively correct for all values in the range given
        below.

    monitf : None or callable monitf(t, npts, npde, x, u, fmon,
        data=None)
        `monitf` must supply and evaluate a remesh monitor function to
        indicate the solution behaviour of interest.

        Parameters
        ~~~~~~~~~~
        t : float
            The current value of the independent variable t.

        npts : int
            The number of mesh points in the interval [a,b].

        npde : int
            The number of PDEs in the system.

        x : float, ndarray, shape (`npts`)
            The current mesh. `x`[i-1] contains the value of x_i, for i
            = 1,2,...,`npts`.

        u : float, ndarray, shape (`npde`, `npts`)
            `u`[i-1,j-1] contains the value of U_i(x,t) at x = `x`[j-1]
            and time t, for j = 1,2,...,`npts`, for i = 1,2,...,`npde`.

        fmon : float, ndarray, shape (`npts`), to be modified in place
            On exit: `fmon`[i-1] must contain the value of the monitor
            function F^mon(x) at mesh point x = `x`[i-1].

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    comm : dict, communication object, modified in place
        Note: this argument will be (re-)initialized when it is an empty
        ``dict`` or under the following condition: `ind` = 0.

        Communication structure.

        On initial entry: need not be set.

    lrsave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['rsave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lrsave` is computed by the
        function.

        If your supplied `lrsave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lrsave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lrsave` set to at least the lower-bound value returned in
        `lrsave_min`.

    lisave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['isave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lisave` is computed by the
        function.

        If your supplied `lisave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lisave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lisave` set to at least the lower-bound value returned in
        `lisave_min`.

    itask : int
        The task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step in the time direction and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

        `itask` = 4

          Normal computation of output values `u` at t = `tout` but
          without overshooting t = t_crit where t_crit is described
          under the argument `algopt`.

        `itask` = 5

          Take one step in the time direction and return, without
          passing t_crit, where t_crit is described under the argument
          `algopt`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_remesh_keller`` and the underlying ODE solver as
        follows:

        `itrace` <= -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` = 1

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        `itrace` = 2

          Output from the underlying ODE solver is similar to that
          produced when `itrace` = 1, except that the advisory messages
          are given in greater detail.

        `itrace` >= 3

          The output from the underlying ODE solver is similar to that
          produced when `itrace` = 2, except that the advisory messages
          are given in greater detail.

        You are advised to set `itrace` = 0, unless you are experienced
        with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the argument `tout` and the
          remeshing arguments `nrmesh`, `dxmesh`, `trmesh`, `xratio` and
          `con` may be reset between calls to
          ``dim1_parab_remesh_keller``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    lrsave_min : int
        A lower bound on the sufficient size for `comm`['rsave'].

    lisave_min : int
        A lower bound on the sufficient size for `comm`['isave'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `lrsave` = *<value>*.

            Constraint: `lrsave` >= *<value>*.

        (`errno` 1)
            On entry, `lisave` = *<value>*.

            Constraint: `lisave` >= *<value>*.

        (`errno` 1)
            On entry, the point `xfix`[i-1] does not coincide with any
            `x`[j-1]: i = *<value>* and `xfix`[i-1] = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xfix`[i] = *<value>* and
            `xfix`[i-1] = *<value>*.

            Constraint: `xfix`[i] > `xfix`[i-1].

        (`errno` 1)
            On entry, at least one point in `xi` lies outside
            [`x`[0],`x`[`npts`-1]]: `x`[0] = *<value>* and `x`[`npts`-1]
            = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xi`[i] = *<value>* and `xi`[i-1] =
            *<value>*.

            Constraint: `xi`[i] > `xi`[i-1].

        (`errno` 1)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: corresponding elements `atol`[i-1] and
            `rtol`[j-1] cannot both be 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `rtol`[i-1] = *<value>*.

            Constraint: `rtol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `atol`[i-1] = *<value>*.

            Constraint: `atol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `neqn` = *<value>*, `npde` = *<value>*, `npts` =
            *<value>* and `nv` = *<value>*.

            Constraint: `neqn` = `npde`*`npts`+`nv`.

        (`errno` 1)
            On entry, `con` = *<value>*, `npts` = *<value>*.

            Constraint: `con` <= 10.0/(`npts`-1).

        (`errno` 1)
            On entry, `con` = *<value>*, `npts` = *<value>*.

            Constraint: `con` >= 0.1/(`npts`-1).

        (`errno` 1)
            On entry, `xratio` = *<value>*.

            Constraint: `xratio` > 1.0.

        (`errno` 1)
            On entry, `ipminf` = *<value>*.

            Constraint: `ipminf` = 0, 1 or 2.

        (`errno` 1)
            On entry, `dxmesh` = *<value>*.

            Constraint: `dxmesh` >= 0.0.

        (`errno` 1)
            On entry, `nxfix` = *<value>*, `npts` = *<value>*.

            Constraint: `nxfix` <= `npts`-2.

        (`errno` 1)
            On entry, `nxfix` = *<value>*.

            Constraint: `nxfix` >= 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` = 0 when `nv` = 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` >= 0 when `nv` > 0.

        (`errno` 1)
            On entry, `nleft` = *<value>*, `npde` = *<value>*.

            Constraint: `nleft` <= `npde`.

        (`errno` 1)
            On entry, `nleft` = *<value>*.

            Constraint: `nleft` >= 0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `laopt` = *<value>*.

            Constraint: `laopt` = 'F', 'B' or 'S'.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = 'A' or 'M'.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2, 3, 4 or 5.

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 0.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef` or `bndary`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `atol` and `rtol` were too small to start integration.

        (`errno` 8)
            `ires` set to an invalid value in call to `pdedef`,
            `bndary`, or `odedef`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 12)
            In solving ODE system, the maximum number of steps
            `algopt`[14] has been exceeded. `algopt`[14] = *<value>*.

        (`errno` 16)
            `remesh` has been changed between calls to
            ``dim1_parab_remesh_keller``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `atol` and `rtol`.
            `ts` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in `pdedef`, `bndary`, or `odedef`. Integration is
            successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `atol` or `rtol`
            are unlikely to result in a changed solution.

        (`errno` 13)
            Zero error weights encountered during time integration.

        (`errno` 15)
            When using the sparse option `lisave` or `lrsave` is too
            small: `lisave` = *<value>*, `lrsave` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_remesh_keller'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, q[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, res[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[ns].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, q, res, ns, s, ires, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ut_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _nv_pdedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_pdedefh,),
            )
            _vdot_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_pdedefh,),
            )
            _res_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _res_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _res_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(ut)
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ngamma].
        utils._BYREF_INT, # Input, nobc.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nobc].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ut, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, gamma, nobc, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _nobc_bndaryh = utils._EngineIntScalarType.to_py(nobc.contents)
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh,),
            )
            _ut_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npde_bndaryh,),
            )
            _nv_bndaryh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_bndaryh,),
            )
            _vdot_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_bndaryh,),
            )
            _res_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_nobc_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _ibnd_bndaryh,
                    _nobc_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _res_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _t_bndaryh,
                    _ibnd_bndaryh,
                    _nobc_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _res_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _res_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'res'}, res,
            )
            utils._unlink_pointer(res)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_uvinit_name = fun_name + ':uvinit'

    uvinith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, u[npde, npts].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Output, v[nv].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_uvinith(_uvinit, en_data, npde, npts, nxi, x, xi, u, nv, v, _iuser, _ruser):
        """Helper for uvinit."""
        try:
            _npde_uvinith = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_uvinith = utils._EngineIntScalarType.to_py(npts.contents)
            _nxi_uvinith = utils._EngineIntScalarType.to_py(nxi.contents)
            _x_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_uvinith,),
            )
            _xi_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_uvinith,),
            )
            _u_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_uvinith, _npts_uvinith),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _nv_uvinith = utils._EngineIntScalarType.to_py(nv.contents)
            _v_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_uvinith,),
                can_reference_contents=False,
            )
            if data is not None:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _nxi_uvinith,
                    _x_uvinith,
                    _xi_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                    data,
                )
            else:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _nxi_uvinith,
                    _x_uvinith,
                    _xi_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(xi)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _u_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'u'}, u,
            )
            utils._unlink_pointer(u)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _v_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'v'}, v,
            )
            utils._unlink_pointer(v)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_odedef_name = fun_name + ':odedef'

    odedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, rcp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpt[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucptx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, r[nv].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_odedefh(_odedef, en_data, npde, t, nv, v, vdot, nxi, xi, ucp, ucpx, rcp, ucpt, ucptx, r, ires, _iuser, _ruser):
        """Helper for odedef."""
        if odedef is None:
            return
        try:
            _npde_odedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_odedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nv_odedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_odedefh,),
            )
            _vdot_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_odedefh,),
            )
            _nxi_odedefh = utils._EngineIntScalarType.to_py(nxi.contents)
            _xi_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_odedefh,),
            )
            _ucp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpt_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpt,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _r_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_nv_odedefh,),
                can_reference_contents=False,
            )
            _ires_odedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                    data,
                )
            else:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._unlink_pointer(xi)
            utils._unlink_pointer(ucp)
            utils._unlink_pointer(ucpx)
            utils._unlink_pointer(ucpt)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_odedefh, {'fun_name': cb_odedef_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_odedefh, {'fun_name': cb_odedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitf_name = fun_name + ':monitf'

    monitfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, npts].
        utils._BYREF_INT, # Input, nr.
        utils._EngineFloat64ArrayType.c_type, # Input, r[nr].
        utils._EngineFloat64ArrayType.c_type, # Output, fmon[npts].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monitfh(_monitf, en_data, t, npts, npde, x, u, nr, r, fmon, _iuser, _ruser):
        """Helper for monitf."""
        if monitf is None:
            return
        try:
            _t_monitfh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _npts_monitfh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_monitfh = utils._EngineIntScalarType.to_py(npde.contents)
            _x_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_monitfh,),
            )
            _u_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_monitfh, _npts_monitfh),
                sorder=utils._EngineData.default_sorder,
            )
            _fmon_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                fmon,
                (_npts_monitfh,),
                can_reference_contents=False,
            )
            if data is not None:
                monitf(
                    _t_monitfh,
                    _npts_monitfh,
                    _npde_monitfh,
                    _x_monitfh,
                    _u_monitfh,
                    _fmon_monitfh,
                    data,
                )
            else:
                monitf(
                    _t_monitfh,
                    _npts_monitfh,
                    _npde_monitfh,
                    _x_monitfh,
                    _u_monitfh,
                    _fmon_monitfh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(u)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fmon_monitfh, {'fun_name': cb_monitf_name, 'returned': 'fmon'}, fmon,
            )
            utils._unlink_pointer(fmon)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03prft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._UNUSED_FUNCTYPE,
        uvinith,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[neqn].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[npts].
        utils._BYREF_INT, # Input, nleft.
        utils._BYREF_INT, # Input, nv.
        utils._UNUSED_FUNCTYPE,
        odedefh,
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._BYREF_INT, # Input, neqn.
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, laopt(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, algopt[30].
        utils._BYREF_LOGICAL, # Input, remesh.
        utils._BYREF_INT, # Input, nxfix.
        utils._EngineFloat64ArrayType.c_type, # Input, xfix[nxfix].
        utils._BYREF_INT, # Input, nrmesh.
        utils._BYREF_FLOAT64, # Input, dxmesh.
        utils._BYREF_FLOAT64, # Input, trmesh.
        utils._BYREF_INT, # Input, ipminf.
        utils._BYREF_FLOAT64, # Input, xratio.
        utils._BYREF_FLOAT64, # Input, con.
        utils._UNUSED_FUNCTYPE,
        monitfh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Output, lrsave_min.
        utils._BYREF_INT, # Output, lisave_min.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of laopt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    utils._check_callable(uvinit, {'fun_name': fun_name, 'entity': 'uvinit'})
    _uvinit = utils._UNUSED_CALLBACK
    _uvinith = utils._cast_callback(
        uvinith, cb_uvinith,
        {'fun_name': fun_name, 'entity': 'uvinit'},
    )
    _neqn = utils._EngineIntScalarType(
        neqn, {'fun_name': fun_name, 'entity': 'neqn'},
    ).to_ctypes()
    _neqn_py = utils._EngineIntScalarType.to_py(_neqn)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_neqn_py,),
    )
    _u = _u_cls.to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    )
    _x = _x_cls.to_ctypes()
    _nleft = utils._EngineIntScalarType(
        nleft, {'fun_name': fun_name, 'entity': 'nleft'},
    ).to_ctypes()
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    utils._check_callable(odedef, {'fun_name': fun_name, 'entity': 'odedef'}, allow_none=True)
    _odedef = utils._UNUSED_CALLBACK
    _odedefh = utils._cast_callback(
        odedefh, cb_odedefh,
        {'fun_name': fun_name, 'entity': 'odedef'},
    )
    _nxi = utils._EngineIntScalarType(
        nxi, {'fun_name': fun_name, 'entity': 'nxi'},
    ).to_ctypes()
    _nxi_py = utils._EngineIntScalarType.to_py(_nxi)
    _xi = utils._EngineFloat64ArrayType(
        xi, {'fun_name': fun_name, 'entity': 'xi'},
        exp_shape=(_nxi_py,),
    ).to_ctypes()
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    elif _itol_py in (3, 4):
        _rtol_dim1 = _neqn_py
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    elif _itol_py in (2, 4):
        _atol_dim1 = _neqn_py
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _laopt = utils._EngineCharScalarType(
        laopt, {'fun_name': fun_name, 'entity': 'laopt'}, exp_chrlen=1,
    ).to_ctypes()
    _algopt = utils._EngineFloat64ArrayType(
        algopt, {'fun_name': fun_name, 'entity': 'algopt'},
        exp_shape=(30,),
    ).to_ctypes()
    _remesh = utils._EngineLogicalScalarType(
        remesh, {'fun_name': fun_name, 'entity': 'remesh'},
    ).to_ctypes()
    _nxfix = utils._EngineIntScalarType(
        nxfix, {'fun_name': fun_name, 'entity': 'nxfix'},
    ).to_ctypes()
    _nxfix_py = utils._EngineIntScalarType.to_py(_nxfix)
    _xfix = utils._EngineFloat64ArrayType(
        xfix, {'fun_name': fun_name, 'entity': 'xfix'},
        exp_shape=(_nxfix_py,),
    ).to_ctypes()
    _nrmesh = utils._EngineIntScalarType(
        nrmesh, {'fun_name': fun_name, 'entity': 'nrmesh'},
    ).to_ctypes()
    _dxmesh = utils._EngineFloat64ScalarType(
        dxmesh, {'fun_name': fun_name, 'entity': 'dxmesh'},
    ).to_ctypes()
    _trmesh = utils._EngineFloat64ScalarType(
        trmesh, {'fun_name': fun_name, 'entity': 'trmesh'},
    ).to_ctypes()
    _ipminf = utils._EngineIntScalarType(
        ipminf, {'fun_name': fun_name, 'entity': 'ipminf'},
    ).to_ctypes()
    _xratio = utils._EngineFloat64ScalarType(
        xratio, {'fun_name': fun_name, 'entity': 'xratio'},
    ).to_ctypes()
    _con = utils._EngineFloat64ScalarType(
        con, {'fun_name': fun_name, 'entity': 'con'},
    ).to_ctypes()
    utils._check_callable(monitf, {'fun_name': fun_name, 'entity': 'monitf'}, allow_none=True)
    _monitf = utils._UNUSED_CALLBACK
    _monitfh = utils._cast_callback(
        monitfh, cb_monitfh,
        {'fun_name': fun_name, 'entity': 'monitf'},
    )
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _comm_needs_init = (
        utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
        ) or
        _ind_py == 0
    )
    if _comm_needs_init:
        if lrsave is None:
            _lrsave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lrsave = utils._EngineIntScalarType(
                lrsave, {'fun_name': fun_name, 'entity': 'lrsave'},
            ).to_ctypes()
        _lrsave_py = utils._EngineIntScalarType.to_py(_lrsave)
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
    else:
        _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = comm['rsave']
    if _comm_needs_init:
        if lisave is None:
            _lisave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lisave = utils._EngineIntScalarType(
                lisave, {'fun_name': fun_name, 'entity': 'lisave'},
            ).to_ctypes()
        _lisave_py = utils._EngineIntScalarType.to_py(_lisave)
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
    else:
        _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = comm['isave']
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iemode_py = 1 if _comm_needs_init else 0
    _iemode = utils._EngineIntScalarType(
        _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
    ).to_ctypes()
    _lrsave_min = utils._EngineIntScalarType.empty_ctypes()
    _lisave_min = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _uvinit,
        _uvinith,
        _u,
        _npts,
        _x,
        _nleft,
        _nv,
        _odedef,
        _odedefh,
        _nxi,
        _xi,
        _neqn,
        _rtol,
        _atol,
        _itol,
        _norm,
        _laopt,
        _algopt,
        _remesh,
        _nxfix,
        _xfix,
        _nrmesh,
        _dxmesh,
        _trmesh,
        _ipminf,
        _xratio,
        _con,
        _monitf,
        _monitfh,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iemode,
        _lrsave_min,
        _lisave_min,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _norm._length_,
        _laopt._length_,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS and _iemode_py == 1:
        _iemode_py = 0
        _iemode = utils._EngineIntScalarType(
            _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
        ).to_ctypes()
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lrsave_py = max(_lrsave_min_py, _lrsave_py)
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        _lisave_py = max(_lisave_min_py, _lisave_py)
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
        engine_ctypes(
            _en_data,
            utils._UNUSED_PRINT_REC,
            _print_rech,
            _npde,
            _ts,
            _tout,
            _pdedef,
            _pdedefh,
            _bndary,
            _bndaryh,
            _uvinit,
            _uvinith,
            _u,
            _npts,
            _x,
            _nleft,
            _nv,
            _odedef,
            _odedefh,
            _nxi,
            _xi,
            _neqn,
            _rtol,
            _atol,
            _itol,
            _norm,
            _laopt,
            _algopt,
            _remesh,
            _nxfix,
            _xfix,
            _nrmesh,
            _dxmesh,
            _trmesh,
            _ipminf,
            _xratio,
            _con,
            _monitf,
            _monitfh,
            _rsave,
            _lrsave,
            _isave,
            _lisave,
            _itask,
            _itrace,
            _nadv,
            _ind,
            _iemode,
            _lrsave_min,
            _lisave_min,
            _iuser,
            _ruser,
            _cwsav,
            _lwsav,
            _iwsav,
            _rwsav,
            _errbuf,
            _ierr,
            _norm._length_,
            _laopt._length_,
            80,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        if _ierr_py == 1 and _ifmt == 99953:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrsave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lisave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lisave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the point xfix[i-1] does not coincide with any x[j-1]:',
                    'i = {:s} and xfix[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xfix[i] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and xfix[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xfix[i] > xfix[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one point in xi lies outside [x[0],x[npts-1]]:',
                    'x[0] = {:s} and x[npts-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xi[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xi[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xi[i] > xi[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: corresponding elements atol[i-1] and rtol[j-1]',
                    'cannot both be 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and rtol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and atol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqn = {:s}, npde = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npts = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: neqn = npde*npts+nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con <= 10.0/(npts-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con >= 0.1/(npts-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xratio = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xratio > 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ipminf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ipminf = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dxmesh = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dxmesh >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxfix = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nxfix <= npts-2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxfix = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxfix >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi = 0 when nv = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi >= 0 when nv > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nleft = {:s}, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nleft <= npde.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nleft = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nleft >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laopt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: laopt = \'F\', \'B\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'A\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99966:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of atol and rtol.',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99965:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99964:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef or bndary.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99962:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'pdedef, bndary, or odedef. Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'atol and rtol were too small to start integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in call to pdedef, bndary, or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99959:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99958:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in atol or rtol are unlikely',
                    'to result in a changed solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99957:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99956:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In solving ODE system, the maximum number of steps',
                    'algopt[14] has been exceeded.',
                    'algopt[14] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99955:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Zero error weights encountered during time integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99954:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'When using the sparse option lisave or lrsave is too small:',
                    'lisave = {:s}, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 99967:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'remesh has been changed between calls to dim1_parab_remesh_keller.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _x_cls.refresh_py(_x)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind', 'lrsave_min', 'lisave_min'],
        )(
            _ts_py,
            _ind_py,
            _lrsave_min_py,
            _lisave_min_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_convdiff_remesh(npde, ts, tout, pdedef, numflx, bndary, uvinit, u, npts, x, nv, odedef, nxi, xi, neqn, rtol, atol, itol, norm, laopt, algopt, remesh, nxfix, xfix, nrmesh, dxmesh, trmesh, ipminf, xratio, con, monitf, comm, lrsave, lisave, itask, itrace, ind, data=None, io_manager=None):
    u"""
    General system of convection-diffusion PDEs, coupled DAEs, method of
    lines, upwind scheme, remeshing, one space variable.

    For full information please refer to the NAG Library document for
    d03ps

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03psf.html

    Parameters
    ----------
    npde : int
        The number of PDEs to be solved.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    pdedef : None or callable ires = pdedef(npde, t, x, u, ux, nv, v,
        vdot, p, c, d, s, ires, data=None)
        `pdedef` must evaluate the functions P_{i,j}, C_i, D_i and S_i
        which partially define the system of PDEs. P_{i,j} and C_i may
        depend on x, t, U and V; D_i may depend on x, t, U, U_x and V;
        and S_i may depend on x, t, U, V and linearly on V^.. `pdedef`
        is called approximately midway between each pair of mesh points
        in turn by ``dim1_parab_convdiff_remesh``. None may be used for
        `pdedef` for problems in the form (2).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        u : float, ndarray, shape (`npde`)
            `u`[i-1] contains the value of the component U_i(x,t), for i
            = 1,2,...,`npde`.

        ux : float, ndarray, shape (`npde`)
            `ux`[i-1] contains the value of the component dU_i(x,t)/dx,
            for i = 1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in S_j, for j = 1,2,...,`npde`.

        p : float, ndarray, shape (`npde`, `npde`), to be modified in
        place
            On exit: `p`[i-1,j-1] must be set to the value of
            P_{i,j}(x,t,U,V), for j = 1,2,...,`npde`, for i =
            1,2,...,`npde`.

        c : float, ndarray, shape (`npde`), to be modified in place
            On exit: `c`[i-1] must be set to the value of C_i(x,t,U,V),
            for i = 1,2,...,`npde`.

        d : float, ndarray, shape (`npde`), to be modified in place
            On exit: `d`[i-1] must be set to the value of
            D_i(x,t,U,U_x,V), for i = 1,2,...,`npde`.

        s : float, ndarray, shape (`npde`), to be modified in place
            On exit: `s`[i-1] must be set to the value of
            S_i(x,t,U,V,V^.), for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_remesh`` returns to the calling
              function with the error indicator set to `errno` = 4.

    numflx : callable ires = numflx(npde, t, x, nv, v, uleft, uright,
        flux, ires, data=None)
        `numflx` must supply the numerical flux for each PDE given the
        left and right values of the solution vector `u`. `numflx` is
        called approximately midway between each pair of mesh points in
        turn by ``dim1_parab_convdiff_remesh``.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float
            The current value of the space variable x.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        uleft : float, ndarray, shape (`npde`)
            `uleft`[i-1] contains the left value of the component
            U_i(x), for i = 1,2,...,`npde`.

        uright : float, ndarray, shape (`npde`)
            `uright`[i-1] contains the right value of the component
            U_i(x), for i = 1,2,...,`npde`.

        flux : float, ndarray, shape (`npde`), to be modified in place
            On exit: `flux`[i-1] must be set to the numerical flux F^_i,
            for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_remesh`` returns to the calling
              function with the error indicator set to `errno` = 4.

    bndary : callable ires = bndary(npde, npts, t, x, u, nv, v, vdot,
        ibnd, g, ires, data=None)
        `bndary` must evaluate the functions G_i^L and G_i^R which
        describe the physical and numerical boundary conditions, as
        given by (9) and (0).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            The mesh points in the spatial direction. `x`[0] corresponds
            to the left-hand boundary, a, and `x`[`npts`-1] corresponds
            to the right-hand boundary, b.

        u : float, ndarray, shape (`npde`, `npts`)
            `u`[i-1,j-1] contains the value of the component U_i(x,t) at
            x = `x`[j-1], for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

            Note: if banded matrix algebra is to be used then the
            functions G_i^L and G_i^R may depend on the value of
            U_i(x,t) at the boundary point and the two adjacent points
            only.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

            Note: V^._i(t), for i = 1,2,...,`nv`, may only appear
            linearly in G_j^L and G_j^R, for j = 1,2,...,`npde`.

        ibnd : int
            Specifies which boundary conditions are to be evaluated.

            `ibnd` = 0

              `bndary` must evaluate the left-hand boundary condition at
              x = a.

            `ibnd` != 0

              `bndary` must evaluate the right-hand boundary condition
              at x = b.

        g : float, ndarray, shape (`npde`), to be modified in place
            On exit: `g`[i-1] must contain the ith component of either
            G_i^L or G_i^R in (9) and (0), depending on the value of
            `ibnd`, for i = 1,2,...,`npde`.

        ires : int
            Set to -1 or 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may set `ires`
            to force the integration function to take certain actions as
            described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_remesh`` returns to the calling
              function with the error indicator set to `errno` = 4.

    uvinit : callable uvinit(npde, npts, nxi, x, xi, u, nv, v,
        data=None)
        `uvinit` must supply the initial (t = t_0) values of U(x,t) and
        V(t) for all values of x in the interval a <= x <= b.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        npts : int
            The number of mesh points in the interval [a,b].

        nxi : int
            The number of ODE/PDE coupling points.

        x : float, ndarray, shape (`npts`)
            The current mesh. `x`[i-1] contains the value of x_i, for i
            = 1,2,...,`npts`.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling point,
            xi_i, for i = 1,2,...,`nxi`.

        u : float, ndarray, shape (`npde`, `npts`), to be modified in
        place
            On exit: `u`[i-1,j-1] contains the value of the component
            U_i(x_j,t_0), for j = 1,2,...,`npts`, for i =
            1,2,...,`npde`.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`), to be modified in place
            On exit: if `nv` > 0, `v`[i-1] must contain the value of
            component V_i(t_0), for i = 1,2,...,`nv`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    u : float, shape (`neqn`), modified in place
        On entry: if `ind` = 1 the value of `u` must be unchanged from
        the previous call.

        On exit: the computed solution U_i(x_j,t), for j =
        1,2,...,`npts`, for i = 1,2,...,`npde`, and V_k(t), for k =
        1,2,...,`nv`, evaluated at t = `ts`, as follows:

        - `u`[`npde`*(j-1)+i-1] contain U_i(x_j,t), for j =
          1,2,...,`npts`, for i = 1,2,...,`npde`, and

        - `u`[`npts`*`npde`+i-1] contain V_i(t), for i = 1,2,...,`nv`.

    npts : int
        The number of mesh points in the interval [a,b].

    x : float, shape (`npts`), modified in place
        On entry: the mesh points in the space direction. `x`[0] must
        specify the left-hand boundary, a, and `x`[`npts`-1] must
        specify the right-hand boundary, b.

        On exit: the final values of the mesh points.

    nv : int
        The number of coupled ODE components.

    odedef : None or callable ires = odedef(npde, t, nv, v, vdot, nxi,
        xi, ucp, ucpx, ucpt, r, ires, data=None)
        `odedef` must evaluate the functions R, which define the system
        of ODEs, as given in (4).

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        nv : int
            The number of coupled ODEs in the system.

        v : float, ndarray, shape (`nv`)
            If `nv` > 0, `v`[i-1] contains the value of the component
            V_i(t), for i = 1,2,...,`nv`.

        vdot : float, ndarray, shape (`nv`)
            If `nv` > 0, `vdot`[i-1] contains the value of component
            V^._i(t), for i = 1,2,...,`nv`.

        nxi : int
            The number of ODE/PDE coupling points.

        xi : float, ndarray, shape (`nxi`)
            If `nxi` > 0, `xi`[i-1] contains the ODE/PDE coupling point,
            xi_i, for i = 1,2,...,`nxi`.

        ucp : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucp`[i-1,j-1] contains the value of U_i(x,t)
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        ucpx : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpx`[i-1,j-1] contains the value of
            dU_i(x,t)/dx at the coupling point x = xi_j, for j =
            1,2,...,`nxi`, for i = 1,2,...,`npde`.

        ucpt : float, ndarray, shape (`npde`, `nxi`)
            If `nxi` > 0, `ucpt`[i-1,j-1] contains the value of dU_i/dt
            at the coupling point x = xi_j, for j = 1,2,...,`nxi`, for i
            = 1,2,...,`npde`.

        r : float, ndarray, shape (`nv`), to be modified in place
            On exit: `r`[i-1] must contain the ith component of R, for i
            = 1,2,...,`nv`, where R is defined as

                R = L-MV^.-NU_t^*,

            or

                R = -MV^.-NU_t^*.

            The definition of R is determined by the input value of
            `ires`.

        ires : int
            The form of R that must be returned in the array `r`.

            `ires` = 1

              Equation (1) must be used.

            `ires` = -1

              Equation (2) must be used.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should usually remain unchanged. However, you may reset
            `ires` to force the integration function to take certain
            actions, as described below:

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling function with the error
              indicator set to `errno` = 6.

            `ires` = 3

              Indicates to the integrator that the current time step
              should be abandoned and a smaller time step used instead.
              You may wish to set `ires` = 3 when a physically
              meaningless input or output value has been generated. If
              you consecutively set `ires` = 3,
              ``dim1_parab_convdiff_remesh`` returns to the calling
              function with the error indicator set to `errno` = 4.

    nxi : int
        The number of ODE/PDE coupling points.

    xi : float, shape (`nxi`)
        If `nxi` > 0, `xi`[i-1], for i = 1,2,...,`nxi`, must be set to
        the ODE/PDE coupling points.

    neqn : int
        The number of ODEs in the time direction.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; if `itol` in (3, 4): `neqn`;
        otherwise: 0.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; if `itol` in (2, 4): `neqn`;
        otherwise: 0.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. If e_i is
        the estimated local error for `u`[i-1], for i = 1,2,...,`neqn`,
        and || ||, denotes the norm, the error test to be satisfied is
        ||e_i|| < 1.0. `itol` indicates to
        ``dim1_parab_convdiff_remesh`` whether to interpret either or
        both of `rtol` and `atol` as a vector or scalar in the formation
        of the weights w_i used in the calculation of the norm (see the
        description of `norm`):

        [table omitted]

    norm : str, length 1
        The type of norm to be used.

        `norm` = '1'

          Averaged L_1 norm.

        `norm` = '2'

          Averaged L_2 norm.

        If U_norm denotes the norm of the vector `u` of length `neqn`,
        then for the averaged L_1 norm

            U_norm = 1/`neqn`sum_i = 1^`neqn``u`[i-1]/w_i,

        and for the averaged L_2 norm

            U_norm = sqrt(1/`neqn`sum_i = 1^`neqn`(`u`[i-1]/w_i)^2),

        See the description of `itol` for the formulation of the weight
        vector w.

    laopt : str, length 1
        The type of matrix algebra required.

        `laopt` = 'F'

          Full matrix methods to be used.

        `laopt` = 'B'

          Banded matrix methods to be used.

        `laopt` = 'S'

          Sparse matrix methods to be used.

    algopt : float, shape (30)
        May be set to control various options available in the
        integrator. If you wish to employ all the default options,
        `algopt`[0] should be set to 0.0. Default values will also be
        used for any other elements of `algopt` set to zero. The
        permissible values, default values, and meanings are as follows:

        `algopt`[0]

          Selects the ODE integration method to be used. If `algopt`[0]
          = 1.0, a BDF method is used and if `algopt`[0] = 2.0, a Theta
          method is used. The default is `algopt`[0] = 1.0.

        If `algopt`[0] = 2.0, then `algopt`[i-1], for i = 2,3,...,4, are
        not used.

        `algopt`[1]

          Specifies the maximum order of the BDF integration formula to
          be used. `algopt`[1] may be 1.0, 2.0, 3.0, 4.0 or 5.0. The
          default value is `algopt`[1] = 5.0.

        `algopt`[2]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the BDF method. If
          `algopt`[2] = 1.0 a modified Newton iteration is used and if
          `algopt`[2] = 2.0 a functional iteration method is used. If
          functional iteration is selected and the integrator encounters
          difficulty, there is an automatic switch to the modified
          Newton iteration. The default value is `algopt`[2] = 1.0.

        `algopt`[3]

          Specifies whether or not the Petzold error test is to be
          employed. The Petzold error test results in extra overhead but
          is more suitable when algebraic equations are present, such as
          P_{i,j} = 0.0, for j = 1,2,...,`npde`, for some i or when
          there is no V^._i(t) dependence in the coupled ODE system. If
          `algopt`[3] = 1.0, the Petzold test is used. If `algopt`[3] =
          2.0, the Petzold test is not used. The default value is
          `algopt`[3] = 1.0.

        If `algopt`[0] = 1.0, `algopt`[i-1], for i = 5,6,...,7, are not
        used.

        `algopt`[4]

          Specifies the value of Theta to be used in the Theta
          integration method. 0.51 <= `algopt`[4] <= 0.99. The default
          value is `algopt`[4] = 0.55.

        `algopt`[5]

          Specifies what method is to be used to solve the system of
          nonlinear equations arising on each step of the Theta method.
          If `algopt`[5] = 1.0, a modified Newton iteration is used and
          if `algopt`[5] = 2.0, a functional iteration method is used.
          The default value is `algopt`[5] = 1.0.

        `algopt`[6]

          Specifies whether or not the integrator is allowed to switch
          automatically between modified Newton and functional iteration
          methods in order to be more efficient. If `algopt`[6] = 1.0,
          switching is allowed and if `algopt`[6] = 2.0, switching is
          not allowed. The default value is `algopt`[6] = 1.0.

        `algopt`[10]

          Specifies a point in the time direction, t_crit, beyond which
          integration must not be attempted. The use of t_crit is
          described under the argument `itask`. If `algopt`[0] != 0.0, a
          value of 0.0 for `algopt`[10], say, should be specified even
          if `itask` subsequently specifies that t_crit will not be
          used.

        `algopt`[11]

          Specifies the minimum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[11]
          should be set to 0.0.

        `algopt`[12]

          Specifies the maximum absolute step size to be allowed in the
          time integration. If this option is not required, `algopt`[12]
          should be set to 0.0.

        `algopt`[13]

          Specifies the initial step size to be attempted by the
          integrator. If `algopt`[13] = 0.0, the initial step size is
          calculated internally.

        `algopt`[14]

          Specifies the maximum number of steps to be attempted by the
          integrator in any one call. If `algopt`[14] = 0.0, no limit is
          imposed.

        `algopt`[22]

          Specifies what method is to be used to solve the nonlinear
          equations at the initial point to initialize the values of U,
          U_t, V and V^.. If `algopt`[22] = 1.0, a modified Newton
          iteration is used and if `algopt`[22] = 2.0, functional
          iteration is used. The default value is `algopt`[22] = 1.0.

        `algopt`[28] and `algopt`[29] are used only for the sparse
        matrix algebra option, i.e., `laopt` = 'S'.

        `algopt`[28]

          Governs the choice of pivots during the decomposition of the
          first Jacobian matrix. It should lie in the range 0.0 <
          `algopt`[28] < 1.0, with smaller values biasing the algorithm
          towards maintaining sparsity at the expense of numerical
          stability. If `algopt`[28] lies outside the range then the
          default value is used. If the functions regard the Jacobian
          matrix as numerically singular, increasing `algopt`[28]
          towards 1.0 may help, but at the cost of increased fill-in.
          The default value is `algopt`[28] = 0.1.

        `algopt`[29]

          Used as the relative pivot threshold during subsequent
          Jacobian decompositions (see `algopt`[28]) below which an
          internal error is invoked. `algopt`[29] must be greater than
          zero, otherwise the default value is used. If `algopt`[29] is
          greater than 1.0 no check is made on the pivot size, and this
          may be a necessary option if the Jacobian matrix is found to
          be numerically singular (see `algopt`[28]). The default value
          is `algopt`[29] = 0.0001.

    remesh : bool
        Indicates whether or not spatial remeshing should be performed.

        `remesh` = True

          Indicates that spatial remeshing should be performed as
          specified.

        `remesh` = False

          Indicates that spatial remeshing should be suppressed.

        Note: `remesh` should not be changed between consecutive calls
        to ``dim1_parab_convdiff_remesh``. Remeshing can be switched off
        or on at specified times by using appropriate values for the
        arguments `nrmesh` and `trmesh` at each call.

    nxfix : int
        The number of fixed mesh points.

    xfix : float, shape (`nxfix`)
        `xfix`[i-1], for i = 1,2,...,`nxfix`, must contain the value of
        the x coordinate at the ith fixed mesh point.

    nrmesh : int
        Specifies the spatial remeshing frequency and criteria for the
        calculation and adoption of a new mesh.

        `nrmesh` < 0

          Indicates that a new mesh is adopted according to the argument
          `dxmesh`. The mesh is tested every |`nrmesh`| timesteps.

        `nrmesh` = 0

          Indicates that remeshing should take place just once at the
          end of the first time step reached when t > `trmesh`.

        `nrmesh` > 0

          Indicates that remeshing will take place every `nrmesh` time
          steps, with no testing using `dxmesh`.

        Note: `nrmesh` may be changed between consecutive calls to
        ``dim1_parab_convdiff_remesh`` to give greater flexibility over
        the times of remeshing.

    dxmesh : float
        Determines whether a new mesh is adopted when `nrmesh` is set
        less than zero. A possible new mesh is calculated at the end of
        every |`nrmesh`| time steps, but is adopted only if

            x_i^new > x_i^old+`dxmesh`*(x_i+1^old-x_i^old)

        or

            x_i^new < x_i^old-`dxmesh`*(x_i^old-x_i-1^old)

        `dxmesh` thus imposes a lower limit on the difference between
        one mesh and the next.

    trmesh : float
        Specifies when remeshing will take place when `nrmesh` is set to
        zero. Remeshing will occur just once at the end of the first
        time step reached when t is greater than `trmesh`.

        Note: `trmesh` may be changed between consecutive calls to
        ``dim1_parab_convdiff_remesh`` to force remeshing at several
        specified times.

    ipminf : int
        The level of trace information regarding the adaptive remeshing.
        Details are directed to the file object associated with the
        advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

        `ipminf` = 0

          No trace information.

        `ipminf` = 1

          Brief summary of mesh characteristics.

        `ipminf` = 2

          More detailed information, including old and new mesh points,
          mesh sizes and monitor function values.

    xratio : float
        An input bound on the adjacent mesh ratio (greater than 1.0 and
        typically in the range 1.5 to 3.0). The remeshing functions will
        attempt to ensure that

            (x_i-x_{i-1})/`xratio` < x_{i+1}-x_i <
            `xratio`*(x_i-x_{i-1}).

    con : float
        An input bound on the sub-integral of the monitor function
        F^mon(x) over each space step. The remeshing functions will
        attempt to ensure that

            int_x_i^x_{i+1}F^mon(x)dx <=
            `con`int_x_1^x_`npts`F^mon(x)dx,

        . `con` gives you more control over the mesh distribution, e.g.,
        decreasing `con` allows more clustering. A typical value is
        2.0/(`npts`-1), but you are encouraged to experiment with
        different values. Its value is not critical and the mesh should
        be qualitatively correct for all values in the range given
        below.

    monitf : None or callable monitf(t, npts, npde, x, u, fmon,
        data=None)
        `monitf` must supply and evaluate a remesh monitor function to
        indicate the solution behaviour of interest.

        Parameters
        ~~~~~~~~~~
        t : float
            The current value of the independent variable t.

        npts : int
            The number of mesh points in the interval [a,b].

        npde : int
            The number of PDEs in the system.

        x : float, ndarray, shape (`npts`)
            The current mesh. `x`[i-1] contains the value of x_i, for i
            = 1,2,...,`npts`.

        u : float, ndarray, shape (`npde`, `npts`)
            `u`[i-1,j-1] contains the value of U_i(x,t) at x = `x`[j-1]
            and time t, for j = 1,2,...,`npts`, for i = 1,2,...,`npde`.

        fmon : float, ndarray, shape (`npts`), to be modified in place
            On exit: `fmon`[i-1] must contain the value of the monitor
            function F^mon(x) at mesh point x = `x`[i-1].

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    comm : dict, communication object, modified in place
        Note: this argument will be (re-)initialized when it is an empty
        ``dict`` or under the following condition: `ind` = 0.

        Communication structure.

        On initial entry: need not be set.

    lrsave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['rsave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lrsave` is computed by the
        function.

        If your supplied `lrsave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lrsave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lrsave` set to at least the lower-bound value returned in
        `lrsave_min`.

    lisave : None or int
        When performing a new integration, the size to use for the
        communication array `comm`['isave'].

        Otherwise, the value is not referenced.

        An initial estimate for an adequate `lisave` is computed by the
        function.

        If your supplied `lisave` is None or too small, the estimated
        value will be used instead.

        In some cases the estimated value will be sufficient for
        continuation calls to the function.

        When `laopt` = 'S', even the function's initial estimated value
        of `lisave` may be too small.

        If so, the function returns with `errno` = 15.

        You are advised to call the function again with `ind` = 0 and
        `lisave` set to at least the lower-bound value returned in
        `lisave_min`.

    itask : int
        The task to be performed by the ODE integrator.

        `itask` = 1

          Normal computation of output values `u` at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step in the time direction and return.

        `itask` = 3

          Stop at first internal integration point at or beyond t =
          `tout`.

        `itask` = 4

          Normal computation of output values `u` at t = `tout` but
          without overshooting t = t_crit where t_crit is described
          under the argument `algopt`.

        `itask` = 5

          Take one step in the time direction and return, without
          passing t_crit, where t_crit is described under the argument
          `algopt`.

    itrace : int
        The level of trace information required from
        ``dim1_parab_convdiff_remesh`` and the underlying ODE solver.
        `itrace` may take the value -1, 0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages from the PDE solver are printed.

        `itrace` > 0

          Output from the underlying ODE solver is printed. This output
          contains details of Jacobian entries, the nonlinear iteration
          and the time integration during the computation of the ODE
          system.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases. You are advised to set `itrace` = 0, unless you are
        experienced with submodule ``ode``.

    ind : int
        Indicates whether this is a continuation call or a new
        integration.

        `ind` = 0

          Starts or restarts the integration in time.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the arguments `tout`, `nrmesh`
          and `trmesh` may be reset between calls to
          ``dim1_parab_convdiff_remesh``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t corresponding to the solution values in `u`.
        Normally `ts` = `tout`.

    ind : int
        `ind` = 1.

    lrsave_min : int
        A lower bound on the sufficient size for `comm`['rsave'].

    lisave_min : int
        A lower bound on the sufficient size for `comm`['isave'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, on initial entry `ind` = 1.

            Constraint: on initial entry `ind` = 0.

        (`errno` 1)
            On entry, `lrsave` = *<value>*.

            Constraint: `lrsave` >= *<value>*.

        (`errno` 1)
            On entry, `lisave` = *<value>*.

            Constraint: `lisave` >= *<value>*.

        (`errno` 1)
            On entry, the point `xfix`[i-1] does not coincide with any
            `x`[j-1]: i = *<value>* and `xfix`[i-1] = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xfix`[i] = *<value>* and
            `xfix`[i-1] = *<value>*.

            Constraint: `xfix`[i] > `xfix`[i-1].

        (`errno` 1)
            On entry, at least one point in `xi` lies outside
            [`x`[0],`x`[`npts`-1]]: `x`[0] = *<value>* and `x`[`npts`-1]
            = *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `xi`[i] = *<value>* and `xi`[i-1] =
            *<value>*.

            Constraint: `xi`[i] > `xi`[i-1].

        (`errno` 1)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: corresponding elements `atol`[i-1] and
            `rtol`[j-1] cannot both be 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `rtol`[i-1] = *<value>*.

            Constraint: `rtol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>* and `atol`[i-1] = *<value>*.

            Constraint: `atol`[i-1] >= 0.0.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `neqn` = *<value>*, `npde` = *<value>*, `npts` =
            *<value>* and `nv` = *<value>*.

            Constraint: `neqn` = `npde`*`npts`+`nv`.

        (`errno` 1)
            On entry, `con` = *<value>*, `npts` = *<value>*.

            Constraint: `con` <= 10.0/(`npts`-1).

        (`errno` 1)
            On entry, `con` = *<value>*, `npts` = *<value>*.

            Constraint: `con` >= 0.1/(`npts`-1).

        (`errno` 1)
            On entry, `xratio` = *<value>*.

            Constraint: `xratio` > 1.0.

        (`errno` 1)
            On entry, `ipminf` = *<value>*.

            Constraint: `ipminf` = 0, 1 or 2.

        (`errno` 1)
            On entry, `dxmesh` = *<value>*.

            Constraint: `dxmesh` >= 0.0.

        (`errno` 1)
            On entry, `nxfix` = *<value>*, `npts` = *<value>*.

            Constraint: `nxfix` <= `npts`-2.

        (`errno` 1)
            On entry, `nxfix` = *<value>*.

            Constraint: `nxfix` >= 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` = 0 when `nv` = 0.

        (`errno` 1)
            On entry, `nv` = *<value>* and `nxi` = *<value>*.

            Constraint: `nxi` >= 0 when `nv` > 0.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 3.

        (`errno` 1)
            On entry, `laopt` = *<value>*.

            Constraint: `laopt` = 'F', 'B' or 'S'.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1' or '2'.

        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0 or 1.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: `itask` = 1, 2, 3, 4 or 5.

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 0.

        (`errno` 1)
            On entry, `tout`-`ts` is too small: `tout` = *<value>* and
            `ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 4)
            In setting up the ODE system an internal auxiliary was
            unable to initialize the derivative. This could be due to
            your setting `ires` = 3 in `pdedef`, `numflx`, `bndary` or
            `odedef`.

        (`errno` 5)
            Singular Jacobian of ODE system. Check problem formulation.

        (`errno` 7)
            `atol` and `rtol` were too small to start integration.

        (`errno` 8)
            `ires` set to an invalid value in a call to functions
            `pdedef`, `numflx`, `bndary` or `odedef`.

        (`errno` 9)
            Serious error in internal call to an auxiliary. Increase
            `itrace` for further details.

        (`errno` 11)
            Error during Jacobian formulation for ODE system. Increase
            `itrace` for further details.

        (`errno` 12)
            In solving ODE system, the maximum number of steps
            `algopt`[14] has been exceeded. `algopt`[14] = *<value>*.

        (`errno` 14)
            The functions P, D, or C appear to depend on time
            derivatives.

        (`errno` 16)
            `remesh` has been changed between calls to
            ``dim1_parab_convdiff_remesh``.

        (`errno` 17)
            `fmon` is negative at one or more mesh points, or zero mesh
            spacing has been obtained due to a poor monitor function.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Underlying ODE solver cannot make further progress from the
            point `ts` with the supplied values of `atol` and `rtol`.
            `ts` = *<value>*.

        (`errno` 3)
            Repeated errors in an attempted step of underlying ODE
            solver. Integration was successful as far as `ts`: `ts` =
            *<value>*.

        (`errno` 6)
            In evaluating residual of ODE system, `ires` = 2 has been
            set in functions `pdedef`, `numflx`, `bndary` or `odedef`.
            Integration is successful as far as `ts`: `ts` = *<value>*.

        (`errno` 10)
            Integration completed, but small changes in `atol` or `rtol`
            are unlikely to result in a changed solution.

        (`errno` 13)
            Zero error weights encountered during time integration.

        (`errno` 15)
            When using the sparse option `lisave` or `lrsave` is too
            small: `lisave` = *<value>*, `lrsave` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_convdiff_remesh'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
        utils._BYREF_INT, # Input, np.
        utils._BYREF_INT, # Input, nptl.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, c[npde, np].
        utils._EngineFloat64ArrayType.c_type, # Output, d[npde, nptl].
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[npde].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if pdedef is None:
        pdedef_auxil_ctypes = utils._EngineState._get_symbol('d03plpt')
        pdedef_auxil_ctypes.restype = None
        pdedef_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, npde.
            utils._BYREF_FLOAT64, # Input, t.
            utils._EngineFloat64ArrayType.c_type, # Input, xv[nptl].
            utils._BYREF_INT, # Input, np.
            utils._BYREF_INT, # Input, nptl.
            utils._EngineFloat64ArrayType.c_type, # Input, u[npde, nptl].
            utils._EngineFloat64ArrayType.c_type, # Input, ut[npde].
            utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, nptl].
            utils._BYREF_INT, # Input, nv.
            utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
            utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
            utils._EngineFloat64ArrayType.c_type, # Output, p[npde, npde, np].
            utils._EngineFloat64ArrayType.c_type, # Output, c[npde, np].
            utils._EngineFloat64ArrayType.c_type, # Output, d[npde, nptl].
            utils._BYREF_INT, # Input, ns.
            utils._EngineFloat64ArrayType.c_type, # Output, s[npde].
            utils._BYREF_INT, # Input/Output, ires.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_pdedefh(_pdedef, en_data, npde, t, xv, np, nptl, u, ut, ux, nv, v, vdot, p, c, d, ns, s, ires, iuser, ruser):
        """Helper for pdedef."""
        try:
            if pdedef is None:
                pdedef_auxil_ctypes(
                    en_data,
                    npde,
                    t,
                    xv,
                    np,
                    nptl,
                    u,
                    ut,
                    ux,
                    nv,
                    v,
                    vdot,
                    p,
                    c,
                    d,
                    ns,
                    s,
                    ires,
                    iuser,
                    ruser,
                )
                return
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.get_element(xv, {'fun_name': cb_pdedef_name, 'entity': 'xv'}, (0,), validate=False)
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_pdedefh,),
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npde_pdedefh,),
            )
            _nv_pdedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_pdedefh,),
            )
            _vdot_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_pdedefh,),
            )
            _p_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_npde_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _c_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                c,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _d_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                d,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _s_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                s,
                (_npde_pdedefh,),
                can_reference_contents=False,
            )
            _ires_pdedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _c_pdedefh,
                    _d_pdedefh,
                    _s_pdedefh,
                    _ires_pdedefh,
                    data,
                )
            else:
                _ires_pdedefh = pdedef(
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _u_pdedefh,
                    _ux_pdedefh,
                    _nv_pdedefh,
                    _v_pdedefh,
                    _vdot_pdedefh,
                    _p_pdedefh,
                    _c_pdedefh,
                    _d_pdedefh,
                    _s_pdedefh,
                    _ires_pdedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _s_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 's'}, s,
            )
            utils._unlink_pointer(s)
            utils._EngineIntScalarType(
                _ires_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_numflx_name = fun_name + ':numflx'

    numflxh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, uleft[npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uright[npde].
        utils._EngineFloat64ArrayType.c_type, # Output, flux[npde].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
    )

    def cb_numflxh(_numflx, en_data, npde, t, x, nv, v, uleft, uright, flux, ires, _iuser, _ruser, iwsav):
        """Helper for numflx."""
        try:
            _npde_numflxh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_numflxh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_numflxh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _nv_numflxh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_numflxh,),
            )
            _uleft_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                uleft,
                (_npde_numflxh,),
            )
            _uright_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                uright,
                (_npde_numflxh,),
            )
            _flux_numflxh = utils._EngineFloat64ArrayType.from_pointer(
                flux,
                (_npde_numflxh,),
                can_reference_contents=False,
            )
            _ires_numflxh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_numflxh = numflx(
                    _npde_numflxh,
                    _t_numflxh,
                    _x_numflxh,
                    _nv_numflxh,
                    _v_numflxh,
                    _uleft_numflxh,
                    _uright_numflxh,
                    _flux_numflxh,
                    _ires_numflxh,
                    data,
                )
            else:
                _ires_numflxh = numflx(
                    _npde_numflxh,
                    _t_numflxh,
                    _x_numflxh,
                    _nv_numflxh,
                    _v_numflxh,
                    _uleft_numflxh,
                    _uright_numflxh,
                    _flux_numflxh,
                    _ires_numflxh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(uleft)
            utils._unlink_pointer(uright)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _flux_numflxh, {'fun_name': cb_numflx_name, 'returned': 'flux'}, flux,
            )
            utils._unlink_pointer(flux)
            utils._EngineIntScalarType(
                _ires_numflxh, {'fun_name': cb_numflx_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, sdu].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npde, sdux].
        utils._BYREF_INT, # Input, sdu.
        utils._BYREF_INT, # Input, sdux.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, ibnd.
        utils._BYREF_INT, # Input, nbeta.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[nbeta].
        utils._BYREF_INT, # Input, ngamma.
        utils._EngineFloat64ArrayType.c_type, # Output, g[ngamma].
        utils._BYREF_INT, # Input, nres.
        utils._EngineFloat64ArrayType.c_type, # Output, res[nres].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npde, npts, t, x, u, ux, sdu, sdux, nv, v, vdot, ibnd, nbeta, beta, ngamma, g, nres, res, ires, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_bndaryh = utils._EngineIntScalarType.to_py(npts.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_bndaryh,),
            )
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_bndaryh, _npts_bndaryh),
                sorder=utils._EngineData.default_sorder,
            )
            _nv_bndaryh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_bndaryh,),
            )
            _vdot_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_bndaryh,),
            )
            _ibnd_bndaryh = utils._EngineIntScalarType.to_py(ibnd.contents)
            _g_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                g,
                (_npde_bndaryh,),
                can_reference_contents=False,
            )
            _ires_bndaryh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _npts_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _u_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _g_bndaryh,
                    _ires_bndaryh,
                    data,
                )
            else:
                _ires_bndaryh = bndary(
                    _npde_bndaryh,
                    _npts_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _u_bndaryh,
                    _nv_bndaryh,
                    _v_bndaryh,
                    _vdot_bndaryh,
                    _ibnd_bndaryh,
                    _g_bndaryh,
                    _ires_bndaryh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._EngineIntScalarType(
                _ires_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_uvinit_name = fun_name + ':uvinit'

    uvinith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, u[npde, npts].
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Output, v[nv].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_uvinith(_uvinit, en_data, npde, npts, nxi, x, xi, u, nv, v, _iuser, _ruser):
        """Helper for uvinit."""
        try:
            _npde_uvinith = utils._EngineIntScalarType.to_py(npde.contents)
            _npts_uvinith = utils._EngineIntScalarType.to_py(npts.contents)
            _nxi_uvinith = utils._EngineIntScalarType.to_py(nxi.contents)
            _x_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_uvinith,),
            )
            _xi_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_uvinith,),
            )
            _u_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_uvinith, _npts_uvinith),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _nv_uvinith = utils._EngineIntScalarType.to_py(nv.contents)
            _v_uvinith = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_uvinith,),
                can_reference_contents=False,
            )
            if data is not None:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _nxi_uvinith,
                    _x_uvinith,
                    _xi_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                    data,
                )
            else:
                uvinit(
                    _npde_uvinith,
                    _npts_uvinith,
                    _nxi_uvinith,
                    _x_uvinith,
                    _xi_uvinith,
                    _u_uvinith,
                    _nv_uvinith,
                    _v_uvinith,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(xi)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _u_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'u'}, u,
            )
            utils._unlink_pointer(u)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _v_uvinith, {'fun_name': cb_uvinit_name, 'returned': 'v'}, v,
            )
            utils._unlink_pointer(v)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_odedef_name = fun_name + ':odedef'

    odedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, nv.
        utils._EngineFloat64ArrayType.c_type, # Input, v[nv].
        utils._EngineFloat64ArrayType.c_type, # Input, vdot[nv].
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, rcp[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucpt[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Input, ucptx[npde, nxi].
        utils._EngineFloat64ArrayType.c_type, # Output, r[nv].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_odedefh(_odedef, en_data, npde, t, nv, v, vdot, nxi, xi, ucp, ucpx, rcp, ucpt, ucptx, r, ires, _iuser, _ruser):
        """Helper for odedef."""
        if odedef is None:
            return
        try:
            _npde_odedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_odedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _nv_odedefh = utils._EngineIntScalarType.to_py(nv.contents)
            _v_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (_nv_odedefh,),
            )
            _vdot_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                vdot,
                (_nv_odedefh,),
            )
            _nxi_odedefh = utils._EngineIntScalarType.to_py(nxi.contents)
            _xi_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                xi,
                (_nxi_odedefh,),
            )
            _ucp_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucp,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpx_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpx,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _ucpt_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                ucpt,
                (_npde_odedefh, _nxi_odedefh),
                sorder=utils._EngineData.default_sorder,
            )
            _r_odedefh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_nv_odedefh,),
                can_reference_contents=False,
            )
            _ires_odedefh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                    data,
                )
            else:
                _ires_odedefh = odedef(
                    _npde_odedefh,
                    _t_odedefh,
                    _nv_odedefh,
                    _v_odedefh,
                    _vdot_odedefh,
                    _nxi_odedefh,
                    _xi_odedefh,
                    _ucp_odedefh,
                    _ucpx_odedefh,
                    _ucpt_odedefh,
                    _r_odedefh,
                    _ires_odedefh,
                )
            utils._unlink_pointer(v)
            utils._unlink_pointer(vdot)
            utils._unlink_pointer(xi)
            utils._unlink_pointer(ucp)
            utils._unlink_pointer(ucpx)
            utils._unlink_pointer(ucpt)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_odedefh, {'fun_name': cb_odedef_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_odedefh, {'fun_name': cb_odedef_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitf_name = fun_name + ':monitf'

    monitfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, npts].
        utils._BYREF_INT, # Input, nr.
        utils._EngineFloat64ArrayType.c_type, # Input, r[nr].
        utils._EngineFloat64ArrayType.c_type, # Output, fmon[npts].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monitfh(_monitf, en_data, t, npts, npde, x, u, nr, r, fmon, _iuser, _ruser):
        """Helper for monitf."""
        if monitf is None:
            return
        try:
            _t_monitfh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _npts_monitfh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_monitfh = utils._EngineIntScalarType.to_py(npde.contents)
            _x_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_monitfh,),
            )
            _u_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npde_monitfh, _npts_monitfh),
                sorder=utils._EngineData.default_sorder,
            )
            _fmon_monitfh = utils._EngineFloat64ArrayType.from_pointer(
                fmon,
                (_npts_monitfh,),
                can_reference_contents=False,
            )
            if data is not None:
                monitf(
                    _t_monitfh,
                    _npts_monitfh,
                    _npde_monitfh,
                    _x_monitfh,
                    _u_monitfh,
                    _fmon_monitfh,
                    data,
                )
            else:
                monitf(
                    _t_monitfh,
                    _npts_monitfh,
                    _npde_monitfh,
                    _x_monitfh,
                    _u_monitfh,
                    _fmon_monitfh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(u)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fmon_monitfh, {'fun_name': cb_monitf_name, 'returned': 'fmon'}, fmon,
            )
            utils._unlink_pointer(fmon)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03psft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        numflxh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._UNUSED_FUNCTYPE,
        uvinith,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[neqn].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[npts].
        utils._BYREF_INT, # Input, nv.
        utils._UNUSED_FUNCTYPE,
        odedefh,
        utils._BYREF_INT, # Input, nxi.
        utils._EngineFloat64ArrayType.c_type, # Input, xi[nxi].
        utils._BYREF_INT, # Input, neqn.
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._BYREF_STRING, # Input, laopt(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, algopt[30].
        utils._BYREF_LOGICAL, # Input, remesh.
        utils._BYREF_INT, # Input, nxfix.
        utils._EngineFloat64ArrayType.c_type, # Input, xfix[nxfix].
        utils._BYREF_INT, # Input, nrmesh.
        utils._BYREF_FLOAT64, # Input, dxmesh.
        utils._BYREF_FLOAT64, # Input, trmesh.
        utils._BYREF_INT, # Input, ipminf.
        utils._BYREF_FLOAT64, # Input, xratio.
        utils._BYREF_FLOAT64, # Input, con.
        utils._UNUSED_FUNCTYPE,
        monitfh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineIntArrayType.c_type, # Input/Output, isave[lisave].
        utils._BYREF_INT, # Input, lisave.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Output, lrsave_min.
        utils._BYREF_INT, # Output, lisave_min.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of laopt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'}, allow_none=True)
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(numflx, {'fun_name': fun_name, 'entity': 'numflx'})
    _numflx = utils._UNUSED_CALLBACK
    _numflxh = utils._cast_callback(
        numflxh, cb_numflxh,
        {'fun_name': fun_name, 'entity': 'numflx'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    utils._check_callable(uvinit, {'fun_name': fun_name, 'entity': 'uvinit'})
    _uvinit = utils._UNUSED_CALLBACK
    _uvinith = utils._cast_callback(
        uvinith, cb_uvinith,
        {'fun_name': fun_name, 'entity': 'uvinit'},
    )
    _neqn = utils._EngineIntScalarType(
        neqn, {'fun_name': fun_name, 'entity': 'neqn'},
    ).to_ctypes()
    _neqn_py = utils._EngineIntScalarType.to_py(_neqn)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_neqn_py,),
    )
    _u = _u_cls.to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    )
    _x = _x_cls.to_ctypes()
    _nv = utils._EngineIntScalarType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
    ).to_ctypes()
    utils._check_callable(odedef, {'fun_name': fun_name, 'entity': 'odedef'}, allow_none=True)
    _odedef = utils._UNUSED_CALLBACK
    _odedefh = utils._cast_callback(
        odedefh, cb_odedefh,
        {'fun_name': fun_name, 'entity': 'odedef'},
    )
    _nxi = utils._EngineIntScalarType(
        nxi, {'fun_name': fun_name, 'entity': 'nxi'},
    ).to_ctypes()
    _nxi_py = utils._EngineIntScalarType.to_py(_nxi)
    _xi = utils._EngineFloat64ArrayType(
        xi, {'fun_name': fun_name, 'entity': 'xi'},
        exp_shape=(_nxi_py,),
    ).to_ctypes()
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    elif _itol_py in (3, 4):
        _rtol_dim1 = _neqn_py
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    elif _itol_py in (2, 4):
        _atol_dim1 = _neqn_py
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _laopt = utils._EngineCharScalarType(
        laopt, {'fun_name': fun_name, 'entity': 'laopt'}, exp_chrlen=1,
    ).to_ctypes()
    _algopt = utils._EngineFloat64ArrayType(
        algopt, {'fun_name': fun_name, 'entity': 'algopt'},
        exp_shape=(30,),
    ).to_ctypes()
    _remesh = utils._EngineLogicalScalarType(
        remesh, {'fun_name': fun_name, 'entity': 'remesh'},
    ).to_ctypes()
    _nxfix = utils._EngineIntScalarType(
        nxfix, {'fun_name': fun_name, 'entity': 'nxfix'},
    ).to_ctypes()
    _nxfix_py = utils._EngineIntScalarType.to_py(_nxfix)
    _xfix = utils._EngineFloat64ArrayType(
        xfix, {'fun_name': fun_name, 'entity': 'xfix'},
        exp_shape=(_nxfix_py,),
    ).to_ctypes()
    _nrmesh = utils._EngineIntScalarType(
        nrmesh, {'fun_name': fun_name, 'entity': 'nrmesh'},
    ).to_ctypes()
    _dxmesh = utils._EngineFloat64ScalarType(
        dxmesh, {'fun_name': fun_name, 'entity': 'dxmesh'},
    ).to_ctypes()
    _trmesh = utils._EngineFloat64ScalarType(
        trmesh, {'fun_name': fun_name, 'entity': 'trmesh'},
    ).to_ctypes()
    _ipminf = utils._EngineIntScalarType(
        ipminf, {'fun_name': fun_name, 'entity': 'ipminf'},
    ).to_ctypes()
    _xratio = utils._EngineFloat64ScalarType(
        xratio, {'fun_name': fun_name, 'entity': 'xratio'},
    ).to_ctypes()
    _con = utils._EngineFloat64ScalarType(
        con, {'fun_name': fun_name, 'entity': 'con'},
    ).to_ctypes()
    utils._check_callable(monitf, {'fun_name': fun_name, 'entity': 'monitf'}, allow_none=True)
    _monitf = utils._UNUSED_CALLBACK
    _monitfh = utils._cast_callback(
        monitfh, cb_monitfh,
        {'fun_name': fun_name, 'entity': 'monitf'},
    )
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _comm_needs_init = (
        utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rsave', 'isave', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
        ) or
        _ind_py == 0
    )
    if _comm_needs_init:
        if lrsave is None:
            _lrsave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lrsave = utils._EngineIntScalarType(
                lrsave, {'fun_name': fun_name, 'entity': 'lrsave'},
            ).to_ctypes()
        _lrsave_py = utils._EngineIntScalarType.to_py(_lrsave)
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
    else:
        _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = comm['rsave']
    if _comm_needs_init:
        if lisave is None:
            _lisave = utils._EngineIntScalarType.empty_ctypes()
        else:
            _lisave = utils._EngineIntScalarType(
                lisave, {'fun_name': fun_name, 'entity': 'lisave'},
            ).to_ctypes()
        _lisave_py = utils._EngineIntScalarType.to_py(_lisave)
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
    else:
        _lisave_py = utils._EngineIntArrayType.get_shape(comm['isave'], {'fun_name': fun_name, 'entity': 'comm[\'isave\']'}, exp_rank=1, validate=False)[0]
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = comm['isave']
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iemode_py = 1 if _comm_needs_init else 0
    _iemode = utils._EngineIntScalarType(
        _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
    ).to_ctypes()
    _lrsave_min = utils._EngineIntScalarType.empty_ctypes()
    _lisave_min = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
        comm['cwsav'] = _cwsav
    else:
        _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _pdedef,
        _pdedefh,
        _numflx,
        _numflxh,
        _bndary,
        _bndaryh,
        _uvinit,
        _uvinith,
        _u,
        _npts,
        _x,
        _nv,
        _odedef,
        _odedefh,
        _nxi,
        _xi,
        _neqn,
        _rtol,
        _atol,
        _itol,
        _norm,
        _laopt,
        _algopt,
        _remesh,
        _nxfix,
        _xfix,
        _nrmesh,
        _dxmesh,
        _trmesh,
        _ipminf,
        _xratio,
        _con,
        _monitf,
        _monitfh,
        _rsave,
        _lrsave,
        _isave,
        _lisave,
        _itask,
        _itrace,
        _nadv,
        _ind,
        _iemode,
        _lrsave_min,
        _lisave_min,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _norm._length_,
        _laopt._length_,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS and _iemode_py == 1:
        _iemode_py = 0
        _iemode = utils._EngineIntScalarType(
            _iemode_py, {'fun_name': fun_name, 'entity': 'iemode'},
        ).to_ctypes()
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lrsave_py = max(_lrsave_min_py, _lrsave_py)
        _lrsave = utils._EngineIntScalarType(
            _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave_py'},
        ).to_ctypes()
        _rsave = utils._EngineFloat64ArrayType.empty_ctypes(_lrsave_py)
        comm['rsave'] = _rsave
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        _lisave_py = max(_lisave_min_py, _lisave_py)
        _lisave = utils._EngineIntScalarType(
            _lisave_py, {'fun_name': fun_name, 'entity': '_lisave_py'},
        ).to_ctypes()
        _isave = utils._EngineIntArrayType.empty_ctypes(_lisave_py)
        comm['isave'] = _isave
        engine_ctypes(
            _en_data,
            utils._UNUSED_PRINT_REC,
            _print_rech,
            _npde,
            _ts,
            _tout,
            _pdedef,
            _pdedefh,
            _numflx,
            _numflxh,
            _bndary,
            _bndaryh,
            _uvinit,
            _uvinith,
            _u,
            _npts,
            _x,
            _nv,
            _odedef,
            _odedefh,
            _nxi,
            _xi,
            _neqn,
            _rtol,
            _atol,
            _itol,
            _norm,
            _laopt,
            _algopt,
            _remesh,
            _nxfix,
            _xfix,
            _nrmesh,
            _dxmesh,
            _trmesh,
            _ipminf,
            _xratio,
            _con,
            _monitf,
            _monitfh,
            _rsave,
            _lrsave,
            _isave,
            _lisave,
            _itask,
            _itrace,
            _nadv,
            _ind,
            _iemode,
            _lrsave_min,
            _lisave_min,
            _iuser,
            _ruser,
            _cwsav,
            _lwsav,
            _iwsav,
            _rwsav,
            _errbuf,
            _ierr,
            _norm._length_,
            _laopt._length_,
            80,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _nv_py = utils._EngineIntScalarType.to_py(_nv)
        if _ierr_py == 1 and _ifmt == 99953:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, on initial entry ind = 1.',
                    'Constraint: on initial entry ind = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrsave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lisave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lisave >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the point xfix[i-1] does not coincide with any x[j-1]:',
                    'i = {:s} and xfix[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xfix[i] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and xfix[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xfix[i] > xfix[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one point in xi lies outside [x[0],x[npts-1]]:',
                    'x[0] = {:s} and x[npts-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xi[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xi[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xi[i] > xi[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: corresponding elements atol[i-1] and rtol[j-1]',
                    'cannot both be 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and rtol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and atol[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqn = {:s}, npde = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npts = {:s} and nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: neqn = npde*npts+nv.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con <= 10.0/(npts-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con >= 0.1/(npts-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xratio = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xratio > 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ipminf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ipminf = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dxmesh = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dxmesh >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxfix = {:s}, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nxfix <= npts-2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxfix = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxfix >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi = 0 when nv = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s} and nxi = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nv_py),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nxi >= 0 when nv > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laopt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: laopt = \'F\', \'B\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'1\' or \'2\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itask = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts is too small:',
                    'tout = {:s} and ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underlying ODE solver cannot make further progress from the point',
                    'ts with the supplied values of atol and rtol.',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated errors in an attempted step of underlying ODE solver.',
                    'Integration was successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In setting up the ODE system an internal auxiliary was unable to',
                    'initialize the derivative.',
                    'This could be due to your setting ires = 3 in pdedef, numflx,',
                    'bndary or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99965:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular Jacobian of ODE system. Check problem formulation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99964:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In evaluating residual of ODE system, ires = 2 has been set in',
                    'functions pdedef, numflx, bndary or odedef.',
                    'Integration is successful as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'atol and rtol were too small to start integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires set to an invalid value in a call to functions pdedef,',
                    'numflx, bndary or odedef.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Serious error in internal call to an auxiliary.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but small changes in atol or rtol are unlikely',
                    'to result in a changed solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99959:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error during Jacobian formulation for ODE system.',
                    'Increase itrace for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99958:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In solving ODE system, the maximum number of steps',
                    'algopt[14] has been exceeded.',
                    'algopt[14] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99957:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Zero error weights encountered during time integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99956:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The functions P, D, or C appear to depend on time derivatives.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99955:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'When using the sparse option lisave or lrsave is too small:',
                    'lisave = {:s}, lrsave = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'remesh has been changed between calls to dim1_parab_convdiff_remesh.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 99954:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'fmon is negative at one or more mesh points, or zero mesh',
                    'spacing has been obtained due to a poor monitor function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _u_cls.refresh_py(_u)
        _x_cls.refresh_py(_x)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _lrsave_min_py = utils._EngineIntScalarType.to_py(_lrsave_min)
        _lisave_min_py = utils._EngineIntScalarType.to_py(_lisave_min)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind', 'lrsave_min', 'lisave_min'],
        )(
            _ts_py,
            _ind_py,
            _lrsave_min_py,
            _lisave_min_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_parab_euler_roe(uleft, uright, gamma, flux, comm):
    u"""
    Roe's approximate Riemann solver for Euler equations in conservative
    form, for use with ``dim1_parab_convdiff``,
    ``dim1_parab_convdiff_dae`` and ``dim1_parab_convdiff_remesh``.

    For full information please refer to the NAG Library document for
    d03pu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03puf.html

    Parameters
    ----------
    uleft : float, shape (3)
        `uleft`[i-1] must contain the left value of the component U_i,
        for i = 1,2,...,3. That is, `uleft`[0] must contain the left
        value of rho, `uleft`[1] must contain the left value of m and
        `uleft`[2] must contain the left value of e.

    uright : float, shape (3)
        `uright`[i-1] must contain the right value of the component U_i,
        for i = 1,2,...,3. That is, `uright`[0] must contain the right
        value of rho, `uright`[1] must contain the right value of m and
        `uright`[2] must contain the right value of e.

    gamma : float
        The ratio of specific heats, gamma.

    flux : float, shape (3), modified in place
        On exit: `flux`[i-1] contains the numerical flux component F^_i,
        for i = 1,2,...,3.

    comm : dict, communication object
        Communication structure.

        On initial entry: need not be set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `gamma` = *<value>*.

            Constraint: `gamma` > 0.0.

        (`errno` 2)
            Right pressure value pr < 0.0: pr = *<value>*.

        (`errno` 2)
            Left pressure value pl < 0.0: pl = *<value>*.

        (`errno` 2)
            On entry, `uright`[0] = *<value>*.

            Constraint: `uright`[0] >= 0.0.

        (`errno` 2)
            On entry, `uleft`[0] = *<value>*.

            Constraint: `uleft`[0] >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_euler_roe'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03puft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, uleft[3].
        utils._EngineFloat64ArrayType.c_type, # Input, uright[3].
        utils._BYREF_FLOAT64, # Input, gamma.
        utils._EngineFloat64ArrayType.c_type, # Output, flux[3].
        utils._EngineIntArrayType.c_type, # Input, iwsav[505].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uleft = utils._EngineFloat64ArrayType(
        uleft, {'fun_name': fun_name, 'entity': 'uleft'},
        exp_shape=(3,),
    ).to_ctypes()
    _uright = utils._EngineFloat64ArrayType(
        uright, {'fun_name': fun_name, 'entity': 'uright'},
        exp_shape=(3,),
    ).to_ctypes()
    _gamma = utils._EngineFloat64ScalarType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
    ).to_ctypes()
    _flux_cls = utils._EngineFloat64ArrayType(
        flux, {'fun_name': fun_name, 'entity': 'flux'},
        exp_shape=(3,),
        check_contents=False,
    )
    _flux = _flux_cls.to_ctypes(can_reference_contents=False)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav',),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uleft,
        _uright,
        _gamma,
        _flux,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, gamma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: gamma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Right pressure value pr < 0.0: pr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Left pressure value pl < 0.0: pl = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uright[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uright[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uleft[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uleft[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _flux_cls.refresh_py(_flux)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_parab_euler_osher(uleft, uright, gamma, path, flux, comm):
    u"""
    Osher's approximate Riemann solver for Euler equations in
    conservative form, for use with ``dim1_parab_convdiff``,
    ``dim1_parab_convdiff_dae`` and ``dim1_parab_convdiff_remesh``.

    For full information please refer to the NAG Library document for
    d03pv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pvf.html

    Parameters
    ----------
    uleft : float, shape (3)
        `uleft`[i-1] must contain the left value of the component U_i,
        for i = 1,2,...,3. That is, `uleft`[0] must contain the left
        value of rho, `uleft`[1] must contain the left value of m and
        `uleft`[2] must contain the left value of e.

    uright : float, shape (3)
        `uright`[i-1] must contain the right value of the component U_i,
        for i = 1,2,...,3. That is, `uright`[0] must contain the right
        value of rho, `uright`[1] must contain the right value of m and
        `uright`[2] must contain the right value of e.

    gamma : float
        The ratio of specific heats, gamma.

    path : str, length 1
        The variant of the Osher scheme.

        `path` = 'O'

          Original.

        `path` = 'P'

          Physical.

    flux : float, shape (3), modified in place
        On exit: `flux`[i-1] contains the numerical flux component F^_i,
        for i = 1,2,...,3.

    comm : dict, communication object
        Communication structure.

        On initial entry: need not be set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `gamma` = *<value>*.

            Constraint: `gamma` > 0.0.

        (`errno` 1)
            On entry, `path` = *<value>*.

            Constraint: `path` = 'O' or 'P'.

        (`errno` 2)
            Right pressure value pr < 0.0: pr = *<value>*.

        (`errno` 2)
            Left pressure value pl < 0.0: pl = *<value>*.

        (`errno` 2)
            On entry, `uright`[0] = *<value>*.

            Constraint: `uright`[0] >= 0.0.

        (`errno` 2)
            On entry, `uleft`[0] = *<value>*.

            Constraint: `uleft`[0] >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_euler_osher'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03pvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, uleft[3].
        utils._EngineFloat64ArrayType.c_type, # Input, uright[3].
        utils._BYREF_FLOAT64, # Input, gamma.
        utils._BYREF_STRING, # Input, path(len=1).
        utils._EngineFloat64ArrayType.c_type, # Output, flux[3].
        utils._EngineIntArrayType.c_type, # Input, iwsav[505].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of path.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uleft = utils._EngineFloat64ArrayType(
        uleft, {'fun_name': fun_name, 'entity': 'uleft'},
        exp_shape=(3,),
    ).to_ctypes()
    _uright = utils._EngineFloat64ArrayType(
        uright, {'fun_name': fun_name, 'entity': 'uright'},
        exp_shape=(3,),
    ).to_ctypes()
    _gamma = utils._EngineFloat64ScalarType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
    ).to_ctypes()
    _path = utils._EngineCharScalarType(
        path, {'fun_name': fun_name, 'entity': 'path'}, exp_chrlen=1,
    ).to_ctypes()
    _flux_cls = utils._EngineFloat64ArrayType(
        flux, {'fun_name': fun_name, 'entity': 'flux'},
        exp_shape=(3,),
        check_contents=False,
    )
    _flux = _flux_cls.to_ctypes(can_reference_contents=False)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav',),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uleft,
        _uright,
        _gamma,
        _path,
        _flux,
        _iwsav,
        _errbuf,
        _ierr,
        _path._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, gamma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: gamma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, path = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: path = \'O\' or \'P\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Right pressure value pr < 0.0: pr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Left pressure value pl < 0.0: pl = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uright[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uright[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uleft[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uleft[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _flux_cls.refresh_py(_flux)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_parab_euler_hll(uleft, uright, gamma, flux, comm):
    u"""
    Modified HLL Riemann solver for Euler equations in conservative
    form, for use with ``dim1_parab_convdiff``,
    ``dim1_parab_convdiff_dae`` and ``dim1_parab_convdiff_remesh``.

    For full information please refer to the NAG Library document for
    d03pw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pwf.html

    Parameters
    ----------
    uleft : float, shape (3)
        `uleft`[i-1] must contain the left value of the component U_i,
        for i = 1,2,...,3. That is, `uleft`[0] must contain the left
        value of rho, `uleft`[1] must contain the left value of m and
        `uleft`[2] must contain the left value of e.

    uright : float, shape (3)
        `uright`[i-1] must contain the right value of the component U_i,
        for i = 1,2,...,3. That is, `uright`[0] must contain the right
        value of rho, `uright`[1] must contain the right value of m and
        `uright`[2] must contain the right value of e.

    gamma : float
        The ratio of specific heats, gamma.

    flux : float, shape (3), modified in place
        On exit: `flux`[i-1] contains the numerical flux component F^_i,
        for i = 1,2,...,3.

    comm : dict, communication object
        Communication structure.

        On initial entry: need not be set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `gamma` = *<value>*.

            Constraint: `gamma` > 0.0.

        (`errno` 2)
            Right pressure value pr < 0.0: pr = *<value>*.

        (`errno` 2)
            Left pressure value pl < 0.0: pl = *<value>*.

        (`errno` 2)
            On entry, `uright`[0] = *<value>*.

            Constraint: `uright`[0] >= 0.0.

        (`errno` 2)
            On entry, `uleft`[0] = *<value>*.

            Constraint: `uleft`[0] >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_euler_hll'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03pwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, uleft[3].
        utils._EngineFloat64ArrayType.c_type, # Input, uright[3].
        utils._BYREF_FLOAT64, # Input, gamma.
        utils._EngineFloat64ArrayType.c_type, # Output, flux[3].
        utils._EngineIntArrayType.c_type, # Input, iwsav[505].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uleft = utils._EngineFloat64ArrayType(
        uleft, {'fun_name': fun_name, 'entity': 'uleft'},
        exp_shape=(3,),
    ).to_ctypes()
    _uright = utils._EngineFloat64ArrayType(
        uright, {'fun_name': fun_name, 'entity': 'uright'},
        exp_shape=(3,),
    ).to_ctypes()
    _gamma = utils._EngineFloat64ScalarType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
    ).to_ctypes()
    _flux_cls = utils._EngineFloat64ArrayType(
        flux, {'fun_name': fun_name, 'entity': 'flux'},
        exp_shape=(3,),
        check_contents=False,
    )
    _flux = _flux_cls.to_ctypes(can_reference_contents=False)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav',),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uleft,
        _uright,
        _gamma,
        _flux,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, gamma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: gamma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Right pressure value pr < 0.0: pr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Left pressure value pl < 0.0: pl = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uright[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uright[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uleft[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uleft[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _flux_cls.refresh_py(_flux)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_parab_euler_exact(uleft, uright, gamma, tol, niter, flux, comm):
    u"""
    Exact Riemann solver for Euler equations in conservative form, for
    use with ``dim1_parab_convdiff``, ``dim1_parab_convdiff_dae`` and
    ``dim1_parab_convdiff_remesh``.

    For full information please refer to the NAG Library document for
    d03px

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pxf.html

    Parameters
    ----------
    uleft : float, shape (3)
        `uleft`[i-1] must contain the left value of the component U_i,
        for i = 1,2,...,3. That is, `uleft`[0] must contain the left
        value of rho, `uleft`[1] must contain the left value of m and
        `uleft`[2] must contain the left value of e.

    uright : float, shape (3)
        `uright`[i-1] must contain the right value of the component U_i,
        for i = 1,2,...,3. That is, `uright`[0] must contain the right
        value of rho, `uright`[1] must contain the right value of m and
        `uright`[2] must contain the right value of e.

    gamma : float
        The ratio of specific heats, gamma.

    tol : float
        The tolerance to be used in the Newton--Raphson procedure to
        calculate the pressure. If `tol` is set to zero then the default
        value of 1.0*10^(-6) is used.

    niter : int
        The maximum number of Newton--Raphson iterations allowed. If
        `niter` is set to zero then the default value of 20 is used.

    flux : float, shape (3), modified in place
        On exit: `flux`[i-1] contains the numerical flux component F^_i,
        for i = 1,2,...,3.

    comm : dict, communication object
        Communication structure.

        On initial entry: need not be set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `niter` = *<value>*.

            Constraint: `niter` >= 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, `gamma` = *<value>*.

            Constraint: `gamma` > 0.0.

        (`errno` 2)
            Right pressure value pr < 0.0: pr = *<value>*.

        (`errno` 2)
            Left pressure value pl < 0.0: pl = *<value>*.

        (`errno` 2)
            On entry, `uright`[0] = *<value>*.

            Constraint: `uright`[0] >= 0.0.

        (`errno` 2)
            On entry, `uleft`[0] = *<value>*.

            Constraint: `uleft`[0] >= 0.0.

        (`errno` 3)
            A vacuum condition has been detected.

        (`errno` 4)
            Newton--Raphson iteration failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_euler_exact'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03pxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, uleft[3].
        utils._EngineFloat64ArrayType.c_type, # Input, uright[3].
        utils._BYREF_FLOAT64, # Input, gamma.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, niter.
        utils._EngineFloat64ArrayType.c_type, # Output, flux[3].
        utils._EngineIntArrayType.c_type, # Input, iwsav[505].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uleft = utils._EngineFloat64ArrayType(
        uleft, {'fun_name': fun_name, 'entity': 'uleft'},
        exp_shape=(3,),
    ).to_ctypes()
    _uright = utils._EngineFloat64ArrayType(
        uright, {'fun_name': fun_name, 'entity': 'uright'},
        exp_shape=(3,),
    ).to_ctypes()
    _gamma = utils._EngineFloat64ScalarType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _niter = utils._EngineIntScalarType(
        niter, {'fun_name': fun_name, 'entity': 'niter'},
    ).to_ctypes()
    _flux_cls = utils._EngineFloat64ArrayType(
        flux, {'fun_name': fun_name, 'entity': 'flux'},
        exp_shape=(3,),
        check_contents=False,
    )
    _flux = _flux_cls.to_ctypes(can_reference_contents=False)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav',),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uleft,
        _uright,
        _gamma,
        _tol,
        _niter,
        _flux,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, niter = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: niter >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, gamma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: gamma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Right pressure value pr < 0.0: pr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Left pressure value pl < 0.0: pl = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uright[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uright[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uleft[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: uleft[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A vacuum condition has been detected.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Newton--Raphson iteration failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _flux_cls.refresh_py(_flux)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_parab_coll_interp(npde, u, nbkpts, xbkpts, npoly, npts, xp, intpts, itype, up, comm):
    u"""
    PDEs, spatial interpolation with ``dim1_parab_coll`` or
    ``dim1_parab_dae_coll``.

    For full information please refer to the NAG Library document for
    d03py

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pyf.html

    Parameters
    ----------
    npde : int
        The number of PDEs.

    u : float, shape (`npde`, `npts`)
        The PDE part of the original solution returned in the argument
        `u` by the function ``dim1_parab_coll`` or
        ``dim1_parab_dae_coll``.

    nbkpts : int
        The number of break-points.

    xbkpts : float, shape (`nbkpts`)
        `xbkpts`[i-1], for i = 1,2,...,`nbkpts`, must contain the
        break-points as used by ``dim1_parab_coll`` or
        ``dim1_parab_dae_coll``.

    npoly : int
        The degree of the Chebyshev polynomial used for approximation as
        used by ``dim1_parab_coll`` or ``dim1_parab_dae_coll``.

    npts : int
        The number of mesh points as used by ``dim1_parab_coll`` or
        ``dim1_parab_dae_coll``.

    xp : float, shape (`intpts`)
        `xp`[i-1], for i = 1,2,...,`intpts`, must contain the spatial
        interpolation points.

    intpts : int
        The number of interpolation points.

    itype : int
        Specifies the interpolation to be performed.

        `itype` = 1

          The solution at the interpolation points are computed.

        `itype` = 2

          Both the solution and the first derivative at the
          interpolation points are computed.

    up : float, shape (`npde`, `intpts`, `itype`), modified in place
        On exit: if `itype` = 1, `up`[i-1,j-1,0], contains the value of
        the solution U_i(x_j,t_out), at the interpolation points x_j =
        `xp`[j-1], for i = 1,2,...,`npde`, for j = 1,2,...,`intpts`.

        If `itype` = 2, `up`[i-1,j-1,0] contains U_i(x_j,t_out) and
        `up`[i-1,j-1,1] contains dU_i/dx at these points.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``dim1_parab_coll`` or ``dim1_parab_dae_coll``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `itype` = *<value>*.

            Constraint: `itype` = 1 or 2.

        (`errno` 1)
            On entry, i = *<value>*, `xbkpts`[i-1] = *<value>*, j =
            *<value>* and `xbkpts`[j-1] = *<value>*.

            Constraint: `xbkpts`[0] < `xbkpts`[1] < ... <
            `xbkpts`[`nbkpts`-1].

        (`errno` 1)
            On entry, `npts` = *<value>*, `nbkpts` = *<value>* and
            `npoly` = *<value>*.

            Constraint: `npts` = (`nbkpts`-1)*`npoly`+1.

        (`errno` 1)
            On entry, `intpts` = *<value>*.

            Constraint: `intpts` >= 1.

        (`errno` 1)
            On entry, `npoly` = *<value>*.

            Constraint: `npoly` > 0.

        (`errno` 1)
            On entry, `nbkpts` = *<value>*.

            Constraint: `nbkpts` >= 2.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` > 0.

        (`errno` 2)
            On entry, `itype` = 2 and at least one interpolation point
            coincides with a break-point, i.e., interpolation point no
            *<value>* with value *<value>* is close to break-point
            *<value>* with value *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `xp`[i-1] = *<value>*, j =
            *<value>* and `xp`[j-1] = *<value>*.

            Constraint: `xbkpts`[0] <= `xp`[0] < `xp`[1] < ... <
            `xp`[`intpts`-1] <= `xbkpts`[`nbkpts`-1].

        (`errno` 3)
            Extrapolation is not allowed.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_coll_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03pyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, npts].
        utils._BYREF_INT, # Input, nbkpts.
        utils._EngineFloat64ArrayType.c_type, # Input, xbkpts[nbkpts].
        utils._BYREF_INT, # Input, npoly.
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, xp[intpts].
        utils._BYREF_INT, # Input, intpts.
        utils._BYREF_INT, # Input, itype.
        utils._EngineFloat64ArrayType.c_type, # Output, up[npde, intpts, itype].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rsave[lrsave].
        utils._BYREF_INT, # Input, lrsave.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'), (up, 'up')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _u = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_npde_py, _npts_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _nbkpts = utils._EngineIntScalarType(
        nbkpts, {'fun_name': fun_name, 'entity': 'nbkpts'},
    ).to_ctypes()
    _nbkpts_py = utils._EngineIntScalarType.to_py(_nbkpts)
    _xbkpts = utils._EngineFloat64ArrayType(
        xbkpts, {'fun_name': fun_name, 'entity': 'xbkpts'},
        exp_shape=(_nbkpts_py,),
    ).to_ctypes()
    _npoly = utils._EngineIntScalarType(
        npoly, {'fun_name': fun_name, 'entity': 'npoly'},
    ).to_ctypes()
    _intpts = utils._EngineIntScalarType(
        intpts, {'fun_name': fun_name, 'entity': 'intpts'},
    ).to_ctypes()
    _intpts_py = utils._EngineIntScalarType.to_py(_intpts)
    _xp = utils._EngineFloat64ArrayType(
        xp, {'fun_name': fun_name, 'entity': 'xp'},
        exp_shape=(_intpts_py,),
    ).to_ctypes()
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    _up_cls = utils._EngineFloat64ArrayType(
        up, {'fun_name': fun_name, 'entity': 'up'},
        exp_shape=(_npde_py, _intpts_py, _itype_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _up = _up_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rsave',),
    )
    _lrsave_py = utils._EngineFloat64ArrayType.get_shape(comm['rsave'], {'fun_name': fun_name, 'entity': 'comm[\'rsave\']'}, exp_rank=1, validate=False)[0]
    _lrsave = utils._EngineIntScalarType(
        _lrsave_py, {'fun_name': fun_name, 'entity': '_lrsave'},
    ).to_ctypes()
    _rsave = comm['rsave']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _npde,
        _u,
        _nbkpts,
        _xbkpts,
        _npoly,
        _npts,
        _xp,
        _intpts,
        _itype,
        _up,
        _rsave,
        _lrsave,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itype = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xbkpts[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and xbkpts[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xbkpts[0] < xbkpts[1] < ... < xbkpts[nbkpts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}, nbkpts = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: npts = (nbkpts-1)*npoly+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, intpts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: intpts >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoly = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npoly > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nbkpts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nbkpts >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itype = 2 and at least one interpolation',
                    'point coincides with a break-point, i.e.,',
                    'interpolation point no {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'with value {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'is close to break-point {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'with value {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xp[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and xp[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xbkpts[0] <= xp[0] < xp[1] < ... < xp[intpts-1] <= xbkpts[nbkpts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extrapolation is not allowed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _up_cls.refresh_py(_up)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_parab_fd_interp(npde, m, u, npts, x, xp, intpts, itype, up):
    u"""
    PDEs, spatial interpolation with ``dim1_parab_fd``,
    ``dim1_parab_keller``, ``dim1_parab_convdiff``,
    ``dim1_parab_dae_fd``, ``dim1_parab_dae_keller``,
    ``dim1_parab_convdiff_dae``, ``dim1_parab_remesh_fd``,
    ``dim1_parab_remesh_keller`` or ``dim1_parab_convdiff_remesh``.

    For full information please refer to the NAG Library document for
    d03pz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03pzf.html

    Parameters
    ----------
    npde : int
        The number of PDEs.

    m : int
        The coordinate system used. If the call to
        ``dim1_parab_fd_interp`` follows one of the finite difference
        functions then `m` must be the same argument `m` as used in that
        call. For the Keller box scheme only Cartesian coordinate
        systems are valid and so `m` must be set to zero. No check will
        be made by ``dim1_parab_fd_interp`` in this case.

        `m` = 0

          Indicates Cartesian coordinates.

        `m` = 1

          Indicates cylindrical polar coordinates.

        `m` = 2

          Indicates spherical polar coordinates.

    u : float, shape (`npde`, `npts`)
        The PDE part of the original solution returned in the argument
        `u` by the PDE function.

    npts : int
        The number of mesh points.

    x : float, shape (`npts`)
        `x`[i-1], for i = 1,2,...,`npts`, must contain the mesh points
        as used by the PDE function.

    xp : float, shape (`intpts`)
        `xp`[i-1], for i = 1,2,...,`intpts`, must contain the spatial
        interpolation points.

    intpts : int
        The number of interpolation points.

    itype : int
        Specifies the interpolation to be performed.

        `itype` = 1

          The solutions at the interpolation points are computed.

        `itype` = 2

          Both the solutions and their first derivatives at the
          interpolation points are computed.

    up : float, shape (`npde`, `intpts`, `itype`), modified in place
        On exit: if `itype` = 1, `up`[i-1,j-1,0], contains the value of
        the solution U_i(x_j,t_out), at the interpolation points x_j =
        `xp`[j-1], for i = 1,2,...,`npde`, for j = 1,2,...,`intpts`.

        If `itype` = 2, `up`[i-1,j-1,0] contains U_i(x_j,t_out) and
        `up`[i-1,j-1,1] contains dU_i/dx at these points.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `itype` = *<value>*.

            Constraint: `itype` = 1 or 2.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = 0, 1 or 2.

        (`errno` 1)
            On entry, i = *<value>*, `x`[i-1] = *<value>*, j = *<value>*
            and `x`[j-1] = *<value>*.

            Constraint: `x`[0] < `x`[1] < ... < `x`[`npts`-1].

        (`errno` 1)
            On entry, `intpts` = *<value>*.

            Constraint: `intpts` >= 1.

        (`errno` 1)
            On entry, `npts` = *<value>*.

            Constraint: `npts` > 2.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` > 0.

        (`errno` 2)
            On entry, i = *<value>*, `xp`[i-1] = *<value>*, j =
            *<value>* and `xp`[j-1] = *<value>*.

            Constraint: `x`[0] <= `xp`[0] < `xp`[1] < ... <
            `xp`[`intpts`-1] <= `x`[`npts`-1].

        (`errno` 3)
            On entry, interpolating point *<value>* with the value
            *<value>* is outside the `x` range.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim1_parab_fd_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03pzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, u[npde, npts].
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, xp[intpts].
        utils._BYREF_INT, # Input, intpts.
        utils._BYREF_INT, # Input, itype.
        utils._EngineFloat64ArrayType.c_type, # Output, up[npde, intpts, itype].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'), (up, 'up')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _u = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_npde_py, _npts_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _intpts = utils._EngineIntScalarType(
        intpts, {'fun_name': fun_name, 'entity': 'intpts'},
    ).to_ctypes()
    _intpts_py = utils._EngineIntScalarType.to_py(_intpts)
    _xp = utils._EngineFloat64ArrayType(
        xp, {'fun_name': fun_name, 'entity': 'xp'},
        exp_shape=(_intpts_py,),
    ).to_ctypes()
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    _up_cls = utils._EngineFloat64ArrayType(
        up, {'fun_name': fun_name, 'entity': 'up'},
        exp_shape=(_npde_py, _intpts_py, _itype_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _up = _up_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _npde,
        _m,
        _u,
        _npts,
        _x,
        _xp,
        _intpts,
        _itype,
        _up,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itype = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and x[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] < x[1] < ... < x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, intpts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: intpts >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts > 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, xp[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'j = {:s} and xp[j-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[0] <= xp[0] < xp[1] < ... < xp[intpts-1] <= x[npts-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, interpolating point {:s} with the value'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} is outside the x range.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _up_cls.refresh_py(_up)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_gen_order2_rectangle(npde, ts, tout, dt, xmin, xmax, ymin, ymax, nx, ny, tols, tolt, pdedef, bndary, pdeiv, monitr, opti, optr, comm, lenrwk, leniwk, lenlwk, itrace, ind, data=None, io_manager=None):
    u"""
    General system of second-order PDEs, method of lines, finite
    differences, remeshing, two space variables, rectangular region.

    For full information please refer to the NAG Library document for
    d03ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03raf.html

    Parameters
    ----------
    npde : int
        The number of PDEs in the system.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    dt : float, shape (3), modified in place
        On entry: the initial, minimum and maximum time step sizes
        respectively.

        `dt`[0]

          Specifies the initial time step size to be used on the first
          entry, i.e., when `ind` = 0. If `dt`[0] = 0.0 then the default
          value `dt`[0] = 0.01*(`tout`-`ts`) is used. On subsequent
          entries (`ind` = 1), the value of `dt`[0] is not referenced.

        `dt`[1]

          Specifies the minimum time step size to be attempted by the
          integrator. If `dt`[1] = 0.0 the default value `dt`[1] =
          10.0*machine precision is used.

        `dt`[2]

          Specifies the maximum time step size to be attempted by the
          integrator. If `dt`[2] = 0.0 the default value `dt`[2] =
          `tout`-`ts` is used.

        On exit: `dt`[0] contains the time step size for the next time
        step. `dt`[1] and `dt`[2] are unchanged or set to their default
        values if zero on entry.

    xmin : float
        The extents of the rectangular domain in the x-direction, i.e.,
        the x coordinates of the left and right boundaries respectively.

    xmax : float
        The extents of the rectangular domain in the x-direction, i.e.,
        the x coordinates of the left and right boundaries respectively.

    ymin : float
        The extents of the rectangular domain in the y-direction, i.e.,
        the y coordinates of the lower and upper boundaries
        respectively.

    ymax : float
        The extents of the rectangular domain in the y-direction, i.e.,
        the y coordinates of the lower and upper boundaries
        respectively.

    nx : int
        The number of grid points in the x-direction (including the
        boundary points).

    ny : int
        The number of grid points in the y-direction (including the
        boundary points).

    tols : float
        The space tolerance used in the grid refinement strategy (sigma
        in equation (4)). See [Further Comments].

    tolt : float
        The time tolerance used to determine the time step size (tau in
        equation (7)). See [Further Comments].

    pdedef : callable pdedef(npts, npde, t, x, y, u, ut, ux, uy, uxx,
        uxy, uyy, res, data=None)
        `pdedef` must evaluate the functions F_j, for j =
        1,2,...,`npde`, in equation (1) which define the system of PDEs
        at all interior points of the domain.

        Values at points on the boundaries of the domain are ignored and
        will be overwritten by `bndary`. `pdedef` is called for each
        subgrid in turn.

        Parameters
        ~~~~~~~~~~
        npts : int
            The number of grid points in the current grid.

        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            `x`[i-1] contains the x coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        y : float, ndarray, shape (`npts`)
            `y`[i-1] contains the y coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        u : float, ndarray, shape (`npts`, `npde`)
            `u`[i-1,j-1] contains the value of the jth PDE component at
            the ith grid point, for j = 1,2,...,`npde`, for i =
            1,2,...,`npts`.

        ut : float, ndarray, shape (`npts`, `npde`)
            `ut`[i-1,j-1] contains the value of du/dt for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        ux : float, ndarray, shape (`npts`, `npde`)
            `ux`[i-1,j-1] contains the value of du/dx for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        uy : float, ndarray, shape (`npts`, `npde`)
            `uy`[i-1,j-1] contains the value of du/dy for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        uxx : float, ndarray, shape (`npts`, `npde`)
            `uxx`[i-1,j-1] contains the value of d^2u/dx^2 for the jth
            PDE component at the ith grid point, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npts`.

        uxy : float, ndarray, shape (`npts`, `npde`)
            `uxy`[i-1,j-1] contains the value of d^2u/(dxdy) for the jth
            PDE component at the ith grid point, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npts`.

        uyy : float, ndarray, shape (`npts`, `npde`)
            `uyy`[i-1,j-1] contains the value of d^2u/dy^2 for the jth
            PDE component at the ith grid point, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npts`.

        res : float, ndarray, shape (`npts`, `npde`), to be modified in
        place
            On exit: `res`[i-1,j-1] must contain the value of F_j, for j
            = 1,2,...,`npde`, at the ith grid point, for i =
            1,2,...,`npts`, although the residuals at boundary points
            will be ignored (and overwritten later on) and so they need
            not be specified here.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    bndary : callable bndary(npts, npde, t, x, y, u, ut, ux, uy, nbpts,
        lbnd, res, data=None)
        `bndary` must evaluate the functions G_j, for j =
        1,2,...,`npde`, in equation (2) which define the boundary
        conditions at all boundary points of the domain.

        Residuals at interior points must not be altered by this
        function.

        Parameters
        ~~~~~~~~~~
        npts : int
            The number of grid points in the current grid.

        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            `x`[i-1] contains the x coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        y : float, ndarray, shape (`npts`)
            `y`[i-1] contains the y coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        u : float, ndarray, shape (`npts`, `npde`)
            `u`[i-1,j-1] contains the value of the jth PDE component at
            the ith grid point, for j = 1,2,...,`npde`, for i =
            1,2,...,`npts`.

        ut : float, ndarray, shape (`npts`, `npde`)
            `ut`[i-1,j-1] contains the value of du/dt for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        ux : float, ndarray, shape (`npts`, `npde`)
            `ux`[i-1,j-1] contains the value of du/dx for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        uy : float, ndarray, shape (`npts`, `npde`)
            `uy`[i-1,j-1] contains the value of du/dy for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        nbpts : int
            The number of boundary points in the grid.

        lbnd : int, ndarray, shape (`nbpts`)
            `lbnd`[i-1] contains the grid index for the ith boundary
            point, for i = 1,2,...,`nbpts`. Hence the ith boundary point
            has coordinates `x`[`lbnd`[i-1]-1] and `y`[`lbnd`[i-1]-1],
            and the corresponding solution values are
            `u`[`lbnd`[i-1]-1,`npde`-1], etc.

        res : float, ndarray, shape (`npts`, `npde`), to be modified in
        place
            On entry: `res`[i-1,j-1] contains the value of F_j, for i =
            1,2,...,`npde`, at the ith grid point, for i =
            1,2,...,`npts`, as returned by `pdedef`. The residuals at
            the boundary points will be overwritten and so need not have
            been set by `pdedef`.

            On exit: `res`[`lbnd`[i-1]-1,j-1] must contain the value of
            G_j, for j = 1,2,...,`npde`, at the ith boundary point, for
            i = 1,2,...,`nbpts`.

            Note: elements of `res` corresponding to interior points
            must not be altered.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    pdeiv : callable pdeiv(npts, npde, t, x, y, u, data=None)
        `pdeiv` must specify the initial values of the PDE components u
        at all points in the grid. `pdeiv` is not referenced if, on
        entry, `ind` = 1.

        Parameters
        ~~~~~~~~~~
        npts : int
            The number of grid points in the grid.

        npde : int
            The number of PDEs in the system.

        t : float
            The (initial) value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            `x`[i-1] contains the x coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        y : float, ndarray, shape (`npts`)
            `y`[i-1] contains the y coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        u : float, ndarray, shape (`npts`, `npde`), to be modified in
        place
            On exit: `u`[i-1,j-1] must contain the value of the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : callable ierr = monitr(npde, t, dt, dtnew, tlast, nlev,
        ngpts, xpts, ypts, lsol, sol, ierr, data=None)
        `monitr` is called by ``dim2_gen_order2_rectangle`` at the end
        of every successful time step, and may be used to examine or
        print the solution or perform other tasks such as error
        calculations, particularly at the final time step, indicated by
        the argument `tlast`.

        The input arguments contain information about the grid and
        solution at all grid levels used.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t, i.e., the
            time at the end of the integration step just completed.

        dt : float
            The current time step size Delta t, i.e., the time step size
            used for the integration step just completed.

        dtnew : float
            The step size that will be used for the next time step.

        tlast : bool
            Indicates if intermediate or final time step. `tlast` =
            False for an intermediate step, `tlast` = True for the last
            call to `monitr` before returning to your program.

        nlev : int
            The number of grid levels used at time `t`.

        ngpts : int, ndarray, shape (`nlev`)
            `ngpts`[l-1] contains the number of grid points at level l,
            for l = 1,2,...,`nlev`.

        xpts : float, ndarray, shape (sum(`ngpts`))
            Contains the x coordinates of the grid points in each level
            in turn, i.e., `x`[i-1], for l = 1,2,...,`nlev`, for i =
            1,2,...,`ngpts`[l-1].

            So for level l, `x`[i-1] = `xpts`[k+i-1], where k =
            `ngpts`[0]+`ngpts`[1]+...+`ngpts`[l-2], for l =
            1,2,...,`nlev`, for i = 1,2,...,`ngpts`[l-1].

        ypts : float, ndarray, shape (sum(`ngpts`))
            Contains the y coordinates of the grid points in each level
            in turn, i.e., `y`[i-1], for l = 1,2,...,`nlev`, for i =
            1,2,...,`ngpts`[l-1].

            So for level l, `y`[i-1] = `ypts`[k+i-1], where k =
            `ngpts`[0]+`ngpts`[1]+...+`ngpts`[l-2], for l =
            1,2,...,`nlev`, for i = 1,2,...,`ngpts`[l-1].

        lsol : int, ndarray, shape (`nlev`)
            `lsol`[l-1] contains the pointer to the solution in `sol` at
            grid level l and time `t`. (`lsol`[l-1] actually contains
            the array index immediately preceding the start of the
            solution in `sol`.)

        sol : float, ndarray, shape (:)
            Contains the solution `u`[`ngpts`[l-1]-1,`npde`-1] at time
            `t` for each grid level l in turn, positioned according to
            `lsol`, i.e., for level l, `u`[i-1,j-1] =
            `sol`[`lsol`[l-1]+(j-1)*`ngpts`[l-1]+i-1], for l =
            1,2,...,`nlev`, for j = 1,2,...,`npde`, for i =
            1,2,...,`ngpts`[l-1].

        ierr : int
            Will be set to 0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ierr : int
            Should be set to 1 to force a tidy termination and an
            immediate return to the calling program with `errno` = 4.
            `ierr` should remain unchanged otherwise.

    opti : int, shape (4)
        May be set to control various options available in the
        integrator.

        `opti`[0] = 0

          All the default options are employed.

        `opti`[0] > 0

          The default value of `opti`[i-1], for i = 2,3,...,4, can be
          obtained by setting `opti`[i-1] = 0.

        `opti`[0]

          Specifies the maximum number of grid levels allowed (including
          the base grid). `opti`[0] >= 0. The default value is `opti`[0]
          = 3.

        `opti`[1]

          Specifies the maximum number of Jacobian evaluations allowed
          during each nonlinear equations solution. `opti`[1] >= 0. The
          default value is `opti`[1] = 2.

        `opti`[2]

          Specifies the maximum number of Newton iterations in each
          nonlinear equations solution. `opti`[2] >= 0. The default
          value is `opti`[2] = 10.

        `opti`[3]

          Specifies the maximum number of iterations in each linear
          equations solution. `opti`[3] >= 0. The default value is
          `opti`[3] = 100.

    optr : float, shape (3, `npde`)
        May be used to specify the optional vectors u^max, w^s and w^t
        in the space and time monitors (see [Further Comments]).

        If an optional vector is not required then all its components
        should be set to 1.0.

        `optr`[0,j-1], for j = 1,2,...,`npde`, specifies u_j^max, the
        approximate maximum absolute value of the jth component of u, as
        used in (4) and (7). `optr`[0,j-1] > 0.0, for j =
        1,2,...,`npde`.

        `optr`[1,j-1], for j = 1,2,...,`npde`, specifies w_j^s, the
        weighting factors used in the space monitor (see (4)) to
        indicate the relative importance of the jth component of u on
        the space monitor. `optr`[1,j-1] >= 0.0, for j = 1,2,...,`npde`.

        `optr`[2,j-1], for j = 1,2,...,`npde`, specifies w_j^t, the
        weighting factors used in the time monitor (see (6)) to indicate
        the relative importance of the jth component of u on the time
        monitor. `optr`[2,j-1] >= 0.0, for j = 1,2,...,`npde`.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    lenrwk : int
        The required value of `lenrwk` cannot be determined exactly in
        advance, but a suggested value is

            `lenrwk` = maxpts*`npde`*(5*l+18*`npde`+9)+2*maxpts,

        where l = `opti`[0] if `opti`[0] != 0 and l = 3 otherwise, and
        maxpts is the expected maximum number of grid points at any one
        level.

        If during the execution the supplied value is found to be too
        small then the function returns with `errno` = 3 and an
        estimated required size is printed.

    leniwk : int
        The required value of `leniwk` cannot be determined exactly in
        advance, but a suggested value is

            `leniwk` = maxpts*(14+5*m)+7*m+2,

        where maxpts is the expected maximum number of grid points at
        any one level and m = `opti`[0] if `opti`[0] > 0 and m = 3
        otherwise.

        If during the execution the supplied value is found to be too
        small then the function returns with `errno` = 3 and an
        estimated required size is printed.

    lenlwk : int
        The required value of `lenlwk` cannot be determined exactly in
        advanced, but a suggested value is

            `lenlwk` = maxpts+1,

        where maxpts is the expected maximum number of grid points at
        any one level.

        If during the execution the supplied value is found to be too
        small then the function returns with `errno` = 3 and an
        estimated required size is printed.

    itrace : int
        The level of trace information required from
        ``dim2_gen_order2_rectangle``. `itrace` may take the value -1,
        0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed.

        `itrace` > 0

          Output from the underlying solver is printed. This output
          contains details of the time integration, the nonlinear
          iteration and the linear solver.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases.

        Setting `itrace` = 1 allows you to monitor the progress of the
        integration without possibly excessive information.

    ind : int
        Must be set to 0 or 1, alternatively 10 or 11.

        `ind` = 0

          Starts the integration in time. `pdedef` is assumed to be
          serial.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the following parameters may be
          reset between calls to ``dim2_gen_order2_rectangle``: tout,
          dt, tols, tolt, opti, optr, itrace and errno. `pdedef` is
          assumed to be serial.

        `ind` = 10

          Starts the integration in time. `pdedef` is assumed to have
          been parallelized by you, as described in [Parallelism and
          Performance]. In all other respects, this is equivalent to
          `ind` = 0.

        `ind` = 11

          Continues the integration after an earlier exit from the
          function. In this case, only the following parameters may be
          reset between calls to ``dim2_gen_order2_rectangle``: tout,
          dt, tols, tolt, opti, optr, itrace and errno. `pdedef` is
          assumed to have been parallelized by you, as described in
          [Parallelism and Performance]. In all other respects, this is
          equivalent to `ind` = 1.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t which has been reached. Normally `ts` = `tout`.

    ind : int
        `ind` = 1, if `ind` on input was 0 or 1, or `ind` = 11, if `ind`
        on input was 10 or 11.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ind` is not equal to 0 or 1: `ind` = *<value>*.

        (`errno` 1)
            On entry, `lenlwk` too small for initial grid level:
            `lenlwk` = *<value>*, minimum value = *<value>*. Note that
            subsequent levels will require more. Consult document.

        (`errno` 1)
            On entry, `leniwk` too small for initial grid level:
            `leniwk` = *<value>*, minimum value = *<value>*. Note that
            subsequent levels will require more. Consult document.

        (`errno` 1)
            On entry, `lenrwk` too small for initial grid level:
            `lenrwk` = *<value>*, minimum value = *<value>*. Note that
            subsequent levels will require more. Consult document.

        (`errno` 1)
            On entry, `ymax` too close to `ymin`: `ymax` = *<value>* and
            `ymin` = *<value>*.

        (`errno` 1)
            On entry, `xmax` too close to `xmin`: `xmax` = *<value>* and
            `xmin` = *<value>*.

        (`errno` 1)
            On entry, `ymin` = *<value>* and `ymax` = *<value>*.

            Constraint: `ymax` > `ymin`.

        (`errno` 1)
            On entry, `xmin` = *<value>* and `xmax` = *<value>*.

            Constraint: `xmax` > `xmin`.

        (`errno` 1)
            On entry, `ny` = *<value>*.

            Constraint: `ny` >= 4.

        (`errno` 1)
            On entry, `nx` = *<value>*.

            Constraint: `nx` >= 4.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `dt`[0] = *<value>* and `dt`[2] = *<value>*. Note
            that `dt`[2] was reset to default if zero on entry.

            Constraint: if `ind` = 0, `dt`[0] <= `dt`[2].

        (`errno` 1)
            On entry, `dt`[0] = *<value>* and `dt`[1] = *<value>*. Note
            that `dt`[1] was reset to default if zero on entry.

            Constraint: if `ind` = 0, `dt`[0] >= `dt`[1].

        (`errno` 1)
            On entry, `ind` = 0 and `dt`[0] too large: `dt`[0] =
            *<value>* and maximum value = *<value>*.

        (`errno` 1)
            On entry, `ind` = 0 and `dt`[0] too small: `dt`[0] =
            *<value>* and minimum value = *<value>*.

        (`errno` 1)
            On entry, `dt`[0] = *<value>*.

            Constraint: if `ind` = 0, `dt`[0] >= 0.0.

        (`errno` 1)
            On entry, `dt`[1] = *<value>* and `dt`[2] = *<value>*.

            Constraint: `dt`[1] <= `dt`[2].

        (`errno` 1)
            On entry, `dt`[2] = *<value>*.

            Constraint: `dt`[2] >= 0.0.

        (`errno` 1)
            On entry, `dt`[1] = *<value>*.

            Constraint: `dt`[1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>*, j = *<value>* and `optr`[i-1,j-1] =
            *<value>*.

            Constraint: `optr`[i-1,j-1] >= 0.0.

        (`errno` 1)
            On entry, j = *<value>* and `optr`[0,j-1] = *<value>*.

            Constraint: `optr`[0,j-1] > 0.0.

        (`errno` 1)
            On entry, `opti`[0] = *<value>*, i = *<value>* and
            `opti`[i-1] = *<value>*.

            Constraint: if `opti`[0] > 0, `opti`[i-1] >= 0.

        (`errno` 1)
            On entry, `opti`[0] = *<value>*.

            Constraint: `opti`[0] >= 0.

        (`errno` 1)
            On entry, `tolt` = *<value>*.

            Constraint: `tolt` > 0.0.

        (`errno` 1)
            On entry, `tols` = *<value>*.

            Constraint: `tols` > 0.0.

        (`errno` 1)
            On entry, `tout`-`ts` too small: `tout`-`ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 2)
            Attempted time-step smaller than specified minimum. Check
            problem formulation in `pdedef`, `bndary` and `pdeiv`. Try
            increasing `itrace` for more information.

        (`errno` 3)
            One or more of the workspace arrays are too small. Try
            increasing `itrace` for more information.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            IERR set to 1 in `monitr`. Integration completed as far as
            `ts`: `ts` = *<value>*.

        (`errno` 5)
            Integration completed, but maximum number of levels too
            small for required accuracy.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_gen_order2_rectangle'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, y[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uy[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uxx[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uxy[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uyy[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Output, res[npts, npde].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npts, npde, t, x, y, u, ut, ux, uy, uxx, uxy, uyy, res, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npts_pdedefh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_pdedefh,),
            )
            _y_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_npts_pdedefh,),
            )
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _ut_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uy_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uy,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uxx_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uxx,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uxy_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uxy,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uyy_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uyy,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _res_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_npts_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                pdedef(
                    _npts_pdedefh,
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _y_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _uy_pdedefh,
                    _uxx_pdedefh,
                    _uxy_pdedefh,
                    _uyy_pdedefh,
                    _res_pdedefh,
                    data,
                )
            else:
                pdedef(
                    _npts_pdedefh,
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _y_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _uy_pdedefh,
                    _uxx_pdedefh,
                    _uxy_pdedefh,
                    _uyy_pdedefh,
                    _res_pdedefh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(y)
            utils._unlink_pointer(u)
            utils._unlink_pointer(ut)
            utils._unlink_pointer(ux)
            utils._unlink_pointer(uy)
            utils._unlink_pointer(uxx)
            utils._unlink_pointer(uxy)
            utils._unlink_pointer(uyy)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _res_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'res'}, res,
            )
            utils._unlink_pointer(res)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, y[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uy[npts, npde].
        utils._BYREF_INT, # Input, nbpts.
        utils._EngineIntArrayType.c_type, # Input, lbnd[nbpts].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, res[npts, npde].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npts, npde, t, x, y, u, ut, ux, uy, nbpts, lbnd, res, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npts_bndaryh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_bndaryh,),
            )
            _y_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_npts_bndaryh,),
            )
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _ut_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _ux_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _uy_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                uy,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _nbpts_bndaryh = utils._EngineIntScalarType.to_py(nbpts.contents)
            _lbnd_bndaryh = utils._EngineIntArrayType.from_pointer(
                lbnd,
                (_nbpts_bndaryh,),
            )
            _res_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            if data is not None:
                bndary(
                    _npts_bndaryh,
                    _npde_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _y_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _ux_bndaryh,
                    _uy_bndaryh,
                    _nbpts_bndaryh,
                    _lbnd_bndaryh,
                    _res_bndaryh,
                    data,
                )
            else:
                bndary(
                    _npts_bndaryh,
                    _npde_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _y_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _ux_bndaryh,
                    _uy_bndaryh,
                    _nbpts_bndaryh,
                    _lbnd_bndaryh,
                    _res_bndaryh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(y)
            utils._unlink_pointer(u)
            utils._unlink_pointer(ut)
            utils._unlink_pointer(ux)
            utils._unlink_pointer(uy)
            utils._unlink_pointer(lbnd)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _res_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'res'}, res,
            )
            utils._unlink_pointer(res)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_pdeiv_name = fun_name + ':pdeiv'

    pdeivh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, y[npts].
        utils._EngineFloat64ArrayType.c_type, # Output, u[npts, npde].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdeivh(_pdeiv, en_data, npts, npde, t, x, y, u, _iuser, _ruser):
        """Helper for pdeiv."""
        try:
            _npts_pdeivh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_pdeivh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdeivh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdeivh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_pdeivh,),
            )
            _y_pdeivh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_npts_pdeivh,),
            )
            _u_pdeivh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npts_pdeivh, _npde_pdeivh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                pdeiv(
                    _npts_pdeivh,
                    _npde_pdeivh,
                    _t_pdeivh,
                    _x_pdeivh,
                    _y_pdeivh,
                    _u_pdeivh,
                    data,
                )
            else:
                pdeiv(
                    _npts_pdeivh,
                    _npde_pdeivh,
                    _t_pdeivh,
                    _x_pdeivh,
                    _y_pdeivh,
                    _u_pdeivh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _u_pdeivh, {'fun_name': cb_pdeiv_name, 'returned': 'u'}, u,
            )
            utils._unlink_pointer(u)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, dt.
        utils._BYREF_FLOAT64, # Input, dtnew.
        utils._BYREF_LOGICAL, # Input, tlast.
        utils._BYREF_INT, # Input, nlev.
        utils._EngineIntArrayType.c_type, # Input, ngpts[nlev].
        utils._EngineFloat64ArrayType.c_type, # Input, xpts[sum(ngpts[0 : nlev-1])].
        utils._EngineFloat64ArrayType.c_type, # Input, ypts[sum(ngpts[0 : nlev-1])].
        utils._EngineIntArrayType.c_type, # Input, lsol[nlev].
        utils._EngineFloat64ArrayType.c_type, # Input, sol[lsol[nlev-1]+npde*ngpts[nlev-1]].
        utils._BYREF_INT, # Input/Output, ierr.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monitrh(_monitr, en_data, npde, t, dt, dtnew, tlast, nlev, ngpts, xpts, ypts, lsol, sol, ierr, _iuser, _ruser):
        """Helper for monitr."""
        try:
            _npde_monitrh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _dt_monitrh = utils._EngineFloat64ScalarType.to_py(dt.contents)
            _dtnew_monitrh = utils._EngineFloat64ScalarType.to_py(dtnew.contents)
            _tlast_monitrh = utils._EngineLogicalScalarType.to_py(tlast.contents)
            _nlev_monitrh = utils._EngineIntScalarType.to_py(nlev.contents)
            _ngpts_monitrh = utils._EngineIntArrayType.from_pointer(
                ngpts,
                (_nlev_monitrh,),
            )
            _xpts_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                xpts,
                (sum(_ngpts_monitrh[0:_nlev_monitrh-1+1]),),
            )
            _ypts_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                ypts,
                (sum(_ngpts_monitrh[0:_nlev_monitrh-1+1]),),
            )
            _lsol_monitrh = utils._EngineIntArrayType.from_pointer(
                lsol,
                (_nlev_monitrh,),
            )
            _sol_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                sol,
                (utils._EngineIntArrayType.get_element(lsol, {'fun_name': cb_monitr_name, 'entity': 'lsol'}, (_nlev_monitrh-1,), validate=False)+_npde_monitrh*utils._EngineIntArrayType.get_element(ngpts, {'fun_name': cb_monitr_name, 'entity': 'ngpts'}, (_nlev_monitrh-1,), validate=False),),
            )
            _ierr_monitrh = utils._EngineIntScalarType.to_py(ierr.contents)
            if data is not None:
                _ierr_monitrh = monitr(
                    _npde_monitrh,
                    _t_monitrh,
                    _dt_monitrh,
                    _dtnew_monitrh,
                    _tlast_monitrh,
                    _nlev_monitrh,
                    _ngpts_monitrh,
                    _xpts_monitrh,
                    _ypts_monitrh,
                    _lsol_monitrh,
                    _sol_monitrh,
                    _ierr_monitrh,
                    data,
                )
            else:
                _ierr_monitrh = monitr(
                    _npde_monitrh,
                    _t_monitrh,
                    _dt_monitrh,
                    _dtnew_monitrh,
                    _tlast_monitrh,
                    _nlev_monitrh,
                    _ngpts_monitrh,
                    _xpts_monitrh,
                    _ypts_monitrh,
                    _lsol_monitrh,
                    _sol_monitrh,
                    _ierr_monitrh,
                )
            utils._unlink_pointer(ngpts)
            utils._unlink_pointer(xpts)
            utils._unlink_pointer(ypts)
            utils._unlink_pointer(lsol)
            utils._unlink_pointer(sol)
            utils._EngineIntScalarType(
                _ierr_monitrh, {'fun_name': cb_monitr_name, 'returned': 'ierr'},
            ).refresh_ctypes_buffer(ierr)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03raft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dt[3].
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, xmax.
        utils._BYREF_FLOAT64, # Input, ymin.
        utils._BYREF_FLOAT64, # Input, ymax.
        utils._BYREF_INT, # Input, nx.
        utils._BYREF_INT, # Input, ny.
        utils._BYREF_FLOAT64, # Input, tols.
        utils._BYREF_FLOAT64, # Input, tolt.
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._UNUSED_FUNCTYPE,
        pdeivh,
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._EngineIntArrayType.c_type, # Input, opti[4].
        utils._EngineFloat64ArrayType.c_type, # Input, optr[3, npde].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwk[lenrwk].
        utils._BYREF_INT, # Input, lenrwk.
        utils._EngineIntArrayType.c_type, # Input/Output, iwk[leniwk].
        utils._BYREF_INT, # Input, leniwk.
        utils._EngineLogicalArrayType.c_type, # Workspace, lwk.
        utils._BYREF_INT, # Input, lenlwk.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[20].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[25].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((optr, 'optr'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _dt_cls = utils._EngineFloat64ArrayType(
        dt, {'fun_name': fun_name, 'entity': 'dt'},
        exp_shape=(3,),
    )
    _dt = _dt_cls.to_ctypes()
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _xmax = utils._EngineFloat64ScalarType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
    ).to_ctypes()
    _ymin = utils._EngineFloat64ScalarType(
        ymin, {'fun_name': fun_name, 'entity': 'ymin'},
    ).to_ctypes()
    _ymax = utils._EngineFloat64ScalarType(
        ymax, {'fun_name': fun_name, 'entity': 'ymax'},
    ).to_ctypes()
    _nx = utils._EngineIntScalarType(
        nx, {'fun_name': fun_name, 'entity': 'nx'},
    ).to_ctypes()
    _ny = utils._EngineIntScalarType(
        ny, {'fun_name': fun_name, 'entity': 'ny'},
    ).to_ctypes()
    _tols = utils._EngineFloat64ScalarType(
        tols, {'fun_name': fun_name, 'entity': 'tols'},
    ).to_ctypes()
    _tolt = utils._EngineFloat64ScalarType(
        tolt, {'fun_name': fun_name, 'entity': 'tolt'},
    ).to_ctypes()
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    utils._check_callable(pdeiv, {'fun_name': fun_name, 'entity': 'pdeiv'})
    _pdeiv = utils._UNUSED_CALLBACK
    _pdeivh = utils._cast_callback(
        pdeivh, cb_pdeivh,
        {'fun_name': fun_name, 'entity': 'pdeiv'},
    )
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'})
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _opti = utils._EngineIntArrayType(
        opti, {'fun_name': fun_name, 'entity': 'opti'},
        exp_shape=(4,),
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _optr = utils._EngineFloat64ArrayType(
        optr, {'fun_name': fun_name, 'entity': 'optr'},
        exp_shape=(3, _npde_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lenrwk = utils._EngineIntScalarType(
        lenrwk, {'fun_name': fun_name, 'entity': 'lenrwk'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rwk', 'iwk', 'lwsav', 'iwsav', 'rwsav'),
    )
    _lenrwk_py = utils._EngineIntScalarType.to_py(_lenrwk)
    if _comm_needs_init:
        _rwk = utils._EngineFloat64ArrayType.empty_ctypes(_lenrwk_py)
        comm['rwk'] = _rwk
    else:
        _rwk = comm['rwk']
    _leniwk = utils._EngineIntScalarType(
        leniwk, {'fun_name': fun_name, 'entity': 'leniwk'},
    ).to_ctypes()
    _leniwk_py = utils._EngineIntScalarType.to_py(_leniwk)
    if _comm_needs_init:
        _iwk = utils._EngineIntArrayType.empty_ctypes(_leniwk_py)
        comm['iwk'] = _iwk
    else:
        _iwk = comm['iwk']
    _lwk = utils._EngineLogicalArrayType.empty_ctypes(0)
    _lenlwk = utils._EngineIntScalarType(
        lenlwk, {'fun_name': fun_name, 'entity': 'lenlwk'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(20)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(25)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _dt,
        _xmin,
        _xmax,
        _ymin,
        _ymax,
        _nx,
        _ny,
        _tols,
        _tolt,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _pdeiv,
        _pdeivh,
        _monitr,
        _monitrh,
        _opti,
        _optr,
        _rwk,
        _lenrwk,
        _iwk,
        _leniwk,
        _lwk,
        _lenlwk,
        _itrace,
        _nadv,
        _ind,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind is not equal to 0 or 1: ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenlwk too small for initial grid level:',
                    'lenlwk = {:s}, minimum value = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Note that subsequent levels will require more. Consult document.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, leniwk too small for initial grid level:',
                    'leniwk = {:s}, minimum value = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Note that subsequent levels will require more. Consult document.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenrwk too small for initial grid level:',
                    'lenrwk = {:s}, minimum value = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Note that subsequent levels will require more. Consult document.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ymax too close to ymin:',
                    'ymax = {:s} and ymin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmax too close to xmin:',
                    'xmax = {:s} and xmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ymin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ymax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ymax > ymin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xmax > xmin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ny = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ny >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nx >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'dt[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Note that dt[2] was reset to default if zero on entry.',
                    'Constraint: if ind = 0, dt[0] <= dt[2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'dt[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Note that dt[1] was reset to default if zero on entry.',
                    'Constraint: if ind = 0, dt[0] >= dt[1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 0 and dt[0] too large:',
                    'dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'maximum value = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 0 and dt[0] too small:',
                    'dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'minimum value = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if ind = 0, dt[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'dt[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: dt[1] <= dt[2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dt[2] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dt[1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, j = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'optr[i-1,j-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: optr[i-1,j-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, j = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'optr[0,j-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: optr[0,j-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, opti[0] = {:s}, i = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'opti[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: if opti[0] > 0, opti[i-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, opti[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: opti[0] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tolt = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tolt > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tols = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tols > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts too small: tout-ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Attempted time-step smaller than specified minimum.',
                    'Check problem formulation in pdedef, bndary and pdeiv.',
                    'Try increasing itrace for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'One or more of the workspace arrays are too small.',
                    'Try increasing itrace for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99970:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'IERR set to 1 in monitr. Integration completed as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99969:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but maximum number of levels too small',
                    'for required accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _dt_cls.refresh_py(_dt)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind'],
        )(
            _ts_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim2_gen_order2_rectilinear(npde, ts, tout, dt, tols, tolt, inidom, pdedef, bndary, pdeiv, monitr, opti, optr, comm, lenrwk, leniwk, lenlwk, itrace, ind, data=None, io_manager=None):
    u"""
    General system of second-order PDEs, method of lines, finite
    differences, remeshing, two space variables, rectilinear region.

    For full information please refer to the NAG Library document for
    d03rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03rbf.html

    Parameters
    ----------
    npde : int
        The number of PDEs in the system.

    ts : float
        The initial value of the independent variable t.

    tout : float
        The final value of t to which the integration is to be carried
        out.

    dt : float, shape (3), modified in place
        On entry: the initial, minimum and maximum time step sizes
        respectively.

        `dt`[0]

          Specifies the initial time step size to be used on the first
          entry, i.e., when `ind` = 0. If `dt`[0] = 0.0 then the default
          value `dt`[0] = 0.01*(`tout`-`ts`) is used. On subsequent
          entries (`ind` = 1), the value of `dt`[0] is not referenced.

        `dt`[1]

          Specifies the minimum time step size to be attempted by the
          integrator. If `dt`[1] = 0.0 the default value `dt`[1] =
          10.0*machine precision is used.

        `dt`[2]

          Specifies the maximum time step size to be attempted by the
          integrator. If `dt`[2] = 0.0 the default value `dt`[2] =
          `tout`-`ts` is used.

        On exit: `dt`[0] contains the time step size for the next time
        step. `dt`[1] and `dt`[2] are unchanged or set to their default
        values if zero on entry.

    tols : float
        The space tolerance used in the grid refinement strategy (sigma
        in equation (4)). See [Further Comments].

    tolt : float
        The time tolerance used to determine the time step size (tau in
        equation (7)). See [Further Comments].

    inidom : callable (xmin, xmax, ymin, ymax, nx, ny, npts, nrows,
        nbnds, nbpts, ierr) = inidom(maxpts, lrow, irow, icol, llbnd,
        ilbnd, lbnd, ierr, data=None)
        `inidom` must specify the base grid in terms of the data
        structure described in [Description]. `inidom` is not referenced
        if, on entry, `ind` = 1.

        Parameters
        ~~~~~~~~~~
        maxpts : int
            The maximum number of base grid points allowed by the
            available workspace.

        lrow : int, ndarray, shape (`maxpts`), to be modified in place
            On exit: `lrow`[i-1], for i = 1,2,...,`nrows`, must contain
            the base grid index of the first grid point in base grid row
            i.

        irow : int, ndarray, shape (`maxpts`), to be modified in place
            On exit: `irow`[i-1], for i = 1,2,...,`nrows`, must contain
            the virtual row number v_y that corresponds to base grid row
            i.

        icol : int, ndarray, shape (`maxpts`), to be modified in place
            On exit: `icol`[i-1], for i = 1,2,...,`npts`, must contain
            the virtual column number v_x that contains base grid point
            i.

        llbnd : int, ndarray, shape (`maxpts`), to be modified in place
            On exit: `llbnd`[i-1], for i = 1,2,...,`nbnds`, must contain
            the element of `lbnd` corresponding to the start of the ith
            boundary or corner.

            Note: the order of the boundaries and corners in `llbnd`
            must be first all the boundaries and then all the corners.
            The end points of a boundary must not be included in the
            list of points on that boundary. Also, if a corner is shared
            by two pairs of physical boundaries then it has two types
            and must therefore be treated as two corners.

        ilbnd : int, ndarray, shape (`maxpts`), to be modified in place
            On exit: `ilbnd`[i-1], for i = 1,2,...,`nbnds`, must contain
            the type of the ith boundary (or corner), as given in
            [Description].

        lbnd : int, ndarray, shape (`maxpts`), to be modified in place
            On exit: `lbnd`[i-1], for i = 1,2,...,`nbpts`, must contain
            the grid index of the ith boundary point. The order of the
            boundaries is as specified in `llbnd`, but within this
            restriction the order of the points in `lbnd` is arbitrary.

        ierr : int
            Will be initialized by ``dim2_gen_order2_rectilinear`` to
            some value prior to internal calls to `inidom`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        xmin : float
            The extents of the virtual grid in the x-direction, i.e.,
            the x coordinates of the left and right boundaries
            respectively.

        xmax : float
            The extents of the virtual grid in the x-direction, i.e.,
            the x coordinates of the left and right boundaries
            respectively.

        ymin : float
            The extents of the virtual grid in the y-direction, i.e.,
            the y coordinates of the left and right boundaries
            respectively.

        ymax : float
            The extents of the virtual grid in the y-direction, i.e.,
            the y coordinates of the left and right boundaries
            respectively.

        nx : int
            The number of virtual grid points in the x- and y-direction
            respectively (including the boundary points).

        ny : int
            The number of virtual grid points in the x- and y-direction
            respectively (including the boundary points).

        npts : int
            The total number of points in the base grid. If the required
            number of points is greater than `maxpts` then `inidom` must
            be exited immediately with `ierr` set to -1 to avoid
            overwriting memory.

        nrows : int
            The total number of rows of the virtual grid that contain
            base grid points. This is the maximum base row index.

        nbnds : int
            The total number of physical boundaries and corners in the
            base grid.

        nbpts : int
            The total number of boundary points in the base grid.

        ierr : int
            If the required number of grid points is larger than
            `maxpts`, `ierr` must be set to -1 to force a termination of
            the integration and an immediate return to the calling
            program with `errno` = 3. Otherwise, `ierr` should remain
            unchanged.

    pdedef : callable pdedef(npts, npde, t, x, y, u, ut, ux, uy, uxx,
        uxy, uyy, res, data=None)
        `pdedef` must evaluate the functions F_j, for j =
        1,2,...,`npde`, in equation (1) which define the system of PDEs
        at all interior points of the domain.

        Values at points on the boundaries of the domain are ignored and
        will be overwritten by `bndary`. `pdedef` is called for each
        subgrid in turn.

        Parameters
        ~~~~~~~~~~
        npts : int
            The number of grid points in the current grid.

        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            `x`[i-1] contains the x coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        y : float, ndarray, shape (`npts`)
            `y`[i-1] contains the y coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        u : float, ndarray, shape (`npts`, `npde`)
            `u`[i-1,j-1] contains the value of the jth PDE component at
            the ith grid point, for j = 1,2,...,`npde`, for i =
            1,2,...,`npts`.

        ut : float, ndarray, shape (`npts`, `npde`)
            `ut`[i-1,j-1] contains the value of du/dt for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        ux : float, ndarray, shape (`npts`, `npde`)
            `ux`[i-1,j-1] contains the value of du/dx for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        uy : float, ndarray, shape (`npts`, `npde`)
            `uy`[i-1,j-1] contains the value of du/dy for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        uxx : float, ndarray, shape (`npts`, `npde`)
            `uxx`[i-1,j-1] contains the value of d^2u/dx^2 for the jth
            PDE component at the ith grid point, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npts`.

        uxy : float, ndarray, shape (`npts`, `npde`)
            `uxy`[i-1,j-1] contains the value of d^2u/(dxdy) for the jth
            PDE component at the ith grid point, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npts`.

        uyy : float, ndarray, shape (`npts`, `npde`)
            `uyy`[i-1,j-1] contains the value of d^2u/dy^2 for the jth
            PDE component at the ith grid point, for j = 1,2,...,`npde`,
            for i = 1,2,...,`npts`.

        res : float, ndarray, shape (`npts`, `npde`), to be modified in
        place
            On exit: `res`[i-1,j-1] must contain the value of F_j, for j
            = 1,2,...,`npde`, at the ith grid point, for i =
            1,2,...,`npts`, although the residuals at boundary points
            will be ignored (and overwritten later on) and so they need
            not be specified here.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    bndary : callable bndary(npts, npde, t, x, y, u, ut, ux, uy, nbnds,
        nbpts, llbnd, ilbnd, lbnd, res, data=None)
        `bndary` must evaluate the functions G_j, for j =
        1,2,...,`npde`, in equation (2) which define the boundary
        conditions at all boundary points of the domain.

        Residuals at interior points must not be altered by this
        function.

        Parameters
        ~~~~~~~~~~
        npts : int
            The number of grid points in the current grid.

        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            `x`[i-1] contains the x coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        y : float, ndarray, shape (`npts`)
            `y`[i-1] contains the y coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        u : float, ndarray, shape (`npts`, `npde`)
            `u`[i-1,j-1] contains the value of the jth PDE component at
            the ith grid point, for j = 1,2,...,`npde`, for i =
            1,2,...,`npts`.

        ut : float, ndarray, shape (`npts`, `npde`)
            `ut`[i-1,j-1] contains the value of du/dt for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        ux : float, ndarray, shape (`npts`, `npde`)
            `ux`[i-1,j-1] contains the value of du/dx for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        uy : float, ndarray, shape (`npts`, `npde`)
            `uy`[i-1,j-1] contains the value of du/dy for the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        nbnds : int
            The total number of physical boundaries and corners in the
            grid.

        nbpts : int
            The total number of boundary points in the grid.

        llbnd : int, ndarray, shape (`nbnds`)
            `llbnd`[i-1], for i = 1,2,...,`nbnds`, contains the element
            of `lbnd` corresponding to the start of the ith boundary (or
            corner).

        ilbnd : int, ndarray, shape (`nbnds`)
            `ilbnd`[i-1], for i = 1,2,...,`nbnds`, contains the type of
            the ith boundary, as given in [Description].

        lbnd : int, ndarray, shape (`nbpts`)
            `lbnd`[i-1], contains the grid index of the ith boundary
            point, where the order of the boundaries is as specified in
            `llbnd`. Hence the ith boundary point has coordinates
            `x`[`lbnd`[i-1]-1] and `y`[`lbnd`[i-1]-1], and the
            corresponding solution values are `u`[`lbnd`[i-1]-1,j-1],
            for j = 1,2,...,`npde`, for i = 1,2,...,`nbpts`.

        res : float, ndarray, shape (`npts`, `npde`), to be modified in
        place
            On entry: contains function values returned by `pdedef`.

            On exit: `res`[`lbnd`[i-1]-1,j-1] must contain the value of
            G_j, for j = 1,2,...,`npde`, at the ith boundary point, for
            i = 1,2,...,`nbpts`.

            Note: elements of `res` corresponding to interior points,
            i.e., points not included in `lbnd`, must not be altered.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    pdeiv : callable pdeiv(npts, npde, t, x, y, u, data=None)
        `pdeiv` must specify the initial values of the PDE components u
        at all points in the base grid. `pdeiv` is not referenced if, on
        entry, `ind` = 1.

        Parameters
        ~~~~~~~~~~
        npts : int
            The number of grid points in the base grid.

        npde : int
            The number of PDEs in the system.

        t : float
            The (initial) value of the independent variable t.

        x : float, ndarray, shape (`npts`)
            `x`[i-1] contains the x coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        y : float, ndarray, shape (`npts`)
            `y`[i-1] contains the y coordinate of the ith grid point,
            for i = 1,2,...,`npts`.

        u : float, ndarray, shape (`npts`, `npde`), to be modified in
        place
            On exit: `u`[i-1,j-1] must contain the value of the jth PDE
            component at the ith grid point, for j = 1,2,...,`npde`, for
            i = 1,2,...,`npts`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : callable ierr = monitr(npde, t, dt, dtnew, tlast, nlev,
        xmin, ymin, dxb, dyb, lgrid, istruc, nistruc, lsol, sol, nsol,
        ierr, data=None)
        `monitr` is called by ``dim2_gen_order2_rectilinear`` at the end
        of every successful time step, and may be used to examine or
        print the solution or perform other tasks such as error
        calculations, particularly at the final time step, indicated by
        the argument `tlast`.

        Parameters
        ~~~~~~~~~~
        npde : int
            The number of PDEs in the system.

        t : float
            The current value of the independent variable t, i.e., the
            time at the end of the integration step just completed.

        dt : float
            The current time step size Delta t, i.e., the time step size
            used for the integration step just completed.

        dtnew : float
            The time step size that will be used for the next time step.

        tlast : bool
            Indicates if intermediate or final time step. `tlast` =
            False for an intermediate step, `tlast` = True for the last
            call to `monitr` before returning to your program.

        nlev : int
            The number of grid levels used at time `t`.

        xmin : float
            The (x,y) coordinates of the lower-left corner of the
            virtual grid.

        ymin : float
            The (x,y) coordinates of the lower-left corner of the
            virtual grid.

        dxb : float
            The sizes of the base grid spacing in the x- and y-direction
            respectively.

        dyb : float
            The sizes of the base grid spacing in the x- and y-direction
            respectively.

        lgrid : int, ndarray, shape (`nlev`+1)
            Contains pointers to the start of the grid structures in
            `istruc`, and must be passed unchanged to
            ``dim2_gen_order2_rectilinear_extractgrid`` in order to
            extract the grid information.

        istruc : int, ndarray, shape (`nistruc`)
            Contains the grid structures for each grid level and must be
            passed unchanged to
            ``dim2_gen_order2_rectilinear_extractgrid`` in order to
            extract the grid information.

        nistruc : int
            The dimension of the array `istruc`.

        lsol : int, ndarray, shape (`nlev`)
            `lsol`[l-1] contains the pointer to the solution in `sol` at
            grid level l and time `t`. (`lsol`[l-1] actually contains
            the array index immediately preceding the start of the
            solution in `sol`.)

        sol : float, ndarray, shape (`nsol`)
            Contains the solution u at time `t` for each grid level l in
            turn, positioned according to `lsol`. More precisely

                `u`[i-1,j-1] = `sol`[ `lsol`[l-1] + (j-1) * n_l+i -1]

            represents the jth component of the solution at the ith grid
            point in the lth level, for l = 1,2,...,`nlev`, for j =
            1,2,...,`npde`, for i = 1,2,...,n_l, where n_l is the number
            of grid points at level l (obtainable by a call to
            ``dim2_gen_order2_rectilinear_extractgrid``).

        nsol : int
            The dimension of the array `sol`.

        ierr : int
            Will be initialized by ``dim2_gen_order2_rectilinear`` to
            some value prior to internal calls to this argument.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ierr : int
            Should be set to 1 to force a termination of the integration
            and an immediate return to the calling program with `errno`
            = 4. `ierr` should remain unchanged otherwise.

    opti : int, shape (4)
        May be set to control various options available in the
        integrator.

        `opti`[0] = 0

          All the default options are employed.

        `opti`[0] > 0

          The default value of `opti`[i-1], for i = 2,3,...,4, can be
          obtained by setting `opti`[i-1] = 0.

        `opti`[0]

          Specifies the maximum number of grid levels allowed (including
          the base grid). `opti`[0] >= 0. The default value is `opti`[0]
          = 3.

        `opti`[1]

          Specifies the maximum number of Jacobian evaluations allowed
          during each nonlinear equations solution. `opti`[1] >= 0. The
          default value is `opti`[1] = 2.

        `opti`[2]

          Specifies the maximum number of Newton iterations in each
          nonlinear equations solution. `opti`[2] >= 0. The default
          value is `opti`[2] = 10.

        `opti`[3]

          Specifies the maximum number of iterations in each linear
          equations solution. `opti`[3] >= 0. The default value is
          `opti`[3] = 100.

    optr : float, shape (3, `npde`)
        May be used to specify the optional vectors u^max, w^s and w^t
        in the space and time monitors (see [Further Comments]).

        If an optional vector is not required then all its components
        should be set to 1.0.

        `optr`[0,j-1], for j = 1,2,...,`npde`, specifies u_j^max, the
        approximate maximum absolute value of the jth component of u, as
        used in (4) and (7). `optr`[0,j-1] > 0.0, for j =
        1,2,...,`npde`.

        `optr`[1,j-1], for j = 1,2,...,`npde`, specifies w_j^s, the
        weighting factors used in the space monitor (see (4)) to
        indicate the relative importance of the jth component of u on
        the space monitor. `optr`[1,j-1] >= 0.0, for j = 1,2,...,`npde`.

        `optr`[2,j-1], for j = 1,2,...,`npde`, specifies w_j^t, the
        weighting factors used in the time monitor (see (6)) to indicate
        the relative importance of the jth component of u on the time
        monitor. `optr`[2,j-1] >= 0.0, for j = 1,2,...,`npde`.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    lenrwk : int
        The required value of `lenrwk` cannot be determined exactly in
        advance, but a suggested value is

            `lenrwk` = maxpts*`npde`*(5*l+18*`npde`+9)+2*maxpts,

        where l = `opti`[0] if `opti`[0] != 0 and l = 3 otherwise, and
        maxpts is the expected maximum number of grid points at any one
        level.

        If during the execution the supplied value is found to be too
        small then the function returns with `errno` = 3 and an
        estimated required size is printed on the current error message
        unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        Note: the size of `lenrwk` cannot be checked upon initial entry
        to ``dim2_gen_order2_rectilinear`` since the number of grid
        points on the base grid is not known.

    leniwk : int
        The required value of `leniwk` cannot be determined exactly in
        advance, but a suggested value is

            `leniwk` = maxpts*(14+5*m)+7*m+2,

        where maxpts is the expected maximum number of grid points at
        any one level and m = `opti`[0] if `opti`[0] > 0 and m = 3
        otherwise.

        If during the execution the supplied value is found to be too
        small then the function returns with `errno` = 3 and an
        estimated required size is printed on the current error message
        unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        Note: the size of `leniwk` cannot be checked upon initial entry
        to ``dim2_gen_order2_rectilinear`` since the number of grid
        points on the base grid is not known.

    lenlwk : int
        The required value of `lenlwk` cannot be determined exactly in
        advance, but a suggested value is

            `lenlwk` = maxpts+1,

        where maxpts is the expected maximum number of grid points at
        any one level.

        If during the execution the supplied value is found to be too
        small then the function returns with `errno` = 3 and an
        estimated required size is printed.

        Note: the size of `lenlwk` cannot be checked upon initial entry
        to ``dim2_gen_order2_rectilinear`` since the number of grid
        points on the base grid is not known.

    itrace : int
        The level of trace information required from
        ``dim2_gen_order2_rectilinear``. `itrace` may take the value -1,
        0, 1, 2 or 3.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed.

        `itrace` > 0

          Output from the underlying solver is printed. This output
          contains details of the time integration, the nonlinear
          iteration and the linear solver.

        If `itrace` < -1, -1 is assumed and similarly if `itrace` > 3, 3
        is assumed.

        The advisory messages are given in greater detail as `itrace`
        increases.

        Setting `itrace` = 1 allows you to monitor the progress of the
        integration without possibly excessive information.

    ind : int
        Must be set to 0 or 1, alternatively 10 or 11.

        `ind` = 0

          Starts the integration in time. `pdedef` is assumed to be
          serial.

        `ind` = 1

          Continues the integration after an earlier exit from the
          function. In this case, only the following parameters may be
          reset between calls to ``dim2_gen_order2_rectilinear``: tout,
          dt, tols, tolt, opti, optr, itrace and errno. `pdedef` is
          assumed to be serial.

        `ind` = 10

          Starts the integration in time. `pdedef` is assumed to have
          been parallelized by you, as described in [Parallelism and
          Performance]. In all other respects, this is equivalent to
          `ind` = 0.

        `ind` = 11

          Continues the integration after an earlier exit from the
          function. In this case, only the following parameters may be
          reset between calls to ``dim2_gen_order2_rectangle``: tout,
          dt, tols, tolt, opti, optr, itrace and errno. `pdedef` is
          assumed to have been parallelized by you, as described in
          [Parallelism and Performance]. In all other respects, this is
          equivalent to `ind` = 1.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ts : float
        The value of t which has been reached. Normally `ts` = `tout`.

    ind : int
        `ind` = 1, if `ind` on input was 0 or 1, or `ind` = 11, if `ind`
        on input was 10 or 11.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ind` is not equal to 0 or 1: `ind` = *<value>*.

        (`errno` 1)
            On entry, `npde` = *<value>*.

            Constraint: `npde` >= 1.

        (`errno` 1)
            On entry, `dt`[0] = *<value>* and `dt`[2] = *<value>*. Note
            that `dt`[2] was reset to default if zero on entry.

            Constraint: if `ind` = 0, `dt`[0] <= `dt`[2].

        (`errno` 1)
            On entry, `dt`[0] = *<value>* and `dt`[1] = *<value>*. Note
            that `dt`[1] was reset to default if zero on entry.

            Constraint: if `ind` = 0, `dt`[0] >= `dt`[1].

        (`errno` 1)
            On entry, `ind` = 0 and `dt`[0] too large: `dt`[0] =
            *<value>* and maximum value = *<value>*.

        (`errno` 1)
            On entry, `ind` = 0 and `dt`[0] too small: `dt`[0] =
            *<value>* and minimum value = *<value>*.

        (`errno` 1)
            On entry, `dt`[0] = *<value>*.

            Constraint: if `ind` = 0, `dt`[0] >= 0.0.

        (`errno` 1)
            On entry, `dt`[1] = *<value>* and `dt`[2] = *<value>*.

            Constraint: `dt`[1] <= `dt`[2].

        (`errno` 1)
            On entry, `dt`[2] = *<value>*.

            Constraint: `dt`[2] >= 0.0.

        (`errno` 1)
            On entry, `dt`[1] = *<value>*.

            Constraint: `dt`[1] >= 0.0.

        (`errno` 1)
            On entry, i = *<value>*, j = *<value>* and `optr`[i-1,j-1] =
            *<value>*.

            Constraint: `optr`[i-1,j-1] >= 0.0.

        (`errno` 1)
            On entry, j = *<value>* and `optr`[0,j-1] = *<value>*.

            Constraint: `optr`[0,j-1] > 0.0.

        (`errno` 1)
            On entry, `opti`[0] = *<value>*, i = *<value>* and
            `opti`[i-1] = *<value>*.

            Constraint: if `opti`[0] > 0, `opti`[i-1] >= 0.

        (`errno` 1)
            On entry, `opti`[0] = *<value>*.

            Constraint: `opti`[0] >= 0.

        (`errno` 1)
            On entry, `tolt` = *<value>*.

            Constraint: `tolt` > 0.0.

        (`errno` 1)
            On entry, `tols` = *<value>*.

            Constraint: `tols` > 0.0.

        (`errno` 1)
            On entry, `tout`-`ts` too small: `tout`-`ts` = *<value>*.

        (`errno` 1)
            On entry, `tout` = *<value>* and `ts` = *<value>*.

            Constraint: `tout` > `ts`.

        (`errno` 2)
            Attempted time-step smaller than specified minimum. Check
            problem formulation in `pdedef`, `bndary` and `pdeiv`. Try
            increasing `itrace` for more information.

        (`errno` 3)
            One or more of the workspace arrays are too small. Try
            increasing `itrace` for more information.

        (`errno` 6)
            Invalid output argument from `inidom`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            IERR set to 1 in `monitr`. Integration completed as far as
            `ts`: `ts` = *<value>*.

        (`errno` 5)
            Integration completed, but maximum number of levels too
            small for required accuracy.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_gen_order2_rectilinear'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_inidom_name = fun_name + ':inidom'

    inidomh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, maxpts.
        utils._BYREF_FLOAT64, # Input/Output, xmin.
        utils._BYREF_FLOAT64, # Input/Output, xmax.
        utils._BYREF_FLOAT64, # Input/Output, ymin.
        utils._BYREF_FLOAT64, # Input/Output, ymax.
        utils._BYREF_INT, # Output, nx.
        utils._BYREF_INT, # Output, ny.
        utils._BYREF_INT, # Output, npts.
        utils._BYREF_INT, # Output, nrows.
        utils._BYREF_INT, # Output, nbnds.
        utils._BYREF_INT, # Output, nbpts.
        utils._EngineIntArrayType.c_type, # Output, lrow[maxpts].
        utils._EngineIntArrayType.c_type, # Output, irow[maxpts].
        utils._EngineIntArrayType.c_type, # Output, icol[maxpts].
        utils._EngineIntArrayType.c_type, # Output, llbnd[maxpts].
        utils._EngineIntArrayType.c_type, # Output, ilbnd[maxpts].
        utils._EngineIntArrayType.c_type, # Output, lbnd[maxpts].
        utils._BYREF_INT, # Input/Output, ierr.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[20].
    )

    def cb_inidomh(_inidom, en_data, maxpts, xmin, xmax, ymin, ymax, nx, ny, npts, nrows, nbnds, nbpts, lrow, irow, icol, llbnd, ilbnd, lbnd, ierr, _iuser, _ruser, iwsav):
        """Helper for inidom."""
        try:
            _maxpts_inidomh = utils._EngineIntScalarType.to_py(maxpts.contents)
            _lrow_inidomh = utils._EngineIntArrayType.from_pointer(
                lrow,
                (_maxpts_inidomh,),
                can_reference_contents=False,
            )
            _irow_inidomh = utils._EngineIntArrayType.from_pointer(
                irow,
                (_maxpts_inidomh,),
                can_reference_contents=False,
            )
            _icol_inidomh = utils._EngineIntArrayType.from_pointer(
                icol,
                (_maxpts_inidomh,),
                can_reference_contents=False,
            )
            _llbnd_inidomh = utils._EngineIntArrayType.from_pointer(
                llbnd,
                (_maxpts_inidomh,),
                can_reference_contents=False,
            )
            _ilbnd_inidomh = utils._EngineIntArrayType.from_pointer(
                ilbnd,
                (_maxpts_inidomh,),
                can_reference_contents=False,
            )
            _lbnd_inidomh = utils._EngineIntArrayType.from_pointer(
                lbnd,
                (_maxpts_inidomh,),
                can_reference_contents=False,
            )
            _ierr_inidomh = utils._EngineIntScalarType.to_py(ierr.contents)
            if data is not None:
                _xmin_inidomh, _xmax_inidomh, _ymin_inidomh, _ymax_inidomh, _nx_inidomh, _ny_inidomh, _npts_inidomh, _nrows_inidomh, _nbnds_inidomh, _nbpts_inidomh, _ierr_inidomh = inidom(
                    _maxpts_inidomh,
                    _lrow_inidomh,
                    _irow_inidomh,
                    _icol_inidomh,
                    _llbnd_inidomh,
                    _ilbnd_inidomh,
                    _lbnd_inidomh,
                    _ierr_inidomh,
                    data,
                )
            else:
                _xmin_inidomh, _xmax_inidomh, _ymin_inidomh, _ymax_inidomh, _nx_inidomh, _ny_inidomh, _npts_inidomh, _nrows_inidomh, _nbnds_inidomh, _nbpts_inidomh, _ierr_inidomh = inidom(
                    _maxpts_inidomh,
                    _lrow_inidomh,
                    _irow_inidomh,
                    _icol_inidomh,
                    _llbnd_inidomh,
                    _ilbnd_inidomh,
                    _lbnd_inidomh,
                    _ierr_inidomh,
                )
            utils._EngineFloat64ScalarType(
                _xmin_inidomh, {'fun_name': cb_inidom_name, 'returned': 'xmin'},
            ).refresh_ctypes_buffer(xmin)
            utils._EngineFloat64ScalarType(
                _xmax_inidomh, {'fun_name': cb_inidom_name, 'returned': 'xmax'},
            ).refresh_ctypes_buffer(xmax)
            utils._EngineFloat64ScalarType(
                _ymin_inidomh, {'fun_name': cb_inidom_name, 'returned': 'ymin'},
            ).refresh_ctypes_buffer(ymin)
            utils._EngineFloat64ScalarType(
                _ymax_inidomh, {'fun_name': cb_inidom_name, 'returned': 'ymax'},
            ).refresh_ctypes_buffer(ymax)
            utils._EngineIntScalarType(
                _nx_inidomh, {'fun_name': cb_inidom_name, 'returned': 'nx'},
            ).refresh_ctypes_buffer(nx)
            utils._EngineIntScalarType(
                _ny_inidomh, {'fun_name': cb_inidom_name, 'returned': 'ny'},
            ).refresh_ctypes_buffer(ny)
            utils._EngineIntScalarType(
                _npts_inidomh, {'fun_name': cb_inidom_name, 'returned': 'npts'},
            ).refresh_ctypes_buffer(npts)
            utils._EngineIntScalarType(
                _nrows_inidomh, {'fun_name': cb_inidom_name, 'returned': 'nrows'},
            ).refresh_ctypes_buffer(nrows)
            utils._EngineIntScalarType(
                _nbnds_inidomh, {'fun_name': cb_inidom_name, 'returned': 'nbnds'},
            ).refresh_ctypes_buffer(nbnds)
            utils._EngineIntScalarType(
                _nbpts_inidomh, {'fun_name': cb_inidom_name, 'returned': 'nbpts'},
            ).refresh_ctypes_buffer(nbpts)
            utils._EngineIntArrayType.refresh_ctypes_buffer(
                _lrow_inidomh, {'fun_name': cb_inidom_name, 'returned': 'lrow'}, lrow,
            )
            utils._unlink_pointer(lrow)
            utils._EngineIntArrayType.refresh_ctypes_buffer(
                _irow_inidomh, {'fun_name': cb_inidom_name, 'returned': 'irow'}, irow,
            )
            utils._unlink_pointer(irow)
            utils._EngineIntArrayType.refresh_ctypes_buffer(
                _icol_inidomh, {'fun_name': cb_inidom_name, 'returned': 'icol'}, icol,
            )
            utils._unlink_pointer(icol)
            utils._EngineIntArrayType.refresh_ctypes_buffer(
                _llbnd_inidomh, {'fun_name': cb_inidom_name, 'returned': 'llbnd'}, llbnd,
            )
            utils._unlink_pointer(llbnd)
            utils._EngineIntArrayType.refresh_ctypes_buffer(
                _ilbnd_inidomh, {'fun_name': cb_inidom_name, 'returned': 'ilbnd'}, ilbnd,
            )
            utils._unlink_pointer(ilbnd)
            utils._EngineIntArrayType.refresh_ctypes_buffer(
                _lbnd_inidomh, {'fun_name': cb_inidom_name, 'returned': 'lbnd'}, lbnd,
            )
            utils._unlink_pointer(lbnd)
            utils._EngineIntScalarType(
                _ierr_inidomh, {'fun_name': cb_inidom_name, 'returned': 'ierr'},
            ).refresh_ctypes_buffer(ierr)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_pdedef_name = fun_name + ':pdedef'

    pdedefh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, y[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uy[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uxx[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uxy[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uyy[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Output, res[npts, npde].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdedefh(_pdedef, en_data, npts, npde, t, x, y, u, ut, ux, uy, uxx, uxy, uyy, res, _iuser, _ruser):
        """Helper for pdedef."""
        try:
            _npts_pdedefh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_pdedefh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdedefh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_pdedefh,),
            )
            _y_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_npts_pdedefh,),
            )
            _u_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _ut_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _ux_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uy_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uy,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uxx_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uxx,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uxy_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uxy,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _uyy_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                uyy,
                (_npts_pdedefh, _npde_pdedefh),
                sorder=_m_sorder,
            )
            _res_pdedefh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_npts_pdedefh, _npde_pdedefh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                pdedef(
                    _npts_pdedefh,
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _y_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _uy_pdedefh,
                    _uxx_pdedefh,
                    _uxy_pdedefh,
                    _uyy_pdedefh,
                    _res_pdedefh,
                    data,
                )
            else:
                pdedef(
                    _npts_pdedefh,
                    _npde_pdedefh,
                    _t_pdedefh,
                    _x_pdedefh,
                    _y_pdedefh,
                    _u_pdedefh,
                    _ut_pdedefh,
                    _ux_pdedefh,
                    _uy_pdedefh,
                    _uxx_pdedefh,
                    _uxy_pdedefh,
                    _uyy_pdedefh,
                    _res_pdedefh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(y)
            utils._unlink_pointer(u)
            utils._unlink_pointer(ut)
            utils._unlink_pointer(ux)
            utils._unlink_pointer(uy)
            utils._unlink_pointer(uxx)
            utils._unlink_pointer(uxy)
            utils._unlink_pointer(uyy)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _res_pdedefh, {'fun_name': cb_pdedef_name, 'returned': 'res'}, res,
            )
            utils._unlink_pointer(res)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bndary_name = fun_name + ':bndary'

    bndaryh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, y[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, u[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ut[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, ux[npts, npde].
        utils._EngineFloat64ArrayType.c_type, # Input, uy[npts, npde].
        utils._BYREF_INT, # Input, nbnds.
        utils._BYREF_INT, # Input, nbpts.
        utils._EngineIntArrayType.c_type, # Input, llbnd[nbnds].
        utils._EngineIntArrayType.c_type, # Input, ilbnd[nbnds].
        utils._EngineIntArrayType.c_type, # Input, lbnd[nbpts].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, res[npts, npde].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bndaryh(_bndary, en_data, npts, npde, t, x, y, u, ut, ux, uy, nbnds, nbpts, llbnd, ilbnd, lbnd, res, _iuser, _ruser):
        """Helper for bndary."""
        try:
            _npts_bndaryh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_bndaryh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_bndaryh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_bndaryh,),
            )
            _y_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_npts_bndaryh,),
            )
            _u_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _ut_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ut,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _ux_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                ux,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _uy_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                uy,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            _nbnds_bndaryh = utils._EngineIntScalarType.to_py(nbnds.contents)
            _nbpts_bndaryh = utils._EngineIntScalarType.to_py(nbpts.contents)
            _llbnd_bndaryh = utils._EngineIntArrayType.from_pointer(
                llbnd,
                (_nbnds_bndaryh,),
            )
            _ilbnd_bndaryh = utils._EngineIntArrayType.from_pointer(
                ilbnd,
                (_nbnds_bndaryh,),
            )
            _lbnd_bndaryh = utils._EngineIntArrayType.from_pointer(
                lbnd,
                (_nbpts_bndaryh,),
            )
            _res_bndaryh = utils._EngineFloat64ArrayType.from_pointer(
                res,
                (_npts_bndaryh, _npde_bndaryh),
                sorder=_m_sorder,
            )
            if data is not None:
                bndary(
                    _npts_bndaryh,
                    _npde_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _y_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _ux_bndaryh,
                    _uy_bndaryh,
                    _nbnds_bndaryh,
                    _nbpts_bndaryh,
                    _llbnd_bndaryh,
                    _ilbnd_bndaryh,
                    _lbnd_bndaryh,
                    _res_bndaryh,
                    data,
                )
            else:
                bndary(
                    _npts_bndaryh,
                    _npde_bndaryh,
                    _t_bndaryh,
                    _x_bndaryh,
                    _y_bndaryh,
                    _u_bndaryh,
                    _ut_bndaryh,
                    _ux_bndaryh,
                    _uy_bndaryh,
                    _nbnds_bndaryh,
                    _nbpts_bndaryh,
                    _llbnd_bndaryh,
                    _ilbnd_bndaryh,
                    _lbnd_bndaryh,
                    _res_bndaryh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(y)
            utils._unlink_pointer(u)
            utils._unlink_pointer(ut)
            utils._unlink_pointer(ux)
            utils._unlink_pointer(uy)
            utils._unlink_pointer(llbnd)
            utils._unlink_pointer(ilbnd)
            utils._unlink_pointer(lbnd)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _res_bndaryh, {'fun_name': cb_bndary_name, 'returned': 'res'}, res,
            )
            utils._unlink_pointer(res)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_pdeiv_name = fun_name + ':pdeiv'

    pdeivh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npts.
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, x[npts].
        utils._EngineFloat64ArrayType.c_type, # Input, y[npts].
        utils._EngineFloat64ArrayType.c_type, # Output, u[npts, npde].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_pdeivh(_pdeiv, en_data, npts, npde, t, x, y, u, _iuser, _ruser):
        """Helper for pdeiv."""
        try:
            _npts_pdeivh = utils._EngineIntScalarType.to_py(npts.contents)
            _npde_pdeivh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_pdeivh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _x_pdeivh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npts_pdeivh,),
            )
            _y_pdeivh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_npts_pdeivh,),
            )
            _u_pdeivh = utils._EngineFloat64ArrayType.from_pointer(
                u,
                (_npts_pdeivh, _npde_pdeivh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                pdeiv(
                    _npts_pdeivh,
                    _npde_pdeivh,
                    _t_pdeivh,
                    _x_pdeivh,
                    _y_pdeivh,
                    _u_pdeivh,
                    data,
                )
            else:
                pdeiv(
                    _npts_pdeivh,
                    _npde_pdeivh,
                    _t_pdeivh,
                    _x_pdeivh,
                    _y_pdeivh,
                    _u_pdeivh,
                )
            utils._unlink_pointer(x)
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _u_pdeivh, {'fun_name': cb_pdeiv_name, 'returned': 'u'}, u,
            )
            utils._unlink_pointer(u)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, dt.
        utils._BYREF_FLOAT64, # Input, dtnew.
        utils._BYREF_LOGICAL, # Input, tlast.
        utils._BYREF_INT, # Input, nlev.
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, ymin.
        utils._BYREF_FLOAT64, # Input, dxb.
        utils._BYREF_FLOAT64, # Input, dyb.
        utils._EngineIntArrayType.c_type, # Input, lgrid[nlev+1].
        utils._EngineIntArrayType.c_type, # Input, istruc[nistruc].
        utils._BYREF_INT, # Input, nistruc.
        utils._EngineIntArrayType.c_type, # Input, lsol[nlev].
        utils._EngineFloat64ArrayType.c_type, # Input, sol[nsol].
        utils._BYREF_INT, # Input, nsol.
        utils._BYREF_INT, # Input/Output, ierr.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monitrh(_monitr, en_data, npde, t, dt, dtnew, tlast, nlev, xmin, ymin, dxb, dyb, lgrid, istruc, nistruc, lsol, sol, nsol, ierr, _iuser, _ruser):
        """Helper for monitr."""
        try:
            _npde_monitrh = utils._EngineIntScalarType.to_py(npde.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _dt_monitrh = utils._EngineFloat64ScalarType.to_py(dt.contents)
            _dtnew_monitrh = utils._EngineFloat64ScalarType.to_py(dtnew.contents)
            _tlast_monitrh = utils._EngineLogicalScalarType.to_py(tlast.contents)
            _nlev_monitrh = utils._EngineIntScalarType.to_py(nlev.contents)
            _xmin_monitrh = utils._EngineFloat64ScalarType.to_py(xmin.contents)
            _ymin_monitrh = utils._EngineFloat64ScalarType.to_py(ymin.contents)
            _dxb_monitrh = utils._EngineFloat64ScalarType.to_py(dxb.contents)
            _dyb_monitrh = utils._EngineFloat64ScalarType.to_py(dyb.contents)
            _lgrid_monitrh = utils._EngineIntArrayType.from_pointer(
                lgrid,
                (_nlev_monitrh+1,),
            )
            _nistruc_monitrh = utils._EngineIntScalarType.to_py(nistruc.contents)
            _istruc_monitrh = utils._EngineIntArrayType.from_pointer(
                istruc,
                (_nistruc_monitrh,),
            )
            _lsol_monitrh = utils._EngineIntArrayType.from_pointer(
                lsol,
                (_nlev_monitrh,),
            )
            _nsol_monitrh = utils._EngineIntScalarType.to_py(nsol.contents)
            _sol_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                sol,
                (_nsol_monitrh,),
            )
            _ierr_monitrh = utils._EngineIntScalarType.to_py(ierr.contents)
            if data is not None:
                _ierr_monitrh = monitr(
                    _npde_monitrh,
                    _t_monitrh,
                    _dt_monitrh,
                    _dtnew_monitrh,
                    _tlast_monitrh,
                    _nlev_monitrh,
                    _xmin_monitrh,
                    _ymin_monitrh,
                    _dxb_monitrh,
                    _dyb_monitrh,
                    _lgrid_monitrh,
                    _istruc_monitrh,
                    _nistruc_monitrh,
                    _lsol_monitrh,
                    _sol_monitrh,
                    _nsol_monitrh,
                    _ierr_monitrh,
                    data,
                )
            else:
                _ierr_monitrh = monitr(
                    _npde_monitrh,
                    _t_monitrh,
                    _dt_monitrh,
                    _dtnew_monitrh,
                    _tlast_monitrh,
                    _nlev_monitrh,
                    _xmin_monitrh,
                    _ymin_monitrh,
                    _dxb_monitrh,
                    _dyb_monitrh,
                    _lgrid_monitrh,
                    _istruc_monitrh,
                    _nistruc_monitrh,
                    _lsol_monitrh,
                    _sol_monitrh,
                    _nsol_monitrh,
                    _ierr_monitrh,
                )
            utils._unlink_pointer(lgrid)
            utils._unlink_pointer(istruc)
            utils._unlink_pointer(lsol)
            utils._unlink_pointer(sol)
            utils._EngineIntScalarType(
                _ierr_monitrh, {'fun_name': cb_monitr_name, 'returned': 'ierr'},
            ).refresh_ctypes_buffer(ierr)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d03rbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, npde.
        utils._BYREF_FLOAT64, # Input/Output, ts.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dt[3].
        utils._BYREF_FLOAT64, # Input, tols.
        utils._BYREF_FLOAT64, # Input, tolt.
        utils._UNUSED_FUNCTYPE,
        inidomh,
        utils._UNUSED_FUNCTYPE,
        pdedefh,
        utils._UNUSED_FUNCTYPE,
        bndaryh,
        utils._UNUSED_FUNCTYPE,
        pdeivh,
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._EngineIntArrayType.c_type, # Input, opti[4].
        utils._EngineFloat64ArrayType.c_type, # Input, optr[3, npde].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwk[lenrwk].
        utils._BYREF_INT, # Input, lenrwk.
        utils._EngineIntArrayType.c_type, # Input/Output, iwk[leniwk].
        utils._BYREF_INT, # Input, leniwk.
        utils._EngineLogicalArrayType.c_type, # Workspace, lwk.
        utils._BYREF_INT, # Input, lenlwk.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[20].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[25].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((optr, 'optr'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _npde = utils._EngineIntScalarType(
        npde, {'fun_name': fun_name, 'entity': 'npde'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _dt_cls = utils._EngineFloat64ArrayType(
        dt, {'fun_name': fun_name, 'entity': 'dt'},
        exp_shape=(3,),
    )
    _dt = _dt_cls.to_ctypes()
    _tols = utils._EngineFloat64ScalarType(
        tols, {'fun_name': fun_name, 'entity': 'tols'},
    ).to_ctypes()
    _tolt = utils._EngineFloat64ScalarType(
        tolt, {'fun_name': fun_name, 'entity': 'tolt'},
    ).to_ctypes()
    utils._check_callable(inidom, {'fun_name': fun_name, 'entity': 'inidom'})
    _inidom = utils._UNUSED_CALLBACK
    _inidomh = utils._cast_callback(
        inidomh, cb_inidomh,
        {'fun_name': fun_name, 'entity': 'inidom'},
    )
    utils._check_callable(pdedef, {'fun_name': fun_name, 'entity': 'pdedef'})
    _pdedef = utils._UNUSED_CALLBACK
    _pdedefh = utils._cast_callback(
        pdedefh, cb_pdedefh,
        {'fun_name': fun_name, 'entity': 'pdedef'},
    )
    utils._check_callable(bndary, {'fun_name': fun_name, 'entity': 'bndary'})
    _bndary = utils._UNUSED_CALLBACK
    _bndaryh = utils._cast_callback(
        bndaryh, cb_bndaryh,
        {'fun_name': fun_name, 'entity': 'bndary'},
    )
    utils._check_callable(pdeiv, {'fun_name': fun_name, 'entity': 'pdeiv'})
    _pdeiv = utils._UNUSED_CALLBACK
    _pdeivh = utils._cast_callback(
        pdeivh, cb_pdeivh,
        {'fun_name': fun_name, 'entity': 'pdeiv'},
    )
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'})
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _opti = utils._EngineIntArrayType(
        opti, {'fun_name': fun_name, 'entity': 'opti'},
        exp_shape=(4,),
    ).to_ctypes()
    _npde_py = utils._EngineIntScalarType.to_py(_npde)
    _optr = utils._EngineFloat64ArrayType(
        optr, {'fun_name': fun_name, 'entity': 'optr'},
        exp_shape=(3, _npde_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lenrwk = utils._EngineIntScalarType(
        lenrwk, {'fun_name': fun_name, 'entity': 'lenrwk'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rwk', 'iwk', 'lwsav', 'iwsav', 'rwsav'),
    )
    _lenrwk_py = utils._EngineIntScalarType.to_py(_lenrwk)
    if _comm_needs_init:
        _rwk = utils._EngineFloat64ArrayType.empty_ctypes(_lenrwk_py)
        comm['rwk'] = _rwk
    else:
        _rwk = comm['rwk']
    _leniwk = utils._EngineIntScalarType(
        leniwk, {'fun_name': fun_name, 'entity': 'leniwk'},
    ).to_ctypes()
    _leniwk_py = utils._EngineIntScalarType.to_py(_leniwk)
    if _comm_needs_init:
        _iwk = utils._EngineIntArrayType.empty_ctypes(_leniwk_py)
        comm['iwk'] = _iwk
    else:
        _iwk = comm['iwk']
    _lwk = utils._EngineLogicalArrayType.empty_ctypes(0)
    _lenlwk = utils._EngineIntScalarType(
        lenlwk, {'fun_name': fun_name, 'entity': 'lenlwk'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(20)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(25)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _npde,
        _ts,
        _tout,
        _dt,
        _tols,
        _tolt,
        _inidom,
        _inidomh,
        _pdedef,
        _pdedefh,
        _bndary,
        _bndaryh,
        _pdeiv,
        _pdeivh,
        _monitr,
        _monitrh,
        _opti,
        _optr,
        _rwk,
        _lenrwk,
        _iwk,
        _leniwk,
        _lwk,
        _lenlwk,
        _itrace,
        _nadv,
        _ind,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind is not equal to 0 or 1: ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npde = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npde >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'dt[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Note that dt[2] was reset to default if zero on entry.',
                    'Constraint: if ind = 0, dt[0] <= dt[2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'dt[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Note that dt[1] was reset to default if zero on entry.',
                    'Constraint: if ind = 0, dt[0] >= dt[1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 0 and dt[0] too large:',
                    'dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'maximum value = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 0 and dt[0] too small:',
                    'dt[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'minimum value = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if ind = 0, dt[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'dt[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: dt[1] <= dt[2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dt[2] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dt[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dt[1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, j = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'optr[i-1,j-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: optr[i-1,j-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, j = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'optr[0,j-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: optr[0,j-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, opti[0] = {:s}, i = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'opti[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: if opti[0] > 0, opti[i-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, opti[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: opti[0] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tolt = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tolt > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tols = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tols > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout-ts too small: tout-ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tout > ts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Attempted time-step smaller than specified minimum.',
                    'Check problem formulation in pdedef, bndary and pdeiv.',
                    'Try increasing itrace for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'One or more of the workspace arrays are too small.',
                    'Try increasing itrace for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99979:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'IERR set to 1 in monitr. Integration completed as far as ts:',
                    'ts = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99978:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration completed, but maximum number of levels too small',
                    'for required accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Invalid output argument from inidom.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _dt_cls.refresh_py(_dt)
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'ind'],
        )(
            _ts_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim2_gen_order2_rectilinear_extractgrid(level, nlev, xmin, ymin, dxb, dyb, lgrid, istruc, x, y, lenxy):
    u"""
    Extract grid data from ``dim2_gen_order2_rectilinear``.

    For full information please refer to the NAG Library document for
    d03rz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03rzf.html

    Parameters
    ----------
    level : int
        The grid level at which the coordinates are required.

    nlev : int
        `nlev` as supplied to monitr must be passed unchanged to
        ``dim2_gen_order2_rectilinear_extractgrid``

    xmin : float
        `xmin` as supplied to monitr must be passed unchanged to
        ``dim2_gen_order2_rectilinear_extractgrid``

    ymin : float
        `ymin` as supplied to monitr must be passed unchanged to
        ``dim2_gen_order2_rectilinear_extractgrid``

    dxb : float
        `dxb` as supplied to monitr must be passed unchanged to
        ``dim2_gen_order2_rectilinear_extractgrid``

    dyb : float
        `dyb` as supplied to monitr must be passed unchanged to
        ``dim2_gen_order2_rectilinear_extractgrid``

    lgrid : int, shape (`nlev`)
        `lgrid` as supplied to monitr must be passed unchanged to
        ``dim2_gen_order2_rectilinear_extractgrid``.

    istruc : int, shape (`lgrid`[`nlev`-1]+2*nrows+npts+1)
        `istruc` as supplied to monitr must be passed unchanged to
        ``dim2_gen_order2_rectilinear_extractgrid``.

    x : float, shape (`lenxy`), modified in place
        On exit: `x`[i-1] and `y`[i-1] contain the (x,y) coordinates
        respectively of the ith grid point, for i = 1,2,...,`npts`.

    y : float, shape (`lenxy`), modified in place
        On exit: `x`[i-1] and `y`[i-1] contain the (x,y) coordinates
        respectively of the ith grid point, for i = 1,2,...,`npts`.

    lenxy : int
        The dimension of the arrays `x` and `y`.

    Returns
    -------
    npts : int
        The number of grid points in the grid level `level`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `level` = *<value>* and `nlev` = *<value>*.

            Constraint: `level` <= `nlev`.

        (`errno` 1)
            On entry, `level` = *<value>*.

            Constraint: `level` >= 1.

        (`errno` 2)
            On entry, `lenxy` = *<value>*.

            Constraint: `lenxy` >= *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_gen_order2_rectilinear_extractgrid'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03rzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, level.
        utils._BYREF_INT, # Input, nlev.
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, ymin.
        utils._BYREF_FLOAT64, # Input, dxb.
        utils._BYREF_FLOAT64, # Input, dyb.
        utils._EngineIntArrayType.c_type, # Input, lgrid[nlev].
        utils._EngineIntArrayType.c_type, # Input, istruc[*].
        utils._BYREF_INT, # Output, npts.
        utils._EngineFloat64ArrayType.c_type, # Output, x[lenxy].
        utils._EngineFloat64ArrayType.c_type, # Output, y[lenxy].
        utils._BYREF_INT, # Input, lenxy.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _level = utils._EngineIntScalarType(
        level, {'fun_name': fun_name, 'entity': 'level'},
    ).to_ctypes()
    _nlev = utils._EngineIntScalarType(
        nlev, {'fun_name': fun_name, 'entity': 'nlev'},
    ).to_ctypes()
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _ymin = utils._EngineFloat64ScalarType(
        ymin, {'fun_name': fun_name, 'entity': 'ymin'},
    ).to_ctypes()
    _dxb = utils._EngineFloat64ScalarType(
        dxb, {'fun_name': fun_name, 'entity': 'dxb'},
    ).to_ctypes()
    _dyb = utils._EngineFloat64ScalarType(
        dyb, {'fun_name': fun_name, 'entity': 'dyb'},
    ).to_ctypes()
    _nlev_py = utils._EngineIntScalarType.to_py(_nlev)
    _lgrid = utils._EngineIntArrayType(
        lgrid, {'fun_name': fun_name, 'entity': 'lgrid'},
        exp_shape=(_nlev_py,),
    ).to_ctypes()
    if _nlev_py >= 1:
        _vl_nrows_py = utils._EngineIntArrayType.get_element(istruc, {'fun_name': fun_name, 'entity': 'istruc'}, (utils._EngineIntArrayType.get_element(lgrid, {'fun_name': fun_name, 'entity': 'lgrid'}, (_nlev_py-1,))-1,))
    else:
        _vl_nrows_py = 0
    _level_py = utils._EngineIntScalarType.to_py(_level)
    if _level_py >= 1 and _level_py <= _nlev_py:
        if utils._EngineIntArrayType.get_element(lgrid, {'fun_name': fun_name, 'entity': 'lgrid'}, (_level_py-1,)) > 0:
            _vl_npts_py = utils._EngineIntArrayType.get_element(istruc, {'fun_name': fun_name, 'entity': 'istruc'}, ((utils._EngineIntArrayType.get_element(lgrid, {'fun_name': fun_name, 'entity': 'lgrid'}, (_level_py-1,))+_vl_nrows_py+1)-1,))-1
        else:
            _vl_npts_py = 0
    else:
        _vl_npts_py = 0
    _istruc = utils._EngineIntArrayType(
        istruc, {'fun_name': fun_name, 'entity': 'istruc'},
        exp_shape=(utils._EngineIntArrayType.get_element(lgrid, {'fun_name': fun_name, 'entity': 'lgrid'}, (_nlev_py-1,))+2*_vl_nrows_py+_vl_npts_py+1,),
    ).to_ctypes()
    _npts = utils._EngineIntScalarType.empty_ctypes()
    _lenxy = utils._EngineIntScalarType(
        lenxy, {'fun_name': fun_name, 'entity': 'lenxy'},
    ).to_ctypes()
    _lenxy_py = utils._EngineIntScalarType.to_py(_lenxy)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_lenxy_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_lenxy_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _level,
        _nlev,
        _xmin,
        _ymin,
        _dxb,
        _dyb,
        _lgrid,
        _istruc,
        _npts,
        _x,
        _y,
        _lenxy,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, level = {:s} and nlev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: level <= nlev.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, level = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: level >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenxy = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lenxy >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _npts_py = utils._EngineIntScalarType.to_py(_npts)
        return_data = _npts_py
    _err_class.manage()
    return return_data

def dim2_ellip_fd_iter(n1, n2, a, b, c, d, e, aparam, it, r):
    u"""
    Elliptic PDE, solution of finite difference equations by SIP,
    five-point two-dimensional molecule, one iteration.

    For full information please refer to the NAG Library document for
    d03ua

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03uaf.html

    Parameters
    ----------
    n1 : int
        The number of nodes in the first coordinate direction, n_1.

    n2 : int
        The number of nodes in the second coordinate direction, n_2.

    a : float, shape (`n1`, `n2`)
        `a`[i-1,j-1] must contain the coefficient of the 'southerly'
        term involving s_{i,j-1} in the (i,j)th equation of the system
        (2), for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of
        `a`, for j = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    b : float, shape (`n1`, `n2`)
        `b`[i-1,j-1] must contain the coefficient of the 'westerly' term
        involving s_{i-1,j} in the (i,j)th equation of the system (2),
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `b`,
        for i = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    c : float, shape (`n1`, `n2`)
        `c`[i-1,j-1] must contain the coefficient of the 'central' term
        involving s_{ij} in the (i,j)th equation of the system (2), for
        j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `c` are
        checked to ensure that they are nonzero. If any element is found
        to be zero, the corresponding algebraic equation is assumed to
        be s_{ij} = r_{ij}. This feature can be used to define the
        equations for nodes at which, for example, Dirichlet boundary
        conditions are applied, or for nodes external to the problem of
        interest, by setting `c`[i-1,j-1] = 0.0 at appropriate points.
        The corresponding value of `r`[i-1,j-1] is set equal to the
        appropriate value, namely the difference between the prescribed
        value of t_{ij} and the current value of t_{ij} in the Dirichlet
        case, or zero at an external point.

    d : float, shape (`n1`, `n2`)
        `d`[i-1,j-1] must contain the coefficient of the 'easterly' term
        involving s_{i+1,j} in the (i,j)th equation of the system (2),
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `d`,
        for i = `n1`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    e : float, shape (`n1`, `n2`)
        `e`[i-1,j-1] must contain the coefficient of the 'northerly'
        term involving s_{i,j+1} in the (i,j)th equation of the system
        (2), for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of
        `e`, for j = `n2`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        rectangle.

    aparam : float
        The iteration acceleration factor. A value of 1.0 is adequate
        for most typical problems. However, if convergence is slow, the
        value can be reduced, typically to 0.2 or 0.1. If divergence is
        obtained, the value can be increased, typically to 2.0, 5.0 or
        10.0.

    it : int
        The iteration number. It must be initialized, but not
        necessarily to 1, before the first call, and must be incremented
        by one in the calling program for each subsequent call.
        ``dim2_ellip_fd_iter`` uses the counter to select the
        appropriate acceleration argument from a sequence of nine, each
        one being used twice in succession. (Note that the acceleration
        argument depends on the value of `aparam`.)

    r : float, shape (`n1`, `n2`), modified in place
        On entry: `r`[i-1,j-1] must contain the current residual r_{ij}
        on the right-hand side of the (i,j)th equation of the system
        (2), for j = 1,2,...,`n2`, for i = 1,2,...,`n1`.

        On exit: these residuals are overwritten by the corresponding
        components of solution s to the system (2), i.e., the changes to
        be made to the vector t to reduce the residuals supplied.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` > 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` > 1.

        (`errno` 3)
            On entry, `aparam` = *<value>*.

            Constraint: `aparam` > 0.0.

        (`errno` 4)
            On entry, `aparam` = *<value>*, `n1` = *<value>* and `n2` =
            *<value>*.

            Constraint: `aparam` <= ((`n1`-1)^2+(`n2`-1)^2)/2.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim2_ellip_fd_iter'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03uaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, n2.
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, b[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, c[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, d[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Input, e[lda, n2].
        utils._BYREF_FLOAT64, # Input, aparam.
        utils._BYREF_INT, # Input, it.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[lda, n2].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp2.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (d, 'd'), (e, 'e'), (r, 'r')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _n2_py = utils._EngineIntScalarType.to_py(_n2)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _aparam = utils._EngineFloat64ScalarType(
        aparam, {'fun_name': fun_name, 'entity': 'aparam'},
    ).to_ctypes()
    _it = utils._EngineIntScalarType(
        it, {'fun_name': fun_name, 'entity': 'it'},
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n1_py, _n2_py),
        mutual_sorder=_m_sorder,
    )
    _r = _r_cls.to_ctypes()
    _wrksp1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _n2,
        _lda,
        _a,
        _b,
        _c,
        _d,
        _e,
        _aparam,
        _it,
        _r,
        _wrksp1,
        _wrksp2,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: aparam > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}, n1 = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: aparam <= ((n1-1)^2+(n2-1)^2)/2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _r_cls.refresh_py(_r)
        return_data = None
    _err_class.manage()
    return return_data

def dim3_ellip_fd_iter(n1, n2, n3, sda, a, b, c, d, e, f, g, aparam, it, r):
    u"""
    Elliptic PDE, solution of finite difference equations by SIP,
    seven-point three-dimensional molecule, one iteration.

    For full information please refer to the NAG Library document for
    d03ub

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d03/d03ubf.html

    Parameters
    ----------
    n1 : int
        The number of nodes in the first coordinate direction, n_1.

    n2 : int
        The number of nodes in the second coordinate direction, n_2.

    n3 : int
        The number of nodes in the third coordinate direction, n_3.

    sda : int
        The second dimension of the arrays `a`, `b`, `c`, `d`, `e`, `f`,
        `g`, `r`, `wrksp1`, `wrksp2` and `wrksp3`.

    a : float, shape (`n1`, `sda`, `n3`)
        `a`[i-1,j-1,k-1] must contain the coefficient of s_{ij,k-1} in
        the (i,j,k)th equation of the system (2), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `a`,
        for k = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    b : float, shape (`n1`, `sda`, `n3`)
        `b`[i-1,j-1,k-1] must contain the coefficient of s_{i,j-1,k} in
        the (i,j,k)th equation of the system (2), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `b`,
        for j = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    c : float, shape (`n1`, `sda`, `n3`)
        `c`[i-1,j-1,k-1] must contain the coefficient of s_{i-1,j,k} in
        the (i,j,k)th equation of the system (2), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `c`,
        for i = 1, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    d : float, shape (`n1`, `sda`, `n3`)
        `d`[i-1,j-1,k-1] must contain the coefficient of s_{ijk}, the
        'central' term, in the (i,j,k)th equation of the system (2), for
        k = 1,2,...,`n3`, for j = 1,2,...,`n2`, for i = 1,2,...,`n1`.
        The elements of `d` are checked to ensure that they are nonzero.
        If any element is found to be zero, the corresponding algebraic
        equation is assumed to be s_{ijk} = r_{ijk}. This feature can be
        used to define the equations for nodes at which, for example,
        Dirichlet boundary conditions are applied, or for nodes external
        to the problem of interest, by setting `d`[i-1,j-1,k-1] = 0.0 at
        appropriate points. The corresponding value of r_{ijk} is set
        equal to the appropriate value, namely the difference between
        the prescribed value of t_{ijk} and the current value in the
        Dirichlet case, or zero at an external point.

    e : float, shape (`n1`, `sda`, `n3`)
        `e`[i-1,j-1,k-1] must contain the coefficient of s_{i+1,j,k} in
        the (i,j,k)th equation of the system (2), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `e`,
        for i = `n1`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    f : float, shape (`n1`, `sda`, `n3`)
        `f`[i-1,j-1,k-1] must contain the coefficient of s_{i,j+1,k} in
        the (i,j,k)th equation of the system (2), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `f`,
        for j = `n2`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    g : float, shape (`n1`, `sda`, `n3`)
        `g`[i-1,j-1,k-1] must contain the coefficient of s_{i,j,k+1} in
        the (i,j,k)th equation of the system (2), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`. The elements of `g`,
        for k = `n3`, must be zero after incorporating the boundary
        conditions, since they involve nodal values from outside the
        box.

    aparam : float
        The iteration acceleration factor. A value of 1.0 is adequate
        for most typical problems. However, if convergence is slow, the
        value can be reduced, typically to 0.2 or 0.1. If divergence is
        obtained, the value can be increased, typically to 2.0, 5.0 or
        10.0.

    it : int
        The iteration number. It must be initialized, but not
        necessarily to 1, before the first call, and should be
        incremented by one in the calling program for each subsequent
        call. The function uses this counter to select the appropriate
        acceleration argument from a sequence of nine, each one being
        used twice in succession. (Note that the acceleration argument
        depends on the value of `aparam`.)

    r : float, shape (`n1`, `sda`, `n3`), modified in place
        On entry: the current residual r_{ijk} on the right-hand side of
        the (i,j,k)th equation of the system (2), for k = 1,2,...,`n3`,
        for j = 1,2,...,`n2`, for i = 1,2,...,`n1`.

        On exit: these residuals are overwritten by the corresponding
        components of the solution s of the system (2), i.e., the
        changes to be made to the vector t to reduce the residuals
        supplied.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n3` = *<value>*.

            Constraint: `n3` > 1.

        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` > 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` > 1.

        (`errno` 2)
            On entry, `sda` = *<value>* and `n2` = *<value>*.

            Constraint: `sda` >= `n2`.

        (`errno` 3)
            On entry, `aparam` = *<value>*.

            Constraint: `aparam` > 0.0.

        (`errno` 4)
            On entry, `aparam` = *<value>*, `n1` = *<value>*, `n2` =
            *<value>* and `n3` = *<value>*.

            Constraint: `aparam` <=
            ((`n1`-1)^2+(`n2`-1)^2+(`n3`-1)^2)/3.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.pde.dim3_ellip_fd_iter'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d03ubft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, n2.
        utils._BYREF_INT, # Input, n3.
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, sda.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, b[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, c[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, d[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, e[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, f[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Input, g[lda, sda, n3].
        utils._BYREF_FLOAT64, # Input, aparam.
        utils._BYREF_INT, # Input, it.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[lda, sda, n3].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp2.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrksp3.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (d, 'd'), (e, 'e'), (f, 'f'), (g, 'g'), (r, 'r')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _n3 = utils._EngineIntScalarType(
        n3, {'fun_name': fun_name, 'entity': 'n3'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3)[0]
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _sda = utils._EngineIntScalarType(
        sda, {'fun_name': fun_name, 'entity': 'sda'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _n2_py = utils._EngineIntScalarType.to_py(_n2)
    _n3_py = utils._EngineIntScalarType.to_py(_n3)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _g = utils._EngineFloat64ArrayType(
        g, {'fun_name': fun_name, 'entity': 'g'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _aparam = utils._EngineFloat64ScalarType(
        aparam, {'fun_name': fun_name, 'entity': 'aparam'},
    ).to_ctypes()
    _it = utils._EngineIntScalarType(
        it, {'fun_name': fun_name, 'entity': 'it'},
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n1_py, _n2_py, _n3_py),
        mutual_sorder=_m_sorder,
    )
    _r = _r_cls.to_ctypes()
    _wrksp1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wrksp3 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _n2,
        _n3,
        _lda,
        _sda,
        _a,
        _b,
        _c,
        _d,
        _e,
        _f,
        _g,
        _aparam,
        _it,
        _r,
        _wrksp1,
        _wrksp2,
        _wrksp3,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n3 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n3 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sda = {:s} and n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sda >= n2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: aparam > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, aparam = {:s}, n1 = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'n2 = {:s} and n3 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: aparam <= ((n1-1)^2+(n2-1)^2+(n3-1)^2)/3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _r_cls.refresh_py(_r)
        return_data = None
    _err_class.manage()
    return return_data
