# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `sparseig` Chapter.

``sparseig`` - Large Scale Eigenproblems

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module provides functions for computing some eigenvalues and
eigenvectors of large-scale (sparse) standard and generalized eigenvalue
problems.
It provides functions for:

- solution of symmetric eigenvalue problems;

- solution of nonsymmetric eigenvalue problems;

- solution of generalized symmetric-definite eigenvalue problems;

- solution of generalized nonsymmetric eigenvalue problems;

- partial singular value decomposition.

Functions are provided for both real and complex data.

The functions in this module have all been derived from the ARPACK
software suite (see ARPACK Users' Guide), a collection of Fortran 77
functions designed to solve large scale eigenvalue problems.
The interfaces provided in this module have been chosen to combine ease
of use with the flexibility of the original ARPACK software.
The underlying iterative methods and algorithms remain essentially the
same as those in ARPACK and are described fully in ARPACK Users' Guide.

The algorithms used are based upon an algorithmic variant of the Arnoldi
process called the Implicitly Restarted Arnoldi Method.
For symmetric matrices, this reduces to a variant of the Lanczos process
called the Implicitly Restarted Lanczos Method.
These variants may be viewed as a synthesis of the Arnoldi/Lanczos
process with the Implicitly Shifted QR technique that is suitable for
large scale problems.
For many standard problems, a matrix factorization is not required.
Only the action of the matrix on a vector is needed.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def real_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a real nonsymmetric
    sparse (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aaf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _icomm = utils._EngineIntArrayType.empty_ctypes(140)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _lcomm = utils._EngineIntScalarType(
        3*_n_py+3*_ncv_py*_ncv_py+6*_ncv_py+60, {'fun_name': fun_name, 'entity': '_lcomm'},
    ).to_ctypes()
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(3*_n_py+3*_ncv_py*_ncv_py+6*_ncv_py+60)
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric sparse eigenproblem, reverse communication.

    For full information please refer to the NAG Library document for
    f12ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12abf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must be unchanged from its previous exit
        value. Changing `irevcm` to any other value between calls will
        result in an error.

    resid : float, shape (n), modified in place
        On initial entry: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``real_option`` in
        which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        On intermediate entry: must be unchanged from its previous exit.
        Changing `resid` to any other value between calls may result in
        an error exit.

        On intermediate exit: contains the current residual vector.

        On final exit: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        On initial entry: need not be set.

        On intermediate entry: must be unchanged from its previous exit.

        On intermediate exit: contains the current set of Arnoldi basis
        vectors.

        On final exit: contains the final set of Arnoldi basis vectors.

    x : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `x` need not be set.

        If 'Pointers' = NO, `x` must contain the result of y = OPx when
        `irevcm` returns the value -1 or +1.

        It must return the real parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        On final exit: does not contain useful data.

    mx : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        It must return the imaginary parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        On final exit: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product y =
          OPx, where x is stored in `x` (by default) or in the array
          `comm`['comm'] (starting from the location given by the first
          element of `comm`['icomm']) when the option 'Pointers' = 'YES'
          is set in a prior call to ``real_option``. The result y is
          returned in `x` (by default) or in the array `comm`['comm']
          (starting from the location given by the second element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set. If
          B is not symmetric semidefinite then the precomputed values in
          `mx` should not be used (see the explanation under `irevcm` =
          2).

        `irevcm` = 1

          The calling program must compute the matrix-vector product y =
          OPx. This is similar to the case `irevcm` = -1 except that the
          result of the matrix-vector product Bx (as required in some
          computational modes) has already been computed and is
          available in `mx` (by default) or in the array `comm`['comm']
          (starting from the location given by the third element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 2

          The calling program must compute the matrix-vector product y =
          Bx, where x is stored as described in the case `irevcm` = -1
          and y is returned in the location described by the case
          `irevcm` = 1. This computation is requested when solving the
          'Generalized' problem using either 'Shifted Inverse
          Imaginary', 'Shifted Inverse Real'; in these cases B is used
          as an inner-product space and requires that B be symmetric
          semidefinite. If neither A nor B is symmetric semidefinite
          then the problem should be reformulated in a 'Standard' form.

        `irevcm` = 3

          Compute the `nshift` real and imaginary parts of the shifts
          where the real parts are to be returned in the first `nshift`
          locations of the array `x` and the imaginary parts are to be
          returned in the first `nshift` locations of the array `mx`.
          Only complex conjugate pairs of shifts may be applied and the
          pairs must be placed in consecutive locations. This value of
          this argument will only arise if the option 'Supplied Shifts'
          is set in a prior call to ``real_option`` which is intended
          for experienced users only; the default and recommended option
          is to use exact shifts (see ARPACK Users' Guide for details).

        `irevcm` = 4

          Monitoring step: a call to ``real_monit`` can now be made to
          return the number of Arnoldi iterations, the number of
          converged Ritz values, their real and imaginary parts, and the
          corresponding Ritz estimates.

        On final exit: `irevcm` = 5: ``real_iter`` has completed its
        tasks. The value of errno determines whether the iteration has
        been successfully completed, or whether errors have been
        detected. On successful completion ``real_proc`` must be called
        to return the requested eigenvalues and eigenvectors (and/or
        Schur vectors).

    nshift : int
        On intermediate exit: if the option 'Supplied Shifts' is set and
        `irevcm` returns a value of 3, this argument returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 5)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 6)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 7)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 4)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_iter'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, mx[*].
        utils._BYREF_INT, # Output, nshift.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aaf_n_py = 0
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12aaf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aaf_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12aaf_n_py, _vl_f12aaf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12aaf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
        _x_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
    else:
        _x_dim1 = 1
    exp_shape_x = (_x_dim1,)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
    )
    _x = _x_cls.to_ctypes()
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
        _mx_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
    else:
        _mx_dim1 = 1
    exp_shape_mx = (_mx_dim1,)
    _mx_cls = utils._EngineFloat64ArrayType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
        exp_shape=exp_shape_mx,
    )
    _mx = _mx_cls.to_ctypes()
    _nshift = utils._EngineIntScalarType.empty_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12ab ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _irevcm,
        _resid,
        _v,
        _ldv,
        _x,
        _mx,
        _nshift,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 5 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Arnoldi iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _x_cls.refresh_py(_x)
        _mx_cls.refresh_py(_mx)
        _nshift_py = utils._EngineIntScalarType.to_py(_nshift)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'nshift'],
        )(
            _irevcm_py,
            _nshift_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_proc(dr, di, z, sigmar, sigmai, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric sparse eigenproblem, postprocessing for ``real_iter``.

    For full information please refer to the NAG Library document for
    f12ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12acf.html

    Parameters
    ----------
    dr : float, shape (nev+1), modified in place
        On exit: the first `nconv` locations of the array `dr` contain
        the real parts of the converged approximate eigenvalues.

    di : float, shape (nev+1), modified in place
        On exit: the first `nconv` locations of the array `di` contain
        the imaginary parts of the converged approximate eigenvalues.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if int(Re(`comm`['comm'][0]))>=1 and
        int(Re(`comm`['comm'][0]))<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: nev+1;
        otherwise: 1; otherwise: 0.

        On exit: if the default option 'Vectors' = 'RITZ' (see
        ``real_option``) has been selected then `z` contains the final
        set of eigenvectors corresponding to the eigenvalues held in
        `dr` and `di`. The complex eigenvector associated with the
        eigenvalue with positive imaginary part is stored in two
        consecutive columns. The first column holds the real part of the
        eigenvector and the second column holds the imaginary part. The
        eigenvector associated with the eigenvalue with negative
        imaginary part is simply the complex conjugate of the
        eigenvector associated with the positive imaginary part.

    sigmar : float
        If one of the 'Shifted Inverse Real' modes have been selected
        then `sigmar` contains the real part of the shift used;
        otherwise `sigmar` is not referenced.

    sigmai : float
        If one of the 'Shifted Inverse Real' modes have been selected
        then `sigmai` contains the imaginary part of the shift used;
        otherwise `sigmai` is not referenced.

    resid : float, shape (n)
        Must not be modified following a call to ``real_iter`` since it
        contains data required by ``real_proc``.

    v : float, shape (n, ncv), modified in place
        On entry: the ncv columns of `v` contain the Arnoldi basis
        vectors for OP as constructed by ``real_iter``.

        On exit: if the option 'Vectors' = 'SCHUR' has been set, or the
        option 'Vectors' = 'RITZ' has been set and a separate array `z`
        has been passed (i.e., `z` does not equal `v`), then the first
        `nconv` columns of `v` will contain approximate Schur vectors
        that span the desired invariant subspace.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by ``real_iter``.

    Raises
    ------
    NagValueError
        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.
            See the function document for further details.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a real Schur form, there was a failure
            to compute *<value>* eigenvalues in a total of *<value>*
            iterations.

        (`errno` 6)
            The computed Schur form could not be reordered by an
            internal call. This function returned with errno =
            *<value>*. Please contact NAG.

        (`errno` 7)
            In calculating eigenvectors, an internal call returned with
            an error. Please contact NAG.

        (`errno` 8)
            Either the solver function has not been called prior to the
            call of this function or a communication array has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, 'Vectors' = 'SELECT', but this is not yet
            implemented.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_proc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, dr[*].
        utils._EngineFloat64ArrayType.c_type, # Output, di[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input, sigmar.
        utils._BYREF_FLOAT64, # Input, sigmai.
        utils._EngineFloat64ArrayType.c_type, # Input, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1,), validate=False)
    else:
        _vl_f12aaf_nev_py = 0
    _dr_cls = utils._EngineFloat64ArrayType(
        dr, {'fun_name': fun_name, 'entity': 'dr'},
        exp_shape=(_vl_f12aaf_nev_py+1,),
        check_contents=False,
    )
    _dr = _dr_cls.to_ctypes(can_reference_contents=False)
    _di_cls = utils._EngineFloat64ArrayType(
        di, {'fun_name': fun_name, 'entity': 'di'},
        exp_shape=(_vl_f12aaf_nev_py+1,),
        check_contents=False,
    )
    _di = _di_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aaf_n_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12aaf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12aaf_nev_py+1
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_vl_f12aaf_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _sigmar = utils._EngineFloat64ScalarType(
        sigmar, {'fun_name': fun_name, 'entity': 'sigmar'},
    ).to_ctypes()
    _sigmai = utils._EngineFloat64ScalarType(
        sigmai, {'fun_name': fun_name, 'entity': 'sigmai'},
    ).to_ctypes()
    _resid = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12aaf_n_py,),
    ).to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aaf_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12aaf_n_py, _vl_f12aaf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12ac ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _nconv,
        _dr,
        _di,
        _z,
        _ldz,
        _sigmar,
        _sigmai,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = \'SELECT\', but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy,',
                    'as communicated through the argument comm[\'icomm\'], is zero.',
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Got a different count of the number of converged Ritz values',
                    'than the value passed to it through the argument comm[\'icomm\']:',
                    'number counted = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a real Schur form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'This function returned with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In calculating eigenvectors, an internal call returned with an error.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the solver function has not been called prior',
                    'to the call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dr_cls.refresh_py(_dr)
        _di_cls.refresh_py(_di)
        _z_cls.refresh_py(_z)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nconv'],
            )(
                _nconv_py,
            )
    _err_class.manage()
    return return_data

def real_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``real_iter``/``real_proc``/``real_band_solve``).

    Note: this function uses optional algorithmic parameters, see also:
    ``real_iter``, ``real_proc``, ``real_init``.

    For full information please refer to the NAG Library document for
    f12ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12adf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Description] and
        [Optional Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Arnoldi iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are supplied in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Supplied Shifts' : valueless
        During the Arnoldi iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are supplied in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Arnoldi iterations that can be
        performed before exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Arnoldi iterations has reached
        this limit then exits with an error; can still be called
        subsequently to return the number of converged eigenvalues, the
        converged eigenvalues and, if requested, the corresponding
        eigenvectors.

    'Largest Magnitude' : valueless
        Default

        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = NO

        During the iterative process and reverse communication calls to,
        required data can be communicated to and from in one of two
        ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to.
        For example, when irevcm = 1, returns the vector x in x and the
        matrix-vector product Bx in mx and expects the result or the
        linear operation OP(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows to perform fewer copy operations on each
        intermediate exit and entry, but can also lead to less elegant
        code in the calling program.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by ``real_option``
        as follows.

        [table omitted]

    'Random Residual' : valueless
        Default

        To begin the Arnoldi iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Initial Residual' : valueless
        To begin the Arnoldi iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse Imaginary' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse Real' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = RITZ

        The function can optionally compute the Schur vectors and/or the
        eigenvectors corresponding to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of, if z
        is set equal to v (as in [Example]) then the Schur vectors in v
        are overwritten by the eigenvectors computed by.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

        (`errno` 4)
            Either the initialization function has not been called prior
            to the call of this function or a communication array has
            become corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_option'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _icomm = comm['icomm']
    _comm = comm['comm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _optstr,
        _icomm,
        _comm,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Ambiguous keyword: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Second keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 4:
            _ierr_fl = 4
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def real_monit(ritzr, ritzi, rzest, comm):
    u"""
    Provides monitoring information for ``real_iter``.

    For full information please refer to the NAG Library document for
    f12ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aef.html

    Parameters
    ----------
    ritzr : float, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `ritzr`
        contain the real parts of the converged approximate eigenvalues.

    ritzi : float, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `ritzi`
        contain the imaginary parts of the converged approximate
        eigenvalues.

    rzest : float, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `rzest`
        contain the Ritz estimates (error bounds) on the converged
        approximate eigenvalues.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_monit'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, niter.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, ritzr[*].
        utils._EngineFloat64ArrayType.c_type, # Output, ritzi[*].
        utils._EngineFloat64ArrayType.c_type, # Output, rzest[*].
        utils._EngineIntArrayType.c_type, # Input, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input, comm[*].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _niter = utils._EngineIntScalarType.empty_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aaf_ncv_py = 0
    _ritzr_cls = utils._EngineFloat64ArrayType(
        ritzr, {'fun_name': fun_name, 'entity': 'ritzr'},
        exp_shape=(_vl_f12aaf_ncv_py,),
        check_contents=False,
    )
    _ritzr = _ritzr_cls.to_ctypes(can_reference_contents=False)
    _ritzi_cls = utils._EngineFloat64ArrayType(
        ritzi, {'fun_name': fun_name, 'entity': 'ritzi'},
        exp_shape=(_vl_f12aaf_ncv_py,),
        check_contents=False,
    )
    _ritzi = _ritzi_cls.to_ctypes(can_reference_contents=False)
    _rzest_cls = utils._EngineFloat64ArrayType(
        rzest, {'fun_name': fun_name, 'entity': 'rzest'},
        exp_shape=(_vl_f12aaf_ncv_py,),
        check_contents=False,
    )
    _rzest = _rzest_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12aaf_lcomm_py = 0
    _comm = comm['comm']
    engine_ctypes(
        _en_data,
        _niter,
        _nconv,
        _ritzr,
        _ritzi,
        _rzest,
        _icomm,
        _comm,
    )
    utils._handle_cb_exception(excs)
    _ritzr_cls.refresh_py(_ritzr)
    _ritzi_cls.refresh_py(_ritzi)
    _rzest_cls.refresh_py(_rzest)
    _niter_py = utils._EngineIntScalarType.to_py(_niter)
    _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv'],
    )(
        _niter_py,
        _nconv_py,
    )
    return return_data

def real_band_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_band_solve``) computing selected
    eigenvalues and, optionally, eigenvectors of a real nonsymmetric
    banded (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aff.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_band_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _icomm = utils._EngineIntArrayType.empty_ctypes(140)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _lcomm = utils._EngineIntScalarType.c_type(60)
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(60)
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_band_solve(kl, ku, ab, mb, sigmar, sigmai, dr, di, z, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric banded eigenproblem, driver.

    For full information please refer to the NAG Library document for
    f12ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12agf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B.

    ab : float, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    mb : float, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    sigmar : float
        If one of the 'Shifted Inverse Real' modes (see ``real_option``)
        have been selected then `sigmar` must contain the real part of
        the shift used; otherwise `sigmar` is not referenced. [the F12
        Introduction] describes the use of shift and inverse
        transformations.

    sigmai : float
        If one of the 'Shifted Inverse Real' modes (see ``real_option``)
        have been selected then `sigmai` must contain the imaginary part
        of the shift used; otherwise `sigmai` is not referenced. [the
        F12 Introduction] describes the use of shift and inverse
        transformations.

    dr : float, shape (nev+1), modified in place
        On exit: the first `nconv` locations of the array `dr` contain
        the real parts of the converged approximate eigenvalues. The
        number of eigenvalues returned may be one more than the number
        requested by nev since complex values occur as conjugate pairs
        and the second in the pair can be returned in position nev+1 of
        the array.

    di : float, shape (nev+1), modified in place
        On exit: the first `nconv` locations of the array `di` contain
        the imaginary parts of the converged approximate eigenvalues.
        The number of eigenvalues returned may be one more than the
        number requested by nev since complex values occur as conjugate
        pairs and the second in the pair can be returned in position
        nev+1 of the array.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: nev; otherwise:
        1.

        On exit: if the default option 'Vectors' = Ritz has been
        selected then `z` contains the final set of eigenvectors
        corresponding to the eigenvalues held in `dr` and `di`. The
        complex eigenvector associated with the eigenvalue with positive
        imaginary part is stored in two consecutive columns. The first
        column holds the real part of the eigenvector and the second
        column holds the imaginary part. The eigenvector associated with
        the eigenvalue with negative imaginary part is simply the
        complex conjugate of the eigenvector associated with the
        positive imaginary part.

    resid : float, shape (n), modified in place
        On entry: need not be set unless the option 'Initial Residual'
        has been set in a prior call to ``real_option`` in which case
        `resid` must contain an initial residual vector.

        On exit: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        On exit: if the option 'Vectors' (see ``real_option``) has been
        set to Schur or Ritz then the first `nconv`*n elements of `v`
        will contain approximate Schur vectors that span the desired
        invariant subspace.

        The ith Schur vector is stored in the ith column of `v`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 4)
            The option 'Shifted Inverse Imaginary' has been selected and
            `sigmai` = zero on entry; `sigmai` must be nonzero for this
            mode of operation.

        (`errno` 5)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 6)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 10)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 11)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 12)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 13)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 14)
            During calculation of a real Schur form, there was a failure
            to compute a number of eigenvalues. Please contact NAG.

        (`errno` 15)
            The computed Schur form could not be reordered by an
            internal call. Please contact NAG.

        (`errno` 16)
            Error in internal call to compute eigenvectors. Please
            contact NAG.

        (`errno` 17)
            Failure during internal factorization of real banded matrix.
            Please contact NAG.

        (`errno` 18)
            Failure during internal solution of real banded matrix.
            Please contact NAG.

        (`errno` 19)
            Failure during internal factorization of complex banded
            matrix. Please contact NAG.

        (`errno` 20)
            Failure during internal solution of complex banded matrix.
            Please contact NAG.

        (`errno` 21)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*.

        (`errno` 22)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 23)
            Overflow occurred during transformation of Ritz values to
            those of the original problem.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_band_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, *].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, mb[ldmb, *].
        utils._BYREF_INT, # Input, ldmb.
        utils._BYREF_FLOAT64, # Input, sigmar.
        utils._BYREF_FLOAT64, # Input, sigmai.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, dr[*].
        utils._EngineFloat64ArrayType.c_type, # Output, di[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[140].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (mb, 'mb'), (z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aff_n_py = 0
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12aff_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldmb_py = utils._EngineFloat64ArrayType.get_nd(mb, {'fun_name': fun_name, 'entity': 'mb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldmb = utils._EngineIntScalarType(
        _ldmb_py, {'fun_name': fun_name, 'entity': '_ldmb'},
    ).to_ctypes()
    _mb = utils._EngineFloat64ArrayType(
        mb, {'fun_name': fun_name, 'entity': 'mb'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12aff_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _sigmar = utils._EngineFloat64ScalarType(
        sigmar, {'fun_name': fun_name, 'entity': 'sigmar'},
    ).to_ctypes()
    _sigmai = utils._EngineFloat64ScalarType(
        sigmai, {'fun_name': fun_name, 'entity': 'sigmai'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1,), validate=False)
    else:
        _vl_f12aff_nev_py = 0
    _dr_cls = utils._EngineFloat64ArrayType(
        dr, {'fun_name': fun_name, 'entity': 'dr'},
        exp_shape=(_vl_f12aff_nev_py+1,),
        check_contents=False,
    )
    _dr = _dr_cls.to_ctypes(can_reference_contents=False)
    _di_cls = utils._EngineFloat64ArrayType(
        di, {'fun_name': fun_name, 'entity': 'di'},
        exp_shape=(_vl_f12aff_nev_py+1,),
        check_contents=False,
    )
    _di = _di_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
        _z_dim2 = _vl_f12aff_nev_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12aff_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12aff_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aff_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12aff_n_py, _vl_f12aff_ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _routine_name = utils._EngineRoutineName(*b'f12ag ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _kl,
        _ku,
        _ab,
        _ldab,
        _mb,
        _ldmb,
        _sigmar,
        _sigmai,
        _nconv,
        _dr,
        _di,
        _z,
        _ldz,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '2*kl+ku+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Shifted Inverse Imaginary\' has been',
                    'selected and sigmai = zero on entry; sigmai must be nonzero',
                    'for this mode of operation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a real Schur form, there was a failure',
                    'to compute a number of eigenvalues.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 16:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvectors.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 19 and _ifmt == 19:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 20 and _ifmt == 20:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 22:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Lanczos iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 23 and _ifmt == 23:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred during transformation of Ritz values',
                    'to those of the original problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _dr_cls.refresh_py(_dr)
        _di_cls.refresh_py(_di)
        _z_cls.refresh_py(_z)
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def complex_init(n, nev, ncv):
    u"""
    Initialization routine for (``complex_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a complex sparse
    (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12anf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` >= `nev`+1 and `ncv` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineComplex128ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _icomm = utils._EngineIntArrayType.empty_ctypes(140)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _lcomm = utils._EngineIntScalarType(
        3*_n_py+3*_ncv_py*_ncv_py+5*_ncv_py+60, {'fun_name': fun_name, 'entity': '_lcomm'},
    ).to_ctypes()
    _comm = utils._EngineComplex128ArrayType.empty_ctypes(3*_n_py+3*_ncv_py*_ncv_py+5*_ncv_py+60)
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv >= nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def complex_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a complex
    sparse eigenproblem, reverse communication.

    For full information please refer to the NAG Library document for
    f12ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12apf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must be unchanged from its previous exit
        value. Changing `irevcm` to any other value between calls will
        result in an error.

    resid : complex, shape (n), modified in place
        On initial entry: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``complex_option`` in
        which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        On intermediate entry: must be unchanged from its previous exit.
        Changing `resid` to any other value between calls may result in
        an error exit.

        On intermediate exit: contains the current residual vector.

        On final exit: contains the final residual vector.

    v : complex, shape (n, ncv), modified in place
        On initial entry: need not be set.

        On intermediate entry: must be unchanged from its previous exit.

        On intermediate exit: contains the current set of Arnoldi basis
        vectors.

        On final exit: contains the final set of Arnoldi basis vectors.

    x : complex, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+24)-1]=0:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `x` need not be set.

        If 'Pointers' = NO, `x` must contain the result of y = OPx when
        `irevcm` returns the value -1 or +1.

        It must return the computed shifts when `irevcm` returns the
        value 3.

        On intermediate exit: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        On final exit: does not contain useful data.

    mx : complex, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+24)-1]=0:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        On intermediate exit: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        On final exit: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product y =
          OPx, where x is stored in `x` (by default) or in the array
          `comm`['comm'] (starting from the location given by the first
          element of `comm`['icomm']) when the option 'Pointers' = 'YES'
          is set in a prior call to ``complex_option``. The result y is
          returned in `x` (by default) or in the array `comm`['comm']
          (starting from the location given by the second element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 1

          The calling program must compute the matrix-vector product y =
          OPx. This is similar to the case `irevcm` = -1 except that the
          result of the matrix-vector product Bx (as required in some
          computational modes) has already been computed and is
          available in `mx` (by default) or in the array `comm`['comm']
          (starting from the location given by the third element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 2

          The calling program must compute the matrix-vector product y =
          Bx, where x is stored in `x` and y is returned in `mx` (by
          default) or in the array `comm`['comm'] (starting from the
          location given by the second element of `comm`['icomm']) when
          the option 'Pointers' = 'YES' is set.

        `irevcm` = 3

          Compute the `nshift` complex shifts. This value of this
          argument will only arise if the option 'Supplied Shifts' is
          set in a prior call to ``complex_option`` which is intended
          for experienced users only; the default and recommended option
          is to use exact shifts (see ARPACK Users' Guide for details).

        `irevcm` = 4

          Monitoring step: a call to ``complex_monit`` can now be made
          to return the number of Arnoldi iterations, the number of
          converged Ritz values, the array of converged values, and the
          corresponding Ritz estimates.

        On final exit: `irevcm` = 5: ``complex_iter`` has completed its
        tasks. The value of errno determines whether the iteration has
        been successfully completed, or whether errors have been
        detected. On successful completion ``complex_proc`` must be
        called to return the requested eigenvalues and eigenvectors
        (and/or Schur vectors).

    nshift : int
        On intermediate exit: if the option 'Supplied Shifts' is set and
        `irevcm` returns a value of 3, this argument returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 5)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 6)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 7)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 4)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_iter'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, mx[*].
        utils._BYREF_INT, # Output, nshift.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real),), validate=False)
    else:
        _vl_f12anf_n_py = 0
    _resid_cls = utils._EngineComplex128ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12anf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12anf_n_py, _vl_f12anf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+24)-1,), validate=False) == 0:
        _x_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+1)-1,), validate=False)
    else:
        _x_dim1 = 1
    exp_shape_x = (_x_dim1,)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
    )
    _x = _x_cls.to_ctypes()
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+24)-1,), validate=False) == 0:
        _mx_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+1)-1,), validate=False)
    else:
        _mx_dim1 = 1
    exp_shape_mx = (_mx_dim1,)
    _mx_cls = utils._EngineComplex128ArrayType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
        exp_shape=exp_shape_mx,
    )
    _mx = _mx_cls.to_ctypes()
    _nshift = utils._EngineIntScalarType.empty_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12ap ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _irevcm,
        _resid,
        _v,
        _ldv,
        _x,
        _mx,
        _nshift,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 5 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Arnoldi iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _x_cls.refresh_py(_x)
        _mx_cls.refresh_py(_mx)
        _nshift_py = utils._EngineIntScalarType.to_py(_nshift)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'nshift'],
        )(
            _irevcm_py,
            _nshift_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_proc(d, z, sigma, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a complex
    sparse eigenproblem, postprocessing for ``complex_iter``.

    For full information please refer to the NAG Library document for
    f12aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aqf.html

    Parameters
    ----------
    d : complex, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : complex, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+7)-1]=1:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+2)-1]; otherwise: 1.

        On exit: if the default option 'Vectors' = 'RITZ' (see
        ``real_option``) has been selected then `z` contains the final
        set of eigenvectors corresponding to the eigenvalues held in
        `d`. The complex eigenvector associated with an eigenvalue is
        stored in the corresponding of `z`.

    sigma : complex
        If one of the 'Shifted Inverse' (see ``complex_option``) modes
        has been selected then `sigma` contains the shift used;
        otherwise `sigma` is not referenced.

    resid : complex, shape (n)
        Must not be modified following a call to ``complex_iter`` since
        it contains data required by ``complex_proc``.

    v : complex, shape (n, ncv), modified in place
        On entry: the ncv columns of `v` contain the Arnoldi basis
        vectors for OP as constructed by ``complex_iter``.

        On exit: if the option 'Vectors' = 'SCHUR' or 'RITZ' has been
        set and a separate array `z` has been passed (i.e., `z` does not
        equal `v`), then the first `nconv` columns of `v` will contain
        approximate Schur vectors that span the desired invariant
        subspace.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by
        ``complex_option``.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a Schur form, there was a failure to
            compute *<value>* eigenvalues in a total of *<value>*
            iterations.

        (`errno` 6)
            The computed Schur form could not be reordered by an
            internal call. This function returned with errno =
            *<value>*. Please contact NAG.

        (`errno` 7)
            In calculating eigenvectors, an internal call returned with
            an error. The function returned with errno = *<value>*.
            Please contact NAG.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_proc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_COMPLEX128, # Input, sigma.
        utils._EngineComplex128ArrayType.c_type, # Input, resid[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12anf_ncv_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real),), validate=False)
    else:
        _vl_f12anf_n_py = 0
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
        _z_dim2 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2)-1,), validate=False)
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12anf_n_py, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _sigma = utils._EngineComplex128ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _resid = utils._EngineComplex128ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12anf_n_py,),
    ).to_ctypes()
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12anf_n_py, _vl_f12anf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12aq ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _nconv,
        _d,
        _z,
        _ldz,
        _sigma,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy,',
                    'as communicated through the argument comm[\'icomm\'], is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Got a different count of the number of converged Ritz values',
                    'than the value passed to it through the argument comm[\'icomm\']:',
                    'number counted = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a Schur form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'This function returned with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In calculating eigenvectors, an internal call returned with an error.',
                    'The function returned with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def complex_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``complex_iter``/``complex_proc``).

    Note: this function uses optional algorithmic parameters, see also:
    ``complex_iter``, ``complex_proc``, ``complex_band_solve``,
    ``complex_band_init``, ``complex_init``.

    For full information please refer to the NAG Library document for
    f12ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12arf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Description] and
        [Optional Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Arnoldi iterative process, shifts are applied as part
        of the implicit restarting scheme.
        The shift strategy used by default and selected by the option
        'Exact Shifts' is strongly recommended over the alternative
        'Supplied Shifts' and will always be used by
        ``complex_band_solve``.

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.
        This strategy is generally effective and cheaper to apply in
        terms of number of operations than using explicit shifts.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the complex shifts are returned in x
        (or in comm when the option 'Pointers' = 'YES' is set). This
        option should only be used if you are an experienced user since
        this requires some algorithmic knowledge and because more
        operations are usually required than for the implicit shift
        scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Supplied Shifts' : valueless
        During the Arnoldi iterative process, shifts are applied as part
        of the implicit restarting scheme.
        The shift strategy used by default and selected by the option
        'Exact Shifts' is strongly recommended over the alternative
        'Supplied Shifts' and will always be used by
        ``complex_band_solve``.

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.
        This strategy is generally effective and cheaper to apply in
        terms of number of operations than using explicit shifts.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the complex shifts are returned in x
        (or in comm when the option 'Pointers' = 'YES' is set). This
        option should only be used if you are an experienced user since
        this requires some algorithmic knowledge and because more
        operations are usually required than for the implicit shift
        scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Arnoldi iterations that can be
        performed before ``complex_iter`` or ``complex_band_solve``
        exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Arnoldi iterations has reached
        this limit then ``complex_iter`` or ``complex_band_solve`` exits
        with an error; ``complex_band_solve`` returns the number of
        converged eigenvalues, the converged eigenvalues and, if
        requested, the corresponding eigenvectors, while
        ``complex_proc`` can be called subsequent to ``complex_iter`` to
        do the same.

    'Largest Magnitude' : valueless
        Default

        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = 'NO'

        During the iterative process and reverse communication calls to
        ``complex_iter``, required data can be communicated to and from
        ``complex_iter`` in one of two ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to ``complex_iter``.
        For example, when irevcm = 1, ``complex_iter`` returns the
        vector x in x and the matrix-vector product Bx in mx and expects
        the result or the linear operation OP(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows ``complex_iter`` to perform fewer copy
        operations on each intermediate exit and entry, but can also
        lead to less elegant code in the calling program.

        This option has no affect on ``complex_band_solve`` which sets
        'Pointers' = 'YES' internally.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``complex_option`` as follows.

        [table omitted]

    'Random Residual' : valueless
        Default

        To begin the Arnoldi iterative process, ``complex_iter`` and
        ``complex_band_solve`` requires an initial residual vector.
        By default ``complex_iter`` and ``complex_band_solve`` provides
        its own random initial residual vector; this option can also be
        set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``complex_iter`` and
        ``complex_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Initial Residual' : valueless
        To begin the Arnoldi iterative process, ``complex_iter`` and
        ``complex_band_solve`` requires an initial residual vector.
        By default ``complex_iter`` and ``complex_band_solve`` provides
        its own random initial residual vector; this option can also be
        set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``complex_iter`` and
        ``complex_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = 'RITZ'

        The function ``complex_proc`` or ``complex_band_solve`` can
        optionally compute the Schur vectors and/or the eigenvectors
        corresponding to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of ``complex_proc`` or
        ``complex_band_solve``.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of
        ``complex_proc`` or ``complex_band_solve``, if z is set equal to
        v (as in [Example]) then the Schur vectors in v are overwritten
        by the eigenvectors computed by ``complex_proc`` or
        ``complex_band_solve``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

        (`errno` 4)
            Either the initialization function has not been called prior
            to the call of this function or a communication array has
            become corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_option'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    _comm = comm['comm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _optstr,
        _icomm,
        _comm,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Ambiguous keyword: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Second keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 4:
            _ierr_fl = 4
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def complex_monit(ritz, rzest, comm):
    u"""
    Provides monitoring information for ``complex_iter``.

    For full information please refer to the NAG Library document for
    f12as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12asf.html

    Parameters
    ----------
    ritz : complex, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `ritz` contain
        the converged approximate eigenvalues.

    rzest : complex, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `rzest`
        contain the complex Ritz estimates on the converged approximate
        eigenvalues.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_monit'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, niter.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Output, ritz[*].
        utils._EngineComplex128ArrayType.c_type, # Output, rzest[*].
        utils._EngineIntArrayType.c_type, # Input, icomm[*].
        utils._EngineComplex128ArrayType.c_type, # Input, comm[*].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _niter = utils._EngineIntScalarType.empty_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _ritz_cls = utils._EngineComplex128ArrayType(
        ritz, {'fun_name': fun_name, 'entity': 'ritz'},
        exp_shape=(_vl_f12anf_ncv_py,),
        check_contents=False,
    )
    _ritz = _ritz_cls.to_ctypes(can_reference_contents=False)
    _rzest_cls = utils._EngineComplex128ArrayType(
        rzest, {'fun_name': fun_name, 'entity': 'rzest'},
        exp_shape=(_vl_f12anf_ncv_py,),
        check_contents=False,
    )
    _rzest = _rzest_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    _comm = comm['comm']
    engine_ctypes(
        _en_data,
        _niter,
        _nconv,
        _ritz,
        _rzest,
        _icomm,
        _comm,
    )
    utils._handle_cb_exception(excs)
    _ritz_cls.refresh_py(_ritz)
    _rzest_cls.refresh_py(_rzest)
    _niter_py = utils._EngineIntScalarType.to_py(_niter)
    _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv'],
    )(
        _niter_py,
        _nconv_py,
    )
    return return_data

def complex_band_init(n, nev, ncv):
    u"""
    Initialization function for ``complex_band_solve`` computing
    selected eigenvalues and, optionally, eigenvectors of a complex
    banded (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12atf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_band_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineComplex128ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _icomm = utils._EngineIntArrayType.empty_ctypes(140)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _lcomm = utils._EngineIntScalarType.c_type(60)
    _comm = utils._EngineComplex128ArrayType.empty_ctypes(60)
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def complex_band_solve(kl, ku, ab, mb, sigma, d, z, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of complex
    non-Hermitian banded eigenproblem, driver.

    For full information please refer to the NAG Library document for
    f12au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12auf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B.

    ab : complex, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        non-Hermitian matrices (see [the F07 Introduction]).

    mb : complex, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        non-Hermitian matrices (see [the F07 Introduction]).

    sigma : complex
        If the 'Shifted Inverse' mode (see ``complex_option``) has been
        selected then `sigma` must contain the shift used; otherwise
        `sigma` is not referenced. [the F12 Introduction] describes the
        use of shift and invert transformations.

    d : complex, shape (nev), modified in place
        On exit: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : None or complex, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+7)-1]=1: nev;
        otherwise: 1.

        On exit: if the default option 'Vectors' = 'RITZ' (see
        ``complex_option``) has been selected then `z` contains the
        final set of eigenvectors corresponding to the eigenvalues held
        in `d`, otherwise `z` is not referenced. The complex eigenvector
        associated with an eigenvalue `d`[j] is stored in the
        corresponding array section of `z`, namely `z`[i-1,j-1], for j =
        1,2,...,`nconv`, for i = 1,2,...,n.

    resid : complex, shape (n), modified in place
        On entry: need not be set unless the option 'Initial Residual'
        has been set in a prior call to ``complex_option`` in which case
        `resid` must contain an initial residual vector.

        On exit: contains the final residual vector. This can be used as
        the starting residual to improve convergence on the solution of
        a closely related eigenproblem. This has no relation to the
        error residual Ax-lambda x or Ax-lambda Bx.

    v : complex, shape (n, ncv), modified in place
        On exit: if the option 'Vectors' = 'SCHUR' or 'RITZ' (see
        ``complex_option``) has been set and a separate array `z` has
        been passed (i.e., `z` does not equal `v`), then the first
        `nconv` columns of `v` will contain approximate Schur vectors
        that span the desired invariant subspace.

        The jth Schur vector is stored in the ith column of `v`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 5)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 6)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 10)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 11)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 12)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 13)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 14)
            During calculation of a Schur form, there was a failure to
            compute a number of eigenvalues Please contact NAG.

        (`errno` 15)
            The computed Schur form could not be reordered by an
            internal call. Please contact NAG.

        (`errno` 16)
            Error in internal call to compute eigenvectors. Please
            contact NAG.

        (`errno` 17)
            Failure during internal factorization of real banded matrix.
            Please contact NAG.

        (`errno` 18)
            Failure during internal solution of real banded matrix.
            Please contact NAG.

        (`errno` 19)
            Failure during internal factorization of complex banded
            matrix. Please contact NAG.

        (`errno` 20)
            Failure during internal solution of complex banded matrix.
            Please contact NAG.

        (`errno` 21)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*.

        (`errno` 22)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 23)
            Overflow occurred during transformation of Ritz values to
            those of the original problem.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_band_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, *].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input, mb[ldmb, *].
        utils._BYREF_INT, # Input, ldmb.
        utils._BYREF_COMPLEX128, # Input, sigma.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[140].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (mb, 'mb'), (z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12atf_n_py = 0
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12atf_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldmb_py = utils._EngineComplex128ArrayType.get_nd(mb, {'fun_name': fun_name, 'entity': 'mb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldmb = utils._EngineIntScalarType(
        _ldmb_py, {'fun_name': fun_name, 'entity': '_ldmb'},
    ).to_ctypes()
    _mb = utils._EngineComplex128ArrayType(
        mb, {'fun_name': fun_name, 'entity': 'mb'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12atf_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _sigma = utils._EngineComplex128ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (21,), validate=False)
    else:
        _vl_f12atf_nev_py = 0
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12atf_nev_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
        if z is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when \'Vectors\' = \'RITZ\'',
            )
    if z is None:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12atf_nev_py
        else:
            _z_dim2 = 1
        _z = utils._EngineComplex128ArrayType.empty_ctypes((_vl_f12atf_n_py)*(_z_dim2))
    else:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12atf_nev_py
        else:
            _z_dim2 = 1
        exp_shape_z = (_vl_f12atf_n_py, _z_dim2)
        _z_cls = utils._EngineComplex128ArrayType(
            z, {'fun_name': fun_name, 'entity': 'z'},
            exp_shape=exp_shape_z,
            mutual_sorder=_m_sorder,
            check_contents=False,
            allow_none=True,
        )
        _z = _z_cls.to_ctypes(can_reference_contents=False)
    _resid_cls = utils._EngineComplex128ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12atf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12atf_ncv_py = 0
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12atf_n_py, _vl_f12atf_ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _routine_name = utils._EngineRoutineName(*b'f12au ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _kl,
        _ku,
        _ab,
        _ldab,
        _mb,
        _ldmb,
        _sigma,
        _nconv,
        _d,
        _z,
        _ldz,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '2*kl+ku+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a Schur form, there was a failure',
                    'to compute a number of eigenvalues',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 16:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvectors.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 19 and _ifmt == 19:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 20 and _ifmt == 20:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 22:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Arnoldi iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 23 and _ifmt == 23:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred during transformation of Ritz values',
                    'to those of the original problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        if z is not None:
            _z_cls.refresh_py(_z)
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def real_symm_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_symm_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a real symmetric sparse
    (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12faf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev` and `ncv` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _icomm = utils._EngineIntArrayType.empty_ctypes(140)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _lcomm = utils._EngineIntScalarType(
        3*_n_py+_ncv_py*_ncv_py+8*_ncv_py+60, {'fun_name': fun_name, 'entity': '_lcomm'},
    ).to_ctypes()
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(3*_n_py+_ncv_py*_ncv_py+8*_ncv_py+60)
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_symm_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric sparse eigenproblem, reverse communication.

    For full information please refer to the NAG Library document for
    f12fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fbf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must be unchanged from its previous exit
        value. Changing `irevcm` to any other value between calls will
        result in an error.

    resid : float, shape (n), modified in place
        On initial entry: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``real_symm_option``
        in which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        On intermediate entry: must be unchanged from its previous exit.
        Changing `resid` to any other value between calls may result in
        an error exit.

        On intermediate exit: contains the current residual vector.

        On final exit: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        On initial entry: need not be set.

        On intermediate entry: must be unchanged from its previous exit.

        On intermediate exit: contains the current set of Arnoldi basis
        vectors.

        On final exit: contains the final set of Arnoldi basis vectors.

    x : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1;
        otherwise: 0.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `x` need not be set.

        If 'Pointers' = NO, `x` must contain the result of y = OPx when
        `irevcm` returns the value -1 or +1.

        It must return the real parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        On final exit: does not contain useful data.

    mx : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1;
        otherwise: 0.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        It must return the imaginary parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        On final exit: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product y =
          OPx, where x is stored in `x` (by default) or in the array
          `comm`['comm'] (starting from the location given by the first
          element of `comm`['icomm']) when the option 'Pointers' = 'YES'
          is set in a prior call to ``real_symm_option``. The result y
          is returned in `x` (by default) or in the array `comm`['comm']
          (starting from the location given by the second element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 1

          The calling program must compute the matrix-vector product y =
          OPx. This is similar to the case `irevcm` = -1 except that the
          result of the matrix-vector product Bx (as required in some
          computational modes) has already been computed and is
          available in `mx` (by default) or in the array `comm`['comm']
          (starting from the location given by the third element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 2

          The calling program must compute the matrix-vector product y =
          Bx, where x is stored in `x` and y is returned in `mx` (by
          default) or in the array `comm`['comm'] (starting from the
          location given by the second element of `comm`['icomm']) when
          the option 'Pointers' = 'YES' is set.

        `irevcm` = 3

          Compute the `nshift` real and imaginary parts of the shifts
          where the real parts are to be returned in the first `nshift`
          locations of the array `x` and the imaginary parts are to be
          returned in the first `nshift` locations of the array `mx`.
          Only complex conjugate pairs of shifts may be applied and the
          pairs must be placed in consecutive locations. This value of
          this argument will only arise if the option 'Supplied Shifts'
          is set in a prior call to ``real_symm_option`` which is
          intended for experienced users only; the default and
          recommended option is to use exact shifts (see ARPACK Users'
          Guide for details and guidance on the choice of shift
          strategies).

        `irevcm` = 4

          Monitoring step: a call to ``real_symm_monit`` can now be made
          to return the number of Arnoldi iterations, the number of
          converged Ritz values, their real and imaginary parts, and the
          corresponding Ritz estimates.

        On final exit: `irevcm` = 5: ``real_symm_iter`` has completed
        its tasks. The value of errno determines whether the iteration
        has been successfully completed, or whether errors have been
        detected. On successful completion ``real_symm_proc`` must be
        called to return the requested eigenvalues and eigenvectors
        (and/or Schur vectors).

    nshift : int
        On intermediate exit: if the option 'Supplied Shifts' is set and
        `irevcm` returns a value of 3, this argument returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 3)
            Eigenvalues from both ends of the spectrum were requested,
            but the number of eigenvalues (see nev in
            ``real_symm_init``) requested is one.

        (`errno` 4)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 6)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 7)
            Could not build a Lanczos factorization. The size of the
            current Lanczos factorization = *<value>*.

        (`errno` 8)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 9)
            Either the function was called without an initial call to
            the setup function or the communication arrays have become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_iter'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, mx[*].
        utils._BYREF_INT, # Output, nshift.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12faf_n_py = 0
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12faf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12faf_n_py, _vl_f12faf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12faf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
            _x_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
        else:
            _x_dim1 = 1
    else:
        _x_dim1 = 0
    exp_shape_x = (_x_dim1,)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
    )
    _x = _x_cls.to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
            _mx_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
        else:
            _mx_dim1 = 1
    else:
        _mx_dim1 = 0
    exp_shape_mx = (_mx_dim1,)
    _mx_cls = utils._EngineFloat64ArrayType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
        exp_shape=exp_shape_mx,
    )
    _mx = _mx_cls.to_ctypes()
    _nshift = utils._EngineIntScalarType.empty_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12fb ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _irevcm,
        _resid,
        _v,
        _ldv,
        _x,
        _mx,
        _nshift,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 5 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Eigenvalues from both ends of the spectrum were requested,',
                    'but the number of eigenvalues (see nev in real_symm_init) requested is one.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Lanczos iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build a Lanczos factorization. The size',
                    'of the current Lanczos factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was called without an initial call to the setup',
                    'function or the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _x_cls.refresh_py(_x)
        _mx_cls.refresh_py(_mx)
        _nshift_py = utils._EngineIntScalarType.to_py(_nshift)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'nshift'],
        )(
            _irevcm_py,
            _nshift_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_symm_proc(d, z, sigma, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric sparse eigenproblem, postprocessing for
    ``real_symm_iter``.

    For full information please refer to the NAG Library document for
    f12fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fcf.html

    Parameters
    ----------
    d : float, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: ncv; otherwise:
        1; otherwise: 1.

        On exit: if the default option 'Vectors' = 'RITZ' (see
        ``real_symm_option``) has been selected then `z` contains the
        final set of eigenvectors corresponding to the eigenvalues held
        in `d`. The real eigenvector associated with an eigenvalue is
        stored in the corresponding of `z`.

    sigma : float
        If one of the 'Shifted Inverse' (see ``real_symm_option``) modes
        has been selected then `sigma` contains the real shift used;
        otherwise `sigma` is not referenced.

    resid : float, shape (n)
        Must not be modified following a call to ``real_symm_iter``
        since it contains data required by ``real_symm_proc``.

    v : float, shape (n, ncv), modified in place
        On entry: the ncv columns of `v` contain the Lanczos basis
        vectors for OP as constructed by ``real_symm_iter``.

        On exit: if the option 'Vectors' = 'SCHUR' has been set, or the
        option 'Vectors' = 'RITZ' has been set and a separate array `z`
        has been passed (i.e., `z` does not equal `v`), then the first
        `nconv` columns of `v` will contain approximate Schur vectors
        that span the desired invariant subspace.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by
        ``real_symm_iter``.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, 'Vectors' = 'SELECT', but this is not yet
            implemented.

        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a tridiagonal form, there was a
            failure to compute *<value>* eigenvalues in a total of
            *<value>* iterations.

        (`errno` 8)
            Either the function was called out of sequence (following an
            initial call to the setup function and following completion
            of calls to the reverse communication function) or the
            communication arrays have become corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_proc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, d[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._EngineFloat64ArrayType.c_type, # Input, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12faf_ncv_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12faf_n_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12faf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12faf_ncv_py
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12faf_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _resid = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12faf_n_py,),
    ).to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12faf_n_py, _vl_f12faf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12fc ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _nconv,
        _d,
        _z,
        _ldz,
        _sigma,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = \'SELECT\', but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy,',
                    'as communicated through the argument comm[\'icomm\'], is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Got a different count of the number of converged Ritz values',
                    'than the value passed to it through the argument comm[\'icomm\']:',
                    'number counted = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a tridiagonal form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was called out of sequence (following an initial call',
                    'to the setup function and following completion of calls to the reverse',
                    'communication function) or the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def real_symm_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``real_symm_iter``/``real_symm_proc``/``real_symm_band_solve``).

    Note: this function uses optional algorithmic parameters, see also:
    ``real_symm_iter``, ``real_symm_proc``, ``real_symm_init``.

    For full information please refer to the NAG Library document for
    f12fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fdf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Description] and
        [Optional Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Lanczos iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Lanczos iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are returned in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Supplied Shifts' : valueless
        During the Lanczos iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Lanczos iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are returned in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Lanczos iterations that can be
        performed before ``real_symm_iter`` exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Lanczos iterations has reached
        this limit then ``real_symm_iter`` exits with an error;
        ``real_symm_proc`` can still be called subsequently to return
        the number of converged eigenvalues, the converged eigenvalues
        and, if requested, the corresponding eigenvectors.

    'Largest Magnitude' : valueless
        Default

        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Both Ends' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = NO

        During the iterative process and reverse communication calls to,
        required data can be communicated to and from in one of two
        ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to.
        For example, when irevcm = 1, returns the vector x in x and the
        matrix-vector product Bx in mx and expects the result or the
        linear operation OP(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows to perform fewer copy operations on each
        intermediate exit and entry, but can also lead to less elegant
        code in the calling program.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``real_symm_option`` as follows.

        [table omitted]

        Note that setting 'Print Level' >= 30 can result in very lengthy
        'Monitoring' output.

        Note that setting 'Print Level' >= 30 can result in very lengthy
        'Monitoring' output.

    'Random Residual' : valueless
        Default

        To begin the Lanczos iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Initial Residual' : valueless
        To begin the Lanczos iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Buckling' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Cayley' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = RITZ

        The function can optionally compute the Schur vectors and/or the
        eigenvectors corresponding to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of, if z
        is set equal to v (as in [Example]) then the Schur vectors in v
        are overwritten by the eigenvectors computed by.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_option'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _icomm = comm['icomm']
    _comm = comm['comm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _optstr,
        _icomm,
        _comm,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Ambiguous keyword: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Second keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def real_symm_monit(ritz, rzest, comm):
    u"""
    Provides monitoring information for ``real_symm_iter``.

    For full information please refer to the NAG Library document for
    f12fe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fef.html

    Parameters
    ----------
    ritz : float, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `ritz` contain
        the real converged approximate eigenvalues.

    rzest : float, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `rzest`
        contain the Ritz estimates (error bounds) on the real `nconv`
        converged approximate eigenvalues.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_monit'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12feft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, niter.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, ritz[*].
        utils._EngineFloat64ArrayType.c_type, # Output, rzest[*].
        utils._EngineIntArrayType.c_type, # Input, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input, comm[*].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _niter = utils._EngineIntScalarType.empty_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _ritz_cls = utils._EngineFloat64ArrayType(
        ritz, {'fun_name': fun_name, 'entity': 'ritz'},
        exp_shape=(_vl_f12faf_ncv_py,),
        check_contents=False,
    )
    _ritz = _ritz_cls.to_ctypes(can_reference_contents=False)
    _rzest_cls = utils._EngineFloat64ArrayType(
        rzest, {'fun_name': fun_name, 'entity': 'rzest'},
        exp_shape=(_vl_f12faf_ncv_py,),
        check_contents=False,
    )
    _rzest = _rzest_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12faf_lcomm_py = 0
    _comm = comm['comm']
    engine_ctypes(
        _en_data,
        _niter,
        _nconv,
        _ritz,
        _rzest,
        _icomm,
        _comm,
    )
    utils._handle_cb_exception(excs)
    _ritz_cls.refresh_py(_ritz)
    _rzest_cls.refresh_py(_rzest)
    _niter_py = utils._EngineIntScalarType.to_py(_niter)
    _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv'],
    )(
        _niter_py,
        _nconv_py,
    )
    return return_data

def real_symm_band_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_symm_band_solve``) computing
    selected eigenvalues and, optionally, eigenvectors of a real
    symmetric banded (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fff.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_band_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12ffft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _icomm = utils._EngineIntArrayType.empty_ctypes(140)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _lcomm = utils._EngineIntScalarType.c_type(60)
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(60)
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_symm_band_solve(kl, ku, ab, mb, sigma, d, z, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric banded eigenproblem, driver.

    For full information please refer to the NAG Library document for
    f12fg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fgf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B. Since A
        and B are symmetric, the normal case is `ku` = `kl`.

    ab : float, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    mb : float, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    sigma : float
        If one of the 'Shifted Inverse' (see ``real_symm_option``) modes
        has been selected then `sigma` contains the real shift used;
        otherwise `sigma` is not referenced.

    d : float, shape (ncv), modified in place
        On exit: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: nev+1;
        otherwise: 1; otherwise: 1.

        On exit: if the default option 'Vectors' = 'RITZ' (see
        ``real_symm_option``) has been selected then `z` contains the
        final set of eigenvectors corresponding to the eigenvalues held
        in `d`. The real eigenvector associated with eigenvalue i-1, for
        i = 1,2,...,`nconv`, is stored in the ith column of `z`.

    resid : float, shape (n), modified in place
        On entry: need not be set unless the option 'Initial Residual'
        has been set in a prior call to ``real_symm_option`` in which
        case `resid` must contain an initial residual vector.

        On exit: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        On exit: if the option 'Vectors' (see ``real_symm_option``) has
        been set to Schur or Ritz and a separate array `z` has been
        passed then the first `nconv`*n elements of `v` will contain
        approximate Schur vectors that span the desired invariant
        subspace.

        The jth Schur vector is stored in the ith column of `v`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_option``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 4)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 5)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 6)
            Eigenvalues from both ends of the spectrum were requested,
            but the number of eigenvalues (nev in
            ``real_symm_band_init``) requested is one.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 9)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 10)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 11)
            Could not build a Lanczos factorization. The size of the
            current Lanczos factorization = *<value>*.

        (`errno` 12)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 13)
            During calculation of a tridiagonal form, there was a
            failure to compute *<value>* eigenvalues in a total of
            *<value>* iterations.

        (`errno` 14)
            Failure during internal factorization of banded matrix.
            Please contact NAG.

        (`errno` 15)
            Failure during internal solution of banded system. Please
            contact NAG.

        (`errno` 17)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 18)
            Either an initial call to the setup function has not been
            made or the communication arrays have become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 16)
            The maximum number of iterations has been reached: there
            have been *<value>* iterations. There are *<value>*
            converged eigenvalues.
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_band_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, *].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, mb[ldmb, *].
        utils._BYREF_INT, # Input, ldmb.
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, d[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (mb, 'mb'), (z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12fff_n_py = 0
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12fff_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldmb_py = utils._EngineFloat64ArrayType.get_nd(mb, {'fun_name': fun_name, 'entity': 'mb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldmb = utils._EngineIntScalarType(
        _ldmb_py, {'fun_name': fun_name, 'entity': '_ldmb'},
    ).to_ctypes()
    _mb = utils._EngineFloat64ArrayType(
        mb, {'fun_name': fun_name, 'entity': 'mb'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12fff_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12fff_ncv_py = 0
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12fff_ncv_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12fff_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12fff_licomm_py = 0
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (21,), validate=False)
    else:
        _vl_f12fff_nev_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12fff_nev_py+1
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12fff_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12fff_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12fff_n_py, _vl_f12fff_ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _routine_name = utils._EngineRoutineName(*b'f12fg ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _kl,
        _ku,
        _ab,
        _ldab,
        _mb,
        _ldmb,
        _sigma,
        _nconv,
        _d,
        _z,
        _ldz,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '2*kl+ku+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Eigenvalues from both ends of the spectrum were requested,',
                    'but the number of eigenvalues (nev in real_symm_band_init) requested is one.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build a Lanczos factorization. The size',
                    'of the current Lanczos factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a tridiagonal form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of banded system.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 16:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached:',
                    'there have been {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'There are {:s} converged eigenvalues.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Lanczos iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either an initial call to the setup function has not been made',
                    'or the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nconv'],
            )(
                _nconv_py,
            )
    _err_class.manage()
    return return_data
