# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `specfun` Chapter.

``specfun`` - Approximations of Special Functions

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the provision of some commonly occurring
physical and mathematical functions.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/sintro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def log_shifted(x):
    u"""
    ln(1+x).

    For full information please refer to the NAG Library document for
    s01ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s01baf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    lnx1 : float
        The value of the shifted logarithmic function, ln(1+x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > -1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.log_shifted'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s01baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, lnx1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _lnx1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _lnx1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > -1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _lnx1_py = utils._EngineFloat64ScalarType.to_py(_lnx1)
        return_data = _lnx1_py
    _err_class.manage()
    return return_data

def exp_complex(z):
    u"""
    Complex exponential, e^z.

    For full information please refer to the NAG Library document for
    s01ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s01eaf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    Returns
    -------
    ez : complex
        The value of the exponential function.

    Raises
    ------
    NagValueError
        (`errno` 5)
            The imaginary part of argument `z` is so large that the
            result has no precision: `z` = (*<value>*,*<value>*).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Argument `z` causes overflow in real part of result: `z` =
            (*<value>*,*<value>*).

        (`errno` 2)
            Argument `z` causes overflow in imaginary part of result:
            `z` = (*<value>*,*<value>*).

        (`errno` 3)
            Argument `z` causes overflow in both real and imaginary
            parts of result: `z` = (*<value>*,*<value>*).

        (`errno` 4)
            The imaginary part of argument `z` is so large that the
            result is accurate to less than half precision: `z` =
            (*<value>*,*<value>*).

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.exp_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s01eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_COMPLEX128, # Output, ez.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _ez = utils._EngineComplex128ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _z,
        _ez,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument z causes overflow in real part of result:',
                    'z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument z causes overflow in imaginary part of result:',
                    'z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Argument z causes overflow in both real and imaginary parts of result:',
                    'z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The imaginary part of argument z is so large that the result is',
                    'accurate to less than half precision:',
                    'z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The imaginary part of argument z is so large that the result has no',
                    'precision: z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ez_py = utils._EngineComplex128ScalarType.to_py(_ez)
        return_data = _ez_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['ez'],
            )(
                _ez_py,
            )
    _err_class.manage()
    return return_data

def tan(x):
    u"""
    tan(x).

    For full information please refer to the NAG Library document for
    s07aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s07aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    tx : float
        The value of the circular tangent, tan(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            The function has been called with an argument that is too
            large; the default result returned is zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The function has been called with an argument that is too
            close to an odd multiple of pi/2, at which the function is
            infinite; the function has returned a value with the correct
            sign but a more or less arbitrary but large magnitude.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.tan'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s07aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, tx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _tx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _tx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function has been called with an argument that is too large;',
                    'the default result returned is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function has been called with an argument that is too close to',
                    'an odd multiple of pi/2, at which the function is infinite;',
                    'the function has returned a value with the correct sign but a more or less',
                    'arbitrary but large magnitude.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _tx_py = utils._EngineFloat64ScalarType.to_py(_tx)
        return_data = _tx_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['tx'],
            )(
                _tx_py,
            )
    _err_class.manage()
    return return_data

def arcsin(x):
    u"""
    arcsin(x).

    For full information please refer to the NAG Library document for
    s09aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s09aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    asx : float
        The value of the inverse circular sine, arcsin(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.arcsin'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s09aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, asx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _asx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _asx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _asx_py = utils._EngineFloat64ScalarType.to_py(_asx)
        return_data = _asx_py
    _err_class.manage()
    return return_data

def arccos(x):
    u"""
    arccos(x).

    For full information please refer to the NAG Library document for
    s09ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s09abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    acx : float
        The value of the inverse circular cosine, arccos(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.arccos'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s09abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, acx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _acx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _acx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _acx_py = utils._EngineFloat64ScalarType.to_py(_acx)
        return_data = _acx_py
    _err_class.manage()
    return return_data

def tanh(x):
    u"""
    tanh(x).

    For full information please refer to the NAG Library document for
    s10aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s10aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    thx : float
        The value of the hyperbolic tangent, tanh(x).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.tanh'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s10aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, thx.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _thx = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _thx,
    )
    utils._handle_cb_exception(excs)
    _thx_py = utils._EngineFloat64ScalarType.to_py(_thx)
    return_data = _thx_py
    return return_data

def sinh(x):
    u"""
    sinh(x).

    For full information please refer to the NAG Library document for
    s10ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s10abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    shx : float
        The value of the hyperbolic sine, sinh(x).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= E_1.

            The function has been called with an argument too large in
            absolute magnitude. There is a danger of overflow. The
            result returned is the value of sinh(x) at the closest
            argument for which a valid call could be made.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.sinh'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s10abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, shx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _shx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _shx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= E_1.',
                    'The function has been called with an argument too large in absolute',
                    'magnitude. There is a danger of overflow. The result returned is the value',
                    'of sinh(x) at the closest argument for which a valid call could be made.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _shx_py = utils._EngineFloat64ScalarType.to_py(_shx)
        return_data = _shx_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['shx'],
            )(
                _shx_py,
            )
    _err_class.manage()
    return return_data

def cosh(x):
    u"""
    cosh(x).

    For full information please refer to the NAG Library document for
    s10ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s10acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    chx : float
        The value of the hyperbolic cosine, cosh(x).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= E_1.

            The function has been called with an argument too large in
            absolute magnitude. There is a danger of overflow. The
            result returned is the value of cosh(x) at the nearest valid
            argument.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.cosh'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s10acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, chx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _chx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _chx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= E_1.',
                    'The function has been called with an argument too large in absolute',
                    'magnitude. There is a danger of overflow. The result returned is the value',
                    'of cosh(x) at the nearest valid argument.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _chx_py = utils._EngineFloat64ScalarType.to_py(_chx)
        return_data = _chx_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['chx'],
            )(
                _chx_py,
            )
    _err_class.manage()
    return return_data

def arctanh(x):
    u"""
    arctanh(x).

    For full information please refer to the NAG Library document for
    s11aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s11aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    athx : float
        The value of the inverse hyperbolic tangent, arctanh(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) < 1.

            The function has been called with an argument greater than
            or equal to 1.0 in magnitude, for which arctanh is not
            defined.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.arctanh'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s11aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, athx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _athx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _athx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) < 1.',
                    'The function has been called with an argument greater than or equal to',
                    '1.0 in magnitude, for which arctanh is not defined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _athx_py = utils._EngineFloat64ScalarType.to_py(_athx)
        return_data = _athx_py
    _err_class.manage()
    return return_data

def arcsinh(x):
    u"""
    arcsinh(x).

    For full information please refer to the NAG Library document for
    s11ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s11abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ashx : float
        The value of the inverse hyperbolic sine, arcsinh(x).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.arcsinh'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s11abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, ashx.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _ashx = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _ashx,
    )
    utils._handle_cb_exception(excs)
    _ashx_py = utils._EngineFloat64ScalarType.to_py(_ashx)
    return_data = _ashx_py
    return return_data

def arccosh(x):
    u"""
    arccosh(x).

    For full information please refer to the NAG Library document for
    s11ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s11acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    achx : float
        The value of the inverse hyperbolic cosine, arccosh(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 1.0.

            The function has been called with an argument less than 1.0,
            for which arccosh(x) is not defined.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.arccosh'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s11acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, achx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _achx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _achx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 1.0.',
                    'The function has been called with an argument less than 1.0, for which',
                    'arccosh(x) is not defined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _achx_py = utils._EngineFloat64ScalarType.to_py(_achx)
        return_data = _achx_py
    _err_class.manage()
    return return_data

def integral_exp(x):
    u"""
    Exponential integral E_1(x).

    For full information please refer to the NAG Library document for
    s13aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s13aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    e1x : float
        The value of the exponential integral E_1(x).

    Raises
    ------
    NagValueError
        (`errno` 2)
            The evaluation has been abandoned due to the likelihood of
            overflow. The argument `x` < -x_hi.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = 0.0 and the function is infinite.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.integral_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s13aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, e1x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _e1x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _e1x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = 0.0 and the function is infinite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The evaluation has been abandoned due to the likelihood of overflow.',
                    'The argument x < -x_hi.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _e1x_py = utils._EngineFloat64ScalarType.to_py(_e1x)
        return_data = _e1x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['e1x'],
            )(
                _e1x_py,
            )
    _err_class.manage()
    return return_data

def integral_cos(x):
    u"""
    Cosine integral Ci(x).

    For full information please refer to the NAG Library document for
    s13ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s13acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    cix : float
        The value of the cosine integral

            Ci(x) = gamma+ln(x)+int_0^x(cos(u)-1)/udu, x > 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            The function has been called with an argument less than or
            equal to zero for which Ci(x) is not defined.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.integral_cos'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s13acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, cix.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _cix = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _cix,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                    'The function has been called with an argument less than or equal to zero',
                    'for which Ci(x) is not defined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _cix_py = utils._EngineFloat64ScalarType.to_py(_cix)
        return_data = _cix_py
    _err_class.manage()
    return return_data

def integral_sin(x):
    u"""
    Sine integral Si(x).

    For full information please refer to the NAG Library document for
    s13ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s13adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    six : float
        The value of the sine integral

            Si(x) = int_0^xsin(u)/udu.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.integral_sin'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s13adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, six.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _six = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _six,
    )
    utils._handle_cb_exception(excs)
    _six_py = utils._EngineFloat64ScalarType.to_py(_six)
    return_data = _six_py
    return return_data

def gamma(x):
    u"""
    Gamma function.

    For full information please refer to the NAG Library document for
    s14aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    gx : float
        The value of the gamma function Gamma(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= *<value>*.

            The argument is too large, the function returns the
            approximate value of Gamma(x) at the nearest valid argument.

        (`errno` 2)
            On entry, `x` = *<value>*. The function returns zero.

            Constraint: `x` >= *<value>*.

            The argument is too large and negative, the function returns
            zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) >= *<value>*.

            The argument is too close to zero, the function returns the
            approximate value of Gamma(x) at the nearest valid argument.

        (`errno` 4)
            On entry, `x` = *<value>*.

            Constraint: `x` must not be a negative integer.

            The argument is a negative integer, at which values Gamma(x)
            is infinite. The function returns a large positive value.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, gx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _gx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _gx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'The argument is too large, the function returns the approximate value of',
                    'Gamma(x) at the nearest valid argument.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The function returns zero.',
                    'Constraint: x >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'The argument is too large and negative, the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'The argument is too close to zero, the function returns the approximate',
                    'value of Gamma(x) at the nearest valid argument.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x must not be a negative integer.',
                    'The argument is a negative integer, at which values Gamma(x) is infinite.',
                    'The function returns a large positive value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _gx_py = utils._EngineFloat64ScalarType.to_py(_gx)
        return_data = _gx_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['gx'],
            )(
                _gx_py,
            )
    _err_class.manage()
    return return_data

def gamma_log_real(x):
    u"""
    Log gamma function, real argument.

    For full information please refer to the NAG Library document for
    s14ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    lgx : float
        The value of ln(Gamma)(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` <= x_big.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma_log_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, lgx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _lgx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _lgx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1021:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x <= x_big.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _lgx_py = utils._EngineFloat64ScalarType.to_py(_lgx)
        return_data = _lgx_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['lgx'],
            )(
                _lgx_py,
            )
    _err_class.manage()
    return return_data

def polygamma(x):
    u"""
    psi(x)-ln(x).

    For full information please refer to the NAG Library document for
    s14ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    pgx : float
        The value of the function psi(x)-ln(x), where psi is the psi
        function psi(x) = d/dxln(Gamma)(x) = Gamma'(x)/Gamma(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

        (`errno` 2)
            Computation halted due to likelihood of underflow. `x` may
            be too large. `x` = *<value>*.

        (`errno` 3)
            Computation halted due to likelihood of overflow. `x` may be
            too small. `x` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.polygamma'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, pgx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _pgx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _pgx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation halted due to likelihood of underflow. x may be',
                    'too large. x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation halted due to likelihood of overflow. x may be',
                    'too small. x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pgx_py = utils._EngineFloat64ScalarType.to_py(_pgx)
        return_data = _pgx_py
    _err_class.manage()
    return return_data

def polygamma_deriv(x, n, m, ans):
    u"""
    Scaled derivatives of psi(x).

    For full information please refer to the NAG Library document for
    s14ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    n : int
        The index of the first member n of the sequence of functions.

    m : int
        The number of members m required in the sequence w(k,x), for k =
        n,...,n+m-1.

    ans : float, shape (`m`), modified in place
        On exit: the first m elements of `ans` contain the required
        values w(k,x), for k = n,...,n+m-1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 4)
            Computation abandoned due to the likelihood of underflow.

        (`errno` 5)
            Computation abandoned due to the likelihood of overflow.

        (`errno` 6)
            There is not enough internal workspace to continue
            computation. `m` is probably too large.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.polygamma_deriv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Output, ans[m].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _ans_cls = utils._EngineFloat64ArrayType(
        ans, {'fun_name': fun_name, 'entity': 'ans'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _ans = _ans_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _n,
        _m,
        _ans,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned due to the likelihood of underflow.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned due to the likelihood of overflow.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There is not enough internal workspace to continue computation.',
                    'm is probably too large.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ans_cls.refresh_py(_ans)
        return_data = None
    _err_class.manage()
    return return_data

def psi_deriv_real(x, k):
    u"""
    Polygamma function psi^(n)(x) for real x.

    For full information please refer to the NAG Library document for
    s14ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    k : int
        The function psi^(k)(x) to be evaluated.

    Returns
    -------
    pkx : float
        The value of the kth derivative of the psi function psi(x) for
        real x and k = 0,1,...,6.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` is 'too close' to a non-positive integer: `x`
            = *<value>* and nint(`x`) = *<value>*.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` <= 6.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            Evaluation abandoned due to likelihood of underflow.

        (`errno` 3)
            Evaluation abandoned due to likelihood of overflow.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.psi_deriv_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Output, pkx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _pkx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _k,
        _pkx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x is \'too close\' to a non-positive integer:',
                    'x = {:s} and nint(x) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k <= 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Evaluation abandoned due to likelihood of underflow.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Evaluation abandoned due to likelihood of overflow.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pkx_py = utils._EngineFloat64ScalarType.to_py(_pkx)
        return_data = _pkx_py
    _err_class.manage()
    return return_data

def psi_deriv_complex(z, k):
    u"""
    Polygamma function psi^(n)(z) for complex z.

    For full information please refer to the NAG Library document for
    s14af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14aff.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    k : int
        The function psi^(k)(z) to be evaluated.

    Returns
    -------
    pkz : complex
        The value of the kth derivative of the psi function.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, Re(`z`) is 'too close' to a non-positive integer
            when Im(`z`) = 0.0: Re(`z`) = *<value>*, nint(Re(`z`)) =
            *<value>*.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` <= 4.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            Evaluation abandoned due to likelihood of overflow.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.psi_deriv_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_COMPLEX128, # Output, pkz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _pkz = utils._EngineComplex128ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _z,
        _k,
        _pkz,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, Re(z) is \'too close\' to a non-positive integer when',
                    'Im(z) = 0.0: Re(z) = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'nint(Re(z)) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Evaluation abandoned due to likelihood of overflow.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pkz_py = utils._EngineComplex128ScalarType.to_py(_pkz)
        return_data = _pkz_py
    _err_class.manage()
    return return_data

def gamma_log_complex(z):
    u"""
    Logarithm of the gamma function ln(Gamma)(z), complex argument.

    For full information please refer to the NAG Library document for
    s14ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14agf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    Returns
    -------
    lngz : complex
        The value of ln(Gamma)(z).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, Re(`z`) is 'too close' to a non-positive integer
            when Im(`z`) = 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma_log_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_COMPLEX128, # Output, lngz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _lngz = utils._EngineComplex128ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _z,
        _lngz,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, Re(z) is \'too close\' to a non-positive integer when Im(z) = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _lngz_py = utils._EngineComplex128ScalarType.to_py(_lngz)
        return_data = _lngz_py
    _err_class.manage()
    return return_data

def gamma_log_scaled_real(x):
    u"""
    Scaled log gamma function.

    For full information please refer to the NAG Library document for
    s14ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14ahf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    slgx : float
        The value of ln(G(x)), the scaled logarithm of the gamma
        function Gamma(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` <= 0.0. On failure, the function value
            returned is zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma_log_scaled_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14ahft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, slgx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _slgx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _slgx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x <= 0.0. On failure, the function value returned is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _slgx_py = utils._EngineFloat64ScalarType.to_py(_slgx)
        return_data = _slgx_py
    _err_class.manage()
    return return_data

def gamma_vector(n, x, f, ivalid):
    u"""
    Gamma function, vectorized Gamma(x).

    For full information please refer to the NAG Library document for
    s14an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14anf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Gamma(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains the
          approximate value of Gamma(x_i) at the nearest valid argument.
          The threshold value is the same as for `errno` = 1 in
          ``gamma``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in ``gamma``,
          as defined in the Users' Note for your implementation.

        `ivalid`[i-1] = 3

          x_i is too close to zero. `f`[i-1] contains the approximate
          value of Gamma(x_i) at the nearest valid argument. The
          threshold value is the same as for `errno` = 2 in ``gamma``,
          as defined in the Users' Note for your implementation.

        `ivalid`[i-1] = 4

          x_i is a negative integer, at which values Gamma(x_i) are
          infinite. `f`[i-1] contains a large positive value.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def gamma_log_real_vector(n, x, f, ivalid):
    u"""
    Log gamma function, vectorized ln(Gamma(x)).

    For full information please refer to the NAG Library document for
    s14ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14apf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: ln(Gamma)(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i <= 0.

        `ivalid`[i-1] = 2

          x_i is too large and positive. The threshold value is the same
          as for `errno` = 2 in ``gamma_log_real``, as defined in the
          Users' Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma_log_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def gamma_incomplete(a, x, tol):
    u"""
    Incomplete gamma functions P(a,x) and Q(a,x).

    For full information please refer to the NAG Library document for
    s14ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14baf.html

    Parameters
    ----------
    a : float
        The argument a of the functions.

    x : float
        The argument x of the functions.

    tol : float
        The relative accuracy required by you in the results. If
        ``gamma_incomplete`` is entered with `tol` greater than 1.0 or
        less than machine precision, then the value of machine precision
        is used instead.

    Returns
    -------
    p : float
        The value of P(a,x)

    q : float
        The value of P(a,x)

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` > 0.0.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

        (`errno` 3)
            Algorithm fails to terminate in *<value>* iterations.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma_incomplete'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_FLOAT64, # Output, q.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _q = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _x,
        _tol,
        _p,
        _q,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Algorithm fails to terminate in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _q_py = utils._EngineFloat64ScalarType.to_py(_q)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['p', 'q'],
        )(
            _p_py,
            _q_py,
        )
    _err_class.manage()
    return return_data

def gamma_incomplete_vector(n, a, x, tol, p, q, ivalid):
    u"""
    Incomplete gamma functions, vectorized P(a,x) and Q(a,x).

    For full information please refer to the NAG Library document for
    s14bn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14bnf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    a : float, shape (`n`)
        The argument a_i of the function, for i = 1,2,...,`n`.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    tol : float
        The relative accuracy required by you in the results. If
        ``gamma_incomplete_vector`` is entered with `tol` greater than
        1.0 or less than machine precision, then the value of machine
        precision is used instead.

    p : float, shape (`n`), modified in place
        On exit: P(a_i,x_i), the function values.

    q : float, shape (`n`), modified in place
        On exit: Q(a_i,x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for a_i and x_i,
        for i = 1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          a_i <= 0.

        `ivalid`[i-1] = 2

          x_i < 0.

        `ivalid`[i-1] = 3

          Algorithm fails to terminate.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.gamma_incomplete_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14bnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Output, p[n].
        utils._EngineFloat64ArrayType.c_type, # Output, q[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _x,
        _tol,
        _p,
        _q,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _q_cls.refresh_py(_q)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def beta_log_real(a, b):
    u"""
    Logarithm of the beta function ln(B)(a,b).

    For full information please refer to the NAG Library document for
    s14cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14cbf.html

    Parameters
    ----------
    a : float
        The argument a of the function.

    b : float
        The argument b of the function.

    Returns
    -------
    lbab : float
        The value of the logarithm of the beta function, ln(B)(a,b).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` > 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.beta_log_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Output, lbab.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _lbab = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _lbab,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: b > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _lbab_py = utils._EngineFloat64ScalarType.to_py(_lbab)
        return_data = _lbab_py
    _err_class.manage()
    return return_data

def beta_incomplete(a, b, x):
    u"""
    Regularized incomplete beta function I_x(a,b) and its complement
    1-I_x.

    For full information please refer to the NAG Library document for
    s14cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14ccf.html

    Parameters
    ----------
    a : float
        The argument a of the function.

    b : float
        The argument b of the function.

    x : float
        x, upper limit of integration.

    Returns
    -------
    w : float
        The value of the incomplete beta function I_x(a,b) evaluated
        from zero to x.

    w1 : float
        The value of the complement of the incomplete beta function
        1-I_x(a,b), i.e., the incomplete beta function evaluated from x
        to one.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `b` = *<value>*.

            Constraint: `b` >= 0.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` >= 0.0.

        (`errno` 2)
            On entry, `a` and `b` were zero.

            Constraint: `a` or `b` must be nonzero.

        (`errno` 3)
            On entry, `x` = *<value>*.

            Constraint: 0.0 <= `x` <= 1.0.

        (`errno` 4)
            On entry, `x` and `a` were zero.

            Constraint: `x` or `a` must be nonzero.

        (`errno` 5)
            On entry, 1.0-`x` and `b` were zero.

            Constraint: 1.0-`x` or `b` must be nonzero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.beta_incomplete'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, w.
        utils._BYREF_FLOAT64, # Output, w1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _w = utils._EngineFloat64ScalarType.empty_ctypes()
    _w1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _x,
        _w,
        _w1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: b >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a and b were zero.',
                    'Constraint: a or b must be nonzero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 <= x <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x and a were zero.',
                    'Constraint: x or a must be nonzero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, 1.0-x and b were zero.',
                    'Constraint: 1.0-x or b must be nonzero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _w_py = utils._EngineFloat64ScalarType.to_py(_w)
        _w1_py = utils._EngineFloat64ScalarType.to_py(_w1)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['w', 'w1'],
        )(
            _w_py,
            _w1_py,
        )
    _err_class.manage()
    return return_data

def beta_log_real_vector(n, a, b, f, ivalid):
    u"""
    Logarithm of the beta function, vectorized ln(B)(a,b).

    For full information please refer to the NAG Library document for
    s14cp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14cpf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    a : float, shape (`n`)
        The argument a_i of the function, for i = 1,2,...,`n`.

    b : float, shape (`n`)
        The argument b_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: ln(B)(a_i,b_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for a_i and b_i,
        for i = 1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          a_i or b_i <= 0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `a` or `b` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.beta_log_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14cpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _b,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of a or b was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def beta_incomplete_vector(n, a, b, x, w, w1, ivalid):
    u"""
    Regularized incomplete beta function, vectorized I_x(a,b) and its
    complement 1-I_x.

    For full information please refer to the NAG Library document for
    s14cq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14cqf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    a : float, shape (`n`)
        The argument a_i of the function, for i = 1,2,...,`n`.

    b : float, shape (`n`)
        The argument b_i of the function, for i = 1,2,...,`n`.

    x : float, shape (`n`)
        x_i, the upper limit of integration, for i = 1,2,...,`n`.

    w : float, shape (`n`), modified in place
        On exit: the values of the incomplete beta function
        I_x_i(a_i,b_i) evaluated from zero to x_i.

    w1 : float, shape (`n`), modified in place
        On exit: the values of the complement of the incomplete beta
        function 1-I_x_i(a_i,b_i), i.e., the incomplete beta function
        evaluated from x_i to one.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for the ith
        evaluation, for i = 1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          a_i or b_i < 0.

        `ivalid`[i-1] = 2

          Both a_i and b_i = 0.

        `ivalid`[i-1] = 3

          x_i∉[0,1].

        `ivalid`[i-1] = 4

          Both x_i and a_i = 0.

        `ivalid`[i-1] = 5

          Both 1-x_i and b_i = 0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one argument had an invalid value.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.beta_incomplete_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s14cqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, w1[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _w1_cls = utils._EngineFloat64ArrayType(
        w1, {'fun_name': fun_name, 'entity': 'w1'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w1 = _w1_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _b,
        _x,
        _w,
        _w1,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one argument had an invalid value.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _w1_cls.refresh_py(_w1)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def cdf_normal(x):
    u"""
    Cumulative Normal distribution function P(x).

    For full information please refer to the NAG Library document for
    s15ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    p : float
        The value of the cumulative Normal distribution function, P(x).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.cdf_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, p.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _p,
    )
    utils._handle_cb_exception(excs)
    _p_py = utils._EngineFloat64ScalarType.to_py(_p)
    return_data = _p_py
    return return_data

def compcdf_normal(x):
    u"""
    Complement of cumulative Normal distribution function Q(x).

    For full information please refer to the NAG Library document for
    s15ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    q : float
        The value of the complement of the cumulative Normal
        distribution function, Q(x).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.compcdf_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, q.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _q,
    )
    utils._handle_cb_exception(excs)
    _q_py = utils._EngineFloat64ScalarType.to_py(_q)
    return_data = _q_py
    return return_data

def erfc_real(x):
    u"""
    Complement of error function erfc(x).

    For full information please refer to the NAG Library document for
    s15ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    erfc : float
        The value of the complementary error function, erfc(x).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erfc_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, erfc.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _erfc = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _erfc,
    )
    utils._handle_cb_exception(excs)
    _erfc_py = utils._EngineFloat64ScalarType.to_py(_erfc)
    return_data = _erfc_py
    return return_data

def erf_real(x):
    u"""
    Error function erf(x).

    For full information please refer to the NAG Library document for
    s15ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    efx : float
        The value of the error function erf(x).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erf_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, efx.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _efx = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _efx,
    )
    utils._handle_cb_exception(excs)
    _efx_py = utils._EngineFloat64ScalarType.to_py(_efx)
    return_data = _efx_py
    return return_data

def dawson(x):
    u"""
    Dawson's integral.

    For full information please refer to the NAG Library document for
    s15af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15aff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    f : float
        The value of Dawson's Integral, F(x).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.dawson'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, f.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _f = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _f,
    )
    utils._handle_cb_exception(excs)
    _f_py = utils._EngineFloat64ScalarType.to_py(_f)
    return_data = _f_py
    return return_data

def erfcx_real(x):
    u"""
    Scaled complement of error function, erfcx(x).

    For full information please refer to the NAG Library document for
    s15ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15agf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ecx : float
        The value of the scaled complementary error function erfcx(x).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = *<value>* and the constant x_hi = *<value>*.

            Constraint: `x` < x_hi.

        (`errno` 2)
            On entry, abs(`x`) was in the interval [*<value>*,*<value>*)
            where erfcx(`x`) is approximately 1/(sqrt(pi)*abs(`x`)): `x`
            = *<value>*.

        (`errno` 3)
            On entry, `x` = *<value>* and the constant x_neg =
            *<value>*.

            Constraint: `x` >= x_neg.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erfcx_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, ecx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _ecx = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _ecx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and the constant'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x_hi = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x < x_hi.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1021:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(x) was in the interval',
                    '[{:s},{:s}) where erfcx(x) is'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'approximately 1/(sqrt(pi)*abs(x)): x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1031:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and the constant'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x_neg = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x >= x_neg.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ecx_py = utils._EngineFloat64ScalarType.to_py(_ecx)
        return_data = _ecx_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['ecx'],
            )(
                _ecx_py,
            )
    _err_class.manage()
    return return_data

def cdf_normal_vector(n, x, f):
    u"""
    Cumulative Normal distribution function, vectorized P(x).

    For full information please refer to the NAG Library document for
    s15ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15apf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: P(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.cdf_normal_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_cls.refresh_py(_f)
        return_data = None
    _err_class.manage()
    return return_data

def compcdf_normal_vector(n, x, f):
    u"""
    Complement of cumulative Normal distribution function, vectorized
    Q(x).

    For full information please refer to the NAG Library document for
    s15aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15aqf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Q(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.compcdf_normal_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_cls.refresh_py(_f)
        return_data = None
    _err_class.manage()
    return return_data

def erfc_real_vector(n, x, f):
    u"""
    Complement of error function, vectorized erfc(x).

    For full information please refer to the NAG Library document for
    s15ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15arf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: erfc(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erfc_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_cls.refresh_py(_f)
        return_data = None
    _err_class.manage()
    return return_data

def erf_real_vector(n, x, f):
    u"""
    Error function, vectorized erf(x).

    For full information please refer to the NAG Library document for
    s15as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15asf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: erf(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erf_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_cls.refresh_py(_f)
        return_data = None
    _err_class.manage()
    return return_data

def dawson_vector(n, x, f):
    u"""
    Dawson's integral, vectorized.

    For full information please refer to the NAG Library document for
    s15at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15atf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: F(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.dawson_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_cls.refresh_py(_f)
        return_data = None
    _err_class.manage()
    return return_data

def erfcx_real_vector(n, x, f, ivalid):
    u"""
    Scaled complement of error function, vectorized erfcx(x).

    For full information please refer to the NAG Library document for
    s15au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15auf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: erfcx(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. The threshold value is the same
          as for `errno` = 1 in ``erfcx_real``, as defined in the Users'
          Note for your implementation.

        `ivalid`[i-1] = 2

          x_i was in the interval [1/(2sqrt(epsilon)),x_hi). The
          threshold values are the same as for `errno` = 2 in
          ``erfcx_real``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 3

          x_i is too small and positive. The threshold value is the same
          as for `errno` = 3 in ``erfcx_real``, as defined in the Users'
          Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` produced a result with
            reduced accuracy.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erfcx_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x produced a result with reduced accuracy.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def erfc_complex(z):
    u"""
    Scaled complex complement of error function, exp(-z^2)erfc(-iz).

    For full information please refer to the NAG Library document for
    s15dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15ddf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    Returns
    -------
    erfcz : complex
        The value of the function evaluated at z, w(z) =
        e^(-z^2)erfc(-iz).

    Raises
    ------
    NagValueError
        (`errno` 5)
            Result has no precision when entered with argument `z` =
            (*<value>*,*<value>*).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Real part of result overflows when entered with argument `z`
            = (*<value>*,*<value>*).

        (`errno` 2)
            Imaginary part of result overflows when entered with
            argument `z` = (*<value>*,*<value>*).

        (`errno` 3)
            Both real and imaginary parts of result overflow when
            entered with argument `z` = (*<value>*,*<value>*).

        (`errno` 4)
            Result has less than half precision when entered with
            argument `z` = (*<value>*,*<value>*).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erfc_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15ddft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_COMPLEX128, # Output, erfcz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _erfcz = utils._EngineComplex128ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _z,
        _erfcz,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Real part of result overflows when entered with',
                    'argument z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Imaginary part of result overflows when entered with',
                    'argument z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Both real and imaginary parts of result overflow when entered with',
                    'argument z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Result has less than half precision when entered with',
                    'argument z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Result has no precision when entered with',
                    'argument z = ({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _erfcz_py = utils._EngineComplex128ScalarType.to_py(_erfcz)
        return_data = _erfcz_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['erfcz'],
            )(
                _erfcz_py,
            )
    _err_class.manage()
    return return_data

def erfc_complex_vector(n, z, f, ivalid):
    u"""
    Scaled complex complement of error function, vectorized
    exp(-z^2)erfc(-iz).

    For full information please refer to the NAG Library document for
    s15dr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15drf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    z : complex, shape (`n`)
        The argument z_i of the function, for i = 1,2,...,`n`.

    f : complex, shape (`n`), modified in place
        On exit: w(z_i) = e^(-z_i^2), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for z_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          Real part of result overflows.

        `ivalid`[i-1] = 2

          Imaginary part of result overflows.

        `ivalid`[i-1] = 3

          Both real and imaginary part of result overflows.

        `ivalid`[i-1] = 4

          Result has less than half precision.

        `ivalid`[i-1] = 5

          Result has no precision.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `z` produced a result with
            reduced accuracy.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.erfc_complex_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s15drft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, z[n].
        utils._EngineComplex128ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _z = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineComplex128ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _z,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of z produced a result with reduced accuracy.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_y0_real(x):
    u"""
    Bessel function Y_0(x).

    For full information please refer to the NAG Library document for
    s17ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    y0 : float
        The value of the Bessel function Y_0(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= *<value>*.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_y0_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, y0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y0_py = utils._EngineFloat64ScalarType.to_py(_y0)
        return_data = _y0_py
    _err_class.manage()
    return return_data

def bessel_y1_real(x):
    u"""
    Bessel function Y_1(x).

    For full information please refer to the NAG Library document for
    s17ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    y1 : float
        The value of the Bessel function Y_1(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= *<value>*.

            x is too large, the function returns the amplitude of the
            Y_1 oscillation, sqrt(2/(pi*x)).

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            Y_1 is undefined, the function returns zero.

        (`errno` 3)
            `x` is too close to zero and there is danger of overflow,
            `x` = *<value>*.

            Constraint: `x` > *<value>*.

            The function returns the value of Y_1(x) at the smallest
            valid argument.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_y1_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, y1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x is too large, the function returns the amplitude of the Y_1 oscillation,',
                    'sqrt(2/(pi*x)).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                    'Y_1 is undefined, the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'x is too close to zero and there is danger of overflow,',
                    'x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'The function returns the value of Y_1(x) at the smallest valid argument.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y1_py = utils._EngineFloat64ScalarType.to_py(_y1)
        return_data = _y1_py
    _err_class.manage()
    return return_data

def bessel_j0_real(x):
    u"""
    Bessel function J_0(x).

    For full information please refer to the NAG Library document for
    s17ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    j0 : float
        The value of the Bessel function J_0(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large, the function returns the amplitude of
            the J_0 oscillation, sqrt(2/(pi*abs(x))).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_j0_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, j0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _j0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _j0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'abs(x) is too large, the function returns the amplitude of the',
                    'J_0 oscillation, sqrt(2/(pi*abs(x))).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _j0_py = utils._EngineFloat64ScalarType.to_py(_j0)
        return_data = _j0_py
    _err_class.manage()
    return return_data

def bessel_j1_real(x):
    u"""
    Bessel function J_1(x).

    For full information please refer to the NAG Library document for
    s17af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17aff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    j1 : float
        The value of the Bessel function J_1(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large, the function returns the amplitude of
            the J_1 oscillation, sqrt(2/(pi*abs(x))).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_j1_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, j1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _j1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _j1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'abs(x) is too large, the function returns the amplitude of the J_1',
                    'oscillation, sqrt(2/(pi*abs(x))).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _j1_py = utils._EngineFloat64ScalarType.to_py(_j1)
        return_data = _j1_py
    _err_class.manage()
    return return_data

def airy_ai_real(x):
    u"""
    Airy function Ai(x).

    For full information please refer to the NAG Library document for
    s17ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17agf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ai : float
        The value of the Airy function, Ai(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_ai_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, ai.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _ai = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _ai,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and positive. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and negative. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ai_py = utils._EngineFloat64ScalarType.to_py(_ai)
        return_data = _ai_py
    _err_class.manage()
    return return_data

def airy_bi_real(x):
    u"""
    Airy function Bi(x).

    For full information please refer to the NAG Library document for
    s17ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17ahf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    bi : float
        The value of the Airy function, Bi(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_bi_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17ahft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, bi.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _bi = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _bi,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and positive. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and negative. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _bi_py = utils._EngineFloat64ScalarType.to_py(_bi)
        return_data = _bi_py
    _err_class.manage()
    return return_data

def airy_ai_deriv(x):
    u"""
    Airy function Ai'(x).

    For full information please refer to the NAG Library document for
    s17aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17ajf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    aid : float
        The value of the derivative of the Airy function Ai(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_ai_deriv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17ajft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, aid.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _aid = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _aid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and positive. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and negative. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _aid_py = utils._EngineFloat64ScalarType.to_py(_aid)
        return_data = _aid_py
    _err_class.manage()
    return return_data

def airy_bi_deriv(x):
    u"""
    Airy function Bi'(x).

    For full information please refer to the NAG Library document for
    s17ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17akf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    bid : float
        The value of the derivative of the Airy function Bi(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_bi_deriv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17akft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, bid.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _bid = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _bid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and positive. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x is too large and negative. The function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _bid_py = utils._EngineFloat64ScalarType.to_py(_bid)
        return_data = _bid_py
    _err_class.manage()
    return return_data

def bessel_zeros(a, n, mode, rel, x):
    u"""
    Zeros of Bessel functions J_alpha(x), J'_alpha(x), Y_alpha(x) or
    Y'_alpha(x).

    For full information please refer to the NAG Library document for
    s17al

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17alf.html

    Parameters
    ----------
    a : float
        The order alpha of the function.

    n : int
        The number N of zeros required.

    mode : int
        Specifies the form of the function whose zeros are required.

        `mode` = 1

          The zeros of J_alpha(x) are required.

        `mode` = 2

          The zeros of Y_alpha(x) are required;

        `mode` = 3

          The zeros of J'_alpha(x) are required;

        `mode` = 4

          The zeros of Y'_alpha(x) are required.

    rel : float
        The relative accuracy to which the zeros are required.

    x : float, shape (`n`), modified in place
        On exit: the N required zeros of the function specified by
        `mode`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `rel` = *<value>*.

            Constraint: `rel` > 0.0.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` <= 4.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` >= 1.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` <= 100000.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` >= 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_zeros'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17alft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, mode.
        utils._BYREF_FLOAT64, # Input, rel.
        utils._EngineFloat64ArrayType.c_type, # Output, x[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _rel = utils._EngineFloat64ScalarType(
        rel, {'fun_name': fun_name, 'entity': 'rel'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _n,
        _mode,
        _rel,
        _x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rel > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a <= 100000.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        return_data = None
    _err_class.manage()
    return return_data

def bessel_y0_real_vector(n, x, f, ivalid):
    u"""
    Bessel function vectorized Y_0(x).

    For full information please refer to the NAG Library document for
    s17aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17aqf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Y_0(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the Y_0 oscillation, sqrt(2/pi x_i).

        `ivalid`[i-1] = 2

          On entry, x_i <= 0.0, Y_0 is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_y0_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_y1_real_vector(n, x, f, ivalid):
    u"""
    Bessel function vectorized Y_1(x).

    For full information please refer to the NAG Library document for
    s17ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17arf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Y_1(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the Y_1 oscillation, sqrt(2/pi x_i).

        `ivalid`[i-1] = 2

          On entry, x_i <= 0.0, Y_1 is undefined. `f`[i-1] contains 0.0.

        `ivalid`[i-1] = 3

          x_i is too close to zero, there is a danger of overflow. On
          failure, `f`[i-1] contains the value of Y_1(x) at the smallest
          valid argument.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_y1_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_j0_real_vector(n, x, f, ivalid):
    u"""
    Bessel function vectorized J_0(x).

    For full information please refer to the NAG Library document for
    s17as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17asf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: J_0(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the J_0 oscillation, sqrt(2/pi|x_i|).

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_j0_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_j1_real_vector(n, x, f, ivalid):
    u"""
    Bessel function vectorized J_1(x).

    For full information please refer to the NAG Library document for
    s17at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17atf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: J_1(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the J_1 oscillation, sqrt(2/pi|x_i|).

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_j1_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def airy_ai_real_vector(n, x, f, ivalid):
    u"""
    Airy function vectorized Ai(x).

    For full information please refer to the NAG Library document for
    s17au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17auf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Ai(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_ai_real``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_ai_real``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_ai_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def airy_bi_real_vector(n, x, f, ivalid):
    u"""
    Airy function vectorized Bi(x).

    For full information please refer to the NAG Library document for
    s17av

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17avf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Bi(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_bi_real``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_bi_real``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_bi_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17avft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def airy_ai_deriv_vector(n, x, f, ivalid):
    u"""
    Derivatives of the Airy function, vectorized Ai'(x).

    For full information please refer to the NAG Library document for
    s17aw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17awf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Ai'(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_ai_deriv``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_ai_deriv``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_ai_deriv_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17awft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def airy_bi_deriv_vector(n, x, f, ivalid):
    u"""
    Derivatives of the Airy function, vectorized Bi'(x).

    For full information please refer to the NAG Library document for
    s17ax

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17axf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: Bi'(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_bi_deriv``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_bi_deriv``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_bi_deriv_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17axft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_y_complex(fnu, z, n, scal, cy):
    u"""
    Bessel functions Y_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s17dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dcf.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        z, the argument of the functions.

    n : int
        N, the number of members required in the sequence
        Y_nu(z),Y_{nu+1}(z),...,Y_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-|Im(z)|).

    cy : complex, shape (`n`), modified in place
        On exit: the N required function values: `cy`[i-1] contains
        Y_{nu+i-1}(z), for i = 1,2,...,N.

    Returns
    -------
    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. The positions of such components in the array `cy`
        are arbitrary.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `z` = (0.0,0.0).

        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because abs(`z`) = *<value>* < *<value>*.

        (`errno` 2)
            No computation because Re(`z`) = *<value>* > *<value>*,
            `scal` = 'U'.

        (`errno` 3)
            No computation because `fnu`+`n`-1 = *<value>* is too large.

        (`errno` 5)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 6)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 4)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_y_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17dcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, fnu.
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
        utils._BYREF_INT, # Output, nz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, cwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _fnu = utils._EngineFloat64ScalarType(
        fnu, {'fun_name': fun_name, 'entity': 'fnu'},
    ).to_ctypes()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _cy_cls = utils._EngineComplex128ArrayType(
        cy, {'fun_name': fun_name, 'entity': 'cy'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _cy = _cy_cls.to_ctypes(can_reference_contents=False)
    _nz = utils._EngineIntScalarType.empty_ctypes()
    _cwrk = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fnu,
        _z,
        _n,
        _scal,
        _cy,
        _nz,
        _cwrk,
        _errbuf,
        _ierr,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, z = (0.0,0.0).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fnu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fnu >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal has an illegal value: scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1013:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} < {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1111:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'Re(z) = {:s} > {:s}, scal = \'U\'.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1210:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because fnu+n-1 = {:s} is too large.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1310:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1311:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1410:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1411:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 1510:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation -- algorithm termination condition not met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _cy_cls.refresh_py(_cy)
        _nz_py = utils._EngineIntScalarType.to_py(_nz)
        return_data = _nz_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nz'],
            )(
                _nz_py,
            )
    _err_class.manage()
    return return_data

def bessel_j_complex(fnu, z, n, scal, cy):
    u"""
    Bessel functions J_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s17de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17def.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        J_nu(z),J_{nu+1}(z),...,J_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-|Im(z)|).

    cy : complex, shape (`n`), modified in place
        On exit: the N required function values: `cy`[i-1] contains
        J_{nu+i-1}(z), for i = 1,2,...,N.

    Returns
    -------
    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. If `nz` > 0, then elements `cy`[`n`-`nz`],
        `cy`[`n`-`nz`+1],...,`cy`[`n`-1] are set to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because Im(`z`) = *<value>* > *<value>*,
            `scal` = 'U'.

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 4)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 3)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_j_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17deft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, fnu.
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
        utils._BYREF_INT, # Output, nz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _fnu = utils._EngineFloat64ScalarType(
        fnu, {'fun_name': fun_name, 'entity': 'fnu'},
    ).to_ctypes()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _cy_cls = utils._EngineComplex128ArrayType(
        cy, {'fun_name': fun_name, 'entity': 'cy'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _cy = _cy_cls.to_ctypes(can_reference_contents=False)
    _nz = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fnu,
        _z,
        _n,
        _scal,
        _cy,
        _nz,
        _errbuf,
        _ierr,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fnu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fnu >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal has an illegal value: scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'Im(z) = {:s} > {:s}, scal = \'U\'.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1210:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1211:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1310:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1311:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1410:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation -- algorithm termination condition not met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _cy_cls.refresh_py(_cy)
        _nz_py = utils._EngineIntScalarType.to_py(_nz)
        return_data = _nz_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nz'],
            )(
                _nz_py,
            )
    _err_class.manage()
    return return_data

def airy_ai_complex(deriv, z, scal):
    u"""
    Airy functions Ai(z) and Ai'(z), complex z.

    For full information please refer to the NAG Library document for
    s17dg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dgf.html

    Parameters
    ----------
    deriv : str, length 1
        Specifies whether the function or its derivative is required.

        `deriv` = 'F'

          Ai(z) is returned.

        `deriv` = 'D'

          Ai'(z) is returned.

    z : complex
        The argument z of the function.

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The result is returned unscaled.

        `scal` = 'S'

          The result is returned scaled by the factor e^(2z*sqrt(z)/3).

    Returns
    -------
    ai : complex
        The required function or derivative value.

    nz : int
        Indicates whether or not `ai` is set to zero due to underflow.
        This can only occur when `scal` = 'U'.

        `nz` = 0

          `ai` is not set to zero.

        `nz` = 1

          `ai` is set to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `deriv` has an illegal value: `deriv` = *<value>*.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 2)
            No computation because Re(omega) too large, where omega =
            (2/3)*`z`^(3/2).

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_ai_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17dgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, deriv(len=1).
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._BYREF_COMPLEX128, # Output, ai.
        utils._BYREF_INT, # Output, nz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of deriv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _deriv = utils._EngineCharScalarType(
        deriv, {'fun_name': fun_name, 'entity': 'deriv'}, exp_chrlen=1,
    ).to_ctypes()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _ai = utils._EngineComplex128ScalarType.empty_ctypes()
    _nz = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _deriv,
        _z,
        _scal,
        _ai,
        _nz,
        _errbuf,
        _ierr,
        _deriv._length_,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, deriv has an illegal value: deriv = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal has an illegal value: scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because Re(omega) too large, where omega = (2/3)*z^(3/2).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1111:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because Re(omega) too large, where omega = (2/3)*z^(3/2).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1210:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1310:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1311:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1410:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation -- algorithm termination condition not met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ai_py = utils._EngineComplex128ScalarType.to_py(_ai)
        _nz_py = utils._EngineIntScalarType.to_py(_nz)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ai', 'nz'],
        )(
            _ai_py,
            _nz_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def airy_bi_complex(deriv, z, scal):
    u"""
    Airy functions Bi(z) and Bi'(z), complex z.

    For full information please refer to the NAG Library document for
    s17dh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dhf.html

    Parameters
    ----------
    deriv : str, length 1
        Specifies whether the function or its derivative is required.

        `deriv` = 'F'

          Bi(z) is returned.

        `deriv` = 'D'

          Bi'(z) is returned.

    z : complex
        The argument z of the function.

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The result is returned unscaled.

        `scal` = 'S'

          The result is returned scaled by the factor
          e^(|Re(2z*sqrt(z)/3)|).

    Returns
    -------
    bi : complex
        The required function or derivative value.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `deriv` has an illegal value: `deriv` = *<value>*.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 2)
            No computation because Re(`z`) = *<value>* is too large when
            `scal` = 'U'.

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.airy_bi_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17dhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, deriv(len=1).
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._BYREF_COMPLEX128, # Output, bi.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of deriv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _deriv = utils._EngineCharScalarType(
        deriv, {'fun_name': fun_name, 'entity': 'deriv'}, exp_chrlen=1,
    ).to_ctypes()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _bi = utils._EngineComplex128ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _deriv,
        _z,
        _scal,
        _bi,
        _errbuf,
        _ierr,
        _deriv._length_,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, deriv has an illegal value: deriv = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal has an illegal value: scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because Re(z) = {:s} is too large when'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'scal = \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1210:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1310:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1311:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1410:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation -- algorithm termination condition not met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _bi_py = utils._EngineComplex128ScalarType.to_py(_bi)
        return_data = _bi_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['bi'],
            )(
                _bi_py,
            )
    _err_class.manage()
    return return_data

def hankel_complex(m, fnu, z, n, scal, cy):
    u"""
    Hankel functions H_nu+a^(j)(z), j = 1,2, real a >= 0, complex z,
    nu=0,1,2,....

    For full information please refer to the NAG Library document for
    s17dl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dlf.html

    Parameters
    ----------
    m : int
        The kind of functions required.

        `m` = 1

          The functions are H_nu^(1)(z).

        `m` = 2

          The functions are H_nu^(2)(z).

    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        H_nu^(`m`)(z),H_nu+1^(`m`)(z),...,H_nu+N-1^(`m`)(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-iz) when `m`
          = 1, or by the factor e^(iz) when `m` = 2.

    cy : complex, shape (`n`), modified in place
        On exit: the N required function values: `cy`[i-1] contains
        H_nu+i-1^(`m`)(z), for i = 1,2,...,N.

    Returns
    -------
    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. If `nz` > 0, then if Im(z) > 0.0 and `m` = 1, or
        Im(z) < 0.0 and `m` = 2, elements
        `cy`[0],`cy`[1],...,`cy`[`nz`-1] are set to zero. In the
        complementary half-planes, `nz` simply states the number of
        underflows, and not which elements they are.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `z` = (0.0,0.0).

        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `m` has illegal value: `m` = *<value>*.

        (`errno` 2)
            No computation because abs(`z`) = *<value>* < *<value>*.

        (`errno` 3)
            No computation because `fnu`+`n`-1 = *<value>* is too large.

        (`errno` 5)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 6)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 4)
            Results lack precision, `fnu`+`n`-1 = *<value>* > *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.hankel_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17dlft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input, fnu.
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
        utils._BYREF_INT, # Output, nz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _fnu = utils._EngineFloat64ScalarType(
        fnu, {'fun_name': fun_name, 'entity': 'fnu'},
    ).to_ctypes()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _cy_cls = utils._EngineComplex128ArrayType(
        cy, {'fun_name': fun_name, 'entity': 'cy'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _cy = _cy_cls.to_ctypes(can_reference_contents=False)
    _nz = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _fnu,
        _z,
        _n,
        _scal,
        _cy,
        _nz,
        _errbuf,
        _ierr,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, z = (0.0,0.0).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fnu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fnu >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal has an illegal value: scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1013:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1014:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m has illegal value: m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} < {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1210:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because fnu+n-1 = {:s} is too large.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1310:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1311:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision,',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1410:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1411:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1412:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 1510:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation -- algorithm termination condition not met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _cy_cls.refresh_py(_cy)
        _nz_py = utils._EngineIntScalarType.to_py(_nz)
        return_data = _nz_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nz'],
            )(
                _nz_py,
            )
    _err_class.manage()
    return return_data

def struve_h0(x):
    u"""
    Struve function of order 0, H_0(x).

    For full information please refer to the NAG Library document for
    s17ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17gaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    h0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            `x` is too large and the function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.struve_h0'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, h0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _h0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _h0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'x is too large and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _h0_py = utils._EngineFloat64ScalarType.to_py(_h0)
        return_data = _h0_py
    _err_class.manage()
    return return_data

def struve_h1(x):
    u"""
    Struve function of order 1, H_1(x).

    For full information please refer to the NAG Library document for
    s17gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17gbf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    h1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            `x` is too large and the function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.struve_h1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s17gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, h1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _h1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _h1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'x is too large and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _h1_py = utils._EngineFloat64ScalarType.to_py(_h1)
        return_data = _h1_py
    _err_class.manage()
    return return_data

def bessel_k0_real(x):
    u"""
    Modified Bessel function K_0(x).

    For full information please refer to the NAG Library document for
    s18ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    k0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            K_0 is undefined and the function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k0_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, k0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _k0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _k0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                    'K_0 is undefined and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _k0_py = utils._EngineFloat64ScalarType.to_py(_k0)
        return_data = _k0_py
    _err_class.manage()
    return return_data

def bessel_k1_real(x):
    u"""
    Modified Bessel function K_1(x).

    For full information please refer to the NAG Library document for
    s18ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    k1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            K_0 is undefined and the function returns zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > *<value>*.

            `x` is too small, there is a danger of overflow and the
            function returns approximately the largest representable
            value.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k1_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, k1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _k1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _k1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                    'K_0 is undefined and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x is too small, there is a danger of overflow and the function returns',
                    'approximately the largest representable value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _k1_py = utils._EngineFloat64ScalarType.to_py(_k1)
        return_data = _k1_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['k1'],
            )(
                _k1_py,
            )
    _err_class.manage()
    return return_data

def bessel_i0_real(x):
    u"""
    Modified Bessel function I_0(x).

    For full information please refer to the NAG Library document for
    s18ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    i0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large and the function returns the
            approximate value of I_0(x) at the nearest valid argument.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i0_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, i0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _i0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _i0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'abs(x) is too large and the function returns the approximate value of',
                    'I_0(x) at the nearest valid argument.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _i0_py = utils._EngineFloat64ScalarType.to_py(_i0)
        return_data = _i0_py
    _err_class.manage()
    return return_data

def bessel_i1_real(x):
    u"""
    Modified Bessel function I_1(x).

    For full information please refer to the NAG Library document for
    s18af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18aff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    i1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large and the function returns the
            approximate value of I_1(x) at the nearest valid argument.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i1_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, i1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _i1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _i1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'abs(x) is too large and the function returns the approximate value of',
                    'I_1(x) at the nearest valid argument.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _i1_py = utils._EngineFloat64ScalarType.to_py(_i1)
        return_data = _i1_py
    _err_class.manage()
    return return_data

def bessel_k0_real_vector(n, x, f, ivalid):
    u"""
    Modified Bessel function vectorized K_0(x).

    For full information please refer to the NAG Library document for
    s18aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18aqf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: K_0(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i <= 0.0, K_0(x_i) is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k0_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_k1_real_vector(n, x, f, ivalid):
    u"""
    Modified Bessel function vectorized K_1(x).

    For full information please refer to the NAG Library document for
    s18ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18arf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: K_1(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i <= 0.0, K_1(x_i) is undefined. `f`[i-1] contains 0.0.

        `ivalid`[i-1] = 2

          x_i is too small, there is a danger of overflow. `f`[i-1]
          contains zero. The threshold value is the same as for `errno`
          = 2 in ``bessel_k1_real``, as defined in the Users' Note for
          your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k1_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_i0_real_vector(n, x, f, ivalid):
    u"""
    Modified Bessel function vectorized I_0(x).

    For full information please refer to the NAG Library document for
    s18as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18asf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: I_0(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large. `f`[i-1] contains the approximate value of
          I_0(x_i) at the nearest valid argument. The threshold value is
          the same as for `errno` = 1 in ``bessel_i0_real``, as defined
          in the Users' Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i0_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_i1_real_vector(n, x, f, ivalid):
    u"""
    Modified Bessel function vectorized I_1(x).

    For full information please refer to the NAG Library document for
    s18at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18atf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: I_1(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large. `f`[i-1] contains the approximate value of
          I_1(x_i) at the nearest valid argument. The threshold value is
          the same as for `errno` = 1 in ``bessel_i1_real``, as defined
          in the Users' Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i1_real_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_k0_scaled(x):
    u"""
    Scaled modified Bessel function e^xK_0(x).

    For full information please refer to the NAG Library document for
    s18cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18ccf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    sk0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k0_scaled'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, sk0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _sk0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _sk0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _sk0_py = utils._EngineFloat64ScalarType.to_py(_sk0)
        return_data = _sk0_py
    _err_class.manage()
    return return_data

def bessel_k1_scaled(x):
    u"""
    Scaled modified Bessel function e^xK_1(x).

    For full information please refer to the NAG Library document for
    s18cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cdf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    sk1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            K_1 is undefined and the function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > *<value>*.

            The function returns the value of the function at the
            smallest permitted value of the argument.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k1_scaled'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18cdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, sk1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _sk1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _sk1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                    'K_1 is undefined and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'The function returns the value of the function at the smallest',
                    'permitted value of the argument.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _sk1_py = utils._EngineFloat64ScalarType.to_py(_sk1)
        return_data = _sk1_py
    _err_class.manage()
    return return_data

def bessel_i0_scaled(x):
    u"""
    Scaled modified Bessel function e^(-|x|)I_0(x).

    For full information please refer to the NAG Library document for
    s18ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    si0 : float
        The value of the function at x.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i0_scaled'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18ceft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, si0.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _si0 = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _si0,
    )
    utils._handle_cb_exception(excs)
    _si0_py = utils._EngineFloat64ScalarType.to_py(_si0)
    return_data = _si0_py
    return return_data

def bessel_i1_scaled(x):
    u"""
    Scaled modified Bessel function e^(-|x|)I_1(x).

    For full information please refer to the NAG Library document for
    s18cf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    si1 : float
        The value of the function at x.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i1_scaled'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18cfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, si1.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _si1 = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _si1,
    )
    utils._handle_cb_exception(excs)
    _si1_py = utils._EngineFloat64ScalarType.to_py(_si1)
    return_data = _si1_py
    return return_data

def bessel_k0_scaled_vector(n, x, f, ivalid):
    u"""
    Scaled modified Bessel function vectorized e^xK_0(x).

    For full information please refer to the NAG Library document for
    s18cq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cqf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: e^(x_i)K_0(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i <= 0.0, K_0(x_i) is undefined. `f`[i-1] contains
          0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k0_scaled_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18cqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_k1_scaled_vector(n, x, f, ivalid):
    u"""
    Scaled modified Bessel function vectorized e^xK_1(x).

    For full information please refer to the NAG Library document for
    s18cr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18crf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: e^(x_i)K_1(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i <= 0.0, K_1(x_i) is undefined. `f`[i-1] contains
          0.0.

        `ivalid`[i-1] = 2

          x_i is too close to zero, as determined by the value of the
          safe-range parameter ``machine.real_safe``: there is a danger
          of causing overflow. `f`[i-1] contains the reciprocal of the
          safe-range parameter.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k1_scaled_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18crft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_i0_scaled_vector(n, x, f):
    u"""
    Scaled modified Bessel function vectorized e^(-|x|)I_0(x).

    For full information please refer to the NAG Library document for
    s18cs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18csf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: e^(-|x_i|)I_0(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i0_scaled_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18csft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_i1_scaled_vector(n, x, f):
    u"""
    Scaled modified Bessel function vectorized e^(-|x|)I_1(x).

    For full information please refer to the NAG Library document for
    s18ct

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18ctf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: e^(-|x_i|)I_1(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i1_scaled_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18ctft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bessel_k_complex(fnu, z, n, scal, cy):
    u"""
    Modified Bessel functions K_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s18dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18dcf.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        K_nu(z),K_{nu+1}(z),...,K_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^z.

    cy : complex, shape (`n`), modified in place
        On exit: the N required function values: `cy`[i-1] contains
        K_{nu+i-1}(z), for i = 1,2,...,N.

    Returns
    -------
    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. If `nz` > 0 and Re(z) >= 0.0, elements
        `cy`[0],`cy`[1],...,`cy`[`nz`-1] are set to zero. If Re(z) <
        0.0, `nz` simply states the number of underflows, and not which
        elements they are.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `z` = (0.0,0.0).

        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because abs(`z`) = *<value>* < *<value>*.

        (`errno` 3)
            No computation because `fnu`+`n`-1 = *<value>* is too large.

        (`errno` 5)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 6)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 4)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_k_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18dcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, fnu.
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
        utils._BYREF_INT, # Output, nz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _fnu = utils._EngineFloat64ScalarType(
        fnu, {'fun_name': fun_name, 'entity': 'fnu'},
    ).to_ctypes()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _cy_cls = utils._EngineComplex128ArrayType(
        cy, {'fun_name': fun_name, 'entity': 'cy'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _cy = _cy_cls.to_ctypes(can_reference_contents=False)
    _nz = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fnu,
        _z,
        _n,
        _scal,
        _cy,
        _nz,
        _errbuf,
        _ierr,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, z = (0.0,0.0).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fnu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fnu >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal has an illegal value: scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1013:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} < {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1210:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because fnu+n-1 = {:s} is too large.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1310:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1311:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1410:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1411:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1412:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 1510:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation -- algorithm termination condition not met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _cy_cls.refresh_py(_cy)
        _nz_py = utils._EngineIntScalarType.to_py(_nz)
        return_data = _nz_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nz'],
            )(
                _nz_py,
            )
    _err_class.manage()
    return return_data

def bessel_i_complex(fnu, z, n, scal, cy):
    u"""
    Modified Bessel functions I_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s18de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18def.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        I_nu(z),I_{nu+1}(z),...,I_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-|Re(z)|).

    cy : complex, shape (`n`), modified in place
        On exit: the N required function values: `cy`[i-1] contains
        I_{nu+i-1}(z), for i = 1,2,...,N.

    Returns
    -------
    nz : int
        The number of components of `cy` that are set to zero due to
        underflow.

        If `nz` > 0, then elements
        `cy`[`n`-`nz`],`cy`[`n`-`nz`+1],...,`cy`[`n`-1] are set to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because Re(`z`) = *<value>* > *<value>*,
            `scal` = 'U'.

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 4)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 3)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_i_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18deft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, fnu.
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_STRING, # Input, scal(len=1).
        utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
        utils._BYREF_INT, # Output, nz.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _fnu = utils._EngineFloat64ScalarType(
        fnu, {'fun_name': fun_name, 'entity': 'fnu'},
    ).to_ctypes()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _scal = utils._EngineCharScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'}, exp_chrlen=1,
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _cy_cls = utils._EngineComplex128ArrayType(
        cy, {'fun_name': fun_name, 'entity': 'cy'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _cy = _cy_cls.to_ctypes(can_reference_contents=False)
    _nz = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fnu,
        _z,
        _n,
        _scal,
        _cy,
        _nz,
        _errbuf,
        _ierr,
        _scal._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fnu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fnu >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal has an illegal value: scal = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'Re(z) = {:s} > {:s}, scal = \'U\'.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1210:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1211:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Results lack precision because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1310:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1311:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'fnu+n-1 = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1312:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation because',
                    'abs(z) = {:s} > {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1410:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No computation -- algorithm termination condition not met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _cy_cls.refresh_py(_cy)
        _nz_py = utils._EngineIntScalarType.to_py(_nz)
        return_data = _nz_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nz'],
            )(
                _nz_py,
            )
    _err_class.manage()
    return return_data

def struve_l0(x):
    u"""
    Modified Struve function of order 0, L_0(x).

    For full information please refer to the NAG Library document for
    s18ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    l0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The value of L_0(x) is larger than the largest positive
            floating-point number, which is returned.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.struve_l0'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, l0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _l0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _l0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of L_0(x) is larger than the largest positive floating-point',
                    'number, which is returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _l0_py = utils._EngineFloat64ScalarType.to_py(_l0)
        return_data = _l0_py
    _err_class.manage()
    return return_data

def struve_l1(x):
    u"""
    Modified Struve function of order 1, L_1(x).

    For full information please refer to the NAG Library document for
    s18gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gbf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    l1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The value of L_1(x) is larger than the largest positive
            floating-point number, which is returned.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.struve_l1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, l1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _l1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _l1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of L_1(x) is larger than the largest positive floating-point',
                    'number, which is returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _l1_py = utils._EngineFloat64ScalarType.to_py(_l1)
        return_data = _l1_py
    _err_class.manage()
    return return_data

def struve_i0ml0(x):
    u"""
    The function I_0(x)-L_0(x), where I_0(x) is a modified Bessel
    function and L_0(x) is a Struve function.

    For full information please refer to the NAG Library document for
    s18gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gcf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    iml0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.struve_i0ml0'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18gcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, iml0.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _iml0 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _iml0,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _iml0_py = utils._EngineFloat64ScalarType.to_py(_iml0)
        return_data = _iml0_py
    _err_class.manage()
    return return_data

def struve_i1ml1(x):
    u"""
    The function I_1(x)-L_1(x), where I_1(x) is a modified Bessel
    function and L_1(x) is a Struve function.

    For full information please refer to the NAG Library document for
    s18gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gdf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    iml1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.struve_i1ml1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18gdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, iml1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _iml1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _iml1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _iml1_py = utils._EngineFloat64ScalarType.to_py(_iml1)
        return_data = _iml1_py
    _err_class.manage()
    return return_data

def bessel_j_seq_complex(z, a, nl, b):
    u"""
    Bessel function of the 1st kind J_{alpha+/-n}(z).

    For full information please refer to the NAG Library document for
    s18gk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gkf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    a : float
        The order alpha of the first member in the required sequence of
        function values.

    nl : int
        The value of N.

    b : complex, shape (|`nl`|+1), modified in place
        On exit: with the function exits successfully or `errno` = 3,
        the required sequence of function values: `b`[n-1] contains
        J_{alpha+n-1}(z) if `nl` >= 0 and J_{alpha-n+1}(z) otherwise,
        for n = 1,2,...,abs(`nl`)+1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(`nl`) = *<value>*.

            Constraint: abs(`nl`) <= 101.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` < 1.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` >= 0.0.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: when `nl` < 0, `z` != (0.0,0.0).

        (`errno` 2)
            Computation abandoned due to the likelihood of overflow.

        (`errno` 4)
            Computation abandoned due to total loss of precision.

        (`errno` 5)
            Computation abandoned due to failure to satisfy the
            termination condition.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Computation completed but some precision has been lost.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.bessel_j_seq_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s18gkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_INT, # Input, nl.
        utils._EngineComplex128ArrayType.c_type, # Output, b[abs(nl)+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _nl = utils._EngineIntScalarType(
        nl, {'fun_name': fun_name, 'entity': 'nl'},
    ).to_ctypes()
    _nl_py = utils._EngineIntScalarType.to_py(_nl)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(abs(_nl_py)+1,),
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _z,
        _a,
        _nl,
        _b,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(nl) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(nl) <= 101.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when nl < 0, z != (0.0,0.0).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned due to the likelihood of overflow.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation completed but some precision has been lost.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned due to total loss of precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned due to failure to satisfy the termination condition.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def kelvin_ber(x):
    u"""
    Kelvin function ber(x).

    For full information please refer to the NAG Library document for
    s19aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ber : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large for an accurate result to be returned
            and the function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_ber'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, ber.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _ber = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _ber,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'abs(x) is too large for an accurate result to be returned and the function',
                    'returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ber_py = utils._EngineFloat64ScalarType.to_py(_ber)
        return_data = _ber_py
    _err_class.manage()
    return return_data

def kelvin_bei(x):
    u"""
    Kelvin function bei(x).

    For full information please refer to the NAG Library document for
    s19ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    bei : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large for an accurate result to be returned
            and the function returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_bei'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, bei.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _bei = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _bei,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'abs(x) is too large for an accurate result to be returned and the function',
                    'returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _bei_py = utils._EngineFloat64ScalarType.to_py(_bei)
        return_data = _bei_py
    _err_class.manage()
    return return_data

def kelvin_ker(x):
    u"""
    Kelvin function ker(x).

    For full information please refer to the NAG Library document for
    s19ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ker : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*. The function returns zero.

            Constraint: `x` <= *<value>*.

            `x` is too large, the result underflows and the function
            returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            The function is undefined and returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_ker'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, ker.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _ker = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _ker,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The function returns zero.',
                    'Constraint: x <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x is too large, the result underflows and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                    'The function is undefined and returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ker_py = utils._EngineFloat64ScalarType.to_py(_ker)
        return_data = _ker_py
    _err_class.manage()
    return return_data

def kelvin_kei(x):
    u"""
    Kelvin function kei(x).

    For full information please refer to the NAG Library document for
    s19ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    kei : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*. The function returns zero.

            Constraint: `x` <= *<value>*.

            `x` is too large, the result underflows and the function
            returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

            The function is undefined and returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_kei'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, kei.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _kei = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _kei,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The function returns zero.',
                    'Constraint: x <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x is too large, the result underflows and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                    'The function is undefined and returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _kei_py = utils._EngineFloat64ScalarType.to_py(_kei)
        return_data = _kei_py
    _err_class.manage()
    return return_data

def kelvin_ber_vector(n, x, f, ivalid):
    u"""
    Kelvin function vectorized ber(x).

    For full information please refer to the NAG Library document for
    s19an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19anf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: ber(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          abs(x_i) is too large for an accurate result to be returned.
          `f`[i-1] contains zero. The threshold value is the same as for
          `errno` = 1 in ``kelvin_ber``, as defined in the Users' Note
          for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_ber_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def kelvin_bei_vector(n, x, f, ivalid):
    u"""
    Kelvin function vectorized bei(x).

    For full information please refer to the NAG Library document for
    s19ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19apf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: bei(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          abs(x_i) is too large for an accurate result to be returned.
          `f`[i-1] contains zero. The threshold value is the same as for
          `errno` = 1 in ``kelvin_bei``, as defined in the Users' Note
          for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_bei_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def kelvin_ker_vector(n, x, f, ivalid):
    u"""
    Kelvin function vectorized ker(x).

    For full information please refer to the NAG Library document for
    s19aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19aqf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: ker(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large, the result underflows. `f`[i-1] contains
          zero. The threshold value is the same as for `errno` = 1 in
          ``kelvin_ker``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i <= 0.0, the function is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_ker_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def kelvin_kei_vector(n, x, f, ivalid):
    u"""
    Kelvin function vectorized kei(x).

    For full information please refer to the NAG Library document for
    s19ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19arf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: kei(x_i), the function values.

    ivalid : int, shape (`n`), modified in place
        On exit: `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,`n`.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large, the result underflows. `f`[i-1] contains
          zero. The threshold value is the same as for `errno` = 1 in
          ``kelvin_kei``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i < 0.0, the function is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.kelvin_kei_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s19arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Output, ivalid[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def fresnel_s(x):
    u"""
    Fresnel integral S(x).

    For full information please refer to the NAG Library document for
    s20ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    sx : float
        The value of the function at x.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.fresnel_s'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s20acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, sx.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _sx = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _sx,
    )
    utils._handle_cb_exception(excs)
    _sx_py = utils._EngineFloat64ScalarType.to_py(_sx)
    return_data = _sx_py
    return return_data

def fresnel_c(x):
    u"""
    Fresnel integral C(x).

    For full information please refer to the NAG Library document for
    s20ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    cx : float
        The value of the function at x.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.fresnel_c'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s20adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, cx.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _cx = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _cx,
    )
    utils._handle_cb_exception(excs)
    _cx_py = utils._EngineFloat64ScalarType.to_py(_cx)
    return_data = _cx_py
    return return_data

def fresnel_s_vector(n, x, f):
    u"""
    Fresnel integral vectorized S(x).

    For full information please refer to the NAG Library document for
    s20aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20aqf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: S(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.fresnel_s_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s20aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def fresnel_c_vector(n, x, f):
    u"""
    Fresnel integral vectorized C(x).

    For full information please refer to the NAG Library document for
    s20ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20arf.html

    Parameters
    ----------
    n : int
        n, the number of points.

    x : float, shape (`n`)
        The argument x_i of the function, for i = 1,2,...,`n`.

    f : float, shape (`n`), modified in place
        On exit: C(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.fresnel_c_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s20arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ellipint_symm_1_degen(x, y):
    u"""
    Degenerate symmetrised elliptic integral of 1st kind R_C(x,y).

    For full information please refer to the NAG Library document for
    s21ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21baf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    y : float
        The argument y of the function.

    Returns
    -------
    rc : float
        The value of an elementary integral, which occurs as a
        degenerate case of an elliptic integral of the first kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

            The function is undefined.

        (`errno` 2)
            On entry, `y` = 0.0.

            Constraint: `y` != 0.0.

            The function is undefined and returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_symm_1_degen'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Output, rc.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _rc = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _rc,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                    'The function is undefined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, y = 0.0.',
                    'Constraint: y != 0.0.',
                    'The function is undefined and returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rc_py = utils._EngineFloat64ScalarType.to_py(_rc)
        return_data = _rc_py
    _err_class.manage()
    return return_data

def ellipint_symm_1(x, y, z):
    u"""
    Symmetrised elliptic integral of 1st kind R_F(x,y,z).

    For full information please refer to the NAG Library document for
    s21bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bbf.html

    Parameters
    ----------
    x : float
        The arguments x, y and z of the function.

    y : float
        The arguments x, y and z of the function.

    z : float
        The arguments x, y and z of the function.

    Returns
    -------
    rf : float
        The value of the symmetrised elliptic integral of the first
        kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: `x` >= 0.0 and `y` >= 0.0 and `z` >= 0.0.

            The function is undefined.

        (`errno` 2)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: at most one of `x`, `y` and `z` is 0.0.

            The function is undefined and returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_symm_1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Input, z.
        utils._BYREF_FLOAT64, # Output, rf.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _z = utils._EngineFloat64ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _rf = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _z,
        _rf,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x >= 0.0 and y >= 0.0 and z >= 0.0.',
                    'The function is undefined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: at most one of x, y and z is 0.0.',
                    'The function is undefined and returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rf_py = utils._EngineFloat64ScalarType.to_py(_rf)
        return_data = _rf_py
    _err_class.manage()
    return return_data

def ellipint_symm_2(x, y, z):
    u"""
    Symmetrised elliptic integral of 2nd kind R_D(x,y,z).

    For full information please refer to the NAG Library document for
    s21bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bcf.html

    Parameters
    ----------
    x : float
        The arguments x, y and z of the function.

    y : float
        The arguments x, y and z of the function.

    z : float
        The arguments x, y and z of the function.

    Returns
    -------
    rd : float
        The value of the symmetrised elliptic integral of the second
        kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` and `y` are both 0.0.

            Constraint: at most one of `x` and `y` is 0.0.

            The function is undefined.

        (`errno` 1)
            On entry, `x` = *<value>* and `y` = *<value>*.

            Constraint: `x` >= 0.0 and `y` >= 0.0.

            The function is undefined.

        (`errno` 2)
            On entry, `z` = *<value>*.

            Constraint: `z` > 0.0.

            The function is undefined.

        (`errno` 3)
            On entry, L = *<value>*, `x` = *<value>*, `y` = *<value>*
            and `z` = *<value>*.

            Constraint: `z` >= L and (`x` >= L or `y` >= L).

            The function is undefined.

        (`errno` 4)
            On entry, U = *<value>*, `x` = *<value>*, `y` = *<value>*
            and `z` = *<value>*.

            Constraint: `x` < U and `y` < U and `z` < U.

            There is a danger of setting underflow and the function
            returns zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_symm_2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21bcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Input, z.
        utils._BYREF_FLOAT64, # Output, rd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _z = utils._EngineFloat64ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _rd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _z,
        _rd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99909:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x and y are both 0.0.',
                    'Constraint: at most one of x and y is 0.0.',
                    'The function is undefined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and y = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x >= 0.0 and y >= 0.0.',
                    'The function is undefined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: z > 0.0.',
                    'The function is undefined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, L = {:s}, x = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: z >= L and (x >= L or y >= L).',
                    'The function is undefined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, U = {:s}, x = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'y = {:s} and z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x < U and y < U and z < U.',
                    'There is a danger of setting underflow and the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rd_py = utils._EngineFloat64ScalarType.to_py(_rd)
        return_data = _rd_py
    _err_class.manage()
    return return_data

def ellipint_symm_3(x, y, z, r):
    u"""
    Symmetrised elliptic integral of 3rd kind R_J(x,y,z,r).

    For full information please refer to the NAG Library document for
    s21bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bdf.html

    Parameters
    ----------
    x : float
        The arguments x, y, z and rho of the function.

    y : float
        The arguments x, y, z and rho of the function.

    z : float
        The arguments x, y, z and rho of the function.

    r : float
        The arguments x, y, z and rho of the function.

    Returns
    -------
    rj : float
        The value of the symmetrised elliptic integral of the third
        kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: at most one of `x`, `y` and `z` is 0.0.

            The function is undefined.

        (`errno` 1)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: `x` >= 0.0 and `y` >= 0.0 and `z` >= 0.0.

        (`errno` 2)
            On entry, `r` = 0.0.

            Constraint: `r` != 0.0.

        (`errno` 3)
            On entry, L = *<value>*, `r` = *<value>*, `x` = *<value>*,
            `y` = *<value>* and `z` = *<value>*.

            Constraint: abs(`r`) >= L and at most one of `x`, `y` and
            `z` is less than L.

        (`errno` 4)
            On entry, U = *<value>*, `r` = *<value>*, `x` = *<value>*,
            `y` = *<value>* and `z` = *<value>*.

            Constraint: abs(`r`) <= U and `x` <= U and `y` <= U and `z`
            <= U.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_symm_3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21bdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Input, z.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Output, rj.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _z = utils._EngineFloat64ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _rj = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _z,
        _r,
        _rj,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: at most one of x, y and z is 0.0.',
                    'The function is undefined.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x >= 0.0 and y >= 0.0 and z >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = 0.0.',
                    'Constraint: r != 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, L = {:s}, r = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x = {:s}, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[5]),
                    ),
                    'Constraint: abs(r) >= L and at most one of x, y and z is less than L.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, U = {:s}, r = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x = {:s}, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'z = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[5]),
                    ),
                    'Constraint: abs(r) <= U and x <= U and y <= U and z <= U.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rj_py = utils._EngineFloat64ScalarType.to_py(_rj)
        return_data = _rj_py
    _err_class.manage()
    return return_data

def ellipint_legendre_1(phi, dm):
    u"""
    Elliptic integral of 1st kind, Legendre form, F(phi | m).

    For full information please refer to the NAG Library document for
    s21be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bef.html

    Parameters
    ----------
    phi : float
        The arguments phi and m of the function.

    dm : float
        The arguments phi and m of the function.

    Returns
    -------
    f : float
        The value of the classical (Legendre) form of the incomplete
        elliptic integral of the first kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `phi` = *<value>*.

            Constraint: 0 <= `phi` <= pi/2.

            On failure, the function returns zero.

        (`errno` 2)
            On entry, `phi` = *<value>* and `dm` = *<value>*; the
            integral is undefined.

            Constraint: `dm`*sin^2(`phi`) <= 1.0.

            On failure, the function returns zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, sin(`phi`) = 1 and `dm` = 1.0; the integral is
            infinite.

            On failure, the function returns the largest machine number
            (see ``machine.real_largest``).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_legendre_1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21beft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, phi.
        utils._BYREF_FLOAT64, # Input, dm.
        utils._BYREF_FLOAT64, # Output, f.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _phi = utils._EngineFloat64ScalarType(
        phi, {'fun_name': fun_name, 'entity': 'phi'},
    ).to_ctypes()
    _dm = utils._EngineFloat64ScalarType(
        dm, {'fun_name': fun_name, 'entity': 'dm'},
    ).to_ctypes()
    _f = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _phi,
        _dm,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, phi = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= phi <= pi/2.',
                    'On failure, the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, phi = {:s} and dm = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'the integral is undefined.',
                    'Constraint: dm*sin^2(phi) <= 1.0.',
                    'On failure, the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1003:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sin(phi) = 1 and dm = 1.0;',
                    'the integral is infinite.',
                    'On failure, the function returns the largest machine number',
                    '(see machine.real_largest).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_py = utils._EngineFloat64ScalarType.to_py(_f)
        return_data = _f_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['f'],
            )(
                _f_py,
            )
    _err_class.manage()
    return return_data

def ellipint_legendre_2(phi, dm):
    u"""
    Elliptic integral of 2nd kind, Legendre form, E(phi | m).

    For full information please refer to the NAG Library document for
    s21bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bff.html

    Parameters
    ----------
    phi : float
        The arguments phi and m of the function.

    dm : float
        The arguments phi and m of the function.

    Returns
    -------
    e : float
        The value of the classical (Legendre) form of the incomplete
        elliptic integral of the second kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `phi` = *<value>*.

            Constraint: 0 <= `phi` <= pi/2.

        (`errno` 2)
            On entry, `phi` = *<value>* and `dm` = *<value>*; the
            integral is undefined.

            Constraint: `dm`*sin^2(`phi`) <= 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_legendre_2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21bfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, phi.
        utils._BYREF_FLOAT64, # Input, dm.
        utils._BYREF_FLOAT64, # Output, e.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _phi = utils._EngineFloat64ScalarType(
        phi, {'fun_name': fun_name, 'entity': 'phi'},
    ).to_ctypes()
    _dm = utils._EngineFloat64ScalarType(
        dm, {'fun_name': fun_name, 'entity': 'dm'},
    ).to_ctypes()
    _e = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _phi,
        _dm,
        _e,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, phi = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= phi <= pi/2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, phi = {:s} and dm = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'the integral is undefined.',
                    'Constraint: dm*sin^2(phi) <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _e_py = utils._EngineFloat64ScalarType.to_py(_e)
        return_data = _e_py
    _err_class.manage()
    return return_data

def ellipint_legendre_3(dn, phi, dm):
    u"""
    Elliptic integral of 3rd kind, Legendre form, Pi(n;phi | m).

    For full information please refer to the NAG Library document for
    s21bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bgf.html

    Parameters
    ----------
    dn : float
        The arguments n, phi and m of the function.

    phi : float
        The arguments n, phi and m of the function.

    dm : float
        The arguments n, phi and m of the function.

    Returns
    -------
    p : float
        The value of the classical (Legendre) form of the incomplete
        elliptic integral of the third kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `phi` = *<value>*.

            Constraint: 0 <= `phi` <= (pi/2).

        (`errno` 2)
            On entry, `phi` = *<value>* and `dm` = *<value>*; the
            integral is undefined.

            Constraint: `dm`*sin^2(`phi`) <= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, sin(`phi`) = 1 and `dm` = 1.0; the integral is
            infinite.

        (`errno` 4)
            On entry, `phi` = *<value>* and `dn` = *<value>*; the
            integral is infinite.

            Constraint: `dn`*sin^2(`phi`) != 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_legendre_3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21bgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, dn.
        utils._BYREF_FLOAT64, # Input, phi.
        utils._BYREF_FLOAT64, # Input, dm.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _dn = utils._EngineFloat64ScalarType(
        dn, {'fun_name': fun_name, 'entity': 'dn'},
    ).to_ctypes()
    _phi = utils._EngineFloat64ScalarType(
        phi, {'fun_name': fun_name, 'entity': 'phi'},
    ).to_ctypes()
    _dm = utils._EngineFloat64ScalarType(
        dm, {'fun_name': fun_name, 'entity': 'dm'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _dn,
        _phi,
        _dm,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, phi = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= phi <= (pi/2).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, phi = {:s} and dm = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'the integral is undefined.',
                    'Constraint: dm*sin^2(phi) <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1003:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sin(phi) = 1 and dm = 1.0; the integral is infinite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1004:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, phi = {:s} and dn = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'the integral is infinite.',
                    'Constraint: dn*sin^2(phi) != 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def ellipint_complete_1(dm):
    u"""
    Complete elliptic integral of 1st kind, Legendre form, K(m).

    For full information please refer to the NAG Library document for
    s21bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bhf.html

    Parameters
    ----------
    dm : float
        The argument m of the function.

    Returns
    -------
    k : float
        The value of the classical (Legendre) form of the complete
        elliptic integral of the first kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `dm` = *<value>*; the integral is undefined.

            Constraint: `dm` < 1.0.

            On failure, the function returns zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `dm` = 1.0; the integral is infinite.

            On failure, the function returns the largest machine number
            (see ``machine.real_largest``).
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_complete_1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21bhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, dm.
        utils._BYREF_FLOAT64, # Output, k.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _dm = utils._EngineFloat64ScalarType(
        dm, {'fun_name': fun_name, 'entity': 'dm'},
    ).to_ctypes()
    _k = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _dm,
        _k,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dm = {:s}; the integral is undefined.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dm < 1.0.',
                    'On failure, the function returns zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1002:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dm = 1.0; the integral is infinite.',
                    'On failure, the function returns the largest machine number',
                    '(see machine.real_largest).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _k_py = utils._EngineFloat64ScalarType.to_py(_k)
        return_data = _k_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['k'],
            )(
                _k_py,
            )
    _err_class.manage()
    return return_data

def ellipint_complete_2(dm):
    u"""
    Complete elliptic integral of 2nd kind, Legendre form, E(m).

    For full information please refer to the NAG Library document for
    s21bj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bjf.html

    Parameters
    ----------
    dm : float
        The argument m of the function.

    Returns
    -------
    e : float
        The value of the classical (Legendre) form of the complete
        elliptic integral of the second kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `dm` = *<value>*; the integral is undefined.

            Constraint: `dm` <= 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_complete_2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21bjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, dm.
        utils._BYREF_FLOAT64, # Output, e.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _dm = utils._EngineFloat64ScalarType(
        dm, {'fun_name': fun_name, 'entity': 'dm'},
    ).to_ctypes()
    _e = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _dm,
        _e,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dm = {:s}; the integral is undefined.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dm <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _e_py = utils._EngineFloat64ScalarType.to_py(_e)
        return_data = _e_py
    _err_class.manage()
    return return_data

def jacellip_real(u, m):
    u"""
    Jacobian elliptic functions sn, cn and dn of real argument.

    For full information please refer to the NAG Library document for
    s21ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21caf.html

    Parameters
    ----------
    u : float
        The argument u.

    m : float
        The argument m.

    Returns
    -------
    sn : float
        The value of sn(u).

    cn : float
        The value of cn(u).

    dn : float
        The value of dn(u).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(`u`) is too large: abs(`u`) = *<value>* it
            must be less than *<value>*.

        (`errno` 2)
            On entry, abs(`m`) is too large when used in conjunction
            with the supplied argument `u`: abs(`m`) = *<value>* it must
            be less than *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.jacellip_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, u.
        utils._BYREF_FLOAT64, # Input, m.
        utils._BYREF_FLOAT64, # Output, sn.
        utils._BYREF_FLOAT64, # Output, cn.
        utils._BYREF_FLOAT64, # Output, dn.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _u = utils._EngineFloat64ScalarType(
        u, {'fun_name': fun_name, 'entity': 'u'},
    ).to_ctypes()
    _m = utils._EngineFloat64ScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _sn = utils._EngineFloat64ScalarType.empty_ctypes()
    _cn = utils._EngineFloat64ScalarType.empty_ctypes()
    _dn = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _u,
        _m,
        _sn,
        _cn,
        _dn,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(u) is too large: abs(u) = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'it must be less than {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(m) is too large when used in conjunction with the supplied',
                    'argument u: abs(m) = {:s} it must be less than'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _sn_py = utils._EngineFloat64ScalarType.to_py(_sn)
        _cn_py = utils._EngineFloat64ScalarType.to_py(_cn)
        _dn_py = utils._EngineFloat64ScalarType.to_py(_dn)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['sn', 'cn', 'dn'],
        )(
            _sn_py,
            _cn_py,
            _dn_py,
        )
    _err_class.manage()
    return return_data

def jacellip_complex(z, ak2):
    u"""
    Jacobian elliptic functions sn, cn and dn of complex argument.

    For full information please refer to the NAG Library document for
    s21cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21cbf.html

    Parameters
    ----------
    z : complex
        The argument z of the functions.

    ak2 : float
        The value of k^2.

    Returns
    -------
    sn : complex
        The value of the function sn(z).

    cn : complex
        The value of the function cn(z).

    dn : complex
        The value of the function dn(z).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(Im(`z`)) is too large: abs(Im(`z`)) =
            *<value>*. It must be less than *<value>*.

        (`errno` 1)
            On entry, abs(Re(`z`)) is too large: abs(Re(`z`)) =
            *<value>*. It must be less than *<value>*.

        (`errno` 1)
            On entry, `ak2` = *<value>*.

            Constraint: `ak2` <= 1.0.

        (`errno` 1)
            On entry, `ak2` = *<value>*.

            Constraint: `ak2` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.jacellip_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_FLOAT64, # Input, ak2.
        utils._BYREF_COMPLEX128, # Output, sn.
        utils._BYREF_COMPLEX128, # Output, cn.
        utils._BYREF_COMPLEX128, # Output, dn.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _ak2 = utils._EngineFloat64ScalarType(
        ak2, {'fun_name': fun_name, 'entity': 'ak2'},
    ).to_ctypes()
    _sn = utils._EngineComplex128ScalarType.empty_ctypes()
    _cn = utils._EngineComplex128ScalarType.empty_ctypes()
    _dn = utils._EngineComplex128ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _z,
        _ak2,
        _sn,
        _cn,
        _dn,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(Im(z)) is too large:',
                    'abs(Im(z)) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'It must be less than {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(Re(z)) is too large:',
                    'abs(Re(z)) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'It must be less than {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ak2 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ak2 <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ak2 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ak2 >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _sn_py = utils._EngineComplex128ScalarType.to_py(_sn)
        _cn_py = utils._EngineComplex128ScalarType.to_py(_cn)
        _dn_py = utils._EngineComplex128ScalarType.to_py(_dn)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['sn', 'cn', 'dn'],
        )(
            _sn_py,
            _cn_py,
            _dn_py,
        )
    _err_class.manage()
    return return_data

def jactheta_real(k, x, q):
    u"""
    Jacobian theta functions theta_k(x,q) of real argument.

    For full information please refer to the NAG Library document for
    s21cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21ccf.html

    Parameters
    ----------
    k : int
        Denotes the function theta_k(x,q) to be evaluated. Note that `k`
        = 4 is equivalent to `k` = 0.

    x : float
        The argument x of the function.

    q : float
        The argument q of the function.

    Returns
    -------
    jt : float
        The value of one of the Jacobian theta functions theta_0(x,q),
        theta_1(x,q), theta_2(x,q), theta_3(x,q) or theta_4(x,q) for a
        real argument x and non-negative q < 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `q` = *<value>*.

            Constraint: `q` < 1.0.

        (`errno` 1)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` <= 4.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.jactheta_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, q.
        utils._BYREF_FLOAT64, # Output, jt.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _jt = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _k,
        _x,
        _q,
        _jt,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _jt_py = utils._EngineFloat64ScalarType.to_py(_jt)
        return_data = _jt_py
    _err_class.manage()
    return return_data

def ellipint_general_2(z, akp, a, b):
    u"""
    General elliptic integral of 2nd kind F(z,k',a,b) of complex
    argument.

    For full information please refer to the NAG Library document for
    s21da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21daf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    akp : float
        The argument k' of the function.

    a : float
        The argument a of the function.

    b : float
        The argument b of the function.

    Returns
    -------
    f : complex
        The value of the general elliptic integral of the second kind
        F(z,k',a,b) for a complex argument z.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(Im(`z`)) is too large: abs(Im(`z`)) =
            *<value>*. It must not exceed *<value>*.

        (`errno` 1)
            On entry, Re(`z`) is too large: Re(`z`) = *<value>*. It must
            not exceed *<value>*.

        (`errno` 1)
            On entry, Re(`z`) < 0.0: Re(`z`) = *<value>*.

        (`errno` 1)
            On entry, abs(`akp`) is too large: abs(`akp`) = *<value>*.
            It must not exceed *<value>*.

        (`errno` 2)
            The iterative procedure used to evaluate the integral has
            failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.ellipint_general_2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s21daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_FLOAT64, # Input, akp.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_COMPLEX128, # Output, f.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _akp = utils._EngineFloat64ScalarType(
        akp, {'fun_name': fun_name, 'entity': 'akp'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _f = utils._EngineComplex128ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _z,
        _akp,
        _a,
        _b,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(Im(z)) is too large:',
                    'abs(Im(z)) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'It must not exceed {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, Re(z) is too large: Re(z) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'It must not exceed {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, Re(z) < 0.0: Re(z) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(akp) is too large: abs(akp) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'It must not exceed {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iterative procedure used to evaluate the integral has',
                    'failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_py = utils._EngineComplex128ScalarType.to_py(_f)
        return_data = _f_py
    _err_class.manage()
    return return_data

def legendre_p(mode, x, m, nl, p):
    u"""
    Legendre functions of 1st kind P_n^m(x) or -P_n^m - (x).

    For full information please refer to the NAG Library document for
    s22aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22aaf.html

    Parameters
    ----------
    mode : int
        Indicates whether the sequence of function values is to be
        returned unnormalized or normalized.

        `mode` = 1

          The sequence of function values is returned unnormalized.

        `mode` = 2

          The sequence of function values is returned normalized.

    x : float
        The argument x of the function.

    m : int
        The order m of the function.

    nl : int
        The degree N of the last function required in the sequence.

    p : float, shape (`nl`+1), modified in place
        On exit: the required sequence of function values as follows:

        - if `mode` = 1, `p`[n] contains P_n^m(x), for n = 0,1,...,N;

        - if `mode` = 2, `p`[n] contains - P_n^m - (x), for n =
          0,1,...,N.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nl` = *<value>* and abs(`m`) = *<value>*.

            Constraint: `nl`+abs(`m`) <= 55.

        (`errno` 1)
            On entry, abs(`m`) = *<value>*.

            Constraint: abs(`m`) <= 27.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: when abs(`m`) != 0, `nl` >= 0.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: `nl` <= 100 when `m` = 0.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: `nl` >= 0.

        (`errno` 1)
            On entry, abs(`x`) = *<value>*.

            Constraint: abs(`x`) <= 1.0.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` <= 2.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.legendre_p'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s22aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, mode.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, nl.
        utils._EngineFloat64ArrayType.c_type, # Output, p[nl+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _nl = utils._EngineIntScalarType(
        nl, {'fun_name': fun_name, 'entity': 'nl'},
    ).to_ctypes()
    _nl_py = utils._EngineIntScalarType.to_py(_nl)
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_nl_py+1,),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _mode,
        _x,
        _m,
        _nl,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nl = {:s} and abs(m) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nl+abs(m) <= 55.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(m) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(m) <= 27.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when abs(m) != 0, nl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nl <= 100 when m = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, abs(x) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode <= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def hyperg_confl_real(a, b, x):
    u"""
    Real confluent hypergeometric function 1_F_1(a;b;x).

    For full information please refer to the NAG Library document for
    s22ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22baf.html

    Parameters
    ----------
    a : float
        The parameter a of the function.

    b : float
        The parameter b of the function.

    x : float
        The argument x of the function.

    Returns
    -------
    m : float
        The solution M(a,b,x).

        Note: if overflow occurs upon completion, as indicated by
        `errno` = 2, |M(a,b,x)| may be assumed to be too large to be
        representable. `m` will be returned as +/-R_max, where R_max is
        the largest representable real number (see
        ``machine.real_largest``). The sign of `m` should match the sign
        of M(a,b,x). If overflow occurs during a subcalculation, as
        indicated by `errno` = 5, the sign may be incorrect, and the
        true value of M(a,b,x) may or may not be greater than R_max. In
        either case it is advisable to subsequently use
        ``hyperg_confl_real_scaled``.

    Raises
    ------
    NagValueError
        (`errno` 4)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 5)
            Overflow occurred in a subcalculation of M(a,b,x).

            The answer may be completely incorrect.

        (`errno` 11)
            On entry, `a` = *<value>*.

            Constraint: abs(`a`) <= arbnd = *<value>*.

        (`errno` 31)
            On entry, `b` = *<value>*.

            Constraint: abs(`b`) <= arbnd = *<value>*.

        (`errno` 32)
            On entry, `b` = *<value>*.

            M(a,b,x) is undefined when b is zero or a negative integer.

        (`errno` 51)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= arbnd = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of M(a,b,x).

            The returned value may be inaccurate.

        (`errno` 2)
            On completion, overflow occurred in the evaluation of
            M(a,b,x).

        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.
    """

    from ..base import _nag_s_consts

    excs = []
    fun_name = 'naginterfaces.base.specfun.hyperg_confl_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s22baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, m.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _m = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _x,
        _m,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_s_consts.S22BA_IERR_UFLOW and _ifmt == _nag_s_consts.S22BA_IFMT_UFLOW:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underflow occurred during the evaluation of M(a,b,x).',
                    'The returned value may be inaccurate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_OFLOW and _ifmt == _nag_s_consts.S22BA_IFMT_OFLOW_FULLCALC:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On completion, overflow occurred in the evaluation of M(a,b,x).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_WARN_RES and _ifmt == _nag_s_consts.S22BA_IFMT_RES_TOL1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate',
                    'indicates some precision may have been lost.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_RES and _ifmt == _nag_s_consts.S22BA_IFMT_RES_NOPREC:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate',
                    'indicates no accuracy can be guaranteed.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_OFLOW_SUB and _ifmt == _nag_s_consts.S22BA_IFMT_OFLOW_SUBCALC:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred in a subcalculation of M(a,b,x).',
                    'The answer may be completely incorrect.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_A_ARBND and _ifmt == _nag_s_consts.S22BA_IFMT_ARBND:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(a) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_B_ARBND and _ifmt == _nag_s_consts.S22BA_IFMT_ARBND:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(b) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_B_NEGI and _ifmt == _nag_s_consts.S22BA_IFMT_B_NEGI:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'M(a,b,x) is undefined when b is zero or a negative integer.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_X and _ifmt == _nag_s_consts.S22BA_IFMT_ARBND:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _m_py = utils._EngineFloat64ScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def hyperg_confl_real_scaled(ani, adr, bni, bdr, x):
    u"""
    Real confluent hypergeometric function 1_F_1(a;b;x) in scaled form.

    For full information please refer to the NAG Library document for
    s22bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22bbf.html

    Parameters
    ----------
    ani : float
        a_i, the nearest integer to a, satisfying a_i = a-a_r.

    adr : float
        a_r, the signed decimal remainder satisfying a_r = a-a_i and
        |a_r| <= 0.5.

    bni : float
        b_i, the nearest integer to b, satisfying b_i = b-b_r.

    bdr : float
        b_r, the signed decimal remainder satisfying b_r = b-b_i and
        |b_r| <= 0.5.

    x : float
        The argument x of the function.

    Returns
    -------
    frm : float
        m_f, the scaled real component of the solution satisfying m_f =
        M(a,b,x)*2^(-m_s).

        Note: if overflow occurs upon completion, as indicated by
        `errno` = 2, the value of m_f returned may still be correct. If
        overflow occurs in a subcalculation, as indicated by `errno` =
        5, this should not be assumed.

    scm : int
        m_s, the scaling power of two, satisfying m_s =
        log2(M(a,b,x)/(m_f)).

        Note: if overflow occurs upon completion, as indicated by
        `errno` = 2, then m_s >= I_max, where I_max is the largest
        representable integer (see ``machine.integer_max``). If overflow
        occurs during a subcalculation, as indicated by `errno` = 5, m_s
        may or may not be greater than I_max. In either case, `scm` =
        machine.integer_max will have been returned.

    Raises
    ------
    NagValueError
        (`errno` 4)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 5)
            Overflow occurred in a subcalculation of M(a,b,x).

            The answer may be completely incorrect.

        (`errno` 11)
            On entry, `ani` = *<value>*.

            Constraint: abs(`ani`) <= arbnd = *<value>*.

        (`errno` 13)
            `ani` is non-integral.

            On entry, `ani` = *<value>*.

            Constraint: `ani` = floor(`ani`).

        (`errno` 21)
            On entry, `adr` = *<value>*.

            Constraint: abs(`adr`) <= 0.5.

        (`errno` 31)
            On entry, `bni` = *<value>*.

            Constraint: abs(`bni`) <= arbnd = *<value>*.

        (`errno` 32)
            On entry, b = `bni`+`bdr` = *<value>*.

            M(a,b,x) is undefined when b is zero or a negative integer.

        (`errno` 33)
            `bni` is non-integral.

            On entry, `bni` = *<value>*.

            Constraint: `bni` = floor(`bni`).

        (`errno` 41)
            On entry, `bdr` = *<value>*.

            Constraint: abs(`bdr`) <= 0.5.

        (`errno` 51)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= arbnd = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of M(a,b,x).

            The returned value may be inaccurate.

        (`errno` 2)
            On completion, overflow occurred in the evaluation of
            M(a,b,x).

        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.
    """

    from ..base import _nag_s_consts

    excs = []
    fun_name = 'naginterfaces.base.specfun.hyperg_confl_real_scaled'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s22bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, ani.
        utils._BYREF_FLOAT64, # Input, adr.
        utils._BYREF_FLOAT64, # Input, bni.
        utils._BYREF_FLOAT64, # Input, bdr.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, frm.
        utils._BYREF_INT, # Output, scm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ani = utils._EngineFloat64ScalarType(
        ani, {'fun_name': fun_name, 'entity': 'ani'},
    ).to_ctypes()
    _adr = utils._EngineFloat64ScalarType(
        adr, {'fun_name': fun_name, 'entity': 'adr'},
    ).to_ctypes()
    _bni = utils._EngineFloat64ScalarType(
        bni, {'fun_name': fun_name, 'entity': 'bni'},
    ).to_ctypes()
    _bdr = utils._EngineFloat64ScalarType(
        bdr, {'fun_name': fun_name, 'entity': 'bdr'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _frm = utils._EngineFloat64ScalarType.empty_ctypes()
    _scm = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ani,
        _adr,
        _bni,
        _bdr,
        _x,
        _frm,
        _scm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_s_consts.S22BA_IERR_UFLOW and _ifmt == _nag_s_consts.S22BA_IFMT_UFLOW:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underflow occurred during the evaluation of M(a,b,x).',
                    'The returned value may be inaccurate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_OFLOW and _ifmt == _nag_s_consts.S22BA_IFMT_OFLOW_FULLCALC:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On completion, overflow occurred in the evaluation of M(a,b,x).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_WARN_RES and _ifmt == _nag_s_consts.S22BA_IFMT_RES_TOL1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate',
                    'indicates some precision may have been lost.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_RES and _ifmt == _nag_s_consts.S22BA_IFMT_RES_NOPREC:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate',
                    'indicates no accuracy can be guaranteed.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_OFLOW_SUB and _ifmt == _nag_s_consts.S22BA_IFMT_OFLOW_SUBCALC:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred in a subcalculation of M(a,b,x).',
                    'The answer may be completely incorrect.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BB_IERR_AI_ARBND and _ifmt == _nag_s_consts.S22BA_IFMT_ARBND:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ani = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(ani) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BB_IERR_AI_INT and _ifmt == _nag_s_consts.S22BB_IFMT_INT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ani is non-integral.',
                    'On entry, ani = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ani = floor(ani).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BB_IERR_AR and _ifmt == _nag_s_consts.S22BB_IFMT_MODR:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, adr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(adr) <= 0.5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BB_IERR_BI_ARBND and _ifmt == _nag_s_consts.S22BA_IFMT_ARBND:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, bni = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(bni) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BB_IERR_BI_NEGI and _ifmt == _nag_s_consts.S22BA_IFMT_B_NEGI:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = bni+bdr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'M(a,b,x) is undefined when b is zero or a negative integer.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BB_IERR_BI_INT and _ifmt == _nag_s_consts.S22BB_IFMT_INT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'bni is non-integral.',
                    'On entry, bni = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: bni = floor(bni).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BB_IERR_BR and _ifmt == _nag_s_consts.S22BB_IFMT_MODR:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, bdr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(bdr) <= 0.5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BA_IERR_X and _ifmt == _nag_s_consts.S22BA_IFMT_ARBND:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(x) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _frm_py = utils._EngineFloat64ScalarType.to_py(_frm)
        _scm_py = utils._EngineIntScalarType.to_py(_scm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['frm', 'scm'],
        )(
            _frm_py,
            _scm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def hyperg_gauss_real(a, b, c, x):
    u"""
    Real Gauss hypergeometric function 2_F_1(a,b;c;x).

    For full information please refer to the NAG Library document for
    s22be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22bef.html

    Parameters
    ----------
    a : float
        The parameter a.

    b : float
        The parameter b.

    c : float
        The parameter c.

    x : float
        The argument x.

    Returns
    -------
    f21 : float
        The value returned for the Gauss hypergeometric function
        2_F_1(a,b;c;x) for real parameters a,b and c, and real argument
        x.

    Raises
    ------
    NagValueError
        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 4)
            On entry, `x` = *<value>*, c = *<value>*, a+b = *<value>*.

            2_F_1(a,b;c;1) is infinite in the case c <= a+b.

        (`errno` 6)
            Overflow occurred in a subcalculation of 2_F_1(a,b;c;x). The
            result may or may not be infinite.

        (`errno` 9)
            An internal calculation has resulted in an undefined result.

        (`errno` 11)
            On entry, `a` does not satisfy abs(`a`) <= arbnd =
            *<value>*.

        (`errno` 21)
            On entry, `b` does not satisfy abs(`b`) <= arbnd =
            *<value>*.

        (`errno` 31)
            On entry, `c` does not satisfy abs(`c`) <= arbnd =
            *<value>*.

        (`errno` 32)
            On entry, `c` = *<value>*.

            2_F_1(a,b;c;x) is undefined when c is zero or a negative
            integer.

        (`errno` 41)
            On entry, `x` does not satisfy abs(`x`) <= arbnd =
            *<value>*.

        (`errno` 42)
            On entry, `x` = *<value>*.

            In general, 2_F_1(a,b;c;x) is not real valued when x > 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of 2_F_1(a,b;c;x).
            The returned value may be inaccurate.

        (`errno` 2)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.

        (`errno` 5)
            On completion, overflow occurred in the evaluation of
            2_F_1(a,b;c;x).
    """

    from ..base import _nag_s_consts

    excs = []
    fun_name = 'naginterfaces.base.specfun.hyperg_gauss_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s22beft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, c.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, f21.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _c = utils._EngineFloat64ScalarType(
        c, {'fun_name': fun_name, 'entity': 'c'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _f21 = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _c,
        _x,
        _f21,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_s_consts.S22BF_IERR_UFLOW and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underflow occurred during the evaluation of 2_F_1(a,b;c;x).',
                    'The returned value may be inaccurate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_WARN_RES and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate indicates',
                    'some precision may have been lost.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_RES and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate indicates',
                    'no accuracy can be guaranteed.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_X_EQ1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, c = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'a+b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    '2_F_1(a,b;c;1) is infinite in the case c <= a+b.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_OFLOW_SAFE and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On completion, overflow occurred in the evaluation of',
                    '2_F_1(a,b;c;x).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_OFLOW_SUB and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred in a subcalculation of 2_F_1(a,b;c;x).',
                    'The result may or may not be infinite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_NAN and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal calculation has resulted in an undefined result.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BE_IERR_A_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a does not satisfy abs(a) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BE_IERR_B_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b does not satisfy abs(b) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BE_IERR_C_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, c does not satisfy abs(c) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BE_IERR_C_NEGI and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, c = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '2_F_1(a,b;c;x) is undefined when c is zero or a negative integer.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BE_IERR_X_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x does not satisfy abs(x) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BE_IERR_X_GT1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'In general, 2_F_1(a,b;c;x) is not real valued when x > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f21_py = utils._EngineFloat64ScalarType.to_py(_f21)
        return_data = _f21_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['f21'],
            )(
                _f21_py,
            )
    _err_class.manage()
    return return_data

def hyperg_gauss_real_scaled(ani, adr, bni, bdr, cni, cdr, x):
    u"""
    Real Gauss hypergeometric function 2_F_1(a,b;c;x) in scaled form.

    For full information please refer to the NAG Library document for
    s22bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22bff.html

    Parameters
    ----------
    ani : float
        a_i, the nearest integer to a, satisfying a_i = a-a_r.

    adr : float
        a_r, the signed decimal remainder satisfying a_r = a-a_i and
        |a_r| <= 0.5.

    bni : float
        b_i, the nearest integer to b, satisfying b_i = b-b_r.

    bdr : float
        b_r, the signed decimal remainder satisfying b_r = b-b_i and
        |b_r| <= 0.5.

    cni : float
        c_i, the nearest integer to c, satisfying c_i = c-c_r.

    cdr : float
        c_r, the signed decimal remainder satisfying c_r = c-c_i and
        |c_r| <= 0.5.

    x : float
        The argument x.

    Returns
    -------
    frf : float
        f_fr, the scaled real component of the solution satisfying f_fr
        = 2_F_1(a,b;c;x)*2^(-f_sc), i.e., 2_F_1(a,b;c;x) =
        f_fr*2^(f_sc). See [Further Comments] for the behaviour of f_fr
        when a finite or non-finite answer is returned.

    scf : int
        f_sc, the scaling power of two, satisfying f_sc =
        log2(2_F_1(a,b;c;x)/f_fr), i.e., 2_F_1(a,b;c;x) = f_fr*2^(f_sc).
        See [Further Comments] for the behaviour of f_sc when a
        non-finite answer is returned.

    Raises
    ------
    NagValueError
        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 4)
            On entry, `x` = *<value>*, c = *<value>*, a+b = *<value>*.

            2_F_1(a,b;c;1) is infinite in the case c <= a+b.

        (`errno` 6)
            Overflow occurred in a subcalculation of 2_F_1(a,b;c;x). The
            answer may be completely incorrect.

        (`errno` 9)
            An internal calculation has resulted in an undefined result.

        (`errno` 11)
            On entry, `ani` does not satisfy abs(`ani`) <= arbnd =
            *<value>*.

        (`errno` 13)
            `ani` is non-integral.

            On entry, `ani` = *<value>*.

            Constraint: `ani` = floor(`ani`).

        (`errno` 21)
            On entry, `adr` does not satisfy abs(`adr`) <= 0.5.

        (`errno` 31)
            On entry, `bni` does not satisfy abs(`bni`) <= arbnd =
            *<value>*.

        (`errno` 33)
            `bni` is non-integral.

            On entry, `bni` = *<value>*.

            Constraint: `bni` = floor(`bni`).

        (`errno` 41)
            On entry, `bdr` does not satisfy abs(`bdr`) <= 0.5.

        (`errno` 51)
            On entry, `cni` does not satisfy abs(`cni`) <= arbnd =
            *<value>*.

        (`errno` 52)
            On entry, c = `cni`+`cdr` = *<value>*.

            2_F_1(a,b;c;x) is undefined when c is zero or a negative
            integer.

        (`errno` 53)
            `cni` is non-integral.

            On entry, `cni` = *<value>*.

            Constraint: `cni` = floor(`cni`).

        (`errno` 61)
            On entry, `cdr` does not satisfy abs(`cdr`) <= 0.5.

        (`errno` 71)
            On entry, `x` does not satisfy abs(`x`) <= arbnd =
            *<value>*.

        (`errno` 72)
            On entry, `x` = *<value>*.

            In general, 2_F_1(a,b;c;x) is not real valued when x > 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of 2_F_1(a,b;c;x).
            The returned value may be inaccurate.

        (`errno` 2)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.

        (`errno` 5)
            On completion, overflow occurred in the evaluation of
            2_F_1(a,b;c;x).
    """

    from ..base import _nag_s_consts

    excs = []
    fun_name = 'naginterfaces.base.specfun.hyperg_gauss_real_scaled'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s22bfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, ani.
        utils._BYREF_FLOAT64, # Input, adr.
        utils._BYREF_FLOAT64, # Input, bni.
        utils._BYREF_FLOAT64, # Input, bdr.
        utils._BYREF_FLOAT64, # Input, cni.
        utils._BYREF_FLOAT64, # Input, cdr.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, frf.
        utils._BYREF_INT, # Output, scf.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ani = utils._EngineFloat64ScalarType(
        ani, {'fun_name': fun_name, 'entity': 'ani'},
    ).to_ctypes()
    _adr = utils._EngineFloat64ScalarType(
        adr, {'fun_name': fun_name, 'entity': 'adr'},
    ).to_ctypes()
    _bni = utils._EngineFloat64ScalarType(
        bni, {'fun_name': fun_name, 'entity': 'bni'},
    ).to_ctypes()
    _bdr = utils._EngineFloat64ScalarType(
        bdr, {'fun_name': fun_name, 'entity': 'bdr'},
    ).to_ctypes()
    _cni = utils._EngineFloat64ScalarType(
        cni, {'fun_name': fun_name, 'entity': 'cni'},
    ).to_ctypes()
    _cdr = utils._EngineFloat64ScalarType(
        cdr, {'fun_name': fun_name, 'entity': 'cdr'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _frf = utils._EngineFloat64ScalarType.empty_ctypes()
    _scf = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ani,
        _adr,
        _bni,
        _bdr,
        _cni,
        _cdr,
        _x,
        _frf,
        _scf,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_s_consts.S22BF_IERR_UFLOW and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underflow occurred during the evaluation of 2_F_1(a,b;c;x).',
                    'The returned value may be inaccurate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_WARN_RES and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate indicates',
                    'some precision may have been lost.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_RES and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All approximations have completed, and the final residual estimate indicates',
                    'no accuracy can be guaranteed.',
                    'Relative residual = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_X_EQ1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, c = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'a+b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    '2_F_1(a,b;c;1) is infinite in the case c <= a+b.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_OFLOW_SAFE and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On completion, overflow occurred in the evaluation of',
                    '2_F_1(a,b;c;x).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_OFLOW_SUB and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred in a subcalculation of 2_F_1(a,b;c;x).',
                    'The answer may be completely incorrect.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_NAN and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal calculation has resulted in an undefined result.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_ANI_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ani does not satisfy abs(ani) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_ANI_NOTINT and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ani is non-integral.',
                    'On entry, ani = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ani = floor(ani).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_ADR and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, adr does not satisfy abs(adr) <= 0.5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_BNI_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, bni does not satisfy abs(bni) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_BNI_NOTINT and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'bni is non-integral.',
                    'On entry, bni = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: bni = floor(bni).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_BDR and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, bdr does not satisfy abs(bdr) <= 0.5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_CNI_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cni does not satisfy abs(cni) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_C_NEGI and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, c = cni+cdr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '2_F_1(a,b;c;x) is undefined when c is zero or a negative integer.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_CNI_NOTINT and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'cni is non-integral.',
                    'On entry, cni = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cni = floor(cni).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_CDR and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cdr does not satisfy abs(cdr) <= 0.5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_X_ARBND and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x does not satisfy abs(x) <= arbnd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_s_consts.S22BF_IERR_X_GT1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'In general, 2_F_1(a,b;c;x) is not real valued when x > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _frf_py = utils._EngineFloat64ScalarType.to_py(_frf)
        _scf_py = utils._EngineIntScalarType.to_py(_scf)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['frf', 'scf'],
        )(
            _frf_py,
            _scf_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def mathieu_ang_periodic_real(ordval, q, parity, mode, n, x, f, f_deriv):
    u"""
    Calculates values of real periodic angular Mathieu functions.

    For full information please refer to the NAG Library document for
    s22ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22caf.html

    Parameters
    ----------
    ordval : int
        m, the order number of the Mathieu function to be computed.

    q : float
        q, the Mathieu function parameter.

    parity : int
        Specifies whether to compute even or odd Mathieu function.

        `parity` = 0

          Compute even Mathieu function, ce_m(x,q).

        `parity` = 1

          Compute odd Mathieu function, se_m(x,q).

    mode : int
        Specifies whether the Mathieu function or its derivative is
        required.

        `mode` = 0

          Compute Mathieu function values.

        `mode` = 1

          Compute derivative values of Mathieu function.

        `mode` = 2

          Compute both Mathieu function and derivative values.

        `mode` = 3

          Compute neither Mathieu functions nor derivative values,
          returns only `a` (the characteristic value).

    n : int
        n, the number of values to compute.

    x : None or float, shape (`n`)
        Note: when `mode` != 3 this argument must not be None.

        The values of x at which to compute Mathieu function or
        derivative values.

    f : None or float, shape (`n`), modified in place
        On exit: if `mode` = 0 or 2, the Mathieu function values
        ce_m(x,q) or se_m(x,q). If `mode` = 1 or 3, `f` is not used.

    f_deriv : None or float, shape (`n`), modified in place
        On exit: if `mode` = 1 or 2, the Mathieu function derivative
        values ce'_m(x,q) or se'_m(x,q). If `mode` = 0 or 3, `f_deriv`
        is not used.

    Returns
    -------
    a : float
        a, the characteristic value for the Mathieu function.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ordval` = *<value>* and `parity` = *<value>*.

            Constraint: if `parity` = 0, `ordval` >= 0 or if `parity` =
            1, `ordval` >= 1.

        (`errno` 2)
            On entry, `parity` = *<value>*.

            Constraint: `parity` = 0 or 1.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 0, 1, 2 or 3.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.mathieu_ang_periodic_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s22caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ordval.
        utils._BYREF_FLOAT64, # Input, q.
        utils._BYREF_INT, # Input, parity.
        utils._BYREF_INT, # Input, mode.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f_deriv[n].
        utils._BYREF_FLOAT64, # Output, a.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ordval = utils._EngineIntScalarType(
        ordval, {'fun_name': fun_name, 'entity': 'ordval'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _parity = utils._EngineIntScalarType(
        parity, {'fun_name': fun_name, 'entity': 'parity'},
    ).to_ctypes()
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _mode_py = utils._EngineIntScalarType.to_py(_mode)
    if _mode_py != 3:
        if x is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'x'},
                'must not be None when mode != 3',
            )
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if x is None:
        _x = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _x = utils._EngineFloat64ArrayType(
            x, {'fun_name': fun_name, 'entity': 'x'},
            exp_shape=(_n_py,),
            check_contents=_mode_py != 3,
            allow_none=True,
        ).to_ctypes(can_reference_contents=_mode_py != 3)
    if _mode_py in (0, 2):
        if f is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'f'},
                'must not be None when mode in (0, 2)',
            )
    if f is None:
        _f = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _f_cls = utils._EngineFloat64ArrayType(
            f, {'fun_name': fun_name, 'entity': 'f'},
            exp_shape=(_n_py,),
            check_contents=False,
            allow_none=True,
        )
        _f = _f_cls.to_ctypes(can_reference_contents=False)
    if _mode_py in (1, 2):
        if f_deriv is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'f_deriv'},
                'must not be None when mode in (1, 2)',
            )
    if f_deriv is None:
        _f_deriv = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _f_deriv_cls = utils._EngineFloat64ArrayType(
            f_deriv, {'fun_name': fun_name, 'entity': 'f_deriv'},
            exp_shape=(_n_py,),
            check_contents=False,
            allow_none=True,
        )
        _f_deriv = _f_deriv_cls.to_ctypes(can_reference_contents=False)
    _a = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ordval,
        _q,
        _parity,
        _mode,
        _n,
        _x,
        _f,
        _f_deriv,
        _a,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ordval = {:s} and parity = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if parity = 0, ordval >= 0 or',
                    'if parity = 1, ordval >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, parity = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: parity = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1003:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode = 0, 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1004:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if f is not None:
            _f_cls.refresh_py(_f)
        if f_deriv is not None:
            _f_deriv_cls.refresh_py(_f_deriv)
        _a_py = utils._EngineFloat64ScalarType.to_py(_a)
        return_data = _a_py
    _err_class.manage()
    return return_data

def opt_bsm_price(calput, m, n, x, s, t, sigma, r, q, p):
    u"""
    Black--Scholes--Merton option pricing formula.

    For full information please refer to the NAG Library document for
    s30aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30aaf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

    See Also
    --------
    :meth:`naginterfaces.library.examples.roots.contfn_brent_interval_ex.main`
    :meth:`naginterfaces.library.examples.specfun.opt_bsm_price_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_bsm_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_bsm_greeks(calput, m, n, x, s, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma):
    u"""
    Black--Scholes--Merton option pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30abf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    delta : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `delta` contains
        the sensitivity, dP/dS, of the option price to change in the
        price of the underlying asset.

    gamma : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `gamma` contains
        the sensitivity, d^2P/dS^2, of `delta` to change in the price of
        the underlying asset.

    vega : float, shape (`m`, `n`), modified in place
        On exit: `vega`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the volatility of the underlying asset, i.e., dP_{ij}/d
        sigma, for i = 1,2,...,`m` and j = 1,2,...,`n`.

    theta : float, shape (`m`, `n`), modified in place
        On exit: `theta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in time, i.e., -dP_{ij}/dT, for i = 1,2,...,`m` and j =
        1,2,...,`n`, where b = r-q.

    rho : float, shape (`m`, `n`), modified in place
        On exit: `rho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual risk-free interest rate, i.e., -dP_{ij}/dr, for i
        = 1,2,...,`m` and j = 1,2,...,`n`.

    crho : float, shape (`m`, `n`), modified in place
        On exit: `crho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual cost of carry rate, i.e., -dP_{ij}/db, for i =
        1,2,...,`m` and j = 1,2,...,`n`, where b = r-q.

    vanna : float, shape (`m`, `n`), modified in place
        On exit: `vanna`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the asset price, i.e., -d
        Delta_{ij}/dT = -d^2P_{ij}/(dSd sigma), for i = 1,2,...,`m` and
        j = 1,2,...,`n`.

    charm : float, shape (`m`, `n`), modified in place
        On exit: `charm`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT),
        for i = 1,2,...,`m` and j = 1,2,...,`n`.

    speed : float, shape (`m`, `n`), modified in place
        On exit: `speed`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the price of the underlying asset, i.e., -d
        Gamma_{ij}/dS = -d^3P_{ij}/dS^3, for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    colour : float, shape (`m`, `n`), modified in place
        On exit: `colour`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the time, i.e., -d Gamma_{ij}/dT =
        -d^3P_{ij}/(dSdT), for i = 1,2,...,`m` and j = 1,2,...,`n`.

    zomma : float, shape (`m`, `n`), modified in place
        On exit: `zomma`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the volatility of the underlying asset, i.e., -d
        Gamma_{ij}/d sigma = -d^3P_{ij}/(dS^2d sigma), for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vomma : float, shape (`m`, `n`), modified in place
        On exit: `vomma`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the underlying asset, i.e., -d
        Delta_{ij}/d sigma = -d^2P_{ij}/d sigma^2, for i = 1,2,...,`m`
        and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_bsm_greeks'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'), (delta, 'delta'), (gamma, 'gamma'), (vega, 'vega'), (theta, 'theta'), (rho, 'rho'), (crho, 'crho'), (vanna, 'vanna'), (charm, 'charm'), (speed, 'speed'), (colour, 'colour'), (zomma, 'zomma'), (vomma, 'vomma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = {:s}.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_shape(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _vega_cls = utils._EngineFloat64ArrayType(
        vega, {'fun_name': fun_name, 'entity': 'vega'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vega = _vega_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _crho_cls = utils._EngineFloat64ArrayType(
        crho, {'fun_name': fun_name, 'entity': 'crho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _crho = _crho_cls.to_ctypes(can_reference_contents=False)
    _vanna_cls = utils._EngineFloat64ArrayType(
        vanna, {'fun_name': fun_name, 'entity': 'vanna'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vanna = _vanna_cls.to_ctypes(can_reference_contents=False)
    _charm_cls = utils._EngineFloat64ArrayType(
        charm, {'fun_name': fun_name, 'entity': 'charm'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _charm = _charm_cls.to_ctypes(can_reference_contents=False)
    _speed_cls = utils._EngineFloat64ArrayType(
        speed, {'fun_name': fun_name, 'entity': 'speed'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _speed = _speed_cls.to_ctypes(can_reference_contents=False)
    _colour_cls = utils._EngineFloat64ArrayType(
        colour, {'fun_name': fun_name, 'entity': 'colour'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _colour = _colour_cls.to_ctypes(can_reference_contents=False)
    _zomma_cls = utils._EngineFloat64ArrayType(
        zomma, {'fun_name': fun_name, 'entity': 'zomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _zomma = _zomma_cls.to_ctypes(can_reference_contents=False)
    _vomma_cls = utils._EngineFloat64ArrayType(
        vomma, {'fun_name': fun_name, 'entity': 'vomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vomma = _vomma_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _vega_cls.refresh_py(_vega)
        _theta_cls.refresh_py(_theta)
        _rho_cls.refresh_py(_rho)
        _crho_cls.refresh_py(_crho)
        _vanna_cls.refresh_py(_vanna)
        _charm_cls.refresh_py(_charm)
        _speed_cls.refresh_py(_speed)
        _colour_cls.refresh_py(_colour)
        _zomma_cls.refresh_py(_zomma)
        _vomma_cls.refresh_py(_vomma)
        return_data = None
    _err_class.manage()
    return return_data

def opt_lookback_fls_price(calput, m, n, sm, s, t, sigma, r, q, p):
    u"""
    Floating-strike lookback option pricing formula in the
    Black-Scholes-Merton model.

    For full information please refer to the NAG Library document for
    s30ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30baf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of minimum or maximum prices to be used.

    n : int
        The number of times to expiry to be used.

    sm : float, shape (`m`)
        `sm`[i-1] must contain S_min(i), the ith minimum observed price
        of the underlying asset when `calput` = 'C', or S_max(i), the
        maximum observed price when `calput` = 'P', for i = 1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the minimum or maximum observed price S_min(i) or
        S_max(i) at expiry `t`_j for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry with a put option, `sm`[*<value>*] = *<value>*.

            Constraint: for put options, `sm`[i] >= *<value>* for all i.

        (`errno` 4)
            On entry with a call option, `sm`[*<value>*] = *<value>*.

            Constraint: for call options, `sm`[i] <= *<value>* for all
            i.

        (`errno` 4)
            On entry, `sm`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `sm`[i] <= *<value>* for all i.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>* for all i.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_lookback_fls_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, sm[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _sm = utils._EngineFloat64ArrayType(
        sm, {'fun_name': fun_name, 'entity': 'sm'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _sm,
        _s,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry with a put option,',
                    'sm[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for put options, sm[i] >= {:s} for all i.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 114:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry with a call option,',
                    'sm[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for call options, sm[i] <= {:s} for all i.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sm[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= sm[i] <= {:s} for'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: t[i] >= {:s} for all i.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_lookback_fls_greeks(calput, m, n, sm, s, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma):
    u"""
    Floating-strike lookback option pricing formula with Greeks in the
    Black-Scholes-Merton model.

    For full information please refer to the NAG Library document for
    s30bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30bbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of minimum or maximum prices to be used.

    n : int
        The number of times to expiry to be used.

    sm : float, shape (`m`)
        `sm`[i-1] must contain S_min(i), the ith minimum observed price
        of the underlying asset when `calput` = 'C', or S_max(i), the
        maximum observed price when `calput` = 'P', for i = 1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        The annual risk-free interest rate, r, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        The annual continuous yield rate. Note that a rate of 8% should
        be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the minimum or maximum observed price S_min(i) or
        S_max(i) at expiry `t`_j for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    delta : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `delta` contains
        the sensitivity, dP/dS, of the option price to change in the
        price of the underlying asset.

    gamma : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `gamma` contains
        the sensitivity, d^2P/dS^2, of `delta` to change in the price of
        the underlying asset.

    vega : float, shape (`m`, `n`), modified in place
        On exit: `vega`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the volatility of the underlying asset, i.e., dP_{ij}/d
        sigma, for i = 1,2,...,`m` and j = 1,2,...,`n`.

    theta : float, shape (`m`, `n`), modified in place
        On exit: `theta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in time, i.e., -dP_{ij}/dT, for i = 1,2,...,`m` and j =
        1,2,...,`n`, where b = r-q.

    rho : float, shape (`m`, `n`), modified in place
        On exit: `rho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual risk-free interest rate, i.e., -dP_{ij}/dr, for i
        = 1,2,...,`m` and j = 1,2,...,`n`.

    crho : float, shape (`m`, `n`), modified in place
        On exit: `crho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual cost of carry rate, i.e., -dP_{ij}/db, for i =
        1,2,...,`m` and j = 1,2,...,`n`, where b = r-q.

    vanna : float, shape (`m`, `n`), modified in place
        On exit: `vanna`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the asset price, i.e., -d
        Delta_{ij}/dT = -d^2P_{ij}/(dSd sigma), for i = 1,2,...,`m` and
        j = 1,2,...,`n`.

    charm : float, shape (`m`, `n`), modified in place
        On exit: `charm`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT),
        for i = 1,2,...,`m` and j = 1,2,...,`n`.

    speed : float, shape (`m`, `n`), modified in place
        On exit: `speed`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the price of the underlying asset, i.e., -d
        Gamma_{ij}/dS = -d^3P_{ij}/dS^3, for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    colour : float, shape (`m`, `n`), modified in place
        On exit: `colour`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the time, i.e., -d Gamma_{ij}/dT =
        -d^3P_{ij}/(dSdT), for i = 1,2,...,`m` and j = 1,2,...,`n`.

    zomma : float, shape (`m`, `n`), modified in place
        On exit: `zomma`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the volatility of the underlying asset, i.e., -d
        Gamma_{ij}/d sigma = -d^3P_{ij}/(dS^2d sigma), for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vomma : float, shape (`m`, `n`), modified in place
        On exit: `vomma`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the underlying asset, i.e., -d
        Delta_{ij}/d sigma = -d^2P_{ij}/d sigma^2, for i = 1,2,...,`m`
        and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry with a put option, `sm`[*<value>*] = *<value>*.

            Constraint: for put options, `sm`[i] >= *<value>* for all i.

        (`errno` 4)
            On entry with a call option, `sm`[*<value>*] = *<value>*.

            Constraint: for call options, `sm`[i] <= *<value>* for all
            i.

        (`errno` 4)
            On entry, `sm`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `sm`[i] <= *<value>* for all i.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>* for all i.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 12)
            On entry, `r` = *<value>* and `q` = *<value>*.

            Constraint: abs(`r`-`q`) > 10*eps*max(abs(`r`),1), where eps
            is the machine precision.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_lookback_fls_greeks'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, sm[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'), (delta, 'delta'), (gamma, 'gamma'), (vega, 'vega'), (theta, 'theta'), (rho, 'rho'), (crho, 'crho'), (vanna, 'vanna'), (charm, 'charm'), (speed, 'speed'), (colour, 'colour'), (zomma, 'zomma'), (vomma, 'vomma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _sm = utils._EngineFloat64ArrayType(
        sm, {'fun_name': fun_name, 'entity': 'sm'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_shape(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _vega_cls = utils._EngineFloat64ArrayType(
        vega, {'fun_name': fun_name, 'entity': 'vega'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vega = _vega_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _crho_cls = utils._EngineFloat64ArrayType(
        crho, {'fun_name': fun_name, 'entity': 'crho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _crho = _crho_cls.to_ctypes(can_reference_contents=False)
    _vanna_cls = utils._EngineFloat64ArrayType(
        vanna, {'fun_name': fun_name, 'entity': 'vanna'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vanna = _vanna_cls.to_ctypes(can_reference_contents=False)
    _charm_cls = utils._EngineFloat64ArrayType(
        charm, {'fun_name': fun_name, 'entity': 'charm'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _charm = _charm_cls.to_ctypes(can_reference_contents=False)
    _speed_cls = utils._EngineFloat64ArrayType(
        speed, {'fun_name': fun_name, 'entity': 'speed'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _speed = _speed_cls.to_ctypes(can_reference_contents=False)
    _colour_cls = utils._EngineFloat64ArrayType(
        colour, {'fun_name': fun_name, 'entity': 'colour'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _colour = _colour_cls.to_ctypes(can_reference_contents=False)
    _zomma_cls = utils._EngineFloat64ArrayType(
        zomma, {'fun_name': fun_name, 'entity': 'zomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _zomma = _zomma_cls.to_ctypes(can_reference_contents=False)
    _vomma_cls = utils._EngineFloat64ArrayType(
        vomma, {'fun_name': fun_name, 'entity': 'vomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vomma = _vomma_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _sm,
        _s,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry with a put option,',
                    'sm[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for put options, sm[i] >= {:s} for all i.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 114:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry with a call option,',
                    'sm[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for call options, sm[i] <= {:s} for all i.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sm[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= sm[i] <= {:s} for'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: t[i] >= {:s} for all i.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s} and q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: abs(r-q) > 10*eps*max(abs(r),1),',
                    'where eps is the machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _vega_cls.refresh_py(_vega)
        _theta_cls.refresh_py(_theta)
        _rho_cls.refresh_py(_rho)
        _crho_cls.refresh_py(_crho)
        _vanna_cls.refresh_py(_vanna)
        _charm_cls.refresh_py(_charm)
        _speed_cls.refresh_py(_speed)
        _colour_cls.refresh_py(_colour)
        _zomma_cls.refresh_py(_zomma)
        _vomma_cls.refresh_py(_vomma)
        return_data = None
    _err_class.manage()
    return return_data

def opt_binary_con_price(calput, m, n, x, s, k, t, sigma, r, q, p):
    u"""
    Binary option, cash-or-nothing pricing formula.

    For full information please refer to the NAG Library document for
    s30ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30caf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    k : float
        The amount, K, to be paid at expiration if the option is
        in-the-money, i.e., if `s` > `x`[i-1] when `calput` = 'C', or if
        `s` < `x`[i-1] when `calput` = 'P', for i = 1,2,...,m.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.0.

        (`errno` 7)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 8)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 9)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 10)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_binary_con_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._BYREF_FLOAT64, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _k = utils._EngineFloat64ScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _k,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_binary_con_greeks(calput, m, n, x, s, k, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma):
    u"""
    Binary option, cash-or-nothing pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30cbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    k : float
        The amount, K, to be paid at expiration if the option is
        in-the-money, i.e., if `s` > `x`[i-1] when `calput` = 'C', or if
        `s` < `x`[i-1] when `calput` = 'P', for i = 1,2,...,m.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    delta : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `delta` contains
        the sensitivity, dP/dS, of the option price to change in the
        price of the underlying asset.

    gamma : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `gamma` contains
        the sensitivity, d^2P/dS^2, of `delta` to change in the price of
        the underlying asset.

    vega : float, shape (`m`, `n`), modified in place
        On exit: `vega`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the volatility of the underlying asset, i.e., dP_{ij}/d
        sigma, for i = 1,2,...,`m` and j = 1,2,...,`n`.

    theta : float, shape (`m`, `n`), modified in place
        On exit: `theta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in time, i.e., -dP_{ij}/dT, for i = 1,2,...,`m` and j =
        1,2,...,`n`, where b = r-q.

    rho : float, shape (`m`, `n`), modified in place
        On exit: `rho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual risk-free interest rate, i.e., -dP_{ij}/dr, for i
        = 1,2,...,`m` and j = 1,2,...,`n`.

    crho : float, shape (`m`, `n`), modified in place
        On exit: `crho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual cost of carry rate, i.e., -dP_{ij}/db, for i =
        1,2,...,`m` and j = 1,2,...,`n`, where b = r-q.

    vanna : float, shape (`m`, `n`), modified in place
        On exit: `vanna`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the asset price, i.e., -d
        Delta_{ij}/dT = -d^2P_{ij}/(dSd sigma), for i = 1,2,...,`m` and
        j = 1,2,...,`n`.

    charm : float, shape (`m`, `n`), modified in place
        On exit: `charm`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT),
        for i = 1,2,...,`m` and j = 1,2,...,`n`.

    speed : float, shape (`m`, `n`), modified in place
        On exit: `speed`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the price of the underlying asset, i.e., -d
        Gamma_{ij}/dS = -d^3P_{ij}/dS^3, for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    colour : float, shape (`m`, `n`), modified in place
        On exit: `colour`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the time, i.e., -d Gamma_{ij}/dT =
        -d^3P_{ij}/(dSdT), for i = 1,2,...,`m` and j = 1,2,...,`n`.

    zomma : float, shape (`m`, `n`), modified in place
        On exit: `zomma`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the volatility of the underlying asset, i.e., -d
        Gamma_{ij}/d sigma = -d^3P_{ij}/(dS^2d sigma), for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vomma : float, shape (`m`, `n`), modified in place
        On exit: `vomma`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the underlying asset, i.e., -d
        Delta_{ij}/d sigma = -d^2P_{ij}/d sigma^2, for i = 1,2,...,`m`
        and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.0.

        (`errno` 7)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 8)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 9)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 10)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_binary_con_greeks'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._BYREF_FLOAT64, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'), (delta, 'delta'), (gamma, 'gamma'), (vega, 'vega'), (theta, 'theta'), (rho, 'rho'), (crho, 'crho'), (vanna, 'vanna'), (charm, 'charm'), (speed, 'speed'), (colour, 'colour'), (zomma, 'zomma'), (vomma, 'vomma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _k = utils._EngineFloat64ScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_shape(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _vega_cls = utils._EngineFloat64ArrayType(
        vega, {'fun_name': fun_name, 'entity': 'vega'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vega = _vega_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _crho_cls = utils._EngineFloat64ArrayType(
        crho, {'fun_name': fun_name, 'entity': 'crho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _crho = _crho_cls.to_ctypes(can_reference_contents=False)
    _vanna_cls = utils._EngineFloat64ArrayType(
        vanna, {'fun_name': fun_name, 'entity': 'vanna'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vanna = _vanna_cls.to_ctypes(can_reference_contents=False)
    _charm_cls = utils._EngineFloat64ArrayType(
        charm, {'fun_name': fun_name, 'entity': 'charm'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _charm = _charm_cls.to_ctypes(can_reference_contents=False)
    _speed_cls = utils._EngineFloat64ArrayType(
        speed, {'fun_name': fun_name, 'entity': 'speed'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _speed = _speed_cls.to_ctypes(can_reference_contents=False)
    _colour_cls = utils._EngineFloat64ArrayType(
        colour, {'fun_name': fun_name, 'entity': 'colour'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _colour = _colour_cls.to_ctypes(can_reference_contents=False)
    _zomma_cls = utils._EngineFloat64ArrayType(
        zomma, {'fun_name': fun_name, 'entity': 'zomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _zomma = _zomma_cls.to_ctypes(can_reference_contents=False)
    _vomma_cls = utils._EngineFloat64ArrayType(
        vomma, {'fun_name': fun_name, 'entity': 'vomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vomma = _vomma_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _k,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _vega_cls.refresh_py(_vega)
        _theta_cls.refresh_py(_theta)
        _rho_cls.refresh_py(_rho)
        _crho_cls.refresh_py(_crho)
        _vanna_cls.refresh_py(_vanna)
        _charm_cls.refresh_py(_charm)
        _speed_cls.refresh_py(_speed)
        _colour_cls.refresh_py(_colour)
        _zomma_cls.refresh_py(_zomma)
        _vomma_cls.refresh_py(_vomma)
        return_data = None
    _err_class.manage()
    return return_data

def opt_binary_aon_price(calput, m, n, x, s, t, sigma, r, q, p):
    u"""
    Binary option, asset-or-nothing pricing formula.

    For full information please refer to the NAG Library document for
    s30cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30ccf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_binary_aon_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_binary_aon_greeks(calput, m, n, x, s, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma):
    u"""
    Binary option, asset-or-nothing pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30cdf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    delta : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `delta` contains
        the sensitivity, dP/dS, of the option price to change in the
        price of the underlying asset.

    gamma : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `gamma` contains
        the sensitivity, d^2P/dS^2, of `delta` to change in the price of
        the underlying asset.

    vega : float, shape (`m`, `n`), modified in place
        On exit: `vega`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the volatility of the underlying asset, i.e., dP_{ij}/d
        sigma, for i = 1,2,...,`m` and j = 1,2,...,`n`.

    theta : float, shape (`m`, `n`), modified in place
        On exit: `theta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in time, i.e., -dP_{ij}/dT, for i = 1,2,...,`m` and j =
        1,2,...,`n`, where b = r-q.

    rho : float, shape (`m`, `n`), modified in place
        On exit: `rho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual risk-free interest rate, i.e., -dP_{ij}/dr, for i
        = 1,2,...,`m` and j = 1,2,...,`n`.

    crho : float, shape (`m`, `n`), modified in place
        On exit: `crho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual cost of carry rate, i.e., -dP_{ij}/db, for i =
        1,2,...,`m` and j = 1,2,...,`n`, where b = r-q.

    vanna : float, shape (`m`, `n`), modified in place
        On exit: `vanna`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the asset price, i.e., -d
        Delta_{ij}/dT = -d^2P_{ij}/(dSd sigma), for i = 1,2,...,`m` and
        j = 1,2,...,`n`.

    charm : float, shape (`m`, `n`), modified in place
        On exit: `charm`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT),
        for i = 1,2,...,`m` and j = 1,2,...,`n`.

    speed : float, shape (`m`, `n`), modified in place
        On exit: `speed`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the price of the underlying asset, i.e., -d
        Gamma_{ij}/dS = -d^3P_{ij}/dS^3, for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    colour : float, shape (`m`, `n`), modified in place
        On exit: `colour`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the time, i.e., -d Gamma_{ij}/dT =
        -d^3P_{ij}/(dSdT), for i = 1,2,...,`m` and j = 1,2,...,`n`.

    zomma : float, shape (`m`, `n`), modified in place
        On exit: `zomma`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the volatility of the underlying asset, i.e., -d
        Gamma_{ij}/d sigma = -d^3P_{ij}/(dS^2d sigma), for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vomma : float, shape (`m`, `n`), modified in place
        On exit: `vomma`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the underlying asset, i.e., -d
        Delta_{ij}/d sigma = -d^2P_{ij}/d sigma^2, for i = 1,2,...,`m`
        and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_binary_aon_greeks'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30cdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'), (delta, 'delta'), (gamma, 'gamma'), (vega, 'vega'), (theta, 'theta'), (rho, 'rho'), (crho, 'crho'), (vanna, 'vanna'), (charm, 'charm'), (speed, 'speed'), (colour, 'colour'), (zomma, 'zomma'), (vomma, 'vomma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_shape(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _vega_cls = utils._EngineFloat64ArrayType(
        vega, {'fun_name': fun_name, 'entity': 'vega'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vega = _vega_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _crho_cls = utils._EngineFloat64ArrayType(
        crho, {'fun_name': fun_name, 'entity': 'crho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _crho = _crho_cls.to_ctypes(can_reference_contents=False)
    _vanna_cls = utils._EngineFloat64ArrayType(
        vanna, {'fun_name': fun_name, 'entity': 'vanna'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vanna = _vanna_cls.to_ctypes(can_reference_contents=False)
    _charm_cls = utils._EngineFloat64ArrayType(
        charm, {'fun_name': fun_name, 'entity': 'charm'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _charm = _charm_cls.to_ctypes(can_reference_contents=False)
    _speed_cls = utils._EngineFloat64ArrayType(
        speed, {'fun_name': fun_name, 'entity': 'speed'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _speed = _speed_cls.to_ctypes(can_reference_contents=False)
    _colour_cls = utils._EngineFloat64ArrayType(
        colour, {'fun_name': fun_name, 'entity': 'colour'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _colour = _colour_cls.to_ctypes(can_reference_contents=False)
    _zomma_cls = utils._EngineFloat64ArrayType(
        zomma, {'fun_name': fun_name, 'entity': 'zomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _zomma = _zomma_cls.to_ctypes(can_reference_contents=False)
    _vomma_cls = utils._EngineFloat64ArrayType(
        vomma, {'fun_name': fun_name, 'entity': 'vomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vomma = _vomma_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _vega_cls.refresh_py(_vega)
        _theta_cls.refresh_py(_theta)
        _rho_cls.refresh_py(_rho)
        _crho_cls.refresh_py(_crho)
        _vanna_cls.refresh_py(_vanna)
        _charm_cls.refresh_py(_charm)
        _speed_cls.refresh_py(_speed)
        _colour_cls.refresh_py(_colour)
        _zomma_cls.refresh_py(_zomma)
        _vomma_cls.refresh_py(_vomma)
        return_data = None
    _err_class.manage()
    return return_data

def opt_barrier_std_price(calput, btype, m, n, x, s, h, k, t, sigma, r, q, p):
    u"""
    Standard barrier option pricing formula.

    For full information please refer to the NAG Library document for
    s30fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30faf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    btype : str, length 2
        Indicates the barrier type as In or Out and its relation to the
        price of the underlying asset as Up or Down.

        `btype` = 'DI'

          Down-and-In.

        `btype` = 'DO'

          Down-and-Out.

        `btype` = 'UI'

          Up-and-In.

        `btype` = 'UO'

          Up-and-Out.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    h : float
        The barrier price.

    k : float
        The value of a possible cash rebate to be paid if the option has
        not been knocked in (or out) before expiration.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `btype` = *<value>*.

            Constraint: `btype` = 'DI', 'DO', 'UI' or 'UO'.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 5)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 6)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 7)
            On entry, `h` = *<value>*.

            Constraint: `h` >= *<value>* and `h` <= *<value>*.

        (`errno` 8)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.0.

        (`errno` 9)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 10)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 11)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 12)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 15)
            On entry, `s` and `h` are inconsistent with `btype`: `s` =
            *<value>* and `h` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_barrier_std_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _btype = utils._EngineCharScalarType(
        btype, {'fun_name': fun_name, 'entity': 'btype'}, exp_chrlen=2,
    ).to_ctypes()
    _btype_py = utils._EngineCharScalarType.to_py(_btype)
    if _btype_py.upper() == 'DI':
        _itype = utils._EngineIntScalarType.c_type(1)
    elif _btype_py.upper() == 'DO':
        _itype = utils._EngineIntScalarType.c_type(2)
    elif _btype_py.upper() == 'UI':
        _itype = utils._EngineIntScalarType.c_type(3)
    elif _btype_py.upper() == 'UO':
        _itype = utils._EngineIntScalarType.c_type(4)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, btype = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_btype_py),
                ),
                'Constraint: btype = \'DI\', \'DO\', \'UI\' or \'UO\'.',
            ],
            errcodes=(2,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _k = utils._EngineFloat64ScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _itype,
        _m,
        _n,
        _x,
        _s,
        _h,
        _k,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: h >= {:s} and h <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s and h are inconsistent with btype:',
                    's = {:s} and h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_jumpdiff_merton_price(calput, m, n, x, s, t, sigma, r, lamda, jvol, p):
    u"""
    Jump-diffusion, Merton's model, option pricing formula.

    For full information please refer to the NAG Library document for
    s30ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30jaf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the annual total volatility, including jumps.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    lamda : float
        lambda, the number of expected jumps per year.

    jvol : float
        The proportion of the total volatility associated with jumps.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `lamda` = *<value>*.

            Constraint: `lamda` > 0.0.

        (`errno` 10)
            On entry, `jvol` = *<value>*.

            Constraint: `jvol` >= 0.0 and `jvol` < 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_jumpdiff_merton_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30jaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, lamda.
        utils._BYREF_FLOAT64, # Input, jvol.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _lamda = utils._EngineFloat64ScalarType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
    ).to_ctypes()
    _jvol = utils._EngineFloat64ScalarType(
        jvol, {'fun_name': fun_name, 'entity': 'jvol'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _lamda,
        _jvol,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lamda > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jvol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jvol >= 0.0 and jvol < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_jumpdiff_merton_greeks(calput, m, n, x, s, t, sigma, r, lamda, jvol, p, delta, gamma, vega, theta, rho, vanna, charm, speed, colour, zomma, vomma):
    u"""
    Jump-diffusion, Merton's model, option pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30jbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the annual total volatility, including jumps.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    lamda : float
        lambda, the number of expected jumps per year.

    jvol : float
        The proportion of the total volatility associated with jumps.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    delta : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `delta` contains
        the sensitivity, dP/dS, of the option price to change in the
        price of the underlying asset.

    gamma : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `gamma` contains
        the sensitivity, d^2P/dS^2, of `delta` to change in the price of
        the underlying asset.

    vega : float, shape (`m`, `n`), modified in place
        On exit: `vega`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the volatility of the underlying asset, i.e., dP_{ij}/d
        sigma, for i = 1,2,...,`m` and j = 1,2,...,`n`.

    theta : float, shape (`m`, `n`), modified in place
        On exit: `theta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in time, i.e., -dP_{ij}/dT, for i = 1,2,...,`m` and j =
        1,2,...,`n`, where b = r-q.

    rho : float, shape (`m`, `n`), modified in place
        On exit: `rho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual risk-free interest rate, i.e., -dP_{ij}/dr, for i
        = 1,2,...,`m` and j = 1,2,...,`n`.

    vanna : float, shape (`m`, `n`), modified in place
        On exit: `vanna`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the asset price, i.e., -d
        Delta_{ij}/dT = -d^2P_{ij}/(dSd sigma), for i = 1,2,...,`m` and
        j = 1,2,...,`n`.

    charm : float, shape (`m`, `n`), modified in place
        On exit: `charm`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT),
        for i = 1,2,...,`m` and j = 1,2,...,`n`.

    speed : float, shape (`m`, `n`), modified in place
        On exit: `speed`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the price of the underlying asset, i.e., -d
        Gamma_{ij}/dS = -d^3P_{ij}/dS^3, for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    colour : float, shape (`m`, `n`), modified in place
        On exit: `colour`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the time, i.e., -d Gamma_{ij}/dT =
        -d^3P_{ij}/(dSdT), for i = 1,2,...,`m` and j = 1,2,...,`n`.

    zomma : float, shape (`m`, `n`), modified in place
        On exit: `zomma`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the volatility of the underlying asset, i.e., -d
        Gamma_{ij}/d sigma = -d^3P_{ij}/(dS^2d sigma), for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vomma : float, shape (`m`, `n`), modified in place
        On exit: `vomma`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the underlying asset, i.e., -d
        Delta_{ij}/d sigma = -d^2P_{ij}/d sigma^2, for i = 1,2,...,`m`
        and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `lamda` = *<value>*.

            Constraint: `lamda` > 0.0.

        (`errno` 10)
            On entry, `jvol` = *<value>*.

            Constraint: `jvol` >= 0.0 and `jvol` < 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_jumpdiff_merton_greeks'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30jbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, lamda.
        utils._BYREF_FLOAT64, # Input, jvol.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'), (delta, 'delta'), (gamma, 'gamma'), (vega, 'vega'), (theta, 'theta'), (rho, 'rho'), (vanna, 'vanna'), (charm, 'charm'), (speed, 'speed'), (colour, 'colour'), (zomma, 'zomma'), (vomma, 'vomma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _lamda = utils._EngineFloat64ScalarType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
    ).to_ctypes()
    _jvol = utils._EngineFloat64ScalarType(
        jvol, {'fun_name': fun_name, 'entity': 'jvol'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_shape(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _vega_cls = utils._EngineFloat64ArrayType(
        vega, {'fun_name': fun_name, 'entity': 'vega'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vega = _vega_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _vanna_cls = utils._EngineFloat64ArrayType(
        vanna, {'fun_name': fun_name, 'entity': 'vanna'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vanna = _vanna_cls.to_ctypes(can_reference_contents=False)
    _charm_cls = utils._EngineFloat64ArrayType(
        charm, {'fun_name': fun_name, 'entity': 'charm'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _charm = _charm_cls.to_ctypes(can_reference_contents=False)
    _speed_cls = utils._EngineFloat64ArrayType(
        speed, {'fun_name': fun_name, 'entity': 'speed'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _speed = _speed_cls.to_ctypes(can_reference_contents=False)
    _colour_cls = utils._EngineFloat64ArrayType(
        colour, {'fun_name': fun_name, 'entity': 'colour'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _colour = _colour_cls.to_ctypes(can_reference_contents=False)
    _zomma_cls = utils._EngineFloat64ArrayType(
        zomma, {'fun_name': fun_name, 'entity': 'zomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _zomma = _zomma_cls.to_ctypes(can_reference_contents=False)
    _vomma_cls = utils._EngineFloat64ArrayType(
        vomma, {'fun_name': fun_name, 'entity': 'vomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vomma = _vomma_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _lamda,
        _jvol,
        _p,
        _ldp,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lamda > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jvol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jvol >= 0.0 and jvol < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _vega_cls.refresh_py(_vega)
        _theta_cls.refresh_py(_theta)
        _rho_cls.refresh_py(_rho)
        _vanna_cls.refresh_py(_vanna)
        _charm_cls.refresh_py(_charm)
        _speed_cls.refresh_py(_speed)
        _colour_cls.refresh_py(_colour)
        _zomma_cls.refresh_py(_zomma)
        _vomma_cls.refresh_py(_vomma)
        return_data = None
    _err_class.manage()
    return return_data

def opt_heston_price(calput, m, n, x, s, t, sigmav, kappa, corr, var0, eta, grisk, r, q, p):
    u"""
    Heston's model option pricing formula.

    For full information please refer to the NAG Library document for
    s30na

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30naf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigmav : float
        The volatility, sigma_v, of the volatility process, sqrt(v_t).
        Note that a rate of 20% should be entered as 0.2.

    kappa : float
        kappa, the long term mean reversion rate of the volatility.

    corr : float
        The correlation between the two standard Brownian motions for
        the asset price and the volatility.

    var0 : float
        The initial value of the variance, v_t, of the asset price.

    eta : float
        eta, the long term mean of the variance of the asset price.

    grisk : float
        The risk aversion parameter, gamma, of the representative agent.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i-1] >= *<value>* and `x`[i-1] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i-1] >= *<value>*.

        (`errno` 7)
            On entry, `sigmav` = *<value>*.

            Constraint: `sigmav` > 0.0.

        (`errno` 8)
            On entry, `kappa` = *<value>*.

            Constraint: `kappa` > 0.0.

        (`errno` 9)
            On entry, `corr` = *<value>*.

            Constraint: abs(`corr`) <= 1.0.

        (`errno` 10)
            On entry, `var0` = *<value>*.

            Constraint: `var0` >= 0.0.

        (`errno` 11)
            On entry, `eta` = *<value>*.

            Constraint: `eta` > 0.0.

        (`errno` 12)
            On entry, `grisk` = *<value>*, `sigmav` = *<value>* and
            `kappa` = *<value>*.

            Constraint: 0.0 <= `grisk` <= 1.0 and
            `grisk`*(1.0-`grisk`)*`sigmav`^2 <= `kappa`^2.

        (`errno` 13)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 14)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 17)
            Quadrature has not converged to the specified accuracy.
            However, the result should be a reasonable approximation.

        (`errno` 18)
            Solution cannot be computed accurately. Check values of
            input arguments.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_heston_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30naft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigmav.
        utils._BYREF_FLOAT64, # Input, kappa.
        utils._BYREF_FLOAT64, # Input, corr.
        utils._BYREF_FLOAT64, # Input, var0.
        utils._BYREF_FLOAT64, # Input, eta.
        utils._BYREF_FLOAT64, # Input, grisk.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigmav = utils._EngineFloat64ScalarType(
        sigmav, {'fun_name': fun_name, 'entity': 'sigmav'},
    ).to_ctypes()
    _kappa = utils._EngineFloat64ScalarType(
        kappa, {'fun_name': fun_name, 'entity': 'kappa'},
    ).to_ctypes()
    _corr = utils._EngineFloat64ScalarType(
        corr, {'fun_name': fun_name, 'entity': 'corr'},
    ).to_ctypes()
    _var0 = utils._EngineFloat64ScalarType(
        var0, {'fun_name': fun_name, 'entity': 'var0'},
    ).to_ctypes()
    _eta = utils._EngineFloat64ScalarType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
    ).to_ctypes()
    _grisk = utils._EngineFloat64ScalarType(
        grisk, {'fun_name': fun_name, 'entity': 'grisk'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigmav,
        _kappa,
        _corr,
        _var0,
        _eta,
        _grisk,
        _r,
        _q,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i-1] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i-1] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i-1] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigmav = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigmav > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kappa = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kappa > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, corr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(corr) <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, var0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: var0 >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eta = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eta > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, grisk = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'sigmav = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'kappa = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 0.0 <= grisk <= 1.0',
                    'and grisk*(1.0-grisk)*sigmav^2 <= kappa^2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Quadrature has not converged to the specified accuracy.',
                    'However, the result should be a reasonable approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Solution cannot be computed accurately. Check values of input arguments.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def opt_heston_greeks(calput, m, n, x, s, t, sigmav, kappa, corr, var0, eta, grisk, r, q, p, delta, gamma, vega, theta, rho, vanna, charm, speed, zomma, vomma):
    u"""
    Heston's model option pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30nb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30nbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigmav : float
        The volatility, sigma_v, of the volatility process, sqrt(v_t).
        Note that a rate of 20% should be entered as 0.2.

    kappa : float
        kappa, the long term mean reversion rate of the volatility.

    corr : float
        The correlation between the two standard Brownian motions for
        the asset price and the volatility.

    var0 : float
        The initial value of the variance, v_t, of the asset price.

    eta : float
        eta, the long term mean of the variance of the asset price.

    grisk : float
        The risk aversion parameter, gamma, of the representative agent.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    delta : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `delta` contains
        the sensitivity, dP/dS, of the option price to change in the
        price of the underlying asset.

    gamma : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `gamma` contains
        the sensitivity, d^2P/dS^2, of `delta` to change in the price of
        the underlying asset.

    vega : float, shape (`m`, `n`), modified in place
        On exit: `vega`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the volatility of the underlying asset, i.e., dP_{ij}/d
        sigma, for i = 1,2,...,`m` and j = 1,2,...,`n`.

    theta : float, shape (`m`, `n`), modified in place
        On exit: `theta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in time, i.e., -dP_{ij}/dT, for i = 1,2,...,`m` and j =
        1,2,...,`n`, where b = r-q.

    rho : float, shape (`m`, `n`), modified in place
        On exit: `rho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual risk-free interest rate, i.e., -dP_{ij}/dr, for i
        = 1,2,...,`m` and j = 1,2,...,`n`.

    vanna : float, shape (`m`, `n`), modified in place
        On exit: `vanna`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the asset price, i.e., -d
        Delta_{ij}/dT = -d^2P_{ij}/(dSd sigma), for i = 1,2,...,`m` and
        j = 1,2,...,`n`.

    charm : float, shape (`m`, `n`), modified in place
        On exit: `charm`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT),
        for i = 1,2,...,`m` and j = 1,2,...,`n`.

    speed : float, shape (`m`, `n`), modified in place
        On exit: `speed`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the price of the underlying asset, i.e., -d
        Gamma_{ij}/dS = -d^3P_{ij}/dS^3, for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    zomma : float, shape (`m`, `n`), modified in place
        On exit: `zomma`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the volatility of the underlying asset, i.e., -d
        Gamma_{ij}/d sigma = -d^3P_{ij}/(dS^2d sigma), for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vomma : float, shape (`m`, `n`), modified in place
        On exit: `vomma`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the underlying asset, i.e., -d
        Delta_{ij}/d sigma = -d^2P_{ij}/d sigma^2, for i = 1,2,...,`m`
        and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i-1] >= *<value>* and `x`[i-1] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i-1] >= *<value>*.

        (`errno` 7)
            On entry, `sigmav` = *<value>*.

            Constraint: `sigmav` > 0.0.

        (`errno` 8)
            On entry, `kappa` = *<value>*.

            Constraint: `kappa` > 0.0.

        (`errno` 9)
            On entry, `corr` = *<value>*.

            Constraint: abs(`corr`) <= 1.0.

        (`errno` 10)
            On entry, `var0` = *<value>*.

            Constraint: `var0` >= 0.0.

        (`errno` 11)
            On entry, `eta` = *<value>*.

            Constraint: `eta` > 0.0.

        (`errno` 12)
            On entry, `grisk` = *<value>*, `sigmav` = *<value>* and
            `kappa` = *<value>*.

            Constraint: 0.0 <= `grisk` <= 1.0 and
            `grisk`*(1.0-`grisk`)*`sigmav`^2 <= `kappa`^2.

        (`errno` 13)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 14)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 17)
            Quadrature has not converged to the required accuracy.
            However, the result should be a reasonable approximation.

        (`errno` 18)
            Solution cannot be computed accurately. Check values of
            input arguments.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_heston_greeks'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30nbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigmav.
        utils._BYREF_FLOAT64, # Input, kappa.
        utils._BYREF_FLOAT64, # Input, corr.
        utils._BYREF_FLOAT64, # Input, var0.
        utils._BYREF_FLOAT64, # Input, eta.
        utils._BYREF_FLOAT64, # Input, grisk.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'), (delta, 'delta'), (gamma, 'gamma'), (vega, 'vega'), (theta, 'theta'), (rho, 'rho'), (vanna, 'vanna'), (charm, 'charm'), (speed, 'speed'), (zomma, 'zomma'), (vomma, 'vomma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = {:s}.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigmav = utils._EngineFloat64ScalarType(
        sigmav, {'fun_name': fun_name, 'entity': 'sigmav'},
    ).to_ctypes()
    _kappa = utils._EngineFloat64ScalarType(
        kappa, {'fun_name': fun_name, 'entity': 'kappa'},
    ).to_ctypes()
    _corr = utils._EngineFloat64ScalarType(
        corr, {'fun_name': fun_name, 'entity': 'corr'},
    ).to_ctypes()
    _var0 = utils._EngineFloat64ScalarType(
        var0, {'fun_name': fun_name, 'entity': 'var0'},
    ).to_ctypes()
    _eta = utils._EngineFloat64ScalarType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
    ).to_ctypes()
    _grisk = utils._EngineFloat64ScalarType(
        grisk, {'fun_name': fun_name, 'entity': 'grisk'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_shape(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _vega_cls = utils._EngineFloat64ArrayType(
        vega, {'fun_name': fun_name, 'entity': 'vega'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vega = _vega_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _vanna_cls = utils._EngineFloat64ArrayType(
        vanna, {'fun_name': fun_name, 'entity': 'vanna'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vanna = _vanna_cls.to_ctypes(can_reference_contents=False)
    _charm_cls = utils._EngineFloat64ArrayType(
        charm, {'fun_name': fun_name, 'entity': 'charm'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _charm = _charm_cls.to_ctypes(can_reference_contents=False)
    _speed_cls = utils._EngineFloat64ArrayType(
        speed, {'fun_name': fun_name, 'entity': 'speed'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _speed = _speed_cls.to_ctypes(can_reference_contents=False)
    _zomma_cls = utils._EngineFloat64ArrayType(
        zomma, {'fun_name': fun_name, 'entity': 'zomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _zomma = _zomma_cls.to_ctypes(can_reference_contents=False)
    _vomma_cls = utils._EngineFloat64ArrayType(
        vomma, {'fun_name': fun_name, 'entity': 'vomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vomma = _vomma_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigmav,
        _kappa,
        _corr,
        _var0,
        _eta,
        _grisk,
        _r,
        _q,
        _p,
        _ldp,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _vanna,
        _charm,
        _speed,
        _zomma,
        _vomma,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i-1] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i-1] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i-1] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigmav = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigmav > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kappa = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kappa > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, corr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(corr) <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, var0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: var0 >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eta = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eta > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, grisk = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'sigmav = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'kappa = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 0.0 <= grisk <= 1.0',
                    'and grisk*(1.0-grisk)*sigmav^2 <= kappa^2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Quadrature has not converged to the required accuracy.',
                    'However, the result should be a reasonable approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Solution cannot be computed accurately. Check values of input arguments.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _vega_cls.refresh_py(_vega)
        _theta_cls.refresh_py(_theta)
        _rho_cls.refresh_py(_rho)
        _vanna_cls.refresh_py(_vanna)
        _charm_cls.refresh_py(_charm)
        _speed_cls.refresh_py(_speed)
        _zomma_cls.refresh_py(_zomma)
        _vomma_cls.refresh_py(_vomma)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def opt_heston_term(calput, m, numts, x, fwd, disc, ts, t, alpha, lamda, corr, sigmat, var0, p):
    u"""
    Heston's model option pricing with term structure.

    For full information please refer to the NAG Library document for
    s30nc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30ncf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        m, the number of strike prices to be used.

    numts : int
        n_s, the number of subintervals into which the time to expiry,
        T, is divided.

    x : float, shape (`m`)
        `x`[i-1] contains the ith strike price, for i = 1,2,...,m.

    fwd : float
        The forward price of the asset.

    disc : float
        The discount factor, where the current price of the underlying
        asset, S_0, is given as S_0 = `disc`*`fwd`.

    ts : float, shape (`numts`)
        `ts`[i-1] must contain the length of the time intervals for
        which the corresponding element of `alpha`, `lamda`, `corr` and
        `sigmat` apply. These should be ordered as they occur in time
        i.e., Delta t_i = t_i-t_{i-1}.

    t : float
        `t` contains the time to expiry. If T > sum Delta t_i then the
        parameters associated with the last time interval are extended
        to the expiry time. If T < sum Delta t_i then the parameters
        specified are used up until the expiry time. The rest are
        ignored.

    alpha : float, shape (`numts`)
        `alpha`[i-1] must contain the value of alpha_t, the value of the
        volatility of the scaled volatility, sqrt(nu), over time
        subinterval Delta t_i.

    lamda : float, shape (`numts`)
        `lamda`[i-1] must contain the value, lambda_t, of the mean
        reversion parameter over the time subinterval Delta t_i.

    corr : float, shape (`numts`)
        `corr`[i-1] must contain the value, rho_t, of the correlation
        parameter over the time subinterval Delta t_i.

    sigmat : float, shape (`numts`)
        `sigmat`[i-1] must contain the value, sigma_t, of the variance
        scale factor over the time subinterval Delta t_i.

    var0 : float
        nu_0, the initial scaled variance.

    p : float, shape (`m`), modified in place
        On exit: `p`[i-1] contains the computed option price at the
        expiry time, T, corresponding to strike `x`[i-1] for the
        specified term structure, for i = 1,2,...,`m`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` != 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `numts` = *<value>*.

            Constraint: `numts` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `x`[i-1] <= *<value>*.

        (`errno` 5)
            On entry, `fwd` = *<value>*.

            Constraint: *<value>* <= `fwd` <= *<value>*.

        (`errno` 6)
            On entry, `disc` = *<value>*.

            Constraint: *<value>* <= `disc` <= *<value>*.

        (`errno` 7)
            On entry, `ts`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `ts`[i-1] <= *<value>*.

        (`errno` 8)
            On entry, `t` = *<value>*.

            Constraint: `t` >= *<value>*.

        (`errno` 9)
            On entry, `alpha`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `alpha`[i-1] <= *<value>*.

        (`errno` 10)
            On entry, `lamda`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `lamda`[i-1] <= *<value>*.

        (`errno` 11)
            On entry, `corr`[*<value>*] = *<value>*.

            Constraint: abs(`corr`[i-1]) <= 1.0.

        (`errno` 12)
            On entry, `sigmat`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `sigmat`[i-1] <= *<value>*.

        (`errno` 13)
            On entry, `var0` = *<value>*.

            Constraint: `var0` > 0.0.

        (`errno` 14)
            Quadrature has not converged to the specified accuracy.
            However, the result should be a reasonable approximation.

        (`errno` 15)
            Solution cannot be computed accurately. Check values of
            input arguments.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_heston_term'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30ncft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, numts.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, fwd.
        utils._BYREF_FLOAT64, # Input, disc.
        utils._EngineFloat64ArrayType.c_type, # Input, ts[numts].
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, alpha[numts].
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[numts].
        utils._EngineFloat64ArrayType.c_type, # Input, corr[numts].
        utils._EngineFloat64ArrayType.c_type, # Input, sigmat[numts].
        utils._BYREF_FLOAT64, # Input, var0.
        utils._EngineFloat64ArrayType.c_type, # Output, p[m].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput != \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _numts = utils._EngineIntScalarType(
        numts, {'fun_name': fun_name, 'entity': 'numts'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _fwd = utils._EngineFloat64ScalarType(
        fwd, {'fun_name': fun_name, 'entity': 'fwd'},
    ).to_ctypes()
    _disc = utils._EngineFloat64ScalarType(
        disc, {'fun_name': fun_name, 'entity': 'disc'},
    ).to_ctypes()
    _numts_py = utils._EngineIntScalarType.to_py(_numts)
    _ts = utils._EngineFloat64ArrayType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
        exp_shape=(_numts_py,),
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _alpha = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_numts_py,),
    ).to_ctypes()
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_numts_py,),
    ).to_ctypes()
    _corr = utils._EngineFloat64ArrayType(
        corr, {'fun_name': fun_name, 'entity': 'corr'},
        exp_shape=(_numts_py,),
    ).to_ctypes()
    _sigmat = utils._EngineFloat64ArrayType(
        sigmat, {'fun_name': fun_name, 'entity': 'sigmat'},
        exp_shape=(_numts_py,),
    ).to_ctypes()
    _var0 = utils._EngineFloat64ScalarType(
        var0, {'fun_name': fun_name, 'entity': 'var0'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _numts,
        _x,
        _fwd,
        _disc,
        _ts,
        _t,
        _alpha,
        _lamda,
        _corr,
        _sigmat,
        _var0,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, numts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: numts >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= x[i-1] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fwd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: {:s} <= fwd <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, disc = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: {:s} <= disc <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ts[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= ts[i-1] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: t >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, alpha[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= alpha[i-1] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= lamda[i-1] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, corr[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: abs(corr[i-1]) <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigmat[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= sigmat[i-1] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, var0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: var0 > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Quadrature has not converged to the specified accuracy.',
                    'However, the result should be a reasonable approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Solution cannot be computed accurately. Check values of input arguments.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_amer_bs_price(calput, m, n, x, s, t, sigma, r, q, p):
    u"""
    American option, Bjerksund and Stensland pricing formula.

    For full information please refer to the NAG Library document for
    s30qc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30qcf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 14)
            On entry, `s` = *<value>* and beta = *<value>*.

            Constraint: `s`^beta < *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_amer_bs_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30qcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _q,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s} and beta = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s^beta < {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_asian_geom_price(calput, m, n, x, s, t, sigma, r, b, p):
    u"""
    Asian option, geometric continuous average rate pricing formula.

    For full information please refer to the NAG Library document for
    s30sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30saf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    b : float
        b, the annual cost of carry rate. Note that a rate of 8% should
        be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_asian_geom_price'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30saft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, b.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_nd(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _b,
        _p,
        _ldp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        return_data = None
    _err_class.manage()
    return return_data

def opt_asian_geom_greeks(calput, m, n, x, s, t, sigma, r, b, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma):
    u"""
    Asian option, geometric continuous average rate pricing formula with
    Greeks.

    For full information please refer to the NAG Library document for
    s30sb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30sbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    m : int
        The number of strike prices to be used.

    n : int
        The number of times to expiry to be used.

    x : float, shape (`m`)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,`m`.

    s : float
        S, the price of the underlying asset.

    t : float, shape (`n`)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,`n`.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    b : float
        b, the annual cost of carry rate. Note that a rate of 8% should
        be entered as 0.08.

    p : float, shape (`m`, `n`), modified in place
        On exit: `p`[i-1,j-1] contains P_{ij}, the option price
        evaluated for the strike price `x`_i at expiry `t`_j for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    delta : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `delta` contains
        the sensitivity, dP/dS, of the option price to change in the
        price of the underlying asset.

    gamma : float, shape (`m`, `n`), modified in place
        On exit: the leading `m`*`n` part of the array `gamma` contains
        the sensitivity, d^2P/dS^2, of `delta` to change in the price of
        the underlying asset.

    vega : float, shape (`m`, `n`), modified in place
        On exit: `vega`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the volatility of the underlying asset, i.e., dP_{ij}/d
        sigma, for i = 1,2,...,`m` and j = 1,2,...,`n`.

    theta : float, shape (`m`, `n`), modified in place
        On exit: `theta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in time, i.e., -dP_{ij}/dT, for i = 1,2,...,`m` and j =
        1,2,...,`n`, where b = r-q.

    rho : float, shape (`m`, `n`), modified in place
        On exit: `rho`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the annual risk-free interest rate, i.e., -dP_{ij}/dr, for i
        = 1,2,...,`m` and j = 1,2,...,`n`.

    crho : float, shape (`m`, `n`), modified in place
        On exit: `delta`[i-1,j-1], contains the first-order Greek
        measuring the sensitivity of the option price P_{ij} to change
        in the price of the underlying asset, i.e., -dP_{ij}/dS, for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vanna : float, shape (`m`, `n`), modified in place
        On exit: `vanna`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the asset price, i.e., -d
        Delta_{ij}/dT = -d^2P_{ij}/(dSd sigma), for i = 1,2,...,`m` and
        j = 1,2,...,`n`.

    charm : float, shape (`m`, `n`), modified in place
        On exit: `charm`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT),
        for i = 1,2,...,`m` and j = 1,2,...,`n`.

    speed : float, shape (`m`, `n`), modified in place
        On exit: `speed`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the price of the underlying asset, i.e., -d
        Gamma_{ij}/dS = -d^3P_{ij}/dS^3, for i = 1,2,...,`m` and j =
        1,2,...,`n`.

    colour : float, shape (`m`, `n`), modified in place
        On exit: `colour`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the time, i.e., -d Gamma_{ij}/dT =
        -d^3P_{ij}/(dSdT), for i = 1,2,...,`m` and j = 1,2,...,`n`.

    zomma : float, shape (`m`, `n`), modified in place
        On exit: `zomma`[i-1,j-1], contains the third-order Greek
        measuring the sensitivity of the second-order Greek Gamma_{ij}
        to change in the volatility of the underlying asset, i.e., -d
        Gamma_{ij}/d sigma = -d^3P_{ij}/(dS^2d sigma), for i =
        1,2,...,`m` and j = 1,2,...,`n`.

    vomma : float, shape (`m`, `n`), modified in place
        On exit: `vomma`[i-1,j-1], contains the second-order Greek
        measuring the sensitivity of the first-order Greek Delta_{ij} to
        change in the volatility of the underlying asset, i.e., -d
        Delta_{ij}/d sigma = -d^2P_{ij}/d sigma^2, for i = 1,2,...,`m`
        and j = 1,2,...,`n`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.specfun.opt_asian_geom_greeks'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('s30sbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, option.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_FLOAT64, # Input, r.
        utils._BYREF_FLOAT64, # Input, b.
        utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
        utils._BYREF_INT, # Input, ldp.
        utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
        utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((p, 'p'), (delta, 'delta'), (gamma, 'gamma'), (vega, 'vega'), (theta, 'theta'), (rho, 'rho'), (crho, 'crho'), (vanna, 'vanna'), (charm, 'charm'), (speed, 'speed'), (colour, 'colour'), (zomma, 'zomma'), (vomma, 'vomma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _calput = utils._EngineCharScalarType(
        calput, {'fun_name': fun_name, 'entity': 'calput'}, exp_chrlen=1,
    ).to_ctypes()
    _calput_py = utils._EngineCharScalarType.to_py(_calput)
    if _calput_py.upper() == 'C':
        _option = utils._EngineIntScalarType.c_type(0)
    elif _calput_py.upper() == 'P':
        _option = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, calput = \'{:s}\'.'.format(
                    utils._STR_FMT_STR.format(_calput_py),
                ),
                'Constraint: calput = \'C\' or \'P\'.',
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _ldp_py = utils._EngineFloat64ArrayType.get_shape(p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ldp = utils._EngineIntScalarType(
        _ldp_py, {'fun_name': fun_name, 'entity': '_ldp'},
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _delta_cls = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _delta = _delta_cls.to_ctypes(can_reference_contents=False)
    _gamma_cls = utils._EngineFloat64ArrayType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _gamma = _gamma_cls.to_ctypes(can_reference_contents=False)
    _vega_cls = utils._EngineFloat64ArrayType(
        vega, {'fun_name': fun_name, 'entity': 'vega'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vega = _vega_cls.to_ctypes(can_reference_contents=False)
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _rho_cls = utils._EngineFloat64ArrayType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _rho = _rho_cls.to_ctypes(can_reference_contents=False)
    _crho_cls = utils._EngineFloat64ArrayType(
        crho, {'fun_name': fun_name, 'entity': 'crho'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _crho = _crho_cls.to_ctypes(can_reference_contents=False)
    _vanna_cls = utils._EngineFloat64ArrayType(
        vanna, {'fun_name': fun_name, 'entity': 'vanna'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vanna = _vanna_cls.to_ctypes(can_reference_contents=False)
    _charm_cls = utils._EngineFloat64ArrayType(
        charm, {'fun_name': fun_name, 'entity': 'charm'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _charm = _charm_cls.to_ctypes(can_reference_contents=False)
    _speed_cls = utils._EngineFloat64ArrayType(
        speed, {'fun_name': fun_name, 'entity': 'speed'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _speed = _speed_cls.to_ctypes(can_reference_contents=False)
    _colour_cls = utils._EngineFloat64ArrayType(
        colour, {'fun_name': fun_name, 'entity': 'colour'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _colour = _colour_cls.to_ctypes(can_reference_contents=False)
    _zomma_cls = utils._EngineFloat64ArrayType(
        zomma, {'fun_name': fun_name, 'entity': 'zomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _zomma = _zomma_cls.to_ctypes(can_reference_contents=False)
    _vomma_cls = utils._EngineFloat64ArrayType(
        vomma, {'fun_name': fun_name, 'entity': 'vomma'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vomma = _vomma_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _option,
        _m,
        _n,
        _x,
        _s,
        _t,
        _sigma,
        _r,
        _b,
        _p,
        _ldp,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[i] >= {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i] <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: s >= {:s} and s <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: t[i] >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _delta_cls.refresh_py(_delta)
        _gamma_cls.refresh_py(_gamma)
        _vega_cls.refresh_py(_vega)
        _theta_cls.refresh_py(_theta)
        _rho_cls.refresh_py(_rho)
        _crho_cls.refresh_py(_crho)
        _vanna_cls.refresh_py(_vanna)
        _charm_cls.refresh_py(_charm)
        _speed_cls.refresh_py(_speed)
        _colour_cls.refresh_py(_colour)
        _zomma_cls.refresh_py(_zomma)
        _vomma_cls.refresh_py(_vomma)
        return_data = None
    _err_class.manage()
    return return_data
