# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `specfun` Chapter.

``specfun`` - Approximations of Special Functions

This module is concerned with the provision of some commonly occurring
physical and mathematical functions.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/sintro.html

See Also
--------
``naginterfaces.library.examples.specfun`` :
    This subpackage contains examples for the ``specfun`` module.
    See also the :ref:`library_specfun_ex` subsection.
"""

# NAG Copyright 2017-2019.

from __future__ import absolute_import

import collections as _collections

from math import sqrt

from ..base import utils as _b_utils
from ..library import machine
from ..library import _utils

def log_shifted(x):
    u"""
    ln(1+x).

    For full information please refer to the NAG Library document for
    s01ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s01baf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    lnx1 : float
        The value of the shifted logarithmic function, ln(1+x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > -1.0.
    """

    from ..base.specfun import log_shifted as b_log_shifted

    _return_scalar = b_log_shifted(
        x,
    )
    return _return_scalar

def exp_complex(z):
    u"""
    Complex exponential, e^z.

    For full information please refer to the NAG Library document for
    s01ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s01eaf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    Returns
    -------
    ez : complex
        The value of the exponential function.

    Raises
    ------
    NagValueError
        (`errno` 5)
            The imaginary part of argument `z` is so large that the
            result has no precision: `z` = (*<value>*,*<value>*).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Argument `z` causes overflow in real part of result: `z` =
            (*<value>*,*<value>*).

        (`errno` 2)
            Argument `z` causes overflow in imaginary part of result:
            `z` = (*<value>*,*<value>*).

        (`errno` 3)
            Argument `z` causes overflow in both real and imaginary
            parts of result: `z` = (*<value>*,*<value>*).

        (`errno` 4)
            The imaginary part of argument `z` is so large that the
            result is accurate to less than half precision: `z` =
            (*<value>*,*<value>*).

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.specfun import exp_complex as b_exp_complex

    try:
        _return_scalar = b_exp_complex(
            z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ez'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def tan(x):
    u"""
    tan(x).

    For full information please refer to the NAG Library document for
    s07aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s07aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    tx : float
        The value of the circular tangent, tan(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            The function has been called with an argument that is too
            large; the default result returned is zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The function has been called with an argument that is too
            close to an odd multiple of pi/2, at which the function is
            infinite; the function has returned a value with the correct
            sign but a more or less arbitrary but large magnitude.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.specfun import tan as b_tan

    try:
        _return_scalar = b_tan(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['tx'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def arcsin(x):
    u"""
    arcsin(x).

    For full information please refer to the NAG Library document for
    s09aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s09aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    asx : float
        The value of the inverse circular sine, arcsin(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.specfun import arcsin as b_arcsin

    _return_scalar = b_arcsin(
        x,
    )
    return _return_scalar

def arccos(x):
    u"""
    arccos(x).

    For full information please refer to the NAG Library document for
    s09ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s09abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    acx : float
        The value of the inverse circular cosine, arccos(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.specfun import arccos as b_arccos

    _return_scalar = b_arccos(
        x,
    )
    return _return_scalar

def tanh(x):
    u"""
    tanh(x).

    For full information please refer to the NAG Library document for
    s10aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s10aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    thx : float
        The value of the hyperbolic tangent, tanh(x).
    """

    from ..base.specfun import tanh as b_tanh

    _return_scalar = b_tanh(
        x,
    )
    return _return_scalar

def sinh(x):
    u"""
    sinh(x).

    For full information please refer to the NAG Library document for
    s10ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s10abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    shx : float
        The value of the hyperbolic sine, sinh(x).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= E_1.

            The function has been called with an argument too large in
            absolute magnitude. There is a danger of overflow. The
            result returned is the value of sinh(x) at the closest
            argument for which a valid call could be made.
    """

    from ..base.specfun import sinh as b_sinh

    try:
        _return_scalar = b_sinh(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['shx'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def cosh(x):
    u"""
    cosh(x).

    For full information please refer to the NAG Library document for
    s10ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s10acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    chx : float
        The value of the hyperbolic cosine, cosh(x).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= E_1.

            The function has been called with an argument too large in
            absolute magnitude. There is a danger of overflow. The
            result returned is the value of cosh(x) at the nearest valid
            argument.
    """

    from ..base.specfun import cosh as b_cosh

    try:
        _return_scalar = b_cosh(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['chx'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def arctanh(x):
    u"""
    arctanh(x).

    For full information please refer to the NAG Library document for
    s11aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s11aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    athx : float
        The value of the inverse hyperbolic tangent, arctanh(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) < 1.

            The function has been called with an argument greater than
            or equal to 1.0 in magnitude, for which arctanh is not
            defined.
    """

    from ..base.specfun import arctanh as b_arctanh

    _return_scalar = b_arctanh(
        x,
    )
    return _return_scalar

def arcsinh(x):
    u"""
    arcsinh(x).

    For full information please refer to the NAG Library document for
    s11ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s11abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ashx : float
        The value of the inverse hyperbolic sine, arcsinh(x).
    """

    from ..base.specfun import arcsinh as b_arcsinh

    _return_scalar = b_arcsinh(
        x,
    )
    return _return_scalar

def arccosh(x):
    u"""
    arccosh(x).

    For full information please refer to the NAG Library document for
    s11ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s11acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    achx : float
        The value of the inverse hyperbolic cosine, arccosh(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 1.0.

            The function has been called with an argument less than 1.0,
            for which arccosh(x) is not defined.
    """

    from ..base.specfun import arccosh as b_arccosh

    _return_scalar = b_arccosh(
        x,
    )
    return _return_scalar

def integral_exp(x):
    u"""
    Exponential integral E_1(x).

    For full information please refer to the NAG Library document for
    s13aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s13aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    e1x : float
        The value of the exponential integral E_1(x).

    Raises
    ------
    NagValueError
        (`errno` 2)
            The evaluation has been abandoned due to the likelihood of
            overflow. The argument `x` < -x_hi.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = 0.0 and the function is infinite.
    """

    from ..base.specfun import integral_exp as b_integral_exp

    try:
        _return_scalar = b_integral_exp(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['e1x'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def integral_cos(x):
    u"""
    Cosine integral Ci(x).

    For full information please refer to the NAG Library document for
    s13ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s13acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    cix : float
        The value of the cosine integral

            Ci(x) = gamma+ln(x)+int_0^x(cos(u)-1)/udu, x > 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            The function has been called with an argument less than or
            equal to zero for which Ci(x) is not defined.
    """

    from ..base.specfun import integral_cos as b_integral_cos

    _return_scalar = b_integral_cos(
        x,
    )
    return _return_scalar

def integral_sin(x):
    u"""
    Sine integral Si(x).

    For full information please refer to the NAG Library document for
    s13ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s13adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    six : float
        The value of the sine integral

            Si(x) = int_0^xsin(u)/udu.
    """

    from ..base.specfun import integral_sin as b_integral_sin

    _return_scalar = b_integral_sin(
        x,
    )
    return _return_scalar

def gamma(x):
    u"""
    Gamma function.

    For full information please refer to the NAG Library document for
    s14aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    gx : float
        The value of the gamma function Gamma(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= *<value>*.

            The argument is too large, the function returns the
            approximate value of Gamma(x) at the nearest valid argument.

        (`errno` 2)
            On entry, `x` = *<value>*. The function returns zero.

            Constraint: `x` >= *<value>*.

            The argument is too large and negative, the function returns
            zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) >= *<value>*.

            The argument is too close to zero, the function returns the
            approximate value of Gamma(x) at the nearest valid argument.

        (`errno` 4)
            On entry, `x` = *<value>*.

            Constraint: `x` must not be a negative integer.

            The argument is a negative integer, at which values Gamma(x)
            is infinite. The function returns a large positive value.
    """

    from ..base.specfun import gamma as b_gamma

    try:
        _return_scalar = b_gamma(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['gx'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def gamma_log_real(x):
    u"""
    Log gamma function, real argument.

    For full information please refer to the NAG Library document for
    s14ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    lgx : float
        The value of ln(Gamma)(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` <= x_big.
    """

    from ..base.specfun import gamma_log_real as b_gamma_log_real

    try:
        _return_scalar = b_gamma_log_real(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['lgx'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def polygamma(x):
    u"""
    psi(x)-ln(x).

    For full information please refer to the NAG Library document for
    s14ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    pgx : float
        The value of the function psi(x)-ln(x), where psi is the psi
        function psi(x) = d/dxln(Gamma)(x) = Gamma'(x)/Gamma(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

        (`errno` 2)
            Computation halted due to likelihood of underflow. `x` may
            be too large. `x` = *<value>*.

        (`errno` 3)
            Computation halted due to likelihood of overflow. `x` may be
            too small. `x` = *<value>*.
    """

    from ..base.specfun import polygamma as b_polygamma

    _return_scalar = b_polygamma(
        x,
    )
    return _return_scalar

def polygamma_deriv(x, n, m):
    u"""
    Scaled derivatives of psi(x).

    For full information please refer to the NAG Library document for
    s14ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    n : int
        The index of the first member n of the sequence of functions.

    m : int
        The number of members m required in the sequence w(k,x), for k =
        n,...,n+m-1.

    Returns
    -------
    ans : float, ndarray, shape (`m`)
        The first m elements of `ans` contain the required values
        w(k,x), for k = n,...,n+m-1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 4)
            Computation abandoned due to the likelihood of underflow.

        (`errno` 5)
            Computation abandoned due to the likelihood of overflow.

        (`errno` 6)
            There is not enough internal workspace to continue
            computation. `m` is probably too large.
    """

    from ..base.specfun import polygamma_deriv as b_polygamma_deriv

    fun_name = 'naginterfaces.library.specfun.polygamma_deriv'

    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _ans = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    b_polygamma_deriv(
        x,
        n,
        _m,
        _ans,
    )
    _ans_py = _ans
    return _ans_py

def psi_deriv_real(x, k):
    u"""
    Polygamma function psi^(n)(x) for real x.

    For full information please refer to the NAG Library document for
    s14ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    k : int
        The function psi^(k)(x) to be evaluated.

    Returns
    -------
    pkx : float
        The value of the kth derivative of the psi function psi(x) for
        real x and k = 0,1,...,6.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` is 'too close' to a non-positive integer: `x`
            = *<value>* and nint(`x`) = *<value>*.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` <= 6.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            Evaluation abandoned due to likelihood of underflow.

        (`errno` 3)
            Evaluation abandoned due to likelihood of overflow.
    """

    from ..base.specfun import psi_deriv_real as b_psi_deriv_real

    _return_scalar = b_psi_deriv_real(
        x,
        k,
    )
    return _return_scalar

def psi_deriv_complex(z, k):
    u"""
    Polygamma function psi^(n)(z) for complex z.

    For full information please refer to the NAG Library document for
    s14af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14aff.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    k : int
        The function psi^(k)(z) to be evaluated.

    Returns
    -------
    pkz : complex
        The value of the kth derivative of the psi function.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, Re(`z`) is 'too close' to a non-positive integer
            when Im(`z`) = 0.0: Re(`z`) = *<value>*, nint(Re(`z`)) =
            *<value>*.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` <= 4.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            Evaluation abandoned due to likelihood of overflow.
    """

    from ..base.specfun import psi_deriv_complex as b_psi_deriv_complex

    _return_scalar = b_psi_deriv_complex(
        z,
        k,
    )
    return _return_scalar

def gamma_log_complex(z):
    u"""
    Logarithm of the gamma function ln(Gamma)(z), complex argument.

    For full information please refer to the NAG Library document for
    s14ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14agf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    Returns
    -------
    lngz : complex
        The value of ln(Gamma)(z).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, Re(`z`) is 'too close' to a non-positive integer
            when Im(`z`) = 0.0.
    """

    from ..base.specfun import gamma_log_complex as b_gamma_log_complex

    _return_scalar = b_gamma_log_complex(
        z,
    )
    return _return_scalar

def gamma_log_scaled_real(x):
    u"""
    Scaled log gamma function.

    For full information please refer to the NAG Library document for
    s14ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14ahf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    slgx : float
        The value of ln(G(x)), the scaled logarithm of the gamma
        function Gamma(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` <= 0.0. On failure, the function value
            returned is zero.
    """

    from ..base.specfun import gamma_log_scaled_real as b_gamma_log_scaled_real

    _return_scalar = b_gamma_log_scaled_real(
        x,
    )
    return _return_scalar

def gamma_vector(x):
    u"""
    Gamma function, vectorized Gamma(x).

    For full information please refer to the NAG Library document for
    s14an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14anf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Gamma(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains the
          approximate value of Gamma(x_i) at the nearest valid argument.
          The threshold value is the same as for `errno` = 1 in
          ``gamma``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in ``gamma``,
          as defined in the Users' Note for your implementation.

        `ivalid`[i-1] = 3

          x_i is too close to zero. `f`[i-1] contains the approximate
          value of Gamma(x_i) at the nearest valid argument. The
          threshold value is the same as for `errno` = 2 in ``gamma``,
          as defined in the Users' Note for your implementation.

        `ivalid`[i-1] = 4

          x_i is a negative integer, at which values Gamma(x_i) are
          infinite. `f`[i-1] contains a large positive value.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import gamma_vector as b_gamma_vector

    fun_name = 'naginterfaces.library.specfun.gamma_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_gamma_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def gamma_log_real_vector(x):
    u"""
    Log gamma function, vectorized ln(Gamma(x)).

    For full information please refer to the NAG Library document for
    s14ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14apf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        ln(Gamma)(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i <= 0.

        `ivalid`[i-1] = 2

          x_i is too large and positive. The threshold value is the same
          as for `errno` = 2 in ``gamma_log_real``, as defined in the
          Users' Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import gamma_log_real_vector as b_gamma_log_real_vector

    fun_name = 'naginterfaces.library.specfun.gamma_log_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_gamma_log_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def gamma_incomplete(a, x, tol):
    u"""
    Incomplete gamma functions P(a,x) and Q(a,x).

    For full information please refer to the NAG Library document for
    s14ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14baf.html

    Parameters
    ----------
    a : float
        The argument a of the functions.

    x : float
        The argument x of the functions.

    tol : float
        The relative accuracy required by you in the results. If
        ``gamma_incomplete`` is entered with `tol` greater than 1.0 or
        less than machine precision, then the value of machine precision
        is used instead.

    Returns
    -------
    p : float
        The value of P(a,x)

    q : float
        The value of P(a,x)

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` > 0.0.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

        (`errno` 3)
            Algorithm fails to terminate in *<value>* iterations.
    """

    from ..base.specfun import gamma_incomplete as b_gamma_incomplete

    _return_scalars = b_gamma_incomplete(
        a,
        x,
        tol,
    )
    _p_py = _return_scalars.p
    _q_py = _return_scalars.q
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'q'],
    )(
        _p_py,
        _q_py,
    )
    return _return_data

def gamma_incomplete_vector(a, x, tol):
    u"""
    Incomplete gamma functions, vectorized P(a,x) and Q(a,x).

    For full information please refer to the NAG Library document for
    s14bn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14bnf.html

    Parameters
    ----------
    a : float, array-like, shape (n)
        The argument a_i of the function, for i = 1,2,...,n.

    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    tol : float
        The relative accuracy required by you in the results. If
        ``gamma_incomplete_vector`` is entered with `tol` greater than
        1.0 or less than machine precision, then the value of machine
        precision is used instead.

    Returns
    -------
    p : float, ndarray, shape (n)
        P(a_i,x_i), the function values.

    q : float, ndarray, shape (n)
        Q(a_i,x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for a_i and x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          a_i <= 0.

        `ivalid`[i-1] = 2

          x_i < 0.

        `ivalid`[i-1] = 3

          Algorithm fails to terminate.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import gamma_incomplete_vector as b_gamma_incomplete_vector

    fun_name = 'naginterfaces.library.specfun.gamma_incomplete_vector'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_gamma_incomplete_vector(
            _n,
            _a,
            _x,
            tol,
            _p,
            _q,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _p_py = _p
    _q_py = _q
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'q', 'ivalid'],
    )(
        _p_py,
        _q_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def beta_log_real(a, b):
    u"""
    Logarithm of the beta function ln(B)(a,b).

    For full information please refer to the NAG Library document for
    s14cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14cbf.html

    Parameters
    ----------
    a : float
        The argument a of the function.

    b : float
        The argument b of the function.

    Returns
    -------
    lbab : float
        The value of the logarithm of the beta function, ln(B)(a,b).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` > 0.0.
    """

    from ..base.specfun import beta_log_real as b_beta_log_real

    _return_scalar = b_beta_log_real(
        a,
        b,
    )
    return _return_scalar

def beta_incomplete(a, b, x):
    u"""
    Regularized incomplete beta function I_x(a,b) and its complement
    1-I_x.

    For full information please refer to the NAG Library document for
    s14cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14ccf.html

    Parameters
    ----------
    a : float
        The argument a of the function.

    b : float
        The argument b of the function.

    x : float
        x, upper limit of integration.

    Returns
    -------
    w : float
        The value of the incomplete beta function I_x(a,b) evaluated
        from zero to x.

    w1 : float
        The value of the complement of the incomplete beta function
        1-I_x(a,b), i.e., the incomplete beta function evaluated from x
        to one.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `b` = *<value>*.

            Constraint: `b` >= 0.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` >= 0.0.

        (`errno` 2)
            On entry, `a` and `b` were zero.

            Constraint: `a` or `b` must be nonzero.

        (`errno` 3)
            On entry, `x` = *<value>*.

            Constraint: 0.0 <= `x` <= 1.0.

        (`errno` 4)
            On entry, `x` and `a` were zero.

            Constraint: `x` or `a` must be nonzero.

        (`errno` 5)
            On entry, 1.0-`x` and `b` were zero.

            Constraint: 1.0-`x` or `b` must be nonzero.
    """

    from ..base.specfun import beta_incomplete as b_beta_incomplete

    _return_scalars = b_beta_incomplete(
        a,
        b,
        x,
    )
    _w_py = _return_scalars.w
    _w1_py = _return_scalars.w1
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['w', 'w1'],
    )(
        _w_py,
        _w1_py,
    )
    return _return_data

def beta_log_real_vector(a, b):
    u"""
    Logarithm of the beta function, vectorized ln(B)(a,b).

    For full information please refer to the NAG Library document for
    s14cp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14cpf.html

    Parameters
    ----------
    a : float, array-like, shape (n)
        The argument a_i of the function, for i = 1,2,...,n.

    b : float, array-like, shape (n)
        The argument b_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        ln(B)(a_i,b_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for a_i and b_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          a_i or b_i <= 0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `a` or `b` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import beta_log_real_vector as b_beta_log_real_vector

    fun_name = 'naginterfaces.library.specfun.beta_log_real_vector'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_beta_log_real_vector(
            _n,
            _a,
            _b,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def beta_incomplete_vector(a, b, x):
    u"""
    Regularized incomplete beta function, vectorized I_x(a,b) and its
    complement 1-I_x.

    For full information please refer to the NAG Library document for
    s14cq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s14cqf.html

    Parameters
    ----------
    a : float, array-like, shape (n)
        The argument a_i of the function, for i = 1,2,...,n.

    b : float, array-like, shape (n)
        The argument b_i of the function, for i = 1,2,...,n.

    x : float, array-like, shape (n)
        x_i, the upper limit of integration, for i = 1,2,...,n.

    Returns
    -------
    w : float, ndarray, shape (n)
        The values of the incomplete beta function I_x_i(a_i,b_i)
        evaluated from zero to x_i.

    w1 : float, ndarray, shape (n)
        The values of the complement of the incomplete beta function
        1-I_x_i(a_i,b_i), i.e., the incomplete beta function evaluated
        from x_i to one.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for the ith evaluation,
        for i = 1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          a_i or b_i < 0.

        `ivalid`[i-1] = 2

          Both a_i and b_i = 0.

        `ivalid`[i-1] = 3

          x_i∉[0,1].

        `ivalid`[i-1] = 4

          Both x_i and a_i = 0.

        `ivalid`[i-1] = 5

          Both 1-x_i and b_i = 0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one argument had an invalid value.

            Check `ivalid` for more information.
    """

    from ..base.specfun import beta_incomplete_vector as b_beta_incomplete_vector

    fun_name = 'naginterfaces.library.specfun.beta_incomplete_vector'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _w1 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_beta_incomplete_vector(
            _n,
            _a,
            _b,
            _x,
            _w,
            _w1,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _w_py = _w
    _w1_py = _w1
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['w', 'w1', 'ivalid'],
    )(
        _w_py,
        _w1_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def cdf_normal(x):
    u"""
    Cumulative Normal distribution function P(x).

    For full information please refer to the NAG Library document for
    s15ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    p : float
        The value of the cumulative Normal distribution function, P(x).
    """

    from ..base.specfun import cdf_normal as b_cdf_normal

    _return_scalar = b_cdf_normal(
        x,
    )
    return _return_scalar

def compcdf_normal(x):
    u"""
    Complement of cumulative Normal distribution function Q(x).

    For full information please refer to the NAG Library document for
    s15ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    q : float
        The value of the complement of the cumulative Normal
        distribution function, Q(x).
    """

    from ..base.specfun import compcdf_normal as b_compcdf_normal

    _return_scalar = b_compcdf_normal(
        x,
    )
    return _return_scalar

def erfc_real(x):
    u"""
    Complement of error function erfc(x).

    For full information please refer to the NAG Library document for
    s15ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    erfc : float
        The value of the complementary error function, erfc(x).
    """

    from ..base.specfun import erfc_real as b_erfc_real

    _return_scalar = b_erfc_real(
        x,
    )
    return _return_scalar

def erf_real(x):
    u"""
    Error function erf(x).

    For full information please refer to the NAG Library document for
    s15ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    efx : float
        The value of the error function erf(x).
    """

    from ..base.specfun import erf_real as b_erf_real

    _return_scalar = b_erf_real(
        x,
    )
    return _return_scalar

def dawson(x):
    u"""
    Dawson's integral.

    For full information please refer to the NAG Library document for
    s15af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15aff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    f : float
        The value of Dawson's Integral, F(x).
    """

    from ..base.specfun import dawson as b_dawson

    _return_scalar = b_dawson(
        x,
    )
    return _return_scalar

def erfcx_real(x):
    u"""
    Scaled complement of error function, erfcx(x).

    For full information please refer to the NAG Library document for
    s15ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15agf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ecx : float
        The value of the scaled complementary error function erfcx(x).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `x` = *<value>* and the constant x_hi = *<value>*.

            Constraint: `x` < x_hi.

        (`errno` 2)
            On entry, abs(`x`) was in the interval [*<value>*,*<value>*)
            where erfcx(`x`) is approximately 1/(sqrt(pi)*abs(`x`)): `x`
            = *<value>*.

        (`errno` 3)
            On entry, `x` = *<value>* and the constant x_neg =
            *<value>*.

            Constraint: `x` >= x_neg.
    """

    from ..base.specfun import erfcx_real as b_erfcx_real

    try:
        _return_scalar = b_erfcx_real(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ecx'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def cdf_normal_vector(x):
    u"""
    Cumulative Normal distribution function, vectorized P(x).

    For full information please refer to the NAG Library document for
    s15ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15apf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        P(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import cdf_normal_vector as b_cdf_normal_vector

    fun_name = 'naginterfaces.library.specfun.cdf_normal_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_cdf_normal_vector(
        _n,
        _x,
        _f,
    )
    _f_py = _f
    return _f_py

def compcdf_normal_vector(x):
    u"""
    Complement of cumulative Normal distribution function, vectorized
    Q(x).

    For full information please refer to the NAG Library document for
    s15aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15aqf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Q(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import compcdf_normal_vector as b_compcdf_normal_vector

    fun_name = 'naginterfaces.library.specfun.compcdf_normal_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_compcdf_normal_vector(
        _n,
        _x,
        _f,
    )
    _f_py = _f
    return _f_py

def erfc_real_vector(x):
    u"""
    Complement of error function, vectorized erfc(x).

    For full information please refer to the NAG Library document for
    s15ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15arf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        erfc(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import erfc_real_vector as b_erfc_real_vector

    fun_name = 'naginterfaces.library.specfun.erfc_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_erfc_real_vector(
        _n,
        _x,
        _f,
    )
    _f_py = _f
    return _f_py

def erf_real_vector(x):
    u"""
    Error function, vectorized erf(x).

    For full information please refer to the NAG Library document for
    s15as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15asf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        erf(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import erf_real_vector as b_erf_real_vector

    fun_name = 'naginterfaces.library.specfun.erf_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_erf_real_vector(
        _n,
        _x,
        _f,
    )
    _f_py = _f
    return _f_py

def dawson_vector(x):
    u"""
    Dawson's integral, vectorized.

    For full information please refer to the NAG Library document for
    s15at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15atf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        F(x_i), the function values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import dawson_vector as b_dawson_vector

    fun_name = 'naginterfaces.library.specfun.dawson_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dawson_vector(
        _n,
        _x,
        _f,
    )
    _f_py = _f
    return _f_py

def erfcx_real_vector(x):
    u"""
    Scaled complement of error function, vectorized erfcx(x).

    For full information please refer to the NAG Library document for
    s15au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15auf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        erfcx(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. The threshold value is the same
          as for `errno` = 1 in ``erfcx_real``, as defined in the Users'
          Note for your implementation.

        `ivalid`[i-1] = 2

          x_i was in the interval [1/(2sqrt(epsilon)),x_hi). The
          threshold values are the same as for `errno` = 2 in
          ``erfcx_real``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 3

          x_i is too small and positive. The threshold value is the same
          as for `errno` = 3 in ``erfcx_real``, as defined in the Users'
          Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` produced a result with
            reduced accuracy.

            Check `ivalid` for more information.
    """

    from ..base.specfun import erfcx_real_vector as b_erfcx_real_vector

    fun_name = 'naginterfaces.library.specfun.erfcx_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_erfcx_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def erfc_complex(z):
    u"""
    Scaled complex complement of error function, exp(-z^2)erfc(-iz).

    For full information please refer to the NAG Library document for
    s15dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15ddf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    Returns
    -------
    erfcz : complex
        The value of the function evaluated at z, w(z) =
        e^(-z^2)erfc(-iz).

    Raises
    ------
    NagValueError
        (`errno` 5)
            Result has no precision when entered with argument `z` =
            (*<value>*,*<value>*).

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Real part of result overflows when entered with argument `z`
            = (*<value>*,*<value>*).

        (`errno` 2)
            Imaginary part of result overflows when entered with
            argument `z` = (*<value>*,*<value>*).

        (`errno` 3)
            Both real and imaginary parts of result overflow when
            entered with argument `z` = (*<value>*,*<value>*).

        (`errno` 4)
            Result has less than half precision when entered with
            argument `z` = (*<value>*,*<value>*).
    """

    from ..base.specfun import erfc_complex as b_erfc_complex

    try:
        _return_scalar = b_erfc_complex(
            z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['erfcz'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def erfc_complex_vector(z):
    u"""
    Scaled complex complement of error function, vectorized
    exp(-z^2)erfc(-iz).

    For full information please refer to the NAG Library document for
    s15dr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s15drf.html

    Parameters
    ----------
    z : complex, array-like, shape (n)
        The argument z_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : complex, ndarray, shape (n)
        w(z_i) = e^(-z_i^2), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for z_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          Real part of result overflows.

        `ivalid`[i-1] = 2

          Imaginary part of result overflows.

        `ivalid`[i-1] = 3

          Both real and imaginary part of result overflows.

        `ivalid`[i-1] = 4

          Result has less than half precision.

        `ivalid`[i-1] = 5

          Result has no precision.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `z` produced a result with
            reduced accuracy.

            Check `ivalid` for more information.
    """

    from ..base.specfun import erfc_complex_vector as b_erfc_complex_vector

    fun_name = 'naginterfaces.library.specfun.erfc_complex_vector'

    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_erfc_complex_vector(
            _n,
            _z,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_y0_real(x):
    u"""
    Bessel function Y_0(x).

    For full information please refer to the NAG Library document for
    s17ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    y0 : float
        The value of the Bessel function Y_0(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= *<value>*.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.
    """

    from ..base.specfun import bessel_y0_real as b_bessel_y0_real

    _return_scalar = b_bessel_y0_real(
        x,
    )
    return _return_scalar

def bessel_y1_real(x):
    u"""
    Bessel function Y_1(x).

    For full information please refer to the NAG Library document for
    s17ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    y1 : float
        The value of the Bessel function Y_1(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= *<value>*.

            x is too large, the function returns the amplitude of the
            Y_1 oscillation, sqrt(2/(pi*x)).

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            Y_1 is undefined, the function returns zero.

        (`errno` 3)
            `x` is too close to zero and there is danger of overflow,
            `x` = *<value>*.

            Constraint: `x` > *<value>*.

            The function returns the value of Y_1(x) at the smallest
            valid argument.
    """

    from ..base.specfun import bessel_y1_real as b_bessel_y1_real

    _return_scalar = b_bessel_y1_real(
        x,
    )
    return _return_scalar

def bessel_j0_real(x):
    u"""
    Bessel function J_0(x).

    For full information please refer to the NAG Library document for
    s17ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    j0 : float
        The value of the Bessel function J_0(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large, the function returns the amplitude of
            the J_0 oscillation, sqrt(2/(pi*abs(x))).
    """

    from ..base.specfun import bessel_j0_real as b_bessel_j0_real

    _return_scalar = b_bessel_j0_real(
        x,
    )
    return _return_scalar

def bessel_j1_real(x):
    u"""
    Bessel function J_1(x).

    For full information please refer to the NAG Library document for
    s17af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17aff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    j1 : float
        The value of the Bessel function J_1(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large, the function returns the amplitude of
            the J_1 oscillation, sqrt(2/(pi*abs(x))).
    """

    from ..base.specfun import bessel_j1_real as b_bessel_j1_real

    _return_scalar = b_bessel_j1_real(
        x,
    )
    return _return_scalar

def airy_ai_real(x):
    u"""
    Airy function Ai(x).

    For full information please refer to the NAG Library document for
    s17ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17agf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ai : float
        The value of the Airy function, Ai(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    from ..base.specfun import airy_ai_real as b_airy_ai_real

    _return_scalar = b_airy_ai_real(
        x,
    )
    return _return_scalar

def airy_bi_real(x):
    u"""
    Airy function Bi(x).

    For full information please refer to the NAG Library document for
    s17ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17ahf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    bi : float
        The value of the Airy function, Bi(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    from ..base.specfun import airy_bi_real as b_airy_bi_real

    _return_scalar = b_airy_bi_real(
        x,
    )
    return _return_scalar

def airy_ai_deriv(x):
    u"""
    Airy function Ai'(x).

    For full information please refer to the NAG Library document for
    s17aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17ajf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    aid : float
        The value of the derivative of the Airy function Ai(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    from ..base.specfun import airy_ai_deriv as b_airy_ai_deriv

    _return_scalar = b_airy_ai_deriv(
        x,
    )
    return _return_scalar

def airy_bi_deriv(x):
    u"""
    Airy function Bi'(x).

    For full information please refer to the NAG Library document for
    s17ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17akf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    bid : float
        The value of the derivative of the Airy function Bi(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            `x` is too large and positive. The function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            `x` is too large and negative. The function returns zero.
    """

    from ..base.specfun import airy_bi_deriv as b_airy_bi_deriv

    _return_scalar = b_airy_bi_deriv(
        x,
    )
    return _return_scalar

def bessel_zeros(a, n, mode, rel=sqrt(machine.precision())):
    u"""
    Zeros of Bessel functions J_alpha(x), J'_alpha(x), Y_alpha(x) or
    Y'_alpha(x).

    For full information please refer to the NAG Library document for
    s17al

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17alf.html

    Parameters
    ----------
    a : float
        The order alpha of the function.

    n : int
        The number N of zeros required.

    mode : int
        Specifies the form of the function whose zeros are required.

        `mode` = 1

          The zeros of J_alpha(x) are required.

        `mode` = 2

          The zeros of Y_alpha(x) are required;

        `mode` = 3

          The zeros of J'_alpha(x) are required;

        `mode` = 4

          The zeros of Y'_alpha(x) are required.

    rel : float, optional
        The relative accuracy to which the zeros are required.

    Returns
    -------
    x : float, ndarray, shape (`n`)
        The N required zeros of the function specified by `mode`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `rel` = *<value>*.

            Constraint: `rel` > 0.0.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` <= 4.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` >= 1.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` <= 100000.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` >= 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.
    """

    from ..base.specfun import bessel_zeros as b_bessel_zeros

    fun_name = 'naginterfaces.library.specfun.bessel_zeros'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_bessel_zeros(
        a,
        _n,
        mode,
        rel,
        _x,
    )
    _x_py = _x
    return _x_py

def bessel_y0_real_vector(x):
    u"""
    Bessel function vectorized Y_0(x).

    For full information please refer to the NAG Library document for
    s17aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17aqf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Y_0(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the Y_0 oscillation, sqrt(2/pi x_i).

        `ivalid`[i-1] = 2

          On entry, x_i <= 0.0, Y_0 is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_y0_real_vector as b_bessel_y0_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_y0_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_y0_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_y1_real_vector(x):
    u"""
    Bessel function vectorized Y_1(x).

    For full information please refer to the NAG Library document for
    s17ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17arf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Y_1(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the Y_1 oscillation, sqrt(2/pi x_i).

        `ivalid`[i-1] = 2

          On entry, x_i <= 0.0, Y_1 is undefined. `f`[i-1] contains 0.0.

        `ivalid`[i-1] = 3

          x_i is too close to zero, there is a danger of overflow. On
          failure, `f`[i-1] contains the value of Y_1(x) at the smallest
          valid argument.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_y1_real_vector as b_bessel_y1_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_y1_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_y1_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_j0_real_vector(x):
    u"""
    Bessel function vectorized J_0(x).

    For full information please refer to the NAG Library document for
    s17as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17asf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        J_0(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the J_0 oscillation, sqrt(2/pi|x_i|).

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_j0_real_vector as b_bessel_j0_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_j0_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_j0_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_j1_real_vector(x):
    u"""
    Bessel function vectorized J_1(x).

    For full information please refer to the NAG Library document for
    s17at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17atf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        J_1(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i is too large. `f`[i-1] contains the amplitude of
          the J_1 oscillation, sqrt(2/pi|x_i|).

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_j1_real_vector as b_bessel_j1_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_j1_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_j1_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def airy_ai_real_vector(x):
    u"""
    Airy function vectorized Ai(x).

    For full information please refer to the NAG Library document for
    s17au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17auf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Ai(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_ai_real``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_ai_real``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import airy_ai_real_vector as b_airy_ai_real_vector

    fun_name = 'naginterfaces.library.specfun.airy_ai_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_airy_ai_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def airy_bi_real_vector(x):
    u"""
    Airy function vectorized Bi(x).

    For full information please refer to the NAG Library document for
    s17av

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17avf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Bi(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_bi_real``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_bi_real``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import airy_bi_real_vector as b_airy_bi_real_vector

    fun_name = 'naginterfaces.library.specfun.airy_bi_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_airy_bi_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def airy_ai_deriv_vector(x):
    u"""
    Derivatives of the Airy function, vectorized Ai'(x).

    For full information please refer to the NAG Library document for
    s17aw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17awf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Ai'(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_ai_deriv``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_ai_deriv``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import airy_ai_deriv_vector as b_airy_ai_deriv_vector

    fun_name = 'naginterfaces.library.specfun.airy_ai_deriv_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_airy_ai_deriv_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def airy_bi_deriv_vector(x):
    u"""
    Derivatives of the Airy function, vectorized Bi'(x).

    For full information please refer to the NAG Library document for
    s17ax

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17axf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        Bi'(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large and positive. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 1 in
          ``airy_bi_deriv``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i is too large and negative. `f`[i-1] contains zero. The
          threshold value is the same as for `errno` = 2 in
          ``airy_bi_deriv``, as defined in the Users' Note for your
          implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import airy_bi_deriv_vector as b_airy_bi_deriv_vector

    fun_name = 'naginterfaces.library.specfun.airy_bi_deriv_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_airy_bi_deriv_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_y_complex(fnu, z, n, scal):
    u"""
    Bessel functions Y_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s17dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dcf.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        z, the argument of the functions.

    n : int
        N, the number of members required in the sequence
        Y_nu(z),Y_{nu+1}(z),...,Y_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-|Im(z)|).

    Returns
    -------
    cy : complex, ndarray, shape (`n`)
        The N required function values: `cy`[i-1] contains
        Y_{nu+i-1}(z), for i = 1,2,...,N.

    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. The positions of such components in the array `cy`
        are arbitrary.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `z` = (0.0,0.0).

        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because abs(`z`) = *<value>* < *<value>*.

        (`errno` 2)
            No computation because Re(`z`) = *<value>* > *<value>*,
            `scal` = 'U'.

        (`errno` 3)
            No computation because `fnu`+`n`-1 = *<value>* is too large.

        (`errno` 5)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 6)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 4)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    from ..base.specfun import bessel_y_complex as b_bessel_y_complex

    fun_name = 'naginterfaces.library.specfun.bessel_y_complex'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _cy = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = b_bessel_y_complex(
            fnu,
            z,
            _n,
            scal,
            _cy,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _cy_py = _cy
    _nz_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cy', 'nz'],
    )(
        _cy_py,
        _nz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_j_complex(fnu, z, n, scal):
    u"""
    Bessel functions J_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s17de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17def.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        J_nu(z),J_{nu+1}(z),...,J_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-|Im(z)|).

    Returns
    -------
    cy : complex, ndarray, shape (`n`)
        The N required function values: `cy`[i-1] contains
        J_{nu+i-1}(z), for i = 1,2,...,N.

    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. If `nz` > 0, then elements `cy`[`n`-`nz`],
        `cy`[`n`-`nz`+1],...,`cy`[`n`-1] are set to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because Im(`z`) = *<value>* > *<value>*,
            `scal` = 'U'.

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 4)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 3)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    from ..base.specfun import bessel_j_complex as b_bessel_j_complex

    fun_name = 'naginterfaces.library.specfun.bessel_j_complex'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _cy = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = b_bessel_j_complex(
            fnu,
            z,
            _n,
            scal,
            _cy,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _cy_py = _cy
    _nz_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cy', 'nz'],
    )(
        _cy_py,
        _nz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def airy_ai_complex(deriv, z, scal):
    u"""
    Airy functions Ai(z) and Ai'(z), complex z.

    For full information please refer to the NAG Library document for
    s17dg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dgf.html

    Parameters
    ----------
    deriv : str, length 1
        Specifies whether the function or its derivative is required.

        `deriv` = 'F'

          Ai(z) is returned.

        `deriv` = 'D'

          Ai'(z) is returned.

    z : complex
        The argument z of the function.

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The result is returned unscaled.

        `scal` = 'S'

          The result is returned scaled by the factor e^(2z*sqrt(z)/3).

    Returns
    -------
    ai : complex
        The required function or derivative value.

    nz : int
        Indicates whether or not `ai` is set to zero due to underflow.
        This can only occur when `scal` = 'U'.

        `nz` = 0

          `ai` is not set to zero.

        `nz` = 1

          `ai` is set to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `deriv` has an illegal value: `deriv` = *<value>*.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 2)
            No computation because Re(omega) too large, where omega =
            (2/3)*`z`^(3/2).

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.
    """

    from ..base.specfun import airy_ai_complex as b_airy_ai_complex

    try:
        _return_scalars = b_airy_ai_complex(
            deriv,
            z,
            scal,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ai_py = _return_scalars.ai
    _nz_py = _return_scalars.nz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ai', 'nz'],
    )(
        _ai_py,
        _nz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def airy_bi_complex(deriv, z, scal):
    u"""
    Airy functions Bi(z) and Bi'(z), complex z.

    For full information please refer to the NAG Library document for
    s17dh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dhf.html

    Parameters
    ----------
    deriv : str, length 1
        Specifies whether the function or its derivative is required.

        `deriv` = 'F'

          Bi(z) is returned.

        `deriv` = 'D'

          Bi'(z) is returned.

    z : complex
        The argument z of the function.

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The result is returned unscaled.

        `scal` = 'S'

          The result is returned scaled by the factor
          e^(|Re(2z*sqrt(z)/3)|).

    Returns
    -------
    bi : complex
        The required function or derivative value.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `deriv` has an illegal value: `deriv` = *<value>*.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 2)
            No computation because Re(`z`) = *<value>* is too large when
            `scal` = 'U'.

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.
    """

    from ..base.specfun import airy_bi_complex as b_airy_bi_complex

    try:
        _return_scalar = b_airy_bi_complex(
            deriv,
            z,
            scal,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _bi_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['bi'],
    )(
        _bi_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _bi_py

def hankel_complex(m, fnu, z, n, scal):
    u"""
    Hankel functions H_nu+a^(j)(z), j = 1,2, real a >= 0, complex z,
    nu=0,1,2,....

    For full information please refer to the NAG Library document for
    s17dl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17dlf.html

    Parameters
    ----------
    m : int
        The kind of functions required.

        `m` = 1

          The functions are H_nu^(1)(z).

        `m` = 2

          The functions are H_nu^(2)(z).

    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        H_nu^(`m`)(z),H_nu+1^(`m`)(z),...,H_nu+N-1^(`m`)(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-iz) when `m`
          = 1, or by the factor e^(iz) when `m` = 2.

    Returns
    -------
    cy : complex, ndarray, shape (`n`)
        The N required function values: `cy`[i-1] contains
        H_nu+i-1^(`m`)(z), for i = 1,2,...,N.

    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. If `nz` > 0, then if Im(z) > 0.0 and `m` = 1, or
        Im(z) < 0.0 and `m` = 2, elements
        `cy`[0],`cy`[1],...,`cy`[`nz`-1] are set to zero. In the
        complementary half-planes, `nz` simply states the number of
        underflows, and not which elements they are.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `z` = (0.0,0.0).

        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `m` has illegal value: `m` = *<value>*.

        (`errno` 2)
            No computation because abs(`z`) = *<value>* < *<value>*.

        (`errno` 3)
            No computation because `fnu`+`n`-1 = *<value>* is too large.

        (`errno` 5)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 6)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 4)
            Results lack precision, `fnu`+`n`-1 = *<value>* > *<value>*.
    """

    from ..base.specfun import hankel_complex as b_hankel_complex

    fun_name = 'naginterfaces.library.specfun.hankel_complex'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _cy = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = b_hankel_complex(
            m,
            fnu,
            z,
            _n,
            scal,
            _cy,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _cy_py = _cy
    _nz_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cy', 'nz'],
    )(
        _cy_py,
        _nz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def struve_h0(x):
    u"""
    Struve function of order 0, H_0(x).

    For full information please refer to the NAG Library document for
    s17ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17gaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    h0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            `x` is too large and the function returns zero.
    """

    from ..base.specfun import struve_h0 as b_struve_h0

    _return_scalar = b_struve_h0(
        x,
    )
    return _return_scalar

def struve_h1(x):
    u"""
    Struve function of order 1, H_1(x).

    For full information please refer to the NAG Library document for
    s17gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s17gbf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    h1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            `x` is too large and the function returns zero.
    """

    from ..base.specfun import struve_h1 as b_struve_h1

    _return_scalar = b_struve_h1(
        x,
    )
    return _return_scalar

def bessel_k0_real(x):
    u"""
    Modified Bessel function K_0(x).

    For full information please refer to the NAG Library document for
    s18ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    k0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            K_0 is undefined and the function returns zero.
    """

    from ..base.specfun import bessel_k0_real as b_bessel_k0_real

    _return_scalar = b_bessel_k0_real(
        x,
    )
    return _return_scalar

def bessel_k1_real(x):
    u"""
    Modified Bessel function K_1(x).

    For full information please refer to the NAG Library document for
    s18ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    k1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            K_0 is undefined and the function returns zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > *<value>*.

            `x` is too small, there is a danger of overflow and the
            function returns approximately the largest representable
            value.
    """

    from ..base.specfun import bessel_k1_real as b_bessel_k1_real

    try:
        _return_scalar = b_bessel_k1_real(
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k1'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def bessel_i0_real(x):
    u"""
    Modified Bessel function I_0(x).

    For full information please refer to the NAG Library document for
    s18ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18aef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    i0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large and the function returns the
            approximate value of I_0(x) at the nearest valid argument.
    """

    from ..base.specfun import bessel_i0_real as b_bessel_i0_real

    _return_scalar = b_bessel_i0_real(
        x,
    )
    return _return_scalar

def bessel_i1_real(x):
    u"""
    Modified Bessel function I_1(x).

    For full information please refer to the NAG Library document for
    s18af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18aff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    i1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large and the function returns the
            approximate value of I_1(x) at the nearest valid argument.
    """

    from ..base.specfun import bessel_i1_real as b_bessel_i1_real

    _return_scalar = b_bessel_i1_real(
        x,
    )
    return _return_scalar

def bessel_k0_real_vector(x):
    u"""
    Modified Bessel function vectorized K_0(x).

    For full information please refer to the NAG Library document for
    s18aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18aqf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        K_0(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i <= 0.0, K_0(x_i) is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_k0_real_vector as b_bessel_k0_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_k0_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_k0_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_k1_real_vector(x):
    u"""
    Modified Bessel function vectorized K_1(x).

    For full information please refer to the NAG Library document for
    s18ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18arf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        K_1(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i <= 0.0, K_1(x_i) is undefined. `f`[i-1] contains 0.0.

        `ivalid`[i-1] = 2

          x_i is too small, there is a danger of overflow. `f`[i-1]
          contains zero. The threshold value is the same as for `errno`
          = 2 in ``bessel_k1_real``, as defined in the Users' Note for
          your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_k1_real_vector as b_bessel_k1_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_k1_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_k1_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_i0_real_vector(x):
    u"""
    Modified Bessel function vectorized I_0(x).

    For full information please refer to the NAG Library document for
    s18as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18asf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        I_0(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large. `f`[i-1] contains the approximate value of
          I_0(x_i) at the nearest valid argument. The threshold value is
          the same as for `errno` = 1 in ``bessel_i0_real``, as defined
          in the Users' Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_i0_real_vector as b_bessel_i0_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_i0_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_i0_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_i1_real_vector(x):
    u"""
    Modified Bessel function vectorized I_1(x).

    For full information please refer to the NAG Library document for
    s18at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18atf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        I_1(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large. `f`[i-1] contains the approximate value of
          I_1(x_i) at the nearest valid argument. The threshold value is
          the same as for `errno` = 1 in ``bessel_i1_real``, as defined
          in the Users' Note for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_i1_real_vector as b_bessel_i1_real_vector

    fun_name = 'naginterfaces.library.specfun.bessel_i1_real_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_i1_real_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_k0_scaled(x):
    u"""
    Scaled modified Bessel function e^xK_0(x).

    For full information please refer to the NAG Library document for
    s18cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18ccf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    sk0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.
    """

    from ..base.specfun import bessel_k0_scaled as b_bessel_k0_scaled

    _return_scalar = b_bessel_k0_scaled(
        x,
    )
    return _return_scalar

def bessel_k1_scaled(x):
    u"""
    Scaled modified Bessel function e^xK_1(x).

    For full information please refer to the NAG Library document for
    s18cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cdf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    sk1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            K_1 is undefined and the function returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > *<value>*.

            The function returns the value of the function at the
            smallest permitted value of the argument.
    """

    from ..base.specfun import bessel_k1_scaled as b_bessel_k1_scaled

    _return_scalar = b_bessel_k1_scaled(
        x,
    )
    return _return_scalar

def bessel_i0_scaled(x):
    u"""
    Scaled modified Bessel function e^(-|x|)I_0(x).

    For full information please refer to the NAG Library document for
    s18ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cef.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    si0 : float
        The value of the function at x.
    """

    from ..base.specfun import bessel_i0_scaled as b_bessel_i0_scaled

    _return_scalar = b_bessel_i0_scaled(
        x,
    )
    return _return_scalar

def bessel_i1_scaled(x):
    u"""
    Scaled modified Bessel function e^(-|x|)I_1(x).

    For full information please refer to the NAG Library document for
    s18cf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cff.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    si1 : float
        The value of the function at x.
    """

    from ..base.specfun import bessel_i1_scaled as b_bessel_i1_scaled

    _return_scalar = b_bessel_i1_scaled(
        x,
    )
    return _return_scalar

def bessel_k0_scaled_vector(x):
    u"""
    Scaled modified Bessel function vectorized e^xK_0(x).

    For full information please refer to the NAG Library document for
    s18cq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18cqf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        e^(x_i)K_0(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i <= 0.0, K_0(x_i) is undefined. `f`[i-1] contains
          0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_k0_scaled_vector as b_bessel_k0_scaled_vector

    fun_name = 'naginterfaces.library.specfun.bessel_k0_scaled_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_k0_scaled_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_k1_scaled_vector(x):
    u"""
    Scaled modified Bessel function vectorized e^xK_1(x).

    For full information please refer to the NAG Library document for
    s18cr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18crf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        e^(x_i)K_1(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, x_i <= 0.0, K_1(x_i) is undefined. `f`[i-1] contains
          0.0.

        `ivalid`[i-1] = 2

          x_i is too close to zero, as determined by the value of the
          safe-range parameter ``machine.real_safe``: there is a danger
          of causing overflow. `f`[i-1] contains the reciprocal of the
          safe-range parameter.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import bessel_k1_scaled_vector as b_bessel_k1_scaled_vector

    fun_name = 'naginterfaces.library.specfun.bessel_k1_scaled_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_k1_scaled_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_i0_scaled_vector(x):
    u"""
    Scaled modified Bessel function vectorized e^(-|x|)I_0(x).

    For full information please refer to the NAG Library document for
    s18cs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18csf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        e^(-|x_i|)I_0(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import bessel_i0_scaled_vector as b_bessel_i0_scaled_vector

    fun_name = 'naginterfaces.library.specfun.bessel_i0_scaled_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_i0_scaled_vector(
            _n,
            _x,
            _f,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f'],
    )(
        _f_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _f_py

def bessel_i1_scaled_vector(x):
    u"""
    Scaled modified Bessel function vectorized e^(-|x|)I_1(x).

    For full information please refer to the NAG Library document for
    s18ct

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18ctf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        e^(-|x_i|)I_1(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import bessel_i1_scaled_vector as b_bessel_i1_scaled_vector

    fun_name = 'naginterfaces.library.specfun.bessel_i1_scaled_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_bessel_i1_scaled_vector(
            _n,
            _x,
            _f,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f'],
    )(
        _f_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _f_py

def bessel_k_complex(fnu, z, n, scal):
    u"""
    Modified Bessel functions K_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s18dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18dcf.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        K_nu(z),K_{nu+1}(z),...,K_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^z.

    Returns
    -------
    cy : complex, ndarray, shape (`n`)
        The N required function values: `cy`[i-1] contains
        K_{nu+i-1}(z), for i = 1,2,...,N.

    nz : int
        The number of components of `cy` that are set to zero due to
        underflow. If `nz` > 0 and Re(z) >= 0.0, elements
        `cy`[0],`cy`[1],...,`cy`[`nz`-1] are set to zero. If Re(z) <
        0.0, `nz` simply states the number of underflows, and not which
        elements they are.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `z` = (0.0,0.0).

        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because abs(`z`) = *<value>* < *<value>*.

        (`errno` 3)
            No computation because `fnu`+`n`-1 = *<value>* is too large.

        (`errno` 5)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 5)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 6)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 4)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    from ..base.specfun import bessel_k_complex as b_bessel_k_complex

    fun_name = 'naginterfaces.library.specfun.bessel_k_complex'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _cy = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = b_bessel_k_complex(
            fnu,
            z,
            _n,
            scal,
            _cy,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _cy_py = _cy
    _nz_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cy', 'nz'],
    )(
        _cy_py,
        _nz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def bessel_i_complex(fnu, z, n, scal):
    u"""
    Modified Bessel functions I_{nu+a}(z), real a >= 0, complex z, nu =
    0,1,2,....

    For full information please refer to the NAG Library document for
    s18de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18def.html

    Parameters
    ----------
    fnu : float
        nu, the order of the first member of the sequence of functions.

    z : complex
        The argument z of the functions.

    n : int
        N, the number of members required in the sequence
        I_nu(z),I_{nu+1}(z),...,I_{nu+N-1}(z).

    scal : str, length 1
        The scaling option.

        `scal` = 'U'

          The results are returned unscaled.

        `scal` = 'S'

          The results are returned scaled by the factor e^(-|Re(z)|).

    Returns
    -------
    cy : complex, ndarray, shape (`n`)
        The N required function values: `cy`[i-1] contains
        I_{nu+i-1}(z), for i = 1,2,...,N.

    nz : int
        The number of components of `cy` that are set to zero due to
        underflow.

        If `nz` > 0, then elements
        `cy`[`n`-`nz`],`cy`[`n`-`nz`+1],...,`cy`[`n`-1] are set to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `fnu` = *<value>*.

            Constraint: `fnu` >= 0.0.

        (`errno` 1)
            On entry, `scal` has an illegal value: `scal` = *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            No computation because Re(`z`) = *<value>* > *<value>*,
            `scal` = 'U'.

        (`errno` 4)
            No computation because abs(`z`) = *<value>* > *<value>*.

        (`errno` 4)
            No computation because `fnu`+`n`-1 = *<value>* > *<value>*.

        (`errno` 5)
            No computation -- algorithm termination condition not met.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Results lack precision because abs(`z`) = *<value>* >
            *<value>*.

        (`errno` 3)
            Results lack precision because `fnu`+`n`-1 = *<value>* >
            *<value>*.
    """

    from ..base.specfun import bessel_i_complex as b_bessel_i_complex

    fun_name = 'naginterfaces.library.specfun.bessel_i_complex'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _cy = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = b_bessel_i_complex(
            fnu,
            z,
            _n,
            scal,
            _cy,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _cy_py = _cy
    _nz_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cy', 'nz'],
    )(
        _cy_py,
        _nz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def struve_l0(x):
    u"""
    Modified Struve function of order 0, L_0(x).

    For full information please refer to the NAG Library document for
    s18ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    l0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The value of L_0(x) is larger than the largest positive
            floating-point number, which is returned.
    """

    from ..base.specfun import struve_l0 as b_struve_l0

    _return_scalar = b_struve_l0(
        x,
    )
    return _return_scalar

def struve_l1(x):
    u"""
    Modified Struve function of order 1, L_1(x).

    For full information please refer to the NAG Library document for
    s18gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gbf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    l1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The value of L_1(x) is larger than the largest positive
            floating-point number, which is returned.
    """

    from ..base.specfun import struve_l1 as b_struve_l1

    _return_scalar = b_struve_l1(
        x,
    )
    return _return_scalar

def struve_i0ml0(x):
    u"""
    The function I_0(x)-L_0(x), where I_0(x) is a modified Bessel
    function and L_0(x) is a Struve function.

    For full information please refer to the NAG Library document for
    s18gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gcf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    iml0 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.
    """

    from ..base.specfun import struve_i0ml0 as b_struve_i0ml0

    _return_scalar = b_struve_i0ml0(
        x,
    )
    return _return_scalar

def struve_i1ml1(x):
    u"""
    The function I_1(x)-L_1(x), where I_1(x) is a modified Bessel
    function and L_1(x) is a Struve function.

    For full information please refer to the NAG Library document for
    s18gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gdf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    iml1 : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.
    """

    from ..base.specfun import struve_i1ml1 as b_struve_i1ml1

    _return_scalar = b_struve_i1ml1(
        x,
    )
    return _return_scalar

def bessel_j_seq_complex(z, a, nl):
    u"""
    Bessel function of the 1st kind J_{alpha+/-n}(z).

    For full information please refer to the NAG Library document for
    s18gk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s18gkf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    a : float
        The order alpha of the first member in the required sequence of
        function values.

    nl : int
        The value of N.

    Returns
    -------
    b : complex, ndarray, shape (|`nl`|+1)
        With the function exits successfully or `errno` = 3, the
        required sequence of function values: `b`[n-1] contains
        J_{alpha+n-1}(z) if `nl` >= 0 and J_{alpha-n+1}(z) otherwise,
        for n = 1,2,...,abs(`nl`)+1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(`nl`) = *<value>*.

            Constraint: abs(`nl`) <= 101.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` < 1.0.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` >= 0.0.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: when `nl` < 0, `z` != (0.0,0.0).

        (`errno` 2)
            Computation abandoned due to the likelihood of overflow.

        (`errno` 4)
            Computation abandoned due to total loss of precision.

        (`errno` 5)
            Computation abandoned due to failure to satisfy the
            termination condition.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Computation completed but some precision has been lost.
    """

    from ..base.specfun import bessel_j_seq_complex as b_bessel_j_seq_complex

    fun_name = 'naginterfaces.library.specfun.bessel_j_seq_complex'

    _nl = _b_utils._EngineIntScalarType(
        nl, {'fun_name': fun_name, 'entity': 'nl'},
    ).to_ctypes()
    _nl_py = _b_utils._EngineIntScalarType.to_py(_nl)
    _b = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (abs(_nl_py)+1,),
    )
    try:
        b_bessel_j_seq_complex(
            z,
            a,
            _nl,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _b_py = _b
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['b'],
    )(
        _b_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _b_py

def kelvin_ber(x):
    u"""
    Kelvin function ber(x).

    For full information please refer to the NAG Library document for
    s19aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19aaf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ber : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large for an accurate result to be returned
            and the function returns zero.
    """

    from ..base.specfun import kelvin_ber as b_kelvin_ber

    _return_scalar = b_kelvin_ber(
        x,
    )
    return _return_scalar

def kelvin_bei(x):
    u"""
    Kelvin function bei(x).

    For full information please refer to the NAG Library document for
    s19ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19abf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    bei : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= *<value>*.

            abs(`x`) is too large for an accurate result to be returned
            and the function returns zero.
    """

    from ..base.specfun import kelvin_bei as b_kelvin_bei

    _return_scalar = b_kelvin_bei(
        x,
    )
    return _return_scalar

def kelvin_ker(x):
    u"""
    Kelvin function ker(x).

    For full information please refer to the NAG Library document for
    s19ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    ker : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*. The function returns zero.

            Constraint: `x` <= *<value>*.

            `x` is too large, the result underflows and the function
            returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.

            The function is undefined and returns zero.
    """

    from ..base.specfun import kelvin_ker as b_kelvin_ker

    _return_scalar = b_kelvin_ker(
        x,
    )
    return _return_scalar

def kelvin_kei(x):
    u"""
    Kelvin function kei(x).

    For full information please refer to the NAG Library document for
    s19ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    kei : float
        The value of the function at x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*. The function returns zero.

            Constraint: `x` <= *<value>*.

            `x` is too large, the result underflows and the function
            returns zero.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

            The function is undefined and returns zero.
    """

    from ..base.specfun import kelvin_kei as b_kelvin_kei

    _return_scalar = b_kelvin_kei(
        x,
    )
    return _return_scalar

def kelvin_ber_vector(x):
    u"""
    Kelvin function vectorized ber(x).

    For full information please refer to the NAG Library document for
    s19an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19anf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        ber(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          abs(x_i) is too large for an accurate result to be returned.
          `f`[i-1] contains zero. The threshold value is the same as for
          `errno` = 1 in ``kelvin_ber``, as defined in the Users' Note
          for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import kelvin_ber_vector as b_kelvin_ber_vector

    fun_name = 'naginterfaces.library.specfun.kelvin_ber_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_kelvin_ber_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def kelvin_bei_vector(x):
    u"""
    Kelvin function vectorized bei(x).

    For full information please refer to the NAG Library document for
    s19ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19apf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        bei(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          abs(x_i) is too large for an accurate result to be returned.
          `f`[i-1] contains zero. The threshold value is the same as for
          `errno` = 1 in ``kelvin_bei``, as defined in the Users' Note
          for your implementation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import kelvin_bei_vector as b_kelvin_bei_vector

    fun_name = 'naginterfaces.library.specfun.kelvin_bei_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_kelvin_bei_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def kelvin_ker_vector(x):
    u"""
    Kelvin function vectorized ker(x).

    For full information please refer to the NAG Library document for
    s19aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19aqf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        ker(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large, the result underflows. `f`[i-1] contains
          zero. The threshold value is the same as for `errno` = 1 in
          ``kelvin_ker``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i <= 0.0, the function is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import kelvin_ker_vector as b_kelvin_ker_vector

    fun_name = 'naginterfaces.library.specfun.kelvin_ker_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_kelvin_ker_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def kelvin_kei_vector(x):
    u"""
    Kelvin function vectorized kei(x).

    For full information please refer to the NAG Library document for
    s19ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s19arf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        kei(x_i), the function values.

    ivalid : int, ndarray, shape (n)
        `ivalid`[i-1] contains the error code for x_i, for i =
        1,2,...,n.

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          x_i is too large, the result underflows. `f`[i-1] contains
          zero. The threshold value is the same as for `errno` = 1 in
          ``kelvin_kei``, as defined in the Users' Note for your
          implementation.

        `ivalid`[i-1] = 2

          x_i < 0.0, the function is undefined. `f`[i-1] contains 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x` was invalid.

            Check `ivalid` for more information.
    """

    from ..base.specfun import kelvin_kei_vector as b_kelvin_kei_vector

    fun_name = 'naginterfaces.library.specfun.kelvin_kei_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_kelvin_kei_vector(
            _n,
            _x,
            _f,
            _ivalid,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'ivalid'],
    )(
        _f_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def fresnel_s(x):
    u"""
    Fresnel integral S(x).

    For full information please refer to the NAG Library document for
    s20ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20acf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    sx : float
        The value of the function at x.
    """

    from ..base.specfun import fresnel_s as b_fresnel_s

    _return_scalar = b_fresnel_s(
        x,
    )
    return _return_scalar

def fresnel_c(x):
    u"""
    Fresnel integral C(x).

    For full information please refer to the NAG Library document for
    s20ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20adf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    cx : float
        The value of the function at x.
    """

    from ..base.specfun import fresnel_c as b_fresnel_c

    _return_scalar = b_fresnel_c(
        x,
    )
    return _return_scalar

def fresnel_s_vector(x):
    u"""
    Fresnel integral vectorized S(x).

    For full information please refer to the NAG Library document for
    s20aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20aqf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        S(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import fresnel_s_vector as b_fresnel_s_vector

    fun_name = 'naginterfaces.library.specfun.fresnel_s_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_fresnel_s_vector(
            _n,
            _x,
            _f,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f'],
    )(
        _f_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _f_py

def fresnel_c_vector(x):
    u"""
    Fresnel integral vectorized C(x).

    For full information please refer to the NAG Library document for
    s20ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s20arf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The argument x_i of the function, for i = 1,2,...,n.

    Returns
    -------
    f : float, ndarray, shape (n)
        C(x_i), the function values.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import fresnel_c_vector as b_fresnel_c_vector

    fun_name = 'naginterfaces.library.specfun.fresnel_c_vector'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_fresnel_c_vector(
            _n,
            _x,
            _f,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _f_py = _f
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f'],
    )(
        _f_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _f_py

def ellipint_symm_1_degen(x, y):
    u"""
    Degenerate symmetrised elliptic integral of 1st kind R_C(x,y).

    For full information please refer to the NAG Library document for
    s21ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21baf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    y : float
        The argument y of the function.

    Returns
    -------
    rc : float
        The value of an elementary integral, which occurs as a
        degenerate case of an elliptic integral of the first kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

            The function is undefined.

        (`errno` 2)
            On entry, `y` = 0.0.

            Constraint: `y` != 0.0.

            The function is undefined and returns zero.
    """

    from ..base.specfun import ellipint_symm_1_degen as b_ellipint_symm_1_degen

    _return_scalar = b_ellipint_symm_1_degen(
        x,
        y,
    )
    return _return_scalar

def ellipint_symm_1(x, y, z):
    u"""
    Symmetrised elliptic integral of 1st kind R_F(x,y,z).

    For full information please refer to the NAG Library document for
    s21bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bbf.html

    Parameters
    ----------
    x : float
        The arguments x, y and z of the function.

    y : float
        The arguments x, y and z of the function.

    z : float
        The arguments x, y and z of the function.

    Returns
    -------
    rf : float
        The value of the symmetrised elliptic integral of the first
        kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: `x` >= 0.0 and `y` >= 0.0 and `z` >= 0.0.

            The function is undefined.

        (`errno` 2)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: at most one of `x`, `y` and `z` is 0.0.

            The function is undefined and returns zero.
    """

    from ..base.specfun import ellipint_symm_1 as b_ellipint_symm_1

    _return_scalar = b_ellipint_symm_1(
        x,
        y,
        z,
    )
    return _return_scalar

def ellipint_symm_2(x, y, z):
    u"""
    Symmetrised elliptic integral of 2nd kind R_D(x,y,z).

    For full information please refer to the NAG Library document for
    s21bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bcf.html

    Parameters
    ----------
    x : float
        The arguments x, y and z of the function.

    y : float
        The arguments x, y and z of the function.

    z : float
        The arguments x, y and z of the function.

    Returns
    -------
    rd : float
        The value of the symmetrised elliptic integral of the second
        kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` and `y` are both 0.0.

            Constraint: at most one of `x` and `y` is 0.0.

            The function is undefined.

        (`errno` 1)
            On entry, `x` = *<value>* and `y` = *<value>*.

            Constraint: `x` >= 0.0 and `y` >= 0.0.

            The function is undefined.

        (`errno` 2)
            On entry, `z` = *<value>*.

            Constraint: `z` > 0.0.

            The function is undefined.

        (`errno` 3)
            On entry, L = *<value>*, `x` = *<value>*, `y` = *<value>*
            and `z` = *<value>*.

            Constraint: `z` >= L and (`x` >= L or `y` >= L).

            The function is undefined.

        (`errno` 4)
            On entry, U = *<value>*, `x` = *<value>*, `y` = *<value>*
            and `z` = *<value>*.

            Constraint: `x` < U and `y` < U and `z` < U.

            There is a danger of setting underflow and the function
            returns zero.
    """

    from ..base.specfun import ellipint_symm_2 as b_ellipint_symm_2

    _return_scalar = b_ellipint_symm_2(
        x,
        y,
        z,
    )
    return _return_scalar

def ellipint_symm_3(x, y, z, r):
    u"""
    Symmetrised elliptic integral of 3rd kind R_J(x,y,z,r).

    For full information please refer to the NAG Library document for
    s21bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bdf.html

    Parameters
    ----------
    x : float
        The arguments x, y, z and rho of the function.

    y : float
        The arguments x, y, z and rho of the function.

    z : float
        The arguments x, y, z and rho of the function.

    r : float
        The arguments x, y, z and rho of the function.

    Returns
    -------
    rj : float
        The value of the symmetrised elliptic integral of the third
        kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: at most one of `x`, `y` and `z` is 0.0.

            The function is undefined.

        (`errno` 1)
            On entry, `x` = *<value>*, `y` = *<value>* and `z` =
            *<value>*.

            Constraint: `x` >= 0.0 and `y` >= 0.0 and `z` >= 0.0.

        (`errno` 2)
            On entry, `r` = 0.0.

            Constraint: `r` != 0.0.

        (`errno` 3)
            On entry, L = *<value>*, `r` = *<value>*, `x` = *<value>*,
            `y` = *<value>* and `z` = *<value>*.

            Constraint: abs(`r`) >= L and at most one of `x`, `y` and
            `z` is less than L.

        (`errno` 4)
            On entry, U = *<value>*, `r` = *<value>*, `x` = *<value>*,
            `y` = *<value>* and `z` = *<value>*.

            Constraint: abs(`r`) <= U and `x` <= U and `y` <= U and `z`
            <= U.
    """

    from ..base.specfun import ellipint_symm_3 as b_ellipint_symm_3

    _return_scalar = b_ellipint_symm_3(
        x,
        y,
        z,
        r,
    )
    return _return_scalar

def ellipint_legendre_1(phi, dm):
    u"""
    Elliptic integral of 1st kind, Legendre form, F(phi | m).

    For full information please refer to the NAG Library document for
    s21be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bef.html

    Parameters
    ----------
    phi : float
        The arguments phi and m of the function.

    dm : float
        The arguments phi and m of the function.

    Returns
    -------
    f : float
        The value of the classical (Legendre) form of the incomplete
        elliptic integral of the first kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `phi` = *<value>*.

            Constraint: 0 <= `phi` <= pi/2.

            On failure, the function returns zero.

        (`errno` 2)
            On entry, `phi` = *<value>* and `dm` = *<value>*; the
            integral is undefined.

            Constraint: `dm`*sin^2(`phi`) <= 1.0.

            On failure, the function returns zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, sin(`phi`) = 1 and `dm` = 1.0; the integral is
            infinite.

            On failure, the function returns the largest machine number
            (see ``machine.real_largest``).
    """

    from ..base.specfun import ellipint_legendre_1 as b_ellipint_legendre_1

    try:
        _return_scalar = b_ellipint_legendre_1(
            phi,
            dm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def ellipint_legendre_2(phi, dm):
    u"""
    Elliptic integral of 2nd kind, Legendre form, E(phi | m).

    For full information please refer to the NAG Library document for
    s21bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bff.html

    Parameters
    ----------
    phi : float
        The arguments phi and m of the function.

    dm : float
        The arguments phi and m of the function.

    Returns
    -------
    e : float
        The value of the classical (Legendre) form of the incomplete
        elliptic integral of the second kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `phi` = *<value>*.

            Constraint: 0 <= `phi` <= pi/2.

        (`errno` 2)
            On entry, `phi` = *<value>* and `dm` = *<value>*; the
            integral is undefined.

            Constraint: `dm`*sin^2(`phi`) <= 1.0.
    """

    from ..base.specfun import ellipint_legendre_2 as b_ellipint_legendre_2

    _return_scalar = b_ellipint_legendre_2(
        phi,
        dm,
    )
    return _return_scalar

def ellipint_legendre_3(dn, phi, dm):
    u"""
    Elliptic integral of 3rd kind, Legendre form, Pi(n;phi | m).

    For full information please refer to the NAG Library document for
    s21bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bgf.html

    Parameters
    ----------
    dn : float
        The arguments n, phi and m of the function.

    phi : float
        The arguments n, phi and m of the function.

    dm : float
        The arguments n, phi and m of the function.

    Returns
    -------
    p : float
        The value of the classical (Legendre) form of the incomplete
        elliptic integral of the third kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `phi` = *<value>*.

            Constraint: 0 <= `phi` <= (pi/2).

        (`errno` 2)
            On entry, `phi` = *<value>* and `dm` = *<value>*; the
            integral is undefined.

            Constraint: `dm`*sin^2(`phi`) <= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, sin(`phi`) = 1 and `dm` = 1.0; the integral is
            infinite.

        (`errno` 4)
            On entry, `phi` = *<value>* and `dn` = *<value>*; the
            integral is infinite.

            Constraint: `dn`*sin^2(`phi`) != 1.0.
    """

    from ..base.specfun import ellipint_legendre_3 as b_ellipint_legendre_3

    try:
        _return_scalar = b_ellipint_legendre_3(
            dn,
            phi,
            dm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def ellipint_complete_1(dm):
    u"""
    Complete elliptic integral of 1st kind, Legendre form, K(m).

    For full information please refer to the NAG Library document for
    s21bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bhf.html

    Parameters
    ----------
    dm : float
        The argument m of the function.

    Returns
    -------
    k : float
        The value of the classical (Legendre) form of the complete
        elliptic integral of the first kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `dm` = *<value>*; the integral is undefined.

            Constraint: `dm` < 1.0.

            On failure, the function returns zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `dm` = 1.0; the integral is infinite.

            On failure, the function returns the largest machine number
            (see ``machine.real_largest``).
    """

    from ..base.specfun import ellipint_complete_1 as b_ellipint_complete_1

    try:
        _return_scalar = b_ellipint_complete_1(
            dm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['k'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def ellipint_complete_2(dm):
    u"""
    Complete elliptic integral of 2nd kind, Legendre form, E(m).

    For full information please refer to the NAG Library document for
    s21bj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21bjf.html

    Parameters
    ----------
    dm : float
        The argument m of the function.

    Returns
    -------
    e : float
        The value of the classical (Legendre) form of the complete
        elliptic integral of the second kind.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `dm` = *<value>*; the integral is undefined.

            Constraint: `dm` <= 1.0.
    """

    from ..base.specfun import ellipint_complete_2 as b_ellipint_complete_2

    _return_scalar = b_ellipint_complete_2(
        dm,
    )
    return _return_scalar

def jacellip_real(u, m):
    u"""
    Jacobian elliptic functions sn, cn and dn of real argument.

    For full information please refer to the NAG Library document for
    s21ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21caf.html

    Parameters
    ----------
    u : float
        The argument u.

    m : float
        The argument m.

    Returns
    -------
    sn : float
        The value of sn(u).

    cn : float
        The value of cn(u).

    dn : float
        The value of dn(u).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(`u`) is too large: abs(`u`) = *<value>* it
            must be less than *<value>*.

        (`errno` 2)
            On entry, abs(`m`) is too large when used in conjunction
            with the supplied argument `u`: abs(`m`) = *<value>* it must
            be less than *<value>*.
    """

    from ..base.specfun import jacellip_real as b_jacellip_real

    _return_scalars = b_jacellip_real(
        u,
        m,
    )
    _sn_py = _return_scalars.sn
    _cn_py = _return_scalars.cn
    _dn_py = _return_scalars.dn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['sn', 'cn', 'dn'],
    )(
        _sn_py,
        _cn_py,
        _dn_py,
    )
    return _return_data

def jacellip_complex(z, ak2):
    u"""
    Jacobian elliptic functions sn, cn and dn of complex argument.

    For full information please refer to the NAG Library document for
    s21cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21cbf.html

    Parameters
    ----------
    z : complex
        The argument z of the functions.

    ak2 : float
        The value of k^2.

    Returns
    -------
    sn : complex
        The value of the function sn(z).

    cn : complex
        The value of the function cn(z).

    dn : complex
        The value of the function dn(z).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(Im(`z`)) is too large: abs(Im(`z`)) =
            *<value>*. It must be less than *<value>*.

        (`errno` 1)
            On entry, abs(Re(`z`)) is too large: abs(Re(`z`)) =
            *<value>*. It must be less than *<value>*.

        (`errno` 1)
            On entry, `ak2` = *<value>*.

            Constraint: `ak2` <= 1.0.

        (`errno` 1)
            On entry, `ak2` = *<value>*.

            Constraint: `ak2` >= 0.0.
    """

    from ..base.specfun import jacellip_complex as b_jacellip_complex

    _return_scalars = b_jacellip_complex(
        z,
        ak2,
    )
    _sn_py = _return_scalars.sn
    _cn_py = _return_scalars.cn
    _dn_py = _return_scalars.dn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['sn', 'cn', 'dn'],
    )(
        _sn_py,
        _cn_py,
        _dn_py,
    )
    return _return_data

def jactheta_real(k, x, q):
    u"""
    Jacobian theta functions theta_k(x,q) of real argument.

    For full information please refer to the NAG Library document for
    s21cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21ccf.html

    Parameters
    ----------
    k : int
        Denotes the function theta_k(x,q) to be evaluated. Note that `k`
        = 4 is equivalent to `k` = 0.

    x : float
        The argument x of the function.

    q : float
        The argument q of the function.

    Returns
    -------
    jt : float
        The value of one of the Jacobian theta functions theta_0(x,q),
        theta_1(x,q), theta_2(x,q), theta_3(x,q) or theta_4(x,q) for a
        real argument x and non-negative q < 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `q` = *<value>*.

            Constraint: `q` < 1.0.

        (`errno` 1)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` <= 4.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.
    """

    from ..base.specfun import jactheta_real as b_jactheta_real

    _return_scalar = b_jactheta_real(
        k,
        x,
        q,
    )
    return _return_scalar

def ellipint_general_2(z, akp, a, b):
    u"""
    General elliptic integral of 2nd kind F(z,k',a,b) of complex
    argument.

    For full information please refer to the NAG Library document for
    s21da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s21daf.html

    Parameters
    ----------
    z : complex
        The argument z of the function.

    akp : float
        The argument k' of the function.

    a : float
        The argument a of the function.

    b : float
        The argument b of the function.

    Returns
    -------
    f : complex
        The value of the general elliptic integral of the second kind
        F(z,k',a,b) for a complex argument z.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, abs(Im(`z`)) is too large: abs(Im(`z`)) =
            *<value>*. It must not exceed *<value>*.

        (`errno` 1)
            On entry, Re(`z`) is too large: Re(`z`) = *<value>*. It must
            not exceed *<value>*.

        (`errno` 1)
            On entry, Re(`z`) < 0.0: Re(`z`) = *<value>*.

        (`errno` 1)
            On entry, abs(`akp`) is too large: abs(`akp`) = *<value>*.
            It must not exceed *<value>*.

        (`errno` 2)
            The iterative procedure used to evaluate the integral has
            failed to converge.
    """

    from ..base.specfun import ellipint_general_2 as b_ellipint_general_2

    _return_scalar = b_ellipint_general_2(
        z,
        akp,
        a,
        b,
    )
    return _return_scalar

def legendre_p(mode, x, m, nl):
    u"""
    Legendre functions of 1st kind P_n^m(x) or -P_n^m - (x).

    For full information please refer to the NAG Library document for
    s22aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22aaf.html

    Parameters
    ----------
    mode : int
        Indicates whether the sequence of function values is to be
        returned unnormalized or normalized.

        `mode` = 1

          The sequence of function values is returned unnormalized.

        `mode` = 2

          The sequence of function values is returned normalized.

    x : float
        The argument x of the function.

    m : int
        The order m of the function.

    nl : int
        The degree N of the last function required in the sequence.

    Returns
    -------
    p : float, ndarray, shape (`nl`+1)
        The required sequence of function values as follows:

        - if `mode` = 1, `p`[n] contains P_n^m(x), for n = 0,1,...,N;

        - if `mode` = 2, `p`[n] contains - P_n^m - (x), for n =
          0,1,...,N.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nl` = *<value>* and abs(`m`) = *<value>*.

            Constraint: `nl`+abs(`m`) <= 55.

        (`errno` 1)
            On entry, abs(`m`) = *<value>*.

            Constraint: abs(`m`) <= 27.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: when abs(`m`) != 0, `nl` >= 0.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: `nl` <= 100 when `m` = 0.

        (`errno` 1)
            On entry, `nl` = *<value>*.

            Constraint: `nl` >= 0.

        (`errno` 1)
            On entry, abs(`x`) = *<value>*.

            Constraint: abs(`x`) <= 1.0.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` <= 2.

        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` >= 1.
    """

    from ..base.specfun import legendre_p as b_legendre_p

    fun_name = 'naginterfaces.library.specfun.legendre_p'

    _nl = _b_utils._EngineIntScalarType(
        nl, {'fun_name': fun_name, 'entity': 'nl'},
    ).to_ctypes()
    _nl_py = _b_utils._EngineIntScalarType.to_py(_nl)
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nl_py+1,),
    )
    b_legendre_p(
        mode,
        x,
        m,
        _nl,
        _p,
    )
    _p_py = _p
    return _p_py

def hyperg_confl_real(a, b, x):
    u"""
    Real confluent hypergeometric function 1_F_1(a;b;x).

    For full information please refer to the NAG Library document for
    s22ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22baf.html

    Parameters
    ----------
    a : float
        The parameter a of the function.

    b : float
        The parameter b of the function.

    x : float
        The argument x of the function.

    Returns
    -------
    m : float
        The solution M(a,b,x).

        Note: if overflow occurs upon completion, as indicated by
        `errno` = 2, |M(a,b,x)| may be assumed to be too large to be
        representable. `m` will be returned as +/-R_max, where R_max is
        the largest representable real number (see
        ``machine.real_largest``). The sign of `m` should match the sign
        of M(a,b,x). If overflow occurs during a subcalculation, as
        indicated by `errno` = 5, the sign may be incorrect, and the
        true value of M(a,b,x) may or may not be greater than R_max. In
        either case it is advisable to subsequently use
        ``hyperg_confl_real_scaled``.

    Raises
    ------
    NagValueError
        (`errno` 4)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 5)
            Overflow occurred in a subcalculation of M(a,b,x).

            The answer may be completely incorrect.

        (`errno` 11)
            On entry, `a` = *<value>*.

            Constraint: abs(`a`) <= arbnd = *<value>*.

        (`errno` 31)
            On entry, `b` = *<value>*.

            Constraint: abs(`b`) <= arbnd = *<value>*.

        (`errno` 32)
            On entry, `b` = *<value>*.

            M(a,b,x) is undefined when b is zero or a negative integer.

        (`errno` 51)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= arbnd = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of M(a,b,x).

            The returned value may be inaccurate.

        (`errno` 2)
            On completion, overflow occurred in the evaluation of
            M(a,b,x).

        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.
    """

    from ..base.specfun import hyperg_confl_real as b_hyperg_confl_real

    try:
        _return_scalar = b_hyperg_confl_real(
            a,
            b,
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _m_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['m'],
    )(
        _m_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _m_py

def hyperg_confl_real_scaled(ani, adr, bni, bdr, x):
    u"""
    Real confluent hypergeometric function 1_F_1(a;b;x) in scaled form.

    For full information please refer to the NAG Library document for
    s22bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22bbf.html

    Parameters
    ----------
    ani : float
        a_i, the nearest integer to a, satisfying a_i = a-a_r.

    adr : float
        a_r, the signed decimal remainder satisfying a_r = a-a_i and
        |a_r| <= 0.5.

    bni : float
        b_i, the nearest integer to b, satisfying b_i = b-b_r.

    bdr : float
        b_r, the signed decimal remainder satisfying b_r = b-b_i and
        |b_r| <= 0.5.

    x : float
        The argument x of the function.

    Returns
    -------
    frm : float
        m_f, the scaled real component of the solution satisfying m_f =
        M(a,b,x)*2^(-m_s).

        Note: if overflow occurs upon completion, as indicated by
        `errno` = 2, the value of m_f returned may still be correct. If
        overflow occurs in a subcalculation, as indicated by `errno` =
        5, this should not be assumed.

    scm : int
        m_s, the scaling power of two, satisfying m_s =
        log2(M(a,b,x)/(m_f)).

        Note: if overflow occurs upon completion, as indicated by
        `errno` = 2, then m_s >= I_max, where I_max is the largest
        representable integer (see ``machine.integer_max``). If overflow
        occurs during a subcalculation, as indicated by `errno` = 5, m_s
        may or may not be greater than I_max. In either case, `scm` =
        machine.integer_max will have been returned.

    Raises
    ------
    NagValueError
        (`errno` 4)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 5)
            Overflow occurred in a subcalculation of M(a,b,x).

            The answer may be completely incorrect.

        (`errno` 11)
            On entry, `ani` = *<value>*.

            Constraint: abs(`ani`) <= arbnd = *<value>*.

        (`errno` 13)
            `ani` is non-integral.

            On entry, `ani` = *<value>*.

            Constraint: `ani` = floor(`ani`).

        (`errno` 21)
            On entry, `adr` = *<value>*.

            Constraint: abs(`adr`) <= 0.5.

        (`errno` 31)
            On entry, `bni` = *<value>*.

            Constraint: abs(`bni`) <= arbnd = *<value>*.

        (`errno` 32)
            On entry, b = `bni`+`bdr` = *<value>*.

            M(a,b,x) is undefined when b is zero or a negative integer.

        (`errno` 33)
            `bni` is non-integral.

            On entry, `bni` = *<value>*.

            Constraint: `bni` = floor(`bni`).

        (`errno` 41)
            On entry, `bdr` = *<value>*.

            Constraint: abs(`bdr`) <= 0.5.

        (`errno` 51)
            On entry, `x` = *<value>*.

            Constraint: abs(`x`) <= arbnd = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of M(a,b,x).

            The returned value may be inaccurate.

        (`errno` 2)
            On completion, overflow occurred in the evaluation of
            M(a,b,x).

        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.
    """

    from ..base.specfun import hyperg_confl_real_scaled as b_hyperg_confl_real_scaled

    try:
        _return_scalars = b_hyperg_confl_real_scaled(
            ani,
            adr,
            bni,
            bdr,
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _frm_py = _return_scalars.frm
    _scm_py = _return_scalars.scm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['frm', 'scm'],
    )(
        _frm_py,
        _scm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def hyperg_gauss_real(a, b, c, x):
    u"""
    Real Gauss hypergeometric function 2_F_1(a,b;c;x).

    For full information please refer to the NAG Library document for
    s22be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22bef.html

    Parameters
    ----------
    a : float
        The parameter a.

    b : float
        The parameter b.

    c : float
        The parameter c.

    x : float
        The argument x.

    Returns
    -------
    f21 : float
        The value returned for the Gauss hypergeometric function
        2_F_1(a,b;c;x) for real parameters a,b and c, and real argument
        x.

    Raises
    ------
    NagValueError
        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 4)
            On entry, `x` = *<value>*, c = *<value>*, a+b = *<value>*.

            2_F_1(a,b;c;1) is infinite in the case c <= a+b.

        (`errno` 6)
            Overflow occurred in a subcalculation of 2_F_1(a,b;c;x). The
            result may or may not be infinite.

        (`errno` 9)
            An internal calculation has resulted in an undefined result.

        (`errno` 11)
            On entry, `a` does not satisfy abs(`a`) <= arbnd =
            *<value>*.

        (`errno` 21)
            On entry, `b` does not satisfy abs(`b`) <= arbnd =
            *<value>*.

        (`errno` 31)
            On entry, `c` does not satisfy abs(`c`) <= arbnd =
            *<value>*.

        (`errno` 32)
            On entry, `c` = *<value>*.

            2_F_1(a,b;c;x) is undefined when c is zero or a negative
            integer.

        (`errno` 41)
            On entry, `x` does not satisfy abs(`x`) <= arbnd =
            *<value>*.

        (`errno` 42)
            On entry, `x` = *<value>*.

            In general, 2_F_1(a,b;c;x) is not real valued when x > 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of 2_F_1(a,b;c;x).
            The returned value may be inaccurate.

        (`errno` 2)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.

        (`errno` 5)
            On completion, overflow occurred in the evaluation of
            2_F_1(a,b;c;x).
    """

    from ..base.specfun import hyperg_gauss_real as b_hyperg_gauss_real

    try:
        _return_scalar = b_hyperg_gauss_real(
            a,
            b,
            c,
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f21'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def hyperg_gauss_real_scaled(ani, adr, bni, bdr, cni, cdr, x):
    u"""
    Real Gauss hypergeometric function 2_F_1(a,b;c;x) in scaled form.

    For full information please refer to the NAG Library document for
    s22bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22bff.html

    Parameters
    ----------
    ani : float
        a_i, the nearest integer to a, satisfying a_i = a-a_r.

    adr : float
        a_r, the signed decimal remainder satisfying a_r = a-a_i and
        |a_r| <= 0.5.

    bni : float
        b_i, the nearest integer to b, satisfying b_i = b-b_r.

    bdr : float
        b_r, the signed decimal remainder satisfying b_r = b-b_i and
        |b_r| <= 0.5.

    cni : float
        c_i, the nearest integer to c, satisfying c_i = c-c_r.

    cdr : float
        c_r, the signed decimal remainder satisfying c_r = c-c_i and
        |c_r| <= 0.5.

    x : float
        The argument x.

    Returns
    -------
    frf : float
        f_fr, the scaled real component of the solution satisfying f_fr
        = 2_F_1(a,b;c;x)*2^(-f_sc), i.e., 2_F_1(a,b;c;x) =
        f_fr*2^(f_sc). See [Further Comments] for the behaviour of f_fr
        when a finite or non-finite answer is returned.

    scf : int
        f_sc, the scaling power of two, satisfying f_sc =
        log2(2_F_1(a,b;c;x)/f_fr), i.e., 2_F_1(a,b;c;x) = f_fr*2^(f_sc).
        See [Further Comments] for the behaviour of f_sc when a
        non-finite answer is returned.

    Raises
    ------
    NagValueError
        (`errno` 3)
            All approximations have completed, and the final residual
            estimate indicates no accuracy can be guaranteed.

            Relative residual = *<value>*.

        (`errno` 4)
            On entry, `x` = *<value>*, c = *<value>*, a+b = *<value>*.

            2_F_1(a,b;c;1) is infinite in the case c <= a+b.

        (`errno` 6)
            Overflow occurred in a subcalculation of 2_F_1(a,b;c;x). The
            answer may be completely incorrect.

        (`errno` 9)
            An internal calculation has resulted in an undefined result.

        (`errno` 11)
            On entry, `ani` does not satisfy abs(`ani`) <= arbnd =
            *<value>*.

        (`errno` 13)
            `ani` is non-integral.

            On entry, `ani` = *<value>*.

            Constraint: `ani` = floor(`ani`).

        (`errno` 21)
            On entry, `adr` does not satisfy abs(`adr`) <= 0.5.

        (`errno` 31)
            On entry, `bni` does not satisfy abs(`bni`) <= arbnd =
            *<value>*.

        (`errno` 33)
            `bni` is non-integral.

            On entry, `bni` = *<value>*.

            Constraint: `bni` = floor(`bni`).

        (`errno` 41)
            On entry, `bdr` does not satisfy abs(`bdr`) <= 0.5.

        (`errno` 51)
            On entry, `cni` does not satisfy abs(`cni`) <= arbnd =
            *<value>*.

        (`errno` 52)
            On entry, c = `cni`+`cdr` = *<value>*.

            2_F_1(a,b;c;x) is undefined when c is zero or a negative
            integer.

        (`errno` 53)
            `cni` is non-integral.

            On entry, `cni` = *<value>*.

            Constraint: `cni` = floor(`cni`).

        (`errno` 61)
            On entry, `cdr` does not satisfy abs(`cdr`) <= 0.5.

        (`errno` 71)
            On entry, `x` does not satisfy abs(`x`) <= arbnd =
            *<value>*.

        (`errno` 72)
            On entry, `x` = *<value>*.

            In general, 2_F_1(a,b;c;x) is not real valued when x > 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Underflow occurred during the evaluation of 2_F_1(a,b;c;x).
            The returned value may be inaccurate.

        (`errno` 2)
            All approximations have completed, and the final residual
            estimate indicates some precision may have been lost.

            Relative residual = *<value>*.

        (`errno` 5)
            On completion, overflow occurred in the evaluation of
            2_F_1(a,b;c;x).
    """

    from ..base.specfun import hyperg_gauss_real_scaled as b_hyperg_gauss_real_scaled

    try:
        _return_scalars = b_hyperg_gauss_real_scaled(
            ani,
            adr,
            bni,
            bdr,
            cni,
            cdr,
            x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _frf_py = _return_scalars.frf
    _scf_py = _return_scalars.scf
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['frf', 'scf'],
    )(
        _frf_py,
        _scf_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def mathieu_ang_periodic_real(ordval, q, parity, mode, x=None):
    u"""
    Calculates values of real periodic angular Mathieu functions.

    For full information please refer to the NAG Library document for
    s22ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s22caf.html

    Parameters
    ----------
    ordval : int
        m, the order number of the Mathieu function to be computed.

    q : float
        q, the Mathieu function parameter.

    parity : int
        Specifies whether to compute even or odd Mathieu function.

        `parity` = 0

          Compute even Mathieu function, ce_m(x,q).

        `parity` = 1

          Compute odd Mathieu function, se_m(x,q).

    mode : int
        Specifies whether the Mathieu function or its derivative is
        required.

        `mode` = 0

          Compute Mathieu function values.

        `mode` = 1

          Compute derivative values of Mathieu function.

        `mode` = 2

          Compute both Mathieu function and derivative values.

        `mode` = 3

          Compute neither Mathieu functions nor derivative values,
          returns only `a` (the characteristic value).

    x : None or float, array-like, shape (n), optional
        The values of x at which to compute Mathieu function or
        derivative values.

    Returns
    -------
    f : None or float, ndarray, shape (n)
        If `mode` = 0 or 2, the Mathieu function values ce_m(x,q) or
        se_m(x,q). If `mode` = 1 or 3, `f` is not used.

    f_deriv : None or float, ndarray, shape (n)
        If `mode` = 1 or 2, the Mathieu function derivative values
        ce'_m(x,q) or se'_m(x,q). If `mode` = 0 or 3, `f_deriv` is not
        used.

    a : float
        a, the characteristic value for the Mathieu function.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ordval` = *<value>* and `parity` = *<value>*.

            Constraint: if `parity` = 0, `ordval` >= 0 or if `parity` =
            1, `ordval` >= 1.

        (`errno` 2)
            On entry, `parity` = *<value>*.

            Constraint: `parity` = 0 or 1.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 0, 1, 2 or 3.

        (`errno` 4)
            On entry, n = *<value>*.

            Constraint: n >= 0.
    """

    from ..base.specfun import mathieu_ang_periodic_real as b_mathieu_ang_periodic_real

    fun_name = 'naginterfaces.library.specfun.mathieu_ang_periodic_real'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
        allow_none=True,
    )
    _n = _b_utils._EngineIntScalarType(
        0 if x is None else _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1, allow_none=True)[0], {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _mode = _b_utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _mode_py = _b_utils._EngineIntScalarType.to_py(_mode)
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    if _mode_py in (0, 2):
        _f = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_n_py,),
        )
    else:
        _f = None
    if _mode_py in (1, 2):
        _f_deriv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_n_py,),
        )
    else:
        _f_deriv = None
    _return_scalar = b_mathieu_ang_periodic_real(
        ordval,
        q,
        parity,
        _mode,
        _n,
        _x,
        _f,
        _f_deriv,
    )
    if _mode_py in (0, 2):
        _f_py = _f
    else:
        _f_py = None
    if _mode_py in (1, 2):
        _f_deriv_py = _f_deriv
    else:
        _f_deriv_py = None
    _a_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['f', 'f_deriv', 'a'],
    )(
        _f_py,
        _f_deriv_py,
        _a_py,
    )
    return _return_data

def opt_bsm_price(calput, x, s, t, sigma, r, q):
    u"""
    Black--Scholes--Merton option pricing formula.

    For full information please refer to the NAG Library document for
    s30aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30aaf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

    See Also
    --------
    :meth:`naginterfaces.library.examples.roots.contfn_brent_interval_ex.main`
    :meth:`naginterfaces.library.examples.specfun.opt_bsm_price_ex.main`
    """

    from ..base.specfun import opt_bsm_price as b_opt_bsm_price

    fun_name = 'naginterfaces.library.specfun.opt_bsm_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_bsm_price(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        q,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_bsm_greeks(calput, x, s, t, sigma, r, q):
    u"""
    Black--Scholes--Merton option pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30abf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    delta : float, ndarray, shape (m, n)
        The leading m*n part of the array `delta` contains the
        sensitivity, dP/dS, of the option price to change in the price
        of the underlying asset.

    gamma : float, ndarray, shape (m, n)
        The leading m*n part of the array `gamma` contains the
        sensitivity, d^2P/dS^2, of `delta` to change in the price of the
        underlying asset.

    vega : float, ndarray, shape (m, n)
        `vega`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the
        volatility of the underlying asset, i.e., dP_{ij}/d sigma, for i
        = 1,2,...,m and j = 1,2,...,n.

    theta : float, ndarray, shape (m, n)
        `theta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in time, i.e.,
        -dP_{ij}/dT, for i = 1,2,...,m and j = 1,2,...,n, where b = r-q.

    rho : float, ndarray, shape (m, n)
        `rho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        risk-free interest rate, i.e., -dP_{ij}/dr, for i = 1,2,...,m
        and j = 1,2,...,n.

    crho : float, ndarray, shape (m, n)
        `crho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        cost of carry rate, i.e., -dP_{ij}/db, for i = 1,2,...,m and j =
        1,2,...,n, where b = r-q.

    vanna : float, ndarray, shape (m, n)
        `vanna`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the asset price, i.e., -d Delta_{ij}/dT =
        -d^2P_{ij}/(dSd sigma), for i = 1,2,...,m and j = 1,2,...,n.

    charm : float, ndarray, shape (m, n)
        `charm`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    speed : float, ndarray, shape (m, n)
        `speed`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the price of the underlying asset, i.e., -d Gamma_{ij}/dS =
        -d^3P_{ij}/dS^3, for i = 1,2,...,m and j = 1,2,...,n.

    colour : float, ndarray, shape (m, n)
        `colour`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the time, i.e., -d Gamma_{ij}/dT = -d^3P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    zomma : float, ndarray, shape (m, n)
        `zomma`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the volatility of the underlying asset, i.e., -d Gamma_{ij}/d
        sigma = -d^3P_{ij}/(dS^2d sigma), for i = 1,2,...,m and j =
        1,2,...,n.

    vomma : float, ndarray, shape (m, n)
        `vomma`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the underlying asset, i.e., -d Delta_{ij}/d sigma
        = -d^2P_{ij}/d sigma^2, for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    from ..base.specfun import opt_bsm_greeks as b_opt_bsm_greeks

    fun_name = 'naginterfaces.library.specfun.opt_bsm_greeks'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _delta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _gamma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vega = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _rho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _crho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vanna = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _charm = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _speed = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _colour = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _zomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_bsm_greeks(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        q,
        _p,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
    )
    _p_py = _p
    _delta_py = _delta
    _gamma_py = _gamma
    _vega_py = _vega
    _theta_py = _theta
    _rho_py = _rho
    _crho_py = _crho
    _vanna_py = _vanna
    _charm_py = _charm
    _speed_py = _speed
    _colour_py = _colour
    _zomma_py = _zomma
    _vomma_py = _vomma
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'delta', 'gamma', 'vega', 'theta', 'rho', 'crho', 'vanna', 'charm', 'speed', 'colour', 'zomma', 'vomma'],
    )(
        _p_py,
        _delta_py,
        _gamma_py,
        _vega_py,
        _theta_py,
        _rho_py,
        _crho_py,
        _vanna_py,
        _charm_py,
        _speed_py,
        _colour_py,
        _zomma_py,
        _vomma_py,
    )
    return _return_data

def opt_lookback_fls_price(calput, sm, s, t, sigma, r, q):
    u"""
    Floating-strike lookback option pricing formula in the
    Black-Scholes-Merton model.

    For full information please refer to the NAG Library document for
    s30ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30baf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    sm : float, array-like, shape (m)
        `sm`[i-1] must contain S_min(i), the ith minimum observed price
        of the underlying asset when `calput` = 'C', or S_max(i), the
        maximum observed price when `calput` = 'P', for i = 1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        minimum or maximum observed price S_min(i) or S_max(i) at expiry
        `t`_j for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry with a put option, `sm`[*<value>*] = *<value>*.

            Constraint: for put options, `sm`[i] >= *<value>* for all i.

        (`errno` 4)
            On entry with a call option, `sm`[*<value>*] = *<value>*.

            Constraint: for call options, `sm`[i] <= *<value>* for all
            i.

        (`errno` 4)
            On entry, `sm`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `sm`[i] <= *<value>* for all i.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>* for all i.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    from ..base.specfun import opt_lookback_fls_price as b_opt_lookback_fls_price

    fun_name = 'naginterfaces.library.specfun.opt_lookback_fls_price'

    _sm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'sm'},
        sm,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_sm, {'fun_name': fun_name, 'entity': 'sm'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_lookback_fls_price(
        calput,
        _m,
        _n,
        _sm,
        s,
        _t,
        sigma,
        r,
        q,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_lookback_fls_greeks(calput, sm, s, t, sigma, r, q):
    u"""
    Floating-strike lookback option pricing formula with Greeks in the
    Black-Scholes-Merton model.

    For full information please refer to the NAG Library document for
    s30bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30bbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    sm : float, array-like, shape (m)
        `sm`[i-1] must contain S_min(i), the ith minimum observed price
        of the underlying asset when `calput` = 'C', or S_max(i), the
        maximum observed price when `calput` = 'P', for i = 1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        The annual risk-free interest rate, r, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        The annual continuous yield rate. Note that a rate of 8% should
        be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        minimum or maximum observed price S_min(i) or S_max(i) at expiry
        `t`_j for i = 1,2,...,m and j = 1,2,...,n.

    delta : float, ndarray, shape (m, n)
        The leading m*n part of the array `delta` contains the
        sensitivity, dP/dS, of the option price to change in the price
        of the underlying asset.

    gamma : float, ndarray, shape (m, n)
        The leading m*n part of the array `gamma` contains the
        sensitivity, d^2P/dS^2, of `delta` to change in the price of the
        underlying asset.

    vega : float, ndarray, shape (m, n)
        `vega`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the
        volatility of the underlying asset, i.e., dP_{ij}/d sigma, for i
        = 1,2,...,m and j = 1,2,...,n.

    theta : float, ndarray, shape (m, n)
        `theta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in time, i.e.,
        -dP_{ij}/dT, for i = 1,2,...,m and j = 1,2,...,n, where b = r-q.

    rho : float, ndarray, shape (m, n)
        `rho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        risk-free interest rate, i.e., -dP_{ij}/dr, for i = 1,2,...,m
        and j = 1,2,...,n.

    crho : float, ndarray, shape (m, n)
        `crho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        cost of carry rate, i.e., -dP_{ij}/db, for i = 1,2,...,m and j =
        1,2,...,n, where b = r-q.

    vanna : float, ndarray, shape (m, n)
        `vanna`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the asset price, i.e., -d Delta_{ij}/dT =
        -d^2P_{ij}/(dSd sigma), for i = 1,2,...,m and j = 1,2,...,n.

    charm : float, ndarray, shape (m, n)
        `charm`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    speed : float, ndarray, shape (m, n)
        `speed`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the price of the underlying asset, i.e., -d Gamma_{ij}/dS =
        -d^3P_{ij}/dS^3, for i = 1,2,...,m and j = 1,2,...,n.

    colour : float, ndarray, shape (m, n)
        `colour`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the time, i.e., -d Gamma_{ij}/dT = -d^3P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    zomma : float, ndarray, shape (m, n)
        `zomma`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the volatility of the underlying asset, i.e., -d Gamma_{ij}/d
        sigma = -d^3P_{ij}/(dS^2d sigma), for i = 1,2,...,m and j =
        1,2,...,n.

    vomma : float, ndarray, shape (m, n)
        `vomma`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the underlying asset, i.e., -d Delta_{ij}/d sigma
        = -d^2P_{ij}/d sigma^2, for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry with a put option, `sm`[*<value>*] = *<value>*.

            Constraint: for put options, `sm`[i] >= *<value>* for all i.

        (`errno` 4)
            On entry with a call option, `sm`[*<value>*] = *<value>*.

            Constraint: for call options, `sm`[i] <= *<value>* for all
            i.

        (`errno` 4)
            On entry, `sm`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `sm`[i] <= *<value>* for all i.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>* for all i.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 12)
            On entry, `r` = *<value>* and `q` = *<value>*.

            Constraint: abs(`r`-`q`) > 10*eps*max(abs(`r`),1), where eps
            is the machine precision.
    """

    from ..base.specfun import opt_lookback_fls_greeks as b_opt_lookback_fls_greeks

    fun_name = 'naginterfaces.library.specfun.opt_lookback_fls_greeks'

    _sm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'sm'},
        sm,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_sm, {'fun_name': fun_name, 'entity': 'sm'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _delta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _gamma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vega = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _rho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _crho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vanna = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _charm = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _speed = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _colour = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _zomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_lookback_fls_greeks(
        calput,
        _m,
        _n,
        _sm,
        s,
        _t,
        sigma,
        r,
        q,
        _p,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
    )
    _p_py = _p
    _delta_py = _delta
    _gamma_py = _gamma
    _vega_py = _vega
    _theta_py = _theta
    _rho_py = _rho
    _crho_py = _crho
    _vanna_py = _vanna
    _charm_py = _charm
    _speed_py = _speed
    _colour_py = _colour
    _zomma_py = _zomma
    _vomma_py = _vomma
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'delta', 'gamma', 'vega', 'theta', 'rho', 'crho', 'vanna', 'charm', 'speed', 'colour', 'zomma', 'vomma'],
    )(
        _p_py,
        _delta_py,
        _gamma_py,
        _vega_py,
        _theta_py,
        _rho_py,
        _crho_py,
        _vanna_py,
        _charm_py,
        _speed_py,
        _colour_py,
        _zomma_py,
        _vomma_py,
    )
    return _return_data

def opt_binary_con_price(calput, x, s, k, t, sigma, r, q):
    u"""
    Binary option, cash-or-nothing pricing formula.

    For full information please refer to the NAG Library document for
    s30ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30caf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    k : float
        The amount, K, to be paid at expiration if the option is
        in-the-money, i.e., if `s` > `x`[i-1] when `calput` = 'C', or if
        `s` < `x`[i-1] when `calput` = 'P', for i = 1,2,...,m.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.0.

        (`errno` 7)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 8)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 9)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 10)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    from ..base.specfun import opt_binary_con_price as b_opt_binary_con_price

    fun_name = 'naginterfaces.library.specfun.opt_binary_con_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_binary_con_price(
        calput,
        _m,
        _n,
        _x,
        s,
        k,
        _t,
        sigma,
        r,
        q,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_binary_con_greeks(calput, x, s, k, t, sigma, r, q):
    u"""
    Binary option, cash-or-nothing pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30cbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    k : float
        The amount, K, to be paid at expiration if the option is
        in-the-money, i.e., if `s` > `x`[i-1] when `calput` = 'C', or if
        `s` < `x`[i-1] when `calput` = 'P', for i = 1,2,...,m.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    delta : float, ndarray, shape (m, n)
        The leading m*n part of the array `delta` contains the
        sensitivity, dP/dS, of the option price to change in the price
        of the underlying asset.

    gamma : float, ndarray, shape (m, n)
        The leading m*n part of the array `gamma` contains the
        sensitivity, d^2P/dS^2, of `delta` to change in the price of the
        underlying asset.

    vega : float, ndarray, shape (m, n)
        `vega`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the
        volatility of the underlying asset, i.e., dP_{ij}/d sigma, for i
        = 1,2,...,m and j = 1,2,...,n.

    theta : float, ndarray, shape (m, n)
        `theta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in time, i.e.,
        -dP_{ij}/dT, for i = 1,2,...,m and j = 1,2,...,n, where b = r-q.

    rho : float, ndarray, shape (m, n)
        `rho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        risk-free interest rate, i.e., -dP_{ij}/dr, for i = 1,2,...,m
        and j = 1,2,...,n.

    crho : float, ndarray, shape (m, n)
        `crho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        cost of carry rate, i.e., -dP_{ij}/db, for i = 1,2,...,m and j =
        1,2,...,n, where b = r-q.

    vanna : float, ndarray, shape (m, n)
        `vanna`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the asset price, i.e., -d Delta_{ij}/dT =
        -d^2P_{ij}/(dSd sigma), for i = 1,2,...,m and j = 1,2,...,n.

    charm : float, ndarray, shape (m, n)
        `charm`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    speed : float, ndarray, shape (m, n)
        `speed`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the price of the underlying asset, i.e., -d Gamma_{ij}/dS =
        -d^3P_{ij}/dS^3, for i = 1,2,...,m and j = 1,2,...,n.

    colour : float, ndarray, shape (m, n)
        `colour`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the time, i.e., -d Gamma_{ij}/dT = -d^3P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    zomma : float, ndarray, shape (m, n)
        `zomma`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the volatility of the underlying asset, i.e., -d Gamma_{ij}/d
        sigma = -d^3P_{ij}/(dS^2d sigma), for i = 1,2,...,m and j =
        1,2,...,n.

    vomma : float, ndarray, shape (m, n)
        `vomma`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the underlying asset, i.e., -d Delta_{ij}/d sigma
        = -d^2P_{ij}/d sigma^2, for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.0.

        (`errno` 7)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 8)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 9)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 10)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    from ..base.specfun import opt_binary_con_greeks as b_opt_binary_con_greeks

    fun_name = 'naginterfaces.library.specfun.opt_binary_con_greeks'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _delta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _gamma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vega = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _rho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _crho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vanna = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _charm = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _speed = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _colour = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _zomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_binary_con_greeks(
        calput,
        _m,
        _n,
        _x,
        s,
        k,
        _t,
        sigma,
        r,
        q,
        _p,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
    )
    _p_py = _p
    _delta_py = _delta
    _gamma_py = _gamma
    _vega_py = _vega
    _theta_py = _theta
    _rho_py = _rho
    _crho_py = _crho
    _vanna_py = _vanna
    _charm_py = _charm
    _speed_py = _speed
    _colour_py = _colour
    _zomma_py = _zomma
    _vomma_py = _vomma
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'delta', 'gamma', 'vega', 'theta', 'rho', 'crho', 'vanna', 'charm', 'speed', 'colour', 'zomma', 'vomma'],
    )(
        _p_py,
        _delta_py,
        _gamma_py,
        _vega_py,
        _theta_py,
        _rho_py,
        _crho_py,
        _vanna_py,
        _charm_py,
        _speed_py,
        _colour_py,
        _zomma_py,
        _vomma_py,
    )
    return _return_data

def opt_binary_aon_price(calput, x, s, t, sigma, r, q):
    u"""
    Binary option, asset-or-nothing pricing formula.

    For full information please refer to the NAG Library document for
    s30cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30ccf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    from ..base.specfun import opt_binary_aon_price as b_opt_binary_aon_price

    fun_name = 'naginterfaces.library.specfun.opt_binary_aon_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_binary_aon_price(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        q,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_binary_aon_greeks(calput, x, s, t, sigma, r, q):
    u"""
    Binary option, asset-or-nothing pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30cdf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    delta : float, ndarray, shape (m, n)
        The leading m*n part of the array `delta` contains the
        sensitivity, dP/dS, of the option price to change in the price
        of the underlying asset.

    gamma : float, ndarray, shape (m, n)
        The leading m*n part of the array `gamma` contains the
        sensitivity, d^2P/dS^2, of `delta` to change in the price of the
        underlying asset.

    vega : float, ndarray, shape (m, n)
        `vega`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the
        volatility of the underlying asset, i.e., dP_{ij}/d sigma, for i
        = 1,2,...,m and j = 1,2,...,n.

    theta : float, ndarray, shape (m, n)
        `theta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in time, i.e.,
        -dP_{ij}/dT, for i = 1,2,...,m and j = 1,2,...,n, where b = r-q.

    rho : float, ndarray, shape (m, n)
        `rho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        risk-free interest rate, i.e., -dP_{ij}/dr, for i = 1,2,...,m
        and j = 1,2,...,n.

    crho : float, ndarray, shape (m, n)
        `crho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        cost of carry rate, i.e., -dP_{ij}/db, for i = 1,2,...,m and j =
        1,2,...,n, where b = r-q.

    vanna : float, ndarray, shape (m, n)
        `vanna`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the asset price, i.e., -d Delta_{ij}/dT =
        -d^2P_{ij}/(dSd sigma), for i = 1,2,...,m and j = 1,2,...,n.

    charm : float, ndarray, shape (m, n)
        `charm`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    speed : float, ndarray, shape (m, n)
        `speed`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the price of the underlying asset, i.e., -d Gamma_{ij}/dS =
        -d^3P_{ij}/dS^3, for i = 1,2,...,m and j = 1,2,...,n.

    colour : float, ndarray, shape (m, n)
        `colour`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the time, i.e., -d Gamma_{ij}/dT = -d^3P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    zomma : float, ndarray, shape (m, n)
        `zomma`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the volatility of the underlying asset, i.e., -d Gamma_{ij}/d
        sigma = -d^3P_{ij}/(dS^2d sigma), for i = 1,2,...,m and j =
        1,2,...,n.

    vomma : float, ndarray, shape (m, n)
        `vomma`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the underlying asset, i.e., -d Delta_{ij}/d sigma
        = -d^2P_{ij}/d sigma^2, for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.
    """

    from ..base.specfun import opt_binary_aon_greeks as b_opt_binary_aon_greeks

    fun_name = 'naginterfaces.library.specfun.opt_binary_aon_greeks'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _delta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _gamma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vega = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _rho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _crho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vanna = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _charm = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _speed = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _colour = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _zomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_binary_aon_greeks(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        q,
        _p,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
    )
    _p_py = _p
    _delta_py = _delta
    _gamma_py = _gamma
    _vega_py = _vega
    _theta_py = _theta
    _rho_py = _rho
    _crho_py = _crho
    _vanna_py = _vanna
    _charm_py = _charm
    _speed_py = _speed
    _colour_py = _colour
    _zomma_py = _zomma
    _vomma_py = _vomma
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'delta', 'gamma', 'vega', 'theta', 'rho', 'crho', 'vanna', 'charm', 'speed', 'colour', 'zomma', 'vomma'],
    )(
        _p_py,
        _delta_py,
        _gamma_py,
        _vega_py,
        _theta_py,
        _rho_py,
        _crho_py,
        _vanna_py,
        _charm_py,
        _speed_py,
        _colour_py,
        _zomma_py,
        _vomma_py,
    )
    return _return_data

def opt_barrier_std_price(calput, btype, x, s, h, k, t, sigma, r, q):
    u"""
    Standard barrier option pricing formula.

    For full information please refer to the NAG Library document for
    s30fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30faf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    btype : str, length 2
        Indicates the barrier type as In or Out and its relation to the
        price of the underlying asset as Up or Down.

        `btype` = 'DI'

          Down-and-In.

        `btype` = 'DO'

          Down-and-Out.

        `btype` = 'UI'

          Up-and-In.

        `btype` = 'UO'

          Up-and-Out.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    h : float
        The barrier price.

    k : float
        The value of a possible cash rebate to be paid if the option has
        not been knocked in (or out) before expiration.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, `btype` = *<value>*.

            Constraint: `btype` = 'DI', 'DO', 'UI' or 'UO'.

        (`errno` 3)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 4)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 5)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 6)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 7)
            On entry, `h` = *<value>*.

            Constraint: `h` >= *<value>* and `h` <= *<value>*.

        (`errno` 8)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.0.

        (`errno` 9)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 10)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 11)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 12)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 15)
            On entry, `s` and `h` are inconsistent with `btype`: `s` =
            *<value>* and `h` = *<value>*.
    """

    from ..base.specfun import opt_barrier_std_price as b_opt_barrier_std_price

    fun_name = 'naginterfaces.library.specfun.opt_barrier_std_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_barrier_std_price(
        calput,
        btype,
        _m,
        _n,
        _x,
        s,
        h,
        k,
        _t,
        sigma,
        r,
        q,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_jumpdiff_merton_price(calput, x, s, t, sigma, r, lamda, jvol):
    u"""
    Jump-diffusion, Merton's model, option pricing formula.

    For full information please refer to the NAG Library document for
    s30ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30jaf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the annual total volatility, including jumps.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    lamda : float
        lambda, the number of expected jumps per year.

    jvol : float
        The proportion of the total volatility associated with jumps.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `lamda` = *<value>*.

            Constraint: `lamda` > 0.0.

        (`errno` 10)
            On entry, `jvol` = *<value>*.

            Constraint: `jvol` >= 0.0 and `jvol` < 1.0.
    """

    from ..base.specfun import opt_jumpdiff_merton_price as b_opt_jumpdiff_merton_price

    fun_name = 'naginterfaces.library.specfun.opt_jumpdiff_merton_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_jumpdiff_merton_price(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        lamda,
        jvol,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_jumpdiff_merton_greeks(calput, x, s, t, sigma, r, lamda, jvol):
    u"""
    Jump-diffusion, Merton's model, option pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30jbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the annual total volatility, including jumps.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    lamda : float
        lambda, the number of expected jumps per year.

    jvol : float
        The proportion of the total volatility associated with jumps.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    delta : float, ndarray, shape (m, n)
        The leading m*n part of the array `delta` contains the
        sensitivity, dP/dS, of the option price to change in the price
        of the underlying asset.

    gamma : float, ndarray, shape (m, n)
        The leading m*n part of the array `gamma` contains the
        sensitivity, d^2P/dS^2, of `delta` to change in the price of the
        underlying asset.

    vega : float, ndarray, shape (m, n)
        `vega`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the
        volatility of the underlying asset, i.e., dP_{ij}/d sigma, for i
        = 1,2,...,m and j = 1,2,...,n.

    theta : float, ndarray, shape (m, n)
        `theta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in time, i.e.,
        -dP_{ij}/dT, for i = 1,2,...,m and j = 1,2,...,n, where b = r-q.

    rho : float, ndarray, shape (m, n)
        `rho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        risk-free interest rate, i.e., -dP_{ij}/dr, for i = 1,2,...,m
        and j = 1,2,...,n.

    vanna : float, ndarray, shape (m, n)
        `vanna`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the asset price, i.e., -d Delta_{ij}/dT =
        -d^2P_{ij}/(dSd sigma), for i = 1,2,...,m and j = 1,2,...,n.

    charm : float, ndarray, shape (m, n)
        `charm`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    speed : float, ndarray, shape (m, n)
        `speed`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the price of the underlying asset, i.e., -d Gamma_{ij}/dS =
        -d^3P_{ij}/dS^3, for i = 1,2,...,m and j = 1,2,...,n.

    colour : float, ndarray, shape (m, n)
        `colour`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the time, i.e., -d Gamma_{ij}/dT = -d^3P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    zomma : float, ndarray, shape (m, n)
        `zomma`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the volatility of the underlying asset, i.e., -d Gamma_{ij}/d
        sigma = -d^3P_{ij}/(dS^2d sigma), for i = 1,2,...,m and j =
        1,2,...,n.

    vomma : float, ndarray, shape (m, n)
        `vomma`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the underlying asset, i.e., -d Delta_{ij}/d sigma
        = -d^2P_{ij}/d sigma^2, for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `lamda` = *<value>*.

            Constraint: `lamda` > 0.0.

        (`errno` 10)
            On entry, `jvol` = *<value>*.

            Constraint: `jvol` >= 0.0 and `jvol` < 1.0.
    """

    from ..base.specfun import opt_jumpdiff_merton_greeks as b_opt_jumpdiff_merton_greeks

    fun_name = 'naginterfaces.library.specfun.opt_jumpdiff_merton_greeks'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _delta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _gamma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vega = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _rho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vanna = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _charm = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _speed = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _colour = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _zomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_jumpdiff_merton_greeks(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        lamda,
        jvol,
        _p,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
    )
    _p_py = _p
    _delta_py = _delta
    _gamma_py = _gamma
    _vega_py = _vega
    _theta_py = _theta
    _rho_py = _rho
    _vanna_py = _vanna
    _charm_py = _charm
    _speed_py = _speed
    _colour_py = _colour
    _zomma_py = _zomma
    _vomma_py = _vomma
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'delta', 'gamma', 'vega', 'theta', 'rho', 'vanna', 'charm', 'speed', 'colour', 'zomma', 'vomma'],
    )(
        _p_py,
        _delta_py,
        _gamma_py,
        _vega_py,
        _theta_py,
        _rho_py,
        _vanna_py,
        _charm_py,
        _speed_py,
        _colour_py,
        _zomma_py,
        _vomma_py,
    )
    return _return_data

def opt_heston_price(calput, x, s, t, sigmav, kappa, corr, var0, eta, grisk, r, q):
    u"""
    Heston's model option pricing formula.

    For full information please refer to the NAG Library document for
    s30na

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30naf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigmav : float
        The volatility, sigma_v, of the volatility process, sqrt(v_t).
        Note that a rate of 20% should be entered as 0.2.

    kappa : float
        kappa, the long term mean reversion rate of the volatility.

    corr : float
        The correlation between the two standard Brownian motions for
        the asset price and the volatility.

    var0 : float
        The initial value of the variance, v_t, of the asset price.

    eta : float
        eta, the long term mean of the variance of the asset price.

    grisk : float
        The risk aversion parameter, gamma, of the representative agent.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i-1] >= *<value>* and `x`[i-1] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i-1] >= *<value>*.

        (`errno` 7)
            On entry, `sigmav` = *<value>*.

            Constraint: `sigmav` > 0.0.

        (`errno` 8)
            On entry, `kappa` = *<value>*.

            Constraint: `kappa` > 0.0.

        (`errno` 9)
            On entry, `corr` = *<value>*.

            Constraint: abs(`corr`) <= 1.0.

        (`errno` 10)
            On entry, `var0` = *<value>*.

            Constraint: `var0` >= 0.0.

        (`errno` 11)
            On entry, `eta` = *<value>*.

            Constraint: `eta` > 0.0.

        (`errno` 12)
            On entry, `grisk` = *<value>*, `sigmav` = *<value>* and
            `kappa` = *<value>*.

            Constraint: 0.0 <= `grisk` <= 1.0 and
            `grisk`*(1.0-`grisk`)*`sigmav`^2 <= `kappa`^2.

        (`errno` 13)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 14)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 17)
            Quadrature has not converged to the specified accuracy.
            However, the result should be a reasonable approximation.

        (`errno` 18)
            Solution cannot be computed accurately. Check values of
            input arguments.
    """

    from ..base.specfun import opt_heston_price as b_opt_heston_price

    fun_name = 'naginterfaces.library.specfun.opt_heston_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    try:
        b_opt_heston_price(
            calput,
            _m,
            _n,
            _x,
            s,
            _t,
            sigmav,
            kappa,
            corr,
            var0,
            eta,
            grisk,
            r,
            q,
            _p,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _p_py = _p
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p'],
    )(
        _p_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _p_py

def opt_heston_greeks(calput, x, s, t, sigmav, kappa, corr, var0, eta, grisk, r, q):
    u"""
    Heston's model option pricing formula with Greeks.

    For full information please refer to the NAG Library document for
    s30nb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30nbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigmav : float
        The volatility, sigma_v, of the volatility process, sqrt(v_t).
        Note that a rate of 20% should be entered as 0.2.

    kappa : float
        kappa, the long term mean reversion rate of the volatility.

    corr : float
        The correlation between the two standard Brownian motions for
        the asset price and the volatility.

    var0 : float
        The initial value of the variance, v_t, of the asset price.

    eta : float
        eta, the long term mean of the variance of the asset price.

    grisk : float
        The risk aversion parameter, gamma, of the representative agent.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    delta : float, ndarray, shape (m, n)
        The leading m*n part of the array `delta` contains the
        sensitivity, dP/dS, of the option price to change in the price
        of the underlying asset.

    gamma : float, ndarray, shape (m, n)
        The leading m*n part of the array `gamma` contains the
        sensitivity, d^2P/dS^2, of `delta` to change in the price of the
        underlying asset.

    vega : float, ndarray, shape (m, n)
        `vega`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the
        volatility of the underlying asset, i.e., dP_{ij}/d sigma, for i
        = 1,2,...,m and j = 1,2,...,n.

    theta : float, ndarray, shape (m, n)
        `theta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in time, i.e.,
        -dP_{ij}/dT, for i = 1,2,...,m and j = 1,2,...,n, where b = r-q.

    rho : float, ndarray, shape (m, n)
        `rho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        risk-free interest rate, i.e., -dP_{ij}/dr, for i = 1,2,...,m
        and j = 1,2,...,n.

    vanna : float, ndarray, shape (m, n)
        `vanna`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the asset price, i.e., -d Delta_{ij}/dT =
        -d^2P_{ij}/(dSd sigma), for i = 1,2,...,m and j = 1,2,...,n.

    charm : float, ndarray, shape (m, n)
        `charm`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    speed : float, ndarray, shape (m, n)
        `speed`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the price of the underlying asset, i.e., -d Gamma_{ij}/dS =
        -d^3P_{ij}/dS^3, for i = 1,2,...,m and j = 1,2,...,n.

    zomma : float, ndarray, shape (m, n)
        `zomma`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the volatility of the underlying asset, i.e., -d Gamma_{ij}/d
        sigma = -d^3P_{ij}/(dS^2d sigma), for i = 1,2,...,m and j =
        1,2,...,n.

    vomma : float, ndarray, shape (m, n)
        `vomma`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the underlying asset, i.e., -d Delta_{ij}/d sigma
        = -d^2P_{ij}/d sigma^2, for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i-1] >= *<value>* and `x`[i-1] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i-1] >= *<value>*.

        (`errno` 7)
            On entry, `sigmav` = *<value>*.

            Constraint: `sigmav` > 0.0.

        (`errno` 8)
            On entry, `kappa` = *<value>*.

            Constraint: `kappa` > 0.0.

        (`errno` 9)
            On entry, `corr` = *<value>*.

            Constraint: abs(`corr`) <= 1.0.

        (`errno` 10)
            On entry, `var0` = *<value>*.

            Constraint: `var0` >= 0.0.

        (`errno` 11)
            On entry, `eta` = *<value>*.

            Constraint: `eta` > 0.0.

        (`errno` 12)
            On entry, `grisk` = *<value>*, `sigmav` = *<value>* and
            `kappa` = *<value>*.

            Constraint: 0.0 <= `grisk` <= 1.0 and
            `grisk`*(1.0-`grisk`)*`sigmav`^2 <= `kappa`^2.

        (`errno` 13)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 14)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 17)
            Quadrature has not converged to the required accuracy.
            However, the result should be a reasonable approximation.

        (`errno` 18)
            Solution cannot be computed accurately. Check values of
            input arguments.
    """

    from ..base.specfun import opt_heston_greeks as b_opt_heston_greeks

    fun_name = 'naginterfaces.library.specfun.opt_heston_greeks'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _delta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _gamma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vega = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _rho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vanna = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _charm = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _speed = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _zomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    try:
        b_opt_heston_greeks(
            calput,
            _m,
            _n,
            _x,
            s,
            _t,
            sigmav,
            kappa,
            corr,
            var0,
            eta,
            grisk,
            r,
            q,
            _p,
            _delta,
            _gamma,
            _vega,
            _theta,
            _rho,
            _vanna,
            _charm,
            _speed,
            _zomma,
            _vomma,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _p_py = _p
    _delta_py = _delta
    _gamma_py = _gamma
    _vega_py = _vega
    _theta_py = _theta
    _rho_py = _rho
    _vanna_py = _vanna
    _charm_py = _charm
    _speed_py = _speed
    _zomma_py = _zomma
    _vomma_py = _vomma
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'delta', 'gamma', 'vega', 'theta', 'rho', 'vanna', 'charm', 'speed', 'zomma', 'vomma'],
    )(
        _p_py,
        _delta_py,
        _gamma_py,
        _vega_py,
        _theta_py,
        _rho_py,
        _vanna_py,
        _charm_py,
        _speed_py,
        _zomma_py,
        _vomma_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def opt_heston_term(calput, x, fwd, disc, ts, t, alpha, lamda, corr, sigmat, var0):
    u"""
    Heston's model option pricing with term structure.

    For full information please refer to the NAG Library document for
    s30nc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30ncf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] contains the ith strike price, for i = 1,2,...,m.

    fwd : float
        The forward price of the asset.

    disc : float
        The discount factor, where the current price of the underlying
        asset, S_0, is given as S_0 = `disc`*`fwd`.

    ts : float, array-like, shape (numts)
        `ts`[i-1] must contain the length of the time intervals for
        which the corresponding element of `alpha`, `lamda`, `corr` and
        `sigmat` apply. These should be ordered as they occur in time
        i.e., Delta t_i = t_i-t_{i-1}.

    t : float
        `t` contains the time to expiry. If T > sum Delta t_i then the
        parameters associated with the last time interval are extended
        to the expiry time. If T < sum Delta t_i then the parameters
        specified are used up until the expiry time. The rest are
        ignored.

    alpha : float, array-like, shape (numts)
        `alpha`[i-1] must contain the value of alpha_t, the value of the
        volatility of the scaled volatility, sqrt(nu), over time
        subinterval Delta t_i.

    lamda : float, array-like, shape (numts)
        `lamda`[i-1] must contain the value, lambda_t, of the mean
        reversion parameter over the time subinterval Delta t_i.

    corr : float, array-like, shape (numts)
        `corr`[i-1] must contain the value, rho_t, of the correlation
        parameter over the time subinterval Delta t_i.

    sigmat : float, array-like, shape (numts)
        `sigmat`[i-1] must contain the value, sigma_t, of the variance
        scale factor over the time subinterval Delta t_i.

    var0 : float
        nu_0, the initial scaled variance.

    Returns
    -------
    p : float, ndarray, shape (m)
        `p`[i-1] contains the computed option price at the expiry time,
        T, corresponding to strike `x`[i-1] for the specified term
        structure, for i = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` != 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, numts = *<value>*.

            Constraint: numts >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `x`[i-1] <= *<value>*.

        (`errno` 5)
            On entry, `fwd` = *<value>*.

            Constraint: *<value>* <= `fwd` <= *<value>*.

        (`errno` 6)
            On entry, `disc` = *<value>*.

            Constraint: *<value>* <= `disc` <= *<value>*.

        (`errno` 7)
            On entry, `ts`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `ts`[i-1] <= *<value>*.

        (`errno` 8)
            On entry, `t` = *<value>*.

            Constraint: `t` >= *<value>*.

        (`errno` 9)
            On entry, `alpha`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `alpha`[i-1] <= *<value>*.

        (`errno` 10)
            On entry, `lamda`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `lamda`[i-1] <= *<value>*.

        (`errno` 11)
            On entry, `corr`[*<value>*] = *<value>*.

            Constraint: abs(`corr`[i-1]) <= 1.0.

        (`errno` 12)
            On entry, `sigmat`[*<value>*] = *<value>*.

            Constraint: *<value>* <= `sigmat`[i-1] <= *<value>*.

        (`errno` 13)
            On entry, `var0` = *<value>*.

            Constraint: `var0` > 0.0.

        (`errno` 14)
            Quadrature has not converged to the specified accuracy.
            However, the result should be a reasonable approximation.

        (`errno` 15)
            Solution cannot be computed accurately. Check values of
            input arguments.
    """

    from ..base.specfun import opt_heston_term as b_opt_heston_term

    fun_name = 'naginterfaces.library.specfun.opt_heston_term'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _ts = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ts'},
        ts,
        copy=False,
    )
    _numts_py = _b_utils._EngineFloat64ArrayType.get_shape(_ts, {'fun_name': fun_name, 'entity': 'ts'}, exp_rank=1)[0]
    _numts = _b_utils._EngineIntScalarType(
        _numts_py, {'fun_name': fun_name, 'entity': '_numts'},
    ).to_ctypes()
    _alpha = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'alpha'},
        alpha,
        copy=False,
    )
    _lamda = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lamda'},
        lamda,
        copy=False,
    )
    _corr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'corr'},
        corr,
        copy=False,
    )
    _sigmat = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'sigmat'},
        sigmat,
        copy=False,
    )
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    b_opt_heston_term(
        calput,
        _m,
        _numts,
        _x,
        fwd,
        disc,
        _ts,
        t,
        _alpha,
        _lamda,
        _corr,
        _sigmat,
        var0,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_amer_bs_price(calput, x, s, t, sigma, r, q):
    u"""
    American option, Bjerksund and Stensland pricing formula.

    For full information please refer to the NAG Library document for
    s30qc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30qcf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    q : float
        q, the annual continuous yield rate. Note that a rate of 8%
        should be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.

        (`errno` 9)
            On entry, `q` = *<value>*.

            Constraint: `q` >= 0.0.

        (`errno` 14)
            On entry, `s` = *<value>* and beta = *<value>*.

            Constraint: `s`^beta < *<value>*.
    """

    from ..base.specfun import opt_amer_bs_price as b_opt_amer_bs_price

    fun_name = 'naginterfaces.library.specfun.opt_amer_bs_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_amer_bs_price(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        q,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_asian_geom_price(calput, x, s, t, sigma, r, b):
    u"""
    Asian option, geometric continuous average rate pricing formula.

    For full information please refer to the NAG Library document for
    s30sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30saf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    b : float
        b, the annual cost of carry rate. Note that a rate of 8% should
        be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.
    """

    from ..base.specfun import opt_asian_geom_price as b_opt_asian_geom_price

    fun_name = 'naginterfaces.library.specfun.opt_asian_geom_price'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_asian_geom_price(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        b,
        _p,
    )
    _p_py = _p
    return _p_py

def opt_asian_geom_greeks(calput, x, s, t, sigma, r, b):
    u"""
    Asian option, geometric continuous average rate pricing formula with
    Greeks.

    For full information please refer to the NAG Library document for
    s30sb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/s/s30sbf.html

    Parameters
    ----------
    calput : str, length 1
        Determines whether the option is a call or a put.

        `calput` = 'C'

          A call; the holder has a right to buy.

        `calput` = 'P'

          A put; the holder has a right to sell.

    x : float, array-like, shape (m)
        `x`[i-1] must contain X_i, the ith strike price, for i =
        1,2,...,m.

    s : float
        S, the price of the underlying asset.

    t : float, array-like, shape (n)
        `t`[i-1] must contain T_i, the ith time, in years, to expiry,
        for i = 1,2,...,n.

    sigma : float
        sigma, the volatility of the underlying asset. Note that a rate
        of 15% should be entered as 0.15.

    r : float
        r, the annual risk-free interest rate, continuously compounded.
        Note that a rate of 5% should be entered as 0.05.

    b : float
        b, the annual cost of carry rate. Note that a rate of 8% should
        be entered as 0.08.

    Returns
    -------
    p : float, ndarray, shape (m, n)
        `p`[i-1,j-1] contains P_{ij}, the option price evaluated for the
        strike price `x`_i at expiry `t`_j for i = 1,2,...,m and j =
        1,2,...,n.

    delta : float, ndarray, shape (m, n)
        The leading m*n part of the array `delta` contains the
        sensitivity, dP/dS, of the option price to change in the price
        of the underlying asset.

    gamma : float, ndarray, shape (m, n)
        The leading m*n part of the array `gamma` contains the
        sensitivity, d^2P/dS^2, of `delta` to change in the price of the
        underlying asset.

    vega : float, ndarray, shape (m, n)
        `vega`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the
        volatility of the underlying asset, i.e., dP_{ij}/d sigma, for i
        = 1,2,...,m and j = 1,2,...,n.

    theta : float, ndarray, shape (m, n)
        `theta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in time, i.e.,
        -dP_{ij}/dT, for i = 1,2,...,m and j = 1,2,...,n, where b = r-q.

    rho : float, ndarray, shape (m, n)
        `rho`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the annual
        risk-free interest rate, i.e., -dP_{ij}/dr, for i = 1,2,...,m
        and j = 1,2,...,n.

    crho : float, ndarray, shape (m, n)
        `delta`[i-1,j-1], contains the first-order Greek measuring the
        sensitivity of the option price P_{ij} to change in the price of
        the underlying asset, i.e., -dP_{ij}/dS, for i = 1,2,...,m and j
        = 1,2,...,n.

    vanna : float, ndarray, shape (m, n)
        `vanna`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the asset price, i.e., -d Delta_{ij}/dT =
        -d^2P_{ij}/(dSd sigma), for i = 1,2,...,m and j = 1,2,...,n.

    charm : float, ndarray, shape (m, n)
        `charm`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        time, i.e., -d Delta_{ij}/dT = -d^2P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    speed : float, ndarray, shape (m, n)
        `speed`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the price of the underlying asset, i.e., -d Gamma_{ij}/dS =
        -d^3P_{ij}/dS^3, for i = 1,2,...,m and j = 1,2,...,n.

    colour : float, ndarray, shape (m, n)
        `colour`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the time, i.e., -d Gamma_{ij}/dT = -d^3P_{ij}/(dSdT), for i =
        1,2,...,m and j = 1,2,...,n.

    zomma : float, ndarray, shape (m, n)
        `zomma`[i-1,j-1], contains the third-order Greek measuring the
        sensitivity of the second-order Greek Gamma_{ij} to change in
        the volatility of the underlying asset, i.e., -d Gamma_{ij}/d
        sigma = -d^3P_{ij}/(dS^2d sigma), for i = 1,2,...,m and j =
        1,2,...,n.

    vomma : float, ndarray, shape (m, n)
        `vomma`[i-1,j-1], contains the second-order Greek measuring the
        sensitivity of the first-order Greek Delta_{ij} to change in the
        volatility of the underlying asset, i.e., -d Delta_{ij}/d sigma
        = -d^2P_{ij}/d sigma^2, for i = 1,2,...,m and j = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `calput` = *<value>*.

            Constraint: `calput` = 'C' or 'P'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 4)
            On entry, `x`[*<value>*] = *<value>*.

            Constraint: `x`[i] >= *<value>* and `x`[i] <= *<value>*.

        (`errno` 5)
            On entry, `s` = *<value>*.

            Constraint: `s` >= *<value>* and `s` <= *<value>*.

        (`errno` 6)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i] >= *<value>*.

        (`errno` 7)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 8)
            On entry, `r` = *<value>*.

            Constraint: `r` >= 0.0.
    """

    from ..base.specfun import opt_asian_geom_greeks as b_opt_asian_geom_greeks

    fun_name = 'naginterfaces.library.specfun.opt_asian_geom_greeks'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _delta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _gamma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vega = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _rho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _crho = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vanna = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _charm = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _speed = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _colour = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _zomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _vomma = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_opt_asian_geom_greeks(
        calput,
        _m,
        _n,
        _x,
        s,
        _t,
        sigma,
        r,
        b,
        _p,
        _delta,
        _gamma,
        _vega,
        _theta,
        _rho,
        _crho,
        _vanna,
        _charm,
        _speed,
        _colour,
        _zomma,
        _vomma,
    )
    _p_py = _p
    _delta_py = _delta
    _gamma_py = _gamma
    _vega_py = _vega
    _theta_py = _theta
    _rho_py = _rho
    _crho_py = _crho
    _vanna_py = _vanna
    _charm_py = _charm
    _speed_py = _speed
    _colour_py = _colour
    _zomma_py = _zomma
    _vomma_py = _vomma
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p', 'delta', 'gamma', 'vega', 'theta', 'rho', 'crho', 'vanna', 'charm', 'speed', 'colour', 'zomma', 'vomma'],
    )(
        _p_py,
        _delta_py,
        _gamma_py,
        _vega_py,
        _theta_py,
        _rho_py,
        _crho_py,
        _vanna_py,
        _charm_py,
        _speed_py,
        _colour_py,
        _zomma_py,
        _vomma_py,
    )
    return _return_data
