#!/usr/bin/env python
"""
Tests of the Python particulars in the naginterfaces package.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2017-2019.

# pylint: disable=protected-access,too-many-lines

import array
import ctypes
import unittest
import numpy as np

from naginterfaces.base import utils as b_utils
from naginterfaces.library import _utils as np_utils

class TestBadGood(object):
    """
    Base class for some required test fixtures.
    """

    def test_bad(self):
        """
        Bad input.
        """
        raise NotImplementedError # pragma: no cover

    def test_good(self):
        """
        Good input.
        """
        raise NotImplementedError # pragma: no cover

class InputTest(unittest.TestCase):
    """
    Test base class for functions processing
    data before passing it to the Engine.
    """

    def setUp(self):
        """
        Define the general error locus.
        """
        import sys

        self.locus = {'fun_name': 'z99zz'}
        self.arg_name = 'x'
        self.locus['entity'] = self.arg_name
        self.locus_re = (
            'function ' + self.locus['fun_name'] +
            '.*argument ' + self.arg_name
        )

        if sys.version_info < (3,):
            self.my_assert_raises_regex = self.assertRaisesRegexp
        else:
            self.my_assert_raises_regex = self.assertRaisesRegex # pylint: disable=no-member

class AttributeTest(unittest.TestCase):
    """
    Tests for subpackage/submodule attributes.
    """

    def test_engine_libdir(self):
        """
        Test that ENGINE_LIBDIR looks reasonable.
        """
        import os
        from naginterfaces import ENGINE_LIBDIR
        self.assertTrue(os.path.isdir(ENGINE_LIBDIR))

class ContentsTest(unittest.TestCase):
    """
    Tests for package binary contents.
    """
    import platform

    def file_present_and_exec(self, name):
        """
        Verify the presence of `name`.
        """
        import os
        import platform
        from naginterfaces import ENGINE_LIBDIR
        from naginterfaces.kusari import _PRODUCT_CODE

        bin_name = name if platform.system() != 'Windows' else name + '.exe'
        bin_path = os.path.join(ENGINE_LIBDIR, bin_name)

        if _PRODUCT_CODE.endswith('L'):
            self.assertTrue(os.path.isfile(bin_path))
            self.assertTrue(os.access(bin_path, os.X_OK))
        else:
            self.assertFalse(os.path.isfile(bin_path))

    def test_hostid(self):
        """
        khostid is present.
        """
        self.file_present_and_exec('khostid')

    def test_lcheck(self):
        """
        klcheck is present.
        """
        self.file_present_and_exec('klcheck')

    @unittest.skipUnless(
        platform.system() == 'Windows',
        'klreg only exists on Windows'
    )
    def test_lreg(self):
        """
        klreg is present.
        """
        self.file_present_and_exec('klreg')

    @unittest.skipUnless(
        platform.system() == 'Windows',
        'key_gui only exists on Windows'
    )
    def test_key_gui(self):
        """
        kusari_key_gui is present.
        """
        self.file_present_and_exec('kusari_key_gui')

class AsciiTestScalarChar(InputTest):
    """
    Tests for strings needing to be ASCII.
    """

    class_to_test = b_utils._EngineCharScalarType
    err_msg = 'must comprise only ASCII characters'

    def test_bad_bytes(self):
        """
        Non-ASCII bytes literal passed.
        """
        input_x = b'Pi\xf1ata'
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            self.locus_re + '.*' + self.err_msg,
            self.class_to_test,
            input_x, self.locus, None,
        )

    def test_good_long_bytes(self):
        """
        Non-ASCII bytes literal passed.
        """
        input_x = b'Pi\xf1ata'
        self.class_to_test(
            input_x, self.locus, exp_chrlen=2,
        )

    def test_good_bytes(self):
        """
        ASCII bytes literal passed.
        """
        input_x = b'Pineapple'
        self.class_to_test(
            input_x, self.locus, None,
        )

    def test_bad_default(self):
        """
        Non-ASCII default literal passed.
        """
        input_x = 'Pi\xf1ata'
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            self.locus_re + '.*' + self.err_msg,
            self.class_to_test,
            input_x, self.locus, None,
        )

    def test_good_default(self):
        """
        ASCII default literal passed.
        """
        input_x = 'Pineapple'
        self.class_to_test(
            input_x, self.locus, None,
        )

    def test_bad_unicode(self):
        """
        Non-ASCII unicode literal passed.
        """
        input_x = u'Pi\xf1ata'
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            self.locus_re + '.*' + self.err_msg,
            self.class_to_test,
            input_x, self.locus, None,
        )

    def test_good_unicode(self):
        """
        ASCII unicode literal passed.
        """
        input_x = u'Pineapple'
        self.class_to_test(
            input_x, self.locus, None,
        )

class AsciiTestArrayChar(InputTest):
    """
    Tests for string arrays needing to be ASCII.
    """

    class_to_test = b_utils._EngineCharArrayType
    err_msg = 'must comprise only ASCII characters'

    def test_bad_numpy_bytes(self):
        """
        Non-ASCII numpy bytes array passed, checking enabled.
        """
        input_x = np.array([b'Pi\xf1ata'])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            self.locus_re + '.*' + self.err_msg,
            self.class_to_test,
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(1,),
        )

    def test_good_long_numpy_bytes(self):
        """
        Non-ASCII numpy bytes array passed, checking enabled.
        """
        input_x = np.array([b'Pi\xf1ata'])
        self.class_to_test(
            input_x, self.locus, exp_chrlen=2,
            exp_shape=(1,),
        )

    def test_bad_numpy_bytes_disabled(self):
        """
        Non-ASCII numpy bytes array passed, checking disabled.
        """
        input_x = np.array([b'Pi\xf1ata'])
        self.class_to_test(
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(1,),
            check_contents=False
        )

    def test_bad_numpy_str_multidim(self):
        """
        Non-ASCII numpy str array passed, checking enabled.
        """
        input_x = np.array(['Colada', 'Pi\xf1a']).reshape((1, 2))
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            self.locus_re + r'.*element \[0, 1\].*' + self.err_msg,
            self.class_to_test,
            input_x, self.locus,
            exp_chrlen=None, exp_shape=input_x.shape, mutual_sorder='C',
        )

    def test_good_long_numpy_str_multidim(self): # pylint: disable=invalid-name
        """
        Non-ASCII numpy str array passed, checking enabled.
        """
        input_x = np.array(['Colada', 'Pi\xf1a']).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus,
            exp_chrlen=2, exp_shape=input_x.shape, mutual_sorder='C',
        )

    def test_bad_numpy_str_multidim_disabled(self): # pylint: disable=invalid-name
        """
        Non-ASCII numpy str array passed, checking disabled.
        """
        input_x = np.array(['Colada', 'Pi\xf1a']).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus,
            exp_chrlen=None, exp_shape=input_x.shape, mutual_sorder='C',
            check_contents=False
        )

    def test_good_list_unicode(self):
        """
        ASCII unicode list passed.
        """
        input_x = [u'Pine', u'Apple']
        self.class_to_test(
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(2,),
        )

class LenTestScalarChar(InputTest):
    """
    Tests for strings being longer than expected.
    """

    class_to_test = b_utils._EngineCharScalarType

    def test_good_long_bytes(self):
        """
        Long bytes literal passed.
        """
        input_x = b'Hello'
        x_cls = self.class_to_test(
            input_x, self.locus, 4,
        )
        self.assertEqual(
            x_cls.to_ctypes()[:x_cls.chrlen],
            b'Hell',
        )

    def test_good_long_skipped(self):
        """
        Long bytes literal passed, checking suppressed.
        """
        input_x = b'Hello'
        x_cls = self.class_to_test(
            input_x, self.locus, None,
        )
        self.assertEqual(
            x_cls.to_ctypes()[:x_cls.chrlen],
            b'Hello',
        )

    def test_good_empty(self):
        """
        Short-enough (empty) default literal passed.
        """
        input_x = ''
        the_inst = self.class_to_test(
            input_x, self.locus, None,
        )
        self.assertEqual(
            the_inst.chrlen,
            0
        )

    def test_good_bytes(self):
        """
        Short-enough bytes literal passed.
        """
        input_x = b'Hello'
        self.class_to_test(
            input_x, self.locus, len(input_x),
        )

    def test_good_long_default(self):
        """
        Long default literal passed.
        """
        input_x = 'Hello'
        x_cls = self.class_to_test(
            input_x, self.locus, 4,
        )
        self.assertEqual(
            x_cls.to_ctypes()[:x_cls.chrlen],
            b'Hell',
        )

    def test_good_default(self):
        """
        Short-enough default literal passed.
        """
        input_x = 'Hello'
        self.class_to_test(
            input_x, self.locus, len(input_x),
        )

    def test_good_long_unicode(self):
        """
        Long unicode literal passed.
        """
        input_x = u'Hello'
        x_cls = self.class_to_test(
            input_x, self.locus, 4,
        )
        self.assertEqual(
            x_cls.to_ctypes()[:x_cls.chrlen],
            b'Hell',
        )

    def test_good_unicode(self):
        """
        Short-enough unicode literal passed.
        """
        input_x = u'Hello'
        self.class_to_test(
            input_x, self.locus, len(input_x),
        )

    def test_good_long_single(self):
        """
        Single character expected.
        """
        input_x = 'HW'
        x_cls = self.class_to_test(
            input_x, self.locus, 1,
        )
        self.assertEqual(
            x_cls.to_ctypes()[:x_cls.chrlen],
            b'H',
        )

class LenTestArrayChar(InputTest):
    """
    Tests for string arrays being longer than expected.
    """

    class_to_test = b_utils._EngineCharArrayType

    def test_good_long_numpy_bytes(self):
        """
        Long numpy bytes array passed, checking enabled.
        """
        input_x = np.array([b'Hello', b'World!'])
        x_cls = self.class_to_test(
            input_x, self.locus, exp_chrlen=5, exp_shape=(2,),
        )
        self.assertEqual(
            x_cls.to_ctypes()[:],
            b'HelloWorld',
        )

    def test_good_long_numpy_bytes_disabled(self): # pylint: disable=invalid-name
        """
        Long numpy bytes array passed, checking disabled.
        """
        input_x = np.array([b'Hello', b'World!'])
        self.class_to_test(
            input_x, self.locus, exp_chrlen=5, exp_shape=(2,),
            check_contents=False
        )

    def test_good_long_numpy_str_multidim(self): # pylint: disable=invalid-name
        """
        Long numpy str array passed, checking enabled.
        """
        input_x = np.array(['Hello', 'World!']).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus,
            exp_chrlen=5, exp_shape=input_x.shape, mutual_sorder='C',
        )

    def test_good_long_numpy_str_multidim_disabled(self): # pylint: disable=invalid-name
        """
        Long numpy str array passed, checking disabled.
        """
        input_x = np.array(['Hello', 'World!']).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus,
            exp_chrlen=5, exp_shape=input_x.shape, mutual_sorder='C',
            check_contents=False
        )

    def test_good_list_empty(self):
        """
        Empty list passed, contents checked.
        """
        input_x = []
        the_inst = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=None, exp_shape=(len(input_x),),
        )
        self.assertEqual(
            the_inst.chrlen,
            0
        )

    def test_good_list_empty_suppressed(self):
        """
        Empty list passed, checking suppressed.
        """
        input_x = []
        the_inst = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=None, exp_shape=(len(input_x),), check_contents=False,
        )
        self.assertEqual(
            the_inst.chrlen,
            0
        )

    def test_good_list_unicode(self):
        """
        Short-enough unicode list passed.
        """
        input_x = [u'Pine', u'Apple']
        self.class_to_test(
            input_x, self.locus, exp_chrlen=5, exp_shape=(2,),
        )

    def test_good_numpy_str_multidim(self):
        """
        Short-enough numpy str array passed.
        """
        input_x = np.array(['Hello', 'World!']).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus,
            exp_chrlen=6, exp_shape=input_x.shape, mutual_sorder='C'
        )

    def test_good_ndarray_empty(self):
        """
        Empty ndarray passed, contents checked.
        """
        input_x = np.array((0, 0), dtype='S6')
        the_inst = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=None, exp_shape=input_x.shape,
        )
        self.assertEqual(
            the_inst.chrlen,
            6
        )

    def test_good_ndarray_empty_suppressed(self): # pylint: disable=invalid-name
        """
        Empty ndarray passed, checking suppressed.
        """
        input_x = np.array((0, 0), dtype='S6')
        the_inst = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=None, exp_shape=input_x.shape, check_contents=False,
        )
        self.assertEqual(
            the_inst.chrlen,
            6
        )

class CastTestScalar(object):
    """
    Abstract suite of unit tests for the scalar casting classes.
    """

    @classmethod
    def setUpClass(cls): # pylint: disable=invalid-name
        """
        Define the abstract attributes.
        """
        cls.arg_name = None
        cls.locus = None

        cls.class_to_test = lambda *args: None # pragma: no cover
        cls.base_scalar = [None]
        cls.input_x_ctype = lambda *args: None # pragma: no cover
        cls.input_x_ptype = None

    def casted_value_eq(self, casted):
        """
        Compare the value of the cast with the expected base scalar.
        """
        self.assertEqual( # pylint: disable=no-member
            casted.value,
            self.base_scalar,
        )

    def run_test(self, input_x):
        """
        Test correct operation.
        """
        casted = self.class_to_test(
            input_x, self.locus,
        ).to_ctypes()
        self.assertIsInstance( # pylint: disable=no-member
            casted,
            self.input_x_ctype
        )
        self.casted_value_eq(casted)
        back_casted = self.class_to_test.to_py(casted)
        self.assertIsInstance( # pylint: disable=no-member
            back_casted,
            self.input_x_ptype
        )
        back_casted_val = (
            back_casted.value if isinstance(back_casted, b_utils.Handle)
            else back_casted
        )
        val_to_check = (
            input_x.value if isinstance(
                input_x,
                (
                    self.input_x_ctype,
                    ctypes.c_int, ctypes.c_long, ctypes.c_longlong
                )
            )
            else input_x
        )
        self.assertEqual( # pylint: disable=no-member
            back_casted_val,
            val_to_check,
        )

    def run_test_none(self):
        """
        Test correct operation.
        """
        casted = self.class_to_test(
            None, self.locus, True,
        ).to_ctypes()
        self.assertIsNone( # pylint: disable=no-member
            casted,
        )
        self.assertIsNone( # pylint: disable=no-member
            self.class_to_test.to_py(casted),
        )

    def test_good_none(self):
        """
        Test correct operation with None.
        """
        self.run_test_none()

class CastTestScalarChar(CastTestScalar, InputTest):
    """
    Cast-check subsuite.

    Overrides run_test.
    Overrides run_test_none.
    """

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineCharScalarType
        cls.base_scalar = b'42.0'
        cls.input_x_ctype = ctypes.c_char * len(cls.base_scalar)

    def run_test(self, input_x):
        """
        Test correct operation.
        """
        casted = self.class_to_test(
            input_x, self.locus, 4,
        ).to_ctypes()
        self.assertIsInstance(
            casted,
            self.input_x_ctype
        )
        self.casted_value_eq(casted)
        back_casted = self.class_to_test.to_py(casted)
        self.assertIsInstance(back_casted, str)
        back_casted_val = back_casted
        val_to_check = (
            input_x.decode(encoding='ascii') if isinstance(input_x, bytes)
            else input_x
        )
        self.assertEqual(
            back_casted_val,
            val_to_check,
        )

    def run_test_none(self):
        """
        Test correct operation.
        """
        casted = self.class_to_test(
            None, self.locus, 42,
            allow_none=True
        ).to_ctypes()
        self.assertIsNone(  # pylint: disable=no-member
            casted,
        )
        self.assertIsNone( # pylint: disable=no-member
            self.class_to_test.to_py(casted),
        )

    def test_good_bytes(self):
        """
        Test correct operation with byte strings.
        """
        import sys
        if sys.version_info < (3,):
            self.run_test('42.0')
        else:
            self.run_test(b'42.0')

    def test_good_numpy_bytes(self):
        """
        Test correct operation with numpy byte strings.
        """
        import sys
        if sys.version_info < (3,):
            self.run_test(np.string_(self.base_scalar))
        else:
            self.run_test(np.bytes_(self.base_scalar))

    def test_good_unicode(self):
        """
        Test correct operation with unicode strings.
        """
        import sys
        if sys.version_info < (3,):
            self.run_test(u'42.0')
        else:
            self.run_test('42.0')

    def test_good_numpy_unicode(self):
        """
        Test correct operation with numpy unicode strings.
        """
        import sys
        if sys.version_info < (3,):
            self.run_test(np.unicode_('42.0'))
        else:
            self.run_test(np.str_('42.0'))

    def test_padding(self):
        """
        Test correct padding with blanks.
        """
        input_x = '42'
        casted = self.class_to_test(
            input_x, self.locus, 3
        ).to_ctypes()
        self.assertIsInstance(
            casted,
            ctypes.c_char * 3
        )
        self.assertEqual(
            casted.value,
            b'42 '
        )

class CastTestScalarComplex(CastTestScalar):
    """
    Overrides casted_value_eq.
    """

    def casted_value_eq(
            self,
            casted
    ):
        """
        Comparison of complex values.
        """
        self.assertEqual( # pylint: disable=no-member
            casted.real,
            self.base_scalar.real, # pylint: disable=no-member
        )
        self.assertEqual( # pylint: disable=no-member
            casted.imag,
            self.base_scalar.imag, # pylint: disable=no-member
        )

class CastTestScalarComplex64(CastTestScalarComplex, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineComplex64ScalarType
        cls.input_x_ctype = cls.class_to_test.c_type
        cls.input_x_ptype = np.complex64
        cls.base_scalar = cls.input_x_ctype(42.0, 43.0)

    def test_good(self):
        """
        Test correct operation.
        """
        self.run_test(np.complex64(42.0 + 43.0j))

class CastTestScalarComplex128(CastTestScalarComplex, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineComplex128ScalarType
        cls.input_x_ctype = cls.class_to_test.c_type
        cls.base_scalar = cls.input_x_ctype(42.0, 43.0)
        cls.input_x_ptype = complex

    def test_good(self):
        """
        Test correct operation.
        """
        self.run_test(42.0 + 43.0j)

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        self.run_test(np.complex128(42.0 + 43.0j))

class CastTestScalarCptr(CastTestScalar, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineCptrScalarType
        cls.base_scalar = 42
        cls.input_x_ctype = b_utils.Handle
        cls.input_x_ptype = b_utils.Handle

    def test_good(self):
        """
        Test correct operation.
        """
        self.run_test(b_utils.Handle(self.base_scalar))

class CastTestScalarFloat32(CastTestScalar, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineFloat32ScalarType
        cls.base_scalar = 42.0
        cls.input_x_ctype = ctypes.c_float
        cls.input_x_ptype = np.float32

    def test_good_ctypes(self):
        """
        Test correct operation for ctypes input.
        """
        self.run_test(self.input_x_ctype(self.base_scalar))

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        self.run_test(np.float32(self.base_scalar))

class CastTestScalarFloat64(CastTestScalar, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineFloat64ScalarType
        cls.base_scalar = 42.0
        cls.input_x_ctype = ctypes.c_double
        cls.input_x_ptype = float

    def test_good(self):
        """
        Test correct operation.
        """
        self.run_test(self.base_scalar)

    def test_good_ctypes(self):
        """
        Test correct operation for ctypes input.
        """
        self.run_test(self.input_x_ctype(self.base_scalar))

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        self.run_test(np.float64(self.base_scalar))

class CastTestScalarInt(CastTestScalar, InputTest):
    """
    Cast-check subsuite.
    """
    import sys

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineIntScalarType
        cls.base_scalar = 42
        cls.input_x_ctype = b_utils._EngineState._int_model['ctypes']
        cls.input_x_ptype = int

    def test_good(self):
        """
        Test correct operation.
        """
        self.run_test(self.base_scalar)

    @unittest.skipUnless(
        (
            sys.version_info < (3,) and
            b_utils._IS_IL32
        ),
        'int and long are equivalent'
    )
    def test_good_64(self):
        """
        Test correct operation if builtin long is bigger than int.
        """
        self.run_test(long(self.base_scalar)) # pylint: disable=undefined-variable

    def test_good_ctypes_32(self):
        """
        Test correct operation for ctypes 32-bit input.
        """
        self.run_test(
            b_utils._SYS_CTYPES_INTS[32](self.base_scalar)
        )

    def test_good_ctypes_64(self):
        """
        Test correct operation for ctypes 64-bit input.
        """
        self.run_test(
            b_utils._SYS_CTYPES_INTS[64](self.base_scalar)
        )

    def test_good_numpy_32(self):
        """
        Test correct operation for numpy 32-bit input.
        f"""
        self.run_test(np.int32(self.base_scalar))

    def test_good_numpy_64(self):
        """
        Test correct operation for numpy 64-bit input.
        """
        self.run_test(np.int64(self.base_scalar))

class CastTestScalarLogical(CastTestScalar, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineLogicalScalarType
        cls.base_scalar = (
            -1 if b_utils._EngineState._compilers['Fortran'] == 'Intel'
            else 1
        )
        cls.input_x_ctype = b_utils._EngineState._int_model['ctypes']
        cls.input_x_ptype = bool

    def test_good(self):
        """
        Test correct operation.
        """
        self.run_test(True)

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        self.run_test(np.bool_(42))

class CastTestArray(object):
    """
    Abstract suite of unit tests for the array casting classes.
    """

    @classmethod
    def setUpClass(cls): # pylint: disable=invalid-name
        """
        Define the abstract attributes.
        """
        cls.arg_name = None
        cls.locus = None

        cls.class_to_test = lambda *args, **kwargs: None # pragma: no cover
        cls.base_list = [None]
        cls.modified_list = [None]
        cls.modified_array = None
        cls.input_x_ctype = lambda *args, **kwargs: None # pragma: no cover
        cls.base_ctype = ctypes.Array

    def modify_1dim(self, buff):
        """
        Modify buff.
        """
        for m_i, m_el in enumerate(self.modified_list):
            buff[m_i] = (
                m_el.value if isinstance(m_el, ctypes._SimpleCData)
                else self.input_x_ctype(m_el).value
            )

    def run_test_1dim(self, input_x, can_ref=True):
        """
        Test correct operation.
        """
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_shape=(len(input_x),), check_contents=can_ref,
        )
        casted = casted_cls.to_ctypes(can_reference_contents=can_ref)
        self.assertIsInstance( # pylint: disable=no-member
            casted,
            self.base_ctype,
        )
        self.assertIs( # pylint: disable=no-member
            casted._type_,
            self.input_x_ctype
        )
        if can_ref:
            for i in range(len(input_x)):
                self.assertEqual( # pylint: disable=no-member
                    casted[i],
                    self.base_list[i],
                )
        self.modify_1dim(casted)
        casted_cls.refresh_py(casted)
        if len(self.modified_list) > 0: # pylint: disable=len-as-condition
            if isinstance(input_x, list):
                self.assertIsInstance( # pylint: disable=no-member
                    input_x[0],
                    type(self.modified_list[0])
                )
        for i, x_el in enumerate(input_x):

            if isinstance(x_el, ctypes._SimpleCData):
                self.assertEqual( # pylint: disable=no-member
                    x_el.value,
                    self.modified_list[i].value,
                )
            else:
                self.assertEqual( # pylint: disable=no-member
                    x_el,
                    self.modified_list[i],
                )

    def run_test_none_1dim(self, refresh_is_implemented=True):
        """
        Test correct operation.
        """
        casted_cls = self.class_to_test(
            None, self.locus,
            exp_shape=(42,), check_contents=True, allow_none=True
        )
        casted = casted_cls.to_ctypes(can_reference_contents=True)
        self.assertIsNone(  # pylint: disable=no-member
            casted,
        )
        self.assertEqual( # pylint: disable=no-member
            casted_cls.rank, 1,
        )
        self.assertEqual( # pylint: disable=no-member
            casted_cls.shape, (0,),
        )
        self.assertEqual( # pylint: disable=no-member
            casted_cls.size, 0,
        )

        if refresh_is_implemented:
            casted_cls.refresh_py(casted)
        else:
            self.assertRaises( # pylint: disable=no-member
                NotImplementedError,
                casted_cls.refresh_py,
                casted,
            )

    def modify_ndim(self, buff, sorder):
        """
        Modify buff.
        """
        for m_i, m_el in enumerate(self.modified_array.flatten(order=sorder)):
            buff[m_i] = m_el

    def run_test_multidim(self, input_x, sorder='C'):
        """
        Test correct operation.
        """
        if input_x.size != 0:
            self.assertFalse( # pylint: disable=no-member
                np.array_equal(
                    input_x,
                    self.modified_array,
                )
            )
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_shape=input_x.shape, mutual_sorder=sorder,
        )
        casted = casted_cls.to_ctypes()
        self.assertIsInstance( # pylint: disable=no-member
            casted,
            self.base_ctype,
        )
        self.assertIs( # pylint: disable=no-member
            casted._type_,
            self.input_x_ctype
        )
        for i in range(input_x.size):
            self.assertEqual( # pylint: disable=no-member
                casted[i],
                input_x.flatten(order=sorder)[i],
            )
        self.modify_ndim(casted, sorder)
        casted_cls.refresh_py(casted)
        self.assertTrue( # pylint: disable=no-member
            np.issubdtype(
                input_x.dtype,
                self.modified_array.dtype
            )
        )
        np.testing.assert_equal(
            input_x,
            self.modified_array,
        )

    def run_test_none_multidim(self):
        """
        Test correct operation.
        """
        casted_cls = self.class_to_test(
            None, self.locus,
            exp_shape=(42, 43), check_contents=True, allow_none=True
        )
        casted = casted_cls.to_ctypes(can_reference_contents=True)
        self.assertIsNone( # pylint: disable=no-member
            casted,
        )
        self.assertEqual( # pylint: disable=no-member
            casted_cls.rank, 2,
        )
        self.assertEqual( # pylint: disable=no-member
            casted_cls.shape, (0, 0),
        )
        self.assertEqual( # pylint: disable=no-member
            casted_cls.size, 0,
        )
        casted_cls.refresh_py(casted)

class CastTestArrayChar(CastTestArray, InputTest):
    """
    Cast-check subsuite.

    Overrides modify_1dim.
    Overrides run_test_1dim.
    Overrides run_test_none_1dim.
    Overrides run_test_none_multidim.
    """

    # pylint: disable=too-many-public-methods

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineCharArrayType
        cls.base_str = b'HelloWorld'
        cls.modified_list = ['World', 'Hello']
        cls.input_x_ctype = ctypes.c_char_p

    def modify_1dim(self, buff):
        import sys

        if (
                len(self.modified_list) > 0 and # pylint: disable=len-as-condition
                (
                    (
                        sys.version_info >= (3,) and
                        isinstance(self.modified_list[0], str)
                    ) or
                    (
                        sys.version_info < (3,) and
                        isinstance(self.modified_list[0], unicode) # pylint: disable=undefined-variable
                    )
                )
        ):
            buff[:] = ''.join(self.modified_list).encode('ascii')
        else:
            buff[:] = b''.join(self.modified_list)

    def run_test_1dim(self, input_x, can_ref=True): # pylint: disable=unused-argument
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=5, exp_shape=(len(input_x),), check_contents=can_ref,
        )
        casted = casted_cls.to_ctypes(can_reference_contents=can_ref)
        self.assertIsInstance(
            casted,
            ctypes.c_char * len(casted)
        )
        self.assertEqual(
            casted[:],
            self.base_str,
        )
        self.modify_1dim(casted)
        casted_cls.refresh_py(casted)
        if len(self.modified_list) > 0: # pylint: disable=len-as-condition
            self.assertIsInstance(
                input_x[0],
                type(self.modified_list[0])
            )
        for i, x_el in enumerate(input_x):
            self.assertEqual(
                x_el,
                self.modified_list[i],
            )

    def run_test_none_1dim(self, refresh_is_implemented=True):
        """
        Test correct operation.
        """
        casted_cls = self.class_to_test(
            None, self.locus, exp_chrlen=None,
            exp_shape=(42,), check_contents=True, allow_none=True
        )
        casted = casted_cls.to_ctypes(can_reference_contents=True)
        self.assertIsNone(casted)
        self.assertEqual(casted_cls.rank, 1)
        self.assertEqual(casted_cls.shape, tuple([0]))
        self.assertEqual(casted_cls.size, 0)
        self.assertEqual(casted_cls.chrlen, 0)

        if refresh_is_implemented:
            casted_cls.refresh_py(casted)
        else:
            self.fail('inconceivable') # pragma: no cover

    def run_test_none_multidim(self):
        """
        Test correct operation.
        """
        casted_cls = self.class_to_test(
            None, self.locus, exp_chrlen=5,
            exp_shape=(42, 43), check_contents=True, allow_none=True
        )
        casted = casted_cls.to_ctypes(can_reference_contents=True)
        self.assertIsNone(casted)
        self.assertEqual(casted_cls.rank, 2)
        self.assertEqual(casted_cls.shape, (0, 0))
        self.assertEqual(casted_cls.size, 0)
        casted_cls.refresh_py(casted)

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 empty input.
        """
        self.base_str = b''
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_bytes(self):
        """
        Test correct operation for rank-1 bytes input.
        """
        self.modified_list = [b'World', b'Hello']
        self.run_test_1dim([b'Hello', b'World'])

    def test_good_list_1dim_padded_star(self):
        """
        Test correct operation and padding for rank-1 char-* input.
        """
        input_x = ['Hello', 'World!']
        data_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=None, exp_shape=(len(input_x),),
        )
        casted = data_cls.to_ctypes()
        self.assertEqual(
            casted[:],
            b'Hello World!',
        )
        self.assertEqual(
            data_cls.chrlen,
            6,
        )
        casted[:] = b'World!Hello '
        data_cls.refresh_py(casted)
        self.assertEqual(
            input_x,
            ['World!', 'Hello'],
        )

    def test_good_list_1dim_padded_len(self):
        """
        Test correct operation and padding for rank-1 char-len input.
        """
        input_x = ['Hello', 'World!']
        data_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=7, exp_shape=(len(input_x),),
        )
        casted = data_cls.to_ctypes()
        self.assertEqual(
            casted[:],
            b'Hello  World! ',
        )
        self.assertEqual(
            data_cls.chrlen,
            7,
        )
        casted[:] = b'World! Hello  '
        data_cls.refresh_py(casted)
        self.assertEqual(
            input_x,
            ['World!', 'Hello'],
        )

    def test_good_list_1dim_truncated(self):
        """
        Test correct operation and truncation for rank-1 input.
        """
        input_x = ['Hel', 'Wor']
        data_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=3, exp_shape=(len(input_x),),
        )
        casted = data_cls.to_ctypes()
        self.assertEqual(
            casted[:],
            b'HelWor',
        )
        self.assertEqual(
            data_cls.chrlen,
            3,
        )
        casted[:] = b'WorHel'
        data_cls.refresh_py(casted)
        self.assertEqual(
            input_x,
            ['Wor', 'Hel'],
        )

    def test_good_list_1dim_unicode(self):
        """
        Test correct operation for rank-1 unicode input.
        """
        self.modified_list = [u'World', u'Hello']
        self.run_test_1dim([u'Hello', u'World'])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        input_x = ['Colada', 'Pi\xf1a']
        data_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=3, exp_shape=(len(input_x),), check_contents=False
        )
        casted = data_cls.to_ctypes(can_reference_contents=False)
        self.assertEqual(
            casted[:],
            b'\0'*6,
        )
        self.assertEqual(
            data_cls.chrlen,
            3,
        )

    def test_good_ndarray_1dim_bytes(self):
        """
        Test correct operation for rank-1 bytes input.
        """
        self.modified_list = np.array([b'World', b'Hello'])
        self.run_test_1dim(np.array([b'Hello', b'World']))

    def test_good_ndarray_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_1dim(np.array(['Hello', 'World']))

    def test_good_ndarray_1dim_unicode(self):
        """
        Test correct operation for rank-1 bytes input.
        """
        self.modified_list = np.array([u'World', u'Hello'])
        self.run_test_1dim(np.array([u'Hello', u'World']))

    def test_good_ndarray_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        input_x = np.array(['Colada', 'Pi\xf1a'])
        data_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=3, exp_shape=(len(input_x),), check_contents=False
        )
        casted = data_cls.to_ctypes(can_reference_contents=False)
        self.assertEqual(
            casted[:],
            b'\0'*6,
        )
        self.assertEqual(
            data_cls.chrlen,
            3,
        )

    def test_good_ndarray_multi_empty(self):
        """
        Test correct operation for empty multidimensional input.
        """
        input_x = np.array([[], []], dtype=str)
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=0, exp_shape=input_x.shape, mutual_sorder='C',
        )
        casted = casted_cls.to_ctypes()
        self.assertIsInstance(
            casted,
            ctypes.c_char * len(casted)
        )
        self.assertEqual(
            casted[:],
            b'',
        )

    def test_good_ndarray_multidim_c(self):
        """
        Test correct operation for C-contiguous multidimensional input.
        """
        input_x = np.array(['Hel', 'Wo', 'lo', 'rld']).reshape((2, 2))
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=3, exp_shape=input_x.shape, mutual_sorder='C',
        )
        casted = casted_cls.to_ctypes()
        self.assertIsInstance(
            casted,
            ctypes.c_char * len(casted)
        )
        self.assertEqual(
            casted[:],
            b'HelWo lo rld',
        )
        casted[:] = b'Hello Wo rld'
        casted_cls.refresh_py(casted)
        np.testing.assert_equal(
            input_x,
            np.array([['Hel', 'lo'], ['Wo', 'rld']]),
        )

    def test_good_ndarray_multidim_f(self):
        """
        Test correct operation for Fortran-contiguous multidimensional input.
        """
        input_x = np.array(
            ['Hel', 'lo', 'Wo', 'rld']
        ).reshape((2, 2), order='F')
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=3, exp_shape=input_x.shape, mutual_sorder='F',
        )
        casted = casted_cls.to_ctypes()
        self.assertIsInstance(
            casted,
            ctypes.c_char * len(casted)
        )
        self.assertEqual(
            casted[:],
            b'Hello Wo rld',
        )
        casted[:] = b'HelrldWo lo '
        casted_cls.refresh_py(casted)
        np.testing.assert_equal(
            input_x,
            np.array([['Hel', 'Wo'], ['rld', 'lo']]),
        )

    def test_good_ndar_mdim_padded_star(self):
        """
        Test correct operation and padding for multidimensional char-* input.
        """
        input_x = np.array(
            [u'Hello', u'World!', u'Ola', u'Mundo']
        ).reshape((2, 2))
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=None, exp_shape=input_x.shape, mutual_sorder='C',
        )
        casted = casted_cls.to_ctypes()
        self.assertEqual(
            casted[:],
            b'Hello World!Ola   Mundo ',
        )
        self.assertEqual(
            casted_cls.chrlen,
            6,
        )
        casted[:] = b'Hello Ola   Mundo World!'
        casted_cls.refresh_py(casted)
        np.testing.assert_equal(
            input_x,
            np.array([[u'Hello', u'Ola'], [u'Mundo', u'World!']]),
        )

    def test_good_ndar_mdim_padded_len(self):
        """
        Test correct operation and padding for multidimensional char-len input.
        """
        input_x = np.array(
            [u'Hello', u'World!', u'Ola', u'Mundo']
        ).reshape((2, 2))
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_chrlen=7, exp_shape=input_x.shape, mutual_sorder='C',
        )
        casted = casted_cls.to_ctypes()
        self.assertEqual(
            casted[:],
            b'Hello  World! Ola    Mundo  ',
        )
        self.assertEqual(
            casted_cls.chrlen,
            7,
        )
        casted[:] = b'Hello  Ola    Mundo  World! '
        casted_cls.refresh_py(casted)
        np.testing.assert_equal(
            input_x,
            np.array([[u'Hello', u'Ola'], [u'Mundo', u'World!']]),
        )

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim()

    def test_good_none_multidim(self):
        """
        Test correct operation for multidimensional input.
        """
        self.run_test_none_multidim()

class CastTestArrayComplex(CastTestArray):
    """
    Overrides modify_1dim.
    Overrides modify_ndim.
    Overrides run_test_1dim.
    Overrides run_test_multidim.
    """

    def modify_1dim(self, buff):
        for m_i, m_el in enumerate(self.modified_list):
            buff[m_i].real = m_el.real
            buff[m_i].imag = m_el.imag

    def modify_ndim(self, buff, sorder):
        """
        Modify buff.
        """
        for m_i, m_el in enumerate(self.modified_array.flatten(order=sorder)):
            buff[m_i].real = m_el.real
            buff[m_i].imag = m_el.imag

    def run_test_1dim(self, input_x, can_ref=True):
        """
        Test correct operation.
        """
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_shape=(len(input_x),), check_contents=can_ref,
        )
        casted = casted_cls.to_ctypes(can_reference_contents=can_ref)
        self.assertIsInstance( # pylint: disable=no-member
            casted,
            self.base_ctype,
        )
        self.assertIs( # pylint: disable=no-member
            casted._type_,
            self.input_x_ctype
        )
        if can_ref:
            for i in range(len(input_x)):
                self.assertEqual( # pylint: disable=no-member
                    casted[i].real,
                    self.base_list[i].real,
                )
                self.assertEqual( # pylint: disable=no-member
                    casted[i].imag,
                    self.base_list[i].imag,
                )
        self.modify_1dim(casted)
        casted_cls.refresh_py(casted)
        if len(self.modified_list) > 0: # pylint: disable=len-as-condition
            self.assertIsInstance( # pylint: disable=no-member
                input_x[0],
                type(self.modified_list[0])
            )
        for i, x_el in enumerate(input_x):
            self.assertEqual( # pylint: disable=no-member
                x_el.real,
                self.modified_list[i].real,
            )
            self.assertEqual( # pylint: disable=no-member
                x_el.imag,
                self.modified_list[i].imag,
            )

    def run_test_multidim(self, input_x, sorder='C'):
        """
        Test correct operation.
        """
        self.assertFalse( # pylint: disable=no-member
            np.array_equal(
                input_x,
                self.modified_array,
            )
        )
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_shape=input_x.shape, mutual_sorder=sorder,
        )
        casted = casted_cls.to_ctypes()
        self.assertIsInstance( # pylint: disable=no-member
            casted,
            self.base_ctype,
        )
        self.assertIs( # pylint: disable=no-member
            casted._type_,
            self.input_x_ctype
        )
        for i in range(input_x.size):
            self.assertEqual( # pylint: disable=no-member
                casted[i].real,
                input_x.flatten(order=sorder)[i].real
            )
            self.assertEqual( # pylint: disable=no-member
                casted[i].imag,
                input_x.flatten(order=sorder)[i].imag
            )
        self.modify_ndim(casted, sorder)
        casted_cls.refresh_py(casted)
        self.assertTrue( # pylint: disable=no-member
            np.issubdtype(
                input_x.dtype,
                self.modified_array.dtype
            )
        )
        np.testing.assert_equal(
            input_x,
            self.modified_array,
        )

class CastTestArrayComplex64(CastTestArrayComplex, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineComplex64ArrayType
        cls.input_x_ctype = cls.class_to_test.c_type._type_
        cls.base_list = [cls.input_x_ctype(42, i) for i in range(10)]
        cls.modified_list = [
            np.complex64(complex(-42., -i)) for i in range(10)
        ]
        cls.modified_array = np.array(cls.modified_list).reshape((2, 5))

    def test_good_list_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_1dim([np.complex64(complex(42, i)) for i in range(10)])

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        self.run_test_1dim([None]*len(self.base_list), can_ref=False)

    def test_good_ndarray_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(
            np.array([complex(42, i) for i in range(10)], dtype=np.complex64)
        )

    def test_good_ndarray_multidim_c(self):
        """
        Test correct operation for C-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array([
                complex(42, 0), complex(44, 2), complex(46, 4), complex(48, 6),
                complex(50, 8),
                complex(43, 1), complex(45, 3), complex(47, 5), complex(49, 7),
                complex(51, 9)
            ], dtype=np.complex64).reshape((2, 5))
        )

    def test_good_ndarray_multidim_f(self):
        """
        Test correct operation for Fortran-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array(
                [complex(42, i) for i in range(10)], dtype=np.complex64
            ).reshape((2, 5), order='F'),
            sorder='F',
        )

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim()

    def test_good_none_multidim(self):
        """
        Test correct operation for multidimensional input.
        """
        self.run_test_none_multidim()

class CastTestArrayComplex128(CastTestArrayComplex, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineComplex128ArrayType
        cls.input_x_ctype = cls.class_to_test.c_type._type_
        cls.base_list = [cls.input_x_ctype(42, i) for i in range(10)]
        cls.modified_list = [complex(-42, -i) for i in range(10)]
        cls.modified_array = np.array(cls.modified_list).reshape((2, 5))

    def test_good_list_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_1dim([complex(42, i) for i in range(10)])

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        self.run_test_1dim([None]*len(self.base_list), can_ref=False)

    def test_good_ndarray_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_list = [np.complex128(z) for z in self.modified_list]
        self.run_test_1dim(
            np.array([complex(42, i) for i in range(10)], dtype=np.complex128)
        )

    def test_good_ndarray_multidim_c(self):
        """
        Test correct operation for C-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array([
                complex(42, 0), complex(44, 2), complex(46, 4), complex(48, 6),
                complex(50, 8),
                complex(43, 1), complex(45, 3), complex(47, 5), complex(49, 7),
                complex(51, 9)
            ], dtype=np.complex128).reshape((2, 5))
        )

    def test_good_ndarray_multidim_f(self):
        """
        Test correct operation for Fortran-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array(
                [complex(42, i) for i in range(10)], dtype=np.complex128
            ).reshape((2, 5), order='F'),
            sorder='F',
        )

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim()

    def test_good_none_multidim(self):
        """
        Test correct operation for multidimensional input.
        """
        self.run_test_none_multidim()

class CastTestArrayCptr(CastTestArray, InputTest):
    """
    Cast-check subsuite.

    Overrides run_test_1dim.
    """

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineCptrArrayType
        cls.base_list = [b_utils.Handle(42 + i) for i in range(10)]
        cls.input_x_ctype = cls.class_to_test.c_type._type_

    def run_test_1dim(self, input_x, can_ref=True):
        """
        Test correct operation.
        Modification phase is omitted.
        """
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_shape=(len(input_x),), check_contents=can_ref,
        )
        casted = casted_cls.to_ctypes(can_reference_contents=can_ref)
        self.assertIsInstance( # pylint: disable=no-member
            casted,
            self.base_ctype,
        )
        self.assertIs( # pylint: disable=no-member
            casted._type_,
            self.input_x_ctype
        )
        if can_ref:
            for i in range(len(input_x)):
                self.assertEqual( # pylint: disable=no-member
                    casted[i].value,
                    self.base_list[i].value,
                )
        else:
            self.fail('inconceivable') # pragma: no cover
        self.assertRaises(
            NotImplementedError,
            casted_cls.refresh_py,
            casted,
        )

    def test_good_ctypes_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 10
            _type_ = b_utils.Handle
        self.run_test_1dim(CtypesTestArray(*self.base_list))

    def test_good_list_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_1dim(self.base_list)

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        casted_cls = self.class_to_test(
            [None]*10, self.locus,
            exp_shape=(10,), check_contents=False,
        )
        casted = casted_cls.to_ctypes(can_reference_contents=False)
        for c_el in casted:
            self.assertIsNone(c_el.value)

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim(refresh_is_implemented=False)

class CastTestArrayFloat32(CastTestArray, InputTest):
    """
    Cast-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineFloat32ArrayType
        cls.base_list = [42.0 + i for i in range(10)]
        cls.modified_list = [np.float32(42.0 - i) for i in range(10)]
        cls.modified_array = np.array(cls.modified_list).reshape((2, 5))
        cls.input_x_ctype = ctypes.c_float

    def test_good_array_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(array.array('f', self.base_list))

    def test_good_ctypes_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 10
            _type_ = ctypes.c_float
        self.run_test_1dim(CtypesTestArray(*self.base_list))

    def test_good_list_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = [
            self.input_x_ctype(x) for x in self.modified_list
        ]
        self.run_test_1dim([self.input_x_ctype(x) for x in self.base_list])

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        self.modified_list = [
            self.input_x_ctype(x) for x in self.modified_list
        ]
        self.run_test_1dim(
            [self.input_x_ctype(x) for x in self.base_list], can_ref=False
        )

    def test_good_ndarray_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(np.array(self.base_list, dtype=np.float32))

    def test_good_ndarray_multidim_c(self):
        """
        Test correct operation for C-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array(
                [42, 44, 46, 48, 50, 43, 45, 47, 49, 51], dtype=np.float32
            ).reshape((2, 5))
        )

    def test_good_ndarray_multidim_f(self):
        """
        Test correct operation for Fortran-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array(
                self.base_list, dtype=np.float32
            ).reshape((2, 5), order='F'),
            sorder='F',
        )

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim()

    def test_good_none_multidim(self):
        """
        Test correct operation for multidimensional input.
        """
        self.run_test_none_multidim()

class CastTestArrayFloat64(CastTestArray, InputTest):
    """
    Cast-check subsuite.
    """
    import sys

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineFloat64ArrayType
        cls.base_list = [42.0 + i for i in range(10)]
        cls.modified_list = [42.0 - i for i in range(10)]
        cls.modified_array = np.array(
            cls.modified_list[:-2]
        ).reshape((2, 2, 2))
        cls.input_x_ctype = ctypes.c_double

    def test_good_array_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(array.array('d', self.base_list))

    def test_good_ctypes_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 10
            _type_ = ctypes.c_double
        self.run_test_1dim(CtypesTestArray(*self.base_list))

    def test_good_list_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = [
            self.input_x_ctype(x) for x in self.modified_list
        ]
        self.run_test_1dim([self.input_x_ctype(x) for x in self.base_list])

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        self.modified_list = [
            self.input_x_ctype(x) for x in self.modified_list
        ]
        self.run_test_1dim(
            [self.input_x_ctype(x) for x in self.base_list], can_ref=False
        )

    def test_good_ndarray_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_list = [np.float64(x) for x in self.modified_list]
        self.run_test_1dim(np.array(self.base_list, dtype=np.float64))

    def test_good_ndarray_multidim_c(self):
        """
        Test correct operation for C-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array(
                [42, 44, 46, 48, 50, 43, 45, 47], dtype=np.float64
            ).reshape((2, 2, 2))
        )

    def test_good_ndarray_multidim_f(self):
        """
        Test correct operation for Fortran-contiguous multidimensional input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_multidim(
            np.array(
                self.base_list[:-2], dtype=np.float64
            ).reshape((2, 2, 2), order='F'),
            sorder='F',
        )

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim()

    def test_good_none_multidim(self):
        """
        Test correct operation for multidimensional input.
        """
        self.run_test_none_multidim()

class CastTestArrayInt(CastTestArray, InputTest):
    """
    Cast-check subsuite.
    """

    # pylint: disable=too-many-public-methods

    import sys

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineIntArrayType
        cls.base_list = [42 + i for i in range(10)]
        cls.modified_list = [42 - i for i in range(10)]
        cls.modified_array = np.array(
            cls.modified_list[:-2]
        ).reshape((2, 2, 2))
        cls.input_x_ctype = b_utils._EngineState._int_model['ctypes']

    def test_good_array_1dim_l(self):
        """
        Test correct operation for rank-1 "l" input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(array.array('l', self.base_list))

    def test_good_array_1dim_l_noref(self):
        """
        Test correct operation for rank-1 "l" input, referencing suppressed.
        """
        if (
                b_utils._EngineState._int_model['bits'] == 64 or
                b_utils._IS_IL32
        ):
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        else: # pragma: no cover
            self.fail('inconceivable')
        self.run_test_1dim(array.array('l', self.base_list), can_ref=False)

    @unittest.skipUnless(
        sys.version_info >= (3,),
        'no "q" array typecode'
    )
    def test_good_array_1dim_q(self):
        """
        Test correct operation for rank-1 "q" input.
        """
        self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(array.array('q', self.base_list))

    @unittest.skipUnless(
        sys.version_info >= (3,),
        'no "q" array typecode'
    )
    def test_good_array_1dim_q_noref(self):
        """
        Test correct operation for rank-1 "q" input, referencing suppressed.
        """
        if b_utils._EngineState._int_model['bits'] == 64:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(
            array.array('q', [2**40]*len(self.base_list)), can_ref=False
        )

    def test_good_ctypes_1dim_32(self):
        """
        Test correct operation for rank-1 input.
        """
        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 10
            _type_ = b_utils._SYS_CTYPES_INTS[32]
        self.run_test_1dim(CtypesTestArray(*self.base_list))

    def test_good_ctypes_1dim_64(self):
        """
        Test correct operation for rank-1 input.
        """
        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 10
            _type_ = b_utils._SYS_CTYPES_INTS[64]
        self.run_test_1dim(CtypesTestArray(*self.base_list))

    def test_good_list_1dim_32(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = [
            b_utils._SYS_CTYPES_INTS[32](x)
            for x in self.modified_list
        ]
        self.run_test_1dim(
            [
                b_utils._SYS_CTYPES_INTS[32](x)
                for x in self.base_list
            ]
        )

    def test_good_list_1dim_64(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = [
            b_utils._SYS_CTYPES_INTS[64](x)
            for x in self.modified_list
        ]
        self.run_test_1dim(
            [
                b_utils._SYS_CTYPES_INTS[64](x)
                for x in self.base_list
            ]
        )

    @unittest.skipUnless(
        (
            sys.version_info < (3,) and
            b_utils._IS_IL32
        ),
        'int and long are equivalent'
    )
    def test_good_list_1dim_long(self):
        """
        Test correct operation for rank-1 input.
        """
        # pylint: disable=undefined-variable
        self.modified_list = [ # pragma: no cover
            long(x) for x in self.modified_list # pragma: no cover
        ] # pragma: no cover
        # No-op to work around strange coverage behaviour:
        pass # pylint: disable=unnecessary-pass
        self.run_test_1dim( # pragma: no cover
            [long(x) for x in self.base_list] # pragma: no cover
        ) # pragma: no cover

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        self.run_test_1dim([None]*10, can_ref=False)

    def test_good_ndarray_1dim_32(self):
        """
        Test correct operation for rank-1 input.
        """
        if b_utils._EngineState._int_model['bits'] == 32:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_list = [np.int32(x) for x in self.modified_list]
        self.run_test_1dim(np.array(self.base_list, dtype=np.int32))

    def test_good_ndarray_1dim_32_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        if b_utils._EngineState._int_model['bits'] == 32:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(
            np.empty(len(self.base_list), dtype=np.int32), can_ref=False
        )

    def test_good_ndarray_1dim_64(self):
        """
        Test correct operation for rank-1 input.
        """
        if b_utils._EngineState._int_model['bits'] == 64:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_list = [np.int64(x) for x in self.modified_list]
        self.run_test_1dim(np.array(self.base_list, dtype=np.int64))

    def test_good_ndarray_1dim_64_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        if b_utils._EngineState._int_model['bits'] == 64:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.run_test_1dim(
            np.array([2**40]*len(self.base_list), dtype=np.int64),
            can_ref=False
        )

    def test_good_ndarray_multi_empty(self):
        """
        Test correct operation for empty multidimensional input.
        """
        if b_utils._EngineState._int_model['bits'] == 64:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_array = np.empty((0, 10), dtype=np.int64)
        self.run_test_multidim(np.empty((0, 10), dtype=np.int64))

    def test_good_ndarray_multidim_f_32(self):
        """
        Test correct operation for Fortran-contiguous muiltidimensional input.
        """
        if b_utils._EngineState._int_model['bits'] == 32:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_array = np.array(
            self.modified_list[:-2], dtype=np.int32
        ).reshape((2, 2, 2))
        self.run_test_multidim(
            np.array(
                self.base_list[:-2], dtype=np.int32
            ).reshape((2, 2, 2), order='F'),
            sorder='F',
        )

    def test_good_ndarray_multidim_c_32(self):
        """
        Test correct operation for C-contiguous muiltidimensional input.
        """
        if b_utils._EngineState._int_model['bits'] == 32:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_array = np.array(
            self.modified_list[:-2], dtype=np.int32
        ).reshape((2, 2, 2))
        self.run_test_multidim(
            np.array(self.base_list[:-2], dtype=np.int32).reshape((2, 2, 2))
        )

    def test_good_ndarray_multidim_f_64(self):
        """
        Test correct operation for Fortran-contiguous muiltidimensional input.
        """
        if b_utils._EngineState._int_model['bits'] == 64:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_array = np.array(
            self.modified_list[:-2], dtype=np.int64
        ).reshape((2, 2, 2))
        self.run_test_multidim(
            np.array(
                self.base_list[:-2], dtype=np.int64
            ).reshape((2, 2, 2), order='F'),
            sorder='F',
        )

    def test_good_ndarray_multidim_c_64(self):
        """
        Test correct operation for C-contiguous muiltidimensional input.
        """
        if b_utils._EngineState._int_model['bits'] == 64:
            self.base_ctype = ctypes.POINTER(self.input_x_ctype)
        self.modified_array = np.array(
            self.modified_list[:-2], dtype=np.int64
        ).reshape((2, 2, 2))
        self.run_test_multidim(
            np.array(self.base_list[:-2], dtype=np.int64).reshape((2, 2, 2))
        )

    def test_overwrite(self):
        """
        Avoiding weird NumPy memory overwrite.
        """
        if b_utils._EngineState._int_model['bits'] == 32:
            different_dtype = np.int64
        elif b_utils._EngineState._int_model['bits'] == 64:
            different_dtype = np.int32
        else:
            self.fail('inconceivable') # pragma: no cover
        casted_cls = self.class_to_test(
            np.array(
                [
                    [1, 0, 1],
                    [0, 1, 0]
                ],
                dtype=different_dtype,
            ),
            self.locus,
            exp_shape=(2, 3), mutual_sorder='C',
        )
        casted = casted_cls._to_ctypes_from_ndarray()
        self.assertEqual(
            casted[:6],
            [1, 0, 1, 0, 1, 0],
            msg='initial cast',
        )
        np.empty((3, 2), dtype='U2')
        self.assertEqual(
            casted[:6],
            [1, 0, 1, 0, 1, 0],
            msg='corrupted',
        )

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim()

    def test_good_none_multidim(self):
        """
        Test correct operation for multidimensional input.
        """
        self.run_test_none_multidim()

class CastTestArrayLogical(CastTestArray, InputTest):
    """
    Cast-check subsuite.

    Overrides run_test_multidim.
    """

    @classmethod
    def setUpClass(cls):
        CastTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineLogicalArrayType
        cls.base_list = [
            b_utils._EngineState._literal_true_int for _i in range(10)
        ]
        cls.modified_list = [False]*10
        cls.modified_array = np.array([False]*4).reshape((2, 2))
        cls.input_x_ctype = b_utils._EngineState._int_model['ctypes']

    def run_test_multidim(self, input_x, sorder='C'):
        """
        Test correct operation.
        """
        self.assertFalse(
            np.array_equal(
                input_x,
                self.modified_array,
            )
        )
        casted_cls = self.class_to_test(
            input_x, self.locus,
            exp_shape=input_x.shape, mutual_sorder=sorder,
        )
        casted = casted_cls.to_ctypes()
        self.assertIsInstance(
            casted,
            self.base_ctype,
        )
        self.assertIs(
            casted._type_,
            self.input_x_ctype
        )
        for i in range(input_x.size):
            self.assertEqual(
                casted[i],
                b_utils._EngineState._from_bool_to_int(
                    input_x.flatten(order=sorder)[i]
                ),
            )
        casted[:] = [False]*4
        casted_cls.refresh_py(casted)
        self.assertTrue(
            np.issubdtype(
                input_x.dtype,
                self.modified_array.dtype
            )
        )
        np.testing.assert_equal(
            input_x,
            self.modified_array,
        )

    def test_good_list_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_1dim([bool(i) for i in self.base_list])

    def test_good_list_1dim_empty(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = []
        self.run_test_1dim([])

    def test_good_list_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        self.run_test_1dim([bool(i) for i in self.base_list], can_ref=False)

    def test_good_ndarray_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.modified_list = [np.bool_(x) for x in self.modified_list]
        self.run_test_1dim(
            np.array([bool(i) for i in self.base_list], dtype=np.bool_)
        )

    def test_good_ndarray_1dim_noref(self):
        """
        Test correct operation for rank-1 input, referencing suppressed.
        """
        self.run_test_1dim(
            np.empty(len(self.base_list), dtype=bool), can_ref=False
        )

    def test_good_ndarray_multidim_c(self):
        """
        Test correct operation for C-contiguous multidimensional input.
        """
        self.run_test_multidim(
            np.array([True, True, False, False]).reshape((2, 2))
        )

    def test_good_ndarray_multidim_f(self):
        """
        Test correct operation for Fortran-contiguous multidimensional input.
        """
        self.run_test_multidim(
            np.array([True, True, False, False]).reshape((2, 2), order='F'),
            sorder='F',
        )

    def test_overwrite(self):
        """
        Avoiding weird NumPy memory overwrite.
        """
        casted = self.class_to_test._to_ctypes_from_flat_data(
            np.array(
                [True, False, True, False, True, False],
                dtype=bool,
            )
        )
        self.assertEqual(
            casted[:6],
            [
                b_utils._EngineState._literal_true_int, 0,
                b_utils._EngineState._literal_true_int, 0,
                b_utils._EngineState._literal_true_int, 0
            ],
            msg='initial cast',
        )
        np.empty((3, 2), dtype='U2')
        self.assertEqual(
            casted[:6],
            [
                b_utils._EngineState._literal_true_int, 0,
                b_utils._EngineState._literal_true_int, 0,
                b_utils._EngineState._literal_true_int, 0
            ],
            msg='corrupted',
        )

    def test_good_none_1dim(self):
        """
        Test correct operation for rank-1 input.
        """
        self.run_test_none_1dim()

    def test_good_none_multidim(self):
        """
        Test correct operation for multidimensional input.
        """
        self.run_test_none_multidim()

class ElementTestArray(object):
    """
    Tests of accessing array elements.
    """

    # pylint: disable=too-few-public-methods

    class_to_test = None
    locus = None

    @classmethod
    def setUpClass(cls): # pylint: disable=invalid-name
        # pylint: disable=missing-docstring
        cls.input_x = np.array(
            [[41, 42], [43, 44]],
            dtype=cls.class_to_test._underlying_scalar.numpy_type,
        )
        cls.fun = cls.class_to_test._check_and_get_element

    def good_array_validate(self):
        """
        Correct operation.
        """
        data_el = self.fun(
            array.array(
                self.class_to_test.array_typecodes[0],
                self.input_x.flatten()
            ), self.locus, (0,), True,
        )
        self.assertEqual( # pylint: disable=no-member
            data_el,
            self.input_x[0, 0], # pylint: disable=unsubscriptable-object
        )

    def good_ctarray_validate(self):
        """
        Correct operation.
        """
        data_el = self.fun(
            (self.class_to_test.c_type._type_*self.input_x.size)(
                *self.input_x.flatten()
            ),
            self.locus, (0,), True,
        )
        self.assertEqual( # pylint: disable=no-member
            data_el,
            self.input_x[0, 0], # pylint: disable=unsubscriptable-object
        )

    def test_bad_index(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=no-member
            b_utils.NagIndexError,
            'requested index.* 4 .*is out of range',
            self.fun,
            self.input_x.flatten(), self.locus, (self.input_x.size,), True,
        )

    def test_bad_get_index(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=no-member
            b_utils.NagIndexError,
            'requested index.* .4, 4. .*is out of range',
            self.class_to_test.get_element,
            self.input_x, self.locus, (self.input_x.size, self.input_x.size),
        )

    def test_bad_list(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=no-member
            b_utils.NagTypeError,
            'must be an instance of',
            self.fun,
            [None], self.locus, (0,), True,
        )

    def test_good_get_no_validate(self):
        """
        Correct operation.
        """
        data_el = self.class_to_test.get_element(
            [None], self.locus, (0,), validate=False,
        )
        self.assertIsNone( # pylint: disable=no-member
            data_el,
        )

    def test_good_get_validate(self):
        """
        Correct operation.
        """
        data_el = self.class_to_test.get_element(
            list(self.input_x.flatten()), self.locus, (0,),
        )
        self.assertEqual( # pylint: disable=no-member
            data_el,
            self.input_x[0, 0], # pylint: disable=unsubscriptable-object
        )

    def test_good_ndarray_1_validate(self):
        """
        Correct operation, 1-dimensional data.
        """
        data_el = self.fun(
            self.input_x.flatten(), self.locus, (0,), True,
        )
        self.assertEqual( # pylint: disable=no-member
            data_el,
            self.input_x[0, 0], # pylint: disable=unsubscriptable-object
        )

    def test_good_ndarray_validate(self):
        """
        Correct operation, multidimensional data.
        """
        data_el = self.fun(
            self.input_x, self.locus, (1, 0), True,
        )
        self.assertEqual( # pylint: disable=no-member
            data_el,
            self.input_x[1, 0], # pylint: disable=unsubscriptable-object
        )

    def good_return_type(self):
        """
        Python data should be returned.
        """
        data_el = self.fun(
            [self.class_to_test._underlying_scalar.c_type(self.input_x[0, 0])],
            self.locus, (0,), True,
        )
        self.assertEqual( # pylint: disable=no-member
            data_el,
            self.input_x[0, 0], # pylint: disable=unsubscriptable-object
        )

class ElementTestArrayChar(ElementTestArray, InputTest):
    """
    Element-test subsuite.
    """

    class_to_test = b_utils._EngineCharArrayType

    def test_return_type_bytes(self):
        """
        str should be returned.
        """
        data_el = self.fun(
            [b'Hello'], self.locus, (0,), True,
        )
        self.assertEqual( # pylint: disable=no-member
            data_el, 'Hello',
        )

    def test_return_type_unicode(self):
        """
        str should be returned.
        """
        data_el = self.fun(
            [u'Hello'], self.locus, (0,), True,
        )
        self.assertEqual( # pylint: disable=no-member
            data_el, 'Hello',
        )

    def test_bad_np_nonascii(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            'only ASCII',
            self.fun,
            np.array(['Pi\xf1a']), self.locus, (0,), True,
        )

class ElementTestArrayComplex64(ElementTestArray, InputTest):
    """
    Element-test subsuite.
    """

    class_to_test = b_utils._EngineComplex64ArrayType

class ElementTestArrayComplex128(ElementTestArray, InputTest):
    """
    Element-test subsuite.
    """

    class_to_test = b_utils._EngineComplex128ArrayType

class ElementTestArrayFloat32(ElementTestArray, InputTest):
    """
    Element-test subsuite.
    """

    class_to_test = b_utils._EngineFloat32ArrayType

    def test_good_array_validate(self):
        """
        Correct operation.
        """
        self.good_array_validate()

    def test_good_ctarray_validate(self):
        """
        Correct operation.
        """
        self.good_ctarray_validate()

    def test_good_return_type(self):
        """
        Correct operation.
        """
        self.good_return_type()

    def test_bad_ctype(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            '_type_.* must be',
            self.fun,
            (ctypes.c_int*1)(42), self.locus, (0,), True,
        )

    def test_bad_dtype(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'dtype.* must be',
            self.fun,
            np.array([42.5]), self.locus, (0,), True,
        )

    def test_bad_typecode(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'typecode.* must be',
            self.fun,
            array.array('l', [42]), self.locus, (0,), True,
        )

class ElementTestArrayFloat64(ElementTestArray, InputTest):
    """
    Element-test subsuite.
    """

    class_to_test = b_utils._EngineFloat64ArrayType

    def test_good_array_validate(self):
        """
        Correct operation.
        """
        self.good_array_validate()

    def test_good_ctarray_validate(self):
        """
        Correct operation.
        """
        self.good_ctarray_validate()

    def test_good_return_type(self):
        """
        Correct operation.
        """
        self.good_return_type()

class ElementTestArrayInt(ElementTestArray, InputTest):
    """
    Element-test subsuite.
    """

    class_to_test = b_utils._EngineIntArrayType

    def test_good_array_validate(self):
        """
        Correct operation.
        """
        self.good_array_validate()

    def test_good_ctarray_validate(self):
        """
        Correct operation.
        """
        self.good_ctarray_validate()

    def test_good_return_type(self):
        """
        Correct operation.
        """
        self.good_return_type()

    @unittest.skipUnless(
        b_utils._EngineState._int_model['bits'] == 32,
        'integer overflow not possible for this data type',
    )
    def test_bad_ctype(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOverflowError,
            'value must not overflow',
            self.fun,
            (b_utils._SYS_CTYPES_INTS[64]*1)(2**40),
            self.locus, (0,), True,
        )

    @unittest.skipUnless(
        b_utils._EngineState._int_model['bits'] == 32,
        'integer overflow not possible for this data type',
    )
    def test_bad_np_oflow(self):
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOverflowError,
            'value must not overflow',
            self.fun,
            np.array([2**40], dtype=np.int64), self.locus, (0,), True,
        )

    @unittest.skipUnless(
        (
            not b_utils._IS_IL32 and
            b_utils._EngineState._int_model['bits'] == 32
        ),
        'array of "l" cannot overflow when builtin int is 32-bit',
    )
    # Don't have I32LP64 config. currently.
    def test_bad_typecode(self): # pragma: no cover
        """
        Bad call.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOverflowError,
            'value must not overflow',
            self.fun,
            array.array('l', [2**40]), self.locus, (0,), True,
        )

class ElementTestArrayLogical(ElementTestArray, InputTest):
    """
    Element-test subsuite.
    """

    class_to_test = b_utils._EngineLogicalArrayType

    @classmethod
    def setUpClass(cls):
        cls.input_x = np.array(
            [[True, True], [False, False]],
            dtype=cls.class_to_test._underlying_scalar.numpy_type,
        )
        cls.fun = cls.class_to_test._check_and_get_element

class EmptyTestScalar(object):
    """
    Tests of returning empty ctypes arrays.
    """

    class_to_test = None
    chrlen = None

    @classmethod
    def setUp(cls): # pylint: disable=invalid-name
        """
        Set up.
        """
        cls.empty_ctypes = cls.class_to_test.empty_ctypes(chrlen=cls.chrlen)

    def test_good(self):
        """
        Correct operation.
        """
        self.assertIs( # pylint: disable=no-member
            self.empty_ctypes._type_,
            self.class_to_test.c_type._type_
        )

class EmptyTestScalarChar(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.

    Extends test_good.
    """

    class_to_test = b_utils._EngineCharScalarType
    chrlen = 2
    empty_len = chrlen

    def test_good(self):
        """
        Check the length.
        """
        EmptyTestScalar.test_good(self)
        self.assertEqual(
            len(self.empty_ctypes),
            self.empty_len,
        )

    def test_good_null_chrlen(self):
        """
        Correct operation.
        """
        self.chrlen = -1
        self.empty_len = 0
        self.empty_ctypes = self.class_to_test.empty_ctypes(chrlen=self.chrlen)
        self.test_good()

class EmptyTestScalarComplex64(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.

    Overrides test_good.
    """

    class_to_test = b_utils._EngineComplex64ScalarType

    def test_good(self):
        """
        Correct operation.
        """
        self.assertIsInstance(
            self.empty_ctypes,
            self.class_to_test.c_type
        )

class EmptyTestScalarComplex128(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineComplex128ScalarType

    def test_good(self):
        """
        Correct operation.
        """
        self.assertIsInstance(
            self.empty_ctypes,
            self.class_to_test.c_type
        )

class EmptyTestScalarCptr(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineCptrScalarType

    def test_good(self):
        self.assertIsInstance( # pylint: disable=no-member
            self.empty_ctypes, b_utils.Handle,
        )

class EmptyTestScalarFloat32(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineFloat32ScalarType

class EmptyTestScalarFloat64(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineFloat64ScalarType

class EmptyTestScalarInt(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineIntScalarType

class EmptyTestScalarLogical(EmptyTestScalar, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineLogicalScalarType

class EmptyTestArray(object):
    """
    Tests of returning empty arrays.
    """

    base_instance = ctypes.Array
    class_to_test = None
    shape = (2, 3)
    empty_shape = shape
    size = 100
    sorder = 'F'
    chrlen = None
    empty_chrlen = None

    @classmethod
    def setUp(cls): # pylint: disable=invalid-name
        """
        Set up.
        """
        cls.empty_ctypes = cls.class_to_test.empty_ctypes(
            cls.size, chrlen=cls.chrlen
        )
        cls.empty_ndarray = cls.class_to_test.empty_ndarray(
            cls.shape, chrlen=cls.chrlen, sorder=cls.sorder
        )

    def test_good_ctypes(self):
        """
        Correct operation.
        """
        self.assertIsInstance( # pylint: disable=no-member
            self.empty_ctypes,
            self.base_instance,
        )
        self.assertIs( # pylint: disable=no-member
            self.empty_ctypes._type_,
            self.class_to_test.c_type._type_
        )
        exp_size = self.size
        if self.chrlen is not None:
            exp_size *= self.chrlen
        self.good_ctypes_size(exp_size)

    def test_good_ctypes_null(self):
        """
        Correct operation.
        """
        self.size = -1
        self.empty_ctypes = self.class_to_test.empty_ctypes(
            self.size, chrlen=self.chrlen
        )
        self.assertIsInstance( # pylint: disable=no-member
            self.empty_ctypes,
            self.base_instance,
        )
        self.assertIs( # pylint: disable=no-member
            self.empty_ctypes._type_,
            self.class_to_test.c_type._type_
        )
        exp_size = 0
        if self.chrlen is not None:
            exp_size *= self.chrlen
        self.good_ctypes_size(exp_size)

    def good_ctypes_size(self, exp_size):
        """
        Verify expected size.
        """
        self.assertEqual( # pylint: disable=no-member
            len(self.empty_ctypes),
            exp_size,
        )

    def good_ndarray(self):
        """
        Verify the expected properties of the ndarray.
        """
        self.assertIsInstance( # pylint: disable=no-member
            self.empty_ndarray,
            np.ndarray,
        )
        self.assertTrue( # pylint: disable=no-member
            np.issubdtype(
                self.empty_ndarray.dtype,
                self.class_to_test._underlying_scalar.numpy_type
            )
        )
        self.assertEqual( # pylint: disable=no-member
            self.empty_ndarray.shape,
            self.empty_shape,
        )
        if self.sorder == 'F':
            self.assertTrue(self.empty_ndarray.flags.f_contiguous) # pylint: disable=no-member
        else:
            self.assertTrue(self.empty_ndarray.flags.c_contiguous) # pylint: disable=no-member
        if self.chrlen is not None:
            self.assertEqual( # pylint: disable=no-member
                int(b_utils._byteorder_stripped_dtype(self.empty_ndarray)[1:]),
                self.empty_chrlen
            )

    def test_good_ndarray(self):
        """
        Correct operation.
        """
        self.good_ndarray()

    def test_good_ndarray_default_order(self):
        """
        Correct operation.
        """
        self.sorder = 'C'
        self.empty_ndarray = self.class_to_test.empty_ndarray(
            self.shape, chrlen=self.chrlen,
        )
        self.good_ndarray()

    def test_good_ndarray_nege(self):
        """
        Correct operation with a negative extent supplied.
        """
        self.shape = (2, -1)
        self.empty_shape = (2, 0)
        self.empty_ndarray = self.class_to_test.empty_ndarray(
            self.shape, chrlen=self.chrlen,
        )
        self.good_ndarray()

class EmptyTestArrayChar(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineCharArrayType
    chrlen = 2
    empty_chrlen = 2

    def test_good_ctypes_null_chrlen(self):
        """
        Correct operation.
        """
        self.chrlen = -1
        self.empty_ctypes = self.class_to_test.empty_ctypes(
            self.size, chrlen=self.chrlen
        )
        self.assertIsInstance( # pylint: disable=no-member
            self.empty_ctypes,
            self.base_instance,
        )
        self.assertIs( # pylint: disable=no-member
            self.empty_ctypes._type_,
            self.class_to_test.c_type._type_
        )
        exp_size = 0
        self.good_ctypes_size(exp_size)

    def test_good_ctypes_nonec(self):
        """
        Correct operation.
        """
        self.chrlen = None
        self.empty_ctypes = self.class_to_test.empty_ctypes(
            self.size, chrlen=self.chrlen
        )
        self.assertIsInstance( # pylint: disable=no-member
            self.empty_ctypes,
            self.base_instance,
        )
        self.assertIs( # pylint: disable=no-member
            self.empty_ctypes._type_,
            self.class_to_test.c_type._type_
        )
        exp_size = 0
        self.good_ctypes_size(exp_size)

    def test_good_ndarray_nege(self):
        """
        Correct operation with a negative extent supplied.
        """
        self.shape = (2, -1)
        self.empty_shape = (2, 0)
        self.empty_ndarray = self.class_to_test.empty_ndarray(
            self.shape, chrlen=self.chrlen,
        )
        self.good_ndarray()

    def test_good_ndarray_negc(self):
        """
        Correct operation with a negative chrlen supplied.
        """
        self.sorder = 'C'
        self.chrlen = -1
        self.empty_chrlen = 1
        self.empty_ndarray = self.class_to_test.empty_ndarray(
            self.shape, chrlen=self.chrlen,
        )
        self.good_ndarray()
        self.chrlen = 2
        self.empty_chrlen = None

    def test_good_ndarray_nonec(self):
        """
        Correct operation with a None chrlen supplied.
        """
        self.sorder = 'C'
        self.chrlen = None
        self.empty_chrlen = 0
        self.empty_ndarray = self.class_to_test.empty_ndarray(
            self.shape, chrlen=self.chrlen,
        )
        self.good_ndarray()
        self.chrlen = 2
        self.empty_chrlen = None

class EmptyTestArrayComplex64(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineComplex64ArrayType

class EmptyTestArrayComplex128(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineComplex128ArrayType

class EmptyTestArrayCptr(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineCptrArrayType

    def setUp(self): # pylint: disable=invalid-name
        """
        Set up.
        """
        self.empty_ctypes = self.class_to_test.empty_ctypes(
            self.size, chrlen=self.chrlen
        )
        self.assertRaises(
            NotImplementedError,
            self.class_to_test.empty_ndarray,
            self.shape, chrlen=self.chrlen, sorder=self.sorder,
        )

    def test_good_ndarray_nege(self):
        pass

    def test_good_ndarray_default_order(self):
        pass

    def test_good_ndarray(self):
        pass

class EmptyTestArrayFloat32(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineFloat32ArrayType

class EmptyTestArrayFloat64(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineFloat64ArrayType

class EmptyTestArrayInt(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineIntArrayType

class EmptyTestArrayLogical(EmptyTestArray, unittest.TestCase):
    """
    Empty-test subsuite.
    """

    class_to_test = b_utils._EngineLogicalArrayType

class LeadDimTest(InputTest):
    """
    Tests computation of lead dimensions.
    """

    input_x_c = np.array([42. + i for i in range(10)]).reshape((2, 5))

    def test_good_c_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c, self.locus,
                self.input_x_c.ndim, 'C', 'l',
            ),
            5
        )

    def test_good_f_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                np.asfortranarray(self.input_x_c),
                self.locus, self.input_x_c.ndim, 'F', 'l',
            ),
            2
        )

    def test_good_vector_row(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c.reshape((1, 10)),
                self.locus, self.input_x_c.ndim, 'C', 'l',
            ),
            10
        )

    def test_good_vector_column(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c.reshape((10, 1)),
                self.locus, self.input_x_c.ndim, 'C', 'l',
            ),
            1
        )

    def test_good_none(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                None,
                self.locus, self.input_x_c.ndim, 'C', 'l',
                allow_none=True,
            ),
            0
        )

class MaxTest(InputTest):
    """
    Tests the safe arraymax method.
    """
    input_x = np.array([42, 44, 41, 43])

    def test_good_float64(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType(
                np.array(self.input_x, dtype=float),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_max(0, self.input_x.size - 1),
            44.
        )

    def test_good_int(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_max(0, self.input_x.size - 1),
            44,
        )

    def test_good_int_sub(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_max(0, 0),
            self.input_x[0],
        )

    def test_good_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                array.array('l', np.array(self.input_x, dtype=int)),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_max(0, self.input_x.size - 1),
            44,
        )

    def test_good_ctarray(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                (b_utils._EngineIntArrayType.c_type._type_*self.input_x.size)(
                    *self.input_x.flatten()
                ),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_max(0, self.input_x.size - 1),
            44,
        )

    def test_good_list(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                [i for i in self.input_x],
                self.locus, exp_shape=(len(self.input_x),),
            ).get_max(0, self.input_x.size - 1),
            44,
        )

    def test_good_list_ct(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                [
                    b_utils._SYS_CTYPES_INTS[32](i)
                    for i in self.input_x
                ],
                self.locus, exp_shape=(len(self.input_x),),
            ).get_max(0, self.input_x.size - 1),
            44,
        )

class MonotonicTest(InputTest):
    """
    Tests the safe monotonicity method.
    """
    input_x = np.array([1, 2, 3, 4])

    def test_good_float64(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineFloat64ArrayType(
                np.array(self.input_x, dtype=float),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, self.input_x.size - 1, 'increasing'),
        )

    def test_good_int(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, self.input_x.size - 1, 'increasing'),
        )

    def test_good_int_sub(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, 1, 'increasing'),
        )

    def test_good_array(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                array.array('l', np.array(self.input_x, dtype=int)),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, 1, 'increasing'),
        )

    def test_good_ctarray(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                (b_utils._EngineIntArrayType.c_type._type_*self.input_x.size)(
                    *self.input_x.flatten()
                ),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, 1, 'increasing'),
        )

    def test_good_list(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                [i for i in self.input_x],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, 1, 'increasing'),
        )

    def test_good_list_ct(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                [
                    b_utils._SYS_CTYPES_INTS[32](i)
                    for i in self.input_x
                ],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, 1, 'increasing'),
        )

    def test_not_incr(self):
        """
        Bad input.
        """
        self.assertFalse(
            b_utils._EngineIntArrayType(
                [1, 2, 4, 3],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, self.input_x.size - 1, 'increasing'),
        )

    def test_is_decr(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                [1, 2, 4, 3],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(2, self.input_x.size - 1, 'decreasing'),
        )

    def test_not_decr(self):
        """
        Bad input.
        """
        self.assertFalse(
            b_utils._EngineIntArrayType(
                [1, 2, 4, 3],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, self.input_x.size - 2, 'decreasing'),
        )

    def test_is_either(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                [1, 2, 4, 3],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(2, self.input_x.size - 1, 'either'),
        )

    def test_not_either(self):
        """
        Bad input.
        """
        self.assertFalse(
            b_utils._EngineIntArrayType(
                [1, 2, 4, 3],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_monotonic(0, self.input_x.size - 1, 'either'),
        )

class NonNegativeTest(InputTest):
    """
    Tests the safe non-negativity method.
    """
    input_x = np.array([1, 2, 3, 0])

    def test_good_float64(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineFloat64ArrayType(
                np.array(self.input_x, dtype=float),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, self.input_x.size - 1),
        )

    def test_good_int(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, self.input_x.size - 1),
        )

    def test_good_int_sub(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, 1),
        )

    def test_good_array(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                array.array('l', np.array(self.input_x, dtype=int)),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, 1),
        )

    def test_good_ctarray(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                (b_utils._EngineIntArrayType.c_type._type_*self.input_x.size)(
                    *self.input_x.flatten()
                ),
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, 1),
        )

    def test_good_list(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                [i for i in self.input_x],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, 1),
        )

    def test_good_list_ct(self):
        """
        Good input.
        """
        self.assertTrue(
            b_utils._EngineIntArrayType(
                [
                    b_utils._SYS_CTYPES_INTS[32](i)
                    for i in self.input_x
                ],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, 1),
        )

    def test_not_incr(self):
        """
        Bad input.
        """
        self.assertFalse(
            b_utils._EngineIntArrayType(
                [1, 2, -4, 3],
                self.locus, exp_shape=(len(self.input_x),),
            ).is_nonnegative(0, self.input_x.size - 1),
        )

class ProductTest(InputTest):
    """
    Tests the safe arrayproduct method.
    """
    input_x = np.array([1, 2, 3, 4])

    def test_good_float64(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType(
                np.array(self.input_x, dtype=float),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_product(0, self.input_x.size - 1),
            24.,
        )

    def test_good_int(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_product(0, self.input_x.size - 1),
            24,
        )

    def test_good_int_sub(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_product(0, 1),
            2,
        )

    def test_good_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                array.array('l', np.array(self.input_x, dtype=int)),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_product(0, 1),
            2,
        )

    def test_good_ctarray(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                (b_utils._EngineIntArrayType.c_type._type_*self.input_x.size)(
                    *self.input_x.flatten()
                ),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_product(0, 1),
            2,
        )

    def test_good_list(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                [i for i in self.input_x],
                self.locus, exp_shape=(len(self.input_x),),
            ).get_product(0, 1),
            2,
        )

    def test_good_list_ct(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                [
                    b_utils._SYS_CTYPES_INTS[32](i)
                    for i in self.input_x
                ],
                self.locus, exp_shape=(len(self.input_x),),
            ).get_product(0, 1),
            2,
        )

class SecondDimTest(InputTest):
    """
    Tests computation of 'second' dimensions.
    """

    input_x_c = np.array([42. + i for i in range(10)]).reshape((2, 5))

    def test_good_c_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c, self.locus,
                self.input_x_c.ndim, 'C', 's',
            ),
            2
        )

    def test_good_c3_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c.reshape((2, 5, 1)), self.locus,
                3, 'C', 's',
            ),
            5
        )

    def test_good_f_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                np.asfortranarray(self.input_x_c),
                self.locus, self.input_x_c.ndim, 'F', 's',
            ),
            5
        )

    def test_good_cf_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c.reshape((2, 5, 1), order='F'), self.locus,
                3, 'F', 's',
            ),
            5
        )

    def test_good_vector_row(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c.reshape((1, 10)),
                self.locus, self.input_x_c.ndim, 'C', 's',
            ),
            1
        )

    def test_good_vector_column(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                self.input_x_c.reshape((10, 1)),
                self.locus, self.input_x_c.ndim, 'C', 's',
            ),
            10
        )

    def test_good_none(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType.get_nd(
                None,
                self.locus, self.input_x_c.ndim, 'C', 's',
                allow_none=True,
            ),
            0
        )

class SumTest(InputTest):
    """
    Tests the safe arraysum method.
    """
    input_x = np.array([1, 2, 3, 4])

    def test_good_float64(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineFloat64ArrayType(
                np.array(self.input_x, dtype=float),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_sum(0, self.input_x.size - 1),
            10.,
        )

    def test_good_int(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_sum(0, self.input_x.size - 1),
            10,
        )

    def test_good_int_sub(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                np.array(self.input_x, dtype=int),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_sum(0, 1),
            3,
        )

    def test_good_array(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                array.array('l', np.array(self.input_x, dtype=int)),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_sum(0, 1),
            3,
        )

    def test_good_ctarray(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                (b_utils._EngineIntArrayType.c_type._type_*self.input_x.size)(
                    *self.input_x.flatten()
                ),
                self.locus, exp_shape=(len(self.input_x),),
            ).get_sum(0, 1),
            3,
        )

    def test_good_list(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                [i for i in self.input_x],
                self.locus, exp_shape=(len(self.input_x),),
            ).get_sum(0, 1),
            3,
        )

    def test_good_list_ct(self):
        """
        Good input.
        """
        self.assertEqual(
            b_utils._EngineIntArrayType(
                [
                    b_utils._SYS_CTYPES_INTS[32](i)
                    for i in self.input_x
                ],
                self.locus, exp_shape=(len(self.input_x),),
            ).get_sum(0, 1),
            3,
        )

class ComplexToFloatTest(InputTest):
    """
    Tests of the complex <-> 2d float utilities.
    """

    def test_bad_to_float(self):
        """
        Bad input.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagException,
            'Inconceivable data instance',
            np_utils._complex_to_float_array,
            np.array([42]), 'C',
        )

    def test_bad_to_complex(self):
        """
        Bad input.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagException,
            'Inconceivable data instance',
            np_utils._float_array_to_complex,
            np.array([42]),
        )

    def test_good_to_float32_f(self):
        """
        Good call.
        """
        float_array = np_utils._complex_to_float_array(
            np.array([42 + 43j], dtype=np.complex64),
            'F',
        )
        self.assertTrue(float_array.flags.f_contiguous)
        np.testing.assert_equal(
            float_array,
            np.array([[42], [43]], dtype=np.float32),
        )

    def test_good_to_float32_c(self):
        """
        Good call.
        """
        float_array = np_utils._complex_to_float_array(
            np.array([42 + 43j, 44 + 45j], dtype=np.complex64),
            'C',
        )
        self.assertTrue(float_array.flags.c_contiguous)
        np.testing.assert_equal(
            float_array,
            np.array([[42, 44], [43, 45]], dtype=np.float32),
        )

    def test_good_to_float64_f(self):
        """
        Good call.
        """
        float_array = np_utils._complex_to_float_array(
            np.array([42 + 43j], dtype=complex),
            'F',
        )
        self.assertTrue(float_array.flags.f_contiguous)
        np.testing.assert_equal(
            float_array,
            np.array([[42], [43]], dtype=float),
        )

    def test_good_to_float64_c(self):
        """
        Good call.
        """
        float_array = np_utils._complex_to_float_array(
            np.array([42 + 43j], dtype=complex),
            'C',
        )
        self.assertTrue(float_array.flags.c_contiguous)
        np.testing.assert_equal(
            float_array,
            np.array([[42], [43]], dtype=float),
        )

    def test_good_to_complex64(self): # pylint: disable=no-self-use
        """
        Good call.
        """
        complex_array = np_utils._float_array_to_complex(
            np.array([[42, 44], [43, 45]], dtype=np.float32),
        )
        np.testing.assert_equal(
            complex_array,
            np.array([42 + 43j, 44 + 45j], dtype=np.complex64),
        )

    def test_good_to_complex128(self): # pylint: disable=no-self-use
        """
        Good call.
        """
        complex_array = np_utils._float_array_to_complex(
            np.array([[42, 44], [43, 45]], dtype=float),
        )
        np.testing.assert_equal(
            complex_array,
            np.array([42 + 43j, 44 + 45j], dtype=complex),
        )

class OrderTest(InputTest):
    """
    Tests of uniform storage-ordering requirements.
    """

    # pylint: disable=too-many-public-methods

    base_list = [42. + i for i in range(10)]

    def test_bad(self):
        """
        Bad input.
        """
        input_x_c = np.array(self.base_list).reshape((2, 5))
        input_x_f = np.asfortranarray(input_x_c)
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                'function ' + self.locus['fun_name'] + ', argument x_f'
                '.*arrays on input.*consistent storage order'
            ),
            b_utils._get_mutual_sorder,
            self.locus, ((input_x_c, 'x_c'), (input_x_f, 'x_f'),),
        )

    def test_bad_cb_vs_wrap(self):
        """
        Bad input.
        """
        input_x_c = np.array(self.base_list).reshape((2, 5))
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                'function ' + self.locus['fun_name'] +
                ".*must use the same storage order.*"
                "callback storage order 'C'.*function used storage order 'F'$"
            ),
            np_utils._check_callback_sorder,
            self.locus, ((input_x_c, 'x_c'),), **{'wrapper_sorder': 'F'}
        )

    def test_bad_cb(self):
        """
        Bad input.
        """
        input_x_c = np.array(self.base_list).reshape((2, 5))
        input_x_f = np.array(self.base_list).reshape((2, 5), order='F')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                'function ' + self.locus['fun_name'] + ', returned x_f'
                '.*arrays returned.*consistent storage order'
            ),
            np_utils._check_callback_sorder,
            self.locus,
            ((input_x_c, 'x_c'), (input_x_f, 'x_f')),
            **{'wrapper_sorder': 'F'}
        )

    def test_bad_cb_list(self):
        """
        Bad input.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagException,
            'returned x.*not a multidimensional ndarray',
            np_utils._check_callback_sorder,
            self.locus,
            ((self.base_list, 'x'),),
            **{'wrapper_sorder': 'F'}
        )

    def test_bad_cb_spiked(self):
        """
        Bad input.
        """
        input_x = np.array(self.base_list).reshape((10, 1))
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                'function ' + self.locus['fun_name'] +
                ".*must use the same storage order.*"
                "callback storage order 'F'.*function used storage order 'C'$"
            ),
            np_utils._check_callback_sorder,
            self.locus,
            ((input_x, 'x'),),
            **{'wrapper_sorder': 'C'}
        )

    def test_bad_disctg_c(self):
        """
        Bad input, totally discontiguous, C storage to start.
        """
        input_x_c = np.array(self.base_list).reshape((2, 5))
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                'function ' + self.locus['fun_name'] +
                ".*all arrays in the argument list must be contiguous.*"
            ),
            b_utils._get_mutual_sorder,
            self.locus, ((input_x_c[:, :-1], 'x_c'),),
        )

    def test_bad_disctg_f(self):
        """
        Bad input, totally discontiguous, Fortran storage to start.
        """
        input_x_f = np.array(self.base_list).reshape((2, 5), order='F')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                'function ' + self.locus['fun_name'] +
                ".*all arrays in the argument list must be contiguous.*"
            ),
            b_utils._get_mutual_sorder,
            self.locus, ((input_x_f[:-1, :], 'x_f'),),
        )

    def test_bad_empty(self):
        """
        Bad input, no arrays, invalid spiked_sorder.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            (
                'function ' + self.locus['fun_name'] +
                ".*supplied spiked_sorder was not.*"
            ),
            b_utils._get_mutual_sorder,
            self.locus, (), spiked_sorder='?',
        )

    def test_good_empty(self):
        """
        Good input.
        """
        spiked_sorder = 'F'
        self.assertEqual(
            b_utils._get_mutual_sorder(
                self.locus, (), spiked_sorder=spiked_sorder,
            ),
            spiked_sorder,
        )

    def test_good_f_and_vector(self):
        """
        Good input.
        """
        spiked_sorder = (
            'C' if np.lib.NumpyVersion(np.__version__) < '1.12.0'
            else 'F'
        )
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                (
                    np.array(self.base_list).reshape(
                        (2, 5), order=spiked_sorder),
                    's'
                ),
                (np.array(self.base_list).reshape((1, 10)), 'b'),
            )
        )
        self.assertEqual(sorder, spiked_sorder)

    def test_good_vector_and_c(self):
        """
        Good input.
        """
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                (np.array(self.base_list).reshape((1, 10)), 'x1'),
                (np.array(self.base_list).reshape((2, 5)), 'x2'),
            ),
        )
        self.assertEqual(sorder, 'C')

    def test_good_list_and_c(self):
        """
        Good input.
        """
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                ([4, 3, 2, 1], 'x1'),
                (np.array(self.base_list).reshape((2, 5)), 'x2'),
            ),
        )
        self.assertEqual(sorder, 'C')

    def test_good_vector_and_vector(self):
        """
        Good input.
        """
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                (np.array(self.base_list).reshape((1, 10)), 'x1'),
                (np.array(self.base_list).reshape((1, 10)), 'x2'),
            ),
        )

        spiked_sorder = (
            'C' if np.lib.NumpyVersion(np.__version__) < '1.12.0'
            else 'F'
        )
        self.assertEqual(sorder, spiked_sorder)

    def test_good_all_none(self):
        """
        Good input.
        """
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                (None, 'x1'),
                (None, 'x2'),
            ),
        )
        self.assertEqual(sorder, b_utils._EngineData.default_sorder)

    def test_good_some_none(self):
        """
        Good input.
        """
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                (None, 'x1'),
                (np.array(self.base_list).reshape((2, 5), order='F'), 'fx1'),
            ),
        )
        self.assertEqual(sorder, 'F')

    def test_good_cb(self):
        """
        Good input.
        """
        np_utils._check_callback_sorder(
            self.locus,
            (
                (np.array(self.base_list).reshape((2, 5)), 'x1'),
                (np.array(self.base_list).reshape((2, 5)), 'x2'),
            ),
            wrapper_sorder='C'
        )

    def test_good_cb_spiked_default(self):
        """
        Good input.
        """
        np_utils._check_callback_sorder(
            self.locus,
            (
                (np.array(self.base_list).reshape((1, 10)), 'x1'),
            ),
            spiked_sorder=b_utils._EngineData.default_spiked_sorder,
            wrapper_sorder=b_utils._EngineData.default_spiked_sorder,
        )

    def test_good_cb_spiked_f(self):
        """
        Good input.
        """
        np_utils._check_callback_sorder(
            self.locus,
            (
                (np.array(self.base_list).reshape((1, 10)), 'x1'),
            ),
            spiked_sorder='F',
            wrapper_sorder='F',
        )

    def test_good_cb_spiked_c(self):
        """
        Good input.
        """
        np_utils._check_callback_sorder(
            self.locus,
            (
                (np.array(self.base_list).reshape((1, 10)), 'x1'),
            ),
            spiked_sorder='C',
            wrapper_sorder='C',
        )

    def test_good_cb_no_wrap_order(self):
        """
        Good input, default order.
        """
        np_utils._check_callback_sorder(
            self.locus,
            (
                (np.array(self.base_list).reshape((2, 5)), 'x1'),
                (np.array(self.base_list).reshape((2, 5)), 'x2'),
            ),
        )

    def test_good_spiked_default(self):
        """
        Good input.
        """
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                (
                    np.array(self.base_list).reshape(
                        (len(self.base_list), 1)
                    ), 'x1',
                ),
            ),
        )
        self.assertEqual(sorder, 'F')

    def test_good_spiked(self):
        """
        Good input.
        """
        sorder = b_utils._get_mutual_sorder(
            self.locus,
            (
                (
                    np.array(self.base_list).reshape(
                        (len(self.base_list), 1)
                    ), 'x1',
                ),
            ),
            spiked_sorder='C',
        )
        self.assertEqual(sorder, 'C')

    def test_bad_spiked_type(self):
        """
        Bad spiked_sorder supplied.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be an instance of.*str',
            b_utils._get_mutual_sorder,
            self.locus,
            (
                (
                    np.array(self.base_list).reshape(
                        (len(self.base_list), 1)
                    ), 'x1',
                ),
            ),
            spiked_sorder=42,
        )

    def test_bad_spiked_value(self):
        """
        Bad spiked_sorder supplied.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagValueError,
            self.locus_re + '.*supplied spiked_sorder was not \'',
            b_utils._get_mutual_sorder,
            self.locus,
            (
                (
                    np.array(self.base_list).reshape(
                        (len(self.base_list), 1)
                    ), 'x1',
                ),
            ),
            spiked_sorder='X',
        )

class OverflowTestScalarInt(InputTest):
    """
    Tests of overflow checking.
    """

    import sys

    class_to_test = b_utils._EngineIntScalarType
    err_msg = 'value must not overflow.*'
    ok_32 = 2**20
    ok_64 = 2**40
    ok_128 = 2**80

    if b_utils._EngineState._int_model['bits'] == 64:

        @unittest.skip(
            'ctypes int64 type cannot be outside 64-bit range'
        )
        def test_bad_ctypes(self):
            """
            Overflowing ctypes_int64.
            """
            self.fail('inconceivable') # pragma: no cover

        def test_good_ctypes_32(self):
            """
            In-64-bit-range ctypes int32 (i.e., any).
            """
            input_x = b_utils._SYS_CTYPES_INTS[32](-self.ok_32)
            self.class_to_test(
                input_x, self.locus,
            )

        def test_good_ctypes_64(self):
            """
            In-64-bit-range ctypes int64 (i.e., any).
            """
            input_x = b_utils._SYS_CTYPES_INTS[64](self.ok_64)
            self.class_to_test(
                input_x, self.locus,
            )

        @unittest.skipIf(
            sys.version_info < (3,),
            'builtin int is at most 64-bit on this platform, ' +
            'so cannot be out of range'
        )
        def test_bad_int(self):
            """
            Overflowing 64-bit builtin int.
            """
            input_x = self.ok_128
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus,
            )

        def test_good_int(self):
            """
            In-64-bit-range builtin int.
            """
            input_x = self.ok_64
            self.class_to_test(
                input_x, self.locus,
            )

        @unittest.skip(
            'numpy.int64 cannot be outside 64-bit range'
        )
        def test_bad_numpy(self):
            """
            Overflowing numpy.int64.
            """
            self.fail('inconceivable') # pragma: no cover

        def test_good_numpy_32(self):
            """
            In-64-bit-range numpy.int32 (i.e., any).
            """
            input_x = np.int32(self.ok_32)
            self.class_to_test(
                input_x, self.locus,
            )

        def test_good_numpy_64(self):
            """
            In-64-bit-range numpy.int64 (i.e., any).
            """
            input_x = np.int64(-self.ok_64)
            self.class_to_test(
                input_x, self.locus,
            )

    else:

        def test_bad_ctypes(self):
            """
            Overflowing 64-bit ctypes.
            """
            input_x = b_utils._SYS_CTYPES_INTS[64](self.ok_64)
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus,
            )

        def test_good_ctypes_32(self):
            """
            In-32-bit-range ctypes int32.
            """
            input_x = b_utils._SYS_CTYPES_INTS[32](self.ok_32)
            self.class_to_test(
                input_x, self.locus,
            )

        def test_good_ctypes_64(self):
            """
            In-32-bit-range ctypes int64.
            """
            input_x = b_utils._SYS_CTYPES_INTS[64](-self.ok_32)
            self.class_to_test(
                input_x, self.locus,
            )

        def test_bad_int(self):
            """
            Overflowing 64-bit builtin int.
            """
            input_x = self.ok_64
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus,
            )

        def test_good_int(self):
            """
            In-32-bit-range builtin int.
            """
            input_x = self.ok_32
            self.class_to_test(
                input_x, self.locus,
            )

        def test_bad_numpy(self):
            """
            Overflowing numpy.int64.
            """
            input_x = np.int64(self.ok_64)
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus,
            )

        def test_good_numpy_32(self):
            """
            In-32-bit-range numpy.int32.
            """
            input_x = np.int32(-self.ok_32)
            self.class_to_test(
                input_x, self.locus,
            )

        def test_good_numpy(self):
            """
            In-32-bit-range numpy.int64.
            """
            input_x = np.int64(self.ok_32)
            self.class_to_test(
                input_x, self.locus,
            )

class OverflowTestArrayInt(InputTest):
    """
    Tests of overflow checking.
    """

    # pylint: disable=too-many-public-methods

    import sys

    class_to_test = b_utils._EngineIntArrayType
    err_msg = 'value must not overflow.*'
    ok_32 = 2**20
    ok_64 = 2**40
    ok_128 = 2**80

    if b_utils._EngineState._int_model['bits'] == 64:

        @unittest.skip(
            'array of "l" (or "q") cannot be outside 64-bit range'
        )
        def test_bad_array(self):
            """
            Overflowing array of "l" or "q".
            """
            self.fail('inconceivable') # pragma: no cover

        def test_good_array_l(self):
            """
            In-64-bit-range array of "l" (i.e., any).
            """
            input_x = array.array('l', [42])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        @unittest.skipUnless(
            sys.version_info >= (3,),
            'no "q" array typecode'
        )
        def test_good_array_q(self):
            """
            In-64-bit-range array of "q" (i.e., any).
            """
            input_x = array.array('q', [42])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        @unittest.skip(
            'ctypes Array of int64 type cannot be outside 64-bit range'
        )
        def test_bad_ctypes(self):
            """
            Overflowing Array of ctypes_int64.
            """
            self.fail('inconceivable') # pragma: no cover

        def test_good_ctypes_32(self):
            """
            In-64-bit-range ctypes int32 (i.e., any).
            """

            class CtypesTestArray32(ctypes.Array):
                # pylint: disable=missing-docstring,too-few-public-methods
                _length_ = 1
                _type_ = b_utils._SYS_CTYPES_INTS[32]

            input_x = CtypesTestArray32(-self.ok_32)
            self.class_to_test(
                input_x, self.locus, (1,)
            )

        def test_good_ctypes_64(self):
            """
            In-64-bit-range ctypes int64 (i.e., any).
            """

            class CtypesTestArray64(ctypes.Array):
                # pylint: disable=missing-docstring,too-few-public-methods
                _length_ = 1
                _type_ = b_utils._SYS_CTYPES_INTS[64]

            input_x = CtypesTestArray64(self.ok_64)
            self.class_to_test(
                input_x, self.locus, (1,)
            )

        @unittest.skipUnless(
            (
                not b_utils._IS_IL32 and
                sys.version_info >= (3,)
            ),
            'builtin int is at most 64-bit on this platform, ' +
            'so cannot be out of range'
        )
        def test_bad_list(self):
            """
            Overflowing list, checking enabled.
            """
            input_x = [self.ok_64, self.ok_128]
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*element 1.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus, (len(input_x),),
            )

        @unittest.skipUnless(
            (
                not b_utils._IS_IL32 and
                sys.version_info >= (3,)
            ),
            'builtin int is at most 64-bit on this platform, ' +
            'so cannot be out of range'
        )
        def test_bad_list_disabled(self):
            """
            Overflowing list, checking disabled.
            """
            input_x = [self.ok_64, self.ok_128]
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
                check_contents=False
            )

        def test_good_list(self):
            """
            In-64-bit-range list.
            """
            input_x = [self.ok_64, self.ok_32]
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        @unittest.skip(
            'numpy.ndarray of numpy.int64 cannot be outside 64-bit range'
        )
        def test_bad_numpy(self):
            """
            Overflowing rank-1 numpy.ndarray of numpy.int64.
            """
            self.fail('inconceivable') # pragma: no cover

        @unittest.skip(
            'numpy.ndarray of numpy.int64 cannot be outside 64-bit range'
        )
        def test_bad_numpy_multidim(self):
            """
            Overflowing multidimensional numpy.ndarray of numpy.int64.
            """
            self.fail('inconceivable') # pragma: no cover

        def test_good_numpy_32(self):
            """
            In-64-bit-range numpy.ndarray of numpy.int32 (i.e., any).
            """
            input_x = np.array([self.ok_32, -self.ok_64], dtype=np.int64)
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        def test_good_numpy_64(self):
            """
            In-64-bit-range numpy.ndarray of numpy.int64 (i.e., any).
            """
            input_x = np.array([self.ok_32, -self.ok_32], dtype=np.int32)
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

    else:

        @unittest.skipUnless(
            not b_utils._IS_IL32,
            'array of "l" cannot overflow when builtin int is 32-bit'
        )
        # Don't have I32LP64 config. currently.
        def test_bad_array_l(self): # pragma: no cover
            """
            Overflowing array of "l", checking enabled.
            """
            input_x = array.array('l', [self.ok_32, self.ok_64])
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*element 1.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus, (len(input_x),),
            )

        @unittest.skipUnless(
            not b_utils._IS_IL32,
            'array of "l" cannot overflow when builtin int is 32-bit'
        )
        # Don't have I32LP64 config. currently.
        def test_bad_array_l_disabled(self): # pragma: no cover
            """
            Overflowing array of "l", checking disabled.
            """
            input_x = array.array('l', [self.ok_32, self.ok_64])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
                check_contents=False
            )

        @unittest.skipUnless(
            sys.version_info >= (3,),
            'no "q" array typecode'
        )
        def test_bad_array_q(self):
            """
            Overflowing array of "q", checking enabled.
            """
            input_x = array.array('q', [self.ok_32, self.ok_64])
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*element 1.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus, (len(input_x),),
            )

        @unittest.skipUnless(
            sys.version_info >= (3,),
            'no "q" array typecode'
        )
        def test_bad_array_q_disabled(self):
            """
            Overflowing array of "q", checking disabled.
            """
            input_x = array.array('q', [self.ok_32, self.ok_64])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
                check_contents=False
            )

        def test_good_array_l(self):
            """
            In-32-bit-range array of "l".
            """
            input_x = array.array('l', [42])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        @unittest.skipUnless(
            sys.version_info >= (3,),
            'no "q" array typecode'
        )
        def test_good_array_q(self):
            """
            In-32-bit-range array of "q".
            """
            input_x = array.array('q', [42])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        def test_bad_ctypes(self):
            """
            Overflowing Array of ctypes int64, checking enabled.
            """

            class CtypesTestArray64(ctypes.Array):
                # pylint: disable=missing-docstring,too-few-public-methods
                _length_ = 2
                _type_ = b_utils._SYS_CTYPES_INTS[64]

            input_x = CtypesTestArray64(*[self.ok_32, self.ok_64])
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*element 1.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus, (len(input_x),),
            )

        def test_bad_ctypes_disabled(self):
            """
            Overflowing Array of ctypes int64, checking disabled.
            """

            class CtypesTestArray64(ctypes.Array):
                # pylint: disable=missing-docstring,too-few-public-methods
                _length_ = 2
                _type_ = b_utils._SYS_CTYPES_INTS[64]

            input_x = CtypesTestArray64(*[self.ok_32, self.ok_64])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
                check_contents=False
            )

        def test_good_ctypes_32(self):
            """
            In-32-bit-range Array of ctypes int32.
            """

            class CtypesTestArray32(ctypes.Array):
                # pylint: disable=missing-docstring,too-few-public-methods
                _length_ = 1
                _type_ = b_utils._SYS_CTYPES_INTS[32]

            input_x = CtypesTestArray32(-self.ok_32)
            self.class_to_test(
                input_x, self.locus, (1,)
            )

        def test_good_ctypes_64(self):
            """
            In-32-bit-range Array of ctypes int64.
            """

            class CtypesTestArray64(ctypes.Array):
                # pylint: disable=missing-docstring,too-few-public-methods
                _length_ = 1
                _type_ = b_utils._SYS_CTYPES_INTS[64]

            input_x = CtypesTestArray64(self.ok_32)
            self.class_to_test(
                input_x, self.locus, (1,)
            )

        def test_bad_list(self):
            """
            Overflowing list, checking enabled.
            """
            input_x = [self.ok_32, self.ok_64]
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*element 1.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus, (len(input_x),),
            )

        def test_bad_list_disabled(self):
            """
            Overflowing list, checking disabled.
            """
            input_x = [self.ok_32, self.ok_64]
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
                check_contents=False
            )

        def test_good_list(self):
            """
            In-32-bit-range list.
            """
            input_x = [-self.ok_32, self.ok_32]
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        def test_bad_numpy(self):
            """
            Overflowing rank-1 numpy.ndarray of numpy.int64,
            checking enabled.
            """
            input_x = np.array([self.ok_32, self.ok_64])
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + '.*element 1.*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus, (len(input_x),),
            )

        def test_bad_numpy_disabled(self):
            """
            Overflowing rank-1 numpy.ndarray of numpy.int64,
            checking disabled.
            """
            input_x = np.array([self.ok_32, self.ok_64])
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
                check_contents=False
            )

        def test_bad_numpy_multidim(self):
            """
            Overflowing multidimensional numpy.ndarray of numpy.int64,
            checking enabled.
            """
            input_x = np.array([self.ok_32, self.ok_64]).reshape((1, 2))
            self.my_assert_raises_regex( #pylint: disable=deprecated-method
                b_utils.NagOverflowError,
                self.locus_re + r'.*element \[0, 1\].*' + self.err_msg + '$',
                self.class_to_test,
                input_x, self.locus, input_x.shape,
            )

        def test_bad_numpy_multidim_dis(self):
            """
            Overflowing multidimensional numpy.ndarray of numpy.int64,
            checking disabled.
            """
            input_x = np.array([self.ok_32, self.ok_64]).reshape((1, 2))
            self.class_to_test(
                input_x, self.locus, input_x.shape, 'C',
                check_contents=False
            )

        def test_good_numpy_32(self):
            """
            In-32-bit-range numpy.ndarray of numpy.int32.
            """

            input_x = np.array([self.ok_32, -self.ok_32], dtype=np.int64)
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

        def test_good_numpy(self):
            """
            In-32-bit-range numpy.ndarray of numpy.int64.
            """
            input_x = np.array([self.ok_32, -self.ok_32], dtype=np.int32)
            self.class_to_test(
                input_x, self.locus, (len(input_x),),
            )

class TypeTestScalarChar(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    import sys

    class_to_test = b_utils._EngineCharScalarType

    if sys.version_info < (3,):
        valid_names = 'numpy.string_, numpy.unicode_, str or unicode'
    else:
        valid_names = 'bytes, numpy.bytes_, numpy.str_ or str'

    def test_bad(self):
        input_x = 42
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus, None,
        )

    def test_bad_array(self):
        """
        Make sure list of single chars is rejected.
        """
        input_x = ['4', '2']
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus, None,
        )

    def test_good(self):
        input_x = '42.0'
        self.class_to_test(
            input_x, self.locus, None,
        )

    def test_good_bytes(self):
        """
        Test correct operation for bytes input.
        """
        input_x = b'42.0'
        self.class_to_test(
            input_x, self.locus, None,
        )

    def test_good_unicode(self):
        """
        Test correct operation for unicode input.
        """
        input_x = u'42.0'
        self.class_to_test(
            input_x, self.locus, None,
        )

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        input_x = np.str_('42.0')
        self.class_to_test(
            input_x, self.locus, None,
        )

    def test_good_numpy_bytes(self):
        """
        Test correct operation for numpy bytes input.
        """
        input_x = np.bytes_('42.0')
        self.class_to_test(
            input_x, self.locus, None,
        )

    def test_good_numpy_unicode(self):
        """
        Test correct operation for numpy unicode input.
        """
        input_x = np.unicode_('42.0')
        self.class_to_test(
            input_x, self.locus, None,
        )

class TypeTestScalarComplex64(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineComplex64ScalarType
    valid_names = 'numpy.complex64'

    def test_bad(self):
        input_x = 42
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus,
        )

    def test_good(self):
        input_x = np.complex64(42.0 + 43.0j)
        self.class_to_test(
            input_x, self.locus,
        )

class TypeTestScalarComplex128(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineComplex128ScalarType
    valid_names = 'complex or numpy.complex128'

    def test_bad(self):
        input_x = 42
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus,
        )

    def test_good(self):
        input_x = 42.0 + 43.0j
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        input_x = np.complex128(42.0 + 43.0j)
        self.class_to_test(
            input_x, self.locus,
        )

class TypeTestScalarCptr(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineCptrScalarType
    valid_names = 'naginterfaces.*Handle'

    def test_bad_shallow_copy(self):
        """
        Shallow copy attempted.
        """
        import copy
        handle = b_utils.Handle()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagAttributeError,
            'shallow copy not permitted',
            copy.copy,
            handle
        )

    def test_bad_deep_copy(self):
        """
        Deep copy attempted.
        """
        import copy
        handle = b_utils.Handle()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagAttributeError,
            'deep copy not permitted',
            copy.deepcopy,
            handle
        )

    def test_bad(self):
        input_x = 42
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re +
                '.*must be an instance of ' + self.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus,
        )

    def test_good(self):
        input_x = b_utils.Handle()
        self.class_to_test(
            input_x, self.locus,
        )

class TypeTestScalarFloat32(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineFloat32ScalarType
    valid_names = 'numpy.float32'

    def test_bad(self):
        input_x = 42
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus,
        )

    @unittest.skip('no builtin allowed for this datatype')
    def test_good(self):
        self.fail('inconceivable') # pragma: no cover

    def test_good_ctypes(self):
        """
        Test correct operation for ctypes input.
        """
        input_x = ctypes.c_float(42.0)
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        input_x = np.float32(42.0)
        self.class_to_test(
            input_x, self.locus,
        )

class TypeTestScalarFloat64(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineFloat64ScalarType
    valid_names = 'ctypes.c_double, float or numpy.float64'

    def test_bad(self):
        input_x = 42
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus,
        )

    def test_good(self):
        input_x = 42.0
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_ctypes(self):
        """
        Test correct operation for ctypes input.
        """
        input_x = ctypes.c_double(42.0)
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        input_x = np.float64(42.0)
        self.class_to_test(
            input_x, self.locus,
        )

class TypeTestScalarInt(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    import sys

    class_to_test = b_utils._EngineIntScalarType

    if b_utils._IS_IL32:
        ctypes_int32_name = 'ctypes.c_long'
        ctypes_int64_name = 'ctypes.c_longlong'
    else:
        ctypes_int32_name = 'ctypes.c_int'
        ctypes_int64_name = 'ctypes.c_long'

    if (
            sys.version_info < (3,) and
            b_utils._IS_IL32
    ):
        builtin_names = 'int, long'
    else:
        builtin_names = 'int'

    valid_names = (
        ctypes_int32_name + ', ' + ctypes_int64_name + ', ' +
        builtin_names + ', ' +
        'numpy.int32 or numpy.int64'
    )

    def test_bad(self):
        input_x = 42.0
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus,
        )

    def test_good(self):
        input_x = 42
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_ctypes_32(self):
        """
        Test correct operation for 32-bit ctypes input.
        """
        input_x = b_utils._SYS_CTYPES_INTS[32](42)
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_ctypes_64(self):
        """
        Test correct operation for 64-bit ctypes input.
        """
        input_x = b_utils._SYS_CTYPES_INTS[64](42)
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_numpy_32(self):
        """
        Test correct operation for 32-bit numpy input.
        """
        input_x = np.int32(42)
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_numpy_64(self):
        """
        Test correct operation for 64-bit numpy input.
        """
        input_x = np.int64(42)
        self.class_to_test(
            input_x, self.locus,
        )

class TypeTestScalarLogical(TestBadGood, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineLogicalScalarType

    valid_names = 'bool or numpy.bool_'

    def test_bad(self):
        input_x = 42
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_names + '$',
            self.class_to_test,
            input_x, self.locus,
        )

    def test_good(self):
        input_x = True
        self.class_to_test(
            input_x, self.locus,
        )

    def test_good_numpy(self):
        """
        Test correct operation for numpy input.
        """
        input_x = np.bool_(42)
        self.class_to_test(
            input_x, self.locus,
        )

class TestBadGoodContainer(object):
    """
    Base class for required test fixtures in array testing.
    """

    heterog_list_msg = 'must contain elements all of the same instance'
    multidim_container_msg = 'multidimensional.*numpy.ndarray'

    def test_bad_container(self):
        """
        Bad rank-1 container.
        """
        raise NotImplementedError # pragma: no cover

    def test_bad_container_multi(self):
        """
        Bad multidim container.
        """
        raise NotImplementedError # pragma: no cover

    def test_bad_array(self):
        """
        Bad input array.array.
        """
        raise NotImplementedError # pragma: no cover

    def test_good_array(self):
        """
        Good input array.array.
        """
        raise NotImplementedError # pragma: no cover

    def test_bad_ctypes(self):
        """
        Bad input ctypes.Array.
        """
        raise NotImplementedError # pragma: no cover

    def test_good_ctypes(self):
        """
        Good input ctypes.Array.
        """
        raise NotImplementedError # pragma: no cover

    def test_bad_list(self):
        """
        Bad input list.
        """
        raise NotImplementedError # pragma: no cover

    def test_good_list(self):
        """
        Good input list.
        """
        raise NotImplementedError # pragma: no cover

    def test_heterog_list(self):
        """
        Valid types in input list, but heterogeneous.
        """
        raise NotImplementedError # pragma: no cover

    def test_bad_ndarray(self):
        """
        Bad input np.ndarray.
        """
        raise NotImplementedError # pragma: no cover

    def test_good_ndarray(self):
        """
        Good rank-1 input np.ndarray.
        """
        raise NotImplementedError # pragma: no cover

    def test_good_ndarray_multidim(self):
        """
        Good multidimensional input np.ndarray.
        """
        raise NotImplementedError # pragma: no cover

class TestBadGoodShape(object):
    """
    Base class for required test fixtures in array shape testing.
    """

    # pylint: disable=no-member

    def test_bad_1dim(self):
        """
        Bad rank-1 input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have length.*$',
            self.class_to_test,
            self.good_input, self.locus,
            exp_shape=(len(self.good_input) - 1,)
        )

    def test_bad_get_container(self):
        """
        Bad container in get_shape for rank-1 input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be an instance of .*$',
            self.class_to_test.get_shape,
            tuple(self.good_input), self.locus,
            exp_rank=1,
        )

    def test_bad_get_1dim(self):
        """
        Bad get_shape for rank-1 input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have rank 2$',
            self.class_to_test.get_shape,
            self.good_input, self.locus,
            exp_rank=2,
        )

    def test_good_get_no_validate(self):
        """
        Correct operation.
        """
        self.assertEqual(
            self.class_to_test.get_shape(
                [None], self.locus, exp_rank=1, validate=False,
            ),
            (1,),
        )

    def test_good_1dim(self):
        """
        Good rank-1 input data.
        """
        self.class_to_test(
            self.good_input, self.locus,
            exp_shape=(len(self.good_input),)
        )

    def test_good_1dim_nocheck_nd(self):
        """
        Good rank-1 input data, checking disabled.
        """
        self.class_to_test(
            np.array(self.good_input), self.locus,
            exp_shape=(None,)
        )

    def test_good_1dim_nocheck(self):
        """
        Good rank-1 input data, checking disabled.
        """
        self.class_to_test(
            list(self.good_input), self.locus,
            exp_shape=(None,)
        )

    def test_good_get_1dim(self):
        """
        Good get_shape for rank-1 input data.
        """
        shape = self.class_to_test.get_shape(
            self.good_input, self.locus,
            exp_rank=1,
        )
        self.assertIsInstance(shape, tuple)
        self.assertEqual(shape, (len(self.good_input),))

    def test_bad_get_multidim(self):
        """
        Bad get_shape for multidimensional input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have rank 1$',
            self.class_to_test.get_shape,
            self.good_input.reshape((1, len(self.good_input))),
            self.locus,
            exp_rank=1,
        )

    def test_bad_multidim(self):
        """
        Bad multidimensional input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have shape.*$',
            self.class_to_test,
            self.good_input.reshape((1, len(self.good_input))),
            self.locus,
            exp_shape=(1, len(self.good_input) - 1)
        )

    def test_bad_multidim_nocheck(self):
        """
        Bad multidimensional input data, checking disabled in one dimension.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + r'.*must have shape \(:, 1\)$',
            self.class_to_test,
            self.good_input.reshape((1, len(self.good_input))),
            self.locus,
            exp_shape=(None, len(self.good_input) - 1)
        )

    def test_bad_multidim_nonneg(self):
        """
        Bad multidimensional input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re +
            '.*expected extent in dimension 2 .* must not be negative',
            self.class_to_test,
            self.good_input,
            self.locus,
            exp_shape=(1, -len(self.good_input)),
        )

    def test_bad_multidim_rank(self):
        """
        Bad multidimensional input data, rank too small.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have shape.*$',
            self.class_to_test,
            self.good_input.reshape((1, len(self.good_input))),
            self.locus,
            exp_shape=(1, len(self.good_input) - 1, 1)
        )

    def test_good_get_multidim(self):
        """
        Good get_shape for multidimensional input data.
        """
        good_input_reshaped = self.good_input.reshape(
            (1, len(self.good_input))
        )
        shape = self.class_to_test.get_shape(
            good_input_reshaped, self.locus,
            exp_rank=2,
        )
        self.assertIsInstance(shape, tuple)
        self.assertEqual(shape, (1, len(self.good_input)))

    def test_good_multidim(self):
        """
        Good multidimensional input data.
        """
        good_input_reshaped = self.good_input.reshape(
            (1, len(self.good_input))
        )
        self.class_to_test(
            good_input_reshaped, self.locus,
            exp_shape=good_input_reshaped.shape, mutual_sorder='C',
        )

    def test_good_multidim_nocheck(self):
        """
        Good multidimensional input data, checking skipped in one dimension.
        """
        good_input_reshaped = self.good_input.reshape(
            (1, len(self.good_input))
        )
        self.class_to_test(
            good_input_reshaped, self.locus,
            exp_shape=(1, None), mutual_sorder='C',
        )

    def test_good_multidim_nonecheck(self):
        """
        Good multidimensional input data, checking skipped in all dimensions.
        """
        good_input_reshaped = self.good_input.reshape(
            (1, len(self.good_input))
        )
        self.class_to_test(
            good_input_reshaped, self.locus,
            exp_shape=(None, None), mutual_sorder='C',
        )

    def test_good_null_nd(self):
        """
        Good null ndarray.
        """
        good_input = np.empty(
            (0, 1),
            dtype=self.class_to_test._underlying_scalar.numpy_type,
        )
        self.class_to_test(
            good_input, self.locus,
            exp_shape=(None, 0), mutual_sorder='C',
        )

    def test_bad_1dim_rank(self):
        """
        Bad one-dimensional input data, rank too small.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have shape.*$',
            self.class_to_test,
            self.good_input,
            self.locus,
            exp_shape=(len(self.good_input), 2)
        )

class ShapeTestArrayChar(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.

    Overrides test_bad_1dim.
    Overrides test_bad_1dim_rank.
    Overrides test_bad_multidim.
    Overrides test_bad_multidim_nocheck.
    Overrides test_bad_multidim_nonneg.
    Overrides test_bad_multidim_rank.
    Overrides test_bad_rank_1dim.
    Overrides test_bad_rank_multidim.
    Overrides test_good_1dim.
    Overrides test_good_1dim_nocheck_nd.
    Overrides test_good_1dim_nocheck.
    Overrides test_good_multidim.
    Overrides test_good_multidim_nocheck.
    Overrides test_good_multidim_nonecheck.
    Overrides test_good_null_nd.
    Overrides test_good_rank_1dim.
    Overrides test_good_rank_multidim.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineCharArrayType
        cls.good_input = np.array(['42', '43'])

    def test_bad_1dim(self):
        """
        Bad rank-1 input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method,no-member
            b_utils.NagShapeError,
            self.locus_re + '.*must have length.*$',
            self.class_to_test,
            self.good_input, self.locus,
            exp_chrlen=None, exp_shape=(len(self.good_input) - 1,)
        )

    def test_bad_multidim(self):
        """
        Bad multidimensional input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method,no-member
            b_utils.NagShapeError,
            self.locus_re + '.*must have shape.*$',
            self.class_to_test,
            self.good_input.reshape((1, len(self.good_input))),
            self.locus,
            exp_chrlen=2, exp_shape=(1, len(self.good_input) - 1)
        )

    def test_bad_multidim_nocheck(self):
        """
        Bad multidimensional input data, checking disabled in one dimension.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method,no-member
            b_utils.NagShapeError,
            self.locus_re + r'.*must have shape \(:, 1\)$',
            self.class_to_test,
            self.good_input.reshape((1, len(self.good_input))),
            self.locus,
            exp_chrlen=2, exp_shape=(None, len(self.good_input) - 1)
        )

    def test_bad_multidim_nonneg(self):
        """
        Bad multidimensional input data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re +
            '.*expected extent in dimension 2 .* must not be negative',
            self.class_to_test,
            self.good_input,
            self.locus,
            exp_chrlen=2, exp_shape=(1, -len(self.good_input)),
        )

    def test_bad_multidim_rank(self):
        """
        Bad multidimensional input data, rank too small.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method,no-member
            b_utils.NagShapeError,
            self.locus_re + '.*must have shape.*$',
            self.class_to_test,
            self.good_input.reshape((1, len(self.good_input))),
            self.locus,
            exp_chrlen=2, exp_shape=(1, len(self.good_input) - 1, 1)
        )

    def test_good_1dim(self):
        """
        Good rank-1 input data.
        """
        self.class_to_test(
            self.good_input, self.locus,
            exp_chrlen=2, exp_shape=(len(self.good_input),)
        )

    def test_good_1dim_nocheck_nd(self):
        """
        Good rank-1 input data, checking disabled.
        """
        self.class_to_test(
            np.array(self.good_input), self.locus,
            exp_chrlen=2, exp_shape=(None,)
        )

    def test_good_1dim_nocheck(self):
        """
        Good rank-1 input data, checking disabled.
        """
        self.class_to_test(
            list(self.good_input), self.locus,
            exp_chrlen=2, exp_shape=(None,)
        )

    def test_good_multidim(self):
        """
        Good multidimensional input data.
        """
        good_input_reshaped = self.good_input.reshape(
            (1, len(self.good_input))
        )
        self.class_to_test(
            good_input_reshaped, self.locus,
            exp_chrlen=2, exp_shape=good_input_reshaped.shape,
            mutual_sorder='C',
        )

    def test_good_multidim_nocheck(self):
        """
        Good multidimensional input data, checking disabled in one dimension.
        """
        good_input_reshaped = self.good_input.reshape(
            (1, len(self.good_input))
        )
        self.class_to_test(
            good_input_reshaped, self.locus,
            exp_chrlen=2, exp_shape=(1, None),
            mutual_sorder='C',
        )

    def test_good_multidim_nonecheck(self):
        """
        Good multidimensional input data, checking disabled in all dimensions.
        """
        good_input_reshaped = self.good_input.reshape(
            (1, len(self.good_input))
        )
        self.class_to_test(
            good_input_reshaped, self.locus,
            exp_chrlen=2, exp_shape=(None, None),
            mutual_sorder='C',
        )

    def test_good_null_nd(self):
        """
        Good null ndarray.
        """
        good_input = np.empty(
            (0, 1),
            dtype='S2',
        )
        self.class_to_test(
            good_input, self.locus,
            exp_chrlen=2, exp_shape=(None, 0), mutual_sorder='C',
        )

    def test_bad_1dim_rank(self):
        """
        Bad one-dimensional input data, rank too small.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have shape.*$',
            self.class_to_test,
            self.good_input,
            self.locus,
            exp_chrlen=2, exp_shape=(len(self.good_input), 2)
        )

class ShapeTestArrayComplex64(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineComplex64ArrayType
        cls.good_input = np.array([np.complex64(42), np.complex64(43)])

class ShapeTestArrayComplex128(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineComplex128ArrayType
        cls.good_input = np.array([np.complex128(42), np.complex128(43)])

class ShapeTestArrayCptr(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineCptrArrayType
        cls.good_input = [b_utils.Handle(42 + i) for i in range(10)]

    def test_good_1dim_nocheck_nd(self):
        pass

    def test_bad_get_multidim(self):
        pass

    def test_good_null_nd(self):
        pass

    def test_good_multidim(self):
        pass

    def test_good_multidim_nocheck(self):
        pass

    def test_good_multidim_nonecheck(self):
        pass

    def test_good_get_multidim(self):
        pass

    def test_bad_multidim_nonneg(self):
        pass

    def test_bad_multidim(self):
        pass

    def test_bad_multidim_nocheck(self):
        pass

    def test_bad_1dim_rank(self):
        pass

    def test_bad_multidim_rank(self):
        pass

    def test_bad_get_1dim(self):
        pass

class ShapeTestArrayFloat32(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineFloat32ArrayType
        cls.good_input = np.array([np.float32(42), np.float32(43)])

class ShapeTestArrayFloat64(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineFloat64ArrayType
        cls.good_input = np.array([np.float64(42), np.float64(43)])

class ShapeTestArrayInt(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineIntArrayType
        cls.good_input = np.array([42, 43])

class ShapeTestArrayLogical(TestBadGoodShape, InputTest):
    """
    Type-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        cls.class_to_test = b_utils._EngineLogicalArrayType
        cls.good_input = np.array([True, False])

class TypeTestArrayChar(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    import sys

    class_to_test = b_utils._EngineCharArrayType
    scalar_test_class = TypeTestScalarChar
    valid_container_names = 'list or numpy.ndarray'

    if sys.version_info < (3,):
        valid_np_dtype_names = 'numpy.string_ or numpy.unicode_'
        heterog_list = [np.string_('42.0'), np.unicode_('43.0')]
    else:
        valid_np_dtype_names = 'numpy.bytes_ or numpy.str_'
        heterog_list = [np.bytes_('42.0'), np.str_('43.0')]

    def test_bad_container(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, exp_chrlen=None, exp_shape=(1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(1, 1)
        )

    def test_bad_array(self):
        """
        Not an accepted container at all.
        """
        input_x = array.array('i', [42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(len(input_x),)
        )

    @unittest.skip(
        'array.array of chars not supported: ' +
        'is of single chars; i.e., like a string'
    )
    def test_good_array(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ctypes(self):
        """
        Not an accepted container at all.
        """

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_int

        input_x = CtypesTestArray(*[42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(len(input_x),)
        )

    @unittest.skip(
        'ctypes.Array for strings not supported: ' +
        'of single c_chars is like a string; ' +
        'no perceived use case for ctypes.Array of c_char_p\'s'
    )
    def test_good_ctypes(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_list(self):
        input_x = ['42.0', None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(len(input_x),)
        )

    def test_good_list(self):
        input_x = ['42.0', '43.0']
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_list_bytes(self):
        """
        Test correct operation for a list of byte strings.
        """
        input_x = [b'42.0', b'43.0']
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_list_unicode(self):
        """
        Test correct operation for a list of unicode strings.
        """
        input_x = [u'42.0', u'43.0']
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_list_numpy(self):
        """
        Test correct operation for a list of numpy strings.
        """
        input_x = [np.str_('42.0'), np.str_('43.0')]
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_list_numpy_bytes(self):
        """
        Test correct operation for a list of numpy byte strings.
        """
        input_x = [np.bytes_('42.0'), np.bytes_('43.0')]
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_list_numpy_unicode(self):
        """
        Test correct operation for a list of numpy unicode strings.
        """
        input_x = [np.unicode_('42.0'), np.unicode_('43.0')]
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_heterog_list(self):
        input_x = self.heterog_list
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.heterog_list_msg + '$',
            self.class_to_test,
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_bad_ndarray(self):
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*dtype.*' + self.valid_np_dtype_names + '$',
            self.class_to_test,
            input_x, self.locus, exp_chrlen=None,
            exp_shape=(len(input_x),)
        )

    def test_good_ndarray(self):
        input_x = np.array(['42.0', '43.0'])
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_ndarray_bytes(self):
        """
        Test correct operation for ndarray of byte strings.
        """
        input_x = np.array([b'42.0', b'43.0'])
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_ndarray_unicode(self):
        """
        Test correct operation for ndarray of unicode strings.
        """
        input_x = np.array([u'42.0', u'43.0'])
        self.class_to_test(
            input_x, self.locus, exp_chrlen=4,
            exp_shape=(len(input_x),)
        )

    def test_good_ndarray_multidim(self):
        input_x = np.array(['42.0', '43.0']).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus,
            exp_chrlen=4, exp_shape=input_x.shape, mutual_sorder='C',
        )

class TypeTestArrayComplex64(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineComplex64ArrayType
    scalar_test_class = TypeTestScalarComplex64
    valid_container_names = 'list or numpy.ndarray'
    valid_np_dtype_names = 'numpy.complex64'

    def test_bad_container(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, (1, 1)
        )

    def test_bad_array(self):
        """
        Not an accepted container at all.
        """
        input_x = array.array('i', [42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'array.array has no separate complex type'
    )
    def test_good_array(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ctypes(self):
        """
        Not an accepted container at all.
        """

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_int

        input_x = CtypesTestArray(*[42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'ctypes.Array has no separate complex type'
    )
    def test_good_ctypes(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_list(self):
        input_x = [np.complex64(42.0 + 43.0j), None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list(self):
        input_x = [np.complex64(42.0), np.complex64(42.0 + 43.0j)]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'no heterogeneity possible: underlying complex64 types are equivalent'
    )
    def test_heterog_list(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ndarray(self):
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*dtype.*' + self.valid_np_dtype_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray(self):
        input_x = np.array([np.complex64(42.0), np.complex64(42.0 + 43.0j)])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray_multidim(self):
        input_x = np.array(
            [np.complex64(42.0), np.complex64(42.0 + 43.0j)]
        ).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus, input_x.shape, 'C'
        )

class TypeTestArrayComplex128(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineComplex128ArrayType
    scalar_test_class = TypeTestScalarComplex128
    valid_container_names = 'list or numpy.ndarray'
    valid_np_dtype_names = 'numpy.complex128'

    def test_bad_container(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, (1, 1)
        )

    def test_bad_array(self):
        """
        Not an accepted container at all.
        """
        input_x = array.array('i', [42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'array.array has no separate complex type'
    )
    def test_good_array(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ctypes(self):
        """
        Not an accepted container at all.
        """

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_int

        input_x = CtypesTestArray(*[42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'ctypes.Array has no separate complex type'
    )
    def test_good_ctypes(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_list(self):
        input_x = [42.0 + 43.0j, None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list(self):
        input_x = [complex(42.0), np.complex128(42.0 + 43.0j)]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'no heterogeneity possible: underlying complex128 types are equivalent'
    )
    def test_heterog_list(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ndarray(self):
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*dtype.*' + self.valid_np_dtype_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray(self):
        input_x = np.array([complex(42.0), np.complex(42.0 + 43.0j)])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray_multidim(self):
        input_x = np.array(
            [np.complex128(42.0), np.complex128(42.0 + 43.0j)]
        ).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus, input_x.shape, 'C'
        )

class TypeTestArrayCptr(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    import sys

    class_to_test = b_utils._EngineCptrArrayType
    scalar_test_class = TypeTestScalarCptr
    valid_container_names = 'ctypes.Array or list'

    def test_bad_container(self):
        input_x = (b_utils.Handle(42), b_utils.Handle(43))
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, (1, 1)
        )

    def test_bad_array(self):
        input_x = array.array('d', [42.0, 43.0])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'array.array not accepted'
    )
    def test_good_array(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ctypes(self):

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_double

        input_x = CtypesTestArray(*[42.0, 43.0])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*_type_.*must be Handle$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ctypes(self):

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = b_utils.Handle

        input_x = CtypesTestArray(*[b_utils.Handle(42), b_utils.Handle(43)])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_list(self):
        input_x = [b_utils.Handle(42), None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list(self):
        input_x = [b_utils.Handle(42), b_utils.Handle(43)]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'no heterogeneity possible'
    )
    def test_heterog_list(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ndarray(self):
        input_x = np.array(['Hello', 'World'])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'ndarray not accepted'
    )
    def test_good_ndarray(self):
        self.fail('inconceivable') # pragma: no cover

    @unittest.skip(
        'ndarray not accepted'
    )
    def test_good_ndarray_multidim(self):
        self.fail('inconceivable') # pragma: no cover

class TypeTestArrayFloat32(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineFloat32ArrayType
    scalar_test_class = TypeTestScalarFloat32
    valid_container_names = 'array.array, ctypes.Array, list or numpy.ndarray'
    valid_np_dtype_names = 'numpy.float32'

    def test_bad_container(self):
        input_x = (42, 43)
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, (1, 1)
        )

    def test_bad_array(self):
        input_x = array.array('i', [42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*typecode.*"f"$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_array(self):
        input_x = array.array('f', [42.0, 43.0])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_ctypes(self):

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_int

        input_x = CtypesTestArray(*[42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*_type_.*ctypes.c_float$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ctypes(self):

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_float

        input_x = CtypesTestArray(*[42.0, 43.0])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_list(self):
        input_x = [np.float32(42.0), None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list(self):
        input_x = [ctypes.c_float(42.0), ctypes.c_float(43.0)]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_heterog_list(self):
        input_x = [ctypes.c_float(42.0), np.float32(43.0)]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.heterog_list_msg + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_ndarray(self):
        input_x = np.array(['Hello', 'World'])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*dtype.*' + self.valid_np_dtype_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray(self):
        input_x = np.array([42.0, 43.0], dtype=np.float32)
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray_multidim(self):
        input_x = np.array([42.0, 43.0], dtype=np.float32).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus, input_x.shape, 'C'
        )

class TypeTestArrayFloat64(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    class_to_test = b_utils._EngineFloat64ArrayType
    scalar_test_class = TypeTestScalarFloat64
    valid_container_names = 'array.array, ctypes.Array, list or numpy.ndarray'
    valid_np_dtype_names = 'numpy.float64'

    def test_bad_container(self):
        input_x = (42, 43)
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, (1, 1)
        )

    def test_bad_array(self):
        input_x = array.array('i', [42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*typecode.*"d"$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_array(self):
        input_x = array.array('d', [42.0, 43.0])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_ctypes(self):

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_int

        input_x = CtypesTestArray(*[42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*_type_.*ctypes.c_double$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ctypes(self):

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_double

        input_x = CtypesTestArray(*[42.0, 43.0])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_list(self):
        input_x = [42.0, None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list(self):
        input_x = [42.0, 43.0]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_heterog_list(self):
        input_x = [ctypes.c_double(42.0), np.float64(43.0)]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.heterog_list_msg + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_ndarray(self):
        input_x = np.array(['Hello', 'World'])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*dtype.*' + self.valid_np_dtype_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray(self):
        input_x = np.array([42.0, 43.0])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray_multidim(self):
        input_x = np.array([42.0, 43.0]).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus, input_x.shape, 'C'
        )

class TypeTestArrayInt(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    import sys

    class_to_test = b_utils._EngineIntArrayType
    scalar_test_class = TypeTestScalarInt
    valid_container_names = 'array.array, ctypes.Array, list or numpy.ndarray'

    if sys.version_info < (3,):
        valid_array_type_names = '"l"'
    else:
        valid_array_type_names = '"l" or "q"'

    valid_c_type_names = (
        scalar_test_class.ctypes_int32_name + ' or ' +
        scalar_test_class.ctypes_int64_name
    )
    valid_np_dtype_names = 'numpy.int32 or numpy.int64'

    def test_bad_container(self):
        input_x = (42, 43)
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, (1, 1)
        )

    def test_bad_array(self):
        input_x = array.array('d', [42.0, 43.0])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*typecode.*' + self.valid_array_type_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_array(self):
        input_x = array.array('l', [42, 43])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skipUnless(
        sys.version_info >= (3,),
        'no "q" array typecode'
    )
    def test_good_array_q(self):
        """
        Test correct operation for an array of "q".
        """
        input_x = array.array('q', [42, 43])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_ctypes(self):

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_double

        input_x = CtypesTestArray(*[42.0, 43.0])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*_type_.*' + self.valid_c_type_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ctypes(self):

        class CtypesTestArray32(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = b_utils._SYS_CTYPES_INTS[32]

        input_x = CtypesTestArray32(*[42, 43])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ctypes_64(self):
        """
        Test correct operation for an array of ctypes int64s.
        """

        class CtypesTestArray64(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = b_utils._SYS_CTYPES_INTS[64]

        input_x = CtypesTestArray64(*[42, 43])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_list(self):
        input_x = [np.int32(42), None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list(self):
        input_x = [42, 43]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list_ctypes_32(self):
        """
        Test correct operation for a list of ctypes int32s.
        """
        input_x = [
            b_utils._SYS_CTYPES_INTS[32](42),
            b_utils._SYS_CTYPES_INTS[32](43)
        ]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list_ctypes_64(self):
        """
        Test correct operation for a list of ctypes int64s.
        """
        input_x = [
            b_utils._SYS_CTYPES_INTS[64](42),
            b_utils._SYS_CTYPES_INTS[64](43)
        ]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_heterog_list(self):
        input_x = [b_utils._SYS_CTYPES_INTS[32](42), np.int32(43)]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.heterog_list_msg + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_ndarray(self):
        input_x = np.array(['Hello', 'World'])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*dtype.*' + self.valid_np_dtype_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray(self):
        input_x = np.array([42, 43], dtype=np.int32)
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray_64(self):
        """
        Test correct operation for an ndarray of int64s.
        """
        input_x = np.array([42, 43], dtype=np.int64)
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray_multidim(self):
        input_x = np.array([42, 43], dtype=np.int64).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus, input_x.shape, 'C'
        )

class TypeTestArrayLogical(TestBadGoodContainer, InputTest):
    """
    Type-check subsuite.
    """

    import sys

    class_to_test = b_utils._EngineLogicalArrayType
    scalar_test_class = TypeTestScalarLogical
    valid_container_names = 'list or numpy.ndarray'
    valid_np_dtype_names = 'numpy.bool_'

    def test_bad_container(self):
        input_x = (True, False)
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (1,)
        )

    def test_bad_container_multi(self):
        input_x = ('42', '43')
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.multidim_container_msg + '$',
            self.class_to_test,
            input_x, self.locus, (1, 1)
        )

    def test_bad_array(self):
        """
        Not an accepted container at all.
        """
        input_x = array.array('i', [42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'array.array for logical not supported: ' +
        'this would need to be some array of zero/nonzero bytes'
    )
    def test_good_array(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_ctypes(self):
        """
        Not an accepted container at all.
        """

        class CtypesTestArray(ctypes.Array):
            # pylint: disable=missing-docstring,too-few-public-methods
            _length_ = 2
            _type_ = ctypes.c_int

        input_x = CtypesTestArray(*[42, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.valid_container_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    @unittest.skip(
        'ctypes.Array for logical not supported: ' +
        'this would need to be some array of zero/nonzero bytes'
    )
    def test_good_ctypes(self):
        self.fail('inconceivable') # pragma: no cover

    def test_bad_list(self):
        input_x = [True, None]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            (
                self.locus_re + '.*element 1.*' +
                self.scalar_test_class.valid_names + '$'
            ),
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_list(self):
        input_x = [True, False]
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_heterog_list(self):
        input_x = [True, np.bool_(False)]
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*' + self.heterog_list_msg + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_bad_ndarray(self):
        input_x = np.array([43, 43])
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*dtype.*' + self.valid_np_dtype_names + '$',
            self.class_to_test,
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray(self):
        input_x = np.array([True, np.bool_(0)])
        self.class_to_test(
            input_x, self.locus, (len(input_x),)
        )

    def test_good_ndarray_multidim(self):
        input_x = np.array([True, np.bool_(0)]).reshape((1, 2))
        self.class_to_test(
            input_x, self.locus, input_x.shape, 'C'
        )

class DebugTest(InputTest):
    """
    Debug test fixtures.
    """

    def test_default_cm(self):
        """
        Correct default operation as contextmanager.
        """

        with b_utils._Debug():
            self.assertEqual(
                b_utils._EngineState._debug_level,
                b_utils._EngineState._debug_args_short,
            )

        self.assertEqual(
            b_utils._EngineState._debug_level,
            b_utils._EngineState._debug_off
        )

    def test_invalid_level(self):
        """
        Invalid level requested.
        """

        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagValueError,
            '.* argument level.*must be',
            b_utils._EngineState._set_debug_level, -42
        )

    def test_flag_cm(self):
        """
        Correct operation with different debug level, as contextmanager.
        """

        # Set a non-default initial debug level:
        initial_level = b_utils._EngineState._debug_args_short
        b_utils._EngineState._set_debug_level(initial_level)

        with b_utils._Debug(level=b_utils._EngineState._debug_naninf_counts):
            self.assertEqual(
                b_utils._EngineState._debug_level,
                b_utils._EngineState._debug_naninf_counts,
            )

        self.assertEqual(
            b_utils._EngineState._debug_level,
            initial_level,
        )

    def test_to_file_cm(self):
        """
        Correct operation to named file, as contextmanager.

        Call twice to verify that the state of the root logger is
        restored and reinitialized correctly.
        """
        import logging
        import os
        import tempfile

        tf_fd, tf_path = tempfile.mkstemp()

        for i in range(2):
            with b_utils._Debug(filename=tf_path):
                logging.debug('Hello{:d}'.format(i))

            with open(tf_path, 'r') as test_fo:
                test_str_lines = test_fo.readlines()

            if i == 0:
                self.assertEqual(
                    test_str_lines, ['DEBUG:root:Hello0\n']
                )
            elif i == 1:
                self.assertEqual(
                    test_str_lines,
                    [
                        'DEBUG:root:Hello0\n',
                        'DEBUG:root:Hello1\n',
                    ]
                )
            else:
                self.fail('inconceivable') # pragma: no cover

        os.close(tf_fd)
        os.unlink(tf_path)

    def test_already_handler(self):
        """
        Correct behaviour if a handler already registered.
        """
        import logging
        import os
        import tempfile
        tf_fd, tf_path = tempfile.mkstemp()
        logger = logging.getLogger()
        logger.setLevel(logging.DEBUG)
        my_fh = logging.FileHandler(tf_path)
        formatter = logging.Formatter(logging.BASIC_FORMAT)
        my_fh.setFormatter(formatter)
        logger.addHandler(my_fh)
        logging.debug('Hello0')
        with b_utils._Debug(filename=tf_path):
            logging.debug('Hello1')
        logging.debug('Hello2')
        with open(tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines,
            [
                'DEBUG:root:Hello0\n',
                'DEBUG:root:Hello1\n',
                'DEBUG:root:Hello2\n',
            ]
        )
        for handler in logging.getLogger().handlers:
            handler.close()
        logging.getLogger().handlers = []
        os.close(tf_fd)
        os.unlink(tf_path)

class OutputResetTest(InputTest):
    """
    Test the NAG Lib output resetter context manager.
    """

    def test_good(self):
        """
        Correct operation.
        """
        with b_utils._NagLibOutput():
            self.assertEqual(
                b_utils._EngineState._get_unit('err'),
                6,
            )
            self.assertEqual(
                b_utils._EngineState._get_unit('adv'),
                6,
            )

        self.assertEqual(
            b_utils._EngineState._get_unit('err'),
            b_utils.FileObjManager.errunit,
        )
        self.assertEqual(
            b_utils._EngineState._get_unit('adv'),
            b_utils.FileObjManager.advunit,
        )

class CommTest(InputTest):
    """
    Tests for comm blobs.
    """

    def setUp(self):
        InputTest.setUp(self)
        self.fun = b_utils._check_comm
        self.locus['entity'] = 'Z'

    def test_bad_instance(self):
        """
        Bad instance supplied.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'function ' + self.locus['fun_name'] +
            '.*argument Z.*must be an instance of dict$',
            self.fun,
            (), self.locus,
        )

    def test_bad_req_keys(self):
        """
        Bad compulsory keys in supplied dict.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagKeyError,
            'function ' + self.locus['fun_name'] +
            '.*argument Z.*key \'hello\' must be present.*' +
            ' initializer.*$',
            self.fun,
            {'goodbye': 42}, self.locus, req_keys=('hello',),
        )

    def test_bad_opt_keys(self):
        """
        Bad optional keys in supplied dict.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagKeyError,
            'function ' + self.locus['fun_name'] +
            '.*argument Z.*key \'hi\' must be present$',
            self.fun,
            {'hello': 43, 'goodbye': 42}, self.locus,
            opt_keys=('hello', 'hi'),
        )

    def test_bad_all_keys_empty(self):
        """
        No keys expected in empty dict.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagException,
            'function.*redundant',
            self.fun,
            {}, self.locus,
        )

    def test_bad_all_keys(self):
        """
        No keys expected in nonempty dict.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagException,
            'function.*redundant',
            self.fun,
            {'hello': 43, 'goodbye': 42}, self.locus,
        )

    def test_bad_empty(self):
        """
        Bad empty dict.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagKeyError,
            'function ' + self.locus['fun_name'] +
            '.*argument Z.*must not be empty.* initializer.*$',
            self.fun,
            {}, self.locus, req_keys=('hello',),
        )

    def test_good_instance_needs_init(self):
        """
        Valid instance supplied.
        """
        init_comm = self.fun(
            {}, self.locus, opt_keys=('hello',),
        )
        self.assertTrue(init_comm)

    def test_good_req_keys(self):
        """
        Good keys in supplied dict.
        """
        init_comm = self.fun(
            {'hello': 1, 'goodbye': 2}, self.locus,
            req_keys=('hello',),
        )
        self.assertFalse(init_comm)

    def test_good_opt_keys_needs_init(self):
        """
        Good keys in supplied dict.
        """
        init_comm = self.fun(
            {'hello': 1, 'goodbye': 2}, self.locus,
            req_keys=('hello',), opt_keys=('hi',),
        )
        self.assertTrue(init_comm)

    def test_good_opt_keys_no_init(self):
        """
        Good keys in supplied dict.
        """
        init_comm = self.fun(
            {'hello': 1, 'goodbye': 2}, self.locus,
            req_keys=('hello',), opt_keys=('goodbye',),
        )
        self.assertFalse(init_comm)

    def test_bad_none(self):
        """
        None not permitted.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'function ' + self.locus['fun_name'] +
            '.*argument Z.*must be an instance of dict$',
            self.fun,
            None, self.locus,
        )

    def test_good_none(self):
        """
        None permirred.
        """
        init_comm = self.fun(
            None, self.locus, allow_none=True,
        )
        self.assertTrue(init_comm)

class EngineDataTest(unittest.TestCase):
    """
    Miscellaneous tests of EngineData methods.
    """
    class_to_test = b_utils._EngineData

    def setUp(self):
        import sys

        if sys.version_info < (3,):
            self.my_assert_raises_regex = self.assertRaisesRegexp
        else:
            self.my_assert_raises_regex = self.assertRaisesRegex # pylint: disable=no-member

    def test_print_rec_default(self):
        """
        Storage of no print callback.
        """
        en_data = self.class_to_test()
        self.assertIsNone(en_data.y90_print_rec)

    def test_print_rec_supplied(self):
        """
        Storage of no print callback.
        """

        def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail):
            # pylint: disable=missing-docstring
            pass # pragma: no cover

        b_utils._EngineState._set_debug_level(
            b_utils._EngineState._debug_args_short
        )
        en_data = self.class_to_test(py_c_print_rec=cb_py_c_print_rec)
        self.assertIsNotNone(en_data.y90_print_rec)
        b_utils._EngineState._set_debug_level(b_utils._EngineState._debug_off)

    def test_sorder_default(self):
        """
        Default storage order.
        """
        en_data = self.class_to_test()
        self.assertEqual(
            en_data.storage_order,
            b_utils._NAG_ED_ROW_MAJOR.value
        )

    def test_sorder(self):
        """
        Set of storage order.
        """
        en_data = self.class_to_test(sorder='F')
        self.assertEqual(
            en_data.storage_order,
            b_utils._NAG_ED_COL_MAJOR.value
        )

class _EngineStateTest(unittest.TestCase):
    """
    Miscellaneous tests of _EngineState state.
    """

    def setUp(self):
        import sys

        if sys.version_info < (3,):
            self.my_assert_raises_regex = self.assertRaisesRegexp
        else:
            self.my_assert_raises_regex = self.assertRaisesRegex # pylint: disable=no-member

    def test_no_instance(self):
        """
        Instantiation prohibited.
        """
        try:
            b_utils._EngineState()
        except b_utils.NagTypeError as exc:
            self.assertIn(
                'instances',
                str(exc)
            )
            self.assertIn(
                'not permitted',
                str(exc)
            )
        else:
            self.fail('NagTypeError not raised') # pragma: no cover

class FileObjManagerTest(unittest.TestCase):
    """
    FileObjManager test fixtures.
    """

    # pylint: disable=too-many-public-methods

    import platform
    import sys

    if sys.version_info < (3, ):
        allowed_insts = 'file or io.IOBase'
    else:
        allowed_insts = 'io.IOBase'

    @classmethod
    def setUpClass(cls):
        """
        Make some file objects.
        """
        import tempfile
        cls.tf_fd, cls.tf_path = tempfile.mkstemp()
        cls.tf_fo = open(cls.tf_path, 'r')

    @classmethod
    def tearDownClass(cls):
        """
        Destroy the file objects.
        """
        import os
        cls.tf_fo.close()
        os.close(cls.tf_fd)
        os.unlink(cls.tf_path)

    def setUp(self):
        import sys

        if sys.version_info < (3,):
            self.my_assert_raises_regex = self.assertRaisesRegexp
        else:
            self.my_assert_raises_regex = self.assertRaisesRegex # pylint: disable=no-member

    def test_none_fileobjs(self): # pylint: disable=no-self-use
        """
        __init__ can be supplied no arguments.
        """
        b_utils.FileObjManager()

    def test_adv_unit(self):
        """
        Class has set adv unit correctly.
        """
        fom = b_utils.FileObjManager()
        self.assertEqual(
            b_utils._EngineState._get_unit('adv'), fom.advunit,
        )

    def test_default_adv_mapping(self):
        """
        stdout.
        """
        import sys

        fom = b_utils.FileObjManager()
        self.assertEqual(
            fom._mappings[fom.advunit], sys.stdout,
        )

    def test_default_err_mapping(self):
        """
        stderr.
        """
        import sys

        fom = b_utils.FileObjManager()
        self.assertEqual(
            fom._mappings[fom.errunit], sys.stderr,
        )

    def test_err_unit(self):
        """
        Class has set err unit correctly.
        """
        fom = b_utils.FileObjManager()
        self.assertEqual(
            b_utils._EngineState._get_unit('err'), fom.errunit,
        )

    def test_register_to_unit_bad(self):
        """
        Bad instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*_register_to_unit, argument fileobj.*must be '
            'an instance of ' +
            self.allowed_insts,
            fom._register_to_unit,
            'Hello', 53,
            {'fun_name': '_register_to_unit', 'entity': 'fileobj'},
        )

    def test_register_to_unit_bad_fd(self):
        """
        Indescribable object supplied.
        """
        import io
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*_register_to_unit, argument fileobj.*must '
            'implement the fileno method',
            fom._register_to_unit,
            io.IOBase(), 53,
            {'fun_name': '_register_to_unit', 'entity': 'fileobj'},
        )

    def test_register_to_unit_good(self):
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager()
        fom._register_to_unit(
            self.tf_fo,
            53,
            {'fun_name': '_register_to_unit', 'entity': 'fileobj'}
        )
        self.assertIn(
            53, fom._mappings
        )
        self.assertEqual(
            fom._mappings[53], self.tf_fo
        )

    def test_register_to_advunit_bad(self):
        """
        Bad instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*register_to_advunit, argument fileobj.*must be '
            'an instance of ' +
            self.allowed_insts,
            fom.register_to_advunit,
            'Hello',
        )

    def test_register_to_advunit_bad_fd(self):
        """
        Indescribable object supplied.
        """
        import io
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*register_to_advunit, argument fileobj.*must '
            'implement the fileno method',
            fom.register_to_advunit,
            io.IOBase()
        )

    def test_register_to_advunit_good(self):
        """
        Correct operation.
        """
        import sys
        fom = b_utils.FileObjManager()
        fom.register_to_advunit(self.tf_fo)
        self.assertIn(
            fom.advunit, fom._mappings
        )
        self.assertEqual(
            fom._mappings[fom.advunit], self.tf_fo
        )
        fom.deregister_advunit()
        self.assertEqual(
            fom._mappings[fom.advunit], sys.stdout,
        )

    def test_register_to_errunit_bad(self):
        """
        Bad instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*register_to_errunit, argument fileobj.*must be '
            'an instance of ' +
            self.allowed_insts,
            fom.register_to_errunit,
            'Hello',
        )

    def test_register_to_errunit_bad_fd(self):
        """
        Indescribable object supplied.
        """
        import io
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*register_to_errunit, argument fileobj.*must '
            'implement the fileno method',
            fom.register_to_errunit,
            io.IOBase()
        )

    def test_register_to_errunit_good(self):
        """
        Correct operation.
        """
        import sys
        fom = b_utils.FileObjManager()
        fom.register_to_errunit(self.tf_fo)
        self.assertIn(
            fom.errunit, fom._mappings
        )
        self.assertEqual(
            fom._mappings[fom.errunit], self.tf_fo
        )
        fom.deregister_errunit()
        self.assertEqual(
            fom._mappings[fom.errunit], sys.stderr,
        )

    def test_register_bad_inst(self):
        """
        Bad non-tuple instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'function .*register, argument fileobjs.*must be an '
            'instance of tuple',
            fom.register,
            'Hello',
        )

    def test_register_bad_fd(self):
        """
        Indescribable object supplied.
        """
        import io
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*register, argument fileobj.*must implement '
            'the fileno method',
            fom.register,
            (io.IOBase(),),
        )

    def test_register_bad_fo(self):
        """
        Bad instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*register, argument fileobj.*must be an '
            'instance of ' + self.allowed_insts,
            fom.register,
            ('Hello',),
        )

    def test_register(self):
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager()
        fom.register((self.tf_fo,))
        self.assertIn(
            self.tf_fo, fom._mappings.values()
        )
        fom.deregister((self.tf_fo,))

    def test_unit_large_check(self):
        """
        Unit number out of range, checking enabled.
        """
        import os
        import tempfile
        fom = b_utils.FileObjManager()
        fos = []
        for i in range(100):
            fos.append(tempfile.mkstemp())
        tf_fo = open(fos[-1][1], 'r')
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagException,
            r'(?m).*unit number will be out of range.*\n\*\*' +
            ' Please contact NAG.$',
            fom.register, (tf_fo,),
        )
        tf_fo.close()
        for i in range(100):
            os.close(fos[i][0])
            os.unlink(fos[i][1])

    def test_unit_large_nocheck(self): # pylint: disable=no-self-use
        """
        Unit number out of range, checking suppressed.
        """
        b_utils.FileObjManager._fileno_to_unit(100, check_range=False)

    def test_init_bad_inst(self):
        """
        Bad non-tuple instance supplied.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'function .*__init__, argument fileobjs.*must be an '
            'instance of tuple',
            b_utils.FileObjManager,
            'Hello',
        )

    def test_init_bad_fd(self):
        """
        Indescribable object supplied.
        """
        import io
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*__init__, argument fileobj.*must implement '
            'the fileno method',
            b_utils.FileObjManager,
            (io.IOBase(),),
        )

    def test_init_bad_fo(self):
        """
        Bad instance supplied.
        """
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*__init__, argument fileobj.*must be an '
            'instance of ' + self.allowed_insts,
            b_utils.FileObjManager,
            ('Hello',),
        )

    def test_init(self):
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager((self.tf_fo,))
        self.assertIn(
            self.tf_fo, fom._mappings.values()
        )
        fom.deregister((self.tf_fo,))

    @unittest.skipIf(
        platform.system() == 'Windows',
        'tempfile.TemporaryFile returns a wrapped file object'
    )
    def test_init_tempfile_posix(self):
        """
        Correct operation.
        """
        import tempfile

        with tempfile.TemporaryFile() as tf_fo:
            fom = b_utils.FileObjManager((tf_fo,))
            self.assertIn(
                tf_fo, fom._mappings.values()
            )
            fom.deregister((tf_fo,))

    @unittest.skipIf(
        platform.system() != 'Windows',
        'tempfile.TemporaryFile returns a file object'
    )
    def test_init_tempfile(self):
        """
        Correct operation.
        """
        import tempfile

        with tempfile.TemporaryFile() as tf_fo:
            fom = b_utils.FileObjManager((tf_fo.file,))
            self.assertIn(
                tf_fo.file, fom._mappings.values()
            )
            fom.deregister((tf_fo.file,))

    def test_init_namedtempfile(self):
        """
        Correct operation.
        """
        import tempfile

        with tempfile.NamedTemporaryFile() as tf_fo:
            fom = b_utils.FileObjManager((tf_fo.file,))
            self.assertIn(
                tf_fo.file, fom._mappings.values()
            )
            fom.deregister((tf_fo.file,))

    def test_unit_from_bad_fd(self):
        """
        Indescribable object supplied.
        """
        import io
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*unit_from_fileobj, argument fileobj.*must '
            'implement the fileno method',
            fom.unit_from_fileobj,
            io.IOBase(),
        )

    def test_unit_from_bad_fo(self):
        """
        Bad instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*unit_from_fileobj, argument fileobj.*must be an '
            'instance of ' +
            self.allowed_insts,
            fom.unit_from_fileobj,
            ('Hello',),
        )

    def test_unit_from_reg(self):
        """
        fileobj gets registered.
        """
        fom = b_utils.FileObjManager()
        unit = fom._fileobj_to_unit(self.tf_fo, {'fun_name': 'z99zz'})
        self.assertNotIn(
            unit, fom._mappings
        )
        unit2 = fom.unit_from_fileobj(self.tf_fo)
        self.assertIn(
            unit, fom._mappings
        )
        self.assertEqual(
            unit, unit2,
        )

    def test_deregister_bad_inst(self):
        """
        Bad non-tuple instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'function .*deregister, argument fileobjs.*must be an '
            'instance of tuple',
            fom.deregister,
            'Hello',
        )

    def test_deregister_bad_fd(self):
        """
        Indescribable object supplied.
        """
        import io
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*deregister, argument fileobj.*must implement '
            'the fileno method',
            fom.deregister,
            (io.IOBase(),),
        )

    def test_deregister_bad_fo(self):
        """
        Bad instance supplied.
        """
        fom = b_utils.FileObjManager()
        self.my_assert_raises_regex( #pylint: disable=deprecated-method
            b_utils.NagOSError,
            'function .*deregister, argument fileobj.*must be an '
            'instance of ' + self.allowed_insts,
            fom.deregister,
            ('Hello',),
        )

    def test_deregister(self):
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager()
        fom.register((self.tf_fo,))
        fom.deregister((self.tf_fo,))
        self.assertNotIn(
            self.tf_fo, fom._mappings.values()
        )

    def test_deregister_unregistered(self):
        """
        Correct operation for unregistered unit.
        """
        fom = b_utils.FileObjManager()
        fom.deregister((self.tf_fo,))
        self.assertNotIn(
            self.tf_fo, fom._mappings.values()
        )

    def test_type_compat(self): # pylint: disable=no-self-use
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager()
        b_utils.FileObjManager._check_type_compat(
            fom, {'fun_name': 'z99zz', 'entity': 'fom'}
        )

    def test_type_compat_none(self): # pylint: disable=no-self-use
        """
        Correct operation.
        """
        b_utils.FileObjManager._check_type_compat(
            None, {'fun_name': 'z99zz', 'entity': 'fom'}
        )

    def test_type_incompat(self):
        """
        Bad instance.
        """
        np_x = np.empty(1)
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            'function .*z99zz, argument np_x.*must be an '
            'instance of.*FileObjManager',
            b_utils.FileObjManager._check_type_compat,
            np_x, {'fun_name': 'z99zz', 'entity': 'np_x'}
        )

class PrintRecTest(unittest.TestCase):
    """
    Test fixtures for the Python-supplied record writer.
    """

    @classmethod
    def setUpClass(cls):
        """
        Make some file objects.
        """
        import tempfile
        cls.tf_fd, cls.tf_path = tempfile.mkstemp()

    @classmethod
    def tearDownClass(cls):
        """
        Destroy the file objects.
        """
        import os
        os.close(cls.tf_fd)
        os.unlink(cls.tf_path)

    def setUp(self):
        self.tf_fo = open(self.tf_path, 'w')
        self.en_data = b_utils._EngineData()

    def tearDown(self):
        self.tf_fo.close()

    def test_logging(self):
        """
        Correct operation through the logging module.
        """
        import logging
        self.tf_fo.close()
        logger = logging.getLogger('test.logging')
        logger.setLevel(logging.DEBUG)
        my_fh = logging.FileHandler(self.tf_path)
        formatter = logging.Formatter(logging.BASIC_FORMAT)
        my_fh.setFormatter(formatter)
        logger.addHandler(my_fh)
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            -1,
            b'Hello',
            None,
            excs,
            ifail,
            logger=logging.getLogger('test.logging')
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['DEBUG:test.logging:Hello\n']
        )
        for handler in logging.getLogger('test.logging').handlers:
            handler.close()
        logging.getLogger().handlers = []

    def test_global_manager_unregged(self):
        """
        Correct operation through the global manager.
        """
        nout = 53
        self.assertNotIn(
            nout, b_utils.GLOBAL_IO_MANAGER._mappings
        )
        b_utils.GLOBAL_IO_MANAGER.register_to_advunit(self.tf_fo)
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data, nout, b'Hello', None, excs, ifail,
            fun_name='test.global_manager_unregged',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['test.global_manager_unregged: Hello\n']
        )

    def test_global_manager_regged(self):
        """
        Correct operation through the global manager.
        """
        b_utils.GLOBAL_IO_MANAGER.register((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            b_utils.GLOBAL_IO_MANAGER.unit_from_fileobj(self.tf_fo),
            b'Hello',
            None,
            excs,
            ifail,
            fun_name='test.global_manager_regged',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['test.global_manager_regged: Hello\n']
        )

    def test_local_manager_unregged(self):
        """
        Correct operation through a local manager.
        """
        fom = b_utils.FileObjManager()
        fom.register_to_advunit(self.tf_fo)
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data, 53, b'Hello', fom, excs, ifail,
            fun_name='test.local_manager_unregged',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['test.local_manager_unregged: Hello\n']
        )

    def test_local_manager_regged(self):
        """
        Correct operation through a local manager.
        """
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello',
            fom,
            excs,
            ifail,
            fun_name='test.local_manager_regged',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['test.local_manager_regged: Hello\n']
        )

    def test_stripped(self):
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello   ',
            fom,
            excs,
            ifail,
            fun_name='test.stripped',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['test.stripped: Hello\n']
        )

    def test_bad(self):
        """
        Write failure.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'r')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello   ',
            fom,
            excs,
            ifail,
            fun_name='test.bad',
        )
        self.assertEqual(len(excs), 1)
        self.assertEqual(
            ifail.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, fom._print_err
        )

    def test_good_bufferedbase(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'wb')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello',
            fom,
            excs,
            ifail,
            fun_name='test.good_bufferedbase',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'rb') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, [b'test.good_bufferedbase: Hello\n']
        )

    def test_good_rawbase(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'wb', buffering=0)
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello',
            fom,
            excs,
            ifail,
            fun_name='test.good_rawbase',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'rb', buffering=0) as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, [b'test.good_rawbase: Hello\n']
        )

    def test_good_mixed(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'wb', buffering=0)
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello',
            fom,
            excs,
            ifail,
            fun_name='test.good_mixed',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['test.good_mixed: Hello\n']
        )

    def test_good_no_locus(self):
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager(locus_in_output=False)
        fom.register_to_advunit(self.tf_fo)
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data, 53, b'Hello', fom, excs, ifail,
            fun_name='test.local_manager_unregged',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['Hello\n']
        )

    def test_good_bufferedbase_no_locus(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'wb')
        fom = b_utils.FileObjManager((self.tf_fo,), locus_in_output=False)
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello',
            fom,
            excs,
            ifail,
            fun_name='test.good_bufferedbase',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'rb') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, [b'Hello\n']
        )

    def test_good_rawbase_no_locus(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'wb', buffering=0)
        fom = b_utils.FileObjManager((self.tf_fo,), locus_in_output=False)
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello',
            fom,
            excs,
            ifail,
            fun_name='test.good_rawbase',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'rb', buffering=0) as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, [b'Hello\n']
        )

    def test_good_mixed_no_locus(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'wb', buffering=0)
        fom = b_utils.FileObjManager((self.tf_fo,), locus_in_output=False)
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_print_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            b'Hello',
            fom,
            excs,
            ifail,
            fun_name='test.good_mixed',
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.tf_fo.close()
        with open(self.tf_path, 'r') as test_fo:
            test_str_lines = test_fo.readlines()
        self.assertEqual(
            test_str_lines, ['Hello\n']
        )

class ReadRecTest(unittest.TestCase):
    """
    Test fixtures for the Python-supplied record reader.
    """
    import sys

    @classmethod
    def setUpClass(cls):
        """
        Make some file objects.
        """
        import tempfile
        cls.tf_fd, cls.tf_path = tempfile.mkstemp()

    @classmethod
    def tearDownClass(cls):
        """
        Destroy the file objects.
        """
        import os
        os.close(cls.tf_fd)
        os.unlink(cls.tf_path)

    def setUp(self):
        import sys

        write_kwargs = {'mode': 'w'}
        if sys.version_info >= (3,):
            write_kwargs['newline'] = '\n'
        with open(self.tf_path, **write_kwargs) as file_fo:
            file_fo.write('Hello\r\n\r')
        self.tf_fo = open(self.tf_path, 'r')
        self.en_data = b_utils._EngineData()
        self.rec_len = 80
        self.rec = ctypes.create_string_buffer(self.rec_len)
        self.rec = ctypes.cast(self.rec, ctypes.POINTER(ctypes.c_char))
        self.errbuf = b_utils._EngineErrbuf()

        if sys.version_info < (3,):
            self.my_assert_regex = self.assertRegexpMatches
        else:
            self.my_assert_regex = self.assertRegex # pylint: disable=no-member

    def tearDown(self):
        self.tf_fo.close()

    def test_bad_global_only_unregged(self):
        """
        Unregistered unit through the global manager, no local manager.
        """
        nin = 53
        self.assertNotIn(
            nin, b_utils.GLOBAL_IO_MANAGER._mappings
        )
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            nin,
            self.rec, self.errbuf, self.rec_len,
            None,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagOSError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_read_rec.*no file object registered'
        )
        self.assertEqual(
            ifail.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, b_utils.GLOBAL_IO_MANAGER._read_err_unassoc
        )

    def test_global_only_regged(self):
        """
        Correct operation through the global manager, no local manager.
        """
        b_utils.GLOBAL_IO_MANAGER.register((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            b_utils.GLOBAL_IO_MANAGER.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            None,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            self.rec[:self.rec_len], b'Hello' + b' ' * (self.rec_len - 5)
        )

    def test_bad_local_unregged(self):
        """
        Unregistered unit.
        """
        nin = 53
        self.assertNotIn(
            nin, b_utils.GLOBAL_IO_MANAGER._mappings
        )
        fom = b_utils.FileObjManager()
        self.assertNotIn(
            nin, fom._mappings
        )
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            nin,
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagOSError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_read_rec.*no file object registered'
        )
        self.assertEqual(
            ifail.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err_unassoc
        )

    def test_good_local_unregged(self):
        """
        Correct operation, unit not registered locally but globally.
        """
        b_utils.GLOBAL_IO_MANAGER.register((self.tf_fo,))
        fom = b_utils.FileObjManager()
        self.assertNotIn(
            b_utils.GLOBAL_IO_MANAGER.unit_from_fileobj(self.tf_fo),
            fom._mappings
        )
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            b_utils.GLOBAL_IO_MANAGER.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            self.rec[:self.rec_len], b'Hello' + b' ' * (self.rec_len - 5)
        )

    def test_good_local_regged(self):
        """
        Correct operation, unit registered locally.
        """
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            self.rec[:self.rec_len], b'Hello' + b' ' * (self.rec_len - 5)
        )

    def test_bad(self):
        """
        Read failure.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'w')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 1)
        self.assertEqual(
            ifail.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err
        )

    def test_eof(self):
        """
        EOF encountered.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'w')
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'r')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )

    def test_good_buffered(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'rb')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            self.rec[:self.rec_len], b'Hello' + b' ' * (self.rec_len - 5)
        )

    def test_good_raw(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'rb', buffering=0)
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            ifail.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            self.rec[:self.rec_len], b'Hello' + b' ' * (self.rec_len - 5)
        )

    def test_bad_unicode_binary(self):
        """
        Unicode content opened for reading in binary mode.
        """
        import sys

        self.tf_fo.close()
        with open(self.tf_path, 'w') as file_fo:
            if sys.version_info < (3,):
                file_fo.write('Pi\xc3\xb1ata')
            else:
                file_fo.write(u'Pi\xf1ata')
        self.tf_fo = open(self.tf_path, 'rb')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagValueError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_read_rec.*must comprise only ASCII characters'
        )
        self.assertEqual(
            ifail.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err_nonascii
        )

    def test_bad_unicode_text(self):
        """
        Unicode content opened for reading in text mode.
        """
        import sys

        self.tf_fo.close()
        with open(self.tf_path, 'w') as file_fo:
            if sys.version_info < (3,):
                file_fo.write('Pi\xc3\xb1ata')
            else:
                file_fo.write(u'Pi\xf1ata')
        self.tf_fo = open(self.tf_path, 'r')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        ifail = ctypes.c_int()
        b_utils._py_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.errbuf, self.rec_len,
            fom,
            excs,
            ifail,
        )
        self.assertEqual(self.errbuf[0], b'\0')
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagValueError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_read_rec.*must comprise only ASCII characters'
        )
        self.assertEqual(
            ifail.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err_nonascii
        )

class NonadvReadRecTest(unittest.TestCase):
    """
    Test fixtures for the Python-supplied nonadvancing record reader.
    """
    import sys

    @classmethod
    def setUpClass(cls):
        """
        Make some file objects.
        """
        import tempfile
        cls.tf_fd, cls.tf_path = tempfile.mkstemp()

    @classmethod
    def tearDownClass(cls):
        """
        Destroy the file objects.
        """
        import os
        os.close(cls.tf_fd)
        os.unlink(cls.tf_path)

    def setUp(self):
        import sys

        write_kwargs = {'mode': 'w'}
        if sys.version_info >= (3,):
            write_kwargs['newline'] = '\n'
        with open(self.tf_path, **write_kwargs) as file_fo:
            file_fo.write('Hello\r\nWorld\n')
        self.tf_fo = open(self.tf_path, 'r')
        self.en_data = b_utils._EngineData()
        self.rec_len = 5
        self.rec = ctypes.create_string_buffer(self.rec_len)
        self.rec = ctypes.cast(self.rec, ctypes.POINTER(ctypes.c_char))

        if sys.version_info < (3,):
            self.my_assert_regex = self.assertRegexpMatches
        else:
            self.my_assert_regex = self.assertRegex # pylint: disable=no-member

    def tearDown(self):
        self.tf_fo.close()

    def test_bad_global_only_unregged(self):
        """
        Unregistered unit through the global manager, no local manager.
        """
        nin = 53
        self.assertNotIn(
            nin, b_utils.GLOBAL_IO_MANAGER._mappings
        )
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            nin,
            self.rec, self.rec_len,
            None,
            excs, inform,
        )
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagOSError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_nonadv_read_rec.*no file object registered'
        )
        self.assertEqual(
            readlen, 0
        )
        self.assertEqual(
            inform.value, 3
        )
        self.assertEqual(
            self.en_data.hlperr, b_utils.GLOBAL_IO_MANAGER._read_err_unassoc
        )

    def test_global_only_regged(self):
        """
        Correct operation through the global manager, no local manager.
        """
        b_utils.GLOBAL_IO_MANAGER.register((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            b_utils.GLOBAL_IO_MANAGER.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            None,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            readlen, self.rec_len
        )
        self.assertEqual(
            inform.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(self.rec[:self.rec_len], b'Hello')

    def test_bad_local_unregged(self):
        """
        Unregistered unit.
        """
        nin = 53
        self.assertNotIn(
            nin, b_utils.GLOBAL_IO_MANAGER._mappings
        )
        fom = b_utils.FileObjManager()
        self.assertNotIn(
            nin, fom._mappings
        )
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            nin,
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagOSError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_nonadv_read_rec.*no file object registered'
        )
        self.assertEqual(
            readlen, 0
        )
        self.assertEqual(
            inform.value, 3
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err_unassoc
        )

    def test_good_local_unregged(self):
        """
        Correct operation, unit not registered locally but globally.
        """
        b_utils.GLOBAL_IO_MANAGER.register((self.tf_fo,))
        fom = b_utils.FileObjManager()
        self.assertNotIn(
            b_utils.GLOBAL_IO_MANAGER.unit_from_fileobj(self.tf_fo),
            fom._mappings
        )
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            b_utils.GLOBAL_IO_MANAGER.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            readlen, self.rec_len
        )
        self.assertEqual(
            inform.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(self.rec[:self.rec_len], b'Hello')

    def test_good_local_regged(self):
        """
        Correct operation, unit registered locally.
        """
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            readlen, self.rec_len
        )
        self.assertEqual(
            inform.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(self.rec[:self.rec_len], b'Hello')

    def test_bad(self):
        """
        Read failure.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'w')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 1)
        self.assertEqual(
            readlen, 0
        )
        self.assertEqual(
            inform.value, 3
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err
        )

    def test_eof(self):
        """
        EOF encountered.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'w')
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'r')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            readlen, 0
        )
        self.assertEqual(
            inform.value, 2
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )

    def test_eor(self):
        """
        Reading past end-of-record encountered.
        """
        self.rec_len = 6
        self.rec = ctypes.create_string_buffer(self.rec_len)
        self.rec = ctypes.cast(self.rec, ctypes.POINTER(ctypes.c_char))
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            readlen, 5
        )
        self.assertEqual(
            inform.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(self.rec[:self.rec_len], b'Hello ')

    def test_good_multi(self):
        """
        Correct operation.
        """
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            inform.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            readlen, self.rec_len
        )
        self.assertEqual(self.rec[:self.rec_len], b'Hello')
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            inform.value, 1
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            readlen, 0
        )
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            inform.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(
            readlen, self.rec_len
        )
        self.assertEqual(self.rec[:self.rec_len], b'World')

    def test_good_buffered(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'rb')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            readlen, self.rec_len
        )
        self.assertEqual(
            inform.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(self.rec[:self.rec_len], b'Hello')

    def test_good_raw(self):
        """
        Correct operation.
        """
        self.tf_fo.close()
        self.tf_fo = open(self.tf_path, 'rb', buffering=0)
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 0)
        self.assertEqual(
            readlen, self.rec_len
        )
        self.assertEqual(
            inform.value, 0
        )
        self.assertEqual(
            self.en_data.hlperr, 0
        )
        self.assertEqual(self.rec[:self.rec_len], b'Hello')

    def test_bad_unicode_binary(self):
        """
        Unicode content opened for reading in binary mode.
        """
        import sys

        self.tf_fo.close()
        with open(self.tf_path, 'w') as file_fo:
            if sys.version_info < (3,):
                file_fo.write('Pi\xc3\xb1ata')
            else:
                file_fo.write(u'Pi\xf1ata')
        self.tf_fo = open(self.tf_path, 'rb')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagValueError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_nonadv_read_rec.*must comprise only '
            'ASCII characters'
        )
        self.assertEqual(
            readlen, 0
        )
        self.assertEqual(
            inform.value, 3
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err_nonascii
        )

    def test_bad_unicode_text(self):
        """
        Unicode content opened for reading in text mode.
        """
        import sys

        self.tf_fo.close()
        with open(self.tf_path, 'w') as file_fo:
            if sys.version_info < (3,):
                file_fo.write('Pi\xc3\xb1ata')
            else:
                file_fo.write(u'Pi\xf1ata')
        self.tf_fo = open(self.tf_path, 'r')
        fom = b_utils.FileObjManager((self.tf_fo,))
        excs = []
        inform = ctypes.c_int()
        readlen = b_utils._py_nonadv_read_rec(
            self.en_data,
            fom.unit_from_fileobj(self.tf_fo),
            self.rec, self.rec_len,
            fom,
            excs, inform,
        )
        self.assertEqual(len(excs), 1)
        self.assertIsInstance(excs[0][1], b_utils.NagValueError)
        self.my_assert_regex( #pylint: disable=deprecated-method
            str(excs[0][1]),
            'function .*_py_nonadv_read_rec.*must comprise only '
            'ASCII characters'
        )
        self.assertEqual(
            readlen, 0
        )
        self.assertEqual(
            inform.value, 3
        )
        self.assertEqual(
            self.en_data.hlperr, fom._read_err_nonascii
        )

class NdarrayCastTest(InputTest): # pylint: disable=too-many-public-methods
    """
    Test ndarray type-check and copy or cast.
    """

    def setUp(self):
        InputTest.setUp(self)
        self.locus = {'fun_name': 'z99zz', 'entity': 'x'}

    def test_good_check(self):
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43])
        np_utils._check_is_ndarray(self.locus, input_x)

    def test_good_check_none(self):
        """
        Input is ndarray.
        """
        input_x = None
        np_utils._check_is_ndarray(self.locus, input_x, allow_none=True)

    def test_good_check_rank(self):
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43])
        np_utils._check_valid_ndarray(self.locus, input_x, exp_rank=1)

    def test_good_check_shape(self):
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43, 44, 45]).reshape((2, 2))
        np_utils._check_valid_ndarray(self.locus, input_x, exp_shape=(None, 2))

    def test_good_check_class(self):
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43])
        np_utils._check_valid_ndarray(
            self.locus, input_x, exp_class=b_utils._EngineIntArrayType
        )

    def test_good_check_rank_and_shape(self):
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43])
        np_utils._check_valid_ndarray(
            self.locus, input_x, exp_rank=1, exp_shape=(2,)
        )

    def test_good_check_rank_and_class(self):
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43])
        np_utils._check_valid_ndarray(
            self.locus, input_x, exp_rank=1,
            exp_class=b_utils._EngineIntArrayType
        )

    def test_good_check_shape_and_class(self):
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43])
        np_utils._check_valid_ndarray(
            self.locus, input_x, exp_shape=(2,),
            exp_class=b_utils._EngineIntArrayType
        )

    def test_good_check_rank_shape_and_class(self): # pylint: disable=invalid-name
        """
        Input is ndarray.
        """
        input_x = np.array([42, 43])
        np_utils._check_valid_ndarray(
            self.locus, input_x, exp_rank=1, exp_shape=(2,),
            exp_class=b_utils._EngineIntArrayType
        )

    def test_bad_check(self):
        """
        Input is not ndarray.
        """
        input_x = [42, 43]
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be None or an instance of numpy.ndarray$',
            np_utils._check_is_ndarray,
            self.locus, input_x,
            exp_rank=2, exp_shape=(2, 1),
            exp_class=b_utils._EngineFloat64ArrayType,
            allow_none=True,
        )

    def test_bad_check_none(self):
        """
        Input is not ndarray.
        """
        input_x = None
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be an instance of numpy.ndarray$',
            np_utils._check_is_ndarray,
            self.locus, input_x, allow_none=False,
        )

    def test_bad_check_null_shape(self):
        """
        Shape check should reject shapeless ndarrays of unusual data.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            '.*must have non-empty shape',
            np_utils._check_valid_ndarray,
            self.locus, np.array(42),
        )

    def test_bad_check_rank(self):
        """
        Input has incorrect rank.
        """
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have rank 2$',
            np_utils._check_valid_ndarray,
            self.locus, input_x, exp_rank=2,
            exp_class=b_utils._EngineIntArrayType,
        )

    def test_bad_check_shape(self):
        """
        Input has incorrect shape.
        """
        input_x = np.array([[42, 43, 44]])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + r'.*must have shape \(2, 1\)$',
            np_utils._check_valid_ndarray,
            self.locus, input_x,
            exp_shape=(2, 1),
            exp_class=b_utils._EngineIntArrayType,
        )

    def test_bad_check_shape_neg(self):
        """
        Input has a negative expected extent.
        """
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re +
            r'.*expected extent in dimension 2 .* must not be negative',
            np_utils._check_valid_ndarray,
            self.locus, input_x,
            exp_rank=2, exp_shape=(2, -1),
            exp_class=b_utils._EngineIntArrayType,
        )

    def test_bad_check_dtype(self):
        """
        Input has incorrect dtype.
        """
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be an instance of numpy.float64$',
            np_utils._check_valid_ndarray,
            self.locus, input_x,
            exp_rank=2, exp_shape=(2, 1),
            exp_class=b_utils._EngineFloat64ArrayType,
        )

    def test_bad_copy_rank(self):
        """
        Input has incorrect rank.
        """
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have rank 2$',
            np_utils._to_ndarray,
            self.locus, input_x, copy=True, exp_rank=2,
        )

    def test_good_none(self):
        """
        Good call.
        """
        input_x = None
        copy_x = np_utils._to_ndarray(
            self.locus, input_x, copy=True, allow_none=True,
        )
        self.assertIsNone(copy_x)

    def test_good_scalar_promotion(self):
        """
        Good call.
        """
        input_x = 42
        copy_x = np_utils._to_ndarray(
            self.locus, input_x, copy=True, allow_none=True,
        )
        self.assertIsInstance(copy_x, np.ndarray)
        self.assertEqual(copy_x.shape, (1,))
        self.assertEqual(copy_x[0], input_x)

    def test_bad_copy_none(self):
        """
        Input must not be None.
        """
        input_x = None
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + '.*must have non-empty shape$',
            np_utils._to_ndarray,
            self.locus, input_x, copy=True, allow_none=False,
        )

    def test_good_copy_good_rank(self):
        """
        Good call.
        """
        input_x = np.array([42, 43])
        copy_x = np_utils._to_ndarray(
            self.locus, input_x, copy=True, exp_rank=input_x.ndim
        )
        self.assertIsNot(input_x, copy_x)
        self.assertTrue(all(input_x == copy_x))

    def test_bad_copy_shape(self):
        """
        Input has incorrect shape.
        """
        input_x = np.array([[42, 43, 44]])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagShapeError,
            self.locus_re + r'.*must have shape \(2, 1\)$',
            np_utils._to_ndarray,
            self.locus, input_x, copy=True, exp_shape=(2, 1),
        )

    def test_good_copy_good_shape(self):
        """
        Good call.
        """
        input_x = np.array([42, 43])
        copy_x = np_utils._to_ndarray(
            self.locus, input_x, copy=True, exp_shape=input_x.shape
        )
        self.assertIsNot(input_x, copy_x)
        self.assertTrue(all(input_x == copy_x))

    def test_good_class(self):
        """
        Good call.
        """

        class MyArray(np.ndarray):
            # pylint: disable=missing-docstring,too-few-public-methods
            pass

        input_x = MyArray((2, 2))
        input_x[0, :] = [42, 43]
        input_x[1, :] = [44, 45]
        copy_x = np_utils._to_ndarray(
            self.locus, input_x, copy=False, exp_shape=input_x.shape
        )
        self.assertIsInstance(copy_x, MyArray)

    def test_bad_dtype(self):
        """
        Input has incorrect dtype.
        """
        input_x = np.array([42, 43])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be an instance of numpy.float64$',
            np_utils._to_ndarray,
            self.locus, input_x, copy=True, exp_rank=1,
            exp_class=b_utils._EngineFloat64ArrayType,
        )

    def test_bad_uncastable(self):
        """
        Input cannot be cast by np.array.
        """
        input_x = [ctypes.c_double(42.0), ctypes.c_double(43.0)]
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*numpy could not convert',
            np_utils._to_ndarray,
            self.locus, input_x, copy=False,
        )

class UpdateTestArray(object):
    """
    Abstract suite of unit tests for the array updating classes (for
    callbacks).
    """

    @classmethod
    def setUpClass(cls): # pylint: disable=invalid-name
        """
        Define the abstract attributes.
        """
        cls.arg_name = None
        cls.locus = None

        cls.class_to_test = lambda *args, **kwargs: None # pragma: no cover
        cls.exp_dtype = None
        cls.sentinel = None
        cls.sentinel_i = 1
        cls.sentinel_j = 0
        cls.chrlen = None
        cls.sorder = 'F'
        cls.mirror_update = True

    def create_pointer(
            self,
            from_x_list,
    ):
        """
        Create the ctypes data.
        """
        from_x_a = np.array(
            from_x_list,
            dtype=self.class_to_test._underlying_scalar.numpy_type,
            order=self.sorder)
        from_x_p = from_x_a.ctypes.data_as(self.class_to_test.c_type)
        return from_x_p, from_x_a.shape

    def update_value(
            self,
            x_array,
    ):
        """
        Update the buffer.
        """
        new_value = self.class_to_test._underlying_scalar.numpy_type(
            self.sentinel
        )
        if x_array.ndim == 1:
            x_array[self.sentinel_i] = new_value
        else:
            x_array[self.sentinel_i, self.sentinel_j] = new_value

    def updated_value_eq(
            self,
            from_x_p,
            shape,
    ):
        """
        Comparison of values.
        """
        # pylint: disable=no-member
        if len(shape) == 1:
            old_value = from_x_p[self.sentinel_i]
        elif self.sorder == 'C':
            old_value = from_x_p[2*self.sentinel_i + self.sentinel_j]
        elif self.sorder == 'F':
            old_value = from_x_p[2*self.sentinel_j + self.sentinel_i]
        else:
            self.fail('inconceivable') # pragma: no cover
        self.assertEqual(
            old_value,
            self.sentinel,
        )

    def run_testfrom_pointer(self, from_x_list, as_ctypes_array):
        """
        Test correct operation.
        """
        old_as_ctypes_array = (
            b_utils._EngineState._cast_pointer_to_ctypes_array
        )
        b_utils._EngineState._cast_pointer_to_ctypes_array = as_ctypes_array
        from_x_p, x_shape = self.create_pointer(from_x_list)
        x_array = self.class_to_test.from_pointer(
            from_x_p, shape=x_shape, chrlen=self.chrlen, sorder=self.sorder,
        )
        self.assertIsInstance( # pylint: disable=no-member
            x_array,
            np.ndarray,
        )
        self.assertTrue( # pylint: disable=no-member
            np.issubdtype(x_array.dtype, self.exp_dtype)
        )
        exp_shape = x_shape
        self.assertEqual( # pylint: disable=no-member
            x_array.shape,
            exp_shape,
        )
        if x_array.ndim > 1:
            self.assertEqual( # pylint: disable=no-member
                b_utils._get_mutual_sorder(
                    None, ((x_array, 'x'),),
                ),
                self.sorder,
            )
        if self.mirror_update:
            self.update_value(x_array)
            self.updated_value_eq(from_x_p, x_array.shape)
        b_utils._EngineState._cast_pointer_to_ctypes_array = (
            old_as_ctypes_array
        )

    def run_test_from_pointer_cast(self, from_x_list):
        """
        Test correct operation.
        """
        self.run_testfrom_pointer(from_x_list, True)

    def run_test_from_pointer_as_array( # pylint: disable=invalid-name
            self,
            from_x_list
    ):
        """
        Test correct operation.
        """
        self.run_testfrom_pointer(from_x_list, False)

    def buffers_equal(self, buff1, buff2):
        """
        Compare two buffers.
        """
        for i, buff1_i in enumerate(buff1):
            self.assertEqual( # pylint: disable=no-member
                buff1_i,
                buff2[i]
            )

    def check_refresh(self, x_data, x_p):
        """
        Test success of the refresh process.
        """
        raise NotImplementedError # pragma: no cover

    def refresh_noop(self, x_data, x_p):
        """
        The refresh is expected to do no back conversion or checking.
        """
        self.buffers_equal([self.sentinel]*x_data.size, x_p)

    def run_test_refresh(self, x_data):
        """
        Test correct operation.
        """
        self.assertIsInstance(x_data, np.ndarray) # pylint:disable=no-member
        x_mould = np.array([self.sentinel]*x_data.size).reshape(x_data.shape)
        x_p, _ = self.create_pointer(x_mould)
        self.class_to_test.refresh_ctypes_buffer(
            x_data, self.locus, x_p, chrlen=self.chrlen,
        )
        if x_data.size != 0:
            self.check_refresh(x_data, x_p)

class UpdateTestArrayChar(UpdateTestArray, InputTest):
    """
    Update-check subsuite.

    Overrides buffers_equal.
    Overrides create_pointer.
    Overrides update value.
    Overrides updated_value_eq.
    """

    # pylint: disable=too-many-public-methods

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineCharArrayType
        cls.chrlen = 6
        cls.exp_dtype = np.dtype('S' + str(cls.chrlen))
        cls.sentinel = b'Monde '
        cls.mirror_update = False

    def setUp(self):
        InputTest.setUp(self)
        self.from_x_list = ['Hello', 'World!']
        self.flat_buffer = b'Hello World!'

    def buffers_equal(
            self,
            _,
            buff2,
    ):
        """
        Comparison of char values.
        """
        import sys

        for i, buffc in enumerate(self.flat_buffer):

            if sys.version_info < (3,):
                self.assertEqual(
                    buffc,
                    buff2[i]
                )
            else:
                self.assertEqual(
                    bytes([buffc]),
                    buff2[i]
                )

    def create_pointer(
            self,
            from_x_list,
    ):
        """
        Use to_ctypes.
        """
        from_x_a = np.array(
            from_x_list,
            dtype=self.class_to_test._underlying_scalar.numpy_type,
            order=self.sorder)
        from_x_ca = b_utils._EngineCharArrayType(
            from_x_a,
            {'fun_name': 'create_pointer', 'entity': 'from_x_a'},
            exp_chrlen=6, exp_shape=from_x_a.shape, mutual_sorder=self.sorder,
        ).to_ctypes()
        from_x_p = ctypes.cast(from_x_ca, ctypes.POINTER(ctypes.c_char))
        return from_x_p, from_x_a.shape

    def test_good_from_pointer_cast(self):
        """
        Test correct operation.
        """
        self.run_test_from_pointer_cast(self.from_x_list)

    def test_good_from_pointer_cast_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        from_x_list = [['Hello', 'To'], ['The', 'World!']]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_list = [['Hello', 'To'], ['The', 'World!']]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_as_array(self):
        """
        Test correct operation.
        """
        self.run_test_from_pointer_as_array(self.from_x_list)

    def test_good_from_pointer_noref(self):
        """
        Test correct operation.
        """
        from_x_p, x_shape = self.create_pointer(self.from_x_list)
        x_array = self.class_to_test.from_pointer(
            from_x_p, shape=x_shape, can_reference_contents=False,
            chrlen=self.chrlen, sorder=self.sorder,
        )
        self.assertIsInstance( # pylint: disable=no-member
            x_array,
            np.ndarray,
        )
        self.assertTrue( # pylint: disable=no-member
            np.issubdtype(x_array.dtype, self.exp_dtype)
        )
        exp_shape = x_shape
        self.assertEqual( # pylint: disable=no-member
            x_array.shape,
            exp_shape,
        )
        np.testing.assert_equal(
            x_array,
            [b'\0', b'\0']
        )

    check_refresh = UpdateTestArray.refresh_noop

    def check_refresh_1d(self, x_data, x_p):
        """
        Correct ASCII values refreshed back.
        """
        if x_data.ndim != 1:
            raise ValueError( # pragma: no cover
                'This is a test for 1d data.'
            )
        self.buffers_equal(None, x_p)

    def check_refresh_multidim(self, x_data, x_p):
        """
        Correct ASCII values refreshed back.
        """
        if x_data.ndim <= 1:
            raise ValueError( # pragma: no cover
                'This is a test for multi-d data.'
            )
        if self.sorder == 'C':
            self.flat_buffer = b'Hello To    The   World!'
        elif self.sorder == 'F':
            self.flat_buffer = b'Hello The   To    World!'
        else:
            self.fail('inconceivable') # pragma: no cover
        self.buffers_equal(None, x_p)

    def test_good_refresh_bytes(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([b'Hello', b'World!'])
        self.check_refresh = self.check_refresh_1d
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_unicode(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([u'Hello', u'World!'])
        self.check_refresh = self.check_refresh_1d
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_b_md_empty(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([[], []], dtype=np.bytes_)
        self.check_refresh = self.check_refresh_multidim
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_bytes_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array(
            [[b'Hello', b'To'], [b'The', b'World!']], order=self.sorder
        )
        self.check_refresh = self.check_refresh_multidim
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_bytes_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array(
            [[b'Hello', b'To'], [b'The', b'World!']], order=self.sorder
        )
        self.check_refresh = self.check_refresh_multidim
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_unicode_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array(
            [[u'Hello', u'To'], [u'The', u'World!']], order=self.sorder
        )
        self.check_refresh = self.check_refresh_multidim
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_unicode_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array(
            [[u'Hello', u'To'], [u'The', u'World!']], order=self.sorder
        )
        self.check_refresh = self.check_refresh_multidim
        self.run_test_refresh(from_x_ndarray)

    def test_bad_refresh(self):
        """
        Attempting to update with non-ASCII data.
        """
        from_x_ndarray = np.array(['Pinata', 'Pi\xf1ata'])
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagValueError,
            self.locus_re + '.*must comprise only ASCII characters',
            self.run_test_refresh,
            from_x_ndarray
        )

    def test_good_long_refresh_len(self):
        """
        Attempting to update with long data.
        """
        from_x_ndarray = np.array(['Hello ', 'World!!'])
        self.run_test_refresh(
            from_x_ndarray,
        )

class UpdateTestArrayComplex(UpdateTestArray):
    """
    Overrides buffers_equal.
    Overrides updated_value_eq.
    """

    def buffers_equal(
            self,
            buff1,
            buff2,
    ):
        """
        Comparison of complex values.
        """
        for i, buff1_i in enumerate(buff1):
            self.assertEqual( # pylint: disable=no-member
                buff1_i.real,
                buff2[i].real,
            )
            self.assertEqual( # pylint: disable=no-member
                buff1_i.imag,
                buff2[i].imag,
            )

    def check_refresh(self, x_data, x_p):
        raise NotImplementedError # pragma: no cover

    def updated_value_eq(
            self,
            from_x_p,
            shape,
    ):
        """
        Comparison of complex values.
        """
        # pylint: disable=no-member
        if len(shape) == 1:
            old_value = from_x_p[self.sentinel_i]
        elif self.sorder == 'C':
            old_value = from_x_p[2*self.sentinel_i + self.sentinel_j]
        elif self.sorder == 'F':
            old_value = from_x_p[2*self.sentinel_j + self.sentinel_i]
        else:
            self.fail('inconceivable') # pragma: no cover
        self.assertEqual(
            old_value.real,
            self.sentinel.real,
        )
        self.assertEqual(
            old_value.imag,
            self.sentinel.imag,
        )

class UpdateTestArrayComplex64(UpdateTestArrayComplex, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineComplex64ArrayType
        cls.exp_dtype = cls.class_to_test._underlying_scalar.numpy_type
        cls.sentinel = 41. + 40.j

    def test_good_from_pointer_cast(self):
        """
        Test correct operation.
        """
        from_x_list = [43. + 45.j, 44. + 46.j]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        from_x_list = [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_list = [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_as_array(self):
        """
        Test correct operation.
        """
        from_x_list = [43. + 45.j, 44. + 46.j]
        self.run_test_from_pointer_as_array(from_x_list)

    check_refresh = UpdateTestArray.refresh_noop

    def test_good_refresh(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([43. + 44.j, 45. + 46.j], dtype=np.complex64)
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_f(self):
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array(
            [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]],
            dtype=np.complex64, order=self.sorder
        )
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_c(self):
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array(
            [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]],
            dtype=np.complex64, order=self.sorder
        )
        self.run_test_refresh(from_x_ndarray)

class UpdateTestArrayComplex128(UpdateTestArrayComplex, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineComplex128ArrayType
        cls.exp_dtype = cls.class_to_test._underlying_scalar.numpy_type
        cls.sentinel = 41. + 40.j

    def test_good_from_pointer_cast(self):
        """
        Test correct operation.
        """
        from_x_list = [43. + 45.j, 44. + 46.j]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        from_x_list = [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_list = [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_as_array(self):
        """
        Test correct operation.
        """
        from_x_list = [43. + 45.j, 44. + 46.j]
        self.run_test_from_pointer_as_array(from_x_list)

    check_refresh = UpdateTestArray.refresh_noop

    def test_good_refresh(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([43. + 44.j, 45. + 46.j])
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_f(self):
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array(
            [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]],
            order=self.sorder
        )
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_c(self):
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array(
            [[43. + 45.j, 44. + 46.j], [47. + 48.j, 49. + 50.j]],
            order=self.sorder
        )
        self.run_test_refresh(from_x_ndarray)

class UpdateTestArrayCptr(UpdateTestArray, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineCptrArrayType
        cls.exp_dtype = cls.class_to_test._underlying_scalar.numpy_type

    def check_refresh(self, x_data, x_p):
        raise NotImplementedError # pragma: no cover

    def test_bad_from_pointer(self):
        """
        Test correct operation.
        """
        self.assertRaises(
            NotImplementedError,
            self.class_to_test.from_pointer,
            None, None,
        )

    def test_bad_refresh(self):
        """
        Test correct operation.
        """
        self.assertRaises(
            NotImplementedError,
            self.class_to_test.refresh_ctypes_buffer,
            None, None, None, None,
        )

class UpdateTestArrayFloat32(UpdateTestArray, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineFloat32ArrayType
        cls.exp_dtype = cls.class_to_test._underlying_scalar.numpy_type
        cls.sentinel = 41.0

    def buffers_equal(self, buff1, buff2):
        """
        Compare two buffers strictly as float32s.
        """
        for i, buff1_i in enumerate(buff1):
            self.assertEqual(
                np.float32(buff1_i),
                np.float32(buff2[i])
            )
    def test_good_from_pointer_cast(self):
        """
        Test correct operation.
        """
        from_x_list = [43., 44.]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        from_x_list = [[43., 44.], [45., 46.]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_list = [[43., 44.], [45., 46.]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_as_array(self):
        """
        Test correct operation.
        """
        from_x_list = [43., 44.]
        self.run_test_from_pointer_as_array(from_x_list)

    check_refresh = UpdateTestArray.refresh_noop

    def test_good_refresh(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([43., 44.], dtype=np.float32)
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_f(self):
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array(
            [[43., 44.], [45., 46.]], dtype=np.float32, order=self.sorder
        )
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_c(self):
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array(
            [[43., 44.], [45., 46.]], dtype=np.float32, order=self.sorder
        )
        self.run_test_refresh(from_x_ndarray)

class UpdateTestArrayFloat64(UpdateTestArray, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineFloat64ArrayType
        cls.exp_dtype = cls.class_to_test._underlying_scalar.numpy_type
        cls.sentinel = 41.0

    def test_good_from_pointer_cast(self):
        """
        Test correct operation.
        """
        from_x_list = [43., 44.]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        from_x_list = [[43., 44.], [45., 46.]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_list = [[43., 44.], [45., 46.]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_as_array(self):
        """
        Test correct operation.
        """
        from_x_list = [43., 44.]
        self.run_test_from_pointer_as_array(from_x_list)

    check_refresh = UpdateTestArray.refresh_noop

    def test_good_refresh(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([43., 44.])
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_f(self):
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array([[43., 44.], [45., 46.]], order=self.sorder)
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_c(self):
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array([[43., 44.], [45., 46.]], order=self.sorder)
        self.run_test_refresh(from_x_ndarray)

class UpdateTestArrayInt(UpdateTestArray, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineIntArrayType
        cls.exp_dtype = cls.class_to_test._underlying_scalar.numpy_type
        cls.sentinel = 41

    def test_good_from_pointer_cast(self):
        """
        Test correct operation.
        """
        from_x_list = [43, 44]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        from_x_list = [[43, 44], [45, 46]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_list = [[43, 44], [45, 46]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_as_array(self):
        """
        Test correct operation.
        """
        import sys
        import warnings

        from_x_list = [43, 44]

        if (
                b_utils._EngineState._int_model['bits'] == 64 and
                sys.version_info < (3,)
        ):
            # Silence warning about Python 2 NumPy problem,
            # from numpy/ctypeslib.py:
            # (A builtin ctypes object gave a PEP3118 format string that does
            # not match its itemsize).
            # Since we do not ship with the '_cast_pointer_to_ctypes_array'
            # run-time facility disabled, this is merely an annoyance
            # during testing.
            warnings.simplefilter('ignore', RuntimeWarning)

        self.run_test_from_pointer_as_array(from_x_list)
        warnings.simplefilter('default', RuntimeWarning)

    check_refresh = UpdateTestArray.refresh_noop

    def test_good_refresh(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array(
            [43, 44], dtype=b_utils._EngineState._int_model['numpy']
        )
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_f(self):
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array([[43, 44], [45, 46]], order=self.sorder)
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_c(self):
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array([[43, 44], [45, 46]], order=self.sorder)
        self.run_test_refresh(from_x_ndarray)

class UpdateTestArrayLogical(UpdateTestArray, InputTest):
    """
    Update-check subsuite.

    Overrides create_pointer.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestArray.setUpClass()
        cls.class_to_test = b_utils._EngineLogicalArrayType
        cls.exp_dtype = cls.class_to_test._underlying_scalar.numpy_type
        cls.sentinel = False
        cls.mirror_update = False

    def create_pointer(
            self,
            from_x_list,
    ):
        """
        Create the ctypes data from intermediate int representation.
        """
        from_x_a = np.array(
            from_x_list,
            dtype=self.class_to_test._underlying_scalar.numpy_type,
            order=self.sorder)
        from_x_ia = from_x_a.astype(b_utils._EngineState._int_model['numpy'])
        from_x_p = from_x_ia.ctypes.data_as(self.class_to_test.c_type)
        return from_x_p, from_x_a.shape

    def test_good_from_pointer_cast(self):
        """
        Test correct operation.
        """
        from_x_list = [True, True]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_f(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        from_x_list = [[True, False], [True, False]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_cast_multidim_c(self): # pylint: disable=invalid-name
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_list = [[True, False], [True, False]]
        self.run_test_from_pointer_cast(from_x_list)

    def test_good_from_pointer_as_array(self):
        """
        Test correct operation.
        """
        from_x_list = [True, True]
        self.run_test_from_pointer_as_array(from_x_list)

    def test_good_from_pointer_noref(self):
        """
        Test correct operation.
        """
        from_x_list = [True, True]
        from_x_p, x_shape = self.create_pointer(from_x_list)
        x_array = self.class_to_test.from_pointer(
            from_x_p, shape=x_shape, can_reference_contents=False,
            sorder=self.sorder,
        )
        self.assertIsInstance( # pylint: disable=no-member
            x_array,
            np.ndarray,
        )
        self.assertTrue( # pylint: disable=no-member
            np.issubdtype(x_array.dtype, self.exp_dtype)
        )
        exp_shape = x_shape
        self.assertEqual( # pylint: disable=no-member
            x_array.shape,
            exp_shape,
        )
        np.testing.assert_equal(
            x_array,
            [False]*2,
        )

    check_refresh = UpdateTestArray.refresh_noop

    def check_refresh_1d(self, x_data, x_p):
        """
        Correct integer values refreshed back.
        """
        if x_data.ndim != 1:
            raise ValueError( # pragma: no cover
                'This is a test for 1d data.'
            )
        self.buffers_equal([
            b_utils._EngineState._literal_true_int, 0,
        ], x_p)

    def check_refresh_multidim(self, x_data, x_p):
        """
        Correct integer values refreshed back.
        """
        if x_data.ndim <= 1:
            raise ValueError( # pragma: no cover
                'This is a test for multi-d data.'
            )
        if self.sorder == 'C':
            self.buffers_equal([
                b_utils._EngineState._literal_true_int, 0,
                b_utils._EngineState._literal_true_int, 0,
            ], x_p)
        elif self.sorder == 'F':
            self.buffers_equal([
                b_utils._EngineState._literal_true_int,
                b_utils._EngineState._literal_true_int,
                0, 0,
            ], x_p)
        else:
            self.fail('inconceivable') # pragma: no cover

    def test_good_refresh(self):
        """
        Test correct operation.
        """
        from_x_ndarray = np.array([True, False])
        self.check_refresh = self.check_refresh_1d
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_f(self):
        """
        Test correct operation.
        """
        self.sorder = 'F'
        from_x_ndarray = np.array(
            [[True, False], [True, False]], order=self.sorder
        )
        self.check_refresh = self.check_refresh_multidim
        self.run_test_refresh(from_x_ndarray)

    def test_good_refresh_multidim_c(self):
        """
        Test correct operation.
        """
        self.sorder = 'C'
        from_x_ndarray = np.array(
            [[True, False], [True, False]], order=self.sorder
        )
        self.check_refresh = self.check_refresh_multidim
        self.run_test_refresh(from_x_ndarray)

class UpdateTestScalar(object):
    """
    Abstract suite of unit tests for the scalar updating classes (for
    callbacks).
    """

    @classmethod
    def setUpClass(cls): # pylint: disable=invalid-name
        """
        Define the abstract attributes.
        """
        cls.arg_name = None
        cls.locus = None

        cls.class_to_test = lambda *args, **kwargs: None # pragma: no cover
        cls.sentinel = None

    def check_refresh(self, x_data, x_p):
        """
        Test success of the refresh process.
        """
        compare_data = (
            x_data
            if not isinstance(x_data, self.class_to_test.c_type)
            else x_data.value
        )
        self.assertEqual( # pylint: disable=no-member
            compare_data,
            x_p.contents.value,
        )

    def create_pointer(
            self,
            x_data,
    ):
        """
        Create the incoming pointer.
        """
        return ctypes.pointer(
            self.class_to_test(x_data, self.locus).to_ctypes()
        )

    def run_test_refresh(self, x_data):
        """
        Test correct operation.
        """
        x_p = self.create_pointer(self.sentinel)
        self.class_to_test(
            x_data, self.locus
        ).refresh_ctypes_buffer(x_p)
        self.check_refresh(x_data, x_p)

class UpdateTestScalarChar(UpdateTestScalar, InputTest):
    """
    Update-check subsuite.

    Overrides check_refresh.
    Overrides create_pointer.
    Overrides run_test_refresh.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineCharScalarType
        cls.sentinel = 'World'
        cls.chrlen = 5

    def check_refresh(self, x_data, x_p):
        """
        Test success of the refresh process.
        """
        import sys
        compare_data = (
            x_data.encode('ascii')
            if sys.version_info >= (3,) and isinstance(x_data, str)
            else x_data
        )
        self.assertEqual(
            compare_data,
            x_p[:self.chrlen],
        )

    def create_pointer(
            self,
            x_data,
    ):
        """
        Create the incoming pointer as a blank char*.
        """
        ct_arr = ctypes.pointer(
            self.class_to_test(
                x_data,
                self.locus,
                self.chrlen
            ).to_ctypes()
        )
        return ctypes.cast(ct_arr, self.class_to_test.c_type)

    def run_test_refresh(self, x_data):
        """
        Test correct operation.
        """
        x_p = self.create_pointer(self.sentinel)
        self.class_to_test(
            x_data, self.locus, self.chrlen,
        ).refresh_ctypes_buffer(x_p)
        self.check_refresh(x_data, x_p)

    def test_good_none(self):
        """
        Test correct operation, None.
        """
        x_p = self.create_pointer(self.sentinel)
        self.class_to_test(
            None, self.locus, self.chrlen, allow_none=True,
        ).refresh_ctypes_buffer(x_p)

    def test_good_bytes(self):
        """
        Test correct operation.
        """
        self.run_test_refresh(b'Hello')

    def test_good_unicode(self):
        """
        Test correct operation.
        """
        self.run_test_refresh(u'Hello')

class UpdateTestScalarComplex64(UpdateTestScalar, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineComplex64ScalarType
        cls.sentinel = np.complex64(-53. + 53.j)

    def check_refresh(self, x_data, x_p):
        """
        Test success of the refresh process.
        """
        self.assertEqual(
            x_data.real,
            x_p.contents.real,
        )
        self.assertEqual(
            x_data.imag,
            x_p.contents.imag,
        )

    def test_good_none(self):
        """
        Test correct operation, None.
        """
        x_p = self.create_pointer(self.sentinel)
        self.class_to_test(
            None, self.locus, allow_none=True,
        ).refresh_ctypes_buffer(x_p)

    def test_good_numpy(self):
        """
        Test correct operation, numpy data.
        """
        self.run_test_refresh(np.complex64(42. + 43.j))

class UpdateTestScalarComplex128(UpdateTestScalar, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineComplex128ScalarType
        cls.sentinel = -53. + 53.j

    def check_refresh(self, x_data, x_p):
        """
        Test success of the refresh process.
        """
        self.assertEqual(
            x_data.real,
            x_p.contents.real,
        )
        self.assertEqual(
            x_data.imag,
            x_p.contents.imag,
        )

    def test_good(self):
        """
        Test correct operation, native data.
        """
        self.run_test_refresh(42. + 43.j)

    def test_good_numpy(self):
        """
        Test correct operation, numpy data.
        """
        self.run_test_refresh(np.complex128(42. + 43.j))

class UpdateTestScalarFloat32(UpdateTestScalar, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineFloat32ScalarType
        cls.sentinel = np.float32(-53.)

    def test_good_none_retval(self):
        """
        Test correct operation, None.
        """
        self.assertIsNone(self.class_to_test.to_py_from_retval(None))

    def test_good_numpy(self):
        """
        Test correct operation, numpy data.
        """
        self.run_test_refresh(np.float32(42.))

    def test_good_ctypes(self):
        """
        Test correct operation, ctypes data.
        """
        self.run_test_refresh(self.class_to_test.c_type(42.))

class UpdateTestScalarFloat64(UpdateTestScalar, InputTest):
    """
    Update-check subsuite.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineFloat64ScalarType
        cls.sentinel = -53.

    def test_good(self):
        """
        Test correct operation, native data.
        """
        self.run_test_refresh(42.)

    def test_good_none(self):
        """
        Test correct operation, None.
        """
        x_p = self.create_pointer(self.sentinel)
        self.class_to_test(
            None, self.locus, allow_none=True,
        ).refresh_ctypes_buffer(x_p)

    def test_good_none_retval(self):
        """
        Test correct operation, None.
        """
        self.assertIsNone(self.class_to_test.to_py_from_retval(None))

    def test_good_numpy(self):
        """
        Test correct operation, numpy data.
        """
        self.run_test_refresh(np.float64(42.))

    def test_good_ctypes(self):
        """
        Test correct operation, ctypes data.
        """
        self.run_test_refresh(self.class_to_test.c_type(42.))

class UpdateTestScalarInt(UpdateTestScalar, InputTest):
    """
    Update-check subsuite.

    Overrides check_refresh.
    """

    import sys

    @classmethod
    def setUpClass(cls):
        UpdateTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineIntScalarType
        cls.sentinel = -53

    def check_refresh(self, x_data, x_p):
        """
        Test success of the refresh process, allowing mismatching ints.
        """
        compare_data = (
            x_data
            if not isinstance(
                x_data,
                (
                    b_utils._SYS_CTYPES_INTS[32],
                    b_utils._SYS_CTYPES_INTS[64],
                )
            )
            else x_data.value
        )
        self.assertEqual(
            compare_data,
            x_p.contents.value,
        )

    def test_good(self):
        """
        Test correct operation, native data.
        """
        self.run_test_refresh(42)

    @unittest.skipUnless(
        (
            sys.version_info < (3,) and
            b_utils._IS_IL32
        ),
        'int and long are equivalent'
    )
    def test_good_long(self):
        """
        Test correct operation, long native data.
        """
        self.run_test_refresh(long(42)) # pylint: disable=undefined-variable

    def test_good_none_retval(self):
        """
        Test correct operation, None.
        """
        self.assertIsNone(self.class_to_test.to_py_from_retval(None))

    def test_good_numpy32(self):
        """
        Test correct operation, numpy 32-bit data.
        """
        self.run_test_refresh(np.int32(42))

    def test_good_numpy64(self):
        """
        Test correct operation, numpy 64-bit data.
        """
        self.run_test_refresh(np.int64(42))

    def test_good_ctypes32(self):
        """
        Test correct operation, ctypes 32-bit data.
        """
        self.run_test_refresh(b_utils._SYS_CTYPES_INTS[32](42))

    def test_good_ctypes64(self):
        """
        Test correct operation, ctypes 64-bit data.
        """
        self.run_test_refresh(b_utils._SYS_CTYPES_INTS[64](42))

class UpdateTestScalarLogical(UpdateTestScalar, InputTest):
    """
    Update-check subsuite.

    Overrides check_refresh.
    """

    @classmethod
    def setUpClass(cls):
        UpdateTestScalar.setUpClass()
        cls.class_to_test = b_utils._EngineLogicalScalarType
        cls.sentinel = False

    def check_refresh(self, x_data, x_p):
        """
        Test success of the refresh process.
        """
        compare_data = b_utils._EngineState._literal_true_int if x_data else 0
        self.assertEqual(
            compare_data,
            x_p.contents.value,
        )

    def test_good(self):
        """
        Test correct operation, native data.
        """
        self.run_test_refresh(True)

    def test_good_none_retval(self):
        """
        Test correct operation, None.
        """
        self.assertIsNone(self.class_to_test.to_py_from_retval(None))

    def test_good_numpy(self):
        """
        Test correct operation, numpy data.
        """
        self.run_test_refresh(np.bool_(True))

class CallableTest(InputTest):
    """
    Test set for parameter checking of callables.
    """

    def setUp(self):
        InputTest.setUp(self)
        self.fun = b_utils._check_callable

    def test_bad_no_none(self):
        """
        Not callable, None not permitted.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be callable$',
            self.fun,
            42, self.locus,
        )

    def test_bad_none(self):
        """
        Not callable, None permitted.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be None or callable$',
            self.fun,
            42, self.locus, allow_none=True,
        )

    def test_bad_none_not_none(self):
        """
        None, None not permitted.
        """
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagTypeError,
            self.locus_re + '.*must be callable$',
            self.fun,
            None, self.locus,
        )

    def test_good_callable(self):
        """
        Good call.
        """
        self.fun(self, self.locus)

    def test_good_none(self):
        """
        Good call.
        """
        self.fun(None, self.locus, allow_none=True)

class ErrorsTest(unittest.TestCase):
    """
    Test set for warnings/errors/exception handling.
    """

    # pylint: disable=too-many-public-methods

    from naginterfaces.kusari import _PRODUCT_CODE

    def setUp(self):
        """
        Define the general error locus.
        """
        import sys

        self.locus = {'fun_name': 'z99zz', 'errcodes': (42, 43, 44)}
        self.locus_re = (
            'function ' + self.locus['fun_name'] + ', code 42:43,44'
        )

        if sys.version_info < (3,):
            self.my_assert_regex = self.assertRegexpMatches
            self.my_assert_raises_regex = self.assertRaisesRegexp
        else:
            self.my_assert_regex = self.assertRegex # pylint: disable=no-member
            self.my_assert_raises_regex = self.assertRaisesRegex # pylint: disable=no-member

    def test_warn_as_warning(self):
        """
        Warning raised as a warning.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            self.locus['cb_excs'] = [b_utils.UserCallbackTerminate('spam')]
            b_utils._raise_warning(
                b_utils.NagCallbackTerminateWarning, ['Hello'], self.locus,
            )
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagCallbackTerminateWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*' + self.locus_re + r'.*\n.*\*\* Hello$'
        )

    def test_warn_as_warning_short(self):
        """
        Warning raised as a warning, short errcodes.
        """
        import warnings

        self.locus['errcodes'] = (42,)

        with warnings.catch_warnings(record=True) as w_ctxt:
            b_utils._raise_warning(
                b_utils.NagAlgorithmicWarning, ['Hello'], self.locus,
            )
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagAlgorithmicWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            (
                '(?m).*function ' + self.locus['fun_name'] +
                r', code 42\)\n\*\* Hello$'
            )
        )

    def test_warn_as_exception(self):
        """
        Warning raised as an exception.
        """
        import warnings

        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        with self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagAlgorithmicWarning, # pylint: disable=bad-continuation
            '(?m).*' + self.locus_re + r'.*\n.*\*\* Hello$' # pylint: disable=bad-continuation
        ) as w_ctxt:
            b_utils._raise_warning(
                b_utils.NagAlgorithmicWarning, ['Hello'], self.locus,
            )
        self.assertEqual(
            w_ctxt.exception.errno,
            42
        )
        self.assertEqual(
            w_ctxt.exception._errcodes,
            self.locus['errcodes'],
        )
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)

    def test_warn_handling(self):
        """
        Handle a warning.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            b_utils._EngineWarningExit(
                fun_name=self.locus['fun_name'],
                msg_lines=['Hello'],
                errcodes=self.locus['errcodes'],
            ).manage()
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagAlgorithmicWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*' + self.locus_re + r'.*\n.*\*\* Hello$'
        )

    def test_major_warn_handling(self):
        """
        Handle a major warning.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            b_utils._EngineMajorWarningExit(
                fun_name=self.locus['fun_name'],
                msg_lines=['Hello'],
                errcodes=self.locus['errcodes'],
            ).manage()
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagAlgorithmicMajorWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*' + self.locus_re + r'.*\n.*\*\* Hello$'
        )

    def test_cb_warn_handling(self):
        """
        Handle a callback warning.
        """
        import warnings

        cb_excs = [b_utils.UserCallbackTerminate(42), ValueError('hello')]

        with warnings.catch_warnings(record=True) as w_ctxt:
            b_utils._UserCallbackExit(
                fun_name=self.locus['fun_name'],
                msg_lines=['Hello'],
                errcodes=self.locus['errcodes'],
                cb_excs=cb_excs,
            ).manage()
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagCallbackTerminateWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*' + self.locus_re + r'.*\n.*\*\* Hello$'
        )
        self.assertIs(w_ctxt[0].message.cb_excs, cb_excs)

    def test_handle_err(self):
        """
        Handle an error.
        """
        with self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagValueError, # pylint: disable=bad-continuation
            ( # pylint: disable=bad-continuation
                '(?m).*' + 'function ' + self.locus['fun_name'] +
                ', argument x, code 42:43,44'
            )
        ) as w_ctxt:
            b_utils._EngineErrorExit(
                fun_name=self.locus['fun_name'],
                msg_lines=['Hello'],
                entity='x',
                errcodes=self.locus['errcodes'],
            ).manage()
        self.assertEqual(
            w_ctxt.exception.errno,
            42
        )
        self.assertEqual(
            w_ctxt.exception._errcodes,
            (42, 43, 44),
        )

    def test_handle_unexpected(self):
        """
        Handle an unexpected exit.
        """
        with self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagException, # pylint: disable=bad-continuation
            ( # pylint: disable=bad-continuation
                '(?m).*' + 'function ' + self.locus['fun_name'] +
                ', code -99:43,44' +
                r'.*\n.*\*\* Unexpected error.\n\*\* Please contact NAG.$'
            )
        ) as w_ctxt:
            b_utils._EngineUnexpectedExit(
                fun_name=self.locus['fun_name'],
                msg_lines=['Hello'],
                errcodes=self.locus['errcodes'],
            ).manage()
        self.assertEqual(
            w_ctxt.exception.errno,
            -99
        )
        self.assertEqual(
            w_ctxt.exception._errcodes,
            (-99, 43, 44),
        )

    def test_handle_unexpected_blank(self):
        """
        Handle an unexpected exit with no further error codes.
        """
        with self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagException, # pylint: disable=bad-continuation
            ( # pylint: disable=bad-continuation
                '(?m).*' + 'function ' + self.locus['fun_name'] +
                ', code -99[^:]' +
                r'.*\n.*\*\* Unexpected error.\n\*\* Please contact NAG.$'
            )
        ) as w_ctxt:
            b_utils._EngineUnexpectedExit(
                fun_name=self.locus['fun_name'],
                msg_lines=['Hello'],
                errcodes=(),
            ).manage()
        self.assertEqual(
            w_ctxt.exception.errno,
            -99
        )
        self.assertEqual(
            w_ctxt.exception._errcodes,
            (-99,),
        )

    @unittest.skipUnless(
        _PRODUCT_CODE.endswith('L'),
        'Library in unlicensed',
    )
    def test_handle_nolic(self):
        """
        Handle licence error.
        """
        with self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagLicenceError, # pylint: disable=bad-continuation
            r'(?m)\n\*\* KUSARI.*\n\*\*' + # pylint: disable=bad-continuation
            r' PRODUCT.*\n\*\* Please contact NAG.$' # pylint: disable=bad-continuation
        ) as w_ctxt:
            b_utils._EngineLicenceErrorExit().manage()
        self.assertEqual(
            w_ctxt.exception.errno,
            b_utils._IERR_NO_LIC,
        )
        self.assertEqual(
            w_ctxt.exception._errcodes,
            (b_utils._IERR_NO_LIC,),
        )

    def test_handle_no_cb_excs(self): # pylint: disable=no-self-use
        """
        Correct operation when no excs.
        """
        b_utils._handle_cb_exception([])

    def test_handle_cb_excs(self):
        """
        Correct operation when an exc.
        """
        import sys

        excs = []

        try:
            raise b_utils.UserCallbackTerminate(42)
        except b_utils.UserCallbackTerminate:
            excs.append(sys.exc_info())

        try:
            raise ValueError('Hello!')
        except ValueError:
            excs.append(sys.exc_info())

        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            ValueError,
            'Hello!',
            b_utils._handle_cb_exception,
            excs
        )

    def test_raise_exc_info(self):
        """
        Correct operation.
        """
        import sys

        exc = None

        try:
            raise ValueError('Hello!')
        except ValueError:
            exc = sys.exc_info()

        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            ValueError,
            'Hello!',
            b_utils._raise_exc_info,
            exc
        )

    @unittest.skipUnless(
        _PRODUCT_CODE.endswith('L'),
        'Library in unlicensed',
    )
    def test_lic_fail(self):
        """
        Correct operation.
        """
        self.assertRaises( # pylint: disable=no-member
            b_utils.NagLicenceError,
            b_utils._EngineState._handle_global_errors,
            'z99zz', None, b_utils._IERR_NO_LIC,
        )

    def test_alloc_fail(self):
        """
        Correct operation.
        """
        self.assertRaises( # pylint: disable=no-member
            b_utils.NagMemoryError,
            b_utils._EngineState._handle_global_errors,
            'z99zz', None, b_utils._IERR_ALLOC_FAIL,
        )

    def test_bad_global_zero(self):
        """
        Attempting to handle zero ifail.
        """
        self.assertRaises( # pylint: disable=no-member
            b_utils.NagException,
            b_utils._EngineState._handle_global_errors,
            'z99zz', None, 0,
        )

    def test_withdrawn_base(self):
        """
        Withdrawn decorator called directly via base.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            b_utils._withdrawal_test()
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagWithdrawalWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*scheduled for withdrawal.*Mark 42.\n.*no replacement'
        )

    def test_withdrawn_warn_base(self):
        """
        Withdrawn decorator called directly via base.
        """
        import inspect
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            b_utils._withdrawal_warn_test()
        self.assertEqual(len(w_ctxt), 2)
        self.assertIs(w_ctxt[0].category, b_utils.NagWithdrawalWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*scheduled for withdrawal.*Mark 42.\n.*no replacement'
        )
        self.assertEqual(
            w_ctxt[0].filename,
            inspect.getframeinfo(inspect.currentframe())[0]
        )
        self.assertIs(w_ctxt[1].category, b_utils.NagAlgorithmicWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[1].message),
            '(?m).*spam\n.*eggs'
        )
        self.assertEqual(
            w_ctxt[1].filename,
            inspect.getframeinfo(inspect.currentframe())[0]
        )

    def test_withdrawn_library(self):
        """
        Withdrawn decorator called directly via library interfaces.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            np_utils._withdrawal_test()
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagWithdrawalWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*scheduled for withdrawal.*Mark 42.\n.*' +
            'following advice.*\nReplacement advice.$'
        )

    def test_withdrawn_warn_library(self):
        """
        Withdrawn decorator called directly via library.
        """
        import inspect
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            np_utils._withdrawal_warn_test()
        self.assertEqual(len(w_ctxt), 2)
        self.assertIs(w_ctxt[0].category, b_utils.NagWithdrawalWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*scheduled for withdrawal.*Mark 42.\n.*no replacement'
        )
        self.assertEqual(
            w_ctxt[0].filename,
            inspect.getframeinfo(inspect.currentframe())[0]
        )
        self.assertIs(w_ctxt[1].category, b_utils.NagAlgorithmicWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[1].message),
            '(?m).*spam\n.*eggs'
        )
        self.assertEqual(
            w_ctxt[1].filename,
            inspect.getframeinfo(inspect.currentframe())[0]
        )

    def test_withdrawn_nag4py(self):
        """
        Withdrawn subpackage imported.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            import naginterfaces.nag4py as _
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagWithdrawalWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            'This subpackage is scheduled for withdrawal.*Mark 28.'
        )

    def test_deprecated_base(self):
        """
        Withdrawn decorator called directly via base.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            b_utils._deprecation_test()
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagDeprecatedWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*is deprecated.\n.*' +
            'following advice.*\nReplacement base advice.$'
        )

    def test_deprecated_library(self):
        """
        Withdrawn decorator called directly via library interfaces.
        """
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            np_utils._deprecation_test()
        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagDeprecatedWarning)
        self.my_assert_regex( # pylint: disable=deprecated-method
            str(w_ctxt[0].message),
            '(?m).*is deprecated.\n.*' +
            'following advice.*\nReplacement advice.$'
        )

class KusariTest(unittest.TestCase):
    """
    Test set for Kusari wrappers.
    """

    from naginterfaces.kusari import _PRODUCT_CODE

    def setUp(self):
        """
        Define the general error locus.
        """
        import sys

        from naginterfaces.base.info import impl_details as b_impl_details

        if sys.version_info < (3,):
            self.my_assert_regex = self.assertRegexpMatches
            self.my_assert_raises_regex = self.assertRaisesRegexp
        else:
            self.my_assert_regex = self.assertRegex # pylint: disable=no-member
            self.my_assert_raises_regex = self.assertRaisesRegex # pylint: disable=no-member

        self.pcode = b_impl_details()['product_code']

    @unittest.skipUnless(
        _PRODUCT_CODE.endswith('L'),
        'Library in unlicensed',
    )
    def test_hostid(self):
        """
        hostid output.
        """
        self.my_assert_regex( # pylint: disable=deprecated-method
            b_utils._capture_hostid(),
            '^KUSARI ID = .*'
        )

    def test_hostid_direct(self):
        """
        hostid output.
        """
        import sys

        if sys.version_info < (3,):
            import StringIO # pylint: disable=import-error
        else:
            import io

        from naginterfaces.kusari import hostid

        if sys.version_info < (3,):
            io_mod = StringIO
        else:
            io_mod = io

        capture = io_mod.StringIO()
        sys.stdout = capture
        hostid()
        sys.stdout = sys.__stdout__

        self.my_assert_regex( # pylint: disable=deprecated-method
            capture.getvalue(),
            '(?m)^KUSARI ID = .*\nNAG Library product code ='
        )

    def test_lcheck_lic_q(self):
        """
        lcheck output, licence available, quiet.
        """
        self.my_assert_regex( # pylint: disable=deprecated-method
            b_utils._capture_lcheck(self.pcode, quiet=True),
            '^Licence available;'
        )

    def test_lcheck_lic(self):
        """
        lcheck output, licence available, verbose.
        """
        klines = b_utils._capture_lcheck(self.pcode).splitlines()

        if self.pcode.endswith('L'):
            self.assertTrue(len(klines) > 2)
        else:
            self.assertTrue(len(klines) == 1)

        was_avail = False

        for line in klines:

            if line.startswith('Licence available;'):
                was_avail = True

        self.assertTrue(was_avail)

    def test_lcheck_nolic_q(self):
        """
        lcheck output, licence unavailable, quiet.
        """
        import os

        kfile = os.environ['NAG_KUSARI_FILE']
        del os.environ['NAG_KUSARI_FILE']

        if self.pcode.endswith('L'):
            re_check = (
                '^Error: Licence not found; this product requires a key for ' +
                self.pcode
            )
        else:
            re_check = 'Licence available'

        self.my_assert_regex( # pylint: disable=deprecated-method
            b_utils._capture_lcheck(self.pcode, quiet=True),
            re_check,
        )
        os.environ['NAG_KUSARI_FILE'] = kfile

    def test_lcheck_nolic(self):
        """
        lcheck output, licence available, verbose.
        """
        import os

        kfile = os.environ['NAG_KUSARI_FILE']
        del os.environ['NAG_KUSARI_FILE']

        klines = b_utils._capture_lcheck(self.pcode).splitlines()

        if self.pcode.endswith('L'):
            self.assertTrue(len(klines) > 2)
        else:
            self.assertTrue(len(klines) == 1)

        was_avail = True

        for line in klines:

            if line.startswith(
                    'Error: Licence not found; this product requires a key '
                    'for ' + self.pcode
            ):
                was_avail = False

        if self.pcode.endswith('L'):
            self.assertFalse(was_avail)
        else:
            self.assertTrue(was_avail)

        os.environ['NAG_KUSARI_FILE'] = kfile

class AuxilsTest(InputTest):
    """
    Test set for wrappers for internal Engine auxiliaries.
    """

    def test_e04ptan_ok(self):
        """
        Valid operation of e04ptan wrapper.
        """
        from naginterfaces.base import opt
        handle = opt.handle_init(42)
        nvar, nnzu, nnzuc, nnzua = b_utils._e04ptan(handle)
        self.assertEqual(nvar.value, 42)
        self.assertEqual(nnzu.value, 0)
        self.assertEqual(nnzuc.value, 0)
        self.assertEqual(nnzua.value, 0)
        opt.handle_free(handle)

    def test_e04ptan_null_handle(self):
        """
        Exception handling when handle is null.
        """
        handle = b_utils.Handle()
        self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagValueError,
            'handle does not belong',
            b_utils._e04ptan, handle,
        )

    def test_f08mdan(self):
        """
        Valid operation of f08mdan wrapper.
        """
        # pylint: disable=invalid-name
        lq, liq = b_utils._f08mdan(False, 100)
        exp_lq = 10500
        exp_liq = 900
        self.assertEqual(lq.value, exp_lq)
        self.assertEqual(liq.value, exp_liq)

    def test_g02zkbn_do_init(self):
        """
        Valid operation of g02zkbn wrapper.
        """
        # pylint: disable=invalid-name
        ip = 5
        sdres, rip, tdch, liopts, lopts, lstate, calcib = b_utils._g02zkbn(
            0, 'G02QGF', ip, 10, None, None,
        )
        self.assertEqual(sdres.value, 0)
        self.assertEqual(rip.value, ip)
        self.assertEqual(tdch.value, 0)
        self.assertEqual(liopts.value, 0)
        self.assertEqual(lopts.value, 0)
        self.assertEqual(lstate.value, 0)
        self.assertEqual(calcib.value, 1)

    def test_g02zkbn_donot_init_no_state(self):
        """
        Valid operation of g02zkbn wrapper.
        """
        # pylint: disable=invalid-name
        from naginterfaces.base import correg
        comm = {}
        correg.optset('Initialize = G02QGF', comm)
        ip = 5
        sdres, rip, tdch, liopts, lopts, lstate, calcib = b_utils._g02zkbn(
            1, 'G02QGF', ip, 10, comm, None,
        )
        self.assertEqual(sdres.value, 0)
        self.assertEqual(rip.value, ip)
        self.assertEqual(tdch.value, 0)
        self.assertEqual(liopts.value, 100)
        self.assertEqual(lopts.value, 100)
        self.assertEqual(lstate.value, 0)
        self.assertEqual(calcib.value, 1)

    def test_g02zkbn_donot_init_with_state(self):
        """
        Valid operation of g02zkbn wrapper.
        """
        # pylint: disable=invalid-name
        from naginterfaces.base import correg, rand
        comm = {}
        correg.optset('Initialize = G02QGF', comm)
        statecomm = rand.init_repeat(
            genid=3, subid=0, seed=[42], lseed=1,
        )
        ip = 5
        sdres, rip, tdch, liopts, lopts, lstate, calcib = b_utils._g02zkbn(
            1, 'G02QGF', ip, 10, comm, statecomm,
        )
        self.assertEqual(sdres.value, 0)
        self.assertEqual(rip.value, ip)
        self.assertEqual(tdch.value, 0)
        self.assertEqual(liopts.value, 100)
        self.assertEqual(lopts.value, 100)
        self.assertEqual(lstate.value, 0)
        self.assertEqual(calcib.value, 1)

    def test_g02zkbn_options(self):
        """
        Valid operation of g02zkbn wrapper.
        """
        # pylint: disable=invalid-name
        from naginterfaces.base import correg, rand
        comm = {}
        correg.optset('Initialize = G02QGF', comm)
        correg.optset('Return Residuals = Yes', comm)
        correg.optset('Matrix Returned = Covariance', comm)
        correg.optset('Interval Method = Bootstrap XY', comm)
        statecomm = rand.init_repeat(
            genid=3, subid=0, seed=[42], lseed=1,
        )
        ip = 5
        sdres, rip, tdch, liopts, lopts, lstate, calcib = b_utils._g02zkbn(
            1, 'G02QGF', ip, 10, comm, statecomm,
        )
        self.assertEqual(sdres.value, 10)
        self.assertEqual(rip.value, ip)
        self.assertEqual(tdch.value, 10)
        self.assertEqual(liopts.value, 100)
        self.assertEqual(lopts.value, 100)
        self.assertEqual(lstate.value, 1260)
        self.assertEqual(calcib.value, 1)

if __name__ == '__main__': # pragma: no cover
    unittest.main()
